#include <tree_sitter/parser.h>

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#define LANGUAGE_VERSION 8
#define STATE_COUNT 2600
#define SYMBOL_COUNT 210
#define ALIAS_COUNT 10
#define TOKEN_COUNT 110
#define EXTERNAL_TOKEN_COUNT 1
#define MAX_ALIAS_SEQUENCE_LENGTH 8

enum {
  sym__automatic_semicolon = 1,
  sym_hash_bang_line = 2,
  anon_sym_export = 3,
  anon_sym_STAR = 4,
  anon_sym_default = 5,
  anon_sym_LBRACE = 6,
  anon_sym_COMMA = 7,
  anon_sym_RBRACE = 8,
  anon_sym_as = 9,
  anon_sym_import = 10,
  anon_sym_from = 11,
  anon_sym_var = 12,
  anon_sym_let = 13,
  anon_sym_const = 14,
  anon_sym_if = 15,
  anon_sym_else = 16,
  anon_sym_switch = 17,
  anon_sym_for = 18,
  anon_sym_LPAREN = 19,
  anon_sym_RPAREN = 20,
  anon_sym_in = 21,
  anon_sym_of = 22,
  anon_sym_while = 23,
  anon_sym_do = 24,
  anon_sym_try = 25,
  anon_sym_with = 26,
  anon_sym_break = 27,
  anon_sym_continue = 28,
  anon_sym_debugger = 29,
  anon_sym_return = 30,
  anon_sym_throw = 31,
  anon_sym_SEMI = 32,
  anon_sym_COLON = 33,
  anon_sym_case = 34,
  anon_sym_catch = 35,
  anon_sym_finally = 36,
  anon_sym_yield = 37,
  anon_sym_LBRACK = 38,
  anon_sym_RBRACK = 39,
  anon_sym_LT = 40,
  anon_sym_GT = 41,
  anon_sym_SLASH = 42,
  sym_jsx_text = 43,
  sym_jsx_identifier = 44,
  anon_sym_DOT = 45,
  anon_sym_EQ = 46,
  anon_sym_class = 47,
  anon_sym_extends = 48,
  anon_sym_async = 49,
  anon_sym_function = 50,
  anon_sym_EQ_GT = 51,
  anon_sym_new = 52,
  anon_sym_await = 53,
  anon_sym_PLUS_EQ = 54,
  anon_sym_DASH_EQ = 55,
  anon_sym_STAR_EQ = 56,
  anon_sym_SLASH_EQ = 57,
  anon_sym_PERCENT_EQ = 58,
  anon_sym_CARET_EQ = 59,
  anon_sym_AMP_EQ = 60,
  anon_sym_PIPE_EQ = 61,
  anon_sym_GT_GT_EQ = 62,
  anon_sym_GT_GT_GT_EQ = 63,
  anon_sym_LT_LT_EQ = 64,
  anon_sym_DOT_DOT_DOT = 65,
  anon_sym_QMARK = 66,
  anon_sym_AMP_AMP = 67,
  anon_sym_PIPE_PIPE = 68,
  anon_sym_GT_GT = 69,
  anon_sym_GT_GT_GT = 70,
  anon_sym_LT_LT = 71,
  anon_sym_AMP = 72,
  anon_sym_CARET = 73,
  anon_sym_PIPE = 74,
  anon_sym_PLUS = 75,
  anon_sym_DASH = 76,
  anon_sym_PERCENT = 77,
  anon_sym_LT_EQ = 78,
  anon_sym_EQ_EQ = 79,
  anon_sym_EQ_EQ_EQ = 80,
  anon_sym_BANG_EQ = 81,
  anon_sym_BANG_EQ_EQ = 82,
  anon_sym_GT_EQ = 83,
  anon_sym_instanceof = 84,
  anon_sym_BANG = 85,
  anon_sym_TILDE = 86,
  anon_sym_typeof = 87,
  anon_sym_void = 88,
  anon_sym_delete = 89,
  anon_sym_PLUS_PLUS = 90,
  anon_sym_DASH_DASH = 91,
  sym_comment = 92,
  sym_string = 93,
  anon_sym_BQUOTE = 94,
  sym__template_chars = 95,
  anon_sym_DOLLAR_LBRACE = 96,
  sym_regex = 97,
  sym_number = 98,
  sym_identifier = 99,
  sym_this = 100,
  sym_super = 101,
  sym_true = 102,
  sym_false = 103,
  sym_null = 104,
  sym_undefined = 105,
  anon_sym_AT = 106,
  anon_sym_static = 107,
  anon_sym_get = 108,
  anon_sym_set = 109,
  sym_program = 110,
  sym_export_statement = 111,
  sym_export_clause = 112,
  sym__import_export_specifier = 113,
  sym__declaration = 114,
  sym_import_statement = 115,
  sym_import_clause = 116,
  sym__from_clause = 117,
  sym_namespace_import = 118,
  sym_named_imports = 119,
  sym_expression_statement = 120,
  sym_variable_declaration = 121,
  sym_lexical_declaration = 122,
  sym_variable_declarator = 123,
  sym_statement_block = 124,
  sym_if_statement = 125,
  sym_switch_statement = 126,
  sym_for_statement = 127,
  sym_for_in_statement = 128,
  sym_for_of_statement = 129,
  sym_while_statement = 130,
  sym_do_statement = 131,
  sym_try_statement = 132,
  sym_with_statement = 133,
  sym_break_statement = 134,
  sym_continue_statement = 135,
  sym_debugger_statement = 136,
  sym_return_statement = 137,
  sym_throw_statement = 138,
  sym_empty_statement = 139,
  sym_labeled_statement = 140,
  sym_switch_body = 141,
  sym_switch_case = 142,
  sym_switch_default = 143,
  sym_catch_clause = 144,
  sym_finally_clause = 145,
  sym_parenthesized_expression = 146,
  sym__expressions = 147,
  sym__expression = 148,
  sym_yield_expression = 149,
  sym_object = 150,
  sym_assignment_pattern = 151,
  sym_array = 152,
  sym_jsx_element = 153,
  sym_jsx_fragment = 154,
  sym_jsx_expression = 155,
  sym_jsx_opening_element = 156,
  sym_nested_identifier = 157,
  sym_jsx_namespace_name = 158,
  sym_jsx_closing_element = 159,
  sym_jsx_self_closing_element = 160,
  sym_jsx_attribute = 161,
  sym_anonymous_class = 162,
  sym_class = 163,
  sym_class_heritage = 164,
  sym_function = 165,
  sym_arrow_function = 166,
  sym_generator_function = 167,
  sym_call_expression = 168,
  sym_new_expression = 169,
  sym_await_expression = 170,
  sym_member_expression = 171,
  sym_subscript_expression = 172,
  sym_assignment_expression = 173,
  sym_augmented_assignment_expression = 174,
  sym__initializer = 175,
  sym_spread_element = 176,
  sym_ternary_expression = 177,
  sym_binary_expression = 178,
  sym_unary_expression = 179,
  sym_update_expression = 180,
  sym_sequence_expression = 181,
  sym_template_string = 182,
  sym_template_substitution = 183,
  sym_arguments = 184,
  sym_decorator = 185,
  sym__identifier_reference = 186,
  sym_decorator_member_expression = 187,
  sym_decorator_call_expression = 188,
  sym_class_body = 189,
  sym_public_field_definition = 190,
  sym_formal_parameters = 191,
  sym_rest_parameter = 192,
  sym_method_definition = 193,
  sym_pair = 194,
  sym__property_name = 195,
  sym_computed_property_name = 196,
  aux_sym_program_repeat1 = 197,
  aux_sym_export_statement_repeat1 = 198,
  aux_sym_export_clause_repeat1 = 199,
  aux_sym_named_imports_repeat1 = 200,
  aux_sym_variable_declaration_repeat1 = 201,
  aux_sym_switch_body_repeat1 = 202,
  aux_sym_object_repeat1 = 203,
  aux_sym_array_repeat1 = 204,
  aux_sym_jsx_element_repeat1 = 205,
  aux_sym_jsx_opening_element_repeat1 = 206,
  aux_sym_template_string_repeat1 = 207,
  aux_sym_class_body_repeat1 = 208,
  aux_sym_formal_parameters_repeat1 = 209,
  alias_sym_array_pattern = 210,
  alias_sym_call_expression = 211,
  alias_sym_export_specifier = 212,
  alias_sym_identifier = 213,
  alias_sym_import_specifier = 214,
  alias_sym_member_expression = 215,
  alias_sym_object_pattern = 216,
  alias_sym_property_identifier = 217,
  alias_sym_shorthand_property_identifier = 218,
  alias_sym_statement_identifier = 219,
};

static const char *ts_symbol_names[] = {
  [sym__automatic_semicolon] = "_automatic_semicolon",
  [ts_builtin_sym_end] = "END",
  [sym_hash_bang_line] = "hash_bang_line",
  [anon_sym_export] = "export",
  [anon_sym_STAR] = "*",
  [anon_sym_default] = "default",
  [anon_sym_LBRACE] = "{",
  [anon_sym_COMMA] = ",",
  [anon_sym_RBRACE] = "}",
  [anon_sym_as] = "as",
  [anon_sym_import] = "import",
  [anon_sym_from] = "from",
  [anon_sym_var] = "var",
  [anon_sym_let] = "let",
  [anon_sym_const] = "const",
  [anon_sym_if] = "if",
  [anon_sym_else] = "else",
  [anon_sym_switch] = "switch",
  [anon_sym_for] = "for",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN] = ")",
  [anon_sym_in] = "in",
  [anon_sym_of] = "of",
  [anon_sym_while] = "while",
  [anon_sym_do] = "do",
  [anon_sym_try] = "try",
  [anon_sym_with] = "with",
  [anon_sym_break] = "break",
  [anon_sym_continue] = "continue",
  [anon_sym_debugger] = "debugger",
  [anon_sym_return] = "return",
  [anon_sym_throw] = "throw",
  [anon_sym_SEMI] = ";",
  [anon_sym_COLON] = ":",
  [anon_sym_case] = "case",
  [anon_sym_catch] = "catch",
  [anon_sym_finally] = "finally",
  [anon_sym_yield] = "yield",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_LT] = "<",
  [anon_sym_GT] = ">",
  [anon_sym_SLASH] = "/",
  [sym_jsx_text] = "jsx_text",
  [sym_jsx_identifier] = "jsx_identifier",
  [anon_sym_DOT] = ".",
  [anon_sym_EQ] = "=",
  [anon_sym_class] = "class",
  [anon_sym_extends] = "extends",
  [anon_sym_async] = "async",
  [anon_sym_function] = "function",
  [anon_sym_EQ_GT] = "=>",
  [anon_sym_new] = "new",
  [anon_sym_await] = "await",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_GT_GT_GT_EQ] = ">>>=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_DOT_DOT_DOT] = "...",
  [anon_sym_QMARK] = "?",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_GT_GT_GT] = ">>>",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_AMP] = "&",
  [anon_sym_CARET] = "^",
  [anon_sym_PIPE] = "|",
  [anon_sym_PLUS] = "+",
  [anon_sym_DASH] = "-",
  [anon_sym_PERCENT] = "%",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_EQ_EQ_EQ] = "===",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_BANG_EQ_EQ] = "!==",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_instanceof] = "instanceof",
  [anon_sym_BANG] = "!",
  [anon_sym_TILDE] = "~",
  [anon_sym_typeof] = "typeof",
  [anon_sym_void] = "void",
  [anon_sym_delete] = "delete",
  [anon_sym_PLUS_PLUS] = "++",
  [anon_sym_DASH_DASH] = "--",
  [sym_comment] = "comment",
  [sym_string] = "string",
  [anon_sym_BQUOTE] = "`",
  [sym__template_chars] = "_template_chars",
  [anon_sym_DOLLAR_LBRACE] = "${",
  [sym_regex] = "regex",
  [sym_number] = "number",
  [sym_identifier] = "identifier",
  [sym_this] = "this",
  [sym_super] = "super",
  [sym_true] = "true",
  [sym_false] = "false",
  [sym_null] = "null",
  [sym_undefined] = "undefined",
  [anon_sym_AT] = "@",
  [anon_sym_static] = "static",
  [anon_sym_get] = "get",
  [anon_sym_set] = "set",
  [sym_program] = "program",
  [sym_export_statement] = "export_statement",
  [sym_export_clause] = "export_clause",
  [sym__import_export_specifier] = "_import_export_specifier",
  [sym__declaration] = "_declaration",
  [sym_import_statement] = "import_statement",
  [sym_import_clause] = "import_clause",
  [sym__from_clause] = "_from_clause",
  [sym_namespace_import] = "namespace_import",
  [sym_named_imports] = "named_imports",
  [sym_expression_statement] = "expression_statement",
  [sym_variable_declaration] = "variable_declaration",
  [sym_lexical_declaration] = "lexical_declaration",
  [sym_variable_declarator] = "variable_declarator",
  [sym_statement_block] = "statement_block",
  [sym_if_statement] = "if_statement",
  [sym_switch_statement] = "switch_statement",
  [sym_for_statement] = "for_statement",
  [sym_for_in_statement] = "for_in_statement",
  [sym_for_of_statement] = "for_of_statement",
  [sym_while_statement] = "while_statement",
  [sym_do_statement] = "do_statement",
  [sym_try_statement] = "try_statement",
  [sym_with_statement] = "with_statement",
  [sym_break_statement] = "break_statement",
  [sym_continue_statement] = "continue_statement",
  [sym_debugger_statement] = "debugger_statement",
  [sym_return_statement] = "return_statement",
  [sym_throw_statement] = "throw_statement",
  [sym_empty_statement] = "empty_statement",
  [sym_labeled_statement] = "labeled_statement",
  [sym_switch_body] = "switch_body",
  [sym_switch_case] = "switch_case",
  [sym_switch_default] = "switch_default",
  [sym_catch_clause] = "catch_clause",
  [sym_finally_clause] = "finally_clause",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym__expressions] = "_expressions",
  [sym__expression] = "_expression",
  [sym_yield_expression] = "yield_expression",
  [sym_object] = "object",
  [sym_assignment_pattern] = "assignment_pattern",
  [sym_array] = "array",
  [sym_jsx_element] = "jsx_element",
  [sym_jsx_fragment] = "jsx_fragment",
  [sym_jsx_expression] = "jsx_expression",
  [sym_jsx_opening_element] = "jsx_opening_element",
  [sym_nested_identifier] = "nested_identifier",
  [sym_jsx_namespace_name] = "jsx_namespace_name",
  [sym_jsx_closing_element] = "jsx_closing_element",
  [sym_jsx_self_closing_element] = "jsx_self_closing_element",
  [sym_jsx_attribute] = "jsx_attribute",
  [sym_anonymous_class] = "anonymous_class",
  [sym_class] = "class",
  [sym_class_heritage] = "class_heritage",
  [sym_function] = "function",
  [sym_arrow_function] = "arrow_function",
  [sym_generator_function] = "generator_function",
  [sym_call_expression] = "call_expression",
  [sym_new_expression] = "new_expression",
  [sym_await_expression] = "await_expression",
  [sym_member_expression] = "member_expression",
  [sym_subscript_expression] = "subscript_expression",
  [sym_assignment_expression] = "assignment_expression",
  [sym_augmented_assignment_expression] = "augmented_assignment_expression",
  [sym__initializer] = "_initializer",
  [sym_spread_element] = "spread_element",
  [sym_ternary_expression] = "ternary_expression",
  [sym_binary_expression] = "binary_expression",
  [sym_unary_expression] = "unary_expression",
  [sym_update_expression] = "update_expression",
  [sym_sequence_expression] = "sequence_expression",
  [sym_template_string] = "template_string",
  [sym_template_substitution] = "template_substitution",
  [sym_arguments] = "arguments",
  [sym_decorator] = "decorator",
  [sym__identifier_reference] = "_identifier_reference",
  [sym_decorator_member_expression] = "decorator_member_expression",
  [sym_decorator_call_expression] = "decorator_call_expression",
  [sym_class_body] = "class_body",
  [sym_public_field_definition] = "public_field_definition",
  [sym_formal_parameters] = "formal_parameters",
  [sym_rest_parameter] = "rest_parameter",
  [sym_method_definition] = "method_definition",
  [sym_pair] = "pair",
  [sym__property_name] = "_property_name",
  [sym_computed_property_name] = "computed_property_name",
  [aux_sym_program_repeat1] = "program_repeat1",
  [aux_sym_export_statement_repeat1] = "export_statement_repeat1",
  [aux_sym_export_clause_repeat1] = "export_clause_repeat1",
  [aux_sym_named_imports_repeat1] = "named_imports_repeat1",
  [aux_sym_variable_declaration_repeat1] = "variable_declaration_repeat1",
  [aux_sym_switch_body_repeat1] = "switch_body_repeat1",
  [aux_sym_object_repeat1] = "object_repeat1",
  [aux_sym_array_repeat1] = "array_repeat1",
  [aux_sym_jsx_element_repeat1] = "jsx_element_repeat1",
  [aux_sym_jsx_opening_element_repeat1] = "jsx_opening_element_repeat1",
  [aux_sym_template_string_repeat1] = "template_string_repeat1",
  [aux_sym_class_body_repeat1] = "class_body_repeat1",
  [aux_sym_formal_parameters_repeat1] = "formal_parameters_repeat1",
  [alias_sym_array_pattern] = "array_pattern",
  [alias_sym_call_expression] = "call_expression",
  [alias_sym_export_specifier] = "export_specifier",
  [alias_sym_identifier] = "identifier",
  [alias_sym_import_specifier] = "import_specifier",
  [alias_sym_member_expression] = "member_expression",
  [alias_sym_object_pattern] = "object_pattern",
  [alias_sym_property_identifier] = "property_identifier",
  [alias_sym_shorthand_property_identifier] = "shorthand_property_identifier",
  [alias_sym_statement_identifier] = "statement_identifier",
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [sym__automatic_semicolon] = {
    .visible = false,
    .named = true,
  },
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_hash_bang_line] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_export] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_default] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_as] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_import] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_from] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_var] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_let] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_const] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_switch] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_in] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_of] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_do] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_try] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_with] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_debugger] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_throw] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_case] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_catch] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_finally] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_yield] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [sym_jsx_text] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_class] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_extends] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_async] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_function] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_new] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_await] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_instanceof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_typeof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_void] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_delete] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_DASH] = {
    .visible = true,
    .named = false,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_string] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_BQUOTE] = {
    .visible = true,
    .named = false,
  },
  [sym__template_chars] = {
    .visible = false,
    .named = true,
  },
  [anon_sym_DOLLAR_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [sym_regex] = {
    .visible = true,
    .named = true,
  },
  [sym_number] = {
    .visible = true,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_this] = {
    .visible = true,
    .named = true,
  },
  [sym_super] = {
    .visible = true,
    .named = true,
  },
  [sym_true] = {
    .visible = true,
    .named = true,
  },
  [sym_false] = {
    .visible = true,
    .named = true,
  },
  [sym_null] = {
    .visible = true,
    .named = true,
  },
  [sym_undefined] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_static] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_get] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_set] = {
    .visible = true,
    .named = false,
  },
  [sym_program] = {
    .visible = true,
    .named = true,
  },
  [sym_export_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_export_clause] = {
    .visible = true,
    .named = true,
  },
  [sym__import_export_specifier] = {
    .visible = false,
    .named = true,
  },
  [sym__declaration] = {
    .visible = false,
    .named = true,
  },
  [sym_import_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_import_clause] = {
    .visible = true,
    .named = true,
  },
  [sym__from_clause] = {
    .visible = false,
    .named = true,
  },
  [sym_namespace_import] = {
    .visible = true,
    .named = true,
  },
  [sym_named_imports] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_lexical_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_variable_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_statement_block] = {
    .visible = true,
    .named = true,
  },
  [sym_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_in_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_of_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_while_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_do_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_try_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_with_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_break_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_debugger_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_return_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_throw_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_empty_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_labeled_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_body] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_case] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_default] = {
    .visible = true,
    .named = true,
  },
  [sym_catch_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_finally_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym__expressions] = {
    .visible = false,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
  },
  [sym_yield_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_object] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_pattern] = {
    .visible = true,
    .named = true,
  },
  [sym_array] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_element] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_fragment] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_opening_element] = {
    .visible = true,
    .named = true,
  },
  [sym_nested_identifier] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_namespace_name] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_closing_element] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_self_closing_element] = {
    .visible = true,
    .named = true,
  },
  [sym_jsx_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_anonymous_class] = {
    .visible = true,
    .named = true,
  },
  [sym_class] = {
    .visible = true,
    .named = true,
  },
  [sym_class_heritage] = {
    .visible = true,
    .named = true,
  },
  [sym_function] = {
    .visible = true,
    .named = true,
  },
  [sym_arrow_function] = {
    .visible = true,
    .named = true,
  },
  [sym_generator_function] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_new_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_await_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_member_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_subscript_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_augmented_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym__initializer] = {
    .visible = false,
    .named = true,
  },
  [sym_spread_element] = {
    .visible = true,
    .named = true,
  },
  [sym_ternary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_update_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_sequence_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_template_string] = {
    .visible = true,
    .named = true,
  },
  [sym_template_substitution] = {
    .visible = true,
    .named = true,
  },
  [sym_arguments] = {
    .visible = true,
    .named = true,
  },
  [sym_decorator] = {
    .visible = true,
    .named = true,
  },
  [sym__identifier_reference] = {
    .visible = false,
    .named = true,
  },
  [sym_decorator_member_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_decorator_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_class_body] = {
    .visible = true,
    .named = true,
  },
  [sym_public_field_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_formal_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_rest_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_method_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_pair] = {
    .visible = true,
    .named = true,
  },
  [sym__property_name] = {
    .visible = false,
    .named = true,
  },
  [sym_computed_property_name] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_program_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_export_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_export_clause_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_named_imports_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_variable_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_switch_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_object_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_array_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_jsx_element_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_jsx_opening_element_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_template_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_class_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_formal_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_array_pattern] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_export_specifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_import_specifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_member_expression] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_object_pattern] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_property_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_shorthand_property_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_statement_identifier] = {
    .visible = true,
    .named = true,
  },
};

static TSSymbol ts_alias_sequences[26][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [1] = {
    [0] = alias_sym_identifier,
  },
  [2] = {
    [0] = alias_sym_property_identifier,
  },
  [3] = {
    [0] = alias_sym_object_pattern,
  },
  [4] = {
    [0] = alias_sym_array_pattern,
  },
  [5] = {
    [1] = alias_sym_member_expression,
  },
  [6] = {
    [1] = alias_sym_call_expression,
  },
  [7] = {
    [1] = alias_sym_shorthand_property_identifier,
  },
  [8] = {
    [0] = alias_sym_shorthand_property_identifier,
  },
  [9] = {
    [1] = alias_sym_identifier,
  },
  [10] = {
    [1] = alias_sym_object_pattern,
  },
  [11] = {
    [1] = alias_sym_array_pattern,
  },
  [12] = {
    [1] = alias_sym_statement_identifier,
  },
  [13] = {
    [0] = alias_sym_identifier,
    [2] = alias_sym_property_identifier,
  },
  [14] = {
    [0] = alias_sym_statement_identifier,
  },
  [15] = {
    [2] = alias_sym_property_identifier,
  },
  [16] = {
    [0] = alias_sym_member_expression,
  },
  [17] = {
    [1] = alias_sym_export_specifier,
  },
  [18] = {
    [1] = alias_sym_import_specifier,
  },
  [19] = {
    [0] = alias_sym_identifier,
    [2] = alias_sym_identifier,
  },
  [20] = {
    [2] = alias_sym_identifier,
  },
  [21] = {
    [0] = alias_sym_member_expression,
    [2] = alias_sym_property_identifier,
  },
  [22] = {
    [2] = alias_sym_object_pattern,
  },
  [23] = {
    [2] = alias_sym_array_pattern,
  },
  [24] = {
    [3] = alias_sym_object_pattern,
  },
  [25] = {
    [3] = alias_sym_array_pattern,
  },
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  switch (state) {
    case 0:
      if (lookahead == 0)
        ADVANCE(1);
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '#')
        ADVANCE(8);
      if (lookahead == '$')
        ADVANCE(10);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(31);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == ']')
        ADVANCE(71);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(0);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if ((lookahead < '[' || lookahead > '^'))
        ADVANCE(12);
      END_STATE();
    case 1:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 2:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=')
        ADVANCE(3);
      END_STATE();
    case 3:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      if (lookahead == '=')
        ADVANCE(4);
      END_STATE();
    case 4:
      ACCEPT_TOKEN(anon_sym_BANG_EQ_EQ);
      END_STATE();
    case 5:
      if (lookahead == '\"')
        ADVANCE(6);
      if (lookahead == '\\')
        ADVANCE(7);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(5);
      END_STATE();
    case 6:
      ACCEPT_TOKEN(sym_string);
      END_STATE();
    case 7:
      if (lookahead == '\n')
        ADVANCE(5);
      if (lookahead != 0)
        ADVANCE(5);
      END_STATE();
    case 8:
      if (lookahead == '!')
        ADVANCE(9);
      END_STATE();
    case 9:
      ACCEPT_TOKEN(sym_hash_bang_line);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(9);
      END_STATE();
    case 10:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '{')
        ADVANCE(11);
      if (lookahead != 0 &&
          lookahead != '\t' &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '/') &&
          (lookahead < ':' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~') &&
          lookahead != 160 &&
          lookahead != 8203 &&
          lookahead != 8288 &&
          lookahead != 65279)
        ADVANCE(12);
      END_STATE();
    case 11:
      ACCEPT_TOKEN(anon_sym_DOLLAR_LBRACE);
      END_STATE();
    case 12:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead != 0 &&
          lookahead != '\t' &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '/') &&
          (lookahead < ':' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~') &&
          lookahead != 160 &&
          lookahead != 8203 &&
          lookahead != 8288 &&
          lookahead != 65279)
        ADVANCE(12);
      END_STATE();
    case 13:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=')
        ADVANCE(14);
      END_STATE();
    case 14:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 15:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&')
        ADVANCE(16);
      if (lookahead == '=')
        ADVANCE(17);
      END_STATE();
    case 16:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 17:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 18:
      if (lookahead == '\'')
        ADVANCE(6);
      if (lookahead == '\\')
        ADVANCE(19);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(18);
      END_STATE();
    case 19:
      if (lookahead == '\n')
        ADVANCE(18);
      if (lookahead != 0)
        ADVANCE(18);
      END_STATE();
    case 20:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 21:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 22:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '=')
        ADVANCE(23);
      END_STATE();
    case 23:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 24:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+')
        ADVANCE(25);
      if (lookahead == '=')
        ADVANCE(26);
      END_STATE();
    case 25:
      ACCEPT_TOKEN(anon_sym_PLUS_PLUS);
      END_STATE();
    case 26:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 27:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 28:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-')
        ADVANCE(29);
      if (lookahead == '=')
        ADVANCE(30);
      END_STATE();
    case 29:
      ACCEPT_TOKEN(anon_sym_DASH_DASH);
      END_STATE();
    case 30:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 31:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.')
        ADVANCE(32);
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(34);
      END_STATE();
    case 32:
      if (lookahead == '.')
        ADVANCE(33);
      END_STATE();
    case 33:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_DOT);
      END_STATE();
    case 34:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == 'E')
        ADVANCE(35);
      if (lookahead == 'e')
        ADVANCE(35);
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(34);
      END_STATE();
    case 35:
      if (lookahead == '+')
        ADVANCE(36);
      if (lookahead == '-')
        ADVANCE(36);
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(37);
      END_STATE();
    case 36:
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(37);
      END_STATE();
    case 37:
      ACCEPT_TOKEN(sym_number);
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(37);
      END_STATE();
    case 38:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*')
        ADVANCE(39);
      if (lookahead == '/')
        ADVANCE(42);
      if (lookahead == '=')
        ADVANCE(43);
      END_STATE();
    case 39:
      if (lookahead == '*')
        ADVANCE(40);
      if (lookahead != 0)
        ADVANCE(39);
      END_STATE();
    case 40:
      if (lookahead == '*')
        ADVANCE(40);
      if (lookahead == '/')
        ADVANCE(41);
      if (lookahead != 0)
        ADVANCE(39);
      END_STATE();
    case 41:
      ACCEPT_TOKEN(sym_comment);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(42);
      END_STATE();
    case 43:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == '.')
        ADVANCE(34);
      if (lookahead == 'B')
        ADVANCE(45);
      if (lookahead == 'E')
        ADVANCE(35);
      if (lookahead == 'O')
        ADVANCE(47);
      if (lookahead == 'X')
        ADVANCE(49);
      if (lookahead == 'b')
        ADVANCE(45);
      if (lookahead == 'e')
        ADVANCE(35);
      if (lookahead == 'o')
        ADVANCE(47);
      if (lookahead == 'x')
        ADVANCE(49);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      END_STATE();
    case 45:
      if (lookahead == '0' ||
          lookahead == '1')
        ADVANCE(46);
      END_STATE();
    case 46:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == '0' ||
          lookahead == '1')
        ADVANCE(46);
      END_STATE();
    case 47:
      if (('0' <= lookahead && lookahead <= '7'))
        ADVANCE(48);
      END_STATE();
    case 48:
      ACCEPT_TOKEN(sym_number);
      if (('0' <= lookahead && lookahead <= '7'))
        ADVANCE(48);
      END_STATE();
    case 49:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f'))
        ADVANCE(50);
      END_STATE();
    case 50:
      ACCEPT_TOKEN(sym_number);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f'))
        ADVANCE(50);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == '.')
        ADVANCE(34);
      if (lookahead == 'E')
        ADVANCE(35);
      if (lookahead == 'e')
        ADVANCE(35);
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 53:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 54:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<')
        ADVANCE(55);
      if (lookahead == '=')
        ADVANCE(57);
      END_STATE();
    case 55:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=')
        ADVANCE(56);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 57:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 58:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=')
        ADVANCE(59);
      if (lookahead == '>')
        ADVANCE(61);
      END_STATE();
    case 59:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      if (lookahead == '=')
        ADVANCE(60);
      END_STATE();
    case 60:
      ACCEPT_TOKEN(anon_sym_EQ_EQ_EQ);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_EQ_GT);
      END_STATE();
    case 62:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=')
        ADVANCE(63);
      if (lookahead == '>')
        ADVANCE(64);
      END_STATE();
    case 63:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 64:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=')
        ADVANCE(65);
      if (lookahead == '>')
        ADVANCE(66);
      END_STATE();
    case 65:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_GT_GT_GT);
      if (lookahead == '=')
        ADVANCE(67);
      END_STATE();
    case 67:
      ACCEPT_TOKEN(anon_sym_GT_GT_GT_EQ);
      END_STATE();
    case 68:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 69:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 70:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 72:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=')
        ADVANCE(73);
      END_STATE();
    case 73:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 74:
      ACCEPT_TOKEN(anon_sym_BQUOTE);
      END_STATE();
    case 75:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 76:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=')
        ADVANCE(77);
      if (lookahead == '|')
        ADVANCE(78);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 78:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 80:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 81:
      if (lookahead == 0)
        ADVANCE(1);
      if (lookahead == '!')
        ADVANCE(82);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '#')
        ADVANCE(8);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(85);
      if (lookahead == '/')
        ADVANCE(86);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(109);
      if (lookahead == '=')
        ADVANCE(110);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(81);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if ((lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 83:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+')
        ADVANCE(25);
      END_STATE();
    case 84:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-')
        ADVANCE(29);
      END_STATE();
    case 85:
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(34);
      END_STATE();
    case 86:
      if (lookahead == '*')
        ADVANCE(87);
      if (lookahead == '/')
        ADVANCE(99);
      if (lookahead == '[')
        ADVANCE(100);
      if (lookahead == '\\')
        ADVANCE(106);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(104);
      END_STATE();
    case 87:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(88);
      if (lookahead == '/')
        ADVANCE(97);
      if (lookahead == '[')
        ADVANCE(89);
      if (lookahead == '\\')
        ADVANCE(98);
      if (lookahead != 0)
        ADVANCE(87);
      END_STATE();
    case 88:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(88);
      if (lookahead == '/')
        ADVANCE(41);
      if (lookahead == '[')
        ADVANCE(89);
      if (lookahead == '\\')
        ADVANCE(98);
      if (lookahead != 0)
        ADVANCE(87);
      END_STATE();
    case 89:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(90);
      if (lookahead == '\\')
        ADVANCE(91);
      if (lookahead == ']')
        ADVANCE(87);
      if (lookahead != 0)
        ADVANCE(89);
      END_STATE();
    case 90:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(90);
      if (lookahead == '/')
        ADVANCE(41);
      if (lookahead == '\\')
        ADVANCE(91);
      if (lookahead == ']')
        ADVANCE(87);
      if (lookahead != 0)
        ADVANCE(89);
      END_STATE();
    case 91:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(90);
      if (lookahead == '\\')
        ADVANCE(91);
      if (lookahead == ']')
        ADVANCE(92);
      if (lookahead != 0)
        ADVANCE(89);
      END_STATE();
    case 92:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(93);
      if (lookahead == '/')
        ADVANCE(96);
      if (lookahead == '[')
        ADVANCE(89);
      if (lookahead == '\\')
        ADVANCE(94);
      if (lookahead == ']')
        ADVANCE(87);
      if (lookahead != 0)
        ADVANCE(92);
      END_STATE();
    case 93:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(93);
      if (lookahead == '/')
        ADVANCE(41);
      if (lookahead == '[')
        ADVANCE(89);
      if (lookahead == '\\')
        ADVANCE(94);
      if (lookahead == ']')
        ADVANCE(87);
      if (lookahead != 0)
        ADVANCE(92);
      END_STATE();
    case 94:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(93);
      if (lookahead == '\\')
        ADVANCE(95);
      if (lookahead == ']')
        ADVANCE(92);
      if (lookahead != 0)
        ADVANCE(92);
      END_STATE();
    case 95:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(93);
      if (lookahead == '/')
        ADVANCE(96);
      if (lookahead == '[')
        ADVANCE(89);
      if (lookahead == '\\')
        ADVANCE(94);
      if (lookahead == ']')
        ADVANCE(92);
      if (lookahead != 0)
        ADVANCE(92);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(sym_regex);
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(90);
      if (lookahead == '\\')
        ADVANCE(91);
      if (lookahead == ']')
        ADVANCE(87);
      if (('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(96);
      if (lookahead != 0)
        ADVANCE(89);
      END_STATE();
    case 97:
      ACCEPT_TOKEN(sym_regex);
      if (lookahead == '*')
        ADVANCE(40);
      if (('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(97);
      if (lookahead != 0)
        ADVANCE(39);
      END_STATE();
    case 98:
      if (lookahead == '\n')
        ADVANCE(39);
      if (lookahead == '*')
        ADVANCE(88);
      if (lookahead != 0)
        ADVANCE(87);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(sym_comment);
      if (('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(99);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(42);
      END_STATE();
    case 100:
      if (lookahead == '\\')
        ADVANCE(101);
      if (lookahead == ']')
        ADVANCE(104);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(100);
      END_STATE();
    case 101:
      if (lookahead == '\\')
        ADVANCE(101);
      if (lookahead == ']')
        ADVANCE(102);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(100);
      END_STATE();
    case 102:
      if (lookahead == '/')
        ADVANCE(103);
      if (lookahead == '[')
        ADVANCE(100);
      if (lookahead == '\\')
        ADVANCE(107);
      if (lookahead == ']')
        ADVANCE(104);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(102);
      END_STATE();
    case 103:
      ACCEPT_TOKEN(sym_regex);
      if (lookahead == '\\')
        ADVANCE(101);
      if (lookahead == ']')
        ADVANCE(104);
      if (('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(103);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(100);
      END_STATE();
    case 104:
      if (lookahead == '/')
        ADVANCE(105);
      if (lookahead == '[')
        ADVANCE(100);
      if (lookahead == '\\')
        ADVANCE(106);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(104);
      END_STATE();
    case 105:
      ACCEPT_TOKEN(sym_regex);
      if (('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(105);
      END_STATE();
    case 106:
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(104);
      END_STATE();
    case 107:
      if (lookahead == '\\')
        ADVANCE(108);
      if (lookahead == ']')
        ADVANCE(102);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(102);
      END_STATE();
    case 108:
      if (lookahead == '/')
        ADVANCE(103);
      if (lookahead == '[')
        ADVANCE(100);
      if (lookahead == '\\')
        ADVANCE(107);
      if (lookahead == ']')
        ADVANCE(102);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(102);
      END_STATE();
    case 109:
      ACCEPT_TOKEN(anon_sym_LT);
      END_STATE();
    case 110:
      if (lookahead == '>')
        ADVANCE(61);
      END_STATE();
    case 111:
      if (lookahead == 0)
        ADVANCE(1);
      if (lookahead == '!')
        ADVANCE(82);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(31);
      if (lookahead == '/')
        ADVANCE(86);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(109);
      if (lookahead == '=')
        ADVANCE(113);
      if (lookahead == '>')
        ADVANCE(114);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == ']')
        ADVANCE(71);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(111);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if ((lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 112:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '>')
        ADVANCE(61);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_GT);
      END_STATE();
    case 115:
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(115);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 116:
      if (lookahead == '*')
        ADVANCE(39);
      if (lookahead == '/')
        ADVANCE(42);
      END_STATE();
    case 117:
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(117);
      END_STATE();
    case 118:
      if (lookahead == '!')
        ADVANCE(82);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(85);
      if (lookahead == '/')
        ADVANCE(86);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(109);
      if (lookahead == '=')
        ADVANCE(110);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(118);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 119:
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(123);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(126);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(119);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&')
        ADVANCE(16);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (('0' <= lookahead && lookahead <= '9'))
        ADVANCE(34);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*')
        ADVANCE(87);
      if (lookahead == '/')
        ADVANCE(99);
      if (lookahead == '[')
        ADVANCE(100);
      if (lookahead == '\\')
        ADVANCE(106);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(104);
      END_STATE();
    case 124:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<')
        ADVANCE(125);
      if (lookahead == '=')
        ADVANCE(57);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      END_STATE();
    case 126:
      if (lookahead == '=')
        ADVANCE(59);
      if (lookahead == '>')
        ADVANCE(61);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=')
        ADVANCE(63);
      if (lookahead == '>')
        ADVANCE(128);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '>')
        ADVANCE(129);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_GT_GT_GT);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '|')
        ADVANCE(78);
      END_STATE();
    case 132:
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == '=')
        ADVANCE(135);
      if (lookahead == '>')
        ADVANCE(114);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(132);
      if (lookahead == '$' ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(136);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*')
        ADVANCE(39);
      if (lookahead == '/')
        ADVANCE(42);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '-')
        ADVANCE(137);
      if (lookahead == '$' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(136);
      if (lookahead != 0 &&
          lookahead != '\t' &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          (lookahead < ' ' || lookahead > '~') &&
          lookahead != 160 &&
          lookahead != 8203 &&
          lookahead != 8288 &&
          lookahead != 65279)
        ADVANCE(12);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(sym_jsx_identifier);
      if (lookahead == '$' ||
          lookahead == '-' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z'))
        ADVANCE(137);
      END_STATE();
    case 138:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(138);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 139:
      if (lookahead == '=')
        ADVANCE(3);
      END_STATE();
    case 140:
      if (lookahead == '$')
        ADVANCE(141);
      if (lookahead == '/')
        ADVANCE(146);
      if (lookahead == '\\')
        ADVANCE(144);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        ADVANCE(156);
      if (lookahead != 0)
        ADVANCE(142);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '{')
        ADVANCE(11);
      if (lookahead != 0 &&
          lookahead != '$' &&
          lookahead != '`')
        ADVANCE(142);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(143);
      if (lookahead == '\\')
        ADVANCE(144);
      if (lookahead != 0 &&
          lookahead != '`')
        ADVANCE(142);
      END_STATE();
    case 143:
      if (lookahead != 0 &&
          lookahead != '$' &&
          lookahead != '`' &&
          lookahead != '{')
        ADVANCE(142);
      END_STATE();
    case 144:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(145);
      if (lookahead == '\\')
        ADVANCE(144);
      if (lookahead == '`')
        ADVANCE(142);
      if (lookahead != 0)
        ADVANCE(142);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(143);
      if (lookahead == '\\')
        ADVANCE(144);
      if (lookahead == '{')
        ADVANCE(142);
      if (lookahead != 0 &&
          lookahead != '`')
        ADVANCE(142);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(143);
      if (lookahead == '*')
        ADVANCE(147);
      if (lookahead == '/')
        ADVANCE(152);
      if (lookahead == '\\')
        ADVANCE(144);
      if (lookahead != 0 &&
          lookahead != '`')
        ADVANCE(142);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(148);
      if (lookahead == '*')
        ADVANCE(149);
      if (lookahead == '\\')
        ADVANCE(150);
      if (lookahead == '`')
        ADVANCE(39);
      if (lookahead != 0)
        ADVANCE(147);
      END_STATE();
    case 148:
      if (lookahead == '*')
        ADVANCE(149);
      if (lookahead == '$' ||
          lookahead == '`' ||
          lookahead == '{')
        ADVANCE(39);
      if (lookahead != 0)
        ADVANCE(147);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(148);
      if (lookahead == '*')
        ADVANCE(149);
      if (lookahead == '/')
        ADVANCE(41);
      if (lookahead == '\\')
        ADVANCE(150);
      if (lookahead == '`')
        ADVANCE(39);
      if (lookahead != 0)
        ADVANCE(147);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(151);
      if (lookahead == '*')
        ADVANCE(149);
      if (lookahead == '\\')
        ADVANCE(150);
      if (lookahead == '`')
        ADVANCE(147);
      if (lookahead != 0)
        ADVANCE(147);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(148);
      if (lookahead == '*')
        ADVANCE(149);
      if (lookahead == '\\')
        ADVANCE(150);
      if (lookahead == '`')
        ADVANCE(39);
      if (lookahead == '{')
        ADVANCE(147);
      if (lookahead != 0)
        ADVANCE(147);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '$')
        ADVANCE(153);
      if (lookahead == '\\')
        ADVANCE(154);
      if (lookahead == '`')
        ADVANCE(42);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(152);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '$' ||
          lookahead == '`' ||
          lookahead == '{')
        ADVANCE(42);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(152);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '$')
        ADVANCE(155);
      if (lookahead == '\\')
        ADVANCE(154);
      if (lookahead == '`')
        ADVANCE(152);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(152);
      END_STATE();
    case 155:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '$')
        ADVANCE(153);
      if (lookahead == '\\')
        ADVANCE(154);
      if (lookahead == '`')
        ADVANCE(42);
      if (lookahead == '{')
        ADVANCE(152);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(152);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(sym__template_chars);
      if (lookahead == '$')
        ADVANCE(141);
      if (lookahead == '/')
        ADVANCE(146);
      if (lookahead == '\\')
        ADVANCE(144);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        ADVANCE(156);
      if (lookahead != 0 &&
          lookahead != '`')
        ADVANCE(142);
      END_STATE();
    case 157:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(157);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 158:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(158);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=')
        ADVANCE(59);
      END_STATE();
    case 160:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(161);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(160);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 161:
      if (lookahead == '=')
        ADVANCE(59);
      END_STATE();
    case 162:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(162);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 163:
      if (lookahead == '/')
        ADVANCE(164);
      if (lookahead == '<')
        ADVANCE(109);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        ADVANCE(169);
      if (lookahead != 0 &&
          lookahead != '>' &&
          lookahead != '}')
        ADVANCE(168);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(sym_jsx_text);
      if (lookahead == '*')
        ADVANCE(165);
      if (lookahead == '/')
        ADVANCE(167);
      if (lookahead != 0 &&
          lookahead != '<' &&
          lookahead != '>' &&
          lookahead != '{' &&
          lookahead != '}')
        ADVANCE(168);
      END_STATE();
    case 165:
      ACCEPT_TOKEN(sym_jsx_text);
      if (lookahead == '*')
        ADVANCE(166);
      if (lookahead == '<' ||
          lookahead == '>' ||
          lookahead == '{' ||
          lookahead == '}')
        ADVANCE(39);
      if (lookahead != 0)
        ADVANCE(165);
      END_STATE();
    case 166:
      ACCEPT_TOKEN(sym_jsx_text);
      if (lookahead == '*')
        ADVANCE(166);
      if (lookahead == '/')
        ADVANCE(41);
      if (lookahead == '<' ||
          lookahead == '>' ||
          lookahead == '{' ||
          lookahead == '}')
        ADVANCE(39);
      if (lookahead != 0)
        ADVANCE(165);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '<' ||
          lookahead == '>' ||
          lookahead == '{' ||
          lookahead == '}')
        ADVANCE(42);
      if (lookahead != 0 &&
          lookahead != '\n')
        ADVANCE(167);
      END_STATE();
    case 168:
      ACCEPT_TOKEN(sym_jsx_text);
      if (lookahead != 0 &&
          lookahead != '<' &&
          lookahead != '>' &&
          lookahead != '{' &&
          lookahead != '}')
        ADVANCE(168);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(sym_jsx_text);
      if (lookahead == '/')
        ADVANCE(164);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        ADVANCE(169);
      if (lookahead != 0 &&
          lookahead != '<' &&
          lookahead != '>' &&
          lookahead != '{' &&
          lookahead != '}')
        ADVANCE(168);
      END_STATE();
    case 170:
      if (lookahead == 0)
        ADVANCE(1);
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(123);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(126);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(170);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if ((lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^'))
        ADVANCE(12);
      END_STATE();
    case 171:
      if (lookahead == 0)
        ADVANCE(1);
      if (lookahead == '!')
        ADVANCE(82);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(85);
      if (lookahead == '/')
        ADVANCE(86);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(109);
      if (lookahead == '=')
        ADVANCE(110);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(171);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if ((lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 172:
      if (lookahead == 0)
        ADVANCE(1);
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(123);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(172);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if ((lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^'))
        ADVANCE(12);
      END_STATE();
    case 173:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(173);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 174:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(161);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(174);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 175:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(175);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 176:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(176);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 177:
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '=')
        ADVANCE(135);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(177);
      END_STATE();
    case 178:
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(178);
      END_STATE();
    case 179:
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(123);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(126);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == ']')
        ADVANCE(71);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(179);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 180:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == ']')
        ADVANCE(71);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(180);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 181:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == ']')
        ADVANCE(71);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(181);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 182:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(161);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(182);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 183:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(183);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 184:
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(123);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(126);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(184);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 185:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(185);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 186:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(186);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 187:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(161);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(187);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 188:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(188);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 189:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(161);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(189);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 190:
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '=')
        ADVANCE(135);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(190);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 191:
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '.')
        ADVANCE(85);
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '=')
        ADVANCE(135);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(191);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 192:
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '=')
        ADVANCE(135);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(192);
      END_STATE();
    case 193:
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '.')
        ADVANCE(85);
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '=')
        ADVANCE(135);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(193);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 194:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(194);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 195:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ':')
        ADVANCE(52);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(195);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 196:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(196);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 197:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(197);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 198:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '}')
        ADVANCE(79);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(198);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 199:
      if (lookahead == '!')
        ADVANCE(2);
      if (lookahead == '\"')
        ADVANCE(5);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '\'')
        ADVANCE(18);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(122);
      if (lookahead == '/')
        ADVANCE(123);
      if (lookahead == '0')
        ADVANCE(44);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(126);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '@')
        ADVANCE(69);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '{')
        ADVANCE(75);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '~')
        ADVANCE(80);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(199);
      if (('1' <= lookahead && lookahead <= '9'))
        ADVANCE(51);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 200:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(58);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(200);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 201:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(13);
      if (lookahead == '&')
        ADVANCE(15);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(22);
      if (lookahead == '+')
        ADVANCE(24);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(28);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(38);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(54);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(62);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(72);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(76);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(201);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 202:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(161);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(202);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 203:
      if (lookahead == '!')
        ADVANCE(139);
      if (lookahead == '%')
        ADVANCE(120);
      if (lookahead == '&')
        ADVANCE(121);
      if (lookahead == '(')
        ADVANCE(20);
      if (lookahead == ')')
        ADVANCE(21);
      if (lookahead == '*')
        ADVANCE(112);
      if (lookahead == '+')
        ADVANCE(83);
      if (lookahead == ',')
        ADVANCE(27);
      if (lookahead == '-')
        ADVANCE(84);
      if (lookahead == '.')
        ADVANCE(133);
      if (lookahead == '/')
        ADVANCE(134);
      if (lookahead == ';')
        ADVANCE(53);
      if (lookahead == '<')
        ADVANCE(124);
      if (lookahead == '=')
        ADVANCE(159);
      if (lookahead == '>')
        ADVANCE(127);
      if (lookahead == '?')
        ADVANCE(68);
      if (lookahead == '[')
        ADVANCE(70);
      if (lookahead == '^')
        ADVANCE(130);
      if (lookahead == '`')
        ADVANCE(74);
      if (lookahead == '|')
        ADVANCE(131);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(203);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    case 204:
      if (lookahead == '/')
        ADVANCE(116);
      if (lookahead == '\t' ||
          lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == ' ' ||
          lookahead == 160 ||
          lookahead == 8203 ||
          lookahead == 8288 ||
          lookahead == 65279)
        SKIP(204);
      if (lookahead != 0 &&
          (lookahead < ' ' || lookahead > '#') &&
          (lookahead < '%' || lookahead > '@') &&
          (lookahead < '[' || lookahead > '^') &&
          lookahead != '`' &&
          (lookahead < '{' || lookahead > '~'))
        ADVANCE(12);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  switch (state) {
    case 0:
      if (lookahead == 'a')
        ADVANCE(1);
      if (lookahead == 'b')
        ADVANCE(10);
      if (lookahead == 'c')
        ADVANCE(15);
      if (lookahead == 'd')
        ADVANCE(35);
      if (lookahead == 'e')
        ADVANCE(53);
      if (lookahead == 'f')
        ADVANCE(67);
      if (lookahead == 'g')
        ADVANCE(90);
      if (lookahead == 'i')
        ADVANCE(93);
      if (lookahead == 'l')
        ADVANCE(109);
      if (lookahead == 'n')
        ADVANCE(112);
      if (lookahead == 'o')
        ADVANCE(118);
      if (lookahead == 'r')
        ADVANCE(120);
      if (lookahead == 's')
        ADVANCE(126);
      if (lookahead == 't')
        ADVANCE(143);
      if (lookahead == 'u')
        ADVANCE(159);
      if (lookahead == 'v')
        ADVANCE(168);
      if (lookahead == 'w')
        ADVANCE(174);
      if (lookahead == 'y')
        ADVANCE(182);
      END_STATE();
    case 1:
      if (lookahead == 's')
        ADVANCE(2);
      if (lookahead == 'w')
        ADVANCE(6);
      END_STATE();
    case 2:
      ACCEPT_TOKEN(anon_sym_as);
      if (lookahead == 'y')
        ADVANCE(3);
      END_STATE();
    case 3:
      if (lookahead == 'n')
        ADVANCE(4);
      END_STATE();
    case 4:
      if (lookahead == 'c')
        ADVANCE(5);
      END_STATE();
    case 5:
      ACCEPT_TOKEN(anon_sym_async);
      END_STATE();
    case 6:
      if (lookahead == 'a')
        ADVANCE(7);
      END_STATE();
    case 7:
      if (lookahead == 'i')
        ADVANCE(8);
      END_STATE();
    case 8:
      if (lookahead == 't')
        ADVANCE(9);
      END_STATE();
    case 9:
      ACCEPT_TOKEN(anon_sym_await);
      END_STATE();
    case 10:
      if (lookahead == 'r')
        ADVANCE(11);
      END_STATE();
    case 11:
      if (lookahead == 'e')
        ADVANCE(12);
      END_STATE();
    case 12:
      if (lookahead == 'a')
        ADVANCE(13);
      END_STATE();
    case 13:
      if (lookahead == 'k')
        ADVANCE(14);
      END_STATE();
    case 14:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 15:
      if (lookahead == 'a')
        ADVANCE(16);
      if (lookahead == 'l')
        ADVANCE(22);
      if (lookahead == 'o')
        ADVANCE(26);
      END_STATE();
    case 16:
      if (lookahead == 's')
        ADVANCE(17);
      if (lookahead == 't')
        ADVANCE(19);
      END_STATE();
    case 17:
      if (lookahead == 'e')
        ADVANCE(18);
      END_STATE();
    case 18:
      ACCEPT_TOKEN(anon_sym_case);
      END_STATE();
    case 19:
      if (lookahead == 'c')
        ADVANCE(20);
      END_STATE();
    case 20:
      if (lookahead == 'h')
        ADVANCE(21);
      END_STATE();
    case 21:
      ACCEPT_TOKEN(anon_sym_catch);
      END_STATE();
    case 22:
      if (lookahead == 'a')
        ADVANCE(23);
      END_STATE();
    case 23:
      if (lookahead == 's')
        ADVANCE(24);
      END_STATE();
    case 24:
      if (lookahead == 's')
        ADVANCE(25);
      END_STATE();
    case 25:
      ACCEPT_TOKEN(anon_sym_class);
      END_STATE();
    case 26:
      if (lookahead == 'n')
        ADVANCE(27);
      END_STATE();
    case 27:
      if (lookahead == 's')
        ADVANCE(28);
      if (lookahead == 't')
        ADVANCE(30);
      END_STATE();
    case 28:
      if (lookahead == 't')
        ADVANCE(29);
      END_STATE();
    case 29:
      ACCEPT_TOKEN(anon_sym_const);
      END_STATE();
    case 30:
      if (lookahead == 'i')
        ADVANCE(31);
      END_STATE();
    case 31:
      if (lookahead == 'n')
        ADVANCE(32);
      END_STATE();
    case 32:
      if (lookahead == 'u')
        ADVANCE(33);
      END_STATE();
    case 33:
      if (lookahead == 'e')
        ADVANCE(34);
      END_STATE();
    case 34:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 35:
      if (lookahead == 'e')
        ADVANCE(36);
      if (lookahead == 'o')
        ADVANCE(52);
      END_STATE();
    case 36:
      if (lookahead == 'b')
        ADVANCE(37);
      if (lookahead == 'f')
        ADVANCE(43);
      if (lookahead == 'l')
        ADVANCE(48);
      END_STATE();
    case 37:
      if (lookahead == 'u')
        ADVANCE(38);
      END_STATE();
    case 38:
      if (lookahead == 'g')
        ADVANCE(39);
      END_STATE();
    case 39:
      if (lookahead == 'g')
        ADVANCE(40);
      END_STATE();
    case 40:
      if (lookahead == 'e')
        ADVANCE(41);
      END_STATE();
    case 41:
      if (lookahead == 'r')
        ADVANCE(42);
      END_STATE();
    case 42:
      ACCEPT_TOKEN(anon_sym_debugger);
      END_STATE();
    case 43:
      if (lookahead == 'a')
        ADVANCE(44);
      END_STATE();
    case 44:
      if (lookahead == 'u')
        ADVANCE(45);
      END_STATE();
    case 45:
      if (lookahead == 'l')
        ADVANCE(46);
      END_STATE();
    case 46:
      if (lookahead == 't')
        ADVANCE(47);
      END_STATE();
    case 47:
      ACCEPT_TOKEN(anon_sym_default);
      END_STATE();
    case 48:
      if (lookahead == 'e')
        ADVANCE(49);
      END_STATE();
    case 49:
      if (lookahead == 't')
        ADVANCE(50);
      END_STATE();
    case 50:
      if (lookahead == 'e')
        ADVANCE(51);
      END_STATE();
    case 51:
      ACCEPT_TOKEN(anon_sym_delete);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_do);
      END_STATE();
    case 53:
      if (lookahead == 'l')
        ADVANCE(54);
      if (lookahead == 'x')
        ADVANCE(57);
      END_STATE();
    case 54:
      if (lookahead == 's')
        ADVANCE(55);
      END_STATE();
    case 55:
      if (lookahead == 'e')
        ADVANCE(56);
      END_STATE();
    case 56:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 57:
      if (lookahead == 'p')
        ADVANCE(58);
      if (lookahead == 't')
        ADVANCE(62);
      END_STATE();
    case 58:
      if (lookahead == 'o')
        ADVANCE(59);
      END_STATE();
    case 59:
      if (lookahead == 'r')
        ADVANCE(60);
      END_STATE();
    case 60:
      if (lookahead == 't')
        ADVANCE(61);
      END_STATE();
    case 61:
      ACCEPT_TOKEN(anon_sym_export);
      END_STATE();
    case 62:
      if (lookahead == 'e')
        ADVANCE(63);
      END_STATE();
    case 63:
      if (lookahead == 'n')
        ADVANCE(64);
      END_STATE();
    case 64:
      if (lookahead == 'd')
        ADVANCE(65);
      END_STATE();
    case 65:
      if (lookahead == 's')
        ADVANCE(66);
      END_STATE();
    case 66:
      ACCEPT_TOKEN(anon_sym_extends);
      END_STATE();
    case 67:
      if (lookahead == 'a')
        ADVANCE(68);
      if (lookahead == 'i')
        ADVANCE(72);
      if (lookahead == 'o')
        ADVANCE(78);
      if (lookahead == 'r')
        ADVANCE(80);
      if (lookahead == 'u')
        ADVANCE(83);
      END_STATE();
    case 68:
      if (lookahead == 'l')
        ADVANCE(69);
      END_STATE();
    case 69:
      if (lookahead == 's')
        ADVANCE(70);
      END_STATE();
    case 70:
      if (lookahead == 'e')
        ADVANCE(71);
      END_STATE();
    case 71:
      ACCEPT_TOKEN(sym_false);
      END_STATE();
    case 72:
      if (lookahead == 'n')
        ADVANCE(73);
      END_STATE();
    case 73:
      if (lookahead == 'a')
        ADVANCE(74);
      END_STATE();
    case 74:
      if (lookahead == 'l')
        ADVANCE(75);
      END_STATE();
    case 75:
      if (lookahead == 'l')
        ADVANCE(76);
      END_STATE();
    case 76:
      if (lookahead == 'y')
        ADVANCE(77);
      END_STATE();
    case 77:
      ACCEPT_TOKEN(anon_sym_finally);
      END_STATE();
    case 78:
      if (lookahead == 'r')
        ADVANCE(79);
      END_STATE();
    case 79:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 80:
      if (lookahead == 'o')
        ADVANCE(81);
      END_STATE();
    case 81:
      if (lookahead == 'm')
        ADVANCE(82);
      END_STATE();
    case 82:
      ACCEPT_TOKEN(anon_sym_from);
      END_STATE();
    case 83:
      if (lookahead == 'n')
        ADVANCE(84);
      END_STATE();
    case 84:
      if (lookahead == 'c')
        ADVANCE(85);
      END_STATE();
    case 85:
      if (lookahead == 't')
        ADVANCE(86);
      END_STATE();
    case 86:
      if (lookahead == 'i')
        ADVANCE(87);
      END_STATE();
    case 87:
      if (lookahead == 'o')
        ADVANCE(88);
      END_STATE();
    case 88:
      if (lookahead == 'n')
        ADVANCE(89);
      END_STATE();
    case 89:
      ACCEPT_TOKEN(anon_sym_function);
      END_STATE();
    case 90:
      if (lookahead == 'e')
        ADVANCE(91);
      END_STATE();
    case 91:
      if (lookahead == 't')
        ADVANCE(92);
      END_STATE();
    case 92:
      ACCEPT_TOKEN(anon_sym_get);
      END_STATE();
    case 93:
      if (lookahead == 'f')
        ADVANCE(94);
      if (lookahead == 'm')
        ADVANCE(95);
      if (lookahead == 'n')
        ADVANCE(100);
      END_STATE();
    case 94:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 95:
      if (lookahead == 'p')
        ADVANCE(96);
      END_STATE();
    case 96:
      if (lookahead == 'o')
        ADVANCE(97);
      END_STATE();
    case 97:
      if (lookahead == 'r')
        ADVANCE(98);
      END_STATE();
    case 98:
      if (lookahead == 't')
        ADVANCE(99);
      END_STATE();
    case 99:
      ACCEPT_TOKEN(anon_sym_import);
      END_STATE();
    case 100:
      ACCEPT_TOKEN(anon_sym_in);
      if (lookahead == 's')
        ADVANCE(101);
      END_STATE();
    case 101:
      if (lookahead == 't')
        ADVANCE(102);
      END_STATE();
    case 102:
      if (lookahead == 'a')
        ADVANCE(103);
      END_STATE();
    case 103:
      if (lookahead == 'n')
        ADVANCE(104);
      END_STATE();
    case 104:
      if (lookahead == 'c')
        ADVANCE(105);
      END_STATE();
    case 105:
      if (lookahead == 'e')
        ADVANCE(106);
      END_STATE();
    case 106:
      if (lookahead == 'o')
        ADVANCE(107);
      END_STATE();
    case 107:
      if (lookahead == 'f')
        ADVANCE(108);
      END_STATE();
    case 108:
      ACCEPT_TOKEN(anon_sym_instanceof);
      END_STATE();
    case 109:
      if (lookahead == 'e')
        ADVANCE(110);
      END_STATE();
    case 110:
      if (lookahead == 't')
        ADVANCE(111);
      END_STATE();
    case 111:
      ACCEPT_TOKEN(anon_sym_let);
      END_STATE();
    case 112:
      if (lookahead == 'e')
        ADVANCE(113);
      if (lookahead == 'u')
        ADVANCE(115);
      END_STATE();
    case 113:
      if (lookahead == 'w')
        ADVANCE(114);
      END_STATE();
    case 114:
      ACCEPT_TOKEN(anon_sym_new);
      END_STATE();
    case 115:
      if (lookahead == 'l')
        ADVANCE(116);
      END_STATE();
    case 116:
      if (lookahead == 'l')
        ADVANCE(117);
      END_STATE();
    case 117:
      ACCEPT_TOKEN(sym_null);
      END_STATE();
    case 118:
      if (lookahead == 'f')
        ADVANCE(119);
      END_STATE();
    case 119:
      ACCEPT_TOKEN(anon_sym_of);
      END_STATE();
    case 120:
      if (lookahead == 'e')
        ADVANCE(121);
      END_STATE();
    case 121:
      if (lookahead == 't')
        ADVANCE(122);
      END_STATE();
    case 122:
      if (lookahead == 'u')
        ADVANCE(123);
      END_STATE();
    case 123:
      if (lookahead == 'r')
        ADVANCE(124);
      END_STATE();
    case 124:
      if (lookahead == 'n')
        ADVANCE(125);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 126:
      if (lookahead == 'e')
        ADVANCE(127);
      if (lookahead == 't')
        ADVANCE(129);
      if (lookahead == 'u')
        ADVANCE(134);
      if (lookahead == 'w')
        ADVANCE(138);
      END_STATE();
    case 127:
      if (lookahead == 't')
        ADVANCE(128);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(anon_sym_set);
      END_STATE();
    case 129:
      if (lookahead == 'a')
        ADVANCE(130);
      END_STATE();
    case 130:
      if (lookahead == 't')
        ADVANCE(131);
      END_STATE();
    case 131:
      if (lookahead == 'i')
        ADVANCE(132);
      END_STATE();
    case 132:
      if (lookahead == 'c')
        ADVANCE(133);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(anon_sym_static);
      END_STATE();
    case 134:
      if (lookahead == 'p')
        ADVANCE(135);
      END_STATE();
    case 135:
      if (lookahead == 'e')
        ADVANCE(136);
      END_STATE();
    case 136:
      if (lookahead == 'r')
        ADVANCE(137);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(sym_super);
      END_STATE();
    case 138:
      if (lookahead == 'i')
        ADVANCE(139);
      END_STATE();
    case 139:
      if (lookahead == 't')
        ADVANCE(140);
      END_STATE();
    case 140:
      if (lookahead == 'c')
        ADVANCE(141);
      END_STATE();
    case 141:
      if (lookahead == 'h')
        ADVANCE(142);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(anon_sym_switch);
      END_STATE();
    case 143:
      if (lookahead == 'h')
        ADVANCE(144);
      if (lookahead == 'r')
        ADVANCE(150);
      if (lookahead == 'y')
        ADVANCE(154);
      END_STATE();
    case 144:
      if (lookahead == 'i')
        ADVANCE(145);
      if (lookahead == 'r')
        ADVANCE(147);
      END_STATE();
    case 145:
      if (lookahead == 's')
        ADVANCE(146);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(sym_this);
      END_STATE();
    case 147:
      if (lookahead == 'o')
        ADVANCE(148);
      END_STATE();
    case 148:
      if (lookahead == 'w')
        ADVANCE(149);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(anon_sym_throw);
      END_STATE();
    case 150:
      if (lookahead == 'u')
        ADVANCE(151);
      if (lookahead == 'y')
        ADVANCE(153);
      END_STATE();
    case 151:
      if (lookahead == 'e')
        ADVANCE(152);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_true);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(anon_sym_try);
      END_STATE();
    case 154:
      if (lookahead == 'p')
        ADVANCE(155);
      END_STATE();
    case 155:
      if (lookahead == 'e')
        ADVANCE(156);
      END_STATE();
    case 156:
      if (lookahead == 'o')
        ADVANCE(157);
      END_STATE();
    case 157:
      if (lookahead == 'f')
        ADVANCE(158);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(anon_sym_typeof);
      END_STATE();
    case 159:
      if (lookahead == 'n')
        ADVANCE(160);
      END_STATE();
    case 160:
      if (lookahead == 'd')
        ADVANCE(161);
      END_STATE();
    case 161:
      if (lookahead == 'e')
        ADVANCE(162);
      END_STATE();
    case 162:
      if (lookahead == 'f')
        ADVANCE(163);
      END_STATE();
    case 163:
      if (lookahead == 'i')
        ADVANCE(164);
      END_STATE();
    case 164:
      if (lookahead == 'n')
        ADVANCE(165);
      END_STATE();
    case 165:
      if (lookahead == 'e')
        ADVANCE(166);
      END_STATE();
    case 166:
      if (lookahead == 'd')
        ADVANCE(167);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(sym_undefined);
      END_STATE();
    case 168:
      if (lookahead == 'a')
        ADVANCE(169);
      if (lookahead == 'o')
        ADVANCE(171);
      END_STATE();
    case 169:
      if (lookahead == 'r')
        ADVANCE(170);
      END_STATE();
    case 170:
      ACCEPT_TOKEN(anon_sym_var);
      END_STATE();
    case 171:
      if (lookahead == 'i')
        ADVANCE(172);
      END_STATE();
    case 172:
      if (lookahead == 'd')
        ADVANCE(173);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(anon_sym_void);
      END_STATE();
    case 174:
      if (lookahead == 'h')
        ADVANCE(175);
      if (lookahead == 'i')
        ADVANCE(179);
      END_STATE();
    case 175:
      if (lookahead == 'i')
        ADVANCE(176);
      END_STATE();
    case 176:
      if (lookahead == 'l')
        ADVANCE(177);
      END_STATE();
    case 177:
      if (lookahead == 'e')
        ADVANCE(178);
      END_STATE();
    case 178:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 179:
      if (lookahead == 't')
        ADVANCE(180);
      END_STATE();
    case 180:
      if (lookahead == 'h')
        ADVANCE(181);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(anon_sym_with);
      END_STATE();
    case 182:
      if (lookahead == 'i')
        ADVANCE(183);
      END_STATE();
    case 183:
      if (lookahead == 'e')
        ADVANCE(184);
      END_STATE();
    case 184:
      if (lookahead == 'l')
        ADVANCE(185);
      END_STATE();
    case 185:
      if (lookahead == 'd')
        ADVANCE(186);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(anon_sym_yield);
      END_STATE();
    default:
      return false;
  }
}

static TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 81},
  [2] = {.lex_state = 81},
  [3] = {.lex_state = 111},
  [4] = {.lex_state = 111},
  [5] = {.lex_state = 111},
  [6] = {.lex_state = 81},
  [7] = {.lex_state = 81},
  [8] = {.lex_state = 0},
  [9] = {.lex_state = 0},
  [10] = {.lex_state = 0},
  [11] = {.lex_state = 111},
  [12] = {.lex_state = 0},
  [13] = {.lex_state = 81},
  [14] = {.lex_state = 0},
  [15] = {.lex_state = 0},
  [16] = {.lex_state = 115, .external_lex_state = 1},
  [17] = {.lex_state = 115, .external_lex_state = 1},
  [18] = {.lex_state = 117, .external_lex_state = 1},
  [19] = {.lex_state = 118, .external_lex_state = 1},
  [20] = {.lex_state = 81},
  [21] = {.lex_state = 111},
  [22] = {.lex_state = 119, .external_lex_state = 1},
  [23] = {.lex_state = 111},
  [24] = {.lex_state = 132},
  [25] = {.lex_state = 81},
  [26] = {.lex_state = 138, .external_lex_state = 1},
  [27] = {.lex_state = 111},
  [28] = {.lex_state = 81},
  [29] = {.lex_state = 81},
  [30] = {.lex_state = 81},
  [31] = {.lex_state = 81},
  [32] = {.lex_state = 81},
  [33] = {.lex_state = 140},
  [34] = {.lex_state = 157, .external_lex_state = 1},
  [35] = {.lex_state = 111},
  [36] = {.lex_state = 81},
  [37] = {.lex_state = 158, .external_lex_state = 1},
  [38] = {.lex_state = 0},
  [39] = {.lex_state = 117, .external_lex_state = 1},
  [40] = {.lex_state = 160, .external_lex_state = 1},
  [41] = {.lex_state = 162, .external_lex_state = 1},
  [42] = {.lex_state = 162, .external_lex_state = 1},
  [43] = {.lex_state = 163},
  [44] = {.lex_state = 170, .external_lex_state = 1},
  [45] = {.lex_state = 170, .external_lex_state = 1},
  [46] = {.lex_state = 158, .external_lex_state = 1},
  [47] = {.lex_state = 0},
  [48] = {.lex_state = 81},
  [49] = {.lex_state = 81},
  [50] = {.lex_state = 81},
  [51] = {.lex_state = 81},
  [52] = {.lex_state = 81},
  [53] = {.lex_state = 111},
  [54] = {.lex_state = 81},
  [55] = {.lex_state = 81},
  [56] = {.lex_state = 111},
  [57] = {.lex_state = 115, .external_lex_state = 1},
  [58] = {.lex_state = 111},
  [59] = {.lex_state = 171, .external_lex_state = 1},
  [60] = {.lex_state = 81},
  [61] = {.lex_state = 81},
  [62] = {.lex_state = 111},
  [63] = {.lex_state = 172, .external_lex_state = 1},
  [64] = {.lex_state = 0},
  [65] = {.lex_state = 0},
  [66] = {.lex_state = 0},
  [67] = {.lex_state = 0},
  [68] = {.lex_state = 111},
  [69] = {.lex_state = 173, .external_lex_state = 1},
  [70] = {.lex_state = 81},
  [71] = {.lex_state = 174, .external_lex_state = 1},
  [72] = {.lex_state = 175, .external_lex_state = 1},
  [73] = {.lex_state = 81},
  [74] = {.lex_state = 176, .external_lex_state = 1},
  [75] = {.lex_state = 162, .external_lex_state = 1},
  [76] = {.lex_state = 111},
  [77] = {.lex_state = 162, .external_lex_state = 1},
  [78] = {.lex_state = 111},
  [79] = {.lex_state = 111},
  [80] = {.lex_state = 111},
  [81] = {.lex_state = 111},
  [82] = {.lex_state = 81},
  [83] = {.lex_state = 111},
  [84] = {.lex_state = 117, .external_lex_state = 1},
  [85] = {.lex_state = 111},
  [86] = {.lex_state = 81},
  [87] = {.lex_state = 81},
  [88] = {.lex_state = 111},
  [89] = {.lex_state = 111},
  [90] = {.lex_state = 177, .external_lex_state = 1},
  [91] = {.lex_state = 178, .external_lex_state = 1},
  [92] = {.lex_state = 177, .external_lex_state = 1},
  [93] = {.lex_state = 177, .external_lex_state = 1},
  [94] = {.lex_state = 178, .external_lex_state = 1},
  [95] = {.lex_state = 81},
  [96] = {.lex_state = 81},
  [97] = {.lex_state = 0},
  [98] = {.lex_state = 81},
  [99] = {.lex_state = 111},
  [100] = {.lex_state = 111},
  [101] = {.lex_state = 0},
  [102] = {.lex_state = 179},
  [103] = {.lex_state = 111},
  [104] = {.lex_state = 132},
  [105] = {.lex_state = 81},
  [106] = {.lex_state = 180},
  [107] = {.lex_state = 111},
  [108] = {.lex_state = 81},
  [109] = {.lex_state = 81},
  [110] = {.lex_state = 81},
  [111] = {.lex_state = 81},
  [112] = {.lex_state = 81},
  [113] = {.lex_state = 81},
  [114] = {.lex_state = 140},
  [115] = {.lex_state = 180},
  [116] = {.lex_state = 111},
  [117] = {.lex_state = 180},
  [118] = {.lex_state = 111},
  [119] = {.lex_state = 181},
  [120] = {.lex_state = 181},
  [121] = {.lex_state = 111},
  [122] = {.lex_state = 181},
  [123] = {.lex_state = 163},
  [124] = {.lex_state = 181},
  [125] = {.lex_state = 180},
  [126] = {.lex_state = 0},
  [127] = {.lex_state = 81},
  [128] = {.lex_state = 81},
  [129] = {.lex_state = 111},
  [130] = {.lex_state = 111},
  [131] = {.lex_state = 0},
  [132] = {.lex_state = 0},
  [133] = {.lex_state = 0},
  [134] = {.lex_state = 0},
  [135] = {.lex_state = 0},
  [136] = {.lex_state = 81},
  [137] = {.lex_state = 138, .external_lex_state = 1},
  [138] = {.lex_state = 111},
  [139] = {.lex_state = 157, .external_lex_state = 1},
  [140] = {.lex_state = 81},
  [141] = {.lex_state = 160, .external_lex_state = 1},
  [142] = {.lex_state = 160, .external_lex_state = 1},
  [143] = {.lex_state = 81},
  [144] = {.lex_state = 111},
  [145] = {.lex_state = 111},
  [146] = {.lex_state = 81},
  [147] = {.lex_state = 111},
  [148] = {.lex_state = 117, .external_lex_state = 1},
  [149] = {.lex_state = 111},
  [150] = {.lex_state = 117, .external_lex_state = 1},
  [151] = {.lex_state = 111},
  [152] = {.lex_state = 111},
  [153] = {.lex_state = 111},
  [154] = {.lex_state = 138, .external_lex_state = 1},
  [155] = {.lex_state = 117, .external_lex_state = 1},
  [156] = {.lex_state = 182, .external_lex_state = 1},
  [157] = {.lex_state = 81},
  [158] = {.lex_state = 117, .external_lex_state = 1},
  [159] = {.lex_state = 160, .external_lex_state = 1},
  [160] = {.lex_state = 111},
  [161] = {.lex_state = 179},
  [162] = {.lex_state = 183, .external_lex_state = 1},
  [163] = {.lex_state = 180},
  [164] = {.lex_state = 81},
  [165] = {.lex_state = 81},
  [166] = {.lex_state = 81},
  [167] = {.lex_state = 81},
  [168] = {.lex_state = 81},
  [169] = {.lex_state = 81},
  [170] = {.lex_state = 180},
  [171] = {.lex_state = 180},
  [172] = {.lex_state = 181},
  [173] = {.lex_state = 181},
  [174] = {.lex_state = 181},
  [175] = {.lex_state = 180},
  [176] = {.lex_state = 111},
  [177] = {.lex_state = 0},
  [178] = {.lex_state = 111},
  [179] = {.lex_state = 163},
  [180] = {.lex_state = 132},
  [181] = {.lex_state = 132},
  [182] = {.lex_state = 132},
  [183] = {.lex_state = 132},
  [184] = {.lex_state = 111},
  [185] = {.lex_state = 81},
  [186] = {.lex_state = 81},
  [187] = {.lex_state = 0},
  [188] = {.lex_state = 182, .external_lex_state = 1},
  [189] = {.lex_state = 111},
  [190] = {.lex_state = 81},
  [191] = {.lex_state = 81},
  [192] = {.lex_state = 81},
  [193] = {.lex_state = 81},
  [194] = {.lex_state = 81},
  [195] = {.lex_state = 0},
  [196] = {.lex_state = 182, .external_lex_state = 1},
  [197] = {.lex_state = 81},
  [198] = {.lex_state = 0},
  [199] = {.lex_state = 0},
  [200] = {.lex_state = 111},
  [201] = {.lex_state = 182, .external_lex_state = 1},
  [202] = {.lex_state = 160, .external_lex_state = 1},
  [203] = {.lex_state = 182, .external_lex_state = 1},
  [204] = {.lex_state = 160, .external_lex_state = 1},
  [205] = {.lex_state = 182, .external_lex_state = 1},
  [206] = {.lex_state = 182, .external_lex_state = 1},
  [207] = {.lex_state = 182, .external_lex_state = 1},
  [208] = {.lex_state = 81},
  [209] = {.lex_state = 140},
  [210] = {.lex_state = 81},
  [211] = {.lex_state = 81},
  [212] = {.lex_state = 81},
  [213] = {.lex_state = 81},
  [214] = {.lex_state = 182, .external_lex_state = 1},
  [215] = {.lex_state = 111},
  [216] = {.lex_state = 81},
  [217] = {.lex_state = 182, .external_lex_state = 1},
  [218] = {.lex_state = 111},
  [219] = {.lex_state = 111},
  [220] = {.lex_state = 111},
  [221] = {.lex_state = 81},
  [222] = {.lex_state = 111},
  [223] = {.lex_state = 81},
  [224] = {.lex_state = 81},
  [225] = {.lex_state = 81},
  [226] = {.lex_state = 81},
  [227] = {.lex_state = 81},
  [228] = {.lex_state = 81},
  [229] = {.lex_state = 182, .external_lex_state = 1},
  [230] = {.lex_state = 182, .external_lex_state = 1},
  [231] = {.lex_state = 182, .external_lex_state = 1},
  [232] = {.lex_state = 111},
  [233] = {.lex_state = 132},
  [234] = {.lex_state = 163},
  [235] = {.lex_state = 182, .external_lex_state = 1},
  [236] = {.lex_state = 163},
  [237] = {.lex_state = 111},
  [238] = {.lex_state = 81},
  [239] = {.lex_state = 81},
  [240] = {.lex_state = 81},
  [241] = {.lex_state = 81},
  [242] = {.lex_state = 81},
  [243] = {.lex_state = 117, .external_lex_state = 1},
  [244] = {.lex_state = 184, .external_lex_state = 1},
  [245] = {.lex_state = 185, .external_lex_state = 1},
  [246] = {.lex_state = 81},
  [247] = {.lex_state = 81},
  [248] = {.lex_state = 81},
  [249] = {.lex_state = 81},
  [250] = {.lex_state = 81},
  [251] = {.lex_state = 185, .external_lex_state = 1},
  [252] = {.lex_state = 186, .external_lex_state = 1},
  [253] = {.lex_state = 187, .external_lex_state = 1},
  [254] = {.lex_state = 188, .external_lex_state = 1},
  [255] = {.lex_state = 188, .external_lex_state = 1},
  [256] = {.lex_state = 186, .external_lex_state = 1},
  [257] = {.lex_state = 0},
  [258] = {.lex_state = 111},
  [259] = {.lex_state = 115, .external_lex_state = 1},
  [260] = {.lex_state = 111},
  [261] = {.lex_state = 111},
  [262] = {.lex_state = 81},
  [263] = {.lex_state = 81},
  [264] = {.lex_state = 81},
  [265] = {.lex_state = 0},
  [266] = {.lex_state = 0},
  [267] = {.lex_state = 111},
  [268] = {.lex_state = 81},
  [269] = {.lex_state = 81},
  [270] = {.lex_state = 81},
  [271] = {.lex_state = 0},
  [272] = {.lex_state = 0},
  [273] = {.lex_state = 111},
  [274] = {.lex_state = 111},
  [275] = {.lex_state = 111},
  [276] = {.lex_state = 111},
  [277] = {.lex_state = 81},
  [278] = {.lex_state = 111},
  [279] = {.lex_state = 111},
  [280] = {.lex_state = 111},
  [281] = {.lex_state = 111},
  [282] = {.lex_state = 111},
  [283] = {.lex_state = 81},
  [284] = {.lex_state = 81},
  [285] = {.lex_state = 81},
  [286] = {.lex_state = 81},
  [287] = {.lex_state = 181},
  [288] = {.lex_state = 81},
  [289] = {.lex_state = 183, .external_lex_state = 1},
  [290] = {.lex_state = 0},
  [291] = {.lex_state = 81},
  [292] = {.lex_state = 111},
  [293] = {.lex_state = 170},
  [294] = {.lex_state = 173},
  [295] = {.lex_state = 81},
  [296] = {.lex_state = 81},
  [297] = {.lex_state = 81},
  [298] = {.lex_state = 81},
  [299] = {.lex_state = 81},
  [300] = {.lex_state = 173},
  [301] = {.lex_state = 173},
  [302] = {.lex_state = 181},
  [303] = {.lex_state = 181},
  [304] = {.lex_state = 181},
  [305] = {.lex_state = 173},
  [306] = {.lex_state = 0},
  [307] = {.lex_state = 81},
  [308] = {.lex_state = 81},
  [309] = {.lex_state = 189, .external_lex_state = 1},
  [310] = {.lex_state = 111},
  [311] = {.lex_state = 111},
  [312] = {.lex_state = 111},
  [313] = {.lex_state = 111},
  [314] = {.lex_state = 189, .external_lex_state = 1},
  [315] = {.lex_state = 183, .external_lex_state = 1},
  [316] = {.lex_state = 111},
  [317] = {.lex_state = 189, .external_lex_state = 1},
  [318] = {.lex_state = 81},
  [319] = {.lex_state = 0},
  [320] = {.lex_state = 111},
  [321] = {.lex_state = 157, .external_lex_state = 1},
  [322] = {.lex_state = 111},
  [323] = {.lex_state = 111},
  [324] = {.lex_state = 81},
  [325] = {.lex_state = 111},
  [326] = {.lex_state = 0},
  [327] = {.lex_state = 111},
  [328] = {.lex_state = 111},
  [329] = {.lex_state = 81},
  [330] = {.lex_state = 111},
  [331] = {.lex_state = 81},
  [332] = {.lex_state = 111},
  [333] = {.lex_state = 111},
  [334] = {.lex_state = 111},
  [335] = {.lex_state = 117, .external_lex_state = 1},
  [336] = {.lex_state = 190, .external_lex_state = 1},
  [337] = {.lex_state = 111},
  [338] = {.lex_state = 111},
  [339] = {.lex_state = 111},
  [340] = {.lex_state = 111},
  [341] = {.lex_state = 111},
  [342] = {.lex_state = 111},
  [343] = {.lex_state = 190, .external_lex_state = 1},
  [344] = {.lex_state = 181},
  [345] = {.lex_state = 111},
  [346] = {.lex_state = 111},
  [347] = {.lex_state = 178, .external_lex_state = 1},
  [348] = {.lex_state = 111},
  [349] = {.lex_state = 81},
  [350] = {.lex_state = 178, .external_lex_state = 1},
  [351] = {.lex_state = 178, .external_lex_state = 1},
  [352] = {.lex_state = 178, .external_lex_state = 1},
  [353] = {.lex_state = 111},
  [354] = {.lex_state = 178, .external_lex_state = 1},
  [355] = {.lex_state = 180},
  [356] = {.lex_state = 180},
  [357] = {.lex_state = 180},
  [358] = {.lex_state = 111},
  [359] = {.lex_state = 181},
  [360] = {.lex_state = 181},
  [361] = {.lex_state = 111},
  [362] = {.lex_state = 111},
  [363] = {.lex_state = 111},
  [364] = {.lex_state = 81},
  [365] = {.lex_state = 81},
  [366] = {.lex_state = 0},
  [367] = {.lex_state = 0},
  [368] = {.lex_state = 0},
  [369] = {.lex_state = 0},
  [370] = {.lex_state = 81},
  [371] = {.lex_state = 0},
  [372] = {.lex_state = 0},
  [373] = {.lex_state = 115, .external_lex_state = 1},
  [374] = {.lex_state = 115, .external_lex_state = 1},
  [375] = {.lex_state = 117, .external_lex_state = 1},
  [376] = {.lex_state = 118, .external_lex_state = 1},
  [377] = {.lex_state = 81},
  [378] = {.lex_state = 111},
  [379] = {.lex_state = 81},
  [380] = {.lex_state = 138, .external_lex_state = 1},
  [381] = {.lex_state = 111},
  [382] = {.lex_state = 157, .external_lex_state = 1},
  [383] = {.lex_state = 81},
  [384] = {.lex_state = 117, .external_lex_state = 1},
  [385] = {.lex_state = 170, .external_lex_state = 1},
  [386] = {.lex_state = 170, .external_lex_state = 1},
  [387] = {.lex_state = 81},
  [388] = {.lex_state = 111},
  [389] = {.lex_state = 111},
  [390] = {.lex_state = 81},
  [391] = {.lex_state = 81},
  [392] = {.lex_state = 111},
  [393] = {.lex_state = 138, .external_lex_state = 1},
  [394] = {.lex_state = 81},
  [395] = {.lex_state = 117, .external_lex_state = 1},
  [396] = {.lex_state = 162, .external_lex_state = 1},
  [397] = {.lex_state = 162, .external_lex_state = 1},
  [398] = {.lex_state = 181},
  [399] = {.lex_state = 111},
  [400] = {.lex_state = 111},
  [401] = {.lex_state = 111},
  [402] = {.lex_state = 111},
  [403] = {.lex_state = 111},
  [404] = {.lex_state = 111},
  [405] = {.lex_state = 111},
  [406] = {.lex_state = 181},
  [407] = {.lex_state = 181},
  [408] = {.lex_state = 181},
  [409] = {.lex_state = 111},
  [410] = {.lex_state = 111},
  [411] = {.lex_state = 163},
  [412] = {.lex_state = 132},
  [413] = {.lex_state = 132},
  [414] = {.lex_state = 132},
  [415] = {.lex_state = 132},
  [416] = {.lex_state = 111},
  [417] = {.lex_state = 81},
  [418] = {.lex_state = 0},
  [419] = {.lex_state = 181},
  [420] = {.lex_state = 111},
  [421] = {.lex_state = 0},
  [422] = {.lex_state = 81},
  [423] = {.lex_state = 81},
  [424] = {.lex_state = 81},
  [425] = {.lex_state = 81},
  [426] = {.lex_state = 81},
  [427] = {.lex_state = 0},
  [428] = {.lex_state = 181},
  [429] = {.lex_state = 111},
  [430] = {.lex_state = 81},
  [431] = {.lex_state = 0},
  [432] = {.lex_state = 0},
  [433] = {.lex_state = 111},
  [434] = {.lex_state = 181},
  [435] = {.lex_state = 181},
  [436] = {.lex_state = 181},
  [437] = {.lex_state = 181},
  [438] = {.lex_state = 111},
  [439] = {.lex_state = 181},
  [440] = {.lex_state = 181},
  [441] = {.lex_state = 181},
  [442] = {.lex_state = 140},
  [443] = {.lex_state = 0},
  [444] = {.lex_state = 81},
  [445] = {.lex_state = 81},
  [446] = {.lex_state = 81},
  [447] = {.lex_state = 181},
  [448] = {.lex_state = 111},
  [449] = {.lex_state = 111},
  [450] = {.lex_state = 81},
  [451] = {.lex_state = 181},
  [452] = {.lex_state = 182, .external_lex_state = 1},
  [453] = {.lex_state = 81},
  [454] = {.lex_state = 81},
  [455] = {.lex_state = 81},
  [456] = {.lex_state = 81},
  [457] = {.lex_state = 81},
  [458] = {.lex_state = 81},
  [459] = {.lex_state = 181},
  [460] = {.lex_state = 0},
  [461] = {.lex_state = 181},
  [462] = {.lex_state = 111},
  [463] = {.lex_state = 0},
  [464] = {.lex_state = 181},
  [465] = {.lex_state = 111},
  [466] = {.lex_state = 132},
  [467] = {.lex_state = 181},
  [468] = {.lex_state = 163},
  [469] = {.lex_state = 181},
  [470] = {.lex_state = 81},
  [471] = {.lex_state = 111},
  [472] = {.lex_state = 81},
  [473] = {.lex_state = 81},
  [474] = {.lex_state = 111},
  [475] = {.lex_state = 81},
  [476] = {.lex_state = 162, .external_lex_state = 1},
  [477] = {.lex_state = 81},
  [478] = {.lex_state = 81},
  [479] = {.lex_state = 81},
  [480] = {.lex_state = 81},
  [481] = {.lex_state = 81},
  [482] = {.lex_state = 81},
  [483] = {.lex_state = 81},
  [484] = {.lex_state = 81},
  [485] = {.lex_state = 0},
  [486] = {.lex_state = 0},
  [487] = {.lex_state = 81},
  [488] = {.lex_state = 0},
  [489] = {.lex_state = 81},
  [490] = {.lex_state = 81},
  [491] = {.lex_state = 111},
  [492] = {.lex_state = 111},
  [493] = {.lex_state = 0},
  [494] = {.lex_state = 0},
  [495] = {.lex_state = 111},
  [496] = {.lex_state = 111},
  [497] = {.lex_state = 111},
  [498] = {.lex_state = 111},
  [499] = {.lex_state = 111},
  [500] = {.lex_state = 183, .external_lex_state = 1},
  [501] = {.lex_state = 111},
  [502] = {.lex_state = 111},
  [503] = {.lex_state = 111},
  [504] = {.lex_state = 111},
  [505] = {.lex_state = 111},
  [506] = {.lex_state = 111},
  [507] = {.lex_state = 181},
  [508] = {.lex_state = 111},
  [509] = {.lex_state = 181},
  [510] = {.lex_state = 81},
  [511] = {.lex_state = 81},
  [512] = {.lex_state = 81},
  [513] = {.lex_state = 0},
  [514] = {.lex_state = 181},
  [515] = {.lex_state = 181},
  [516] = {.lex_state = 181},
  [517] = {.lex_state = 81},
  [518] = {.lex_state = 81},
  [519] = {.lex_state = 81},
  [520] = {.lex_state = 183, .external_lex_state = 1},
  [521] = {.lex_state = 81},
  [522] = {.lex_state = 81},
  [523] = {.lex_state = 81},
  [524] = {.lex_state = 81},
  [525] = {.lex_state = 111},
  [526] = {.lex_state = 181},
  [527] = {.lex_state = 181},
  [528] = {.lex_state = 111},
  [529] = {.lex_state = 132},
  [530] = {.lex_state = 163},
  [531] = {.lex_state = 111},
  [532] = {.lex_state = 132},
  [533] = {.lex_state = 163},
  [534] = {.lex_state = 111},
  [535] = {.lex_state = 132},
  [536] = {.lex_state = 132},
  [537] = {.lex_state = 132},
  [538] = {.lex_state = 132},
  [539] = {.lex_state = 132},
  [540] = {.lex_state = 163},
  [541] = {.lex_state = 111},
  [542] = {.lex_state = 81},
  [543] = {.lex_state = 132},
  [544] = {.lex_state = 81},
  [545] = {.lex_state = 182, .external_lex_state = 1},
  [546] = {.lex_state = 81},
  [547] = {.lex_state = 191, .external_lex_state = 1},
  [548] = {.lex_state = 192, .external_lex_state = 1},
  [549] = {.lex_state = 81},
  [550] = {.lex_state = 111},
  [551] = {.lex_state = 193, .external_lex_state = 1},
  [552] = {.lex_state = 117, .external_lex_state = 1},
  [553] = {.lex_state = 111},
  [554] = {.lex_state = 192, .external_lex_state = 1},
  [555] = {.lex_state = 111},
  [556] = {.lex_state = 111},
  [557] = {.lex_state = 119},
  [558] = {.lex_state = 180},
  [559] = {.lex_state = 81},
  [560] = {.lex_state = 81},
  [561] = {.lex_state = 81},
  [562] = {.lex_state = 81},
  [563] = {.lex_state = 81},
  [564] = {.lex_state = 180},
  [565] = {.lex_state = 180},
  [566] = {.lex_state = 181},
  [567] = {.lex_state = 181},
  [568] = {.lex_state = 181},
  [569] = {.lex_state = 180},
  [570] = {.lex_state = 0},
  [571] = {.lex_state = 111},
  [572] = {.lex_state = 0},
  [573] = {.lex_state = 170, .external_lex_state = 1},
  [574] = {.lex_state = 182, .external_lex_state = 1},
  [575] = {.lex_state = 111},
  [576] = {.lex_state = 111},
  [577] = {.lex_state = 111},
  [578] = {.lex_state = 111},
  [579] = {.lex_state = 111},
  [580] = {.lex_state = 194, .external_lex_state = 1},
  [581] = {.lex_state = 160, .external_lex_state = 1},
  [582] = {.lex_state = 0},
  [583] = {.lex_state = 0},
  [584] = {.lex_state = 182, .external_lex_state = 1},
  [585] = {.lex_state = 160, .external_lex_state = 1},
  [586] = {.lex_state = 160, .external_lex_state = 1},
  [587] = {.lex_state = 81},
  [588] = {.lex_state = 0},
  [589] = {.lex_state = 0},
  [590] = {.lex_state = 111},
  [591] = {.lex_state = 170, .external_lex_state = 1},
  [592] = {.lex_state = 111},
  [593] = {.lex_state = 111},
  [594] = {.lex_state = 181},
  [595] = {.lex_state = 182, .external_lex_state = 1},
  [596] = {.lex_state = 140},
  [597] = {.lex_state = 160, .external_lex_state = 1},
  [598] = {.lex_state = 111},
  [599] = {.lex_state = 194, .external_lex_state = 1},
  [600] = {.lex_state = 160, .external_lex_state = 1},
  [601] = {.lex_state = 111},
  [602] = {.lex_state = 182, .external_lex_state = 1},
  [603] = {.lex_state = 81},
  [604] = {.lex_state = 181},
  [605] = {.lex_state = 111},
  [606] = {.lex_state = 111},
  [607] = {.lex_state = 111},
  [608] = {.lex_state = 181},
  [609] = {.lex_state = 81},
  [610] = {.lex_state = 81},
  [611] = {.lex_state = 81},
  [612] = {.lex_state = 81},
  [613] = {.lex_state = 182, .external_lex_state = 1},
  [614] = {.lex_state = 160, .external_lex_state = 1},
  [615] = {.lex_state = 117, .external_lex_state = 1},
  [616] = {.lex_state = 179},
  [617] = {.lex_state = 157},
  [618] = {.lex_state = 81},
  [619] = {.lex_state = 81},
  [620] = {.lex_state = 81},
  [621] = {.lex_state = 81},
  [622] = {.lex_state = 81},
  [623] = {.lex_state = 157},
  [624] = {.lex_state = 157},
  [625] = {.lex_state = 174},
  [626] = {.lex_state = 181},
  [627] = {.lex_state = 181},
  [628] = {.lex_state = 157},
  [629] = {.lex_state = 0},
  [630] = {.lex_state = 160, .external_lex_state = 1},
  [631] = {.lex_state = 160, .external_lex_state = 1},
  [632] = {.lex_state = 160, .external_lex_state = 1},
  [633] = {.lex_state = 163},
  [634] = {.lex_state = 81},
  [635] = {.lex_state = 181},
  [636] = {.lex_state = 111},
  [637] = {.lex_state = 132},
  [638] = {.lex_state = 132},
  [639] = {.lex_state = 132},
  [640] = {.lex_state = 132},
  [641] = {.lex_state = 132},
  [642] = {.lex_state = 132},
  [643] = {.lex_state = 163},
  [644] = {.lex_state = 163},
  [645] = {.lex_state = 182, .external_lex_state = 1},
  [646] = {.lex_state = 163},
  [647] = {.lex_state = 81},
  [648] = {.lex_state = 81},
  [649] = {.lex_state = 0},
  [650] = {.lex_state = 117, .external_lex_state = 1},
  [651] = {.lex_state = 111},
  [652] = {.lex_state = 187, .external_lex_state = 1},
  [653] = {.lex_state = 81},
  [654] = {.lex_state = 81},
  [655] = {.lex_state = 81},
  [656] = {.lex_state = 0},
  [657] = {.lex_state = 187, .external_lex_state = 1},
  [658] = {.lex_state = 187, .external_lex_state = 1},
  [659] = {.lex_state = 81},
  [660] = {.lex_state = 81},
  [661] = {.lex_state = 81},
  [662] = {.lex_state = 81},
  [663] = {.lex_state = 81},
  [664] = {.lex_state = 81},
  [665] = {.lex_state = 81},
  [666] = {.lex_state = 115, .external_lex_state = 1},
  [667] = {.lex_state = 81},
  [668] = {.lex_state = 111},
  [669] = {.lex_state = 115, .external_lex_state = 1},
  [670] = {.lex_state = 111},
  [671] = {.lex_state = 111},
  [672] = {.lex_state = 0},
  [673] = {.lex_state = 171, .external_lex_state = 1},
  [674] = {.lex_state = 0},
  [675] = {.lex_state = 0},
  [676] = {.lex_state = 0},
  [677] = {.lex_state = 0},
  [678] = {.lex_state = 111},
  [679] = {.lex_state = 171, .external_lex_state = 1},
  [680] = {.lex_state = 81},
  [681] = {.lex_state = 179},
  [682] = {.lex_state = 180},
  [683] = {.lex_state = 81},
  [684] = {.lex_state = 81},
  [685] = {.lex_state = 81},
  [686] = {.lex_state = 81},
  [687] = {.lex_state = 81},
  [688] = {.lex_state = 180},
  [689] = {.lex_state = 180},
  [690] = {.lex_state = 181},
  [691] = {.lex_state = 181},
  [692] = {.lex_state = 181},
  [693] = {.lex_state = 180},
  [694] = {.lex_state = 0},
  [695] = {.lex_state = 0},
  [696] = {.lex_state = 111},
  [697] = {.lex_state = 111},
  [698] = {.lex_state = 111},
  [699] = {.lex_state = 111},
  [700] = {.lex_state = 111},
  [701] = {.lex_state = 111},
  [702] = {.lex_state = 111},
  [703] = {.lex_state = 111},
  [704] = {.lex_state = 181},
  [705] = {.lex_state = 111},
  [706] = {.lex_state = 111},
  [707] = {.lex_state = 81},
  [708] = {.lex_state = 111},
  [709] = {.lex_state = 111},
  [710] = {.lex_state = 111},
  [711] = {.lex_state = 111},
  [712] = {.lex_state = 111},
  [713] = {.lex_state = 111},
  [714] = {.lex_state = 111},
  [715] = {.lex_state = 111},
  [716] = {.lex_state = 111},
  [717] = {.lex_state = 0},
  [718] = {.lex_state = 0},
  [719] = {.lex_state = 0},
  [720] = {.lex_state = 0},
  [721] = {.lex_state = 157, .external_lex_state = 1},
  [722] = {.lex_state = 111},
  [723] = {.lex_state = 81},
  [724] = {.lex_state = 81},
  [725] = {.lex_state = 138, .external_lex_state = 1},
  [726] = {.lex_state = 81},
  [727] = {.lex_state = 162, .external_lex_state = 1},
  [728] = {.lex_state = 162, .external_lex_state = 1},
  [729] = {.lex_state = 195, .external_lex_state = 1},
  [730] = {.lex_state = 0},
  [731] = {.lex_state = 183, .external_lex_state = 1},
  [732] = {.lex_state = 181},
  [733] = {.lex_state = 81},
  [734] = {.lex_state = 81},
  [735] = {.lex_state = 0},
  [736] = {.lex_state = 181},
  [737] = {.lex_state = 181},
  [738] = {.lex_state = 81},
  [739] = {.lex_state = 81},
  [740] = {.lex_state = 81},
  [741] = {.lex_state = 81},
  [742] = {.lex_state = 81},
  [743] = {.lex_state = 81},
  [744] = {.lex_state = 81},
  [745] = {.lex_state = 170, .external_lex_state = 1},
  [746] = {.lex_state = 196, .external_lex_state = 1},
  [747] = {.lex_state = 81},
  [748] = {.lex_state = 81},
  [749] = {.lex_state = 81},
  [750] = {.lex_state = 81},
  [751] = {.lex_state = 81},
  [752] = {.lex_state = 196, .external_lex_state = 1},
  [753] = {.lex_state = 197, .external_lex_state = 1},
  [754] = {.lex_state = 189, .external_lex_state = 1},
  [755] = {.lex_state = 198, .external_lex_state = 1},
  [756] = {.lex_state = 198, .external_lex_state = 1},
  [757] = {.lex_state = 197, .external_lex_state = 1},
  [758] = {.lex_state = 0},
  [759] = {.lex_state = 111},
  [760] = {.lex_state = 81},
  [761] = {.lex_state = 111},
  [762] = {.lex_state = 81},
  [763] = {.lex_state = 111},
  [764] = {.lex_state = 183, .external_lex_state = 1},
  [765] = {.lex_state = 181},
  [766] = {.lex_state = 111},
  [767] = {.lex_state = 111},
  [768] = {.lex_state = 111},
  [769] = {.lex_state = 81},
  [770] = {.lex_state = 81},
  [771] = {.lex_state = 111},
  [772] = {.lex_state = 81},
  [773] = {.lex_state = 111},
  [774] = {.lex_state = 81},
  [775] = {.lex_state = 111},
  [776] = {.lex_state = 190, .external_lex_state = 1},
  [777] = {.lex_state = 111},
  [778] = {.lex_state = 190, .external_lex_state = 1},
  [779] = {.lex_state = 111},
  [780] = {.lex_state = 111},
  [781] = {.lex_state = 190, .external_lex_state = 1},
  [782] = {.lex_state = 111},
  [783] = {.lex_state = 178, .external_lex_state = 1},
  [784] = {.lex_state = 111},
  [785] = {.lex_state = 178, .external_lex_state = 1},
  [786] = {.lex_state = 111},
  [787] = {.lex_state = 81},
  [788] = {.lex_state = 81},
  [789] = {.lex_state = 81},
  [790] = {.lex_state = 81},
  [791] = {.lex_state = 81},
  [792] = {.lex_state = 111},
  [793] = {.lex_state = 115, .external_lex_state = 1},
  [794] = {.lex_state = 111},
  [795] = {.lex_state = 171, .external_lex_state = 1},
  [796] = {.lex_state = 81},
  [797] = {.lex_state = 172, .external_lex_state = 1},
  [798] = {.lex_state = 111},
  [799] = {.lex_state = 117, .external_lex_state = 1},
  [800] = {.lex_state = 81},
  [801] = {.lex_state = 178, .external_lex_state = 1},
  [802] = {.lex_state = 178, .external_lex_state = 1},
  [803] = {.lex_state = 81},
  [804] = {.lex_state = 0},
  [805] = {.lex_state = 81},
  [806] = {.lex_state = 81},
  [807] = {.lex_state = 81},
  [808] = {.lex_state = 111},
  [809] = {.lex_state = 111},
  [810] = {.lex_state = 81},
  [811] = {.lex_state = 111},
  [812] = {.lex_state = 117, .external_lex_state = 1},
  [813] = {.lex_state = 111},
  [814] = {.lex_state = 117, .external_lex_state = 1},
  [815] = {.lex_state = 111},
  [816] = {.lex_state = 111},
  [817] = {.lex_state = 117, .external_lex_state = 1},
  [818] = {.lex_state = 117, .external_lex_state = 1},
  [819] = {.lex_state = 81},
  [820] = {.lex_state = 81},
  [821] = {.lex_state = 81},
  [822] = {.lex_state = 0},
  [823] = {.lex_state = 0},
  [824] = {.lex_state = 81},
  [825] = {.lex_state = 81},
  [826] = {.lex_state = 111},
  [827] = {.lex_state = 111},
  [828] = {.lex_state = 81},
  [829] = {.lex_state = 81},
  [830] = {.lex_state = 111},
  [831] = {.lex_state = 111},
  [832] = {.lex_state = 81},
  [833] = {.lex_state = 111},
  [834] = {.lex_state = 190, .external_lex_state = 1},
  [835] = {.lex_state = 178, .external_lex_state = 1},
  [836] = {.lex_state = 190, .external_lex_state = 1},
  [837] = {.lex_state = 190, .external_lex_state = 1},
  [838] = {.lex_state = 178, .external_lex_state = 1},
  [839] = {.lex_state = 81},
  [840] = {.lex_state = 81},
  [841] = {.lex_state = 111},
  [842] = {.lex_state = 111},
  [843] = {.lex_state = 81},
  [844] = {.lex_state = 81},
  [845] = {.lex_state = 81},
  [846] = {.lex_state = 81},
  [847] = {.lex_state = 181},
  [848] = {.lex_state = 111},
  [849] = {.lex_state = 181},
  [850] = {.lex_state = 111},
  [851] = {.lex_state = 111},
  [852] = {.lex_state = 181},
  [853] = {.lex_state = 181},
  [854] = {.lex_state = 111},
  [855] = {.lex_state = 132},
  [856] = {.lex_state = 163},
  [857] = {.lex_state = 111},
  [858] = {.lex_state = 132},
  [859] = {.lex_state = 111},
  [860] = {.lex_state = 132},
  [861] = {.lex_state = 181},
  [862] = {.lex_state = 111},
  [863] = {.lex_state = 0},
  [864] = {.lex_state = 181},
  [865] = {.lex_state = 181},
  [866] = {.lex_state = 111},
  [867] = {.lex_state = 111},
  [868] = {.lex_state = 111},
  [869] = {.lex_state = 111},
  [870] = {.lex_state = 111},
  [871] = {.lex_state = 180},
  [872] = {.lex_state = 181},
  [873] = {.lex_state = 0},
  [874] = {.lex_state = 0},
  [875] = {.lex_state = 181},
  [876] = {.lex_state = 181},
  [877] = {.lex_state = 181},
  [878] = {.lex_state = 81},
  [879] = {.lex_state = 0},
  [880] = {.lex_state = 111},
  [881] = {.lex_state = 0},
  [882] = {.lex_state = 0},
  [883] = {.lex_state = 111},
  [884] = {.lex_state = 181},
  [885] = {.lex_state = 173, .external_lex_state = 1},
  [886] = {.lex_state = 175, .external_lex_state = 1},
  [887] = {.lex_state = 176, .external_lex_state = 1},
  [888] = {.lex_state = 111},
  [889] = {.lex_state = 111},
  [890] = {.lex_state = 181},
  [891] = {.lex_state = 181},
  [892] = {.lex_state = 180},
  [893] = {.lex_state = 181},
  [894] = {.lex_state = 0},
  [895] = {.lex_state = 181},
  [896] = {.lex_state = 181},
  [897] = {.lex_state = 111},
  [898] = {.lex_state = 111},
  [899] = {.lex_state = 111},
  [900] = {.lex_state = 181},
  [901] = {.lex_state = 181},
  [902] = {.lex_state = 111},
  [903] = {.lex_state = 174},
  [904] = {.lex_state = 181},
  [905] = {.lex_state = 181},
  [906] = {.lex_state = 181},
  [907] = {.lex_state = 0},
  [908] = {.lex_state = 0},
  [909] = {.lex_state = 132},
  [910] = {.lex_state = 181},
  [911] = {.lex_state = 81},
  [912] = {.lex_state = 0},
  [913] = {.lex_state = 81},
  [914] = {.lex_state = 81},
  [915] = {.lex_state = 81},
  [916] = {.lex_state = 0},
  [917] = {.lex_state = 0},
  [918] = {.lex_state = 81},
  [919] = {.lex_state = 111},
  [920] = {.lex_state = 111},
  [921] = {.lex_state = 111},
  [922] = {.lex_state = 81},
  [923] = {.lex_state = 81},
  [924] = {.lex_state = 0},
  [925] = {.lex_state = 0},
  [926] = {.lex_state = 0},
  [927] = {.lex_state = 0},
  [928] = {.lex_state = 81},
  [929] = {.lex_state = 0},
  [930] = {.lex_state = 0},
  [931] = {.lex_state = 115, .external_lex_state = 1},
  [932] = {.lex_state = 115, .external_lex_state = 1},
  [933] = {.lex_state = 117, .external_lex_state = 1},
  [934] = {.lex_state = 118, .external_lex_state = 1},
  [935] = {.lex_state = 81},
  [936] = {.lex_state = 81},
  [937] = {.lex_state = 157, .external_lex_state = 1},
  [938] = {.lex_state = 81},
  [939] = {.lex_state = 117, .external_lex_state = 1},
  [940] = {.lex_state = 160, .external_lex_state = 1},
  [941] = {.lex_state = 160, .external_lex_state = 1},
  [942] = {.lex_state = 81},
  [943] = {.lex_state = 81},
  [944] = {.lex_state = 81},
  [945] = {.lex_state = 138, .external_lex_state = 1},
  [946] = {.lex_state = 81},
  [947] = {.lex_state = 162, .external_lex_state = 1},
  [948] = {.lex_state = 162, .external_lex_state = 1},
  [949] = {.lex_state = 0},
  [950] = {.lex_state = 0},
  [951] = {.lex_state = 81},
  [952] = {.lex_state = 0},
  [953] = {.lex_state = 182, .external_lex_state = 1},
  [954] = {.lex_state = 0},
  [955] = {.lex_state = 0},
  [956] = {.lex_state = 0},
  [957] = {.lex_state = 0},
  [958] = {.lex_state = 111},
  [959] = {.lex_state = 182, .external_lex_state = 1},
  [960] = {.lex_state = 81},
  [961] = {.lex_state = 117, .external_lex_state = 1},
  [962] = {.lex_state = 81},
  [963] = {.lex_state = 111},
  [964] = {.lex_state = 111},
  [965] = {.lex_state = 81},
  [966] = {.lex_state = 111},
  [967] = {.lex_state = 111},
  [968] = {.lex_state = 111},
  [969] = {.lex_state = 111},
  [970] = {.lex_state = 181},
  [971] = {.lex_state = 181},
  [972] = {.lex_state = 181},
  [973] = {.lex_state = 81},
  [974] = {.lex_state = 181},
  [975] = {.lex_state = 181},
  [976] = {.lex_state = 174},
  [977] = {.lex_state = 181},
  [978] = {.lex_state = 181},
  [979] = {.lex_state = 181},
  [980] = {.lex_state = 183, .external_lex_state = 1},
  [981] = {.lex_state = 111},
  [982] = {.lex_state = 132},
  [983] = {.lex_state = 132},
  [984] = {.lex_state = 181},
  [985] = {.lex_state = 111},
  [986] = {.lex_state = 132},
  [987] = {.lex_state = 132},
  [988] = {.lex_state = 182, .external_lex_state = 1},
  [989] = {.lex_state = 81},
  [990] = {.lex_state = 81},
  [991] = {.lex_state = 163},
  [992] = {.lex_state = 111},
  [993] = {.lex_state = 132},
  [994] = {.lex_state = 132},
  [995] = {.lex_state = 132},
  [996] = {.lex_state = 182, .external_lex_state = 1},
  [997] = {.lex_state = 132},
  [998] = {.lex_state = 163},
  [999] = {.lex_state = 111},
  [1000] = {.lex_state = 0},
  [1001] = {.lex_state = 181},
  [1002] = {.lex_state = 81},
  [1003] = {.lex_state = 81},
  [1004] = {.lex_state = 111},
  [1005] = {.lex_state = 111},
  [1006] = {.lex_state = 111},
  [1007] = {.lex_state = 111},
  [1008] = {.lex_state = 117, .external_lex_state = 1},
  [1009] = {.lex_state = 111},
  [1010] = {.lex_state = 111},
  [1011] = {.lex_state = 117, .external_lex_state = 1},
  [1012] = {.lex_state = 0},
  [1013] = {.lex_state = 111},
  [1014] = {.lex_state = 111},
  [1015] = {.lex_state = 182, .external_lex_state = 1},
  [1016] = {.lex_state = 111},
  [1017] = {.lex_state = 181},
  [1018] = {.lex_state = 81},
  [1019] = {.lex_state = 81},
  [1020] = {.lex_state = 81},
  [1021] = {.lex_state = 0},
  [1022] = {.lex_state = 181},
  [1023] = {.lex_state = 181},
  [1024] = {.lex_state = 81},
  [1025] = {.lex_state = 81},
  [1026] = {.lex_state = 81},
  [1027] = {.lex_state = 81},
  [1028] = {.lex_state = 81},
  [1029] = {.lex_state = 81},
  [1030] = {.lex_state = 81},
  [1031] = {.lex_state = 170, .external_lex_state = 1},
  [1032] = {.lex_state = 111},
  [1033] = {.lex_state = 170, .external_lex_state = 1},
  [1034] = {.lex_state = 181},
  [1035] = {.lex_state = 0},
  [1036] = {.lex_state = 170, .external_lex_state = 1},
  [1037] = {.lex_state = 182, .external_lex_state = 1},
  [1038] = {.lex_state = 160, .external_lex_state = 1},
  [1039] = {.lex_state = 0},
  [1040] = {.lex_state = 170, .external_lex_state = 1},
  [1041] = {.lex_state = 170, .external_lex_state = 1},
  [1042] = {.lex_state = 111},
  [1043] = {.lex_state = 182, .external_lex_state = 1},
  [1044] = {.lex_state = 140},
  [1045] = {.lex_state = 81},
  [1046] = {.lex_state = 181},
  [1047] = {.lex_state = 181},
  [1048] = {.lex_state = 182, .external_lex_state = 1},
  [1049] = {.lex_state = 111},
  [1050] = {.lex_state = 111},
  [1051] = {.lex_state = 194, .external_lex_state = 1},
  [1052] = {.lex_state = 81},
  [1053] = {.lex_state = 111},
  [1054] = {.lex_state = 111},
  [1055] = {.lex_state = 174},
  [1056] = {.lex_state = 81},
  [1057] = {.lex_state = 81},
  [1058] = {.lex_state = 81},
  [1059] = {.lex_state = 0},
  [1060] = {.lex_state = 174},
  [1061] = {.lex_state = 174},
  [1062] = {.lex_state = 81},
  [1063] = {.lex_state = 81},
  [1064] = {.lex_state = 81},
  [1065] = {.lex_state = 81},
  [1066] = {.lex_state = 81},
  [1067] = {.lex_state = 81},
  [1068] = {.lex_state = 81},
  [1069] = {.lex_state = 81},
  [1070] = {.lex_state = 170},
  [1071] = {.lex_state = 173},
  [1072] = {.lex_state = 81},
  [1073] = {.lex_state = 81},
  [1074] = {.lex_state = 81},
  [1075] = {.lex_state = 81},
  [1076] = {.lex_state = 81},
  [1077] = {.lex_state = 173},
  [1078] = {.lex_state = 173},
  [1079] = {.lex_state = 181},
  [1080] = {.lex_state = 181},
  [1081] = {.lex_state = 181},
  [1082] = {.lex_state = 173},
  [1083] = {.lex_state = 0},
  [1084] = {.lex_state = 163},
  [1085] = {.lex_state = 111},
  [1086] = {.lex_state = 111},
  [1087] = {.lex_state = 111},
  [1088] = {.lex_state = 111},
  [1089] = {.lex_state = 111},
  [1090] = {.lex_state = 132},
  [1091] = {.lex_state = 111},
  [1092] = {.lex_state = 132},
  [1093] = {.lex_state = 132},
  [1094] = {.lex_state = 163},
  [1095] = {.lex_state = 132},
  [1096] = {.lex_state = 187, .external_lex_state = 1},
  [1097] = {.lex_state = 0},
  [1098] = {.lex_state = 182, .external_lex_state = 1},
  [1099] = {.lex_state = 187, .external_lex_state = 1},
  [1100] = {.lex_state = 187, .external_lex_state = 1},
  [1101] = {.lex_state = 187, .external_lex_state = 1},
  [1102] = {.lex_state = 81},
  [1103] = {.lex_state = 187, .external_lex_state = 1},
  [1104] = {.lex_state = 187, .external_lex_state = 1},
  [1105] = {.lex_state = 174},
  [1106] = {.lex_state = 187, .external_lex_state = 1},
  [1107] = {.lex_state = 187, .external_lex_state = 1},
  [1108] = {.lex_state = 187, .external_lex_state = 1},
  [1109] = {.lex_state = 111},
  [1110] = {.lex_state = 115, .external_lex_state = 1},
  [1111] = {.lex_state = 111},
  [1112] = {.lex_state = 111},
  [1113] = {.lex_state = 111},
  [1114] = {.lex_state = 171, .external_lex_state = 1},
  [1115] = {.lex_state = 111},
  [1116] = {.lex_state = 171, .external_lex_state = 1},
  [1117] = {.lex_state = 0},
  [1118] = {.lex_state = 171, .external_lex_state = 1},
  [1119] = {.lex_state = 0},
  [1120] = {.lex_state = 171, .external_lex_state = 1},
  [1121] = {.lex_state = 171, .external_lex_state = 1},
  [1122] = {.lex_state = 111},
  [1123] = {.lex_state = 0},
  [1124] = {.lex_state = 181},
  [1125] = {.lex_state = 81},
  [1126] = {.lex_state = 81},
  [1127] = {.lex_state = 81},
  [1128] = {.lex_state = 0},
  [1129] = {.lex_state = 181},
  [1130] = {.lex_state = 181},
  [1131] = {.lex_state = 81},
  [1132] = {.lex_state = 81},
  [1133] = {.lex_state = 81},
  [1134] = {.lex_state = 0},
  [1135] = {.lex_state = 81},
  [1136] = {.lex_state = 81},
  [1137] = {.lex_state = 81},
  [1138] = {.lex_state = 81},
  [1139] = {.lex_state = 111},
  [1140] = {.lex_state = 111},
  [1141] = {.lex_state = 111},
  [1142] = {.lex_state = 111},
  [1143] = {.lex_state = 111},
  [1144] = {.lex_state = 111},
  [1145] = {.lex_state = 111},
  [1146] = {.lex_state = 111},
  [1147] = {.lex_state = 111},
  [1148] = {.lex_state = 181},
  [1149] = {.lex_state = 111},
  [1150] = {.lex_state = 81},
  [1151] = {.lex_state = 111},
  [1152] = {.lex_state = 81},
  [1153] = {.lex_state = 111},
  [1154] = {.lex_state = 81},
  [1155] = {.lex_state = 81},
  [1156] = {.lex_state = 81},
  [1157] = {.lex_state = 81},
  [1158] = {.lex_state = 81},
  [1159] = {.lex_state = 81},
  [1160] = {.lex_state = 190, .external_lex_state = 1},
  [1161] = {.lex_state = 190, .external_lex_state = 1},
  [1162] = {.lex_state = 190, .external_lex_state = 1},
  [1163] = {.lex_state = 81},
  [1164] = {.lex_state = 81},
  [1165] = {.lex_state = 111},
  [1166] = {.lex_state = 81},
  [1167] = {.lex_state = 81},
  [1168] = {.lex_state = 81},
  [1169] = {.lex_state = 81},
  [1170] = {.lex_state = 0},
  [1171] = {.lex_state = 181},
  [1172] = {.lex_state = 181},
  [1173] = {.lex_state = 81},
  [1174] = {.lex_state = 181},
  [1175] = {.lex_state = 181},
  [1176] = {.lex_state = 174},
  [1177] = {.lex_state = 181},
  [1178] = {.lex_state = 181},
  [1179] = {.lex_state = 181},
  [1180] = {.lex_state = 189, .external_lex_state = 1},
  [1181] = {.lex_state = 81},
  [1182] = {.lex_state = 81},
  [1183] = {.lex_state = 0},
  [1184] = {.lex_state = 189, .external_lex_state = 1},
  [1185] = {.lex_state = 189, .external_lex_state = 1},
  [1186] = {.lex_state = 81},
  [1187] = {.lex_state = 81},
  [1188] = {.lex_state = 81},
  [1189] = {.lex_state = 81},
  [1190] = {.lex_state = 81},
  [1191] = {.lex_state = 81},
  [1192] = {.lex_state = 81},
  [1193] = {.lex_state = 111},
  [1194] = {.lex_state = 181},
  [1195] = {.lex_state = 111},
  [1196] = {.lex_state = 111},
  [1197] = {.lex_state = 111},
  [1198] = {.lex_state = 111},
  [1199] = {.lex_state = 183, .external_lex_state = 1},
  [1200] = {.lex_state = 81},
  [1201] = {.lex_state = 111},
  [1202] = {.lex_state = 111},
  [1203] = {.lex_state = 111},
  [1204] = {.lex_state = 190, .external_lex_state = 1},
  [1205] = {.lex_state = 190, .external_lex_state = 1},
  [1206] = {.lex_state = 111},
  [1207] = {.lex_state = 190, .external_lex_state = 1},
  [1208] = {.lex_state = 117, .external_lex_state = 1},
  [1209] = {.lex_state = 187, .external_lex_state = 1},
  [1210] = {.lex_state = 81},
  [1211] = {.lex_state = 81},
  [1212] = {.lex_state = 81},
  [1213] = {.lex_state = 0},
  [1214] = {.lex_state = 0},
  [1215] = {.lex_state = 111},
  [1216] = {.lex_state = 81},
  [1217] = {.lex_state = 111},
  [1218] = {.lex_state = 111},
  [1219] = {.lex_state = 117, .external_lex_state = 1},
  [1220] = {.lex_state = 111},
  [1221] = {.lex_state = 178, .external_lex_state = 1},
  [1222] = {.lex_state = 111},
  [1223] = {.lex_state = 178, .external_lex_state = 1},
  [1224] = {.lex_state = 81},
  [1225] = {.lex_state = 111},
  [1226] = {.lex_state = 111},
  [1227] = {.lex_state = 81},
  [1228] = {.lex_state = 81},
  [1229] = {.lex_state = 138, .external_lex_state = 1},
  [1230] = {.lex_state = 81},
  [1231] = {.lex_state = 162, .external_lex_state = 1},
  [1232] = {.lex_state = 162, .external_lex_state = 1},
  [1233] = {.lex_state = 111},
  [1234] = {.lex_state = 0},
  [1235] = {.lex_state = 111},
  [1236] = {.lex_state = 111},
  [1237] = {.lex_state = 0},
  [1238] = {.lex_state = 0},
  [1239] = {.lex_state = 111},
  [1240] = {.lex_state = 111},
  [1241] = {.lex_state = 111},
  [1242] = {.lex_state = 111},
  [1243] = {.lex_state = 111},
  [1244] = {.lex_state = 111},
  [1245] = {.lex_state = 111},
  [1246] = {.lex_state = 111},
  [1247] = {.lex_state = 0},
  [1248] = {.lex_state = 170, .external_lex_state = 1},
  [1249] = {.lex_state = 0},
  [1250] = {.lex_state = 0},
  [1251] = {.lex_state = 0},
  [1252] = {.lex_state = 0},
  [1253] = {.lex_state = 111},
  [1254] = {.lex_state = 170, .external_lex_state = 1},
  [1255] = {.lex_state = 111},
  [1256] = {.lex_state = 111},
  [1257] = {.lex_state = 81},
  [1258] = {.lex_state = 81},
  [1259] = {.lex_state = 111},
  [1260] = {.lex_state = 179},
  [1261] = {.lex_state = 157},
  [1262] = {.lex_state = 81},
  [1263] = {.lex_state = 81},
  [1264] = {.lex_state = 81},
  [1265] = {.lex_state = 81},
  [1266] = {.lex_state = 81},
  [1267] = {.lex_state = 157},
  [1268] = {.lex_state = 157},
  [1269] = {.lex_state = 111},
  [1270] = {.lex_state = 174},
  [1271] = {.lex_state = 181},
  [1272] = {.lex_state = 181},
  [1273] = {.lex_state = 157},
  [1274] = {.lex_state = 0},
  [1275] = {.lex_state = 111},
  [1276] = {.lex_state = 111},
  [1277] = {.lex_state = 81},
  [1278] = {.lex_state = 81},
  [1279] = {.lex_state = 81},
  [1280] = {.lex_state = 178, .external_lex_state = 1},
  [1281] = {.lex_state = 81},
  [1282] = {.lex_state = 81},
  [1283] = {.lex_state = 81},
  [1284] = {.lex_state = 81},
  [1285] = {.lex_state = 81},
  [1286] = {.lex_state = 178, .external_lex_state = 1},
  [1287] = {.lex_state = 199, .external_lex_state = 1},
  [1288] = {.lex_state = 200, .external_lex_state = 1},
  [1289] = {.lex_state = 81},
  [1290] = {.lex_state = 81},
  [1291] = {.lex_state = 81},
  [1292] = {.lex_state = 81},
  [1293] = {.lex_state = 81},
  [1294] = {.lex_state = 200, .external_lex_state = 1},
  [1295] = {.lex_state = 201, .external_lex_state = 1},
  [1296] = {.lex_state = 111},
  [1297] = {.lex_state = 202, .external_lex_state = 1},
  [1298] = {.lex_state = 203, .external_lex_state = 1},
  [1299] = {.lex_state = 203, .external_lex_state = 1},
  [1300] = {.lex_state = 201, .external_lex_state = 1},
  [1301] = {.lex_state = 0},
  [1302] = {.lex_state = 179},
  [1303] = {.lex_state = 180},
  [1304] = {.lex_state = 81},
  [1305] = {.lex_state = 81},
  [1306] = {.lex_state = 81},
  [1307] = {.lex_state = 81},
  [1308] = {.lex_state = 81},
  [1309] = {.lex_state = 180},
  [1310] = {.lex_state = 180},
  [1311] = {.lex_state = 181},
  [1312] = {.lex_state = 181},
  [1313] = {.lex_state = 181},
  [1314] = {.lex_state = 180},
  [1315] = {.lex_state = 0},
  [1316] = {.lex_state = 81},
  [1317] = {.lex_state = 111},
  [1318] = {.lex_state = 111},
  [1319] = {.lex_state = 181},
  [1320] = {.lex_state = 111},
  [1321] = {.lex_state = 181},
  [1322] = {.lex_state = 181},
  [1323] = {.lex_state = 181},
  [1324] = {.lex_state = 111},
  [1325] = {.lex_state = 181},
  [1326] = {.lex_state = 111},
  [1327] = {.lex_state = 132},
  [1328] = {.lex_state = 181},
  [1329] = {.lex_state = 111},
  [1330] = {.lex_state = 181},
  [1331] = {.lex_state = 111},
  [1332] = {.lex_state = 181},
  [1333] = {.lex_state = 181},
  [1334] = {.lex_state = 0},
  [1335] = {.lex_state = 181},
  [1336] = {.lex_state = 181},
  [1337] = {.lex_state = 181},
  [1338] = {.lex_state = 0},
  [1339] = {.lex_state = 181},
  [1340] = {.lex_state = 181},
  [1341] = {.lex_state = 181},
  [1342] = {.lex_state = 181},
  [1343] = {.lex_state = 111},
  [1344] = {.lex_state = 180},
  [1345] = {.lex_state = 81},
  [1346] = {.lex_state = 111},
  [1347] = {.lex_state = 111},
  [1348] = {.lex_state = 111},
  [1349] = {.lex_state = 111},
  [1350] = {.lex_state = 0},
  [1351] = {.lex_state = 181},
  [1352] = {.lex_state = 0},
  [1353] = {.lex_state = 0},
  [1354] = {.lex_state = 0},
  [1355] = {.lex_state = 0},
  [1356] = {.lex_state = 0},
  [1357] = {.lex_state = 111},
  [1358] = {.lex_state = 81},
  [1359] = {.lex_state = 81},
  [1360] = {.lex_state = 81},
  [1361] = {.lex_state = 81},
  [1362] = {.lex_state = 81},
  [1363] = {.lex_state = 115, .external_lex_state = 1},
  [1364] = {.lex_state = 81},
  [1365] = {.lex_state = 204, .external_lex_state = 1},
  [1366] = {.lex_state = 81},
  [1367] = {.lex_state = 117, .external_lex_state = 1},
  [1368] = {.lex_state = 81},
  [1369] = {.lex_state = 178, .external_lex_state = 1},
  [1370] = {.lex_state = 178, .external_lex_state = 1},
  [1371] = {.lex_state = 81},
  [1372] = {.lex_state = 0},
  [1373] = {.lex_state = 81},
  [1374] = {.lex_state = 81},
  [1375] = {.lex_state = 81},
  [1376] = {.lex_state = 81},
  [1377] = {.lex_state = 81},
  [1378] = {.lex_state = 81},
  [1379] = {.lex_state = 117, .external_lex_state = 1},
  [1380] = {.lex_state = 81},
  [1381] = {.lex_state = 117, .external_lex_state = 1},
  [1382] = {.lex_state = 81},
  [1383] = {.lex_state = 81},
  [1384] = {.lex_state = 117, .external_lex_state = 1},
  [1385] = {.lex_state = 117, .external_lex_state = 1},
  [1386] = {.lex_state = 81},
  [1387] = {.lex_state = 81},
  [1388] = {.lex_state = 81},
  [1389] = {.lex_state = 81},
  [1390] = {.lex_state = 81},
  [1391] = {.lex_state = 190, .external_lex_state = 1},
  [1392] = {.lex_state = 190, .external_lex_state = 1},
  [1393] = {.lex_state = 190, .external_lex_state = 1},
  [1394] = {.lex_state = 81},
  [1395] = {.lex_state = 81},
  [1396] = {.lex_state = 111},
  [1397] = {.lex_state = 81},
  [1398] = {.lex_state = 81},
  [1399] = {.lex_state = 81},
  [1400] = {.lex_state = 81},
  [1401] = {.lex_state = 81},
  [1402] = {.lex_state = 182, .external_lex_state = 1},
  [1403] = {.lex_state = 0},
  [1404] = {.lex_state = 182, .external_lex_state = 1},
  [1405] = {.lex_state = 0},
  [1406] = {.lex_state = 182, .external_lex_state = 1},
  [1407] = {.lex_state = 182, .external_lex_state = 1},
  [1408] = {.lex_state = 111},
  [1409] = {.lex_state = 0},
  [1410] = {.lex_state = 111},
  [1411] = {.lex_state = 111},
  [1412] = {.lex_state = 111},
  [1413] = {.lex_state = 111},
  [1414] = {.lex_state = 181},
  [1415] = {.lex_state = 81},
  [1416] = {.lex_state = 182, .external_lex_state = 1},
  [1417] = {.lex_state = 111},
  [1418] = {.lex_state = 132},
  [1419] = {.lex_state = 132},
  [1420] = {.lex_state = 132},
  [1421] = {.lex_state = 163},
  [1422] = {.lex_state = 132},
  [1423] = {.lex_state = 182, .external_lex_state = 1},
  [1424] = {.lex_state = 182, .external_lex_state = 1},
  [1425] = {.lex_state = 0},
  [1426] = {.lex_state = 192, .external_lex_state = 1},
  [1427] = {.lex_state = 0},
  [1428] = {.lex_state = 111},
  [1429] = {.lex_state = 81},
  [1430] = {.lex_state = 111},
  [1431] = {.lex_state = 81},
  [1432] = {.lex_state = 111},
  [1433] = {.lex_state = 111},
  [1434] = {.lex_state = 111},
  [1435] = {.lex_state = 117, .external_lex_state = 1},
  [1436] = {.lex_state = 111},
  [1437] = {.lex_state = 181},
  [1438] = {.lex_state = 181},
  [1439] = {.lex_state = 181},
  [1440] = {.lex_state = 81},
  [1441] = {.lex_state = 181},
  [1442] = {.lex_state = 181},
  [1443] = {.lex_state = 174},
  [1444] = {.lex_state = 181},
  [1445] = {.lex_state = 181},
  [1446] = {.lex_state = 181},
  [1447] = {.lex_state = 170, .external_lex_state = 1},
  [1448] = {.lex_state = 170, .external_lex_state = 1},
  [1449] = {.lex_state = 170, .external_lex_state = 1},
  [1450] = {.lex_state = 170, .external_lex_state = 1},
  [1451] = {.lex_state = 181},
  [1452] = {.lex_state = 182, .external_lex_state = 1},
  [1453] = {.lex_state = 181},
  [1454] = {.lex_state = 174},
  [1455] = {.lex_state = 174},
  [1456] = {.lex_state = 174},
  [1457] = {.lex_state = 81},
  [1458] = {.lex_state = 174},
  [1459] = {.lex_state = 174},
  [1460] = {.lex_state = 160, .external_lex_state = 1},
  [1461] = {.lex_state = 174},
  [1462] = {.lex_state = 174},
  [1463] = {.lex_state = 174},
  [1464] = {.lex_state = 174},
  [1465] = {.lex_state = 181},
  [1466] = {.lex_state = 81},
  [1467] = {.lex_state = 81},
  [1468] = {.lex_state = 81},
  [1469] = {.lex_state = 0},
  [1470] = {.lex_state = 181},
  [1471] = {.lex_state = 181},
  [1472] = {.lex_state = 81},
  [1473] = {.lex_state = 81},
  [1474] = {.lex_state = 81},
  [1475] = {.lex_state = 81},
  [1476] = {.lex_state = 81},
  [1477] = {.lex_state = 81},
  [1478] = {.lex_state = 81},
  [1479] = {.lex_state = 182, .external_lex_state = 1},
  [1480] = {.lex_state = 182, .external_lex_state = 1},
  [1481] = {.lex_state = 163},
  [1482] = {.lex_state = 111},
  [1483] = {.lex_state = 163},
  [1484] = {.lex_state = 111},
  [1485] = {.lex_state = 111},
  [1486] = {.lex_state = 111},
  [1487] = {.lex_state = 111},
  [1488] = {.lex_state = 111},
  [1489] = {.lex_state = 111},
  [1490] = {.lex_state = 170, .external_lex_state = 1},
  [1491] = {.lex_state = 187, .external_lex_state = 1},
  [1492] = {.lex_state = 81},
  [1493] = {.lex_state = 115, .external_lex_state = 1},
  [1494] = {.lex_state = 81},
  [1495] = {.lex_state = 171, .external_lex_state = 1},
  [1496] = {.lex_state = 171, .external_lex_state = 1},
  [1497] = {.lex_state = 171, .external_lex_state = 1},
  [1498] = {.lex_state = 171, .external_lex_state = 1},
  [1499] = {.lex_state = 171, .external_lex_state = 1},
  [1500] = {.lex_state = 181},
  [1501] = {.lex_state = 181},
  [1502] = {.lex_state = 181},
  [1503] = {.lex_state = 81},
  [1504] = {.lex_state = 181},
  [1505] = {.lex_state = 181},
  [1506] = {.lex_state = 174},
  [1507] = {.lex_state = 181},
  [1508] = {.lex_state = 181},
  [1509] = {.lex_state = 181},
  [1510] = {.lex_state = 111},
  [1511] = {.lex_state = 111},
  [1512] = {.lex_state = 111},
  [1513] = {.lex_state = 111},
  [1514] = {.lex_state = 111},
  [1515] = {.lex_state = 181},
  [1516] = {.lex_state = 111},
  [1517] = {.lex_state = 111},
  [1518] = {.lex_state = 111},
  [1519] = {.lex_state = 111},
  [1520] = {.lex_state = 111},
  [1521] = {.lex_state = 81},
  [1522] = {.lex_state = 81},
  [1523] = {.lex_state = 138, .external_lex_state = 1},
  [1524] = {.lex_state = 81},
  [1525] = {.lex_state = 162, .external_lex_state = 1},
  [1526] = {.lex_state = 162, .external_lex_state = 1},
  [1527] = {.lex_state = 81},
  [1528] = {.lex_state = 81},
  [1529] = {.lex_state = 81},
  [1530] = {.lex_state = 81},
  [1531] = {.lex_state = 81},
  [1532] = {.lex_state = 81},
  [1533] = {.lex_state = 111},
  [1534] = {.lex_state = 181},
  [1535] = {.lex_state = 81},
  [1536] = {.lex_state = 111},
  [1537] = {.lex_state = 111},
  [1538] = {.lex_state = 181},
  [1539] = {.lex_state = 111},
  [1540] = {.lex_state = 181},
  [1541] = {.lex_state = 111},
  [1542] = {.lex_state = 181},
  [1543] = {.lex_state = 81},
  [1544] = {.lex_state = 189, .external_lex_state = 1},
  [1545] = {.lex_state = 189, .external_lex_state = 1},
  [1546] = {.lex_state = 81},
  [1547] = {.lex_state = 189, .external_lex_state = 1},
  [1548] = {.lex_state = 189, .external_lex_state = 1},
  [1549] = {.lex_state = 174},
  [1550] = {.lex_state = 189, .external_lex_state = 1},
  [1551] = {.lex_state = 189, .external_lex_state = 1},
  [1552] = {.lex_state = 189, .external_lex_state = 1},
  [1553] = {.lex_state = 111},
  [1554] = {.lex_state = 111},
  [1555] = {.lex_state = 81},
  [1556] = {.lex_state = 81},
  [1557] = {.lex_state = 190, .external_lex_state = 1},
  [1558] = {.lex_state = 111},
  [1559] = {.lex_state = 111},
  [1560] = {.lex_state = 0},
  [1561] = {.lex_state = 171, .external_lex_state = 1},
  [1562] = {.lex_state = 0},
  [1563] = {.lex_state = 0},
  [1564] = {.lex_state = 0},
  [1565] = {.lex_state = 0},
  [1566] = {.lex_state = 111},
  [1567] = {.lex_state = 171, .external_lex_state = 1},
  [1568] = {.lex_state = 81},
  [1569] = {.lex_state = 111},
  [1570] = {.lex_state = 111},
  [1571] = {.lex_state = 111},
  [1572] = {.lex_state = 81},
  [1573] = {.lex_state = 111},
  [1574] = {.lex_state = 111},
  [1575] = {.lex_state = 190, .external_lex_state = 1},
  [1576] = {.lex_state = 190, .external_lex_state = 1},
  [1577] = {.lex_state = 190, .external_lex_state = 1},
  [1578] = {.lex_state = 81},
  [1579] = {.lex_state = 81},
  [1580] = {.lex_state = 111},
  [1581] = {.lex_state = 81},
  [1582] = {.lex_state = 81},
  [1583] = {.lex_state = 81},
  [1584] = {.lex_state = 81},
  [1585] = {.lex_state = 117, .external_lex_state = 1},
  [1586] = {.lex_state = 111},
  [1587] = {.lex_state = 111},
  [1588] = {.lex_state = 81},
  [1589] = {.lex_state = 111},
  [1590] = {.lex_state = 111},
  [1591] = {.lex_state = 111},
  [1592] = {.lex_state = 111},
  [1593] = {.lex_state = 170, .external_lex_state = 1},
  [1594] = {.lex_state = 111},
  [1595] = {.lex_state = 170, .external_lex_state = 1},
  [1596] = {.lex_state = 0},
  [1597] = {.lex_state = 170, .external_lex_state = 1},
  [1598] = {.lex_state = 0},
  [1599] = {.lex_state = 170, .external_lex_state = 1},
  [1600] = {.lex_state = 170, .external_lex_state = 1},
  [1601] = {.lex_state = 111},
  [1602] = {.lex_state = 187, .external_lex_state = 1},
  [1603] = {.lex_state = 0},
  [1604] = {.lex_state = 111},
  [1605] = {.lex_state = 111},
  [1606] = {.lex_state = 111},
  [1607] = {.lex_state = 81},
  [1608] = {.lex_state = 81},
  [1609] = {.lex_state = 0},
  [1610] = {.lex_state = 0},
  [1611] = {.lex_state = 0},
  [1612] = {.lex_state = 0},
  [1613] = {.lex_state = 81},
  [1614] = {.lex_state = 0},
  [1615] = {.lex_state = 0},
  [1616] = {.lex_state = 115, .external_lex_state = 1},
  [1617] = {.lex_state = 115, .external_lex_state = 1},
  [1618] = {.lex_state = 117, .external_lex_state = 1},
  [1619] = {.lex_state = 118, .external_lex_state = 1},
  [1620] = {.lex_state = 81},
  [1621] = {.lex_state = 111},
  [1622] = {.lex_state = 81},
  [1623] = {.lex_state = 138, .external_lex_state = 1},
  [1624] = {.lex_state = 111},
  [1625] = {.lex_state = 157, .external_lex_state = 1},
  [1626] = {.lex_state = 117, .external_lex_state = 1},
  [1627] = {.lex_state = 170, .external_lex_state = 1},
  [1628] = {.lex_state = 170, .external_lex_state = 1},
  [1629] = {.lex_state = 111},
  [1630] = {.lex_state = 81},
  [1631] = {.lex_state = 174},
  [1632] = {.lex_state = 81},
  [1633] = {.lex_state = 81},
  [1634] = {.lex_state = 81},
  [1635] = {.lex_state = 0},
  [1636] = {.lex_state = 174},
  [1637] = {.lex_state = 174},
  [1638] = {.lex_state = 81},
  [1639] = {.lex_state = 81},
  [1640] = {.lex_state = 111},
  [1641] = {.lex_state = 81},
  [1642] = {.lex_state = 81},
  [1643] = {.lex_state = 81},
  [1644] = {.lex_state = 81},
  [1645] = {.lex_state = 81},
  [1646] = {.lex_state = 81},
  [1647] = {.lex_state = 111},
  [1648] = {.lex_state = 181},
  [1649] = {.lex_state = 81},
  [1650] = {.lex_state = 111},
  [1651] = {.lex_state = 181},
  [1652] = {.lex_state = 111},
  [1653] = {.lex_state = 181},
  [1654] = {.lex_state = 81},
  [1655] = {.lex_state = 202, .external_lex_state = 1},
  [1656] = {.lex_state = 81},
  [1657] = {.lex_state = 81},
  [1658] = {.lex_state = 81},
  [1659] = {.lex_state = 0},
  [1660] = {.lex_state = 202, .external_lex_state = 1},
  [1661] = {.lex_state = 202, .external_lex_state = 1},
  [1662] = {.lex_state = 81},
  [1663] = {.lex_state = 81},
  [1664] = {.lex_state = 81},
  [1665] = {.lex_state = 81},
  [1666] = {.lex_state = 81},
  [1667] = {.lex_state = 81},
  [1668] = {.lex_state = 81},
  [1669] = {.lex_state = 81},
  [1670] = {.lex_state = 181},
  [1671] = {.lex_state = 81},
  [1672] = {.lex_state = 81},
  [1673] = {.lex_state = 81},
  [1674] = {.lex_state = 0},
  [1675] = {.lex_state = 181},
  [1676] = {.lex_state = 181},
  [1677] = {.lex_state = 81},
  [1678] = {.lex_state = 81},
  [1679] = {.lex_state = 81},
  [1680] = {.lex_state = 81},
  [1681] = {.lex_state = 81},
  [1682] = {.lex_state = 81},
  [1683] = {.lex_state = 81},
  [1684] = {.lex_state = 81},
  [1685] = {.lex_state = 111},
  [1686] = {.lex_state = 81},
  [1687] = {.lex_state = 81},
  [1688] = {.lex_state = 81},
  [1689] = {.lex_state = 81},
  [1690] = {.lex_state = 81},
  [1691] = {.lex_state = 181},
  [1692] = {.lex_state = 181},
  [1693] = {.lex_state = 111},
  [1694] = {.lex_state = 181},
  [1695] = {.lex_state = 181},
  [1696] = {.lex_state = 181},
  [1697] = {.lex_state = 181},
  [1698] = {.lex_state = 181},
  [1699] = {.lex_state = 181},
  [1700] = {.lex_state = 181},
  [1701] = {.lex_state = 181},
  [1702] = {.lex_state = 181},
  [1703] = {.lex_state = 0},
  [1704] = {.lex_state = 0},
  [1705] = {.lex_state = 0},
  [1706] = {.lex_state = 117, .external_lex_state = 1},
  [1707] = {.lex_state = 187, .external_lex_state = 1},
  [1708] = {.lex_state = 81},
  [1709] = {.lex_state = 81},
  [1710] = {.lex_state = 81},
  [1711] = {.lex_state = 117, .external_lex_state = 1},
  [1712] = {.lex_state = 81},
  [1713] = {.lex_state = 178, .external_lex_state = 1},
  [1714] = {.lex_state = 81},
  [1715] = {.lex_state = 178, .external_lex_state = 1},
  [1716] = {.lex_state = 81},
  [1717] = {.lex_state = 111},
  [1718] = {.lex_state = 81},
  [1719] = {.lex_state = 81},
  [1720] = {.lex_state = 81},
  [1721] = {.lex_state = 138, .external_lex_state = 1},
  [1722] = {.lex_state = 81},
  [1723] = {.lex_state = 162, .external_lex_state = 1},
  [1724] = {.lex_state = 162, .external_lex_state = 1},
  [1725] = {.lex_state = 81},
  [1726] = {.lex_state = 0},
  [1727] = {.lex_state = 0},
  [1728] = {.lex_state = 0},
  [1729] = {.lex_state = 81},
  [1730] = {.lex_state = 81},
  [1731] = {.lex_state = 81},
  [1732] = {.lex_state = 81},
  [1733] = {.lex_state = 81},
  [1734] = {.lex_state = 81},
  [1735] = {.lex_state = 81},
  [1736] = {.lex_state = 81},
  [1737] = {.lex_state = 81},
  [1738] = {.lex_state = 81},
  [1739] = {.lex_state = 81},
  [1740] = {.lex_state = 81},
  [1741] = {.lex_state = 81},
  [1742] = {.lex_state = 81},
  [1743] = {.lex_state = 81},
  [1744] = {.lex_state = 111},
  [1745] = {.lex_state = 181},
  [1746] = {.lex_state = 81},
  [1747] = {.lex_state = 111},
  [1748] = {.lex_state = 111},
  [1749] = {.lex_state = 181},
  [1750] = {.lex_state = 111},
  [1751] = {.lex_state = 181},
  [1752] = {.lex_state = 111},
  [1753] = {.lex_state = 182, .external_lex_state = 1},
  [1754] = {.lex_state = 182, .external_lex_state = 1},
  [1755] = {.lex_state = 182, .external_lex_state = 1},
  [1756] = {.lex_state = 111},
  [1757] = {.lex_state = 0},
  [1758] = {.lex_state = 181},
  [1759] = {.lex_state = 182, .external_lex_state = 1},
  [1760] = {.lex_state = 163},
  [1761] = {.lex_state = 132},
  [1762] = {.lex_state = 132},
  [1763] = {.lex_state = 132},
  [1764] = {.lex_state = 132},
  [1765] = {.lex_state = 132},
  [1766] = {.lex_state = 132},
  [1767] = {.lex_state = 163},
  [1768] = {.lex_state = 0},
  [1769] = {.lex_state = 111},
  [1770] = {.lex_state = 181},
  [1771] = {.lex_state = 111},
  [1772] = {.lex_state = 111},
  [1773] = {.lex_state = 111},
  [1774] = {.lex_state = 111},
  [1775] = {.lex_state = 111},
  [1776] = {.lex_state = 111},
  [1777] = {.lex_state = 111},
  [1778] = {.lex_state = 181},
  [1779] = {.lex_state = 81},
  [1780] = {.lex_state = 174},
  [1781] = {.lex_state = 81},
  [1782] = {.lex_state = 181},
  [1783] = {.lex_state = 181},
  [1784] = {.lex_state = 181},
  [1785] = {.lex_state = 81},
  [1786] = {.lex_state = 181},
  [1787] = {.lex_state = 181},
  [1788] = {.lex_state = 174},
  [1789] = {.lex_state = 181},
  [1790] = {.lex_state = 181},
  [1791] = {.lex_state = 181},
  [1792] = {.lex_state = 163},
  [1793] = {.lex_state = 163},
  [1794] = {.lex_state = 163},
  [1795] = {.lex_state = 163},
  [1796] = {.lex_state = 187, .external_lex_state = 1},
  [1797] = {.lex_state = 181},
  [1798] = {.lex_state = 81},
  [1799] = {.lex_state = 111},
  [1800] = {.lex_state = 111},
  [1801] = {.lex_state = 111},
  [1802] = {.lex_state = 81},
  [1803] = {.lex_state = 190, .external_lex_state = 1},
  [1804] = {.lex_state = 190, .external_lex_state = 1},
  [1805] = {.lex_state = 190, .external_lex_state = 1},
  [1806] = {.lex_state = 81},
  [1807] = {.lex_state = 81},
  [1808] = {.lex_state = 111},
  [1809] = {.lex_state = 81},
  [1810] = {.lex_state = 81},
  [1811] = {.lex_state = 81},
  [1812] = {.lex_state = 81},
  [1813] = {.lex_state = 111},
  [1814] = {.lex_state = 181},
  [1815] = {.lex_state = 111},
  [1816] = {.lex_state = 181},
  [1817] = {.lex_state = 111},
  [1818] = {.lex_state = 181},
  [1819] = {.lex_state = 81},
  [1820] = {.lex_state = 81},
  [1821] = {.lex_state = 81},
  [1822] = {.lex_state = 81},
  [1823] = {.lex_state = 81},
  [1824] = {.lex_state = 81},
  [1825] = {.lex_state = 81},
  [1826] = {.lex_state = 181},
  [1827] = {.lex_state = 189, .external_lex_state = 1},
  [1828] = {.lex_state = 81},
  [1829] = {.lex_state = 111},
  [1830] = {.lex_state = 171, .external_lex_state = 1},
  [1831] = {.lex_state = 111},
  [1832] = {.lex_state = 171, .external_lex_state = 1},
  [1833] = {.lex_state = 0},
  [1834] = {.lex_state = 171, .external_lex_state = 1},
  [1835] = {.lex_state = 0},
  [1836] = {.lex_state = 171, .external_lex_state = 1},
  [1837] = {.lex_state = 171, .external_lex_state = 1},
  [1838] = {.lex_state = 111},
  [1839] = {.lex_state = 0},
  [1840] = {.lex_state = 111},
  [1841] = {.lex_state = 111},
  [1842] = {.lex_state = 81},
  [1843] = {.lex_state = 81},
  [1844] = {.lex_state = 81},
  [1845] = {.lex_state = 81},
  [1846] = {.lex_state = 81},
  [1847] = {.lex_state = 81},
  [1848] = {.lex_state = 111},
  [1849] = {.lex_state = 181},
  [1850] = {.lex_state = 81},
  [1851] = {.lex_state = 111},
  [1852] = {.lex_state = 111},
  [1853] = {.lex_state = 181},
  [1854] = {.lex_state = 111},
  [1855] = {.lex_state = 181},
  [1856] = {.lex_state = 111},
  [1857] = {.lex_state = 111},
  [1858] = {.lex_state = 111},
  [1859] = {.lex_state = 111},
  [1860] = {.lex_state = 111},
  [1861] = {.lex_state = 170, .external_lex_state = 1},
  [1862] = {.lex_state = 170, .external_lex_state = 1},
  [1863] = {.lex_state = 170, .external_lex_state = 1},
  [1864] = {.lex_state = 170, .external_lex_state = 1},
  [1865] = {.lex_state = 111},
  [1866] = {.lex_state = 170, .external_lex_state = 1},
  [1867] = {.lex_state = 81},
  [1868] = {.lex_state = 81},
  [1869] = {.lex_state = 81},
  [1870] = {.lex_state = 81},
  [1871] = {.lex_state = 111},
  [1872] = {.lex_state = 115, .external_lex_state = 1},
  [1873] = {.lex_state = 111},
  [1874] = {.lex_state = 171, .external_lex_state = 1},
  [1875] = {.lex_state = 81},
  [1876] = {.lex_state = 172, .external_lex_state = 1},
  [1877] = {.lex_state = 111},
  [1878] = {.lex_state = 117, .external_lex_state = 1},
  [1879] = {.lex_state = 81},
  [1880] = {.lex_state = 178, .external_lex_state = 1},
  [1881] = {.lex_state = 178, .external_lex_state = 1},
  [1882] = {.lex_state = 81},
  [1883] = {.lex_state = 0},
  [1884] = {.lex_state = 81},
  [1885] = {.lex_state = 81},
  [1886] = {.lex_state = 81},
  [1887] = {.lex_state = 111},
  [1888] = {.lex_state = 111},
  [1889] = {.lex_state = 81},
  [1890] = {.lex_state = 111},
  [1891] = {.lex_state = 117, .external_lex_state = 1},
  [1892] = {.lex_state = 111},
  [1893] = {.lex_state = 117, .external_lex_state = 1},
  [1894] = {.lex_state = 111},
  [1895] = {.lex_state = 111},
  [1896] = {.lex_state = 117, .external_lex_state = 1},
  [1897] = {.lex_state = 117, .external_lex_state = 1},
  [1898] = {.lex_state = 81},
  [1899] = {.lex_state = 81},
  [1900] = {.lex_state = 81},
  [1901] = {.lex_state = 0},
  [1902] = {.lex_state = 0},
  [1903] = {.lex_state = 81},
  [1904] = {.lex_state = 111},
  [1905] = {.lex_state = 111},
  [1906] = {.lex_state = 111},
  [1907] = {.lex_state = 81},
  [1908] = {.lex_state = 81},
  [1909] = {.lex_state = 174},
  [1910] = {.lex_state = 174},
  [1911] = {.lex_state = 174},
  [1912] = {.lex_state = 81},
  [1913] = {.lex_state = 174},
  [1914] = {.lex_state = 174},
  [1915] = {.lex_state = 111},
  [1916] = {.lex_state = 174},
  [1917] = {.lex_state = 174},
  [1918] = {.lex_state = 174},
  [1919] = {.lex_state = 174},
  [1920] = {.lex_state = 174},
  [1921] = {.lex_state = 81},
  [1922] = {.lex_state = 81},
  [1923] = {.lex_state = 81},
  [1924] = {.lex_state = 81},
  [1925] = {.lex_state = 81},
  [1926] = {.lex_state = 81},
  [1927] = {.lex_state = 202, .external_lex_state = 1},
  [1928] = {.lex_state = 202, .external_lex_state = 1},
  [1929] = {.lex_state = 202, .external_lex_state = 1},
  [1930] = {.lex_state = 81},
  [1931] = {.lex_state = 202, .external_lex_state = 1},
  [1932] = {.lex_state = 202, .external_lex_state = 1},
  [1933] = {.lex_state = 111},
  [1934] = {.lex_state = 174},
  [1935] = {.lex_state = 202, .external_lex_state = 1},
  [1936] = {.lex_state = 202, .external_lex_state = 1},
  [1937] = {.lex_state = 202, .external_lex_state = 1},
  [1938] = {.lex_state = 181},
  [1939] = {.lex_state = 181},
  [1940] = {.lex_state = 181},
  [1941] = {.lex_state = 81},
  [1942] = {.lex_state = 181},
  [1943] = {.lex_state = 181},
  [1944] = {.lex_state = 111},
  [1945] = {.lex_state = 174},
  [1946] = {.lex_state = 181},
  [1947] = {.lex_state = 181},
  [1948] = {.lex_state = 181},
  [1949] = {.lex_state = 111},
  [1950] = {.lex_state = 111},
  [1951] = {.lex_state = 111},
  [1952] = {.lex_state = 111},
  [1953] = {.lex_state = 111},
  [1954] = {.lex_state = 181},
  [1955] = {.lex_state = 81},
  [1956] = {.lex_state = 81},
  [1957] = {.lex_state = 81},
  [1958] = {.lex_state = 81},
  [1959] = {.lex_state = 81},
  [1960] = {.lex_state = 81},
  [1961] = {.lex_state = 111},
  [1962] = {.lex_state = 190, .external_lex_state = 1},
  [1963] = {.lex_state = 190, .external_lex_state = 1},
  [1964] = {.lex_state = 190, .external_lex_state = 1},
  [1965] = {.lex_state = 81},
  [1966] = {.lex_state = 81},
  [1967] = {.lex_state = 111},
  [1968] = {.lex_state = 81},
  [1969] = {.lex_state = 81},
  [1970] = {.lex_state = 81},
  [1971] = {.lex_state = 81},
  [1972] = {.lex_state = 117, .external_lex_state = 1},
  [1973] = {.lex_state = 81},
  [1974] = {.lex_state = 81},
  [1975] = {.lex_state = 81},
  [1976] = {.lex_state = 81},
  [1977] = {.lex_state = 187, .external_lex_state = 1},
  [1978] = {.lex_state = 111},
  [1979] = {.lex_state = 181},
  [1980] = {.lex_state = 111},
  [1981] = {.lex_state = 181},
  [1982] = {.lex_state = 111},
  [1983] = {.lex_state = 181},
  [1984] = {.lex_state = 81},
  [1985] = {.lex_state = 81},
  [1986] = {.lex_state = 81},
  [1987] = {.lex_state = 81},
  [1988] = {.lex_state = 81},
  [1989] = {.lex_state = 81},
  [1990] = {.lex_state = 81},
  [1991] = {.lex_state = 0},
  [1992] = {.lex_state = 111},
  [1993] = {.lex_state = 132},
  [1994] = {.lex_state = 163},
  [1995] = {.lex_state = 111},
  [1996] = {.lex_state = 132},
  [1997] = {.lex_state = 111},
  [1998] = {.lex_state = 132},
  [1999] = {.lex_state = 132},
  [2000] = {.lex_state = 132},
  [2001] = {.lex_state = 111},
  [2002] = {.lex_state = 111},
  [2003] = {.lex_state = 111},
  [2004] = {.lex_state = 181},
  [2005] = {.lex_state = 174},
  [2006] = {.lex_state = 181},
  [2007] = {.lex_state = 81},
  [2008] = {.lex_state = 181},
  [2009] = {.lex_state = 111},
  [2010] = {.lex_state = 81},
  [2011] = {.lex_state = 81},
  [2012] = {.lex_state = 81},
  [2013] = {.lex_state = 81},
  [2014] = {.lex_state = 81},
  [2015] = {.lex_state = 81},
  [2016] = {.lex_state = 111},
  [2017] = {.lex_state = 181},
  [2018] = {.lex_state = 81},
  [2019] = {.lex_state = 111},
  [2020] = {.lex_state = 111},
  [2021] = {.lex_state = 181},
  [2022] = {.lex_state = 111},
  [2023] = {.lex_state = 181},
  [2024] = {.lex_state = 81},
  [2025] = {.lex_state = 81},
  [2026] = {.lex_state = 81},
  [2027] = {.lex_state = 81},
  [2028] = {.lex_state = 81},
  [2029] = {.lex_state = 81},
  [2030] = {.lex_state = 189, .external_lex_state = 1},
  [2031] = {.lex_state = 171, .external_lex_state = 1},
  [2032] = {.lex_state = 171, .external_lex_state = 1},
  [2033] = {.lex_state = 171, .external_lex_state = 1},
  [2034] = {.lex_state = 171, .external_lex_state = 1},
  [2035] = {.lex_state = 171, .external_lex_state = 1},
  [2036] = {.lex_state = 111},
  [2037] = {.lex_state = 181},
  [2038] = {.lex_state = 111},
  [2039] = {.lex_state = 181},
  [2040] = {.lex_state = 111},
  [2041] = {.lex_state = 181},
  [2042] = {.lex_state = 81},
  [2043] = {.lex_state = 81},
  [2044] = {.lex_state = 111},
  [2045] = {.lex_state = 81},
  [2046] = {.lex_state = 81},
  [2047] = {.lex_state = 81},
  [2048] = {.lex_state = 81},
  [2049] = {.lex_state = 81},
  [2050] = {.lex_state = 111},
  [2051] = {.lex_state = 0},
  [2052] = {.lex_state = 117, .external_lex_state = 1},
  [2053] = {.lex_state = 187, .external_lex_state = 1},
  [2054] = {.lex_state = 81},
  [2055] = {.lex_state = 81},
  [2056] = {.lex_state = 81},
  [2057] = {.lex_state = 0},
  [2058] = {.lex_state = 0},
  [2059] = {.lex_state = 111},
  [2060] = {.lex_state = 81},
  [2061] = {.lex_state = 111},
  [2062] = {.lex_state = 111},
  [2063] = {.lex_state = 117, .external_lex_state = 1},
  [2064] = {.lex_state = 111},
  [2065] = {.lex_state = 178, .external_lex_state = 1},
  [2066] = {.lex_state = 111},
  [2067] = {.lex_state = 178, .external_lex_state = 1},
  [2068] = {.lex_state = 111},
  [2069] = {.lex_state = 111},
  [2070] = {.lex_state = 111},
  [2071] = {.lex_state = 81},
  [2072] = {.lex_state = 81},
  [2073] = {.lex_state = 0},
  [2074] = {.lex_state = 0},
  [2075] = {.lex_state = 0},
  [2076] = {.lex_state = 0},
  [2077] = {.lex_state = 81},
  [2078] = {.lex_state = 0},
  [2079] = {.lex_state = 0},
  [2080] = {.lex_state = 115, .external_lex_state = 1},
  [2081] = {.lex_state = 115, .external_lex_state = 1},
  [2082] = {.lex_state = 117, .external_lex_state = 1},
  [2083] = {.lex_state = 118, .external_lex_state = 1},
  [2084] = {.lex_state = 81},
  [2085] = {.lex_state = 111},
  [2086] = {.lex_state = 81},
  [2087] = {.lex_state = 138, .external_lex_state = 1},
  [2088] = {.lex_state = 111},
  [2089] = {.lex_state = 157, .external_lex_state = 1},
  [2090] = {.lex_state = 111},
  [2091] = {.lex_state = 117, .external_lex_state = 1},
  [2092] = {.lex_state = 170, .external_lex_state = 1},
  [2093] = {.lex_state = 170, .external_lex_state = 1},
  [2094] = {.lex_state = 81},
  [2095] = {.lex_state = 111},
  [2096] = {.lex_state = 111},
  [2097] = {.lex_state = 81},
  [2098] = {.lex_state = 81},
  [2099] = {.lex_state = 138, .external_lex_state = 1},
  [2100] = {.lex_state = 81},
  [2101] = {.lex_state = 162, .external_lex_state = 1},
  [2102] = {.lex_state = 162, .external_lex_state = 1},
  [2103] = {.lex_state = 111},
  [2104] = {.lex_state = 0},
  [2105] = {.lex_state = 111},
  [2106] = {.lex_state = 111},
  [2107] = {.lex_state = 0},
  [2108] = {.lex_state = 0},
  [2109] = {.lex_state = 111},
  [2110] = {.lex_state = 111},
  [2111] = {.lex_state = 111},
  [2112] = {.lex_state = 111},
  [2113] = {.lex_state = 111},
  [2114] = {.lex_state = 111},
  [2115] = {.lex_state = 111},
  [2116] = {.lex_state = 111},
  [2117] = {.lex_state = 0},
  [2118] = {.lex_state = 170, .external_lex_state = 1},
  [2119] = {.lex_state = 0},
  [2120] = {.lex_state = 0},
  [2121] = {.lex_state = 0},
  [2122] = {.lex_state = 0},
  [2123] = {.lex_state = 111},
  [2124] = {.lex_state = 170, .external_lex_state = 1},
  [2125] = {.lex_state = 111},
  [2126] = {.lex_state = 81},
  [2127] = {.lex_state = 81},
  [2128] = {.lex_state = 174},
  [2129] = {.lex_state = 81},
  [2130] = {.lex_state = 111},
  [2131] = {.lex_state = 111},
  [2132] = {.lex_state = 111},
  [2133] = {.lex_state = 111},
  [2134] = {.lex_state = 111},
  [2135] = {.lex_state = 111},
  [2136] = {.lex_state = 202, .external_lex_state = 1},
  [2137] = {.lex_state = 81},
  [2138] = {.lex_state = 181},
  [2139] = {.lex_state = 81},
  [2140] = {.lex_state = 81},
  [2141] = {.lex_state = 81},
  [2142] = {.lex_state = 81},
  [2143] = {.lex_state = 81},
  [2144] = {.lex_state = 81},
  [2145] = {.lex_state = 81},
  [2146] = {.lex_state = 81},
  [2147] = {.lex_state = 81},
  [2148] = {.lex_state = 111},
  [2149] = {.lex_state = 181},
  [2150] = {.lex_state = 81},
  [2151] = {.lex_state = 111},
  [2152] = {.lex_state = 111},
  [2153] = {.lex_state = 181},
  [2154] = {.lex_state = 111},
  [2155] = {.lex_state = 181},
  [2156] = {.lex_state = 81},
  [2157] = {.lex_state = 111},
  [2158] = {.lex_state = 81},
  [2159] = {.lex_state = 81},
  [2160] = {.lex_state = 81},
  [2161] = {.lex_state = 81},
  [2162] = {.lex_state = 81},
  [2163] = {.lex_state = 81},
  [2164] = {.lex_state = 81},
  [2165] = {.lex_state = 111},
  [2166] = {.lex_state = 132},
  [2167] = {.lex_state = 132},
  [2168] = {.lex_state = 111},
  [2169] = {.lex_state = 132},
  [2170] = {.lex_state = 111},
  [2171] = {.lex_state = 111},
  [2172] = {.lex_state = 111},
  [2173] = {.lex_state = 111},
  [2174] = {.lex_state = 111},
  [2175] = {.lex_state = 111},
  [2176] = {.lex_state = 181},
  [2177] = {.lex_state = 111},
  [2178] = {.lex_state = 181},
  [2179] = {.lex_state = 111},
  [2180] = {.lex_state = 181},
  [2181] = {.lex_state = 111},
  [2182] = {.lex_state = 181},
  [2183] = {.lex_state = 81},
  [2184] = {.lex_state = 81},
  [2185] = {.lex_state = 81},
  [2186] = {.lex_state = 81},
  [2187] = {.lex_state = 81},
  [2188] = {.lex_state = 81},
  [2189] = {.lex_state = 81},
  [2190] = {.lex_state = 81},
  [2191] = {.lex_state = 81},
  [2192] = {.lex_state = 81},
  [2193] = {.lex_state = 81},
  [2194] = {.lex_state = 81},
  [2195] = {.lex_state = 81},
  [2196] = {.lex_state = 111},
  [2197] = {.lex_state = 111},
  [2198] = {.lex_state = 111},
  [2199] = {.lex_state = 111},
  [2200] = {.lex_state = 111},
  [2201] = {.lex_state = 111},
  [2202] = {.lex_state = 111},
  [2203] = {.lex_state = 111},
  [2204] = {.lex_state = 111},
  [2205] = {.lex_state = 111},
  [2206] = {.lex_state = 0},
  [2207] = {.lex_state = 171, .external_lex_state = 1},
  [2208] = {.lex_state = 0},
  [2209] = {.lex_state = 0},
  [2210] = {.lex_state = 0},
  [2211] = {.lex_state = 0},
  [2212] = {.lex_state = 111},
  [2213] = {.lex_state = 171, .external_lex_state = 1},
  [2214] = {.lex_state = 81},
  [2215] = {.lex_state = 111},
  [2216] = {.lex_state = 111},
  [2217] = {.lex_state = 111},
  [2218] = {.lex_state = 81},
  [2219] = {.lex_state = 81},
  [2220] = {.lex_state = 81},
  [2221] = {.lex_state = 81},
  [2222] = {.lex_state = 111},
  [2223] = {.lex_state = 115, .external_lex_state = 1},
  [2224] = {.lex_state = 111},
  [2225] = {.lex_state = 171, .external_lex_state = 1},
  [2226] = {.lex_state = 81},
  [2227] = {.lex_state = 172, .external_lex_state = 1},
  [2228] = {.lex_state = 111},
  [2229] = {.lex_state = 117, .external_lex_state = 1},
  [2230] = {.lex_state = 81},
  [2231] = {.lex_state = 178, .external_lex_state = 1},
  [2232] = {.lex_state = 178, .external_lex_state = 1},
  [2233] = {.lex_state = 81},
  [2234] = {.lex_state = 0},
  [2235] = {.lex_state = 81},
  [2236] = {.lex_state = 81},
  [2237] = {.lex_state = 81},
  [2238] = {.lex_state = 111},
  [2239] = {.lex_state = 111},
  [2240] = {.lex_state = 81},
  [2241] = {.lex_state = 111},
  [2242] = {.lex_state = 117, .external_lex_state = 1},
  [2243] = {.lex_state = 111},
  [2244] = {.lex_state = 117, .external_lex_state = 1},
  [2245] = {.lex_state = 111},
  [2246] = {.lex_state = 111},
  [2247] = {.lex_state = 117, .external_lex_state = 1},
  [2248] = {.lex_state = 117, .external_lex_state = 1},
  [2249] = {.lex_state = 81},
  [2250] = {.lex_state = 81},
  [2251] = {.lex_state = 81},
  [2252] = {.lex_state = 0},
  [2253] = {.lex_state = 0},
  [2254] = {.lex_state = 81},
  [2255] = {.lex_state = 81},
  [2256] = {.lex_state = 111},
  [2257] = {.lex_state = 111},
  [2258] = {.lex_state = 81},
  [2259] = {.lex_state = 81},
  [2260] = {.lex_state = 111},
  [2261] = {.lex_state = 111},
  [2262] = {.lex_state = 190, .external_lex_state = 1},
  [2263] = {.lex_state = 190, .external_lex_state = 1},
  [2264] = {.lex_state = 190, .external_lex_state = 1},
  [2265] = {.lex_state = 81},
  [2266] = {.lex_state = 81},
  [2267] = {.lex_state = 111},
  [2268] = {.lex_state = 81},
  [2269] = {.lex_state = 81},
  [2270] = {.lex_state = 81},
  [2271] = {.lex_state = 81},
  [2272] = {.lex_state = 117, .external_lex_state = 1},
  [2273] = {.lex_state = 111},
  [2274] = {.lex_state = 111},
  [2275] = {.lex_state = 81},
  [2276] = {.lex_state = 111},
  [2277] = {.lex_state = 111},
  [2278] = {.lex_state = 111},
  [2279] = {.lex_state = 111},
  [2280] = {.lex_state = 170, .external_lex_state = 1},
  [2281] = {.lex_state = 111},
  [2282] = {.lex_state = 170, .external_lex_state = 1},
  [2283] = {.lex_state = 0},
  [2284] = {.lex_state = 170, .external_lex_state = 1},
  [2285] = {.lex_state = 0},
  [2286] = {.lex_state = 170, .external_lex_state = 1},
  [2287] = {.lex_state = 170, .external_lex_state = 1},
  [2288] = {.lex_state = 111},
  [2289] = {.lex_state = 187, .external_lex_state = 1},
  [2290] = {.lex_state = 0},
  [2291] = {.lex_state = 174},
  [2292] = {.lex_state = 202, .external_lex_state = 1},
  [2293] = {.lex_state = 181},
  [2294] = {.lex_state = 111},
  [2295] = {.lex_state = 181},
  [2296] = {.lex_state = 111},
  [2297] = {.lex_state = 181},
  [2298] = {.lex_state = 111},
  [2299] = {.lex_state = 181},
  [2300] = {.lex_state = 81},
  [2301] = {.lex_state = 81},
  [2302] = {.lex_state = 81},
  [2303] = {.lex_state = 81},
  [2304] = {.lex_state = 81},
  [2305] = {.lex_state = 81},
  [2306] = {.lex_state = 81},
  [2307] = {.lex_state = 81},
  [2308] = {.lex_state = 0},
  [2309] = {.lex_state = 132},
  [2310] = {.lex_state = 111},
  [2311] = {.lex_state = 132},
  [2312] = {.lex_state = 132},
  [2313] = {.lex_state = 132},
  [2314] = {.lex_state = 132},
  [2315] = {.lex_state = 81},
  [2316] = {.lex_state = 81},
  [2317] = {.lex_state = 81},
  [2318] = {.lex_state = 81},
  [2319] = {.lex_state = 81},
  [2320] = {.lex_state = 81},
  [2321] = {.lex_state = 111},
  [2322] = {.lex_state = 111},
  [2323] = {.lex_state = 111},
  [2324] = {.lex_state = 111},
  [2325] = {.lex_state = 111},
  [2326] = {.lex_state = 111},
  [2327] = {.lex_state = 171, .external_lex_state = 1},
  [2328] = {.lex_state = 111},
  [2329] = {.lex_state = 171, .external_lex_state = 1},
  [2330] = {.lex_state = 0},
  [2331] = {.lex_state = 171, .external_lex_state = 1},
  [2332] = {.lex_state = 0},
  [2333] = {.lex_state = 171, .external_lex_state = 1},
  [2334] = {.lex_state = 171, .external_lex_state = 1},
  [2335] = {.lex_state = 111},
  [2336] = {.lex_state = 0},
  [2337] = {.lex_state = 117, .external_lex_state = 1},
  [2338] = {.lex_state = 187, .external_lex_state = 1},
  [2339] = {.lex_state = 81},
  [2340] = {.lex_state = 81},
  [2341] = {.lex_state = 81},
  [2342] = {.lex_state = 0},
  [2343] = {.lex_state = 0},
  [2344] = {.lex_state = 111},
  [2345] = {.lex_state = 81},
  [2346] = {.lex_state = 111},
  [2347] = {.lex_state = 111},
  [2348] = {.lex_state = 117, .external_lex_state = 1},
  [2349] = {.lex_state = 111},
  [2350] = {.lex_state = 178, .external_lex_state = 1},
  [2351] = {.lex_state = 111},
  [2352] = {.lex_state = 178, .external_lex_state = 1},
  [2353] = {.lex_state = 111},
  [2354] = {.lex_state = 111},
  [2355] = {.lex_state = 111},
  [2356] = {.lex_state = 81},
  [2357] = {.lex_state = 81},
  [2358] = {.lex_state = 138, .external_lex_state = 1},
  [2359] = {.lex_state = 81},
  [2360] = {.lex_state = 162, .external_lex_state = 1},
  [2361] = {.lex_state = 162, .external_lex_state = 1},
  [2362] = {.lex_state = 111},
  [2363] = {.lex_state = 0},
  [2364] = {.lex_state = 111},
  [2365] = {.lex_state = 111},
  [2366] = {.lex_state = 0},
  [2367] = {.lex_state = 0},
  [2368] = {.lex_state = 111},
  [2369] = {.lex_state = 111},
  [2370] = {.lex_state = 111},
  [2371] = {.lex_state = 111},
  [2372] = {.lex_state = 111},
  [2373] = {.lex_state = 111},
  [2374] = {.lex_state = 111},
  [2375] = {.lex_state = 111},
  [2376] = {.lex_state = 0},
  [2377] = {.lex_state = 170, .external_lex_state = 1},
  [2378] = {.lex_state = 0},
  [2379] = {.lex_state = 0},
  [2380] = {.lex_state = 0},
  [2381] = {.lex_state = 0},
  [2382] = {.lex_state = 111},
  [2383] = {.lex_state = 170, .external_lex_state = 1},
  [2384] = {.lex_state = 111},
  [2385] = {.lex_state = 111},
  [2386] = {.lex_state = 81},
  [2387] = {.lex_state = 81},
  [2388] = {.lex_state = 111},
  [2389] = {.lex_state = 81},
  [2390] = {.lex_state = 81},
  [2391] = {.lex_state = 81},
  [2392] = {.lex_state = 81},
  [2393] = {.lex_state = 81},
  [2394] = {.lex_state = 81},
  [2395] = {.lex_state = 111},
  [2396] = {.lex_state = 181},
  [2397] = {.lex_state = 81},
  [2398] = {.lex_state = 111},
  [2399] = {.lex_state = 111},
  [2400] = {.lex_state = 181},
  [2401] = {.lex_state = 111},
  [2402] = {.lex_state = 181},
  [2403] = {.lex_state = 111},
  [2404] = {.lex_state = 111},
  [2405] = {.lex_state = 111},
  [2406] = {.lex_state = 111},
  [2407] = {.lex_state = 111},
  [2408] = {.lex_state = 170, .external_lex_state = 1},
  [2409] = {.lex_state = 170, .external_lex_state = 1},
  [2410] = {.lex_state = 170, .external_lex_state = 1},
  [2411] = {.lex_state = 170, .external_lex_state = 1},
  [2412] = {.lex_state = 111},
  [2413] = {.lex_state = 170, .external_lex_state = 1},
  [2414] = {.lex_state = 81},
  [2415] = {.lex_state = 81},
  [2416] = {.lex_state = 81},
  [2417] = {.lex_state = 81},
  [2418] = {.lex_state = 81},
  [2419] = {.lex_state = 81},
  [2420] = {.lex_state = 81},
  [2421] = {.lex_state = 81},
  [2422] = {.lex_state = 81},
  [2423] = {.lex_state = 81},
  [2424] = {.lex_state = 81},
  [2425] = {.lex_state = 81},
  [2426] = {.lex_state = 81},
  [2427] = {.lex_state = 81},
  [2428] = {.lex_state = 132},
  [2429] = {.lex_state = 171, .external_lex_state = 1},
  [2430] = {.lex_state = 171, .external_lex_state = 1},
  [2431] = {.lex_state = 171, .external_lex_state = 1},
  [2432] = {.lex_state = 171, .external_lex_state = 1},
  [2433] = {.lex_state = 171, .external_lex_state = 1},
  [2434] = {.lex_state = 111},
  [2435] = {.lex_state = 111},
  [2436] = {.lex_state = 0},
  [2437] = {.lex_state = 171, .external_lex_state = 1},
  [2438] = {.lex_state = 0},
  [2439] = {.lex_state = 0},
  [2440] = {.lex_state = 0},
  [2441] = {.lex_state = 0},
  [2442] = {.lex_state = 111},
  [2443] = {.lex_state = 171, .external_lex_state = 1},
  [2444] = {.lex_state = 81},
  [2445] = {.lex_state = 111},
  [2446] = {.lex_state = 111},
  [2447] = {.lex_state = 111},
  [2448] = {.lex_state = 81},
  [2449] = {.lex_state = 111},
  [2450] = {.lex_state = 111},
  [2451] = {.lex_state = 190, .external_lex_state = 1},
  [2452] = {.lex_state = 190, .external_lex_state = 1},
  [2453] = {.lex_state = 190, .external_lex_state = 1},
  [2454] = {.lex_state = 81},
  [2455] = {.lex_state = 81},
  [2456] = {.lex_state = 111},
  [2457] = {.lex_state = 81},
  [2458] = {.lex_state = 81},
  [2459] = {.lex_state = 81},
  [2460] = {.lex_state = 81},
  [2461] = {.lex_state = 117, .external_lex_state = 1},
  [2462] = {.lex_state = 111},
  [2463] = {.lex_state = 111},
  [2464] = {.lex_state = 81},
  [2465] = {.lex_state = 111},
  [2466] = {.lex_state = 111},
  [2467] = {.lex_state = 111},
  [2468] = {.lex_state = 111},
  [2469] = {.lex_state = 170, .external_lex_state = 1},
  [2470] = {.lex_state = 111},
  [2471] = {.lex_state = 170, .external_lex_state = 1},
  [2472] = {.lex_state = 0},
  [2473] = {.lex_state = 170, .external_lex_state = 1},
  [2474] = {.lex_state = 0},
  [2475] = {.lex_state = 170, .external_lex_state = 1},
  [2476] = {.lex_state = 170, .external_lex_state = 1},
  [2477] = {.lex_state = 111},
  [2478] = {.lex_state = 187, .external_lex_state = 1},
  [2479] = {.lex_state = 0},
  [2480] = {.lex_state = 111},
  [2481] = {.lex_state = 181},
  [2482] = {.lex_state = 111},
  [2483] = {.lex_state = 181},
  [2484] = {.lex_state = 111},
  [2485] = {.lex_state = 181},
  [2486] = {.lex_state = 81},
  [2487] = {.lex_state = 81},
  [2488] = {.lex_state = 111},
  [2489] = {.lex_state = 81},
  [2490] = {.lex_state = 81},
  [2491] = {.lex_state = 81},
  [2492] = {.lex_state = 81},
  [2493] = {.lex_state = 81},
  [2494] = {.lex_state = 111},
  [2495] = {.lex_state = 0},
  [2496] = {.lex_state = 81},
  [2497] = {.lex_state = 81},
  [2498] = {.lex_state = 81},
  [2499] = {.lex_state = 81},
  [2500] = {.lex_state = 81},
  [2501] = {.lex_state = 81},
  [2502] = {.lex_state = 171, .external_lex_state = 1},
  [2503] = {.lex_state = 111},
  [2504] = {.lex_state = 171, .external_lex_state = 1},
  [2505] = {.lex_state = 0},
  [2506] = {.lex_state = 171, .external_lex_state = 1},
  [2507] = {.lex_state = 0},
  [2508] = {.lex_state = 171, .external_lex_state = 1},
  [2509] = {.lex_state = 171, .external_lex_state = 1},
  [2510] = {.lex_state = 111},
  [2511] = {.lex_state = 0},
  [2512] = {.lex_state = 111},
  [2513] = {.lex_state = 111},
  [2514] = {.lex_state = 81},
  [2515] = {.lex_state = 81},
  [2516] = {.lex_state = 81},
  [2517] = {.lex_state = 81},
  [2518] = {.lex_state = 81},
  [2519] = {.lex_state = 81},
  [2520] = {.lex_state = 111},
  [2521] = {.lex_state = 181},
  [2522] = {.lex_state = 81},
  [2523] = {.lex_state = 111},
  [2524] = {.lex_state = 111},
  [2525] = {.lex_state = 181},
  [2526] = {.lex_state = 111},
  [2527] = {.lex_state = 181},
  [2528] = {.lex_state = 111},
  [2529] = {.lex_state = 111},
  [2530] = {.lex_state = 111},
  [2531] = {.lex_state = 111},
  [2532] = {.lex_state = 111},
  [2533] = {.lex_state = 170, .external_lex_state = 1},
  [2534] = {.lex_state = 170, .external_lex_state = 1},
  [2535] = {.lex_state = 170, .external_lex_state = 1},
  [2536] = {.lex_state = 170, .external_lex_state = 1},
  [2537] = {.lex_state = 111},
  [2538] = {.lex_state = 170, .external_lex_state = 1},
  [2539] = {.lex_state = 81},
  [2540] = {.lex_state = 81},
  [2541] = {.lex_state = 81},
  [2542] = {.lex_state = 81},
  [2543] = {.lex_state = 81},
  [2544] = {.lex_state = 81},
  [2545] = {.lex_state = 111},
  [2546] = {.lex_state = 111},
  [2547] = {.lex_state = 111},
  [2548] = {.lex_state = 111},
  [2549] = {.lex_state = 111},
  [2550] = {.lex_state = 111},
  [2551] = {.lex_state = 111},
  [2552] = {.lex_state = 111},
  [2553] = {.lex_state = 171, .external_lex_state = 1},
  [2554] = {.lex_state = 171, .external_lex_state = 1},
  [2555] = {.lex_state = 171, .external_lex_state = 1},
  [2556] = {.lex_state = 171, .external_lex_state = 1},
  [2557] = {.lex_state = 171, .external_lex_state = 1},
  [2558] = {.lex_state = 111},
  [2559] = {.lex_state = 181},
  [2560] = {.lex_state = 111},
  [2561] = {.lex_state = 181},
  [2562] = {.lex_state = 111},
  [2563] = {.lex_state = 181},
  [2564] = {.lex_state = 81},
  [2565] = {.lex_state = 81},
  [2566] = {.lex_state = 111},
  [2567] = {.lex_state = 81},
  [2568] = {.lex_state = 81},
  [2569] = {.lex_state = 81},
  [2570] = {.lex_state = 81},
  [2571] = {.lex_state = 81},
  [2572] = {.lex_state = 111},
  [2573] = {.lex_state = 0},
  [2574] = {.lex_state = 111},
  [2575] = {.lex_state = 111},
  [2576] = {.lex_state = 111},
  [2577] = {.lex_state = 111},
  [2578] = {.lex_state = 111},
  [2579] = {.lex_state = 111},
  [2580] = {.lex_state = 81},
  [2581] = {.lex_state = 81},
  [2582] = {.lex_state = 81},
  [2583] = {.lex_state = 81},
  [2584] = {.lex_state = 81},
  [2585] = {.lex_state = 81},
  [2586] = {.lex_state = 111},
  [2587] = {.lex_state = 111},
  [2588] = {.lex_state = 111},
  [2589] = {.lex_state = 111},
  [2590] = {.lex_state = 111},
  [2591] = {.lex_state = 111},
  [2592] = {.lex_state = 111},
  [2593] = {.lex_state = 111},
  [2594] = {.lex_state = 111},
  [2595] = {.lex_state = 111},
  [2596] = {.lex_state = 111},
  [2597] = {.lex_state = 111},
  [2598] = {.lex_state = 111},
  [2599] = {.lex_state = 111},
};

enum {
  ts_external_token__automatic_semicolon,
};

static TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token__automatic_semicolon] = sym__automatic_semicolon,
};

static bool ts_external_scanner_states[2][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token__automatic_semicolon] = true,
  },
};

static uint16_t ts_parse_table[STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [sym__automatic_semicolon] = ACTIONS(1),
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_hash_bang_line] = ACTIONS(1),
    [anon_sym_export] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(3),
    [anon_sym_default] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_as] = ACTIONS(3),
    [anon_sym_import] = ACTIONS(1),
    [anon_sym_from] = ACTIONS(1),
    [anon_sym_var] = ACTIONS(1),
    [anon_sym_let] = ACTIONS(1),
    [anon_sym_const] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_switch] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [anon_sym_in] = ACTIONS(3),
    [anon_sym_of] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_do] = ACTIONS(1),
    [anon_sym_try] = ACTIONS(1),
    [anon_sym_with] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_debugger] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_throw] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_case] = ACTIONS(1),
    [anon_sym_catch] = ACTIONS(1),
    [anon_sym_finally] = ACTIONS(1),
    [anon_sym_yield] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(3),
    [anon_sym_GT] = ACTIONS(3),
    [anon_sym_SLASH] = ACTIONS(3),
    [anon_sym_DOT] = ACTIONS(3),
    [anon_sym_EQ] = ACTIONS(3),
    [anon_sym_class] = ACTIONS(1),
    [anon_sym_extends] = ACTIONS(1),
    [anon_sym_async] = ACTIONS(1),
    [anon_sym_function] = ACTIONS(1),
    [anon_sym_EQ_GT] = ACTIONS(1),
    [anon_sym_new] = ACTIONS(1),
    [anon_sym_await] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_DASH_EQ] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(3),
    [anon_sym_GT_GT_GT] = ACTIONS(3),
    [anon_sym_LT_LT] = ACTIONS(3),
    [anon_sym_AMP] = ACTIONS(3),
    [anon_sym_CARET] = ACTIONS(3),
    [anon_sym_PIPE] = ACTIONS(3),
    [anon_sym_PLUS] = ACTIONS(3),
    [anon_sym_DASH] = ACTIONS(3),
    [anon_sym_PERCENT] = ACTIONS(3),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(3),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(3),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_instanceof] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(3),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_typeof] = ACTIONS(1),
    [anon_sym_void] = ACTIONS(1),
    [anon_sym_delete] = ACTIONS(1),
    [anon_sym_PLUS_PLUS] = ACTIONS(1),
    [anon_sym_DASH_DASH] = ACTIONS(1),
    [sym_comment] = ACTIONS(1),
    [sym_string] = ACTIONS(1),
    [anon_sym_BQUOTE] = ACTIONS(1),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(1),
    [sym_number] = ACTIONS(1),
    [sym_this] = ACTIONS(1),
    [sym_super] = ACTIONS(1),
    [sym_true] = ACTIONS(1),
    [sym_false] = ACTIONS(1),
    [sym_null] = ACTIONS(1),
    [sym_undefined] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym_static] = ACTIONS(1),
    [anon_sym_get] = ACTIONS(1),
    [anon_sym_set] = ACTIONS(1),
  },
  [1] = {
    [sym_program] = STATE(38),
    [sym_export_statement] = STATE(48),
    [sym__declaration] = STATE(48),
    [sym_import_statement] = STATE(48),
    [sym_expression_statement] = STATE(48),
    [sym_variable_declaration] = STATE(48),
    [sym_lexical_declaration] = STATE(48),
    [sym_statement_block] = STATE(48),
    [sym_if_statement] = STATE(48),
    [sym_switch_statement] = STATE(48),
    [sym_for_statement] = STATE(48),
    [sym_for_in_statement] = STATE(48),
    [sym_for_of_statement] = STATE(48),
    [sym_while_statement] = STATE(48),
    [sym_do_statement] = STATE(48),
    [sym_try_statement] = STATE(48),
    [sym_with_statement] = STATE(48),
    [sym_break_statement] = STATE(48),
    [sym_continue_statement] = STATE(48),
    [sym_debugger_statement] = STATE(48),
    [sym_return_statement] = STATE(48),
    [sym_throw_statement] = STATE(48),
    [sym_empty_statement] = STATE(48),
    [sym_labeled_statement] = STATE(48),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(48),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [ts_builtin_sym_end] = ACTIONS(5),
    [sym_hash_bang_line] = ACTIONS(7),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2] = {
    [sym_export_statement] = STATE(50),
    [sym__declaration] = STATE(50),
    [sym_import_statement] = STATE(50),
    [sym_expression_statement] = STATE(50),
    [sym_variable_declaration] = STATE(50),
    [sym_lexical_declaration] = STATE(50),
    [sym_statement_block] = STATE(50),
    [sym_if_statement] = STATE(50),
    [sym_switch_statement] = STATE(50),
    [sym_for_statement] = STATE(50),
    [sym_for_in_statement] = STATE(50),
    [sym_for_of_statement] = STATE(50),
    [sym_while_statement] = STATE(50),
    [sym_do_statement] = STATE(50),
    [sym_try_statement] = STATE(50),
    [sym_with_statement] = STATE(50),
    [sym_break_statement] = STATE(50),
    [sym_continue_statement] = STATE(50),
    [sym_debugger_statement] = STATE(50),
    [sym_return_statement] = STATE(50),
    [sym_throw_statement] = STATE(50),
    [sym_empty_statement] = STATE(50),
    [sym_labeled_statement] = STATE(50),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(50),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [ts_builtin_sym_end] = ACTIONS(87),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [3] = {
    [sym_export_clause] = STATE(57),
    [sym__declaration] = STATE(58),
    [sym_variable_declaration] = STATE(58),
    [sym_lexical_declaration] = STATE(58),
    [sym_class] = STATE(59),
    [sym_function] = STATE(59),
    [sym_generator_function] = STATE(59),
    [sym_decorator] = STATE(60),
    [aux_sym_export_statement_repeat1] = STATE(60),
    [anon_sym_STAR] = ACTIONS(89),
    [anon_sym_default] = ACTIONS(91),
    [anon_sym_LBRACE] = ACTIONS(93),
    [anon_sym_var] = ACTIONS(95),
    [anon_sym_let] = ACTIONS(97),
    [anon_sym_const] = ACTIONS(97),
    [anon_sym_class] = ACTIONS(99),
    [anon_sym_async] = ACTIONS(101),
    [anon_sym_function] = ACTIONS(103),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [4] = {
    [sym_export_statement] = STATE(79),
    [sym__declaration] = STATE(79),
    [sym_import_statement] = STATE(79),
    [sym_expression_statement] = STATE(79),
    [sym_variable_declaration] = STATE(79),
    [sym_lexical_declaration] = STATE(79),
    [sym_statement_block] = STATE(79),
    [sym_if_statement] = STATE(79),
    [sym_switch_statement] = STATE(79),
    [sym_for_statement] = STATE(79),
    [sym_for_in_statement] = STATE(79),
    [sym_for_of_statement] = STATE(79),
    [sym_while_statement] = STATE(79),
    [sym_do_statement] = STATE(79),
    [sym_try_statement] = STATE(79),
    [sym_with_statement] = STATE(79),
    [sym_break_statement] = STATE(79),
    [sym_continue_statement] = STATE(79),
    [sym_debugger_statement] = STATE(79),
    [sym_return_statement] = STATE(79),
    [sym_throw_statement] = STATE(79),
    [sym_empty_statement] = STATE(79),
    [sym_labeled_statement] = STATE(79),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(79),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(109),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [5] = {
    [sym_import_clause] = STATE(86),
    [sym_namespace_import] = STATE(87),
    [sym_named_imports] = STATE(87),
    [anon_sym_STAR] = ACTIONS(133),
    [anon_sym_LBRACE] = ACTIONS(135),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(137),
    [sym_identifier] = ACTIONS(139),
  },
  [6] = {
    [sym_variable_declarator] = STATE(91),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [7] = {
    [sym_variable_declarator] = STATE(94),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [8] = {
    [sym_parenthesized_expression] = STATE(96),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [9] = {
    [sym_parenthesized_expression] = STATE(97),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [10] = {
    [anon_sym_LPAREN] = ACTIONS(149),
    [sym_comment] = ACTIONS(71),
  },
  [11] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(118),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(120),
    [sym_assignment_pattern] = STATE(121),
    [sym_array] = STATE(122),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(118),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [sym_rest_parameter] = STATE(121),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(155),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(165),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(175),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(189),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(193),
    [anon_sym_set] = ACTIONS(193),
  },
  [12] = {
    [sym_parenthesized_expression] = STATE(128),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [13] = {
    [sym_export_statement] = STATE(140),
    [sym__declaration] = STATE(140),
    [sym_import_statement] = STATE(140),
    [sym_expression_statement] = STATE(140),
    [sym_variable_declaration] = STATE(140),
    [sym_lexical_declaration] = STATE(140),
    [sym_statement_block] = STATE(140),
    [sym_if_statement] = STATE(140),
    [sym_switch_statement] = STATE(140),
    [sym_for_statement] = STATE(140),
    [sym_for_in_statement] = STATE(140),
    [sym_for_of_statement] = STATE(140),
    [sym_while_statement] = STATE(140),
    [sym_do_statement] = STATE(140),
    [sym_try_statement] = STATE(140),
    [sym_with_statement] = STATE(140),
    [sym_break_statement] = STATE(140),
    [sym_continue_statement] = STATE(140),
    [sym_debugger_statement] = STATE(140),
    [sym_return_statement] = STATE(140),
    [sym_throw_statement] = STATE(140),
    [sym_empty_statement] = STATE(140),
    [sym_labeled_statement] = STATE(140),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [14] = {
    [sym_statement_block] = STATE(145),
    [anon_sym_LBRACE] = ACTIONS(217),
    [sym_comment] = ACTIONS(71),
  },
  [15] = {
    [sym_parenthesized_expression] = STATE(146),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [16] = {
    [sym__automatic_semicolon] = ACTIONS(219),
    [anon_sym_SEMI] = ACTIONS(219),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(221),
  },
  [17] = {
    [sym__automatic_semicolon] = ACTIONS(223),
    [anon_sym_SEMI] = ACTIONS(223),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(225),
  },
  [18] = {
    [sym__automatic_semicolon] = ACTIONS(227),
    [anon_sym_SEMI] = ACTIONS(227),
    [sym_comment] = ACTIONS(71),
  },
  [19] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(155),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(155),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(229),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(229),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [20] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(158),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(158),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [21] = {
    [ts_builtin_sym_end] = ACTIONS(235),
    [anon_sym_export] = ACTIONS(237),
    [anon_sym_LBRACE] = ACTIONS(235),
    [anon_sym_RBRACE] = ACTIONS(235),
    [anon_sym_import] = ACTIONS(237),
    [anon_sym_var] = ACTIONS(237),
    [anon_sym_let] = ACTIONS(237),
    [anon_sym_const] = ACTIONS(237),
    [anon_sym_if] = ACTIONS(237),
    [anon_sym_switch] = ACTIONS(237),
    [anon_sym_for] = ACTIONS(237),
    [anon_sym_LPAREN] = ACTIONS(235),
    [anon_sym_while] = ACTIONS(237),
    [anon_sym_do] = ACTIONS(237),
    [anon_sym_try] = ACTIONS(237),
    [anon_sym_with] = ACTIONS(237),
    [anon_sym_break] = ACTIONS(237),
    [anon_sym_continue] = ACTIONS(237),
    [anon_sym_debugger] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_throw] = ACTIONS(237),
    [anon_sym_SEMI] = ACTIONS(235),
    [anon_sym_yield] = ACTIONS(237),
    [anon_sym_LBRACK] = ACTIONS(235),
    [anon_sym_LT] = ACTIONS(235),
    [anon_sym_class] = ACTIONS(237),
    [anon_sym_async] = ACTIONS(237),
    [anon_sym_function] = ACTIONS(237),
    [anon_sym_EQ_GT] = ACTIONS(235),
    [anon_sym_new] = ACTIONS(237),
    [anon_sym_await] = ACTIONS(237),
    [anon_sym_PLUS] = ACTIONS(237),
    [anon_sym_DASH] = ACTIONS(237),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_TILDE] = ACTIONS(235),
    [anon_sym_typeof] = ACTIONS(237),
    [anon_sym_void] = ACTIONS(237),
    [anon_sym_delete] = ACTIONS(237),
    [anon_sym_PLUS_PLUS] = ACTIONS(235),
    [anon_sym_DASH_DASH] = ACTIONS(235),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(235),
    [anon_sym_BQUOTE] = ACTIONS(235),
    [sym_regex] = ACTIONS(237),
    [sym_number] = ACTIONS(235),
    [sym_identifier] = ACTIONS(237),
    [sym_this] = ACTIONS(237),
    [sym_super] = ACTIONS(237),
    [sym_true] = ACTIONS(237),
    [sym_false] = ACTIONS(237),
    [sym_null] = ACTIONS(237),
    [sym_undefined] = ACTIONS(237),
    [anon_sym_AT] = ACTIONS(235),
    [anon_sym_get] = ACTIONS(237),
    [anon_sym_set] = ACTIONS(237),
  },
  [22] = {
    [sym_parenthesized_expression] = STATE(159),
    [sym__expression] = STATE(159),
    [sym_yield_expression] = STATE(159),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(159),
    [sym_jsx_fragment] = STATE(159),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(159),
    [sym_anonymous_class] = STATE(159),
    [sym_class] = STATE(159),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(159),
    [sym_generator_function] = STATE(159),
    [sym_call_expression] = STATE(159),
    [sym_new_expression] = STATE(159),
    [sym_await_expression] = STATE(159),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(159),
    [sym_augmented_assignment_expression] = STATE(159),
    [sym_ternary_expression] = STATE(159),
    [sym_binary_expression] = STATE(159),
    [sym_unary_expression] = STATE(159),
    [sym_update_expression] = STATE(159),
    [sym_template_string] = STATE(159),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(239),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_SEMI] = ACTIONS(239),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(243),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(65),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(245),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(247),
    [sym_number] = ACTIONS(245),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(247),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(247),
    [sym_false] = ACTIONS(247),
    [sym_null] = ACTIONS(247),
    [sym_undefined] = ACTIONS(247),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [23] = {
    [sym_parenthesized_expression] = STATE(172),
    [sym__expression] = STATE(172),
    [sym_yield_expression] = STATE(172),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(172),
    [sym_jsx_fragment] = STATE(172),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(172),
    [sym_anonymous_class] = STATE(172),
    [sym_class] = STATE(172),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(172),
    [sym_generator_function] = STATE(172),
    [sym_call_expression] = STATE(172),
    [sym_new_expression] = STATE(172),
    [sym_await_expression] = STATE(172),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(172),
    [sym_augmented_assignment_expression] = STATE(172),
    [sym_spread_element] = STATE(176),
    [sym_ternary_expression] = STATE(172),
    [sym_binary_expression] = STATE(172),
    [sym_unary_expression] = STATE(172),
    [sym_update_expression] = STATE(172),
    [sym_template_string] = STATE(172),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(178),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(253),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(271),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(273),
    [sym_number] = ACTIONS(271),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(273),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(273),
    [sym_false] = ACTIONS(273),
    [sym_null] = ACTIONS(273),
    [sym_undefined] = ACTIONS(273),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [24] = {
    [sym_nested_identifier] = STATE(182),
    [sym_jsx_namespace_name] = STATE(183),
    [anon_sym_GT] = ACTIONS(279),
    [sym_jsx_identifier] = ACTIONS(281),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(283),
  },
  [25] = {
    [sym_class_heritage] = STATE(187),
    [sym_class_body] = STATE(188),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(289),
  },
  [26] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_function] = ACTIONS(302),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(308),
  },
  [27] = {
    [sym_formal_parameters] = STATE(199),
    [anon_sym_STAR] = ACTIONS(310),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(314),
  },
  [28] = {
    [sym_statement_block] = STATE(201),
    [sym_parenthesized_expression] = STATE(202),
    [sym__expression] = STATE(202),
    [sym_yield_expression] = STATE(202),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(202),
    [sym_jsx_fragment] = STATE(202),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(202),
    [sym_anonymous_class] = STATE(202),
    [sym_class] = STATE(202),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(202),
    [sym_generator_function] = STATE(202),
    [sym_call_expression] = STATE(202),
    [sym_new_expression] = STATE(202),
    [sym_await_expression] = STATE(202),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(202),
    [sym_augmented_assignment_expression] = STATE(202),
    [sym_ternary_expression] = STATE(202),
    [sym_binary_expression] = STATE(202),
    [sym_unary_expression] = STATE(202),
    [sym_update_expression] = STATE(202),
    [sym_template_string] = STATE(202),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(318),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(320),
    [sym_number] = ACTIONS(318),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(320),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(320),
    [sym_false] = ACTIONS(320),
    [sym_null] = ACTIONS(320),
    [sym_undefined] = ACTIONS(320),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [29] = {
    [sym_parenthesized_expression] = STATE(203),
    [sym__expression] = STATE(203),
    [sym_yield_expression] = STATE(203),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(203),
    [sym_jsx_fragment] = STATE(203),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(203),
    [sym_anonymous_class] = STATE(203),
    [sym_class] = STATE(203),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(203),
    [sym_generator_function] = STATE(203),
    [sym_call_expression] = STATE(203),
    [sym_new_expression] = STATE(203),
    [sym_await_expression] = STATE(203),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(203),
    [sym_augmented_assignment_expression] = STATE(203),
    [sym_ternary_expression] = STATE(203),
    [sym_binary_expression] = STATE(203),
    [sym_unary_expression] = STATE(203),
    [sym_update_expression] = STATE(203),
    [sym_template_string] = STATE(203),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(322),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(324),
    [sym_number] = ACTIONS(322),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(324),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(324),
    [sym_false] = ACTIONS(324),
    [sym_null] = ACTIONS(324),
    [sym_undefined] = ACTIONS(324),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [30] = {
    [sym_parenthesized_expression] = STATE(204),
    [sym__expression] = STATE(204),
    [sym_yield_expression] = STATE(204),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(204),
    [sym_jsx_fragment] = STATE(204),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(204),
    [sym_anonymous_class] = STATE(204),
    [sym_class] = STATE(204),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(204),
    [sym_generator_function] = STATE(204),
    [sym_call_expression] = STATE(204),
    [sym_new_expression] = STATE(204),
    [sym_await_expression] = STATE(204),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(204),
    [sym_augmented_assignment_expression] = STATE(204),
    [sym_ternary_expression] = STATE(204),
    [sym_binary_expression] = STATE(204),
    [sym_unary_expression] = STATE(204),
    [sym_update_expression] = STATE(204),
    [sym_template_string] = STATE(204),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(326),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(328),
    [sym_number] = ACTIONS(326),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(328),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(328),
    [sym_false] = ACTIONS(328),
    [sym_null] = ACTIONS(328),
    [sym_undefined] = ACTIONS(328),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [31] = {
    [sym_parenthesized_expression] = STATE(205),
    [sym__expression] = STATE(205),
    [sym_yield_expression] = STATE(205),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(205),
    [sym_jsx_fragment] = STATE(205),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(205),
    [sym_anonymous_class] = STATE(205),
    [sym_class] = STATE(205),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(205),
    [sym_generator_function] = STATE(205),
    [sym_call_expression] = STATE(205),
    [sym_new_expression] = STATE(205),
    [sym_await_expression] = STATE(205),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(205),
    [sym_augmented_assignment_expression] = STATE(205),
    [sym_ternary_expression] = STATE(205),
    [sym_binary_expression] = STATE(205),
    [sym_unary_expression] = STATE(205),
    [sym_update_expression] = STATE(205),
    [sym_template_string] = STATE(205),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(330),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(332),
    [sym_number] = ACTIONS(330),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(332),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(332),
    [sym_false] = ACTIONS(332),
    [sym_null] = ACTIONS(332),
    [sym_undefined] = ACTIONS(332),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [32] = {
    [sym_parenthesized_expression] = STATE(206),
    [sym__expression] = STATE(206),
    [sym_yield_expression] = STATE(206),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(206),
    [sym_jsx_fragment] = STATE(206),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(206),
    [sym_anonymous_class] = STATE(206),
    [sym_class] = STATE(206),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(206),
    [sym_generator_function] = STATE(206),
    [sym_call_expression] = STATE(206),
    [sym_new_expression] = STATE(206),
    [sym_await_expression] = STATE(206),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(206),
    [sym_augmented_assignment_expression] = STATE(206),
    [sym_ternary_expression] = STATE(206),
    [sym_binary_expression] = STATE(206),
    [sym_unary_expression] = STATE(206),
    [sym_update_expression] = STATE(206),
    [sym_template_string] = STATE(206),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(334),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(336),
    [sym_number] = ACTIONS(334),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(336),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(336),
    [sym_false] = ACTIONS(336),
    [sym_null] = ACTIONS(336),
    [sym_undefined] = ACTIONS(336),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [33] = {
    [sym_template_substitution] = STATE(209),
    [aux_sym_template_string_repeat1] = STATE(209),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(338),
    [sym__template_chars] = ACTIONS(340),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(342),
  },
  [34] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(350),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [35] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_DOT] = ACTIONS(352),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [36] = {
    [sym__identifier_reference] = STATE(219),
    [sym_decorator_member_expression] = STATE(220),
    [sym_decorator_call_expression] = STATE(221),
    [anon_sym_async] = ACTIONS(360),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(362),
    [anon_sym_get] = ACTIONS(360),
    [anon_sym_set] = ACTIONS(360),
  },
  [37] = {
    [sym__initializer] = STATE(196),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [38] = {
    [ts_builtin_sym_end] = ACTIONS(364),
    [sym_comment] = ACTIONS(71),
  },
  [39] = {
    [sym__automatic_semicolon] = ACTIONS(366),
    [anon_sym_SEMI] = ACTIONS(366),
    [sym_comment] = ACTIONS(71),
  },
  [40] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(368),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(372),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(368),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [41] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [42] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [43] = {
    [sym_jsx_element] = STATE(236),
    [sym_jsx_expression] = STATE(236),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(235),
    [sym_jsx_self_closing_element] = STATE(236),
    [aux_sym_jsx_element_repeat1] = STATE(236),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(394),
    [sym_jsx_text] = ACTIONS(396),
    [sym_comment] = ACTIONS(71),
  },
  [44] = {
    [sym__automatic_semicolon] = ACTIONS(398),
    [ts_builtin_sym_end] = ACTIONS(400),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [45] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(398),
    [ts_builtin_sym_end] = ACTIONS(400),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [46] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [47] = {
    [anon_sym_EQ_GT] = ACTIONS(304),
    [sym_comment] = ACTIONS(71),
  },
  [48] = {
    [sym_export_statement] = STATE(238),
    [sym__declaration] = STATE(238),
    [sym_import_statement] = STATE(238),
    [sym_expression_statement] = STATE(238),
    [sym_variable_declaration] = STATE(238),
    [sym_lexical_declaration] = STATE(238),
    [sym_statement_block] = STATE(238),
    [sym_if_statement] = STATE(238),
    [sym_switch_statement] = STATE(238),
    [sym_for_statement] = STATE(238),
    [sym_for_in_statement] = STATE(238),
    [sym_for_of_statement] = STATE(238),
    [sym_while_statement] = STATE(238),
    [sym_do_statement] = STATE(238),
    [sym_try_statement] = STATE(238),
    [sym_with_statement] = STATE(238),
    [sym_break_statement] = STATE(238),
    [sym_continue_statement] = STATE(238),
    [sym_debugger_statement] = STATE(238),
    [sym_return_statement] = STATE(238),
    [sym_throw_statement] = STATE(238),
    [sym_empty_statement] = STATE(238),
    [sym_labeled_statement] = STATE(238),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(238),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [ts_builtin_sym_end] = ACTIONS(87),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [49] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(404),
    [anon_sym_class] = ACTIONS(406),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [50] = {
    [sym_export_statement] = STATE(238),
    [sym__declaration] = STATE(238),
    [sym_import_statement] = STATE(238),
    [sym_expression_statement] = STATE(238),
    [sym_variable_declaration] = STATE(238),
    [sym_lexical_declaration] = STATE(238),
    [sym_statement_block] = STATE(238),
    [sym_if_statement] = STATE(238),
    [sym_switch_statement] = STATE(238),
    [sym_for_statement] = STATE(238),
    [sym_for_in_statement] = STATE(238),
    [sym_for_of_statement] = STATE(238),
    [sym_while_statement] = STATE(238),
    [sym_do_statement] = STATE(238),
    [sym_try_statement] = STATE(238),
    [sym_with_statement] = STATE(238),
    [sym_break_statement] = STATE(238),
    [sym_continue_statement] = STATE(238),
    [sym_debugger_statement] = STATE(238),
    [sym_return_statement] = STATE(238),
    [sym_throw_statement] = STATE(238),
    [sym_empty_statement] = STATE(238),
    [sym_labeled_statement] = STATE(238),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(238),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [ts_builtin_sym_end] = ACTIONS(408),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [51] = {
    [sym__from_clause] = STATE(243),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [52] = {
    [sym_parenthesized_expression] = STATE(253),
    [sym__expression] = STATE(253),
    [sym_yield_expression] = STATE(253),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(253),
    [sym_jsx_fragment] = STATE(253),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(253),
    [sym_anonymous_class] = STATE(253),
    [sym_class] = STATE(253),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(253),
    [sym_generator_function] = STATE(253),
    [sym_call_expression] = STATE(253),
    [sym_new_expression] = STATE(253),
    [sym_await_expression] = STATE(253),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(253),
    [sym_augmented_assignment_expression] = STATE(253),
    [sym_ternary_expression] = STATE(253),
    [sym_binary_expression] = STATE(253),
    [sym_unary_expression] = STATE(253),
    [sym_update_expression] = STATE(253),
    [sym_template_string] = STATE(253),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(428),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(430),
    [sym_number] = ACTIONS(428),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(430),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(430),
    [sym_false] = ACTIONS(430),
    [sym_null] = ACTIONS(430),
    [sym_undefined] = ACTIONS(430),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [53] = {
    [sym__import_export_specifier] = STATE(261),
    [anon_sym_COMMA] = ACTIONS(436),
    [anon_sym_RBRACE] = ACTIONS(438),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [54] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(442),
  },
  [55] = {
    [anon_sym_function] = ACTIONS(444),
    [sym_comment] = ACTIONS(71),
  },
  [56] = {
    [sym_formal_parameters] = STATE(266),
    [anon_sym_STAR] = ACTIONS(446),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(448),
  },
  [57] = {
    [sym__from_clause] = STATE(243),
    [sym__automatic_semicolon] = ACTIONS(450),
    [anon_sym_from] = ACTIONS(410),
    [anon_sym_SEMI] = ACTIONS(450),
    [sym_comment] = ACTIONS(71),
  },
  [58] = {
    [ts_builtin_sym_end] = ACTIONS(452),
    [anon_sym_export] = ACTIONS(454),
    [anon_sym_LBRACE] = ACTIONS(452),
    [anon_sym_RBRACE] = ACTIONS(452),
    [anon_sym_import] = ACTIONS(454),
    [anon_sym_var] = ACTIONS(454),
    [anon_sym_let] = ACTIONS(454),
    [anon_sym_const] = ACTIONS(454),
    [anon_sym_if] = ACTIONS(454),
    [anon_sym_switch] = ACTIONS(454),
    [anon_sym_for] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(452),
    [anon_sym_while] = ACTIONS(454),
    [anon_sym_do] = ACTIONS(454),
    [anon_sym_try] = ACTIONS(454),
    [anon_sym_with] = ACTIONS(454),
    [anon_sym_break] = ACTIONS(454),
    [anon_sym_continue] = ACTIONS(454),
    [anon_sym_debugger] = ACTIONS(454),
    [anon_sym_return] = ACTIONS(454),
    [anon_sym_throw] = ACTIONS(454),
    [anon_sym_SEMI] = ACTIONS(452),
    [anon_sym_yield] = ACTIONS(454),
    [anon_sym_LBRACK] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(452),
    [anon_sym_class] = ACTIONS(454),
    [anon_sym_async] = ACTIONS(454),
    [anon_sym_function] = ACTIONS(454),
    [anon_sym_EQ_GT] = ACTIONS(452),
    [anon_sym_new] = ACTIONS(454),
    [anon_sym_await] = ACTIONS(454),
    [anon_sym_PLUS] = ACTIONS(454),
    [anon_sym_DASH] = ACTIONS(454),
    [anon_sym_BANG] = ACTIONS(452),
    [anon_sym_TILDE] = ACTIONS(452),
    [anon_sym_typeof] = ACTIONS(454),
    [anon_sym_void] = ACTIONS(454),
    [anon_sym_delete] = ACTIONS(454),
    [anon_sym_PLUS_PLUS] = ACTIONS(452),
    [anon_sym_DASH_DASH] = ACTIONS(452),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(452),
    [anon_sym_BQUOTE] = ACTIONS(452),
    [sym_regex] = ACTIONS(454),
    [sym_number] = ACTIONS(452),
    [sym_identifier] = ACTIONS(454),
    [sym_this] = ACTIONS(454),
    [sym_super] = ACTIONS(454),
    [sym_true] = ACTIONS(454),
    [sym_false] = ACTIONS(454),
    [sym_null] = ACTIONS(454),
    [sym_undefined] = ACTIONS(454),
    [anon_sym_AT] = ACTIONS(452),
    [anon_sym_get] = ACTIONS(454),
    [anon_sym_set] = ACTIONS(454),
  },
  [59] = {
    [sym__automatic_semicolon] = ACTIONS(398),
    [ts_builtin_sym_end] = ACTIONS(400),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(400),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_BANG] = ACTIONS(400),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [60] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(456),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [61] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [62] = {
    [sym_object] = STATE(279),
    [sym_assignment_pattern] = STATE(280),
    [sym_array] = STATE(281),
    [sym_spread_element] = STATE(280),
    [sym_decorator] = STATE(282),
    [sym_method_definition] = STATE(280),
    [sym_pair] = STATE(280),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_export_statement_repeat1] = STATE(282),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_COMMA] = ACTIONS(466),
    [anon_sym_RBRACE] = ACTIONS(466),
    [anon_sym_LBRACK] = ACTIONS(468),
    [anon_sym_async] = ACTIONS(470),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(472),
    [sym_number] = ACTIONS(472),
    [sym_identifier] = ACTIONS(474),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(478),
    [anon_sym_set] = ACTIONS(478),
  },
  [63] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(486),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(486),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(480),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [64] = {
    [sym_parenthesized_expression] = STATE(283),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [65] = {
    [anon_sym_LPAREN] = ACTIONS(488),
    [sym_comment] = ACTIONS(71),
  },
  [66] = {
    [sym_parenthesized_expression] = STATE(285),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [67] = {
    [sym_parenthesized_expression] = STATE(286),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [68] = {
    [sym_parenthesized_expression] = STATE(287),
    [sym__expression] = STATE(287),
    [sym_yield_expression] = STATE(287),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(287),
    [sym_jsx_fragment] = STATE(287),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(287),
    [sym_anonymous_class] = STATE(287),
    [sym_class] = STATE(287),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(287),
    [sym_generator_function] = STATE(287),
    [sym_call_expression] = STATE(287),
    [sym_new_expression] = STATE(287),
    [sym_await_expression] = STATE(287),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(287),
    [sym_augmented_assignment_expression] = STATE(287),
    [sym_spread_element] = STATE(176),
    [sym_ternary_expression] = STATE(287),
    [sym_binary_expression] = STATE(287),
    [sym_unary_expression] = STATE(287),
    [sym_update_expression] = STATE(287),
    [sym_template_string] = STATE(287),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(178),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(253),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(490),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(492),
    [sym_number] = ACTIONS(490),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(492),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(492),
    [sym_false] = ACTIONS(492),
    [sym_null] = ACTIONS(492),
    [sym_undefined] = ACTIONS(492),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [69] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(292),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(494),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(499),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(505),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(508),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_async] = ACTIONS(460),
    [anon_sym_function] = ACTIONS(302),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(510),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [70] = {
    [sym_parenthesized_expression] = STATE(302),
    [sym__expression] = STATE(302),
    [sym_yield_expression] = STATE(302),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(302),
    [sym_jsx_fragment] = STATE(302),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(302),
    [sym_anonymous_class] = STATE(302),
    [sym_class] = STATE(302),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(302),
    [sym_generator_function] = STATE(302),
    [sym_call_expression] = STATE(302),
    [sym_new_expression] = STATE(302),
    [sym_await_expression] = STATE(302),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(302),
    [sym_augmented_assignment_expression] = STATE(302),
    [sym_ternary_expression] = STATE(302),
    [sym_binary_expression] = STATE(302),
    [sym_unary_expression] = STATE(302),
    [sym_update_expression] = STATE(302),
    [sym_template_string] = STATE(302),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(530),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(532),
    [sym_number] = ACTIONS(530),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(532),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(532),
    [sym_false] = ACTIONS(532),
    [sym_null] = ACTIONS(532),
    [sym_undefined] = ACTIONS(532),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [71] = {
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(538),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(541),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [72] = {
    [sym__initializer] = STATE(309),
    [aux_sym_object_repeat1] = STATE(292),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(543),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(546),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [73] = {
    [sym__identifier_reference] = STATE(311),
    [sym_decorator_member_expression] = STATE(312),
    [sym_decorator_call_expression] = STATE(313),
    [anon_sym_async] = ACTIONS(551),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(553),
    [anon_sym_get] = ACTIONS(551),
    [anon_sym_set] = ACTIONS(551),
  },
  [74] = {
    [sym__initializer] = STATE(196),
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(292),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(555),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(505),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(508),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_async] = ACTIONS(460),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [75] = {
    [sym__initializer] = STATE(314),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [76] = {
    [aux_sym_object_repeat1] = STATE(316),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(558),
    [sym_comment] = ACTIONS(71),
  },
  [77] = {
    [sym__initializer] = STATE(317),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [78] = {
    [sym_formal_parameters] = STATE(319),
    [anon_sym_LPAREN] = ACTIONS(312),
    [anon_sym_COLON] = ACTIONS(560),
    [sym_comment] = ACTIONS(71),
  },
  [79] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(562),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [80] = {
    [sym_decorator] = STATE(327),
    [sym_method_definition] = STATE(325),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(327),
    [anon_sym_export] = ACTIONS(566),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_class] = ACTIONS(568),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [81] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(558),
    [sym_comment] = ACTIONS(71),
  },
  [82] = {
    [anon_sym_as] = ACTIONS(576),
    [sym_comment] = ACTIONS(71),
  },
  [83] = {
    [sym__import_export_specifier] = STATE(332),
    [anon_sym_COMMA] = ACTIONS(578),
    [anon_sym_RBRACE] = ACTIONS(580),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [84] = {
    [sym__automatic_semicolon] = ACTIONS(582),
    [anon_sym_SEMI] = ACTIONS(582),
    [sym_comment] = ACTIONS(71),
  },
  [85] = {
    [anon_sym_COMMA] = ACTIONS(584),
    [anon_sym_from] = ACTIONS(586),
    [sym_comment] = ACTIONS(71),
  },
  [86] = {
    [sym__from_clause] = STATE(335),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [87] = {
    [anon_sym_from] = ACTIONS(586),
    [sym_comment] = ACTIONS(71),
  },
  [88] = {
    [sym_object] = STATE(279),
    [sym_assignment_pattern] = STATE(340),
    [sym_array] = STATE(281),
    [sym_spread_element] = STATE(340),
    [sym_decorator] = STATE(341),
    [sym_method_definition] = STATE(340),
    [sym_pair] = STATE(340),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_export_statement_repeat1] = STATE(341),
    [aux_sym_object_repeat1] = STATE(342),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(588),
    [anon_sym_LBRACK] = ACTIONS(468),
    [anon_sym_async] = ACTIONS(590),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(472),
    [sym_number] = ACTIONS(472),
    [sym_identifier] = ACTIONS(592),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(594),
    [anon_sym_set] = ACTIONS(594),
  },
  [89] = {
    [sym_parenthesized_expression] = STATE(344),
    [sym__expression] = STATE(344),
    [sym_yield_expression] = STATE(344),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(344),
    [sym_jsx_fragment] = STATE(344),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(344),
    [sym_anonymous_class] = STATE(344),
    [sym_class] = STATE(344),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(344),
    [sym_generator_function] = STATE(344),
    [sym_call_expression] = STATE(344),
    [sym_new_expression] = STATE(344),
    [sym_await_expression] = STATE(344),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(344),
    [sym_augmented_assignment_expression] = STATE(344),
    [sym_spread_element] = STATE(345),
    [sym_ternary_expression] = STATE(344),
    [sym_binary_expression] = STATE(344),
    [sym_unary_expression] = STATE(344),
    [sym_update_expression] = STATE(344),
    [sym_template_string] = STATE(344),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(346),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(596),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(598),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(600),
    [sym_number] = ACTIONS(598),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(600),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(600),
    [sym_false] = ACTIONS(600),
    [sym_null] = ACTIONS(600),
    [sym_undefined] = ACTIONS(600),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [90] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [91] = {
    [aux_sym_variable_declaration_repeat1] = STATE(350),
    [sym__automatic_semicolon] = ACTIONS(606),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(606),
    [sym_comment] = ACTIONS(71),
  },
  [92] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [93] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [94] = {
    [aux_sym_variable_declaration_repeat1] = STATE(354),
    [sym__automatic_semicolon] = ACTIONS(614),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(614),
    [sym_comment] = ACTIONS(71),
  },
  [95] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(358),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(358),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [96] = {
    [sym_export_statement] = STATE(383),
    [sym__declaration] = STATE(383),
    [sym_import_statement] = STATE(383),
    [sym_expression_statement] = STATE(383),
    [sym_variable_declaration] = STATE(383),
    [sym_lexical_declaration] = STATE(383),
    [sym_statement_block] = STATE(383),
    [sym_if_statement] = STATE(383),
    [sym_switch_statement] = STATE(383),
    [sym_for_statement] = STATE(383),
    [sym_for_in_statement] = STATE(383),
    [sym_for_of_statement] = STATE(383),
    [sym_while_statement] = STATE(383),
    [sym_do_statement] = STATE(383),
    [sym_try_statement] = STATE(383),
    [sym_with_statement] = STATE(383),
    [sym_break_statement] = STATE(383),
    [sym_continue_statement] = STATE(383),
    [sym_debugger_statement] = STATE(383),
    [sym_return_statement] = STATE(383),
    [sym_throw_statement] = STATE(383),
    [sym_empty_statement] = STATE(383),
    [sym_labeled_statement] = STATE(383),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [97] = {
    [sym_switch_body] = STATE(389),
    [anon_sym_LBRACE] = ACTIONS(666),
    [sym_comment] = ACTIONS(71),
  },
  [98] = {
    [sym_expression_statement] = STATE(394),
    [sym_variable_declaration] = STATE(394),
    [sym_lexical_declaration] = STATE(394),
    [sym_empty_statement] = STATE(394),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(396),
    [sym_array] = STATE(397),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(668),
    [anon_sym_let] = ACTIONS(670),
    [anon_sym_const] = ACTIONS(670),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(674),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [99] = {
    [sym_object] = STATE(279),
    [sym_assignment_pattern] = STATE(402),
    [sym_array] = STATE(281),
    [sym_spread_element] = STATE(402),
    [sym_decorator] = STATE(403),
    [sym_method_definition] = STATE(402),
    [sym_pair] = STATE(402),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_export_statement_repeat1] = STATE(403),
    [aux_sym_object_repeat1] = STATE(404),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(676),
    [anon_sym_LBRACK] = ACTIONS(468),
    [anon_sym_async] = ACTIONS(678),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(472),
    [sym_number] = ACTIONS(472),
    [sym_identifier] = ACTIONS(680),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(682),
    [anon_sym_set] = ACTIONS(682),
  },
  [100] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(405),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(120),
    [sym_assignment_pattern] = STATE(121),
    [sym_array] = STATE(122),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(405),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [sym_rest_parameter] = STATE(121),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(155),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(165),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(175),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(189),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(193),
    [anon_sym_set] = ACTIONS(193),
  },
  [101] = {
    [anon_sym_LBRACE] = ACTIONS(684),
    [anon_sym_EQ_GT] = ACTIONS(684),
    [sym_comment] = ACTIONS(71),
  },
  [102] = {
    [sym_parenthesized_expression] = STATE(406),
    [sym__expression] = STATE(406),
    [sym_yield_expression] = STATE(406),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(406),
    [sym_jsx_fragment] = STATE(406),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(406),
    [sym_anonymous_class] = STATE(406),
    [sym_class] = STATE(406),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(406),
    [sym_generator_function] = STATE(406),
    [sym_call_expression] = STATE(406),
    [sym_new_expression] = STATE(406),
    [sym_await_expression] = STATE(406),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(406),
    [sym_augmented_assignment_expression] = STATE(406),
    [sym_ternary_expression] = STATE(406),
    [sym_binary_expression] = STATE(406),
    [sym_unary_expression] = STATE(406),
    [sym_update_expression] = STATE(406),
    [sym_template_string] = STATE(406),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(239),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(177),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(688),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(690),
    [sym_number] = ACTIONS(688),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(690),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(690),
    [sym_false] = ACTIONS(690),
    [sym_null] = ACTIONS(690),
    [sym_undefined] = ACTIONS(690),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [103] = {
    [sym_parenthesized_expression] = STATE(408),
    [sym__expression] = STATE(408),
    [sym_yield_expression] = STATE(408),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(408),
    [sym_jsx_fragment] = STATE(408),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(408),
    [sym_anonymous_class] = STATE(408),
    [sym_class] = STATE(408),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(408),
    [sym_generator_function] = STATE(408),
    [sym_call_expression] = STATE(408),
    [sym_new_expression] = STATE(408),
    [sym_await_expression] = STATE(408),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(408),
    [sym_augmented_assignment_expression] = STATE(408),
    [sym_spread_element] = STATE(409),
    [sym_ternary_expression] = STATE(408),
    [sym_binary_expression] = STATE(408),
    [sym_unary_expression] = STATE(408),
    [sym_update_expression] = STATE(408),
    [sym_template_string] = STATE(408),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(410),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(692),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(694),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(696),
    [sym_number] = ACTIONS(694),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(696),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(696),
    [sym_false] = ACTIONS(696),
    [sym_null] = ACTIONS(696),
    [sym_undefined] = ACTIONS(696),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [104] = {
    [sym_nested_identifier] = STATE(414),
    [sym_jsx_namespace_name] = STATE(415),
    [anon_sym_GT] = ACTIONS(698),
    [sym_jsx_identifier] = ACTIONS(700),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(702),
  },
  [105] = {
    [sym_class_heritage] = STATE(418),
    [sym_class_body] = STATE(419),
    [anon_sym_LBRACE] = ACTIONS(704),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(706),
  },
  [106] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(427),
    [aux_sym_formal_parameters_repeat1] = STATE(429),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(708),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_RPAREN] = ACTIONS(711),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(720),
    [anon_sym_PLUS_EQ] = ACTIONS(722),
    [anon_sym_DASH_EQ] = ACTIONS(722),
    [anon_sym_STAR_EQ] = ACTIONS(722),
    [anon_sym_SLASH_EQ] = ACTIONS(722),
    [anon_sym_PERCENT_EQ] = ACTIONS(722),
    [anon_sym_CARET_EQ] = ACTIONS(722),
    [anon_sym_AMP_EQ] = ACTIONS(722),
    [anon_sym_PIPE_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_LT_LT_EQ] = ACTIONS(722),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(724),
  },
  [107] = {
    [sym_formal_parameters] = STATE(432),
    [anon_sym_STAR] = ACTIONS(726),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(728),
  },
  [108] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(435),
    [sym__expression] = STATE(435),
    [sym_yield_expression] = STATE(435),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(435),
    [sym_jsx_fragment] = STATE(435),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(435),
    [sym_anonymous_class] = STATE(435),
    [sym_class] = STATE(435),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(435),
    [sym_generator_function] = STATE(435),
    [sym_call_expression] = STATE(435),
    [sym_new_expression] = STATE(435),
    [sym_await_expression] = STATE(435),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(435),
    [sym_augmented_assignment_expression] = STATE(435),
    [sym_ternary_expression] = STATE(435),
    [sym_binary_expression] = STATE(435),
    [sym_unary_expression] = STATE(435),
    [sym_update_expression] = STATE(435),
    [sym_template_string] = STATE(435),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(732),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(734),
    [sym_number] = ACTIONS(732),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(734),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(734),
    [sym_false] = ACTIONS(734),
    [sym_null] = ACTIONS(734),
    [sym_undefined] = ACTIONS(734),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [109] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [110] = {
    [sym_parenthesized_expression] = STATE(437),
    [sym__expression] = STATE(437),
    [sym_yield_expression] = STATE(437),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(437),
    [sym_jsx_fragment] = STATE(437),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(437),
    [sym_anonymous_class] = STATE(437),
    [sym_class] = STATE(437),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(437),
    [sym_generator_function] = STATE(437),
    [sym_call_expression] = STATE(437),
    [sym_new_expression] = STATE(437),
    [sym_await_expression] = STATE(437),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(437),
    [sym_augmented_assignment_expression] = STATE(437),
    [sym_ternary_expression] = STATE(437),
    [sym_binary_expression] = STATE(437),
    [sym_unary_expression] = STATE(437),
    [sym_update_expression] = STATE(437),
    [sym_template_string] = STATE(437),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(740),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(742),
    [sym_number] = ACTIONS(740),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(742),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(742),
    [sym_false] = ACTIONS(742),
    [sym_null] = ACTIONS(742),
    [sym_undefined] = ACTIONS(742),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [111] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(744),
  },
  [112] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [113] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [114] = {
    [sym_template_substitution] = STATE(442),
    [aux_sym_template_string_repeat1] = STATE(442),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(754),
    [sym__template_chars] = ACTIONS(756),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(342),
  },
  [115] = {
    [sym__initializer] = STATE(447),
    [aux_sym_formal_parameters_repeat1] = STATE(448),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(758),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(761),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_EQ_GT] = ACTIONS(720),
    [anon_sym_PLUS_EQ] = ACTIONS(768),
    [anon_sym_DASH_EQ] = ACTIONS(768),
    [anon_sym_STAR_EQ] = ACTIONS(768),
    [anon_sym_SLASH_EQ] = ACTIONS(768),
    [anon_sym_PERCENT_EQ] = ACTIONS(768),
    [anon_sym_CARET_EQ] = ACTIONS(768),
    [anon_sym_AMP_EQ] = ACTIONS(768),
    [anon_sym_PIPE_EQ] = ACTIONS(768),
    [anon_sym_GT_GT_EQ] = ACTIONS(768),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(768),
    [anon_sym_LT_LT_EQ] = ACTIONS(768),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [116] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_DOT] = ACTIONS(766),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [117] = {
    [sym__initializer] = STATE(428),
    [aux_sym_formal_parameters_repeat1] = STATE(429),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(708),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_RPAREN] = ACTIONS(711),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_PLUS_EQ] = ACTIONS(722),
    [anon_sym_DASH_EQ] = ACTIONS(722),
    [anon_sym_STAR_EQ] = ACTIONS(722),
    [anon_sym_SLASH_EQ] = ACTIONS(722),
    [anon_sym_PERCENT_EQ] = ACTIONS(722),
    [anon_sym_CARET_EQ] = ACTIONS(722),
    [anon_sym_AMP_EQ] = ACTIONS(722),
    [anon_sym_PIPE_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_LT_LT_EQ] = ACTIONS(722),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [118] = {
    [anon_sym_RPAREN] = ACTIONS(774),
    [sym_comment] = ACTIONS(71),
  },
  [119] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(778),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(368),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [120] = {
    [sym__initializer] = STATE(461),
    [aux_sym_formal_parameters_repeat1] = STATE(462),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(758),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(798),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [121] = {
    [aux_sym_formal_parameters_repeat1] = STATE(448),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(803),
    [sym_comment] = ACTIONS(71),
  },
  [122] = {
    [sym__initializer] = STATE(464),
    [aux_sym_formal_parameters_repeat1] = STATE(465),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(758),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(805),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [123] = {
    [sym_jsx_element] = STATE(468),
    [sym_jsx_expression] = STATE(468),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(467),
    [sym_jsx_self_closing_element] = STATE(468),
    [aux_sym_jsx_element_repeat1] = STATE(468),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(808),
    [sym_jsx_text] = ACTIONS(810),
    [sym_comment] = ACTIONS(71),
  },
  [124] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [125] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_PLUS_EQ] = ACTIONS(768),
    [anon_sym_DASH_EQ] = ACTIONS(768),
    [anon_sym_STAR_EQ] = ACTIONS(768),
    [anon_sym_SLASH_EQ] = ACTIONS(768),
    [anon_sym_PERCENT_EQ] = ACTIONS(768),
    [anon_sym_CARET_EQ] = ACTIONS(768),
    [anon_sym_AMP_EQ] = ACTIONS(768),
    [anon_sym_PIPE_EQ] = ACTIONS(768),
    [anon_sym_GT_GT_EQ] = ACTIONS(768),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(768),
    [anon_sym_LT_LT_EQ] = ACTIONS(768),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [126] = {
    [anon_sym_EQ_GT] = ACTIONS(720),
    [sym_comment] = ACTIONS(71),
  },
  [127] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(812),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [128] = {
    [sym_export_statement] = STATE(471),
    [sym__declaration] = STATE(471),
    [sym_import_statement] = STATE(471),
    [sym_expression_statement] = STATE(471),
    [sym_variable_declaration] = STATE(471),
    [sym_lexical_declaration] = STATE(471),
    [sym_statement_block] = STATE(471),
    [sym_if_statement] = STATE(471),
    [sym_switch_statement] = STATE(471),
    [sym_for_statement] = STATE(471),
    [sym_for_in_statement] = STATE(471),
    [sym_for_of_statement] = STATE(471),
    [sym_while_statement] = STATE(471),
    [sym_do_statement] = STATE(471),
    [sym_try_statement] = STATE(471),
    [sym_with_statement] = STATE(471),
    [sym_break_statement] = STATE(471),
    [sym_continue_statement] = STATE(471),
    [sym_debugger_statement] = STATE(471),
    [sym_return_statement] = STATE(471),
    [sym_throw_statement] = STATE(471),
    [sym_empty_statement] = STATE(471),
    [sym_labeled_statement] = STATE(471),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [129] = {
    [sym_export_clause] = STATE(57),
    [sym__declaration] = STATE(58),
    [sym_variable_declaration] = STATE(58),
    [sym_lexical_declaration] = STATE(58),
    [sym_class] = STATE(59),
    [sym_function] = STATE(59),
    [sym_generator_function] = STATE(59),
    [sym_decorator] = STATE(475),
    [aux_sym_export_statement_repeat1] = STATE(475),
    [anon_sym_STAR] = ACTIONS(89),
    [anon_sym_default] = ACTIONS(91),
    [anon_sym_LBRACE] = ACTIONS(93),
    [anon_sym_var] = ACTIONS(95),
    [anon_sym_let] = ACTIONS(97),
    [anon_sym_const] = ACTIONS(97),
    [anon_sym_class] = ACTIONS(814),
    [anon_sym_async] = ACTIONS(816),
    [anon_sym_function] = ACTIONS(818),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [130] = {
    [sym_export_statement] = STATE(79),
    [sym__declaration] = STATE(79),
    [sym_import_statement] = STATE(79),
    [sym_expression_statement] = STATE(79),
    [sym_variable_declaration] = STATE(79),
    [sym_lexical_declaration] = STATE(79),
    [sym_statement_block] = STATE(79),
    [sym_if_statement] = STATE(79),
    [sym_switch_statement] = STATE(79),
    [sym_for_statement] = STATE(79),
    [sym_for_in_statement] = STATE(79),
    [sym_for_of_statement] = STATE(79),
    [sym_while_statement] = STATE(79),
    [sym_do_statement] = STATE(79),
    [sym_try_statement] = STATE(79),
    [sym_with_statement] = STATE(79),
    [sym_break_statement] = STATE(79),
    [sym_continue_statement] = STATE(79),
    [sym_debugger_statement] = STATE(79),
    [sym_return_statement] = STATE(79),
    [sym_throw_statement] = STATE(79),
    [sym_empty_statement] = STATE(79),
    [sym_labeled_statement] = STATE(79),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(79),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(820),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [131] = {
    [sym_parenthesized_expression] = STATE(477),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [132] = {
    [anon_sym_LPAREN] = ACTIONS(822),
    [sym_comment] = ACTIONS(71),
  },
  [133] = {
    [sym_parenthesized_expression] = STATE(479),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [134] = {
    [sym_statement_block] = STATE(480),
    [anon_sym_LBRACE] = ACTIONS(217),
    [sym_comment] = ACTIONS(71),
  },
  [135] = {
    [sym_parenthesized_expression] = STATE(481),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [136] = {
    [sym_class_heritage] = STATE(187),
    [sym_class_body] = STATE(188),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(824),
  },
  [137] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_function] = ACTIONS(826),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(308),
  },
  [138] = {
    [sym_formal_parameters] = STATE(486),
    [anon_sym_STAR] = ACTIONS(828),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(830),
  },
  [139] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(832),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [140] = {
    [anon_sym_while] = ACTIONS(834),
    [sym_comment] = ACTIONS(71),
  },
  [141] = {
    [sym__automatic_semicolon] = ACTIONS(398),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(400),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [142] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(398),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(400),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [143] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(836),
    [anon_sym_class] = ACTIONS(838),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [144] = {
    [sym_export_statement] = STATE(492),
    [sym__declaration] = STATE(492),
    [sym_import_statement] = STATE(492),
    [sym_expression_statement] = STATE(492),
    [sym_variable_declaration] = STATE(492),
    [sym_lexical_declaration] = STATE(492),
    [sym_statement_block] = STATE(492),
    [sym_if_statement] = STATE(492),
    [sym_switch_statement] = STATE(492),
    [sym_for_statement] = STATE(492),
    [sym_for_in_statement] = STATE(492),
    [sym_for_of_statement] = STATE(492),
    [sym_while_statement] = STATE(492),
    [sym_do_statement] = STATE(492),
    [sym_try_statement] = STATE(492),
    [sym_with_statement] = STATE(492),
    [sym_break_statement] = STATE(492),
    [sym_continue_statement] = STATE(492),
    [sym_debugger_statement] = STATE(492),
    [sym_return_statement] = STATE(492),
    [sym_throw_statement] = STATE(492),
    [sym_empty_statement] = STATE(492),
    [sym_labeled_statement] = STATE(492),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(492),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(840),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [145] = {
    [sym_catch_clause] = STATE(495),
    [sym_finally_clause] = STATE(496),
    [ts_builtin_sym_end] = ACTIONS(842),
    [anon_sym_export] = ACTIONS(844),
    [anon_sym_LBRACE] = ACTIONS(842),
    [anon_sym_RBRACE] = ACTIONS(842),
    [anon_sym_import] = ACTIONS(844),
    [anon_sym_var] = ACTIONS(844),
    [anon_sym_let] = ACTIONS(844),
    [anon_sym_const] = ACTIONS(844),
    [anon_sym_if] = ACTIONS(844),
    [anon_sym_switch] = ACTIONS(844),
    [anon_sym_for] = ACTIONS(844),
    [anon_sym_LPAREN] = ACTIONS(842),
    [anon_sym_while] = ACTIONS(844),
    [anon_sym_do] = ACTIONS(844),
    [anon_sym_try] = ACTIONS(844),
    [anon_sym_with] = ACTIONS(844),
    [anon_sym_break] = ACTIONS(844),
    [anon_sym_continue] = ACTIONS(844),
    [anon_sym_debugger] = ACTIONS(844),
    [anon_sym_return] = ACTIONS(844),
    [anon_sym_throw] = ACTIONS(844),
    [anon_sym_SEMI] = ACTIONS(842),
    [anon_sym_catch] = ACTIONS(846),
    [anon_sym_finally] = ACTIONS(848),
    [anon_sym_yield] = ACTIONS(844),
    [anon_sym_LBRACK] = ACTIONS(842),
    [anon_sym_LT] = ACTIONS(842),
    [anon_sym_class] = ACTIONS(844),
    [anon_sym_async] = ACTIONS(844),
    [anon_sym_function] = ACTIONS(844),
    [anon_sym_EQ_GT] = ACTIONS(842),
    [anon_sym_new] = ACTIONS(844),
    [anon_sym_await] = ACTIONS(844),
    [anon_sym_PLUS] = ACTIONS(844),
    [anon_sym_DASH] = ACTIONS(844),
    [anon_sym_BANG] = ACTIONS(842),
    [anon_sym_TILDE] = ACTIONS(842),
    [anon_sym_typeof] = ACTIONS(844),
    [anon_sym_void] = ACTIONS(844),
    [anon_sym_delete] = ACTIONS(844),
    [anon_sym_PLUS_PLUS] = ACTIONS(842),
    [anon_sym_DASH_DASH] = ACTIONS(842),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(842),
    [anon_sym_BQUOTE] = ACTIONS(842),
    [sym_regex] = ACTIONS(844),
    [sym_number] = ACTIONS(842),
    [sym_identifier] = ACTIONS(844),
    [sym_this] = ACTIONS(844),
    [sym_super] = ACTIONS(844),
    [sym_true] = ACTIONS(844),
    [sym_false] = ACTIONS(844),
    [sym_null] = ACTIONS(844),
    [sym_undefined] = ACTIONS(844),
    [anon_sym_AT] = ACTIONS(842),
    [anon_sym_get] = ACTIONS(844),
    [anon_sym_set] = ACTIONS(844),
  },
  [146] = {
    [sym_export_statement] = STATE(497),
    [sym__declaration] = STATE(497),
    [sym_import_statement] = STATE(497),
    [sym_expression_statement] = STATE(497),
    [sym_variable_declaration] = STATE(497),
    [sym_lexical_declaration] = STATE(497),
    [sym_statement_block] = STATE(497),
    [sym_if_statement] = STATE(497),
    [sym_switch_statement] = STATE(497),
    [sym_for_statement] = STATE(497),
    [sym_for_in_statement] = STATE(497),
    [sym_for_of_statement] = STATE(497),
    [sym_while_statement] = STATE(497),
    [sym_do_statement] = STATE(497),
    [sym_try_statement] = STATE(497),
    [sym_with_statement] = STATE(497),
    [sym_break_statement] = STATE(497),
    [sym_continue_statement] = STATE(497),
    [sym_debugger_statement] = STATE(497),
    [sym_return_statement] = STATE(497),
    [sym_throw_statement] = STATE(497),
    [sym_empty_statement] = STATE(497),
    [sym_labeled_statement] = STATE(497),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [147] = {
    [ts_builtin_sym_end] = ACTIONS(850),
    [anon_sym_export] = ACTIONS(852),
    [anon_sym_LBRACE] = ACTIONS(850),
    [anon_sym_RBRACE] = ACTIONS(850),
    [anon_sym_import] = ACTIONS(852),
    [anon_sym_var] = ACTIONS(852),
    [anon_sym_let] = ACTIONS(852),
    [anon_sym_const] = ACTIONS(852),
    [anon_sym_if] = ACTIONS(852),
    [anon_sym_switch] = ACTIONS(852),
    [anon_sym_for] = ACTIONS(852),
    [anon_sym_LPAREN] = ACTIONS(850),
    [anon_sym_while] = ACTIONS(852),
    [anon_sym_do] = ACTIONS(852),
    [anon_sym_try] = ACTIONS(852),
    [anon_sym_with] = ACTIONS(852),
    [anon_sym_break] = ACTIONS(852),
    [anon_sym_continue] = ACTIONS(852),
    [anon_sym_debugger] = ACTIONS(852),
    [anon_sym_return] = ACTIONS(852),
    [anon_sym_throw] = ACTIONS(852),
    [anon_sym_SEMI] = ACTIONS(850),
    [anon_sym_yield] = ACTIONS(852),
    [anon_sym_LBRACK] = ACTIONS(850),
    [anon_sym_LT] = ACTIONS(850),
    [anon_sym_class] = ACTIONS(852),
    [anon_sym_async] = ACTIONS(852),
    [anon_sym_function] = ACTIONS(852),
    [anon_sym_EQ_GT] = ACTIONS(850),
    [anon_sym_new] = ACTIONS(852),
    [anon_sym_await] = ACTIONS(852),
    [anon_sym_PLUS] = ACTIONS(852),
    [anon_sym_DASH] = ACTIONS(852),
    [anon_sym_BANG] = ACTIONS(850),
    [anon_sym_TILDE] = ACTIONS(850),
    [anon_sym_typeof] = ACTIONS(852),
    [anon_sym_void] = ACTIONS(852),
    [anon_sym_delete] = ACTIONS(852),
    [anon_sym_PLUS_PLUS] = ACTIONS(850),
    [anon_sym_DASH_DASH] = ACTIONS(850),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(850),
    [anon_sym_BQUOTE] = ACTIONS(850),
    [sym_regex] = ACTIONS(852),
    [sym_number] = ACTIONS(850),
    [sym_identifier] = ACTIONS(852),
    [sym_this] = ACTIONS(852),
    [sym_super] = ACTIONS(852),
    [sym_true] = ACTIONS(852),
    [sym_false] = ACTIONS(852),
    [sym_null] = ACTIONS(852),
    [sym_undefined] = ACTIONS(852),
    [anon_sym_AT] = ACTIONS(850),
    [anon_sym_get] = ACTIONS(852),
    [anon_sym_set] = ACTIONS(852),
  },
  [148] = {
    [sym__automatic_semicolon] = ACTIONS(854),
    [anon_sym_SEMI] = ACTIONS(854),
    [sym_comment] = ACTIONS(71),
  },
  [149] = {
    [ts_builtin_sym_end] = ACTIONS(856),
    [anon_sym_export] = ACTIONS(858),
    [anon_sym_LBRACE] = ACTIONS(856),
    [anon_sym_RBRACE] = ACTIONS(856),
    [anon_sym_import] = ACTIONS(858),
    [anon_sym_var] = ACTIONS(858),
    [anon_sym_let] = ACTIONS(858),
    [anon_sym_const] = ACTIONS(858),
    [anon_sym_if] = ACTIONS(858),
    [anon_sym_switch] = ACTIONS(858),
    [anon_sym_for] = ACTIONS(858),
    [anon_sym_LPAREN] = ACTIONS(856),
    [anon_sym_while] = ACTIONS(858),
    [anon_sym_do] = ACTIONS(858),
    [anon_sym_try] = ACTIONS(858),
    [anon_sym_with] = ACTIONS(858),
    [anon_sym_break] = ACTIONS(858),
    [anon_sym_continue] = ACTIONS(858),
    [anon_sym_debugger] = ACTIONS(858),
    [anon_sym_return] = ACTIONS(858),
    [anon_sym_throw] = ACTIONS(858),
    [anon_sym_SEMI] = ACTIONS(856),
    [anon_sym_yield] = ACTIONS(858),
    [anon_sym_LBRACK] = ACTIONS(856),
    [anon_sym_LT] = ACTIONS(856),
    [anon_sym_class] = ACTIONS(858),
    [anon_sym_async] = ACTIONS(858),
    [anon_sym_function] = ACTIONS(858),
    [anon_sym_EQ_GT] = ACTIONS(856),
    [anon_sym_new] = ACTIONS(858),
    [anon_sym_await] = ACTIONS(858),
    [anon_sym_PLUS] = ACTIONS(858),
    [anon_sym_DASH] = ACTIONS(858),
    [anon_sym_BANG] = ACTIONS(856),
    [anon_sym_TILDE] = ACTIONS(856),
    [anon_sym_typeof] = ACTIONS(858),
    [anon_sym_void] = ACTIONS(858),
    [anon_sym_delete] = ACTIONS(858),
    [anon_sym_PLUS_PLUS] = ACTIONS(856),
    [anon_sym_DASH_DASH] = ACTIONS(856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(856),
    [anon_sym_BQUOTE] = ACTIONS(856),
    [sym_regex] = ACTIONS(858),
    [sym_number] = ACTIONS(856),
    [sym_identifier] = ACTIONS(858),
    [sym_this] = ACTIONS(858),
    [sym_super] = ACTIONS(858),
    [sym_true] = ACTIONS(858),
    [sym_false] = ACTIONS(858),
    [sym_null] = ACTIONS(858),
    [sym_undefined] = ACTIONS(858),
    [anon_sym_AT] = ACTIONS(856),
    [anon_sym_get] = ACTIONS(858),
    [anon_sym_set] = ACTIONS(858),
  },
  [150] = {
    [sym__automatic_semicolon] = ACTIONS(860),
    [anon_sym_SEMI] = ACTIONS(860),
    [sym_comment] = ACTIONS(71),
  },
  [151] = {
    [ts_builtin_sym_end] = ACTIONS(862),
    [anon_sym_export] = ACTIONS(864),
    [anon_sym_LBRACE] = ACTIONS(862),
    [anon_sym_RBRACE] = ACTIONS(862),
    [anon_sym_import] = ACTIONS(864),
    [anon_sym_var] = ACTIONS(864),
    [anon_sym_let] = ACTIONS(864),
    [anon_sym_const] = ACTIONS(864),
    [anon_sym_if] = ACTIONS(864),
    [anon_sym_switch] = ACTIONS(864),
    [anon_sym_for] = ACTIONS(864),
    [anon_sym_LPAREN] = ACTIONS(862),
    [anon_sym_while] = ACTIONS(864),
    [anon_sym_do] = ACTIONS(864),
    [anon_sym_try] = ACTIONS(864),
    [anon_sym_with] = ACTIONS(864),
    [anon_sym_break] = ACTIONS(864),
    [anon_sym_continue] = ACTIONS(864),
    [anon_sym_debugger] = ACTIONS(864),
    [anon_sym_return] = ACTIONS(864),
    [anon_sym_throw] = ACTIONS(864),
    [anon_sym_SEMI] = ACTIONS(862),
    [anon_sym_yield] = ACTIONS(864),
    [anon_sym_LBRACK] = ACTIONS(862),
    [anon_sym_LT] = ACTIONS(862),
    [anon_sym_class] = ACTIONS(864),
    [anon_sym_async] = ACTIONS(864),
    [anon_sym_function] = ACTIONS(864),
    [anon_sym_EQ_GT] = ACTIONS(862),
    [anon_sym_new] = ACTIONS(864),
    [anon_sym_await] = ACTIONS(864),
    [anon_sym_PLUS] = ACTIONS(864),
    [anon_sym_DASH] = ACTIONS(864),
    [anon_sym_BANG] = ACTIONS(862),
    [anon_sym_TILDE] = ACTIONS(862),
    [anon_sym_typeof] = ACTIONS(864),
    [anon_sym_void] = ACTIONS(864),
    [anon_sym_delete] = ACTIONS(864),
    [anon_sym_PLUS_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(862),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(862),
    [anon_sym_BQUOTE] = ACTIONS(862),
    [sym_regex] = ACTIONS(864),
    [sym_number] = ACTIONS(862),
    [sym_identifier] = ACTIONS(864),
    [sym_this] = ACTIONS(864),
    [sym_super] = ACTIONS(864),
    [sym_true] = ACTIONS(864),
    [sym_false] = ACTIONS(864),
    [sym_null] = ACTIONS(864),
    [sym_undefined] = ACTIONS(864),
    [anon_sym_AT] = ACTIONS(862),
    [anon_sym_get] = ACTIONS(864),
    [anon_sym_set] = ACTIONS(864),
  },
  [152] = {
    [ts_builtin_sym_end] = ACTIONS(866),
    [anon_sym_export] = ACTIONS(868),
    [anon_sym_LBRACE] = ACTIONS(866),
    [anon_sym_RBRACE] = ACTIONS(866),
    [anon_sym_import] = ACTIONS(868),
    [anon_sym_var] = ACTIONS(868),
    [anon_sym_let] = ACTIONS(868),
    [anon_sym_const] = ACTIONS(868),
    [anon_sym_if] = ACTIONS(868),
    [anon_sym_switch] = ACTIONS(868),
    [anon_sym_for] = ACTIONS(868),
    [anon_sym_LPAREN] = ACTIONS(866),
    [anon_sym_while] = ACTIONS(868),
    [anon_sym_do] = ACTIONS(868),
    [anon_sym_try] = ACTIONS(868),
    [anon_sym_with] = ACTIONS(868),
    [anon_sym_break] = ACTIONS(868),
    [anon_sym_continue] = ACTIONS(868),
    [anon_sym_debugger] = ACTIONS(868),
    [anon_sym_return] = ACTIONS(868),
    [anon_sym_throw] = ACTIONS(868),
    [anon_sym_SEMI] = ACTIONS(866),
    [anon_sym_yield] = ACTIONS(868),
    [anon_sym_LBRACK] = ACTIONS(866),
    [anon_sym_LT] = ACTIONS(866),
    [anon_sym_class] = ACTIONS(868),
    [anon_sym_async] = ACTIONS(868),
    [anon_sym_function] = ACTIONS(868),
    [anon_sym_EQ_GT] = ACTIONS(866),
    [anon_sym_new] = ACTIONS(868),
    [anon_sym_await] = ACTIONS(868),
    [anon_sym_PLUS] = ACTIONS(868),
    [anon_sym_DASH] = ACTIONS(868),
    [anon_sym_BANG] = ACTIONS(866),
    [anon_sym_TILDE] = ACTIONS(866),
    [anon_sym_typeof] = ACTIONS(868),
    [anon_sym_void] = ACTIONS(868),
    [anon_sym_delete] = ACTIONS(868),
    [anon_sym_PLUS_PLUS] = ACTIONS(866),
    [anon_sym_DASH_DASH] = ACTIONS(866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(866),
    [anon_sym_BQUOTE] = ACTIONS(866),
    [sym_regex] = ACTIONS(868),
    [sym_number] = ACTIONS(866),
    [sym_identifier] = ACTIONS(868),
    [sym_this] = ACTIONS(868),
    [sym_super] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_null] = ACTIONS(868),
    [sym_undefined] = ACTIONS(868),
    [anon_sym_AT] = ACTIONS(866),
    [anon_sym_get] = ACTIONS(868),
    [anon_sym_set] = ACTIONS(868),
  },
  [153] = {
    [sym_object] = STATE(279),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(281),
    [sym_spread_element] = STATE(76),
    [sym_decorator] = STATE(504),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_export_statement_repeat1] = STATE(504),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(870),
    [anon_sym_LBRACK] = ACTIONS(468),
    [anon_sym_async] = ACTIONS(872),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(472),
    [sym_number] = ACTIONS(472),
    [sym_identifier] = ACTIONS(874),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(876),
    [anon_sym_set] = ACTIONS(876),
  },
  [154] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [155] = {
    [sym__automatic_semicolon] = ACTIONS(878),
    [anon_sym_SEMI] = ACTIONS(878),
    [sym_comment] = ACTIONS(71),
  },
  [156] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [157] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(838),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [158] = {
    [sym__automatic_semicolon] = ACTIONS(880),
    [anon_sym_SEMI] = ACTIONS(880),
    [sym_comment] = ACTIONS(71),
  },
  [159] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(882),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(882),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [160] = {
    [sym_parenthesized_expression] = STATE(507),
    [sym__expression] = STATE(507),
    [sym_yield_expression] = STATE(507),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(507),
    [sym_jsx_fragment] = STATE(507),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(507),
    [sym_anonymous_class] = STATE(507),
    [sym_class] = STATE(507),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(507),
    [sym_generator_function] = STATE(507),
    [sym_call_expression] = STATE(507),
    [sym_new_expression] = STATE(507),
    [sym_await_expression] = STATE(507),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(507),
    [sym_augmented_assignment_expression] = STATE(507),
    [sym_spread_element] = STATE(508),
    [sym_ternary_expression] = STATE(507),
    [sym_binary_expression] = STATE(507),
    [sym_unary_expression] = STATE(507),
    [sym_update_expression] = STATE(507),
    [sym_template_string] = STATE(507),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(884),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(884),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(886),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(888),
    [sym_number] = ACTIONS(886),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(888),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(888),
    [sym_false] = ACTIONS(888),
    [sym_null] = ACTIONS(888),
    [sym_undefined] = ACTIONS(888),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [161] = {
    [sym_parenthesized_expression] = STATE(509),
    [sym__expression] = STATE(509),
    [sym_yield_expression] = STATE(509),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(509),
    [sym_jsx_fragment] = STATE(509),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(509),
    [sym_anonymous_class] = STATE(509),
    [sym_class] = STATE(509),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(509),
    [sym_generator_function] = STATE(509),
    [sym_call_expression] = STATE(509),
    [sym_new_expression] = STATE(509),
    [sym_await_expression] = STATE(509),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(509),
    [sym_augmented_assignment_expression] = STATE(509),
    [sym_ternary_expression] = STATE(509),
    [sym_binary_expression] = STATE(509),
    [sym_unary_expression] = STATE(509),
    [sym_update_expression] = STATE(509),
    [sym_template_string] = STATE(509),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(239),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(265),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(890),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(892),
    [sym_number] = ACTIONS(890),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(892),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(892),
    [sym_false] = ACTIONS(892),
    [sym_null] = ACTIONS(892),
    [sym_undefined] = ACTIONS(892),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [162] = {
    [sym__automatic_semicolon] = ACTIONS(894),
    [anon_sym_STAR] = ACTIONS(894),
    [anon_sym_COMMA] = ACTIONS(894),
    [anon_sym_RBRACE] = ACTIONS(894),
    [anon_sym_LPAREN] = ACTIONS(894),
    [anon_sym_RPAREN] = ACTIONS(894),
    [anon_sym_in] = ACTIONS(896),
    [anon_sym_of] = ACTIONS(894),
    [anon_sym_SEMI] = ACTIONS(894),
    [anon_sym_LBRACK] = ACTIONS(894),
    [anon_sym_LT] = ACTIONS(896),
    [anon_sym_GT] = ACTIONS(896),
    [anon_sym_SLASH] = ACTIONS(896),
    [anon_sym_DOT] = ACTIONS(894),
    [anon_sym_EQ] = ACTIONS(896),
    [anon_sym_QMARK] = ACTIONS(894),
    [anon_sym_AMP_AMP] = ACTIONS(894),
    [anon_sym_PIPE_PIPE] = ACTIONS(894),
    [anon_sym_GT_GT] = ACTIONS(896),
    [anon_sym_GT_GT_GT] = ACTIONS(894),
    [anon_sym_LT_LT] = ACTIONS(894),
    [anon_sym_AMP] = ACTIONS(896),
    [anon_sym_CARET] = ACTIONS(894),
    [anon_sym_PIPE] = ACTIONS(896),
    [anon_sym_PLUS] = ACTIONS(896),
    [anon_sym_DASH] = ACTIONS(896),
    [anon_sym_PERCENT] = ACTIONS(894),
    [anon_sym_LT_EQ] = ACTIONS(894),
    [anon_sym_EQ_EQ] = ACTIONS(896),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(894),
    [anon_sym_BANG_EQ] = ACTIONS(896),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(894),
    [anon_sym_GT_EQ] = ACTIONS(894),
    [anon_sym_instanceof] = ACTIONS(894),
    [anon_sym_PLUS_PLUS] = ACTIONS(894),
    [anon_sym_DASH_DASH] = ACTIONS(894),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(894),
  },
  [163] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(513),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_RBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(898),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(900),
    [anon_sym_PLUS_EQ] = ACTIONS(902),
    [anon_sym_DASH_EQ] = ACTIONS(902),
    [anon_sym_STAR_EQ] = ACTIONS(902),
    [anon_sym_SLASH_EQ] = ACTIONS(902),
    [anon_sym_PERCENT_EQ] = ACTIONS(902),
    [anon_sym_CARET_EQ] = ACTIONS(902),
    [anon_sym_AMP_EQ] = ACTIONS(902),
    [anon_sym_PIPE_EQ] = ACTIONS(902),
    [anon_sym_GT_GT_EQ] = ACTIONS(902),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(902),
    [anon_sym_LT_LT_EQ] = ACTIONS(902),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(904),
  },
  [164] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(514),
    [sym__expression] = STATE(514),
    [sym_yield_expression] = STATE(514),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(514),
    [sym_jsx_fragment] = STATE(514),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(514),
    [sym_anonymous_class] = STATE(514),
    [sym_class] = STATE(514),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(514),
    [sym_generator_function] = STATE(514),
    [sym_call_expression] = STATE(514),
    [sym_new_expression] = STATE(514),
    [sym_await_expression] = STATE(514),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(514),
    [sym_augmented_assignment_expression] = STATE(514),
    [sym_ternary_expression] = STATE(514),
    [sym_binary_expression] = STATE(514),
    [sym_unary_expression] = STATE(514),
    [sym_update_expression] = STATE(514),
    [sym_template_string] = STATE(514),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(906),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(908),
    [sym_number] = ACTIONS(906),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(908),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(908),
    [sym_false] = ACTIONS(908),
    [sym_null] = ACTIONS(908),
    [sym_undefined] = ACTIONS(908),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [165] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [166] = {
    [sym_parenthesized_expression] = STATE(515),
    [sym__expression] = STATE(515),
    [sym_yield_expression] = STATE(515),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(515),
    [sym_jsx_fragment] = STATE(515),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(515),
    [sym_anonymous_class] = STATE(515),
    [sym_class] = STATE(515),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(515),
    [sym_generator_function] = STATE(515),
    [sym_call_expression] = STATE(515),
    [sym_new_expression] = STATE(515),
    [sym_await_expression] = STATE(515),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(515),
    [sym_augmented_assignment_expression] = STATE(515),
    [sym_ternary_expression] = STATE(515),
    [sym_binary_expression] = STATE(515),
    [sym_unary_expression] = STATE(515),
    [sym_update_expression] = STATE(515),
    [sym_template_string] = STATE(515),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(910),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(912),
    [sym_number] = ACTIONS(910),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(912),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(912),
    [sym_false] = ACTIONS(912),
    [sym_null] = ACTIONS(912),
    [sym_undefined] = ACTIONS(912),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [167] = {
    [sym_parenthesized_expression] = STATE(516),
    [sym__expression] = STATE(516),
    [sym_yield_expression] = STATE(516),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(516),
    [sym_jsx_fragment] = STATE(516),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(516),
    [sym_anonymous_class] = STATE(516),
    [sym_class] = STATE(516),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(516),
    [sym_generator_function] = STATE(516),
    [sym_call_expression] = STATE(516),
    [sym_new_expression] = STATE(516),
    [sym_await_expression] = STATE(516),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(516),
    [sym_augmented_assignment_expression] = STATE(516),
    [sym_ternary_expression] = STATE(516),
    [sym_binary_expression] = STATE(516),
    [sym_unary_expression] = STATE(516),
    [sym_update_expression] = STATE(516),
    [sym_template_string] = STATE(516),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(914),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(916),
    [sym_number] = ACTIONS(914),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(916),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(916),
    [sym_false] = ACTIONS(916),
    [sym_null] = ACTIONS(916),
    [sym_undefined] = ACTIONS(916),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [168] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [169] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [170] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(898),
    [anon_sym_EQ_GT] = ACTIONS(900),
    [anon_sym_PLUS_EQ] = ACTIONS(920),
    [anon_sym_DASH_EQ] = ACTIONS(920),
    [anon_sym_STAR_EQ] = ACTIONS(920),
    [anon_sym_SLASH_EQ] = ACTIONS(920),
    [anon_sym_PERCENT_EQ] = ACTIONS(920),
    [anon_sym_CARET_EQ] = ACTIONS(920),
    [anon_sym_AMP_EQ] = ACTIONS(920),
    [anon_sym_PIPE_EQ] = ACTIONS(920),
    [anon_sym_GT_GT_EQ] = ACTIONS(920),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(920),
    [anon_sym_LT_LT_EQ] = ACTIONS(920),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [171] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_RBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(898),
    [anon_sym_PLUS_EQ] = ACTIONS(902),
    [anon_sym_DASH_EQ] = ACTIONS(902),
    [anon_sym_STAR_EQ] = ACTIONS(902),
    [anon_sym_SLASH_EQ] = ACTIONS(902),
    [anon_sym_PERCENT_EQ] = ACTIONS(902),
    [anon_sym_CARET_EQ] = ACTIONS(902),
    [anon_sym_AMP_EQ] = ACTIONS(902),
    [anon_sym_PIPE_EQ] = ACTIONS(902),
    [anon_sym_GT_GT_EQ] = ACTIONS(902),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(902),
    [anon_sym_LT_LT_EQ] = ACTIONS(902),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [172] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(525),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(924),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [173] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(898),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [174] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(898),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [175] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(898),
    [anon_sym_PLUS_EQ] = ACTIONS(920),
    [anon_sym_DASH_EQ] = ACTIONS(920),
    [anon_sym_STAR_EQ] = ACTIONS(920),
    [anon_sym_SLASH_EQ] = ACTIONS(920),
    [anon_sym_PERCENT_EQ] = ACTIONS(920),
    [anon_sym_CARET_EQ] = ACTIONS(920),
    [anon_sym_AMP_EQ] = ACTIONS(920),
    [anon_sym_PIPE_EQ] = ACTIONS(920),
    [anon_sym_GT_GT_EQ] = ACTIONS(920),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(920),
    [anon_sym_LT_LT_EQ] = ACTIONS(920),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [176] = {
    [aux_sym_array_repeat1] = STATE(525),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(924),
    [sym_comment] = ACTIONS(71),
  },
  [177] = {
    [anon_sym_EQ_GT] = ACTIONS(900),
    [sym_comment] = ACTIONS(71),
  },
  [178] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(924),
    [sym_comment] = ACTIONS(71),
  },
  [179] = {
    [sym_jsx_element] = STATE(530),
    [sym_jsx_expression] = STATE(530),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(530),
    [aux_sym_jsx_element_repeat1] = STATE(530),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(942),
    [sym_jsx_text] = ACTIONS(944),
    [sym_comment] = ACTIONS(71),
  },
  [180] = {
    [sym_jsx_expression] = STATE(538),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(538),
    [aux_sym_jsx_opening_element_repeat1] = STATE(538),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(950),
    [anon_sym_SLASH] = ACTIONS(952),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [181] = {
    [sym_jsx_expression] = STATE(543),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(543),
    [aux_sym_jsx_opening_element_repeat1] = STATE(543),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(962),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [182] = {
    [sym_jsx_expression] = STATE(543),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(543),
    [aux_sym_jsx_opening_element_repeat1] = STATE(543),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(962),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [183] = {
    [sym_jsx_expression] = STATE(543),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(543),
    [aux_sym_jsx_opening_element_repeat1] = STATE(543),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(962),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [184] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(556),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(968),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [185] = {
    [sym_parenthesized_expression] = STATE(566),
    [sym__expression] = STATE(566),
    [sym_yield_expression] = STATE(566),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(566),
    [sym_jsx_fragment] = STATE(566),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(566),
    [sym_anonymous_class] = STATE(566),
    [sym_class] = STATE(566),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(566),
    [sym_generator_function] = STATE(566),
    [sym_call_expression] = STATE(566),
    [sym_new_expression] = STATE(566),
    [sym_await_expression] = STATE(566),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(566),
    [sym_augmented_assignment_expression] = STATE(566),
    [sym_ternary_expression] = STATE(566),
    [sym_binary_expression] = STATE(566),
    [sym_unary_expression] = STATE(566),
    [sym_update_expression] = STATE(566),
    [sym_template_string] = STATE(566),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1000),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1002),
    [sym_number] = ACTIONS(1000),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(1002),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1002),
    [sym_false] = ACTIONS(1002),
    [sym_null] = ACTIONS(1002),
    [sym_undefined] = ACTIONS(1002),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [186] = {
    [sym_class_heritage] = STATE(572),
    [sym_class_body] = STATE(573),
    [anon_sym_LBRACE] = ACTIONS(1008),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [187] = {
    [sym_class_body] = STATE(574),
    [anon_sym_LBRACE] = ACTIONS(285),
    [sym_comment] = ACTIONS(71),
  },
  [188] = {
    [sym__automatic_semicolon] = ACTIONS(1012),
    [anon_sym_STAR] = ACTIONS(1012),
    [anon_sym_COMMA] = ACTIONS(1012),
    [anon_sym_RBRACE] = ACTIONS(1012),
    [anon_sym_LPAREN] = ACTIONS(1012),
    [anon_sym_RPAREN] = ACTIONS(1012),
    [anon_sym_in] = ACTIONS(1014),
    [anon_sym_SEMI] = ACTIONS(1012),
    [anon_sym_LBRACK] = ACTIONS(1012),
    [anon_sym_LT] = ACTIONS(1014),
    [anon_sym_GT] = ACTIONS(1014),
    [anon_sym_SLASH] = ACTIONS(1014),
    [anon_sym_DOT] = ACTIONS(1012),
    [anon_sym_QMARK] = ACTIONS(1012),
    [anon_sym_AMP_AMP] = ACTIONS(1012),
    [anon_sym_PIPE_PIPE] = ACTIONS(1012),
    [anon_sym_GT_GT] = ACTIONS(1014),
    [anon_sym_GT_GT_GT] = ACTIONS(1012),
    [anon_sym_LT_LT] = ACTIONS(1012),
    [anon_sym_AMP] = ACTIONS(1014),
    [anon_sym_CARET] = ACTIONS(1012),
    [anon_sym_PIPE] = ACTIONS(1014),
    [anon_sym_PLUS] = ACTIONS(1014),
    [anon_sym_DASH] = ACTIONS(1014),
    [anon_sym_PERCENT] = ACTIONS(1012),
    [anon_sym_LT_EQ] = ACTIONS(1012),
    [anon_sym_EQ_EQ] = ACTIONS(1014),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1012),
    [anon_sym_BANG_EQ] = ACTIONS(1014),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1012),
    [anon_sym_GT_EQ] = ACTIONS(1012),
    [anon_sym_instanceof] = ACTIONS(1012),
    [anon_sym_PLUS_PLUS] = ACTIONS(1012),
    [anon_sym_DASH_DASH] = ACTIONS(1012),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1012),
  },
  [189] = {
    [sym_object] = STATE(578),
    [sym_assignment_pattern] = STATE(121),
    [sym_array] = STATE(579),
    [sym_rest_parameter] = STATE(121),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_RPAREN] = ACTIONS(155),
    [anon_sym_LBRACK] = ACTIONS(1016),
    [anon_sym_async] = ACTIONS(1018),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(175),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1020),
    [anon_sym_get] = ACTIONS(1018),
    [anon_sym_set] = ACTIONS(1018),
  },
  [190] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1022),
  },
  [191] = {
    [sym_parenthesized_expression] = STATE(581),
    [sym__expression] = STATE(581),
    [sym_yield_expression] = STATE(581),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(581),
    [sym_jsx_fragment] = STATE(581),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(581),
    [sym_anonymous_class] = STATE(581),
    [sym_class] = STATE(581),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(581),
    [sym_generator_function] = STATE(581),
    [sym_call_expression] = STATE(581),
    [sym_new_expression] = STATE(581),
    [sym_await_expression] = STATE(581),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(581),
    [sym_augmented_assignment_expression] = STATE(581),
    [sym_ternary_expression] = STATE(581),
    [sym_binary_expression] = STATE(581),
    [sym_unary_expression] = STATE(581),
    [sym_update_expression] = STATE(581),
    [sym_template_string] = STATE(581),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1024),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1026),
    [sym_number] = ACTIONS(1024),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1026),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1026),
    [sym_false] = ACTIONS(1026),
    [sym_null] = ACTIONS(1026),
    [sym_undefined] = ACTIONS(1026),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [192] = {
    [sym_formal_parameters] = STATE(583),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1028),
  },
  [193] = {
    [sym_statement_block] = STATE(584),
    [sym_parenthesized_expression] = STATE(585),
    [sym__expression] = STATE(585),
    [sym_yield_expression] = STATE(585),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(585),
    [sym_jsx_fragment] = STATE(585),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(585),
    [sym_anonymous_class] = STATE(585),
    [sym_class] = STATE(585),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(585),
    [sym_generator_function] = STATE(585),
    [sym_call_expression] = STATE(585),
    [sym_new_expression] = STATE(585),
    [sym_await_expression] = STATE(585),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(585),
    [sym_augmented_assignment_expression] = STATE(585),
    [sym_ternary_expression] = STATE(585),
    [sym_binary_expression] = STATE(585),
    [sym_unary_expression] = STATE(585),
    [sym_update_expression] = STATE(585),
    [sym_template_string] = STATE(585),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1030),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1032),
    [sym_number] = ACTIONS(1030),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1032),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1032),
    [sym_false] = ACTIONS(1032),
    [sym_null] = ACTIONS(1032),
    [sym_undefined] = ACTIONS(1032),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [194] = {
    [sym_parenthesized_expression] = STATE(586),
    [sym__expression] = STATE(586),
    [sym_yield_expression] = STATE(586),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(586),
    [sym_jsx_fragment] = STATE(586),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(586),
    [sym_anonymous_class] = STATE(586),
    [sym_class] = STATE(586),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(586),
    [sym_generator_function] = STATE(586),
    [sym_call_expression] = STATE(586),
    [sym_new_expression] = STATE(586),
    [sym_await_expression] = STATE(586),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(586),
    [sym_augmented_assignment_expression] = STATE(586),
    [sym_ternary_expression] = STATE(586),
    [sym_binary_expression] = STATE(586),
    [sym_unary_expression] = STATE(586),
    [sym_update_expression] = STATE(586),
    [sym_template_string] = STATE(586),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1034),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1036),
    [sym_number] = ACTIONS(1034),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1036),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1036),
    [sym_false] = ACTIONS(1036),
    [sym_null] = ACTIONS(1036),
    [sym_undefined] = ACTIONS(1036),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [195] = {
    [anon_sym_EQ_GT] = ACTIONS(1038),
    [sym_comment] = ACTIONS(71),
  },
  [196] = {
    [sym__automatic_semicolon] = ACTIONS(1040),
    [anon_sym_STAR] = ACTIONS(1040),
    [anon_sym_COMMA] = ACTIONS(1040),
    [anon_sym_RBRACE] = ACTIONS(1040),
    [anon_sym_LPAREN] = ACTIONS(1040),
    [anon_sym_RPAREN] = ACTIONS(1040),
    [anon_sym_in] = ACTIONS(1042),
    [anon_sym_SEMI] = ACTIONS(1040),
    [anon_sym_LBRACK] = ACTIONS(1040),
    [anon_sym_LT] = ACTIONS(1042),
    [anon_sym_GT] = ACTIONS(1042),
    [anon_sym_SLASH] = ACTIONS(1042),
    [anon_sym_DOT] = ACTIONS(1040),
    [anon_sym_QMARK] = ACTIONS(1040),
    [anon_sym_AMP_AMP] = ACTIONS(1040),
    [anon_sym_PIPE_PIPE] = ACTIONS(1040),
    [anon_sym_GT_GT] = ACTIONS(1042),
    [anon_sym_GT_GT_GT] = ACTIONS(1040),
    [anon_sym_LT_LT] = ACTIONS(1040),
    [anon_sym_AMP] = ACTIONS(1042),
    [anon_sym_CARET] = ACTIONS(1040),
    [anon_sym_PIPE] = ACTIONS(1042),
    [anon_sym_PLUS] = ACTIONS(1042),
    [anon_sym_DASH] = ACTIONS(1042),
    [anon_sym_PERCENT] = ACTIONS(1040),
    [anon_sym_LT_EQ] = ACTIONS(1040),
    [anon_sym_EQ_EQ] = ACTIONS(1042),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1040),
    [anon_sym_BANG_EQ] = ACTIONS(1042),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1040),
    [anon_sym_GT_EQ] = ACTIONS(1040),
    [anon_sym_instanceof] = ACTIONS(1040),
    [anon_sym_PLUS_PLUS] = ACTIONS(1040),
    [anon_sym_DASH_DASH] = ACTIONS(1040),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1040),
  },
  [197] = {
    [sym_formal_parameters] = STATE(589),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1044),
  },
  [198] = {
    [sym_formal_parameters] = STATE(583),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [199] = {
    [sym_statement_block] = STATE(591),
    [anon_sym_LBRACE] = ACTIONS(1046),
    [sym_comment] = ACTIONS(71),
  },
  [200] = {
    [sym_export_statement] = STATE(592),
    [sym__declaration] = STATE(592),
    [sym_import_statement] = STATE(592),
    [sym_expression_statement] = STATE(592),
    [sym_variable_declaration] = STATE(592),
    [sym_lexical_declaration] = STATE(592),
    [sym_statement_block] = STATE(592),
    [sym_if_statement] = STATE(592),
    [sym_switch_statement] = STATE(592),
    [sym_for_statement] = STATE(592),
    [sym_for_in_statement] = STATE(592),
    [sym_for_of_statement] = STATE(592),
    [sym_while_statement] = STATE(592),
    [sym_do_statement] = STATE(592),
    [sym_try_statement] = STATE(592),
    [sym_with_statement] = STATE(592),
    [sym_break_statement] = STATE(592),
    [sym_continue_statement] = STATE(592),
    [sym_debugger_statement] = STATE(592),
    [sym_return_statement] = STATE(592),
    [sym_throw_statement] = STATE(592),
    [sym_empty_statement] = STATE(592),
    [sym_labeled_statement] = STATE(592),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(592),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(870),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [201] = {
    [sym__automatic_semicolon] = ACTIONS(1048),
    [anon_sym_STAR] = ACTIONS(1048),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(1048),
    [anon_sym_RPAREN] = ACTIONS(1048),
    [anon_sym_in] = ACTIONS(1050),
    [anon_sym_SEMI] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(1048),
    [anon_sym_LT] = ACTIONS(1050),
    [anon_sym_GT] = ACTIONS(1050),
    [anon_sym_SLASH] = ACTIONS(1050),
    [anon_sym_DOT] = ACTIONS(1048),
    [anon_sym_QMARK] = ACTIONS(1048),
    [anon_sym_AMP_AMP] = ACTIONS(1048),
    [anon_sym_PIPE_PIPE] = ACTIONS(1048),
    [anon_sym_GT_GT] = ACTIONS(1050),
    [anon_sym_GT_GT_GT] = ACTIONS(1048),
    [anon_sym_LT_LT] = ACTIONS(1048),
    [anon_sym_AMP] = ACTIONS(1050),
    [anon_sym_CARET] = ACTIONS(1048),
    [anon_sym_PIPE] = ACTIONS(1050),
    [anon_sym_PLUS] = ACTIONS(1050),
    [anon_sym_DASH] = ACTIONS(1050),
    [anon_sym_PERCENT] = ACTIONS(1048),
    [anon_sym_LT_EQ] = ACTIONS(1048),
    [anon_sym_EQ_EQ] = ACTIONS(1050),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1048),
    [anon_sym_BANG_EQ] = ACTIONS(1050),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1048),
    [anon_sym_GT_EQ] = ACTIONS(1048),
    [anon_sym_instanceof] = ACTIONS(1048),
    [anon_sym_PLUS_PLUS] = ACTIONS(1048),
    [anon_sym_DASH_DASH] = ACTIONS(1048),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1048),
  },
  [202] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1048),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [203] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1052),
    [anon_sym_STAR] = ACTIONS(1052),
    [anon_sym_COMMA] = ACTIONS(1052),
    [anon_sym_RBRACE] = ACTIONS(1052),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(1052),
    [anon_sym_in] = ACTIONS(1054),
    [anon_sym_SEMI] = ACTIONS(1052),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1054),
    [anon_sym_GT] = ACTIONS(1054),
    [anon_sym_SLASH] = ACTIONS(1054),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1052),
    [anon_sym_AMP_AMP] = ACTIONS(1052),
    [anon_sym_PIPE_PIPE] = ACTIONS(1052),
    [anon_sym_GT_GT] = ACTIONS(1054),
    [anon_sym_GT_GT_GT] = ACTIONS(1052),
    [anon_sym_LT_LT] = ACTIONS(1052),
    [anon_sym_AMP] = ACTIONS(1054),
    [anon_sym_CARET] = ACTIONS(1052),
    [anon_sym_PIPE] = ACTIONS(1054),
    [anon_sym_PLUS] = ACTIONS(1054),
    [anon_sym_DASH] = ACTIONS(1054),
    [anon_sym_PERCENT] = ACTIONS(1052),
    [anon_sym_LT_EQ] = ACTIONS(1052),
    [anon_sym_EQ_EQ] = ACTIONS(1054),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1052),
    [anon_sym_BANG_EQ] = ACTIONS(1054),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1052),
    [anon_sym_GT_EQ] = ACTIONS(1052),
    [anon_sym_instanceof] = ACTIONS(1052),
    [anon_sym_PLUS_PLUS] = ACTIONS(1052),
    [anon_sym_DASH_DASH] = ACTIONS(1052),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [204] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1056),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(1056),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [205] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1058),
    [anon_sym_STAR] = ACTIONS(1058),
    [anon_sym_COMMA] = ACTIONS(1058),
    [anon_sym_RBRACE] = ACTIONS(1058),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(1058),
    [anon_sym_in] = ACTIONS(1060),
    [anon_sym_SEMI] = ACTIONS(1058),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1060),
    [anon_sym_GT] = ACTIONS(1060),
    [anon_sym_SLASH] = ACTIONS(1060),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1058),
    [anon_sym_AMP_AMP] = ACTIONS(1058),
    [anon_sym_PIPE_PIPE] = ACTIONS(1058),
    [anon_sym_GT_GT] = ACTIONS(1060),
    [anon_sym_GT_GT_GT] = ACTIONS(1058),
    [anon_sym_LT_LT] = ACTIONS(1058),
    [anon_sym_AMP] = ACTIONS(1060),
    [anon_sym_CARET] = ACTIONS(1058),
    [anon_sym_PIPE] = ACTIONS(1060),
    [anon_sym_PLUS] = ACTIONS(1060),
    [anon_sym_DASH] = ACTIONS(1060),
    [anon_sym_PERCENT] = ACTIONS(1058),
    [anon_sym_LT_EQ] = ACTIONS(1058),
    [anon_sym_EQ_EQ] = ACTIONS(1060),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1058),
    [anon_sym_BANG_EQ] = ACTIONS(1060),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1058),
    [anon_sym_GT_EQ] = ACTIONS(1058),
    [anon_sym_instanceof] = ACTIONS(1058),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [206] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1062),
    [anon_sym_STAR] = ACTIONS(1062),
    [anon_sym_COMMA] = ACTIONS(1062),
    [anon_sym_RBRACE] = ACTIONS(1062),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(1062),
    [anon_sym_in] = ACTIONS(1064),
    [anon_sym_SEMI] = ACTIONS(1062),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1064),
    [anon_sym_GT] = ACTIONS(1064),
    [anon_sym_SLASH] = ACTIONS(1064),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1062),
    [anon_sym_AMP_AMP] = ACTIONS(1062),
    [anon_sym_PIPE_PIPE] = ACTIONS(1062),
    [anon_sym_GT_GT] = ACTIONS(1064),
    [anon_sym_GT_GT_GT] = ACTIONS(1062),
    [anon_sym_LT_LT] = ACTIONS(1062),
    [anon_sym_AMP] = ACTIONS(1064),
    [anon_sym_CARET] = ACTIONS(1062),
    [anon_sym_PIPE] = ACTIONS(1064),
    [anon_sym_PLUS] = ACTIONS(1064),
    [anon_sym_DASH] = ACTIONS(1064),
    [anon_sym_PERCENT] = ACTIONS(1062),
    [anon_sym_LT_EQ] = ACTIONS(1062),
    [anon_sym_EQ_EQ] = ACTIONS(1064),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1062),
    [anon_sym_BANG_EQ] = ACTIONS(1064),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1062),
    [anon_sym_GT_EQ] = ACTIONS(1062),
    [anon_sym_instanceof] = ACTIONS(1062),
    [anon_sym_PLUS_PLUS] = ACTIONS(1062),
    [anon_sym_DASH_DASH] = ACTIONS(1062),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [207] = {
    [sym__automatic_semicolon] = ACTIONS(1066),
    [anon_sym_STAR] = ACTIONS(1066),
    [anon_sym_COMMA] = ACTIONS(1066),
    [anon_sym_RBRACE] = ACTIONS(1066),
    [anon_sym_LPAREN] = ACTIONS(1066),
    [anon_sym_RPAREN] = ACTIONS(1066),
    [anon_sym_in] = ACTIONS(1068),
    [anon_sym_SEMI] = ACTIONS(1066),
    [anon_sym_LBRACK] = ACTIONS(1066),
    [anon_sym_LT] = ACTIONS(1068),
    [anon_sym_GT] = ACTIONS(1068),
    [anon_sym_SLASH] = ACTIONS(1068),
    [anon_sym_DOT] = ACTIONS(1066),
    [anon_sym_QMARK] = ACTIONS(1066),
    [anon_sym_AMP_AMP] = ACTIONS(1066),
    [anon_sym_PIPE_PIPE] = ACTIONS(1066),
    [anon_sym_GT_GT] = ACTIONS(1068),
    [anon_sym_GT_GT_GT] = ACTIONS(1066),
    [anon_sym_LT_LT] = ACTIONS(1066),
    [anon_sym_AMP] = ACTIONS(1068),
    [anon_sym_CARET] = ACTIONS(1066),
    [anon_sym_PIPE] = ACTIONS(1068),
    [anon_sym_PLUS] = ACTIONS(1068),
    [anon_sym_DASH] = ACTIONS(1068),
    [anon_sym_PERCENT] = ACTIONS(1066),
    [anon_sym_LT_EQ] = ACTIONS(1066),
    [anon_sym_EQ_EQ] = ACTIONS(1068),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1066),
    [anon_sym_BANG_EQ] = ACTIONS(1068),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1066),
    [anon_sym_GT_EQ] = ACTIONS(1066),
    [anon_sym_instanceof] = ACTIONS(1066),
    [anon_sym_PLUS_PLUS] = ACTIONS(1066),
    [anon_sym_DASH_DASH] = ACTIONS(1066),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1066),
  },
  [208] = {
    [sym_parenthesized_expression] = STATE(594),
    [sym__expressions] = STATE(593),
    [sym__expression] = STATE(594),
    [sym_yield_expression] = STATE(594),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(594),
    [sym_jsx_fragment] = STATE(594),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(594),
    [sym_anonymous_class] = STATE(594),
    [sym_class] = STATE(594),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(594),
    [sym_generator_function] = STATE(594),
    [sym_call_expression] = STATE(594),
    [sym_new_expression] = STATE(594),
    [sym_await_expression] = STATE(594),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(594),
    [sym_augmented_assignment_expression] = STATE(594),
    [sym_ternary_expression] = STATE(594),
    [sym_binary_expression] = STATE(594),
    [sym_unary_expression] = STATE(594),
    [sym_update_expression] = STATE(594),
    [sym_sequence_expression] = STATE(593),
    [sym_template_string] = STATE(594),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1070),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1072),
    [sym_number] = ACTIONS(1070),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1072),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1072),
    [sym_false] = ACTIONS(1072),
    [sym_null] = ACTIONS(1072),
    [sym_undefined] = ACTIONS(1072),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [209] = {
    [sym_template_substitution] = STATE(596),
    [aux_sym_template_string_repeat1] = STATE(596),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1074),
    [sym__template_chars] = ACTIONS(1076),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(342),
  },
  [210] = {
    [sym_parenthesized_expression] = STATE(597),
    [sym__expression] = STATE(597),
    [sym_yield_expression] = STATE(597),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(597),
    [sym_jsx_fragment] = STATE(597),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(597),
    [sym_anonymous_class] = STATE(597),
    [sym_class] = STATE(597),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(597),
    [sym_generator_function] = STATE(597),
    [sym_call_expression] = STATE(597),
    [sym_new_expression] = STATE(597),
    [sym_await_expression] = STATE(597),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(597),
    [sym_augmented_assignment_expression] = STATE(597),
    [sym_ternary_expression] = STATE(597),
    [sym_binary_expression] = STATE(597),
    [sym_unary_expression] = STATE(597),
    [sym_update_expression] = STATE(597),
    [sym_template_string] = STATE(597),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1078),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1080),
    [sym_number] = ACTIONS(1078),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1080),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1080),
    [sym_false] = ACTIONS(1080),
    [sym_null] = ACTIONS(1080),
    [sym_undefined] = ACTIONS(1080),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [211] = {
    [sym_export_statement] = STATE(598),
    [sym__declaration] = STATE(598),
    [sym_import_statement] = STATE(598),
    [sym_expression_statement] = STATE(598),
    [sym_variable_declaration] = STATE(598),
    [sym_lexical_declaration] = STATE(598),
    [sym_statement_block] = STATE(598),
    [sym_if_statement] = STATE(598),
    [sym_switch_statement] = STATE(598),
    [sym_for_statement] = STATE(598),
    [sym_for_in_statement] = STATE(598),
    [sym_for_of_statement] = STATE(598),
    [sym_while_statement] = STATE(598),
    [sym_do_statement] = STATE(598),
    [sym_try_statement] = STATE(598),
    [sym_with_statement] = STATE(598),
    [sym_break_statement] = STATE(598),
    [sym_continue_statement] = STATE(598),
    [sym_debugger_statement] = STATE(598),
    [sym_return_statement] = STATE(598),
    [sym_throw_statement] = STATE(598),
    [sym_empty_statement] = STATE(598),
    [sym_labeled_statement] = STATE(598),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [212] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1082),
  },
  [213] = {
    [sym_parenthesized_expression] = STATE(600),
    [sym__expression] = STATE(600),
    [sym_yield_expression] = STATE(600),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(600),
    [sym_jsx_fragment] = STATE(600),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(600),
    [sym_anonymous_class] = STATE(600),
    [sym_class] = STATE(600),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(600),
    [sym_generator_function] = STATE(600),
    [sym_call_expression] = STATE(600),
    [sym_new_expression] = STATE(600),
    [sym_await_expression] = STATE(600),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(600),
    [sym_augmented_assignment_expression] = STATE(600),
    [sym_ternary_expression] = STATE(600),
    [sym_binary_expression] = STATE(600),
    [sym_unary_expression] = STATE(600),
    [sym_update_expression] = STATE(600),
    [sym_template_string] = STATE(600),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1084),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1086),
    [sym_number] = ACTIONS(1084),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1086),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1086),
    [sym_false] = ACTIONS(1086),
    [sym_null] = ACTIONS(1086),
    [sym_undefined] = ACTIONS(1086),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [214] = {
    [sym__automatic_semicolon] = ACTIONS(1088),
    [anon_sym_STAR] = ACTIONS(1088),
    [anon_sym_COMMA] = ACTIONS(1088),
    [anon_sym_RBRACE] = ACTIONS(1088),
    [anon_sym_LPAREN] = ACTIONS(1088),
    [anon_sym_RPAREN] = ACTIONS(1088),
    [anon_sym_in] = ACTIONS(1090),
    [anon_sym_SEMI] = ACTIONS(1088),
    [anon_sym_LBRACK] = ACTIONS(1088),
    [anon_sym_LT] = ACTIONS(1090),
    [anon_sym_GT] = ACTIONS(1090),
    [anon_sym_SLASH] = ACTIONS(1090),
    [anon_sym_DOT] = ACTIONS(1088),
    [anon_sym_QMARK] = ACTIONS(1088),
    [anon_sym_AMP_AMP] = ACTIONS(1088),
    [anon_sym_PIPE_PIPE] = ACTIONS(1088),
    [anon_sym_GT_GT] = ACTIONS(1090),
    [anon_sym_GT_GT_GT] = ACTIONS(1088),
    [anon_sym_LT_LT] = ACTIONS(1088),
    [anon_sym_AMP] = ACTIONS(1090),
    [anon_sym_CARET] = ACTIONS(1088),
    [anon_sym_PIPE] = ACTIONS(1090),
    [anon_sym_PLUS] = ACTIONS(1090),
    [anon_sym_DASH] = ACTIONS(1090),
    [anon_sym_PERCENT] = ACTIONS(1088),
    [anon_sym_LT_EQ] = ACTIONS(1088),
    [anon_sym_EQ_EQ] = ACTIONS(1090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1088),
    [anon_sym_BANG_EQ] = ACTIONS(1090),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1088),
    [anon_sym_GT_EQ] = ACTIONS(1088),
    [anon_sym_instanceof] = ACTIONS(1088),
    [anon_sym_PLUS_PLUS] = ACTIONS(1088),
    [anon_sym_DASH_DASH] = ACTIONS(1088),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1088),
  },
  [215] = {
    [sym_parenthesized_expression] = STATE(604),
    [sym__expression] = STATE(604),
    [sym_yield_expression] = STATE(604),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(604),
    [sym_jsx_fragment] = STATE(604),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(604),
    [sym_anonymous_class] = STATE(604),
    [sym_class] = STATE(604),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(604),
    [sym_generator_function] = STATE(604),
    [sym_call_expression] = STATE(604),
    [sym_new_expression] = STATE(604),
    [sym_await_expression] = STATE(604),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(604),
    [sym_augmented_assignment_expression] = STATE(604),
    [sym_spread_element] = STATE(605),
    [sym_ternary_expression] = STATE(604),
    [sym_binary_expression] = STATE(604),
    [sym_unary_expression] = STATE(604),
    [sym_update_expression] = STATE(604),
    [sym_template_string] = STATE(604),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(606),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(1094),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1096),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1098),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1100),
    [sym_number] = ACTIONS(1098),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1100),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1100),
    [sym_false] = ACTIONS(1100),
    [sym_null] = ACTIONS(1100),
    [sym_undefined] = ACTIONS(1100),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [216] = {
    [sym_parenthesized_expression] = STATE(608),
    [sym__expressions] = STATE(607),
    [sym__expression] = STATE(608),
    [sym_yield_expression] = STATE(608),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(608),
    [sym_jsx_fragment] = STATE(608),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(608),
    [sym_anonymous_class] = STATE(608),
    [sym_class] = STATE(608),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(608),
    [sym_generator_function] = STATE(608),
    [sym_call_expression] = STATE(608),
    [sym_new_expression] = STATE(608),
    [sym_await_expression] = STATE(608),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(608),
    [sym_augmented_assignment_expression] = STATE(608),
    [sym_ternary_expression] = STATE(608),
    [sym_binary_expression] = STATE(608),
    [sym_unary_expression] = STATE(608),
    [sym_update_expression] = STATE(608),
    [sym_sequence_expression] = STATE(607),
    [sym_template_string] = STATE(608),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1102),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1104),
    [sym_number] = ACTIONS(1102),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1104),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1104),
    [sym_false] = ACTIONS(1104),
    [sym_null] = ACTIONS(1104),
    [sym_undefined] = ACTIONS(1104),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [217] = {
    [sym__automatic_semicolon] = ACTIONS(1106),
    [anon_sym_STAR] = ACTIONS(1106),
    [anon_sym_COMMA] = ACTIONS(1106),
    [anon_sym_RBRACE] = ACTIONS(1106),
    [anon_sym_LPAREN] = ACTIONS(1106),
    [anon_sym_RPAREN] = ACTIONS(1106),
    [anon_sym_in] = ACTIONS(1108),
    [anon_sym_SEMI] = ACTIONS(1106),
    [anon_sym_LBRACK] = ACTIONS(1106),
    [anon_sym_LT] = ACTIONS(1108),
    [anon_sym_GT] = ACTIONS(1108),
    [anon_sym_SLASH] = ACTIONS(1108),
    [anon_sym_DOT] = ACTIONS(1106),
    [anon_sym_QMARK] = ACTIONS(1106),
    [anon_sym_AMP_AMP] = ACTIONS(1106),
    [anon_sym_PIPE_PIPE] = ACTIONS(1106),
    [anon_sym_GT_GT] = ACTIONS(1108),
    [anon_sym_GT_GT_GT] = ACTIONS(1106),
    [anon_sym_LT_LT] = ACTIONS(1106),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_CARET] = ACTIONS(1106),
    [anon_sym_PIPE] = ACTIONS(1108),
    [anon_sym_PLUS] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1108),
    [anon_sym_PERCENT] = ACTIONS(1106),
    [anon_sym_LT_EQ] = ACTIONS(1106),
    [anon_sym_EQ_EQ] = ACTIONS(1108),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1106),
    [anon_sym_BANG_EQ] = ACTIONS(1108),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1106),
    [anon_sym_GT_EQ] = ACTIONS(1106),
    [anon_sym_instanceof] = ACTIONS(1106),
    [anon_sym_PLUS_PLUS] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1106),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1106),
  },
  [218] = {
    [anon_sym_export] = ACTIONS(1110),
    [anon_sym_LPAREN] = ACTIONS(1110),
    [anon_sym_DOT] = ACTIONS(1110),
    [anon_sym_class] = ACTIONS(1110),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(1110),
  },
  [219] = {
    [sym_arguments] = STATE(610),
    [anon_sym_export] = ACTIONS(1112),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_DOT] = ACTIONS(1114),
    [anon_sym_class] = ACTIONS(1112),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(1112),
  },
  [220] = {
    [sym_arguments] = STATE(612),
    [anon_sym_export] = ACTIONS(1116),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_DOT] = ACTIONS(1118),
    [anon_sym_class] = ACTIONS(1116),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(1116),
  },
  [221] = {
    [anon_sym_export] = ACTIONS(1120),
    [anon_sym_class] = ACTIONS(1120),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(1120),
  },
  [222] = {
    [ts_builtin_sym_end] = ACTIONS(1122),
    [anon_sym_export] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_RBRACE] = ACTIONS(1122),
    [anon_sym_import] = ACTIONS(1124),
    [anon_sym_var] = ACTIONS(1124),
    [anon_sym_let] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_LPAREN] = ACTIONS(1122),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_try] = ACTIONS(1124),
    [anon_sym_with] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_debugger] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_throw] = ACTIONS(1124),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1122),
    [anon_sym_LT] = ACTIONS(1122),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_async] = ACTIONS(1124),
    [anon_sym_function] = ACTIONS(1124),
    [anon_sym_EQ_GT] = ACTIONS(1122),
    [anon_sym_new] = ACTIONS(1124),
    [anon_sym_await] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_typeof] = ACTIONS(1124),
    [anon_sym_void] = ACTIONS(1124),
    [anon_sym_delete] = ACTIONS(1124),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1122),
    [anon_sym_BQUOTE] = ACTIONS(1122),
    [sym_regex] = ACTIONS(1124),
    [sym_number] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1124),
    [sym_this] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_null] = ACTIONS(1124),
    [sym_undefined] = ACTIONS(1124),
    [anon_sym_AT] = ACTIONS(1122),
    [anon_sym_get] = ACTIONS(1124),
    [anon_sym_set] = ACTIONS(1124),
  },
  [223] = {
    [sym_parenthesized_expression] = STATE(613),
    [sym__expression] = STATE(613),
    [sym_yield_expression] = STATE(613),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(613),
    [sym_jsx_fragment] = STATE(613),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(613),
    [sym_anonymous_class] = STATE(613),
    [sym_class] = STATE(613),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(613),
    [sym_generator_function] = STATE(613),
    [sym_call_expression] = STATE(613),
    [sym_new_expression] = STATE(613),
    [sym_await_expression] = STATE(613),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(613),
    [sym_augmented_assignment_expression] = STATE(613),
    [sym_ternary_expression] = STATE(613),
    [sym_binary_expression] = STATE(613),
    [sym_unary_expression] = STATE(613),
    [sym_update_expression] = STATE(613),
    [sym_template_string] = STATE(613),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1126),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1128),
    [sym_number] = ACTIONS(1126),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1128),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [sym_null] = ACTIONS(1128),
    [sym_undefined] = ACTIONS(1128),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [224] = {
    [sym_parenthesized_expression] = STATE(614),
    [sym__expression] = STATE(614),
    [sym_yield_expression] = STATE(614),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(614),
    [sym_jsx_fragment] = STATE(614),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(614),
    [sym_anonymous_class] = STATE(614),
    [sym_class] = STATE(614),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(614),
    [sym_generator_function] = STATE(614),
    [sym_call_expression] = STATE(614),
    [sym_new_expression] = STATE(614),
    [sym_await_expression] = STATE(614),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(614),
    [sym_augmented_assignment_expression] = STATE(614),
    [sym_ternary_expression] = STATE(614),
    [sym_binary_expression] = STATE(614),
    [sym_unary_expression] = STATE(614),
    [sym_update_expression] = STATE(614),
    [sym_sequence_expression] = STATE(615),
    [sym_template_string] = STATE(614),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1130),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1132),
    [sym_number] = ACTIONS(1130),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1132),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1132),
    [sym_false] = ACTIONS(1132),
    [sym_null] = ACTIONS(1132),
    [sym_undefined] = ACTIONS(1132),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [225] = {
    [sym_parenthesized_expression] = STATE(625),
    [sym__expression] = STATE(625),
    [sym_yield_expression] = STATE(625),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(625),
    [sym_jsx_fragment] = STATE(625),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(625),
    [sym_anonymous_class] = STATE(625),
    [sym_class] = STATE(625),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(625),
    [sym_generator_function] = STATE(625),
    [sym_call_expression] = STATE(625),
    [sym_new_expression] = STATE(625),
    [sym_await_expression] = STATE(625),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(625),
    [sym_augmented_assignment_expression] = STATE(625),
    [sym_ternary_expression] = STATE(625),
    [sym_binary_expression] = STATE(625),
    [sym_unary_expression] = STATE(625),
    [sym_update_expression] = STATE(625),
    [sym_template_string] = STATE(625),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1150),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1152),
    [sym_number] = ACTIONS(1150),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(1152),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1152),
    [sym_false] = ACTIONS(1152),
    [sym_null] = ACTIONS(1152),
    [sym_undefined] = ACTIONS(1152),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [226] = {
    [sym_parenthesized_expression] = STATE(630),
    [sym__expression] = STATE(630),
    [sym_yield_expression] = STATE(630),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(630),
    [sym_jsx_fragment] = STATE(630),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(630),
    [sym_anonymous_class] = STATE(630),
    [sym_class] = STATE(630),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(630),
    [sym_generator_function] = STATE(630),
    [sym_call_expression] = STATE(630),
    [sym_new_expression] = STATE(630),
    [sym_await_expression] = STATE(630),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(630),
    [sym_augmented_assignment_expression] = STATE(630),
    [sym_ternary_expression] = STATE(630),
    [sym_binary_expression] = STATE(630),
    [sym_unary_expression] = STATE(630),
    [sym_update_expression] = STATE(630),
    [sym_template_string] = STATE(630),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1158),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1160),
    [sym_number] = ACTIONS(1158),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1160),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1160),
    [sym_false] = ACTIONS(1160),
    [sym_null] = ACTIONS(1160),
    [sym_undefined] = ACTIONS(1160),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [227] = {
    [sym_parenthesized_expression] = STATE(631),
    [sym__expression] = STATE(631),
    [sym_yield_expression] = STATE(631),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(631),
    [sym_jsx_fragment] = STATE(631),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(631),
    [sym_anonymous_class] = STATE(631),
    [sym_class] = STATE(631),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(631),
    [sym_generator_function] = STATE(631),
    [sym_call_expression] = STATE(631),
    [sym_new_expression] = STATE(631),
    [sym_await_expression] = STATE(631),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(631),
    [sym_augmented_assignment_expression] = STATE(631),
    [sym_ternary_expression] = STATE(631),
    [sym_binary_expression] = STATE(631),
    [sym_unary_expression] = STATE(631),
    [sym_update_expression] = STATE(631),
    [sym_template_string] = STATE(631),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1162),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1164),
    [sym_number] = ACTIONS(1162),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1164),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1164),
    [sym_false] = ACTIONS(1164),
    [sym_null] = ACTIONS(1164),
    [sym_undefined] = ACTIONS(1164),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [228] = {
    [sym_parenthesized_expression] = STATE(632),
    [sym__expression] = STATE(632),
    [sym_yield_expression] = STATE(632),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(632),
    [sym_jsx_fragment] = STATE(632),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(632),
    [sym_anonymous_class] = STATE(632),
    [sym_class] = STATE(632),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(632),
    [sym_generator_function] = STATE(632),
    [sym_call_expression] = STATE(632),
    [sym_new_expression] = STATE(632),
    [sym_await_expression] = STATE(632),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(632),
    [sym_augmented_assignment_expression] = STATE(632),
    [sym_ternary_expression] = STATE(632),
    [sym_binary_expression] = STATE(632),
    [sym_unary_expression] = STATE(632),
    [sym_update_expression] = STATE(632),
    [sym_template_string] = STATE(632),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1166),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1168),
    [sym_number] = ACTIONS(1166),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(1168),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [sym_null] = ACTIONS(1168),
    [sym_undefined] = ACTIONS(1168),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [229] = {
    [sym__automatic_semicolon] = ACTIONS(1062),
    [anon_sym_STAR] = ACTIONS(1062),
    [anon_sym_COMMA] = ACTIONS(1062),
    [anon_sym_RBRACE] = ACTIONS(1062),
    [anon_sym_LPAREN] = ACTIONS(1062),
    [anon_sym_RPAREN] = ACTIONS(1062),
    [anon_sym_in] = ACTIONS(1064),
    [anon_sym_SEMI] = ACTIONS(1062),
    [anon_sym_LBRACK] = ACTIONS(1062),
    [anon_sym_LT] = ACTIONS(1064),
    [anon_sym_GT] = ACTIONS(1064),
    [anon_sym_SLASH] = ACTIONS(1064),
    [anon_sym_DOT] = ACTIONS(1062),
    [anon_sym_QMARK] = ACTIONS(1062),
    [anon_sym_AMP_AMP] = ACTIONS(1062),
    [anon_sym_PIPE_PIPE] = ACTIONS(1062),
    [anon_sym_GT_GT] = ACTIONS(1064),
    [anon_sym_GT_GT_GT] = ACTIONS(1062),
    [anon_sym_LT_LT] = ACTIONS(1062),
    [anon_sym_AMP] = ACTIONS(1064),
    [anon_sym_CARET] = ACTIONS(1062),
    [anon_sym_PIPE] = ACTIONS(1064),
    [anon_sym_PLUS] = ACTIONS(1064),
    [anon_sym_DASH] = ACTIONS(1064),
    [anon_sym_PERCENT] = ACTIONS(1062),
    [anon_sym_LT_EQ] = ACTIONS(1062),
    [anon_sym_EQ_EQ] = ACTIONS(1064),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1062),
    [anon_sym_BANG_EQ] = ACTIONS(1064),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1062),
    [anon_sym_GT_EQ] = ACTIONS(1062),
    [anon_sym_instanceof] = ACTIONS(1062),
    [anon_sym_PLUS_PLUS] = ACTIONS(1062),
    [anon_sym_DASH_DASH] = ACTIONS(1062),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1062),
  },
  [230] = {
    [sym__automatic_semicolon] = ACTIONS(1170),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_COMMA] = ACTIONS(1170),
    [anon_sym_RBRACE] = ACTIONS(1170),
    [anon_sym_LPAREN] = ACTIONS(1170),
    [anon_sym_RPAREN] = ACTIONS(1170),
    [anon_sym_in] = ACTIONS(1172),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym_LBRACK] = ACTIONS(1170),
    [anon_sym_LT] = ACTIONS(1172),
    [anon_sym_GT] = ACTIONS(1172),
    [anon_sym_SLASH] = ACTIONS(1172),
    [anon_sym_DOT] = ACTIONS(1170),
    [anon_sym_QMARK] = ACTIONS(1170),
    [anon_sym_AMP_AMP] = ACTIONS(1170),
    [anon_sym_PIPE_PIPE] = ACTIONS(1170),
    [anon_sym_GT_GT] = ACTIONS(1172),
    [anon_sym_GT_GT_GT] = ACTIONS(1170),
    [anon_sym_LT_LT] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1172),
    [anon_sym_CARET] = ACTIONS(1170),
    [anon_sym_PIPE] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PERCENT] = ACTIONS(1170),
    [anon_sym_LT_EQ] = ACTIONS(1170),
    [anon_sym_EQ_EQ] = ACTIONS(1172),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1170),
    [anon_sym_BANG_EQ] = ACTIONS(1172),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1170),
    [anon_sym_GT_EQ] = ACTIONS(1170),
    [anon_sym_instanceof] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1170),
  },
  [231] = {
    [sym__automatic_semicolon] = ACTIONS(1174),
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_COMMA] = ACTIONS(1174),
    [anon_sym_RBRACE] = ACTIONS(1174),
    [anon_sym_LPAREN] = ACTIONS(1174),
    [anon_sym_RPAREN] = ACTIONS(1174),
    [anon_sym_in] = ACTIONS(1176),
    [anon_sym_SEMI] = ACTIONS(1174),
    [anon_sym_LBRACK] = ACTIONS(1174),
    [anon_sym_LT] = ACTIONS(1176),
    [anon_sym_GT] = ACTIONS(1176),
    [anon_sym_SLASH] = ACTIONS(1176),
    [anon_sym_DOT] = ACTIONS(1174),
    [anon_sym_QMARK] = ACTIONS(1174),
    [anon_sym_AMP_AMP] = ACTIONS(1174),
    [anon_sym_PIPE_PIPE] = ACTIONS(1174),
    [anon_sym_GT_GT] = ACTIONS(1176),
    [anon_sym_GT_GT_GT] = ACTIONS(1174),
    [anon_sym_LT_LT] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1176),
    [anon_sym_CARET] = ACTIONS(1174),
    [anon_sym_PIPE] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PERCENT] = ACTIONS(1174),
    [anon_sym_LT_EQ] = ACTIONS(1174),
    [anon_sym_EQ_EQ] = ACTIONS(1176),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1174),
    [anon_sym_BANG_EQ] = ACTIONS(1176),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1174),
    [anon_sym_GT_EQ] = ACTIONS(1174),
    [anon_sym_instanceof] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1174),
  },
  [232] = {
    [sym_parenthesized_expression] = STATE(635),
    [sym__expression] = STATE(635),
    [sym_yield_expression] = STATE(635),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(635),
    [sym_jsx_fragment] = STATE(635),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(635),
    [sym_anonymous_class] = STATE(635),
    [sym_class] = STATE(635),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(635),
    [sym_generator_function] = STATE(635),
    [sym_call_expression] = STATE(635),
    [sym_new_expression] = STATE(635),
    [sym_await_expression] = STATE(635),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(635),
    [sym_augmented_assignment_expression] = STATE(635),
    [sym_spread_element] = STATE(636),
    [sym_ternary_expression] = STATE(635),
    [sym_binary_expression] = STATE(635),
    [sym_unary_expression] = STATE(635),
    [sym_update_expression] = STATE(635),
    [sym_sequence_expression] = STATE(636),
    [sym_template_string] = STATE(635),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_RBRACE] = ACTIONS(1178),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1180),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1182),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1184),
    [sym_number] = ACTIONS(1182),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1184),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1184),
    [sym_false] = ACTIONS(1184),
    [sym_null] = ACTIONS(1184),
    [sym_undefined] = ACTIONS(1184),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [233] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(1186),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [234] = {
    [sym_jsx_element] = STATE(644),
    [sym_jsx_expression] = STATE(644),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(643),
    [sym_jsx_self_closing_element] = STATE(644),
    [aux_sym_jsx_element_repeat1] = STATE(644),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(1192),
    [sym_jsx_text] = ACTIONS(1194),
    [sym_comment] = ACTIONS(71),
  },
  [235] = {
    [sym__automatic_semicolon] = ACTIONS(1196),
    [anon_sym_STAR] = ACTIONS(1196),
    [anon_sym_COMMA] = ACTIONS(1196),
    [anon_sym_RBRACE] = ACTIONS(1196),
    [anon_sym_LPAREN] = ACTIONS(1196),
    [anon_sym_RPAREN] = ACTIONS(1196),
    [anon_sym_in] = ACTIONS(1198),
    [anon_sym_SEMI] = ACTIONS(1196),
    [anon_sym_LBRACK] = ACTIONS(1196),
    [anon_sym_LT] = ACTIONS(1198),
    [anon_sym_GT] = ACTIONS(1198),
    [anon_sym_SLASH] = ACTIONS(1198),
    [anon_sym_DOT] = ACTIONS(1196),
    [anon_sym_QMARK] = ACTIONS(1196),
    [anon_sym_AMP_AMP] = ACTIONS(1196),
    [anon_sym_PIPE_PIPE] = ACTIONS(1196),
    [anon_sym_GT_GT] = ACTIONS(1198),
    [anon_sym_GT_GT_GT] = ACTIONS(1196),
    [anon_sym_LT_LT] = ACTIONS(1196),
    [anon_sym_AMP] = ACTIONS(1198),
    [anon_sym_CARET] = ACTIONS(1196),
    [anon_sym_PIPE] = ACTIONS(1198),
    [anon_sym_PLUS] = ACTIONS(1198),
    [anon_sym_DASH] = ACTIONS(1198),
    [anon_sym_PERCENT] = ACTIONS(1196),
    [anon_sym_LT_EQ] = ACTIONS(1196),
    [anon_sym_EQ_EQ] = ACTIONS(1198),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1196),
    [anon_sym_BANG_EQ] = ACTIONS(1198),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1196),
    [anon_sym_GT_EQ] = ACTIONS(1196),
    [anon_sym_instanceof] = ACTIONS(1196),
    [anon_sym_PLUS_PLUS] = ACTIONS(1196),
    [anon_sym_DASH_DASH] = ACTIONS(1196),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1196),
  },
  [236] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(645),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(394),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [237] = {
    [ts_builtin_sym_end] = ACTIONS(1202),
    [anon_sym_export] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_RBRACE] = ACTIONS(1202),
    [anon_sym_import] = ACTIONS(1204),
    [anon_sym_var] = ACTIONS(1204),
    [anon_sym_let] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_LPAREN] = ACTIONS(1202),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_try] = ACTIONS(1204),
    [anon_sym_with] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_debugger] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_throw] = ACTIONS(1204),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym_yield] = ACTIONS(1204),
    [anon_sym_LBRACK] = ACTIONS(1202),
    [anon_sym_LT] = ACTIONS(1202),
    [anon_sym_class] = ACTIONS(1204),
    [anon_sym_async] = ACTIONS(1204),
    [anon_sym_function] = ACTIONS(1204),
    [anon_sym_EQ_GT] = ACTIONS(1202),
    [anon_sym_new] = ACTIONS(1204),
    [anon_sym_await] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_typeof] = ACTIONS(1204),
    [anon_sym_void] = ACTIONS(1204),
    [anon_sym_delete] = ACTIONS(1204),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1202),
    [anon_sym_BQUOTE] = ACTIONS(1202),
    [sym_regex] = ACTIONS(1204),
    [sym_number] = ACTIONS(1202),
    [sym_identifier] = ACTIONS(1204),
    [sym_this] = ACTIONS(1204),
    [sym_super] = ACTIONS(1204),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [sym_null] = ACTIONS(1204),
    [sym_undefined] = ACTIONS(1204),
    [anon_sym_AT] = ACTIONS(1202),
    [anon_sym_get] = ACTIONS(1204),
    [anon_sym_set] = ACTIONS(1204),
  },
  [238] = {
    [sym_export_statement] = STATE(238),
    [sym__declaration] = STATE(238),
    [sym_import_statement] = STATE(238),
    [sym_expression_statement] = STATE(238),
    [sym_variable_declaration] = STATE(238),
    [sym_lexical_declaration] = STATE(238),
    [sym_statement_block] = STATE(238),
    [sym_if_statement] = STATE(238),
    [sym_switch_statement] = STATE(238),
    [sym_for_statement] = STATE(238),
    [sym_for_in_statement] = STATE(238),
    [sym_for_of_statement] = STATE(238),
    [sym_while_statement] = STATE(238),
    [sym_do_statement] = STATE(238),
    [sym_try_statement] = STATE(238),
    [sym_with_statement] = STATE(238),
    [sym_break_statement] = STATE(238),
    [sym_continue_statement] = STATE(238),
    [sym_debugger_statement] = STATE(238),
    [sym_return_statement] = STATE(238),
    [sym_throw_statement] = STATE(238),
    [sym_empty_statement] = STATE(238),
    [sym_labeled_statement] = STATE(238),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(238),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [ts_builtin_sym_end] = ACTIONS(1206),
    [anon_sym_export] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1211),
    [anon_sym_import] = ACTIONS(1214),
    [anon_sym_var] = ACTIONS(1217),
    [anon_sym_let] = ACTIONS(1220),
    [anon_sym_const] = ACTIONS(1220),
    [anon_sym_if] = ACTIONS(1223),
    [anon_sym_switch] = ACTIONS(1226),
    [anon_sym_for] = ACTIONS(1229),
    [anon_sym_LPAREN] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(1235),
    [anon_sym_do] = ACTIONS(1238),
    [anon_sym_try] = ACTIONS(1241),
    [anon_sym_with] = ACTIONS(1244),
    [anon_sym_break] = ACTIONS(1247),
    [anon_sym_continue] = ACTIONS(1250),
    [anon_sym_debugger] = ACTIONS(1253),
    [anon_sym_return] = ACTIONS(1256),
    [anon_sym_throw] = ACTIONS(1259),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_yield] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1271),
    [anon_sym_class] = ACTIONS(1274),
    [anon_sym_async] = ACTIONS(1277),
    [anon_sym_function] = ACTIONS(1280),
    [anon_sym_EQ_GT] = ACTIONS(1283),
    [anon_sym_new] = ACTIONS(1286),
    [anon_sym_await] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_BANG] = ACTIONS(1295),
    [anon_sym_TILDE] = ACTIONS(1295),
    [anon_sym_typeof] = ACTIONS(1292),
    [anon_sym_void] = ACTIONS(1292),
    [anon_sym_delete] = ACTIONS(1292),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1301),
    [anon_sym_BQUOTE] = ACTIONS(1304),
    [sym_regex] = ACTIONS(1307),
    [sym_number] = ACTIONS(1301),
    [sym_identifier] = ACTIONS(1310),
    [sym_this] = ACTIONS(1307),
    [sym_super] = ACTIONS(1313),
    [sym_true] = ACTIONS(1307),
    [sym_false] = ACTIONS(1307),
    [sym_null] = ACTIONS(1307),
    [sym_undefined] = ACTIONS(1307),
    [anon_sym_AT] = ACTIONS(1316),
    [anon_sym_get] = ACTIONS(1319),
    [anon_sym_set] = ACTIONS(1319),
  },
  [239] = {
    [sym__declaration] = STATE(267),
    [sym_variable_declaration] = STATE(267),
    [sym_lexical_declaration] = STATE(267),
    [sym_class] = STATE(59),
    [sym_function] = STATE(59),
    [sym_generator_function] = STATE(59),
    [sym_decorator] = STATE(60),
    [aux_sym_export_statement_repeat1] = STATE(60),
    [anon_sym_default] = ACTIONS(1322),
    [anon_sym_var] = ACTIONS(95),
    [anon_sym_let] = ACTIONS(97),
    [anon_sym_const] = ACTIONS(97),
    [anon_sym_class] = ACTIONS(99),
    [anon_sym_async] = ACTIONS(101),
    [anon_sym_function] = ACTIONS(103),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [240] = {
    [sym_class_heritage] = STATE(649),
    [sym_class_body] = STATE(574),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1324),
  },
  [241] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(1326),
    [anon_sym_class] = ACTIONS(1326),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(1328),
  },
  [242] = {
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1331),
  },
  [243] = {
    [sym__automatic_semicolon] = ACTIONS(1333),
    [anon_sym_SEMI] = ACTIONS(1333),
    [sym_comment] = ACTIONS(71),
  },
  [244] = {
    [sym_parenthesized_expression] = STATE(652),
    [sym__expression] = STATE(652),
    [sym_yield_expression] = STATE(652),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(652),
    [sym_jsx_fragment] = STATE(652),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(652),
    [sym_anonymous_class] = STATE(652),
    [sym_class] = STATE(652),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(652),
    [sym_generator_function] = STATE(652),
    [sym_call_expression] = STATE(652),
    [sym_new_expression] = STATE(652),
    [sym_await_expression] = STATE(652),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(652),
    [sym_augmented_assignment_expression] = STATE(652),
    [sym_ternary_expression] = STATE(652),
    [sym_binary_expression] = STATE(652),
    [sym_unary_expression] = STATE(652),
    [sym_update_expression] = STATE(652),
    [sym_template_string] = STATE(652),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(239),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_SEMI] = ACTIONS(239),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(243),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(422),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1335),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1337),
    [sym_number] = ACTIONS(1335),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(1337),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1337),
    [sym_false] = ACTIONS(1337),
    [sym_null] = ACTIONS(1337),
    [sym_undefined] = ACTIONS(1337),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [245] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(656),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(1339),
    [anon_sym_function] = ACTIONS(826),
    [anon_sym_EQ_GT] = ACTIONS(1341),
    [anon_sym_PLUS_EQ] = ACTIONS(1343),
    [anon_sym_DASH_EQ] = ACTIONS(1343),
    [anon_sym_STAR_EQ] = ACTIONS(1343),
    [anon_sym_SLASH_EQ] = ACTIONS(1343),
    [anon_sym_PERCENT_EQ] = ACTIONS(1343),
    [anon_sym_CARET_EQ] = ACTIONS(1343),
    [anon_sym_AMP_EQ] = ACTIONS(1343),
    [anon_sym_PIPE_EQ] = ACTIONS(1343),
    [anon_sym_GT_GT_EQ] = ACTIONS(1343),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1343),
    [anon_sym_LT_LT_EQ] = ACTIONS(1343),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(1345),
  },
  [246] = {
    [sym_statement_block] = STATE(201),
    [sym_parenthesized_expression] = STATE(657),
    [sym__expression] = STATE(657),
    [sym_yield_expression] = STATE(657),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(657),
    [sym_jsx_fragment] = STATE(657),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(657),
    [sym_anonymous_class] = STATE(657),
    [sym_class] = STATE(657),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(657),
    [sym_generator_function] = STATE(657),
    [sym_call_expression] = STATE(657),
    [sym_new_expression] = STATE(657),
    [sym_await_expression] = STATE(657),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(657),
    [sym_augmented_assignment_expression] = STATE(657),
    [sym_ternary_expression] = STATE(657),
    [sym_binary_expression] = STATE(657),
    [sym_unary_expression] = STATE(657),
    [sym_update_expression] = STATE(657),
    [sym_template_string] = STATE(657),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1347),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1349),
    [sym_number] = ACTIONS(1347),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(1349),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1349),
    [sym_false] = ACTIONS(1349),
    [sym_null] = ACTIONS(1349),
    [sym_undefined] = ACTIONS(1349),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [247] = {
    [sym_parenthesized_expression] = STATE(203),
    [sym__expression] = STATE(203),
    [sym_yield_expression] = STATE(203),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(203),
    [sym_jsx_fragment] = STATE(203),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(203),
    [sym_anonymous_class] = STATE(203),
    [sym_class] = STATE(203),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(203),
    [sym_generator_function] = STATE(203),
    [sym_call_expression] = STATE(203),
    [sym_new_expression] = STATE(203),
    [sym_await_expression] = STATE(203),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(203),
    [sym_augmented_assignment_expression] = STATE(203),
    [sym_ternary_expression] = STATE(203),
    [sym_binary_expression] = STATE(203),
    [sym_unary_expression] = STATE(203),
    [sym_update_expression] = STATE(203),
    [sym_template_string] = STATE(203),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(322),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(324),
    [sym_number] = ACTIONS(322),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(324),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(324),
    [sym_false] = ACTIONS(324),
    [sym_null] = ACTIONS(324),
    [sym_undefined] = ACTIONS(324),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [248] = {
    [sym_parenthesized_expression] = STATE(658),
    [sym__expression] = STATE(658),
    [sym_yield_expression] = STATE(658),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(658),
    [sym_jsx_fragment] = STATE(658),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(658),
    [sym_anonymous_class] = STATE(658),
    [sym_class] = STATE(658),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(658),
    [sym_generator_function] = STATE(658),
    [sym_call_expression] = STATE(658),
    [sym_new_expression] = STATE(658),
    [sym_await_expression] = STATE(658),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(658),
    [sym_augmented_assignment_expression] = STATE(658),
    [sym_ternary_expression] = STATE(658),
    [sym_binary_expression] = STATE(658),
    [sym_unary_expression] = STATE(658),
    [sym_update_expression] = STATE(658),
    [sym_template_string] = STATE(658),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1351),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1353),
    [sym_number] = ACTIONS(1351),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(1353),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1353),
    [sym_false] = ACTIONS(1353),
    [sym_null] = ACTIONS(1353),
    [sym_undefined] = ACTIONS(1353),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [249] = {
    [sym_parenthesized_expression] = STATE(205),
    [sym__expression] = STATE(205),
    [sym_yield_expression] = STATE(205),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(205),
    [sym_jsx_fragment] = STATE(205),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(205),
    [sym_anonymous_class] = STATE(205),
    [sym_class] = STATE(205),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(205),
    [sym_generator_function] = STATE(205),
    [sym_call_expression] = STATE(205),
    [sym_new_expression] = STATE(205),
    [sym_await_expression] = STATE(205),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(205),
    [sym_augmented_assignment_expression] = STATE(205),
    [sym_ternary_expression] = STATE(205),
    [sym_binary_expression] = STATE(205),
    [sym_unary_expression] = STATE(205),
    [sym_update_expression] = STATE(205),
    [sym_template_string] = STATE(205),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(330),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(332),
    [sym_number] = ACTIONS(330),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(332),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(332),
    [sym_false] = ACTIONS(332),
    [sym_null] = ACTIONS(332),
    [sym_undefined] = ACTIONS(332),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [250] = {
    [sym_parenthesized_expression] = STATE(206),
    [sym__expression] = STATE(206),
    [sym_yield_expression] = STATE(206),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(206),
    [sym_jsx_fragment] = STATE(206),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(206),
    [sym_anonymous_class] = STATE(206),
    [sym_class] = STATE(206),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(206),
    [sym_generator_function] = STATE(206),
    [sym_call_expression] = STATE(206),
    [sym_new_expression] = STATE(206),
    [sym_await_expression] = STATE(206),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(206),
    [sym_augmented_assignment_expression] = STATE(206),
    [sym_ternary_expression] = STATE(206),
    [sym_binary_expression] = STATE(206),
    [sym_unary_expression] = STATE(206),
    [sym_update_expression] = STATE(206),
    [sym_template_string] = STATE(206),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(334),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(336),
    [sym_number] = ACTIONS(334),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(336),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(336),
    [sym_false] = ACTIONS(336),
    [sym_null] = ACTIONS(336),
    [sym_undefined] = ACTIONS(336),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [251] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(1339),
    [anon_sym_EQ_GT] = ACTIONS(1341),
    [anon_sym_PLUS_EQ] = ACTIONS(1357),
    [anon_sym_DASH_EQ] = ACTIONS(1357),
    [anon_sym_STAR_EQ] = ACTIONS(1357),
    [anon_sym_SLASH_EQ] = ACTIONS(1357),
    [anon_sym_PERCENT_EQ] = ACTIONS(1357),
    [anon_sym_CARET_EQ] = ACTIONS(1357),
    [anon_sym_AMP_EQ] = ACTIONS(1357),
    [anon_sym_PIPE_EQ] = ACTIONS(1357),
    [anon_sym_GT_GT_EQ] = ACTIONS(1357),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1357),
    [anon_sym_LT_LT_EQ] = ACTIONS(1357),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [252] = {
    [sym__initializer] = STATE(196),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(1339),
    [anon_sym_PLUS_EQ] = ACTIONS(1343),
    [anon_sym_DASH_EQ] = ACTIONS(1343),
    [anon_sym_STAR_EQ] = ACTIONS(1343),
    [anon_sym_SLASH_EQ] = ACTIONS(1343),
    [anon_sym_PERCENT_EQ] = ACTIONS(1343),
    [anon_sym_CARET_EQ] = ACTIONS(1343),
    [anon_sym_AMP_EQ] = ACTIONS(1343),
    [anon_sym_PIPE_EQ] = ACTIONS(1343),
    [anon_sym_GT_GT_EQ] = ACTIONS(1343),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1343),
    [anon_sym_LT_LT_EQ] = ACTIONS(1343),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [253] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1333),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(1333),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [254] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(1339),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [255] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(1339),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [256] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(1339),
    [anon_sym_PLUS_EQ] = ACTIONS(1357),
    [anon_sym_DASH_EQ] = ACTIONS(1357),
    [anon_sym_STAR_EQ] = ACTIONS(1357),
    [anon_sym_SLASH_EQ] = ACTIONS(1357),
    [anon_sym_PERCENT_EQ] = ACTIONS(1357),
    [anon_sym_CARET_EQ] = ACTIONS(1357),
    [anon_sym_AMP_EQ] = ACTIONS(1357),
    [anon_sym_PIPE_EQ] = ACTIONS(1357),
    [anon_sym_GT_GT_EQ] = ACTIONS(1357),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1357),
    [anon_sym_LT_LT_EQ] = ACTIONS(1357),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [257] = {
    [anon_sym_EQ_GT] = ACTIONS(1341),
    [sym_comment] = ACTIONS(71),
  },
  [258] = {
    [anon_sym_RBRACE] = ACTIONS(1377),
    [sym_comment] = ACTIONS(71),
  },
  [259] = {
    [sym__automatic_semicolon] = ACTIONS(1379),
    [anon_sym_from] = ACTIONS(1379),
    [anon_sym_SEMI] = ACTIONS(1379),
    [sym_comment] = ACTIONS(71),
  },
  [260] = {
    [anon_sym_COMMA] = ACTIONS(1381),
    [anon_sym_RBRACE] = ACTIONS(1381),
    [anon_sym_as] = ACTIONS(1383),
    [sym_comment] = ACTIONS(71),
  },
  [261] = {
    [aux_sym_export_clause_repeat1] = STATE(670),
    [anon_sym_COMMA] = ACTIONS(1385),
    [anon_sym_RBRACE] = ACTIONS(1387),
    [sym_comment] = ACTIONS(71),
  },
  [262] = {
    [sym_class_heritage] = STATE(672),
    [sym_class_body] = STATE(673),
    [anon_sym_LBRACE] = ACTIONS(1389),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [263] = {
    [sym_formal_parameters] = STATE(675),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1391),
  },
  [264] = {
    [sym_formal_parameters] = STATE(677),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1393),
  },
  [265] = {
    [sym_formal_parameters] = STATE(675),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [266] = {
    [sym_statement_block] = STATE(679),
    [anon_sym_LBRACE] = ACTIONS(1395),
    [sym_comment] = ACTIONS(71),
  },
  [267] = {
    [ts_builtin_sym_end] = ACTIONS(1397),
    [anon_sym_export] = ACTIONS(1399),
    [anon_sym_LBRACE] = ACTIONS(1397),
    [anon_sym_RBRACE] = ACTIONS(1397),
    [anon_sym_import] = ACTIONS(1399),
    [anon_sym_var] = ACTIONS(1399),
    [anon_sym_let] = ACTIONS(1399),
    [anon_sym_const] = ACTIONS(1399),
    [anon_sym_if] = ACTIONS(1399),
    [anon_sym_switch] = ACTIONS(1399),
    [anon_sym_for] = ACTIONS(1399),
    [anon_sym_LPAREN] = ACTIONS(1397),
    [anon_sym_while] = ACTIONS(1399),
    [anon_sym_do] = ACTIONS(1399),
    [anon_sym_try] = ACTIONS(1399),
    [anon_sym_with] = ACTIONS(1399),
    [anon_sym_break] = ACTIONS(1399),
    [anon_sym_continue] = ACTIONS(1399),
    [anon_sym_debugger] = ACTIONS(1399),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_throw] = ACTIONS(1399),
    [anon_sym_SEMI] = ACTIONS(1397),
    [anon_sym_yield] = ACTIONS(1399),
    [anon_sym_LBRACK] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1397),
    [anon_sym_class] = ACTIONS(1399),
    [anon_sym_async] = ACTIONS(1399),
    [anon_sym_function] = ACTIONS(1399),
    [anon_sym_EQ_GT] = ACTIONS(1397),
    [anon_sym_new] = ACTIONS(1399),
    [anon_sym_await] = ACTIONS(1399),
    [anon_sym_PLUS] = ACTIONS(1399),
    [anon_sym_DASH] = ACTIONS(1399),
    [anon_sym_BANG] = ACTIONS(1397),
    [anon_sym_TILDE] = ACTIONS(1397),
    [anon_sym_typeof] = ACTIONS(1399),
    [anon_sym_void] = ACTIONS(1399),
    [anon_sym_delete] = ACTIONS(1399),
    [anon_sym_PLUS_PLUS] = ACTIONS(1397),
    [anon_sym_DASH_DASH] = ACTIONS(1397),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1397),
    [anon_sym_BQUOTE] = ACTIONS(1397),
    [sym_regex] = ACTIONS(1399),
    [sym_number] = ACTIONS(1397),
    [sym_identifier] = ACTIONS(1399),
    [sym_this] = ACTIONS(1399),
    [sym_super] = ACTIONS(1399),
    [sym_true] = ACTIONS(1399),
    [sym_false] = ACTIONS(1399),
    [sym_null] = ACTIONS(1399),
    [sym_undefined] = ACTIONS(1399),
    [anon_sym_AT] = ACTIONS(1397),
    [anon_sym_get] = ACTIONS(1399),
    [anon_sym_set] = ACTIONS(1399),
  },
  [268] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1401),
  },
  [269] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(1326),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(1328),
  },
  [270] = {
    [sym_parenthesized_expression] = STATE(690),
    [sym__expression] = STATE(690),
    [sym_yield_expression] = STATE(690),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(690),
    [sym_jsx_fragment] = STATE(690),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(690),
    [sym_anonymous_class] = STATE(690),
    [sym_class] = STATE(690),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(690),
    [sym_generator_function] = STATE(690),
    [sym_call_expression] = STATE(690),
    [sym_new_expression] = STATE(690),
    [sym_await_expression] = STATE(690),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(690),
    [sym_augmented_assignment_expression] = STATE(690),
    [sym_ternary_expression] = STATE(690),
    [sym_binary_expression] = STATE(690),
    [sym_unary_expression] = STATE(690),
    [sym_update_expression] = STATE(690),
    [sym_template_string] = STATE(690),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1419),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1421),
    [sym_number] = ACTIONS(1419),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(1421),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1421),
    [sym_false] = ACTIONS(1421),
    [sym_null] = ACTIONS(1421),
    [sym_undefined] = ACTIONS(1421),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [271] = {
    [anon_sym_LPAREN] = ACTIONS(503),
    [sym_comment] = ACTIONS(71),
  },
  [272] = {
    [sym_formal_parameters] = STATE(695),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [273] = {
    [sym_object] = STATE(279),
    [sym_assignment_pattern] = STATE(700),
    [sym_array] = STATE(281),
    [sym_spread_element] = STATE(700),
    [sym_decorator] = STATE(701),
    [sym_method_definition] = STATE(700),
    [sym_pair] = STATE(700),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_export_statement_repeat1] = STATE(701),
    [aux_sym_object_repeat1] = STATE(702),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1427),
    [anon_sym_LBRACK] = ACTIONS(468),
    [anon_sym_async] = ACTIONS(1429),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(472),
    [sym_number] = ACTIONS(472),
    [sym_identifier] = ACTIONS(1431),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(1433),
    [anon_sym_set] = ACTIONS(1433),
  },
  [274] = {
    [sym_parenthesized_expression] = STATE(704),
    [sym__expression] = STATE(704),
    [sym_yield_expression] = STATE(704),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(704),
    [sym_jsx_fragment] = STATE(704),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(704),
    [sym_anonymous_class] = STATE(704),
    [sym_class] = STATE(704),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(704),
    [sym_generator_function] = STATE(704),
    [sym_call_expression] = STATE(704),
    [sym_new_expression] = STATE(704),
    [sym_await_expression] = STATE(704),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(704),
    [sym_augmented_assignment_expression] = STATE(704),
    [sym_spread_element] = STATE(705),
    [sym_ternary_expression] = STATE(704),
    [sym_binary_expression] = STATE(704),
    [sym_unary_expression] = STATE(704),
    [sym_update_expression] = STATE(704),
    [sym_template_string] = STATE(704),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(706),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(1435),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1437),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1439),
    [sym_number] = ACTIONS(1437),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1439),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1439),
    [sym_false] = ACTIONS(1439),
    [sym_null] = ACTIONS(1439),
    [sym_undefined] = ACTIONS(1439),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [275] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [anon_sym_STAR] = ACTIONS(1441),
    [anon_sym_COMMA] = ACTIONS(1443),
    [anon_sym_RBRACE] = ACTIONS(1443),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [276] = {
    [sym__initializer] = STATE(708),
    [anon_sym_COMMA] = ACTIONS(1443),
    [anon_sym_RBRACE] = ACTIONS(1443),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [277] = {
    [sym__identifier_reference] = STATE(710),
    [sym_decorator_member_expression] = STATE(711),
    [sym_decorator_call_expression] = STATE(712),
    [anon_sym_async] = ACTIONS(1447),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1449),
    [anon_sym_get] = ACTIONS(1447),
    [anon_sym_set] = ACTIONS(1447),
  },
  [278] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [anon_sym_COMMA] = ACTIONS(1443),
    [anon_sym_RBRACE] = ACTIONS(1443),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [279] = {
    [sym__initializer] = STATE(713),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [280] = {
    [anon_sym_COMMA] = ACTIONS(1451),
    [anon_sym_RBRACE] = ACTIONS(1451),
    [sym_comment] = ACTIONS(71),
  },
  [281] = {
    [sym__initializer] = STATE(714),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [282] = {
    [sym_decorator] = STATE(716),
    [sym_method_definition] = STATE(715),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(716),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [283] = {
    [sym_export_statement] = STATE(722),
    [sym__declaration] = STATE(722),
    [sym_import_statement] = STATE(722),
    [sym_expression_statement] = STATE(722),
    [sym_variable_declaration] = STATE(722),
    [sym_lexical_declaration] = STATE(722),
    [sym_statement_block] = STATE(722),
    [sym_if_statement] = STATE(722),
    [sym_switch_statement] = STATE(722),
    [sym_for_statement] = STATE(722),
    [sym_for_in_statement] = STATE(722),
    [sym_for_of_statement] = STATE(722),
    [sym_while_statement] = STATE(722),
    [sym_do_statement] = STATE(722),
    [sym_try_statement] = STATE(722),
    [sym_with_statement] = STATE(722),
    [sym_break_statement] = STATE(722),
    [sym_continue_statement] = STATE(722),
    [sym_debugger_statement] = STATE(722),
    [sym_return_statement] = STATE(722),
    [sym_throw_statement] = STATE(722),
    [sym_empty_statement] = STATE(722),
    [sym_labeled_statement] = STATE(722),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [284] = {
    [sym_expression_statement] = STATE(726),
    [sym_variable_declaration] = STATE(726),
    [sym_lexical_declaration] = STATE(726),
    [sym_empty_statement] = STATE(726),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(727),
    [sym_array] = STATE(728),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(1463),
    [anon_sym_let] = ACTIONS(1465),
    [anon_sym_const] = ACTIONS(1465),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1467),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [285] = {
    [sym_export_statement] = STATE(471),
    [sym__declaration] = STATE(471),
    [sym_import_statement] = STATE(471),
    [sym_expression_statement] = STATE(471),
    [sym_variable_declaration] = STATE(471),
    [sym_lexical_declaration] = STATE(471),
    [sym_statement_block] = STATE(471),
    [sym_if_statement] = STATE(471),
    [sym_switch_statement] = STATE(471),
    [sym_for_statement] = STATE(471),
    [sym_for_in_statement] = STATE(471),
    [sym_for_of_statement] = STATE(471),
    [sym_while_statement] = STATE(471),
    [sym_do_statement] = STATE(471),
    [sym_try_statement] = STATE(471),
    [sym_with_statement] = STATE(471),
    [sym_break_statement] = STATE(471),
    [sym_continue_statement] = STATE(471),
    [sym_debugger_statement] = STATE(471),
    [sym_return_statement] = STATE(471),
    [sym_throw_statement] = STATE(471),
    [sym_empty_statement] = STATE(471),
    [sym_labeled_statement] = STATE(471),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [286] = {
    [sym_export_statement] = STATE(497),
    [sym__declaration] = STATE(497),
    [sym_import_statement] = STATE(497),
    [sym_expression_statement] = STATE(497),
    [sym_variable_declaration] = STATE(497),
    [sym_lexical_declaration] = STATE(497),
    [sym_statement_block] = STATE(497),
    [sym_if_statement] = STATE(497),
    [sym_switch_statement] = STATE(497),
    [sym_for_statement] = STATE(497),
    [sym_for_in_statement] = STATE(497),
    [sym_for_of_statement] = STATE(497),
    [sym_while_statement] = STATE(497),
    [sym_do_statement] = STATE(497),
    [sym_try_statement] = STATE(497),
    [sym_with_statement] = STATE(497),
    [sym_break_statement] = STATE(497),
    [sym_continue_statement] = STATE(497),
    [sym_debugger_statement] = STATE(497),
    [sym_return_statement] = STATE(497),
    [sym_throw_statement] = STATE(497),
    [sym_empty_statement] = STATE(497),
    [sym_labeled_statement] = STATE(497),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [287] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(525),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1469),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [288] = {
    [sym__property_name] = STATE(730),
    [sym_computed_property_name] = STATE(730),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1471),
    [sym_number] = ACTIONS(1471),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [289] = {
    [sym__automatic_semicolon] = ACTIONS(1473),
    [anon_sym_STAR] = ACTIONS(1473),
    [anon_sym_COMMA] = ACTIONS(1473),
    [anon_sym_RBRACE] = ACTIONS(1473),
    [anon_sym_LPAREN] = ACTIONS(1473),
    [anon_sym_RPAREN] = ACTIONS(1473),
    [anon_sym_in] = ACTIONS(1475),
    [anon_sym_of] = ACTIONS(1473),
    [anon_sym_SEMI] = ACTIONS(1473),
    [anon_sym_LBRACK] = ACTIONS(1473),
    [anon_sym_LT] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1475),
    [anon_sym_SLASH] = ACTIONS(1475),
    [anon_sym_DOT] = ACTIONS(1473),
    [anon_sym_EQ] = ACTIONS(1475),
    [anon_sym_QMARK] = ACTIONS(1473),
    [anon_sym_AMP_AMP] = ACTIONS(1473),
    [anon_sym_PIPE_PIPE] = ACTIONS(1473),
    [anon_sym_GT_GT] = ACTIONS(1475),
    [anon_sym_GT_GT_GT] = ACTIONS(1473),
    [anon_sym_LT_LT] = ACTIONS(1473),
    [anon_sym_AMP] = ACTIONS(1475),
    [anon_sym_CARET] = ACTIONS(1473),
    [anon_sym_PIPE] = ACTIONS(1475),
    [anon_sym_PLUS] = ACTIONS(1475),
    [anon_sym_DASH] = ACTIONS(1475),
    [anon_sym_PERCENT] = ACTIONS(1473),
    [anon_sym_LT_EQ] = ACTIONS(1473),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1473),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1473),
    [anon_sym_GT_EQ] = ACTIONS(1473),
    [anon_sym_instanceof] = ACTIONS(1473),
    [anon_sym_PLUS_PLUS] = ACTIONS(1473),
    [anon_sym_DASH_DASH] = ACTIONS(1473),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1473),
  },
  [290] = {
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_EQ_GT] = ACTIONS(1038),
    [sym_comment] = ACTIONS(71),
  },
  [291] = {
    [sym__property_name] = STATE(730),
    [sym_computed_property_name] = STATE(730),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1471),
    [sym_number] = ACTIONS(1471),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [292] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1477),
    [sym_comment] = ACTIONS(71),
  },
  [293] = {
    [sym_parenthesized_expression] = STATE(732),
    [sym__expression] = STATE(732),
    [sym_yield_expression] = STATE(732),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(732),
    [sym_jsx_fragment] = STATE(732),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(732),
    [sym_anonymous_class] = STATE(732),
    [sym_class] = STATE(732),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(732),
    [sym_generator_function] = STATE(732),
    [sym_call_expression] = STATE(732),
    [sym_new_expression] = STATE(732),
    [sym_await_expression] = STATE(732),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(732),
    [sym_augmented_assignment_expression] = STATE(732),
    [sym_ternary_expression] = STATE(732),
    [sym_binary_expression] = STATE(732),
    [sym_unary_expression] = STATE(732),
    [sym_update_expression] = STATE(732),
    [sym_template_string] = STATE(732),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_RBRACE] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(524),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1479),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1481),
    [sym_number] = ACTIONS(1479),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1481),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1481),
    [sym_false] = ACTIONS(1481),
    [sym_null] = ACTIONS(1481),
    [sym_undefined] = ACTIONS(1481),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [294] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(735),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(1483),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(1485),
    [anon_sym_PLUS_EQ] = ACTIONS(1487),
    [anon_sym_DASH_EQ] = ACTIONS(1487),
    [anon_sym_STAR_EQ] = ACTIONS(1487),
    [anon_sym_SLASH_EQ] = ACTIONS(1487),
    [anon_sym_PERCENT_EQ] = ACTIONS(1487),
    [anon_sym_CARET_EQ] = ACTIONS(1487),
    [anon_sym_AMP_EQ] = ACTIONS(1487),
    [anon_sym_PIPE_EQ] = ACTIONS(1487),
    [anon_sym_GT_GT_EQ] = ACTIONS(1487),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1487),
    [anon_sym_LT_LT_EQ] = ACTIONS(1487),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(1489),
  },
  [295] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(736),
    [sym__expression] = STATE(736),
    [sym_yield_expression] = STATE(736),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(736),
    [sym_jsx_fragment] = STATE(736),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(736),
    [sym_anonymous_class] = STATE(736),
    [sym_class] = STATE(736),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(736),
    [sym_generator_function] = STATE(736),
    [sym_call_expression] = STATE(736),
    [sym_new_expression] = STATE(736),
    [sym_await_expression] = STATE(736),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(736),
    [sym_augmented_assignment_expression] = STATE(736),
    [sym_ternary_expression] = STATE(736),
    [sym_binary_expression] = STATE(736),
    [sym_unary_expression] = STATE(736),
    [sym_update_expression] = STATE(736),
    [sym_template_string] = STATE(736),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1491),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1493),
    [sym_number] = ACTIONS(1491),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1493),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1493),
    [sym_false] = ACTIONS(1493),
    [sym_null] = ACTIONS(1493),
    [sym_undefined] = ACTIONS(1493),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [296] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [297] = {
    [sym_parenthesized_expression] = STATE(737),
    [sym__expression] = STATE(737),
    [sym_yield_expression] = STATE(737),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(737),
    [sym_jsx_fragment] = STATE(737),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(737),
    [sym_anonymous_class] = STATE(737),
    [sym_class] = STATE(737),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(737),
    [sym_generator_function] = STATE(737),
    [sym_call_expression] = STATE(737),
    [sym_new_expression] = STATE(737),
    [sym_await_expression] = STATE(737),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(737),
    [sym_augmented_assignment_expression] = STATE(737),
    [sym_ternary_expression] = STATE(737),
    [sym_binary_expression] = STATE(737),
    [sym_unary_expression] = STATE(737),
    [sym_update_expression] = STATE(737),
    [sym_template_string] = STATE(737),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1495),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1497),
    [sym_number] = ACTIONS(1495),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1497),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1497),
    [sym_false] = ACTIONS(1497),
    [sym_null] = ACTIONS(1497),
    [sym_undefined] = ACTIONS(1497),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [298] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [299] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [300] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(1483),
    [anon_sym_EQ_GT] = ACTIONS(1485),
    [anon_sym_PLUS_EQ] = ACTIONS(1501),
    [anon_sym_DASH_EQ] = ACTIONS(1501),
    [anon_sym_STAR_EQ] = ACTIONS(1501),
    [anon_sym_SLASH_EQ] = ACTIONS(1501),
    [anon_sym_PERCENT_EQ] = ACTIONS(1501),
    [anon_sym_CARET_EQ] = ACTIONS(1501),
    [anon_sym_AMP_EQ] = ACTIONS(1501),
    [anon_sym_PIPE_EQ] = ACTIONS(1501),
    [anon_sym_GT_GT_EQ] = ACTIONS(1501),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1501),
    [anon_sym_LT_LT_EQ] = ACTIONS(1501),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [301] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(1483),
    [anon_sym_PLUS_EQ] = ACTIONS(1487),
    [anon_sym_DASH_EQ] = ACTIONS(1487),
    [anon_sym_STAR_EQ] = ACTIONS(1487),
    [anon_sym_SLASH_EQ] = ACTIONS(1487),
    [anon_sym_PERCENT_EQ] = ACTIONS(1487),
    [anon_sym_CARET_EQ] = ACTIONS(1487),
    [anon_sym_AMP_EQ] = ACTIONS(1487),
    [anon_sym_PIPE_EQ] = ACTIONS(1487),
    [anon_sym_GT_GT_EQ] = ACTIONS(1487),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1487),
    [anon_sym_LT_LT_EQ] = ACTIONS(1487),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [302] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(1505),
    [anon_sym_RBRACE] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [303] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(1483),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [304] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(1483),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [305] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(1483),
    [anon_sym_PLUS_EQ] = ACTIONS(1501),
    [anon_sym_DASH_EQ] = ACTIONS(1501),
    [anon_sym_STAR_EQ] = ACTIONS(1501),
    [anon_sym_SLASH_EQ] = ACTIONS(1501),
    [anon_sym_PERCENT_EQ] = ACTIONS(1501),
    [anon_sym_CARET_EQ] = ACTIONS(1501),
    [anon_sym_AMP_EQ] = ACTIONS(1501),
    [anon_sym_PIPE_EQ] = ACTIONS(1501),
    [anon_sym_GT_GT_EQ] = ACTIONS(1501),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(1501),
    [anon_sym_LT_LT_EQ] = ACTIONS(1501),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [306] = {
    [anon_sym_EQ_GT] = ACTIONS(1485),
    [sym_comment] = ACTIONS(71),
  },
  [307] = {
    [sym_export_statement] = STATE(598),
    [sym__declaration] = STATE(598),
    [sym_import_statement] = STATE(598),
    [sym_expression_statement] = STATE(598),
    [sym_variable_declaration] = STATE(598),
    [sym_lexical_declaration] = STATE(598),
    [sym_statement_block] = STATE(598),
    [sym_if_statement] = STATE(598),
    [sym_switch_statement] = STATE(598),
    [sym_for_statement] = STATE(598),
    [sym_for_in_statement] = STATE(598),
    [sym_for_of_statement] = STATE(598),
    [sym_while_statement] = STATE(598),
    [sym_do_statement] = STATE(598),
    [sym_try_statement] = STATE(598),
    [sym_with_statement] = STATE(598),
    [sym_break_statement] = STATE(598),
    [sym_continue_statement] = STATE(598),
    [sym_debugger_statement] = STATE(598),
    [sym_return_statement] = STATE(598),
    [sym_throw_statement] = STATE(598),
    [sym_empty_statement] = STATE(598),
    [sym_labeled_statement] = STATE(598),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [308] = {
    [sym_parenthesized_expression] = STATE(754),
    [sym__expression] = STATE(754),
    [sym_yield_expression] = STATE(754),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(754),
    [sym_jsx_fragment] = STATE(754),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(754),
    [sym_anonymous_class] = STATE(754),
    [sym_class] = STATE(754),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(754),
    [sym_generator_function] = STATE(754),
    [sym_call_expression] = STATE(754),
    [sym_new_expression] = STATE(754),
    [sym_await_expression] = STATE(754),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(754),
    [sym_augmented_assignment_expression] = STATE(754),
    [sym_ternary_expression] = STATE(754),
    [sym_binary_expression] = STATE(754),
    [sym_unary_expression] = STATE(754),
    [sym_update_expression] = STATE(754),
    [sym_template_string] = STATE(754),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1539),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1541),
    [sym_number] = ACTIONS(1539),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(1541),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1541),
    [sym_false] = ACTIONS(1541),
    [sym_null] = ACTIONS(1541),
    [sym_undefined] = ACTIONS(1541),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [309] = {
    [sym__automatic_semicolon] = ACTIONS(1088),
    [anon_sym_STAR] = ACTIONS(1088),
    [anon_sym_COMMA] = ACTIONS(1547),
    [anon_sym_RBRACE] = ACTIONS(1550),
    [anon_sym_LPAREN] = ACTIONS(1088),
    [anon_sym_in] = ACTIONS(1090),
    [anon_sym_SEMI] = ACTIONS(1088),
    [anon_sym_LBRACK] = ACTIONS(1088),
    [anon_sym_LT] = ACTIONS(1090),
    [anon_sym_GT] = ACTIONS(1090),
    [anon_sym_SLASH] = ACTIONS(1090),
    [anon_sym_DOT] = ACTIONS(1088),
    [anon_sym_QMARK] = ACTIONS(1088),
    [anon_sym_AMP_AMP] = ACTIONS(1088),
    [anon_sym_PIPE_PIPE] = ACTIONS(1088),
    [anon_sym_GT_GT] = ACTIONS(1090),
    [anon_sym_GT_GT_GT] = ACTIONS(1088),
    [anon_sym_LT_LT] = ACTIONS(1088),
    [anon_sym_AMP] = ACTIONS(1090),
    [anon_sym_CARET] = ACTIONS(1088),
    [anon_sym_PIPE] = ACTIONS(1090),
    [anon_sym_PLUS] = ACTIONS(1090),
    [anon_sym_DASH] = ACTIONS(1090),
    [anon_sym_PERCENT] = ACTIONS(1088),
    [anon_sym_LT_EQ] = ACTIONS(1088),
    [anon_sym_EQ_EQ] = ACTIONS(1090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1088),
    [anon_sym_BANG_EQ] = ACTIONS(1090),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1088),
    [anon_sym_GT_EQ] = ACTIONS(1088),
    [anon_sym_instanceof] = ACTIONS(1088),
    [anon_sym_PLUS_PLUS] = ACTIONS(1088),
    [anon_sym_DASH_DASH] = ACTIONS(1088),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1088),
  },
  [310] = {
    [anon_sym_export] = ACTIONS(1552),
    [anon_sym_STAR] = ACTIONS(1110),
    [anon_sym_LPAREN] = ACTIONS(1110),
    [anon_sym_LBRACK] = ACTIONS(1110),
    [anon_sym_DOT] = ACTIONS(1552),
    [anon_sym_class] = ACTIONS(1552),
    [anon_sym_async] = ACTIONS(1552),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1110),
    [sym_number] = ACTIONS(1110),
    [sym_identifier] = ACTIONS(1552),
    [anon_sym_AT] = ACTIONS(1110),
    [anon_sym_get] = ACTIONS(1552),
    [anon_sym_set] = ACTIONS(1552),
  },
  [311] = {
    [sym_arguments] = STATE(761),
    [anon_sym_export] = ACTIONS(1554),
    [anon_sym_STAR] = ACTIONS(1112),
    [anon_sym_LPAREN] = ACTIONS(1556),
    [anon_sym_LBRACK] = ACTIONS(1112),
    [anon_sym_DOT] = ACTIONS(1558),
    [anon_sym_class] = ACTIONS(1554),
    [anon_sym_async] = ACTIONS(1554),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1112),
    [sym_number] = ACTIONS(1112),
    [sym_identifier] = ACTIONS(1554),
    [anon_sym_AT] = ACTIONS(1112),
    [anon_sym_get] = ACTIONS(1554),
    [anon_sym_set] = ACTIONS(1554),
  },
  [312] = {
    [sym_arguments] = STATE(763),
    [anon_sym_export] = ACTIONS(1560),
    [anon_sym_STAR] = ACTIONS(1116),
    [anon_sym_LPAREN] = ACTIONS(1556),
    [anon_sym_LBRACK] = ACTIONS(1116),
    [anon_sym_DOT] = ACTIONS(1562),
    [anon_sym_class] = ACTIONS(1560),
    [anon_sym_async] = ACTIONS(1560),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1116),
    [sym_number] = ACTIONS(1116),
    [sym_identifier] = ACTIONS(1560),
    [anon_sym_AT] = ACTIONS(1116),
    [anon_sym_get] = ACTIONS(1560),
    [anon_sym_set] = ACTIONS(1560),
  },
  [313] = {
    [anon_sym_export] = ACTIONS(1564),
    [anon_sym_STAR] = ACTIONS(1120),
    [anon_sym_LBRACK] = ACTIONS(1120),
    [anon_sym_class] = ACTIONS(1564),
    [anon_sym_async] = ACTIONS(1564),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1120),
    [sym_number] = ACTIONS(1120),
    [sym_identifier] = ACTIONS(1564),
    [anon_sym_AT] = ACTIONS(1120),
    [anon_sym_get] = ACTIONS(1564),
    [anon_sym_set] = ACTIONS(1564),
  },
  [314] = {
    [sym__automatic_semicolon] = ACTIONS(1170),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_COMMA] = ACTIONS(1566),
    [anon_sym_RBRACE] = ACTIONS(1569),
    [anon_sym_LPAREN] = ACTIONS(1170),
    [anon_sym_in] = ACTIONS(1172),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym_LBRACK] = ACTIONS(1170),
    [anon_sym_LT] = ACTIONS(1172),
    [anon_sym_GT] = ACTIONS(1172),
    [anon_sym_SLASH] = ACTIONS(1172),
    [anon_sym_DOT] = ACTIONS(1170),
    [anon_sym_QMARK] = ACTIONS(1170),
    [anon_sym_AMP_AMP] = ACTIONS(1170),
    [anon_sym_PIPE_PIPE] = ACTIONS(1170),
    [anon_sym_GT_GT] = ACTIONS(1172),
    [anon_sym_GT_GT_GT] = ACTIONS(1170),
    [anon_sym_LT_LT] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1172),
    [anon_sym_CARET] = ACTIONS(1170),
    [anon_sym_PIPE] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PERCENT] = ACTIONS(1170),
    [anon_sym_LT_EQ] = ACTIONS(1170),
    [anon_sym_EQ_EQ] = ACTIONS(1172),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1170),
    [anon_sym_BANG_EQ] = ACTIONS(1172),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1170),
    [anon_sym_GT_EQ] = ACTIONS(1170),
    [anon_sym_instanceof] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1170),
  },
  [315] = {
    [sym__automatic_semicolon] = ACTIONS(1571),
    [anon_sym_STAR] = ACTIONS(1571),
    [anon_sym_COMMA] = ACTIONS(1571),
    [anon_sym_RBRACE] = ACTIONS(1571),
    [anon_sym_LPAREN] = ACTIONS(1571),
    [anon_sym_RPAREN] = ACTIONS(1571),
    [anon_sym_in] = ACTIONS(1573),
    [anon_sym_of] = ACTIONS(1571),
    [anon_sym_SEMI] = ACTIONS(1571),
    [anon_sym_LBRACK] = ACTIONS(1571),
    [anon_sym_LT] = ACTIONS(1573),
    [anon_sym_GT] = ACTIONS(1573),
    [anon_sym_SLASH] = ACTIONS(1573),
    [anon_sym_DOT] = ACTIONS(1571),
    [anon_sym_EQ] = ACTIONS(1573),
    [anon_sym_QMARK] = ACTIONS(1571),
    [anon_sym_AMP_AMP] = ACTIONS(1571),
    [anon_sym_PIPE_PIPE] = ACTIONS(1571),
    [anon_sym_GT_GT] = ACTIONS(1573),
    [anon_sym_GT_GT_GT] = ACTIONS(1571),
    [anon_sym_LT_LT] = ACTIONS(1571),
    [anon_sym_AMP] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(1571),
    [anon_sym_PIPE] = ACTIONS(1573),
    [anon_sym_PLUS] = ACTIONS(1573),
    [anon_sym_DASH] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1571),
    [anon_sym_LT_EQ] = ACTIONS(1571),
    [anon_sym_EQ_EQ] = ACTIONS(1573),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1571),
    [anon_sym_BANG_EQ] = ACTIONS(1573),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1571),
    [anon_sym_GT_EQ] = ACTIONS(1571),
    [anon_sym_instanceof] = ACTIONS(1571),
    [anon_sym_PLUS_PLUS] = ACTIONS(1571),
    [anon_sym_DASH_DASH] = ACTIONS(1571),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1571),
  },
  [316] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1575),
    [sym_comment] = ACTIONS(71),
  },
  [317] = {
    [sym__automatic_semicolon] = ACTIONS(1174),
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_COMMA] = ACTIONS(1577),
    [anon_sym_RBRACE] = ACTIONS(1580),
    [anon_sym_LPAREN] = ACTIONS(1174),
    [anon_sym_in] = ACTIONS(1176),
    [anon_sym_SEMI] = ACTIONS(1174),
    [anon_sym_LBRACK] = ACTIONS(1174),
    [anon_sym_LT] = ACTIONS(1176),
    [anon_sym_GT] = ACTIONS(1176),
    [anon_sym_SLASH] = ACTIONS(1176),
    [anon_sym_DOT] = ACTIONS(1174),
    [anon_sym_QMARK] = ACTIONS(1174),
    [anon_sym_AMP_AMP] = ACTIONS(1174),
    [anon_sym_PIPE_PIPE] = ACTIONS(1174),
    [anon_sym_GT_GT] = ACTIONS(1176),
    [anon_sym_GT_GT_GT] = ACTIONS(1174),
    [anon_sym_LT_LT] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1176),
    [anon_sym_CARET] = ACTIONS(1174),
    [anon_sym_PIPE] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PERCENT] = ACTIONS(1174),
    [anon_sym_LT_EQ] = ACTIONS(1174),
    [anon_sym_EQ_EQ] = ACTIONS(1176),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1174),
    [anon_sym_BANG_EQ] = ACTIONS(1176),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1174),
    [anon_sym_GT_EQ] = ACTIONS(1174),
    [anon_sym_instanceof] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1174),
  },
  [318] = {
    [sym_parenthesized_expression] = STATE(765),
    [sym__expression] = STATE(765),
    [sym_yield_expression] = STATE(765),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(765),
    [sym_jsx_fragment] = STATE(765),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(765),
    [sym_anonymous_class] = STATE(765),
    [sym_class] = STATE(765),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(765),
    [sym_generator_function] = STATE(765),
    [sym_call_expression] = STATE(765),
    [sym_new_expression] = STATE(765),
    [sym_await_expression] = STATE(765),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(765),
    [sym_augmented_assignment_expression] = STATE(765),
    [sym_ternary_expression] = STATE(765),
    [sym_binary_expression] = STATE(765),
    [sym_unary_expression] = STATE(765),
    [sym_update_expression] = STATE(765),
    [sym_template_string] = STATE(765),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1582),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1584),
    [sym_number] = ACTIONS(1582),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1584),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1584),
    [sym_false] = ACTIONS(1584),
    [sym_null] = ACTIONS(1584),
    [sym_undefined] = ACTIONS(1584),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [319] = {
    [sym_statement_block] = STATE(767),
    [anon_sym_LBRACE] = ACTIONS(1586),
    [sym_comment] = ACTIONS(71),
  },
  [320] = {
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [321] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(1592),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [322] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1211),
    [anon_sym_RBRACE] = ACTIONS(1206),
    [anon_sym_import] = ACTIONS(1214),
    [anon_sym_var] = ACTIONS(1217),
    [anon_sym_let] = ACTIONS(1220),
    [anon_sym_const] = ACTIONS(1220),
    [anon_sym_if] = ACTIONS(1594),
    [anon_sym_switch] = ACTIONS(1226),
    [anon_sym_for] = ACTIONS(1597),
    [anon_sym_LPAREN] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(1600),
    [anon_sym_do] = ACTIONS(1238),
    [anon_sym_try] = ACTIONS(1241),
    [anon_sym_with] = ACTIONS(1603),
    [anon_sym_break] = ACTIONS(1247),
    [anon_sym_continue] = ACTIONS(1250),
    [anon_sym_debugger] = ACTIONS(1253),
    [anon_sym_return] = ACTIONS(1256),
    [anon_sym_throw] = ACTIONS(1259),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym_yield] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1271),
    [anon_sym_class] = ACTIONS(1274),
    [anon_sym_async] = ACTIONS(1277),
    [anon_sym_function] = ACTIONS(1280),
    [anon_sym_EQ_GT] = ACTIONS(1283),
    [anon_sym_new] = ACTIONS(1286),
    [anon_sym_await] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_BANG] = ACTIONS(1295),
    [anon_sym_TILDE] = ACTIONS(1295),
    [anon_sym_typeof] = ACTIONS(1292),
    [anon_sym_void] = ACTIONS(1292),
    [anon_sym_delete] = ACTIONS(1292),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1301),
    [anon_sym_BQUOTE] = ACTIONS(1304),
    [sym_regex] = ACTIONS(1307),
    [sym_number] = ACTIONS(1301),
    [sym_identifier] = ACTIONS(1606),
    [sym_this] = ACTIONS(1307),
    [sym_super] = ACTIONS(1313),
    [sym_true] = ACTIONS(1307),
    [sym_false] = ACTIONS(1307),
    [sym_null] = ACTIONS(1307),
    [sym_undefined] = ACTIONS(1307),
    [anon_sym_AT] = ACTIONS(1316),
    [anon_sym_get] = ACTIONS(1319),
    [anon_sym_set] = ACTIONS(1319),
  },
  [323] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [anon_sym_STAR] = ACTIONS(1441),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [324] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [325] = {
    [aux_sym_object_repeat1] = STATE(768),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1575),
    [sym_comment] = ACTIONS(71),
  },
  [326] = {
    [sym_formal_parameters] = STATE(319),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [327] = {
    [sym_decorator] = STATE(327),
    [aux_sym_export_statement_repeat1] = STATE(327),
    [anon_sym_export] = ACTIONS(1609),
    [anon_sym_STAR] = ACTIONS(1326),
    [anon_sym_LBRACK] = ACTIONS(1326),
    [anon_sym_class] = ACTIONS(1609),
    [anon_sym_async] = ACTIONS(1609),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1326),
    [sym_number] = ACTIONS(1326),
    [sym_identifier] = ACTIONS(1609),
    [anon_sym_AT] = ACTIONS(1611),
    [anon_sym_get] = ACTIONS(1609),
    [anon_sym_set] = ACTIONS(1609),
  },
  [328] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(1614),
    [anon_sym_RBRACE] = ACTIONS(1451),
    [sym_comment] = ACTIONS(71),
  },
  [329] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1617),
  },
  [330] = {
    [anon_sym_RBRACE] = ACTIONS(1619),
    [sym_comment] = ACTIONS(71),
  },
  [331] = {
    [anon_sym_from] = ACTIONS(1621),
    [sym_comment] = ACTIONS(71),
  },
  [332] = {
    [aux_sym_named_imports_repeat1] = STATE(773),
    [anon_sym_COMMA] = ACTIONS(1623),
    [anon_sym_RBRACE] = ACTIONS(1625),
    [sym_comment] = ACTIONS(71),
  },
  [333] = {
    [ts_builtin_sym_end] = ACTIONS(1627),
    [anon_sym_export] = ACTIONS(1629),
    [anon_sym_LBRACE] = ACTIONS(1627),
    [anon_sym_RBRACE] = ACTIONS(1627),
    [anon_sym_import] = ACTIONS(1629),
    [anon_sym_var] = ACTIONS(1629),
    [anon_sym_let] = ACTIONS(1629),
    [anon_sym_const] = ACTIONS(1629),
    [anon_sym_if] = ACTIONS(1629),
    [anon_sym_switch] = ACTIONS(1629),
    [anon_sym_for] = ACTIONS(1629),
    [anon_sym_LPAREN] = ACTIONS(1627),
    [anon_sym_while] = ACTIONS(1629),
    [anon_sym_do] = ACTIONS(1629),
    [anon_sym_try] = ACTIONS(1629),
    [anon_sym_with] = ACTIONS(1629),
    [anon_sym_break] = ACTIONS(1629),
    [anon_sym_continue] = ACTIONS(1629),
    [anon_sym_debugger] = ACTIONS(1629),
    [anon_sym_return] = ACTIONS(1629),
    [anon_sym_throw] = ACTIONS(1629),
    [anon_sym_SEMI] = ACTIONS(1627),
    [anon_sym_yield] = ACTIONS(1629),
    [anon_sym_LBRACK] = ACTIONS(1627),
    [anon_sym_LT] = ACTIONS(1627),
    [anon_sym_class] = ACTIONS(1629),
    [anon_sym_async] = ACTIONS(1629),
    [anon_sym_function] = ACTIONS(1629),
    [anon_sym_EQ_GT] = ACTIONS(1627),
    [anon_sym_new] = ACTIONS(1629),
    [anon_sym_await] = ACTIONS(1629),
    [anon_sym_PLUS] = ACTIONS(1629),
    [anon_sym_DASH] = ACTIONS(1629),
    [anon_sym_BANG] = ACTIONS(1627),
    [anon_sym_TILDE] = ACTIONS(1627),
    [anon_sym_typeof] = ACTIONS(1629),
    [anon_sym_void] = ACTIONS(1629),
    [anon_sym_delete] = ACTIONS(1629),
    [anon_sym_PLUS_PLUS] = ACTIONS(1627),
    [anon_sym_DASH_DASH] = ACTIONS(1627),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1627),
    [anon_sym_BQUOTE] = ACTIONS(1627),
    [sym_regex] = ACTIONS(1629),
    [sym_number] = ACTIONS(1627),
    [sym_identifier] = ACTIONS(1629),
    [sym_this] = ACTIONS(1629),
    [sym_super] = ACTIONS(1629),
    [sym_true] = ACTIONS(1629),
    [sym_false] = ACTIONS(1629),
    [sym_null] = ACTIONS(1629),
    [sym_undefined] = ACTIONS(1629),
    [anon_sym_AT] = ACTIONS(1627),
    [anon_sym_get] = ACTIONS(1629),
    [anon_sym_set] = ACTIONS(1629),
  },
  [334] = {
    [sym_namespace_import] = STATE(774),
    [sym_named_imports] = STATE(774),
    [anon_sym_STAR] = ACTIONS(133),
    [anon_sym_LBRACE] = ACTIONS(135),
    [sym_comment] = ACTIONS(71),
  },
  [335] = {
    [sym__automatic_semicolon] = ACTIONS(1631),
    [anon_sym_SEMI] = ACTIONS(1631),
    [sym_comment] = ACTIONS(71),
  },
  [336] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(480),
    [anon_sym_of] = ACTIONS(480),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_EQ] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
  },
  [337] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(777),
    [anon_sym_STAR] = ACTIONS(1441),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1633),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [338] = {
    [sym__initializer] = STATE(708),
    [aux_sym_object_repeat1] = STATE(777),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1633),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [339] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(777),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1633),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [340] = {
    [aux_sym_object_repeat1] = STATE(779),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1635),
    [sym_comment] = ACTIONS(71),
  },
  [341] = {
    [sym_decorator] = STATE(716),
    [sym_method_definition] = STATE(780),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(716),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [342] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1635),
    [sym_comment] = ACTIONS(71),
  },
  [343] = {
    [sym__automatic_semicolon] = ACTIONS(894),
    [anon_sym_COMMA] = ACTIONS(894),
    [anon_sym_in] = ACTIONS(894),
    [anon_sym_of] = ACTIONS(894),
    [anon_sym_SEMI] = ACTIONS(894),
    [anon_sym_EQ] = ACTIONS(894),
    [sym_comment] = ACTIONS(71),
  },
  [344] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(782),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1637),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [345] = {
    [aux_sym_array_repeat1] = STATE(782),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(1637),
    [sym_comment] = ACTIONS(71),
  },
  [346] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(1637),
    [sym_comment] = ACTIONS(71),
  },
  [347] = {
    [sym__automatic_semicolon] = ACTIONS(1639),
    [anon_sym_COMMA] = ACTIONS(1639),
    [anon_sym_SEMI] = ACTIONS(1639),
    [sym_comment] = ACTIONS(71),
  },
  [348] = {
    [ts_builtin_sym_end] = ACTIONS(1641),
    [anon_sym_export] = ACTIONS(1643),
    [anon_sym_LBRACE] = ACTIONS(1641),
    [anon_sym_RBRACE] = ACTIONS(1641),
    [anon_sym_import] = ACTIONS(1643),
    [anon_sym_var] = ACTIONS(1643),
    [anon_sym_let] = ACTIONS(1643),
    [anon_sym_const] = ACTIONS(1643),
    [anon_sym_if] = ACTIONS(1643),
    [anon_sym_switch] = ACTIONS(1643),
    [anon_sym_for] = ACTIONS(1643),
    [anon_sym_LPAREN] = ACTIONS(1641),
    [anon_sym_while] = ACTIONS(1643),
    [anon_sym_do] = ACTIONS(1643),
    [anon_sym_try] = ACTIONS(1643),
    [anon_sym_with] = ACTIONS(1643),
    [anon_sym_break] = ACTIONS(1643),
    [anon_sym_continue] = ACTIONS(1643),
    [anon_sym_debugger] = ACTIONS(1643),
    [anon_sym_return] = ACTIONS(1643),
    [anon_sym_throw] = ACTIONS(1643),
    [anon_sym_SEMI] = ACTIONS(1641),
    [anon_sym_yield] = ACTIONS(1643),
    [anon_sym_LBRACK] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1641),
    [anon_sym_class] = ACTIONS(1643),
    [anon_sym_async] = ACTIONS(1643),
    [anon_sym_function] = ACTIONS(1643),
    [anon_sym_EQ_GT] = ACTIONS(1641),
    [anon_sym_new] = ACTIONS(1643),
    [anon_sym_await] = ACTIONS(1643),
    [anon_sym_PLUS] = ACTIONS(1643),
    [anon_sym_DASH] = ACTIONS(1643),
    [anon_sym_BANG] = ACTIONS(1641),
    [anon_sym_TILDE] = ACTIONS(1641),
    [anon_sym_typeof] = ACTIONS(1643),
    [anon_sym_void] = ACTIONS(1643),
    [anon_sym_delete] = ACTIONS(1643),
    [anon_sym_PLUS_PLUS] = ACTIONS(1641),
    [anon_sym_DASH_DASH] = ACTIONS(1641),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1641),
    [anon_sym_BQUOTE] = ACTIONS(1641),
    [sym_regex] = ACTIONS(1643),
    [sym_number] = ACTIONS(1641),
    [sym_identifier] = ACTIONS(1643),
    [sym_this] = ACTIONS(1643),
    [sym_super] = ACTIONS(1643),
    [sym_true] = ACTIONS(1643),
    [sym_false] = ACTIONS(1643),
    [sym_null] = ACTIONS(1643),
    [sym_undefined] = ACTIONS(1643),
    [anon_sym_AT] = ACTIONS(1641),
    [anon_sym_get] = ACTIONS(1643),
    [anon_sym_set] = ACTIONS(1643),
  },
  [349] = {
    [sym_variable_declarator] = STATE(783),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [350] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(1645),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(1645),
    [sym_comment] = ACTIONS(71),
  },
  [351] = {
    [sym__automatic_semicolon] = ACTIONS(1647),
    [anon_sym_COMMA] = ACTIONS(1647),
    [anon_sym_SEMI] = ACTIONS(1647),
    [sym_comment] = ACTIONS(71),
  },
  [352] = {
    [sym__automatic_semicolon] = ACTIONS(1649),
    [anon_sym_COMMA] = ACTIONS(1649),
    [anon_sym_SEMI] = ACTIONS(1649),
    [sym_comment] = ACTIONS(71),
  },
  [353] = {
    [ts_builtin_sym_end] = ACTIONS(1651),
    [anon_sym_export] = ACTIONS(1653),
    [anon_sym_LBRACE] = ACTIONS(1651),
    [anon_sym_RBRACE] = ACTIONS(1651),
    [anon_sym_import] = ACTIONS(1653),
    [anon_sym_var] = ACTIONS(1653),
    [anon_sym_let] = ACTIONS(1653),
    [anon_sym_const] = ACTIONS(1653),
    [anon_sym_if] = ACTIONS(1653),
    [anon_sym_switch] = ACTIONS(1653),
    [anon_sym_for] = ACTIONS(1653),
    [anon_sym_LPAREN] = ACTIONS(1651),
    [anon_sym_while] = ACTIONS(1653),
    [anon_sym_do] = ACTIONS(1653),
    [anon_sym_try] = ACTIONS(1653),
    [anon_sym_with] = ACTIONS(1653),
    [anon_sym_break] = ACTIONS(1653),
    [anon_sym_continue] = ACTIONS(1653),
    [anon_sym_debugger] = ACTIONS(1653),
    [anon_sym_return] = ACTIONS(1653),
    [anon_sym_throw] = ACTIONS(1653),
    [anon_sym_SEMI] = ACTIONS(1651),
    [anon_sym_yield] = ACTIONS(1653),
    [anon_sym_LBRACK] = ACTIONS(1651),
    [anon_sym_LT] = ACTIONS(1651),
    [anon_sym_class] = ACTIONS(1653),
    [anon_sym_async] = ACTIONS(1653),
    [anon_sym_function] = ACTIONS(1653),
    [anon_sym_EQ_GT] = ACTIONS(1651),
    [anon_sym_new] = ACTIONS(1653),
    [anon_sym_await] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1653),
    [anon_sym_BANG] = ACTIONS(1651),
    [anon_sym_TILDE] = ACTIONS(1651),
    [anon_sym_typeof] = ACTIONS(1653),
    [anon_sym_void] = ACTIONS(1653),
    [anon_sym_delete] = ACTIONS(1653),
    [anon_sym_PLUS_PLUS] = ACTIONS(1651),
    [anon_sym_DASH_DASH] = ACTIONS(1651),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1651),
    [anon_sym_BQUOTE] = ACTIONS(1651),
    [sym_regex] = ACTIONS(1653),
    [sym_number] = ACTIONS(1651),
    [sym_identifier] = ACTIONS(1653),
    [sym_this] = ACTIONS(1653),
    [sym_super] = ACTIONS(1653),
    [sym_true] = ACTIONS(1653),
    [sym_false] = ACTIONS(1653),
    [sym_null] = ACTIONS(1653),
    [sym_undefined] = ACTIONS(1653),
    [anon_sym_AT] = ACTIONS(1651),
    [anon_sym_get] = ACTIONS(1653),
    [anon_sym_set] = ACTIONS(1653),
  },
  [354] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(1655),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(1655),
    [sym_comment] = ACTIONS(71),
  },
  [355] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(427),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_RPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(720),
    [anon_sym_PLUS_EQ] = ACTIONS(722),
    [anon_sym_DASH_EQ] = ACTIONS(722),
    [anon_sym_STAR_EQ] = ACTIONS(722),
    [anon_sym_SLASH_EQ] = ACTIONS(722),
    [anon_sym_PERCENT_EQ] = ACTIONS(722),
    [anon_sym_CARET_EQ] = ACTIONS(722),
    [anon_sym_AMP_EQ] = ACTIONS(722),
    [anon_sym_PIPE_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_LT_LT_EQ] = ACTIONS(722),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(724),
  },
  [356] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_EQ_GT] = ACTIONS(720),
    [anon_sym_PLUS_EQ] = ACTIONS(768),
    [anon_sym_DASH_EQ] = ACTIONS(768),
    [anon_sym_STAR_EQ] = ACTIONS(768),
    [anon_sym_SLASH_EQ] = ACTIONS(768),
    [anon_sym_PERCENT_EQ] = ACTIONS(768),
    [anon_sym_CARET_EQ] = ACTIONS(768),
    [anon_sym_AMP_EQ] = ACTIONS(768),
    [anon_sym_PIPE_EQ] = ACTIONS(768),
    [anon_sym_GT_GT_EQ] = ACTIONS(768),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(768),
    [anon_sym_LT_LT_EQ] = ACTIONS(768),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [357] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_RPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_PLUS_EQ] = ACTIONS(722),
    [anon_sym_DASH_EQ] = ACTIONS(722),
    [anon_sym_STAR_EQ] = ACTIONS(722),
    [anon_sym_SLASH_EQ] = ACTIONS(722),
    [anon_sym_PERCENT_EQ] = ACTIONS(722),
    [anon_sym_CARET_EQ] = ACTIONS(722),
    [anon_sym_AMP_EQ] = ACTIONS(722),
    [anon_sym_PIPE_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(722),
    [anon_sym_LT_LT_EQ] = ACTIONS(722),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [358] = {
    [anon_sym_RPAREN] = ACTIONS(1657),
    [sym_comment] = ACTIONS(71),
  },
  [359] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [360] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(716),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [361] = {
    [sym_export_clause] = STATE(793),
    [sym__declaration] = STATE(794),
    [sym_variable_declaration] = STATE(794),
    [sym_lexical_declaration] = STATE(794),
    [sym_class] = STATE(795),
    [sym_function] = STATE(795),
    [sym_generator_function] = STATE(795),
    [sym_decorator] = STATE(796),
    [aux_sym_export_statement_repeat1] = STATE(796),
    [anon_sym_STAR] = ACTIONS(1659),
    [anon_sym_default] = ACTIONS(1661),
    [anon_sym_LBRACE] = ACTIONS(93),
    [anon_sym_var] = ACTIONS(1663),
    [anon_sym_let] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1665),
    [anon_sym_class] = ACTIONS(1667),
    [anon_sym_async] = ACTIONS(1669),
    [anon_sym_function] = ACTIONS(1671),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [362] = {
    [sym_export_statement] = STATE(798),
    [sym__declaration] = STATE(798),
    [sym_import_statement] = STATE(798),
    [sym_expression_statement] = STATE(798),
    [sym_variable_declaration] = STATE(798),
    [sym_lexical_declaration] = STATE(798),
    [sym_statement_block] = STATE(798),
    [sym_if_statement] = STATE(798),
    [sym_switch_statement] = STATE(798),
    [sym_for_statement] = STATE(798),
    [sym_for_in_statement] = STATE(798),
    [sym_for_of_statement] = STATE(798),
    [sym_while_statement] = STATE(798),
    [sym_do_statement] = STATE(798),
    [sym_try_statement] = STATE(798),
    [sym_with_statement] = STATE(798),
    [sym_break_statement] = STATE(798),
    [sym_continue_statement] = STATE(798),
    [sym_debugger_statement] = STATE(798),
    [sym_return_statement] = STATE(798),
    [sym_throw_statement] = STATE(798),
    [sym_empty_statement] = STATE(798),
    [sym_labeled_statement] = STATE(798),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(798),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1673),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [363] = {
    [sym_import_clause] = STATE(800),
    [sym_namespace_import] = STATE(87),
    [sym_named_imports] = STATE(87),
    [anon_sym_STAR] = ACTIONS(133),
    [anon_sym_LBRACE] = ACTIONS(135),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1675),
    [sym_identifier] = ACTIONS(139),
  },
  [364] = {
    [sym_variable_declarator] = STATE(801),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [365] = {
    [sym_variable_declarator] = STATE(802),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [366] = {
    [sym_parenthesized_expression] = STATE(803),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [367] = {
    [sym_parenthesized_expression] = STATE(804),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [368] = {
    [anon_sym_LPAREN] = ACTIONS(1677),
    [sym_comment] = ACTIONS(71),
  },
  [369] = {
    [sym_parenthesized_expression] = STATE(806),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [370] = {
    [sym_export_statement] = STATE(807),
    [sym__declaration] = STATE(807),
    [sym_import_statement] = STATE(807),
    [sym_expression_statement] = STATE(807),
    [sym_variable_declaration] = STATE(807),
    [sym_lexical_declaration] = STATE(807),
    [sym_statement_block] = STATE(807),
    [sym_if_statement] = STATE(807),
    [sym_switch_statement] = STATE(807),
    [sym_for_statement] = STATE(807),
    [sym_for_in_statement] = STATE(807),
    [sym_for_of_statement] = STATE(807),
    [sym_while_statement] = STATE(807),
    [sym_do_statement] = STATE(807),
    [sym_try_statement] = STATE(807),
    [sym_with_statement] = STATE(807),
    [sym_break_statement] = STATE(807),
    [sym_continue_statement] = STATE(807),
    [sym_debugger_statement] = STATE(807),
    [sym_return_statement] = STATE(807),
    [sym_throw_statement] = STATE(807),
    [sym_empty_statement] = STATE(807),
    [sym_labeled_statement] = STATE(807),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [371] = {
    [sym_statement_block] = STATE(809),
    [anon_sym_LBRACE] = ACTIONS(1679),
    [sym_comment] = ACTIONS(71),
  },
  [372] = {
    [sym_parenthesized_expression] = STATE(810),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [373] = {
    [sym__automatic_semicolon] = ACTIONS(1681),
    [anon_sym_SEMI] = ACTIONS(1681),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1683),
  },
  [374] = {
    [sym__automatic_semicolon] = ACTIONS(1685),
    [anon_sym_SEMI] = ACTIONS(1685),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1687),
  },
  [375] = {
    [sym__automatic_semicolon] = ACTIONS(1689),
    [anon_sym_SEMI] = ACTIONS(1689),
    [sym_comment] = ACTIONS(71),
  },
  [376] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(817),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(817),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(1691),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(1691),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [377] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(818),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(818),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [378] = {
    [ts_builtin_sym_end] = ACTIONS(235),
    [anon_sym_export] = ACTIONS(237),
    [anon_sym_LBRACE] = ACTIONS(235),
    [anon_sym_RBRACE] = ACTIONS(235),
    [anon_sym_import] = ACTIONS(237),
    [anon_sym_var] = ACTIONS(237),
    [anon_sym_let] = ACTIONS(237),
    [anon_sym_const] = ACTIONS(237),
    [anon_sym_if] = ACTIONS(237),
    [anon_sym_else] = ACTIONS(237),
    [anon_sym_switch] = ACTIONS(237),
    [anon_sym_for] = ACTIONS(237),
    [anon_sym_LPAREN] = ACTIONS(235),
    [anon_sym_while] = ACTIONS(237),
    [anon_sym_do] = ACTIONS(237),
    [anon_sym_try] = ACTIONS(237),
    [anon_sym_with] = ACTIONS(237),
    [anon_sym_break] = ACTIONS(237),
    [anon_sym_continue] = ACTIONS(237),
    [anon_sym_debugger] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_throw] = ACTIONS(237),
    [anon_sym_SEMI] = ACTIONS(235),
    [anon_sym_yield] = ACTIONS(237),
    [anon_sym_LBRACK] = ACTIONS(235),
    [anon_sym_LT] = ACTIONS(235),
    [anon_sym_class] = ACTIONS(237),
    [anon_sym_async] = ACTIONS(237),
    [anon_sym_function] = ACTIONS(237),
    [anon_sym_EQ_GT] = ACTIONS(235),
    [anon_sym_new] = ACTIONS(237),
    [anon_sym_await] = ACTIONS(237),
    [anon_sym_PLUS] = ACTIONS(237),
    [anon_sym_DASH] = ACTIONS(237),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_TILDE] = ACTIONS(235),
    [anon_sym_typeof] = ACTIONS(237),
    [anon_sym_void] = ACTIONS(237),
    [anon_sym_delete] = ACTIONS(237),
    [anon_sym_PLUS_PLUS] = ACTIONS(235),
    [anon_sym_DASH_DASH] = ACTIONS(235),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(235),
    [anon_sym_BQUOTE] = ACTIONS(235),
    [sym_regex] = ACTIONS(237),
    [sym_number] = ACTIONS(235),
    [sym_identifier] = ACTIONS(237),
    [sym_this] = ACTIONS(237),
    [sym_super] = ACTIONS(237),
    [sym_true] = ACTIONS(237),
    [sym_false] = ACTIONS(237),
    [sym_null] = ACTIONS(237),
    [sym_undefined] = ACTIONS(237),
    [anon_sym_AT] = ACTIONS(235),
    [anon_sym_get] = ACTIONS(237),
    [anon_sym_set] = ACTIONS(237),
  },
  [379] = {
    [sym_class_heritage] = STATE(187),
    [sym_class_body] = STATE(188),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1693),
  },
  [380] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_function] = ACTIONS(1695),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(308),
  },
  [381] = {
    [sym_formal_parameters] = STATE(823),
    [anon_sym_STAR] = ACTIONS(1697),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1699),
  },
  [382] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(1701),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [383] = {
    [ts_builtin_sym_end] = ACTIONS(1703),
    [anon_sym_export] = ACTIONS(1705),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_import] = ACTIONS(1705),
    [anon_sym_var] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_else] = ACTIONS(1707),
    [anon_sym_switch] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_do] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [anon_sym_with] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_debugger] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_throw] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_class] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_function] = ACTIONS(1705),
    [anon_sym_EQ_GT] = ACTIONS(1703),
    [anon_sym_new] = ACTIONS(1705),
    [anon_sym_await] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_typeof] = ACTIONS(1705),
    [anon_sym_void] = ACTIONS(1705),
    [anon_sym_delete] = ACTIONS(1705),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1703),
    [anon_sym_BQUOTE] = ACTIONS(1703),
    [sym_regex] = ACTIONS(1705),
    [sym_number] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_this] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_null] = ACTIONS(1705),
    [sym_undefined] = ACTIONS(1705),
    [anon_sym_AT] = ACTIONS(1703),
    [anon_sym_get] = ACTIONS(1705),
    [anon_sym_set] = ACTIONS(1705),
  },
  [384] = {
    [sym__automatic_semicolon] = ACTIONS(1709),
    [anon_sym_SEMI] = ACTIONS(1709),
    [sym_comment] = ACTIONS(71),
  },
  [385] = {
    [sym__automatic_semicolon] = ACTIONS(1711),
    [ts_builtin_sym_end] = ACTIONS(400),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_else] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [386] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1711),
    [ts_builtin_sym_end] = ACTIONS(400),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_else] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [387] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(1713),
    [anon_sym_class] = ACTIONS(1715),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [388] = {
    [sym_switch_case] = STATE(833),
    [sym_switch_default] = STATE(833),
    [aux_sym_switch_body_repeat1] = STATE(833),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(1719),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [389] = {
    [ts_builtin_sym_end] = ACTIONS(1723),
    [anon_sym_export] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1723),
    [anon_sym_RBRACE] = ACTIONS(1723),
    [anon_sym_import] = ACTIONS(1725),
    [anon_sym_var] = ACTIONS(1725),
    [anon_sym_let] = ACTIONS(1725),
    [anon_sym_const] = ACTIONS(1725),
    [anon_sym_if] = ACTIONS(1725),
    [anon_sym_switch] = ACTIONS(1725),
    [anon_sym_for] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_while] = ACTIONS(1725),
    [anon_sym_do] = ACTIONS(1725),
    [anon_sym_try] = ACTIONS(1725),
    [anon_sym_with] = ACTIONS(1725),
    [anon_sym_break] = ACTIONS(1725),
    [anon_sym_continue] = ACTIONS(1725),
    [anon_sym_debugger] = ACTIONS(1725),
    [anon_sym_return] = ACTIONS(1725),
    [anon_sym_throw] = ACTIONS(1725),
    [anon_sym_SEMI] = ACTIONS(1723),
    [anon_sym_yield] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1723),
    [anon_sym_LT] = ACTIONS(1723),
    [anon_sym_class] = ACTIONS(1725),
    [anon_sym_async] = ACTIONS(1725),
    [anon_sym_function] = ACTIONS(1725),
    [anon_sym_EQ_GT] = ACTIONS(1723),
    [anon_sym_new] = ACTIONS(1725),
    [anon_sym_await] = ACTIONS(1725),
    [anon_sym_PLUS] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1725),
    [anon_sym_BANG] = ACTIONS(1723),
    [anon_sym_TILDE] = ACTIONS(1723),
    [anon_sym_typeof] = ACTIONS(1725),
    [anon_sym_void] = ACTIONS(1725),
    [anon_sym_delete] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1723),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1723),
    [anon_sym_BQUOTE] = ACTIONS(1723),
    [sym_regex] = ACTIONS(1725),
    [sym_number] = ACTIONS(1723),
    [sym_identifier] = ACTIONS(1725),
    [sym_this] = ACTIONS(1725),
    [sym_super] = ACTIONS(1725),
    [sym_true] = ACTIONS(1725),
    [sym_false] = ACTIONS(1725),
    [sym_null] = ACTIONS(1725),
    [sym_undefined] = ACTIONS(1725),
    [anon_sym_AT] = ACTIONS(1723),
    [anon_sym_get] = ACTIONS(1725),
    [anon_sym_set] = ACTIONS(1725),
  },
  [390] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(836),
    [sym_array] = STATE(837),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1727),
  },
  [391] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(836),
    [sym_array] = STATE(837),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1727),
  },
  [392] = {
    [anon_sym_LBRACE] = ACTIONS(235),
    [anon_sym_LPAREN] = ACTIONS(235),
    [anon_sym_RPAREN] = ACTIONS(235),
    [anon_sym_SEMI] = ACTIONS(235),
    [anon_sym_yield] = ACTIONS(237),
    [anon_sym_LBRACK] = ACTIONS(235),
    [anon_sym_LT] = ACTIONS(235),
    [anon_sym_class] = ACTIONS(237),
    [anon_sym_async] = ACTIONS(237),
    [anon_sym_function] = ACTIONS(237),
    [anon_sym_EQ_GT] = ACTIONS(235),
    [anon_sym_new] = ACTIONS(237),
    [anon_sym_await] = ACTIONS(237),
    [anon_sym_PLUS] = ACTIONS(237),
    [anon_sym_DASH] = ACTIONS(237),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_TILDE] = ACTIONS(235),
    [anon_sym_typeof] = ACTIONS(237),
    [anon_sym_void] = ACTIONS(237),
    [anon_sym_delete] = ACTIONS(237),
    [anon_sym_PLUS_PLUS] = ACTIONS(235),
    [anon_sym_DASH_DASH] = ACTIONS(235),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(235),
    [anon_sym_BQUOTE] = ACTIONS(235),
    [sym_regex] = ACTIONS(237),
    [sym_number] = ACTIONS(235),
    [sym_identifier] = ACTIONS(237),
    [sym_this] = ACTIONS(237),
    [sym_super] = ACTIONS(237),
    [sym_true] = ACTIONS(237),
    [sym_false] = ACTIONS(237),
    [sym_null] = ACTIONS(237),
    [sym_undefined] = ACTIONS(237),
    [anon_sym_AT] = ACTIONS(235),
    [anon_sym_get] = ACTIONS(237),
    [anon_sym_set] = ACTIONS(237),
  },
  [393] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1729),
    [anon_sym_of] = ACTIONS(1731),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [394] = {
    [sym_expression_statement] = STATE(841),
    [sym_empty_statement] = STATE(841),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [395] = {
    [sym__automatic_semicolon] = ACTIONS(1733),
    [anon_sym_SEMI] = ACTIONS(1733),
    [sym_comment] = ACTIONS(71),
  },
  [396] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1735),
    [anon_sym_of] = ACTIONS(1737),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [397] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(1739),
    [anon_sym_of] = ACTIONS(1741),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [398] = {
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_LBRACE] = ACTIONS(480),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RBRACE] = ACTIONS(480),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_RPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_COLON] = ACTIONS(480),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_RBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(480),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(480),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(480),
  },
  [399] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(848),
    [anon_sym_STAR] = ACTIONS(1441),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [400] = {
    [sym__initializer] = STATE(708),
    [aux_sym_object_repeat1] = STATE(848),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [401] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(848),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [402] = {
    [aux_sym_object_repeat1] = STATE(850),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1745),
    [sym_comment] = ACTIONS(71),
  },
  [403] = {
    [sym_decorator] = STATE(716),
    [sym_method_definition] = STATE(851),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(716),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [404] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1745),
    [sym_comment] = ACTIONS(71),
  },
  [405] = {
    [anon_sym_RPAREN] = ACTIONS(1747),
    [sym_comment] = ACTIONS(71),
  },
  [406] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(882),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [407] = {
    [anon_sym_STAR] = ACTIONS(894),
    [anon_sym_LBRACE] = ACTIONS(894),
    [anon_sym_COMMA] = ACTIONS(894),
    [anon_sym_RBRACE] = ACTIONS(894),
    [anon_sym_LPAREN] = ACTIONS(894),
    [anon_sym_RPAREN] = ACTIONS(894),
    [anon_sym_in] = ACTIONS(896),
    [anon_sym_COLON] = ACTIONS(894),
    [anon_sym_LBRACK] = ACTIONS(894),
    [anon_sym_RBRACK] = ACTIONS(894),
    [anon_sym_LT] = ACTIONS(896),
    [anon_sym_GT] = ACTIONS(896),
    [anon_sym_SLASH] = ACTIONS(896),
    [anon_sym_DOT] = ACTIONS(894),
    [anon_sym_EQ] = ACTIONS(896),
    [anon_sym_QMARK] = ACTIONS(894),
    [anon_sym_AMP_AMP] = ACTIONS(894),
    [anon_sym_PIPE_PIPE] = ACTIONS(894),
    [anon_sym_GT_GT] = ACTIONS(896),
    [anon_sym_GT_GT_GT] = ACTIONS(894),
    [anon_sym_LT_LT] = ACTIONS(894),
    [anon_sym_AMP] = ACTIONS(896),
    [anon_sym_CARET] = ACTIONS(894),
    [anon_sym_PIPE] = ACTIONS(896),
    [anon_sym_PLUS] = ACTIONS(896),
    [anon_sym_DASH] = ACTIONS(896),
    [anon_sym_PERCENT] = ACTIONS(894),
    [anon_sym_LT_EQ] = ACTIONS(894),
    [anon_sym_EQ_EQ] = ACTIONS(896),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(894),
    [anon_sym_BANG_EQ] = ACTIONS(896),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(894),
    [anon_sym_GT_EQ] = ACTIONS(894),
    [anon_sym_instanceof] = ACTIONS(894),
    [anon_sym_PLUS_PLUS] = ACTIONS(894),
    [anon_sym_DASH_DASH] = ACTIONS(894),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(894),
  },
  [408] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(854),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1749),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [409] = {
    [aux_sym_array_repeat1] = STATE(854),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(1749),
    [sym_comment] = ACTIONS(71),
  },
  [410] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(1749),
    [sym_comment] = ACTIONS(71),
  },
  [411] = {
    [sym_jsx_element] = STATE(856),
    [sym_jsx_expression] = STATE(856),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(856),
    [aux_sym_jsx_element_repeat1] = STATE(856),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(1751),
    [sym_jsx_text] = ACTIONS(1753),
    [sym_comment] = ACTIONS(71),
  },
  [412] = {
    [sym_jsx_expression] = STATE(858),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(858),
    [aux_sym_jsx_opening_element_repeat1] = STATE(858),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(950),
    [anon_sym_SLASH] = ACTIONS(1755),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [413] = {
    [sym_jsx_expression] = STATE(860),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(860),
    [aux_sym_jsx_opening_element_repeat1] = STATE(860),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(1757),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [414] = {
    [sym_jsx_expression] = STATE(860),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(860),
    [aux_sym_jsx_opening_element_repeat1] = STATE(860),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(1757),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [415] = {
    [sym_jsx_expression] = STATE(860),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(860),
    [aux_sym_jsx_opening_element_repeat1] = STATE(860),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(1757),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [416] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(862),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(1759),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [417] = {
    [sym_class_heritage] = STATE(863),
    [sym_class_body] = STATE(864),
    [anon_sym_LBRACE] = ACTIONS(704),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [418] = {
    [sym_class_body] = STATE(865),
    [anon_sym_LBRACE] = ACTIONS(704),
    [sym_comment] = ACTIONS(71),
  },
  [419] = {
    [anon_sym_STAR] = ACTIONS(1012),
    [anon_sym_LBRACE] = ACTIONS(1012),
    [anon_sym_COMMA] = ACTIONS(1012),
    [anon_sym_RBRACE] = ACTIONS(1012),
    [anon_sym_LPAREN] = ACTIONS(1012),
    [anon_sym_RPAREN] = ACTIONS(1012),
    [anon_sym_in] = ACTIONS(1014),
    [anon_sym_COLON] = ACTIONS(1012),
    [anon_sym_LBRACK] = ACTIONS(1012),
    [anon_sym_RBRACK] = ACTIONS(1012),
    [anon_sym_LT] = ACTIONS(1014),
    [anon_sym_GT] = ACTIONS(1014),
    [anon_sym_SLASH] = ACTIONS(1014),
    [anon_sym_DOT] = ACTIONS(1012),
    [anon_sym_QMARK] = ACTIONS(1012),
    [anon_sym_AMP_AMP] = ACTIONS(1012),
    [anon_sym_PIPE_PIPE] = ACTIONS(1012),
    [anon_sym_GT_GT] = ACTIONS(1014),
    [anon_sym_GT_GT_GT] = ACTIONS(1012),
    [anon_sym_LT_LT] = ACTIONS(1012),
    [anon_sym_AMP] = ACTIONS(1014),
    [anon_sym_CARET] = ACTIONS(1012),
    [anon_sym_PIPE] = ACTIONS(1014),
    [anon_sym_PLUS] = ACTIONS(1014),
    [anon_sym_DASH] = ACTIONS(1014),
    [anon_sym_PERCENT] = ACTIONS(1012),
    [anon_sym_LT_EQ] = ACTIONS(1012),
    [anon_sym_EQ_EQ] = ACTIONS(1014),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1012),
    [anon_sym_BANG_EQ] = ACTIONS(1014),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1012),
    [anon_sym_GT_EQ] = ACTIONS(1012),
    [anon_sym_instanceof] = ACTIONS(1012),
    [anon_sym_PLUS_PLUS] = ACTIONS(1012),
    [anon_sym_DASH_DASH] = ACTIONS(1012),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1012),
  },
  [420] = {
    [sym_object] = STATE(868),
    [sym_assignment_pattern] = STATE(869),
    [sym_array] = STATE(870),
    [sym_rest_parameter] = STATE(869),
    [anon_sym_LBRACE] = ACTIONS(464),
    [anon_sym_LBRACK] = ACTIONS(1016),
    [anon_sym_async] = ACTIONS(1761),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(175),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1763),
    [anon_sym_get] = ACTIONS(1761),
    [anon_sym_set] = ACTIONS(1761),
  },
  [421] = {
    [anon_sym_LBRACE] = ACTIONS(1765),
    [anon_sym_EQ_GT] = ACTIONS(1765),
    [sym_comment] = ACTIONS(71),
  },
  [422] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1767),
  },
  [423] = {
    [sym_parenthesized_expression] = STATE(872),
    [sym__expression] = STATE(872),
    [sym_yield_expression] = STATE(872),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(872),
    [sym_jsx_fragment] = STATE(872),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(872),
    [sym_anonymous_class] = STATE(872),
    [sym_class] = STATE(872),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(872),
    [sym_generator_function] = STATE(872),
    [sym_call_expression] = STATE(872),
    [sym_new_expression] = STATE(872),
    [sym_await_expression] = STATE(872),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(872),
    [sym_augmented_assignment_expression] = STATE(872),
    [sym_ternary_expression] = STATE(872),
    [sym_binary_expression] = STATE(872),
    [sym_unary_expression] = STATE(872),
    [sym_update_expression] = STATE(872),
    [sym_template_string] = STATE(872),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1769),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1771),
    [sym_number] = ACTIONS(1769),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1771),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1771),
    [sym_false] = ACTIONS(1771),
    [sym_null] = ACTIONS(1771),
    [sym_undefined] = ACTIONS(1771),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [424] = {
    [sym_formal_parameters] = STATE(874),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1773),
  },
  [425] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(876),
    [sym__expression] = STATE(876),
    [sym_yield_expression] = STATE(876),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(876),
    [sym_jsx_fragment] = STATE(876),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(876),
    [sym_anonymous_class] = STATE(876),
    [sym_class] = STATE(876),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(876),
    [sym_generator_function] = STATE(876),
    [sym_call_expression] = STATE(876),
    [sym_new_expression] = STATE(876),
    [sym_await_expression] = STATE(876),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(876),
    [sym_augmented_assignment_expression] = STATE(876),
    [sym_ternary_expression] = STATE(876),
    [sym_binary_expression] = STATE(876),
    [sym_unary_expression] = STATE(876),
    [sym_update_expression] = STATE(876),
    [sym_template_string] = STATE(876),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1775),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1777),
    [sym_number] = ACTIONS(1775),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1777),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1777),
    [sym_false] = ACTIONS(1777),
    [sym_null] = ACTIONS(1777),
    [sym_undefined] = ACTIONS(1777),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [426] = {
    [sym_parenthesized_expression] = STATE(877),
    [sym__expression] = STATE(877),
    [sym_yield_expression] = STATE(877),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(877),
    [sym_jsx_fragment] = STATE(877),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(877),
    [sym_anonymous_class] = STATE(877),
    [sym_class] = STATE(877),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(877),
    [sym_generator_function] = STATE(877),
    [sym_call_expression] = STATE(877),
    [sym_new_expression] = STATE(877),
    [sym_await_expression] = STATE(877),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(877),
    [sym_augmented_assignment_expression] = STATE(877),
    [sym_ternary_expression] = STATE(877),
    [sym_binary_expression] = STATE(877),
    [sym_unary_expression] = STATE(877),
    [sym_update_expression] = STATE(877),
    [sym_template_string] = STATE(877),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1779),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1781),
    [sym_number] = ACTIONS(1779),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1781),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1781),
    [sym_false] = ACTIONS(1781),
    [sym_null] = ACTIONS(1781),
    [sym_undefined] = ACTIONS(1781),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [427] = {
    [anon_sym_EQ_GT] = ACTIONS(1783),
    [sym_comment] = ACTIONS(71),
  },
  [428] = {
    [anon_sym_STAR] = ACTIONS(1040),
    [anon_sym_LBRACE] = ACTIONS(1040),
    [anon_sym_COMMA] = ACTIONS(1040),
    [anon_sym_RBRACE] = ACTIONS(1040),
    [anon_sym_LPAREN] = ACTIONS(1040),
    [anon_sym_RPAREN] = ACTIONS(1040),
    [anon_sym_in] = ACTIONS(1042),
    [anon_sym_COLON] = ACTIONS(1040),
    [anon_sym_LBRACK] = ACTIONS(1040),
    [anon_sym_RBRACK] = ACTIONS(1040),
    [anon_sym_LT] = ACTIONS(1042),
    [anon_sym_GT] = ACTIONS(1042),
    [anon_sym_SLASH] = ACTIONS(1042),
    [anon_sym_DOT] = ACTIONS(1040),
    [anon_sym_QMARK] = ACTIONS(1040),
    [anon_sym_AMP_AMP] = ACTIONS(1040),
    [anon_sym_PIPE_PIPE] = ACTIONS(1040),
    [anon_sym_GT_GT] = ACTIONS(1042),
    [anon_sym_GT_GT_GT] = ACTIONS(1040),
    [anon_sym_LT_LT] = ACTIONS(1040),
    [anon_sym_AMP] = ACTIONS(1042),
    [anon_sym_CARET] = ACTIONS(1040),
    [anon_sym_PIPE] = ACTIONS(1042),
    [anon_sym_PLUS] = ACTIONS(1042),
    [anon_sym_DASH] = ACTIONS(1042),
    [anon_sym_PERCENT] = ACTIONS(1040),
    [anon_sym_LT_EQ] = ACTIONS(1040),
    [anon_sym_EQ_EQ] = ACTIONS(1042),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1040),
    [anon_sym_BANG_EQ] = ACTIONS(1042),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1040),
    [anon_sym_GT_EQ] = ACTIONS(1040),
    [anon_sym_instanceof] = ACTIONS(1040),
    [anon_sym_PLUS_PLUS] = ACTIONS(1040),
    [anon_sym_DASH_DASH] = ACTIONS(1040),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1040),
  },
  [429] = {
    [aux_sym_formal_parameters_repeat1] = STATE(880),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(1785),
    [sym_comment] = ACTIONS(71),
  },
  [430] = {
    [sym_formal_parameters] = STATE(882),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1787),
  },
  [431] = {
    [sym_formal_parameters] = STATE(874),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [432] = {
    [sym_statement_block] = STATE(884),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [433] = {
    [sym_export_statement] = STATE(888),
    [sym__declaration] = STATE(888),
    [sym_import_statement] = STATE(888),
    [sym_expression_statement] = STATE(888),
    [sym_variable_declaration] = STATE(888),
    [sym_lexical_declaration] = STATE(888),
    [sym_statement_block] = STATE(888),
    [sym_if_statement] = STATE(888),
    [sym_switch_statement] = STATE(888),
    [sym_for_statement] = STATE(888),
    [sym_for_in_statement] = STATE(888),
    [sym_for_of_statement] = STATE(888),
    [sym_while_statement] = STATE(888),
    [sym_do_statement] = STATE(888),
    [sym_try_statement] = STATE(888),
    [sym_with_statement] = STATE(888),
    [sym_break_statement] = STATE(888),
    [sym_continue_statement] = STATE(888),
    [sym_debugger_statement] = STATE(888),
    [sym_return_statement] = STATE(888),
    [sym_throw_statement] = STATE(888),
    [sym_empty_statement] = STATE(888),
    [sym_labeled_statement] = STATE(888),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(402),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(402),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(889),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(402),
    [sym_pair] = STATE(402),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(888),
    [aux_sym_export_statement_repeat1] = STATE(889),
    [aux_sym_object_repeat1] = STATE(404),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(676),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(1791),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(1793),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(1795),
    [anon_sym_set] = ACTIONS(1795),
  },
  [434] = {
    [anon_sym_STAR] = ACTIONS(1048),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(1048),
    [anon_sym_RPAREN] = ACTIONS(1048),
    [anon_sym_in] = ACTIONS(1050),
    [anon_sym_COLON] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(1048),
    [anon_sym_RBRACK] = ACTIONS(1048),
    [anon_sym_LT] = ACTIONS(1050),
    [anon_sym_GT] = ACTIONS(1050),
    [anon_sym_SLASH] = ACTIONS(1050),
    [anon_sym_DOT] = ACTIONS(1048),
    [anon_sym_QMARK] = ACTIONS(1048),
    [anon_sym_AMP_AMP] = ACTIONS(1048),
    [anon_sym_PIPE_PIPE] = ACTIONS(1048),
    [anon_sym_GT_GT] = ACTIONS(1050),
    [anon_sym_GT_GT_GT] = ACTIONS(1048),
    [anon_sym_LT_LT] = ACTIONS(1048),
    [anon_sym_AMP] = ACTIONS(1050),
    [anon_sym_CARET] = ACTIONS(1048),
    [anon_sym_PIPE] = ACTIONS(1050),
    [anon_sym_PLUS] = ACTIONS(1050),
    [anon_sym_DASH] = ACTIONS(1050),
    [anon_sym_PERCENT] = ACTIONS(1048),
    [anon_sym_LT_EQ] = ACTIONS(1048),
    [anon_sym_EQ_EQ] = ACTIONS(1050),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1048),
    [anon_sym_BANG_EQ] = ACTIONS(1050),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1048),
    [anon_sym_GT_EQ] = ACTIONS(1048),
    [anon_sym_instanceof] = ACTIONS(1048),
    [anon_sym_PLUS_PLUS] = ACTIONS(1048),
    [anon_sym_DASH_DASH] = ACTIONS(1048),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1048),
  },
  [435] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1048),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [436] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1052),
    [anon_sym_LBRACE] = ACTIONS(1052),
    [anon_sym_COMMA] = ACTIONS(1052),
    [anon_sym_RBRACE] = ACTIONS(1052),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1052),
    [anon_sym_in] = ACTIONS(1054),
    [anon_sym_COLON] = ACTIONS(1052),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1052),
    [anon_sym_LT] = ACTIONS(1054),
    [anon_sym_GT] = ACTIONS(1054),
    [anon_sym_SLASH] = ACTIONS(1054),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1052),
    [anon_sym_AMP_AMP] = ACTIONS(1052),
    [anon_sym_PIPE_PIPE] = ACTIONS(1052),
    [anon_sym_GT_GT] = ACTIONS(1054),
    [anon_sym_GT_GT_GT] = ACTIONS(1052),
    [anon_sym_LT_LT] = ACTIONS(1052),
    [anon_sym_AMP] = ACTIONS(1054),
    [anon_sym_CARET] = ACTIONS(1052),
    [anon_sym_PIPE] = ACTIONS(1054),
    [anon_sym_PLUS] = ACTIONS(1054),
    [anon_sym_DASH] = ACTIONS(1054),
    [anon_sym_PERCENT] = ACTIONS(1052),
    [anon_sym_LT_EQ] = ACTIONS(1052),
    [anon_sym_EQ_EQ] = ACTIONS(1054),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1052),
    [anon_sym_BANG_EQ] = ACTIONS(1054),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1052),
    [anon_sym_GT_EQ] = ACTIONS(1052),
    [anon_sym_instanceof] = ACTIONS(1052),
    [anon_sym_PLUS_PLUS] = ACTIONS(1052),
    [anon_sym_DASH_DASH] = ACTIONS(1052),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [437] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1056),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [438] = {
    [anon_sym_COMMA] = ACTIONS(1797),
    [anon_sym_RPAREN] = ACTIONS(1797),
    [sym_comment] = ACTIONS(71),
  },
  [439] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1058),
    [anon_sym_LBRACE] = ACTIONS(1058),
    [anon_sym_COMMA] = ACTIONS(1058),
    [anon_sym_RBRACE] = ACTIONS(1058),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1058),
    [anon_sym_in] = ACTIONS(1060),
    [anon_sym_COLON] = ACTIONS(1058),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1058),
    [anon_sym_LT] = ACTIONS(1060),
    [anon_sym_GT] = ACTIONS(1060),
    [anon_sym_SLASH] = ACTIONS(1060),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1058),
    [anon_sym_AMP_AMP] = ACTIONS(1058),
    [anon_sym_PIPE_PIPE] = ACTIONS(1058),
    [anon_sym_GT_GT] = ACTIONS(1060),
    [anon_sym_GT_GT_GT] = ACTIONS(1058),
    [anon_sym_LT_LT] = ACTIONS(1058),
    [anon_sym_AMP] = ACTIONS(1060),
    [anon_sym_CARET] = ACTIONS(1058),
    [anon_sym_PIPE] = ACTIONS(1060),
    [anon_sym_PLUS] = ACTIONS(1060),
    [anon_sym_DASH] = ACTIONS(1060),
    [anon_sym_PERCENT] = ACTIONS(1058),
    [anon_sym_LT_EQ] = ACTIONS(1058),
    [anon_sym_EQ_EQ] = ACTIONS(1060),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1058),
    [anon_sym_BANG_EQ] = ACTIONS(1060),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1058),
    [anon_sym_GT_EQ] = ACTIONS(1058),
    [anon_sym_instanceof] = ACTIONS(1058),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [440] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1062),
    [anon_sym_LBRACE] = ACTIONS(1062),
    [anon_sym_COMMA] = ACTIONS(1062),
    [anon_sym_RBRACE] = ACTIONS(1062),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1062),
    [anon_sym_in] = ACTIONS(1064),
    [anon_sym_COLON] = ACTIONS(1062),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1062),
    [anon_sym_LT] = ACTIONS(1064),
    [anon_sym_GT] = ACTIONS(1064),
    [anon_sym_SLASH] = ACTIONS(1064),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1062),
    [anon_sym_AMP_AMP] = ACTIONS(1062),
    [anon_sym_PIPE_PIPE] = ACTIONS(1062),
    [anon_sym_GT_GT] = ACTIONS(1064),
    [anon_sym_GT_GT_GT] = ACTIONS(1062),
    [anon_sym_LT_LT] = ACTIONS(1062),
    [anon_sym_AMP] = ACTIONS(1064),
    [anon_sym_CARET] = ACTIONS(1062),
    [anon_sym_PIPE] = ACTIONS(1064),
    [anon_sym_PLUS] = ACTIONS(1064),
    [anon_sym_DASH] = ACTIONS(1064),
    [anon_sym_PERCENT] = ACTIONS(1062),
    [anon_sym_LT_EQ] = ACTIONS(1062),
    [anon_sym_EQ_EQ] = ACTIONS(1064),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1062),
    [anon_sym_BANG_EQ] = ACTIONS(1064),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1062),
    [anon_sym_GT_EQ] = ACTIONS(1062),
    [anon_sym_instanceof] = ACTIONS(1062),
    [anon_sym_PLUS_PLUS] = ACTIONS(1062),
    [anon_sym_DASH_DASH] = ACTIONS(1062),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [441] = {
    [anon_sym_STAR] = ACTIONS(1066),
    [anon_sym_LBRACE] = ACTIONS(1066),
    [anon_sym_COMMA] = ACTIONS(1066),
    [anon_sym_RBRACE] = ACTIONS(1066),
    [anon_sym_LPAREN] = ACTIONS(1066),
    [anon_sym_RPAREN] = ACTIONS(1066),
    [anon_sym_in] = ACTIONS(1068),
    [anon_sym_COLON] = ACTIONS(1066),
    [anon_sym_LBRACK] = ACTIONS(1066),
    [anon_sym_RBRACK] = ACTIONS(1066),
    [anon_sym_LT] = ACTIONS(1068),
    [anon_sym_GT] = ACTIONS(1068),
    [anon_sym_SLASH] = ACTIONS(1068),
    [anon_sym_DOT] = ACTIONS(1066),
    [anon_sym_QMARK] = ACTIONS(1066),
    [anon_sym_AMP_AMP] = ACTIONS(1066),
    [anon_sym_PIPE_PIPE] = ACTIONS(1066),
    [anon_sym_GT_GT] = ACTIONS(1068),
    [anon_sym_GT_GT_GT] = ACTIONS(1066),
    [anon_sym_LT_LT] = ACTIONS(1066),
    [anon_sym_AMP] = ACTIONS(1068),
    [anon_sym_CARET] = ACTIONS(1066),
    [anon_sym_PIPE] = ACTIONS(1068),
    [anon_sym_PLUS] = ACTIONS(1068),
    [anon_sym_DASH] = ACTIONS(1068),
    [anon_sym_PERCENT] = ACTIONS(1066),
    [anon_sym_LT_EQ] = ACTIONS(1066),
    [anon_sym_EQ_EQ] = ACTIONS(1068),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1066),
    [anon_sym_BANG_EQ] = ACTIONS(1068),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1066),
    [anon_sym_GT_EQ] = ACTIONS(1066),
    [anon_sym_instanceof] = ACTIONS(1066),
    [anon_sym_PLUS_PLUS] = ACTIONS(1066),
    [anon_sym_DASH_DASH] = ACTIONS(1066),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1066),
  },
  [442] = {
    [sym_template_substitution] = STATE(596),
    [aux_sym_template_string_repeat1] = STATE(596),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1799),
    [sym__template_chars] = ACTIONS(1076),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(342),
  },
  [443] = {
    [anon_sym_LBRACE] = ACTIONS(1801),
    [anon_sym_EQ_GT] = ACTIONS(1801),
    [sym_comment] = ACTIONS(71),
  },
  [444] = {
    [sym_parenthesized_expression] = STATE(891),
    [sym__expression] = STATE(891),
    [sym_yield_expression] = STATE(891),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(891),
    [sym_jsx_fragment] = STATE(891),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(891),
    [sym_anonymous_class] = STATE(891),
    [sym_class] = STATE(891),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(891),
    [sym_generator_function] = STATE(891),
    [sym_call_expression] = STATE(891),
    [sym_new_expression] = STATE(891),
    [sym_await_expression] = STATE(891),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(891),
    [sym_augmented_assignment_expression] = STATE(891),
    [sym_ternary_expression] = STATE(891),
    [sym_binary_expression] = STATE(891),
    [sym_unary_expression] = STATE(891),
    [sym_update_expression] = STATE(891),
    [sym_template_string] = STATE(891),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1803),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1805),
    [sym_number] = ACTIONS(1803),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1805),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1805),
    [sym_false] = ACTIONS(1805),
    [sym_null] = ACTIONS(1805),
    [sym_undefined] = ACTIONS(1805),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [445] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1807),
  },
  [446] = {
    [sym_parenthesized_expression] = STATE(893),
    [sym__expression] = STATE(893),
    [sym_yield_expression] = STATE(893),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(893),
    [sym_jsx_fragment] = STATE(893),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(893),
    [sym_anonymous_class] = STATE(893),
    [sym_class] = STATE(893),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(893),
    [sym_generator_function] = STATE(893),
    [sym_call_expression] = STATE(893),
    [sym_new_expression] = STATE(893),
    [sym_await_expression] = STATE(893),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(893),
    [sym_augmented_assignment_expression] = STATE(893),
    [sym_ternary_expression] = STATE(893),
    [sym_binary_expression] = STATE(893),
    [sym_unary_expression] = STATE(893),
    [sym_update_expression] = STATE(893),
    [sym_template_string] = STATE(893),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1809),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1811),
    [sym_number] = ACTIONS(1809),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1811),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1811),
    [sym_false] = ACTIONS(1811),
    [sym_null] = ACTIONS(1811),
    [sym_undefined] = ACTIONS(1811),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [447] = {
    [anon_sym_STAR] = ACTIONS(1088),
    [anon_sym_COMMA] = ACTIONS(1547),
    [anon_sym_LPAREN] = ACTIONS(1088),
    [anon_sym_RPAREN] = ACTIONS(1547),
    [anon_sym_in] = ACTIONS(1090),
    [anon_sym_LBRACK] = ACTIONS(1088),
    [anon_sym_LT] = ACTIONS(1090),
    [anon_sym_GT] = ACTIONS(1090),
    [anon_sym_SLASH] = ACTIONS(1090),
    [anon_sym_DOT] = ACTIONS(1088),
    [anon_sym_QMARK] = ACTIONS(1088),
    [anon_sym_AMP_AMP] = ACTIONS(1088),
    [anon_sym_PIPE_PIPE] = ACTIONS(1088),
    [anon_sym_GT_GT] = ACTIONS(1090),
    [anon_sym_GT_GT_GT] = ACTIONS(1088),
    [anon_sym_LT_LT] = ACTIONS(1088),
    [anon_sym_AMP] = ACTIONS(1090),
    [anon_sym_CARET] = ACTIONS(1088),
    [anon_sym_PIPE] = ACTIONS(1090),
    [anon_sym_PLUS] = ACTIONS(1090),
    [anon_sym_DASH] = ACTIONS(1090),
    [anon_sym_PERCENT] = ACTIONS(1088),
    [anon_sym_LT_EQ] = ACTIONS(1088),
    [anon_sym_EQ_EQ] = ACTIONS(1090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1088),
    [anon_sym_BANG_EQ] = ACTIONS(1090),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1088),
    [anon_sym_GT_EQ] = ACTIONS(1088),
    [anon_sym_instanceof] = ACTIONS(1088),
    [anon_sym_PLUS_PLUS] = ACTIONS(1088),
    [anon_sym_DASH_DASH] = ACTIONS(1088),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1088),
  },
  [448] = {
    [aux_sym_formal_parameters_repeat1] = STATE(880),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(1813),
    [sym_comment] = ACTIONS(71),
  },
  [449] = {
    [sym_parenthesized_expression] = STATE(896),
    [sym__expression] = STATE(896),
    [sym_yield_expression] = STATE(896),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(896),
    [sym_jsx_fragment] = STATE(896),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(896),
    [sym_anonymous_class] = STATE(896),
    [sym_class] = STATE(896),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(896),
    [sym_generator_function] = STATE(896),
    [sym_call_expression] = STATE(896),
    [sym_new_expression] = STATE(896),
    [sym_await_expression] = STATE(896),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(896),
    [sym_augmented_assignment_expression] = STATE(896),
    [sym_spread_element] = STATE(897),
    [sym_ternary_expression] = STATE(896),
    [sym_binary_expression] = STATE(896),
    [sym_unary_expression] = STATE(896),
    [sym_update_expression] = STATE(896),
    [sym_template_string] = STATE(896),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(898),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(1815),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1096),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1817),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1819),
    [sym_number] = ACTIONS(1817),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1819),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1819),
    [sym_false] = ACTIONS(1819),
    [sym_null] = ACTIONS(1819),
    [sym_undefined] = ACTIONS(1819),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [450] = {
    [sym_parenthesized_expression] = STATE(608),
    [sym__expressions] = STATE(899),
    [sym__expression] = STATE(608),
    [sym_yield_expression] = STATE(608),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(608),
    [sym_jsx_fragment] = STATE(608),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(608),
    [sym_anonymous_class] = STATE(608),
    [sym_class] = STATE(608),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(608),
    [sym_generator_function] = STATE(608),
    [sym_call_expression] = STATE(608),
    [sym_new_expression] = STATE(608),
    [sym_await_expression] = STATE(608),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(608),
    [sym_augmented_assignment_expression] = STATE(608),
    [sym_ternary_expression] = STATE(608),
    [sym_binary_expression] = STATE(608),
    [sym_unary_expression] = STATE(608),
    [sym_update_expression] = STATE(608),
    [sym_sequence_expression] = STATE(899),
    [sym_template_string] = STATE(608),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1102),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1104),
    [sym_number] = ACTIONS(1102),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1104),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1104),
    [sym_false] = ACTIONS(1104),
    [sym_null] = ACTIONS(1104),
    [sym_undefined] = ACTIONS(1104),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [451] = {
    [anon_sym_STAR] = ACTIONS(1106),
    [anon_sym_LBRACE] = ACTIONS(1106),
    [anon_sym_COMMA] = ACTIONS(1106),
    [anon_sym_RBRACE] = ACTIONS(1106),
    [anon_sym_LPAREN] = ACTIONS(1106),
    [anon_sym_RPAREN] = ACTIONS(1106),
    [anon_sym_in] = ACTIONS(1108),
    [anon_sym_COLON] = ACTIONS(1106),
    [anon_sym_LBRACK] = ACTIONS(1106),
    [anon_sym_RBRACK] = ACTIONS(1106),
    [anon_sym_LT] = ACTIONS(1108),
    [anon_sym_GT] = ACTIONS(1108),
    [anon_sym_SLASH] = ACTIONS(1108),
    [anon_sym_DOT] = ACTIONS(1106),
    [anon_sym_QMARK] = ACTIONS(1106),
    [anon_sym_AMP_AMP] = ACTIONS(1106),
    [anon_sym_PIPE_PIPE] = ACTIONS(1106),
    [anon_sym_GT_GT] = ACTIONS(1108),
    [anon_sym_GT_GT_GT] = ACTIONS(1106),
    [anon_sym_LT_LT] = ACTIONS(1106),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_CARET] = ACTIONS(1106),
    [anon_sym_PIPE] = ACTIONS(1108),
    [anon_sym_PLUS] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1108),
    [anon_sym_PERCENT] = ACTIONS(1106),
    [anon_sym_LT_EQ] = ACTIONS(1106),
    [anon_sym_EQ_EQ] = ACTIONS(1108),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1106),
    [anon_sym_BANG_EQ] = ACTIONS(1108),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1106),
    [anon_sym_GT_EQ] = ACTIONS(1106),
    [anon_sym_instanceof] = ACTIONS(1106),
    [anon_sym_PLUS_PLUS] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1106),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1106),
  },
  [452] = {
    [sym__automatic_semicolon] = ACTIONS(1821),
    [anon_sym_STAR] = ACTIONS(1821),
    [anon_sym_COMMA] = ACTIONS(1821),
    [anon_sym_RBRACE] = ACTIONS(1821),
    [anon_sym_LPAREN] = ACTIONS(1821),
    [anon_sym_RPAREN] = ACTIONS(1821),
    [anon_sym_in] = ACTIONS(1823),
    [anon_sym_SEMI] = ACTIONS(1821),
    [anon_sym_LBRACK] = ACTIONS(1821),
    [anon_sym_LT] = ACTIONS(1823),
    [anon_sym_GT] = ACTIONS(1823),
    [anon_sym_SLASH] = ACTIONS(1823),
    [anon_sym_DOT] = ACTIONS(1821),
    [anon_sym_QMARK] = ACTIONS(1821),
    [anon_sym_AMP_AMP] = ACTIONS(1821),
    [anon_sym_PIPE_PIPE] = ACTIONS(1821),
    [anon_sym_GT_GT] = ACTIONS(1823),
    [anon_sym_GT_GT_GT] = ACTIONS(1821),
    [anon_sym_LT_LT] = ACTIONS(1821),
    [anon_sym_AMP] = ACTIONS(1823),
    [anon_sym_CARET] = ACTIONS(1821),
    [anon_sym_PIPE] = ACTIONS(1823),
    [anon_sym_PLUS] = ACTIONS(1823),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_PERCENT] = ACTIONS(1821),
    [anon_sym_LT_EQ] = ACTIONS(1821),
    [anon_sym_EQ_EQ] = ACTIONS(1823),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1821),
    [anon_sym_BANG_EQ] = ACTIONS(1823),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1821),
    [anon_sym_GT_EQ] = ACTIONS(1821),
    [anon_sym_instanceof] = ACTIONS(1821),
    [anon_sym_PLUS_PLUS] = ACTIONS(1821),
    [anon_sym_DASH_DASH] = ACTIONS(1821),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1821),
  },
  [453] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [454] = {
    [sym_parenthesized_expression] = STATE(901),
    [sym__expression] = STATE(901),
    [sym_yield_expression] = STATE(901),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(901),
    [sym_jsx_fragment] = STATE(901),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(901),
    [sym_anonymous_class] = STATE(901),
    [sym_class] = STATE(901),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(901),
    [sym_generator_function] = STATE(901),
    [sym_call_expression] = STATE(901),
    [sym_new_expression] = STATE(901),
    [sym_await_expression] = STATE(901),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(901),
    [sym_augmented_assignment_expression] = STATE(901),
    [sym_ternary_expression] = STATE(901),
    [sym_binary_expression] = STATE(901),
    [sym_unary_expression] = STATE(901),
    [sym_update_expression] = STATE(901),
    [sym_sequence_expression] = STATE(902),
    [sym_template_string] = STATE(901),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1829),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1831),
    [sym_number] = ACTIONS(1829),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1831),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1831),
    [sym_false] = ACTIONS(1831),
    [sym_null] = ACTIONS(1831),
    [sym_undefined] = ACTIONS(1831),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [455] = {
    [sym_parenthesized_expression] = STATE(903),
    [sym__expression] = STATE(903),
    [sym_yield_expression] = STATE(903),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(903),
    [sym_jsx_fragment] = STATE(903),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(903),
    [sym_anonymous_class] = STATE(903),
    [sym_class] = STATE(903),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(903),
    [sym_generator_function] = STATE(903),
    [sym_call_expression] = STATE(903),
    [sym_new_expression] = STATE(903),
    [sym_await_expression] = STATE(903),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(903),
    [sym_augmented_assignment_expression] = STATE(903),
    [sym_ternary_expression] = STATE(903),
    [sym_binary_expression] = STATE(903),
    [sym_unary_expression] = STATE(903),
    [sym_update_expression] = STATE(903),
    [sym_template_string] = STATE(903),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1833),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1835),
    [sym_number] = ACTIONS(1833),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(1835),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1835),
    [sym_false] = ACTIONS(1835),
    [sym_null] = ACTIONS(1835),
    [sym_undefined] = ACTIONS(1835),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [456] = {
    [sym_parenthesized_expression] = STATE(904),
    [sym__expression] = STATE(904),
    [sym_yield_expression] = STATE(904),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(904),
    [sym_jsx_fragment] = STATE(904),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(904),
    [sym_anonymous_class] = STATE(904),
    [sym_class] = STATE(904),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(904),
    [sym_generator_function] = STATE(904),
    [sym_call_expression] = STATE(904),
    [sym_new_expression] = STATE(904),
    [sym_await_expression] = STATE(904),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(904),
    [sym_augmented_assignment_expression] = STATE(904),
    [sym_ternary_expression] = STATE(904),
    [sym_binary_expression] = STATE(904),
    [sym_unary_expression] = STATE(904),
    [sym_update_expression] = STATE(904),
    [sym_template_string] = STATE(904),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1837),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1839),
    [sym_number] = ACTIONS(1837),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1839),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1839),
    [sym_false] = ACTIONS(1839),
    [sym_null] = ACTIONS(1839),
    [sym_undefined] = ACTIONS(1839),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [457] = {
    [sym_parenthesized_expression] = STATE(905),
    [sym__expression] = STATE(905),
    [sym_yield_expression] = STATE(905),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(905),
    [sym_jsx_fragment] = STATE(905),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(905),
    [sym_anonymous_class] = STATE(905),
    [sym_class] = STATE(905),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(905),
    [sym_generator_function] = STATE(905),
    [sym_call_expression] = STATE(905),
    [sym_new_expression] = STATE(905),
    [sym_await_expression] = STATE(905),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(905),
    [sym_augmented_assignment_expression] = STATE(905),
    [sym_ternary_expression] = STATE(905),
    [sym_binary_expression] = STATE(905),
    [sym_unary_expression] = STATE(905),
    [sym_update_expression] = STATE(905),
    [sym_template_string] = STATE(905),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1841),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1843),
    [sym_number] = ACTIONS(1841),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1843),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1843),
    [sym_false] = ACTIONS(1843),
    [sym_null] = ACTIONS(1843),
    [sym_undefined] = ACTIONS(1843),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [458] = {
    [sym_parenthesized_expression] = STATE(906),
    [sym__expression] = STATE(906),
    [sym_yield_expression] = STATE(906),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(906),
    [sym_jsx_fragment] = STATE(906),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(906),
    [sym_anonymous_class] = STATE(906),
    [sym_class] = STATE(906),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(906),
    [sym_generator_function] = STATE(906),
    [sym_call_expression] = STATE(906),
    [sym_new_expression] = STATE(906),
    [sym_await_expression] = STATE(906),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(906),
    [sym_augmented_assignment_expression] = STATE(906),
    [sym_ternary_expression] = STATE(906),
    [sym_binary_expression] = STATE(906),
    [sym_unary_expression] = STATE(906),
    [sym_update_expression] = STATE(906),
    [sym_template_string] = STATE(906),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1845),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1847),
    [sym_number] = ACTIONS(1845),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(1847),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1847),
    [sym_false] = ACTIONS(1847),
    [sym_null] = ACTIONS(1847),
    [sym_undefined] = ACTIONS(1847),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [459] = {
    [anon_sym_STAR] = ACTIONS(1062),
    [anon_sym_LBRACE] = ACTIONS(1062),
    [anon_sym_COMMA] = ACTIONS(1062),
    [anon_sym_RBRACE] = ACTIONS(1062),
    [anon_sym_LPAREN] = ACTIONS(1062),
    [anon_sym_RPAREN] = ACTIONS(1062),
    [anon_sym_in] = ACTIONS(1064),
    [anon_sym_COLON] = ACTIONS(1062),
    [anon_sym_LBRACK] = ACTIONS(1062),
    [anon_sym_RBRACK] = ACTIONS(1062),
    [anon_sym_LT] = ACTIONS(1064),
    [anon_sym_GT] = ACTIONS(1064),
    [anon_sym_SLASH] = ACTIONS(1064),
    [anon_sym_DOT] = ACTIONS(1062),
    [anon_sym_QMARK] = ACTIONS(1062),
    [anon_sym_AMP_AMP] = ACTIONS(1062),
    [anon_sym_PIPE_PIPE] = ACTIONS(1062),
    [anon_sym_GT_GT] = ACTIONS(1064),
    [anon_sym_GT_GT_GT] = ACTIONS(1062),
    [anon_sym_LT_LT] = ACTIONS(1062),
    [anon_sym_AMP] = ACTIONS(1064),
    [anon_sym_CARET] = ACTIONS(1062),
    [anon_sym_PIPE] = ACTIONS(1064),
    [anon_sym_PLUS] = ACTIONS(1064),
    [anon_sym_DASH] = ACTIONS(1064),
    [anon_sym_PERCENT] = ACTIONS(1062),
    [anon_sym_LT_EQ] = ACTIONS(1062),
    [anon_sym_EQ_EQ] = ACTIONS(1064),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1062),
    [anon_sym_BANG_EQ] = ACTIONS(1064),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1062),
    [anon_sym_GT_EQ] = ACTIONS(1062),
    [anon_sym_instanceof] = ACTIONS(1062),
    [anon_sym_PLUS_PLUS] = ACTIONS(1062),
    [anon_sym_DASH_DASH] = ACTIONS(1062),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1062),
  },
  [460] = {
    [anon_sym_LBRACE] = ACTIONS(1849),
    [anon_sym_EQ_GT] = ACTIONS(1849),
    [sym_comment] = ACTIONS(71),
  },
  [461] = {
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_COMMA] = ACTIONS(1566),
    [anon_sym_LPAREN] = ACTIONS(1170),
    [anon_sym_RPAREN] = ACTIONS(1566),
    [anon_sym_in] = ACTIONS(1172),
    [anon_sym_LBRACK] = ACTIONS(1170),
    [anon_sym_LT] = ACTIONS(1172),
    [anon_sym_GT] = ACTIONS(1172),
    [anon_sym_SLASH] = ACTIONS(1172),
    [anon_sym_DOT] = ACTIONS(1170),
    [anon_sym_QMARK] = ACTIONS(1170),
    [anon_sym_AMP_AMP] = ACTIONS(1170),
    [anon_sym_PIPE_PIPE] = ACTIONS(1170),
    [anon_sym_GT_GT] = ACTIONS(1172),
    [anon_sym_GT_GT_GT] = ACTIONS(1170),
    [anon_sym_LT_LT] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1172),
    [anon_sym_CARET] = ACTIONS(1170),
    [anon_sym_PIPE] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PERCENT] = ACTIONS(1170),
    [anon_sym_LT_EQ] = ACTIONS(1170),
    [anon_sym_EQ_EQ] = ACTIONS(1172),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1170),
    [anon_sym_BANG_EQ] = ACTIONS(1172),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1170),
    [anon_sym_GT_EQ] = ACTIONS(1170),
    [anon_sym_instanceof] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1170),
  },
  [462] = {
    [aux_sym_formal_parameters_repeat1] = STATE(880),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(1851),
    [sym_comment] = ACTIONS(71),
  },
  [463] = {
    [anon_sym_LBRACE] = ACTIONS(1853),
    [anon_sym_EQ_GT] = ACTIONS(1853),
    [sym_comment] = ACTIONS(71),
  },
  [464] = {
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_COMMA] = ACTIONS(1577),
    [anon_sym_LPAREN] = ACTIONS(1174),
    [anon_sym_RPAREN] = ACTIONS(1577),
    [anon_sym_in] = ACTIONS(1176),
    [anon_sym_LBRACK] = ACTIONS(1174),
    [anon_sym_LT] = ACTIONS(1176),
    [anon_sym_GT] = ACTIONS(1176),
    [anon_sym_SLASH] = ACTIONS(1176),
    [anon_sym_DOT] = ACTIONS(1174),
    [anon_sym_QMARK] = ACTIONS(1174),
    [anon_sym_AMP_AMP] = ACTIONS(1174),
    [anon_sym_PIPE_PIPE] = ACTIONS(1174),
    [anon_sym_GT_GT] = ACTIONS(1176),
    [anon_sym_GT_GT_GT] = ACTIONS(1174),
    [anon_sym_LT_LT] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1176),
    [anon_sym_CARET] = ACTIONS(1174),
    [anon_sym_PIPE] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PERCENT] = ACTIONS(1174),
    [anon_sym_LT_EQ] = ACTIONS(1174),
    [anon_sym_EQ_EQ] = ACTIONS(1176),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1174),
    [anon_sym_BANG_EQ] = ACTIONS(1176),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1174),
    [anon_sym_GT_EQ] = ACTIONS(1174),
    [anon_sym_instanceof] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1174),
  },
  [465] = {
    [aux_sym_formal_parameters_repeat1] = STATE(880),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(1855),
    [sym_comment] = ACTIONS(71),
  },
  [466] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(1857),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [467] = {
    [anon_sym_STAR] = ACTIONS(1196),
    [anon_sym_LBRACE] = ACTIONS(1196),
    [anon_sym_COMMA] = ACTIONS(1196),
    [anon_sym_RBRACE] = ACTIONS(1196),
    [anon_sym_LPAREN] = ACTIONS(1196),
    [anon_sym_RPAREN] = ACTIONS(1196),
    [anon_sym_in] = ACTIONS(1198),
    [anon_sym_COLON] = ACTIONS(1196),
    [anon_sym_LBRACK] = ACTIONS(1196),
    [anon_sym_RBRACK] = ACTIONS(1196),
    [anon_sym_LT] = ACTIONS(1198),
    [anon_sym_GT] = ACTIONS(1198),
    [anon_sym_SLASH] = ACTIONS(1198),
    [anon_sym_DOT] = ACTIONS(1196),
    [anon_sym_QMARK] = ACTIONS(1196),
    [anon_sym_AMP_AMP] = ACTIONS(1196),
    [anon_sym_PIPE_PIPE] = ACTIONS(1196),
    [anon_sym_GT_GT] = ACTIONS(1198),
    [anon_sym_GT_GT_GT] = ACTIONS(1196),
    [anon_sym_LT_LT] = ACTIONS(1196),
    [anon_sym_AMP] = ACTIONS(1198),
    [anon_sym_CARET] = ACTIONS(1196),
    [anon_sym_PIPE] = ACTIONS(1198),
    [anon_sym_PLUS] = ACTIONS(1198),
    [anon_sym_DASH] = ACTIONS(1198),
    [anon_sym_PERCENT] = ACTIONS(1196),
    [anon_sym_LT_EQ] = ACTIONS(1196),
    [anon_sym_EQ_EQ] = ACTIONS(1198),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1196),
    [anon_sym_BANG_EQ] = ACTIONS(1198),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1196),
    [anon_sym_GT_EQ] = ACTIONS(1196),
    [anon_sym_instanceof] = ACTIONS(1196),
    [anon_sym_PLUS_PLUS] = ACTIONS(1196),
    [anon_sym_DASH_DASH] = ACTIONS(1196),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1196),
  },
  [468] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(910),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(808),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [469] = {
    [anon_sym_STAR] = ACTIONS(1088),
    [anon_sym_LBRACE] = ACTIONS(1088),
    [anon_sym_COMMA] = ACTIONS(1088),
    [anon_sym_RBRACE] = ACTIONS(1088),
    [anon_sym_LPAREN] = ACTIONS(1088),
    [anon_sym_RPAREN] = ACTIONS(1088),
    [anon_sym_in] = ACTIONS(1090),
    [anon_sym_COLON] = ACTIONS(1088),
    [anon_sym_LBRACK] = ACTIONS(1088),
    [anon_sym_RBRACK] = ACTIONS(1088),
    [anon_sym_LT] = ACTIONS(1090),
    [anon_sym_GT] = ACTIONS(1090),
    [anon_sym_SLASH] = ACTIONS(1090),
    [anon_sym_DOT] = ACTIONS(1088),
    [anon_sym_QMARK] = ACTIONS(1088),
    [anon_sym_AMP_AMP] = ACTIONS(1088),
    [anon_sym_PIPE_PIPE] = ACTIONS(1088),
    [anon_sym_GT_GT] = ACTIONS(1090),
    [anon_sym_GT_GT_GT] = ACTIONS(1088),
    [anon_sym_LT_LT] = ACTIONS(1088),
    [anon_sym_AMP] = ACTIONS(1090),
    [anon_sym_CARET] = ACTIONS(1088),
    [anon_sym_PIPE] = ACTIONS(1090),
    [anon_sym_PLUS] = ACTIONS(1090),
    [anon_sym_DASH] = ACTIONS(1090),
    [anon_sym_PERCENT] = ACTIONS(1088),
    [anon_sym_LT_EQ] = ACTIONS(1088),
    [anon_sym_EQ_EQ] = ACTIONS(1090),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1088),
    [anon_sym_BANG_EQ] = ACTIONS(1090),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1088),
    [anon_sym_GT_EQ] = ACTIONS(1088),
    [anon_sym_instanceof] = ACTIONS(1088),
    [anon_sym_PLUS_PLUS] = ACTIONS(1088),
    [anon_sym_DASH_DASH] = ACTIONS(1088),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1088),
  },
  [470] = {
    [sym_class_heritage] = STATE(912),
    [sym_class_body] = STATE(865),
    [anon_sym_LBRACE] = ACTIONS(704),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1859),
  },
  [471] = {
    [ts_builtin_sym_end] = ACTIONS(1861),
    [anon_sym_export] = ACTIONS(1863),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_RBRACE] = ACTIONS(1861),
    [anon_sym_import] = ACTIONS(1863),
    [anon_sym_var] = ACTIONS(1863),
    [anon_sym_let] = ACTIONS(1863),
    [anon_sym_const] = ACTIONS(1863),
    [anon_sym_if] = ACTIONS(1863),
    [anon_sym_switch] = ACTIONS(1863),
    [anon_sym_for] = ACTIONS(1863),
    [anon_sym_LPAREN] = ACTIONS(1861),
    [anon_sym_while] = ACTIONS(1863),
    [anon_sym_do] = ACTIONS(1863),
    [anon_sym_try] = ACTIONS(1863),
    [anon_sym_with] = ACTIONS(1863),
    [anon_sym_break] = ACTIONS(1863),
    [anon_sym_continue] = ACTIONS(1863),
    [anon_sym_debugger] = ACTIONS(1863),
    [anon_sym_return] = ACTIONS(1863),
    [anon_sym_throw] = ACTIONS(1863),
    [anon_sym_SEMI] = ACTIONS(1861),
    [anon_sym_yield] = ACTIONS(1863),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LT] = ACTIONS(1861),
    [anon_sym_class] = ACTIONS(1863),
    [anon_sym_async] = ACTIONS(1863),
    [anon_sym_function] = ACTIONS(1863),
    [anon_sym_EQ_GT] = ACTIONS(1861),
    [anon_sym_new] = ACTIONS(1863),
    [anon_sym_await] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_typeof] = ACTIONS(1863),
    [anon_sym_void] = ACTIONS(1863),
    [anon_sym_delete] = ACTIONS(1863),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1861),
    [anon_sym_BQUOTE] = ACTIONS(1861),
    [sym_regex] = ACTIONS(1863),
    [sym_number] = ACTIONS(1861),
    [sym_identifier] = ACTIONS(1863),
    [sym_this] = ACTIONS(1863),
    [sym_super] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_null] = ACTIONS(1863),
    [sym_undefined] = ACTIONS(1863),
    [anon_sym_AT] = ACTIONS(1861),
    [anon_sym_get] = ACTIONS(1863),
    [anon_sym_set] = ACTIONS(1863),
  },
  [472] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1865),
  },
  [473] = {
    [anon_sym_function] = ACTIONS(1867),
    [sym_comment] = ACTIONS(71),
  },
  [474] = {
    [sym_formal_parameters] = STATE(917),
    [anon_sym_STAR] = ACTIONS(1869),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1871),
  },
  [475] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(1873),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [476] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_else] = ACTIONS(482),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_while] = ACTIONS(482),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(480),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(480),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(480),
  },
  [477] = {
    [sym_export_statement] = STATE(938),
    [sym__declaration] = STATE(938),
    [sym_import_statement] = STATE(938),
    [sym_expression_statement] = STATE(938),
    [sym_variable_declaration] = STATE(938),
    [sym_lexical_declaration] = STATE(938),
    [sym_statement_block] = STATE(938),
    [sym_if_statement] = STATE(938),
    [sym_switch_statement] = STATE(938),
    [sym_for_statement] = STATE(938),
    [sym_for_in_statement] = STATE(938),
    [sym_for_of_statement] = STATE(938),
    [sym_while_statement] = STATE(938),
    [sym_do_statement] = STATE(938),
    [sym_try_statement] = STATE(938),
    [sym_with_statement] = STATE(938),
    [sym_break_statement] = STATE(938),
    [sym_continue_statement] = STATE(938),
    [sym_debugger_statement] = STATE(938),
    [sym_return_statement] = STATE(938),
    [sym_throw_statement] = STATE(938),
    [sym_empty_statement] = STATE(938),
    [sym_labeled_statement] = STATE(938),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [478] = {
    [sym_expression_statement] = STATE(946),
    [sym_variable_declaration] = STATE(946),
    [sym_lexical_declaration] = STATE(946),
    [sym_empty_statement] = STATE(946),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(947),
    [sym_array] = STATE(948),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(1913),
    [anon_sym_let] = ACTIONS(1915),
    [anon_sym_const] = ACTIONS(1915),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1917),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [479] = {
    [sym_export_statement] = STATE(471),
    [sym__declaration] = STATE(471),
    [sym_import_statement] = STATE(471),
    [sym_expression_statement] = STATE(471),
    [sym_variable_declaration] = STATE(471),
    [sym_lexical_declaration] = STATE(471),
    [sym_statement_block] = STATE(471),
    [sym_if_statement] = STATE(471),
    [sym_switch_statement] = STATE(471),
    [sym_for_statement] = STATE(471),
    [sym_for_in_statement] = STATE(471),
    [sym_for_of_statement] = STATE(471),
    [sym_while_statement] = STATE(471),
    [sym_do_statement] = STATE(471),
    [sym_try_statement] = STATE(471),
    [sym_with_statement] = STATE(471),
    [sym_break_statement] = STATE(471),
    [sym_continue_statement] = STATE(471),
    [sym_debugger_statement] = STATE(471),
    [sym_return_statement] = STATE(471),
    [sym_throw_statement] = STATE(471),
    [sym_empty_statement] = STATE(471),
    [sym_labeled_statement] = STATE(471),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [480] = {
    [sym_catch_clause] = STATE(951),
    [sym_finally_clause] = STATE(496),
    [anon_sym_while] = ACTIONS(842),
    [anon_sym_catch] = ACTIONS(1919),
    [anon_sym_finally] = ACTIONS(1921),
    [sym_comment] = ACTIONS(71),
  },
  [481] = {
    [sym_export_statement] = STATE(497),
    [sym__declaration] = STATE(497),
    [sym_import_statement] = STATE(497),
    [sym_expression_statement] = STATE(497),
    [sym_variable_declaration] = STATE(497),
    [sym_lexical_declaration] = STATE(497),
    [sym_statement_block] = STATE(497),
    [sym_if_statement] = STATE(497),
    [sym_switch_statement] = STATE(497),
    [sym_for_statement] = STATE(497),
    [sym_for_in_statement] = STATE(497),
    [sym_for_of_statement] = STATE(497),
    [sym_while_statement] = STATE(497),
    [sym_do_statement] = STATE(497),
    [sym_try_statement] = STATE(497),
    [sym_with_statement] = STATE(497),
    [sym_break_statement] = STATE(497),
    [sym_continue_statement] = STATE(497),
    [sym_debugger_statement] = STATE(497),
    [sym_return_statement] = STATE(497),
    [sym_throw_statement] = STATE(497),
    [sym_empty_statement] = STATE(497),
    [sym_labeled_statement] = STATE(497),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [482] = {
    [sym_class_heritage] = STATE(952),
    [sym_class_body] = STATE(953),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [483] = {
    [sym_formal_parameters] = STATE(955),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1923),
  },
  [484] = {
    [sym_formal_parameters] = STATE(957),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1925),
  },
  [485] = {
    [sym_formal_parameters] = STATE(955),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [486] = {
    [sym_statement_block] = STATE(959),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [sym_comment] = ACTIONS(71),
  },
  [487] = {
    [sym_export_statement] = STATE(598),
    [sym__declaration] = STATE(598),
    [sym_import_statement] = STATE(598),
    [sym_expression_statement] = STATE(598),
    [sym_variable_declaration] = STATE(598),
    [sym_lexical_declaration] = STATE(598),
    [sym_statement_block] = STATE(598),
    [sym_if_statement] = STATE(598),
    [sym_switch_statement] = STATE(598),
    [sym_for_statement] = STATE(598),
    [sym_for_in_statement] = STATE(598),
    [sym_for_of_statement] = STATE(598),
    [sym_while_statement] = STATE(598),
    [sym_do_statement] = STATE(598),
    [sym_try_statement] = STATE(598),
    [sym_with_statement] = STATE(598),
    [sym_break_statement] = STATE(598),
    [sym_continue_statement] = STATE(598),
    [sym_debugger_statement] = STATE(598),
    [sym_return_statement] = STATE(598),
    [sym_throw_statement] = STATE(598),
    [sym_empty_statement] = STATE(598),
    [sym_labeled_statement] = STATE(598),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [488] = {
    [sym_parenthesized_expression] = STATE(961),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [sym_comment] = ACTIONS(71),
  },
  [489] = {
    [sym__declaration] = STATE(267),
    [sym_variable_declaration] = STATE(267),
    [sym_lexical_declaration] = STATE(267),
    [sym_class] = STATE(59),
    [sym_function] = STATE(59),
    [sym_generator_function] = STATE(59),
    [sym_decorator] = STATE(475),
    [aux_sym_export_statement_repeat1] = STATE(475),
    [anon_sym_default] = ACTIONS(1322),
    [anon_sym_var] = ACTIONS(95),
    [anon_sym_let] = ACTIONS(97),
    [anon_sym_const] = ACTIONS(97),
    [anon_sym_class] = ACTIONS(814),
    [anon_sym_async] = ACTIONS(816),
    [anon_sym_function] = ACTIONS(818),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [490] = {
    [sym_class_heritage] = STATE(649),
    [sym_class_body] = STATE(574),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1931),
  },
  [491] = {
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_catch] = ACTIONS(484),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [492] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(1933),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [493] = {
    [sym_statement_block] = STATE(966),
    [anon_sym_LBRACE] = ACTIONS(1935),
    [anon_sym_LPAREN] = ACTIONS(1937),
    [sym_comment] = ACTIONS(71),
  },
  [494] = {
    [sym_statement_block] = STATE(968),
    [anon_sym_LBRACE] = ACTIONS(1939),
    [sym_comment] = ACTIONS(71),
  },
  [495] = {
    [sym_finally_clause] = STATE(969),
    [ts_builtin_sym_end] = ACTIONS(1941),
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_finally] = ACTIONS(848),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [496] = {
    [ts_builtin_sym_end] = ACTIONS(1941),
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [497] = {
    [ts_builtin_sym_end] = ACTIONS(1945),
    [anon_sym_export] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1945),
    [anon_sym_RBRACE] = ACTIONS(1945),
    [anon_sym_import] = ACTIONS(1947),
    [anon_sym_var] = ACTIONS(1947),
    [anon_sym_let] = ACTIONS(1947),
    [anon_sym_const] = ACTIONS(1947),
    [anon_sym_if] = ACTIONS(1947),
    [anon_sym_switch] = ACTIONS(1947),
    [anon_sym_for] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1945),
    [anon_sym_while] = ACTIONS(1947),
    [anon_sym_do] = ACTIONS(1947),
    [anon_sym_try] = ACTIONS(1947),
    [anon_sym_with] = ACTIONS(1947),
    [anon_sym_break] = ACTIONS(1947),
    [anon_sym_continue] = ACTIONS(1947),
    [anon_sym_debugger] = ACTIONS(1947),
    [anon_sym_return] = ACTIONS(1947),
    [anon_sym_throw] = ACTIONS(1947),
    [anon_sym_SEMI] = ACTIONS(1945),
    [anon_sym_yield] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1945),
    [anon_sym_LT] = ACTIONS(1945),
    [anon_sym_class] = ACTIONS(1947),
    [anon_sym_async] = ACTIONS(1947),
    [anon_sym_function] = ACTIONS(1947),
    [anon_sym_EQ_GT] = ACTIONS(1945),
    [anon_sym_new] = ACTIONS(1947),
    [anon_sym_await] = ACTIONS(1947),
    [anon_sym_PLUS] = ACTIONS(1947),
    [anon_sym_DASH] = ACTIONS(1947),
    [anon_sym_BANG] = ACTIONS(1945),
    [anon_sym_TILDE] = ACTIONS(1945),
    [anon_sym_typeof] = ACTIONS(1947),
    [anon_sym_void] = ACTIONS(1947),
    [anon_sym_delete] = ACTIONS(1947),
    [anon_sym_PLUS_PLUS] = ACTIONS(1945),
    [anon_sym_DASH_DASH] = ACTIONS(1945),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1945),
    [anon_sym_BQUOTE] = ACTIONS(1945),
    [sym_regex] = ACTIONS(1947),
    [sym_number] = ACTIONS(1945),
    [sym_identifier] = ACTIONS(1947),
    [sym_this] = ACTIONS(1947),
    [sym_super] = ACTIONS(1947),
    [sym_true] = ACTIONS(1947),
    [sym_false] = ACTIONS(1947),
    [sym_null] = ACTIONS(1947),
    [sym_undefined] = ACTIONS(1947),
    [anon_sym_AT] = ACTIONS(1945),
    [anon_sym_get] = ACTIONS(1947),
    [anon_sym_set] = ACTIONS(1947),
  },
  [498] = {
    [ts_builtin_sym_end] = ACTIONS(1949),
    [anon_sym_export] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1949),
    [anon_sym_RBRACE] = ACTIONS(1949),
    [anon_sym_import] = ACTIONS(1951),
    [anon_sym_var] = ACTIONS(1951),
    [anon_sym_let] = ACTIONS(1951),
    [anon_sym_const] = ACTIONS(1951),
    [anon_sym_if] = ACTIONS(1951),
    [anon_sym_switch] = ACTIONS(1951),
    [anon_sym_for] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1949),
    [anon_sym_while] = ACTIONS(1951),
    [anon_sym_do] = ACTIONS(1951),
    [anon_sym_try] = ACTIONS(1951),
    [anon_sym_with] = ACTIONS(1951),
    [anon_sym_break] = ACTIONS(1951),
    [anon_sym_continue] = ACTIONS(1951),
    [anon_sym_debugger] = ACTIONS(1951),
    [anon_sym_return] = ACTIONS(1951),
    [anon_sym_throw] = ACTIONS(1951),
    [anon_sym_SEMI] = ACTIONS(1949),
    [anon_sym_yield] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1949),
    [anon_sym_LT] = ACTIONS(1949),
    [anon_sym_class] = ACTIONS(1951),
    [anon_sym_async] = ACTIONS(1951),
    [anon_sym_function] = ACTIONS(1951),
    [anon_sym_EQ_GT] = ACTIONS(1949),
    [anon_sym_new] = ACTIONS(1951),
    [anon_sym_await] = ACTIONS(1951),
    [anon_sym_PLUS] = ACTIONS(1951),
    [anon_sym_DASH] = ACTIONS(1951),
    [anon_sym_BANG] = ACTIONS(1949),
    [anon_sym_TILDE] = ACTIONS(1949),
    [anon_sym_typeof] = ACTIONS(1951),
    [anon_sym_void] = ACTIONS(1951),
    [anon_sym_delete] = ACTIONS(1951),
    [anon_sym_PLUS_PLUS] = ACTIONS(1949),
    [anon_sym_DASH_DASH] = ACTIONS(1949),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1949),
    [anon_sym_BQUOTE] = ACTIONS(1949),
    [sym_regex] = ACTIONS(1951),
    [sym_number] = ACTIONS(1949),
    [sym_identifier] = ACTIONS(1951),
    [sym_this] = ACTIONS(1951),
    [sym_super] = ACTIONS(1951),
    [sym_true] = ACTIONS(1951),
    [sym_false] = ACTIONS(1951),
    [sym_null] = ACTIONS(1951),
    [sym_undefined] = ACTIONS(1951),
    [anon_sym_AT] = ACTIONS(1949),
    [anon_sym_get] = ACTIONS(1951),
    [anon_sym_set] = ACTIONS(1951),
  },
  [499] = {
    [ts_builtin_sym_end] = ACTIONS(1953),
    [anon_sym_export] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1953),
    [anon_sym_RBRACE] = ACTIONS(1953),
    [anon_sym_import] = ACTIONS(1955),
    [anon_sym_var] = ACTIONS(1955),
    [anon_sym_let] = ACTIONS(1955),
    [anon_sym_const] = ACTIONS(1955),
    [anon_sym_if] = ACTIONS(1955),
    [anon_sym_switch] = ACTIONS(1955),
    [anon_sym_for] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1953),
    [anon_sym_while] = ACTIONS(1955),
    [anon_sym_do] = ACTIONS(1955),
    [anon_sym_try] = ACTIONS(1955),
    [anon_sym_with] = ACTIONS(1955),
    [anon_sym_break] = ACTIONS(1955),
    [anon_sym_continue] = ACTIONS(1955),
    [anon_sym_debugger] = ACTIONS(1955),
    [anon_sym_return] = ACTIONS(1955),
    [anon_sym_throw] = ACTIONS(1955),
    [anon_sym_SEMI] = ACTIONS(1953),
    [anon_sym_yield] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1953),
    [anon_sym_LT] = ACTIONS(1953),
    [anon_sym_class] = ACTIONS(1955),
    [anon_sym_async] = ACTIONS(1955),
    [anon_sym_function] = ACTIONS(1955),
    [anon_sym_EQ_GT] = ACTIONS(1953),
    [anon_sym_new] = ACTIONS(1955),
    [anon_sym_await] = ACTIONS(1955),
    [anon_sym_PLUS] = ACTIONS(1955),
    [anon_sym_DASH] = ACTIONS(1955),
    [anon_sym_BANG] = ACTIONS(1953),
    [anon_sym_TILDE] = ACTIONS(1953),
    [anon_sym_typeof] = ACTIONS(1955),
    [anon_sym_void] = ACTIONS(1955),
    [anon_sym_delete] = ACTIONS(1955),
    [anon_sym_PLUS_PLUS] = ACTIONS(1953),
    [anon_sym_DASH_DASH] = ACTIONS(1953),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1953),
    [anon_sym_BQUOTE] = ACTIONS(1953),
    [sym_regex] = ACTIONS(1955),
    [sym_number] = ACTIONS(1953),
    [sym_identifier] = ACTIONS(1955),
    [sym_this] = ACTIONS(1955),
    [sym_super] = ACTIONS(1955),
    [sym_true] = ACTIONS(1955),
    [sym_false] = ACTIONS(1955),
    [sym_null] = ACTIONS(1955),
    [sym_undefined] = ACTIONS(1955),
    [anon_sym_AT] = ACTIONS(1953),
    [anon_sym_get] = ACTIONS(1955),
    [anon_sym_set] = ACTIONS(1955),
  },
  [500] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RBRACE] = ACTIONS(480),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_RPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_of] = ACTIONS(480),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(480),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(480),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(480),
  },
  [501] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(292),
    [anon_sym_STAR] = ACTIONS(1441),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [502] = {
    [sym__initializer] = STATE(708),
    [aux_sym_object_repeat1] = STATE(292),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [503] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(292),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(497),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [504] = {
    [sym_decorator] = STATE(716),
    [sym_method_definition] = STATE(325),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(716),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [505] = {
    [ts_builtin_sym_end] = ACTIONS(1957),
    [anon_sym_export] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1957),
    [anon_sym_RBRACE] = ACTIONS(1957),
    [anon_sym_import] = ACTIONS(1959),
    [anon_sym_var] = ACTIONS(1959),
    [anon_sym_let] = ACTIONS(1959),
    [anon_sym_const] = ACTIONS(1959),
    [anon_sym_if] = ACTIONS(1959),
    [anon_sym_switch] = ACTIONS(1959),
    [anon_sym_for] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1957),
    [anon_sym_while] = ACTIONS(1959),
    [anon_sym_do] = ACTIONS(1959),
    [anon_sym_try] = ACTIONS(1959),
    [anon_sym_with] = ACTIONS(1959),
    [anon_sym_break] = ACTIONS(1959),
    [anon_sym_continue] = ACTIONS(1959),
    [anon_sym_debugger] = ACTIONS(1959),
    [anon_sym_return] = ACTIONS(1959),
    [anon_sym_throw] = ACTIONS(1959),
    [anon_sym_SEMI] = ACTIONS(1957),
    [anon_sym_yield] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1957),
    [anon_sym_LT] = ACTIONS(1957),
    [anon_sym_class] = ACTIONS(1959),
    [anon_sym_async] = ACTIONS(1959),
    [anon_sym_function] = ACTIONS(1959),
    [anon_sym_EQ_GT] = ACTIONS(1957),
    [anon_sym_new] = ACTIONS(1959),
    [anon_sym_await] = ACTIONS(1959),
    [anon_sym_PLUS] = ACTIONS(1959),
    [anon_sym_DASH] = ACTIONS(1959),
    [anon_sym_BANG] = ACTIONS(1957),
    [anon_sym_TILDE] = ACTIONS(1957),
    [anon_sym_typeof] = ACTIONS(1959),
    [anon_sym_void] = ACTIONS(1959),
    [anon_sym_delete] = ACTIONS(1959),
    [anon_sym_PLUS_PLUS] = ACTIONS(1957),
    [anon_sym_DASH_DASH] = ACTIONS(1957),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1957),
    [anon_sym_BQUOTE] = ACTIONS(1957),
    [sym_regex] = ACTIONS(1959),
    [sym_number] = ACTIONS(1957),
    [sym_identifier] = ACTIONS(1959),
    [sym_this] = ACTIONS(1959),
    [sym_super] = ACTIONS(1959),
    [sym_true] = ACTIONS(1959),
    [sym_false] = ACTIONS(1959),
    [sym_null] = ACTIONS(1959),
    [sym_undefined] = ACTIONS(1959),
    [anon_sym_AT] = ACTIONS(1957),
    [anon_sym_get] = ACTIONS(1959),
    [anon_sym_set] = ACTIONS(1959),
  },
  [506] = {
    [ts_builtin_sym_end] = ACTIONS(1961),
    [anon_sym_export] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1961),
    [anon_sym_RBRACE] = ACTIONS(1961),
    [anon_sym_import] = ACTIONS(1963),
    [anon_sym_var] = ACTIONS(1963),
    [anon_sym_let] = ACTIONS(1963),
    [anon_sym_const] = ACTIONS(1963),
    [anon_sym_if] = ACTIONS(1963),
    [anon_sym_switch] = ACTIONS(1963),
    [anon_sym_for] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1961),
    [anon_sym_while] = ACTIONS(1963),
    [anon_sym_do] = ACTIONS(1963),
    [anon_sym_try] = ACTIONS(1963),
    [anon_sym_with] = ACTIONS(1963),
    [anon_sym_break] = ACTIONS(1963),
    [anon_sym_continue] = ACTIONS(1963),
    [anon_sym_debugger] = ACTIONS(1963),
    [anon_sym_return] = ACTIONS(1963),
    [anon_sym_throw] = ACTIONS(1963),
    [anon_sym_SEMI] = ACTIONS(1961),
    [anon_sym_yield] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1961),
    [anon_sym_LT] = ACTIONS(1961),
    [anon_sym_class] = ACTIONS(1963),
    [anon_sym_async] = ACTIONS(1963),
    [anon_sym_function] = ACTIONS(1963),
    [anon_sym_EQ_GT] = ACTIONS(1961),
    [anon_sym_new] = ACTIONS(1963),
    [anon_sym_await] = ACTIONS(1963),
    [anon_sym_PLUS] = ACTIONS(1963),
    [anon_sym_DASH] = ACTIONS(1963),
    [anon_sym_BANG] = ACTIONS(1961),
    [anon_sym_TILDE] = ACTIONS(1961),
    [anon_sym_typeof] = ACTIONS(1963),
    [anon_sym_void] = ACTIONS(1963),
    [anon_sym_delete] = ACTIONS(1963),
    [anon_sym_PLUS_PLUS] = ACTIONS(1961),
    [anon_sym_DASH_DASH] = ACTIONS(1961),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1961),
    [anon_sym_BQUOTE] = ACTIONS(1961),
    [sym_regex] = ACTIONS(1963),
    [sym_number] = ACTIONS(1961),
    [sym_identifier] = ACTIONS(1963),
    [sym_this] = ACTIONS(1963),
    [sym_super] = ACTIONS(1963),
    [sym_true] = ACTIONS(1963),
    [sym_false] = ACTIONS(1963),
    [sym_null] = ACTIONS(1963),
    [sym_undefined] = ACTIONS(1963),
    [anon_sym_AT] = ACTIONS(1961),
    [anon_sym_get] = ACTIONS(1963),
    [anon_sym_set] = ACTIONS(1963),
  },
  [507] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(1965),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1965),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [508] = {
    [anon_sym_COMMA] = ACTIONS(1965),
    [anon_sym_RPAREN] = ACTIONS(1965),
    [anon_sym_RBRACK] = ACTIONS(1965),
    [sym_comment] = ACTIONS(71),
  },
  [509] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(882),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [510] = {
    [sym_parenthesized_expression] = STATE(970),
    [sym__expression] = STATE(970),
    [sym_yield_expression] = STATE(970),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(970),
    [sym_jsx_fragment] = STATE(970),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(970),
    [sym_anonymous_class] = STATE(970),
    [sym_class] = STATE(970),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(970),
    [sym_generator_function] = STATE(970),
    [sym_call_expression] = STATE(970),
    [sym_new_expression] = STATE(970),
    [sym_await_expression] = STATE(970),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(970),
    [sym_augmented_assignment_expression] = STATE(970),
    [sym_ternary_expression] = STATE(970),
    [sym_binary_expression] = STATE(970),
    [sym_unary_expression] = STATE(970),
    [sym_update_expression] = STATE(970),
    [sym_template_string] = STATE(970),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1967),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1969),
    [sym_number] = ACTIONS(1967),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1969),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1969),
    [sym_false] = ACTIONS(1969),
    [sym_null] = ACTIONS(1969),
    [sym_undefined] = ACTIONS(1969),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [511] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(971),
    [sym__expression] = STATE(971),
    [sym_yield_expression] = STATE(971),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(971),
    [sym_jsx_fragment] = STATE(971),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(971),
    [sym_anonymous_class] = STATE(971),
    [sym_class] = STATE(971),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(971),
    [sym_generator_function] = STATE(971),
    [sym_call_expression] = STATE(971),
    [sym_new_expression] = STATE(971),
    [sym_await_expression] = STATE(971),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(971),
    [sym_augmented_assignment_expression] = STATE(971),
    [sym_ternary_expression] = STATE(971),
    [sym_binary_expression] = STATE(971),
    [sym_unary_expression] = STATE(971),
    [sym_update_expression] = STATE(971),
    [sym_template_string] = STATE(971),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1971),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1973),
    [sym_number] = ACTIONS(1971),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1973),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1973),
    [sym_false] = ACTIONS(1973),
    [sym_null] = ACTIONS(1973),
    [sym_undefined] = ACTIONS(1973),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [512] = {
    [sym_parenthesized_expression] = STATE(972),
    [sym__expression] = STATE(972),
    [sym_yield_expression] = STATE(972),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(972),
    [sym_jsx_fragment] = STATE(972),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(972),
    [sym_anonymous_class] = STATE(972),
    [sym_class] = STATE(972),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(972),
    [sym_generator_function] = STATE(972),
    [sym_call_expression] = STATE(972),
    [sym_new_expression] = STATE(972),
    [sym_await_expression] = STATE(972),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(972),
    [sym_augmented_assignment_expression] = STATE(972),
    [sym_ternary_expression] = STATE(972),
    [sym_binary_expression] = STATE(972),
    [sym_unary_expression] = STATE(972),
    [sym_update_expression] = STATE(972),
    [sym_template_string] = STATE(972),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1975),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1977),
    [sym_number] = ACTIONS(1975),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1977),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1977),
    [sym_false] = ACTIONS(1977),
    [sym_null] = ACTIONS(1977),
    [sym_undefined] = ACTIONS(1977),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [513] = {
    [anon_sym_EQ_GT] = ACTIONS(1979),
    [sym_comment] = ACTIONS(71),
  },
  [514] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1048),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [515] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1056),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [516] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1505),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [517] = {
    [sym_parenthesized_expression] = STATE(974),
    [sym__expression] = STATE(974),
    [sym_yield_expression] = STATE(974),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(974),
    [sym_jsx_fragment] = STATE(974),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(974),
    [sym_anonymous_class] = STATE(974),
    [sym_class] = STATE(974),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(974),
    [sym_generator_function] = STATE(974),
    [sym_call_expression] = STATE(974),
    [sym_new_expression] = STATE(974),
    [sym_await_expression] = STATE(974),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(974),
    [sym_augmented_assignment_expression] = STATE(974),
    [sym_ternary_expression] = STATE(974),
    [sym_binary_expression] = STATE(974),
    [sym_unary_expression] = STATE(974),
    [sym_update_expression] = STATE(974),
    [sym_template_string] = STATE(974),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1981),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1983),
    [sym_number] = ACTIONS(1981),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1983),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1983),
    [sym_false] = ACTIONS(1983),
    [sym_null] = ACTIONS(1983),
    [sym_undefined] = ACTIONS(1983),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [518] = {
    [sym_parenthesized_expression] = STATE(975),
    [sym__expression] = STATE(975),
    [sym_yield_expression] = STATE(975),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(975),
    [sym_jsx_fragment] = STATE(975),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(975),
    [sym_anonymous_class] = STATE(975),
    [sym_class] = STATE(975),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(975),
    [sym_generator_function] = STATE(975),
    [sym_call_expression] = STATE(975),
    [sym_new_expression] = STATE(975),
    [sym_await_expression] = STATE(975),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(975),
    [sym_augmented_assignment_expression] = STATE(975),
    [sym_ternary_expression] = STATE(975),
    [sym_binary_expression] = STATE(975),
    [sym_unary_expression] = STATE(975),
    [sym_update_expression] = STATE(975),
    [sym_template_string] = STATE(975),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1985),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1987),
    [sym_number] = ACTIONS(1985),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1987),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1987),
    [sym_false] = ACTIONS(1987),
    [sym_null] = ACTIONS(1987),
    [sym_undefined] = ACTIONS(1987),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [519] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [520] = {
    [sym__automatic_semicolon] = ACTIONS(1989),
    [anon_sym_STAR] = ACTIONS(1989),
    [anon_sym_COMMA] = ACTIONS(1989),
    [anon_sym_RBRACE] = ACTIONS(1989),
    [anon_sym_LPAREN] = ACTIONS(1989),
    [anon_sym_RPAREN] = ACTIONS(1989),
    [anon_sym_in] = ACTIONS(1991),
    [anon_sym_of] = ACTIONS(1989),
    [anon_sym_SEMI] = ACTIONS(1989),
    [anon_sym_LBRACK] = ACTIONS(1989),
    [anon_sym_LT] = ACTIONS(1991),
    [anon_sym_GT] = ACTIONS(1991),
    [anon_sym_SLASH] = ACTIONS(1991),
    [anon_sym_DOT] = ACTIONS(1989),
    [anon_sym_EQ] = ACTIONS(1991),
    [anon_sym_QMARK] = ACTIONS(1989),
    [anon_sym_AMP_AMP] = ACTIONS(1989),
    [anon_sym_PIPE_PIPE] = ACTIONS(1989),
    [anon_sym_GT_GT] = ACTIONS(1991),
    [anon_sym_GT_GT_GT] = ACTIONS(1989),
    [anon_sym_LT_LT] = ACTIONS(1989),
    [anon_sym_AMP] = ACTIONS(1991),
    [anon_sym_CARET] = ACTIONS(1989),
    [anon_sym_PIPE] = ACTIONS(1991),
    [anon_sym_PLUS] = ACTIONS(1991),
    [anon_sym_DASH] = ACTIONS(1991),
    [anon_sym_PERCENT] = ACTIONS(1989),
    [anon_sym_LT_EQ] = ACTIONS(1989),
    [anon_sym_EQ_EQ] = ACTIONS(1991),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1989),
    [anon_sym_BANG_EQ] = ACTIONS(1991),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1989),
    [anon_sym_GT_EQ] = ACTIONS(1989),
    [anon_sym_instanceof] = ACTIONS(1989),
    [anon_sym_PLUS_PLUS] = ACTIONS(1989),
    [anon_sym_DASH_DASH] = ACTIONS(1989),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1989),
  },
  [521] = {
    [sym_parenthesized_expression] = STATE(976),
    [sym__expression] = STATE(976),
    [sym_yield_expression] = STATE(976),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(976),
    [sym_jsx_fragment] = STATE(976),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(976),
    [sym_anonymous_class] = STATE(976),
    [sym_class] = STATE(976),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(976),
    [sym_generator_function] = STATE(976),
    [sym_call_expression] = STATE(976),
    [sym_new_expression] = STATE(976),
    [sym_await_expression] = STATE(976),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(976),
    [sym_augmented_assignment_expression] = STATE(976),
    [sym_ternary_expression] = STATE(976),
    [sym_binary_expression] = STATE(976),
    [sym_unary_expression] = STATE(976),
    [sym_update_expression] = STATE(976),
    [sym_template_string] = STATE(976),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1993),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1995),
    [sym_number] = ACTIONS(1993),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(1995),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1995),
    [sym_false] = ACTIONS(1995),
    [sym_null] = ACTIONS(1995),
    [sym_undefined] = ACTIONS(1995),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [522] = {
    [sym_parenthesized_expression] = STATE(977),
    [sym__expression] = STATE(977),
    [sym_yield_expression] = STATE(977),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(977),
    [sym_jsx_fragment] = STATE(977),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(977),
    [sym_anonymous_class] = STATE(977),
    [sym_class] = STATE(977),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(977),
    [sym_generator_function] = STATE(977),
    [sym_call_expression] = STATE(977),
    [sym_new_expression] = STATE(977),
    [sym_await_expression] = STATE(977),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(977),
    [sym_augmented_assignment_expression] = STATE(977),
    [sym_ternary_expression] = STATE(977),
    [sym_binary_expression] = STATE(977),
    [sym_unary_expression] = STATE(977),
    [sym_update_expression] = STATE(977),
    [sym_template_string] = STATE(977),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1997),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1999),
    [sym_number] = ACTIONS(1997),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(1999),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1999),
    [sym_false] = ACTIONS(1999),
    [sym_null] = ACTIONS(1999),
    [sym_undefined] = ACTIONS(1999),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [523] = {
    [sym_parenthesized_expression] = STATE(978),
    [sym__expression] = STATE(978),
    [sym_yield_expression] = STATE(978),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(978),
    [sym_jsx_fragment] = STATE(978),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(978),
    [sym_anonymous_class] = STATE(978),
    [sym_class] = STATE(978),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(978),
    [sym_generator_function] = STATE(978),
    [sym_call_expression] = STATE(978),
    [sym_new_expression] = STATE(978),
    [sym_await_expression] = STATE(978),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(978),
    [sym_augmented_assignment_expression] = STATE(978),
    [sym_ternary_expression] = STATE(978),
    [sym_binary_expression] = STATE(978),
    [sym_unary_expression] = STATE(978),
    [sym_update_expression] = STATE(978),
    [sym_template_string] = STATE(978),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2001),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2003),
    [sym_number] = ACTIONS(2001),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(2003),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2003),
    [sym_false] = ACTIONS(2003),
    [sym_null] = ACTIONS(2003),
    [sym_undefined] = ACTIONS(2003),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [524] = {
    [sym_parenthesized_expression] = STATE(979),
    [sym__expression] = STATE(979),
    [sym_yield_expression] = STATE(979),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(979),
    [sym_jsx_fragment] = STATE(979),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(979),
    [sym_anonymous_class] = STATE(979),
    [sym_class] = STATE(979),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(979),
    [sym_generator_function] = STATE(979),
    [sym_call_expression] = STATE(979),
    [sym_new_expression] = STATE(979),
    [sym_await_expression] = STATE(979),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(979),
    [sym_augmented_assignment_expression] = STATE(979),
    [sym_ternary_expression] = STATE(979),
    [sym_binary_expression] = STATE(979),
    [sym_unary_expression] = STATE(979),
    [sym_update_expression] = STATE(979),
    [sym_template_string] = STATE(979),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2005),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2007),
    [sym_number] = ACTIONS(2005),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(2007),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2007),
    [sym_false] = ACTIONS(2007),
    [sym_null] = ACTIONS(2007),
    [sym_undefined] = ACTIONS(2007),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [525] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(2009),
    [sym_comment] = ACTIONS(71),
  },
  [526] = {
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_COMMA] = ACTIONS(1170),
    [anon_sym_RBRACE] = ACTIONS(1170),
    [anon_sym_LPAREN] = ACTIONS(1170),
    [anon_sym_RPAREN] = ACTIONS(1170),
    [anon_sym_in] = ACTIONS(1172),
    [anon_sym_COLON] = ACTIONS(1170),
    [anon_sym_LBRACK] = ACTIONS(1170),
    [anon_sym_RBRACK] = ACTIONS(1170),
    [anon_sym_LT] = ACTIONS(1172),
    [anon_sym_GT] = ACTIONS(1172),
    [anon_sym_SLASH] = ACTIONS(1172),
    [anon_sym_DOT] = ACTIONS(1170),
    [anon_sym_QMARK] = ACTIONS(1170),
    [anon_sym_AMP_AMP] = ACTIONS(1170),
    [anon_sym_PIPE_PIPE] = ACTIONS(1170),
    [anon_sym_GT_GT] = ACTIONS(1172),
    [anon_sym_GT_GT_GT] = ACTIONS(1170),
    [anon_sym_LT_LT] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1172),
    [anon_sym_CARET] = ACTIONS(1170),
    [anon_sym_PIPE] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PERCENT] = ACTIONS(1170),
    [anon_sym_LT_EQ] = ACTIONS(1170),
    [anon_sym_EQ_EQ] = ACTIONS(1172),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1170),
    [anon_sym_BANG_EQ] = ACTIONS(1172),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1170),
    [anon_sym_GT_EQ] = ACTIONS(1170),
    [anon_sym_instanceof] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1170),
  },
  [527] = {
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_LBRACE] = ACTIONS(1174),
    [anon_sym_COMMA] = ACTIONS(1174),
    [anon_sym_RBRACE] = ACTIONS(1174),
    [anon_sym_LPAREN] = ACTIONS(1174),
    [anon_sym_RPAREN] = ACTIONS(1174),
    [anon_sym_in] = ACTIONS(1176),
    [anon_sym_COLON] = ACTIONS(1174),
    [anon_sym_LBRACK] = ACTIONS(1174),
    [anon_sym_RBRACK] = ACTIONS(1174),
    [anon_sym_LT] = ACTIONS(1176),
    [anon_sym_GT] = ACTIONS(1176),
    [anon_sym_SLASH] = ACTIONS(1176),
    [anon_sym_DOT] = ACTIONS(1174),
    [anon_sym_QMARK] = ACTIONS(1174),
    [anon_sym_AMP_AMP] = ACTIONS(1174),
    [anon_sym_PIPE_PIPE] = ACTIONS(1174),
    [anon_sym_GT_GT] = ACTIONS(1176),
    [anon_sym_GT_GT_GT] = ACTIONS(1174),
    [anon_sym_LT_LT] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1176),
    [anon_sym_CARET] = ACTIONS(1174),
    [anon_sym_PIPE] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PERCENT] = ACTIONS(1174),
    [anon_sym_LT_EQ] = ACTIONS(1174),
    [anon_sym_EQ_EQ] = ACTIONS(1176),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1174),
    [anon_sym_BANG_EQ] = ACTIONS(1176),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1174),
    [anon_sym_GT_EQ] = ACTIONS(1174),
    [anon_sym_instanceof] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1174),
  },
  [528] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(2011),
    [anon_sym_RBRACK] = ACTIONS(1965),
    [sym_comment] = ACTIONS(71),
  },
  [529] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(2014),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [530] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(2016),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [531] = {
    [sym_parenthesized_expression] = STATE(984),
    [sym__expression] = STATE(984),
    [sym_yield_expression] = STATE(984),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(984),
    [sym_jsx_fragment] = STATE(984),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(984),
    [sym_anonymous_class] = STATE(984),
    [sym_class] = STATE(984),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(984),
    [sym_generator_function] = STATE(984),
    [sym_call_expression] = STATE(984),
    [sym_new_expression] = STATE(984),
    [sym_await_expression] = STATE(984),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(984),
    [sym_augmented_assignment_expression] = STATE(984),
    [sym_spread_element] = STATE(985),
    [sym_ternary_expression] = STATE(984),
    [sym_binary_expression] = STATE(984),
    [sym_unary_expression] = STATE(984),
    [sym_update_expression] = STATE(984),
    [sym_sequence_expression] = STATE(985),
    [sym_template_string] = STATE(984),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_RBRACE] = ACTIONS(2018),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1180),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2020),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2022),
    [sym_number] = ACTIONS(2020),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2022),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2022),
    [sym_false] = ACTIONS(2022),
    [sym_null] = ACTIONS(2022),
    [sym_undefined] = ACTIONS(2022),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [532] = {
    [sym_jsx_identifier] = ACTIONS(2024),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2026),
  },
  [533] = {
    [anon_sym_LBRACE] = ACTIONS(2028),
    [anon_sym_LT] = ACTIONS(2028),
    [sym_jsx_text] = ACTIONS(2028),
    [sym_comment] = ACTIONS(71),
  },
  [534] = {
    [anon_sym_GT] = ACTIONS(2030),
    [sym_comment] = ACTIONS(71),
  },
  [535] = {
    [anon_sym_LBRACE] = ACTIONS(2032),
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(2032),
    [anon_sym_SLASH] = ACTIONS(2034),
    [sym_jsx_identifier] = ACTIONS(2032),
    [anon_sym_EQ] = ACTIONS(2036),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2034),
  },
  [536] = {
    [anon_sym_LBRACE] = ACTIONS(2032),
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(2032),
    [anon_sym_SLASH] = ACTIONS(2034),
    [sym_jsx_identifier] = ACTIONS(2032),
    [anon_sym_EQ] = ACTIONS(2036),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2034),
  },
  [537] = {
    [anon_sym_LBRACE] = ACTIONS(2038),
    [anon_sym_GT] = ACTIONS(2038),
    [anon_sym_SLASH] = ACTIONS(2040),
    [sym_jsx_identifier] = ACTIONS(2038),
    [anon_sym_EQ] = ACTIONS(2042),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2040),
  },
  [538] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2044),
    [anon_sym_SLASH] = ACTIONS(2046),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [539] = {
    [sym_jsx_identifier] = ACTIONS(2048),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2050),
  },
  [540] = {
    [anon_sym_LBRACE] = ACTIONS(2052),
    [anon_sym_LT] = ACTIONS(2052),
    [sym_jsx_text] = ACTIONS(2052),
    [sym_comment] = ACTIONS(71),
  },
  [541] = {
    [anon_sym_GT] = ACTIONS(2054),
    [sym_comment] = ACTIONS(71),
  },
  [542] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2056),
  },
  [543] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2058),
    [anon_sym_SLASH] = ACTIONS(2060),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [544] = {
    [sym__property_name] = STATE(1000),
    [sym_computed_property_name] = STATE(1000),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2062),
    [sym_number] = ACTIONS(2062),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [545] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [anon_sym_STAR] = ACTIONS(2064),
    [anon_sym_COMMA] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_else] = ACTIONS(2064),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_RPAREN] = ACTIONS(2064),
    [anon_sym_in] = ACTIONS(2066),
    [anon_sym_while] = ACTIONS(2064),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2066),
    [anon_sym_GT] = ACTIONS(2066),
    [anon_sym_SLASH] = ACTIONS(2066),
    [anon_sym_DOT] = ACTIONS(2064),
    [anon_sym_QMARK] = ACTIONS(2064),
    [anon_sym_AMP_AMP] = ACTIONS(2064),
    [anon_sym_PIPE_PIPE] = ACTIONS(2064),
    [anon_sym_GT_GT] = ACTIONS(2066),
    [anon_sym_GT_GT_GT] = ACTIONS(2064),
    [anon_sym_LT_LT] = ACTIONS(2064),
    [anon_sym_AMP] = ACTIONS(2066),
    [anon_sym_CARET] = ACTIONS(2064),
    [anon_sym_PIPE] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_PERCENT] = ACTIONS(2064),
    [anon_sym_LT_EQ] = ACTIONS(2064),
    [anon_sym_EQ_EQ] = ACTIONS(2066),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2064),
    [anon_sym_BANG_EQ] = ACTIONS(2066),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2064),
    [anon_sym_GT_EQ] = ACTIONS(2064),
    [anon_sym_instanceof] = ACTIONS(2064),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2064),
  },
  [546] = {
    [sym_parenthesized_expression] = STATE(1001),
    [sym__expression] = STATE(1001),
    [sym_yield_expression] = STATE(1001),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1001),
    [sym_jsx_fragment] = STATE(1001),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1001),
    [sym_anonymous_class] = STATE(1001),
    [sym_class] = STATE(1001),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1001),
    [sym_generator_function] = STATE(1001),
    [sym_call_expression] = STATE(1001),
    [sym_new_expression] = STATE(1001),
    [sym_await_expression] = STATE(1001),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1001),
    [sym_augmented_assignment_expression] = STATE(1001),
    [sym_ternary_expression] = STATE(1001),
    [sym_binary_expression] = STATE(1001),
    [sym_unary_expression] = STATE(1001),
    [sym_update_expression] = STATE(1001),
    [sym_template_string] = STATE(1001),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2068),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2070),
    [sym_number] = ACTIONS(2068),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(2070),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2070),
    [sym_false] = ACTIONS(2070),
    [sym_null] = ACTIONS(2070),
    [sym_undefined] = ACTIONS(2070),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [547] = {
    [sym__property_name] = STATE(1000),
    [sym_computed_property_name] = STATE(1000),
    [sym__automatic_semicolon] = ACTIONS(503),
    [anon_sym_STAR] = ACTIONS(2072),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_SEMI] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_EQ] = ACTIONS(503),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2062),
    [sym_number] = ACTIONS(2062),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(2074),
    [anon_sym_set] = ACTIONS(2074),
  },
  [548] = {
    [sym__automatic_semicolon] = ACTIONS(503),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_SEMI] = ACTIONS(503),
    [anon_sym_EQ] = ACTIONS(503),
    [sym_comment] = ACTIONS(71),
  },
  [549] = {
    [sym__identifier_reference] = STATE(1005),
    [sym_decorator_member_expression] = STATE(1006),
    [sym_decorator_call_expression] = STATE(1007),
    [anon_sym_async] = ACTIONS(2076),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2078),
    [anon_sym_get] = ACTIONS(2076),
    [anon_sym_set] = ACTIONS(2076),
  },
  [550] = {
    [sym_public_field_definition] = STATE(1008),
    [sym_method_definition] = STATE(1009),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [551] = {
    [sym__property_name] = STATE(1000),
    [sym_computed_property_name] = STATE(1000),
    [sym__automatic_semicolon] = ACTIONS(503),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_SEMI] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_EQ] = ACTIONS(503),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2062),
    [sym_number] = ACTIONS(2062),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [552] = {
    [sym__automatic_semicolon] = ACTIONS(2080),
    [anon_sym_SEMI] = ACTIONS(2080),
    [sym_comment] = ACTIONS(71),
  },
  [553] = {
    [anon_sym_STAR] = ACTIONS(2082),
    [anon_sym_RBRACE] = ACTIONS(2082),
    [anon_sym_SEMI] = ACTIONS(2080),
    [anon_sym_LBRACK] = ACTIONS(2082),
    [anon_sym_async] = ACTIONS(2084),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2082),
    [sym_number] = ACTIONS(2082),
    [sym_identifier] = ACTIONS(2084),
    [anon_sym_AT] = ACTIONS(2082),
    [anon_sym_static] = ACTIONS(2084),
    [anon_sym_get] = ACTIONS(2084),
    [anon_sym_set] = ACTIONS(2084),
  },
  [554] = {
    [sym__initializer] = STATE(1011),
    [sym_formal_parameters] = STATE(1012),
    [sym__automatic_semicolon] = ACTIONS(2086),
    [anon_sym_LPAREN] = ACTIONS(312),
    [anon_sym_SEMI] = ACTIONS(2086),
    [anon_sym_EQ] = ACTIONS(2088),
    [sym_comment] = ACTIONS(71),
  },
  [555] = {
    [sym_decorator] = STATE(1014),
    [sym_public_field_definition] = STATE(1008),
    [sym_method_definition] = STATE(1009),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(1014),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(2090),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [556] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(2092),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [557] = {
    [sym_parenthesized_expression] = STATE(1017),
    [sym__expression] = STATE(1017),
    [sym_yield_expression] = STATE(1017),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1017),
    [sym_jsx_fragment] = STATE(1017),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1017),
    [sym_anonymous_class] = STATE(1017),
    [sym_class] = STATE(1017),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1017),
    [sym_generator_function] = STATE(1017),
    [sym_call_expression] = STATE(1017),
    [sym_new_expression] = STATE(1017),
    [sym_await_expression] = STATE(1017),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1017),
    [sym_augmented_assignment_expression] = STATE(1017),
    [sym_ternary_expression] = STATE(1017),
    [sym_binary_expression] = STATE(1017),
    [sym_unary_expression] = STATE(1017),
    [sym_update_expression] = STATE(1017),
    [sym_template_string] = STATE(1017),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(994),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2094),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2096),
    [sym_number] = ACTIONS(2094),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(2096),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2096),
    [sym_false] = ACTIONS(2096),
    [sym_null] = ACTIONS(2096),
    [sym_undefined] = ACTIONS(2096),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [558] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(1021),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(2098),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(2100),
    [anon_sym_PLUS_EQ] = ACTIONS(2102),
    [anon_sym_DASH_EQ] = ACTIONS(2102),
    [anon_sym_STAR_EQ] = ACTIONS(2102),
    [anon_sym_SLASH_EQ] = ACTIONS(2102),
    [anon_sym_PERCENT_EQ] = ACTIONS(2102),
    [anon_sym_CARET_EQ] = ACTIONS(2102),
    [anon_sym_AMP_EQ] = ACTIONS(2102),
    [anon_sym_PIPE_EQ] = ACTIONS(2102),
    [anon_sym_GT_GT_EQ] = ACTIONS(2102),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2102),
    [anon_sym_LT_LT_EQ] = ACTIONS(2102),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(2104),
  },
  [559] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(1022),
    [sym__expression] = STATE(1022),
    [sym_yield_expression] = STATE(1022),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1022),
    [sym_jsx_fragment] = STATE(1022),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1022),
    [sym_anonymous_class] = STATE(1022),
    [sym_class] = STATE(1022),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1022),
    [sym_generator_function] = STATE(1022),
    [sym_call_expression] = STATE(1022),
    [sym_new_expression] = STATE(1022),
    [sym_await_expression] = STATE(1022),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1022),
    [sym_augmented_assignment_expression] = STATE(1022),
    [sym_ternary_expression] = STATE(1022),
    [sym_binary_expression] = STATE(1022),
    [sym_unary_expression] = STATE(1022),
    [sym_update_expression] = STATE(1022),
    [sym_template_string] = STATE(1022),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2106),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2108),
    [sym_number] = ACTIONS(2106),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(2108),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2108),
    [sym_false] = ACTIONS(2108),
    [sym_null] = ACTIONS(2108),
    [sym_undefined] = ACTIONS(2108),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [560] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [561] = {
    [sym_parenthesized_expression] = STATE(1023),
    [sym__expression] = STATE(1023),
    [sym_yield_expression] = STATE(1023),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1023),
    [sym_jsx_fragment] = STATE(1023),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1023),
    [sym_anonymous_class] = STATE(1023),
    [sym_class] = STATE(1023),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1023),
    [sym_generator_function] = STATE(1023),
    [sym_call_expression] = STATE(1023),
    [sym_new_expression] = STATE(1023),
    [sym_await_expression] = STATE(1023),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1023),
    [sym_augmented_assignment_expression] = STATE(1023),
    [sym_ternary_expression] = STATE(1023),
    [sym_binary_expression] = STATE(1023),
    [sym_unary_expression] = STATE(1023),
    [sym_update_expression] = STATE(1023),
    [sym_template_string] = STATE(1023),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2110),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2112),
    [sym_number] = ACTIONS(2110),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(2112),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2112),
    [sym_false] = ACTIONS(2112),
    [sym_null] = ACTIONS(2112),
    [sym_undefined] = ACTIONS(2112),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [562] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [563] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [564] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(2098),
    [anon_sym_EQ_GT] = ACTIONS(2100),
    [anon_sym_PLUS_EQ] = ACTIONS(2116),
    [anon_sym_DASH_EQ] = ACTIONS(2116),
    [anon_sym_STAR_EQ] = ACTIONS(2116),
    [anon_sym_SLASH_EQ] = ACTIONS(2116),
    [anon_sym_PERCENT_EQ] = ACTIONS(2116),
    [anon_sym_CARET_EQ] = ACTIONS(2116),
    [anon_sym_AMP_EQ] = ACTIONS(2116),
    [anon_sym_PIPE_EQ] = ACTIONS(2116),
    [anon_sym_GT_GT_EQ] = ACTIONS(2116),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2116),
    [anon_sym_LT_LT_EQ] = ACTIONS(2116),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [565] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(2098),
    [anon_sym_PLUS_EQ] = ACTIONS(2102),
    [anon_sym_DASH_EQ] = ACTIONS(2102),
    [anon_sym_STAR_EQ] = ACTIONS(2102),
    [anon_sym_SLASH_EQ] = ACTIONS(2102),
    [anon_sym_PERCENT_EQ] = ACTIONS(2102),
    [anon_sym_CARET_EQ] = ACTIONS(2102),
    [anon_sym_AMP_EQ] = ACTIONS(2102),
    [anon_sym_PIPE_EQ] = ACTIONS(2102),
    [anon_sym_GT_GT_EQ] = ACTIONS(2102),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2102),
    [anon_sym_LT_LT_EQ] = ACTIONS(2102),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [566] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2120),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [567] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2098),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [568] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2098),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [569] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2098),
    [anon_sym_PLUS_EQ] = ACTIONS(2116),
    [anon_sym_DASH_EQ] = ACTIONS(2116),
    [anon_sym_STAR_EQ] = ACTIONS(2116),
    [anon_sym_SLASH_EQ] = ACTIONS(2116),
    [anon_sym_PERCENT_EQ] = ACTIONS(2116),
    [anon_sym_CARET_EQ] = ACTIONS(2116),
    [anon_sym_AMP_EQ] = ACTIONS(2116),
    [anon_sym_PIPE_EQ] = ACTIONS(2116),
    [anon_sym_GT_GT_EQ] = ACTIONS(2116),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2116),
    [anon_sym_LT_LT_EQ] = ACTIONS(2116),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [570] = {
    [anon_sym_EQ_GT] = ACTIONS(2100),
    [sym_comment] = ACTIONS(71),
  },
  [571] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1032),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(2138),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [572] = {
    [sym_class_body] = STATE(1033),
    [anon_sym_LBRACE] = ACTIONS(1008),
    [sym_comment] = ACTIONS(71),
  },
  [573] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [ts_builtin_sym_end] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_STAR] = ACTIONS(2140),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_COMMA] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_in] = ACTIONS(2142),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2142),
    [anon_sym_GT] = ACTIONS(2142),
    [anon_sym_SLASH] = ACTIONS(2142),
    [anon_sym_DOT] = ACTIONS(2142),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_QMARK] = ACTIONS(2140),
    [anon_sym_AMP_AMP] = ACTIONS(2140),
    [anon_sym_PIPE_PIPE] = ACTIONS(2140),
    [anon_sym_GT_GT] = ACTIONS(2142),
    [anon_sym_GT_GT_GT] = ACTIONS(2140),
    [anon_sym_LT_LT] = ACTIONS(2140),
    [anon_sym_AMP] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(2140),
    [anon_sym_PIPE] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_PERCENT] = ACTIONS(2140),
    [anon_sym_LT_EQ] = ACTIONS(2140),
    [anon_sym_EQ_EQ] = ACTIONS(2142),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2140),
    [anon_sym_BANG_EQ] = ACTIONS(2142),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2140),
    [anon_sym_GT_EQ] = ACTIONS(2140),
    [anon_sym_instanceof] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2142),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [574] = {
    [sym__automatic_semicolon] = ACTIONS(2144),
    [anon_sym_STAR] = ACTIONS(2144),
    [anon_sym_COMMA] = ACTIONS(2144),
    [anon_sym_RBRACE] = ACTIONS(2144),
    [anon_sym_LPAREN] = ACTIONS(2144),
    [anon_sym_RPAREN] = ACTIONS(2144),
    [anon_sym_in] = ACTIONS(2146),
    [anon_sym_SEMI] = ACTIONS(2144),
    [anon_sym_LBRACK] = ACTIONS(2144),
    [anon_sym_LT] = ACTIONS(2146),
    [anon_sym_GT] = ACTIONS(2146),
    [anon_sym_SLASH] = ACTIONS(2146),
    [anon_sym_DOT] = ACTIONS(2144),
    [anon_sym_QMARK] = ACTIONS(2144),
    [anon_sym_AMP_AMP] = ACTIONS(2144),
    [anon_sym_PIPE_PIPE] = ACTIONS(2144),
    [anon_sym_GT_GT] = ACTIONS(2146),
    [anon_sym_GT_GT_GT] = ACTIONS(2144),
    [anon_sym_LT_LT] = ACTIONS(2144),
    [anon_sym_AMP] = ACTIONS(2146),
    [anon_sym_CARET] = ACTIONS(2144),
    [anon_sym_PIPE] = ACTIONS(2146),
    [anon_sym_PLUS] = ACTIONS(2146),
    [anon_sym_DASH] = ACTIONS(2146),
    [anon_sym_PERCENT] = ACTIONS(2144),
    [anon_sym_LT_EQ] = ACTIONS(2144),
    [anon_sym_EQ_EQ] = ACTIONS(2146),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2144),
    [anon_sym_BANG_EQ] = ACTIONS(2146),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2144),
    [anon_sym_GT_EQ] = ACTIONS(2144),
    [anon_sym_instanceof] = ACTIONS(2144),
    [anon_sym_PLUS_PLUS] = ACTIONS(2144),
    [anon_sym_DASH_DASH] = ACTIONS(2144),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2144),
  },
  [575] = {
    [sym_parenthesized_expression] = STATE(1034),
    [sym__expression] = STATE(1034),
    [sym_yield_expression] = STATE(1034),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(1034),
    [sym_jsx_fragment] = STATE(1034),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1034),
    [sym_anonymous_class] = STATE(1034),
    [sym_class] = STATE(1034),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1034),
    [sym_generator_function] = STATE(1034),
    [sym_call_expression] = STATE(1034),
    [sym_new_expression] = STATE(1034),
    [sym_await_expression] = STATE(1034),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(1034),
    [sym_augmented_assignment_expression] = STATE(1034),
    [sym_spread_element] = STATE(705),
    [sym_ternary_expression] = STATE(1034),
    [sym_binary_expression] = STATE(1034),
    [sym_unary_expression] = STATE(1034),
    [sym_update_expression] = STATE(1034),
    [sym_template_string] = STATE(1034),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(706),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(1435),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(263),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2148),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2150),
    [sym_number] = ACTIONS(2148),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(2150),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2150),
    [sym_false] = ACTIONS(2150),
    [sym_null] = ACTIONS(2150),
    [sym_undefined] = ACTIONS(2150),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [576] = {
    [aux_sym_formal_parameters_repeat1] = STATE(429),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(2152),
    [sym_comment] = ACTIONS(71),
  },
  [577] = {
    [sym__initializer] = STATE(708),
    [aux_sym_formal_parameters_repeat1] = STATE(448),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(803),
    [anon_sym_EQ] = ACTIONS(2154),
    [sym_comment] = ACTIONS(71),
  },
  [578] = {
    [sym__initializer] = STATE(713),
    [aux_sym_formal_parameters_repeat1] = STATE(462),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(2156),
    [anon_sym_EQ] = ACTIONS(2154),
    [sym_comment] = ACTIONS(71),
  },
  [579] = {
    [sym__initializer] = STATE(714),
    [aux_sym_formal_parameters_repeat1] = STATE(465),
    [anon_sym_COMMA] = ACTIONS(801),
    [anon_sym_RPAREN] = ACTIONS(2158),
    [anon_sym_EQ] = ACTIONS(2154),
    [sym_comment] = ACTIONS(71),
  },
  [580] = {
    [sym__automatic_semicolon] = ACTIONS(2160),
    [anon_sym_STAR] = ACTIONS(2162),
    [anon_sym_COMMA] = ACTIONS(2160),
    [anon_sym_RBRACE] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(2160),
    [anon_sym_RPAREN] = ACTIONS(2160),
    [anon_sym_in] = ACTIONS(2162),
    [anon_sym_SEMI] = ACTIONS(2160),
    [anon_sym_LBRACK] = ACTIONS(2160),
    [anon_sym_LT] = ACTIONS(2162),
    [anon_sym_GT] = ACTIONS(2162),
    [anon_sym_SLASH] = ACTIONS(2162),
    [anon_sym_DOT] = ACTIONS(2160),
    [anon_sym_EQ] = ACTIONS(2162),
    [anon_sym_PLUS_EQ] = ACTIONS(2160),
    [anon_sym_DASH_EQ] = ACTIONS(2160),
    [anon_sym_STAR_EQ] = ACTIONS(2160),
    [anon_sym_SLASH_EQ] = ACTIONS(2160),
    [anon_sym_PERCENT_EQ] = ACTIONS(2160),
    [anon_sym_CARET_EQ] = ACTIONS(2160),
    [anon_sym_AMP_EQ] = ACTIONS(2160),
    [anon_sym_PIPE_EQ] = ACTIONS(2160),
    [anon_sym_GT_GT_EQ] = ACTIONS(2160),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2160),
    [anon_sym_LT_LT_EQ] = ACTIONS(2160),
    [anon_sym_QMARK] = ACTIONS(2160),
    [anon_sym_AMP_AMP] = ACTIONS(2160),
    [anon_sym_PIPE_PIPE] = ACTIONS(2160),
    [anon_sym_GT_GT] = ACTIONS(2162),
    [anon_sym_GT_GT_GT] = ACTIONS(2162),
    [anon_sym_LT_LT] = ACTIONS(2162),
    [anon_sym_AMP] = ACTIONS(2162),
    [anon_sym_CARET] = ACTIONS(2162),
    [anon_sym_PIPE] = ACTIONS(2162),
    [anon_sym_PLUS] = ACTIONS(2162),
    [anon_sym_DASH] = ACTIONS(2162),
    [anon_sym_PERCENT] = ACTIONS(2162),
    [anon_sym_LT_EQ] = ACTIONS(2160),
    [anon_sym_EQ_EQ] = ACTIONS(2162),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2160),
    [anon_sym_BANG_EQ] = ACTIONS(2162),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2160),
    [anon_sym_GT_EQ] = ACTIONS(2160),
    [anon_sym_instanceof] = ACTIONS(2160),
    [anon_sym_PLUS_PLUS] = ACTIONS(2160),
    [anon_sym_DASH_DASH] = ACTIONS(2160),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2160),
  },
  [581] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2164),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2164),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [582] = {
    [sym_formal_parameters] = STATE(1035),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [583] = {
    [sym_statement_block] = STATE(1036),
    [anon_sym_LBRACE] = ACTIONS(1046),
    [sym_comment] = ACTIONS(71),
  },
  [584] = {
    [sym__automatic_semicolon] = ACTIONS(2166),
    [anon_sym_STAR] = ACTIONS(2166),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_RBRACE] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(2166),
    [anon_sym_RPAREN] = ACTIONS(2166),
    [anon_sym_in] = ACTIONS(2168),
    [anon_sym_SEMI] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(2166),
    [anon_sym_LT] = ACTIONS(2168),
    [anon_sym_GT] = ACTIONS(2168),
    [anon_sym_SLASH] = ACTIONS(2168),
    [anon_sym_DOT] = ACTIONS(2166),
    [anon_sym_QMARK] = ACTIONS(2166),
    [anon_sym_AMP_AMP] = ACTIONS(2166),
    [anon_sym_PIPE_PIPE] = ACTIONS(2166),
    [anon_sym_GT_GT] = ACTIONS(2168),
    [anon_sym_GT_GT_GT] = ACTIONS(2166),
    [anon_sym_LT_LT] = ACTIONS(2166),
    [anon_sym_AMP] = ACTIONS(2168),
    [anon_sym_CARET] = ACTIONS(2166),
    [anon_sym_PIPE] = ACTIONS(2168),
    [anon_sym_PLUS] = ACTIONS(2168),
    [anon_sym_DASH] = ACTIONS(2168),
    [anon_sym_PERCENT] = ACTIONS(2166),
    [anon_sym_LT_EQ] = ACTIONS(2166),
    [anon_sym_EQ_EQ] = ACTIONS(2168),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2166),
    [anon_sym_BANG_EQ] = ACTIONS(2168),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2166),
    [anon_sym_GT_EQ] = ACTIONS(2166),
    [anon_sym_instanceof] = ACTIONS(2166),
    [anon_sym_PLUS_PLUS] = ACTIONS(2166),
    [anon_sym_DASH_DASH] = ACTIONS(2166),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2166),
  },
  [585] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2166),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [586] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2170),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [587] = {
    [sym_statement_block] = STATE(1037),
    [sym_parenthesized_expression] = STATE(1038),
    [sym__expression] = STATE(1038),
    [sym_yield_expression] = STATE(1038),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(1038),
    [sym_jsx_fragment] = STATE(1038),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1038),
    [sym_anonymous_class] = STATE(1038),
    [sym_class] = STATE(1038),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1038),
    [sym_generator_function] = STATE(1038),
    [sym_call_expression] = STATE(1038),
    [sym_new_expression] = STATE(1038),
    [sym_await_expression] = STATE(1038),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(1038),
    [sym_augmented_assignment_expression] = STATE(1038),
    [sym_ternary_expression] = STATE(1038),
    [sym_binary_expression] = STATE(1038),
    [sym_unary_expression] = STATE(1038),
    [sym_update_expression] = STATE(1038),
    [sym_template_string] = STATE(1038),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2172),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2174),
    [sym_number] = ACTIONS(2172),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(2174),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2174),
    [sym_false] = ACTIONS(2174),
    [sym_null] = ACTIONS(2174),
    [sym_undefined] = ACTIONS(2174),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [588] = {
    [sym_formal_parameters] = STATE(1039),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [589] = {
    [sym_statement_block] = STATE(1040),
    [anon_sym_LBRACE] = ACTIONS(1046),
    [sym_comment] = ACTIONS(71),
  },
  [590] = {
    [sym_export_statement] = STATE(1042),
    [sym__declaration] = STATE(1042),
    [sym_import_statement] = STATE(1042),
    [sym_expression_statement] = STATE(1042),
    [sym_variable_declaration] = STATE(1042),
    [sym_lexical_declaration] = STATE(1042),
    [sym_statement_block] = STATE(1042),
    [sym_if_statement] = STATE(1042),
    [sym_switch_statement] = STATE(1042),
    [sym_for_statement] = STATE(1042),
    [sym_for_in_statement] = STATE(1042),
    [sym_for_of_statement] = STATE(1042),
    [sym_while_statement] = STATE(1042),
    [sym_do_statement] = STATE(1042),
    [sym_try_statement] = STATE(1042),
    [sym_with_statement] = STATE(1042),
    [sym_break_statement] = STATE(1042),
    [sym_continue_statement] = STATE(1042),
    [sym_debugger_statement] = STATE(1042),
    [sym_return_statement] = STATE(1042),
    [sym_throw_statement] = STATE(1042),
    [sym_empty_statement] = STATE(1042),
    [sym_labeled_statement] = STATE(1042),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1042),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2176),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [591] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [ts_builtin_sym_end] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_COMMA] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_in] = ACTIONS(2180),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2180),
    [anon_sym_GT] = ACTIONS(2180),
    [anon_sym_SLASH] = ACTIONS(2180),
    [anon_sym_DOT] = ACTIONS(2180),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_QMARK] = ACTIONS(2178),
    [anon_sym_AMP_AMP] = ACTIONS(2178),
    [anon_sym_PIPE_PIPE] = ACTIONS(2178),
    [anon_sym_GT_GT] = ACTIONS(2180),
    [anon_sym_GT_GT_GT] = ACTIONS(2178),
    [anon_sym_LT_LT] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2180),
    [anon_sym_CARET] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_PERCENT] = ACTIONS(2178),
    [anon_sym_LT_EQ] = ACTIONS(2178),
    [anon_sym_EQ_EQ] = ACTIONS(2180),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2178),
    [anon_sym_BANG_EQ] = ACTIONS(2180),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2178),
    [anon_sym_GT_EQ] = ACTIONS(2178),
    [anon_sym_instanceof] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2180),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [592] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2182),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [593] = {
    [anon_sym_RBRACE] = ACTIONS(2184),
    [sym_comment] = ACTIONS(71),
  },
  [594] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2186),
    [anon_sym_RBRACE] = ACTIONS(368),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [595] = {
    [sym__automatic_semicolon] = ACTIONS(2188),
    [anon_sym_STAR] = ACTIONS(2188),
    [anon_sym_COMMA] = ACTIONS(2188),
    [anon_sym_RBRACE] = ACTIONS(2188),
    [anon_sym_LPAREN] = ACTIONS(2188),
    [anon_sym_RPAREN] = ACTIONS(2188),
    [anon_sym_in] = ACTIONS(2190),
    [anon_sym_SEMI] = ACTIONS(2188),
    [anon_sym_LBRACK] = ACTIONS(2188),
    [anon_sym_LT] = ACTIONS(2190),
    [anon_sym_GT] = ACTIONS(2190),
    [anon_sym_SLASH] = ACTIONS(2190),
    [anon_sym_DOT] = ACTIONS(2188),
    [anon_sym_QMARK] = ACTIONS(2188),
    [anon_sym_AMP_AMP] = ACTIONS(2188),
    [anon_sym_PIPE_PIPE] = ACTIONS(2188),
    [anon_sym_GT_GT] = ACTIONS(2190),
    [anon_sym_GT_GT_GT] = ACTIONS(2188),
    [anon_sym_LT_LT] = ACTIONS(2188),
    [anon_sym_AMP] = ACTIONS(2190),
    [anon_sym_CARET] = ACTIONS(2188),
    [anon_sym_PIPE] = ACTIONS(2190),
    [anon_sym_PLUS] = ACTIONS(2190),
    [anon_sym_DASH] = ACTIONS(2190),
    [anon_sym_PERCENT] = ACTIONS(2188),
    [anon_sym_LT_EQ] = ACTIONS(2188),
    [anon_sym_EQ_EQ] = ACTIONS(2190),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2188),
    [anon_sym_BANG_EQ] = ACTIONS(2190),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2188),
    [anon_sym_GT_EQ] = ACTIONS(2188),
    [anon_sym_instanceof] = ACTIONS(2188),
    [anon_sym_PLUS_PLUS] = ACTIONS(2188),
    [anon_sym_DASH_DASH] = ACTIONS(2188),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2188),
  },
  [596] = {
    [sym_template_substitution] = STATE(596),
    [aux_sym_template_string_repeat1] = STATE(596),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2192),
    [sym__template_chars] = ACTIONS(2194),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(2197),
  },
  [597] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [598] = {
    [ts_builtin_sym_end] = ACTIONS(2204),
    [anon_sym_export] = ACTIONS(2206),
    [anon_sym_LBRACE] = ACTIONS(2204),
    [anon_sym_RBRACE] = ACTIONS(2204),
    [anon_sym_import] = ACTIONS(2206),
    [anon_sym_var] = ACTIONS(2206),
    [anon_sym_let] = ACTIONS(2206),
    [anon_sym_const] = ACTIONS(2206),
    [anon_sym_if] = ACTIONS(2206),
    [anon_sym_switch] = ACTIONS(2206),
    [anon_sym_for] = ACTIONS(2206),
    [anon_sym_LPAREN] = ACTIONS(2204),
    [anon_sym_while] = ACTIONS(2206),
    [anon_sym_do] = ACTIONS(2206),
    [anon_sym_try] = ACTIONS(2206),
    [anon_sym_with] = ACTIONS(2206),
    [anon_sym_break] = ACTIONS(2206),
    [anon_sym_continue] = ACTIONS(2206),
    [anon_sym_debugger] = ACTIONS(2206),
    [anon_sym_return] = ACTIONS(2206),
    [anon_sym_throw] = ACTIONS(2206),
    [anon_sym_SEMI] = ACTIONS(2204),
    [anon_sym_yield] = ACTIONS(2206),
    [anon_sym_LBRACK] = ACTIONS(2204),
    [anon_sym_LT] = ACTIONS(2204),
    [anon_sym_class] = ACTIONS(2206),
    [anon_sym_async] = ACTIONS(2206),
    [anon_sym_function] = ACTIONS(2206),
    [anon_sym_EQ_GT] = ACTIONS(2204),
    [anon_sym_new] = ACTIONS(2206),
    [anon_sym_await] = ACTIONS(2206),
    [anon_sym_PLUS] = ACTIONS(2206),
    [anon_sym_DASH] = ACTIONS(2206),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_typeof] = ACTIONS(2206),
    [anon_sym_void] = ACTIONS(2206),
    [anon_sym_delete] = ACTIONS(2206),
    [anon_sym_PLUS_PLUS] = ACTIONS(2204),
    [anon_sym_DASH_DASH] = ACTIONS(2204),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2204),
    [anon_sym_BQUOTE] = ACTIONS(2204),
    [sym_regex] = ACTIONS(2206),
    [sym_number] = ACTIONS(2204),
    [sym_identifier] = ACTIONS(2206),
    [sym_this] = ACTIONS(2206),
    [sym_super] = ACTIONS(2206),
    [sym_true] = ACTIONS(2206),
    [sym_false] = ACTIONS(2206),
    [sym_null] = ACTIONS(2206),
    [sym_undefined] = ACTIONS(2206),
    [anon_sym_AT] = ACTIONS(2204),
    [anon_sym_get] = ACTIONS(2206),
    [anon_sym_set] = ACTIONS(2206),
  },
  [599] = {
    [sym__automatic_semicolon] = ACTIONS(2208),
    [anon_sym_STAR] = ACTIONS(2210),
    [anon_sym_COMMA] = ACTIONS(2208),
    [anon_sym_RBRACE] = ACTIONS(2208),
    [anon_sym_LPAREN] = ACTIONS(2208),
    [anon_sym_RPAREN] = ACTIONS(2208),
    [anon_sym_in] = ACTIONS(2210),
    [anon_sym_SEMI] = ACTIONS(2208),
    [anon_sym_LBRACK] = ACTIONS(2208),
    [anon_sym_LT] = ACTIONS(2210),
    [anon_sym_GT] = ACTIONS(2210),
    [anon_sym_SLASH] = ACTIONS(2210),
    [anon_sym_DOT] = ACTIONS(2208),
    [anon_sym_EQ] = ACTIONS(2210),
    [anon_sym_PLUS_EQ] = ACTIONS(2208),
    [anon_sym_DASH_EQ] = ACTIONS(2208),
    [anon_sym_STAR_EQ] = ACTIONS(2208),
    [anon_sym_SLASH_EQ] = ACTIONS(2208),
    [anon_sym_PERCENT_EQ] = ACTIONS(2208),
    [anon_sym_CARET_EQ] = ACTIONS(2208),
    [anon_sym_AMP_EQ] = ACTIONS(2208),
    [anon_sym_PIPE_EQ] = ACTIONS(2208),
    [anon_sym_GT_GT_EQ] = ACTIONS(2208),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2208),
    [anon_sym_LT_LT_EQ] = ACTIONS(2208),
    [anon_sym_QMARK] = ACTIONS(2208),
    [anon_sym_AMP_AMP] = ACTIONS(2208),
    [anon_sym_PIPE_PIPE] = ACTIONS(2208),
    [anon_sym_GT_GT] = ACTIONS(2210),
    [anon_sym_GT_GT_GT] = ACTIONS(2210),
    [anon_sym_LT_LT] = ACTIONS(2210),
    [anon_sym_AMP] = ACTIONS(2210),
    [anon_sym_CARET] = ACTIONS(2210),
    [anon_sym_PIPE] = ACTIONS(2210),
    [anon_sym_PLUS] = ACTIONS(2210),
    [anon_sym_DASH] = ACTIONS(2210),
    [anon_sym_PERCENT] = ACTIONS(2210),
    [anon_sym_LT_EQ] = ACTIONS(2208),
    [anon_sym_EQ_EQ] = ACTIONS(2210),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2208),
    [anon_sym_BANG_EQ] = ACTIONS(2210),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2208),
    [anon_sym_GT_EQ] = ACTIONS(2208),
    [anon_sym_instanceof] = ACTIONS(2208),
    [anon_sym_PLUS_PLUS] = ACTIONS(2208),
    [anon_sym_DASH_DASH] = ACTIONS(2208),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2208),
  },
  [600] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2212),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2212),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [601] = {
    [sym_parenthesized_expression] = STATE(1046),
    [sym__expression] = STATE(1046),
    [sym_yield_expression] = STATE(1046),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1046),
    [sym_jsx_fragment] = STATE(1046),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1046),
    [sym_anonymous_class] = STATE(1046),
    [sym_class] = STATE(1046),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1046),
    [sym_generator_function] = STATE(1046),
    [sym_call_expression] = STATE(1046),
    [sym_new_expression] = STATE(1046),
    [sym_await_expression] = STATE(1046),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1046),
    [sym_augmented_assignment_expression] = STATE(1046),
    [sym_spread_element] = STATE(508),
    [sym_ternary_expression] = STATE(1046),
    [sym_binary_expression] = STATE(1046),
    [sym_unary_expression] = STATE(1046),
    [sym_update_expression] = STATE(1046),
    [sym_template_string] = STATE(1046),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(884),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(884),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1096),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2214),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2216),
    [sym_number] = ACTIONS(2214),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(2216),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2216),
    [sym_false] = ACTIONS(2216),
    [sym_null] = ACTIONS(2216),
    [sym_undefined] = ACTIONS(2216),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [602] = {
    [sym__automatic_semicolon] = ACTIONS(2218),
    [anon_sym_STAR] = ACTIONS(2218),
    [anon_sym_COMMA] = ACTIONS(2218),
    [anon_sym_RBRACE] = ACTIONS(2218),
    [anon_sym_LPAREN] = ACTIONS(2218),
    [anon_sym_RPAREN] = ACTIONS(2218),
    [anon_sym_in] = ACTIONS(2220),
    [anon_sym_SEMI] = ACTIONS(2218),
    [anon_sym_LBRACK] = ACTIONS(2218),
    [anon_sym_LT] = ACTIONS(2220),
    [anon_sym_GT] = ACTIONS(2220),
    [anon_sym_SLASH] = ACTIONS(2220),
    [anon_sym_DOT] = ACTIONS(2218),
    [anon_sym_QMARK] = ACTIONS(2218),
    [anon_sym_AMP_AMP] = ACTIONS(2218),
    [anon_sym_PIPE_PIPE] = ACTIONS(2218),
    [anon_sym_GT_GT] = ACTIONS(2220),
    [anon_sym_GT_GT_GT] = ACTIONS(2218),
    [anon_sym_LT_LT] = ACTIONS(2218),
    [anon_sym_AMP] = ACTIONS(2220),
    [anon_sym_CARET] = ACTIONS(2218),
    [anon_sym_PIPE] = ACTIONS(2220),
    [anon_sym_PLUS] = ACTIONS(2220),
    [anon_sym_DASH] = ACTIONS(2220),
    [anon_sym_PERCENT] = ACTIONS(2218),
    [anon_sym_LT_EQ] = ACTIONS(2218),
    [anon_sym_EQ_EQ] = ACTIONS(2220),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2218),
    [anon_sym_BANG_EQ] = ACTIONS(2220),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2218),
    [anon_sym_GT_EQ] = ACTIONS(2218),
    [anon_sym_instanceof] = ACTIONS(2218),
    [anon_sym_PLUS_PLUS] = ACTIONS(2218),
    [anon_sym_DASH_DASH] = ACTIONS(2218),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2218),
  },
  [603] = {
    [sym_parenthesized_expression] = STATE(1047),
    [sym__expression] = STATE(1047),
    [sym_yield_expression] = STATE(1047),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1047),
    [sym_jsx_fragment] = STATE(1047),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1047),
    [sym_anonymous_class] = STATE(1047),
    [sym_class] = STATE(1047),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1047),
    [sym_generator_function] = STATE(1047),
    [sym_call_expression] = STATE(1047),
    [sym_new_expression] = STATE(1047),
    [sym_await_expression] = STATE(1047),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1047),
    [sym_augmented_assignment_expression] = STATE(1047),
    [sym_ternary_expression] = STATE(1047),
    [sym_binary_expression] = STATE(1047),
    [sym_unary_expression] = STATE(1047),
    [sym_update_expression] = STATE(1047),
    [sym_template_string] = STATE(1047),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2222),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2224),
    [sym_number] = ACTIONS(2222),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(2224),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2224),
    [sym_false] = ACTIONS(2224),
    [sym_null] = ACTIONS(2224),
    [sym_undefined] = ACTIONS(2224),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [604] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(1049),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2226),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [605] = {
    [aux_sym_array_repeat1] = STATE(1049),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(2226),
    [sym_comment] = ACTIONS(71),
  },
  [606] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(2226),
    [sym_comment] = ACTIONS(71),
  },
  [607] = {
    [anon_sym_RBRACK] = ACTIONS(2228),
    [sym_comment] = ACTIONS(71),
  },
  [608] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2230),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(368),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [609] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2232),
  },
  [610] = {
    [anon_sym_export] = ACTIONS(2234),
    [anon_sym_class] = ACTIONS(2234),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(2234),
  },
  [611] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2236),
  },
  [612] = {
    [anon_sym_export] = ACTIONS(2238),
    [anon_sym_class] = ACTIONS(2238),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(2238),
  },
  [613] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(2200),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(2202),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2202),
    [anon_sym_GT_GT_GT] = ACTIONS(2200),
    [anon_sym_LT_LT] = ACTIONS(2200),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2200),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [614] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2240),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(372),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2240),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [615] = {
    [sym__automatic_semicolon] = ACTIONS(2240),
    [anon_sym_SEMI] = ACTIONS(2240),
    [sym_comment] = ACTIONS(71),
  },
  [616] = {
    [sym_parenthesized_expression] = STATE(1055),
    [sym__expression] = STATE(1055),
    [sym_yield_expression] = STATE(1055),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1055),
    [sym_jsx_fragment] = STATE(1055),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1055),
    [sym_anonymous_class] = STATE(1055),
    [sym_class] = STATE(1055),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1055),
    [sym_generator_function] = STATE(1055),
    [sym_call_expression] = STATE(1055),
    [sym_new_expression] = STATE(1055),
    [sym_await_expression] = STATE(1055),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1055),
    [sym_augmented_assignment_expression] = STATE(1055),
    [sym_ternary_expression] = STATE(1055),
    [sym_binary_expression] = STATE(1055),
    [sym_unary_expression] = STATE(1055),
    [sym_update_expression] = STATE(1055),
    [sym_template_string] = STATE(1055),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_COLON] = ACTIONS(239),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(1144),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2242),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2244),
    [sym_number] = ACTIONS(2242),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(2244),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2244),
    [sym_false] = ACTIONS(2244),
    [sym_null] = ACTIONS(2244),
    [sym_undefined] = ACTIONS(2244),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [617] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(1059),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_COLON] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(2246),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(2248),
    [anon_sym_PLUS_EQ] = ACTIONS(2250),
    [anon_sym_DASH_EQ] = ACTIONS(2250),
    [anon_sym_STAR_EQ] = ACTIONS(2250),
    [anon_sym_SLASH_EQ] = ACTIONS(2250),
    [anon_sym_PERCENT_EQ] = ACTIONS(2250),
    [anon_sym_CARET_EQ] = ACTIONS(2250),
    [anon_sym_AMP_EQ] = ACTIONS(2250),
    [anon_sym_PIPE_EQ] = ACTIONS(2250),
    [anon_sym_GT_GT_EQ] = ACTIONS(2250),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2250),
    [anon_sym_LT_LT_EQ] = ACTIONS(2250),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(2252),
  },
  [618] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(1060),
    [sym__expression] = STATE(1060),
    [sym_yield_expression] = STATE(1060),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1060),
    [sym_jsx_fragment] = STATE(1060),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1060),
    [sym_anonymous_class] = STATE(1060),
    [sym_class] = STATE(1060),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1060),
    [sym_generator_function] = STATE(1060),
    [sym_call_expression] = STATE(1060),
    [sym_new_expression] = STATE(1060),
    [sym_await_expression] = STATE(1060),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1060),
    [sym_augmented_assignment_expression] = STATE(1060),
    [sym_ternary_expression] = STATE(1060),
    [sym_binary_expression] = STATE(1060),
    [sym_unary_expression] = STATE(1060),
    [sym_update_expression] = STATE(1060),
    [sym_template_string] = STATE(1060),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2254),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2256),
    [sym_number] = ACTIONS(2254),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(2256),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2256),
    [sym_false] = ACTIONS(2256),
    [sym_null] = ACTIONS(2256),
    [sym_undefined] = ACTIONS(2256),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [619] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [620] = {
    [sym_parenthesized_expression] = STATE(1061),
    [sym__expression] = STATE(1061),
    [sym_yield_expression] = STATE(1061),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1061),
    [sym_jsx_fragment] = STATE(1061),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1061),
    [sym_anonymous_class] = STATE(1061),
    [sym_class] = STATE(1061),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1061),
    [sym_generator_function] = STATE(1061),
    [sym_call_expression] = STATE(1061),
    [sym_new_expression] = STATE(1061),
    [sym_await_expression] = STATE(1061),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1061),
    [sym_augmented_assignment_expression] = STATE(1061),
    [sym_ternary_expression] = STATE(1061),
    [sym_binary_expression] = STATE(1061),
    [sym_unary_expression] = STATE(1061),
    [sym_update_expression] = STATE(1061),
    [sym_template_string] = STATE(1061),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2258),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2260),
    [sym_number] = ACTIONS(2258),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(2260),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2260),
    [sym_false] = ACTIONS(2260),
    [sym_null] = ACTIONS(2260),
    [sym_undefined] = ACTIONS(2260),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [621] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [622] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [623] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(2246),
    [anon_sym_EQ_GT] = ACTIONS(2248),
    [anon_sym_PLUS_EQ] = ACTIONS(2264),
    [anon_sym_DASH_EQ] = ACTIONS(2264),
    [anon_sym_STAR_EQ] = ACTIONS(2264),
    [anon_sym_SLASH_EQ] = ACTIONS(2264),
    [anon_sym_PERCENT_EQ] = ACTIONS(2264),
    [anon_sym_CARET_EQ] = ACTIONS(2264),
    [anon_sym_AMP_EQ] = ACTIONS(2264),
    [anon_sym_PIPE_EQ] = ACTIONS(2264),
    [anon_sym_GT_GT_EQ] = ACTIONS(2264),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2264),
    [anon_sym_LT_LT_EQ] = ACTIONS(2264),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [624] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_COLON] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(2246),
    [anon_sym_PLUS_EQ] = ACTIONS(2250),
    [anon_sym_DASH_EQ] = ACTIONS(2250),
    [anon_sym_STAR_EQ] = ACTIONS(2250),
    [anon_sym_SLASH_EQ] = ACTIONS(2250),
    [anon_sym_PERCENT_EQ] = ACTIONS(2250),
    [anon_sym_CARET_EQ] = ACTIONS(2250),
    [anon_sym_AMP_EQ] = ACTIONS(2250),
    [anon_sym_PIPE_EQ] = ACTIONS(2250),
    [anon_sym_GT_GT_EQ] = ACTIONS(2250),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2250),
    [anon_sym_LT_LT_EQ] = ACTIONS(2250),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [625] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2268),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [626] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2246),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [627] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2246),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [628] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2246),
    [anon_sym_PLUS_EQ] = ACTIONS(2264),
    [anon_sym_DASH_EQ] = ACTIONS(2264),
    [anon_sym_STAR_EQ] = ACTIONS(2264),
    [anon_sym_SLASH_EQ] = ACTIONS(2264),
    [anon_sym_PERCENT_EQ] = ACTIONS(2264),
    [anon_sym_CARET_EQ] = ACTIONS(2264),
    [anon_sym_AMP_EQ] = ACTIONS(2264),
    [anon_sym_PIPE_EQ] = ACTIONS(2264),
    [anon_sym_GT_GT_EQ] = ACTIONS(2264),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2264),
    [anon_sym_LT_LT_EQ] = ACTIONS(2264),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [629] = {
    [anon_sym_EQ_GT] = ACTIONS(2248),
    [sym_comment] = ACTIONS(71),
  },
  [630] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [631] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [632] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [633] = {
    [anon_sym_LBRACE] = ACTIONS(2286),
    [anon_sym_LT] = ACTIONS(2286),
    [sym_jsx_text] = ACTIONS(2286),
    [sym_comment] = ACTIONS(71),
  },
  [634] = {
    [sym_parenthesized_expression] = STATE(1079),
    [sym__expression] = STATE(1079),
    [sym_yield_expression] = STATE(1079),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1079),
    [sym_jsx_fragment] = STATE(1079),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1079),
    [sym_anonymous_class] = STATE(1079),
    [sym_class] = STATE(1079),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1079),
    [sym_generator_function] = STATE(1079),
    [sym_call_expression] = STATE(1079),
    [sym_new_expression] = STATE(1079),
    [sym_await_expression] = STATE(1079),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1079),
    [sym_augmented_assignment_expression] = STATE(1079),
    [sym_ternary_expression] = STATE(1079),
    [sym_binary_expression] = STATE(1079),
    [sym_unary_expression] = STATE(1079),
    [sym_update_expression] = STATE(1079),
    [sym_template_string] = STATE(1079),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2304),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2306),
    [sym_number] = ACTIONS(2304),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(2306),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2306),
    [sym_false] = ACTIONS(2306),
    [sym_null] = ACTIONS(2306),
    [sym_undefined] = ACTIONS(2306),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [635] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2186),
    [anon_sym_RBRACE] = ACTIONS(2312),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [636] = {
    [anon_sym_RBRACE] = ACTIONS(2312),
    [sym_comment] = ACTIONS(71),
  },
  [637] = {
    [sym_nested_identifier] = STATE(1087),
    [sym_jsx_namespace_name] = STATE(1088),
    [sym_jsx_identifier] = ACTIONS(2314),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2316),
  },
  [638] = {
    [sym_jsx_expression] = STATE(1090),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1090),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1090),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(950),
    [anon_sym_SLASH] = ACTIONS(2318),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [639] = {
    [sym_jsx_expression] = STATE(1092),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1092),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1092),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(2320),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [640] = {
    [sym_jsx_expression] = STATE(1092),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1092),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1092),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(2320),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [641] = {
    [sym_jsx_expression] = STATE(1092),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1092),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1092),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(2320),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [642] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(2322),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [643] = {
    [anon_sym_LBRACE] = ACTIONS(1198),
    [anon_sym_LT] = ACTIONS(1198),
    [sym_jsx_text] = ACTIONS(1198),
    [sym_comment] = ACTIONS(71),
  },
  [644] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(1094),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(1192),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [645] = {
    [sym__automatic_semicolon] = ACTIONS(2324),
    [anon_sym_STAR] = ACTIONS(2324),
    [anon_sym_COMMA] = ACTIONS(2324),
    [anon_sym_RBRACE] = ACTIONS(2324),
    [anon_sym_LPAREN] = ACTIONS(2324),
    [anon_sym_RPAREN] = ACTIONS(2324),
    [anon_sym_in] = ACTIONS(2326),
    [anon_sym_SEMI] = ACTIONS(2324),
    [anon_sym_LBRACK] = ACTIONS(2324),
    [anon_sym_LT] = ACTIONS(2326),
    [anon_sym_GT] = ACTIONS(2326),
    [anon_sym_SLASH] = ACTIONS(2326),
    [anon_sym_DOT] = ACTIONS(2324),
    [anon_sym_QMARK] = ACTIONS(2324),
    [anon_sym_AMP_AMP] = ACTIONS(2324),
    [anon_sym_PIPE_PIPE] = ACTIONS(2324),
    [anon_sym_GT_GT] = ACTIONS(2326),
    [anon_sym_GT_GT_GT] = ACTIONS(2324),
    [anon_sym_LT_LT] = ACTIONS(2324),
    [anon_sym_AMP] = ACTIONS(2326),
    [anon_sym_CARET] = ACTIONS(2324),
    [anon_sym_PIPE] = ACTIONS(2326),
    [anon_sym_PLUS] = ACTIONS(2326),
    [anon_sym_DASH] = ACTIONS(2326),
    [anon_sym_PERCENT] = ACTIONS(2324),
    [anon_sym_LT_EQ] = ACTIONS(2324),
    [anon_sym_EQ_EQ] = ACTIONS(2326),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2324),
    [anon_sym_BANG_EQ] = ACTIONS(2326),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2324),
    [anon_sym_GT_EQ] = ACTIONS(2324),
    [anon_sym_instanceof] = ACTIONS(2324),
    [anon_sym_PLUS_PLUS] = ACTIONS(2324),
    [anon_sym_DASH_DASH] = ACTIONS(2324),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2324),
  },
  [646] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(2328),
    [anon_sym_LT] = ACTIONS(2331),
    [sym_jsx_text] = ACTIONS(2334),
    [sym_comment] = ACTIONS(71),
  },
  [647] = {
    [sym_parenthesized_expression] = STATE(1096),
    [sym__expression] = STATE(1096),
    [sym_yield_expression] = STATE(1096),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1096),
    [sym_jsx_fragment] = STATE(1096),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1096),
    [sym_anonymous_class] = STATE(1096),
    [sym_class] = STATE(1096),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1096),
    [sym_generator_function] = STATE(1096),
    [sym_call_expression] = STATE(1096),
    [sym_new_expression] = STATE(1096),
    [sym_await_expression] = STATE(1096),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1096),
    [sym_augmented_assignment_expression] = STATE(1096),
    [sym_ternary_expression] = STATE(1096),
    [sym_binary_expression] = STATE(1096),
    [sym_unary_expression] = STATE(1096),
    [sym_update_expression] = STATE(1096),
    [sym_template_string] = STATE(1096),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2337),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2339),
    [sym_number] = ACTIONS(2337),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2339),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2339),
    [sym_false] = ACTIONS(2339),
    [sym_null] = ACTIONS(2339),
    [sym_undefined] = ACTIONS(2339),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [648] = {
    [sym_class_heritage] = STATE(1097),
    [sym_class_body] = STATE(1033),
    [anon_sym_LBRACE] = ACTIONS(1008),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [649] = {
    [sym_class_body] = STATE(1098),
    [anon_sym_LBRACE] = ACTIONS(285),
    [sym_comment] = ACTIONS(71),
  },
  [650] = {
    [sym__automatic_semicolon] = ACTIONS(2341),
    [anon_sym_SEMI] = ACTIONS(2341),
    [sym_comment] = ACTIONS(71),
  },
  [651] = {
    [ts_builtin_sym_end] = ACTIONS(2343),
    [anon_sym_export] = ACTIONS(2345),
    [anon_sym_LBRACE] = ACTIONS(2343),
    [anon_sym_RBRACE] = ACTIONS(2343),
    [anon_sym_import] = ACTIONS(2345),
    [anon_sym_var] = ACTIONS(2345),
    [anon_sym_let] = ACTIONS(2345),
    [anon_sym_const] = ACTIONS(2345),
    [anon_sym_if] = ACTIONS(2345),
    [anon_sym_switch] = ACTIONS(2345),
    [anon_sym_for] = ACTIONS(2345),
    [anon_sym_LPAREN] = ACTIONS(2343),
    [anon_sym_while] = ACTIONS(2345),
    [anon_sym_do] = ACTIONS(2345),
    [anon_sym_try] = ACTIONS(2345),
    [anon_sym_with] = ACTIONS(2345),
    [anon_sym_break] = ACTIONS(2345),
    [anon_sym_continue] = ACTIONS(2345),
    [anon_sym_debugger] = ACTIONS(2345),
    [anon_sym_return] = ACTIONS(2345),
    [anon_sym_throw] = ACTIONS(2345),
    [anon_sym_SEMI] = ACTIONS(2343),
    [anon_sym_yield] = ACTIONS(2345),
    [anon_sym_LBRACK] = ACTIONS(2343),
    [anon_sym_LT] = ACTIONS(2343),
    [anon_sym_class] = ACTIONS(2345),
    [anon_sym_async] = ACTIONS(2345),
    [anon_sym_function] = ACTIONS(2345),
    [anon_sym_EQ_GT] = ACTIONS(2343),
    [anon_sym_new] = ACTIONS(2345),
    [anon_sym_await] = ACTIONS(2345),
    [anon_sym_PLUS] = ACTIONS(2345),
    [anon_sym_DASH] = ACTIONS(2345),
    [anon_sym_BANG] = ACTIONS(2343),
    [anon_sym_TILDE] = ACTIONS(2343),
    [anon_sym_typeof] = ACTIONS(2345),
    [anon_sym_void] = ACTIONS(2345),
    [anon_sym_delete] = ACTIONS(2345),
    [anon_sym_PLUS_PLUS] = ACTIONS(2343),
    [anon_sym_DASH_DASH] = ACTIONS(2343),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2343),
    [anon_sym_BQUOTE] = ACTIONS(2343),
    [sym_regex] = ACTIONS(2345),
    [sym_number] = ACTIONS(2343),
    [sym_identifier] = ACTIONS(2345),
    [sym_this] = ACTIONS(2345),
    [sym_super] = ACTIONS(2345),
    [sym_true] = ACTIONS(2345),
    [sym_false] = ACTIONS(2345),
    [sym_null] = ACTIONS(2345),
    [sym_undefined] = ACTIONS(2345),
    [anon_sym_AT] = ACTIONS(2343),
    [anon_sym_get] = ACTIONS(2345),
    [anon_sym_set] = ACTIONS(2345),
  },
  [652] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(882),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(882),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [653] = {
    [sym_parenthesized_expression] = STATE(1099),
    [sym__expression] = STATE(1099),
    [sym_yield_expression] = STATE(1099),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1099),
    [sym_jsx_fragment] = STATE(1099),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1099),
    [sym_anonymous_class] = STATE(1099),
    [sym_class] = STATE(1099),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1099),
    [sym_generator_function] = STATE(1099),
    [sym_call_expression] = STATE(1099),
    [sym_new_expression] = STATE(1099),
    [sym_await_expression] = STATE(1099),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1099),
    [sym_augmented_assignment_expression] = STATE(1099),
    [sym_ternary_expression] = STATE(1099),
    [sym_binary_expression] = STATE(1099),
    [sym_unary_expression] = STATE(1099),
    [sym_update_expression] = STATE(1099),
    [sym_template_string] = STATE(1099),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2347),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2349),
    [sym_number] = ACTIONS(2347),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2349),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2349),
    [sym_false] = ACTIONS(2349),
    [sym_null] = ACTIONS(2349),
    [sym_undefined] = ACTIONS(2349),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [654] = {
    [sym_statement_block] = STATE(584),
    [sym_parenthesized_expression] = STATE(1100),
    [sym__expression] = STATE(1100),
    [sym_yield_expression] = STATE(1100),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1100),
    [sym_jsx_fragment] = STATE(1100),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1100),
    [sym_anonymous_class] = STATE(1100),
    [sym_class] = STATE(1100),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1100),
    [sym_generator_function] = STATE(1100),
    [sym_call_expression] = STATE(1100),
    [sym_new_expression] = STATE(1100),
    [sym_await_expression] = STATE(1100),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1100),
    [sym_augmented_assignment_expression] = STATE(1100),
    [sym_ternary_expression] = STATE(1100),
    [sym_binary_expression] = STATE(1100),
    [sym_unary_expression] = STATE(1100),
    [sym_update_expression] = STATE(1100),
    [sym_template_string] = STATE(1100),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2351),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2353),
    [sym_number] = ACTIONS(2351),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2353),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2353),
    [sym_false] = ACTIONS(2353),
    [sym_null] = ACTIONS(2353),
    [sym_undefined] = ACTIONS(2353),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [655] = {
    [sym_parenthesized_expression] = STATE(1101),
    [sym__expression] = STATE(1101),
    [sym_yield_expression] = STATE(1101),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1101),
    [sym_jsx_fragment] = STATE(1101),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1101),
    [sym_anonymous_class] = STATE(1101),
    [sym_class] = STATE(1101),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1101),
    [sym_generator_function] = STATE(1101),
    [sym_call_expression] = STATE(1101),
    [sym_new_expression] = STATE(1101),
    [sym_await_expression] = STATE(1101),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1101),
    [sym_augmented_assignment_expression] = STATE(1101),
    [sym_ternary_expression] = STATE(1101),
    [sym_binary_expression] = STATE(1101),
    [sym_unary_expression] = STATE(1101),
    [sym_update_expression] = STATE(1101),
    [sym_template_string] = STATE(1101),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2355),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2357),
    [sym_number] = ACTIONS(2355),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2357),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2357),
    [sym_false] = ACTIONS(2357),
    [sym_null] = ACTIONS(2357),
    [sym_undefined] = ACTIONS(2357),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [656] = {
    [anon_sym_EQ_GT] = ACTIONS(2359),
    [sym_comment] = ACTIONS(71),
  },
  [657] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1048),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [658] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1056),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(1056),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [659] = {
    [sym_parenthesized_expression] = STATE(1103),
    [sym__expression] = STATE(1103),
    [sym_yield_expression] = STATE(1103),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1103),
    [sym_jsx_fragment] = STATE(1103),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1103),
    [sym_anonymous_class] = STATE(1103),
    [sym_class] = STATE(1103),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1103),
    [sym_generator_function] = STATE(1103),
    [sym_call_expression] = STATE(1103),
    [sym_new_expression] = STATE(1103),
    [sym_await_expression] = STATE(1103),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1103),
    [sym_augmented_assignment_expression] = STATE(1103),
    [sym_ternary_expression] = STATE(1103),
    [sym_binary_expression] = STATE(1103),
    [sym_unary_expression] = STATE(1103),
    [sym_update_expression] = STATE(1103),
    [sym_template_string] = STATE(1103),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2361),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2363),
    [sym_number] = ACTIONS(2361),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2363),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2363),
    [sym_false] = ACTIONS(2363),
    [sym_null] = ACTIONS(2363),
    [sym_undefined] = ACTIONS(2363),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [660] = {
    [sym_parenthesized_expression] = STATE(1104),
    [sym__expression] = STATE(1104),
    [sym_yield_expression] = STATE(1104),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1104),
    [sym_jsx_fragment] = STATE(1104),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1104),
    [sym_anonymous_class] = STATE(1104),
    [sym_class] = STATE(1104),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1104),
    [sym_generator_function] = STATE(1104),
    [sym_call_expression] = STATE(1104),
    [sym_new_expression] = STATE(1104),
    [sym_await_expression] = STATE(1104),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1104),
    [sym_augmented_assignment_expression] = STATE(1104),
    [sym_ternary_expression] = STATE(1104),
    [sym_binary_expression] = STATE(1104),
    [sym_unary_expression] = STATE(1104),
    [sym_update_expression] = STATE(1104),
    [sym_template_string] = STATE(1104),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2365),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2367),
    [sym_number] = ACTIONS(2365),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2367),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2367),
    [sym_false] = ACTIONS(2367),
    [sym_null] = ACTIONS(2367),
    [sym_undefined] = ACTIONS(2367),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [661] = {
    [sym_parenthesized_expression] = STATE(613),
    [sym__expression] = STATE(613),
    [sym_yield_expression] = STATE(613),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(613),
    [sym_jsx_fragment] = STATE(613),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(613),
    [sym_anonymous_class] = STATE(613),
    [sym_class] = STATE(613),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(613),
    [sym_generator_function] = STATE(613),
    [sym_call_expression] = STATE(613),
    [sym_new_expression] = STATE(613),
    [sym_await_expression] = STATE(613),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(613),
    [sym_augmented_assignment_expression] = STATE(613),
    [sym_ternary_expression] = STATE(613),
    [sym_binary_expression] = STATE(613),
    [sym_unary_expression] = STATE(613),
    [sym_update_expression] = STATE(613),
    [sym_template_string] = STATE(613),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1126),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1128),
    [sym_number] = ACTIONS(1126),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(1128),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [sym_null] = ACTIONS(1128),
    [sym_undefined] = ACTIONS(1128),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [662] = {
    [sym_parenthesized_expression] = STATE(1105),
    [sym__expression] = STATE(1105),
    [sym_yield_expression] = STATE(1105),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1105),
    [sym_jsx_fragment] = STATE(1105),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1105),
    [sym_anonymous_class] = STATE(1105),
    [sym_class] = STATE(1105),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1105),
    [sym_generator_function] = STATE(1105),
    [sym_call_expression] = STATE(1105),
    [sym_new_expression] = STATE(1105),
    [sym_await_expression] = STATE(1105),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1105),
    [sym_augmented_assignment_expression] = STATE(1105),
    [sym_ternary_expression] = STATE(1105),
    [sym_binary_expression] = STATE(1105),
    [sym_unary_expression] = STATE(1105),
    [sym_update_expression] = STATE(1105),
    [sym_template_string] = STATE(1105),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2369),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2371),
    [sym_number] = ACTIONS(2369),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(2371),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2371),
    [sym_false] = ACTIONS(2371),
    [sym_null] = ACTIONS(2371),
    [sym_undefined] = ACTIONS(2371),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [663] = {
    [sym_parenthesized_expression] = STATE(1106),
    [sym__expression] = STATE(1106),
    [sym_yield_expression] = STATE(1106),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1106),
    [sym_jsx_fragment] = STATE(1106),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1106),
    [sym_anonymous_class] = STATE(1106),
    [sym_class] = STATE(1106),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1106),
    [sym_generator_function] = STATE(1106),
    [sym_call_expression] = STATE(1106),
    [sym_new_expression] = STATE(1106),
    [sym_await_expression] = STATE(1106),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1106),
    [sym_augmented_assignment_expression] = STATE(1106),
    [sym_ternary_expression] = STATE(1106),
    [sym_binary_expression] = STATE(1106),
    [sym_unary_expression] = STATE(1106),
    [sym_update_expression] = STATE(1106),
    [sym_template_string] = STATE(1106),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2373),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2375),
    [sym_number] = ACTIONS(2373),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2375),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2375),
    [sym_false] = ACTIONS(2375),
    [sym_null] = ACTIONS(2375),
    [sym_undefined] = ACTIONS(2375),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [664] = {
    [sym_parenthesized_expression] = STATE(1107),
    [sym__expression] = STATE(1107),
    [sym_yield_expression] = STATE(1107),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1107),
    [sym_jsx_fragment] = STATE(1107),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1107),
    [sym_anonymous_class] = STATE(1107),
    [sym_class] = STATE(1107),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1107),
    [sym_generator_function] = STATE(1107),
    [sym_call_expression] = STATE(1107),
    [sym_new_expression] = STATE(1107),
    [sym_await_expression] = STATE(1107),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1107),
    [sym_augmented_assignment_expression] = STATE(1107),
    [sym_ternary_expression] = STATE(1107),
    [sym_binary_expression] = STATE(1107),
    [sym_unary_expression] = STATE(1107),
    [sym_update_expression] = STATE(1107),
    [sym_template_string] = STATE(1107),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2377),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2379),
    [sym_number] = ACTIONS(2377),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2379),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2379),
    [sym_false] = ACTIONS(2379),
    [sym_null] = ACTIONS(2379),
    [sym_undefined] = ACTIONS(2379),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [665] = {
    [sym_parenthesized_expression] = STATE(1108),
    [sym__expression] = STATE(1108),
    [sym_yield_expression] = STATE(1108),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1108),
    [sym_jsx_fragment] = STATE(1108),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1108),
    [sym_anonymous_class] = STATE(1108),
    [sym_class] = STATE(1108),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1108),
    [sym_generator_function] = STATE(1108),
    [sym_call_expression] = STATE(1108),
    [sym_new_expression] = STATE(1108),
    [sym_await_expression] = STATE(1108),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1108),
    [sym_augmented_assignment_expression] = STATE(1108),
    [sym_ternary_expression] = STATE(1108),
    [sym_binary_expression] = STATE(1108),
    [sym_unary_expression] = STATE(1108),
    [sym_update_expression] = STATE(1108),
    [sym_template_string] = STATE(1108),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2381),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2383),
    [sym_number] = ACTIONS(2381),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2383),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2383),
    [sym_false] = ACTIONS(2383),
    [sym_null] = ACTIONS(2383),
    [sym_undefined] = ACTIONS(2383),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [666] = {
    [sym__automatic_semicolon] = ACTIONS(2385),
    [anon_sym_from] = ACTIONS(2385),
    [anon_sym_SEMI] = ACTIONS(2385),
    [sym_comment] = ACTIONS(71),
  },
  [667] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2387),
  },
  [668] = {
    [sym__import_export_specifier] = STATE(1111),
    [anon_sym_RBRACE] = ACTIONS(2389),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [669] = {
    [sym__automatic_semicolon] = ACTIONS(2391),
    [anon_sym_from] = ACTIONS(2391),
    [anon_sym_SEMI] = ACTIONS(2391),
    [sym_comment] = ACTIONS(71),
  },
  [670] = {
    [aux_sym_export_clause_repeat1] = STATE(1113),
    [anon_sym_COMMA] = ACTIONS(2393),
    [anon_sym_RBRACE] = ACTIONS(2389),
    [sym_comment] = ACTIONS(71),
  },
  [671] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1115),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(2395),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [672] = {
    [sym_class_body] = STATE(1116),
    [anon_sym_LBRACE] = ACTIONS(1389),
    [sym_comment] = ACTIONS(71),
  },
  [673] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [ts_builtin_sym_end] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2140),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2140),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [674] = {
    [sym_formal_parameters] = STATE(1117),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [675] = {
    [sym_statement_block] = STATE(1118),
    [anon_sym_LBRACE] = ACTIONS(1395),
    [sym_comment] = ACTIONS(71),
  },
  [676] = {
    [sym_formal_parameters] = STATE(1119),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [677] = {
    [sym_statement_block] = STATE(1120),
    [anon_sym_LBRACE] = ACTIONS(1395),
    [sym_comment] = ACTIONS(71),
  },
  [678] = {
    [sym_export_statement] = STATE(1122),
    [sym__declaration] = STATE(1122),
    [sym_import_statement] = STATE(1122),
    [sym_expression_statement] = STATE(1122),
    [sym_variable_declaration] = STATE(1122),
    [sym_lexical_declaration] = STATE(1122),
    [sym_statement_block] = STATE(1122),
    [sym_if_statement] = STATE(1122),
    [sym_switch_statement] = STATE(1122),
    [sym_for_statement] = STATE(1122),
    [sym_for_in_statement] = STATE(1122),
    [sym_for_of_statement] = STATE(1122),
    [sym_while_statement] = STATE(1122),
    [sym_do_statement] = STATE(1122),
    [sym_try_statement] = STATE(1122),
    [sym_with_statement] = STATE(1122),
    [sym_break_statement] = STATE(1122),
    [sym_continue_statement] = STATE(1122),
    [sym_debugger_statement] = STATE(1122),
    [sym_return_statement] = STATE(1122),
    [sym_throw_statement] = STATE(1122),
    [sym_empty_statement] = STATE(1122),
    [sym_labeled_statement] = STATE(1122),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1122),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2397),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [679] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [ts_builtin_sym_end] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2178),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2178),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [680] = {
    [sym_class_heritage] = STATE(1123),
    [sym_class_body] = STATE(1116),
    [anon_sym_LBRACE] = ACTIONS(1389),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [681] = {
    [sym_parenthesized_expression] = STATE(1124),
    [sym__expression] = STATE(1124),
    [sym_yield_expression] = STATE(1124),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1124),
    [sym_jsx_fragment] = STATE(1124),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1124),
    [sym_anonymous_class] = STATE(1124),
    [sym_class] = STATE(1124),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1124),
    [sym_generator_function] = STATE(1124),
    [sym_call_expression] = STATE(1124),
    [sym_new_expression] = STATE(1124),
    [sym_await_expression] = STATE(1124),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1124),
    [sym_augmented_assignment_expression] = STATE(1124),
    [sym_ternary_expression] = STATE(1124),
    [sym_binary_expression] = STATE(1124),
    [sym_unary_expression] = STATE(1124),
    [sym_update_expression] = STATE(1124),
    [sym_template_string] = STATE(1124),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_RBRACK] = ACTIONS(239),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(1413),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2399),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2401),
    [sym_number] = ACTIONS(2399),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(2401),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2401),
    [sym_false] = ACTIONS(2401),
    [sym_null] = ACTIONS(2401),
    [sym_undefined] = ACTIONS(2401),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [682] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(1128),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_RBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(2403),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(2405),
    [anon_sym_PLUS_EQ] = ACTIONS(2407),
    [anon_sym_DASH_EQ] = ACTIONS(2407),
    [anon_sym_STAR_EQ] = ACTIONS(2407),
    [anon_sym_SLASH_EQ] = ACTIONS(2407),
    [anon_sym_PERCENT_EQ] = ACTIONS(2407),
    [anon_sym_CARET_EQ] = ACTIONS(2407),
    [anon_sym_AMP_EQ] = ACTIONS(2407),
    [anon_sym_PIPE_EQ] = ACTIONS(2407),
    [anon_sym_GT_GT_EQ] = ACTIONS(2407),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2407),
    [anon_sym_LT_LT_EQ] = ACTIONS(2407),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(2409),
  },
  [683] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(1129),
    [sym__expression] = STATE(1129),
    [sym_yield_expression] = STATE(1129),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1129),
    [sym_jsx_fragment] = STATE(1129),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1129),
    [sym_anonymous_class] = STATE(1129),
    [sym_class] = STATE(1129),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1129),
    [sym_generator_function] = STATE(1129),
    [sym_call_expression] = STATE(1129),
    [sym_new_expression] = STATE(1129),
    [sym_await_expression] = STATE(1129),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1129),
    [sym_augmented_assignment_expression] = STATE(1129),
    [sym_ternary_expression] = STATE(1129),
    [sym_binary_expression] = STATE(1129),
    [sym_unary_expression] = STATE(1129),
    [sym_update_expression] = STATE(1129),
    [sym_template_string] = STATE(1129),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2411),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2413),
    [sym_number] = ACTIONS(2411),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(2413),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2413),
    [sym_false] = ACTIONS(2413),
    [sym_null] = ACTIONS(2413),
    [sym_undefined] = ACTIONS(2413),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [684] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [685] = {
    [sym_parenthesized_expression] = STATE(1130),
    [sym__expression] = STATE(1130),
    [sym_yield_expression] = STATE(1130),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1130),
    [sym_jsx_fragment] = STATE(1130),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1130),
    [sym_anonymous_class] = STATE(1130),
    [sym_class] = STATE(1130),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1130),
    [sym_generator_function] = STATE(1130),
    [sym_call_expression] = STATE(1130),
    [sym_new_expression] = STATE(1130),
    [sym_await_expression] = STATE(1130),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1130),
    [sym_augmented_assignment_expression] = STATE(1130),
    [sym_ternary_expression] = STATE(1130),
    [sym_binary_expression] = STATE(1130),
    [sym_unary_expression] = STATE(1130),
    [sym_update_expression] = STATE(1130),
    [sym_template_string] = STATE(1130),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2415),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2417),
    [sym_number] = ACTIONS(2415),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(2417),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2417),
    [sym_false] = ACTIONS(2417),
    [sym_null] = ACTIONS(2417),
    [sym_undefined] = ACTIONS(2417),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [686] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [687] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [688] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(2403),
    [anon_sym_EQ_GT] = ACTIONS(2405),
    [anon_sym_PLUS_EQ] = ACTIONS(2421),
    [anon_sym_DASH_EQ] = ACTIONS(2421),
    [anon_sym_STAR_EQ] = ACTIONS(2421),
    [anon_sym_SLASH_EQ] = ACTIONS(2421),
    [anon_sym_PERCENT_EQ] = ACTIONS(2421),
    [anon_sym_CARET_EQ] = ACTIONS(2421),
    [anon_sym_AMP_EQ] = ACTIONS(2421),
    [anon_sym_PIPE_EQ] = ACTIONS(2421),
    [anon_sym_GT_GT_EQ] = ACTIONS(2421),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2421),
    [anon_sym_LT_LT_EQ] = ACTIONS(2421),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [689] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_RBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(2403),
    [anon_sym_PLUS_EQ] = ACTIONS(2407),
    [anon_sym_DASH_EQ] = ACTIONS(2407),
    [anon_sym_STAR_EQ] = ACTIONS(2407),
    [anon_sym_SLASH_EQ] = ACTIONS(2407),
    [anon_sym_PERCENT_EQ] = ACTIONS(2407),
    [anon_sym_CARET_EQ] = ACTIONS(2407),
    [anon_sym_AMP_EQ] = ACTIONS(2407),
    [anon_sym_PIPE_EQ] = ACTIONS(2407),
    [anon_sym_GT_GT_EQ] = ACTIONS(2407),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2407),
    [anon_sym_LT_LT_EQ] = ACTIONS(2407),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [690] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2425),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [691] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2403),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [692] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2403),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [693] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_RBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(2403),
    [anon_sym_PLUS_EQ] = ACTIONS(2421),
    [anon_sym_DASH_EQ] = ACTIONS(2421),
    [anon_sym_STAR_EQ] = ACTIONS(2421),
    [anon_sym_SLASH_EQ] = ACTIONS(2421),
    [anon_sym_PERCENT_EQ] = ACTIONS(2421),
    [anon_sym_CARET_EQ] = ACTIONS(2421),
    [anon_sym_AMP_EQ] = ACTIONS(2421),
    [anon_sym_PIPE_EQ] = ACTIONS(2421),
    [anon_sym_GT_GT_EQ] = ACTIONS(2421),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2421),
    [anon_sym_LT_LT_EQ] = ACTIONS(2421),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [694] = {
    [anon_sym_EQ_GT] = ACTIONS(2405),
    [sym_comment] = ACTIONS(71),
  },
  [695] = {
    [sym_statement_block] = STATE(1139),
    [anon_sym_LBRACE] = ACTIONS(1586),
    [sym_comment] = ACTIONS(71),
  },
  [696] = {
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RPAREN] = ACTIONS(480),
    [anon_sym_EQ] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
  },
  [697] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(1141),
    [anon_sym_STAR] = ACTIONS(1441),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2443),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [698] = {
    [sym__initializer] = STATE(708),
    [aux_sym_object_repeat1] = STATE(1141),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2443),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_EQ] = ACTIONS(1445),
    [sym_comment] = ACTIONS(71),
  },
  [699] = {
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(1141),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2443),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [700] = {
    [aux_sym_object_repeat1] = STATE(1143),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2445),
    [sym_comment] = ACTIONS(71),
  },
  [701] = {
    [sym_decorator] = STATE(716),
    [sym_method_definition] = STATE(1144),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(716),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(476),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [702] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2445),
    [sym_comment] = ACTIONS(71),
  },
  [703] = {
    [anon_sym_COMMA] = ACTIONS(894),
    [anon_sym_RPAREN] = ACTIONS(894),
    [anon_sym_EQ] = ACTIONS(894),
    [sym_comment] = ACTIONS(71),
  },
  [704] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(1146),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2447),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [705] = {
    [aux_sym_array_repeat1] = STATE(1146),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(2449),
    [sym_comment] = ACTIONS(71),
  },
  [706] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(2449),
    [sym_comment] = ACTIONS(71),
  },
  [707] = {
    [sym_parenthesized_expression] = STATE(1148),
    [sym__expression] = STATE(1148),
    [sym_yield_expression] = STATE(1148),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1148),
    [sym_jsx_fragment] = STATE(1148),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1148),
    [sym_anonymous_class] = STATE(1148),
    [sym_class] = STATE(1148),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1148),
    [sym_generator_function] = STATE(1148),
    [sym_call_expression] = STATE(1148),
    [sym_new_expression] = STATE(1148),
    [sym_await_expression] = STATE(1148),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1148),
    [sym_augmented_assignment_expression] = STATE(1148),
    [sym_ternary_expression] = STATE(1148),
    [sym_binary_expression] = STATE(1148),
    [sym_unary_expression] = STATE(1148),
    [sym_update_expression] = STATE(1148),
    [sym_template_string] = STATE(1148),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2451),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2453),
    [sym_number] = ACTIONS(2451),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2453),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2453),
    [sym_false] = ACTIONS(2453),
    [sym_null] = ACTIONS(2453),
    [sym_undefined] = ACTIONS(2453),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [708] = {
    [anon_sym_COMMA] = ACTIONS(1550),
    [anon_sym_RBRACE] = ACTIONS(1550),
    [anon_sym_RPAREN] = ACTIONS(1550),
    [sym_comment] = ACTIONS(71),
  },
  [709] = {
    [anon_sym_STAR] = ACTIONS(1110),
    [anon_sym_LPAREN] = ACTIONS(1110),
    [anon_sym_LBRACK] = ACTIONS(1110),
    [anon_sym_DOT] = ACTIONS(1552),
    [anon_sym_async] = ACTIONS(1552),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1110),
    [sym_number] = ACTIONS(1110),
    [sym_identifier] = ACTIONS(1552),
    [anon_sym_AT] = ACTIONS(1110),
    [anon_sym_get] = ACTIONS(1552),
    [anon_sym_set] = ACTIONS(1552),
  },
  [710] = {
    [sym_arguments] = STATE(1151),
    [anon_sym_STAR] = ACTIONS(1112),
    [anon_sym_LPAREN] = ACTIONS(2455),
    [anon_sym_LBRACK] = ACTIONS(1112),
    [anon_sym_DOT] = ACTIONS(2457),
    [anon_sym_async] = ACTIONS(1554),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1112),
    [sym_number] = ACTIONS(1112),
    [sym_identifier] = ACTIONS(1554),
    [anon_sym_AT] = ACTIONS(1112),
    [anon_sym_get] = ACTIONS(1554),
    [anon_sym_set] = ACTIONS(1554),
  },
  [711] = {
    [sym_arguments] = STATE(1153),
    [anon_sym_STAR] = ACTIONS(1116),
    [anon_sym_LPAREN] = ACTIONS(2455),
    [anon_sym_LBRACK] = ACTIONS(1116),
    [anon_sym_DOT] = ACTIONS(2459),
    [anon_sym_async] = ACTIONS(1560),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1116),
    [sym_number] = ACTIONS(1116),
    [sym_identifier] = ACTIONS(1560),
    [anon_sym_AT] = ACTIONS(1116),
    [anon_sym_get] = ACTIONS(1560),
    [anon_sym_set] = ACTIONS(1560),
  },
  [712] = {
    [anon_sym_STAR] = ACTIONS(1120),
    [anon_sym_LBRACK] = ACTIONS(1120),
    [anon_sym_async] = ACTIONS(1564),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1120),
    [sym_number] = ACTIONS(1120),
    [sym_identifier] = ACTIONS(1564),
    [anon_sym_AT] = ACTIONS(1120),
    [anon_sym_get] = ACTIONS(1564),
    [anon_sym_set] = ACTIONS(1564),
  },
  [713] = {
    [anon_sym_COMMA] = ACTIONS(1569),
    [anon_sym_RBRACE] = ACTIONS(1569),
    [anon_sym_RPAREN] = ACTIONS(1569),
    [sym_comment] = ACTIONS(71),
  },
  [714] = {
    [anon_sym_COMMA] = ACTIONS(1580),
    [anon_sym_RBRACE] = ACTIONS(1580),
    [anon_sym_RPAREN] = ACTIONS(1580),
    [sym_comment] = ACTIONS(71),
  },
  [715] = {
    [anon_sym_COMMA] = ACTIONS(2461),
    [anon_sym_RBRACE] = ACTIONS(2461),
    [sym_comment] = ACTIONS(71),
  },
  [716] = {
    [sym_decorator] = STATE(716),
    [aux_sym_export_statement_repeat1] = STATE(716),
    [anon_sym_STAR] = ACTIONS(1326),
    [anon_sym_LBRACK] = ACTIONS(1326),
    [anon_sym_async] = ACTIONS(1609),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1326),
    [sym_number] = ACTIONS(1326),
    [sym_identifier] = ACTIONS(1609),
    [anon_sym_AT] = ACTIONS(2463),
    [anon_sym_get] = ACTIONS(1609),
    [anon_sym_set] = ACTIONS(1609),
  },
  [717] = {
    [sym_parenthesized_expression] = STATE(1154),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [718] = {
    [anon_sym_LPAREN] = ACTIONS(2466),
    [sym_comment] = ACTIONS(71),
  },
  [719] = {
    [sym_parenthesized_expression] = STATE(1156),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [720] = {
    [sym_parenthesized_expression] = STATE(1157),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [721] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(2468),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [722] = {
    [anon_sym_export] = ACTIONS(1705),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_import] = ACTIONS(1705),
    [anon_sym_var] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_else] = ACTIONS(2470),
    [anon_sym_switch] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_do] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [anon_sym_with] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_debugger] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_throw] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_class] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_function] = ACTIONS(1705),
    [anon_sym_EQ_GT] = ACTIONS(1703),
    [anon_sym_new] = ACTIONS(1705),
    [anon_sym_await] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_typeof] = ACTIONS(1705),
    [anon_sym_void] = ACTIONS(1705),
    [anon_sym_delete] = ACTIONS(1705),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1703),
    [anon_sym_BQUOTE] = ACTIONS(1703),
    [sym_regex] = ACTIONS(1705),
    [sym_number] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_this] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_null] = ACTIONS(1705),
    [sym_undefined] = ACTIONS(1705),
    [anon_sym_AT] = ACTIONS(1703),
    [anon_sym_get] = ACTIONS(1705),
    [anon_sym_set] = ACTIONS(1705),
  },
  [723] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(1161),
    [sym_array] = STATE(1162),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2472),
  },
  [724] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(1161),
    [sym_array] = STATE(1162),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2472),
  },
  [725] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2474),
    [anon_sym_of] = ACTIONS(2476),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [726] = {
    [sym_expression_statement] = STATE(1165),
    [sym_empty_statement] = STATE(1165),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [727] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2478),
    [anon_sym_of] = ACTIONS(2480),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [728] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2482),
    [anon_sym_of] = ACTIONS(2484),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [729] = {
    [sym__automatic_semicolon] = ACTIONS(1989),
    [anon_sym_STAR] = ACTIONS(1989),
    [anon_sym_COMMA] = ACTIONS(1989),
    [anon_sym_LPAREN] = ACTIONS(2486),
    [anon_sym_in] = ACTIONS(1991),
    [anon_sym_SEMI] = ACTIONS(1989),
    [anon_sym_COLON] = ACTIONS(2489),
    [anon_sym_LBRACK] = ACTIONS(1989),
    [anon_sym_LT] = ACTIONS(1991),
    [anon_sym_GT] = ACTIONS(1991),
    [anon_sym_SLASH] = ACTIONS(1991),
    [anon_sym_DOT] = ACTIONS(1989),
    [anon_sym_EQ] = ACTIONS(1991),
    [anon_sym_QMARK] = ACTIONS(1989),
    [anon_sym_AMP_AMP] = ACTIONS(1989),
    [anon_sym_PIPE_PIPE] = ACTIONS(1989),
    [anon_sym_GT_GT] = ACTIONS(1991),
    [anon_sym_GT_GT_GT] = ACTIONS(1989),
    [anon_sym_LT_LT] = ACTIONS(1989),
    [anon_sym_AMP] = ACTIONS(1991),
    [anon_sym_CARET] = ACTIONS(1989),
    [anon_sym_PIPE] = ACTIONS(1991),
    [anon_sym_PLUS] = ACTIONS(1991),
    [anon_sym_DASH] = ACTIONS(1991),
    [anon_sym_PERCENT] = ACTIONS(1989),
    [anon_sym_LT_EQ] = ACTIONS(1989),
    [anon_sym_EQ_EQ] = ACTIONS(1991),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1989),
    [anon_sym_BANG_EQ] = ACTIONS(1991),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1989),
    [anon_sym_GT_EQ] = ACTIONS(1989),
    [anon_sym_instanceof] = ACTIONS(1989),
    [anon_sym_PLUS_PLUS] = ACTIONS(1989),
    [anon_sym_DASH_DASH] = ACTIONS(1989),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1989),
  },
  [730] = {
    [sym_formal_parameters] = STATE(1170),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [731] = {
    [sym__automatic_semicolon] = ACTIONS(2491),
    [anon_sym_STAR] = ACTIONS(2491),
    [anon_sym_COMMA] = ACTIONS(2491),
    [anon_sym_RBRACE] = ACTIONS(2491),
    [anon_sym_LPAREN] = ACTIONS(2491),
    [anon_sym_RPAREN] = ACTIONS(2491),
    [anon_sym_in] = ACTIONS(2493),
    [anon_sym_of] = ACTIONS(2491),
    [anon_sym_SEMI] = ACTIONS(2491),
    [anon_sym_LBRACK] = ACTIONS(2491),
    [anon_sym_LT] = ACTIONS(2493),
    [anon_sym_GT] = ACTIONS(2493),
    [anon_sym_SLASH] = ACTIONS(2493),
    [anon_sym_DOT] = ACTIONS(2491),
    [anon_sym_EQ] = ACTIONS(2493),
    [anon_sym_QMARK] = ACTIONS(2491),
    [anon_sym_AMP_AMP] = ACTIONS(2491),
    [anon_sym_PIPE_PIPE] = ACTIONS(2491),
    [anon_sym_GT_GT] = ACTIONS(2493),
    [anon_sym_GT_GT_GT] = ACTIONS(2491),
    [anon_sym_LT_LT] = ACTIONS(2491),
    [anon_sym_AMP] = ACTIONS(2493),
    [anon_sym_CARET] = ACTIONS(2491),
    [anon_sym_PIPE] = ACTIONS(2493),
    [anon_sym_PLUS] = ACTIONS(2493),
    [anon_sym_DASH] = ACTIONS(2493),
    [anon_sym_PERCENT] = ACTIONS(2491),
    [anon_sym_LT_EQ] = ACTIONS(2491),
    [anon_sym_EQ_EQ] = ACTIONS(2493),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2491),
    [anon_sym_BANG_EQ] = ACTIONS(2493),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2491),
    [anon_sym_GT_EQ] = ACTIONS(2491),
    [anon_sym_instanceof] = ACTIONS(2491),
    [anon_sym_PLUS_PLUS] = ACTIONS(2491),
    [anon_sym_DASH_DASH] = ACTIONS(2491),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2491),
  },
  [732] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_RBRACE] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [733] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1171),
    [sym__expression] = STATE(1171),
    [sym_yield_expression] = STATE(1171),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1171),
    [sym_jsx_fragment] = STATE(1171),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1171),
    [sym_anonymous_class] = STATE(1171),
    [sym_class] = STATE(1171),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1171),
    [sym_generator_function] = STATE(1171),
    [sym_call_expression] = STATE(1171),
    [sym_new_expression] = STATE(1171),
    [sym_await_expression] = STATE(1171),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1171),
    [sym_augmented_assignment_expression] = STATE(1171),
    [sym_ternary_expression] = STATE(1171),
    [sym_binary_expression] = STATE(1171),
    [sym_unary_expression] = STATE(1171),
    [sym_update_expression] = STATE(1171),
    [sym_template_string] = STATE(1171),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2495),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2497),
    [sym_number] = ACTIONS(2495),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2497),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2497),
    [sym_false] = ACTIONS(2497),
    [sym_null] = ACTIONS(2497),
    [sym_undefined] = ACTIONS(2497),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [734] = {
    [sym_parenthesized_expression] = STATE(1172),
    [sym__expression] = STATE(1172),
    [sym_yield_expression] = STATE(1172),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1172),
    [sym_jsx_fragment] = STATE(1172),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1172),
    [sym_anonymous_class] = STATE(1172),
    [sym_class] = STATE(1172),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1172),
    [sym_generator_function] = STATE(1172),
    [sym_call_expression] = STATE(1172),
    [sym_new_expression] = STATE(1172),
    [sym_await_expression] = STATE(1172),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1172),
    [sym_augmented_assignment_expression] = STATE(1172),
    [sym_ternary_expression] = STATE(1172),
    [sym_binary_expression] = STATE(1172),
    [sym_unary_expression] = STATE(1172),
    [sym_update_expression] = STATE(1172),
    [sym_template_string] = STATE(1172),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2499),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2501),
    [sym_number] = ACTIONS(2499),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2501),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2501),
    [sym_false] = ACTIONS(2501),
    [sym_null] = ACTIONS(2501),
    [sym_undefined] = ACTIONS(2501),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [735] = {
    [anon_sym_EQ_GT] = ACTIONS(2503),
    [sym_comment] = ACTIONS(71),
  },
  [736] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [737] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_RBRACE] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [738] = {
    [sym_parenthesized_expression] = STATE(1174),
    [sym__expression] = STATE(1174),
    [sym_yield_expression] = STATE(1174),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1174),
    [sym_jsx_fragment] = STATE(1174),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1174),
    [sym_anonymous_class] = STATE(1174),
    [sym_class] = STATE(1174),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1174),
    [sym_generator_function] = STATE(1174),
    [sym_call_expression] = STATE(1174),
    [sym_new_expression] = STATE(1174),
    [sym_await_expression] = STATE(1174),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1174),
    [sym_augmented_assignment_expression] = STATE(1174),
    [sym_ternary_expression] = STATE(1174),
    [sym_binary_expression] = STATE(1174),
    [sym_unary_expression] = STATE(1174),
    [sym_update_expression] = STATE(1174),
    [sym_template_string] = STATE(1174),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2505),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2507),
    [sym_number] = ACTIONS(2505),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2507),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2507),
    [sym_false] = ACTIONS(2507),
    [sym_null] = ACTIONS(2507),
    [sym_undefined] = ACTIONS(2507),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [739] = {
    [sym_parenthesized_expression] = STATE(1175),
    [sym__expression] = STATE(1175),
    [sym_yield_expression] = STATE(1175),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1175),
    [sym_jsx_fragment] = STATE(1175),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1175),
    [sym_anonymous_class] = STATE(1175),
    [sym_class] = STATE(1175),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1175),
    [sym_generator_function] = STATE(1175),
    [sym_call_expression] = STATE(1175),
    [sym_new_expression] = STATE(1175),
    [sym_await_expression] = STATE(1175),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1175),
    [sym_augmented_assignment_expression] = STATE(1175),
    [sym_ternary_expression] = STATE(1175),
    [sym_binary_expression] = STATE(1175),
    [sym_unary_expression] = STATE(1175),
    [sym_update_expression] = STATE(1175),
    [sym_template_string] = STATE(1175),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2509),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2511),
    [sym_number] = ACTIONS(2509),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2511),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2511),
    [sym_false] = ACTIONS(2511),
    [sym_null] = ACTIONS(2511),
    [sym_undefined] = ACTIONS(2511),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [740] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [741] = {
    [sym_parenthesized_expression] = STATE(1176),
    [sym__expression] = STATE(1176),
    [sym_yield_expression] = STATE(1176),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1176),
    [sym_jsx_fragment] = STATE(1176),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1176),
    [sym_anonymous_class] = STATE(1176),
    [sym_class] = STATE(1176),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1176),
    [sym_generator_function] = STATE(1176),
    [sym_call_expression] = STATE(1176),
    [sym_new_expression] = STATE(1176),
    [sym_await_expression] = STATE(1176),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1176),
    [sym_augmented_assignment_expression] = STATE(1176),
    [sym_ternary_expression] = STATE(1176),
    [sym_binary_expression] = STATE(1176),
    [sym_unary_expression] = STATE(1176),
    [sym_update_expression] = STATE(1176),
    [sym_template_string] = STATE(1176),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2513),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2515),
    [sym_number] = ACTIONS(2513),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(2515),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2515),
    [sym_false] = ACTIONS(2515),
    [sym_null] = ACTIONS(2515),
    [sym_undefined] = ACTIONS(2515),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [742] = {
    [sym_parenthesized_expression] = STATE(1177),
    [sym__expression] = STATE(1177),
    [sym_yield_expression] = STATE(1177),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1177),
    [sym_jsx_fragment] = STATE(1177),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1177),
    [sym_anonymous_class] = STATE(1177),
    [sym_class] = STATE(1177),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1177),
    [sym_generator_function] = STATE(1177),
    [sym_call_expression] = STATE(1177),
    [sym_new_expression] = STATE(1177),
    [sym_await_expression] = STATE(1177),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1177),
    [sym_augmented_assignment_expression] = STATE(1177),
    [sym_ternary_expression] = STATE(1177),
    [sym_binary_expression] = STATE(1177),
    [sym_unary_expression] = STATE(1177),
    [sym_update_expression] = STATE(1177),
    [sym_template_string] = STATE(1177),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2517),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2519),
    [sym_number] = ACTIONS(2517),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2519),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2519),
    [sym_false] = ACTIONS(2519),
    [sym_null] = ACTIONS(2519),
    [sym_undefined] = ACTIONS(2519),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [743] = {
    [sym_parenthesized_expression] = STATE(1178),
    [sym__expression] = STATE(1178),
    [sym_yield_expression] = STATE(1178),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1178),
    [sym_jsx_fragment] = STATE(1178),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1178),
    [sym_anonymous_class] = STATE(1178),
    [sym_class] = STATE(1178),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1178),
    [sym_generator_function] = STATE(1178),
    [sym_call_expression] = STATE(1178),
    [sym_new_expression] = STATE(1178),
    [sym_await_expression] = STATE(1178),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1178),
    [sym_augmented_assignment_expression] = STATE(1178),
    [sym_ternary_expression] = STATE(1178),
    [sym_binary_expression] = STATE(1178),
    [sym_unary_expression] = STATE(1178),
    [sym_update_expression] = STATE(1178),
    [sym_template_string] = STATE(1178),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2521),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2523),
    [sym_number] = ACTIONS(2521),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2523),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2523),
    [sym_false] = ACTIONS(2523),
    [sym_null] = ACTIONS(2523),
    [sym_undefined] = ACTIONS(2523),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [744] = {
    [sym_parenthesized_expression] = STATE(1179),
    [sym__expression] = STATE(1179),
    [sym_yield_expression] = STATE(1179),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1179),
    [sym_jsx_fragment] = STATE(1179),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1179),
    [sym_anonymous_class] = STATE(1179),
    [sym_class] = STATE(1179),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1179),
    [sym_generator_function] = STATE(1179),
    [sym_call_expression] = STATE(1179),
    [sym_new_expression] = STATE(1179),
    [sym_await_expression] = STATE(1179),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1179),
    [sym_augmented_assignment_expression] = STATE(1179),
    [sym_ternary_expression] = STATE(1179),
    [sym_binary_expression] = STATE(1179),
    [sym_unary_expression] = STATE(1179),
    [sym_update_expression] = STATE(1179),
    [sym_template_string] = STATE(1179),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2525),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2527),
    [sym_number] = ACTIONS(2525),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(2527),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2527),
    [sym_false] = ACTIONS(2527),
    [sym_null] = ACTIONS(2527),
    [sym_undefined] = ACTIONS(2527),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [745] = {
    [sym_parenthesized_expression] = STATE(1180),
    [sym__expression] = STATE(1180),
    [sym_yield_expression] = STATE(1180),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1180),
    [sym_jsx_fragment] = STATE(1180),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1180),
    [sym_anonymous_class] = STATE(1180),
    [sym_class] = STATE(1180),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1180),
    [sym_generator_function] = STATE(1180),
    [sym_call_expression] = STATE(1180),
    [sym_new_expression] = STATE(1180),
    [sym_await_expression] = STATE(1180),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1180),
    [sym_augmented_assignment_expression] = STATE(1180),
    [sym_ternary_expression] = STATE(1180),
    [sym_binary_expression] = STATE(1180),
    [sym_unary_expression] = STATE(1180),
    [sym_update_expression] = STATE(1180),
    [sym_template_string] = STATE(1180),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(239),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_RBRACE] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_SEMI] = ACTIONS(239),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(243),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(1533),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2529),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2531),
    [sym_number] = ACTIONS(2529),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(2531),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2531),
    [sym_false] = ACTIONS(2531),
    [sym_null] = ACTIONS(2531),
    [sym_undefined] = ACTIONS(2531),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [746] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(1183),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_function] = ACTIONS(826),
    [anon_sym_EQ_GT] = ACTIONS(2533),
    [anon_sym_PLUS_EQ] = ACTIONS(2535),
    [anon_sym_DASH_EQ] = ACTIONS(2535),
    [anon_sym_STAR_EQ] = ACTIONS(2535),
    [anon_sym_SLASH_EQ] = ACTIONS(2535),
    [anon_sym_PERCENT_EQ] = ACTIONS(2535),
    [anon_sym_CARET_EQ] = ACTIONS(2535),
    [anon_sym_AMP_EQ] = ACTIONS(2535),
    [anon_sym_PIPE_EQ] = ACTIONS(2535),
    [anon_sym_GT_GT_EQ] = ACTIONS(2535),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2535),
    [anon_sym_LT_LT_EQ] = ACTIONS(2535),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(2537),
  },
  [747] = {
    [sym_statement_block] = STATE(201),
    [sym_parenthesized_expression] = STATE(1184),
    [sym__expression] = STATE(1184),
    [sym_yield_expression] = STATE(1184),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1184),
    [sym_jsx_fragment] = STATE(1184),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1184),
    [sym_anonymous_class] = STATE(1184),
    [sym_class] = STATE(1184),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1184),
    [sym_generator_function] = STATE(1184),
    [sym_call_expression] = STATE(1184),
    [sym_new_expression] = STATE(1184),
    [sym_await_expression] = STATE(1184),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1184),
    [sym_augmented_assignment_expression] = STATE(1184),
    [sym_ternary_expression] = STATE(1184),
    [sym_binary_expression] = STATE(1184),
    [sym_unary_expression] = STATE(1184),
    [sym_update_expression] = STATE(1184),
    [sym_template_string] = STATE(1184),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2539),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2541),
    [sym_number] = ACTIONS(2539),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(2541),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2541),
    [sym_false] = ACTIONS(2541),
    [sym_null] = ACTIONS(2541),
    [sym_undefined] = ACTIONS(2541),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [748] = {
    [sym_parenthesized_expression] = STATE(203),
    [sym__expression] = STATE(203),
    [sym_yield_expression] = STATE(203),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(203),
    [sym_jsx_fragment] = STATE(203),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(203),
    [sym_anonymous_class] = STATE(203),
    [sym_class] = STATE(203),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(203),
    [sym_generator_function] = STATE(203),
    [sym_call_expression] = STATE(203),
    [sym_new_expression] = STATE(203),
    [sym_await_expression] = STATE(203),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(203),
    [sym_augmented_assignment_expression] = STATE(203),
    [sym_ternary_expression] = STATE(203),
    [sym_binary_expression] = STATE(203),
    [sym_unary_expression] = STATE(203),
    [sym_update_expression] = STATE(203),
    [sym_template_string] = STATE(203),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(322),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(324),
    [sym_number] = ACTIONS(322),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(324),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(324),
    [sym_false] = ACTIONS(324),
    [sym_null] = ACTIONS(324),
    [sym_undefined] = ACTIONS(324),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [749] = {
    [sym_parenthesized_expression] = STATE(1185),
    [sym__expression] = STATE(1185),
    [sym_yield_expression] = STATE(1185),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1185),
    [sym_jsx_fragment] = STATE(1185),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1185),
    [sym_anonymous_class] = STATE(1185),
    [sym_class] = STATE(1185),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1185),
    [sym_generator_function] = STATE(1185),
    [sym_call_expression] = STATE(1185),
    [sym_new_expression] = STATE(1185),
    [sym_await_expression] = STATE(1185),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1185),
    [sym_augmented_assignment_expression] = STATE(1185),
    [sym_ternary_expression] = STATE(1185),
    [sym_binary_expression] = STATE(1185),
    [sym_unary_expression] = STATE(1185),
    [sym_update_expression] = STATE(1185),
    [sym_template_string] = STATE(1185),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2543),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2545),
    [sym_number] = ACTIONS(2543),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(2545),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2545),
    [sym_false] = ACTIONS(2545),
    [sym_null] = ACTIONS(2545),
    [sym_undefined] = ACTIONS(2545),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [750] = {
    [sym_parenthesized_expression] = STATE(205),
    [sym__expression] = STATE(205),
    [sym_yield_expression] = STATE(205),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(205),
    [sym_jsx_fragment] = STATE(205),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(205),
    [sym_anonymous_class] = STATE(205),
    [sym_class] = STATE(205),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(205),
    [sym_generator_function] = STATE(205),
    [sym_call_expression] = STATE(205),
    [sym_new_expression] = STATE(205),
    [sym_await_expression] = STATE(205),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(205),
    [sym_augmented_assignment_expression] = STATE(205),
    [sym_ternary_expression] = STATE(205),
    [sym_binary_expression] = STATE(205),
    [sym_unary_expression] = STATE(205),
    [sym_update_expression] = STATE(205),
    [sym_template_string] = STATE(205),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(330),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(332),
    [sym_number] = ACTIONS(330),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(332),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(332),
    [sym_false] = ACTIONS(332),
    [sym_null] = ACTIONS(332),
    [sym_undefined] = ACTIONS(332),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [751] = {
    [sym_parenthesized_expression] = STATE(206),
    [sym__expression] = STATE(206),
    [sym_yield_expression] = STATE(206),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(206),
    [sym_jsx_fragment] = STATE(206),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(206),
    [sym_anonymous_class] = STATE(206),
    [sym_class] = STATE(206),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(206),
    [sym_generator_function] = STATE(206),
    [sym_call_expression] = STATE(206),
    [sym_new_expression] = STATE(206),
    [sym_await_expression] = STATE(206),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(206),
    [sym_augmented_assignment_expression] = STATE(206),
    [sym_ternary_expression] = STATE(206),
    [sym_binary_expression] = STATE(206),
    [sym_unary_expression] = STATE(206),
    [sym_update_expression] = STATE(206),
    [sym_template_string] = STATE(206),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(334),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(336),
    [sym_number] = ACTIONS(334),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(336),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(336),
    [sym_false] = ACTIONS(336),
    [sym_null] = ACTIONS(336),
    [sym_undefined] = ACTIONS(336),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [752] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_EQ_GT] = ACTIONS(2533),
    [anon_sym_PLUS_EQ] = ACTIONS(2549),
    [anon_sym_DASH_EQ] = ACTIONS(2549),
    [anon_sym_STAR_EQ] = ACTIONS(2549),
    [anon_sym_SLASH_EQ] = ACTIONS(2549),
    [anon_sym_PERCENT_EQ] = ACTIONS(2549),
    [anon_sym_CARET_EQ] = ACTIONS(2549),
    [anon_sym_AMP_EQ] = ACTIONS(2549),
    [anon_sym_PIPE_EQ] = ACTIONS(2549),
    [anon_sym_GT_GT_EQ] = ACTIONS(2549),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2549),
    [anon_sym_LT_LT_EQ] = ACTIONS(2549),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [753] = {
    [sym__initializer] = STATE(196),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_PLUS_EQ] = ACTIONS(2535),
    [anon_sym_DASH_EQ] = ACTIONS(2535),
    [anon_sym_STAR_EQ] = ACTIONS(2535),
    [anon_sym_SLASH_EQ] = ACTIONS(2535),
    [anon_sym_PERCENT_EQ] = ACTIONS(2535),
    [anon_sym_CARET_EQ] = ACTIONS(2535),
    [anon_sym_AMP_EQ] = ACTIONS(2535),
    [anon_sym_PIPE_EQ] = ACTIONS(2535),
    [anon_sym_GT_GT_EQ] = ACTIONS(2535),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2535),
    [anon_sym_LT_LT_EQ] = ACTIONS(2535),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [754] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2164),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_RBRACE] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2164),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [755] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [756] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [757] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_PLUS_EQ] = ACTIONS(2549),
    [anon_sym_DASH_EQ] = ACTIONS(2549),
    [anon_sym_STAR_EQ] = ACTIONS(2549),
    [anon_sym_SLASH_EQ] = ACTIONS(2549),
    [anon_sym_PERCENT_EQ] = ACTIONS(2549),
    [anon_sym_CARET_EQ] = ACTIONS(2549),
    [anon_sym_AMP_EQ] = ACTIONS(2549),
    [anon_sym_PIPE_EQ] = ACTIONS(2549),
    [anon_sym_GT_GT_EQ] = ACTIONS(2549),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2549),
    [anon_sym_LT_LT_EQ] = ACTIONS(2549),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [758] = {
    [anon_sym_EQ_GT] = ACTIONS(2533),
    [sym_comment] = ACTIONS(71),
  },
  [759] = {
    [sym_parenthesized_expression] = STATE(1194),
    [sym__expression] = STATE(1194),
    [sym_yield_expression] = STATE(1194),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1194),
    [sym_jsx_fragment] = STATE(1194),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1194),
    [sym_anonymous_class] = STATE(1194),
    [sym_class] = STATE(1194),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1194),
    [sym_generator_function] = STATE(1194),
    [sym_call_expression] = STATE(1194),
    [sym_new_expression] = STATE(1194),
    [sym_await_expression] = STATE(1194),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1194),
    [sym_augmented_assignment_expression] = STATE(1194),
    [sym_spread_element] = STATE(1195),
    [sym_ternary_expression] = STATE(1194),
    [sym_binary_expression] = STATE(1194),
    [sym_unary_expression] = STATE(1194),
    [sym_update_expression] = STATE(1194),
    [sym_template_string] = STATE(1194),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(1196),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(2569),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1096),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2571),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2573),
    [sym_number] = ACTIONS(2571),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(2573),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2573),
    [sym_false] = ACTIONS(2573),
    [sym_null] = ACTIONS(2573),
    [sym_undefined] = ACTIONS(2573),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [760] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2575),
  },
  [761] = {
    [anon_sym_export] = ACTIONS(2577),
    [anon_sym_STAR] = ACTIONS(2234),
    [anon_sym_LBRACK] = ACTIONS(2234),
    [anon_sym_class] = ACTIONS(2577),
    [anon_sym_async] = ACTIONS(2577),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2234),
    [sym_number] = ACTIONS(2234),
    [sym_identifier] = ACTIONS(2577),
    [anon_sym_AT] = ACTIONS(2234),
    [anon_sym_get] = ACTIONS(2577),
    [anon_sym_set] = ACTIONS(2577),
  },
  [762] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2579),
  },
  [763] = {
    [anon_sym_export] = ACTIONS(2581),
    [anon_sym_STAR] = ACTIONS(2238),
    [anon_sym_LBRACK] = ACTIONS(2238),
    [anon_sym_class] = ACTIONS(2581),
    [anon_sym_async] = ACTIONS(2581),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2238),
    [sym_number] = ACTIONS(2238),
    [sym_identifier] = ACTIONS(2581),
    [anon_sym_AT] = ACTIONS(2238),
    [anon_sym_get] = ACTIONS(2581),
    [anon_sym_set] = ACTIONS(2581),
  },
  [764] = {
    [sym__automatic_semicolon] = ACTIONS(2583),
    [anon_sym_STAR] = ACTIONS(2583),
    [anon_sym_COMMA] = ACTIONS(2583),
    [anon_sym_RBRACE] = ACTIONS(2583),
    [anon_sym_LPAREN] = ACTIONS(2583),
    [anon_sym_RPAREN] = ACTIONS(2583),
    [anon_sym_in] = ACTIONS(2585),
    [anon_sym_of] = ACTIONS(2583),
    [anon_sym_SEMI] = ACTIONS(2583),
    [anon_sym_LBRACK] = ACTIONS(2583),
    [anon_sym_LT] = ACTIONS(2585),
    [anon_sym_GT] = ACTIONS(2585),
    [anon_sym_SLASH] = ACTIONS(2585),
    [anon_sym_DOT] = ACTIONS(2583),
    [anon_sym_EQ] = ACTIONS(2585),
    [anon_sym_QMARK] = ACTIONS(2583),
    [anon_sym_AMP_AMP] = ACTIONS(2583),
    [anon_sym_PIPE_PIPE] = ACTIONS(2583),
    [anon_sym_GT_GT] = ACTIONS(2585),
    [anon_sym_GT_GT_GT] = ACTIONS(2583),
    [anon_sym_LT_LT] = ACTIONS(2583),
    [anon_sym_AMP] = ACTIONS(2585),
    [anon_sym_CARET] = ACTIONS(2583),
    [anon_sym_PIPE] = ACTIONS(2585),
    [anon_sym_PLUS] = ACTIONS(2585),
    [anon_sym_DASH] = ACTIONS(2585),
    [anon_sym_PERCENT] = ACTIONS(2583),
    [anon_sym_LT_EQ] = ACTIONS(2583),
    [anon_sym_EQ_EQ] = ACTIONS(2585),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2583),
    [anon_sym_BANG_EQ] = ACTIONS(2585),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2583),
    [anon_sym_GT_EQ] = ACTIONS(2583),
    [anon_sym_instanceof] = ACTIONS(2583),
    [anon_sym_PLUS_PLUS] = ACTIONS(2583),
    [anon_sym_DASH_DASH] = ACTIONS(2583),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2583),
  },
  [765] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2587),
    [anon_sym_RBRACE] = ACTIONS(2587),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [766] = {
    [sym_export_statement] = STATE(79),
    [sym__declaration] = STATE(79),
    [sym_import_statement] = STATE(79),
    [sym_expression_statement] = STATE(79),
    [sym_variable_declaration] = STATE(79),
    [sym_lexical_declaration] = STATE(79),
    [sym_statement_block] = STATE(79),
    [sym_if_statement] = STATE(79),
    [sym_switch_statement] = STATE(79),
    [sym_for_statement] = STATE(79),
    [sym_for_in_statement] = STATE(79),
    [sym_for_of_statement] = STATE(79),
    [sym_while_statement] = STATE(79),
    [sym_do_statement] = STATE(79),
    [sym_try_statement] = STATE(79),
    [sym_with_statement] = STATE(79),
    [sym_break_statement] = STATE(79),
    [sym_continue_statement] = STATE(79),
    [sym_debugger_statement] = STATE(79),
    [sym_return_statement] = STATE(79),
    [sym_throw_statement] = STATE(79),
    [sym_empty_statement] = STATE(79),
    [sym_labeled_statement] = STATE(79),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(79),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(840),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [767] = {
    [anon_sym_STAR] = ACTIONS(2589),
    [anon_sym_COMMA] = ACTIONS(2589),
    [anon_sym_RBRACE] = ACTIONS(2589),
    [anon_sym_SEMI] = ACTIONS(2589),
    [anon_sym_LBRACK] = ACTIONS(2589),
    [anon_sym_async] = ACTIONS(2591),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2589),
    [sym_number] = ACTIONS(2589),
    [sym_identifier] = ACTIONS(2591),
    [anon_sym_AT] = ACTIONS(2589),
    [anon_sym_static] = ACTIONS(2591),
    [anon_sym_get] = ACTIONS(2591),
    [anon_sym_set] = ACTIONS(2591),
  },
  [768] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2593),
    [sym_comment] = ACTIONS(71),
  },
  [769] = {
    [anon_sym_from] = ACTIONS(2595),
    [sym_comment] = ACTIONS(71),
  },
  [770] = {
    [anon_sym_from] = ACTIONS(2597),
    [sym_comment] = ACTIONS(71),
  },
  [771] = {
    [sym__import_export_specifier] = STATE(1201),
    [anon_sym_RBRACE] = ACTIONS(2599),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [772] = {
    [anon_sym_from] = ACTIONS(2601),
    [sym_comment] = ACTIONS(71),
  },
  [773] = {
    [aux_sym_named_imports_repeat1] = STATE(1203),
    [anon_sym_COMMA] = ACTIONS(2603),
    [anon_sym_RBRACE] = ACTIONS(2599),
    [sym_comment] = ACTIONS(71),
  },
  [774] = {
    [anon_sym_from] = ACTIONS(2605),
    [sym_comment] = ACTIONS(71),
  },
  [775] = {
    [ts_builtin_sym_end] = ACTIONS(2607),
    [anon_sym_export] = ACTIONS(2609),
    [anon_sym_LBRACE] = ACTIONS(2607),
    [anon_sym_RBRACE] = ACTIONS(2607),
    [anon_sym_import] = ACTIONS(2609),
    [anon_sym_var] = ACTIONS(2609),
    [anon_sym_let] = ACTIONS(2609),
    [anon_sym_const] = ACTIONS(2609),
    [anon_sym_if] = ACTIONS(2609),
    [anon_sym_switch] = ACTIONS(2609),
    [anon_sym_for] = ACTIONS(2609),
    [anon_sym_LPAREN] = ACTIONS(2607),
    [anon_sym_while] = ACTIONS(2609),
    [anon_sym_do] = ACTIONS(2609),
    [anon_sym_try] = ACTIONS(2609),
    [anon_sym_with] = ACTIONS(2609),
    [anon_sym_break] = ACTIONS(2609),
    [anon_sym_continue] = ACTIONS(2609),
    [anon_sym_debugger] = ACTIONS(2609),
    [anon_sym_return] = ACTIONS(2609),
    [anon_sym_throw] = ACTIONS(2609),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_yield] = ACTIONS(2609),
    [anon_sym_LBRACK] = ACTIONS(2607),
    [anon_sym_LT] = ACTIONS(2607),
    [anon_sym_class] = ACTIONS(2609),
    [anon_sym_async] = ACTIONS(2609),
    [anon_sym_function] = ACTIONS(2609),
    [anon_sym_EQ_GT] = ACTIONS(2607),
    [anon_sym_new] = ACTIONS(2609),
    [anon_sym_await] = ACTIONS(2609),
    [anon_sym_PLUS] = ACTIONS(2609),
    [anon_sym_DASH] = ACTIONS(2609),
    [anon_sym_BANG] = ACTIONS(2607),
    [anon_sym_TILDE] = ACTIONS(2607),
    [anon_sym_typeof] = ACTIONS(2609),
    [anon_sym_void] = ACTIONS(2609),
    [anon_sym_delete] = ACTIONS(2609),
    [anon_sym_PLUS_PLUS] = ACTIONS(2607),
    [anon_sym_DASH_DASH] = ACTIONS(2607),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2607),
    [anon_sym_BQUOTE] = ACTIONS(2607),
    [sym_regex] = ACTIONS(2609),
    [sym_number] = ACTIONS(2607),
    [sym_identifier] = ACTIONS(2609),
    [sym_this] = ACTIONS(2609),
    [sym_super] = ACTIONS(2609),
    [sym_true] = ACTIONS(2609),
    [sym_false] = ACTIONS(2609),
    [sym_null] = ACTIONS(2609),
    [sym_undefined] = ACTIONS(2609),
    [anon_sym_AT] = ACTIONS(2607),
    [anon_sym_get] = ACTIONS(2609),
    [anon_sym_set] = ACTIONS(2609),
  },
  [776] = {
    [sym__automatic_semicolon] = ACTIONS(1473),
    [anon_sym_COMMA] = ACTIONS(1473),
    [anon_sym_in] = ACTIONS(1473),
    [anon_sym_of] = ACTIONS(1473),
    [anon_sym_SEMI] = ACTIONS(1473),
    [anon_sym_EQ] = ACTIONS(1473),
    [sym_comment] = ACTIONS(71),
  },
  [777] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2611),
    [sym_comment] = ACTIONS(71),
  },
  [778] = {
    [sym__automatic_semicolon] = ACTIONS(1571),
    [anon_sym_COMMA] = ACTIONS(1571),
    [anon_sym_in] = ACTIONS(1571),
    [anon_sym_of] = ACTIONS(1571),
    [anon_sym_SEMI] = ACTIONS(1571),
    [anon_sym_EQ] = ACTIONS(1571),
    [sym_comment] = ACTIONS(71),
  },
  [779] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2613),
    [sym_comment] = ACTIONS(71),
  },
  [780] = {
    [aux_sym_object_repeat1] = STATE(1206),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2613),
    [sym_comment] = ACTIONS(71),
  },
  [781] = {
    [sym__automatic_semicolon] = ACTIONS(1989),
    [anon_sym_COMMA] = ACTIONS(1989),
    [anon_sym_in] = ACTIONS(1989),
    [anon_sym_of] = ACTIONS(1989),
    [anon_sym_SEMI] = ACTIONS(1989),
    [anon_sym_EQ] = ACTIONS(1989),
    [sym_comment] = ACTIONS(71),
  },
  [782] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(2615),
    [sym_comment] = ACTIONS(71),
  },
  [783] = {
    [sym__automatic_semicolon] = ACTIONS(2617),
    [anon_sym_COMMA] = ACTIONS(2617),
    [anon_sym_SEMI] = ACTIONS(2617),
    [sym_comment] = ACTIONS(71),
  },
  [784] = {
    [ts_builtin_sym_end] = ACTIONS(2619),
    [anon_sym_export] = ACTIONS(2621),
    [anon_sym_LBRACE] = ACTIONS(2619),
    [anon_sym_RBRACE] = ACTIONS(2619),
    [anon_sym_import] = ACTIONS(2621),
    [anon_sym_var] = ACTIONS(2621),
    [anon_sym_let] = ACTIONS(2621),
    [anon_sym_const] = ACTIONS(2621),
    [anon_sym_if] = ACTIONS(2621),
    [anon_sym_switch] = ACTIONS(2621),
    [anon_sym_for] = ACTIONS(2621),
    [anon_sym_LPAREN] = ACTIONS(2619),
    [anon_sym_while] = ACTIONS(2621),
    [anon_sym_do] = ACTIONS(2621),
    [anon_sym_try] = ACTIONS(2621),
    [anon_sym_with] = ACTIONS(2621),
    [anon_sym_break] = ACTIONS(2621),
    [anon_sym_continue] = ACTIONS(2621),
    [anon_sym_debugger] = ACTIONS(2621),
    [anon_sym_return] = ACTIONS(2621),
    [anon_sym_throw] = ACTIONS(2621),
    [anon_sym_SEMI] = ACTIONS(2619),
    [anon_sym_yield] = ACTIONS(2621),
    [anon_sym_LBRACK] = ACTIONS(2619),
    [anon_sym_LT] = ACTIONS(2619),
    [anon_sym_class] = ACTIONS(2621),
    [anon_sym_async] = ACTIONS(2621),
    [anon_sym_function] = ACTIONS(2621),
    [anon_sym_EQ_GT] = ACTIONS(2619),
    [anon_sym_new] = ACTIONS(2621),
    [anon_sym_await] = ACTIONS(2621),
    [anon_sym_PLUS] = ACTIONS(2621),
    [anon_sym_DASH] = ACTIONS(2621),
    [anon_sym_BANG] = ACTIONS(2619),
    [anon_sym_TILDE] = ACTIONS(2619),
    [anon_sym_typeof] = ACTIONS(2621),
    [anon_sym_void] = ACTIONS(2621),
    [anon_sym_delete] = ACTIONS(2621),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2619),
    [anon_sym_BQUOTE] = ACTIONS(2619),
    [sym_regex] = ACTIONS(2621),
    [sym_number] = ACTIONS(2619),
    [sym_identifier] = ACTIONS(2621),
    [sym_this] = ACTIONS(2621),
    [sym_super] = ACTIONS(2621),
    [sym_true] = ACTIONS(2621),
    [sym_false] = ACTIONS(2621),
    [sym_null] = ACTIONS(2621),
    [sym_undefined] = ACTIONS(2621),
    [anon_sym_AT] = ACTIONS(2619),
    [anon_sym_get] = ACTIONS(2621),
    [anon_sym_set] = ACTIONS(2621),
  },
  [785] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(2617),
    [anon_sym_COMMA] = ACTIONS(2623),
    [anon_sym_SEMI] = ACTIONS(2617),
    [sym_comment] = ACTIONS(71),
  },
  [786] = {
    [ts_builtin_sym_end] = ACTIONS(2626),
    [anon_sym_export] = ACTIONS(2628),
    [anon_sym_LBRACE] = ACTIONS(2626),
    [anon_sym_RBRACE] = ACTIONS(2626),
    [anon_sym_import] = ACTIONS(2628),
    [anon_sym_var] = ACTIONS(2628),
    [anon_sym_let] = ACTIONS(2628),
    [anon_sym_const] = ACTIONS(2628),
    [anon_sym_if] = ACTIONS(2628),
    [anon_sym_switch] = ACTIONS(2628),
    [anon_sym_for] = ACTIONS(2628),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2628),
    [anon_sym_do] = ACTIONS(2628),
    [anon_sym_try] = ACTIONS(2628),
    [anon_sym_with] = ACTIONS(2628),
    [anon_sym_break] = ACTIONS(2628),
    [anon_sym_continue] = ACTIONS(2628),
    [anon_sym_debugger] = ACTIONS(2628),
    [anon_sym_return] = ACTIONS(2628),
    [anon_sym_throw] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2628),
    [anon_sym_async] = ACTIONS(2628),
    [anon_sym_function] = ACTIONS(2628),
    [anon_sym_EQ_GT] = ACTIONS(2626),
    [anon_sym_new] = ACTIONS(2628),
    [anon_sym_await] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_DASH] = ACTIONS(2628),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2626),
    [anon_sym_typeof] = ACTIONS(2628),
    [anon_sym_void] = ACTIONS(2628),
    [anon_sym_delete] = ACTIONS(2628),
    [anon_sym_PLUS_PLUS] = ACTIONS(2626),
    [anon_sym_DASH_DASH] = ACTIONS(2626),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2626),
    [anon_sym_BQUOTE] = ACTIONS(2626),
    [sym_regex] = ACTIONS(2628),
    [sym_number] = ACTIONS(2626),
    [sym_identifier] = ACTIONS(2628),
    [sym_this] = ACTIONS(2628),
    [sym_super] = ACTIONS(2628),
    [sym_true] = ACTIONS(2628),
    [sym_false] = ACTIONS(2628),
    [sym_null] = ACTIONS(2628),
    [sym_undefined] = ACTIONS(2628),
    [anon_sym_AT] = ACTIONS(2626),
    [anon_sym_get] = ACTIONS(2628),
    [anon_sym_set] = ACTIONS(2628),
  },
  [787] = {
    [anon_sym_export] = ACTIONS(1823),
    [anon_sym_LBRACE] = ACTIONS(1821),
    [anon_sym_import] = ACTIONS(1823),
    [anon_sym_var] = ACTIONS(1823),
    [anon_sym_let] = ACTIONS(1823),
    [anon_sym_const] = ACTIONS(1823),
    [anon_sym_if] = ACTIONS(1823),
    [anon_sym_switch] = ACTIONS(1823),
    [anon_sym_for] = ACTIONS(1823),
    [anon_sym_LPAREN] = ACTIONS(1821),
    [anon_sym_while] = ACTIONS(1823),
    [anon_sym_do] = ACTIONS(1823),
    [anon_sym_try] = ACTIONS(1823),
    [anon_sym_with] = ACTIONS(1823),
    [anon_sym_break] = ACTIONS(1823),
    [anon_sym_continue] = ACTIONS(1823),
    [anon_sym_debugger] = ACTIONS(1823),
    [anon_sym_return] = ACTIONS(1823),
    [anon_sym_throw] = ACTIONS(1823),
    [anon_sym_SEMI] = ACTIONS(1821),
    [anon_sym_yield] = ACTIONS(1823),
    [anon_sym_LBRACK] = ACTIONS(1821),
    [anon_sym_LT] = ACTIONS(1821),
    [anon_sym_class] = ACTIONS(1823),
    [anon_sym_async] = ACTIONS(1823),
    [anon_sym_function] = ACTIONS(1823),
    [anon_sym_EQ_GT] = ACTIONS(1821),
    [anon_sym_new] = ACTIONS(1823),
    [anon_sym_await] = ACTIONS(1823),
    [anon_sym_PLUS] = ACTIONS(1823),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_BANG] = ACTIONS(1821),
    [anon_sym_TILDE] = ACTIONS(1821),
    [anon_sym_typeof] = ACTIONS(1823),
    [anon_sym_void] = ACTIONS(1823),
    [anon_sym_delete] = ACTIONS(1823),
    [anon_sym_PLUS_PLUS] = ACTIONS(1821),
    [anon_sym_DASH_DASH] = ACTIONS(1821),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1821),
    [anon_sym_BQUOTE] = ACTIONS(1821),
    [sym_regex] = ACTIONS(1823),
    [sym_number] = ACTIONS(1821),
    [sym_identifier] = ACTIONS(1823),
    [sym_this] = ACTIONS(1823),
    [sym_super] = ACTIONS(1823),
    [sym_true] = ACTIONS(1823),
    [sym_false] = ACTIONS(1823),
    [sym_null] = ACTIONS(1823),
    [sym_undefined] = ACTIONS(1823),
    [anon_sym_AT] = ACTIONS(1821),
    [anon_sym_get] = ACTIONS(1823),
    [anon_sym_set] = ACTIONS(1823),
  },
  [788] = {
    [sym__from_clause] = STATE(1208),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [789] = {
    [sym_parenthesized_expression] = STATE(1209),
    [sym__expression] = STATE(1209),
    [sym_yield_expression] = STATE(1209),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1209),
    [sym_jsx_fragment] = STATE(1209),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1209),
    [sym_anonymous_class] = STATE(1209),
    [sym_class] = STATE(1209),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1209),
    [sym_generator_function] = STATE(1209),
    [sym_call_expression] = STATE(1209),
    [sym_new_expression] = STATE(1209),
    [sym_await_expression] = STATE(1209),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1209),
    [sym_augmented_assignment_expression] = STATE(1209),
    [sym_ternary_expression] = STATE(1209),
    [sym_binary_expression] = STATE(1209),
    [sym_unary_expression] = STATE(1209),
    [sym_update_expression] = STATE(1209),
    [sym_template_string] = STATE(1209),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2630),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2632),
    [sym_number] = ACTIONS(2630),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(2632),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2632),
    [sym_false] = ACTIONS(2632),
    [sym_null] = ACTIONS(2632),
    [sym_undefined] = ACTIONS(2632),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [790] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2634),
  },
  [791] = {
    [anon_sym_function] = ACTIONS(2636),
    [sym_comment] = ACTIONS(71),
  },
  [792] = {
    [sym_formal_parameters] = STATE(1214),
    [anon_sym_STAR] = ACTIONS(2638),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2640),
  },
  [793] = {
    [sym__from_clause] = STATE(1208),
    [sym__automatic_semicolon] = ACTIONS(2642),
    [anon_sym_from] = ACTIONS(410),
    [anon_sym_SEMI] = ACTIONS(2642),
    [sym_comment] = ACTIONS(71),
  },
  [794] = {
    [ts_builtin_sym_end] = ACTIONS(452),
    [anon_sym_export] = ACTIONS(454),
    [anon_sym_LBRACE] = ACTIONS(452),
    [anon_sym_RBRACE] = ACTIONS(452),
    [anon_sym_import] = ACTIONS(454),
    [anon_sym_var] = ACTIONS(454),
    [anon_sym_let] = ACTIONS(454),
    [anon_sym_const] = ACTIONS(454),
    [anon_sym_if] = ACTIONS(454),
    [anon_sym_else] = ACTIONS(454),
    [anon_sym_switch] = ACTIONS(454),
    [anon_sym_for] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(452),
    [anon_sym_while] = ACTIONS(454),
    [anon_sym_do] = ACTIONS(454),
    [anon_sym_try] = ACTIONS(454),
    [anon_sym_with] = ACTIONS(454),
    [anon_sym_break] = ACTIONS(454),
    [anon_sym_continue] = ACTIONS(454),
    [anon_sym_debugger] = ACTIONS(454),
    [anon_sym_return] = ACTIONS(454),
    [anon_sym_throw] = ACTIONS(454),
    [anon_sym_SEMI] = ACTIONS(452),
    [anon_sym_yield] = ACTIONS(454),
    [anon_sym_LBRACK] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(452),
    [anon_sym_class] = ACTIONS(454),
    [anon_sym_async] = ACTIONS(454),
    [anon_sym_function] = ACTIONS(454),
    [anon_sym_EQ_GT] = ACTIONS(452),
    [anon_sym_new] = ACTIONS(454),
    [anon_sym_await] = ACTIONS(454),
    [anon_sym_PLUS] = ACTIONS(454),
    [anon_sym_DASH] = ACTIONS(454),
    [anon_sym_BANG] = ACTIONS(452),
    [anon_sym_TILDE] = ACTIONS(452),
    [anon_sym_typeof] = ACTIONS(454),
    [anon_sym_void] = ACTIONS(454),
    [anon_sym_delete] = ACTIONS(454),
    [anon_sym_PLUS_PLUS] = ACTIONS(452),
    [anon_sym_DASH_DASH] = ACTIONS(452),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(452),
    [anon_sym_BQUOTE] = ACTIONS(452),
    [sym_regex] = ACTIONS(454),
    [sym_number] = ACTIONS(452),
    [sym_identifier] = ACTIONS(454),
    [sym_this] = ACTIONS(454),
    [sym_super] = ACTIONS(454),
    [sym_true] = ACTIONS(454),
    [sym_false] = ACTIONS(454),
    [sym_null] = ACTIONS(454),
    [sym_undefined] = ACTIONS(454),
    [anon_sym_AT] = ACTIONS(452),
    [anon_sym_get] = ACTIONS(454),
    [anon_sym_set] = ACTIONS(454),
  },
  [795] = {
    [sym__automatic_semicolon] = ACTIONS(1711),
    [ts_builtin_sym_end] = ACTIONS(400),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_else] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(400),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_BANG] = ACTIONS(400),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [796] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(2644),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [797] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(486),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(486),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(480),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [798] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2646),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [799] = {
    [sym__automatic_semicolon] = ACTIONS(2648),
    [anon_sym_SEMI] = ACTIONS(2648),
    [sym_comment] = ACTIONS(71),
  },
  [800] = {
    [sym__from_clause] = STATE(1219),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [801] = {
    [aux_sym_variable_declaration_repeat1] = STATE(1221),
    [sym__automatic_semicolon] = ACTIONS(2650),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(2650),
    [sym_comment] = ACTIONS(71),
  },
  [802] = {
    [aux_sym_variable_declaration_repeat1] = STATE(1223),
    [sym__automatic_semicolon] = ACTIONS(2652),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(2652),
    [sym_comment] = ACTIONS(71),
  },
  [803] = {
    [sym_export_statement] = STATE(1224),
    [sym__declaration] = STATE(1224),
    [sym_import_statement] = STATE(1224),
    [sym_expression_statement] = STATE(1224),
    [sym_variable_declaration] = STATE(1224),
    [sym_lexical_declaration] = STATE(1224),
    [sym_statement_block] = STATE(1224),
    [sym_if_statement] = STATE(1224),
    [sym_switch_statement] = STATE(1224),
    [sym_for_statement] = STATE(1224),
    [sym_for_in_statement] = STATE(1224),
    [sym_for_of_statement] = STATE(1224),
    [sym_while_statement] = STATE(1224),
    [sym_do_statement] = STATE(1224),
    [sym_try_statement] = STATE(1224),
    [sym_with_statement] = STATE(1224),
    [sym_break_statement] = STATE(1224),
    [sym_continue_statement] = STATE(1224),
    [sym_debugger_statement] = STATE(1224),
    [sym_return_statement] = STATE(1224),
    [sym_throw_statement] = STATE(1224),
    [sym_empty_statement] = STATE(1224),
    [sym_labeled_statement] = STATE(1224),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [804] = {
    [sym_switch_body] = STATE(1226),
    [anon_sym_LBRACE] = ACTIONS(2654),
    [sym_comment] = ACTIONS(71),
  },
  [805] = {
    [sym_expression_statement] = STATE(1230),
    [sym_variable_declaration] = STATE(1230),
    [sym_lexical_declaration] = STATE(1230),
    [sym_empty_statement] = STATE(1230),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(1231),
    [sym_array] = STATE(1232),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(2656),
    [anon_sym_let] = ACTIONS(2658),
    [anon_sym_const] = ACTIONS(2658),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(2660),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [806] = {
    [sym_export_statement] = STATE(1233),
    [sym__declaration] = STATE(1233),
    [sym_import_statement] = STATE(1233),
    [sym_expression_statement] = STATE(1233),
    [sym_variable_declaration] = STATE(1233),
    [sym_lexical_declaration] = STATE(1233),
    [sym_statement_block] = STATE(1233),
    [sym_if_statement] = STATE(1233),
    [sym_switch_statement] = STATE(1233),
    [sym_for_statement] = STATE(1233),
    [sym_for_in_statement] = STATE(1233),
    [sym_for_of_statement] = STATE(1233),
    [sym_while_statement] = STATE(1233),
    [sym_do_statement] = STATE(1233),
    [sym_try_statement] = STATE(1233),
    [sym_with_statement] = STATE(1233),
    [sym_break_statement] = STATE(1233),
    [sym_continue_statement] = STATE(1233),
    [sym_debugger_statement] = STATE(1233),
    [sym_return_statement] = STATE(1233),
    [sym_throw_statement] = STATE(1233),
    [sym_empty_statement] = STATE(1233),
    [sym_labeled_statement] = STATE(1233),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [807] = {
    [anon_sym_while] = ACTIONS(2662),
    [sym_comment] = ACTIONS(71),
  },
  [808] = {
    [sym_export_statement] = STATE(1236),
    [sym__declaration] = STATE(1236),
    [sym_import_statement] = STATE(1236),
    [sym_expression_statement] = STATE(1236),
    [sym_variable_declaration] = STATE(1236),
    [sym_lexical_declaration] = STATE(1236),
    [sym_statement_block] = STATE(1236),
    [sym_if_statement] = STATE(1236),
    [sym_switch_statement] = STATE(1236),
    [sym_for_statement] = STATE(1236),
    [sym_for_in_statement] = STATE(1236),
    [sym_for_of_statement] = STATE(1236),
    [sym_while_statement] = STATE(1236),
    [sym_do_statement] = STATE(1236),
    [sym_try_statement] = STATE(1236),
    [sym_with_statement] = STATE(1236),
    [sym_break_statement] = STATE(1236),
    [sym_continue_statement] = STATE(1236),
    [sym_debugger_statement] = STATE(1236),
    [sym_return_statement] = STATE(1236),
    [sym_throw_statement] = STATE(1236),
    [sym_empty_statement] = STATE(1236),
    [sym_labeled_statement] = STATE(1236),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1236),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2664),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [809] = {
    [sym_catch_clause] = STATE(1239),
    [sym_finally_clause] = STATE(1240),
    [ts_builtin_sym_end] = ACTIONS(842),
    [anon_sym_export] = ACTIONS(844),
    [anon_sym_LBRACE] = ACTIONS(842),
    [anon_sym_RBRACE] = ACTIONS(842),
    [anon_sym_import] = ACTIONS(844),
    [anon_sym_var] = ACTIONS(844),
    [anon_sym_let] = ACTIONS(844),
    [anon_sym_const] = ACTIONS(844),
    [anon_sym_if] = ACTIONS(844),
    [anon_sym_else] = ACTIONS(844),
    [anon_sym_switch] = ACTIONS(844),
    [anon_sym_for] = ACTIONS(844),
    [anon_sym_LPAREN] = ACTIONS(842),
    [anon_sym_while] = ACTIONS(844),
    [anon_sym_do] = ACTIONS(844),
    [anon_sym_try] = ACTIONS(844),
    [anon_sym_with] = ACTIONS(844),
    [anon_sym_break] = ACTIONS(844),
    [anon_sym_continue] = ACTIONS(844),
    [anon_sym_debugger] = ACTIONS(844),
    [anon_sym_return] = ACTIONS(844),
    [anon_sym_throw] = ACTIONS(844),
    [anon_sym_SEMI] = ACTIONS(842),
    [anon_sym_catch] = ACTIONS(2666),
    [anon_sym_finally] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(844),
    [anon_sym_LBRACK] = ACTIONS(842),
    [anon_sym_LT] = ACTIONS(842),
    [anon_sym_class] = ACTIONS(844),
    [anon_sym_async] = ACTIONS(844),
    [anon_sym_function] = ACTIONS(844),
    [anon_sym_EQ_GT] = ACTIONS(842),
    [anon_sym_new] = ACTIONS(844),
    [anon_sym_await] = ACTIONS(844),
    [anon_sym_PLUS] = ACTIONS(844),
    [anon_sym_DASH] = ACTIONS(844),
    [anon_sym_BANG] = ACTIONS(842),
    [anon_sym_TILDE] = ACTIONS(842),
    [anon_sym_typeof] = ACTIONS(844),
    [anon_sym_void] = ACTIONS(844),
    [anon_sym_delete] = ACTIONS(844),
    [anon_sym_PLUS_PLUS] = ACTIONS(842),
    [anon_sym_DASH_DASH] = ACTIONS(842),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(842),
    [anon_sym_BQUOTE] = ACTIONS(842),
    [sym_regex] = ACTIONS(844),
    [sym_number] = ACTIONS(842),
    [sym_identifier] = ACTIONS(844),
    [sym_this] = ACTIONS(844),
    [sym_super] = ACTIONS(844),
    [sym_true] = ACTIONS(844),
    [sym_false] = ACTIONS(844),
    [sym_null] = ACTIONS(844),
    [sym_undefined] = ACTIONS(844),
    [anon_sym_AT] = ACTIONS(842),
    [anon_sym_get] = ACTIONS(844),
    [anon_sym_set] = ACTIONS(844),
  },
  [810] = {
    [sym_export_statement] = STATE(1241),
    [sym__declaration] = STATE(1241),
    [sym_import_statement] = STATE(1241),
    [sym_expression_statement] = STATE(1241),
    [sym_variable_declaration] = STATE(1241),
    [sym_lexical_declaration] = STATE(1241),
    [sym_statement_block] = STATE(1241),
    [sym_if_statement] = STATE(1241),
    [sym_switch_statement] = STATE(1241),
    [sym_for_statement] = STATE(1241),
    [sym_for_in_statement] = STATE(1241),
    [sym_for_of_statement] = STATE(1241),
    [sym_while_statement] = STATE(1241),
    [sym_do_statement] = STATE(1241),
    [sym_try_statement] = STATE(1241),
    [sym_with_statement] = STATE(1241),
    [sym_break_statement] = STATE(1241),
    [sym_continue_statement] = STATE(1241),
    [sym_debugger_statement] = STATE(1241),
    [sym_return_statement] = STATE(1241),
    [sym_throw_statement] = STATE(1241),
    [sym_empty_statement] = STATE(1241),
    [sym_labeled_statement] = STATE(1241),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [811] = {
    [ts_builtin_sym_end] = ACTIONS(850),
    [anon_sym_export] = ACTIONS(852),
    [anon_sym_LBRACE] = ACTIONS(850),
    [anon_sym_RBRACE] = ACTIONS(850),
    [anon_sym_import] = ACTIONS(852),
    [anon_sym_var] = ACTIONS(852),
    [anon_sym_let] = ACTIONS(852),
    [anon_sym_const] = ACTIONS(852),
    [anon_sym_if] = ACTIONS(852),
    [anon_sym_else] = ACTIONS(852),
    [anon_sym_switch] = ACTIONS(852),
    [anon_sym_for] = ACTIONS(852),
    [anon_sym_LPAREN] = ACTIONS(850),
    [anon_sym_while] = ACTIONS(852),
    [anon_sym_do] = ACTIONS(852),
    [anon_sym_try] = ACTIONS(852),
    [anon_sym_with] = ACTIONS(852),
    [anon_sym_break] = ACTIONS(852),
    [anon_sym_continue] = ACTIONS(852),
    [anon_sym_debugger] = ACTIONS(852),
    [anon_sym_return] = ACTIONS(852),
    [anon_sym_throw] = ACTIONS(852),
    [anon_sym_SEMI] = ACTIONS(850),
    [anon_sym_yield] = ACTIONS(852),
    [anon_sym_LBRACK] = ACTIONS(850),
    [anon_sym_LT] = ACTIONS(850),
    [anon_sym_class] = ACTIONS(852),
    [anon_sym_async] = ACTIONS(852),
    [anon_sym_function] = ACTIONS(852),
    [anon_sym_EQ_GT] = ACTIONS(850),
    [anon_sym_new] = ACTIONS(852),
    [anon_sym_await] = ACTIONS(852),
    [anon_sym_PLUS] = ACTIONS(852),
    [anon_sym_DASH] = ACTIONS(852),
    [anon_sym_BANG] = ACTIONS(850),
    [anon_sym_TILDE] = ACTIONS(850),
    [anon_sym_typeof] = ACTIONS(852),
    [anon_sym_void] = ACTIONS(852),
    [anon_sym_delete] = ACTIONS(852),
    [anon_sym_PLUS_PLUS] = ACTIONS(850),
    [anon_sym_DASH_DASH] = ACTIONS(850),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(850),
    [anon_sym_BQUOTE] = ACTIONS(850),
    [sym_regex] = ACTIONS(852),
    [sym_number] = ACTIONS(850),
    [sym_identifier] = ACTIONS(852),
    [sym_this] = ACTIONS(852),
    [sym_super] = ACTIONS(852),
    [sym_true] = ACTIONS(852),
    [sym_false] = ACTIONS(852),
    [sym_null] = ACTIONS(852),
    [sym_undefined] = ACTIONS(852),
    [anon_sym_AT] = ACTIONS(850),
    [anon_sym_get] = ACTIONS(852),
    [anon_sym_set] = ACTIONS(852),
  },
  [812] = {
    [sym__automatic_semicolon] = ACTIONS(2670),
    [anon_sym_SEMI] = ACTIONS(2670),
    [sym_comment] = ACTIONS(71),
  },
  [813] = {
    [ts_builtin_sym_end] = ACTIONS(856),
    [anon_sym_export] = ACTIONS(858),
    [anon_sym_LBRACE] = ACTIONS(856),
    [anon_sym_RBRACE] = ACTIONS(856),
    [anon_sym_import] = ACTIONS(858),
    [anon_sym_var] = ACTIONS(858),
    [anon_sym_let] = ACTIONS(858),
    [anon_sym_const] = ACTIONS(858),
    [anon_sym_if] = ACTIONS(858),
    [anon_sym_else] = ACTIONS(858),
    [anon_sym_switch] = ACTIONS(858),
    [anon_sym_for] = ACTIONS(858),
    [anon_sym_LPAREN] = ACTIONS(856),
    [anon_sym_while] = ACTIONS(858),
    [anon_sym_do] = ACTIONS(858),
    [anon_sym_try] = ACTIONS(858),
    [anon_sym_with] = ACTIONS(858),
    [anon_sym_break] = ACTIONS(858),
    [anon_sym_continue] = ACTIONS(858),
    [anon_sym_debugger] = ACTIONS(858),
    [anon_sym_return] = ACTIONS(858),
    [anon_sym_throw] = ACTIONS(858),
    [anon_sym_SEMI] = ACTIONS(856),
    [anon_sym_yield] = ACTIONS(858),
    [anon_sym_LBRACK] = ACTIONS(856),
    [anon_sym_LT] = ACTIONS(856),
    [anon_sym_class] = ACTIONS(858),
    [anon_sym_async] = ACTIONS(858),
    [anon_sym_function] = ACTIONS(858),
    [anon_sym_EQ_GT] = ACTIONS(856),
    [anon_sym_new] = ACTIONS(858),
    [anon_sym_await] = ACTIONS(858),
    [anon_sym_PLUS] = ACTIONS(858),
    [anon_sym_DASH] = ACTIONS(858),
    [anon_sym_BANG] = ACTIONS(856),
    [anon_sym_TILDE] = ACTIONS(856),
    [anon_sym_typeof] = ACTIONS(858),
    [anon_sym_void] = ACTIONS(858),
    [anon_sym_delete] = ACTIONS(858),
    [anon_sym_PLUS_PLUS] = ACTIONS(856),
    [anon_sym_DASH_DASH] = ACTIONS(856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(856),
    [anon_sym_BQUOTE] = ACTIONS(856),
    [sym_regex] = ACTIONS(858),
    [sym_number] = ACTIONS(856),
    [sym_identifier] = ACTIONS(858),
    [sym_this] = ACTIONS(858),
    [sym_super] = ACTIONS(858),
    [sym_true] = ACTIONS(858),
    [sym_false] = ACTIONS(858),
    [sym_null] = ACTIONS(858),
    [sym_undefined] = ACTIONS(858),
    [anon_sym_AT] = ACTIONS(856),
    [anon_sym_get] = ACTIONS(858),
    [anon_sym_set] = ACTIONS(858),
  },
  [814] = {
    [sym__automatic_semicolon] = ACTIONS(2672),
    [anon_sym_SEMI] = ACTIONS(2672),
    [sym_comment] = ACTIONS(71),
  },
  [815] = {
    [ts_builtin_sym_end] = ACTIONS(862),
    [anon_sym_export] = ACTIONS(864),
    [anon_sym_LBRACE] = ACTIONS(862),
    [anon_sym_RBRACE] = ACTIONS(862),
    [anon_sym_import] = ACTIONS(864),
    [anon_sym_var] = ACTIONS(864),
    [anon_sym_let] = ACTIONS(864),
    [anon_sym_const] = ACTIONS(864),
    [anon_sym_if] = ACTIONS(864),
    [anon_sym_else] = ACTIONS(864),
    [anon_sym_switch] = ACTIONS(864),
    [anon_sym_for] = ACTIONS(864),
    [anon_sym_LPAREN] = ACTIONS(862),
    [anon_sym_while] = ACTIONS(864),
    [anon_sym_do] = ACTIONS(864),
    [anon_sym_try] = ACTIONS(864),
    [anon_sym_with] = ACTIONS(864),
    [anon_sym_break] = ACTIONS(864),
    [anon_sym_continue] = ACTIONS(864),
    [anon_sym_debugger] = ACTIONS(864),
    [anon_sym_return] = ACTIONS(864),
    [anon_sym_throw] = ACTIONS(864),
    [anon_sym_SEMI] = ACTIONS(862),
    [anon_sym_yield] = ACTIONS(864),
    [anon_sym_LBRACK] = ACTIONS(862),
    [anon_sym_LT] = ACTIONS(862),
    [anon_sym_class] = ACTIONS(864),
    [anon_sym_async] = ACTIONS(864),
    [anon_sym_function] = ACTIONS(864),
    [anon_sym_EQ_GT] = ACTIONS(862),
    [anon_sym_new] = ACTIONS(864),
    [anon_sym_await] = ACTIONS(864),
    [anon_sym_PLUS] = ACTIONS(864),
    [anon_sym_DASH] = ACTIONS(864),
    [anon_sym_BANG] = ACTIONS(862),
    [anon_sym_TILDE] = ACTIONS(862),
    [anon_sym_typeof] = ACTIONS(864),
    [anon_sym_void] = ACTIONS(864),
    [anon_sym_delete] = ACTIONS(864),
    [anon_sym_PLUS_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(862),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(862),
    [anon_sym_BQUOTE] = ACTIONS(862),
    [sym_regex] = ACTIONS(864),
    [sym_number] = ACTIONS(862),
    [sym_identifier] = ACTIONS(864),
    [sym_this] = ACTIONS(864),
    [sym_super] = ACTIONS(864),
    [sym_true] = ACTIONS(864),
    [sym_false] = ACTIONS(864),
    [sym_null] = ACTIONS(864),
    [sym_undefined] = ACTIONS(864),
    [anon_sym_AT] = ACTIONS(862),
    [anon_sym_get] = ACTIONS(864),
    [anon_sym_set] = ACTIONS(864),
  },
  [816] = {
    [ts_builtin_sym_end] = ACTIONS(866),
    [anon_sym_export] = ACTIONS(868),
    [anon_sym_LBRACE] = ACTIONS(866),
    [anon_sym_RBRACE] = ACTIONS(866),
    [anon_sym_import] = ACTIONS(868),
    [anon_sym_var] = ACTIONS(868),
    [anon_sym_let] = ACTIONS(868),
    [anon_sym_const] = ACTIONS(868),
    [anon_sym_if] = ACTIONS(868),
    [anon_sym_else] = ACTIONS(868),
    [anon_sym_switch] = ACTIONS(868),
    [anon_sym_for] = ACTIONS(868),
    [anon_sym_LPAREN] = ACTIONS(866),
    [anon_sym_while] = ACTIONS(868),
    [anon_sym_do] = ACTIONS(868),
    [anon_sym_try] = ACTIONS(868),
    [anon_sym_with] = ACTIONS(868),
    [anon_sym_break] = ACTIONS(868),
    [anon_sym_continue] = ACTIONS(868),
    [anon_sym_debugger] = ACTIONS(868),
    [anon_sym_return] = ACTIONS(868),
    [anon_sym_throw] = ACTIONS(868),
    [anon_sym_SEMI] = ACTIONS(866),
    [anon_sym_yield] = ACTIONS(868),
    [anon_sym_LBRACK] = ACTIONS(866),
    [anon_sym_LT] = ACTIONS(866),
    [anon_sym_class] = ACTIONS(868),
    [anon_sym_async] = ACTIONS(868),
    [anon_sym_function] = ACTIONS(868),
    [anon_sym_EQ_GT] = ACTIONS(866),
    [anon_sym_new] = ACTIONS(868),
    [anon_sym_await] = ACTIONS(868),
    [anon_sym_PLUS] = ACTIONS(868),
    [anon_sym_DASH] = ACTIONS(868),
    [anon_sym_BANG] = ACTIONS(866),
    [anon_sym_TILDE] = ACTIONS(866),
    [anon_sym_typeof] = ACTIONS(868),
    [anon_sym_void] = ACTIONS(868),
    [anon_sym_delete] = ACTIONS(868),
    [anon_sym_PLUS_PLUS] = ACTIONS(866),
    [anon_sym_DASH_DASH] = ACTIONS(866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(866),
    [anon_sym_BQUOTE] = ACTIONS(866),
    [sym_regex] = ACTIONS(868),
    [sym_number] = ACTIONS(866),
    [sym_identifier] = ACTIONS(868),
    [sym_this] = ACTIONS(868),
    [sym_super] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_null] = ACTIONS(868),
    [sym_undefined] = ACTIONS(868),
    [anon_sym_AT] = ACTIONS(866),
    [anon_sym_get] = ACTIONS(868),
    [anon_sym_set] = ACTIONS(868),
  },
  [817] = {
    [sym__automatic_semicolon] = ACTIONS(2674),
    [anon_sym_SEMI] = ACTIONS(2674),
    [sym_comment] = ACTIONS(71),
  },
  [818] = {
    [sym__automatic_semicolon] = ACTIONS(2676),
    [anon_sym_SEMI] = ACTIONS(2676),
    [sym_comment] = ACTIONS(71),
  },
  [819] = {
    [sym_class_heritage] = STATE(1247),
    [sym_class_body] = STATE(1248),
    [anon_sym_LBRACE] = ACTIONS(2678),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [820] = {
    [sym_formal_parameters] = STATE(1250),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2680),
  },
  [821] = {
    [sym_formal_parameters] = STATE(1252),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2682),
  },
  [822] = {
    [sym_formal_parameters] = STATE(1250),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [823] = {
    [sym_statement_block] = STATE(1254),
    [anon_sym_LBRACE] = ACTIONS(2684),
    [sym_comment] = ACTIONS(71),
  },
  [824] = {
    [sym_export_statement] = STATE(1255),
    [sym__declaration] = STATE(1255),
    [sym_import_statement] = STATE(1255),
    [sym_expression_statement] = STATE(1255),
    [sym_variable_declaration] = STATE(1255),
    [sym_lexical_declaration] = STATE(1255),
    [sym_statement_block] = STATE(1255),
    [sym_if_statement] = STATE(1255),
    [sym_switch_statement] = STATE(1255),
    [sym_for_statement] = STATE(1255),
    [sym_for_in_statement] = STATE(1255),
    [sym_for_of_statement] = STATE(1255),
    [sym_while_statement] = STATE(1255),
    [sym_do_statement] = STATE(1255),
    [sym_try_statement] = STATE(1255),
    [sym_with_statement] = STATE(1255),
    [sym_break_statement] = STATE(1255),
    [sym_continue_statement] = STATE(1255),
    [sym_debugger_statement] = STATE(1255),
    [sym_return_statement] = STATE(1255),
    [sym_throw_statement] = STATE(1255),
    [sym_empty_statement] = STATE(1255),
    [sym_labeled_statement] = STATE(1255),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [825] = {
    [sym_export_statement] = STATE(1256),
    [sym__declaration] = STATE(1256),
    [sym_import_statement] = STATE(1256),
    [sym_expression_statement] = STATE(1256),
    [sym_variable_declaration] = STATE(1256),
    [sym_lexical_declaration] = STATE(1256),
    [sym_statement_block] = STATE(1256),
    [sym_if_statement] = STATE(1256),
    [sym_switch_statement] = STATE(1256),
    [sym_for_statement] = STATE(1256),
    [sym_for_in_statement] = STATE(1256),
    [sym_for_of_statement] = STATE(1256),
    [sym_while_statement] = STATE(1256),
    [sym_do_statement] = STATE(1256),
    [sym_try_statement] = STATE(1256),
    [sym_with_statement] = STATE(1256),
    [sym_break_statement] = STATE(1256),
    [sym_continue_statement] = STATE(1256),
    [sym_debugger_statement] = STATE(1256),
    [sym_return_statement] = STATE(1256),
    [sym_throw_statement] = STATE(1256),
    [sym_empty_statement] = STATE(1256),
    [sym_labeled_statement] = STATE(1256),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [826] = {
    [ts_builtin_sym_end] = ACTIONS(1122),
    [anon_sym_export] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_RBRACE] = ACTIONS(1122),
    [anon_sym_import] = ACTIONS(1124),
    [anon_sym_var] = ACTIONS(1124),
    [anon_sym_let] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_LPAREN] = ACTIONS(1122),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_try] = ACTIONS(1124),
    [anon_sym_with] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_debugger] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_throw] = ACTIONS(1124),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1122),
    [anon_sym_LT] = ACTIONS(1122),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_async] = ACTIONS(1124),
    [anon_sym_function] = ACTIONS(1124),
    [anon_sym_EQ_GT] = ACTIONS(1122),
    [anon_sym_new] = ACTIONS(1124),
    [anon_sym_await] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_typeof] = ACTIONS(1124),
    [anon_sym_void] = ACTIONS(1124),
    [anon_sym_delete] = ACTIONS(1124),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1122),
    [anon_sym_BQUOTE] = ACTIONS(1122),
    [sym_regex] = ACTIONS(1124),
    [sym_number] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1124),
    [sym_this] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_null] = ACTIONS(1124),
    [sym_undefined] = ACTIONS(1124),
    [anon_sym_AT] = ACTIONS(1122),
    [anon_sym_get] = ACTIONS(1124),
    [anon_sym_set] = ACTIONS(1124),
  },
  [827] = {
    [ts_builtin_sym_end] = ACTIONS(1202),
    [anon_sym_export] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_RBRACE] = ACTIONS(1202),
    [anon_sym_import] = ACTIONS(1204),
    [anon_sym_var] = ACTIONS(1204),
    [anon_sym_let] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_else] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_LPAREN] = ACTIONS(1202),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_try] = ACTIONS(1204),
    [anon_sym_with] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_debugger] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_throw] = ACTIONS(1204),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym_yield] = ACTIONS(1204),
    [anon_sym_LBRACK] = ACTIONS(1202),
    [anon_sym_LT] = ACTIONS(1202),
    [anon_sym_class] = ACTIONS(1204),
    [anon_sym_async] = ACTIONS(1204),
    [anon_sym_function] = ACTIONS(1204),
    [anon_sym_EQ_GT] = ACTIONS(1202),
    [anon_sym_new] = ACTIONS(1204),
    [anon_sym_await] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_typeof] = ACTIONS(1204),
    [anon_sym_void] = ACTIONS(1204),
    [anon_sym_delete] = ACTIONS(1204),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1202),
    [anon_sym_BQUOTE] = ACTIONS(1202),
    [sym_regex] = ACTIONS(1204),
    [sym_number] = ACTIONS(1202),
    [sym_identifier] = ACTIONS(1204),
    [sym_this] = ACTIONS(1204),
    [sym_super] = ACTIONS(1204),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [sym_null] = ACTIONS(1204),
    [sym_undefined] = ACTIONS(1204),
    [anon_sym_AT] = ACTIONS(1202),
    [anon_sym_get] = ACTIONS(1204),
    [anon_sym_set] = ACTIONS(1204),
  },
  [828] = {
    [sym__declaration] = STATE(1215),
    [sym_variable_declaration] = STATE(1215),
    [sym_lexical_declaration] = STATE(1215),
    [sym_class] = STATE(795),
    [sym_function] = STATE(795),
    [sym_generator_function] = STATE(795),
    [sym_decorator] = STATE(796),
    [aux_sym_export_statement_repeat1] = STATE(796),
    [anon_sym_default] = ACTIONS(2686),
    [anon_sym_var] = ACTIONS(1663),
    [anon_sym_let] = ACTIONS(1665),
    [anon_sym_const] = ACTIONS(1665),
    [anon_sym_class] = ACTIONS(1667),
    [anon_sym_async] = ACTIONS(1669),
    [anon_sym_function] = ACTIONS(1671),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [829] = {
    [sym_class_heritage] = STATE(649),
    [sym_class_body] = STATE(574),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2688),
  },
  [830] = {
    [anon_sym_COLON] = ACTIONS(2690),
    [sym_comment] = ACTIONS(71),
  },
  [831] = {
    [ts_builtin_sym_end] = ACTIONS(2692),
    [anon_sym_export] = ACTIONS(2694),
    [anon_sym_LBRACE] = ACTIONS(2692),
    [anon_sym_RBRACE] = ACTIONS(2692),
    [anon_sym_import] = ACTIONS(2694),
    [anon_sym_var] = ACTIONS(2694),
    [anon_sym_let] = ACTIONS(2694),
    [anon_sym_const] = ACTIONS(2694),
    [anon_sym_if] = ACTIONS(2694),
    [anon_sym_switch] = ACTIONS(2694),
    [anon_sym_for] = ACTIONS(2694),
    [anon_sym_LPAREN] = ACTIONS(2692),
    [anon_sym_while] = ACTIONS(2694),
    [anon_sym_do] = ACTIONS(2694),
    [anon_sym_try] = ACTIONS(2694),
    [anon_sym_with] = ACTIONS(2694),
    [anon_sym_break] = ACTIONS(2694),
    [anon_sym_continue] = ACTIONS(2694),
    [anon_sym_debugger] = ACTIONS(2694),
    [anon_sym_return] = ACTIONS(2694),
    [anon_sym_throw] = ACTIONS(2694),
    [anon_sym_SEMI] = ACTIONS(2692),
    [anon_sym_yield] = ACTIONS(2694),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_LT] = ACTIONS(2692),
    [anon_sym_class] = ACTIONS(2694),
    [anon_sym_async] = ACTIONS(2694),
    [anon_sym_function] = ACTIONS(2694),
    [anon_sym_EQ_GT] = ACTIONS(2692),
    [anon_sym_new] = ACTIONS(2694),
    [anon_sym_await] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2694),
    [anon_sym_DASH] = ACTIONS(2694),
    [anon_sym_BANG] = ACTIONS(2692),
    [anon_sym_TILDE] = ACTIONS(2692),
    [anon_sym_typeof] = ACTIONS(2694),
    [anon_sym_void] = ACTIONS(2694),
    [anon_sym_delete] = ACTIONS(2694),
    [anon_sym_PLUS_PLUS] = ACTIONS(2692),
    [anon_sym_DASH_DASH] = ACTIONS(2692),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2692),
    [anon_sym_BQUOTE] = ACTIONS(2692),
    [sym_regex] = ACTIONS(2694),
    [sym_number] = ACTIONS(2692),
    [sym_identifier] = ACTIONS(2694),
    [sym_this] = ACTIONS(2694),
    [sym_super] = ACTIONS(2694),
    [sym_true] = ACTIONS(2694),
    [sym_false] = ACTIONS(2694),
    [sym_null] = ACTIONS(2694),
    [sym_undefined] = ACTIONS(2694),
    [anon_sym_AT] = ACTIONS(2692),
    [anon_sym_get] = ACTIONS(2694),
    [anon_sym_set] = ACTIONS(2694),
  },
  [832] = {
    [sym_parenthesized_expression] = STATE(1270),
    [sym__expressions] = STATE(1269),
    [sym__expression] = STATE(1270),
    [sym_yield_expression] = STATE(1270),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1270),
    [sym_jsx_fragment] = STATE(1270),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1270),
    [sym_anonymous_class] = STATE(1270),
    [sym_class] = STATE(1270),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1270),
    [sym_generator_function] = STATE(1270),
    [sym_call_expression] = STATE(1270),
    [sym_new_expression] = STATE(1270),
    [sym_await_expression] = STATE(1270),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1270),
    [sym_augmented_assignment_expression] = STATE(1270),
    [sym_ternary_expression] = STATE(1270),
    [sym_binary_expression] = STATE(1270),
    [sym_unary_expression] = STATE(1270),
    [sym_update_expression] = STATE(1270),
    [sym_sequence_expression] = STATE(1269),
    [sym_template_string] = STATE(1270),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2712),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2714),
    [sym_number] = ACTIONS(2712),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(2714),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2714),
    [sym_false] = ACTIONS(2714),
    [sym_null] = ACTIONS(2714),
    [sym_undefined] = ACTIONS(2714),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [833] = {
    [sym_switch_case] = STATE(1276),
    [sym_switch_default] = STATE(1276),
    [aux_sym_switch_body_repeat1] = STATE(1276),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(2720),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [834] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(2722),
    [anon_sym_of] = ACTIONS(2724),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [835] = {
    [aux_sym_variable_declaration_repeat1] = STATE(1280),
    [sym__automatic_semicolon] = ACTIONS(2726),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(2726),
    [sym_comment] = ACTIONS(71),
  },
  [836] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(2728),
    [anon_sym_of] = ACTIONS(2730),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [837] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(2732),
    [anon_sym_of] = ACTIONS(2734),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [838] = {
    [aux_sym_variable_declaration_repeat1] = STATE(1286),
    [sym__automatic_semicolon] = ACTIONS(2736),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(2736),
    [sym_comment] = ACTIONS(71),
  },
  [839] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1296),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1296),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [840] = {
    [sym_parenthesized_expression] = STATE(1311),
    [sym__expression] = STATE(1311),
    [sym_yield_expression] = STATE(1311),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1311),
    [sym_jsx_fragment] = STATE(1311),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1311),
    [sym_anonymous_class] = STATE(1311),
    [sym_class] = STATE(1311),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1311),
    [sym_generator_function] = STATE(1311),
    [sym_call_expression] = STATE(1311),
    [sym_new_expression] = STATE(1311),
    [sym_await_expression] = STATE(1311),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1311),
    [sym_augmented_assignment_expression] = STATE(1311),
    [sym_ternary_expression] = STATE(1311),
    [sym_binary_expression] = STATE(1311),
    [sym_unary_expression] = STATE(1311),
    [sym_update_expression] = STATE(1311),
    [sym_template_string] = STATE(1311),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2778),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2780),
    [sym_number] = ACTIONS(2778),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(2780),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2780),
    [sym_false] = ACTIONS(2780),
    [sym_null] = ACTIONS(2780),
    [sym_undefined] = ACTIONS(2780),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [841] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1317),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1317),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(2786),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [842] = {
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_LPAREN] = ACTIONS(1122),
    [anon_sym_RPAREN] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1122),
    [anon_sym_LT] = ACTIONS(1122),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_async] = ACTIONS(1124),
    [anon_sym_function] = ACTIONS(1124),
    [anon_sym_EQ_GT] = ACTIONS(1122),
    [anon_sym_new] = ACTIONS(1124),
    [anon_sym_await] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_typeof] = ACTIONS(1124),
    [anon_sym_void] = ACTIONS(1124),
    [anon_sym_delete] = ACTIONS(1124),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1122),
    [anon_sym_BQUOTE] = ACTIONS(1122),
    [sym_regex] = ACTIONS(1124),
    [sym_number] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1124),
    [sym_this] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_null] = ACTIONS(1124),
    [sym_undefined] = ACTIONS(1124),
    [anon_sym_AT] = ACTIONS(1122),
    [anon_sym_get] = ACTIONS(1124),
    [anon_sym_set] = ACTIONS(1124),
  },
  [843] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1318),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1318),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [844] = {
    [sym_parenthesized_expression] = STATE(1319),
    [sym__expression] = STATE(1319),
    [sym_yield_expression] = STATE(1319),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1319),
    [sym_jsx_fragment] = STATE(1319),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1319),
    [sym_anonymous_class] = STATE(1319),
    [sym_class] = STATE(1319),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1319),
    [sym_generator_function] = STATE(1319),
    [sym_call_expression] = STATE(1319),
    [sym_new_expression] = STATE(1319),
    [sym_await_expression] = STATE(1319),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1319),
    [sym_augmented_assignment_expression] = STATE(1319),
    [sym_ternary_expression] = STATE(1319),
    [sym_binary_expression] = STATE(1319),
    [sym_unary_expression] = STATE(1319),
    [sym_update_expression] = STATE(1319),
    [sym_template_string] = STATE(1319),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2788),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2790),
    [sym_number] = ACTIONS(2788),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(2790),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2790),
    [sym_false] = ACTIONS(2790),
    [sym_null] = ACTIONS(2790),
    [sym_undefined] = ACTIONS(2790),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [845] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1320),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1320),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [846] = {
    [sym_parenthesized_expression] = STATE(1321),
    [sym__expression] = STATE(1321),
    [sym_yield_expression] = STATE(1321),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1321),
    [sym_jsx_fragment] = STATE(1321),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1321),
    [sym_anonymous_class] = STATE(1321),
    [sym_class] = STATE(1321),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1321),
    [sym_generator_function] = STATE(1321),
    [sym_call_expression] = STATE(1321),
    [sym_new_expression] = STATE(1321),
    [sym_await_expression] = STATE(1321),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1321),
    [sym_augmented_assignment_expression] = STATE(1321),
    [sym_ternary_expression] = STATE(1321),
    [sym_binary_expression] = STATE(1321),
    [sym_unary_expression] = STATE(1321),
    [sym_update_expression] = STATE(1321),
    [sym_template_string] = STATE(1321),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2792),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2794),
    [sym_number] = ACTIONS(2792),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(2794),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2794),
    [sym_false] = ACTIONS(2794),
    [sym_null] = ACTIONS(2794),
    [sym_undefined] = ACTIONS(2794),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [847] = {
    [anon_sym_STAR] = ACTIONS(1473),
    [anon_sym_LBRACE] = ACTIONS(1473),
    [anon_sym_COMMA] = ACTIONS(1473),
    [anon_sym_RBRACE] = ACTIONS(1473),
    [anon_sym_LPAREN] = ACTIONS(1473),
    [anon_sym_RPAREN] = ACTIONS(1473),
    [anon_sym_in] = ACTIONS(1475),
    [anon_sym_COLON] = ACTIONS(1473),
    [anon_sym_LBRACK] = ACTIONS(1473),
    [anon_sym_RBRACK] = ACTIONS(1473),
    [anon_sym_LT] = ACTIONS(1475),
    [anon_sym_GT] = ACTIONS(1475),
    [anon_sym_SLASH] = ACTIONS(1475),
    [anon_sym_DOT] = ACTIONS(1473),
    [anon_sym_EQ] = ACTIONS(1475),
    [anon_sym_QMARK] = ACTIONS(1473),
    [anon_sym_AMP_AMP] = ACTIONS(1473),
    [anon_sym_PIPE_PIPE] = ACTIONS(1473),
    [anon_sym_GT_GT] = ACTIONS(1475),
    [anon_sym_GT_GT_GT] = ACTIONS(1473),
    [anon_sym_LT_LT] = ACTIONS(1473),
    [anon_sym_AMP] = ACTIONS(1475),
    [anon_sym_CARET] = ACTIONS(1473),
    [anon_sym_PIPE] = ACTIONS(1475),
    [anon_sym_PLUS] = ACTIONS(1475),
    [anon_sym_DASH] = ACTIONS(1475),
    [anon_sym_PERCENT] = ACTIONS(1473),
    [anon_sym_LT_EQ] = ACTIONS(1473),
    [anon_sym_EQ_EQ] = ACTIONS(1475),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1473),
    [anon_sym_BANG_EQ] = ACTIONS(1475),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1473),
    [anon_sym_GT_EQ] = ACTIONS(1473),
    [anon_sym_instanceof] = ACTIONS(1473),
    [anon_sym_PLUS_PLUS] = ACTIONS(1473),
    [anon_sym_DASH_DASH] = ACTIONS(1473),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1473),
  },
  [848] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2796),
    [sym_comment] = ACTIONS(71),
  },
  [849] = {
    [anon_sym_STAR] = ACTIONS(1571),
    [anon_sym_LBRACE] = ACTIONS(1571),
    [anon_sym_COMMA] = ACTIONS(1571),
    [anon_sym_RBRACE] = ACTIONS(1571),
    [anon_sym_LPAREN] = ACTIONS(1571),
    [anon_sym_RPAREN] = ACTIONS(1571),
    [anon_sym_in] = ACTIONS(1573),
    [anon_sym_COLON] = ACTIONS(1571),
    [anon_sym_LBRACK] = ACTIONS(1571),
    [anon_sym_RBRACK] = ACTIONS(1571),
    [anon_sym_LT] = ACTIONS(1573),
    [anon_sym_GT] = ACTIONS(1573),
    [anon_sym_SLASH] = ACTIONS(1573),
    [anon_sym_DOT] = ACTIONS(1571),
    [anon_sym_EQ] = ACTIONS(1573),
    [anon_sym_QMARK] = ACTIONS(1571),
    [anon_sym_AMP_AMP] = ACTIONS(1571),
    [anon_sym_PIPE_PIPE] = ACTIONS(1571),
    [anon_sym_GT_GT] = ACTIONS(1573),
    [anon_sym_GT_GT_GT] = ACTIONS(1571),
    [anon_sym_LT_LT] = ACTIONS(1571),
    [anon_sym_AMP] = ACTIONS(1573),
    [anon_sym_CARET] = ACTIONS(1571),
    [anon_sym_PIPE] = ACTIONS(1573),
    [anon_sym_PLUS] = ACTIONS(1573),
    [anon_sym_DASH] = ACTIONS(1573),
    [anon_sym_PERCENT] = ACTIONS(1571),
    [anon_sym_LT_EQ] = ACTIONS(1571),
    [anon_sym_EQ_EQ] = ACTIONS(1573),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1571),
    [anon_sym_BANG_EQ] = ACTIONS(1573),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1571),
    [anon_sym_GT_EQ] = ACTIONS(1571),
    [anon_sym_instanceof] = ACTIONS(1571),
    [anon_sym_PLUS_PLUS] = ACTIONS(1571),
    [anon_sym_DASH_DASH] = ACTIONS(1571),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1571),
  },
  [850] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2798),
    [sym_comment] = ACTIONS(71),
  },
  [851] = {
    [aux_sym_object_repeat1] = STATE(1324),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(2798),
    [sym_comment] = ACTIONS(71),
  },
  [852] = {
    [anon_sym_STAR] = ACTIONS(1821),
    [anon_sym_LBRACE] = ACTIONS(1821),
    [anon_sym_COMMA] = ACTIONS(1821),
    [anon_sym_RBRACE] = ACTIONS(1821),
    [anon_sym_LPAREN] = ACTIONS(1821),
    [anon_sym_RPAREN] = ACTIONS(1821),
    [anon_sym_in] = ACTIONS(1823),
    [anon_sym_COLON] = ACTIONS(1821),
    [anon_sym_LBRACK] = ACTIONS(1821),
    [anon_sym_RBRACK] = ACTIONS(1821),
    [anon_sym_LT] = ACTIONS(1823),
    [anon_sym_GT] = ACTIONS(1823),
    [anon_sym_SLASH] = ACTIONS(1823),
    [anon_sym_DOT] = ACTIONS(1821),
    [anon_sym_QMARK] = ACTIONS(1821),
    [anon_sym_AMP_AMP] = ACTIONS(1821),
    [anon_sym_PIPE_PIPE] = ACTIONS(1821),
    [anon_sym_GT_GT] = ACTIONS(1823),
    [anon_sym_GT_GT_GT] = ACTIONS(1821),
    [anon_sym_LT_LT] = ACTIONS(1821),
    [anon_sym_AMP] = ACTIONS(1823),
    [anon_sym_CARET] = ACTIONS(1821),
    [anon_sym_PIPE] = ACTIONS(1823),
    [anon_sym_PLUS] = ACTIONS(1823),
    [anon_sym_DASH] = ACTIONS(1823),
    [anon_sym_PERCENT] = ACTIONS(1821),
    [anon_sym_LT_EQ] = ACTIONS(1821),
    [anon_sym_EQ_EQ] = ACTIONS(1823),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1821),
    [anon_sym_BANG_EQ] = ACTIONS(1823),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1821),
    [anon_sym_GT_EQ] = ACTIONS(1821),
    [anon_sym_instanceof] = ACTIONS(1821),
    [anon_sym_PLUS_PLUS] = ACTIONS(1821),
    [anon_sym_DASH_DASH] = ACTIONS(1821),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1821),
  },
  [853] = {
    [anon_sym_STAR] = ACTIONS(1989),
    [anon_sym_LBRACE] = ACTIONS(1989),
    [anon_sym_COMMA] = ACTIONS(1989),
    [anon_sym_RBRACE] = ACTIONS(1989),
    [anon_sym_LPAREN] = ACTIONS(1989),
    [anon_sym_RPAREN] = ACTIONS(1989),
    [anon_sym_in] = ACTIONS(1991),
    [anon_sym_COLON] = ACTIONS(1989),
    [anon_sym_LBRACK] = ACTIONS(1989),
    [anon_sym_RBRACK] = ACTIONS(1989),
    [anon_sym_LT] = ACTIONS(1991),
    [anon_sym_GT] = ACTIONS(1991),
    [anon_sym_SLASH] = ACTIONS(1991),
    [anon_sym_DOT] = ACTIONS(1989),
    [anon_sym_EQ] = ACTIONS(1991),
    [anon_sym_QMARK] = ACTIONS(1989),
    [anon_sym_AMP_AMP] = ACTIONS(1989),
    [anon_sym_PIPE_PIPE] = ACTIONS(1989),
    [anon_sym_GT_GT] = ACTIONS(1991),
    [anon_sym_GT_GT_GT] = ACTIONS(1989),
    [anon_sym_LT_LT] = ACTIONS(1989),
    [anon_sym_AMP] = ACTIONS(1991),
    [anon_sym_CARET] = ACTIONS(1989),
    [anon_sym_PIPE] = ACTIONS(1991),
    [anon_sym_PLUS] = ACTIONS(1991),
    [anon_sym_DASH] = ACTIONS(1991),
    [anon_sym_PERCENT] = ACTIONS(1989),
    [anon_sym_LT_EQ] = ACTIONS(1989),
    [anon_sym_EQ_EQ] = ACTIONS(1991),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1989),
    [anon_sym_BANG_EQ] = ACTIONS(1991),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1989),
    [anon_sym_GT_EQ] = ACTIONS(1989),
    [anon_sym_instanceof] = ACTIONS(1989),
    [anon_sym_PLUS_PLUS] = ACTIONS(1989),
    [anon_sym_DASH_DASH] = ACTIONS(1989),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1989),
  },
  [854] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(2800),
    [sym_comment] = ACTIONS(71),
  },
  [855] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(2802),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [856] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(2804),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [857] = {
    [anon_sym_GT] = ACTIONS(2806),
    [sym_comment] = ACTIONS(71),
  },
  [858] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2044),
    [anon_sym_SLASH] = ACTIONS(2808),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [859] = {
    [anon_sym_GT] = ACTIONS(2810),
    [sym_comment] = ACTIONS(71),
  },
  [860] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2058),
    [anon_sym_SLASH] = ACTIONS(2812),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [861] = {
    [anon_sym_STAR] = ACTIONS(2064),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_COMMA] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_RPAREN] = ACTIONS(2064),
    [anon_sym_in] = ACTIONS(2066),
    [anon_sym_COLON] = ACTIONS(2064),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_RBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2066),
    [anon_sym_GT] = ACTIONS(2066),
    [anon_sym_SLASH] = ACTIONS(2066),
    [anon_sym_DOT] = ACTIONS(2064),
    [anon_sym_QMARK] = ACTIONS(2064),
    [anon_sym_AMP_AMP] = ACTIONS(2064),
    [anon_sym_PIPE_PIPE] = ACTIONS(2064),
    [anon_sym_GT_GT] = ACTIONS(2066),
    [anon_sym_GT_GT_GT] = ACTIONS(2064),
    [anon_sym_LT_LT] = ACTIONS(2064),
    [anon_sym_AMP] = ACTIONS(2066),
    [anon_sym_CARET] = ACTIONS(2064),
    [anon_sym_PIPE] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_PERCENT] = ACTIONS(2064),
    [anon_sym_LT_EQ] = ACTIONS(2064),
    [anon_sym_EQ_EQ] = ACTIONS(2066),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2064),
    [anon_sym_BANG_EQ] = ACTIONS(2066),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2064),
    [anon_sym_GT_EQ] = ACTIONS(2064),
    [anon_sym_instanceof] = ACTIONS(2064),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2064),
  },
  [862] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(2814),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [863] = {
    [sym_class_body] = STATE(1333),
    [anon_sym_LBRACE] = ACTIONS(704),
    [sym_comment] = ACTIONS(71),
  },
  [864] = {
    [anon_sym_STAR] = ACTIONS(2140),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_COMMA] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_RPAREN] = ACTIONS(2140),
    [anon_sym_in] = ACTIONS(2142),
    [anon_sym_COLON] = ACTIONS(2140),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_RBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2142),
    [anon_sym_GT] = ACTIONS(2142),
    [anon_sym_SLASH] = ACTIONS(2142),
    [anon_sym_DOT] = ACTIONS(2140),
    [anon_sym_QMARK] = ACTIONS(2140),
    [anon_sym_AMP_AMP] = ACTIONS(2140),
    [anon_sym_PIPE_PIPE] = ACTIONS(2140),
    [anon_sym_GT_GT] = ACTIONS(2142),
    [anon_sym_GT_GT_GT] = ACTIONS(2140),
    [anon_sym_LT_LT] = ACTIONS(2140),
    [anon_sym_AMP] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(2140),
    [anon_sym_PIPE] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_PERCENT] = ACTIONS(2140),
    [anon_sym_LT_EQ] = ACTIONS(2140),
    [anon_sym_EQ_EQ] = ACTIONS(2142),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2140),
    [anon_sym_BANG_EQ] = ACTIONS(2142),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2140),
    [anon_sym_GT_EQ] = ACTIONS(2140),
    [anon_sym_instanceof] = ACTIONS(2140),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2140),
  },
  [865] = {
    [anon_sym_STAR] = ACTIONS(2144),
    [anon_sym_LBRACE] = ACTIONS(2144),
    [anon_sym_COMMA] = ACTIONS(2144),
    [anon_sym_RBRACE] = ACTIONS(2144),
    [anon_sym_LPAREN] = ACTIONS(2144),
    [anon_sym_RPAREN] = ACTIONS(2144),
    [anon_sym_in] = ACTIONS(2146),
    [anon_sym_COLON] = ACTIONS(2144),
    [anon_sym_LBRACK] = ACTIONS(2144),
    [anon_sym_RBRACK] = ACTIONS(2144),
    [anon_sym_LT] = ACTIONS(2146),
    [anon_sym_GT] = ACTIONS(2146),
    [anon_sym_SLASH] = ACTIONS(2146),
    [anon_sym_DOT] = ACTIONS(2144),
    [anon_sym_QMARK] = ACTIONS(2144),
    [anon_sym_AMP_AMP] = ACTIONS(2144),
    [anon_sym_PIPE_PIPE] = ACTIONS(2144),
    [anon_sym_GT_GT] = ACTIONS(2146),
    [anon_sym_GT_GT_GT] = ACTIONS(2144),
    [anon_sym_LT_LT] = ACTIONS(2144),
    [anon_sym_AMP] = ACTIONS(2146),
    [anon_sym_CARET] = ACTIONS(2144),
    [anon_sym_PIPE] = ACTIONS(2146),
    [anon_sym_PLUS] = ACTIONS(2146),
    [anon_sym_DASH] = ACTIONS(2146),
    [anon_sym_PERCENT] = ACTIONS(2144),
    [anon_sym_LT_EQ] = ACTIONS(2144),
    [anon_sym_EQ_EQ] = ACTIONS(2146),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2144),
    [anon_sym_BANG_EQ] = ACTIONS(2146),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2144),
    [anon_sym_GT_EQ] = ACTIONS(2144),
    [anon_sym_instanceof] = ACTIONS(2144),
    [anon_sym_PLUS_PLUS] = ACTIONS(2144),
    [anon_sym_DASH_DASH] = ACTIONS(2144),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2144),
  },
  [866] = {
    [anon_sym_COMMA] = ACTIONS(2816),
    [anon_sym_RPAREN] = ACTIONS(2816),
    [sym_comment] = ACTIONS(71),
  },
  [867] = {
    [sym__initializer] = STATE(708),
    [anon_sym_COMMA] = ACTIONS(2818),
    [anon_sym_RPAREN] = ACTIONS(2818),
    [anon_sym_EQ] = ACTIONS(2154),
    [sym_comment] = ACTIONS(71),
  },
  [868] = {
    [sym__initializer] = STATE(713),
    [anon_sym_COMMA] = ACTIONS(2820),
    [anon_sym_RPAREN] = ACTIONS(2820),
    [anon_sym_EQ] = ACTIONS(2154),
    [sym_comment] = ACTIONS(71),
  },
  [869] = {
    [anon_sym_COMMA] = ACTIONS(2818),
    [anon_sym_RPAREN] = ACTIONS(2818),
    [sym_comment] = ACTIONS(71),
  },
  [870] = {
    [sym__initializer] = STATE(714),
    [anon_sym_COMMA] = ACTIONS(2822),
    [anon_sym_RPAREN] = ACTIONS(2822),
    [anon_sym_EQ] = ACTIONS(2154),
    [sym_comment] = ACTIONS(71),
  },
  [871] = {
    [anon_sym_STAR] = ACTIONS(2162),
    [anon_sym_LBRACE] = ACTIONS(2160),
    [anon_sym_COMMA] = ACTIONS(2160),
    [anon_sym_RBRACE] = ACTIONS(2160),
    [anon_sym_LPAREN] = ACTIONS(2160),
    [anon_sym_RPAREN] = ACTIONS(2160),
    [anon_sym_in] = ACTIONS(2162),
    [anon_sym_COLON] = ACTIONS(2160),
    [anon_sym_LBRACK] = ACTIONS(2160),
    [anon_sym_RBRACK] = ACTIONS(2160),
    [anon_sym_LT] = ACTIONS(2162),
    [anon_sym_GT] = ACTIONS(2162),
    [anon_sym_SLASH] = ACTIONS(2162),
    [anon_sym_DOT] = ACTIONS(2160),
    [anon_sym_EQ] = ACTIONS(2162),
    [anon_sym_PLUS_EQ] = ACTIONS(2160),
    [anon_sym_DASH_EQ] = ACTIONS(2160),
    [anon_sym_STAR_EQ] = ACTIONS(2160),
    [anon_sym_SLASH_EQ] = ACTIONS(2160),
    [anon_sym_PERCENT_EQ] = ACTIONS(2160),
    [anon_sym_CARET_EQ] = ACTIONS(2160),
    [anon_sym_AMP_EQ] = ACTIONS(2160),
    [anon_sym_PIPE_EQ] = ACTIONS(2160),
    [anon_sym_GT_GT_EQ] = ACTIONS(2160),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2160),
    [anon_sym_LT_LT_EQ] = ACTIONS(2160),
    [anon_sym_QMARK] = ACTIONS(2160),
    [anon_sym_AMP_AMP] = ACTIONS(2160),
    [anon_sym_PIPE_PIPE] = ACTIONS(2160),
    [anon_sym_GT_GT] = ACTIONS(2162),
    [anon_sym_GT_GT_GT] = ACTIONS(2162),
    [anon_sym_LT_LT] = ACTIONS(2162),
    [anon_sym_AMP] = ACTIONS(2162),
    [anon_sym_CARET] = ACTIONS(2162),
    [anon_sym_PIPE] = ACTIONS(2162),
    [anon_sym_PLUS] = ACTIONS(2162),
    [anon_sym_DASH] = ACTIONS(2162),
    [anon_sym_PERCENT] = ACTIONS(2162),
    [anon_sym_LT_EQ] = ACTIONS(2160),
    [anon_sym_EQ_EQ] = ACTIONS(2162),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2160),
    [anon_sym_BANG_EQ] = ACTIONS(2162),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2160),
    [anon_sym_GT_EQ] = ACTIONS(2160),
    [anon_sym_instanceof] = ACTIONS(2160),
    [anon_sym_PLUS_PLUS] = ACTIONS(2160),
    [anon_sym_DASH_DASH] = ACTIONS(2160),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2160),
  },
  [872] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2164),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [873] = {
    [sym_formal_parameters] = STATE(1334),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [874] = {
    [sym_statement_block] = STATE(1335),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [875] = {
    [anon_sym_STAR] = ACTIONS(2166),
    [anon_sym_LBRACE] = ACTIONS(2166),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_RBRACE] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(2166),
    [anon_sym_RPAREN] = ACTIONS(2166),
    [anon_sym_in] = ACTIONS(2168),
    [anon_sym_COLON] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(2166),
    [anon_sym_RBRACK] = ACTIONS(2166),
    [anon_sym_LT] = ACTIONS(2168),
    [anon_sym_GT] = ACTIONS(2168),
    [anon_sym_SLASH] = ACTIONS(2168),
    [anon_sym_DOT] = ACTIONS(2166),
    [anon_sym_QMARK] = ACTIONS(2166),
    [anon_sym_AMP_AMP] = ACTIONS(2166),
    [anon_sym_PIPE_PIPE] = ACTIONS(2166),
    [anon_sym_GT_GT] = ACTIONS(2168),
    [anon_sym_GT_GT_GT] = ACTIONS(2166),
    [anon_sym_LT_LT] = ACTIONS(2166),
    [anon_sym_AMP] = ACTIONS(2168),
    [anon_sym_CARET] = ACTIONS(2166),
    [anon_sym_PIPE] = ACTIONS(2168),
    [anon_sym_PLUS] = ACTIONS(2168),
    [anon_sym_DASH] = ACTIONS(2168),
    [anon_sym_PERCENT] = ACTIONS(2166),
    [anon_sym_LT_EQ] = ACTIONS(2166),
    [anon_sym_EQ_EQ] = ACTIONS(2168),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2166),
    [anon_sym_BANG_EQ] = ACTIONS(2168),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2166),
    [anon_sym_GT_EQ] = ACTIONS(2166),
    [anon_sym_instanceof] = ACTIONS(2166),
    [anon_sym_PLUS_PLUS] = ACTIONS(2166),
    [anon_sym_DASH_DASH] = ACTIONS(2166),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2166),
  },
  [876] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2166),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [877] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2170),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [878] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(1337),
    [sym__expression] = STATE(1337),
    [sym_yield_expression] = STATE(1337),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1337),
    [sym_jsx_fragment] = STATE(1337),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1337),
    [sym_anonymous_class] = STATE(1337),
    [sym_class] = STATE(1337),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1337),
    [sym_generator_function] = STATE(1337),
    [sym_call_expression] = STATE(1337),
    [sym_new_expression] = STATE(1337),
    [sym_await_expression] = STATE(1337),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1337),
    [sym_augmented_assignment_expression] = STATE(1337),
    [sym_ternary_expression] = STATE(1337),
    [sym_binary_expression] = STATE(1337),
    [sym_unary_expression] = STATE(1337),
    [sym_update_expression] = STATE(1337),
    [sym_template_string] = STATE(1337),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2824),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2826),
    [sym_number] = ACTIONS(2824),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(2826),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2826),
    [sym_false] = ACTIONS(2826),
    [sym_null] = ACTIONS(2826),
    [sym_undefined] = ACTIONS(2826),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [879] = {
    [anon_sym_LBRACE] = ACTIONS(2828),
    [anon_sym_EQ_GT] = ACTIONS(2828),
    [sym_comment] = ACTIONS(71),
  },
  [880] = {
    [aux_sym_formal_parameters_repeat1] = STATE(880),
    [anon_sym_COMMA] = ACTIONS(2830),
    [anon_sym_RPAREN] = ACTIONS(2818),
    [sym_comment] = ACTIONS(71),
  },
  [881] = {
    [sym_formal_parameters] = STATE(1338),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [882] = {
    [sym_statement_block] = STATE(1339),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [883] = {
    [sym_export_statement] = STATE(888),
    [sym__declaration] = STATE(888),
    [sym_import_statement] = STATE(888),
    [sym_expression_statement] = STATE(888),
    [sym_variable_declaration] = STATE(888),
    [sym_lexical_declaration] = STATE(888),
    [sym_statement_block] = STATE(888),
    [sym_if_statement] = STATE(888),
    [sym_switch_statement] = STATE(888),
    [sym_for_statement] = STATE(888),
    [sym_for_in_statement] = STATE(888),
    [sym_for_of_statement] = STATE(888),
    [sym_while_statement] = STATE(888),
    [sym_do_statement] = STATE(888),
    [sym_try_statement] = STATE(888),
    [sym_with_statement] = STATE(888),
    [sym_break_statement] = STATE(888),
    [sym_continue_statement] = STATE(888),
    [sym_debugger_statement] = STATE(888),
    [sym_return_statement] = STATE(888),
    [sym_throw_statement] = STATE(888),
    [sym_empty_statement] = STATE(888),
    [sym_labeled_statement] = STATE(888),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(888),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2833),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [884] = {
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_COMMA] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_RPAREN] = ACTIONS(2178),
    [anon_sym_in] = ACTIONS(2180),
    [anon_sym_COLON] = ACTIONS(2178),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_RBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2180),
    [anon_sym_GT] = ACTIONS(2180),
    [anon_sym_SLASH] = ACTIONS(2180),
    [anon_sym_DOT] = ACTIONS(2178),
    [anon_sym_QMARK] = ACTIONS(2178),
    [anon_sym_AMP_AMP] = ACTIONS(2178),
    [anon_sym_PIPE_PIPE] = ACTIONS(2178),
    [anon_sym_GT_GT] = ACTIONS(2180),
    [anon_sym_GT_GT_GT] = ACTIONS(2178),
    [anon_sym_LT_LT] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2180),
    [anon_sym_CARET] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_PERCENT] = ACTIONS(2178),
    [anon_sym_LT_EQ] = ACTIONS(2178),
    [anon_sym_EQ_EQ] = ACTIONS(2180),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2178),
    [anon_sym_BANG_EQ] = ACTIONS(2180),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2178),
    [anon_sym_GT_EQ] = ACTIONS(2178),
    [anon_sym_instanceof] = ACTIONS(2178),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2178),
  },
  [885] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(848),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(494),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(499),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(505),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(508),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_async] = ACTIONS(460),
    [anon_sym_function] = ACTIONS(302),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(510),
    [anon_sym_get] = ACTIONS(512),
    [anon_sym_set] = ACTIONS(512),
  },
  [886] = {
    [sym__initializer] = STATE(309),
    [aux_sym_object_repeat1] = STATE(848),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(543),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(546),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(549),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [887] = {
    [sym__initializer] = STATE(196),
    [sym__property_name] = STATE(272),
    [sym_computed_property_name] = STATE(272),
    [aux_sym_object_repeat1] = STATE(848),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(1743),
    [anon_sym_LPAREN] = ACTIONS(555),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_COLON] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(505),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(508),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_async] = ACTIONS(460),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(462),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_number] = ACTIONS(462),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [888] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2835),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [889] = {
    [sym_decorator] = STATE(327),
    [sym_method_definition] = STATE(851),
    [sym__property_name] = STATE(326),
    [sym_computed_property_name] = STATE(326),
    [aux_sym_export_statement_repeat1] = STATE(327),
    [anon_sym_export] = ACTIONS(566),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_class] = ACTIONS(568),
    [anon_sym_async] = ACTIONS(570),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(572),
    [sym_number] = ACTIONS(572),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(574),
    [anon_sym_set] = ACTIONS(574),
  },
  [890] = {
    [anon_sym_STAR] = ACTIONS(2188),
    [anon_sym_LBRACE] = ACTIONS(2188),
    [anon_sym_COMMA] = ACTIONS(2188),
    [anon_sym_RBRACE] = ACTIONS(2188),
    [anon_sym_LPAREN] = ACTIONS(2188),
    [anon_sym_RPAREN] = ACTIONS(2188),
    [anon_sym_in] = ACTIONS(2190),
    [anon_sym_COLON] = ACTIONS(2188),
    [anon_sym_LBRACK] = ACTIONS(2188),
    [anon_sym_RBRACK] = ACTIONS(2188),
    [anon_sym_LT] = ACTIONS(2190),
    [anon_sym_GT] = ACTIONS(2190),
    [anon_sym_SLASH] = ACTIONS(2190),
    [anon_sym_DOT] = ACTIONS(2188),
    [anon_sym_QMARK] = ACTIONS(2188),
    [anon_sym_AMP_AMP] = ACTIONS(2188),
    [anon_sym_PIPE_PIPE] = ACTIONS(2188),
    [anon_sym_GT_GT] = ACTIONS(2190),
    [anon_sym_GT_GT_GT] = ACTIONS(2188),
    [anon_sym_LT_LT] = ACTIONS(2188),
    [anon_sym_AMP] = ACTIONS(2190),
    [anon_sym_CARET] = ACTIONS(2188),
    [anon_sym_PIPE] = ACTIONS(2190),
    [anon_sym_PLUS] = ACTIONS(2190),
    [anon_sym_DASH] = ACTIONS(2190),
    [anon_sym_PERCENT] = ACTIONS(2188),
    [anon_sym_LT_EQ] = ACTIONS(2188),
    [anon_sym_EQ_EQ] = ACTIONS(2190),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2188),
    [anon_sym_BANG_EQ] = ACTIONS(2190),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2188),
    [anon_sym_GT_EQ] = ACTIONS(2188),
    [anon_sym_instanceof] = ACTIONS(2188),
    [anon_sym_PLUS_PLUS] = ACTIONS(2188),
    [anon_sym_DASH_DASH] = ACTIONS(2188),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2188),
  },
  [891] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [892] = {
    [anon_sym_STAR] = ACTIONS(2210),
    [anon_sym_LBRACE] = ACTIONS(2208),
    [anon_sym_COMMA] = ACTIONS(2208),
    [anon_sym_RBRACE] = ACTIONS(2208),
    [anon_sym_LPAREN] = ACTIONS(2208),
    [anon_sym_RPAREN] = ACTIONS(2208),
    [anon_sym_in] = ACTIONS(2210),
    [anon_sym_COLON] = ACTIONS(2208),
    [anon_sym_LBRACK] = ACTIONS(2208),
    [anon_sym_RBRACK] = ACTIONS(2208),
    [anon_sym_LT] = ACTIONS(2210),
    [anon_sym_GT] = ACTIONS(2210),
    [anon_sym_SLASH] = ACTIONS(2210),
    [anon_sym_DOT] = ACTIONS(2208),
    [anon_sym_EQ] = ACTIONS(2210),
    [anon_sym_PLUS_EQ] = ACTIONS(2208),
    [anon_sym_DASH_EQ] = ACTIONS(2208),
    [anon_sym_STAR_EQ] = ACTIONS(2208),
    [anon_sym_SLASH_EQ] = ACTIONS(2208),
    [anon_sym_PERCENT_EQ] = ACTIONS(2208),
    [anon_sym_CARET_EQ] = ACTIONS(2208),
    [anon_sym_AMP_EQ] = ACTIONS(2208),
    [anon_sym_PIPE_EQ] = ACTIONS(2208),
    [anon_sym_GT_GT_EQ] = ACTIONS(2208),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(2208),
    [anon_sym_LT_LT_EQ] = ACTIONS(2208),
    [anon_sym_QMARK] = ACTIONS(2208),
    [anon_sym_AMP_AMP] = ACTIONS(2208),
    [anon_sym_PIPE_PIPE] = ACTIONS(2208),
    [anon_sym_GT_GT] = ACTIONS(2210),
    [anon_sym_GT_GT_GT] = ACTIONS(2210),
    [anon_sym_LT_LT] = ACTIONS(2210),
    [anon_sym_AMP] = ACTIONS(2210),
    [anon_sym_CARET] = ACTIONS(2210),
    [anon_sym_PIPE] = ACTIONS(2210),
    [anon_sym_PLUS] = ACTIONS(2210),
    [anon_sym_DASH] = ACTIONS(2210),
    [anon_sym_PERCENT] = ACTIONS(2210),
    [anon_sym_LT_EQ] = ACTIONS(2208),
    [anon_sym_EQ_EQ] = ACTIONS(2210),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2208),
    [anon_sym_BANG_EQ] = ACTIONS(2210),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2208),
    [anon_sym_GT_EQ] = ACTIONS(2208),
    [anon_sym_instanceof] = ACTIONS(2208),
    [anon_sym_PLUS_PLUS] = ACTIONS(2208),
    [anon_sym_DASH_DASH] = ACTIONS(2208),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2208),
  },
  [893] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2212),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [894] = {
    [anon_sym_LBRACE] = ACTIONS(2837),
    [anon_sym_EQ_GT] = ACTIONS(2837),
    [sym_comment] = ACTIONS(71),
  },
  [895] = {
    [anon_sym_export] = ACTIONS(2218),
    [anon_sym_STAR] = ACTIONS(2218),
    [anon_sym_LBRACE] = ACTIONS(2218),
    [anon_sym_COMMA] = ACTIONS(2218),
    [anon_sym_RBRACE] = ACTIONS(2218),
    [anon_sym_LPAREN] = ACTIONS(2218),
    [anon_sym_RPAREN] = ACTIONS(2218),
    [anon_sym_in] = ACTIONS(2220),
    [anon_sym_COLON] = ACTIONS(2218),
    [anon_sym_LBRACK] = ACTIONS(2218),
    [anon_sym_RBRACK] = ACTIONS(2218),
    [anon_sym_LT] = ACTIONS(2220),
    [anon_sym_GT] = ACTIONS(2220),
    [anon_sym_SLASH] = ACTIONS(2220),
    [anon_sym_DOT] = ACTIONS(2218),
    [anon_sym_class] = ACTIONS(2218),
    [anon_sym_QMARK] = ACTIONS(2218),
    [anon_sym_AMP_AMP] = ACTIONS(2218),
    [anon_sym_PIPE_PIPE] = ACTIONS(2218),
    [anon_sym_GT_GT] = ACTIONS(2220),
    [anon_sym_GT_GT_GT] = ACTIONS(2218),
    [anon_sym_LT_LT] = ACTIONS(2218),
    [anon_sym_AMP] = ACTIONS(2220),
    [anon_sym_CARET] = ACTIONS(2218),
    [anon_sym_PIPE] = ACTIONS(2220),
    [anon_sym_PLUS] = ACTIONS(2220),
    [anon_sym_DASH] = ACTIONS(2220),
    [anon_sym_PERCENT] = ACTIONS(2218),
    [anon_sym_LT_EQ] = ACTIONS(2218),
    [anon_sym_EQ_EQ] = ACTIONS(2220),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2218),
    [anon_sym_BANG_EQ] = ACTIONS(2220),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2218),
    [anon_sym_GT_EQ] = ACTIONS(2218),
    [anon_sym_instanceof] = ACTIONS(2218),
    [anon_sym_PLUS_PLUS] = ACTIONS(2218),
    [anon_sym_DASH_DASH] = ACTIONS(2218),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2218),
    [anon_sym_AT] = ACTIONS(2218),
  },
  [896] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(1343),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2839),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [897] = {
    [aux_sym_array_repeat1] = STATE(1343),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(2839),
    [sym_comment] = ACTIONS(71),
  },
  [898] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(2839),
    [sym_comment] = ACTIONS(71),
  },
  [899] = {
    [anon_sym_RBRACK] = ACTIONS(2841),
    [sym_comment] = ACTIONS(71),
  },
  [900] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2200),
    [anon_sym_LBRACE] = ACTIONS(2200),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(2202),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2202),
    [anon_sym_GT_GT_GT] = ACTIONS(2200),
    [anon_sym_LT_LT] = ACTIONS(2200),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2200),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [901] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(778),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2240),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [902] = {
    [anon_sym_RBRACE] = ACTIONS(2240),
    [anon_sym_RPAREN] = ACTIONS(2240),
    [anon_sym_COLON] = ACTIONS(2240),
    [anon_sym_RBRACK] = ACTIONS(2240),
    [sym_comment] = ACTIONS(71),
  },
  [903] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2843),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [904] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [905] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [906] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [907] = {
    [anon_sym_LBRACE] = ACTIONS(2845),
    [anon_sym_EQ_GT] = ACTIONS(2845),
    [sym_comment] = ACTIONS(71),
  },
  [908] = {
    [anon_sym_LBRACE] = ACTIONS(2847),
    [anon_sym_EQ_GT] = ACTIONS(2847),
    [sym_comment] = ACTIONS(71),
  },
  [909] = {
    [sym_nested_identifier] = STATE(1348),
    [sym_jsx_namespace_name] = STATE(1349),
    [sym_jsx_identifier] = ACTIONS(2849),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2851),
  },
  [910] = {
    [anon_sym_STAR] = ACTIONS(2324),
    [anon_sym_LBRACE] = ACTIONS(2324),
    [anon_sym_COMMA] = ACTIONS(2324),
    [anon_sym_RBRACE] = ACTIONS(2324),
    [anon_sym_LPAREN] = ACTIONS(2324),
    [anon_sym_RPAREN] = ACTIONS(2324),
    [anon_sym_in] = ACTIONS(2326),
    [anon_sym_COLON] = ACTIONS(2324),
    [anon_sym_LBRACK] = ACTIONS(2324),
    [anon_sym_RBRACK] = ACTIONS(2324),
    [anon_sym_LT] = ACTIONS(2326),
    [anon_sym_GT] = ACTIONS(2326),
    [anon_sym_SLASH] = ACTIONS(2326),
    [anon_sym_DOT] = ACTIONS(2324),
    [anon_sym_QMARK] = ACTIONS(2324),
    [anon_sym_AMP_AMP] = ACTIONS(2324),
    [anon_sym_PIPE_PIPE] = ACTIONS(2324),
    [anon_sym_GT_GT] = ACTIONS(2326),
    [anon_sym_GT_GT_GT] = ACTIONS(2324),
    [anon_sym_LT_LT] = ACTIONS(2324),
    [anon_sym_AMP] = ACTIONS(2326),
    [anon_sym_CARET] = ACTIONS(2324),
    [anon_sym_PIPE] = ACTIONS(2326),
    [anon_sym_PLUS] = ACTIONS(2326),
    [anon_sym_DASH] = ACTIONS(2326),
    [anon_sym_PERCENT] = ACTIONS(2324),
    [anon_sym_LT_EQ] = ACTIONS(2324),
    [anon_sym_EQ_EQ] = ACTIONS(2326),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2324),
    [anon_sym_BANG_EQ] = ACTIONS(2326),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2324),
    [anon_sym_GT_EQ] = ACTIONS(2324),
    [anon_sym_instanceof] = ACTIONS(2324),
    [anon_sym_PLUS_PLUS] = ACTIONS(2324),
    [anon_sym_DASH_DASH] = ACTIONS(2324),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2324),
  },
  [911] = {
    [sym_class_heritage] = STATE(1350),
    [sym_class_body] = STATE(1333),
    [anon_sym_LBRACE] = ACTIONS(704),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [912] = {
    [sym_class_body] = STATE(1351),
    [anon_sym_LBRACE] = ACTIONS(704),
    [sym_comment] = ACTIONS(71),
  },
  [913] = {
    [sym_class_heritage] = STATE(1352),
    [sym_class_body] = STATE(573),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [914] = {
    [sym_formal_parameters] = STATE(1354),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2853),
  },
  [915] = {
    [sym_formal_parameters] = STATE(1356),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2855),
  },
  [916] = {
    [sym_formal_parameters] = STATE(1354),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [917] = {
    [sym_statement_block] = STATE(591),
    [anon_sym_LBRACE] = ACTIONS(2857),
    [sym_comment] = ACTIONS(71),
  },
  [918] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2859),
  },
  [919] = {
    [sym_export_clause] = STATE(1363),
    [sym__declaration] = STATE(1364),
    [sym_variable_declaration] = STATE(1364),
    [sym_lexical_declaration] = STATE(1364),
    [sym_class] = STATE(1365),
    [sym_function] = STATE(1365),
    [sym_generator_function] = STATE(1365),
    [sym_decorator] = STATE(1366),
    [aux_sym_export_statement_repeat1] = STATE(1366),
    [anon_sym_STAR] = ACTIONS(2861),
    [anon_sym_default] = ACTIONS(2863),
    [anon_sym_LBRACE] = ACTIONS(93),
    [anon_sym_var] = ACTIONS(2865),
    [anon_sym_let] = ACTIONS(2867),
    [anon_sym_const] = ACTIONS(2867),
    [anon_sym_class] = ACTIONS(2869),
    [anon_sym_async] = ACTIONS(2871),
    [anon_sym_function] = ACTIONS(2873),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [920] = {
    [sym_export_statement] = STATE(888),
    [sym__declaration] = STATE(888),
    [sym_import_statement] = STATE(888),
    [sym_expression_statement] = STATE(888),
    [sym_variable_declaration] = STATE(888),
    [sym_lexical_declaration] = STATE(888),
    [sym_statement_block] = STATE(888),
    [sym_if_statement] = STATE(888),
    [sym_switch_statement] = STATE(888),
    [sym_for_statement] = STATE(888),
    [sym_for_in_statement] = STATE(888),
    [sym_for_of_statement] = STATE(888),
    [sym_while_statement] = STATE(888),
    [sym_do_statement] = STATE(888),
    [sym_try_statement] = STATE(888),
    [sym_with_statement] = STATE(888),
    [sym_break_statement] = STATE(888),
    [sym_continue_statement] = STATE(888),
    [sym_debugger_statement] = STATE(888),
    [sym_return_statement] = STATE(888),
    [sym_throw_statement] = STATE(888),
    [sym_empty_statement] = STATE(888),
    [sym_labeled_statement] = STATE(888),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(888),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(820),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [921] = {
    [sym_import_clause] = STATE(1368),
    [sym_namespace_import] = STATE(87),
    [sym_named_imports] = STATE(87),
    [anon_sym_STAR] = ACTIONS(133),
    [anon_sym_LBRACE] = ACTIONS(135),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2875),
    [sym_identifier] = ACTIONS(139),
  },
  [922] = {
    [sym_variable_declarator] = STATE(1369),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [923] = {
    [sym_variable_declarator] = STATE(1370),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [924] = {
    [sym_parenthesized_expression] = STATE(1371),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [925] = {
    [sym_parenthesized_expression] = STATE(1372),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [926] = {
    [anon_sym_LPAREN] = ACTIONS(2877),
    [sym_comment] = ACTIONS(71),
  },
  [927] = {
    [sym_parenthesized_expression] = STATE(1374),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [928] = {
    [sym_export_statement] = STATE(1375),
    [sym__declaration] = STATE(1375),
    [sym_import_statement] = STATE(1375),
    [sym_expression_statement] = STATE(1375),
    [sym_variable_declaration] = STATE(1375),
    [sym_lexical_declaration] = STATE(1375),
    [sym_statement_block] = STATE(1375),
    [sym_if_statement] = STATE(1375),
    [sym_switch_statement] = STATE(1375),
    [sym_for_statement] = STATE(1375),
    [sym_for_in_statement] = STATE(1375),
    [sym_for_of_statement] = STATE(1375),
    [sym_while_statement] = STATE(1375),
    [sym_do_statement] = STATE(1375),
    [sym_try_statement] = STATE(1375),
    [sym_with_statement] = STATE(1375),
    [sym_break_statement] = STATE(1375),
    [sym_continue_statement] = STATE(1375),
    [sym_debugger_statement] = STATE(1375),
    [sym_return_statement] = STATE(1375),
    [sym_throw_statement] = STATE(1375),
    [sym_empty_statement] = STATE(1375),
    [sym_labeled_statement] = STATE(1375),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [929] = {
    [sym_statement_block] = STATE(1376),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [930] = {
    [sym_parenthesized_expression] = STATE(1377),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [931] = {
    [sym__automatic_semicolon] = ACTIONS(2879),
    [anon_sym_SEMI] = ACTIONS(2879),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2881),
  },
  [932] = {
    [sym__automatic_semicolon] = ACTIONS(2883),
    [anon_sym_SEMI] = ACTIONS(2883),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2885),
  },
  [933] = {
    [sym__automatic_semicolon] = ACTIONS(2887),
    [anon_sym_SEMI] = ACTIONS(2887),
    [sym_comment] = ACTIONS(71),
  },
  [934] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(2889),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(2889),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [935] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1385),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1385),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [936] = {
    [anon_sym_else] = ACTIONS(235),
    [anon_sym_while] = ACTIONS(235),
    [sym_comment] = ACTIONS(71),
  },
  [937] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(2891),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [938] = {
    [anon_sym_else] = ACTIONS(2893),
    [anon_sym_while] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
  },
  [939] = {
    [sym__automatic_semicolon] = ACTIONS(2895),
    [anon_sym_SEMI] = ACTIONS(2895),
    [sym_comment] = ACTIONS(71),
  },
  [940] = {
    [sym__automatic_semicolon] = ACTIONS(2897),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_else] = ACTIONS(400),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(400),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [941] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2897),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_else] = ACTIONS(400),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(400),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [942] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(2899),
    [anon_sym_class] = ACTIONS(838),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [943] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(1392),
    [sym_array] = STATE(1393),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2901),
  },
  [944] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(1392),
    [sym_array] = STATE(1393),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2901),
  },
  [945] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2903),
    [anon_sym_of] = ACTIONS(2905),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [946] = {
    [sym_expression_statement] = STATE(1396),
    [sym_empty_statement] = STATE(1396),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [947] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2907),
    [anon_sym_of] = ACTIONS(2909),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [948] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(2911),
    [anon_sym_of] = ACTIONS(2913),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [949] = {
    [sym_statement_block] = STATE(966),
    [anon_sym_LBRACE] = ACTIONS(217),
    [anon_sym_LPAREN] = ACTIONS(2915),
    [sym_comment] = ACTIONS(71),
  },
  [950] = {
    [sym_statement_block] = STATE(968),
    [anon_sym_LBRACE] = ACTIONS(1586),
    [sym_comment] = ACTIONS(71),
  },
  [951] = {
    [sym_finally_clause] = STATE(969),
    [anon_sym_while] = ACTIONS(1941),
    [anon_sym_finally] = ACTIONS(1921),
    [sym_comment] = ACTIONS(71),
  },
  [952] = {
    [sym_class_body] = STATE(1402),
    [anon_sym_LBRACE] = ACTIONS(285),
    [sym_comment] = ACTIONS(71),
  },
  [953] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [anon_sym_STAR] = ACTIONS(2140),
    [anon_sym_COMMA] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_else] = ACTIONS(2140),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_RPAREN] = ACTIONS(2140),
    [anon_sym_in] = ACTIONS(2142),
    [anon_sym_while] = ACTIONS(2140),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2142),
    [anon_sym_GT] = ACTIONS(2142),
    [anon_sym_SLASH] = ACTIONS(2142),
    [anon_sym_DOT] = ACTIONS(2140),
    [anon_sym_QMARK] = ACTIONS(2140),
    [anon_sym_AMP_AMP] = ACTIONS(2140),
    [anon_sym_PIPE_PIPE] = ACTIONS(2140),
    [anon_sym_GT_GT] = ACTIONS(2142),
    [anon_sym_GT_GT_GT] = ACTIONS(2140),
    [anon_sym_LT_LT] = ACTIONS(2140),
    [anon_sym_AMP] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(2140),
    [anon_sym_PIPE] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_PERCENT] = ACTIONS(2140),
    [anon_sym_LT_EQ] = ACTIONS(2140),
    [anon_sym_EQ_EQ] = ACTIONS(2142),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2140),
    [anon_sym_BANG_EQ] = ACTIONS(2142),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2140),
    [anon_sym_GT_EQ] = ACTIONS(2140),
    [anon_sym_instanceof] = ACTIONS(2140),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2140),
  },
  [954] = {
    [sym_formal_parameters] = STATE(1403),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [955] = {
    [sym_statement_block] = STATE(1404),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [sym_comment] = ACTIONS(71),
  },
  [956] = {
    [sym_formal_parameters] = STATE(1405),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [957] = {
    [sym_statement_block] = STATE(1406),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [sym_comment] = ACTIONS(71),
  },
  [958] = {
    [sym_export_statement] = STATE(592),
    [sym__declaration] = STATE(592),
    [sym_import_statement] = STATE(592),
    [sym_expression_statement] = STATE(592),
    [sym_variable_declaration] = STATE(592),
    [sym_lexical_declaration] = STATE(592),
    [sym_statement_block] = STATE(592),
    [sym_if_statement] = STATE(592),
    [sym_switch_statement] = STATE(592),
    [sym_for_statement] = STATE(592),
    [sym_for_in_statement] = STATE(592),
    [sym_for_of_statement] = STATE(592),
    [sym_while_statement] = STATE(592),
    [sym_do_statement] = STATE(592),
    [sym_try_statement] = STATE(592),
    [sym_with_statement] = STATE(592),
    [sym_break_statement] = STATE(592),
    [sym_continue_statement] = STATE(592),
    [sym_debugger_statement] = STATE(592),
    [sym_return_statement] = STATE(592),
    [sym_throw_statement] = STATE(592),
    [sym_empty_statement] = STATE(592),
    [sym_labeled_statement] = STATE(592),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(592),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2917),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [959] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_COMMA] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_else] = ACTIONS(2178),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_RPAREN] = ACTIONS(2178),
    [anon_sym_in] = ACTIONS(2180),
    [anon_sym_while] = ACTIONS(2178),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2180),
    [anon_sym_GT] = ACTIONS(2180),
    [anon_sym_SLASH] = ACTIONS(2180),
    [anon_sym_DOT] = ACTIONS(2178),
    [anon_sym_QMARK] = ACTIONS(2178),
    [anon_sym_AMP_AMP] = ACTIONS(2178),
    [anon_sym_PIPE_PIPE] = ACTIONS(2178),
    [anon_sym_GT_GT] = ACTIONS(2180),
    [anon_sym_GT_GT_GT] = ACTIONS(2178),
    [anon_sym_LT_LT] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2180),
    [anon_sym_CARET] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_PERCENT] = ACTIONS(2178),
    [anon_sym_LT_EQ] = ACTIONS(2178),
    [anon_sym_EQ_EQ] = ACTIONS(2180),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2178),
    [anon_sym_BANG_EQ] = ACTIONS(2180),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2178),
    [anon_sym_GT_EQ] = ACTIONS(2178),
    [anon_sym_instanceof] = ACTIONS(2178),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2178),
  },
  [960] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(118),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(118),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [961] = {
    [sym__automatic_semicolon] = ACTIONS(2919),
    [anon_sym_SEMI] = ACTIONS(2919),
    [sym_comment] = ACTIONS(71),
  },
  [962] = {
    [sym_class_heritage] = STATE(1409),
    [sym_class_body] = STATE(1402),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [963] = {
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_catch] = ACTIONS(1590),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [964] = {
    [sym_export_statement] = STATE(1411),
    [sym__declaration] = STATE(1411),
    [sym_import_statement] = STATE(1411),
    [sym_expression_statement] = STATE(1411),
    [sym_variable_declaration] = STATE(1411),
    [sym_lexical_declaration] = STATE(1411),
    [sym_statement_block] = STATE(1411),
    [sym_if_statement] = STATE(1411),
    [sym_switch_statement] = STATE(1411),
    [sym_for_statement] = STATE(1411),
    [sym_for_in_statement] = STATE(1411),
    [sym_for_of_statement] = STATE(1411),
    [sym_while_statement] = STATE(1411),
    [sym_do_statement] = STATE(1411),
    [sym_try_statement] = STATE(1411),
    [sym_with_statement] = STATE(1411),
    [sym_break_statement] = STATE(1411),
    [sym_continue_statement] = STATE(1411),
    [sym_debugger_statement] = STATE(1411),
    [sym_return_statement] = STATE(1411),
    [sym_throw_statement] = STATE(1411),
    [sym_empty_statement] = STATE(1411),
    [sym_labeled_statement] = STATE(1411),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1411),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2921),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [965] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2923),
  },
  [966] = {
    [ts_builtin_sym_end] = ACTIONS(2925),
    [anon_sym_export] = ACTIONS(2927),
    [anon_sym_LBRACE] = ACTIONS(2925),
    [anon_sym_RBRACE] = ACTIONS(2925),
    [anon_sym_import] = ACTIONS(2927),
    [anon_sym_var] = ACTIONS(2927),
    [anon_sym_let] = ACTIONS(2927),
    [anon_sym_const] = ACTIONS(2927),
    [anon_sym_if] = ACTIONS(2927),
    [anon_sym_switch] = ACTIONS(2927),
    [anon_sym_for] = ACTIONS(2927),
    [anon_sym_LPAREN] = ACTIONS(2925),
    [anon_sym_while] = ACTIONS(2927),
    [anon_sym_do] = ACTIONS(2927),
    [anon_sym_try] = ACTIONS(2927),
    [anon_sym_with] = ACTIONS(2927),
    [anon_sym_break] = ACTIONS(2927),
    [anon_sym_continue] = ACTIONS(2927),
    [anon_sym_debugger] = ACTIONS(2927),
    [anon_sym_return] = ACTIONS(2927),
    [anon_sym_throw] = ACTIONS(2927),
    [anon_sym_SEMI] = ACTIONS(2925),
    [anon_sym_finally] = ACTIONS(2927),
    [anon_sym_yield] = ACTIONS(2927),
    [anon_sym_LBRACK] = ACTIONS(2925),
    [anon_sym_LT] = ACTIONS(2925),
    [anon_sym_class] = ACTIONS(2927),
    [anon_sym_async] = ACTIONS(2927),
    [anon_sym_function] = ACTIONS(2927),
    [anon_sym_EQ_GT] = ACTIONS(2925),
    [anon_sym_new] = ACTIONS(2927),
    [anon_sym_await] = ACTIONS(2927),
    [anon_sym_PLUS] = ACTIONS(2927),
    [anon_sym_DASH] = ACTIONS(2927),
    [anon_sym_BANG] = ACTIONS(2925),
    [anon_sym_TILDE] = ACTIONS(2925),
    [anon_sym_typeof] = ACTIONS(2927),
    [anon_sym_void] = ACTIONS(2927),
    [anon_sym_delete] = ACTIONS(2927),
    [anon_sym_PLUS_PLUS] = ACTIONS(2925),
    [anon_sym_DASH_DASH] = ACTIONS(2925),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2925),
    [anon_sym_BQUOTE] = ACTIONS(2925),
    [sym_regex] = ACTIONS(2927),
    [sym_number] = ACTIONS(2925),
    [sym_identifier] = ACTIONS(2927),
    [sym_this] = ACTIONS(2927),
    [sym_super] = ACTIONS(2927),
    [sym_true] = ACTIONS(2927),
    [sym_false] = ACTIONS(2927),
    [sym_null] = ACTIONS(2927),
    [sym_undefined] = ACTIONS(2927),
    [anon_sym_AT] = ACTIONS(2925),
    [anon_sym_get] = ACTIONS(2927),
    [anon_sym_set] = ACTIONS(2927),
  },
  [967] = {
    [sym_export_statement] = STATE(79),
    [sym__declaration] = STATE(79),
    [sym_import_statement] = STATE(79),
    [sym_expression_statement] = STATE(79),
    [sym_variable_declaration] = STATE(79),
    [sym_lexical_declaration] = STATE(79),
    [sym_statement_block] = STATE(79),
    [sym_if_statement] = STATE(79),
    [sym_switch_statement] = STATE(79),
    [sym_for_statement] = STATE(79),
    [sym_for_in_statement] = STATE(79),
    [sym_for_of_statement] = STATE(79),
    [sym_while_statement] = STATE(79),
    [sym_do_statement] = STATE(79),
    [sym_try_statement] = STATE(79),
    [sym_with_statement] = STATE(79),
    [sym_break_statement] = STATE(79),
    [sym_continue_statement] = STATE(79),
    [sym_debugger_statement] = STATE(79),
    [sym_return_statement] = STATE(79),
    [sym_throw_statement] = STATE(79),
    [sym_empty_statement] = STATE(79),
    [sym_labeled_statement] = STATE(79),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(79),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2929),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [968] = {
    [ts_builtin_sym_end] = ACTIONS(2931),
    [anon_sym_export] = ACTIONS(2933),
    [anon_sym_LBRACE] = ACTIONS(2931),
    [anon_sym_RBRACE] = ACTIONS(2931),
    [anon_sym_import] = ACTIONS(2933),
    [anon_sym_var] = ACTIONS(2933),
    [anon_sym_let] = ACTIONS(2933),
    [anon_sym_const] = ACTIONS(2933),
    [anon_sym_if] = ACTIONS(2933),
    [anon_sym_switch] = ACTIONS(2933),
    [anon_sym_for] = ACTIONS(2933),
    [anon_sym_LPAREN] = ACTIONS(2931),
    [anon_sym_while] = ACTIONS(2933),
    [anon_sym_do] = ACTIONS(2933),
    [anon_sym_try] = ACTIONS(2933),
    [anon_sym_with] = ACTIONS(2933),
    [anon_sym_break] = ACTIONS(2933),
    [anon_sym_continue] = ACTIONS(2933),
    [anon_sym_debugger] = ACTIONS(2933),
    [anon_sym_return] = ACTIONS(2933),
    [anon_sym_throw] = ACTIONS(2933),
    [anon_sym_SEMI] = ACTIONS(2931),
    [anon_sym_yield] = ACTIONS(2933),
    [anon_sym_LBRACK] = ACTIONS(2931),
    [anon_sym_LT] = ACTIONS(2931),
    [anon_sym_class] = ACTIONS(2933),
    [anon_sym_async] = ACTIONS(2933),
    [anon_sym_function] = ACTIONS(2933),
    [anon_sym_EQ_GT] = ACTIONS(2931),
    [anon_sym_new] = ACTIONS(2933),
    [anon_sym_await] = ACTIONS(2933),
    [anon_sym_PLUS] = ACTIONS(2933),
    [anon_sym_DASH] = ACTIONS(2933),
    [anon_sym_BANG] = ACTIONS(2931),
    [anon_sym_TILDE] = ACTIONS(2931),
    [anon_sym_typeof] = ACTIONS(2933),
    [anon_sym_void] = ACTIONS(2933),
    [anon_sym_delete] = ACTIONS(2933),
    [anon_sym_PLUS_PLUS] = ACTIONS(2931),
    [anon_sym_DASH_DASH] = ACTIONS(2931),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2931),
    [anon_sym_BQUOTE] = ACTIONS(2931),
    [sym_regex] = ACTIONS(2933),
    [sym_number] = ACTIONS(2931),
    [sym_identifier] = ACTIONS(2933),
    [sym_this] = ACTIONS(2933),
    [sym_super] = ACTIONS(2933),
    [sym_true] = ACTIONS(2933),
    [sym_false] = ACTIONS(2933),
    [sym_null] = ACTIONS(2933),
    [sym_undefined] = ACTIONS(2933),
    [anon_sym_AT] = ACTIONS(2931),
    [anon_sym_get] = ACTIONS(2933),
    [anon_sym_set] = ACTIONS(2933),
  },
  [969] = {
    [ts_builtin_sym_end] = ACTIONS(2935),
    [anon_sym_export] = ACTIONS(2937),
    [anon_sym_LBRACE] = ACTIONS(2935),
    [anon_sym_RBRACE] = ACTIONS(2935),
    [anon_sym_import] = ACTIONS(2937),
    [anon_sym_var] = ACTIONS(2937),
    [anon_sym_let] = ACTIONS(2937),
    [anon_sym_const] = ACTIONS(2937),
    [anon_sym_if] = ACTIONS(2937),
    [anon_sym_switch] = ACTIONS(2937),
    [anon_sym_for] = ACTIONS(2937),
    [anon_sym_LPAREN] = ACTIONS(2935),
    [anon_sym_while] = ACTIONS(2937),
    [anon_sym_do] = ACTIONS(2937),
    [anon_sym_try] = ACTIONS(2937),
    [anon_sym_with] = ACTIONS(2937),
    [anon_sym_break] = ACTIONS(2937),
    [anon_sym_continue] = ACTIONS(2937),
    [anon_sym_debugger] = ACTIONS(2937),
    [anon_sym_return] = ACTIONS(2937),
    [anon_sym_throw] = ACTIONS(2937),
    [anon_sym_SEMI] = ACTIONS(2935),
    [anon_sym_yield] = ACTIONS(2937),
    [anon_sym_LBRACK] = ACTIONS(2935),
    [anon_sym_LT] = ACTIONS(2935),
    [anon_sym_class] = ACTIONS(2937),
    [anon_sym_async] = ACTIONS(2937),
    [anon_sym_function] = ACTIONS(2937),
    [anon_sym_EQ_GT] = ACTIONS(2935),
    [anon_sym_new] = ACTIONS(2937),
    [anon_sym_await] = ACTIONS(2937),
    [anon_sym_PLUS] = ACTIONS(2937),
    [anon_sym_DASH] = ACTIONS(2937),
    [anon_sym_BANG] = ACTIONS(2935),
    [anon_sym_TILDE] = ACTIONS(2935),
    [anon_sym_typeof] = ACTIONS(2937),
    [anon_sym_void] = ACTIONS(2937),
    [anon_sym_delete] = ACTIONS(2937),
    [anon_sym_PLUS_PLUS] = ACTIONS(2935),
    [anon_sym_DASH_DASH] = ACTIONS(2935),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2935),
    [anon_sym_BQUOTE] = ACTIONS(2935),
    [sym_regex] = ACTIONS(2937),
    [sym_number] = ACTIONS(2935),
    [sym_identifier] = ACTIONS(2937),
    [sym_this] = ACTIONS(2937),
    [sym_super] = ACTIONS(2937),
    [sym_true] = ACTIONS(2937),
    [sym_false] = ACTIONS(2937),
    [sym_null] = ACTIONS(2937),
    [sym_undefined] = ACTIONS(2937),
    [anon_sym_AT] = ACTIONS(2935),
    [anon_sym_get] = ACTIONS(2937),
    [anon_sym_set] = ACTIONS(2937),
  },
  [970] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2164),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [971] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2166),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [972] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2170),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [973] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(1414),
    [sym__expression] = STATE(1414),
    [sym_yield_expression] = STATE(1414),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(1414),
    [sym_jsx_fragment] = STATE(1414),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1414),
    [sym_anonymous_class] = STATE(1414),
    [sym_class] = STATE(1414),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1414),
    [sym_generator_function] = STATE(1414),
    [sym_call_expression] = STATE(1414),
    [sym_new_expression] = STATE(1414),
    [sym_await_expression] = STATE(1414),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(1414),
    [sym_augmented_assignment_expression] = STATE(1414),
    [sym_ternary_expression] = STATE(1414),
    [sym_binary_expression] = STATE(1414),
    [sym_unary_expression] = STATE(1414),
    [sym_update_expression] = STATE(1414),
    [sym_template_string] = STATE(1414),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2939),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(2941),
    [sym_number] = ACTIONS(2939),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(2941),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(2941),
    [sym_false] = ACTIONS(2941),
    [sym_null] = ACTIONS(2941),
    [sym_undefined] = ACTIONS(2941),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [974] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [975] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2212),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [976] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2943),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [977] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [978] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [979] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [980] = {
    [sym__automatic_semicolon] = ACTIONS(2945),
    [anon_sym_STAR] = ACTIONS(2945),
    [anon_sym_COMMA] = ACTIONS(2945),
    [anon_sym_RBRACE] = ACTIONS(2945),
    [anon_sym_LPAREN] = ACTIONS(2945),
    [anon_sym_RPAREN] = ACTIONS(2945),
    [anon_sym_in] = ACTIONS(2947),
    [anon_sym_of] = ACTIONS(2945),
    [anon_sym_SEMI] = ACTIONS(2945),
    [anon_sym_LBRACK] = ACTIONS(2945),
    [anon_sym_LT] = ACTIONS(2947),
    [anon_sym_GT] = ACTIONS(2947),
    [anon_sym_SLASH] = ACTIONS(2947),
    [anon_sym_DOT] = ACTIONS(2945),
    [anon_sym_EQ] = ACTIONS(2947),
    [anon_sym_QMARK] = ACTIONS(2945),
    [anon_sym_AMP_AMP] = ACTIONS(2945),
    [anon_sym_PIPE_PIPE] = ACTIONS(2945),
    [anon_sym_GT_GT] = ACTIONS(2947),
    [anon_sym_GT_GT_GT] = ACTIONS(2945),
    [anon_sym_LT_LT] = ACTIONS(2945),
    [anon_sym_AMP] = ACTIONS(2947),
    [anon_sym_CARET] = ACTIONS(2945),
    [anon_sym_PIPE] = ACTIONS(2947),
    [anon_sym_PLUS] = ACTIONS(2947),
    [anon_sym_DASH] = ACTIONS(2947),
    [anon_sym_PERCENT] = ACTIONS(2945),
    [anon_sym_LT_EQ] = ACTIONS(2945),
    [anon_sym_EQ_EQ] = ACTIONS(2947),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2945),
    [anon_sym_BANG_EQ] = ACTIONS(2947),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2945),
    [anon_sym_GT_EQ] = ACTIONS(2945),
    [anon_sym_instanceof] = ACTIONS(2945),
    [anon_sym_PLUS_PLUS] = ACTIONS(2945),
    [anon_sym_DASH_DASH] = ACTIONS(2945),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2945),
  },
  [981] = {
    [anon_sym_GT] = ACTIONS(2949),
    [sym_comment] = ACTIONS(71),
  },
  [982] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(2951),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [983] = {
    [anon_sym_LBRACE] = ACTIONS(2953),
    [anon_sym_GT] = ACTIONS(2953),
    [anon_sym_SLASH] = ACTIONS(2286),
    [sym_jsx_identifier] = ACTIONS(2953),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2286),
  },
  [984] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2186),
    [anon_sym_RBRACE] = ACTIONS(2955),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [985] = {
    [anon_sym_RBRACE] = ACTIONS(2955),
    [sym_comment] = ACTIONS(71),
  },
  [986] = {
    [anon_sym_LBRACE] = ACTIONS(2957),
    [anon_sym_GT] = ACTIONS(2957),
    [anon_sym_SLASH] = ACTIONS(2959),
    [sym_jsx_identifier] = ACTIONS(2957),
    [anon_sym_EQ] = ACTIONS(2957),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2959),
  },
  [987] = {
    [anon_sym_LBRACE] = ACTIONS(2961),
    [anon_sym_GT] = ACTIONS(2961),
    [anon_sym_SLASH] = ACTIONS(2963),
    [sym_jsx_identifier] = ACTIONS(2961),
    [anon_sym_EQ] = ACTIONS(2961),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2963),
  },
  [988] = {
    [sym__automatic_semicolon] = ACTIONS(2965),
    [anon_sym_STAR] = ACTIONS(2965),
    [anon_sym_COMMA] = ACTIONS(2965),
    [anon_sym_RBRACE] = ACTIONS(2965),
    [anon_sym_LPAREN] = ACTIONS(2965),
    [anon_sym_RPAREN] = ACTIONS(2965),
    [anon_sym_in] = ACTIONS(2967),
    [anon_sym_SEMI] = ACTIONS(2965),
    [anon_sym_LBRACK] = ACTIONS(2965),
    [anon_sym_LT] = ACTIONS(2967),
    [anon_sym_GT] = ACTIONS(2967),
    [anon_sym_SLASH] = ACTIONS(2967),
    [anon_sym_DOT] = ACTIONS(2965),
    [anon_sym_QMARK] = ACTIONS(2965),
    [anon_sym_AMP_AMP] = ACTIONS(2965),
    [anon_sym_PIPE_PIPE] = ACTIONS(2965),
    [anon_sym_GT_GT] = ACTIONS(2967),
    [anon_sym_GT_GT_GT] = ACTIONS(2965),
    [anon_sym_LT_LT] = ACTIONS(2965),
    [anon_sym_AMP] = ACTIONS(2967),
    [anon_sym_CARET] = ACTIONS(2965),
    [anon_sym_PIPE] = ACTIONS(2967),
    [anon_sym_PLUS] = ACTIONS(2967),
    [anon_sym_DASH] = ACTIONS(2967),
    [anon_sym_PERCENT] = ACTIONS(2965),
    [anon_sym_LT_EQ] = ACTIONS(2965),
    [anon_sym_EQ_EQ] = ACTIONS(2967),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2965),
    [anon_sym_BANG_EQ] = ACTIONS(2967),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2965),
    [anon_sym_GT_EQ] = ACTIONS(2965),
    [anon_sym_instanceof] = ACTIONS(2965),
    [anon_sym_PLUS_PLUS] = ACTIONS(2965),
    [anon_sym_DASH_DASH] = ACTIONS(2965),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2965),
  },
  [989] = {
    [sym_jsx_element] = STATE(1420),
    [sym_jsx_fragment] = STATE(1420),
    [sym_jsx_expression] = STATE(1420),
    [sym_jsx_opening_element] = STATE(1421),
    [sym_jsx_self_closing_element] = STATE(1420),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_LT] = ACTIONS(2969),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2971),
  },
  [990] = {
    [sym_jsx_element] = STATE(1422),
    [sym_jsx_fragment] = STATE(1422),
    [sym_jsx_expression] = STATE(1422),
    [sym_jsx_opening_element] = STATE(1421),
    [sym_jsx_self_closing_element] = STATE(1422),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_LT] = ACTIONS(2969),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2973),
  },
  [991] = {
    [anon_sym_LBRACE] = ACTIONS(2975),
    [anon_sym_LT] = ACTIONS(2975),
    [sym_jsx_text] = ACTIONS(2975),
    [sym_comment] = ACTIONS(71),
  },
  [992] = {
    [anon_sym_GT] = ACTIONS(2977),
    [sym_comment] = ACTIONS(71),
  },
  [993] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(2979),
    [anon_sym_GT] = ACTIONS(2982),
    [anon_sym_SLASH] = ACTIONS(2984),
    [sym_jsx_identifier] = ACTIONS(2986),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2989),
  },
  [994] = {
    [anon_sym_LBRACE] = ACTIONS(2992),
    [anon_sym_GT] = ACTIONS(2992),
    [anon_sym_SLASH] = ACTIONS(2994),
    [sym_jsx_identifier] = ACTIONS(2992),
    [anon_sym_EQ] = ACTIONS(2992),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2994),
  },
  [995] = {
    [anon_sym_LBRACE] = ACTIONS(2996),
    [anon_sym_GT] = ACTIONS(2996),
    [anon_sym_SLASH] = ACTIONS(2998),
    [sym_jsx_identifier] = ACTIONS(2996),
    [anon_sym_EQ] = ACTIONS(2996),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2998),
  },
  [996] = {
    [sym__automatic_semicolon] = ACTIONS(3000),
    [anon_sym_STAR] = ACTIONS(3000),
    [anon_sym_COMMA] = ACTIONS(3000),
    [anon_sym_RBRACE] = ACTIONS(3000),
    [anon_sym_LPAREN] = ACTIONS(3000),
    [anon_sym_RPAREN] = ACTIONS(3000),
    [anon_sym_in] = ACTIONS(3002),
    [anon_sym_SEMI] = ACTIONS(3000),
    [anon_sym_LBRACK] = ACTIONS(3000),
    [anon_sym_LT] = ACTIONS(3002),
    [anon_sym_GT] = ACTIONS(3002),
    [anon_sym_SLASH] = ACTIONS(3002),
    [anon_sym_DOT] = ACTIONS(3000),
    [anon_sym_QMARK] = ACTIONS(3000),
    [anon_sym_AMP_AMP] = ACTIONS(3000),
    [anon_sym_PIPE_PIPE] = ACTIONS(3000),
    [anon_sym_GT_GT] = ACTIONS(3002),
    [anon_sym_GT_GT_GT] = ACTIONS(3000),
    [anon_sym_LT_LT] = ACTIONS(3000),
    [anon_sym_AMP] = ACTIONS(3002),
    [anon_sym_CARET] = ACTIONS(3000),
    [anon_sym_PIPE] = ACTIONS(3002),
    [anon_sym_PLUS] = ACTIONS(3002),
    [anon_sym_DASH] = ACTIONS(3002),
    [anon_sym_PERCENT] = ACTIONS(3000),
    [anon_sym_LT_EQ] = ACTIONS(3000),
    [anon_sym_EQ_EQ] = ACTIONS(3002),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3000),
    [anon_sym_BANG_EQ] = ACTIONS(3002),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3000),
    [anon_sym_GT_EQ] = ACTIONS(3000),
    [anon_sym_instanceof] = ACTIONS(3000),
    [anon_sym_PLUS_PLUS] = ACTIONS(3000),
    [anon_sym_DASH_DASH] = ACTIONS(3000),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3000),
  },
  [997] = {
    [anon_sym_LBRACE] = ACTIONS(3004),
    [anon_sym_GT] = ACTIONS(3004),
    [anon_sym_SLASH] = ACTIONS(3006),
    [sym_jsx_identifier] = ACTIONS(3004),
    [anon_sym_DOT] = ACTIONS(3004),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3006),
  },
  [998] = {
    [anon_sym_LBRACE] = ACTIONS(3008),
    [anon_sym_LT] = ACTIONS(3008),
    [sym_jsx_text] = ACTIONS(3008),
    [sym_comment] = ACTIONS(71),
  },
  [999] = {
    [anon_sym_GT] = ACTIONS(3010),
    [sym_comment] = ACTIONS(71),
  },
  [1000] = {
    [sym_formal_parameters] = STATE(1425),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1001] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(3012),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1002] = {
    [sym__property_name] = STATE(1427),
    [sym_computed_property_name] = STATE(1427),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3014),
    [sym_number] = ACTIONS(3014),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [1003] = {
    [sym__property_name] = STATE(1427),
    [sym_computed_property_name] = STATE(1427),
    [anon_sym_LPAREN] = ACTIONS(503),
    [anon_sym_LBRACK] = ACTIONS(458),
    [anon_sym_async] = ACTIONS(460),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3014),
    [sym_number] = ACTIONS(3014),
    [sym_identifier] = ACTIONS(460),
    [anon_sym_get] = ACTIONS(460),
    [anon_sym_set] = ACTIONS(460),
  },
  [1004] = {
    [anon_sym_STAR] = ACTIONS(1110),
    [anon_sym_LPAREN] = ACTIONS(1110),
    [anon_sym_LBRACK] = ACTIONS(1110),
    [anon_sym_DOT] = ACTIONS(1552),
    [anon_sym_async] = ACTIONS(1552),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1110),
    [sym_number] = ACTIONS(1110),
    [sym_identifier] = ACTIONS(1552),
    [anon_sym_AT] = ACTIONS(1110),
    [anon_sym_static] = ACTIONS(1552),
    [anon_sym_get] = ACTIONS(1552),
    [anon_sym_set] = ACTIONS(1552),
  },
  [1005] = {
    [sym_arguments] = STATE(1430),
    [anon_sym_STAR] = ACTIONS(1112),
    [anon_sym_LPAREN] = ACTIONS(3016),
    [anon_sym_LBRACK] = ACTIONS(1112),
    [anon_sym_DOT] = ACTIONS(3018),
    [anon_sym_async] = ACTIONS(1554),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1112),
    [sym_number] = ACTIONS(1112),
    [sym_identifier] = ACTIONS(1554),
    [anon_sym_AT] = ACTIONS(1112),
    [anon_sym_static] = ACTIONS(1554),
    [anon_sym_get] = ACTIONS(1554),
    [anon_sym_set] = ACTIONS(1554),
  },
  [1006] = {
    [sym_arguments] = STATE(1432),
    [anon_sym_STAR] = ACTIONS(1116),
    [anon_sym_LPAREN] = ACTIONS(3016),
    [anon_sym_LBRACK] = ACTIONS(1116),
    [anon_sym_DOT] = ACTIONS(3020),
    [anon_sym_async] = ACTIONS(1560),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1116),
    [sym_number] = ACTIONS(1116),
    [sym_identifier] = ACTIONS(1560),
    [anon_sym_AT] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1560),
    [anon_sym_get] = ACTIONS(1560),
    [anon_sym_set] = ACTIONS(1560),
  },
  [1007] = {
    [anon_sym_STAR] = ACTIONS(1120),
    [anon_sym_LBRACK] = ACTIONS(1120),
    [anon_sym_async] = ACTIONS(1564),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1120),
    [sym_number] = ACTIONS(1120),
    [sym_identifier] = ACTIONS(1564),
    [anon_sym_AT] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1564),
    [anon_sym_get] = ACTIONS(1564),
    [anon_sym_set] = ACTIONS(1564),
  },
  [1008] = {
    [sym__automatic_semicolon] = ACTIONS(3022),
    [anon_sym_SEMI] = ACTIONS(3022),
    [sym_comment] = ACTIONS(71),
  },
  [1009] = {
    [anon_sym_STAR] = ACTIONS(3024),
    [anon_sym_RBRACE] = ACTIONS(3024),
    [anon_sym_SEMI] = ACTIONS(3022),
    [anon_sym_LBRACK] = ACTIONS(3024),
    [anon_sym_async] = ACTIONS(3026),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3024),
    [sym_number] = ACTIONS(3024),
    [sym_identifier] = ACTIONS(3026),
    [anon_sym_AT] = ACTIONS(3024),
    [anon_sym_static] = ACTIONS(3026),
    [anon_sym_get] = ACTIONS(3026),
    [anon_sym_set] = ACTIONS(3026),
  },
  [1010] = {
    [anon_sym_STAR] = ACTIONS(3024),
    [anon_sym_RBRACE] = ACTIONS(3024),
    [anon_sym_LBRACK] = ACTIONS(3024),
    [anon_sym_async] = ACTIONS(3026),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3024),
    [sym_number] = ACTIONS(3024),
    [sym_identifier] = ACTIONS(3026),
    [anon_sym_AT] = ACTIONS(3024),
    [anon_sym_static] = ACTIONS(3026),
    [anon_sym_get] = ACTIONS(3026),
    [anon_sym_set] = ACTIONS(3026),
  },
  [1011] = {
    [sym__automatic_semicolon] = ACTIONS(3028),
    [anon_sym_SEMI] = ACTIONS(3028),
    [sym_comment] = ACTIONS(71),
  },
  [1012] = {
    [sym_statement_block] = STATE(767),
    [anon_sym_LBRACE] = ACTIONS(3030),
    [sym_comment] = ACTIONS(71),
  },
  [1013] = {
    [sym_public_field_definition] = STATE(1435),
    [sym_method_definition] = STATE(1436),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1014] = {
    [sym_decorator] = STATE(1014),
    [aux_sym_export_statement_repeat1] = STATE(1014),
    [anon_sym_STAR] = ACTIONS(1326),
    [anon_sym_LBRACK] = ACTIONS(1326),
    [anon_sym_async] = ACTIONS(1609),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1326),
    [sym_number] = ACTIONS(1326),
    [sym_identifier] = ACTIONS(1609),
    [anon_sym_AT] = ACTIONS(3032),
    [anon_sym_static] = ACTIONS(1609),
    [anon_sym_get] = ACTIONS(1609),
    [anon_sym_set] = ACTIONS(1609),
  },
  [1015] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [anon_sym_STAR] = ACTIONS(3035),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_else] = ACTIONS(3035),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_RPAREN] = ACTIONS(3035),
    [anon_sym_in] = ACTIONS(3037),
    [anon_sym_while] = ACTIONS(3035),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3037),
    [anon_sym_GT] = ACTIONS(3037),
    [anon_sym_SLASH] = ACTIONS(3037),
    [anon_sym_DOT] = ACTIONS(3035),
    [anon_sym_QMARK] = ACTIONS(3035),
    [anon_sym_AMP_AMP] = ACTIONS(3035),
    [anon_sym_PIPE_PIPE] = ACTIONS(3035),
    [anon_sym_GT_GT] = ACTIONS(3037),
    [anon_sym_GT_GT_GT] = ACTIONS(3035),
    [anon_sym_LT_LT] = ACTIONS(3035),
    [anon_sym_AMP] = ACTIONS(3037),
    [anon_sym_CARET] = ACTIONS(3035),
    [anon_sym_PIPE] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_PERCENT] = ACTIONS(3035),
    [anon_sym_LT_EQ] = ACTIONS(3035),
    [anon_sym_EQ_EQ] = ACTIONS(3037),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3035),
    [anon_sym_BANG_EQ] = ACTIONS(3037),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3035),
    [anon_sym_GT_EQ] = ACTIONS(3035),
    [anon_sym_instanceof] = ACTIONS(3035),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3035),
  },
  [1016] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(3039),
    [anon_sym_RBRACE] = ACTIONS(3024),
    [anon_sym_LBRACK] = ACTIONS(3042),
    [anon_sym_async] = ACTIONS(3045),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3048),
    [sym_number] = ACTIONS(3048),
    [sym_identifier] = ACTIONS(3051),
    [anon_sym_AT] = ACTIONS(3054),
    [anon_sym_static] = ACTIONS(3057),
    [anon_sym_get] = ACTIONS(3060),
    [anon_sym_set] = ACTIONS(3060),
  },
  [1017] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1018] = {
    [sym_parenthesized_expression] = STATE(1437),
    [sym__expression] = STATE(1437),
    [sym_yield_expression] = STATE(1437),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1437),
    [sym_jsx_fragment] = STATE(1437),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1437),
    [sym_anonymous_class] = STATE(1437),
    [sym_class] = STATE(1437),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1437),
    [sym_generator_function] = STATE(1437),
    [sym_call_expression] = STATE(1437),
    [sym_new_expression] = STATE(1437),
    [sym_await_expression] = STATE(1437),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1437),
    [sym_augmented_assignment_expression] = STATE(1437),
    [sym_ternary_expression] = STATE(1437),
    [sym_binary_expression] = STATE(1437),
    [sym_unary_expression] = STATE(1437),
    [sym_update_expression] = STATE(1437),
    [sym_template_string] = STATE(1437),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3063),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3065),
    [sym_number] = ACTIONS(3063),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3065),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3065),
    [sym_false] = ACTIONS(3065),
    [sym_null] = ACTIONS(3065),
    [sym_undefined] = ACTIONS(3065),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1019] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1438),
    [sym__expression] = STATE(1438),
    [sym_yield_expression] = STATE(1438),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1438),
    [sym_jsx_fragment] = STATE(1438),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1438),
    [sym_anonymous_class] = STATE(1438),
    [sym_class] = STATE(1438),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1438),
    [sym_generator_function] = STATE(1438),
    [sym_call_expression] = STATE(1438),
    [sym_new_expression] = STATE(1438),
    [sym_await_expression] = STATE(1438),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1438),
    [sym_augmented_assignment_expression] = STATE(1438),
    [sym_ternary_expression] = STATE(1438),
    [sym_binary_expression] = STATE(1438),
    [sym_unary_expression] = STATE(1438),
    [sym_update_expression] = STATE(1438),
    [sym_template_string] = STATE(1438),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3067),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3069),
    [sym_number] = ACTIONS(3067),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3069),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3069),
    [sym_false] = ACTIONS(3069),
    [sym_null] = ACTIONS(3069),
    [sym_undefined] = ACTIONS(3069),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1020] = {
    [sym_parenthesized_expression] = STATE(1439),
    [sym__expression] = STATE(1439),
    [sym_yield_expression] = STATE(1439),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1439),
    [sym_jsx_fragment] = STATE(1439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1439),
    [sym_anonymous_class] = STATE(1439),
    [sym_class] = STATE(1439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1439),
    [sym_generator_function] = STATE(1439),
    [sym_call_expression] = STATE(1439),
    [sym_new_expression] = STATE(1439),
    [sym_await_expression] = STATE(1439),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1439),
    [sym_augmented_assignment_expression] = STATE(1439),
    [sym_ternary_expression] = STATE(1439),
    [sym_binary_expression] = STATE(1439),
    [sym_unary_expression] = STATE(1439),
    [sym_update_expression] = STATE(1439),
    [sym_template_string] = STATE(1439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3071),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3073),
    [sym_number] = ACTIONS(3071),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3073),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3073),
    [sym_false] = ACTIONS(3073),
    [sym_null] = ACTIONS(3073),
    [sym_undefined] = ACTIONS(3073),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1021] = {
    [anon_sym_EQ_GT] = ACTIONS(3075),
    [sym_comment] = ACTIONS(71),
  },
  [1022] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1023] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1024] = {
    [sym_parenthesized_expression] = STATE(1441),
    [sym__expression] = STATE(1441),
    [sym_yield_expression] = STATE(1441),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1441),
    [sym_jsx_fragment] = STATE(1441),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1441),
    [sym_anonymous_class] = STATE(1441),
    [sym_class] = STATE(1441),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1441),
    [sym_generator_function] = STATE(1441),
    [sym_call_expression] = STATE(1441),
    [sym_new_expression] = STATE(1441),
    [sym_await_expression] = STATE(1441),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1441),
    [sym_augmented_assignment_expression] = STATE(1441),
    [sym_ternary_expression] = STATE(1441),
    [sym_binary_expression] = STATE(1441),
    [sym_unary_expression] = STATE(1441),
    [sym_update_expression] = STATE(1441),
    [sym_template_string] = STATE(1441),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3077),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3079),
    [sym_number] = ACTIONS(3077),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3079),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3079),
    [sym_false] = ACTIONS(3079),
    [sym_null] = ACTIONS(3079),
    [sym_undefined] = ACTIONS(3079),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1025] = {
    [sym_parenthesized_expression] = STATE(1442),
    [sym__expression] = STATE(1442),
    [sym_yield_expression] = STATE(1442),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1442),
    [sym_jsx_fragment] = STATE(1442),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1442),
    [sym_anonymous_class] = STATE(1442),
    [sym_class] = STATE(1442),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1442),
    [sym_generator_function] = STATE(1442),
    [sym_call_expression] = STATE(1442),
    [sym_new_expression] = STATE(1442),
    [sym_await_expression] = STATE(1442),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1442),
    [sym_augmented_assignment_expression] = STATE(1442),
    [sym_ternary_expression] = STATE(1442),
    [sym_binary_expression] = STATE(1442),
    [sym_unary_expression] = STATE(1442),
    [sym_update_expression] = STATE(1442),
    [sym_template_string] = STATE(1442),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3081),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3083),
    [sym_number] = ACTIONS(3081),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3083),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3083),
    [sym_false] = ACTIONS(3083),
    [sym_null] = ACTIONS(3083),
    [sym_undefined] = ACTIONS(3083),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1026] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1027] = {
    [sym_parenthesized_expression] = STATE(1443),
    [sym__expression] = STATE(1443),
    [sym_yield_expression] = STATE(1443),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1443),
    [sym_jsx_fragment] = STATE(1443),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1443),
    [sym_anonymous_class] = STATE(1443),
    [sym_class] = STATE(1443),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1443),
    [sym_generator_function] = STATE(1443),
    [sym_call_expression] = STATE(1443),
    [sym_new_expression] = STATE(1443),
    [sym_await_expression] = STATE(1443),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1443),
    [sym_augmented_assignment_expression] = STATE(1443),
    [sym_ternary_expression] = STATE(1443),
    [sym_binary_expression] = STATE(1443),
    [sym_unary_expression] = STATE(1443),
    [sym_update_expression] = STATE(1443),
    [sym_template_string] = STATE(1443),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3085),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3087),
    [sym_number] = ACTIONS(3085),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3087),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3087),
    [sym_false] = ACTIONS(3087),
    [sym_null] = ACTIONS(3087),
    [sym_undefined] = ACTIONS(3087),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1028] = {
    [sym_parenthesized_expression] = STATE(1444),
    [sym__expression] = STATE(1444),
    [sym_yield_expression] = STATE(1444),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1444),
    [sym_jsx_fragment] = STATE(1444),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1444),
    [sym_anonymous_class] = STATE(1444),
    [sym_class] = STATE(1444),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1444),
    [sym_generator_function] = STATE(1444),
    [sym_call_expression] = STATE(1444),
    [sym_new_expression] = STATE(1444),
    [sym_await_expression] = STATE(1444),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1444),
    [sym_augmented_assignment_expression] = STATE(1444),
    [sym_ternary_expression] = STATE(1444),
    [sym_binary_expression] = STATE(1444),
    [sym_unary_expression] = STATE(1444),
    [sym_update_expression] = STATE(1444),
    [sym_template_string] = STATE(1444),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3089),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3091),
    [sym_number] = ACTIONS(3089),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3091),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3091),
    [sym_false] = ACTIONS(3091),
    [sym_null] = ACTIONS(3091),
    [sym_undefined] = ACTIONS(3091),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1029] = {
    [sym_parenthesized_expression] = STATE(1445),
    [sym__expression] = STATE(1445),
    [sym_yield_expression] = STATE(1445),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1445),
    [sym_jsx_fragment] = STATE(1445),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1445),
    [sym_anonymous_class] = STATE(1445),
    [sym_class] = STATE(1445),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1445),
    [sym_generator_function] = STATE(1445),
    [sym_call_expression] = STATE(1445),
    [sym_new_expression] = STATE(1445),
    [sym_await_expression] = STATE(1445),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1445),
    [sym_augmented_assignment_expression] = STATE(1445),
    [sym_ternary_expression] = STATE(1445),
    [sym_binary_expression] = STATE(1445),
    [sym_unary_expression] = STATE(1445),
    [sym_update_expression] = STATE(1445),
    [sym_template_string] = STATE(1445),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3093),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3095),
    [sym_number] = ACTIONS(3093),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3095),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3095),
    [sym_false] = ACTIONS(3095),
    [sym_null] = ACTIONS(3095),
    [sym_undefined] = ACTIONS(3095),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1030] = {
    [sym_parenthesized_expression] = STATE(1446),
    [sym__expression] = STATE(1446),
    [sym_yield_expression] = STATE(1446),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1446),
    [sym_jsx_fragment] = STATE(1446),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1446),
    [sym_anonymous_class] = STATE(1446),
    [sym_class] = STATE(1446),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1446),
    [sym_generator_function] = STATE(1446),
    [sym_call_expression] = STATE(1446),
    [sym_new_expression] = STATE(1446),
    [sym_await_expression] = STATE(1446),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1446),
    [sym_augmented_assignment_expression] = STATE(1446),
    [sym_ternary_expression] = STATE(1446),
    [sym_binary_expression] = STATE(1446),
    [sym_unary_expression] = STATE(1446),
    [sym_update_expression] = STATE(1446),
    [sym_template_string] = STATE(1446),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3097),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3099),
    [sym_number] = ACTIONS(3097),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3099),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3099),
    [sym_false] = ACTIONS(3099),
    [sym_null] = ACTIONS(3099),
    [sym_undefined] = ACTIONS(3099),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1031] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [ts_builtin_sym_end] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_STAR] = ACTIONS(2064),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_COMMA] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_in] = ACTIONS(2066),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2066),
    [anon_sym_GT] = ACTIONS(2066),
    [anon_sym_SLASH] = ACTIONS(2066),
    [anon_sym_DOT] = ACTIONS(2066),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_QMARK] = ACTIONS(2064),
    [anon_sym_AMP_AMP] = ACTIONS(2064),
    [anon_sym_PIPE_PIPE] = ACTIONS(2064),
    [anon_sym_GT_GT] = ACTIONS(2066),
    [anon_sym_GT_GT_GT] = ACTIONS(2064),
    [anon_sym_LT_LT] = ACTIONS(2064),
    [anon_sym_AMP] = ACTIONS(2066),
    [anon_sym_CARET] = ACTIONS(2064),
    [anon_sym_PIPE] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_PERCENT] = ACTIONS(2064),
    [anon_sym_LT_EQ] = ACTIONS(2064),
    [anon_sym_EQ_EQ] = ACTIONS(2066),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2064),
    [anon_sym_BANG_EQ] = ACTIONS(2066),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2064),
    [anon_sym_GT_EQ] = ACTIONS(2064),
    [anon_sym_instanceof] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2066),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [1032] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(3101),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1033] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [ts_builtin_sym_end] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_STAR] = ACTIONS(3103),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_COMMA] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_in] = ACTIONS(3105),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3105),
    [anon_sym_GT] = ACTIONS(3105),
    [anon_sym_SLASH] = ACTIONS(3105),
    [anon_sym_DOT] = ACTIONS(3105),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3103),
    [anon_sym_AMP_AMP] = ACTIONS(3103),
    [anon_sym_PIPE_PIPE] = ACTIONS(3103),
    [anon_sym_GT_GT] = ACTIONS(3105),
    [anon_sym_GT_GT_GT] = ACTIONS(3103),
    [anon_sym_LT_LT] = ACTIONS(3103),
    [anon_sym_AMP] = ACTIONS(3105),
    [anon_sym_CARET] = ACTIONS(3103),
    [anon_sym_PIPE] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_PERCENT] = ACTIONS(3103),
    [anon_sym_LT_EQ] = ACTIONS(3103),
    [anon_sym_EQ_EQ] = ACTIONS(3105),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3103),
    [anon_sym_BANG_EQ] = ACTIONS(3105),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3103),
    [anon_sym_GT_EQ] = ACTIONS(3103),
    [anon_sym_instanceof] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3105),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [1034] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(1146),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2449),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1035] = {
    [sym_statement_block] = STATE(1448),
    [anon_sym_LBRACE] = ACTIONS(1046),
    [sym_comment] = ACTIONS(71),
  },
  [1036] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [ts_builtin_sym_end] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_STAR] = ACTIONS(3107),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_COMMA] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_in] = ACTIONS(3109),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3109),
    [anon_sym_GT] = ACTIONS(3109),
    [anon_sym_SLASH] = ACTIONS(3109),
    [anon_sym_DOT] = ACTIONS(3109),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_AMP_AMP] = ACTIONS(3107),
    [anon_sym_PIPE_PIPE] = ACTIONS(3107),
    [anon_sym_GT_GT] = ACTIONS(3109),
    [anon_sym_GT_GT_GT] = ACTIONS(3107),
    [anon_sym_LT_LT] = ACTIONS(3107),
    [anon_sym_AMP] = ACTIONS(3109),
    [anon_sym_CARET] = ACTIONS(3107),
    [anon_sym_PIPE] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_PERCENT] = ACTIONS(3107),
    [anon_sym_LT_EQ] = ACTIONS(3107),
    [anon_sym_EQ_EQ] = ACTIONS(3109),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3107),
    [anon_sym_BANG_EQ] = ACTIONS(3109),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3107),
    [anon_sym_GT_EQ] = ACTIONS(3107),
    [anon_sym_instanceof] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3109),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [1037] = {
    [sym__automatic_semicolon] = ACTIONS(3111),
    [anon_sym_STAR] = ACTIONS(3111),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_RBRACE] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(3111),
    [anon_sym_RPAREN] = ACTIONS(3111),
    [anon_sym_in] = ACTIONS(3113),
    [anon_sym_SEMI] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(3111),
    [anon_sym_LT] = ACTIONS(3113),
    [anon_sym_GT] = ACTIONS(3113),
    [anon_sym_SLASH] = ACTIONS(3113),
    [anon_sym_DOT] = ACTIONS(3111),
    [anon_sym_QMARK] = ACTIONS(3111),
    [anon_sym_AMP_AMP] = ACTIONS(3111),
    [anon_sym_PIPE_PIPE] = ACTIONS(3111),
    [anon_sym_GT_GT] = ACTIONS(3113),
    [anon_sym_GT_GT_GT] = ACTIONS(3111),
    [anon_sym_LT_LT] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_CARET] = ACTIONS(3111),
    [anon_sym_PIPE] = ACTIONS(3113),
    [anon_sym_PLUS] = ACTIONS(3113),
    [anon_sym_DASH] = ACTIONS(3113),
    [anon_sym_PERCENT] = ACTIONS(3111),
    [anon_sym_LT_EQ] = ACTIONS(3111),
    [anon_sym_EQ_EQ] = ACTIONS(3113),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3111),
    [anon_sym_BANG_EQ] = ACTIONS(3113),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3111),
    [anon_sym_GT_EQ] = ACTIONS(3111),
    [anon_sym_instanceof] = ACTIONS(3111),
    [anon_sym_PLUS_PLUS] = ACTIONS(3111),
    [anon_sym_DASH_DASH] = ACTIONS(3111),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3111),
  },
  [1038] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3111),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1039] = {
    [sym_statement_block] = STATE(1449),
    [anon_sym_LBRACE] = ACTIONS(1046),
    [sym_comment] = ACTIONS(71),
  },
  [1040] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [ts_builtin_sym_end] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_STAR] = ACTIONS(3115),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_COMMA] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_in] = ACTIONS(3117),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3117),
    [anon_sym_GT] = ACTIONS(3117),
    [anon_sym_SLASH] = ACTIONS(3117),
    [anon_sym_DOT] = ACTIONS(3117),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_QMARK] = ACTIONS(3115),
    [anon_sym_AMP_AMP] = ACTIONS(3115),
    [anon_sym_PIPE_PIPE] = ACTIONS(3115),
    [anon_sym_GT_GT] = ACTIONS(3117),
    [anon_sym_GT_GT_GT] = ACTIONS(3115),
    [anon_sym_LT_LT] = ACTIONS(3115),
    [anon_sym_AMP] = ACTIONS(3117),
    [anon_sym_CARET] = ACTIONS(3115),
    [anon_sym_PIPE] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_PERCENT] = ACTIONS(3115),
    [anon_sym_LT_EQ] = ACTIONS(3115),
    [anon_sym_EQ_EQ] = ACTIONS(3117),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3115),
    [anon_sym_BANG_EQ] = ACTIONS(3117),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3115),
    [anon_sym_GT_EQ] = ACTIONS(3115),
    [anon_sym_instanceof] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3117),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [1041] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_in] = ACTIONS(484),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(484),
    [anon_sym_GT] = ACTIONS(484),
    [anon_sym_SLASH] = ACTIONS(484),
    [anon_sym_DOT] = ACTIONS(484),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(482),
    [anon_sym_AMP_AMP] = ACTIONS(482),
    [anon_sym_PIPE_PIPE] = ACTIONS(482),
    [anon_sym_GT_GT] = ACTIONS(484),
    [anon_sym_GT_GT_GT] = ACTIONS(482),
    [anon_sym_LT_LT] = ACTIONS(482),
    [anon_sym_AMP] = ACTIONS(484),
    [anon_sym_CARET] = ACTIONS(482),
    [anon_sym_PIPE] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_PERCENT] = ACTIONS(482),
    [anon_sym_LT_EQ] = ACTIONS(482),
    [anon_sym_EQ_EQ] = ACTIONS(484),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(482),
    [anon_sym_BANG_EQ] = ACTIONS(484),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(482),
    [anon_sym_GT_EQ] = ACTIONS(482),
    [anon_sym_instanceof] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1042] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(3119),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1043] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_else] = ACTIONS(1588),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_RPAREN] = ACTIONS(1588),
    [anon_sym_in] = ACTIONS(1590),
    [anon_sym_while] = ACTIONS(1588),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1590),
    [anon_sym_SLASH] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1588),
    [anon_sym_QMARK] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1588),
    [anon_sym_PIPE_PIPE] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1590),
    [anon_sym_GT_GT_GT] = ACTIONS(1588),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1590),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_LT_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1588),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1588),
    [anon_sym_instanceof] = ACTIONS(1588),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1588),
  },
  [1044] = {
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3121),
    [sym__template_chars] = ACTIONS(3121),
    [anon_sym_DOLLAR_LBRACE] = ACTIONS(3123),
  },
  [1045] = {
    [sym_parenthesized_expression] = STATE(1451),
    [sym__expression] = STATE(1451),
    [sym_yield_expression] = STATE(1451),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1451),
    [sym_jsx_fragment] = STATE(1451),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1451),
    [sym_anonymous_class] = STATE(1451),
    [sym_class] = STATE(1451),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1451),
    [sym_generator_function] = STATE(1451),
    [sym_call_expression] = STATE(1451),
    [sym_new_expression] = STATE(1451),
    [sym_await_expression] = STATE(1451),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1451),
    [sym_augmented_assignment_expression] = STATE(1451),
    [sym_ternary_expression] = STATE(1451),
    [sym_binary_expression] = STATE(1451),
    [sym_unary_expression] = STATE(1451),
    [sym_update_expression] = STATE(1451),
    [sym_sequence_expression] = STATE(902),
    [sym_template_string] = STATE(1451),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3125),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3127),
    [sym_number] = ACTIONS(3125),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(3127),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3127),
    [sym_false] = ACTIONS(3127),
    [sym_null] = ACTIONS(3127),
    [sym_undefined] = ACTIONS(3127),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [1046] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1965),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1965),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1047] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1505),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1048] = {
    [sym__automatic_semicolon] = ACTIONS(3129),
    [anon_sym_STAR] = ACTIONS(3129),
    [anon_sym_COMMA] = ACTIONS(3129),
    [anon_sym_RBRACE] = ACTIONS(3129),
    [anon_sym_LPAREN] = ACTIONS(3129),
    [anon_sym_RPAREN] = ACTIONS(3129),
    [anon_sym_in] = ACTIONS(3131),
    [anon_sym_SEMI] = ACTIONS(3129),
    [anon_sym_LBRACK] = ACTIONS(3129),
    [anon_sym_LT] = ACTIONS(3131),
    [anon_sym_GT] = ACTIONS(3131),
    [anon_sym_SLASH] = ACTIONS(3131),
    [anon_sym_DOT] = ACTIONS(3129),
    [anon_sym_QMARK] = ACTIONS(3129),
    [anon_sym_AMP_AMP] = ACTIONS(3129),
    [anon_sym_PIPE_PIPE] = ACTIONS(3129),
    [anon_sym_GT_GT] = ACTIONS(3131),
    [anon_sym_GT_GT_GT] = ACTIONS(3129),
    [anon_sym_LT_LT] = ACTIONS(3129),
    [anon_sym_AMP] = ACTIONS(3131),
    [anon_sym_CARET] = ACTIONS(3129),
    [anon_sym_PIPE] = ACTIONS(3131),
    [anon_sym_PLUS] = ACTIONS(3131),
    [anon_sym_DASH] = ACTIONS(3131),
    [anon_sym_PERCENT] = ACTIONS(3129),
    [anon_sym_LT_EQ] = ACTIONS(3129),
    [anon_sym_EQ_EQ] = ACTIONS(3131),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3129),
    [anon_sym_BANG_EQ] = ACTIONS(3131),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3129),
    [anon_sym_GT_EQ] = ACTIONS(3129),
    [anon_sym_instanceof] = ACTIONS(3129),
    [anon_sym_PLUS_PLUS] = ACTIONS(3129),
    [anon_sym_DASH_DASH] = ACTIONS(3129),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3129),
  },
  [1049] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(3133),
    [sym_comment] = ACTIONS(71),
  },
  [1050] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(3135),
    [anon_sym_RPAREN] = ACTIONS(1965),
    [sym_comment] = ACTIONS(71),
  },
  [1051] = {
    [sym__automatic_semicolon] = ACTIONS(3138),
    [anon_sym_STAR] = ACTIONS(3140),
    [anon_sym_COMMA] = ACTIONS(3138),
    [anon_sym_RBRACE] = ACTIONS(3138),
    [anon_sym_LPAREN] = ACTIONS(3138),
    [anon_sym_RPAREN] = ACTIONS(3138),
    [anon_sym_in] = ACTIONS(3140),
    [anon_sym_SEMI] = ACTIONS(3138),
    [anon_sym_LBRACK] = ACTIONS(3138),
    [anon_sym_LT] = ACTIONS(3140),
    [anon_sym_GT] = ACTIONS(3140),
    [anon_sym_SLASH] = ACTIONS(3140),
    [anon_sym_DOT] = ACTIONS(3138),
    [anon_sym_EQ] = ACTIONS(3140),
    [anon_sym_PLUS_EQ] = ACTIONS(3138),
    [anon_sym_DASH_EQ] = ACTIONS(3138),
    [anon_sym_STAR_EQ] = ACTIONS(3138),
    [anon_sym_SLASH_EQ] = ACTIONS(3138),
    [anon_sym_PERCENT_EQ] = ACTIONS(3138),
    [anon_sym_CARET_EQ] = ACTIONS(3138),
    [anon_sym_AMP_EQ] = ACTIONS(3138),
    [anon_sym_PIPE_EQ] = ACTIONS(3138),
    [anon_sym_GT_GT_EQ] = ACTIONS(3138),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3138),
    [anon_sym_LT_LT_EQ] = ACTIONS(3138),
    [anon_sym_QMARK] = ACTIONS(3138),
    [anon_sym_AMP_AMP] = ACTIONS(3138),
    [anon_sym_PIPE_PIPE] = ACTIONS(3138),
    [anon_sym_GT_GT] = ACTIONS(3140),
    [anon_sym_GT_GT_GT] = ACTIONS(3140),
    [anon_sym_LT_LT] = ACTIONS(3140),
    [anon_sym_AMP] = ACTIONS(3140),
    [anon_sym_CARET] = ACTIONS(3140),
    [anon_sym_PIPE] = ACTIONS(3140),
    [anon_sym_PLUS] = ACTIONS(3140),
    [anon_sym_DASH] = ACTIONS(3140),
    [anon_sym_PERCENT] = ACTIONS(3140),
    [anon_sym_LT_EQ] = ACTIONS(3138),
    [anon_sym_EQ_EQ] = ACTIONS(3140),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3138),
    [anon_sym_BANG_EQ] = ACTIONS(3140),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3138),
    [anon_sym_GT_EQ] = ACTIONS(3138),
    [anon_sym_instanceof] = ACTIONS(3138),
    [anon_sym_PLUS_PLUS] = ACTIONS(3138),
    [anon_sym_DASH_DASH] = ACTIONS(3138),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3138),
  },
  [1052] = {
    [sym_parenthesized_expression] = STATE(1453),
    [sym__expression] = STATE(1453),
    [sym_yield_expression] = STATE(1453),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(1453),
    [sym_jsx_fragment] = STATE(1453),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1453),
    [sym_anonymous_class] = STATE(1453),
    [sym_class] = STATE(1453),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1453),
    [sym_generator_function] = STATE(1453),
    [sym_call_expression] = STATE(1453),
    [sym_new_expression] = STATE(1453),
    [sym_await_expression] = STATE(1453),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(1453),
    [sym_augmented_assignment_expression] = STATE(1453),
    [sym_ternary_expression] = STATE(1453),
    [sym_binary_expression] = STATE(1453),
    [sym_unary_expression] = STATE(1453),
    [sym_update_expression] = STATE(1453),
    [sym_sequence_expression] = STATE(902),
    [sym_template_string] = STATE(1453),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3142),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3144),
    [sym_number] = ACTIONS(3142),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(3144),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3144),
    [sym_false] = ACTIONS(3144),
    [sym_null] = ACTIONS(3144),
    [sym_undefined] = ACTIONS(3144),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [1053] = {
    [anon_sym_export] = ACTIONS(3146),
    [anon_sym_LPAREN] = ACTIONS(3146),
    [anon_sym_DOT] = ACTIONS(3146),
    [anon_sym_class] = ACTIONS(3146),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(3146),
  },
  [1054] = {
    [anon_sym_export] = ACTIONS(3148),
    [anon_sym_LPAREN] = ACTIONS(3148),
    [anon_sym_DOT] = ACTIONS(3148),
    [anon_sym_class] = ACTIONS(3148),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(3148),
  },
  [1055] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(882),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1056] = {
    [sym_parenthesized_expression] = STATE(1454),
    [sym__expression] = STATE(1454),
    [sym_yield_expression] = STATE(1454),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1454),
    [sym_jsx_fragment] = STATE(1454),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1454),
    [sym_anonymous_class] = STATE(1454),
    [sym_class] = STATE(1454),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1454),
    [sym_generator_function] = STATE(1454),
    [sym_call_expression] = STATE(1454),
    [sym_new_expression] = STATE(1454),
    [sym_await_expression] = STATE(1454),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1454),
    [sym_augmented_assignment_expression] = STATE(1454),
    [sym_ternary_expression] = STATE(1454),
    [sym_binary_expression] = STATE(1454),
    [sym_unary_expression] = STATE(1454),
    [sym_update_expression] = STATE(1454),
    [sym_template_string] = STATE(1454),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3150),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3152),
    [sym_number] = ACTIONS(3150),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3152),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3152),
    [sym_false] = ACTIONS(3152),
    [sym_null] = ACTIONS(3152),
    [sym_undefined] = ACTIONS(3152),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1057] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1455),
    [sym__expression] = STATE(1455),
    [sym_yield_expression] = STATE(1455),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1455),
    [sym_jsx_fragment] = STATE(1455),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1455),
    [sym_anonymous_class] = STATE(1455),
    [sym_class] = STATE(1455),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1455),
    [sym_generator_function] = STATE(1455),
    [sym_call_expression] = STATE(1455),
    [sym_new_expression] = STATE(1455),
    [sym_await_expression] = STATE(1455),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1455),
    [sym_augmented_assignment_expression] = STATE(1455),
    [sym_ternary_expression] = STATE(1455),
    [sym_binary_expression] = STATE(1455),
    [sym_unary_expression] = STATE(1455),
    [sym_update_expression] = STATE(1455),
    [sym_template_string] = STATE(1455),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3154),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3156),
    [sym_number] = ACTIONS(3154),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3156),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3156),
    [sym_false] = ACTIONS(3156),
    [sym_null] = ACTIONS(3156),
    [sym_undefined] = ACTIONS(3156),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1058] = {
    [sym_parenthesized_expression] = STATE(1456),
    [sym__expression] = STATE(1456),
    [sym_yield_expression] = STATE(1456),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1456),
    [sym_jsx_fragment] = STATE(1456),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1456),
    [sym_anonymous_class] = STATE(1456),
    [sym_class] = STATE(1456),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1456),
    [sym_generator_function] = STATE(1456),
    [sym_call_expression] = STATE(1456),
    [sym_new_expression] = STATE(1456),
    [sym_await_expression] = STATE(1456),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1456),
    [sym_augmented_assignment_expression] = STATE(1456),
    [sym_ternary_expression] = STATE(1456),
    [sym_binary_expression] = STATE(1456),
    [sym_unary_expression] = STATE(1456),
    [sym_update_expression] = STATE(1456),
    [sym_template_string] = STATE(1456),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3158),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3160),
    [sym_number] = ACTIONS(3158),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3160),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3160),
    [sym_false] = ACTIONS(3160),
    [sym_null] = ACTIONS(3160),
    [sym_undefined] = ACTIONS(3160),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1059] = {
    [anon_sym_EQ_GT] = ACTIONS(3162),
    [sym_comment] = ACTIONS(71),
  },
  [1060] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1061] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(1056),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1062] = {
    [sym_parenthesized_expression] = STATE(1458),
    [sym__expression] = STATE(1458),
    [sym_yield_expression] = STATE(1458),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1458),
    [sym_jsx_fragment] = STATE(1458),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1458),
    [sym_anonymous_class] = STATE(1458),
    [sym_class] = STATE(1458),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1458),
    [sym_generator_function] = STATE(1458),
    [sym_call_expression] = STATE(1458),
    [sym_new_expression] = STATE(1458),
    [sym_await_expression] = STATE(1458),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1458),
    [sym_augmented_assignment_expression] = STATE(1458),
    [sym_ternary_expression] = STATE(1458),
    [sym_binary_expression] = STATE(1458),
    [sym_unary_expression] = STATE(1458),
    [sym_update_expression] = STATE(1458),
    [sym_template_string] = STATE(1458),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3164),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3166),
    [sym_number] = ACTIONS(3164),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3166),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3166),
    [sym_false] = ACTIONS(3166),
    [sym_null] = ACTIONS(3166),
    [sym_undefined] = ACTIONS(3166),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1063] = {
    [sym_parenthesized_expression] = STATE(1459),
    [sym__expression] = STATE(1459),
    [sym_yield_expression] = STATE(1459),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1459),
    [sym_jsx_fragment] = STATE(1459),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1459),
    [sym_anonymous_class] = STATE(1459),
    [sym_class] = STATE(1459),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1459),
    [sym_generator_function] = STATE(1459),
    [sym_call_expression] = STATE(1459),
    [sym_new_expression] = STATE(1459),
    [sym_await_expression] = STATE(1459),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1459),
    [sym_augmented_assignment_expression] = STATE(1459),
    [sym_ternary_expression] = STATE(1459),
    [sym_binary_expression] = STATE(1459),
    [sym_unary_expression] = STATE(1459),
    [sym_update_expression] = STATE(1459),
    [sym_template_string] = STATE(1459),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3168),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3170),
    [sym_number] = ACTIONS(3168),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3170),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3170),
    [sym_false] = ACTIONS(3170),
    [sym_null] = ACTIONS(3170),
    [sym_undefined] = ACTIONS(3170),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1064] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1065] = {
    [sym_parenthesized_expression] = STATE(1460),
    [sym__expression] = STATE(1460),
    [sym_yield_expression] = STATE(1460),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(1460),
    [sym_jsx_fragment] = STATE(1460),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1460),
    [sym_anonymous_class] = STATE(1460),
    [sym_class] = STATE(1460),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1460),
    [sym_generator_function] = STATE(1460),
    [sym_call_expression] = STATE(1460),
    [sym_new_expression] = STATE(1460),
    [sym_await_expression] = STATE(1460),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(1460),
    [sym_augmented_assignment_expression] = STATE(1460),
    [sym_ternary_expression] = STATE(1460),
    [sym_binary_expression] = STATE(1460),
    [sym_unary_expression] = STATE(1460),
    [sym_update_expression] = STATE(1460),
    [sym_template_string] = STATE(1460),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3172),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3174),
    [sym_number] = ACTIONS(3172),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(3174),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3174),
    [sym_false] = ACTIONS(3174),
    [sym_null] = ACTIONS(3174),
    [sym_undefined] = ACTIONS(3174),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1066] = {
    [sym_parenthesized_expression] = STATE(1461),
    [sym__expression] = STATE(1461),
    [sym_yield_expression] = STATE(1461),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1461),
    [sym_jsx_fragment] = STATE(1461),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1461),
    [sym_anonymous_class] = STATE(1461),
    [sym_class] = STATE(1461),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1461),
    [sym_generator_function] = STATE(1461),
    [sym_call_expression] = STATE(1461),
    [sym_new_expression] = STATE(1461),
    [sym_await_expression] = STATE(1461),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1461),
    [sym_augmented_assignment_expression] = STATE(1461),
    [sym_ternary_expression] = STATE(1461),
    [sym_binary_expression] = STATE(1461),
    [sym_unary_expression] = STATE(1461),
    [sym_update_expression] = STATE(1461),
    [sym_template_string] = STATE(1461),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3176),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3178),
    [sym_number] = ACTIONS(3176),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3178),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3178),
    [sym_false] = ACTIONS(3178),
    [sym_null] = ACTIONS(3178),
    [sym_undefined] = ACTIONS(3178),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1067] = {
    [sym_parenthesized_expression] = STATE(1462),
    [sym__expression] = STATE(1462),
    [sym_yield_expression] = STATE(1462),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1462),
    [sym_jsx_fragment] = STATE(1462),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1462),
    [sym_anonymous_class] = STATE(1462),
    [sym_class] = STATE(1462),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1462),
    [sym_generator_function] = STATE(1462),
    [sym_call_expression] = STATE(1462),
    [sym_new_expression] = STATE(1462),
    [sym_await_expression] = STATE(1462),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1462),
    [sym_augmented_assignment_expression] = STATE(1462),
    [sym_ternary_expression] = STATE(1462),
    [sym_binary_expression] = STATE(1462),
    [sym_unary_expression] = STATE(1462),
    [sym_update_expression] = STATE(1462),
    [sym_template_string] = STATE(1462),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3180),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3182),
    [sym_number] = ACTIONS(3180),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3182),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3182),
    [sym_false] = ACTIONS(3182),
    [sym_null] = ACTIONS(3182),
    [sym_undefined] = ACTIONS(3182),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1068] = {
    [sym_parenthesized_expression] = STATE(1463),
    [sym__expression] = STATE(1463),
    [sym_yield_expression] = STATE(1463),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1463),
    [sym_jsx_fragment] = STATE(1463),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1463),
    [sym_anonymous_class] = STATE(1463),
    [sym_class] = STATE(1463),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1463),
    [sym_generator_function] = STATE(1463),
    [sym_call_expression] = STATE(1463),
    [sym_new_expression] = STATE(1463),
    [sym_await_expression] = STATE(1463),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1463),
    [sym_augmented_assignment_expression] = STATE(1463),
    [sym_ternary_expression] = STATE(1463),
    [sym_binary_expression] = STATE(1463),
    [sym_unary_expression] = STATE(1463),
    [sym_update_expression] = STATE(1463),
    [sym_template_string] = STATE(1463),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3184),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3186),
    [sym_number] = ACTIONS(3184),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3186),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3186),
    [sym_false] = ACTIONS(3186),
    [sym_null] = ACTIONS(3186),
    [sym_undefined] = ACTIONS(3186),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1069] = {
    [sym_parenthesized_expression] = STATE(1464),
    [sym__expression] = STATE(1464),
    [sym_yield_expression] = STATE(1464),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1464),
    [sym_jsx_fragment] = STATE(1464),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1464),
    [sym_anonymous_class] = STATE(1464),
    [sym_class] = STATE(1464),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1464),
    [sym_generator_function] = STATE(1464),
    [sym_call_expression] = STATE(1464),
    [sym_new_expression] = STATE(1464),
    [sym_await_expression] = STATE(1464),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1464),
    [sym_augmented_assignment_expression] = STATE(1464),
    [sym_ternary_expression] = STATE(1464),
    [sym_binary_expression] = STATE(1464),
    [sym_unary_expression] = STATE(1464),
    [sym_update_expression] = STATE(1464),
    [sym_template_string] = STATE(1464),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3188),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3190),
    [sym_number] = ACTIONS(3188),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3190),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3190),
    [sym_false] = ACTIONS(3190),
    [sym_null] = ACTIONS(3190),
    [sym_undefined] = ACTIONS(3190),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1070] = {
    [sym_parenthesized_expression] = STATE(1465),
    [sym__expression] = STATE(1465),
    [sym_yield_expression] = STATE(1465),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1465),
    [sym_jsx_fragment] = STATE(1465),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1465),
    [sym_anonymous_class] = STATE(1465),
    [sym_class] = STATE(1465),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1465),
    [sym_generator_function] = STATE(1465),
    [sym_call_expression] = STATE(1465),
    [sym_new_expression] = STATE(1465),
    [sym_await_expression] = STATE(1465),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1465),
    [sym_augmented_assignment_expression] = STATE(1465),
    [sym_ternary_expression] = STATE(1465),
    [sym_binary_expression] = STATE(1465),
    [sym_unary_expression] = STATE(1465),
    [sym_update_expression] = STATE(1465),
    [sym_template_string] = STATE(1465),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_RBRACE] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(2298),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3192),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3194),
    [sym_number] = ACTIONS(3192),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3194),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3194),
    [sym_false] = ACTIONS(3194),
    [sym_null] = ACTIONS(3194),
    [sym_undefined] = ACTIONS(3194),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1071] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(1469),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(3196),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(3198),
    [anon_sym_PLUS_EQ] = ACTIONS(3200),
    [anon_sym_DASH_EQ] = ACTIONS(3200),
    [anon_sym_STAR_EQ] = ACTIONS(3200),
    [anon_sym_SLASH_EQ] = ACTIONS(3200),
    [anon_sym_PERCENT_EQ] = ACTIONS(3200),
    [anon_sym_CARET_EQ] = ACTIONS(3200),
    [anon_sym_AMP_EQ] = ACTIONS(3200),
    [anon_sym_PIPE_EQ] = ACTIONS(3200),
    [anon_sym_GT_GT_EQ] = ACTIONS(3200),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3200),
    [anon_sym_LT_LT_EQ] = ACTIONS(3200),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(3202),
  },
  [1072] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(1470),
    [sym__expression] = STATE(1470),
    [sym_yield_expression] = STATE(1470),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1470),
    [sym_jsx_fragment] = STATE(1470),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1470),
    [sym_anonymous_class] = STATE(1470),
    [sym_class] = STATE(1470),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1470),
    [sym_generator_function] = STATE(1470),
    [sym_call_expression] = STATE(1470),
    [sym_new_expression] = STATE(1470),
    [sym_await_expression] = STATE(1470),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1470),
    [sym_augmented_assignment_expression] = STATE(1470),
    [sym_ternary_expression] = STATE(1470),
    [sym_binary_expression] = STATE(1470),
    [sym_unary_expression] = STATE(1470),
    [sym_update_expression] = STATE(1470),
    [sym_template_string] = STATE(1470),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3204),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3206),
    [sym_number] = ACTIONS(3204),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3206),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3206),
    [sym_false] = ACTIONS(3206),
    [sym_null] = ACTIONS(3206),
    [sym_undefined] = ACTIONS(3206),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1073] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1074] = {
    [sym_parenthesized_expression] = STATE(1471),
    [sym__expression] = STATE(1471),
    [sym_yield_expression] = STATE(1471),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1471),
    [sym_jsx_fragment] = STATE(1471),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1471),
    [sym_anonymous_class] = STATE(1471),
    [sym_class] = STATE(1471),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1471),
    [sym_generator_function] = STATE(1471),
    [sym_call_expression] = STATE(1471),
    [sym_new_expression] = STATE(1471),
    [sym_await_expression] = STATE(1471),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1471),
    [sym_augmented_assignment_expression] = STATE(1471),
    [sym_ternary_expression] = STATE(1471),
    [sym_binary_expression] = STATE(1471),
    [sym_unary_expression] = STATE(1471),
    [sym_update_expression] = STATE(1471),
    [sym_template_string] = STATE(1471),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3208),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3210),
    [sym_number] = ACTIONS(3208),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3210),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3210),
    [sym_false] = ACTIONS(3210),
    [sym_null] = ACTIONS(3210),
    [sym_undefined] = ACTIONS(3210),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1075] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1076] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1077] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(3196),
    [anon_sym_EQ_GT] = ACTIONS(3198),
    [anon_sym_PLUS_EQ] = ACTIONS(3214),
    [anon_sym_DASH_EQ] = ACTIONS(3214),
    [anon_sym_STAR_EQ] = ACTIONS(3214),
    [anon_sym_SLASH_EQ] = ACTIONS(3214),
    [anon_sym_PERCENT_EQ] = ACTIONS(3214),
    [anon_sym_CARET_EQ] = ACTIONS(3214),
    [anon_sym_AMP_EQ] = ACTIONS(3214),
    [anon_sym_PIPE_EQ] = ACTIONS(3214),
    [anon_sym_GT_GT_EQ] = ACTIONS(3214),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3214),
    [anon_sym_LT_LT_EQ] = ACTIONS(3214),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1078] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_RBRACE] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(3196),
    [anon_sym_PLUS_EQ] = ACTIONS(3200),
    [anon_sym_DASH_EQ] = ACTIONS(3200),
    [anon_sym_STAR_EQ] = ACTIONS(3200),
    [anon_sym_SLASH_EQ] = ACTIONS(3200),
    [anon_sym_PERCENT_EQ] = ACTIONS(3200),
    [anon_sym_CARET_EQ] = ACTIONS(3200),
    [anon_sym_AMP_EQ] = ACTIONS(3200),
    [anon_sym_PIPE_EQ] = ACTIONS(3200),
    [anon_sym_GT_GT_EQ] = ACTIONS(3200),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3200),
    [anon_sym_LT_LT_EQ] = ACTIONS(3200),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [1079] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(1505),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1080] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3196),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1081] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3196),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1082] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_RBRACE] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3196),
    [anon_sym_PLUS_EQ] = ACTIONS(3214),
    [anon_sym_DASH_EQ] = ACTIONS(3214),
    [anon_sym_STAR_EQ] = ACTIONS(3214),
    [anon_sym_SLASH_EQ] = ACTIONS(3214),
    [anon_sym_PERCENT_EQ] = ACTIONS(3214),
    [anon_sym_CARET_EQ] = ACTIONS(3214),
    [anon_sym_AMP_EQ] = ACTIONS(3214),
    [anon_sym_PIPE_EQ] = ACTIONS(3214),
    [anon_sym_GT_GT_EQ] = ACTIONS(3214),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3214),
    [anon_sym_LT_LT_EQ] = ACTIONS(3214),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1083] = {
    [anon_sym_EQ_GT] = ACTIONS(3198),
    [sym_comment] = ACTIONS(71),
  },
  [1084] = {
    [anon_sym_LBRACE] = ACTIONS(3234),
    [anon_sym_LT] = ACTIONS(3234),
    [sym_jsx_text] = ACTIONS(3234),
    [sym_comment] = ACTIONS(71),
  },
  [1085] = {
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(3236),
    [sym_comment] = ACTIONS(71),
  },
  [1086] = {
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(3238),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [1087] = {
    [anon_sym_GT] = ACTIONS(3238),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [1088] = {
    [anon_sym_GT] = ACTIONS(3238),
    [sym_comment] = ACTIONS(71),
  },
  [1089] = {
    [anon_sym_GT] = ACTIONS(3240),
    [sym_comment] = ACTIONS(71),
  },
  [1090] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2044),
    [anon_sym_SLASH] = ACTIONS(3242),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1091] = {
    [anon_sym_GT] = ACTIONS(3244),
    [sym_comment] = ACTIONS(71),
  },
  [1092] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2058),
    [anon_sym_SLASH] = ACTIONS(3246),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1093] = {
    [sym_nested_identifier] = STATE(1487),
    [sym_jsx_namespace_name] = STATE(1488),
    [sym_jsx_identifier] = ACTIONS(3248),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3250),
  },
  [1094] = {
    [anon_sym_LBRACE] = ACTIONS(2326),
    [anon_sym_LT] = ACTIONS(2326),
    [sym_jsx_text] = ACTIONS(2326),
    [sym_comment] = ACTIONS(71),
  },
  [1095] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [1096] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3252),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(3252),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1097] = {
    [sym_class_body] = STATE(1490),
    [anon_sym_LBRACE] = ACTIONS(1008),
    [sym_comment] = ACTIONS(71),
  },
  [1098] = {
    [sym__automatic_semicolon] = ACTIONS(3254),
    [anon_sym_STAR] = ACTIONS(3254),
    [anon_sym_COMMA] = ACTIONS(3254),
    [anon_sym_RBRACE] = ACTIONS(3254),
    [anon_sym_LPAREN] = ACTIONS(3254),
    [anon_sym_RPAREN] = ACTIONS(3254),
    [anon_sym_in] = ACTIONS(3256),
    [anon_sym_SEMI] = ACTIONS(3254),
    [anon_sym_LBRACK] = ACTIONS(3254),
    [anon_sym_LT] = ACTIONS(3256),
    [anon_sym_GT] = ACTIONS(3256),
    [anon_sym_SLASH] = ACTIONS(3256),
    [anon_sym_DOT] = ACTIONS(3254),
    [anon_sym_QMARK] = ACTIONS(3254),
    [anon_sym_AMP_AMP] = ACTIONS(3254),
    [anon_sym_PIPE_PIPE] = ACTIONS(3254),
    [anon_sym_GT_GT] = ACTIONS(3256),
    [anon_sym_GT_GT_GT] = ACTIONS(3254),
    [anon_sym_LT_LT] = ACTIONS(3254),
    [anon_sym_AMP] = ACTIONS(3256),
    [anon_sym_CARET] = ACTIONS(3254),
    [anon_sym_PIPE] = ACTIONS(3256),
    [anon_sym_PLUS] = ACTIONS(3256),
    [anon_sym_DASH] = ACTIONS(3256),
    [anon_sym_PERCENT] = ACTIONS(3254),
    [anon_sym_LT_EQ] = ACTIONS(3254),
    [anon_sym_EQ_EQ] = ACTIONS(3256),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3254),
    [anon_sym_BANG_EQ] = ACTIONS(3256),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3254),
    [anon_sym_GT_EQ] = ACTIONS(3254),
    [anon_sym_instanceof] = ACTIONS(3254),
    [anon_sym_PLUS_PLUS] = ACTIONS(3254),
    [anon_sym_DASH_DASH] = ACTIONS(3254),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3254),
  },
  [1099] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2164),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2164),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1100] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2166),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1101] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2170),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1102] = {
    [sym_statement_block] = STATE(1037),
    [sym_parenthesized_expression] = STATE(1491),
    [sym__expression] = STATE(1491),
    [sym_yield_expression] = STATE(1491),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1491),
    [sym_jsx_fragment] = STATE(1491),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1491),
    [sym_anonymous_class] = STATE(1491),
    [sym_class] = STATE(1491),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1491),
    [sym_generator_function] = STATE(1491),
    [sym_call_expression] = STATE(1491),
    [sym_new_expression] = STATE(1491),
    [sym_await_expression] = STATE(1491),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1491),
    [sym_augmented_assignment_expression] = STATE(1491),
    [sym_ternary_expression] = STATE(1491),
    [sym_binary_expression] = STATE(1491),
    [sym_unary_expression] = STATE(1491),
    [sym_update_expression] = STATE(1491),
    [sym_template_string] = STATE(1491),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3258),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3260),
    [sym_number] = ACTIONS(3258),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(3260),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3260),
    [sym_false] = ACTIONS(3260),
    [sym_null] = ACTIONS(3260),
    [sym_undefined] = ACTIONS(3260),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [1103] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1104] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2212),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2212),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1105] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3262),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1106] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1107] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1108] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1109] = {
    [anon_sym_COMMA] = ACTIONS(3264),
    [anon_sym_RBRACE] = ACTIONS(3264),
    [sym_comment] = ACTIONS(71),
  },
  [1110] = {
    [sym__automatic_semicolon] = ACTIONS(3266),
    [anon_sym_from] = ACTIONS(3266),
    [anon_sym_SEMI] = ACTIONS(3266),
    [sym_comment] = ACTIONS(71),
  },
  [1111] = {
    [anon_sym_COMMA] = ACTIONS(3268),
    [anon_sym_RBRACE] = ACTIONS(3268),
    [sym_comment] = ACTIONS(71),
  },
  [1112] = {
    [sym__import_export_specifier] = STATE(1111),
    [anon_sym_RBRACE] = ACTIONS(3270),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [1113] = {
    [aux_sym_export_clause_repeat1] = STATE(1113),
    [anon_sym_COMMA] = ACTIONS(3272),
    [anon_sym_RBRACE] = ACTIONS(3275),
    [sym_comment] = ACTIONS(71),
  },
  [1114] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [ts_builtin_sym_end] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2064),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2064),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [1115] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(3277),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1116] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [ts_builtin_sym_end] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3103),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3103),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [1117] = {
    [sym_statement_block] = STATE(1496),
    [anon_sym_LBRACE] = ACTIONS(1395),
    [sym_comment] = ACTIONS(71),
  },
  [1118] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [ts_builtin_sym_end] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3107),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3107),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [1119] = {
    [sym_statement_block] = STATE(1497),
    [anon_sym_LBRACE] = ACTIONS(1395),
    [sym_comment] = ACTIONS(71),
  },
  [1120] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [ts_builtin_sym_end] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3115),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3115),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [1121] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1122] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(3279),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1123] = {
    [sym_class_body] = STATE(1499),
    [anon_sym_LBRACE] = ACTIONS(1389),
    [sym_comment] = ACTIONS(71),
  },
  [1124] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(882),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1125] = {
    [sym_parenthesized_expression] = STATE(1500),
    [sym__expression] = STATE(1500),
    [sym_yield_expression] = STATE(1500),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1500),
    [sym_jsx_fragment] = STATE(1500),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1500),
    [sym_anonymous_class] = STATE(1500),
    [sym_class] = STATE(1500),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1500),
    [sym_generator_function] = STATE(1500),
    [sym_call_expression] = STATE(1500),
    [sym_new_expression] = STATE(1500),
    [sym_await_expression] = STATE(1500),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1500),
    [sym_augmented_assignment_expression] = STATE(1500),
    [sym_ternary_expression] = STATE(1500),
    [sym_binary_expression] = STATE(1500),
    [sym_unary_expression] = STATE(1500),
    [sym_update_expression] = STATE(1500),
    [sym_template_string] = STATE(1500),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3281),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3283),
    [sym_number] = ACTIONS(3281),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3283),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3283),
    [sym_false] = ACTIONS(3283),
    [sym_null] = ACTIONS(3283),
    [sym_undefined] = ACTIONS(3283),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1126] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1501),
    [sym__expression] = STATE(1501),
    [sym_yield_expression] = STATE(1501),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1501),
    [sym_jsx_fragment] = STATE(1501),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1501),
    [sym_anonymous_class] = STATE(1501),
    [sym_class] = STATE(1501),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1501),
    [sym_generator_function] = STATE(1501),
    [sym_call_expression] = STATE(1501),
    [sym_new_expression] = STATE(1501),
    [sym_await_expression] = STATE(1501),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1501),
    [sym_augmented_assignment_expression] = STATE(1501),
    [sym_ternary_expression] = STATE(1501),
    [sym_binary_expression] = STATE(1501),
    [sym_unary_expression] = STATE(1501),
    [sym_update_expression] = STATE(1501),
    [sym_template_string] = STATE(1501),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3285),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3287),
    [sym_number] = ACTIONS(3285),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3287),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3287),
    [sym_false] = ACTIONS(3287),
    [sym_null] = ACTIONS(3287),
    [sym_undefined] = ACTIONS(3287),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1127] = {
    [sym_parenthesized_expression] = STATE(1502),
    [sym__expression] = STATE(1502),
    [sym_yield_expression] = STATE(1502),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1502),
    [sym_jsx_fragment] = STATE(1502),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1502),
    [sym_anonymous_class] = STATE(1502),
    [sym_class] = STATE(1502),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1502),
    [sym_generator_function] = STATE(1502),
    [sym_call_expression] = STATE(1502),
    [sym_new_expression] = STATE(1502),
    [sym_await_expression] = STATE(1502),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1502),
    [sym_augmented_assignment_expression] = STATE(1502),
    [sym_ternary_expression] = STATE(1502),
    [sym_binary_expression] = STATE(1502),
    [sym_unary_expression] = STATE(1502),
    [sym_update_expression] = STATE(1502),
    [sym_template_string] = STATE(1502),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3289),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3291),
    [sym_number] = ACTIONS(3289),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3291),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3291),
    [sym_false] = ACTIONS(3291),
    [sym_null] = ACTIONS(3291),
    [sym_undefined] = ACTIONS(3291),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1128] = {
    [anon_sym_EQ_GT] = ACTIONS(3293),
    [sym_comment] = ACTIONS(71),
  },
  [1129] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1048),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1130] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(1056),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1131] = {
    [sym_parenthesized_expression] = STATE(1504),
    [sym__expression] = STATE(1504),
    [sym_yield_expression] = STATE(1504),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1504),
    [sym_jsx_fragment] = STATE(1504),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1504),
    [sym_anonymous_class] = STATE(1504),
    [sym_class] = STATE(1504),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1504),
    [sym_generator_function] = STATE(1504),
    [sym_call_expression] = STATE(1504),
    [sym_new_expression] = STATE(1504),
    [sym_await_expression] = STATE(1504),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1504),
    [sym_augmented_assignment_expression] = STATE(1504),
    [sym_ternary_expression] = STATE(1504),
    [sym_binary_expression] = STATE(1504),
    [sym_unary_expression] = STATE(1504),
    [sym_update_expression] = STATE(1504),
    [sym_template_string] = STATE(1504),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3295),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3297),
    [sym_number] = ACTIONS(3295),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3297),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3297),
    [sym_false] = ACTIONS(3297),
    [sym_null] = ACTIONS(3297),
    [sym_undefined] = ACTIONS(3297),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1132] = {
    [sym_parenthesized_expression] = STATE(1505),
    [sym__expression] = STATE(1505),
    [sym_yield_expression] = STATE(1505),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1505),
    [sym_jsx_fragment] = STATE(1505),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1505),
    [sym_anonymous_class] = STATE(1505),
    [sym_class] = STATE(1505),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1505),
    [sym_generator_function] = STATE(1505),
    [sym_call_expression] = STATE(1505),
    [sym_new_expression] = STATE(1505),
    [sym_await_expression] = STATE(1505),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1505),
    [sym_augmented_assignment_expression] = STATE(1505),
    [sym_ternary_expression] = STATE(1505),
    [sym_binary_expression] = STATE(1505),
    [sym_unary_expression] = STATE(1505),
    [sym_update_expression] = STATE(1505),
    [sym_template_string] = STATE(1505),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3299),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3301),
    [sym_number] = ACTIONS(3299),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3301),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3301),
    [sym_false] = ACTIONS(3301),
    [sym_null] = ACTIONS(3301),
    [sym_undefined] = ACTIONS(3301),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1133] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1134] = {
    [anon_sym_LPAREN] = ACTIONS(2489),
    [sym_comment] = ACTIONS(71),
  },
  [1135] = {
    [sym_parenthesized_expression] = STATE(1506),
    [sym__expression] = STATE(1506),
    [sym_yield_expression] = STATE(1506),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1506),
    [sym_jsx_fragment] = STATE(1506),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1506),
    [sym_anonymous_class] = STATE(1506),
    [sym_class] = STATE(1506),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1506),
    [sym_generator_function] = STATE(1506),
    [sym_call_expression] = STATE(1506),
    [sym_new_expression] = STATE(1506),
    [sym_await_expression] = STATE(1506),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1506),
    [sym_augmented_assignment_expression] = STATE(1506),
    [sym_ternary_expression] = STATE(1506),
    [sym_binary_expression] = STATE(1506),
    [sym_unary_expression] = STATE(1506),
    [sym_update_expression] = STATE(1506),
    [sym_template_string] = STATE(1506),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3303),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3305),
    [sym_number] = ACTIONS(3303),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3305),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3305),
    [sym_false] = ACTIONS(3305),
    [sym_null] = ACTIONS(3305),
    [sym_undefined] = ACTIONS(3305),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1136] = {
    [sym_parenthesized_expression] = STATE(1507),
    [sym__expression] = STATE(1507),
    [sym_yield_expression] = STATE(1507),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1507),
    [sym_jsx_fragment] = STATE(1507),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1507),
    [sym_anonymous_class] = STATE(1507),
    [sym_class] = STATE(1507),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1507),
    [sym_generator_function] = STATE(1507),
    [sym_call_expression] = STATE(1507),
    [sym_new_expression] = STATE(1507),
    [sym_await_expression] = STATE(1507),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1507),
    [sym_augmented_assignment_expression] = STATE(1507),
    [sym_ternary_expression] = STATE(1507),
    [sym_binary_expression] = STATE(1507),
    [sym_unary_expression] = STATE(1507),
    [sym_update_expression] = STATE(1507),
    [sym_template_string] = STATE(1507),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3307),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3309),
    [sym_number] = ACTIONS(3307),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3309),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3309),
    [sym_false] = ACTIONS(3309),
    [sym_null] = ACTIONS(3309),
    [sym_undefined] = ACTIONS(3309),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1137] = {
    [sym_parenthesized_expression] = STATE(1508),
    [sym__expression] = STATE(1508),
    [sym_yield_expression] = STATE(1508),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1508),
    [sym_jsx_fragment] = STATE(1508),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1508),
    [sym_anonymous_class] = STATE(1508),
    [sym_class] = STATE(1508),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1508),
    [sym_generator_function] = STATE(1508),
    [sym_call_expression] = STATE(1508),
    [sym_new_expression] = STATE(1508),
    [sym_await_expression] = STATE(1508),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1508),
    [sym_augmented_assignment_expression] = STATE(1508),
    [sym_ternary_expression] = STATE(1508),
    [sym_binary_expression] = STATE(1508),
    [sym_unary_expression] = STATE(1508),
    [sym_update_expression] = STATE(1508),
    [sym_template_string] = STATE(1508),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3311),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3313),
    [sym_number] = ACTIONS(3311),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3313),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3313),
    [sym_false] = ACTIONS(3313),
    [sym_null] = ACTIONS(3313),
    [sym_undefined] = ACTIONS(3313),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1138] = {
    [sym_parenthesized_expression] = STATE(1509),
    [sym__expression] = STATE(1509),
    [sym_yield_expression] = STATE(1509),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1509),
    [sym_jsx_fragment] = STATE(1509),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1509),
    [sym_anonymous_class] = STATE(1509),
    [sym_class] = STATE(1509),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1509),
    [sym_generator_function] = STATE(1509),
    [sym_call_expression] = STATE(1509),
    [sym_new_expression] = STATE(1509),
    [sym_await_expression] = STATE(1509),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1509),
    [sym_augmented_assignment_expression] = STATE(1509),
    [sym_ternary_expression] = STATE(1509),
    [sym_binary_expression] = STATE(1509),
    [sym_unary_expression] = STATE(1509),
    [sym_update_expression] = STATE(1509),
    [sym_template_string] = STATE(1509),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3315),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3317),
    [sym_number] = ACTIONS(3315),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3317),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3317),
    [sym_false] = ACTIONS(3317),
    [sym_null] = ACTIONS(3317),
    [sym_undefined] = ACTIONS(3317),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1139] = {
    [anon_sym_STAR] = ACTIONS(3319),
    [anon_sym_COMMA] = ACTIONS(3319),
    [anon_sym_RBRACE] = ACTIONS(3319),
    [anon_sym_SEMI] = ACTIONS(3319),
    [anon_sym_LBRACK] = ACTIONS(3319),
    [anon_sym_async] = ACTIONS(3321),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3319),
    [sym_number] = ACTIONS(3319),
    [sym_identifier] = ACTIONS(3321),
    [anon_sym_AT] = ACTIONS(3319),
    [anon_sym_static] = ACTIONS(3321),
    [anon_sym_get] = ACTIONS(3321),
    [anon_sym_set] = ACTIONS(3321),
  },
  [1140] = {
    [anon_sym_COMMA] = ACTIONS(1473),
    [anon_sym_RPAREN] = ACTIONS(1473),
    [anon_sym_EQ] = ACTIONS(1473),
    [sym_comment] = ACTIONS(71),
  },
  [1141] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(3323),
    [sym_comment] = ACTIONS(71),
  },
  [1142] = {
    [anon_sym_COMMA] = ACTIONS(1571),
    [anon_sym_RPAREN] = ACTIONS(1571),
    [anon_sym_EQ] = ACTIONS(1571),
    [sym_comment] = ACTIONS(71),
  },
  [1143] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(3325),
    [sym_comment] = ACTIONS(71),
  },
  [1144] = {
    [aux_sym_object_repeat1] = STATE(1512),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(3325),
    [sym_comment] = ACTIONS(71),
  },
  [1145] = {
    [anon_sym_LPAREN] = ACTIONS(2489),
    [anon_sym_COLON] = ACTIONS(2489),
    [anon_sym_EQ] = ACTIONS(1989),
    [sym_comment] = ACTIONS(71),
  },
  [1146] = {
    [aux_sym_array_repeat1] = STATE(528),
    [anon_sym_COMMA] = ACTIONS(249),
    [anon_sym_RBRACK] = ACTIONS(3327),
    [sym_comment] = ACTIONS(71),
  },
  [1147] = {
    [anon_sym_COMMA] = ACTIONS(1989),
    [anon_sym_RPAREN] = ACTIONS(1989),
    [anon_sym_EQ] = ACTIONS(1989),
    [sym_comment] = ACTIONS(71),
  },
  [1148] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_RBRACE] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1149] = {
    [sym_parenthesized_expression] = STATE(1515),
    [sym__expression] = STATE(1515),
    [sym_yield_expression] = STATE(1515),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1515),
    [sym_jsx_fragment] = STATE(1515),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1515),
    [sym_anonymous_class] = STATE(1515),
    [sym_class] = STATE(1515),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1515),
    [sym_generator_function] = STATE(1515),
    [sym_call_expression] = STATE(1515),
    [sym_new_expression] = STATE(1515),
    [sym_await_expression] = STATE(1515),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1515),
    [sym_augmented_assignment_expression] = STATE(1515),
    [sym_spread_element] = STATE(1516),
    [sym_ternary_expression] = STATE(1515),
    [sym_binary_expression] = STATE(1515),
    [sym_unary_expression] = STATE(1515),
    [sym_update_expression] = STATE(1515),
    [sym_template_string] = STATE(1515),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(1517),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(3329),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1096),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3331),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3333),
    [sym_number] = ACTIONS(3331),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(3333),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3333),
    [sym_false] = ACTIONS(3333),
    [sym_null] = ACTIONS(3333),
    [sym_undefined] = ACTIONS(3333),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1150] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3335),
  },
  [1151] = {
    [anon_sym_STAR] = ACTIONS(2234),
    [anon_sym_LBRACK] = ACTIONS(2234),
    [anon_sym_async] = ACTIONS(2577),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2234),
    [sym_number] = ACTIONS(2234),
    [sym_identifier] = ACTIONS(2577),
    [anon_sym_AT] = ACTIONS(2234),
    [anon_sym_get] = ACTIONS(2577),
    [anon_sym_set] = ACTIONS(2577),
  },
  [1152] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3337),
  },
  [1153] = {
    [anon_sym_STAR] = ACTIONS(2238),
    [anon_sym_LBRACK] = ACTIONS(2238),
    [anon_sym_async] = ACTIONS(2581),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2238),
    [sym_number] = ACTIONS(2238),
    [sym_identifier] = ACTIONS(2581),
    [anon_sym_AT] = ACTIONS(2238),
    [anon_sym_get] = ACTIONS(2581),
    [anon_sym_set] = ACTIONS(2581),
  },
  [1154] = {
    [sym_export_statement] = STATE(1520),
    [sym__declaration] = STATE(1520),
    [sym_import_statement] = STATE(1520),
    [sym_expression_statement] = STATE(1520),
    [sym_variable_declaration] = STATE(1520),
    [sym_lexical_declaration] = STATE(1520),
    [sym_statement_block] = STATE(1520),
    [sym_if_statement] = STATE(1520),
    [sym_switch_statement] = STATE(1520),
    [sym_for_statement] = STATE(1520),
    [sym_for_in_statement] = STATE(1520),
    [sym_for_of_statement] = STATE(1520),
    [sym_while_statement] = STATE(1520),
    [sym_do_statement] = STATE(1520),
    [sym_try_statement] = STATE(1520),
    [sym_with_statement] = STATE(1520),
    [sym_break_statement] = STATE(1520),
    [sym_continue_statement] = STATE(1520),
    [sym_debugger_statement] = STATE(1520),
    [sym_return_statement] = STATE(1520),
    [sym_throw_statement] = STATE(1520),
    [sym_empty_statement] = STATE(1520),
    [sym_labeled_statement] = STATE(1520),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1155] = {
    [sym_expression_statement] = STATE(1524),
    [sym_variable_declaration] = STATE(1524),
    [sym_lexical_declaration] = STATE(1524),
    [sym_empty_statement] = STATE(1524),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(1525),
    [sym_array] = STATE(1526),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(3339),
    [anon_sym_let] = ACTIONS(3341),
    [anon_sym_const] = ACTIONS(3341),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3343),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1156] = {
    [sym_export_statement] = STATE(1233),
    [sym__declaration] = STATE(1233),
    [sym_import_statement] = STATE(1233),
    [sym_expression_statement] = STATE(1233),
    [sym_variable_declaration] = STATE(1233),
    [sym_lexical_declaration] = STATE(1233),
    [sym_statement_block] = STATE(1233),
    [sym_if_statement] = STATE(1233),
    [sym_switch_statement] = STATE(1233),
    [sym_for_statement] = STATE(1233),
    [sym_for_in_statement] = STATE(1233),
    [sym_for_of_statement] = STATE(1233),
    [sym_while_statement] = STATE(1233),
    [sym_do_statement] = STATE(1233),
    [sym_try_statement] = STATE(1233),
    [sym_with_statement] = STATE(1233),
    [sym_break_statement] = STATE(1233),
    [sym_continue_statement] = STATE(1233),
    [sym_debugger_statement] = STATE(1233),
    [sym_return_statement] = STATE(1233),
    [sym_throw_statement] = STATE(1233),
    [sym_empty_statement] = STATE(1233),
    [sym_labeled_statement] = STATE(1233),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1157] = {
    [sym_export_statement] = STATE(1241),
    [sym__declaration] = STATE(1241),
    [sym_import_statement] = STATE(1241),
    [sym_expression_statement] = STATE(1241),
    [sym_variable_declaration] = STATE(1241),
    [sym_lexical_declaration] = STATE(1241),
    [sym_statement_block] = STATE(1241),
    [sym_if_statement] = STATE(1241),
    [sym_switch_statement] = STATE(1241),
    [sym_for_statement] = STATE(1241),
    [sym_for_in_statement] = STATE(1241),
    [sym_for_of_statement] = STATE(1241),
    [sym_while_statement] = STATE(1241),
    [sym_do_statement] = STATE(1241),
    [sym_try_statement] = STATE(1241),
    [sym_with_statement] = STATE(1241),
    [sym_break_statement] = STATE(1241),
    [sym_continue_statement] = STATE(1241),
    [sym_debugger_statement] = STATE(1241),
    [sym_return_statement] = STATE(1241),
    [sym_throw_statement] = STATE(1241),
    [sym_empty_statement] = STATE(1241),
    [sym_labeled_statement] = STATE(1241),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1158] = {
    [sym_export_statement] = STATE(1255),
    [sym__declaration] = STATE(1255),
    [sym_import_statement] = STATE(1255),
    [sym_expression_statement] = STATE(1255),
    [sym_variable_declaration] = STATE(1255),
    [sym_lexical_declaration] = STATE(1255),
    [sym_statement_block] = STATE(1255),
    [sym_if_statement] = STATE(1255),
    [sym_switch_statement] = STATE(1255),
    [sym_for_statement] = STATE(1255),
    [sym_for_in_statement] = STATE(1255),
    [sym_for_of_statement] = STATE(1255),
    [sym_while_statement] = STATE(1255),
    [sym_do_statement] = STATE(1255),
    [sym_try_statement] = STATE(1255),
    [sym_with_statement] = STATE(1255),
    [sym_break_statement] = STATE(1255),
    [sym_continue_statement] = STATE(1255),
    [sym_debugger_statement] = STATE(1255),
    [sym_return_statement] = STATE(1255),
    [sym_throw_statement] = STATE(1255),
    [sym_empty_statement] = STATE(1255),
    [sym_labeled_statement] = STATE(1255),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1159] = {
    [sym_export_statement] = STATE(1256),
    [sym__declaration] = STATE(1256),
    [sym_import_statement] = STATE(1256),
    [sym_expression_statement] = STATE(1256),
    [sym_variable_declaration] = STATE(1256),
    [sym_lexical_declaration] = STATE(1256),
    [sym_statement_block] = STATE(1256),
    [sym_if_statement] = STATE(1256),
    [sym_switch_statement] = STATE(1256),
    [sym_for_statement] = STATE(1256),
    [sym_for_in_statement] = STATE(1256),
    [sym_for_of_statement] = STATE(1256),
    [sym_while_statement] = STATE(1256),
    [sym_do_statement] = STATE(1256),
    [sym_try_statement] = STATE(1256),
    [sym_with_statement] = STATE(1256),
    [sym_break_statement] = STATE(1256),
    [sym_continue_statement] = STATE(1256),
    [sym_debugger_statement] = STATE(1256),
    [sym_return_statement] = STATE(1256),
    [sym_throw_statement] = STATE(1256),
    [sym_empty_statement] = STATE(1256),
    [sym_labeled_statement] = STATE(1256),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1160] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(3345),
    [anon_sym_of] = ACTIONS(3347),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1161] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(3349),
    [anon_sym_of] = ACTIONS(3351),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1162] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(3353),
    [anon_sym_of] = ACTIONS(3355),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1163] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1533),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1533),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1164] = {
    [sym_parenthesized_expression] = STATE(1534),
    [sym__expression] = STATE(1534),
    [sym_yield_expression] = STATE(1534),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1534),
    [sym_jsx_fragment] = STATE(1534),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1534),
    [sym_anonymous_class] = STATE(1534),
    [sym_class] = STATE(1534),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1534),
    [sym_generator_function] = STATE(1534),
    [sym_call_expression] = STATE(1534),
    [sym_new_expression] = STATE(1534),
    [sym_await_expression] = STATE(1534),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1534),
    [sym_augmented_assignment_expression] = STATE(1534),
    [sym_ternary_expression] = STATE(1534),
    [sym_binary_expression] = STATE(1534),
    [sym_unary_expression] = STATE(1534),
    [sym_update_expression] = STATE(1534),
    [sym_template_string] = STATE(1534),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3357),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3359),
    [sym_number] = ACTIONS(3357),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3359),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3359),
    [sym_false] = ACTIONS(3359),
    [sym_null] = ACTIONS(3359),
    [sym_undefined] = ACTIONS(3359),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1165] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1536),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1536),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(3361),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1166] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1537),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1537),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1167] = {
    [sym_parenthesized_expression] = STATE(1538),
    [sym__expression] = STATE(1538),
    [sym_yield_expression] = STATE(1538),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1538),
    [sym_jsx_fragment] = STATE(1538),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1538),
    [sym_anonymous_class] = STATE(1538),
    [sym_class] = STATE(1538),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1538),
    [sym_generator_function] = STATE(1538),
    [sym_call_expression] = STATE(1538),
    [sym_new_expression] = STATE(1538),
    [sym_await_expression] = STATE(1538),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1538),
    [sym_augmented_assignment_expression] = STATE(1538),
    [sym_ternary_expression] = STATE(1538),
    [sym_binary_expression] = STATE(1538),
    [sym_unary_expression] = STATE(1538),
    [sym_update_expression] = STATE(1538),
    [sym_template_string] = STATE(1538),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3363),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3365),
    [sym_number] = ACTIONS(3363),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3365),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3365),
    [sym_false] = ACTIONS(3365),
    [sym_null] = ACTIONS(3365),
    [sym_undefined] = ACTIONS(3365),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1168] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1539),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1539),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1169] = {
    [sym_parenthesized_expression] = STATE(1540),
    [sym__expression] = STATE(1540),
    [sym_yield_expression] = STATE(1540),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1540),
    [sym_jsx_fragment] = STATE(1540),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1540),
    [sym_anonymous_class] = STATE(1540),
    [sym_class] = STATE(1540),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1540),
    [sym_generator_function] = STATE(1540),
    [sym_call_expression] = STATE(1540),
    [sym_new_expression] = STATE(1540),
    [sym_await_expression] = STATE(1540),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1540),
    [sym_augmented_assignment_expression] = STATE(1540),
    [sym_ternary_expression] = STATE(1540),
    [sym_binary_expression] = STATE(1540),
    [sym_unary_expression] = STATE(1540),
    [sym_update_expression] = STATE(1540),
    [sym_template_string] = STATE(1540),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3367),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3369),
    [sym_number] = ACTIONS(3367),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3369),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3369),
    [sym_false] = ACTIONS(3369),
    [sym_null] = ACTIONS(3369),
    [sym_undefined] = ACTIONS(3369),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1170] = {
    [sym_statement_block] = STATE(1541),
    [anon_sym_LBRACE] = ACTIONS(1586),
    [sym_comment] = ACTIONS(71),
  },
  [1171] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_RBRACE] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1172] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_RBRACE] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1173] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(1542),
    [sym__expression] = STATE(1542),
    [sym_yield_expression] = STATE(1542),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1542),
    [sym_jsx_fragment] = STATE(1542),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1542),
    [sym_anonymous_class] = STATE(1542),
    [sym_class] = STATE(1542),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1542),
    [sym_generator_function] = STATE(1542),
    [sym_call_expression] = STATE(1542),
    [sym_new_expression] = STATE(1542),
    [sym_await_expression] = STATE(1542),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1542),
    [sym_augmented_assignment_expression] = STATE(1542),
    [sym_ternary_expression] = STATE(1542),
    [sym_binary_expression] = STATE(1542),
    [sym_unary_expression] = STATE(1542),
    [sym_update_expression] = STATE(1542),
    [sym_template_string] = STATE(1542),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3371),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3373),
    [sym_number] = ACTIONS(3371),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(3373),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3373),
    [sym_false] = ACTIONS(3373),
    [sym_null] = ACTIONS(3373),
    [sym_undefined] = ACTIONS(3373),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [1174] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1175] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_RBRACE] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1176] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3375),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1177] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1178] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1179] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1180] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(882),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_RBRACE] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(882),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1181] = {
    [sym_statement_block] = STATE(584),
    [sym_parenthesized_expression] = STATE(1544),
    [sym__expression] = STATE(1544),
    [sym_yield_expression] = STATE(1544),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1544),
    [sym_jsx_fragment] = STATE(1544),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1544),
    [sym_anonymous_class] = STATE(1544),
    [sym_class] = STATE(1544),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1544),
    [sym_generator_function] = STATE(1544),
    [sym_call_expression] = STATE(1544),
    [sym_new_expression] = STATE(1544),
    [sym_await_expression] = STATE(1544),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1544),
    [sym_augmented_assignment_expression] = STATE(1544),
    [sym_ternary_expression] = STATE(1544),
    [sym_binary_expression] = STATE(1544),
    [sym_unary_expression] = STATE(1544),
    [sym_update_expression] = STATE(1544),
    [sym_template_string] = STATE(1544),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3377),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3379),
    [sym_number] = ACTIONS(3377),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3379),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3379),
    [sym_false] = ACTIONS(3379),
    [sym_null] = ACTIONS(3379),
    [sym_undefined] = ACTIONS(3379),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1182] = {
    [sym_parenthesized_expression] = STATE(1545),
    [sym__expression] = STATE(1545),
    [sym_yield_expression] = STATE(1545),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1545),
    [sym_jsx_fragment] = STATE(1545),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1545),
    [sym_anonymous_class] = STATE(1545),
    [sym_class] = STATE(1545),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1545),
    [sym_generator_function] = STATE(1545),
    [sym_call_expression] = STATE(1545),
    [sym_new_expression] = STATE(1545),
    [sym_await_expression] = STATE(1545),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1545),
    [sym_augmented_assignment_expression] = STATE(1545),
    [sym_ternary_expression] = STATE(1545),
    [sym_binary_expression] = STATE(1545),
    [sym_unary_expression] = STATE(1545),
    [sym_update_expression] = STATE(1545),
    [sym_template_string] = STATE(1545),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3381),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3383),
    [sym_number] = ACTIONS(3381),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3383),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3383),
    [sym_false] = ACTIONS(3383),
    [sym_null] = ACTIONS(3383),
    [sym_undefined] = ACTIONS(3383),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1183] = {
    [anon_sym_EQ_GT] = ACTIONS(3385),
    [sym_comment] = ACTIONS(71),
  },
  [1184] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1048),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_RBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1185] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1056),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_RBRACE] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(1056),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1186] = {
    [sym_parenthesized_expression] = STATE(1547),
    [sym__expression] = STATE(1547),
    [sym_yield_expression] = STATE(1547),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1547),
    [sym_jsx_fragment] = STATE(1547),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1547),
    [sym_anonymous_class] = STATE(1547),
    [sym_class] = STATE(1547),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1547),
    [sym_generator_function] = STATE(1547),
    [sym_call_expression] = STATE(1547),
    [sym_new_expression] = STATE(1547),
    [sym_await_expression] = STATE(1547),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1547),
    [sym_augmented_assignment_expression] = STATE(1547),
    [sym_ternary_expression] = STATE(1547),
    [sym_binary_expression] = STATE(1547),
    [sym_unary_expression] = STATE(1547),
    [sym_update_expression] = STATE(1547),
    [sym_template_string] = STATE(1547),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3387),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3389),
    [sym_number] = ACTIONS(3387),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3389),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3389),
    [sym_false] = ACTIONS(3389),
    [sym_null] = ACTIONS(3389),
    [sym_undefined] = ACTIONS(3389),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1187] = {
    [sym_parenthesized_expression] = STATE(1548),
    [sym__expression] = STATE(1548),
    [sym_yield_expression] = STATE(1548),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1548),
    [sym_jsx_fragment] = STATE(1548),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1548),
    [sym_anonymous_class] = STATE(1548),
    [sym_class] = STATE(1548),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1548),
    [sym_generator_function] = STATE(1548),
    [sym_call_expression] = STATE(1548),
    [sym_new_expression] = STATE(1548),
    [sym_await_expression] = STATE(1548),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1548),
    [sym_augmented_assignment_expression] = STATE(1548),
    [sym_ternary_expression] = STATE(1548),
    [sym_binary_expression] = STATE(1548),
    [sym_unary_expression] = STATE(1548),
    [sym_update_expression] = STATE(1548),
    [sym_template_string] = STATE(1548),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3391),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3393),
    [sym_number] = ACTIONS(3391),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3393),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3393),
    [sym_false] = ACTIONS(3393),
    [sym_null] = ACTIONS(3393),
    [sym_undefined] = ACTIONS(3393),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1188] = {
    [sym_parenthesized_expression] = STATE(613),
    [sym__expression] = STATE(613),
    [sym_yield_expression] = STATE(613),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(613),
    [sym_jsx_fragment] = STATE(613),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(613),
    [sym_anonymous_class] = STATE(613),
    [sym_class] = STATE(613),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(613),
    [sym_generator_function] = STATE(613),
    [sym_call_expression] = STATE(613),
    [sym_new_expression] = STATE(613),
    [sym_await_expression] = STATE(613),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(613),
    [sym_augmented_assignment_expression] = STATE(613),
    [sym_ternary_expression] = STATE(613),
    [sym_binary_expression] = STATE(613),
    [sym_unary_expression] = STATE(613),
    [sym_update_expression] = STATE(613),
    [sym_template_string] = STATE(613),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1126),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1128),
    [sym_number] = ACTIONS(1126),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(1128),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [sym_null] = ACTIONS(1128),
    [sym_undefined] = ACTIONS(1128),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1189] = {
    [sym_parenthesized_expression] = STATE(1549),
    [sym__expression] = STATE(1549),
    [sym_yield_expression] = STATE(1549),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1549),
    [sym_jsx_fragment] = STATE(1549),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1549),
    [sym_anonymous_class] = STATE(1549),
    [sym_class] = STATE(1549),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1549),
    [sym_generator_function] = STATE(1549),
    [sym_call_expression] = STATE(1549),
    [sym_new_expression] = STATE(1549),
    [sym_await_expression] = STATE(1549),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1549),
    [sym_augmented_assignment_expression] = STATE(1549),
    [sym_ternary_expression] = STATE(1549),
    [sym_binary_expression] = STATE(1549),
    [sym_unary_expression] = STATE(1549),
    [sym_update_expression] = STATE(1549),
    [sym_template_string] = STATE(1549),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3395),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3397),
    [sym_number] = ACTIONS(3395),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3397),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3397),
    [sym_false] = ACTIONS(3397),
    [sym_null] = ACTIONS(3397),
    [sym_undefined] = ACTIONS(3397),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1190] = {
    [sym_parenthesized_expression] = STATE(1550),
    [sym__expression] = STATE(1550),
    [sym_yield_expression] = STATE(1550),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1550),
    [sym_jsx_fragment] = STATE(1550),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1550),
    [sym_anonymous_class] = STATE(1550),
    [sym_class] = STATE(1550),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1550),
    [sym_generator_function] = STATE(1550),
    [sym_call_expression] = STATE(1550),
    [sym_new_expression] = STATE(1550),
    [sym_await_expression] = STATE(1550),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1550),
    [sym_augmented_assignment_expression] = STATE(1550),
    [sym_ternary_expression] = STATE(1550),
    [sym_binary_expression] = STATE(1550),
    [sym_unary_expression] = STATE(1550),
    [sym_update_expression] = STATE(1550),
    [sym_template_string] = STATE(1550),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3399),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3401),
    [sym_number] = ACTIONS(3399),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3401),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3401),
    [sym_false] = ACTIONS(3401),
    [sym_null] = ACTIONS(3401),
    [sym_undefined] = ACTIONS(3401),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1191] = {
    [sym_parenthesized_expression] = STATE(1551),
    [sym__expression] = STATE(1551),
    [sym_yield_expression] = STATE(1551),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1551),
    [sym_jsx_fragment] = STATE(1551),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1551),
    [sym_anonymous_class] = STATE(1551),
    [sym_class] = STATE(1551),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1551),
    [sym_generator_function] = STATE(1551),
    [sym_call_expression] = STATE(1551),
    [sym_new_expression] = STATE(1551),
    [sym_await_expression] = STATE(1551),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1551),
    [sym_augmented_assignment_expression] = STATE(1551),
    [sym_ternary_expression] = STATE(1551),
    [sym_binary_expression] = STATE(1551),
    [sym_unary_expression] = STATE(1551),
    [sym_update_expression] = STATE(1551),
    [sym_template_string] = STATE(1551),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3403),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3405),
    [sym_number] = ACTIONS(3403),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3405),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3405),
    [sym_false] = ACTIONS(3405),
    [sym_null] = ACTIONS(3405),
    [sym_undefined] = ACTIONS(3405),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1192] = {
    [sym_parenthesized_expression] = STATE(1552),
    [sym__expression] = STATE(1552),
    [sym_yield_expression] = STATE(1552),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1552),
    [sym_jsx_fragment] = STATE(1552),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1552),
    [sym_anonymous_class] = STATE(1552),
    [sym_class] = STATE(1552),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1552),
    [sym_generator_function] = STATE(1552),
    [sym_call_expression] = STATE(1552),
    [sym_new_expression] = STATE(1552),
    [sym_await_expression] = STATE(1552),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1552),
    [sym_augmented_assignment_expression] = STATE(1552),
    [sym_ternary_expression] = STATE(1552),
    [sym_binary_expression] = STATE(1552),
    [sym_unary_expression] = STATE(1552),
    [sym_update_expression] = STATE(1552),
    [sym_template_string] = STATE(1552),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3407),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3409),
    [sym_number] = ACTIONS(3407),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(3409),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3409),
    [sym_false] = ACTIONS(3409),
    [sym_null] = ACTIONS(3409),
    [sym_undefined] = ACTIONS(3409),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1193] = {
    [anon_sym_export] = ACTIONS(2220),
    [anon_sym_STAR] = ACTIONS(2218),
    [anon_sym_LBRACK] = ACTIONS(2218),
    [anon_sym_class] = ACTIONS(2220),
    [anon_sym_async] = ACTIONS(2220),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2218),
    [sym_number] = ACTIONS(2218),
    [sym_identifier] = ACTIONS(2220),
    [anon_sym_AT] = ACTIONS(2218),
    [anon_sym_get] = ACTIONS(2220),
    [anon_sym_set] = ACTIONS(2220),
  },
  [1194] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(1554),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3411),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1195] = {
    [aux_sym_array_repeat1] = STATE(1554),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(3411),
    [sym_comment] = ACTIONS(71),
  },
  [1196] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(3411),
    [sym_comment] = ACTIONS(71),
  },
  [1197] = {
    [anon_sym_export] = ACTIONS(3413),
    [anon_sym_STAR] = ACTIONS(3146),
    [anon_sym_LPAREN] = ACTIONS(3146),
    [anon_sym_LBRACK] = ACTIONS(3146),
    [anon_sym_DOT] = ACTIONS(3413),
    [anon_sym_class] = ACTIONS(3413),
    [anon_sym_async] = ACTIONS(3413),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3146),
    [sym_number] = ACTIONS(3146),
    [sym_identifier] = ACTIONS(3413),
    [anon_sym_AT] = ACTIONS(3146),
    [anon_sym_get] = ACTIONS(3413),
    [anon_sym_set] = ACTIONS(3413),
  },
  [1198] = {
    [anon_sym_export] = ACTIONS(3415),
    [anon_sym_STAR] = ACTIONS(3148),
    [anon_sym_LPAREN] = ACTIONS(3148),
    [anon_sym_LBRACK] = ACTIONS(3148),
    [anon_sym_DOT] = ACTIONS(3415),
    [anon_sym_class] = ACTIONS(3415),
    [anon_sym_async] = ACTIONS(3415),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3148),
    [sym_number] = ACTIONS(3148),
    [sym_identifier] = ACTIONS(3415),
    [anon_sym_AT] = ACTIONS(3148),
    [anon_sym_get] = ACTIONS(3415),
    [anon_sym_set] = ACTIONS(3415),
  },
  [1199] = {
    [sym__automatic_semicolon] = ACTIONS(3417),
    [anon_sym_STAR] = ACTIONS(3417),
    [anon_sym_COMMA] = ACTIONS(3417),
    [anon_sym_RBRACE] = ACTIONS(3417),
    [anon_sym_LPAREN] = ACTIONS(3417),
    [anon_sym_RPAREN] = ACTIONS(3417),
    [anon_sym_in] = ACTIONS(3419),
    [anon_sym_of] = ACTIONS(3417),
    [anon_sym_SEMI] = ACTIONS(3417),
    [anon_sym_LBRACK] = ACTIONS(3417),
    [anon_sym_LT] = ACTIONS(3419),
    [anon_sym_GT] = ACTIONS(3419),
    [anon_sym_SLASH] = ACTIONS(3419),
    [anon_sym_DOT] = ACTIONS(3417),
    [anon_sym_EQ] = ACTIONS(3419),
    [anon_sym_QMARK] = ACTIONS(3417),
    [anon_sym_AMP_AMP] = ACTIONS(3417),
    [anon_sym_PIPE_PIPE] = ACTIONS(3417),
    [anon_sym_GT_GT] = ACTIONS(3419),
    [anon_sym_GT_GT_GT] = ACTIONS(3417),
    [anon_sym_LT_LT] = ACTIONS(3417),
    [anon_sym_AMP] = ACTIONS(3419),
    [anon_sym_CARET] = ACTIONS(3417),
    [anon_sym_PIPE] = ACTIONS(3419),
    [anon_sym_PLUS] = ACTIONS(3419),
    [anon_sym_DASH] = ACTIONS(3419),
    [anon_sym_PERCENT] = ACTIONS(3417),
    [anon_sym_LT_EQ] = ACTIONS(3417),
    [anon_sym_EQ_EQ] = ACTIONS(3419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3417),
    [anon_sym_BANG_EQ] = ACTIONS(3419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3417),
    [anon_sym_GT_EQ] = ACTIONS(3417),
    [anon_sym_instanceof] = ACTIONS(3417),
    [anon_sym_PLUS_PLUS] = ACTIONS(3417),
    [anon_sym_DASH_DASH] = ACTIONS(3417),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3417),
  },
  [1200] = {
    [anon_sym_from] = ACTIONS(3421),
    [sym_comment] = ACTIONS(71),
  },
  [1201] = {
    [anon_sym_COMMA] = ACTIONS(3423),
    [anon_sym_RBRACE] = ACTIONS(3423),
    [sym_comment] = ACTIONS(71),
  },
  [1202] = {
    [sym__import_export_specifier] = STATE(1201),
    [anon_sym_RBRACE] = ACTIONS(3425),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [1203] = {
    [aux_sym_named_imports_repeat1] = STATE(1203),
    [anon_sym_COMMA] = ACTIONS(3427),
    [anon_sym_RBRACE] = ACTIONS(3430),
    [sym_comment] = ACTIONS(71),
  },
  [1204] = {
    [sym__automatic_semicolon] = ACTIONS(2491),
    [anon_sym_COMMA] = ACTIONS(2491),
    [anon_sym_in] = ACTIONS(2491),
    [anon_sym_of] = ACTIONS(2491),
    [anon_sym_SEMI] = ACTIONS(2491),
    [anon_sym_EQ] = ACTIONS(2491),
    [sym_comment] = ACTIONS(71),
  },
  [1205] = {
    [sym__automatic_semicolon] = ACTIONS(2583),
    [anon_sym_COMMA] = ACTIONS(2583),
    [anon_sym_in] = ACTIONS(2583),
    [anon_sym_of] = ACTIONS(2583),
    [anon_sym_SEMI] = ACTIONS(2583),
    [anon_sym_EQ] = ACTIONS(2583),
    [sym_comment] = ACTIONS(71),
  },
  [1206] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(3432),
    [sym_comment] = ACTIONS(71),
  },
  [1207] = {
    [sym__automatic_semicolon] = ACTIONS(2945),
    [anon_sym_COMMA] = ACTIONS(2945),
    [anon_sym_in] = ACTIONS(2945),
    [anon_sym_of] = ACTIONS(2945),
    [anon_sym_SEMI] = ACTIONS(2945),
    [anon_sym_EQ] = ACTIONS(2945),
    [sym_comment] = ACTIONS(71),
  },
  [1208] = {
    [sym__automatic_semicolon] = ACTIONS(3434),
    [anon_sym_SEMI] = ACTIONS(3434),
    [sym_comment] = ACTIONS(71),
  },
  [1209] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3434),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(3434),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1210] = {
    [sym_class_heritage] = STATE(1560),
    [sym_class_body] = STATE(1561),
    [anon_sym_LBRACE] = ACTIONS(3436),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [1211] = {
    [sym_formal_parameters] = STATE(1563),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3438),
  },
  [1212] = {
    [sym_formal_parameters] = STATE(1565),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3440),
  },
  [1213] = {
    [sym_formal_parameters] = STATE(1563),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1214] = {
    [sym_statement_block] = STATE(1567),
    [anon_sym_LBRACE] = ACTIONS(3442),
    [sym_comment] = ACTIONS(71),
  },
  [1215] = {
    [ts_builtin_sym_end] = ACTIONS(1397),
    [anon_sym_export] = ACTIONS(1399),
    [anon_sym_LBRACE] = ACTIONS(1397),
    [anon_sym_RBRACE] = ACTIONS(1397),
    [anon_sym_import] = ACTIONS(1399),
    [anon_sym_var] = ACTIONS(1399),
    [anon_sym_let] = ACTIONS(1399),
    [anon_sym_const] = ACTIONS(1399),
    [anon_sym_if] = ACTIONS(1399),
    [anon_sym_else] = ACTIONS(1399),
    [anon_sym_switch] = ACTIONS(1399),
    [anon_sym_for] = ACTIONS(1399),
    [anon_sym_LPAREN] = ACTIONS(1397),
    [anon_sym_while] = ACTIONS(1399),
    [anon_sym_do] = ACTIONS(1399),
    [anon_sym_try] = ACTIONS(1399),
    [anon_sym_with] = ACTIONS(1399),
    [anon_sym_break] = ACTIONS(1399),
    [anon_sym_continue] = ACTIONS(1399),
    [anon_sym_debugger] = ACTIONS(1399),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_throw] = ACTIONS(1399),
    [anon_sym_SEMI] = ACTIONS(1397),
    [anon_sym_yield] = ACTIONS(1399),
    [anon_sym_LBRACK] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1397),
    [anon_sym_class] = ACTIONS(1399),
    [anon_sym_async] = ACTIONS(1399),
    [anon_sym_function] = ACTIONS(1399),
    [anon_sym_EQ_GT] = ACTIONS(1397),
    [anon_sym_new] = ACTIONS(1399),
    [anon_sym_await] = ACTIONS(1399),
    [anon_sym_PLUS] = ACTIONS(1399),
    [anon_sym_DASH] = ACTIONS(1399),
    [anon_sym_BANG] = ACTIONS(1397),
    [anon_sym_TILDE] = ACTIONS(1397),
    [anon_sym_typeof] = ACTIONS(1399),
    [anon_sym_void] = ACTIONS(1399),
    [anon_sym_delete] = ACTIONS(1399),
    [anon_sym_PLUS_PLUS] = ACTIONS(1397),
    [anon_sym_DASH_DASH] = ACTIONS(1397),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1397),
    [anon_sym_BQUOTE] = ACTIONS(1397),
    [sym_regex] = ACTIONS(1399),
    [sym_number] = ACTIONS(1397),
    [sym_identifier] = ACTIONS(1399),
    [sym_this] = ACTIONS(1399),
    [sym_super] = ACTIONS(1399),
    [sym_true] = ACTIONS(1399),
    [sym_false] = ACTIONS(1399),
    [sym_null] = ACTIONS(1399),
    [sym_undefined] = ACTIONS(1399),
    [anon_sym_AT] = ACTIONS(1397),
    [anon_sym_get] = ACTIONS(1399),
    [anon_sym_set] = ACTIONS(1399),
  },
  [1216] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3444),
  },
  [1217] = {
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [1218] = {
    [ts_builtin_sym_end] = ACTIONS(1627),
    [anon_sym_export] = ACTIONS(1629),
    [anon_sym_LBRACE] = ACTIONS(1627),
    [anon_sym_RBRACE] = ACTIONS(1627),
    [anon_sym_import] = ACTIONS(1629),
    [anon_sym_var] = ACTIONS(1629),
    [anon_sym_let] = ACTIONS(1629),
    [anon_sym_const] = ACTIONS(1629),
    [anon_sym_if] = ACTIONS(1629),
    [anon_sym_else] = ACTIONS(1629),
    [anon_sym_switch] = ACTIONS(1629),
    [anon_sym_for] = ACTIONS(1629),
    [anon_sym_LPAREN] = ACTIONS(1627),
    [anon_sym_while] = ACTIONS(1629),
    [anon_sym_do] = ACTIONS(1629),
    [anon_sym_try] = ACTIONS(1629),
    [anon_sym_with] = ACTIONS(1629),
    [anon_sym_break] = ACTIONS(1629),
    [anon_sym_continue] = ACTIONS(1629),
    [anon_sym_debugger] = ACTIONS(1629),
    [anon_sym_return] = ACTIONS(1629),
    [anon_sym_throw] = ACTIONS(1629),
    [anon_sym_SEMI] = ACTIONS(1627),
    [anon_sym_yield] = ACTIONS(1629),
    [anon_sym_LBRACK] = ACTIONS(1627),
    [anon_sym_LT] = ACTIONS(1627),
    [anon_sym_class] = ACTIONS(1629),
    [anon_sym_async] = ACTIONS(1629),
    [anon_sym_function] = ACTIONS(1629),
    [anon_sym_EQ_GT] = ACTIONS(1627),
    [anon_sym_new] = ACTIONS(1629),
    [anon_sym_await] = ACTIONS(1629),
    [anon_sym_PLUS] = ACTIONS(1629),
    [anon_sym_DASH] = ACTIONS(1629),
    [anon_sym_BANG] = ACTIONS(1627),
    [anon_sym_TILDE] = ACTIONS(1627),
    [anon_sym_typeof] = ACTIONS(1629),
    [anon_sym_void] = ACTIONS(1629),
    [anon_sym_delete] = ACTIONS(1629),
    [anon_sym_PLUS_PLUS] = ACTIONS(1627),
    [anon_sym_DASH_DASH] = ACTIONS(1627),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1627),
    [anon_sym_BQUOTE] = ACTIONS(1627),
    [sym_regex] = ACTIONS(1629),
    [sym_number] = ACTIONS(1627),
    [sym_identifier] = ACTIONS(1629),
    [sym_this] = ACTIONS(1629),
    [sym_super] = ACTIONS(1629),
    [sym_true] = ACTIONS(1629),
    [sym_false] = ACTIONS(1629),
    [sym_null] = ACTIONS(1629),
    [sym_undefined] = ACTIONS(1629),
    [anon_sym_AT] = ACTIONS(1627),
    [anon_sym_get] = ACTIONS(1629),
    [anon_sym_set] = ACTIONS(1629),
  },
  [1219] = {
    [sym__automatic_semicolon] = ACTIONS(3446),
    [anon_sym_SEMI] = ACTIONS(3446),
    [sym_comment] = ACTIONS(71),
  },
  [1220] = {
    [ts_builtin_sym_end] = ACTIONS(1641),
    [anon_sym_export] = ACTIONS(1643),
    [anon_sym_LBRACE] = ACTIONS(1641),
    [anon_sym_RBRACE] = ACTIONS(1641),
    [anon_sym_import] = ACTIONS(1643),
    [anon_sym_var] = ACTIONS(1643),
    [anon_sym_let] = ACTIONS(1643),
    [anon_sym_const] = ACTIONS(1643),
    [anon_sym_if] = ACTIONS(1643),
    [anon_sym_else] = ACTIONS(1643),
    [anon_sym_switch] = ACTIONS(1643),
    [anon_sym_for] = ACTIONS(1643),
    [anon_sym_LPAREN] = ACTIONS(1641),
    [anon_sym_while] = ACTIONS(1643),
    [anon_sym_do] = ACTIONS(1643),
    [anon_sym_try] = ACTIONS(1643),
    [anon_sym_with] = ACTIONS(1643),
    [anon_sym_break] = ACTIONS(1643),
    [anon_sym_continue] = ACTIONS(1643),
    [anon_sym_debugger] = ACTIONS(1643),
    [anon_sym_return] = ACTIONS(1643),
    [anon_sym_throw] = ACTIONS(1643),
    [anon_sym_SEMI] = ACTIONS(1641),
    [anon_sym_yield] = ACTIONS(1643),
    [anon_sym_LBRACK] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1641),
    [anon_sym_class] = ACTIONS(1643),
    [anon_sym_async] = ACTIONS(1643),
    [anon_sym_function] = ACTIONS(1643),
    [anon_sym_EQ_GT] = ACTIONS(1641),
    [anon_sym_new] = ACTIONS(1643),
    [anon_sym_await] = ACTIONS(1643),
    [anon_sym_PLUS] = ACTIONS(1643),
    [anon_sym_DASH] = ACTIONS(1643),
    [anon_sym_BANG] = ACTIONS(1641),
    [anon_sym_TILDE] = ACTIONS(1641),
    [anon_sym_typeof] = ACTIONS(1643),
    [anon_sym_void] = ACTIONS(1643),
    [anon_sym_delete] = ACTIONS(1643),
    [anon_sym_PLUS_PLUS] = ACTIONS(1641),
    [anon_sym_DASH_DASH] = ACTIONS(1641),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1641),
    [anon_sym_BQUOTE] = ACTIONS(1641),
    [sym_regex] = ACTIONS(1643),
    [sym_number] = ACTIONS(1641),
    [sym_identifier] = ACTIONS(1643),
    [sym_this] = ACTIONS(1643),
    [sym_super] = ACTIONS(1643),
    [sym_true] = ACTIONS(1643),
    [sym_false] = ACTIONS(1643),
    [sym_null] = ACTIONS(1643),
    [sym_undefined] = ACTIONS(1643),
    [anon_sym_AT] = ACTIONS(1641),
    [anon_sym_get] = ACTIONS(1643),
    [anon_sym_set] = ACTIONS(1643),
  },
  [1221] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(3448),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(3448),
    [sym_comment] = ACTIONS(71),
  },
  [1222] = {
    [ts_builtin_sym_end] = ACTIONS(1651),
    [anon_sym_export] = ACTIONS(1653),
    [anon_sym_LBRACE] = ACTIONS(1651),
    [anon_sym_RBRACE] = ACTIONS(1651),
    [anon_sym_import] = ACTIONS(1653),
    [anon_sym_var] = ACTIONS(1653),
    [anon_sym_let] = ACTIONS(1653),
    [anon_sym_const] = ACTIONS(1653),
    [anon_sym_if] = ACTIONS(1653),
    [anon_sym_else] = ACTIONS(1653),
    [anon_sym_switch] = ACTIONS(1653),
    [anon_sym_for] = ACTIONS(1653),
    [anon_sym_LPAREN] = ACTIONS(1651),
    [anon_sym_while] = ACTIONS(1653),
    [anon_sym_do] = ACTIONS(1653),
    [anon_sym_try] = ACTIONS(1653),
    [anon_sym_with] = ACTIONS(1653),
    [anon_sym_break] = ACTIONS(1653),
    [anon_sym_continue] = ACTIONS(1653),
    [anon_sym_debugger] = ACTIONS(1653),
    [anon_sym_return] = ACTIONS(1653),
    [anon_sym_throw] = ACTIONS(1653),
    [anon_sym_SEMI] = ACTIONS(1651),
    [anon_sym_yield] = ACTIONS(1653),
    [anon_sym_LBRACK] = ACTIONS(1651),
    [anon_sym_LT] = ACTIONS(1651),
    [anon_sym_class] = ACTIONS(1653),
    [anon_sym_async] = ACTIONS(1653),
    [anon_sym_function] = ACTIONS(1653),
    [anon_sym_EQ_GT] = ACTIONS(1651),
    [anon_sym_new] = ACTIONS(1653),
    [anon_sym_await] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1653),
    [anon_sym_BANG] = ACTIONS(1651),
    [anon_sym_TILDE] = ACTIONS(1651),
    [anon_sym_typeof] = ACTIONS(1653),
    [anon_sym_void] = ACTIONS(1653),
    [anon_sym_delete] = ACTIONS(1653),
    [anon_sym_PLUS_PLUS] = ACTIONS(1651),
    [anon_sym_DASH_DASH] = ACTIONS(1651),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1651),
    [anon_sym_BQUOTE] = ACTIONS(1651),
    [sym_regex] = ACTIONS(1653),
    [sym_number] = ACTIONS(1651),
    [sym_identifier] = ACTIONS(1653),
    [sym_this] = ACTIONS(1653),
    [sym_super] = ACTIONS(1653),
    [sym_true] = ACTIONS(1653),
    [sym_false] = ACTIONS(1653),
    [sym_null] = ACTIONS(1653),
    [sym_undefined] = ACTIONS(1653),
    [anon_sym_AT] = ACTIONS(1651),
    [anon_sym_get] = ACTIONS(1653),
    [anon_sym_set] = ACTIONS(1653),
  },
  [1223] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(3450),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(3450),
    [sym_comment] = ACTIONS(71),
  },
  [1224] = {
    [ts_builtin_sym_end] = ACTIONS(1703),
    [anon_sym_export] = ACTIONS(1705),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_import] = ACTIONS(1705),
    [anon_sym_var] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_else] = ACTIONS(3452),
    [anon_sym_switch] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_do] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [anon_sym_with] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_debugger] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_throw] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_class] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_function] = ACTIONS(1705),
    [anon_sym_EQ_GT] = ACTIONS(1703),
    [anon_sym_new] = ACTIONS(1705),
    [anon_sym_await] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_typeof] = ACTIONS(1705),
    [anon_sym_void] = ACTIONS(1705),
    [anon_sym_delete] = ACTIONS(1705),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1703),
    [anon_sym_BQUOTE] = ACTIONS(1703),
    [sym_regex] = ACTIONS(1705),
    [sym_number] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_this] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_null] = ACTIONS(1705),
    [sym_undefined] = ACTIONS(1705),
    [anon_sym_AT] = ACTIONS(1703),
    [anon_sym_get] = ACTIONS(1705),
    [anon_sym_set] = ACTIONS(1705),
  },
  [1225] = {
    [sym_switch_case] = STATE(1574),
    [sym_switch_default] = STATE(1574),
    [aux_sym_switch_body_repeat1] = STATE(1574),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(3454),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [1226] = {
    [ts_builtin_sym_end] = ACTIONS(1723),
    [anon_sym_export] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1723),
    [anon_sym_RBRACE] = ACTIONS(1723),
    [anon_sym_import] = ACTIONS(1725),
    [anon_sym_var] = ACTIONS(1725),
    [anon_sym_let] = ACTIONS(1725),
    [anon_sym_const] = ACTIONS(1725),
    [anon_sym_if] = ACTIONS(1725),
    [anon_sym_else] = ACTIONS(1725),
    [anon_sym_switch] = ACTIONS(1725),
    [anon_sym_for] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_while] = ACTIONS(1725),
    [anon_sym_do] = ACTIONS(1725),
    [anon_sym_try] = ACTIONS(1725),
    [anon_sym_with] = ACTIONS(1725),
    [anon_sym_break] = ACTIONS(1725),
    [anon_sym_continue] = ACTIONS(1725),
    [anon_sym_debugger] = ACTIONS(1725),
    [anon_sym_return] = ACTIONS(1725),
    [anon_sym_throw] = ACTIONS(1725),
    [anon_sym_SEMI] = ACTIONS(1723),
    [anon_sym_yield] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1723),
    [anon_sym_LT] = ACTIONS(1723),
    [anon_sym_class] = ACTIONS(1725),
    [anon_sym_async] = ACTIONS(1725),
    [anon_sym_function] = ACTIONS(1725),
    [anon_sym_EQ_GT] = ACTIONS(1723),
    [anon_sym_new] = ACTIONS(1725),
    [anon_sym_await] = ACTIONS(1725),
    [anon_sym_PLUS] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1725),
    [anon_sym_BANG] = ACTIONS(1723),
    [anon_sym_TILDE] = ACTIONS(1723),
    [anon_sym_typeof] = ACTIONS(1725),
    [anon_sym_void] = ACTIONS(1725),
    [anon_sym_delete] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1723),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1723),
    [anon_sym_BQUOTE] = ACTIONS(1723),
    [sym_regex] = ACTIONS(1725),
    [sym_number] = ACTIONS(1723),
    [sym_identifier] = ACTIONS(1725),
    [sym_this] = ACTIONS(1725),
    [sym_super] = ACTIONS(1725),
    [sym_true] = ACTIONS(1725),
    [sym_false] = ACTIONS(1725),
    [sym_null] = ACTIONS(1725),
    [sym_undefined] = ACTIONS(1725),
    [anon_sym_AT] = ACTIONS(1723),
    [anon_sym_get] = ACTIONS(1725),
    [anon_sym_set] = ACTIONS(1725),
  },
  [1227] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(1576),
    [sym_array] = STATE(1577),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3456),
  },
  [1228] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(1576),
    [sym_array] = STATE(1577),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3456),
  },
  [1229] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3458),
    [anon_sym_of] = ACTIONS(3460),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1230] = {
    [sym_expression_statement] = STATE(1580),
    [sym_empty_statement] = STATE(1580),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1231] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3462),
    [anon_sym_of] = ACTIONS(3464),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1232] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3466),
    [anon_sym_of] = ACTIONS(3468),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1233] = {
    [ts_builtin_sym_end] = ACTIONS(1861),
    [anon_sym_export] = ACTIONS(1863),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_RBRACE] = ACTIONS(1861),
    [anon_sym_import] = ACTIONS(1863),
    [anon_sym_var] = ACTIONS(1863),
    [anon_sym_let] = ACTIONS(1863),
    [anon_sym_const] = ACTIONS(1863),
    [anon_sym_if] = ACTIONS(1863),
    [anon_sym_else] = ACTIONS(1863),
    [anon_sym_switch] = ACTIONS(1863),
    [anon_sym_for] = ACTIONS(1863),
    [anon_sym_LPAREN] = ACTIONS(1861),
    [anon_sym_while] = ACTIONS(1863),
    [anon_sym_do] = ACTIONS(1863),
    [anon_sym_try] = ACTIONS(1863),
    [anon_sym_with] = ACTIONS(1863),
    [anon_sym_break] = ACTIONS(1863),
    [anon_sym_continue] = ACTIONS(1863),
    [anon_sym_debugger] = ACTIONS(1863),
    [anon_sym_return] = ACTIONS(1863),
    [anon_sym_throw] = ACTIONS(1863),
    [anon_sym_SEMI] = ACTIONS(1861),
    [anon_sym_yield] = ACTIONS(1863),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LT] = ACTIONS(1861),
    [anon_sym_class] = ACTIONS(1863),
    [anon_sym_async] = ACTIONS(1863),
    [anon_sym_function] = ACTIONS(1863),
    [anon_sym_EQ_GT] = ACTIONS(1861),
    [anon_sym_new] = ACTIONS(1863),
    [anon_sym_await] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_typeof] = ACTIONS(1863),
    [anon_sym_void] = ACTIONS(1863),
    [anon_sym_delete] = ACTIONS(1863),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1861),
    [anon_sym_BQUOTE] = ACTIONS(1861),
    [sym_regex] = ACTIONS(1863),
    [sym_number] = ACTIONS(1861),
    [sym_identifier] = ACTIONS(1863),
    [sym_this] = ACTIONS(1863),
    [sym_super] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_null] = ACTIONS(1863),
    [sym_undefined] = ACTIONS(1863),
    [anon_sym_AT] = ACTIONS(1861),
    [anon_sym_get] = ACTIONS(1863),
    [anon_sym_set] = ACTIONS(1863),
  },
  [1234] = {
    [sym_parenthesized_expression] = STATE(1585),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [sym_comment] = ACTIONS(71),
  },
  [1235] = {
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_catch] = ACTIONS(484),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1236] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(3470),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1237] = {
    [sym_statement_block] = STATE(1589),
    [anon_sym_LBRACE] = ACTIONS(3472),
    [anon_sym_LPAREN] = ACTIONS(3474),
    [sym_comment] = ACTIONS(71),
  },
  [1238] = {
    [sym_statement_block] = STATE(1591),
    [anon_sym_LBRACE] = ACTIONS(3476),
    [sym_comment] = ACTIONS(71),
  },
  [1239] = {
    [sym_finally_clause] = STATE(1592),
    [ts_builtin_sym_end] = ACTIONS(1941),
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_else] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_finally] = ACTIONS(2668),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [1240] = {
    [ts_builtin_sym_end] = ACTIONS(1941),
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_else] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [1241] = {
    [ts_builtin_sym_end] = ACTIONS(1945),
    [anon_sym_export] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1945),
    [anon_sym_RBRACE] = ACTIONS(1945),
    [anon_sym_import] = ACTIONS(1947),
    [anon_sym_var] = ACTIONS(1947),
    [anon_sym_let] = ACTIONS(1947),
    [anon_sym_const] = ACTIONS(1947),
    [anon_sym_if] = ACTIONS(1947),
    [anon_sym_else] = ACTIONS(1947),
    [anon_sym_switch] = ACTIONS(1947),
    [anon_sym_for] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1945),
    [anon_sym_while] = ACTIONS(1947),
    [anon_sym_do] = ACTIONS(1947),
    [anon_sym_try] = ACTIONS(1947),
    [anon_sym_with] = ACTIONS(1947),
    [anon_sym_break] = ACTIONS(1947),
    [anon_sym_continue] = ACTIONS(1947),
    [anon_sym_debugger] = ACTIONS(1947),
    [anon_sym_return] = ACTIONS(1947),
    [anon_sym_throw] = ACTIONS(1947),
    [anon_sym_SEMI] = ACTIONS(1945),
    [anon_sym_yield] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1945),
    [anon_sym_LT] = ACTIONS(1945),
    [anon_sym_class] = ACTIONS(1947),
    [anon_sym_async] = ACTIONS(1947),
    [anon_sym_function] = ACTIONS(1947),
    [anon_sym_EQ_GT] = ACTIONS(1945),
    [anon_sym_new] = ACTIONS(1947),
    [anon_sym_await] = ACTIONS(1947),
    [anon_sym_PLUS] = ACTIONS(1947),
    [anon_sym_DASH] = ACTIONS(1947),
    [anon_sym_BANG] = ACTIONS(1945),
    [anon_sym_TILDE] = ACTIONS(1945),
    [anon_sym_typeof] = ACTIONS(1947),
    [anon_sym_void] = ACTIONS(1947),
    [anon_sym_delete] = ACTIONS(1947),
    [anon_sym_PLUS_PLUS] = ACTIONS(1945),
    [anon_sym_DASH_DASH] = ACTIONS(1945),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1945),
    [anon_sym_BQUOTE] = ACTIONS(1945),
    [sym_regex] = ACTIONS(1947),
    [sym_number] = ACTIONS(1945),
    [sym_identifier] = ACTIONS(1947),
    [sym_this] = ACTIONS(1947),
    [sym_super] = ACTIONS(1947),
    [sym_true] = ACTIONS(1947),
    [sym_false] = ACTIONS(1947),
    [sym_null] = ACTIONS(1947),
    [sym_undefined] = ACTIONS(1947),
    [anon_sym_AT] = ACTIONS(1945),
    [anon_sym_get] = ACTIONS(1947),
    [anon_sym_set] = ACTIONS(1947),
  },
  [1242] = {
    [ts_builtin_sym_end] = ACTIONS(1949),
    [anon_sym_export] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1949),
    [anon_sym_RBRACE] = ACTIONS(1949),
    [anon_sym_import] = ACTIONS(1951),
    [anon_sym_var] = ACTIONS(1951),
    [anon_sym_let] = ACTIONS(1951),
    [anon_sym_const] = ACTIONS(1951),
    [anon_sym_if] = ACTIONS(1951),
    [anon_sym_else] = ACTIONS(1951),
    [anon_sym_switch] = ACTIONS(1951),
    [anon_sym_for] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1949),
    [anon_sym_while] = ACTIONS(1951),
    [anon_sym_do] = ACTIONS(1951),
    [anon_sym_try] = ACTIONS(1951),
    [anon_sym_with] = ACTIONS(1951),
    [anon_sym_break] = ACTIONS(1951),
    [anon_sym_continue] = ACTIONS(1951),
    [anon_sym_debugger] = ACTIONS(1951),
    [anon_sym_return] = ACTIONS(1951),
    [anon_sym_throw] = ACTIONS(1951),
    [anon_sym_SEMI] = ACTIONS(1949),
    [anon_sym_yield] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1949),
    [anon_sym_LT] = ACTIONS(1949),
    [anon_sym_class] = ACTIONS(1951),
    [anon_sym_async] = ACTIONS(1951),
    [anon_sym_function] = ACTIONS(1951),
    [anon_sym_EQ_GT] = ACTIONS(1949),
    [anon_sym_new] = ACTIONS(1951),
    [anon_sym_await] = ACTIONS(1951),
    [anon_sym_PLUS] = ACTIONS(1951),
    [anon_sym_DASH] = ACTIONS(1951),
    [anon_sym_BANG] = ACTIONS(1949),
    [anon_sym_TILDE] = ACTIONS(1949),
    [anon_sym_typeof] = ACTIONS(1951),
    [anon_sym_void] = ACTIONS(1951),
    [anon_sym_delete] = ACTIONS(1951),
    [anon_sym_PLUS_PLUS] = ACTIONS(1949),
    [anon_sym_DASH_DASH] = ACTIONS(1949),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1949),
    [anon_sym_BQUOTE] = ACTIONS(1949),
    [sym_regex] = ACTIONS(1951),
    [sym_number] = ACTIONS(1949),
    [sym_identifier] = ACTIONS(1951),
    [sym_this] = ACTIONS(1951),
    [sym_super] = ACTIONS(1951),
    [sym_true] = ACTIONS(1951),
    [sym_false] = ACTIONS(1951),
    [sym_null] = ACTIONS(1951),
    [sym_undefined] = ACTIONS(1951),
    [anon_sym_AT] = ACTIONS(1949),
    [anon_sym_get] = ACTIONS(1951),
    [anon_sym_set] = ACTIONS(1951),
  },
  [1243] = {
    [ts_builtin_sym_end] = ACTIONS(1953),
    [anon_sym_export] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1953),
    [anon_sym_RBRACE] = ACTIONS(1953),
    [anon_sym_import] = ACTIONS(1955),
    [anon_sym_var] = ACTIONS(1955),
    [anon_sym_let] = ACTIONS(1955),
    [anon_sym_const] = ACTIONS(1955),
    [anon_sym_if] = ACTIONS(1955),
    [anon_sym_else] = ACTIONS(1955),
    [anon_sym_switch] = ACTIONS(1955),
    [anon_sym_for] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1953),
    [anon_sym_while] = ACTIONS(1955),
    [anon_sym_do] = ACTIONS(1955),
    [anon_sym_try] = ACTIONS(1955),
    [anon_sym_with] = ACTIONS(1955),
    [anon_sym_break] = ACTIONS(1955),
    [anon_sym_continue] = ACTIONS(1955),
    [anon_sym_debugger] = ACTIONS(1955),
    [anon_sym_return] = ACTIONS(1955),
    [anon_sym_throw] = ACTIONS(1955),
    [anon_sym_SEMI] = ACTIONS(1953),
    [anon_sym_yield] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1953),
    [anon_sym_LT] = ACTIONS(1953),
    [anon_sym_class] = ACTIONS(1955),
    [anon_sym_async] = ACTIONS(1955),
    [anon_sym_function] = ACTIONS(1955),
    [anon_sym_EQ_GT] = ACTIONS(1953),
    [anon_sym_new] = ACTIONS(1955),
    [anon_sym_await] = ACTIONS(1955),
    [anon_sym_PLUS] = ACTIONS(1955),
    [anon_sym_DASH] = ACTIONS(1955),
    [anon_sym_BANG] = ACTIONS(1953),
    [anon_sym_TILDE] = ACTIONS(1953),
    [anon_sym_typeof] = ACTIONS(1955),
    [anon_sym_void] = ACTIONS(1955),
    [anon_sym_delete] = ACTIONS(1955),
    [anon_sym_PLUS_PLUS] = ACTIONS(1953),
    [anon_sym_DASH_DASH] = ACTIONS(1953),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1953),
    [anon_sym_BQUOTE] = ACTIONS(1953),
    [sym_regex] = ACTIONS(1955),
    [sym_number] = ACTIONS(1953),
    [sym_identifier] = ACTIONS(1955),
    [sym_this] = ACTIONS(1955),
    [sym_super] = ACTIONS(1955),
    [sym_true] = ACTIONS(1955),
    [sym_false] = ACTIONS(1955),
    [sym_null] = ACTIONS(1955),
    [sym_undefined] = ACTIONS(1955),
    [anon_sym_AT] = ACTIONS(1953),
    [anon_sym_get] = ACTIONS(1955),
    [anon_sym_set] = ACTIONS(1955),
  },
  [1244] = {
    [ts_builtin_sym_end] = ACTIONS(1957),
    [anon_sym_export] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1957),
    [anon_sym_RBRACE] = ACTIONS(1957),
    [anon_sym_import] = ACTIONS(1959),
    [anon_sym_var] = ACTIONS(1959),
    [anon_sym_let] = ACTIONS(1959),
    [anon_sym_const] = ACTIONS(1959),
    [anon_sym_if] = ACTIONS(1959),
    [anon_sym_else] = ACTIONS(1959),
    [anon_sym_switch] = ACTIONS(1959),
    [anon_sym_for] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1957),
    [anon_sym_while] = ACTIONS(1959),
    [anon_sym_do] = ACTIONS(1959),
    [anon_sym_try] = ACTIONS(1959),
    [anon_sym_with] = ACTIONS(1959),
    [anon_sym_break] = ACTIONS(1959),
    [anon_sym_continue] = ACTIONS(1959),
    [anon_sym_debugger] = ACTIONS(1959),
    [anon_sym_return] = ACTIONS(1959),
    [anon_sym_throw] = ACTIONS(1959),
    [anon_sym_SEMI] = ACTIONS(1957),
    [anon_sym_yield] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1957),
    [anon_sym_LT] = ACTIONS(1957),
    [anon_sym_class] = ACTIONS(1959),
    [anon_sym_async] = ACTIONS(1959),
    [anon_sym_function] = ACTIONS(1959),
    [anon_sym_EQ_GT] = ACTIONS(1957),
    [anon_sym_new] = ACTIONS(1959),
    [anon_sym_await] = ACTIONS(1959),
    [anon_sym_PLUS] = ACTIONS(1959),
    [anon_sym_DASH] = ACTIONS(1959),
    [anon_sym_BANG] = ACTIONS(1957),
    [anon_sym_TILDE] = ACTIONS(1957),
    [anon_sym_typeof] = ACTIONS(1959),
    [anon_sym_void] = ACTIONS(1959),
    [anon_sym_delete] = ACTIONS(1959),
    [anon_sym_PLUS_PLUS] = ACTIONS(1957),
    [anon_sym_DASH_DASH] = ACTIONS(1957),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1957),
    [anon_sym_BQUOTE] = ACTIONS(1957),
    [sym_regex] = ACTIONS(1959),
    [sym_number] = ACTIONS(1957),
    [sym_identifier] = ACTIONS(1959),
    [sym_this] = ACTIONS(1959),
    [sym_super] = ACTIONS(1959),
    [sym_true] = ACTIONS(1959),
    [sym_false] = ACTIONS(1959),
    [sym_null] = ACTIONS(1959),
    [sym_undefined] = ACTIONS(1959),
    [anon_sym_AT] = ACTIONS(1957),
    [anon_sym_get] = ACTIONS(1959),
    [anon_sym_set] = ACTIONS(1959),
  },
  [1245] = {
    [ts_builtin_sym_end] = ACTIONS(1961),
    [anon_sym_export] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1961),
    [anon_sym_RBRACE] = ACTIONS(1961),
    [anon_sym_import] = ACTIONS(1963),
    [anon_sym_var] = ACTIONS(1963),
    [anon_sym_let] = ACTIONS(1963),
    [anon_sym_const] = ACTIONS(1963),
    [anon_sym_if] = ACTIONS(1963),
    [anon_sym_else] = ACTIONS(1963),
    [anon_sym_switch] = ACTIONS(1963),
    [anon_sym_for] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1961),
    [anon_sym_while] = ACTIONS(1963),
    [anon_sym_do] = ACTIONS(1963),
    [anon_sym_try] = ACTIONS(1963),
    [anon_sym_with] = ACTIONS(1963),
    [anon_sym_break] = ACTIONS(1963),
    [anon_sym_continue] = ACTIONS(1963),
    [anon_sym_debugger] = ACTIONS(1963),
    [anon_sym_return] = ACTIONS(1963),
    [anon_sym_throw] = ACTIONS(1963),
    [anon_sym_SEMI] = ACTIONS(1961),
    [anon_sym_yield] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1961),
    [anon_sym_LT] = ACTIONS(1961),
    [anon_sym_class] = ACTIONS(1963),
    [anon_sym_async] = ACTIONS(1963),
    [anon_sym_function] = ACTIONS(1963),
    [anon_sym_EQ_GT] = ACTIONS(1961),
    [anon_sym_new] = ACTIONS(1963),
    [anon_sym_await] = ACTIONS(1963),
    [anon_sym_PLUS] = ACTIONS(1963),
    [anon_sym_DASH] = ACTIONS(1963),
    [anon_sym_BANG] = ACTIONS(1961),
    [anon_sym_TILDE] = ACTIONS(1961),
    [anon_sym_typeof] = ACTIONS(1963),
    [anon_sym_void] = ACTIONS(1963),
    [anon_sym_delete] = ACTIONS(1963),
    [anon_sym_PLUS_PLUS] = ACTIONS(1961),
    [anon_sym_DASH_DASH] = ACTIONS(1961),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1961),
    [anon_sym_BQUOTE] = ACTIONS(1961),
    [sym_regex] = ACTIONS(1963),
    [sym_number] = ACTIONS(1961),
    [sym_identifier] = ACTIONS(1963),
    [sym_this] = ACTIONS(1963),
    [sym_super] = ACTIONS(1963),
    [sym_true] = ACTIONS(1963),
    [sym_false] = ACTIONS(1963),
    [sym_null] = ACTIONS(1963),
    [sym_undefined] = ACTIONS(1963),
    [anon_sym_AT] = ACTIONS(1961),
    [anon_sym_get] = ACTIONS(1963),
    [anon_sym_set] = ACTIONS(1963),
  },
  [1246] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1594),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(3478),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1247] = {
    [sym_class_body] = STATE(1595),
    [anon_sym_LBRACE] = ACTIONS(2678),
    [sym_comment] = ACTIONS(71),
  },
  [1248] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [ts_builtin_sym_end] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_STAR] = ACTIONS(2140),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_COMMA] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_else] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_in] = ACTIONS(2142),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2142),
    [anon_sym_GT] = ACTIONS(2142),
    [anon_sym_SLASH] = ACTIONS(2142),
    [anon_sym_DOT] = ACTIONS(2142),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_QMARK] = ACTIONS(2140),
    [anon_sym_AMP_AMP] = ACTIONS(2140),
    [anon_sym_PIPE_PIPE] = ACTIONS(2140),
    [anon_sym_GT_GT] = ACTIONS(2142),
    [anon_sym_GT_GT_GT] = ACTIONS(2140),
    [anon_sym_LT_LT] = ACTIONS(2140),
    [anon_sym_AMP] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(2140),
    [anon_sym_PIPE] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_PERCENT] = ACTIONS(2140),
    [anon_sym_LT_EQ] = ACTIONS(2140),
    [anon_sym_EQ_EQ] = ACTIONS(2142),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2140),
    [anon_sym_BANG_EQ] = ACTIONS(2142),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2140),
    [anon_sym_GT_EQ] = ACTIONS(2140),
    [anon_sym_instanceof] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2142),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [1249] = {
    [sym_formal_parameters] = STATE(1596),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1250] = {
    [sym_statement_block] = STATE(1597),
    [anon_sym_LBRACE] = ACTIONS(2684),
    [sym_comment] = ACTIONS(71),
  },
  [1251] = {
    [sym_formal_parameters] = STATE(1598),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1252] = {
    [sym_statement_block] = STATE(1599),
    [anon_sym_LBRACE] = ACTIONS(2684),
    [sym_comment] = ACTIONS(71),
  },
  [1253] = {
    [sym_export_statement] = STATE(1601),
    [sym__declaration] = STATE(1601),
    [sym_import_statement] = STATE(1601),
    [sym_expression_statement] = STATE(1601),
    [sym_variable_declaration] = STATE(1601),
    [sym_lexical_declaration] = STATE(1601),
    [sym_statement_block] = STATE(1601),
    [sym_if_statement] = STATE(1601),
    [sym_switch_statement] = STATE(1601),
    [sym_for_statement] = STATE(1601),
    [sym_for_in_statement] = STATE(1601),
    [sym_for_of_statement] = STATE(1601),
    [sym_while_statement] = STATE(1601),
    [sym_do_statement] = STATE(1601),
    [sym_try_statement] = STATE(1601),
    [sym_with_statement] = STATE(1601),
    [sym_break_statement] = STATE(1601),
    [sym_continue_statement] = STATE(1601),
    [sym_debugger_statement] = STATE(1601),
    [sym_return_statement] = STATE(1601),
    [sym_throw_statement] = STATE(1601),
    [sym_empty_statement] = STATE(1601),
    [sym_labeled_statement] = STATE(1601),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1601),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(3480),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1254] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [ts_builtin_sym_end] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_COMMA] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_else] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_in] = ACTIONS(2180),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2180),
    [anon_sym_GT] = ACTIONS(2180),
    [anon_sym_SLASH] = ACTIONS(2180),
    [anon_sym_DOT] = ACTIONS(2180),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_QMARK] = ACTIONS(2178),
    [anon_sym_AMP_AMP] = ACTIONS(2178),
    [anon_sym_PIPE_PIPE] = ACTIONS(2178),
    [anon_sym_GT_GT] = ACTIONS(2180),
    [anon_sym_GT_GT_GT] = ACTIONS(2178),
    [anon_sym_LT_LT] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2180),
    [anon_sym_CARET] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_PERCENT] = ACTIONS(2178),
    [anon_sym_LT_EQ] = ACTIONS(2178),
    [anon_sym_EQ_EQ] = ACTIONS(2180),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2178),
    [anon_sym_BANG_EQ] = ACTIONS(2180),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2178),
    [anon_sym_GT_EQ] = ACTIONS(2178),
    [anon_sym_instanceof] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2180),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [1255] = {
    [ts_builtin_sym_end] = ACTIONS(2204),
    [anon_sym_export] = ACTIONS(2206),
    [anon_sym_LBRACE] = ACTIONS(2204),
    [anon_sym_RBRACE] = ACTIONS(2204),
    [anon_sym_import] = ACTIONS(2206),
    [anon_sym_var] = ACTIONS(2206),
    [anon_sym_let] = ACTIONS(2206),
    [anon_sym_const] = ACTIONS(2206),
    [anon_sym_if] = ACTIONS(2206),
    [anon_sym_else] = ACTIONS(2206),
    [anon_sym_switch] = ACTIONS(2206),
    [anon_sym_for] = ACTIONS(2206),
    [anon_sym_LPAREN] = ACTIONS(2204),
    [anon_sym_while] = ACTIONS(2206),
    [anon_sym_do] = ACTIONS(2206),
    [anon_sym_try] = ACTIONS(2206),
    [anon_sym_with] = ACTIONS(2206),
    [anon_sym_break] = ACTIONS(2206),
    [anon_sym_continue] = ACTIONS(2206),
    [anon_sym_debugger] = ACTIONS(2206),
    [anon_sym_return] = ACTIONS(2206),
    [anon_sym_throw] = ACTIONS(2206),
    [anon_sym_SEMI] = ACTIONS(2204),
    [anon_sym_yield] = ACTIONS(2206),
    [anon_sym_LBRACK] = ACTIONS(2204),
    [anon_sym_LT] = ACTIONS(2204),
    [anon_sym_class] = ACTIONS(2206),
    [anon_sym_async] = ACTIONS(2206),
    [anon_sym_function] = ACTIONS(2206),
    [anon_sym_EQ_GT] = ACTIONS(2204),
    [anon_sym_new] = ACTIONS(2206),
    [anon_sym_await] = ACTIONS(2206),
    [anon_sym_PLUS] = ACTIONS(2206),
    [anon_sym_DASH] = ACTIONS(2206),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_typeof] = ACTIONS(2206),
    [anon_sym_void] = ACTIONS(2206),
    [anon_sym_delete] = ACTIONS(2206),
    [anon_sym_PLUS_PLUS] = ACTIONS(2204),
    [anon_sym_DASH_DASH] = ACTIONS(2204),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2204),
    [anon_sym_BQUOTE] = ACTIONS(2204),
    [sym_regex] = ACTIONS(2206),
    [sym_number] = ACTIONS(2204),
    [sym_identifier] = ACTIONS(2206),
    [sym_this] = ACTIONS(2206),
    [sym_super] = ACTIONS(2206),
    [sym_true] = ACTIONS(2206),
    [sym_false] = ACTIONS(2206),
    [sym_null] = ACTIONS(2206),
    [sym_undefined] = ACTIONS(2206),
    [anon_sym_AT] = ACTIONS(2204),
    [anon_sym_get] = ACTIONS(2206),
    [anon_sym_set] = ACTIONS(2206),
  },
  [1256] = {
    [ts_builtin_sym_end] = ACTIONS(3482),
    [anon_sym_export] = ACTIONS(3484),
    [anon_sym_LBRACE] = ACTIONS(3482),
    [anon_sym_RBRACE] = ACTIONS(3482),
    [anon_sym_import] = ACTIONS(3484),
    [anon_sym_var] = ACTIONS(3484),
    [anon_sym_let] = ACTIONS(3484),
    [anon_sym_const] = ACTIONS(3484),
    [anon_sym_if] = ACTIONS(3484),
    [anon_sym_switch] = ACTIONS(3484),
    [anon_sym_for] = ACTIONS(3484),
    [anon_sym_LPAREN] = ACTIONS(3482),
    [anon_sym_while] = ACTIONS(3484),
    [anon_sym_do] = ACTIONS(3484),
    [anon_sym_try] = ACTIONS(3484),
    [anon_sym_with] = ACTIONS(3484),
    [anon_sym_break] = ACTIONS(3484),
    [anon_sym_continue] = ACTIONS(3484),
    [anon_sym_debugger] = ACTIONS(3484),
    [anon_sym_return] = ACTIONS(3484),
    [anon_sym_throw] = ACTIONS(3484),
    [anon_sym_SEMI] = ACTIONS(3482),
    [anon_sym_yield] = ACTIONS(3484),
    [anon_sym_LBRACK] = ACTIONS(3482),
    [anon_sym_LT] = ACTIONS(3482),
    [anon_sym_class] = ACTIONS(3484),
    [anon_sym_async] = ACTIONS(3484),
    [anon_sym_function] = ACTIONS(3484),
    [anon_sym_EQ_GT] = ACTIONS(3482),
    [anon_sym_new] = ACTIONS(3484),
    [anon_sym_await] = ACTIONS(3484),
    [anon_sym_PLUS] = ACTIONS(3484),
    [anon_sym_DASH] = ACTIONS(3484),
    [anon_sym_BANG] = ACTIONS(3482),
    [anon_sym_TILDE] = ACTIONS(3482),
    [anon_sym_typeof] = ACTIONS(3484),
    [anon_sym_void] = ACTIONS(3484),
    [anon_sym_delete] = ACTIONS(3484),
    [anon_sym_PLUS_PLUS] = ACTIONS(3482),
    [anon_sym_DASH_DASH] = ACTIONS(3482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3482),
    [anon_sym_BQUOTE] = ACTIONS(3482),
    [sym_regex] = ACTIONS(3484),
    [sym_number] = ACTIONS(3482),
    [sym_identifier] = ACTIONS(3484),
    [sym_this] = ACTIONS(3484),
    [sym_super] = ACTIONS(3484),
    [sym_true] = ACTIONS(3484),
    [sym_false] = ACTIONS(3484),
    [sym_null] = ACTIONS(3484),
    [sym_undefined] = ACTIONS(3484),
    [anon_sym_AT] = ACTIONS(3482),
    [anon_sym_get] = ACTIONS(3484),
    [anon_sym_set] = ACTIONS(3484),
  },
  [1257] = {
    [sym_parenthesized_expression] = STATE(1602),
    [sym__expression] = STATE(1602),
    [sym_yield_expression] = STATE(1602),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1602),
    [sym_jsx_fragment] = STATE(1602),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1602),
    [sym_anonymous_class] = STATE(1602),
    [sym_class] = STATE(1602),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1602),
    [sym_generator_function] = STATE(1602),
    [sym_call_expression] = STATE(1602),
    [sym_new_expression] = STATE(1602),
    [sym_await_expression] = STATE(1602),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1602),
    [sym_augmented_assignment_expression] = STATE(1602),
    [sym_ternary_expression] = STATE(1602),
    [sym_binary_expression] = STATE(1602),
    [sym_unary_expression] = STATE(1602),
    [sym_update_expression] = STATE(1602),
    [sym_template_string] = STATE(1602),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3486),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3488),
    [sym_number] = ACTIONS(3486),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(3488),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3488),
    [sym_false] = ACTIONS(3488),
    [sym_null] = ACTIONS(3488),
    [sym_undefined] = ACTIONS(3488),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [1258] = {
    [sym_class_heritage] = STATE(1603),
    [sym_class_body] = STATE(1595),
    [anon_sym_LBRACE] = ACTIONS(2678),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [1259] = {
    [sym_export_statement] = STATE(1629),
    [sym__declaration] = STATE(1629),
    [sym_import_statement] = STATE(1629),
    [sym_expression_statement] = STATE(1629),
    [sym_variable_declaration] = STATE(1629),
    [sym_lexical_declaration] = STATE(1629),
    [sym_statement_block] = STATE(1629),
    [sym_if_statement] = STATE(1629),
    [sym_switch_statement] = STATE(1629),
    [sym_for_statement] = STATE(1629),
    [sym_for_in_statement] = STATE(1629),
    [sym_for_of_statement] = STATE(1629),
    [sym_while_statement] = STATE(1629),
    [sym_do_statement] = STATE(1629),
    [sym_try_statement] = STATE(1629),
    [sym_with_statement] = STATE(1629),
    [sym_break_statement] = STATE(1629),
    [sym_continue_statement] = STATE(1629),
    [sym_debugger_statement] = STATE(1629),
    [sym_return_statement] = STATE(1629),
    [sym_throw_statement] = STATE(1629),
    [sym_empty_statement] = STATE(1629),
    [sym_labeled_statement] = STATE(1629),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1629),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_default] = ACTIONS(3492),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_RBRACE] = ACTIONS(3496),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_case] = ACTIONS(3492),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1260] = {
    [sym_parenthesized_expression] = STATE(1631),
    [sym__expression] = STATE(1631),
    [sym_yield_expression] = STATE(1631),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1631),
    [sym_jsx_fragment] = STATE(1631),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1631),
    [sym_anonymous_class] = STATE(1631),
    [sym_class] = STATE(1631),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1631),
    [sym_generator_function] = STATE(1631),
    [sym_call_expression] = STATE(1631),
    [sym_new_expression] = STATE(1631),
    [sym_await_expression] = STATE(1631),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1631),
    [sym_augmented_assignment_expression] = STATE(1631),
    [sym_ternary_expression] = STATE(1631),
    [sym_binary_expression] = STATE(1631),
    [sym_unary_expression] = STATE(1631),
    [sym_update_expression] = STATE(1631),
    [sym_template_string] = STATE(1631),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_COLON] = ACTIONS(239),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(2706),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3538),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3540),
    [sym_number] = ACTIONS(3538),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(3540),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3540),
    [sym_false] = ACTIONS(3540),
    [sym_null] = ACTIONS(3540),
    [sym_undefined] = ACTIONS(3540),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1261] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(1635),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_COLON] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(3542),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(3544),
    [anon_sym_PLUS_EQ] = ACTIONS(3546),
    [anon_sym_DASH_EQ] = ACTIONS(3546),
    [anon_sym_STAR_EQ] = ACTIONS(3546),
    [anon_sym_SLASH_EQ] = ACTIONS(3546),
    [anon_sym_PERCENT_EQ] = ACTIONS(3546),
    [anon_sym_CARET_EQ] = ACTIONS(3546),
    [anon_sym_AMP_EQ] = ACTIONS(3546),
    [anon_sym_PIPE_EQ] = ACTIONS(3546),
    [anon_sym_GT_GT_EQ] = ACTIONS(3546),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3546),
    [anon_sym_LT_LT_EQ] = ACTIONS(3546),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(3548),
  },
  [1262] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(1636),
    [sym__expression] = STATE(1636),
    [sym_yield_expression] = STATE(1636),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1636),
    [sym_jsx_fragment] = STATE(1636),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1636),
    [sym_anonymous_class] = STATE(1636),
    [sym_class] = STATE(1636),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1636),
    [sym_generator_function] = STATE(1636),
    [sym_call_expression] = STATE(1636),
    [sym_new_expression] = STATE(1636),
    [sym_await_expression] = STATE(1636),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1636),
    [sym_augmented_assignment_expression] = STATE(1636),
    [sym_ternary_expression] = STATE(1636),
    [sym_binary_expression] = STATE(1636),
    [sym_unary_expression] = STATE(1636),
    [sym_update_expression] = STATE(1636),
    [sym_template_string] = STATE(1636),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3550),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3552),
    [sym_number] = ACTIONS(3550),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(3552),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3552),
    [sym_false] = ACTIONS(3552),
    [sym_null] = ACTIONS(3552),
    [sym_undefined] = ACTIONS(3552),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1263] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1264] = {
    [sym_parenthesized_expression] = STATE(1637),
    [sym__expression] = STATE(1637),
    [sym_yield_expression] = STATE(1637),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1637),
    [sym_jsx_fragment] = STATE(1637),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1637),
    [sym_anonymous_class] = STATE(1637),
    [sym_class] = STATE(1637),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1637),
    [sym_generator_function] = STATE(1637),
    [sym_call_expression] = STATE(1637),
    [sym_new_expression] = STATE(1637),
    [sym_await_expression] = STATE(1637),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1637),
    [sym_augmented_assignment_expression] = STATE(1637),
    [sym_ternary_expression] = STATE(1637),
    [sym_binary_expression] = STATE(1637),
    [sym_unary_expression] = STATE(1637),
    [sym_update_expression] = STATE(1637),
    [sym_template_string] = STATE(1637),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3554),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3556),
    [sym_number] = ACTIONS(3554),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(3556),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3556),
    [sym_false] = ACTIONS(3556),
    [sym_null] = ACTIONS(3556),
    [sym_undefined] = ACTIONS(3556),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1265] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1266] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1267] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(3542),
    [anon_sym_EQ_GT] = ACTIONS(3544),
    [anon_sym_PLUS_EQ] = ACTIONS(3560),
    [anon_sym_DASH_EQ] = ACTIONS(3560),
    [anon_sym_STAR_EQ] = ACTIONS(3560),
    [anon_sym_SLASH_EQ] = ACTIONS(3560),
    [anon_sym_PERCENT_EQ] = ACTIONS(3560),
    [anon_sym_CARET_EQ] = ACTIONS(3560),
    [anon_sym_AMP_EQ] = ACTIONS(3560),
    [anon_sym_PIPE_EQ] = ACTIONS(3560),
    [anon_sym_GT_GT_EQ] = ACTIONS(3560),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3560),
    [anon_sym_LT_LT_EQ] = ACTIONS(3560),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1268] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_COLON] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(3542),
    [anon_sym_PLUS_EQ] = ACTIONS(3546),
    [anon_sym_DASH_EQ] = ACTIONS(3546),
    [anon_sym_STAR_EQ] = ACTIONS(3546),
    [anon_sym_SLASH_EQ] = ACTIONS(3546),
    [anon_sym_PERCENT_EQ] = ACTIONS(3546),
    [anon_sym_CARET_EQ] = ACTIONS(3546),
    [anon_sym_AMP_EQ] = ACTIONS(3546),
    [anon_sym_PIPE_EQ] = ACTIONS(3546),
    [anon_sym_GT_GT_EQ] = ACTIONS(3546),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3546),
    [anon_sym_LT_LT_EQ] = ACTIONS(3546),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [1269] = {
    [anon_sym_COLON] = ACTIONS(3562),
    [sym_comment] = ACTIONS(71),
  },
  [1270] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(3566),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(368),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1271] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3542),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1272] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3542),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1273] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_COLON] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3542),
    [anon_sym_PLUS_EQ] = ACTIONS(3560),
    [anon_sym_DASH_EQ] = ACTIONS(3560),
    [anon_sym_STAR_EQ] = ACTIONS(3560),
    [anon_sym_SLASH_EQ] = ACTIONS(3560),
    [anon_sym_PERCENT_EQ] = ACTIONS(3560),
    [anon_sym_CARET_EQ] = ACTIONS(3560),
    [anon_sym_AMP_EQ] = ACTIONS(3560),
    [anon_sym_PIPE_EQ] = ACTIONS(3560),
    [anon_sym_GT_GT_EQ] = ACTIONS(3560),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3560),
    [anon_sym_LT_LT_EQ] = ACTIONS(3560),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1274] = {
    [anon_sym_EQ_GT] = ACTIONS(3544),
    [sym_comment] = ACTIONS(71),
  },
  [1275] = {
    [ts_builtin_sym_end] = ACTIONS(3584),
    [anon_sym_export] = ACTIONS(3586),
    [anon_sym_LBRACE] = ACTIONS(3584),
    [anon_sym_RBRACE] = ACTIONS(3584),
    [anon_sym_import] = ACTIONS(3586),
    [anon_sym_var] = ACTIONS(3586),
    [anon_sym_let] = ACTIONS(3586),
    [anon_sym_const] = ACTIONS(3586),
    [anon_sym_if] = ACTIONS(3586),
    [anon_sym_switch] = ACTIONS(3586),
    [anon_sym_for] = ACTIONS(3586),
    [anon_sym_LPAREN] = ACTIONS(3584),
    [anon_sym_while] = ACTIONS(3586),
    [anon_sym_do] = ACTIONS(3586),
    [anon_sym_try] = ACTIONS(3586),
    [anon_sym_with] = ACTIONS(3586),
    [anon_sym_break] = ACTIONS(3586),
    [anon_sym_continue] = ACTIONS(3586),
    [anon_sym_debugger] = ACTIONS(3586),
    [anon_sym_return] = ACTIONS(3586),
    [anon_sym_throw] = ACTIONS(3586),
    [anon_sym_SEMI] = ACTIONS(3584),
    [anon_sym_yield] = ACTIONS(3586),
    [anon_sym_LBRACK] = ACTIONS(3584),
    [anon_sym_LT] = ACTIONS(3584),
    [anon_sym_class] = ACTIONS(3586),
    [anon_sym_async] = ACTIONS(3586),
    [anon_sym_function] = ACTIONS(3586),
    [anon_sym_EQ_GT] = ACTIONS(3584),
    [anon_sym_new] = ACTIONS(3586),
    [anon_sym_await] = ACTIONS(3586),
    [anon_sym_PLUS] = ACTIONS(3586),
    [anon_sym_DASH] = ACTIONS(3586),
    [anon_sym_BANG] = ACTIONS(3584),
    [anon_sym_TILDE] = ACTIONS(3584),
    [anon_sym_typeof] = ACTIONS(3586),
    [anon_sym_void] = ACTIONS(3586),
    [anon_sym_delete] = ACTIONS(3586),
    [anon_sym_PLUS_PLUS] = ACTIONS(3584),
    [anon_sym_DASH_DASH] = ACTIONS(3584),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3584),
    [anon_sym_BQUOTE] = ACTIONS(3584),
    [sym_regex] = ACTIONS(3586),
    [sym_number] = ACTIONS(3584),
    [sym_identifier] = ACTIONS(3586),
    [sym_this] = ACTIONS(3586),
    [sym_super] = ACTIONS(3586),
    [sym_true] = ACTIONS(3586),
    [sym_false] = ACTIONS(3586),
    [sym_null] = ACTIONS(3586),
    [sym_undefined] = ACTIONS(3586),
    [anon_sym_AT] = ACTIONS(3584),
    [anon_sym_get] = ACTIONS(3586),
    [anon_sym_set] = ACTIONS(3586),
  },
  [1276] = {
    [sym_switch_case] = STATE(1276),
    [sym_switch_default] = STATE(1276),
    [aux_sym_switch_body_repeat1] = STATE(1276),
    [anon_sym_default] = ACTIONS(3588),
    [anon_sym_RBRACE] = ACTIONS(3591),
    [anon_sym_case] = ACTIONS(3593),
    [sym_comment] = ACTIONS(71),
  },
  [1277] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1647),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1647),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1278] = {
    [sym_parenthesized_expression] = STATE(1648),
    [sym__expression] = STATE(1648),
    [sym_yield_expression] = STATE(1648),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1648),
    [sym_jsx_fragment] = STATE(1648),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1648),
    [sym_anonymous_class] = STATE(1648),
    [sym_class] = STATE(1648),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1648),
    [sym_generator_function] = STATE(1648),
    [sym_call_expression] = STATE(1648),
    [sym_new_expression] = STATE(1648),
    [sym_await_expression] = STATE(1648),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1648),
    [sym_augmented_assignment_expression] = STATE(1648),
    [sym_ternary_expression] = STATE(1648),
    [sym_binary_expression] = STATE(1648),
    [sym_unary_expression] = STATE(1648),
    [sym_update_expression] = STATE(1648),
    [sym_template_string] = STATE(1648),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3596),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3598),
    [sym_number] = ACTIONS(3596),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3598),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3598),
    [sym_false] = ACTIONS(3598),
    [sym_null] = ACTIONS(3598),
    [sym_undefined] = ACTIONS(3598),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1279] = {
    [anon_sym_LBRACE] = ACTIONS(1641),
    [anon_sym_LPAREN] = ACTIONS(1641),
    [anon_sym_SEMI] = ACTIONS(1641),
    [anon_sym_yield] = ACTIONS(1643),
    [anon_sym_LBRACK] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1641),
    [anon_sym_class] = ACTIONS(1643),
    [anon_sym_async] = ACTIONS(1643),
    [anon_sym_function] = ACTIONS(1643),
    [anon_sym_EQ_GT] = ACTIONS(1641),
    [anon_sym_new] = ACTIONS(1643),
    [anon_sym_await] = ACTIONS(1643),
    [anon_sym_PLUS] = ACTIONS(1643),
    [anon_sym_DASH] = ACTIONS(1643),
    [anon_sym_BANG] = ACTIONS(1641),
    [anon_sym_TILDE] = ACTIONS(1641),
    [anon_sym_typeof] = ACTIONS(1643),
    [anon_sym_void] = ACTIONS(1643),
    [anon_sym_delete] = ACTIONS(1643),
    [anon_sym_PLUS_PLUS] = ACTIONS(1641),
    [anon_sym_DASH_DASH] = ACTIONS(1641),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1641),
    [anon_sym_BQUOTE] = ACTIONS(1641),
    [sym_regex] = ACTIONS(1643),
    [sym_number] = ACTIONS(1641),
    [sym_identifier] = ACTIONS(1643),
    [sym_this] = ACTIONS(1643),
    [sym_super] = ACTIONS(1643),
    [sym_true] = ACTIONS(1643),
    [sym_false] = ACTIONS(1643),
    [sym_null] = ACTIONS(1643),
    [sym_undefined] = ACTIONS(1643),
    [anon_sym_AT] = ACTIONS(1641),
    [anon_sym_get] = ACTIONS(1643),
    [anon_sym_set] = ACTIONS(1643),
  },
  [1280] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(3600),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(3600),
    [sym_comment] = ACTIONS(71),
  },
  [1281] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1650),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1650),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1282] = {
    [sym_parenthesized_expression] = STATE(1651),
    [sym__expression] = STATE(1651),
    [sym_yield_expression] = STATE(1651),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1651),
    [sym_jsx_fragment] = STATE(1651),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1651),
    [sym_anonymous_class] = STATE(1651),
    [sym_class] = STATE(1651),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1651),
    [sym_generator_function] = STATE(1651),
    [sym_call_expression] = STATE(1651),
    [sym_new_expression] = STATE(1651),
    [sym_await_expression] = STATE(1651),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1651),
    [sym_augmented_assignment_expression] = STATE(1651),
    [sym_ternary_expression] = STATE(1651),
    [sym_binary_expression] = STATE(1651),
    [sym_unary_expression] = STATE(1651),
    [sym_update_expression] = STATE(1651),
    [sym_template_string] = STATE(1651),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3602),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3604),
    [sym_number] = ACTIONS(3602),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3604),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3604),
    [sym_false] = ACTIONS(3604),
    [sym_null] = ACTIONS(3604),
    [sym_undefined] = ACTIONS(3604),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1283] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1652),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1652),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1284] = {
    [sym_parenthesized_expression] = STATE(1653),
    [sym__expression] = STATE(1653),
    [sym_yield_expression] = STATE(1653),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1653),
    [sym_jsx_fragment] = STATE(1653),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1653),
    [sym_anonymous_class] = STATE(1653),
    [sym_class] = STATE(1653),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1653),
    [sym_generator_function] = STATE(1653),
    [sym_call_expression] = STATE(1653),
    [sym_new_expression] = STATE(1653),
    [sym_await_expression] = STATE(1653),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1653),
    [sym_augmented_assignment_expression] = STATE(1653),
    [sym_ternary_expression] = STATE(1653),
    [sym_binary_expression] = STATE(1653),
    [sym_unary_expression] = STATE(1653),
    [sym_update_expression] = STATE(1653),
    [sym_template_string] = STATE(1653),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3606),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3608),
    [sym_number] = ACTIONS(3606),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3608),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3608),
    [sym_false] = ACTIONS(3608),
    [sym_null] = ACTIONS(3608),
    [sym_undefined] = ACTIONS(3608),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1285] = {
    [anon_sym_LBRACE] = ACTIONS(1651),
    [anon_sym_LPAREN] = ACTIONS(1651),
    [anon_sym_SEMI] = ACTIONS(1651),
    [anon_sym_yield] = ACTIONS(1653),
    [anon_sym_LBRACK] = ACTIONS(1651),
    [anon_sym_LT] = ACTIONS(1651),
    [anon_sym_class] = ACTIONS(1653),
    [anon_sym_async] = ACTIONS(1653),
    [anon_sym_function] = ACTIONS(1653),
    [anon_sym_EQ_GT] = ACTIONS(1651),
    [anon_sym_new] = ACTIONS(1653),
    [anon_sym_await] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1653),
    [anon_sym_BANG] = ACTIONS(1651),
    [anon_sym_TILDE] = ACTIONS(1651),
    [anon_sym_typeof] = ACTIONS(1653),
    [anon_sym_void] = ACTIONS(1653),
    [anon_sym_delete] = ACTIONS(1653),
    [anon_sym_PLUS_PLUS] = ACTIONS(1651),
    [anon_sym_DASH_DASH] = ACTIONS(1651),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1651),
    [anon_sym_BQUOTE] = ACTIONS(1651),
    [sym_regex] = ACTIONS(1653),
    [sym_number] = ACTIONS(1651),
    [sym_identifier] = ACTIONS(1653),
    [sym_this] = ACTIONS(1653),
    [sym_super] = ACTIONS(1653),
    [sym_true] = ACTIONS(1653),
    [sym_false] = ACTIONS(1653),
    [sym_null] = ACTIONS(1653),
    [sym_undefined] = ACTIONS(1653),
    [anon_sym_AT] = ACTIONS(1651),
    [anon_sym_get] = ACTIONS(1653),
    [anon_sym_set] = ACTIONS(1653),
  },
  [1286] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(3610),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(3610),
    [sym_comment] = ACTIONS(71),
  },
  [1287] = {
    [sym_parenthesized_expression] = STATE(1655),
    [sym__expression] = STATE(1655),
    [sym_yield_expression] = STATE(1655),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1655),
    [sym_jsx_fragment] = STATE(1655),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1655),
    [sym_anonymous_class] = STATE(1655),
    [sym_class] = STATE(1655),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1655),
    [sym_generator_function] = STATE(1655),
    [sym_call_expression] = STATE(1655),
    [sym_new_expression] = STATE(1655),
    [sym_await_expression] = STATE(1655),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1655),
    [sym_augmented_assignment_expression] = STATE(1655),
    [sym_ternary_expression] = STATE(1655),
    [sym_binary_expression] = STATE(1655),
    [sym_unary_expression] = STATE(1655),
    [sym_update_expression] = STATE(1655),
    [sym_template_string] = STATE(1655),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(239),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_COMMA] = ACTIONS(239),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_RPAREN] = ACTIONS(239),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_SEMI] = ACTIONS(239),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(243),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(2748),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3612),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3614),
    [sym_number] = ACTIONS(3612),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(3614),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3614),
    [sym_false] = ACTIONS(3614),
    [sym_null] = ACTIONS(3614),
    [sym_undefined] = ACTIONS(3614),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1288] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(1659),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_RPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(3616),
    [anon_sym_function] = ACTIONS(826),
    [anon_sym_EQ_GT] = ACTIONS(3618),
    [anon_sym_PLUS_EQ] = ACTIONS(3620),
    [anon_sym_DASH_EQ] = ACTIONS(3620),
    [anon_sym_STAR_EQ] = ACTIONS(3620),
    [anon_sym_SLASH_EQ] = ACTIONS(3620),
    [anon_sym_PERCENT_EQ] = ACTIONS(3620),
    [anon_sym_CARET_EQ] = ACTIONS(3620),
    [anon_sym_AMP_EQ] = ACTIONS(3620),
    [anon_sym_PIPE_EQ] = ACTIONS(3620),
    [anon_sym_GT_GT_EQ] = ACTIONS(3620),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3620),
    [anon_sym_LT_LT_EQ] = ACTIONS(3620),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(3622),
  },
  [1289] = {
    [sym_statement_block] = STATE(201),
    [sym_parenthesized_expression] = STATE(1660),
    [sym__expression] = STATE(1660),
    [sym_yield_expression] = STATE(1660),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1660),
    [sym_jsx_fragment] = STATE(1660),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1660),
    [sym_anonymous_class] = STATE(1660),
    [sym_class] = STATE(1660),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1660),
    [sym_generator_function] = STATE(1660),
    [sym_call_expression] = STATE(1660),
    [sym_new_expression] = STATE(1660),
    [sym_await_expression] = STATE(1660),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1660),
    [sym_augmented_assignment_expression] = STATE(1660),
    [sym_ternary_expression] = STATE(1660),
    [sym_binary_expression] = STATE(1660),
    [sym_unary_expression] = STATE(1660),
    [sym_update_expression] = STATE(1660),
    [sym_template_string] = STATE(1660),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3624),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3626),
    [sym_number] = ACTIONS(3624),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(3626),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3626),
    [sym_false] = ACTIONS(3626),
    [sym_null] = ACTIONS(3626),
    [sym_undefined] = ACTIONS(3626),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1290] = {
    [sym_parenthesized_expression] = STATE(203),
    [sym__expression] = STATE(203),
    [sym_yield_expression] = STATE(203),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(203),
    [sym_jsx_fragment] = STATE(203),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(203),
    [sym_anonymous_class] = STATE(203),
    [sym_class] = STATE(203),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(203),
    [sym_generator_function] = STATE(203),
    [sym_call_expression] = STATE(203),
    [sym_new_expression] = STATE(203),
    [sym_await_expression] = STATE(203),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(203),
    [sym_augmented_assignment_expression] = STATE(203),
    [sym_ternary_expression] = STATE(203),
    [sym_binary_expression] = STATE(203),
    [sym_unary_expression] = STATE(203),
    [sym_update_expression] = STATE(203),
    [sym_template_string] = STATE(203),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(322),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(324),
    [sym_number] = ACTIONS(322),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(324),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(324),
    [sym_false] = ACTIONS(324),
    [sym_null] = ACTIONS(324),
    [sym_undefined] = ACTIONS(324),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1291] = {
    [sym_parenthesized_expression] = STATE(1661),
    [sym__expression] = STATE(1661),
    [sym_yield_expression] = STATE(1661),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1661),
    [sym_jsx_fragment] = STATE(1661),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1661),
    [sym_anonymous_class] = STATE(1661),
    [sym_class] = STATE(1661),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1661),
    [sym_generator_function] = STATE(1661),
    [sym_call_expression] = STATE(1661),
    [sym_new_expression] = STATE(1661),
    [sym_await_expression] = STATE(1661),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1661),
    [sym_augmented_assignment_expression] = STATE(1661),
    [sym_ternary_expression] = STATE(1661),
    [sym_binary_expression] = STATE(1661),
    [sym_unary_expression] = STATE(1661),
    [sym_update_expression] = STATE(1661),
    [sym_template_string] = STATE(1661),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3628),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3630),
    [sym_number] = ACTIONS(3628),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(3630),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3630),
    [sym_false] = ACTIONS(3630),
    [sym_null] = ACTIONS(3630),
    [sym_undefined] = ACTIONS(3630),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1292] = {
    [sym_parenthesized_expression] = STATE(205),
    [sym__expression] = STATE(205),
    [sym_yield_expression] = STATE(205),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(205),
    [sym_jsx_fragment] = STATE(205),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(205),
    [sym_anonymous_class] = STATE(205),
    [sym_class] = STATE(205),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(205),
    [sym_generator_function] = STATE(205),
    [sym_call_expression] = STATE(205),
    [sym_new_expression] = STATE(205),
    [sym_await_expression] = STATE(205),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(205),
    [sym_augmented_assignment_expression] = STATE(205),
    [sym_ternary_expression] = STATE(205),
    [sym_binary_expression] = STATE(205),
    [sym_unary_expression] = STATE(205),
    [sym_update_expression] = STATE(205),
    [sym_template_string] = STATE(205),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(330),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(332),
    [sym_number] = ACTIONS(330),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(332),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(332),
    [sym_false] = ACTIONS(332),
    [sym_null] = ACTIONS(332),
    [sym_undefined] = ACTIONS(332),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1293] = {
    [sym_parenthesized_expression] = STATE(206),
    [sym__expression] = STATE(206),
    [sym_yield_expression] = STATE(206),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(206),
    [sym_jsx_fragment] = STATE(206),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(206),
    [sym_anonymous_class] = STATE(206),
    [sym_class] = STATE(206),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(206),
    [sym_generator_function] = STATE(206),
    [sym_call_expression] = STATE(206),
    [sym_new_expression] = STATE(206),
    [sym_await_expression] = STATE(206),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(206),
    [sym_augmented_assignment_expression] = STATE(206),
    [sym_ternary_expression] = STATE(206),
    [sym_binary_expression] = STATE(206),
    [sym_unary_expression] = STATE(206),
    [sym_update_expression] = STATE(206),
    [sym_template_string] = STATE(206),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(334),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(336),
    [sym_number] = ACTIONS(334),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(336),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(336),
    [sym_false] = ACTIONS(336),
    [sym_null] = ACTIONS(336),
    [sym_undefined] = ACTIONS(336),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1294] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(3616),
    [anon_sym_EQ_GT] = ACTIONS(3618),
    [anon_sym_PLUS_EQ] = ACTIONS(3634),
    [anon_sym_DASH_EQ] = ACTIONS(3634),
    [anon_sym_STAR_EQ] = ACTIONS(3634),
    [anon_sym_SLASH_EQ] = ACTIONS(3634),
    [anon_sym_PERCENT_EQ] = ACTIONS(3634),
    [anon_sym_CARET_EQ] = ACTIONS(3634),
    [anon_sym_AMP_EQ] = ACTIONS(3634),
    [anon_sym_PIPE_EQ] = ACTIONS(3634),
    [anon_sym_GT_GT_EQ] = ACTIONS(3634),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3634),
    [anon_sym_LT_LT_EQ] = ACTIONS(3634),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1295] = {
    [sym__initializer] = STATE(196),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_RPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(3616),
    [anon_sym_PLUS_EQ] = ACTIONS(3620),
    [anon_sym_DASH_EQ] = ACTIONS(3620),
    [anon_sym_STAR_EQ] = ACTIONS(3620),
    [anon_sym_SLASH_EQ] = ACTIONS(3620),
    [anon_sym_PERCENT_EQ] = ACTIONS(3620),
    [anon_sym_CARET_EQ] = ACTIONS(3620),
    [anon_sym_AMP_EQ] = ACTIONS(3620),
    [anon_sym_PIPE_EQ] = ACTIONS(3620),
    [anon_sym_GT_GT_EQ] = ACTIONS(3620),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3620),
    [anon_sym_LT_LT_EQ] = ACTIONS(3620),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [1296] = {
    [anon_sym_RPAREN] = ACTIONS(3636),
    [sym_comment] = ACTIONS(71),
  },
  [1297] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(368),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1298] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3616),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1299] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3616),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1300] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3616),
    [anon_sym_PLUS_EQ] = ACTIONS(3634),
    [anon_sym_DASH_EQ] = ACTIONS(3634),
    [anon_sym_STAR_EQ] = ACTIONS(3634),
    [anon_sym_SLASH_EQ] = ACTIONS(3634),
    [anon_sym_PERCENT_EQ] = ACTIONS(3634),
    [anon_sym_CARET_EQ] = ACTIONS(3634),
    [anon_sym_AMP_EQ] = ACTIONS(3634),
    [anon_sym_PIPE_EQ] = ACTIONS(3634),
    [anon_sym_GT_GT_EQ] = ACTIONS(3634),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3634),
    [anon_sym_LT_LT_EQ] = ACTIONS(3634),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1301] = {
    [anon_sym_EQ_GT] = ACTIONS(3618),
    [sym_comment] = ACTIONS(71),
  },
  [1302] = {
    [sym_parenthesized_expression] = STATE(1670),
    [sym__expression] = STATE(1670),
    [sym_yield_expression] = STATE(1670),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1670),
    [sym_jsx_fragment] = STATE(1670),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1670),
    [sym_anonymous_class] = STATE(1670),
    [sym_class] = STATE(1670),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1670),
    [sym_generator_function] = STATE(1670),
    [sym_call_expression] = STATE(1670),
    [sym_new_expression] = STATE(1670),
    [sym_await_expression] = STATE(1670),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1670),
    [sym_augmented_assignment_expression] = STATE(1670),
    [sym_ternary_expression] = STATE(1670),
    [sym_binary_expression] = STATE(1670),
    [sym_unary_expression] = STATE(1670),
    [sym_update_expression] = STATE(1670),
    [sym_template_string] = STATE(1670),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_STAR] = ACTIONS(239),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(239),
    [anon_sym_in] = ACTIONS(241),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(686),
    [anon_sym_GT] = ACTIONS(241),
    [anon_sym_SLASH] = ACTIONS(241),
    [anon_sym_DOT] = ACTIONS(241),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_QMARK] = ACTIONS(239),
    [anon_sym_AMP_AMP] = ACTIONS(239),
    [anon_sym_PIPE_PIPE] = ACTIONS(239),
    [anon_sym_GT_GT] = ACTIONS(241),
    [anon_sym_GT_GT_GT] = ACTIONS(239),
    [anon_sym_LT_LT] = ACTIONS(239),
    [anon_sym_AMP] = ACTIONS(241),
    [anon_sym_CARET] = ACTIONS(239),
    [anon_sym_PIPE] = ACTIONS(241),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_PERCENT] = ACTIONS(239),
    [anon_sym_LT_EQ] = ACTIONS(239),
    [anon_sym_EQ_EQ] = ACTIONS(241),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(239),
    [anon_sym_BANG_EQ] = ACTIONS(241),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(239),
    [anon_sym_GT_EQ] = ACTIONS(239),
    [anon_sym_instanceof] = ACTIONS(241),
    [anon_sym_BANG] = ACTIONS(2772),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3642),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3644),
    [sym_number] = ACTIONS(3642),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3644),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3644),
    [sym_false] = ACTIONS(3644),
    [sym_null] = ACTIONS(3644),
    [sym_undefined] = ACTIONS(3644),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1303] = {
    [sym__initializer] = STATE(428),
    [sym_formal_parameters] = STATE(1674),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_RPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(3646),
    [anon_sym_function] = ACTIONS(718),
    [anon_sym_EQ_GT] = ACTIONS(3648),
    [anon_sym_PLUS_EQ] = ACTIONS(3650),
    [anon_sym_DASH_EQ] = ACTIONS(3650),
    [anon_sym_STAR_EQ] = ACTIONS(3650),
    [anon_sym_SLASH_EQ] = ACTIONS(3650),
    [anon_sym_PERCENT_EQ] = ACTIONS(3650),
    [anon_sym_CARET_EQ] = ACTIONS(3650),
    [anon_sym_AMP_EQ] = ACTIONS(3650),
    [anon_sym_PIPE_EQ] = ACTIONS(3650),
    [anon_sym_GT_GT_EQ] = ACTIONS(3650),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3650),
    [anon_sym_LT_LT_EQ] = ACTIONS(3650),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(3652),
  },
  [1304] = {
    [sym_statement_block] = STATE(434),
    [sym_parenthesized_expression] = STATE(1675),
    [sym__expression] = STATE(1675),
    [sym_yield_expression] = STATE(1675),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1675),
    [sym_jsx_fragment] = STATE(1675),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1675),
    [sym_anonymous_class] = STATE(1675),
    [sym_class] = STATE(1675),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1675),
    [sym_generator_function] = STATE(1675),
    [sym_call_expression] = STATE(1675),
    [sym_new_expression] = STATE(1675),
    [sym_await_expression] = STATE(1675),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1675),
    [sym_augmented_assignment_expression] = STATE(1675),
    [sym_ternary_expression] = STATE(1675),
    [sym_binary_expression] = STATE(1675),
    [sym_unary_expression] = STATE(1675),
    [sym_update_expression] = STATE(1675),
    [sym_template_string] = STATE(1675),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3654),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3656),
    [sym_number] = ACTIONS(3654),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3656),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3656),
    [sym_false] = ACTIONS(3656),
    [sym_null] = ACTIONS(3656),
    [sym_undefined] = ACTIONS(3656),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1305] = {
    [sym_parenthesized_expression] = STATE(436),
    [sym__expression] = STATE(436),
    [sym_yield_expression] = STATE(436),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(436),
    [sym_jsx_fragment] = STATE(436),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(436),
    [sym_anonymous_class] = STATE(436),
    [sym_class] = STATE(436),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(436),
    [sym_generator_function] = STATE(436),
    [sym_call_expression] = STATE(436),
    [sym_new_expression] = STATE(436),
    [sym_await_expression] = STATE(436),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(436),
    [sym_augmented_assignment_expression] = STATE(436),
    [sym_ternary_expression] = STATE(436),
    [sym_binary_expression] = STATE(436),
    [sym_unary_expression] = STATE(436),
    [sym_update_expression] = STATE(436),
    [sym_template_string] = STATE(436),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(736),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(738),
    [sym_number] = ACTIONS(736),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(738),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(738),
    [sym_false] = ACTIONS(738),
    [sym_null] = ACTIONS(738),
    [sym_undefined] = ACTIONS(738),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1306] = {
    [sym_parenthesized_expression] = STATE(1676),
    [sym__expression] = STATE(1676),
    [sym_yield_expression] = STATE(1676),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1676),
    [sym_jsx_fragment] = STATE(1676),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1676),
    [sym_anonymous_class] = STATE(1676),
    [sym_class] = STATE(1676),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1676),
    [sym_generator_function] = STATE(1676),
    [sym_call_expression] = STATE(1676),
    [sym_new_expression] = STATE(1676),
    [sym_await_expression] = STATE(1676),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1676),
    [sym_augmented_assignment_expression] = STATE(1676),
    [sym_ternary_expression] = STATE(1676),
    [sym_binary_expression] = STATE(1676),
    [sym_unary_expression] = STATE(1676),
    [sym_update_expression] = STATE(1676),
    [sym_template_string] = STATE(1676),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3658),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3660),
    [sym_number] = ACTIONS(3658),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3660),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3660),
    [sym_false] = ACTIONS(3660),
    [sym_null] = ACTIONS(3660),
    [sym_undefined] = ACTIONS(3660),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1307] = {
    [sym_parenthesized_expression] = STATE(439),
    [sym__expression] = STATE(439),
    [sym_yield_expression] = STATE(439),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(439),
    [sym_jsx_fragment] = STATE(439),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(439),
    [sym_anonymous_class] = STATE(439),
    [sym_class] = STATE(439),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(439),
    [sym_generator_function] = STATE(439),
    [sym_call_expression] = STATE(439),
    [sym_new_expression] = STATE(439),
    [sym_await_expression] = STATE(439),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(439),
    [sym_augmented_assignment_expression] = STATE(439),
    [sym_ternary_expression] = STATE(439),
    [sym_binary_expression] = STATE(439),
    [sym_unary_expression] = STATE(439),
    [sym_update_expression] = STATE(439),
    [sym_template_string] = STATE(439),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(746),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(748),
    [sym_number] = ACTIONS(746),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(748),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(748),
    [sym_false] = ACTIONS(748),
    [sym_null] = ACTIONS(748),
    [sym_undefined] = ACTIONS(748),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1308] = {
    [sym_parenthesized_expression] = STATE(440),
    [sym__expression] = STATE(440),
    [sym_yield_expression] = STATE(440),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(440),
    [sym_jsx_fragment] = STATE(440),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(440),
    [sym_anonymous_class] = STATE(440),
    [sym_class] = STATE(440),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(440),
    [sym_generator_function] = STATE(440),
    [sym_call_expression] = STATE(440),
    [sym_new_expression] = STATE(440),
    [sym_await_expression] = STATE(440),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(440),
    [sym_augmented_assignment_expression] = STATE(440),
    [sym_ternary_expression] = STATE(440),
    [sym_binary_expression] = STATE(440),
    [sym_unary_expression] = STATE(440),
    [sym_update_expression] = STATE(440),
    [sym_template_string] = STATE(440),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(750),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(752),
    [sym_number] = ACTIONS(750),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(752),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(752),
    [sym_false] = ACTIONS(752),
    [sym_null] = ACTIONS(752),
    [sym_undefined] = ACTIONS(752),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1309] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_EQ] = ACTIONS(3646),
    [anon_sym_EQ_GT] = ACTIONS(3648),
    [anon_sym_PLUS_EQ] = ACTIONS(3664),
    [anon_sym_DASH_EQ] = ACTIONS(3664),
    [anon_sym_STAR_EQ] = ACTIONS(3664),
    [anon_sym_SLASH_EQ] = ACTIONS(3664),
    [anon_sym_PERCENT_EQ] = ACTIONS(3664),
    [anon_sym_CARET_EQ] = ACTIONS(3664),
    [anon_sym_AMP_EQ] = ACTIONS(3664),
    [anon_sym_PIPE_EQ] = ACTIONS(3664),
    [anon_sym_GT_GT_EQ] = ACTIONS(3664),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3664),
    [anon_sym_LT_LT_EQ] = ACTIONS(3664),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1310] = {
    [sym__initializer] = STATE(428),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_LPAREN] = ACTIONS(291),
    [anon_sym_RPAREN] = ACTIONS(291),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(714),
    [anon_sym_EQ] = ACTIONS(3646),
    [anon_sym_PLUS_EQ] = ACTIONS(3650),
    [anon_sym_DASH_EQ] = ACTIONS(3650),
    [anon_sym_STAR_EQ] = ACTIONS(3650),
    [anon_sym_SLASH_EQ] = ACTIONS(3650),
    [anon_sym_PERCENT_EQ] = ACTIONS(3650),
    [anon_sym_CARET_EQ] = ACTIONS(3650),
    [anon_sym_AMP_EQ] = ACTIONS(3650),
    [anon_sym_PIPE_EQ] = ACTIONS(3650),
    [anon_sym_GT_GT_EQ] = ACTIONS(3650),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3650),
    [anon_sym_LT_LT_EQ] = ACTIONS(3650),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(291),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
  },
  [1311] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3668),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1312] = {
    [sym__initializer] = STATE(526),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3646),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1313] = {
    [sym__initializer] = STATE(527),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3646),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1314] = {
    [sym__initializer] = STATE(469),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_RPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(3646),
    [anon_sym_PLUS_EQ] = ACTIONS(3664),
    [anon_sym_DASH_EQ] = ACTIONS(3664),
    [anon_sym_STAR_EQ] = ACTIONS(3664),
    [anon_sym_SLASH_EQ] = ACTIONS(3664),
    [anon_sym_PERCENT_EQ] = ACTIONS(3664),
    [anon_sym_CARET_EQ] = ACTIONS(3664),
    [anon_sym_AMP_EQ] = ACTIONS(3664),
    [anon_sym_PIPE_EQ] = ACTIONS(3664),
    [anon_sym_GT_GT_EQ] = ACTIONS(3664),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3664),
    [anon_sym_LT_LT_EQ] = ACTIONS(3664),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1315] = {
    [anon_sym_EQ_GT] = ACTIONS(3648),
    [sym_comment] = ACTIONS(71),
  },
  [1316] = {
    [sym_export_statement] = STATE(1685),
    [sym__declaration] = STATE(1685),
    [sym_import_statement] = STATE(1685),
    [sym_expression_statement] = STATE(1685),
    [sym_variable_declaration] = STATE(1685),
    [sym_lexical_declaration] = STATE(1685),
    [sym_statement_block] = STATE(1685),
    [sym_if_statement] = STATE(1685),
    [sym_switch_statement] = STATE(1685),
    [sym_for_statement] = STATE(1685),
    [sym_for_in_statement] = STATE(1685),
    [sym_for_of_statement] = STATE(1685),
    [sym_while_statement] = STATE(1685),
    [sym_do_statement] = STATE(1685),
    [sym_try_statement] = STATE(1685),
    [sym_with_statement] = STATE(1685),
    [sym_break_statement] = STATE(1685),
    [sym_continue_statement] = STATE(1685),
    [sym_debugger_statement] = STATE(1685),
    [sym_return_statement] = STATE(1685),
    [sym_throw_statement] = STATE(1685),
    [sym_empty_statement] = STATE(1685),
    [sym_labeled_statement] = STATE(1685),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1317] = {
    [anon_sym_RPAREN] = ACTIONS(3686),
    [sym_comment] = ACTIONS(71),
  },
  [1318] = {
    [anon_sym_RPAREN] = ACTIONS(3688),
    [sym_comment] = ACTIONS(71),
  },
  [1319] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3690),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1320] = {
    [anon_sym_RPAREN] = ACTIONS(3692),
    [sym_comment] = ACTIONS(71),
  },
  [1321] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3694),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1322] = {
    [anon_sym_STAR] = ACTIONS(2491),
    [anon_sym_LBRACE] = ACTIONS(2491),
    [anon_sym_COMMA] = ACTIONS(2491),
    [anon_sym_RBRACE] = ACTIONS(2491),
    [anon_sym_LPAREN] = ACTIONS(2491),
    [anon_sym_RPAREN] = ACTIONS(2491),
    [anon_sym_in] = ACTIONS(2493),
    [anon_sym_COLON] = ACTIONS(2491),
    [anon_sym_LBRACK] = ACTIONS(2491),
    [anon_sym_RBRACK] = ACTIONS(2491),
    [anon_sym_LT] = ACTIONS(2493),
    [anon_sym_GT] = ACTIONS(2493),
    [anon_sym_SLASH] = ACTIONS(2493),
    [anon_sym_DOT] = ACTIONS(2491),
    [anon_sym_EQ] = ACTIONS(2493),
    [anon_sym_QMARK] = ACTIONS(2491),
    [anon_sym_AMP_AMP] = ACTIONS(2491),
    [anon_sym_PIPE_PIPE] = ACTIONS(2491),
    [anon_sym_GT_GT] = ACTIONS(2493),
    [anon_sym_GT_GT_GT] = ACTIONS(2491),
    [anon_sym_LT_LT] = ACTIONS(2491),
    [anon_sym_AMP] = ACTIONS(2493),
    [anon_sym_CARET] = ACTIONS(2491),
    [anon_sym_PIPE] = ACTIONS(2493),
    [anon_sym_PLUS] = ACTIONS(2493),
    [anon_sym_DASH] = ACTIONS(2493),
    [anon_sym_PERCENT] = ACTIONS(2491),
    [anon_sym_LT_EQ] = ACTIONS(2491),
    [anon_sym_EQ_EQ] = ACTIONS(2493),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2491),
    [anon_sym_BANG_EQ] = ACTIONS(2493),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2491),
    [anon_sym_GT_EQ] = ACTIONS(2491),
    [anon_sym_instanceof] = ACTIONS(2491),
    [anon_sym_PLUS_PLUS] = ACTIONS(2491),
    [anon_sym_DASH_DASH] = ACTIONS(2491),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2491),
  },
  [1323] = {
    [anon_sym_STAR] = ACTIONS(2583),
    [anon_sym_LBRACE] = ACTIONS(2583),
    [anon_sym_COMMA] = ACTIONS(2583),
    [anon_sym_RBRACE] = ACTIONS(2583),
    [anon_sym_LPAREN] = ACTIONS(2583),
    [anon_sym_RPAREN] = ACTIONS(2583),
    [anon_sym_in] = ACTIONS(2585),
    [anon_sym_COLON] = ACTIONS(2583),
    [anon_sym_LBRACK] = ACTIONS(2583),
    [anon_sym_RBRACK] = ACTIONS(2583),
    [anon_sym_LT] = ACTIONS(2585),
    [anon_sym_GT] = ACTIONS(2585),
    [anon_sym_SLASH] = ACTIONS(2585),
    [anon_sym_DOT] = ACTIONS(2583),
    [anon_sym_EQ] = ACTIONS(2585),
    [anon_sym_QMARK] = ACTIONS(2583),
    [anon_sym_AMP_AMP] = ACTIONS(2583),
    [anon_sym_PIPE_PIPE] = ACTIONS(2583),
    [anon_sym_GT_GT] = ACTIONS(2585),
    [anon_sym_GT_GT_GT] = ACTIONS(2583),
    [anon_sym_LT_LT] = ACTIONS(2583),
    [anon_sym_AMP] = ACTIONS(2585),
    [anon_sym_CARET] = ACTIONS(2583),
    [anon_sym_PIPE] = ACTIONS(2585),
    [anon_sym_PLUS] = ACTIONS(2585),
    [anon_sym_DASH] = ACTIONS(2585),
    [anon_sym_PERCENT] = ACTIONS(2583),
    [anon_sym_LT_EQ] = ACTIONS(2583),
    [anon_sym_EQ_EQ] = ACTIONS(2585),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2583),
    [anon_sym_BANG_EQ] = ACTIONS(2585),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2583),
    [anon_sym_GT_EQ] = ACTIONS(2583),
    [anon_sym_instanceof] = ACTIONS(2583),
    [anon_sym_PLUS_PLUS] = ACTIONS(2583),
    [anon_sym_DASH_DASH] = ACTIONS(2583),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2583),
  },
  [1324] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(3696),
    [sym_comment] = ACTIONS(71),
  },
  [1325] = {
    [anon_sym_STAR] = ACTIONS(2945),
    [anon_sym_LBRACE] = ACTIONS(2945),
    [anon_sym_COMMA] = ACTIONS(2945),
    [anon_sym_RBRACE] = ACTIONS(2945),
    [anon_sym_LPAREN] = ACTIONS(2945),
    [anon_sym_RPAREN] = ACTIONS(2945),
    [anon_sym_in] = ACTIONS(2947),
    [anon_sym_COLON] = ACTIONS(2945),
    [anon_sym_LBRACK] = ACTIONS(2945),
    [anon_sym_RBRACK] = ACTIONS(2945),
    [anon_sym_LT] = ACTIONS(2947),
    [anon_sym_GT] = ACTIONS(2947),
    [anon_sym_SLASH] = ACTIONS(2947),
    [anon_sym_DOT] = ACTIONS(2945),
    [anon_sym_EQ] = ACTIONS(2947),
    [anon_sym_QMARK] = ACTIONS(2945),
    [anon_sym_AMP_AMP] = ACTIONS(2945),
    [anon_sym_PIPE_PIPE] = ACTIONS(2945),
    [anon_sym_GT_GT] = ACTIONS(2947),
    [anon_sym_GT_GT_GT] = ACTIONS(2945),
    [anon_sym_LT_LT] = ACTIONS(2945),
    [anon_sym_AMP] = ACTIONS(2947),
    [anon_sym_CARET] = ACTIONS(2945),
    [anon_sym_PIPE] = ACTIONS(2947),
    [anon_sym_PLUS] = ACTIONS(2947),
    [anon_sym_DASH] = ACTIONS(2947),
    [anon_sym_PERCENT] = ACTIONS(2945),
    [anon_sym_LT_EQ] = ACTIONS(2945),
    [anon_sym_EQ_EQ] = ACTIONS(2947),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2945),
    [anon_sym_BANG_EQ] = ACTIONS(2947),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2945),
    [anon_sym_GT_EQ] = ACTIONS(2945),
    [anon_sym_instanceof] = ACTIONS(2945),
    [anon_sym_PLUS_PLUS] = ACTIONS(2945),
    [anon_sym_DASH_DASH] = ACTIONS(2945),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2945),
  },
  [1326] = {
    [anon_sym_GT] = ACTIONS(3698),
    [sym_comment] = ACTIONS(71),
  },
  [1327] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(3700),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [1328] = {
    [anon_sym_STAR] = ACTIONS(2965),
    [anon_sym_LBRACE] = ACTIONS(2965),
    [anon_sym_COMMA] = ACTIONS(2965),
    [anon_sym_RBRACE] = ACTIONS(2965),
    [anon_sym_LPAREN] = ACTIONS(2965),
    [anon_sym_RPAREN] = ACTIONS(2965),
    [anon_sym_in] = ACTIONS(2967),
    [anon_sym_COLON] = ACTIONS(2965),
    [anon_sym_LBRACK] = ACTIONS(2965),
    [anon_sym_RBRACK] = ACTIONS(2965),
    [anon_sym_LT] = ACTIONS(2967),
    [anon_sym_GT] = ACTIONS(2967),
    [anon_sym_SLASH] = ACTIONS(2967),
    [anon_sym_DOT] = ACTIONS(2965),
    [anon_sym_QMARK] = ACTIONS(2965),
    [anon_sym_AMP_AMP] = ACTIONS(2965),
    [anon_sym_PIPE_PIPE] = ACTIONS(2965),
    [anon_sym_GT_GT] = ACTIONS(2967),
    [anon_sym_GT_GT_GT] = ACTIONS(2965),
    [anon_sym_LT_LT] = ACTIONS(2965),
    [anon_sym_AMP] = ACTIONS(2967),
    [anon_sym_CARET] = ACTIONS(2965),
    [anon_sym_PIPE] = ACTIONS(2967),
    [anon_sym_PLUS] = ACTIONS(2967),
    [anon_sym_DASH] = ACTIONS(2967),
    [anon_sym_PERCENT] = ACTIONS(2965),
    [anon_sym_LT_EQ] = ACTIONS(2965),
    [anon_sym_EQ_EQ] = ACTIONS(2967),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2965),
    [anon_sym_BANG_EQ] = ACTIONS(2967),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2965),
    [anon_sym_GT_EQ] = ACTIONS(2965),
    [anon_sym_instanceof] = ACTIONS(2965),
    [anon_sym_PLUS_PLUS] = ACTIONS(2965),
    [anon_sym_DASH_DASH] = ACTIONS(2965),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(2965),
  },
  [1329] = {
    [anon_sym_GT] = ACTIONS(3702),
    [sym_comment] = ACTIONS(71),
  },
  [1330] = {
    [anon_sym_STAR] = ACTIONS(3000),
    [anon_sym_LBRACE] = ACTIONS(3000),
    [anon_sym_COMMA] = ACTIONS(3000),
    [anon_sym_RBRACE] = ACTIONS(3000),
    [anon_sym_LPAREN] = ACTIONS(3000),
    [anon_sym_RPAREN] = ACTIONS(3000),
    [anon_sym_in] = ACTIONS(3002),
    [anon_sym_COLON] = ACTIONS(3000),
    [anon_sym_LBRACK] = ACTIONS(3000),
    [anon_sym_RBRACK] = ACTIONS(3000),
    [anon_sym_LT] = ACTIONS(3002),
    [anon_sym_GT] = ACTIONS(3002),
    [anon_sym_SLASH] = ACTIONS(3002),
    [anon_sym_DOT] = ACTIONS(3000),
    [anon_sym_QMARK] = ACTIONS(3000),
    [anon_sym_AMP_AMP] = ACTIONS(3000),
    [anon_sym_PIPE_PIPE] = ACTIONS(3000),
    [anon_sym_GT_GT] = ACTIONS(3002),
    [anon_sym_GT_GT_GT] = ACTIONS(3000),
    [anon_sym_LT_LT] = ACTIONS(3000),
    [anon_sym_AMP] = ACTIONS(3002),
    [anon_sym_CARET] = ACTIONS(3000),
    [anon_sym_PIPE] = ACTIONS(3002),
    [anon_sym_PLUS] = ACTIONS(3002),
    [anon_sym_DASH] = ACTIONS(3002),
    [anon_sym_PERCENT] = ACTIONS(3000),
    [anon_sym_LT_EQ] = ACTIONS(3000),
    [anon_sym_EQ_EQ] = ACTIONS(3002),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3000),
    [anon_sym_BANG_EQ] = ACTIONS(3002),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3000),
    [anon_sym_GT_EQ] = ACTIONS(3000),
    [anon_sym_instanceof] = ACTIONS(3000),
    [anon_sym_PLUS_PLUS] = ACTIONS(3000),
    [anon_sym_DASH_DASH] = ACTIONS(3000),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3000),
  },
  [1331] = {
    [anon_sym_GT] = ACTIONS(3704),
    [sym_comment] = ACTIONS(71),
  },
  [1332] = {
    [anon_sym_STAR] = ACTIONS(3035),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_RPAREN] = ACTIONS(3035),
    [anon_sym_in] = ACTIONS(3037),
    [anon_sym_COLON] = ACTIONS(3035),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_RBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3037),
    [anon_sym_GT] = ACTIONS(3037),
    [anon_sym_SLASH] = ACTIONS(3037),
    [anon_sym_DOT] = ACTIONS(3035),
    [anon_sym_QMARK] = ACTIONS(3035),
    [anon_sym_AMP_AMP] = ACTIONS(3035),
    [anon_sym_PIPE_PIPE] = ACTIONS(3035),
    [anon_sym_GT_GT] = ACTIONS(3037),
    [anon_sym_GT_GT_GT] = ACTIONS(3035),
    [anon_sym_LT_LT] = ACTIONS(3035),
    [anon_sym_AMP] = ACTIONS(3037),
    [anon_sym_CARET] = ACTIONS(3035),
    [anon_sym_PIPE] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_PERCENT] = ACTIONS(3035),
    [anon_sym_LT_EQ] = ACTIONS(3035),
    [anon_sym_EQ_EQ] = ACTIONS(3037),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3035),
    [anon_sym_BANG_EQ] = ACTIONS(3037),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3035),
    [anon_sym_GT_EQ] = ACTIONS(3035),
    [anon_sym_instanceof] = ACTIONS(3035),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3035),
  },
  [1333] = {
    [anon_sym_STAR] = ACTIONS(3103),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_COMMA] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_RPAREN] = ACTIONS(3103),
    [anon_sym_in] = ACTIONS(3105),
    [anon_sym_COLON] = ACTIONS(3103),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_RBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3105),
    [anon_sym_GT] = ACTIONS(3105),
    [anon_sym_SLASH] = ACTIONS(3105),
    [anon_sym_DOT] = ACTIONS(3103),
    [anon_sym_QMARK] = ACTIONS(3103),
    [anon_sym_AMP_AMP] = ACTIONS(3103),
    [anon_sym_PIPE_PIPE] = ACTIONS(3103),
    [anon_sym_GT_GT] = ACTIONS(3105),
    [anon_sym_GT_GT_GT] = ACTIONS(3103),
    [anon_sym_LT_LT] = ACTIONS(3103),
    [anon_sym_AMP] = ACTIONS(3105),
    [anon_sym_CARET] = ACTIONS(3103),
    [anon_sym_PIPE] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_PERCENT] = ACTIONS(3103),
    [anon_sym_LT_EQ] = ACTIONS(3103),
    [anon_sym_EQ_EQ] = ACTIONS(3105),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3103),
    [anon_sym_BANG_EQ] = ACTIONS(3105),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3103),
    [anon_sym_GT_EQ] = ACTIONS(3103),
    [anon_sym_instanceof] = ACTIONS(3103),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3103),
  },
  [1334] = {
    [sym_statement_block] = STATE(1696),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [1335] = {
    [anon_sym_STAR] = ACTIONS(3107),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_COMMA] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_RPAREN] = ACTIONS(3107),
    [anon_sym_in] = ACTIONS(3109),
    [anon_sym_COLON] = ACTIONS(3107),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_RBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3109),
    [anon_sym_GT] = ACTIONS(3109),
    [anon_sym_SLASH] = ACTIONS(3109),
    [anon_sym_DOT] = ACTIONS(3107),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_AMP_AMP] = ACTIONS(3107),
    [anon_sym_PIPE_PIPE] = ACTIONS(3107),
    [anon_sym_GT_GT] = ACTIONS(3109),
    [anon_sym_GT_GT_GT] = ACTIONS(3107),
    [anon_sym_LT_LT] = ACTIONS(3107),
    [anon_sym_AMP] = ACTIONS(3109),
    [anon_sym_CARET] = ACTIONS(3107),
    [anon_sym_PIPE] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_PERCENT] = ACTIONS(3107),
    [anon_sym_LT_EQ] = ACTIONS(3107),
    [anon_sym_EQ_EQ] = ACTIONS(3109),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3107),
    [anon_sym_BANG_EQ] = ACTIONS(3109),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3107),
    [anon_sym_GT_EQ] = ACTIONS(3107),
    [anon_sym_instanceof] = ACTIONS(3107),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3107),
  },
  [1336] = {
    [anon_sym_STAR] = ACTIONS(3111),
    [anon_sym_LBRACE] = ACTIONS(3111),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_RBRACE] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(3111),
    [anon_sym_RPAREN] = ACTIONS(3111),
    [anon_sym_in] = ACTIONS(3113),
    [anon_sym_COLON] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(3111),
    [anon_sym_RBRACK] = ACTIONS(3111),
    [anon_sym_LT] = ACTIONS(3113),
    [anon_sym_GT] = ACTIONS(3113),
    [anon_sym_SLASH] = ACTIONS(3113),
    [anon_sym_DOT] = ACTIONS(3111),
    [anon_sym_QMARK] = ACTIONS(3111),
    [anon_sym_AMP_AMP] = ACTIONS(3111),
    [anon_sym_PIPE_PIPE] = ACTIONS(3111),
    [anon_sym_GT_GT] = ACTIONS(3113),
    [anon_sym_GT_GT_GT] = ACTIONS(3111),
    [anon_sym_LT_LT] = ACTIONS(3111),
    [anon_sym_AMP] = ACTIONS(3113),
    [anon_sym_CARET] = ACTIONS(3111),
    [anon_sym_PIPE] = ACTIONS(3113),
    [anon_sym_PLUS] = ACTIONS(3113),
    [anon_sym_DASH] = ACTIONS(3113),
    [anon_sym_PERCENT] = ACTIONS(3111),
    [anon_sym_LT_EQ] = ACTIONS(3111),
    [anon_sym_EQ_EQ] = ACTIONS(3113),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3111),
    [anon_sym_BANG_EQ] = ACTIONS(3113),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3111),
    [anon_sym_GT_EQ] = ACTIONS(3111),
    [anon_sym_instanceof] = ACTIONS(3111),
    [anon_sym_PLUS_PLUS] = ACTIONS(3111),
    [anon_sym_DASH_DASH] = ACTIONS(3111),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3111),
  },
  [1337] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3111),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1338] = {
    [sym_statement_block] = STATE(1697),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [1339] = {
    [anon_sym_STAR] = ACTIONS(3115),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_COMMA] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_RPAREN] = ACTIONS(3115),
    [anon_sym_in] = ACTIONS(3117),
    [anon_sym_COLON] = ACTIONS(3115),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_RBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3117),
    [anon_sym_GT] = ACTIONS(3117),
    [anon_sym_SLASH] = ACTIONS(3117),
    [anon_sym_DOT] = ACTIONS(3115),
    [anon_sym_QMARK] = ACTIONS(3115),
    [anon_sym_AMP_AMP] = ACTIONS(3115),
    [anon_sym_PIPE_PIPE] = ACTIONS(3115),
    [anon_sym_GT_GT] = ACTIONS(3117),
    [anon_sym_GT_GT_GT] = ACTIONS(3115),
    [anon_sym_LT_LT] = ACTIONS(3115),
    [anon_sym_AMP] = ACTIONS(3117),
    [anon_sym_CARET] = ACTIONS(3115),
    [anon_sym_PIPE] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_PERCENT] = ACTIONS(3115),
    [anon_sym_LT_EQ] = ACTIONS(3115),
    [anon_sym_EQ_EQ] = ACTIONS(3117),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3115),
    [anon_sym_BANG_EQ] = ACTIONS(3117),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3115),
    [anon_sym_GT_EQ] = ACTIONS(3115),
    [anon_sym_instanceof] = ACTIONS(3115),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3115),
  },
  [1340] = {
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_else] = ACTIONS(482),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_RPAREN] = ACTIONS(482),
    [anon_sym_in] = ACTIONS(484),
    [anon_sym_while] = ACTIONS(482),
    [anon_sym_COLON] = ACTIONS(482),
    [anon_sym_catch] = ACTIONS(482),
    [anon_sym_finally] = ACTIONS(482),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_RBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(484),
    [anon_sym_GT] = ACTIONS(484),
    [anon_sym_SLASH] = ACTIONS(484),
    [anon_sym_DOT] = ACTIONS(482),
    [anon_sym_QMARK] = ACTIONS(482),
    [anon_sym_AMP_AMP] = ACTIONS(482),
    [anon_sym_PIPE_PIPE] = ACTIONS(482),
    [anon_sym_GT_GT] = ACTIONS(484),
    [anon_sym_GT_GT_GT] = ACTIONS(482),
    [anon_sym_LT_LT] = ACTIONS(482),
    [anon_sym_AMP] = ACTIONS(484),
    [anon_sym_CARET] = ACTIONS(482),
    [anon_sym_PIPE] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_PERCENT] = ACTIONS(482),
    [anon_sym_LT_EQ] = ACTIONS(482),
    [anon_sym_EQ_EQ] = ACTIONS(484),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(482),
    [anon_sym_BANG_EQ] = ACTIONS(484),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(482),
    [anon_sym_GT_EQ] = ACTIONS(482),
    [anon_sym_instanceof] = ACTIONS(482),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(482),
  },
  [1341] = {
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_else] = ACTIONS(1588),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_RPAREN] = ACTIONS(1588),
    [anon_sym_in] = ACTIONS(1590),
    [anon_sym_while] = ACTIONS(1588),
    [anon_sym_COLON] = ACTIONS(1588),
    [anon_sym_catch] = ACTIONS(1588),
    [anon_sym_finally] = ACTIONS(1588),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_RBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1590),
    [anon_sym_SLASH] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1588),
    [anon_sym_QMARK] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1588),
    [anon_sym_PIPE_PIPE] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1590),
    [anon_sym_GT_GT_GT] = ACTIONS(1588),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1590),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_LT_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1588),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1588),
    [anon_sym_instanceof] = ACTIONS(1588),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(1588),
  },
  [1342] = {
    [anon_sym_export] = ACTIONS(3129),
    [anon_sym_STAR] = ACTIONS(3129),
    [anon_sym_LBRACE] = ACTIONS(3129),
    [anon_sym_COMMA] = ACTIONS(3129),
    [anon_sym_RBRACE] = ACTIONS(3129),
    [anon_sym_LPAREN] = ACTIONS(3129),
    [anon_sym_RPAREN] = ACTIONS(3129),
    [anon_sym_in] = ACTIONS(3131),
    [anon_sym_COLON] = ACTIONS(3129),
    [anon_sym_LBRACK] = ACTIONS(3129),
    [anon_sym_RBRACK] = ACTIONS(3129),
    [anon_sym_LT] = ACTIONS(3131),
    [anon_sym_GT] = ACTIONS(3131),
    [anon_sym_SLASH] = ACTIONS(3131),
    [anon_sym_DOT] = ACTIONS(3129),
    [anon_sym_class] = ACTIONS(3129),
    [anon_sym_QMARK] = ACTIONS(3129),
    [anon_sym_AMP_AMP] = ACTIONS(3129),
    [anon_sym_PIPE_PIPE] = ACTIONS(3129),
    [anon_sym_GT_GT] = ACTIONS(3131),
    [anon_sym_GT_GT_GT] = ACTIONS(3129),
    [anon_sym_LT_LT] = ACTIONS(3129),
    [anon_sym_AMP] = ACTIONS(3131),
    [anon_sym_CARET] = ACTIONS(3129),
    [anon_sym_PIPE] = ACTIONS(3131),
    [anon_sym_PLUS] = ACTIONS(3131),
    [anon_sym_DASH] = ACTIONS(3131),
    [anon_sym_PERCENT] = ACTIONS(3129),
    [anon_sym_LT_EQ] = ACTIONS(3129),
    [anon_sym_EQ_EQ] = ACTIONS(3131),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3129),
    [anon_sym_BANG_EQ] = ACTIONS(3131),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3129),
    [anon_sym_GT_EQ] = ACTIONS(3129),
    [anon_sym_instanceof] = ACTIONS(3129),
    [anon_sym_PLUS_PLUS] = ACTIONS(3129),
    [anon_sym_DASH_DASH] = ACTIONS(3129),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3129),
    [anon_sym_AT] = ACTIONS(3129),
  },
  [1343] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(3706),
    [sym_comment] = ACTIONS(71),
  },
  [1344] = {
    [anon_sym_STAR] = ACTIONS(3140),
    [anon_sym_LBRACE] = ACTIONS(3138),
    [anon_sym_COMMA] = ACTIONS(3138),
    [anon_sym_RBRACE] = ACTIONS(3138),
    [anon_sym_LPAREN] = ACTIONS(3138),
    [anon_sym_RPAREN] = ACTIONS(3138),
    [anon_sym_in] = ACTIONS(3140),
    [anon_sym_COLON] = ACTIONS(3138),
    [anon_sym_LBRACK] = ACTIONS(3138),
    [anon_sym_RBRACK] = ACTIONS(3138),
    [anon_sym_LT] = ACTIONS(3140),
    [anon_sym_GT] = ACTIONS(3140),
    [anon_sym_SLASH] = ACTIONS(3140),
    [anon_sym_DOT] = ACTIONS(3138),
    [anon_sym_EQ] = ACTIONS(3140),
    [anon_sym_PLUS_EQ] = ACTIONS(3138),
    [anon_sym_DASH_EQ] = ACTIONS(3138),
    [anon_sym_STAR_EQ] = ACTIONS(3138),
    [anon_sym_SLASH_EQ] = ACTIONS(3138),
    [anon_sym_PERCENT_EQ] = ACTIONS(3138),
    [anon_sym_CARET_EQ] = ACTIONS(3138),
    [anon_sym_AMP_EQ] = ACTIONS(3138),
    [anon_sym_PIPE_EQ] = ACTIONS(3138),
    [anon_sym_GT_GT_EQ] = ACTIONS(3138),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(3138),
    [anon_sym_LT_LT_EQ] = ACTIONS(3138),
    [anon_sym_QMARK] = ACTIONS(3138),
    [anon_sym_AMP_AMP] = ACTIONS(3138),
    [anon_sym_PIPE_PIPE] = ACTIONS(3138),
    [anon_sym_GT_GT] = ACTIONS(3140),
    [anon_sym_GT_GT_GT] = ACTIONS(3140),
    [anon_sym_LT_LT] = ACTIONS(3140),
    [anon_sym_AMP] = ACTIONS(3140),
    [anon_sym_CARET] = ACTIONS(3140),
    [anon_sym_PIPE] = ACTIONS(3140),
    [anon_sym_PLUS] = ACTIONS(3140),
    [anon_sym_DASH] = ACTIONS(3140),
    [anon_sym_PERCENT] = ACTIONS(3140),
    [anon_sym_LT_EQ] = ACTIONS(3138),
    [anon_sym_EQ_EQ] = ACTIONS(3140),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3138),
    [anon_sym_BANG_EQ] = ACTIONS(3140),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3138),
    [anon_sym_GT_EQ] = ACTIONS(3138),
    [anon_sym_instanceof] = ACTIONS(3138),
    [anon_sym_PLUS_PLUS] = ACTIONS(3138),
    [anon_sym_DASH_DASH] = ACTIONS(3138),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3138),
  },
  [1345] = {
    [sym_parenthesized_expression] = STATE(1699),
    [sym__expression] = STATE(1699),
    [sym_yield_expression] = STATE(1699),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1699),
    [sym_jsx_fragment] = STATE(1699),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1699),
    [sym_anonymous_class] = STATE(1699),
    [sym_class] = STATE(1699),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1699),
    [sym_generator_function] = STATE(1699),
    [sym_call_expression] = STATE(1699),
    [sym_new_expression] = STATE(1699),
    [sym_await_expression] = STATE(1699),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1699),
    [sym_augmented_assignment_expression] = STATE(1699),
    [sym_ternary_expression] = STATE(1699),
    [sym_binary_expression] = STATE(1699),
    [sym_unary_expression] = STATE(1699),
    [sym_update_expression] = STATE(1699),
    [sym_template_string] = STATE(1699),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3708),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3710),
    [sym_number] = ACTIONS(3708),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(3710),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3710),
    [sym_false] = ACTIONS(3710),
    [sym_null] = ACTIONS(3710),
    [sym_undefined] = ACTIONS(3710),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1346] = {
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(3712),
    [sym_comment] = ACTIONS(71),
  },
  [1347] = {
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(3714),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [1348] = {
    [anon_sym_GT] = ACTIONS(3714),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [1349] = {
    [anon_sym_GT] = ACTIONS(3714),
    [sym_comment] = ACTIONS(71),
  },
  [1350] = {
    [sym_class_body] = STATE(1702),
    [anon_sym_LBRACE] = ACTIONS(704),
    [sym_comment] = ACTIONS(71),
  },
  [1351] = {
    [anon_sym_STAR] = ACTIONS(3254),
    [anon_sym_LBRACE] = ACTIONS(3254),
    [anon_sym_COMMA] = ACTIONS(3254),
    [anon_sym_RBRACE] = ACTIONS(3254),
    [anon_sym_LPAREN] = ACTIONS(3254),
    [anon_sym_RPAREN] = ACTIONS(3254),
    [anon_sym_in] = ACTIONS(3256),
    [anon_sym_COLON] = ACTIONS(3254),
    [anon_sym_LBRACK] = ACTIONS(3254),
    [anon_sym_RBRACK] = ACTIONS(3254),
    [anon_sym_LT] = ACTIONS(3256),
    [anon_sym_GT] = ACTIONS(3256),
    [anon_sym_SLASH] = ACTIONS(3256),
    [anon_sym_DOT] = ACTIONS(3254),
    [anon_sym_QMARK] = ACTIONS(3254),
    [anon_sym_AMP_AMP] = ACTIONS(3254),
    [anon_sym_PIPE_PIPE] = ACTIONS(3254),
    [anon_sym_GT_GT] = ACTIONS(3256),
    [anon_sym_GT_GT_GT] = ACTIONS(3254),
    [anon_sym_LT_LT] = ACTIONS(3254),
    [anon_sym_AMP] = ACTIONS(3256),
    [anon_sym_CARET] = ACTIONS(3254),
    [anon_sym_PIPE] = ACTIONS(3256),
    [anon_sym_PLUS] = ACTIONS(3256),
    [anon_sym_DASH] = ACTIONS(3256),
    [anon_sym_PERCENT] = ACTIONS(3254),
    [anon_sym_LT_EQ] = ACTIONS(3254),
    [anon_sym_EQ_EQ] = ACTIONS(3256),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3254),
    [anon_sym_BANG_EQ] = ACTIONS(3256),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3254),
    [anon_sym_GT_EQ] = ACTIONS(3254),
    [anon_sym_instanceof] = ACTIONS(3254),
    [anon_sym_PLUS_PLUS] = ACTIONS(3254),
    [anon_sym_DASH_DASH] = ACTIONS(3254),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3254),
  },
  [1352] = {
    [sym_class_body] = STATE(1033),
    [anon_sym_LBRACE] = ACTIONS(285),
    [sym_comment] = ACTIONS(71),
  },
  [1353] = {
    [sym_formal_parameters] = STATE(1703),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1354] = {
    [sym_statement_block] = STATE(1036),
    [anon_sym_LBRACE] = ACTIONS(2857),
    [sym_comment] = ACTIONS(71),
  },
  [1355] = {
    [sym_formal_parameters] = STATE(1704),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1356] = {
    [sym_statement_block] = STATE(1040),
    [anon_sym_LBRACE] = ACTIONS(2857),
    [sym_comment] = ACTIONS(71),
  },
  [1357] = {
    [sym_export_statement] = STATE(592),
    [sym__declaration] = STATE(592),
    [sym_import_statement] = STATE(592),
    [sym_expression_statement] = STATE(592),
    [sym_variable_declaration] = STATE(592),
    [sym_lexical_declaration] = STATE(592),
    [sym_statement_block] = STATE(592),
    [sym_if_statement] = STATE(592),
    [sym_switch_statement] = STATE(592),
    [sym_for_statement] = STATE(592),
    [sym_for_in_statement] = STATE(592),
    [sym_for_of_statement] = STATE(592),
    [sym_while_statement] = STATE(592),
    [sym_do_statement] = STATE(592),
    [sym_try_statement] = STATE(592),
    [sym_with_statement] = STATE(592),
    [sym_break_statement] = STATE(592),
    [sym_continue_statement] = STATE(592),
    [sym_debugger_statement] = STATE(592),
    [sym_return_statement] = STATE(592),
    [sym_throw_statement] = STATE(592),
    [sym_empty_statement] = STATE(592),
    [sym_labeled_statement] = STATE(592),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(592),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(2176),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1358] = {
    [sym_class_heritage] = STATE(1705),
    [sym_class_body] = STATE(1033),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [1359] = {
    [sym__from_clause] = STATE(1706),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [1360] = {
    [sym_parenthesized_expression] = STATE(1707),
    [sym__expression] = STATE(1707),
    [sym_yield_expression] = STATE(1707),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1707),
    [sym_jsx_fragment] = STATE(1707),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1707),
    [sym_anonymous_class] = STATE(1707),
    [sym_class] = STATE(1707),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1707),
    [sym_generator_function] = STATE(1707),
    [sym_call_expression] = STATE(1707),
    [sym_new_expression] = STATE(1707),
    [sym_await_expression] = STATE(1707),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1707),
    [sym_augmented_assignment_expression] = STATE(1707),
    [sym_ternary_expression] = STATE(1707),
    [sym_binary_expression] = STATE(1707),
    [sym_unary_expression] = STATE(1707),
    [sym_update_expression] = STATE(1707),
    [sym_template_string] = STATE(1707),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3716),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3718),
    [sym_number] = ACTIONS(3716),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(3718),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3718),
    [sym_false] = ACTIONS(3718),
    [sym_null] = ACTIONS(3718),
    [sym_undefined] = ACTIONS(3718),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [1361] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3720),
  },
  [1362] = {
    [anon_sym_function] = ACTIONS(3722),
    [sym_comment] = ACTIONS(71),
  },
  [1363] = {
    [sym__from_clause] = STATE(1706),
    [sym__automatic_semicolon] = ACTIONS(3724),
    [anon_sym_from] = ACTIONS(410),
    [anon_sym_SEMI] = ACTIONS(3724),
    [sym_comment] = ACTIONS(71),
  },
  [1364] = {
    [anon_sym_else] = ACTIONS(452),
    [anon_sym_while] = ACTIONS(452),
    [sym_comment] = ACTIONS(71),
  },
  [1365] = {
    [sym__automatic_semicolon] = ACTIONS(2897),
    [anon_sym_else] = ACTIONS(400),
    [anon_sym_while] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
  },
  [1366] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(3726),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [1367] = {
    [sym__automatic_semicolon] = ACTIONS(3728),
    [anon_sym_SEMI] = ACTIONS(3728),
    [sym_comment] = ACTIONS(71),
  },
  [1368] = {
    [sym__from_clause] = STATE(1711),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [1369] = {
    [aux_sym_variable_declaration_repeat1] = STATE(1713),
    [sym__automatic_semicolon] = ACTIONS(3730),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(3730),
    [sym_comment] = ACTIONS(71),
  },
  [1370] = {
    [aux_sym_variable_declaration_repeat1] = STATE(1715),
    [sym__automatic_semicolon] = ACTIONS(3732),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(3732),
    [sym_comment] = ACTIONS(71),
  },
  [1371] = {
    [sym_export_statement] = STATE(1716),
    [sym__declaration] = STATE(1716),
    [sym_import_statement] = STATE(1716),
    [sym_expression_statement] = STATE(1716),
    [sym_variable_declaration] = STATE(1716),
    [sym_lexical_declaration] = STATE(1716),
    [sym_statement_block] = STATE(1716),
    [sym_if_statement] = STATE(1716),
    [sym_switch_statement] = STATE(1716),
    [sym_for_statement] = STATE(1716),
    [sym_for_in_statement] = STATE(1716),
    [sym_for_of_statement] = STATE(1716),
    [sym_while_statement] = STATE(1716),
    [sym_do_statement] = STATE(1716),
    [sym_try_statement] = STATE(1716),
    [sym_with_statement] = STATE(1716),
    [sym_break_statement] = STATE(1716),
    [sym_continue_statement] = STATE(1716),
    [sym_debugger_statement] = STATE(1716),
    [sym_return_statement] = STATE(1716),
    [sym_throw_statement] = STATE(1716),
    [sym_empty_statement] = STATE(1716),
    [sym_labeled_statement] = STATE(1716),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1372] = {
    [sym_switch_body] = STATE(1718),
    [anon_sym_LBRACE] = ACTIONS(3734),
    [sym_comment] = ACTIONS(71),
  },
  [1373] = {
    [sym_expression_statement] = STATE(1722),
    [sym_variable_declaration] = STATE(1722),
    [sym_lexical_declaration] = STATE(1722),
    [sym_empty_statement] = STATE(1722),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(1723),
    [sym_array] = STATE(1724),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(3736),
    [anon_sym_let] = ACTIONS(3738),
    [anon_sym_const] = ACTIONS(3738),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3740),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1374] = {
    [sym_export_statement] = STATE(1725),
    [sym__declaration] = STATE(1725),
    [sym_import_statement] = STATE(1725),
    [sym_expression_statement] = STATE(1725),
    [sym_variable_declaration] = STATE(1725),
    [sym_lexical_declaration] = STATE(1725),
    [sym_statement_block] = STATE(1725),
    [sym_if_statement] = STATE(1725),
    [sym_switch_statement] = STATE(1725),
    [sym_for_statement] = STATE(1725),
    [sym_for_in_statement] = STATE(1725),
    [sym_for_of_statement] = STATE(1725),
    [sym_while_statement] = STATE(1725),
    [sym_do_statement] = STATE(1725),
    [sym_try_statement] = STATE(1725),
    [sym_with_statement] = STATE(1725),
    [sym_break_statement] = STATE(1725),
    [sym_continue_statement] = STATE(1725),
    [sym_debugger_statement] = STATE(1725),
    [sym_return_statement] = STATE(1725),
    [sym_throw_statement] = STATE(1725),
    [sym_empty_statement] = STATE(1725),
    [sym_labeled_statement] = STATE(1725),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1375] = {
    [anon_sym_while] = ACTIONS(3742),
    [sym_comment] = ACTIONS(71),
  },
  [1376] = {
    [sym_catch_clause] = STATE(1729),
    [sym_finally_clause] = STATE(1730),
    [anon_sym_else] = ACTIONS(842),
    [anon_sym_while] = ACTIONS(842),
    [anon_sym_catch] = ACTIONS(3744),
    [anon_sym_finally] = ACTIONS(3746),
    [sym_comment] = ACTIONS(71),
  },
  [1377] = {
    [sym_export_statement] = STATE(1731),
    [sym__declaration] = STATE(1731),
    [sym_import_statement] = STATE(1731),
    [sym_expression_statement] = STATE(1731),
    [sym_variable_declaration] = STATE(1731),
    [sym_lexical_declaration] = STATE(1731),
    [sym_statement_block] = STATE(1731),
    [sym_if_statement] = STATE(1731),
    [sym_switch_statement] = STATE(1731),
    [sym_for_statement] = STATE(1731),
    [sym_for_in_statement] = STATE(1731),
    [sym_for_of_statement] = STATE(1731),
    [sym_while_statement] = STATE(1731),
    [sym_do_statement] = STATE(1731),
    [sym_try_statement] = STATE(1731),
    [sym_with_statement] = STATE(1731),
    [sym_break_statement] = STATE(1731),
    [sym_continue_statement] = STATE(1731),
    [sym_debugger_statement] = STATE(1731),
    [sym_return_statement] = STATE(1731),
    [sym_throw_statement] = STATE(1731),
    [sym_empty_statement] = STATE(1731),
    [sym_labeled_statement] = STATE(1731),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1378] = {
    [anon_sym_else] = ACTIONS(850),
    [anon_sym_while] = ACTIONS(850),
    [sym_comment] = ACTIONS(71),
  },
  [1379] = {
    [sym__automatic_semicolon] = ACTIONS(3748),
    [anon_sym_SEMI] = ACTIONS(3748),
    [sym_comment] = ACTIONS(71),
  },
  [1380] = {
    [anon_sym_else] = ACTIONS(856),
    [anon_sym_while] = ACTIONS(856),
    [sym_comment] = ACTIONS(71),
  },
  [1381] = {
    [sym__automatic_semicolon] = ACTIONS(3750),
    [anon_sym_SEMI] = ACTIONS(3750),
    [sym_comment] = ACTIONS(71),
  },
  [1382] = {
    [anon_sym_else] = ACTIONS(862),
    [anon_sym_while] = ACTIONS(862),
    [sym_comment] = ACTIONS(71),
  },
  [1383] = {
    [anon_sym_else] = ACTIONS(866),
    [anon_sym_while] = ACTIONS(866),
    [sym_comment] = ACTIONS(71),
  },
  [1384] = {
    [sym__automatic_semicolon] = ACTIONS(3752),
    [anon_sym_SEMI] = ACTIONS(3752),
    [sym_comment] = ACTIONS(71),
  },
  [1385] = {
    [sym__automatic_semicolon] = ACTIONS(3754),
    [anon_sym_SEMI] = ACTIONS(3754),
    [sym_comment] = ACTIONS(71),
  },
  [1386] = {
    [sym_export_statement] = STATE(1736),
    [sym__declaration] = STATE(1736),
    [sym_import_statement] = STATE(1736),
    [sym_expression_statement] = STATE(1736),
    [sym_variable_declaration] = STATE(1736),
    [sym_lexical_declaration] = STATE(1736),
    [sym_statement_block] = STATE(1736),
    [sym_if_statement] = STATE(1736),
    [sym_switch_statement] = STATE(1736),
    [sym_for_statement] = STATE(1736),
    [sym_for_in_statement] = STATE(1736),
    [sym_for_of_statement] = STATE(1736),
    [sym_while_statement] = STATE(1736),
    [sym_do_statement] = STATE(1736),
    [sym_try_statement] = STATE(1736),
    [sym_with_statement] = STATE(1736),
    [sym_break_statement] = STATE(1736),
    [sym_continue_statement] = STATE(1736),
    [sym_debugger_statement] = STATE(1736),
    [sym_return_statement] = STATE(1736),
    [sym_throw_statement] = STATE(1736),
    [sym_empty_statement] = STATE(1736),
    [sym_labeled_statement] = STATE(1736),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1387] = {
    [sym_export_statement] = STATE(1256),
    [sym__declaration] = STATE(1256),
    [sym_import_statement] = STATE(1256),
    [sym_expression_statement] = STATE(1256),
    [sym_variable_declaration] = STATE(1256),
    [sym_lexical_declaration] = STATE(1256),
    [sym_statement_block] = STATE(1256),
    [sym_if_statement] = STATE(1256),
    [sym_switch_statement] = STATE(1256),
    [sym_for_statement] = STATE(1256),
    [sym_for_in_statement] = STATE(1256),
    [sym_for_of_statement] = STATE(1256),
    [sym_while_statement] = STATE(1256),
    [sym_do_statement] = STATE(1256),
    [sym_try_statement] = STATE(1256),
    [sym_with_statement] = STATE(1256),
    [sym_break_statement] = STATE(1256),
    [sym_continue_statement] = STATE(1256),
    [sym_debugger_statement] = STATE(1256),
    [sym_return_statement] = STATE(1256),
    [sym_throw_statement] = STATE(1256),
    [sym_empty_statement] = STATE(1256),
    [sym_labeled_statement] = STATE(1256),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1388] = {
    [anon_sym_else] = ACTIONS(1122),
    [anon_sym_while] = ACTIONS(1122),
    [sym_comment] = ACTIONS(71),
  },
  [1389] = {
    [anon_sym_else] = ACTIONS(1202),
    [anon_sym_while] = ACTIONS(1202),
    [sym_comment] = ACTIONS(71),
  },
  [1390] = {
    [sym__declaration] = STATE(1708),
    [sym_variable_declaration] = STATE(1708),
    [sym_lexical_declaration] = STATE(1708),
    [sym_class] = STATE(1365),
    [sym_function] = STATE(1365),
    [sym_generator_function] = STATE(1365),
    [sym_decorator] = STATE(1366),
    [aux_sym_export_statement_repeat1] = STATE(1366),
    [anon_sym_default] = ACTIONS(3756),
    [anon_sym_var] = ACTIONS(2865),
    [anon_sym_let] = ACTIONS(2867),
    [anon_sym_const] = ACTIONS(2867),
    [anon_sym_class] = ACTIONS(2869),
    [anon_sym_async] = ACTIONS(2871),
    [anon_sym_function] = ACTIONS(2873),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [1391] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(3758),
    [anon_sym_of] = ACTIONS(3760),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1392] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(3762),
    [anon_sym_of] = ACTIONS(3764),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1393] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(3766),
    [anon_sym_of] = ACTIONS(3768),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1394] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1744),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1744),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1395] = {
    [sym_parenthesized_expression] = STATE(1745),
    [sym__expression] = STATE(1745),
    [sym_yield_expression] = STATE(1745),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1745),
    [sym_jsx_fragment] = STATE(1745),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1745),
    [sym_anonymous_class] = STATE(1745),
    [sym_class] = STATE(1745),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1745),
    [sym_generator_function] = STATE(1745),
    [sym_call_expression] = STATE(1745),
    [sym_new_expression] = STATE(1745),
    [sym_await_expression] = STATE(1745),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1745),
    [sym_augmented_assignment_expression] = STATE(1745),
    [sym_ternary_expression] = STATE(1745),
    [sym_binary_expression] = STATE(1745),
    [sym_unary_expression] = STATE(1745),
    [sym_update_expression] = STATE(1745),
    [sym_template_string] = STATE(1745),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3770),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3772),
    [sym_number] = ACTIONS(3770),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3772),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3772),
    [sym_false] = ACTIONS(3772),
    [sym_null] = ACTIONS(3772),
    [sym_undefined] = ACTIONS(3772),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1396] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1747),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1747),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(3774),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1397] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1748),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1748),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1398] = {
    [sym_parenthesized_expression] = STATE(1749),
    [sym__expression] = STATE(1749),
    [sym_yield_expression] = STATE(1749),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1749),
    [sym_jsx_fragment] = STATE(1749),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1749),
    [sym_anonymous_class] = STATE(1749),
    [sym_class] = STATE(1749),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1749),
    [sym_generator_function] = STATE(1749),
    [sym_call_expression] = STATE(1749),
    [sym_new_expression] = STATE(1749),
    [sym_await_expression] = STATE(1749),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1749),
    [sym_augmented_assignment_expression] = STATE(1749),
    [sym_ternary_expression] = STATE(1749),
    [sym_binary_expression] = STATE(1749),
    [sym_unary_expression] = STATE(1749),
    [sym_update_expression] = STATE(1749),
    [sym_template_string] = STATE(1749),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3776),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3778),
    [sym_number] = ACTIONS(3776),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3778),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3778),
    [sym_false] = ACTIONS(3778),
    [sym_null] = ACTIONS(3778),
    [sym_undefined] = ACTIONS(3778),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1399] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1750),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1750),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1400] = {
    [sym_parenthesized_expression] = STATE(1751),
    [sym__expression] = STATE(1751),
    [sym_yield_expression] = STATE(1751),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1751),
    [sym_jsx_fragment] = STATE(1751),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1751),
    [sym_anonymous_class] = STATE(1751),
    [sym_class] = STATE(1751),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1751),
    [sym_generator_function] = STATE(1751),
    [sym_call_expression] = STATE(1751),
    [sym_new_expression] = STATE(1751),
    [sym_await_expression] = STATE(1751),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1751),
    [sym_augmented_assignment_expression] = STATE(1751),
    [sym_ternary_expression] = STATE(1751),
    [sym_binary_expression] = STATE(1751),
    [sym_unary_expression] = STATE(1751),
    [sym_update_expression] = STATE(1751),
    [sym_template_string] = STATE(1751),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3780),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3782),
    [sym_number] = ACTIONS(3780),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3782),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3782),
    [sym_false] = ACTIONS(3782),
    [sym_null] = ACTIONS(3782),
    [sym_undefined] = ACTIONS(3782),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1401] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3784),
  },
  [1402] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [anon_sym_STAR] = ACTIONS(3103),
    [anon_sym_COMMA] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_else] = ACTIONS(3103),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_RPAREN] = ACTIONS(3103),
    [anon_sym_in] = ACTIONS(3105),
    [anon_sym_while] = ACTIONS(3103),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3105),
    [anon_sym_GT] = ACTIONS(3105),
    [anon_sym_SLASH] = ACTIONS(3105),
    [anon_sym_DOT] = ACTIONS(3103),
    [anon_sym_QMARK] = ACTIONS(3103),
    [anon_sym_AMP_AMP] = ACTIONS(3103),
    [anon_sym_PIPE_PIPE] = ACTIONS(3103),
    [anon_sym_GT_GT] = ACTIONS(3105),
    [anon_sym_GT_GT_GT] = ACTIONS(3103),
    [anon_sym_LT_LT] = ACTIONS(3103),
    [anon_sym_AMP] = ACTIONS(3105),
    [anon_sym_CARET] = ACTIONS(3103),
    [anon_sym_PIPE] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_PERCENT] = ACTIONS(3103),
    [anon_sym_LT_EQ] = ACTIONS(3103),
    [anon_sym_EQ_EQ] = ACTIONS(3105),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3103),
    [anon_sym_BANG_EQ] = ACTIONS(3105),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3103),
    [anon_sym_GT_EQ] = ACTIONS(3103),
    [anon_sym_instanceof] = ACTIONS(3103),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3103),
  },
  [1403] = {
    [sym_statement_block] = STATE(1753),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [sym_comment] = ACTIONS(71),
  },
  [1404] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [anon_sym_STAR] = ACTIONS(3107),
    [anon_sym_COMMA] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_else] = ACTIONS(3107),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_RPAREN] = ACTIONS(3107),
    [anon_sym_in] = ACTIONS(3109),
    [anon_sym_while] = ACTIONS(3107),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3109),
    [anon_sym_GT] = ACTIONS(3109),
    [anon_sym_SLASH] = ACTIONS(3109),
    [anon_sym_DOT] = ACTIONS(3107),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_AMP_AMP] = ACTIONS(3107),
    [anon_sym_PIPE_PIPE] = ACTIONS(3107),
    [anon_sym_GT_GT] = ACTIONS(3109),
    [anon_sym_GT_GT_GT] = ACTIONS(3107),
    [anon_sym_LT_LT] = ACTIONS(3107),
    [anon_sym_AMP] = ACTIONS(3109),
    [anon_sym_CARET] = ACTIONS(3107),
    [anon_sym_PIPE] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_PERCENT] = ACTIONS(3107),
    [anon_sym_LT_EQ] = ACTIONS(3107),
    [anon_sym_EQ_EQ] = ACTIONS(3109),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3107),
    [anon_sym_BANG_EQ] = ACTIONS(3109),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3107),
    [anon_sym_GT_EQ] = ACTIONS(3107),
    [anon_sym_instanceof] = ACTIONS(3107),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3107),
  },
  [1405] = {
    [sym_statement_block] = STATE(1754),
    [anon_sym_LBRACE] = ACTIONS(1927),
    [sym_comment] = ACTIONS(71),
  },
  [1406] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [anon_sym_STAR] = ACTIONS(3115),
    [anon_sym_COMMA] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_else] = ACTIONS(3115),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_RPAREN] = ACTIONS(3115),
    [anon_sym_in] = ACTIONS(3117),
    [anon_sym_while] = ACTIONS(3115),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3117),
    [anon_sym_GT] = ACTIONS(3117),
    [anon_sym_SLASH] = ACTIONS(3117),
    [anon_sym_DOT] = ACTIONS(3115),
    [anon_sym_QMARK] = ACTIONS(3115),
    [anon_sym_AMP_AMP] = ACTIONS(3115),
    [anon_sym_PIPE_PIPE] = ACTIONS(3115),
    [anon_sym_GT_GT] = ACTIONS(3117),
    [anon_sym_GT_GT_GT] = ACTIONS(3115),
    [anon_sym_LT_LT] = ACTIONS(3115),
    [anon_sym_AMP] = ACTIONS(3117),
    [anon_sym_CARET] = ACTIONS(3115),
    [anon_sym_PIPE] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_PERCENT] = ACTIONS(3115),
    [anon_sym_LT_EQ] = ACTIONS(3115),
    [anon_sym_EQ_EQ] = ACTIONS(3117),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3115),
    [anon_sym_BANG_EQ] = ACTIONS(3117),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3115),
    [anon_sym_GT_EQ] = ACTIONS(3115),
    [anon_sym_instanceof] = ACTIONS(3115),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3115),
  },
  [1407] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_else] = ACTIONS(482),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_RPAREN] = ACTIONS(482),
    [anon_sym_in] = ACTIONS(484),
    [anon_sym_while] = ACTIONS(482),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(484),
    [anon_sym_GT] = ACTIONS(484),
    [anon_sym_SLASH] = ACTIONS(484),
    [anon_sym_DOT] = ACTIONS(482),
    [anon_sym_QMARK] = ACTIONS(482),
    [anon_sym_AMP_AMP] = ACTIONS(482),
    [anon_sym_PIPE_PIPE] = ACTIONS(482),
    [anon_sym_GT_GT] = ACTIONS(484),
    [anon_sym_GT_GT_GT] = ACTIONS(482),
    [anon_sym_LT_LT] = ACTIONS(482),
    [anon_sym_AMP] = ACTIONS(484),
    [anon_sym_CARET] = ACTIONS(482),
    [anon_sym_PIPE] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_PERCENT] = ACTIONS(482),
    [anon_sym_LT_EQ] = ACTIONS(482),
    [anon_sym_EQ_EQ] = ACTIONS(484),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(482),
    [anon_sym_BANG_EQ] = ACTIONS(484),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(482),
    [anon_sym_GT_EQ] = ACTIONS(482),
    [anon_sym_instanceof] = ACTIONS(482),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(482),
  },
  [1408] = {
    [ts_builtin_sym_end] = ACTIONS(3786),
    [anon_sym_export] = ACTIONS(3788),
    [anon_sym_LBRACE] = ACTIONS(3786),
    [anon_sym_RBRACE] = ACTIONS(3786),
    [anon_sym_import] = ACTIONS(3788),
    [anon_sym_var] = ACTIONS(3788),
    [anon_sym_let] = ACTIONS(3788),
    [anon_sym_const] = ACTIONS(3788),
    [anon_sym_if] = ACTIONS(3788),
    [anon_sym_switch] = ACTIONS(3788),
    [anon_sym_for] = ACTIONS(3788),
    [anon_sym_LPAREN] = ACTIONS(3786),
    [anon_sym_while] = ACTIONS(3788),
    [anon_sym_do] = ACTIONS(3788),
    [anon_sym_try] = ACTIONS(3788),
    [anon_sym_with] = ACTIONS(3788),
    [anon_sym_break] = ACTIONS(3788),
    [anon_sym_continue] = ACTIONS(3788),
    [anon_sym_debugger] = ACTIONS(3788),
    [anon_sym_return] = ACTIONS(3788),
    [anon_sym_throw] = ACTIONS(3788),
    [anon_sym_SEMI] = ACTIONS(3786),
    [anon_sym_yield] = ACTIONS(3788),
    [anon_sym_LBRACK] = ACTIONS(3786),
    [anon_sym_LT] = ACTIONS(3786),
    [anon_sym_class] = ACTIONS(3788),
    [anon_sym_async] = ACTIONS(3788),
    [anon_sym_function] = ACTIONS(3788),
    [anon_sym_EQ_GT] = ACTIONS(3786),
    [anon_sym_new] = ACTIONS(3788),
    [anon_sym_await] = ACTIONS(3788),
    [anon_sym_PLUS] = ACTIONS(3788),
    [anon_sym_DASH] = ACTIONS(3788),
    [anon_sym_BANG] = ACTIONS(3786),
    [anon_sym_TILDE] = ACTIONS(3786),
    [anon_sym_typeof] = ACTIONS(3788),
    [anon_sym_void] = ACTIONS(3788),
    [anon_sym_delete] = ACTIONS(3788),
    [anon_sym_PLUS_PLUS] = ACTIONS(3786),
    [anon_sym_DASH_DASH] = ACTIONS(3786),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3786),
    [anon_sym_BQUOTE] = ACTIONS(3786),
    [sym_regex] = ACTIONS(3788),
    [sym_number] = ACTIONS(3786),
    [sym_identifier] = ACTIONS(3788),
    [sym_this] = ACTIONS(3788),
    [sym_super] = ACTIONS(3788),
    [sym_true] = ACTIONS(3788),
    [sym_false] = ACTIONS(3788),
    [sym_null] = ACTIONS(3788),
    [sym_undefined] = ACTIONS(3788),
    [anon_sym_AT] = ACTIONS(3786),
    [anon_sym_get] = ACTIONS(3788),
    [anon_sym_set] = ACTIONS(3788),
  },
  [1409] = {
    [sym_class_body] = STATE(1755),
    [anon_sym_LBRACE] = ACTIONS(285),
    [sym_comment] = ACTIONS(71),
  },
  [1410] = {
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1411] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(3790),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1412] = {
    [anon_sym_RPAREN] = ACTIONS(3792),
    [sym_comment] = ACTIONS(71),
  },
  [1413] = {
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1414] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(3111),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1415] = {
    [sym_parenthesized_expression] = STATE(1758),
    [sym__expression] = STATE(1758),
    [sym_yield_expression] = STATE(1758),
    [sym_object] = STATE(173),
    [sym_array] = STATE(174),
    [sym_jsx_element] = STATE(1758),
    [sym_jsx_fragment] = STATE(1758),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1758),
    [sym_anonymous_class] = STATE(1758),
    [sym_class] = STATE(1758),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1758),
    [sym_generator_function] = STATE(1758),
    [sym_call_expression] = STATE(1758),
    [sym_new_expression] = STATE(1758),
    [sym_await_expression] = STATE(1758),
    [sym_member_expression] = STATE(175),
    [sym_subscript_expression] = STATE(175),
    [sym_assignment_expression] = STATE(1758),
    [sym_augmented_assignment_expression] = STATE(1758),
    [sym_ternary_expression] = STATE(1758),
    [sym_binary_expression] = STATE(1758),
    [sym_unary_expression] = STATE(1758),
    [sym_update_expression] = STATE(1758),
    [sym_template_string] = STATE(1758),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(177),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(251),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(255),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(257),
    [anon_sym_new] = ACTIONS(259),
    [anon_sym_await] = ACTIONS(261),
    [anon_sym_PLUS] = ACTIONS(265),
    [anon_sym_DASH] = ACTIONS(265),
    [anon_sym_BANG] = ACTIONS(267),
    [anon_sym_TILDE] = ACTIONS(267),
    [anon_sym_typeof] = ACTIONS(265),
    [anon_sym_void] = ACTIONS(265),
    [anon_sym_delete] = ACTIONS(265),
    [anon_sym_PLUS_PLUS] = ACTIONS(269),
    [anon_sym_DASH_DASH] = ACTIONS(269),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3794),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3796),
    [sym_number] = ACTIONS(3794),
    [sym_identifier] = ACTIONS(275),
    [sym_this] = ACTIONS(3796),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3796),
    [sym_false] = ACTIONS(3796),
    [sym_null] = ACTIONS(3796),
    [sym_undefined] = ACTIONS(3796),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(277),
    [anon_sym_set] = ACTIONS(277),
  },
  [1416] = {
    [sym__automatic_semicolon] = ACTIONS(3798),
    [anon_sym_STAR] = ACTIONS(3798),
    [anon_sym_COMMA] = ACTIONS(3798),
    [anon_sym_RBRACE] = ACTIONS(3798),
    [anon_sym_LPAREN] = ACTIONS(3798),
    [anon_sym_RPAREN] = ACTIONS(3798),
    [anon_sym_in] = ACTIONS(3800),
    [anon_sym_SEMI] = ACTIONS(3798),
    [anon_sym_LBRACK] = ACTIONS(3798),
    [anon_sym_LT] = ACTIONS(3800),
    [anon_sym_GT] = ACTIONS(3800),
    [anon_sym_SLASH] = ACTIONS(3800),
    [anon_sym_DOT] = ACTIONS(3798),
    [anon_sym_QMARK] = ACTIONS(3798),
    [anon_sym_AMP_AMP] = ACTIONS(3798),
    [anon_sym_PIPE_PIPE] = ACTIONS(3798),
    [anon_sym_GT_GT] = ACTIONS(3800),
    [anon_sym_GT_GT_GT] = ACTIONS(3798),
    [anon_sym_LT_LT] = ACTIONS(3798),
    [anon_sym_AMP] = ACTIONS(3800),
    [anon_sym_CARET] = ACTIONS(3798),
    [anon_sym_PIPE] = ACTIONS(3800),
    [anon_sym_PLUS] = ACTIONS(3800),
    [anon_sym_DASH] = ACTIONS(3800),
    [anon_sym_PERCENT] = ACTIONS(3798),
    [anon_sym_LT_EQ] = ACTIONS(3798),
    [anon_sym_EQ_EQ] = ACTIONS(3800),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3798),
    [anon_sym_BANG_EQ] = ACTIONS(3800),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3798),
    [anon_sym_GT_EQ] = ACTIONS(3798),
    [anon_sym_instanceof] = ACTIONS(3798),
    [anon_sym_PLUS_PLUS] = ACTIONS(3798),
    [anon_sym_DASH_DASH] = ACTIONS(3798),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3798),
  },
  [1417] = {
    [anon_sym_GT] = ACTIONS(3802),
    [sym_comment] = ACTIONS(71),
  },
  [1418] = {
    [anon_sym_LBRACE] = ACTIONS(3804),
    [anon_sym_GT] = ACTIONS(3804),
    [anon_sym_SLASH] = ACTIONS(3234),
    [sym_jsx_identifier] = ACTIONS(3804),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3234),
  },
  [1419] = {
    [sym_nested_identifier] = STATE(1763),
    [sym_jsx_namespace_name] = STATE(1764),
    [anon_sym_GT] = ACTIONS(3806),
    [sym_jsx_identifier] = ACTIONS(3808),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3810),
  },
  [1420] = {
    [anon_sym_LBRACE] = ACTIONS(3812),
    [anon_sym_GT] = ACTIONS(3812),
    [anon_sym_SLASH] = ACTIONS(3814),
    [sym_jsx_identifier] = ACTIONS(3812),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3814),
  },
  [1421] = {
    [sym_jsx_element] = STATE(1767),
    [sym_jsx_expression] = STATE(1767),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(1766),
    [sym_jsx_self_closing_element] = STATE(1767),
    [aux_sym_jsx_element_repeat1] = STATE(1767),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(3816),
    [sym_jsx_text] = ACTIONS(3818),
    [sym_comment] = ACTIONS(71),
  },
  [1422] = {
    [anon_sym_LBRACE] = ACTIONS(3820),
    [anon_sym_GT] = ACTIONS(3820),
    [anon_sym_SLASH] = ACTIONS(3822),
    [sym_jsx_identifier] = ACTIONS(3820),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3822),
  },
  [1423] = {
    [sym__automatic_semicolon] = ACTIONS(3824),
    [anon_sym_STAR] = ACTIONS(3824),
    [anon_sym_COMMA] = ACTIONS(3824),
    [anon_sym_RBRACE] = ACTIONS(3824),
    [anon_sym_LPAREN] = ACTIONS(3824),
    [anon_sym_RPAREN] = ACTIONS(3824),
    [anon_sym_in] = ACTIONS(3826),
    [anon_sym_SEMI] = ACTIONS(3824),
    [anon_sym_LBRACK] = ACTIONS(3824),
    [anon_sym_LT] = ACTIONS(3826),
    [anon_sym_GT] = ACTIONS(3826),
    [anon_sym_SLASH] = ACTIONS(3826),
    [anon_sym_DOT] = ACTIONS(3824),
    [anon_sym_QMARK] = ACTIONS(3824),
    [anon_sym_AMP_AMP] = ACTIONS(3824),
    [anon_sym_PIPE_PIPE] = ACTIONS(3824),
    [anon_sym_GT_GT] = ACTIONS(3826),
    [anon_sym_GT_GT_GT] = ACTIONS(3824),
    [anon_sym_LT_LT] = ACTIONS(3824),
    [anon_sym_AMP] = ACTIONS(3826),
    [anon_sym_CARET] = ACTIONS(3824),
    [anon_sym_PIPE] = ACTIONS(3826),
    [anon_sym_PLUS] = ACTIONS(3826),
    [anon_sym_DASH] = ACTIONS(3826),
    [anon_sym_PERCENT] = ACTIONS(3824),
    [anon_sym_LT_EQ] = ACTIONS(3824),
    [anon_sym_EQ_EQ] = ACTIONS(3826),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3824),
    [anon_sym_BANG_EQ] = ACTIONS(3826),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3824),
    [anon_sym_GT_EQ] = ACTIONS(3824),
    [anon_sym_instanceof] = ACTIONS(3824),
    [anon_sym_PLUS_PLUS] = ACTIONS(3824),
    [anon_sym_DASH_DASH] = ACTIONS(3824),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3824),
  },
  [1424] = {
    [sym__automatic_semicolon] = ACTIONS(3828),
    [anon_sym_STAR] = ACTIONS(3828),
    [anon_sym_COMMA] = ACTIONS(3828),
    [anon_sym_RBRACE] = ACTIONS(3828),
    [anon_sym_LPAREN] = ACTIONS(3828),
    [anon_sym_RPAREN] = ACTIONS(3828),
    [anon_sym_in] = ACTIONS(3830),
    [anon_sym_SEMI] = ACTIONS(3828),
    [anon_sym_LBRACK] = ACTIONS(3828),
    [anon_sym_LT] = ACTIONS(3830),
    [anon_sym_GT] = ACTIONS(3830),
    [anon_sym_SLASH] = ACTIONS(3830),
    [anon_sym_DOT] = ACTIONS(3828),
    [anon_sym_QMARK] = ACTIONS(3828),
    [anon_sym_AMP_AMP] = ACTIONS(3828),
    [anon_sym_PIPE_PIPE] = ACTIONS(3828),
    [anon_sym_GT_GT] = ACTIONS(3830),
    [anon_sym_GT_GT_GT] = ACTIONS(3828),
    [anon_sym_LT_LT] = ACTIONS(3828),
    [anon_sym_AMP] = ACTIONS(3830),
    [anon_sym_CARET] = ACTIONS(3828),
    [anon_sym_PIPE] = ACTIONS(3830),
    [anon_sym_PLUS] = ACTIONS(3830),
    [anon_sym_DASH] = ACTIONS(3830),
    [anon_sym_PERCENT] = ACTIONS(3828),
    [anon_sym_LT_EQ] = ACTIONS(3828),
    [anon_sym_EQ_EQ] = ACTIONS(3830),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3828),
    [anon_sym_BANG_EQ] = ACTIONS(3830),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3828),
    [anon_sym_GT_EQ] = ACTIONS(3828),
    [anon_sym_instanceof] = ACTIONS(3828),
    [anon_sym_PLUS_PLUS] = ACTIONS(3828),
    [anon_sym_DASH_DASH] = ACTIONS(3828),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3828),
  },
  [1425] = {
    [sym_statement_block] = STATE(1139),
    [anon_sym_LBRACE] = ACTIONS(3030),
    [sym_comment] = ACTIONS(71),
  },
  [1426] = {
    [sym__automatic_semicolon] = ACTIONS(2489),
    [anon_sym_LPAREN] = ACTIONS(2489),
    [anon_sym_SEMI] = ACTIONS(2489),
    [anon_sym_EQ] = ACTIONS(2489),
    [sym_comment] = ACTIONS(71),
  },
  [1427] = {
    [sym_formal_parameters] = STATE(1768),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1428] = {
    [sym_parenthesized_expression] = STATE(1770),
    [sym__expression] = STATE(1770),
    [sym_yield_expression] = STATE(1770),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(1770),
    [sym_jsx_fragment] = STATE(1770),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1770),
    [sym_anonymous_class] = STATE(1770),
    [sym_class] = STATE(1770),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1770),
    [sym_generator_function] = STATE(1770),
    [sym_call_expression] = STATE(1770),
    [sym_new_expression] = STATE(1770),
    [sym_await_expression] = STATE(1770),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(1770),
    [sym_augmented_assignment_expression] = STATE(1770),
    [sym_spread_element] = STATE(1771),
    [sym_ternary_expression] = STATE(1770),
    [sym_binary_expression] = STATE(1770),
    [sym_unary_expression] = STATE(1770),
    [sym_update_expression] = STATE(1770),
    [sym_template_string] = STATE(1770),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [aux_sym_array_repeat1] = STATE(1772),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(3832),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1096),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3834),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3836),
    [sym_number] = ACTIONS(3834),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(3836),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3836),
    [sym_false] = ACTIONS(3836),
    [sym_null] = ACTIONS(3836),
    [sym_undefined] = ACTIONS(3836),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1429] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3838),
  },
  [1430] = {
    [anon_sym_STAR] = ACTIONS(2234),
    [anon_sym_LBRACK] = ACTIONS(2234),
    [anon_sym_async] = ACTIONS(2577),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2234),
    [sym_number] = ACTIONS(2234),
    [sym_identifier] = ACTIONS(2577),
    [anon_sym_AT] = ACTIONS(2234),
    [anon_sym_static] = ACTIONS(2577),
    [anon_sym_get] = ACTIONS(2577),
    [anon_sym_set] = ACTIONS(2577),
  },
  [1431] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3840),
  },
  [1432] = {
    [anon_sym_STAR] = ACTIONS(2238),
    [anon_sym_LBRACK] = ACTIONS(2238),
    [anon_sym_async] = ACTIONS(2581),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2238),
    [sym_number] = ACTIONS(2238),
    [sym_identifier] = ACTIONS(2581),
    [anon_sym_AT] = ACTIONS(2238),
    [anon_sym_static] = ACTIONS(2581),
    [anon_sym_get] = ACTIONS(2581),
    [anon_sym_set] = ACTIONS(2581),
  },
  [1433] = {
    [anon_sym_STAR] = ACTIONS(3842),
    [anon_sym_RBRACE] = ACTIONS(3842),
    [anon_sym_LBRACK] = ACTIONS(3842),
    [anon_sym_async] = ACTIONS(3844),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3842),
    [sym_number] = ACTIONS(3842),
    [sym_identifier] = ACTIONS(3844),
    [anon_sym_AT] = ACTIONS(3842),
    [anon_sym_static] = ACTIONS(3844),
    [anon_sym_get] = ACTIONS(3844),
    [anon_sym_set] = ACTIONS(3844),
  },
  [1434] = {
    [sym_export_statement] = STATE(1776),
    [sym__declaration] = STATE(1776),
    [sym_import_statement] = STATE(1776),
    [sym_expression_statement] = STATE(1776),
    [sym_variable_declaration] = STATE(1776),
    [sym_lexical_declaration] = STATE(1776),
    [sym_statement_block] = STATE(1776),
    [sym_if_statement] = STATE(1776),
    [sym_switch_statement] = STATE(1776),
    [sym_for_statement] = STATE(1776),
    [sym_for_in_statement] = STATE(1776),
    [sym_for_of_statement] = STATE(1776),
    [sym_while_statement] = STATE(1776),
    [sym_do_statement] = STATE(1776),
    [sym_try_statement] = STATE(1776),
    [sym_with_statement] = STATE(1776),
    [sym_break_statement] = STATE(1776),
    [sym_continue_statement] = STATE(1776),
    [sym_debugger_statement] = STATE(1776),
    [sym_return_statement] = STATE(1776),
    [sym_throw_statement] = STATE(1776),
    [sym_empty_statement] = STATE(1776),
    [sym_labeled_statement] = STATE(1776),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1776),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(3846),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1435] = {
    [sym__automatic_semicolon] = ACTIONS(3848),
    [anon_sym_SEMI] = ACTIONS(3848),
    [sym_comment] = ACTIONS(71),
  },
  [1436] = {
    [anon_sym_STAR] = ACTIONS(3842),
    [anon_sym_RBRACE] = ACTIONS(3842),
    [anon_sym_SEMI] = ACTIONS(3848),
    [anon_sym_LBRACK] = ACTIONS(3842),
    [anon_sym_async] = ACTIONS(3844),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3842),
    [sym_number] = ACTIONS(3842),
    [sym_identifier] = ACTIONS(3844),
    [anon_sym_AT] = ACTIONS(3842),
    [anon_sym_static] = ACTIONS(3844),
    [anon_sym_get] = ACTIONS(3844),
    [anon_sym_set] = ACTIONS(3844),
  },
  [1437] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1438] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1439] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1440] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(1778),
    [sym__expression] = STATE(1778),
    [sym_yield_expression] = STATE(1778),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(1778),
    [sym_jsx_fragment] = STATE(1778),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1778),
    [sym_anonymous_class] = STATE(1778),
    [sym_class] = STATE(1778),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1778),
    [sym_generator_function] = STATE(1778),
    [sym_call_expression] = STATE(1778),
    [sym_new_expression] = STATE(1778),
    [sym_await_expression] = STATE(1778),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(1778),
    [sym_augmented_assignment_expression] = STATE(1778),
    [sym_ternary_expression] = STATE(1778),
    [sym_binary_expression] = STATE(1778),
    [sym_unary_expression] = STATE(1778),
    [sym_update_expression] = STATE(1778),
    [sym_template_string] = STATE(1778),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3850),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3852),
    [sym_number] = ACTIONS(3850),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(3852),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3852),
    [sym_false] = ACTIONS(3852),
    [sym_null] = ACTIONS(3852),
    [sym_undefined] = ACTIONS(3852),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1441] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1442] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1443] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3854),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1444] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1445] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1446] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1447] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [ts_builtin_sym_end] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_STAR] = ACTIONS(3035),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_in] = ACTIONS(3037),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3037),
    [anon_sym_GT] = ACTIONS(3037),
    [anon_sym_SLASH] = ACTIONS(3037),
    [anon_sym_DOT] = ACTIONS(3037),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_QMARK] = ACTIONS(3035),
    [anon_sym_AMP_AMP] = ACTIONS(3035),
    [anon_sym_PIPE_PIPE] = ACTIONS(3035),
    [anon_sym_GT_GT] = ACTIONS(3037),
    [anon_sym_GT_GT_GT] = ACTIONS(3035),
    [anon_sym_LT_LT] = ACTIONS(3035),
    [anon_sym_AMP] = ACTIONS(3037),
    [anon_sym_CARET] = ACTIONS(3035),
    [anon_sym_PIPE] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_PERCENT] = ACTIONS(3035),
    [anon_sym_LT_EQ] = ACTIONS(3035),
    [anon_sym_EQ_EQ] = ACTIONS(3037),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3035),
    [anon_sym_BANG_EQ] = ACTIONS(3037),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3035),
    [anon_sym_GT_EQ] = ACTIONS(3035),
    [anon_sym_instanceof] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3037),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [1448] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [ts_builtin_sym_end] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_STAR] = ACTIONS(3856),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_COMMA] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_in] = ACTIONS(3858),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_GT] = ACTIONS(3858),
    [anon_sym_SLASH] = ACTIONS(3858),
    [anon_sym_DOT] = ACTIONS(3858),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_QMARK] = ACTIONS(3856),
    [anon_sym_AMP_AMP] = ACTIONS(3856),
    [anon_sym_PIPE_PIPE] = ACTIONS(3856),
    [anon_sym_GT_GT] = ACTIONS(3858),
    [anon_sym_GT_GT_GT] = ACTIONS(3856),
    [anon_sym_LT_LT] = ACTIONS(3856),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_CARET] = ACTIONS(3856),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_PERCENT] = ACTIONS(3856),
    [anon_sym_LT_EQ] = ACTIONS(3856),
    [anon_sym_EQ_EQ] = ACTIONS(3858),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3856),
    [anon_sym_BANG_EQ] = ACTIONS(3858),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3856),
    [anon_sym_GT_EQ] = ACTIONS(3856),
    [anon_sym_instanceof] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3858),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [1449] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [ts_builtin_sym_end] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_STAR] = ACTIONS(3860),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_COMMA] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_in] = ACTIONS(3862),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_GT] = ACTIONS(3862),
    [anon_sym_SLASH] = ACTIONS(3862),
    [anon_sym_DOT] = ACTIONS(3862),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_QMARK] = ACTIONS(3860),
    [anon_sym_AMP_AMP] = ACTIONS(3860),
    [anon_sym_PIPE_PIPE] = ACTIONS(3860),
    [anon_sym_GT_GT] = ACTIONS(3862),
    [anon_sym_GT_GT_GT] = ACTIONS(3860),
    [anon_sym_LT_LT] = ACTIONS(3860),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_CARET] = ACTIONS(3860),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_PERCENT] = ACTIONS(3860),
    [anon_sym_LT_EQ] = ACTIONS(3860),
    [anon_sym_EQ_EQ] = ACTIONS(3862),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3860),
    [anon_sym_BANG_EQ] = ACTIONS(3862),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3860),
    [anon_sym_GT_EQ] = ACTIONS(3860),
    [anon_sym_instanceof] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3862),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [1450] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_in] = ACTIONS(1590),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1590),
    [anon_sym_SLASH] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1590),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_QMARK] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1588),
    [anon_sym_PIPE_PIPE] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1590),
    [anon_sym_GT_GT_GT] = ACTIONS(1588),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1590),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_LT_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1588),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1588),
    [anon_sym_instanceof] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1590),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [1451] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(2186),
    [anon_sym_RBRACE] = ACTIONS(2240),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1452] = {
    [sym__automatic_semicolon] = ACTIONS(3864),
    [anon_sym_STAR] = ACTIONS(3864),
    [anon_sym_COMMA] = ACTIONS(3864),
    [anon_sym_RBRACE] = ACTIONS(3864),
    [anon_sym_LPAREN] = ACTIONS(3864),
    [anon_sym_RPAREN] = ACTIONS(3864),
    [anon_sym_in] = ACTIONS(3866),
    [anon_sym_SEMI] = ACTIONS(3864),
    [anon_sym_LBRACK] = ACTIONS(3864),
    [anon_sym_LT] = ACTIONS(3866),
    [anon_sym_GT] = ACTIONS(3866),
    [anon_sym_SLASH] = ACTIONS(3866),
    [anon_sym_DOT] = ACTIONS(3864),
    [anon_sym_QMARK] = ACTIONS(3864),
    [anon_sym_AMP_AMP] = ACTIONS(3864),
    [anon_sym_PIPE_PIPE] = ACTIONS(3864),
    [anon_sym_GT_GT] = ACTIONS(3866),
    [anon_sym_GT_GT_GT] = ACTIONS(3864),
    [anon_sym_LT_LT] = ACTIONS(3864),
    [anon_sym_AMP] = ACTIONS(3866),
    [anon_sym_CARET] = ACTIONS(3864),
    [anon_sym_PIPE] = ACTIONS(3866),
    [anon_sym_PLUS] = ACTIONS(3866),
    [anon_sym_DASH] = ACTIONS(3866),
    [anon_sym_PERCENT] = ACTIONS(3864),
    [anon_sym_LT_EQ] = ACTIONS(3864),
    [anon_sym_EQ_EQ] = ACTIONS(3866),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3864),
    [anon_sym_BANG_EQ] = ACTIONS(3866),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3864),
    [anon_sym_GT_EQ] = ACTIONS(3864),
    [anon_sym_instanceof] = ACTIONS(3864),
    [anon_sym_PLUS_PLUS] = ACTIONS(3864),
    [anon_sym_DASH_DASH] = ACTIONS(3864),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3864),
  },
  [1453] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(2230),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2240),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1454] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2164),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1455] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1456] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1457] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(1780),
    [sym__expression] = STATE(1780),
    [sym_yield_expression] = STATE(1780),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1780),
    [sym_jsx_fragment] = STATE(1780),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1780),
    [sym_anonymous_class] = STATE(1780),
    [sym_class] = STATE(1780),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1780),
    [sym_generator_function] = STATE(1780),
    [sym_call_expression] = STATE(1780),
    [sym_new_expression] = STATE(1780),
    [sym_await_expression] = STATE(1780),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1780),
    [sym_augmented_assignment_expression] = STATE(1780),
    [sym_ternary_expression] = STATE(1780),
    [sym_binary_expression] = STATE(1780),
    [sym_unary_expression] = STATE(1780),
    [sym_update_expression] = STATE(1780),
    [sym_template_string] = STATE(1780),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3868),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3870),
    [sym_number] = ACTIONS(3868),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3870),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3870),
    [sym_false] = ACTIONS(3870),
    [sym_null] = ACTIONS(3870),
    [sym_undefined] = ACTIONS(3870),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1458] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1459] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1460] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3872),
    [anon_sym_STAR] = ACTIONS(370),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(3872),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(348),
    [anon_sym_GT] = ACTIONS(348),
    [anon_sym_SLASH] = ACTIONS(374),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(376),
    [anon_sym_AMP_AMP] = ACTIONS(378),
    [anon_sym_PIPE_PIPE] = ACTIONS(380),
    [anon_sym_GT_GT] = ACTIONS(374),
    [anon_sym_GT_GT_GT] = ACTIONS(370),
    [anon_sym_LT_LT] = ACTIONS(370),
    [anon_sym_AMP] = ACTIONS(382),
    [anon_sym_CARET] = ACTIONS(380),
    [anon_sym_PIPE] = ACTIONS(384),
    [anon_sym_PLUS] = ACTIONS(386),
    [anon_sym_DASH] = ACTIONS(386),
    [anon_sym_PERCENT] = ACTIONS(370),
    [anon_sym_LT_EQ] = ACTIONS(388),
    [anon_sym_EQ_EQ] = ACTIONS(348),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(388),
    [anon_sym_BANG_EQ] = ACTIONS(348),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(388),
    [anon_sym_GT_EQ] = ACTIONS(388),
    [anon_sym_instanceof] = ACTIONS(388),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1461] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3874),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1462] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1463] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1464] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1465] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1466] = {
    [sym_parenthesized_expression] = STATE(1782),
    [sym__expression] = STATE(1782),
    [sym_yield_expression] = STATE(1782),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1782),
    [sym_jsx_fragment] = STATE(1782),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1782),
    [sym_anonymous_class] = STATE(1782),
    [sym_class] = STATE(1782),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1782),
    [sym_generator_function] = STATE(1782),
    [sym_call_expression] = STATE(1782),
    [sym_new_expression] = STATE(1782),
    [sym_await_expression] = STATE(1782),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1782),
    [sym_augmented_assignment_expression] = STATE(1782),
    [sym_ternary_expression] = STATE(1782),
    [sym_binary_expression] = STATE(1782),
    [sym_unary_expression] = STATE(1782),
    [sym_update_expression] = STATE(1782),
    [sym_template_string] = STATE(1782),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3876),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3878),
    [sym_number] = ACTIONS(3876),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3878),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3878),
    [sym_false] = ACTIONS(3878),
    [sym_null] = ACTIONS(3878),
    [sym_undefined] = ACTIONS(3878),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1467] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1783),
    [sym__expression] = STATE(1783),
    [sym_yield_expression] = STATE(1783),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1783),
    [sym_jsx_fragment] = STATE(1783),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1783),
    [sym_anonymous_class] = STATE(1783),
    [sym_class] = STATE(1783),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1783),
    [sym_generator_function] = STATE(1783),
    [sym_call_expression] = STATE(1783),
    [sym_new_expression] = STATE(1783),
    [sym_await_expression] = STATE(1783),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1783),
    [sym_augmented_assignment_expression] = STATE(1783),
    [sym_ternary_expression] = STATE(1783),
    [sym_binary_expression] = STATE(1783),
    [sym_unary_expression] = STATE(1783),
    [sym_update_expression] = STATE(1783),
    [sym_template_string] = STATE(1783),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3880),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3882),
    [sym_number] = ACTIONS(3880),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3882),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3882),
    [sym_false] = ACTIONS(3882),
    [sym_null] = ACTIONS(3882),
    [sym_undefined] = ACTIONS(3882),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1468] = {
    [sym_parenthesized_expression] = STATE(1784),
    [sym__expression] = STATE(1784),
    [sym_yield_expression] = STATE(1784),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1784),
    [sym_jsx_fragment] = STATE(1784),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1784),
    [sym_anonymous_class] = STATE(1784),
    [sym_class] = STATE(1784),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1784),
    [sym_generator_function] = STATE(1784),
    [sym_call_expression] = STATE(1784),
    [sym_new_expression] = STATE(1784),
    [sym_await_expression] = STATE(1784),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1784),
    [sym_augmented_assignment_expression] = STATE(1784),
    [sym_ternary_expression] = STATE(1784),
    [sym_binary_expression] = STATE(1784),
    [sym_unary_expression] = STATE(1784),
    [sym_update_expression] = STATE(1784),
    [sym_template_string] = STATE(1784),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3884),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3886),
    [sym_number] = ACTIONS(3884),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3886),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3886),
    [sym_false] = ACTIONS(3886),
    [sym_null] = ACTIONS(3886),
    [sym_undefined] = ACTIONS(3886),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1469] = {
    [anon_sym_EQ_GT] = ACTIONS(3888),
    [sym_comment] = ACTIONS(71),
  },
  [1470] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1471] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1472] = {
    [sym_parenthesized_expression] = STATE(1786),
    [sym__expression] = STATE(1786),
    [sym_yield_expression] = STATE(1786),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1786),
    [sym_jsx_fragment] = STATE(1786),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1786),
    [sym_anonymous_class] = STATE(1786),
    [sym_class] = STATE(1786),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1786),
    [sym_generator_function] = STATE(1786),
    [sym_call_expression] = STATE(1786),
    [sym_new_expression] = STATE(1786),
    [sym_await_expression] = STATE(1786),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1786),
    [sym_augmented_assignment_expression] = STATE(1786),
    [sym_ternary_expression] = STATE(1786),
    [sym_binary_expression] = STATE(1786),
    [sym_unary_expression] = STATE(1786),
    [sym_update_expression] = STATE(1786),
    [sym_template_string] = STATE(1786),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3890),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3892),
    [sym_number] = ACTIONS(3890),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3892),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3892),
    [sym_false] = ACTIONS(3892),
    [sym_null] = ACTIONS(3892),
    [sym_undefined] = ACTIONS(3892),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1473] = {
    [sym_parenthesized_expression] = STATE(1787),
    [sym__expression] = STATE(1787),
    [sym_yield_expression] = STATE(1787),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1787),
    [sym_jsx_fragment] = STATE(1787),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1787),
    [sym_anonymous_class] = STATE(1787),
    [sym_class] = STATE(1787),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1787),
    [sym_generator_function] = STATE(1787),
    [sym_call_expression] = STATE(1787),
    [sym_new_expression] = STATE(1787),
    [sym_await_expression] = STATE(1787),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1787),
    [sym_augmented_assignment_expression] = STATE(1787),
    [sym_ternary_expression] = STATE(1787),
    [sym_binary_expression] = STATE(1787),
    [sym_unary_expression] = STATE(1787),
    [sym_update_expression] = STATE(1787),
    [sym_template_string] = STATE(1787),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3894),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3896),
    [sym_number] = ACTIONS(3894),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3896),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3896),
    [sym_false] = ACTIONS(3896),
    [sym_null] = ACTIONS(3896),
    [sym_undefined] = ACTIONS(3896),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1474] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1475] = {
    [sym_parenthesized_expression] = STATE(1788),
    [sym__expression] = STATE(1788),
    [sym_yield_expression] = STATE(1788),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1788),
    [sym_jsx_fragment] = STATE(1788),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1788),
    [sym_anonymous_class] = STATE(1788),
    [sym_class] = STATE(1788),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1788),
    [sym_generator_function] = STATE(1788),
    [sym_call_expression] = STATE(1788),
    [sym_new_expression] = STATE(1788),
    [sym_await_expression] = STATE(1788),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1788),
    [sym_augmented_assignment_expression] = STATE(1788),
    [sym_ternary_expression] = STATE(1788),
    [sym_binary_expression] = STATE(1788),
    [sym_unary_expression] = STATE(1788),
    [sym_update_expression] = STATE(1788),
    [sym_template_string] = STATE(1788),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3898),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3900),
    [sym_number] = ACTIONS(3898),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(3900),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3900),
    [sym_false] = ACTIONS(3900),
    [sym_null] = ACTIONS(3900),
    [sym_undefined] = ACTIONS(3900),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1476] = {
    [sym_parenthesized_expression] = STATE(1789),
    [sym__expression] = STATE(1789),
    [sym_yield_expression] = STATE(1789),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1789),
    [sym_jsx_fragment] = STATE(1789),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1789),
    [sym_anonymous_class] = STATE(1789),
    [sym_class] = STATE(1789),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1789),
    [sym_generator_function] = STATE(1789),
    [sym_call_expression] = STATE(1789),
    [sym_new_expression] = STATE(1789),
    [sym_await_expression] = STATE(1789),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1789),
    [sym_augmented_assignment_expression] = STATE(1789),
    [sym_ternary_expression] = STATE(1789),
    [sym_binary_expression] = STATE(1789),
    [sym_unary_expression] = STATE(1789),
    [sym_update_expression] = STATE(1789),
    [sym_template_string] = STATE(1789),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3902),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3904),
    [sym_number] = ACTIONS(3902),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3904),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3904),
    [sym_false] = ACTIONS(3904),
    [sym_null] = ACTIONS(3904),
    [sym_undefined] = ACTIONS(3904),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1477] = {
    [sym_parenthesized_expression] = STATE(1790),
    [sym__expression] = STATE(1790),
    [sym_yield_expression] = STATE(1790),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1790),
    [sym_jsx_fragment] = STATE(1790),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1790),
    [sym_anonymous_class] = STATE(1790),
    [sym_class] = STATE(1790),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1790),
    [sym_generator_function] = STATE(1790),
    [sym_call_expression] = STATE(1790),
    [sym_new_expression] = STATE(1790),
    [sym_await_expression] = STATE(1790),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1790),
    [sym_augmented_assignment_expression] = STATE(1790),
    [sym_ternary_expression] = STATE(1790),
    [sym_binary_expression] = STATE(1790),
    [sym_unary_expression] = STATE(1790),
    [sym_update_expression] = STATE(1790),
    [sym_template_string] = STATE(1790),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3906),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3908),
    [sym_number] = ACTIONS(3906),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3908),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3908),
    [sym_false] = ACTIONS(3908),
    [sym_null] = ACTIONS(3908),
    [sym_undefined] = ACTIONS(3908),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1478] = {
    [sym_parenthesized_expression] = STATE(1791),
    [sym__expression] = STATE(1791),
    [sym_yield_expression] = STATE(1791),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(1791),
    [sym_jsx_fragment] = STATE(1791),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1791),
    [sym_anonymous_class] = STATE(1791),
    [sym_class] = STATE(1791),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1791),
    [sym_generator_function] = STATE(1791),
    [sym_call_expression] = STATE(1791),
    [sym_new_expression] = STATE(1791),
    [sym_await_expression] = STATE(1791),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(1791),
    [sym_augmented_assignment_expression] = STATE(1791),
    [sym_ternary_expression] = STATE(1791),
    [sym_binary_expression] = STATE(1791),
    [sym_unary_expression] = STATE(1791),
    [sym_update_expression] = STATE(1791),
    [sym_template_string] = STATE(1791),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3910),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3912),
    [sym_number] = ACTIONS(3910),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(3912),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3912),
    [sym_false] = ACTIONS(3912),
    [sym_null] = ACTIONS(3912),
    [sym_undefined] = ACTIONS(3912),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1479] = {
    [sym__automatic_semicolon] = ACTIONS(3914),
    [anon_sym_STAR] = ACTIONS(3914),
    [anon_sym_COMMA] = ACTIONS(3914),
    [anon_sym_RBRACE] = ACTIONS(3914),
    [anon_sym_LPAREN] = ACTIONS(3914),
    [anon_sym_RPAREN] = ACTIONS(3914),
    [anon_sym_in] = ACTIONS(3916),
    [anon_sym_SEMI] = ACTIONS(3914),
    [anon_sym_LBRACK] = ACTIONS(3914),
    [anon_sym_LT] = ACTIONS(3916),
    [anon_sym_GT] = ACTIONS(3916),
    [anon_sym_SLASH] = ACTIONS(3916),
    [anon_sym_DOT] = ACTIONS(3914),
    [anon_sym_QMARK] = ACTIONS(3914),
    [anon_sym_AMP_AMP] = ACTIONS(3914),
    [anon_sym_PIPE_PIPE] = ACTIONS(3914),
    [anon_sym_GT_GT] = ACTIONS(3916),
    [anon_sym_GT_GT_GT] = ACTIONS(3914),
    [anon_sym_LT_LT] = ACTIONS(3914),
    [anon_sym_AMP] = ACTIONS(3916),
    [anon_sym_CARET] = ACTIONS(3914),
    [anon_sym_PIPE] = ACTIONS(3916),
    [anon_sym_PLUS] = ACTIONS(3916),
    [anon_sym_DASH] = ACTIONS(3916),
    [anon_sym_PERCENT] = ACTIONS(3914),
    [anon_sym_LT_EQ] = ACTIONS(3914),
    [anon_sym_EQ_EQ] = ACTIONS(3916),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3914),
    [anon_sym_BANG_EQ] = ACTIONS(3916),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3914),
    [anon_sym_GT_EQ] = ACTIONS(3914),
    [anon_sym_instanceof] = ACTIONS(3914),
    [anon_sym_PLUS_PLUS] = ACTIONS(3914),
    [anon_sym_DASH_DASH] = ACTIONS(3914),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3914),
  },
  [1480] = {
    [sym__automatic_semicolon] = ACTIONS(3918),
    [anon_sym_STAR] = ACTIONS(3918),
    [anon_sym_COMMA] = ACTIONS(3918),
    [anon_sym_RBRACE] = ACTIONS(3918),
    [anon_sym_LPAREN] = ACTIONS(3918),
    [anon_sym_RPAREN] = ACTIONS(3918),
    [anon_sym_in] = ACTIONS(3920),
    [anon_sym_SEMI] = ACTIONS(3918),
    [anon_sym_LBRACK] = ACTIONS(3918),
    [anon_sym_LT] = ACTIONS(3920),
    [anon_sym_GT] = ACTIONS(3920),
    [anon_sym_SLASH] = ACTIONS(3920),
    [anon_sym_DOT] = ACTIONS(3918),
    [anon_sym_QMARK] = ACTIONS(3918),
    [anon_sym_AMP_AMP] = ACTIONS(3918),
    [anon_sym_PIPE_PIPE] = ACTIONS(3918),
    [anon_sym_GT_GT] = ACTIONS(3920),
    [anon_sym_GT_GT_GT] = ACTIONS(3918),
    [anon_sym_LT_LT] = ACTIONS(3918),
    [anon_sym_AMP] = ACTIONS(3920),
    [anon_sym_CARET] = ACTIONS(3918),
    [anon_sym_PIPE] = ACTIONS(3920),
    [anon_sym_PLUS] = ACTIONS(3920),
    [anon_sym_DASH] = ACTIONS(3920),
    [anon_sym_PERCENT] = ACTIONS(3918),
    [anon_sym_LT_EQ] = ACTIONS(3918),
    [anon_sym_EQ_EQ] = ACTIONS(3920),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3918),
    [anon_sym_BANG_EQ] = ACTIONS(3920),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3918),
    [anon_sym_GT_EQ] = ACTIONS(3918),
    [anon_sym_instanceof] = ACTIONS(3918),
    [anon_sym_PLUS_PLUS] = ACTIONS(3918),
    [anon_sym_DASH_DASH] = ACTIONS(3918),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3918),
  },
  [1481] = {
    [anon_sym_LBRACE] = ACTIONS(2967),
    [anon_sym_LT] = ACTIONS(2967),
    [sym_jsx_text] = ACTIONS(2967),
    [sym_comment] = ACTIONS(71),
  },
  [1482] = {
    [anon_sym_GT] = ACTIONS(3922),
    [sym_comment] = ACTIONS(71),
  },
  [1483] = {
    [anon_sym_LBRACE] = ACTIONS(3002),
    [anon_sym_LT] = ACTIONS(3002),
    [sym_jsx_text] = ACTIONS(3002),
    [sym_comment] = ACTIONS(71),
  },
  [1484] = {
    [anon_sym_GT] = ACTIONS(3924),
    [sym_comment] = ACTIONS(71),
  },
  [1485] = {
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(3926),
    [sym_comment] = ACTIONS(71),
  },
  [1486] = {
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(3928),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [1487] = {
    [anon_sym_GT] = ACTIONS(3928),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [1488] = {
    [anon_sym_GT] = ACTIONS(3928),
    [sym_comment] = ACTIONS(71),
  },
  [1489] = {
    [ts_builtin_sym_end] = ACTIONS(3930),
    [anon_sym_export] = ACTIONS(3932),
    [anon_sym_LBRACE] = ACTIONS(3930),
    [anon_sym_RBRACE] = ACTIONS(3930),
    [anon_sym_import] = ACTIONS(3932),
    [anon_sym_var] = ACTIONS(3932),
    [anon_sym_let] = ACTIONS(3932),
    [anon_sym_const] = ACTIONS(3932),
    [anon_sym_if] = ACTIONS(3932),
    [anon_sym_switch] = ACTIONS(3932),
    [anon_sym_for] = ACTIONS(3932),
    [anon_sym_LPAREN] = ACTIONS(3930),
    [anon_sym_while] = ACTIONS(3932),
    [anon_sym_do] = ACTIONS(3932),
    [anon_sym_try] = ACTIONS(3932),
    [anon_sym_with] = ACTIONS(3932),
    [anon_sym_break] = ACTIONS(3932),
    [anon_sym_continue] = ACTIONS(3932),
    [anon_sym_debugger] = ACTIONS(3932),
    [anon_sym_return] = ACTIONS(3932),
    [anon_sym_throw] = ACTIONS(3932),
    [anon_sym_SEMI] = ACTIONS(3930),
    [anon_sym_yield] = ACTIONS(3932),
    [anon_sym_LBRACK] = ACTIONS(3930),
    [anon_sym_LT] = ACTIONS(3930),
    [anon_sym_class] = ACTIONS(3932),
    [anon_sym_async] = ACTIONS(3932),
    [anon_sym_function] = ACTIONS(3932),
    [anon_sym_EQ_GT] = ACTIONS(3930),
    [anon_sym_new] = ACTIONS(3932),
    [anon_sym_await] = ACTIONS(3932),
    [anon_sym_PLUS] = ACTIONS(3932),
    [anon_sym_DASH] = ACTIONS(3932),
    [anon_sym_BANG] = ACTIONS(3930),
    [anon_sym_TILDE] = ACTIONS(3930),
    [anon_sym_typeof] = ACTIONS(3932),
    [anon_sym_void] = ACTIONS(3932),
    [anon_sym_delete] = ACTIONS(3932),
    [anon_sym_PLUS_PLUS] = ACTIONS(3930),
    [anon_sym_DASH_DASH] = ACTIONS(3930),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3930),
    [anon_sym_BQUOTE] = ACTIONS(3930),
    [sym_regex] = ACTIONS(3932),
    [sym_number] = ACTIONS(3930),
    [sym_identifier] = ACTIONS(3932),
    [sym_this] = ACTIONS(3932),
    [sym_super] = ACTIONS(3932),
    [sym_true] = ACTIONS(3932),
    [sym_false] = ACTIONS(3932),
    [sym_null] = ACTIONS(3932),
    [sym_undefined] = ACTIONS(3932),
    [anon_sym_AT] = ACTIONS(3930),
    [anon_sym_get] = ACTIONS(3932),
    [anon_sym_set] = ACTIONS(3932),
  },
  [1490] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [ts_builtin_sym_end] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_COMMA] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_in] = ACTIONS(3936),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3936),
    [anon_sym_GT] = ACTIONS(3936),
    [anon_sym_SLASH] = ACTIONS(3936),
    [anon_sym_DOT] = ACTIONS(3936),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_QMARK] = ACTIONS(3934),
    [anon_sym_AMP_AMP] = ACTIONS(3934),
    [anon_sym_PIPE_PIPE] = ACTIONS(3934),
    [anon_sym_GT_GT] = ACTIONS(3936),
    [anon_sym_GT_GT_GT] = ACTIONS(3934),
    [anon_sym_LT_LT] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3936),
    [anon_sym_CARET] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_PERCENT] = ACTIONS(3934),
    [anon_sym_LT_EQ] = ACTIONS(3934),
    [anon_sym_EQ_EQ] = ACTIONS(3936),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3934),
    [anon_sym_BANG_EQ] = ACTIONS(3936),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3934),
    [anon_sym_GT_EQ] = ACTIONS(3934),
    [anon_sym_instanceof] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3936),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [1491] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3111),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1492] = {
    [sym_parenthesized_expression] = STATE(1796),
    [sym__expression] = STATE(1796),
    [sym_yield_expression] = STATE(1796),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1796),
    [sym_jsx_fragment] = STATE(1796),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1796),
    [sym_anonymous_class] = STATE(1796),
    [sym_class] = STATE(1796),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1796),
    [sym_generator_function] = STATE(1796),
    [sym_call_expression] = STATE(1796),
    [sym_new_expression] = STATE(1796),
    [sym_await_expression] = STATE(1796),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1796),
    [sym_augmented_assignment_expression] = STATE(1796),
    [sym_ternary_expression] = STATE(1796),
    [sym_binary_expression] = STATE(1796),
    [sym_unary_expression] = STATE(1796),
    [sym_update_expression] = STATE(1796),
    [sym_template_string] = STATE(1796),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3938),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(3940),
    [sym_number] = ACTIONS(3938),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(3940),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(3940),
    [sym_false] = ACTIONS(3940),
    [sym_null] = ACTIONS(3940),
    [sym_undefined] = ACTIONS(3940),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [1493] = {
    [sym__automatic_semicolon] = ACTIONS(3942),
    [anon_sym_from] = ACTIONS(3942),
    [anon_sym_SEMI] = ACTIONS(3942),
    [sym_comment] = ACTIONS(71),
  },
  [1494] = {
    [sym__import_export_specifier] = STATE(1111),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [1495] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [ts_builtin_sym_end] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3035),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3035),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [1496] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [ts_builtin_sym_end] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3856),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3856),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [1497] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [ts_builtin_sym_end] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3860),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3860),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [1498] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [1499] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [ts_builtin_sym_end] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3934),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3934),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [1500] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2164),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1501] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2166),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1502] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2170),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1503] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(1797),
    [sym__expression] = STATE(1797),
    [sym_yield_expression] = STATE(1797),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(1797),
    [sym_jsx_fragment] = STATE(1797),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1797),
    [sym_anonymous_class] = STATE(1797),
    [sym_class] = STATE(1797),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1797),
    [sym_generator_function] = STATE(1797),
    [sym_call_expression] = STATE(1797),
    [sym_new_expression] = STATE(1797),
    [sym_await_expression] = STATE(1797),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(1797),
    [sym_augmented_assignment_expression] = STATE(1797),
    [sym_ternary_expression] = STATE(1797),
    [sym_binary_expression] = STATE(1797),
    [sym_unary_expression] = STATE(1797),
    [sym_update_expression] = STATE(1797),
    [sym_template_string] = STATE(1797),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3944),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3946),
    [sym_number] = ACTIONS(3944),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(3946),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3946),
    [sym_false] = ACTIONS(3946),
    [sym_null] = ACTIONS(3946),
    [sym_undefined] = ACTIONS(3946),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1504] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1505] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2212),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1506] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3948),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1507] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1508] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1509] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(2200),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1510] = {
    [anon_sym_COMMA] = ACTIONS(2491),
    [anon_sym_RPAREN] = ACTIONS(2491),
    [anon_sym_EQ] = ACTIONS(2491),
    [sym_comment] = ACTIONS(71),
  },
  [1511] = {
    [anon_sym_COMMA] = ACTIONS(2583),
    [anon_sym_RPAREN] = ACTIONS(2583),
    [anon_sym_EQ] = ACTIONS(2583),
    [sym_comment] = ACTIONS(71),
  },
  [1512] = {
    [aux_sym_object_repeat1] = STATE(328),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(3950),
    [sym_comment] = ACTIONS(71),
  },
  [1513] = {
    [anon_sym_COMMA] = ACTIONS(2945),
    [anon_sym_RPAREN] = ACTIONS(2945),
    [anon_sym_EQ] = ACTIONS(2945),
    [sym_comment] = ACTIONS(71),
  },
  [1514] = {
    [anon_sym_STAR] = ACTIONS(2218),
    [anon_sym_LBRACK] = ACTIONS(2218),
    [anon_sym_async] = ACTIONS(2220),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2218),
    [sym_number] = ACTIONS(2218),
    [sym_identifier] = ACTIONS(2220),
    [anon_sym_AT] = ACTIONS(2218),
    [anon_sym_get] = ACTIONS(2220),
    [anon_sym_set] = ACTIONS(2220),
  },
  [1515] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(1801),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3952),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1516] = {
    [aux_sym_array_repeat1] = STATE(1801),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(3952),
    [sym_comment] = ACTIONS(71),
  },
  [1517] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(3952),
    [sym_comment] = ACTIONS(71),
  },
  [1518] = {
    [anon_sym_STAR] = ACTIONS(3146),
    [anon_sym_LPAREN] = ACTIONS(3146),
    [anon_sym_LBRACK] = ACTIONS(3146),
    [anon_sym_DOT] = ACTIONS(3413),
    [anon_sym_async] = ACTIONS(3413),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3146),
    [sym_number] = ACTIONS(3146),
    [sym_identifier] = ACTIONS(3413),
    [anon_sym_AT] = ACTIONS(3146),
    [anon_sym_get] = ACTIONS(3413),
    [anon_sym_set] = ACTIONS(3413),
  },
  [1519] = {
    [anon_sym_STAR] = ACTIONS(3148),
    [anon_sym_LPAREN] = ACTIONS(3148),
    [anon_sym_LBRACK] = ACTIONS(3148),
    [anon_sym_DOT] = ACTIONS(3415),
    [anon_sym_async] = ACTIONS(3415),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3148),
    [sym_number] = ACTIONS(3148),
    [sym_identifier] = ACTIONS(3415),
    [anon_sym_AT] = ACTIONS(3148),
    [anon_sym_get] = ACTIONS(3415),
    [anon_sym_set] = ACTIONS(3415),
  },
  [1520] = {
    [anon_sym_export] = ACTIONS(1705),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_import] = ACTIONS(1705),
    [anon_sym_var] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_else] = ACTIONS(3954),
    [anon_sym_switch] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_do] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [anon_sym_with] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_debugger] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_throw] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_class] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_function] = ACTIONS(1705),
    [anon_sym_EQ_GT] = ACTIONS(1703),
    [anon_sym_new] = ACTIONS(1705),
    [anon_sym_await] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_typeof] = ACTIONS(1705),
    [anon_sym_void] = ACTIONS(1705),
    [anon_sym_delete] = ACTIONS(1705),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1703),
    [anon_sym_BQUOTE] = ACTIONS(1703),
    [sym_regex] = ACTIONS(1705),
    [sym_number] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_this] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_null] = ACTIONS(1705),
    [sym_undefined] = ACTIONS(1705),
    [anon_sym_AT] = ACTIONS(1703),
    [anon_sym_get] = ACTIONS(1705),
    [anon_sym_set] = ACTIONS(1705),
  },
  [1521] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(1804),
    [sym_array] = STATE(1805),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3956),
  },
  [1522] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(1804),
    [sym_array] = STATE(1805),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3956),
  },
  [1523] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3958),
    [anon_sym_of] = ACTIONS(3960),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1524] = {
    [sym_expression_statement] = STATE(1808),
    [sym_empty_statement] = STATE(1808),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1525] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3962),
    [anon_sym_of] = ACTIONS(3964),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1526] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(3966),
    [anon_sym_of] = ACTIONS(3968),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1527] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1813),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1813),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1528] = {
    [sym_parenthesized_expression] = STATE(1814),
    [sym__expression] = STATE(1814),
    [sym_yield_expression] = STATE(1814),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1814),
    [sym_jsx_fragment] = STATE(1814),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1814),
    [sym_anonymous_class] = STATE(1814),
    [sym_class] = STATE(1814),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1814),
    [sym_generator_function] = STATE(1814),
    [sym_call_expression] = STATE(1814),
    [sym_new_expression] = STATE(1814),
    [sym_await_expression] = STATE(1814),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1814),
    [sym_augmented_assignment_expression] = STATE(1814),
    [sym_ternary_expression] = STATE(1814),
    [sym_binary_expression] = STATE(1814),
    [sym_unary_expression] = STATE(1814),
    [sym_update_expression] = STATE(1814),
    [sym_template_string] = STATE(1814),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3970),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3972),
    [sym_number] = ACTIONS(3970),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3972),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3972),
    [sym_false] = ACTIONS(3972),
    [sym_null] = ACTIONS(3972),
    [sym_undefined] = ACTIONS(3972),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1529] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1815),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1815),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1530] = {
    [sym_parenthesized_expression] = STATE(1816),
    [sym__expression] = STATE(1816),
    [sym_yield_expression] = STATE(1816),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1816),
    [sym_jsx_fragment] = STATE(1816),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1816),
    [sym_anonymous_class] = STATE(1816),
    [sym_class] = STATE(1816),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1816),
    [sym_generator_function] = STATE(1816),
    [sym_call_expression] = STATE(1816),
    [sym_new_expression] = STATE(1816),
    [sym_await_expression] = STATE(1816),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1816),
    [sym_augmented_assignment_expression] = STATE(1816),
    [sym_ternary_expression] = STATE(1816),
    [sym_binary_expression] = STATE(1816),
    [sym_unary_expression] = STATE(1816),
    [sym_update_expression] = STATE(1816),
    [sym_template_string] = STATE(1816),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3974),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3976),
    [sym_number] = ACTIONS(3974),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3976),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3976),
    [sym_false] = ACTIONS(3976),
    [sym_null] = ACTIONS(3976),
    [sym_undefined] = ACTIONS(3976),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1531] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1817),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1817),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1532] = {
    [sym_parenthesized_expression] = STATE(1818),
    [sym__expression] = STATE(1818),
    [sym_yield_expression] = STATE(1818),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1818),
    [sym_jsx_fragment] = STATE(1818),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1818),
    [sym_anonymous_class] = STATE(1818),
    [sym_class] = STATE(1818),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1818),
    [sym_generator_function] = STATE(1818),
    [sym_call_expression] = STATE(1818),
    [sym_new_expression] = STATE(1818),
    [sym_await_expression] = STATE(1818),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1818),
    [sym_augmented_assignment_expression] = STATE(1818),
    [sym_ternary_expression] = STATE(1818),
    [sym_binary_expression] = STATE(1818),
    [sym_unary_expression] = STATE(1818),
    [sym_update_expression] = STATE(1818),
    [sym_template_string] = STATE(1818),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3978),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(3980),
    [sym_number] = ACTIONS(3978),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(3980),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(3980),
    [sym_false] = ACTIONS(3980),
    [sym_null] = ACTIONS(3980),
    [sym_undefined] = ACTIONS(3980),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1533] = {
    [anon_sym_RPAREN] = ACTIONS(3982),
    [sym_comment] = ACTIONS(71),
  },
  [1534] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3984),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1535] = {
    [sym_export_statement] = STATE(1685),
    [sym__declaration] = STATE(1685),
    [sym_import_statement] = STATE(1685),
    [sym_expression_statement] = STATE(1685),
    [sym_variable_declaration] = STATE(1685),
    [sym_lexical_declaration] = STATE(1685),
    [sym_statement_block] = STATE(1685),
    [sym_if_statement] = STATE(1685),
    [sym_switch_statement] = STATE(1685),
    [sym_for_statement] = STATE(1685),
    [sym_for_in_statement] = STATE(1685),
    [sym_for_of_statement] = STATE(1685),
    [sym_while_statement] = STATE(1685),
    [sym_do_statement] = STATE(1685),
    [sym_try_statement] = STATE(1685),
    [sym_with_statement] = STATE(1685),
    [sym_break_statement] = STATE(1685),
    [sym_continue_statement] = STATE(1685),
    [sym_debugger_statement] = STATE(1685),
    [sym_return_statement] = STATE(1685),
    [sym_throw_statement] = STATE(1685),
    [sym_empty_statement] = STATE(1685),
    [sym_labeled_statement] = STATE(1685),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1536] = {
    [anon_sym_RPAREN] = ACTIONS(3986),
    [sym_comment] = ACTIONS(71),
  },
  [1537] = {
    [anon_sym_RPAREN] = ACTIONS(3988),
    [sym_comment] = ACTIONS(71),
  },
  [1538] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3990),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1539] = {
    [anon_sym_RPAREN] = ACTIONS(3992),
    [sym_comment] = ACTIONS(71),
  },
  [1540] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3994),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1541] = {
    [anon_sym_STAR] = ACTIONS(3996),
    [anon_sym_COMMA] = ACTIONS(3996),
    [anon_sym_RBRACE] = ACTIONS(3996),
    [anon_sym_SEMI] = ACTIONS(3996),
    [anon_sym_LBRACK] = ACTIONS(3996),
    [anon_sym_async] = ACTIONS(3998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3996),
    [sym_number] = ACTIONS(3996),
    [sym_identifier] = ACTIONS(3998),
    [anon_sym_AT] = ACTIONS(3996),
    [anon_sym_static] = ACTIONS(3998),
    [anon_sym_get] = ACTIONS(3998),
    [anon_sym_set] = ACTIONS(3998),
  },
  [1542] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_RBRACE] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1543] = {
    [sym_parenthesized_expression] = STATE(1826),
    [sym__expression] = STATE(1826),
    [sym_yield_expression] = STATE(1826),
    [sym_object] = STATE(303),
    [sym_array] = STATE(304),
    [sym_jsx_element] = STATE(1826),
    [sym_jsx_fragment] = STATE(1826),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1826),
    [sym_anonymous_class] = STATE(1826),
    [sym_class] = STATE(1826),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1826),
    [sym_generator_function] = STATE(1826),
    [sym_call_expression] = STATE(1826),
    [sym_new_expression] = STATE(1826),
    [sym_await_expression] = STATE(1826),
    [sym_member_expression] = STATE(305),
    [sym_subscript_expression] = STATE(305),
    [sym_assignment_expression] = STATE(1826),
    [sym_augmented_assignment_expression] = STATE(1826),
    [sym_ternary_expression] = STATE(1826),
    [sym_binary_expression] = STATE(1826),
    [sym_unary_expression] = STATE(1826),
    [sym_update_expression] = STATE(1826),
    [sym_template_string] = STATE(1826),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(306),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(514),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(516),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(518),
    [anon_sym_new] = ACTIONS(520),
    [anon_sym_await] = ACTIONS(522),
    [anon_sym_PLUS] = ACTIONS(524),
    [anon_sym_DASH] = ACTIONS(524),
    [anon_sym_BANG] = ACTIONS(526),
    [anon_sym_TILDE] = ACTIONS(526),
    [anon_sym_typeof] = ACTIONS(524),
    [anon_sym_void] = ACTIONS(524),
    [anon_sym_delete] = ACTIONS(524),
    [anon_sym_PLUS_PLUS] = ACTIONS(528),
    [anon_sym_DASH_DASH] = ACTIONS(528),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4000),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4002),
    [sym_number] = ACTIONS(4000),
    [sym_identifier] = ACTIONS(534),
    [sym_this] = ACTIONS(4002),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4002),
    [sym_false] = ACTIONS(4002),
    [sym_null] = ACTIONS(4002),
    [sym_undefined] = ACTIONS(4002),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(536),
    [anon_sym_set] = ACTIONS(536),
  },
  [1544] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2166),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_RBRACE] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1545] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2170),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_RBRACE] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1546] = {
    [sym_statement_block] = STATE(1037),
    [sym_parenthesized_expression] = STATE(1827),
    [sym__expression] = STATE(1827),
    [sym_yield_expression] = STATE(1827),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(1827),
    [sym_jsx_fragment] = STATE(1827),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1827),
    [sym_anonymous_class] = STATE(1827),
    [sym_class] = STATE(1827),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1827),
    [sym_generator_function] = STATE(1827),
    [sym_call_expression] = STATE(1827),
    [sym_new_expression] = STATE(1827),
    [sym_await_expression] = STATE(1827),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(1827),
    [sym_augmented_assignment_expression] = STATE(1827),
    [sym_ternary_expression] = STATE(1827),
    [sym_binary_expression] = STATE(1827),
    [sym_unary_expression] = STATE(1827),
    [sym_update_expression] = STATE(1827),
    [sym_template_string] = STATE(1827),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4004),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4006),
    [sym_number] = ACTIONS(4004),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(4006),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4006),
    [sym_false] = ACTIONS(4006),
    [sym_null] = ACTIONS(4006),
    [sym_undefined] = ACTIONS(4006),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1547] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1548] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2212),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_RBRACE] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2212),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1549] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(4008),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1550] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1551] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1552] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1553] = {
    [anon_sym_export] = ACTIONS(3131),
    [anon_sym_STAR] = ACTIONS(3129),
    [anon_sym_LBRACK] = ACTIONS(3129),
    [anon_sym_class] = ACTIONS(3131),
    [anon_sym_async] = ACTIONS(3131),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3129),
    [sym_number] = ACTIONS(3129),
    [sym_identifier] = ACTIONS(3131),
    [anon_sym_AT] = ACTIONS(3129),
    [anon_sym_get] = ACTIONS(3131),
    [anon_sym_set] = ACTIONS(3131),
  },
  [1554] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(4010),
    [sym_comment] = ACTIONS(71),
  },
  [1555] = {
    [anon_sym_from] = ACTIONS(4012),
    [sym_comment] = ACTIONS(71),
  },
  [1556] = {
    [sym__import_export_specifier] = STATE(1201),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(440),
  },
  [1557] = {
    [sym__automatic_semicolon] = ACTIONS(3417),
    [anon_sym_COMMA] = ACTIONS(3417),
    [anon_sym_in] = ACTIONS(3417),
    [anon_sym_of] = ACTIONS(3417),
    [anon_sym_SEMI] = ACTIONS(3417),
    [anon_sym_EQ] = ACTIONS(3417),
    [sym_comment] = ACTIONS(71),
  },
  [1558] = {
    [ts_builtin_sym_end] = ACTIONS(2343),
    [anon_sym_export] = ACTIONS(2345),
    [anon_sym_LBRACE] = ACTIONS(2343),
    [anon_sym_RBRACE] = ACTIONS(2343),
    [anon_sym_import] = ACTIONS(2345),
    [anon_sym_var] = ACTIONS(2345),
    [anon_sym_let] = ACTIONS(2345),
    [anon_sym_const] = ACTIONS(2345),
    [anon_sym_if] = ACTIONS(2345),
    [anon_sym_else] = ACTIONS(2345),
    [anon_sym_switch] = ACTIONS(2345),
    [anon_sym_for] = ACTIONS(2345),
    [anon_sym_LPAREN] = ACTIONS(2343),
    [anon_sym_while] = ACTIONS(2345),
    [anon_sym_do] = ACTIONS(2345),
    [anon_sym_try] = ACTIONS(2345),
    [anon_sym_with] = ACTIONS(2345),
    [anon_sym_break] = ACTIONS(2345),
    [anon_sym_continue] = ACTIONS(2345),
    [anon_sym_debugger] = ACTIONS(2345),
    [anon_sym_return] = ACTIONS(2345),
    [anon_sym_throw] = ACTIONS(2345),
    [anon_sym_SEMI] = ACTIONS(2343),
    [anon_sym_yield] = ACTIONS(2345),
    [anon_sym_LBRACK] = ACTIONS(2343),
    [anon_sym_LT] = ACTIONS(2343),
    [anon_sym_class] = ACTIONS(2345),
    [anon_sym_async] = ACTIONS(2345),
    [anon_sym_function] = ACTIONS(2345),
    [anon_sym_EQ_GT] = ACTIONS(2343),
    [anon_sym_new] = ACTIONS(2345),
    [anon_sym_await] = ACTIONS(2345),
    [anon_sym_PLUS] = ACTIONS(2345),
    [anon_sym_DASH] = ACTIONS(2345),
    [anon_sym_BANG] = ACTIONS(2343),
    [anon_sym_TILDE] = ACTIONS(2343),
    [anon_sym_typeof] = ACTIONS(2345),
    [anon_sym_void] = ACTIONS(2345),
    [anon_sym_delete] = ACTIONS(2345),
    [anon_sym_PLUS_PLUS] = ACTIONS(2343),
    [anon_sym_DASH_DASH] = ACTIONS(2343),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2343),
    [anon_sym_BQUOTE] = ACTIONS(2343),
    [sym_regex] = ACTIONS(2345),
    [sym_number] = ACTIONS(2343),
    [sym_identifier] = ACTIONS(2345),
    [sym_this] = ACTIONS(2345),
    [sym_super] = ACTIONS(2345),
    [sym_true] = ACTIONS(2345),
    [sym_false] = ACTIONS(2345),
    [sym_null] = ACTIONS(2345),
    [sym_undefined] = ACTIONS(2345),
    [anon_sym_AT] = ACTIONS(2343),
    [anon_sym_get] = ACTIONS(2345),
    [anon_sym_set] = ACTIONS(2345),
  },
  [1559] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1831),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(4014),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1560] = {
    [sym_class_body] = STATE(1832),
    [anon_sym_LBRACE] = ACTIONS(3436),
    [sym_comment] = ACTIONS(71),
  },
  [1561] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [ts_builtin_sym_end] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_else] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2140),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2140),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [1562] = {
    [sym_formal_parameters] = STATE(1833),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1563] = {
    [sym_statement_block] = STATE(1834),
    [anon_sym_LBRACE] = ACTIONS(3442),
    [sym_comment] = ACTIONS(71),
  },
  [1564] = {
    [sym_formal_parameters] = STATE(1835),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1565] = {
    [sym_statement_block] = STATE(1836),
    [anon_sym_LBRACE] = ACTIONS(3442),
    [sym_comment] = ACTIONS(71),
  },
  [1566] = {
    [sym_export_statement] = STATE(1838),
    [sym__declaration] = STATE(1838),
    [sym_import_statement] = STATE(1838),
    [sym_expression_statement] = STATE(1838),
    [sym_variable_declaration] = STATE(1838),
    [sym_lexical_declaration] = STATE(1838),
    [sym_statement_block] = STATE(1838),
    [sym_if_statement] = STATE(1838),
    [sym_switch_statement] = STATE(1838),
    [sym_for_statement] = STATE(1838),
    [sym_for_in_statement] = STATE(1838),
    [sym_for_of_statement] = STATE(1838),
    [sym_while_statement] = STATE(1838),
    [sym_do_statement] = STATE(1838),
    [sym_try_statement] = STATE(1838),
    [sym_with_statement] = STATE(1838),
    [sym_break_statement] = STATE(1838),
    [sym_continue_statement] = STATE(1838),
    [sym_debugger_statement] = STATE(1838),
    [sym_return_statement] = STATE(1838),
    [sym_throw_statement] = STATE(1838),
    [sym_empty_statement] = STATE(1838),
    [sym_labeled_statement] = STATE(1838),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1838),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4016),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1567] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [ts_builtin_sym_end] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_else] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2178),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2178),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [1568] = {
    [sym_class_heritage] = STATE(1839),
    [sym_class_body] = STATE(1832),
    [anon_sym_LBRACE] = ACTIONS(3436),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [1569] = {
    [ts_builtin_sym_end] = ACTIONS(2607),
    [anon_sym_export] = ACTIONS(2609),
    [anon_sym_LBRACE] = ACTIONS(2607),
    [anon_sym_RBRACE] = ACTIONS(2607),
    [anon_sym_import] = ACTIONS(2609),
    [anon_sym_var] = ACTIONS(2609),
    [anon_sym_let] = ACTIONS(2609),
    [anon_sym_const] = ACTIONS(2609),
    [anon_sym_if] = ACTIONS(2609),
    [anon_sym_else] = ACTIONS(2609),
    [anon_sym_switch] = ACTIONS(2609),
    [anon_sym_for] = ACTIONS(2609),
    [anon_sym_LPAREN] = ACTIONS(2607),
    [anon_sym_while] = ACTIONS(2609),
    [anon_sym_do] = ACTIONS(2609),
    [anon_sym_try] = ACTIONS(2609),
    [anon_sym_with] = ACTIONS(2609),
    [anon_sym_break] = ACTIONS(2609),
    [anon_sym_continue] = ACTIONS(2609),
    [anon_sym_debugger] = ACTIONS(2609),
    [anon_sym_return] = ACTIONS(2609),
    [anon_sym_throw] = ACTIONS(2609),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_yield] = ACTIONS(2609),
    [anon_sym_LBRACK] = ACTIONS(2607),
    [anon_sym_LT] = ACTIONS(2607),
    [anon_sym_class] = ACTIONS(2609),
    [anon_sym_async] = ACTIONS(2609),
    [anon_sym_function] = ACTIONS(2609),
    [anon_sym_EQ_GT] = ACTIONS(2607),
    [anon_sym_new] = ACTIONS(2609),
    [anon_sym_await] = ACTIONS(2609),
    [anon_sym_PLUS] = ACTIONS(2609),
    [anon_sym_DASH] = ACTIONS(2609),
    [anon_sym_BANG] = ACTIONS(2607),
    [anon_sym_TILDE] = ACTIONS(2607),
    [anon_sym_typeof] = ACTIONS(2609),
    [anon_sym_void] = ACTIONS(2609),
    [anon_sym_delete] = ACTIONS(2609),
    [anon_sym_PLUS_PLUS] = ACTIONS(2607),
    [anon_sym_DASH_DASH] = ACTIONS(2607),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2607),
    [anon_sym_BQUOTE] = ACTIONS(2607),
    [sym_regex] = ACTIONS(2609),
    [sym_number] = ACTIONS(2607),
    [sym_identifier] = ACTIONS(2609),
    [sym_this] = ACTIONS(2609),
    [sym_super] = ACTIONS(2609),
    [sym_true] = ACTIONS(2609),
    [sym_false] = ACTIONS(2609),
    [sym_null] = ACTIONS(2609),
    [sym_undefined] = ACTIONS(2609),
    [anon_sym_AT] = ACTIONS(2607),
    [anon_sym_get] = ACTIONS(2609),
    [anon_sym_set] = ACTIONS(2609),
  },
  [1570] = {
    [ts_builtin_sym_end] = ACTIONS(2619),
    [anon_sym_export] = ACTIONS(2621),
    [anon_sym_LBRACE] = ACTIONS(2619),
    [anon_sym_RBRACE] = ACTIONS(2619),
    [anon_sym_import] = ACTIONS(2621),
    [anon_sym_var] = ACTIONS(2621),
    [anon_sym_let] = ACTIONS(2621),
    [anon_sym_const] = ACTIONS(2621),
    [anon_sym_if] = ACTIONS(2621),
    [anon_sym_else] = ACTIONS(2621),
    [anon_sym_switch] = ACTIONS(2621),
    [anon_sym_for] = ACTIONS(2621),
    [anon_sym_LPAREN] = ACTIONS(2619),
    [anon_sym_while] = ACTIONS(2621),
    [anon_sym_do] = ACTIONS(2621),
    [anon_sym_try] = ACTIONS(2621),
    [anon_sym_with] = ACTIONS(2621),
    [anon_sym_break] = ACTIONS(2621),
    [anon_sym_continue] = ACTIONS(2621),
    [anon_sym_debugger] = ACTIONS(2621),
    [anon_sym_return] = ACTIONS(2621),
    [anon_sym_throw] = ACTIONS(2621),
    [anon_sym_SEMI] = ACTIONS(2619),
    [anon_sym_yield] = ACTIONS(2621),
    [anon_sym_LBRACK] = ACTIONS(2619),
    [anon_sym_LT] = ACTIONS(2619),
    [anon_sym_class] = ACTIONS(2621),
    [anon_sym_async] = ACTIONS(2621),
    [anon_sym_function] = ACTIONS(2621),
    [anon_sym_EQ_GT] = ACTIONS(2619),
    [anon_sym_new] = ACTIONS(2621),
    [anon_sym_await] = ACTIONS(2621),
    [anon_sym_PLUS] = ACTIONS(2621),
    [anon_sym_DASH] = ACTIONS(2621),
    [anon_sym_BANG] = ACTIONS(2619),
    [anon_sym_TILDE] = ACTIONS(2619),
    [anon_sym_typeof] = ACTIONS(2621),
    [anon_sym_void] = ACTIONS(2621),
    [anon_sym_delete] = ACTIONS(2621),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2619),
    [anon_sym_BQUOTE] = ACTIONS(2619),
    [sym_regex] = ACTIONS(2621),
    [sym_number] = ACTIONS(2619),
    [sym_identifier] = ACTIONS(2621),
    [sym_this] = ACTIONS(2621),
    [sym_super] = ACTIONS(2621),
    [sym_true] = ACTIONS(2621),
    [sym_false] = ACTIONS(2621),
    [sym_null] = ACTIONS(2621),
    [sym_undefined] = ACTIONS(2621),
    [anon_sym_AT] = ACTIONS(2619),
    [anon_sym_get] = ACTIONS(2621),
    [anon_sym_set] = ACTIONS(2621),
  },
  [1571] = {
    [ts_builtin_sym_end] = ACTIONS(2626),
    [anon_sym_export] = ACTIONS(2628),
    [anon_sym_LBRACE] = ACTIONS(2626),
    [anon_sym_RBRACE] = ACTIONS(2626),
    [anon_sym_import] = ACTIONS(2628),
    [anon_sym_var] = ACTIONS(2628),
    [anon_sym_let] = ACTIONS(2628),
    [anon_sym_const] = ACTIONS(2628),
    [anon_sym_if] = ACTIONS(2628),
    [anon_sym_else] = ACTIONS(2628),
    [anon_sym_switch] = ACTIONS(2628),
    [anon_sym_for] = ACTIONS(2628),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2628),
    [anon_sym_do] = ACTIONS(2628),
    [anon_sym_try] = ACTIONS(2628),
    [anon_sym_with] = ACTIONS(2628),
    [anon_sym_break] = ACTIONS(2628),
    [anon_sym_continue] = ACTIONS(2628),
    [anon_sym_debugger] = ACTIONS(2628),
    [anon_sym_return] = ACTIONS(2628),
    [anon_sym_throw] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2628),
    [anon_sym_async] = ACTIONS(2628),
    [anon_sym_function] = ACTIONS(2628),
    [anon_sym_EQ_GT] = ACTIONS(2626),
    [anon_sym_new] = ACTIONS(2628),
    [anon_sym_await] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_DASH] = ACTIONS(2628),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2626),
    [anon_sym_typeof] = ACTIONS(2628),
    [anon_sym_void] = ACTIONS(2628),
    [anon_sym_delete] = ACTIONS(2628),
    [anon_sym_PLUS_PLUS] = ACTIONS(2626),
    [anon_sym_DASH_DASH] = ACTIONS(2626),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2626),
    [anon_sym_BQUOTE] = ACTIONS(2626),
    [sym_regex] = ACTIONS(2628),
    [sym_number] = ACTIONS(2626),
    [sym_identifier] = ACTIONS(2628),
    [sym_this] = ACTIONS(2628),
    [sym_super] = ACTIONS(2628),
    [sym_true] = ACTIONS(2628),
    [sym_false] = ACTIONS(2628),
    [sym_null] = ACTIONS(2628),
    [sym_undefined] = ACTIONS(2628),
    [anon_sym_AT] = ACTIONS(2626),
    [anon_sym_get] = ACTIONS(2628),
    [anon_sym_set] = ACTIONS(2628),
  },
  [1572] = {
    [sym_export_statement] = STATE(1840),
    [sym__declaration] = STATE(1840),
    [sym_import_statement] = STATE(1840),
    [sym_expression_statement] = STATE(1840),
    [sym_variable_declaration] = STATE(1840),
    [sym_lexical_declaration] = STATE(1840),
    [sym_statement_block] = STATE(1840),
    [sym_if_statement] = STATE(1840),
    [sym_switch_statement] = STATE(1840),
    [sym_for_statement] = STATE(1840),
    [sym_for_in_statement] = STATE(1840),
    [sym_for_of_statement] = STATE(1840),
    [sym_while_statement] = STATE(1840),
    [sym_do_statement] = STATE(1840),
    [sym_try_statement] = STATE(1840),
    [sym_with_statement] = STATE(1840),
    [sym_break_statement] = STATE(1840),
    [sym_continue_statement] = STATE(1840),
    [sym_debugger_statement] = STATE(1840),
    [sym_return_statement] = STATE(1840),
    [sym_throw_statement] = STATE(1840),
    [sym_empty_statement] = STATE(1840),
    [sym_labeled_statement] = STATE(1840),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1573] = {
    [ts_builtin_sym_end] = ACTIONS(2692),
    [anon_sym_export] = ACTIONS(2694),
    [anon_sym_LBRACE] = ACTIONS(2692),
    [anon_sym_RBRACE] = ACTIONS(2692),
    [anon_sym_import] = ACTIONS(2694),
    [anon_sym_var] = ACTIONS(2694),
    [anon_sym_let] = ACTIONS(2694),
    [anon_sym_const] = ACTIONS(2694),
    [anon_sym_if] = ACTIONS(2694),
    [anon_sym_else] = ACTIONS(2694),
    [anon_sym_switch] = ACTIONS(2694),
    [anon_sym_for] = ACTIONS(2694),
    [anon_sym_LPAREN] = ACTIONS(2692),
    [anon_sym_while] = ACTIONS(2694),
    [anon_sym_do] = ACTIONS(2694),
    [anon_sym_try] = ACTIONS(2694),
    [anon_sym_with] = ACTIONS(2694),
    [anon_sym_break] = ACTIONS(2694),
    [anon_sym_continue] = ACTIONS(2694),
    [anon_sym_debugger] = ACTIONS(2694),
    [anon_sym_return] = ACTIONS(2694),
    [anon_sym_throw] = ACTIONS(2694),
    [anon_sym_SEMI] = ACTIONS(2692),
    [anon_sym_yield] = ACTIONS(2694),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_LT] = ACTIONS(2692),
    [anon_sym_class] = ACTIONS(2694),
    [anon_sym_async] = ACTIONS(2694),
    [anon_sym_function] = ACTIONS(2694),
    [anon_sym_EQ_GT] = ACTIONS(2692),
    [anon_sym_new] = ACTIONS(2694),
    [anon_sym_await] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2694),
    [anon_sym_DASH] = ACTIONS(2694),
    [anon_sym_BANG] = ACTIONS(2692),
    [anon_sym_TILDE] = ACTIONS(2692),
    [anon_sym_typeof] = ACTIONS(2694),
    [anon_sym_void] = ACTIONS(2694),
    [anon_sym_delete] = ACTIONS(2694),
    [anon_sym_PLUS_PLUS] = ACTIONS(2692),
    [anon_sym_DASH_DASH] = ACTIONS(2692),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2692),
    [anon_sym_BQUOTE] = ACTIONS(2692),
    [sym_regex] = ACTIONS(2694),
    [sym_number] = ACTIONS(2692),
    [sym_identifier] = ACTIONS(2694),
    [sym_this] = ACTIONS(2694),
    [sym_super] = ACTIONS(2694),
    [sym_true] = ACTIONS(2694),
    [sym_false] = ACTIONS(2694),
    [sym_null] = ACTIONS(2694),
    [sym_undefined] = ACTIONS(2694),
    [anon_sym_AT] = ACTIONS(2692),
    [anon_sym_get] = ACTIONS(2694),
    [anon_sym_set] = ACTIONS(2694),
  },
  [1574] = {
    [sym_switch_case] = STATE(1276),
    [sym_switch_default] = STATE(1276),
    [aux_sym_switch_body_repeat1] = STATE(1276),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(4018),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [1575] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(4020),
    [anon_sym_of] = ACTIONS(4022),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1576] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(4024),
    [anon_sym_of] = ACTIONS(4026),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1577] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(4028),
    [anon_sym_of] = ACTIONS(4030),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1578] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1848),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1848),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1579] = {
    [sym_parenthesized_expression] = STATE(1849),
    [sym__expression] = STATE(1849),
    [sym_yield_expression] = STATE(1849),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1849),
    [sym_jsx_fragment] = STATE(1849),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1849),
    [sym_anonymous_class] = STATE(1849),
    [sym_class] = STATE(1849),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1849),
    [sym_generator_function] = STATE(1849),
    [sym_call_expression] = STATE(1849),
    [sym_new_expression] = STATE(1849),
    [sym_await_expression] = STATE(1849),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1849),
    [sym_augmented_assignment_expression] = STATE(1849),
    [sym_ternary_expression] = STATE(1849),
    [sym_binary_expression] = STATE(1849),
    [sym_unary_expression] = STATE(1849),
    [sym_update_expression] = STATE(1849),
    [sym_template_string] = STATE(1849),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4032),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4034),
    [sym_number] = ACTIONS(4032),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4034),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4034),
    [sym_false] = ACTIONS(4034),
    [sym_null] = ACTIONS(4034),
    [sym_undefined] = ACTIONS(4034),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1580] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1851),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1851),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(4036),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1581] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1852),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1852),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1582] = {
    [sym_parenthesized_expression] = STATE(1853),
    [sym__expression] = STATE(1853),
    [sym_yield_expression] = STATE(1853),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1853),
    [sym_jsx_fragment] = STATE(1853),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1853),
    [sym_anonymous_class] = STATE(1853),
    [sym_class] = STATE(1853),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1853),
    [sym_generator_function] = STATE(1853),
    [sym_call_expression] = STATE(1853),
    [sym_new_expression] = STATE(1853),
    [sym_await_expression] = STATE(1853),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1853),
    [sym_augmented_assignment_expression] = STATE(1853),
    [sym_ternary_expression] = STATE(1853),
    [sym_binary_expression] = STATE(1853),
    [sym_unary_expression] = STATE(1853),
    [sym_update_expression] = STATE(1853),
    [sym_template_string] = STATE(1853),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4038),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4040),
    [sym_number] = ACTIONS(4038),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4040),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4040),
    [sym_false] = ACTIONS(4040),
    [sym_null] = ACTIONS(4040),
    [sym_undefined] = ACTIONS(4040),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1583] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(1854),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(1854),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1584] = {
    [sym_parenthesized_expression] = STATE(1855),
    [sym__expression] = STATE(1855),
    [sym_yield_expression] = STATE(1855),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1855),
    [sym_jsx_fragment] = STATE(1855),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1855),
    [sym_anonymous_class] = STATE(1855),
    [sym_class] = STATE(1855),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1855),
    [sym_generator_function] = STATE(1855),
    [sym_call_expression] = STATE(1855),
    [sym_new_expression] = STATE(1855),
    [sym_await_expression] = STATE(1855),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1855),
    [sym_augmented_assignment_expression] = STATE(1855),
    [sym_ternary_expression] = STATE(1855),
    [sym_binary_expression] = STATE(1855),
    [sym_unary_expression] = STATE(1855),
    [sym_update_expression] = STATE(1855),
    [sym_template_string] = STATE(1855),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4042),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4044),
    [sym_number] = ACTIONS(4042),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4044),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4044),
    [sym_false] = ACTIONS(4044),
    [sym_null] = ACTIONS(4044),
    [sym_undefined] = ACTIONS(4044),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1585] = {
    [sym__automatic_semicolon] = ACTIONS(4046),
    [anon_sym_SEMI] = ACTIONS(4046),
    [sym_comment] = ACTIONS(71),
  },
  [1586] = {
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_catch] = ACTIONS(1590),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [1587] = {
    [sym_export_statement] = STATE(1858),
    [sym__declaration] = STATE(1858),
    [sym_import_statement] = STATE(1858),
    [sym_expression_statement] = STATE(1858),
    [sym_variable_declaration] = STATE(1858),
    [sym_lexical_declaration] = STATE(1858),
    [sym_statement_block] = STATE(1858),
    [sym_if_statement] = STATE(1858),
    [sym_switch_statement] = STATE(1858),
    [sym_for_statement] = STATE(1858),
    [sym_for_in_statement] = STATE(1858),
    [sym_for_of_statement] = STATE(1858),
    [sym_while_statement] = STATE(1858),
    [sym_do_statement] = STATE(1858),
    [sym_try_statement] = STATE(1858),
    [sym_with_statement] = STATE(1858),
    [sym_break_statement] = STATE(1858),
    [sym_continue_statement] = STATE(1858),
    [sym_debugger_statement] = STATE(1858),
    [sym_return_statement] = STATE(1858),
    [sym_throw_statement] = STATE(1858),
    [sym_empty_statement] = STATE(1858),
    [sym_labeled_statement] = STATE(1858),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1858),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4048),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1588] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4050),
  },
  [1589] = {
    [ts_builtin_sym_end] = ACTIONS(2925),
    [anon_sym_export] = ACTIONS(2927),
    [anon_sym_LBRACE] = ACTIONS(2925),
    [anon_sym_RBRACE] = ACTIONS(2925),
    [anon_sym_import] = ACTIONS(2927),
    [anon_sym_var] = ACTIONS(2927),
    [anon_sym_let] = ACTIONS(2927),
    [anon_sym_const] = ACTIONS(2927),
    [anon_sym_if] = ACTIONS(2927),
    [anon_sym_else] = ACTIONS(2927),
    [anon_sym_switch] = ACTIONS(2927),
    [anon_sym_for] = ACTIONS(2927),
    [anon_sym_LPAREN] = ACTIONS(2925),
    [anon_sym_while] = ACTIONS(2927),
    [anon_sym_do] = ACTIONS(2927),
    [anon_sym_try] = ACTIONS(2927),
    [anon_sym_with] = ACTIONS(2927),
    [anon_sym_break] = ACTIONS(2927),
    [anon_sym_continue] = ACTIONS(2927),
    [anon_sym_debugger] = ACTIONS(2927),
    [anon_sym_return] = ACTIONS(2927),
    [anon_sym_throw] = ACTIONS(2927),
    [anon_sym_SEMI] = ACTIONS(2925),
    [anon_sym_finally] = ACTIONS(2927),
    [anon_sym_yield] = ACTIONS(2927),
    [anon_sym_LBRACK] = ACTIONS(2925),
    [anon_sym_LT] = ACTIONS(2925),
    [anon_sym_class] = ACTIONS(2927),
    [anon_sym_async] = ACTIONS(2927),
    [anon_sym_function] = ACTIONS(2927),
    [anon_sym_EQ_GT] = ACTIONS(2925),
    [anon_sym_new] = ACTIONS(2927),
    [anon_sym_await] = ACTIONS(2927),
    [anon_sym_PLUS] = ACTIONS(2927),
    [anon_sym_DASH] = ACTIONS(2927),
    [anon_sym_BANG] = ACTIONS(2925),
    [anon_sym_TILDE] = ACTIONS(2925),
    [anon_sym_typeof] = ACTIONS(2927),
    [anon_sym_void] = ACTIONS(2927),
    [anon_sym_delete] = ACTIONS(2927),
    [anon_sym_PLUS_PLUS] = ACTIONS(2925),
    [anon_sym_DASH_DASH] = ACTIONS(2925),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2925),
    [anon_sym_BQUOTE] = ACTIONS(2925),
    [sym_regex] = ACTIONS(2927),
    [sym_number] = ACTIONS(2925),
    [sym_identifier] = ACTIONS(2927),
    [sym_this] = ACTIONS(2927),
    [sym_super] = ACTIONS(2927),
    [sym_true] = ACTIONS(2927),
    [sym_false] = ACTIONS(2927),
    [sym_null] = ACTIONS(2927),
    [sym_undefined] = ACTIONS(2927),
    [anon_sym_AT] = ACTIONS(2925),
    [anon_sym_get] = ACTIONS(2927),
    [anon_sym_set] = ACTIONS(2927),
  },
  [1590] = {
    [sym_export_statement] = STATE(798),
    [sym__declaration] = STATE(798),
    [sym_import_statement] = STATE(798),
    [sym_expression_statement] = STATE(798),
    [sym_variable_declaration] = STATE(798),
    [sym_lexical_declaration] = STATE(798),
    [sym_statement_block] = STATE(798),
    [sym_if_statement] = STATE(798),
    [sym_switch_statement] = STATE(798),
    [sym_for_statement] = STATE(798),
    [sym_for_in_statement] = STATE(798),
    [sym_for_of_statement] = STATE(798),
    [sym_while_statement] = STATE(798),
    [sym_do_statement] = STATE(798),
    [sym_try_statement] = STATE(798),
    [sym_with_statement] = STATE(798),
    [sym_break_statement] = STATE(798),
    [sym_continue_statement] = STATE(798),
    [sym_debugger_statement] = STATE(798),
    [sym_return_statement] = STATE(798),
    [sym_throw_statement] = STATE(798),
    [sym_empty_statement] = STATE(798),
    [sym_labeled_statement] = STATE(798),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(798),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4052),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1591] = {
    [ts_builtin_sym_end] = ACTIONS(2931),
    [anon_sym_export] = ACTIONS(2933),
    [anon_sym_LBRACE] = ACTIONS(2931),
    [anon_sym_RBRACE] = ACTIONS(2931),
    [anon_sym_import] = ACTIONS(2933),
    [anon_sym_var] = ACTIONS(2933),
    [anon_sym_let] = ACTIONS(2933),
    [anon_sym_const] = ACTIONS(2933),
    [anon_sym_if] = ACTIONS(2933),
    [anon_sym_else] = ACTIONS(2933),
    [anon_sym_switch] = ACTIONS(2933),
    [anon_sym_for] = ACTIONS(2933),
    [anon_sym_LPAREN] = ACTIONS(2931),
    [anon_sym_while] = ACTIONS(2933),
    [anon_sym_do] = ACTIONS(2933),
    [anon_sym_try] = ACTIONS(2933),
    [anon_sym_with] = ACTIONS(2933),
    [anon_sym_break] = ACTIONS(2933),
    [anon_sym_continue] = ACTIONS(2933),
    [anon_sym_debugger] = ACTIONS(2933),
    [anon_sym_return] = ACTIONS(2933),
    [anon_sym_throw] = ACTIONS(2933),
    [anon_sym_SEMI] = ACTIONS(2931),
    [anon_sym_yield] = ACTIONS(2933),
    [anon_sym_LBRACK] = ACTIONS(2931),
    [anon_sym_LT] = ACTIONS(2931),
    [anon_sym_class] = ACTIONS(2933),
    [anon_sym_async] = ACTIONS(2933),
    [anon_sym_function] = ACTIONS(2933),
    [anon_sym_EQ_GT] = ACTIONS(2931),
    [anon_sym_new] = ACTIONS(2933),
    [anon_sym_await] = ACTIONS(2933),
    [anon_sym_PLUS] = ACTIONS(2933),
    [anon_sym_DASH] = ACTIONS(2933),
    [anon_sym_BANG] = ACTIONS(2931),
    [anon_sym_TILDE] = ACTIONS(2931),
    [anon_sym_typeof] = ACTIONS(2933),
    [anon_sym_void] = ACTIONS(2933),
    [anon_sym_delete] = ACTIONS(2933),
    [anon_sym_PLUS_PLUS] = ACTIONS(2931),
    [anon_sym_DASH_DASH] = ACTIONS(2931),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2931),
    [anon_sym_BQUOTE] = ACTIONS(2931),
    [sym_regex] = ACTIONS(2933),
    [sym_number] = ACTIONS(2931),
    [sym_identifier] = ACTIONS(2933),
    [sym_this] = ACTIONS(2933),
    [sym_super] = ACTIONS(2933),
    [sym_true] = ACTIONS(2933),
    [sym_false] = ACTIONS(2933),
    [sym_null] = ACTIONS(2933),
    [sym_undefined] = ACTIONS(2933),
    [anon_sym_AT] = ACTIONS(2931),
    [anon_sym_get] = ACTIONS(2933),
    [anon_sym_set] = ACTIONS(2933),
  },
  [1592] = {
    [ts_builtin_sym_end] = ACTIONS(2935),
    [anon_sym_export] = ACTIONS(2937),
    [anon_sym_LBRACE] = ACTIONS(2935),
    [anon_sym_RBRACE] = ACTIONS(2935),
    [anon_sym_import] = ACTIONS(2937),
    [anon_sym_var] = ACTIONS(2937),
    [anon_sym_let] = ACTIONS(2937),
    [anon_sym_const] = ACTIONS(2937),
    [anon_sym_if] = ACTIONS(2937),
    [anon_sym_else] = ACTIONS(2937),
    [anon_sym_switch] = ACTIONS(2937),
    [anon_sym_for] = ACTIONS(2937),
    [anon_sym_LPAREN] = ACTIONS(2935),
    [anon_sym_while] = ACTIONS(2937),
    [anon_sym_do] = ACTIONS(2937),
    [anon_sym_try] = ACTIONS(2937),
    [anon_sym_with] = ACTIONS(2937),
    [anon_sym_break] = ACTIONS(2937),
    [anon_sym_continue] = ACTIONS(2937),
    [anon_sym_debugger] = ACTIONS(2937),
    [anon_sym_return] = ACTIONS(2937),
    [anon_sym_throw] = ACTIONS(2937),
    [anon_sym_SEMI] = ACTIONS(2935),
    [anon_sym_yield] = ACTIONS(2937),
    [anon_sym_LBRACK] = ACTIONS(2935),
    [anon_sym_LT] = ACTIONS(2935),
    [anon_sym_class] = ACTIONS(2937),
    [anon_sym_async] = ACTIONS(2937),
    [anon_sym_function] = ACTIONS(2937),
    [anon_sym_EQ_GT] = ACTIONS(2935),
    [anon_sym_new] = ACTIONS(2937),
    [anon_sym_await] = ACTIONS(2937),
    [anon_sym_PLUS] = ACTIONS(2937),
    [anon_sym_DASH] = ACTIONS(2937),
    [anon_sym_BANG] = ACTIONS(2935),
    [anon_sym_TILDE] = ACTIONS(2935),
    [anon_sym_typeof] = ACTIONS(2937),
    [anon_sym_void] = ACTIONS(2937),
    [anon_sym_delete] = ACTIONS(2937),
    [anon_sym_PLUS_PLUS] = ACTIONS(2935),
    [anon_sym_DASH_DASH] = ACTIONS(2935),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2935),
    [anon_sym_BQUOTE] = ACTIONS(2935),
    [sym_regex] = ACTIONS(2937),
    [sym_number] = ACTIONS(2935),
    [sym_identifier] = ACTIONS(2937),
    [sym_this] = ACTIONS(2937),
    [sym_super] = ACTIONS(2937),
    [sym_true] = ACTIONS(2937),
    [sym_false] = ACTIONS(2937),
    [sym_null] = ACTIONS(2937),
    [sym_undefined] = ACTIONS(2937),
    [anon_sym_AT] = ACTIONS(2935),
    [anon_sym_get] = ACTIONS(2937),
    [anon_sym_set] = ACTIONS(2937),
  },
  [1593] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [ts_builtin_sym_end] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_STAR] = ACTIONS(2064),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_COMMA] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_else] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_in] = ACTIONS(2066),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2066),
    [anon_sym_GT] = ACTIONS(2066),
    [anon_sym_SLASH] = ACTIONS(2066),
    [anon_sym_DOT] = ACTIONS(2066),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_QMARK] = ACTIONS(2064),
    [anon_sym_AMP_AMP] = ACTIONS(2064),
    [anon_sym_PIPE_PIPE] = ACTIONS(2064),
    [anon_sym_GT_GT] = ACTIONS(2066),
    [anon_sym_GT_GT_GT] = ACTIONS(2064),
    [anon_sym_LT_LT] = ACTIONS(2064),
    [anon_sym_AMP] = ACTIONS(2066),
    [anon_sym_CARET] = ACTIONS(2064),
    [anon_sym_PIPE] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_PERCENT] = ACTIONS(2064),
    [anon_sym_LT_EQ] = ACTIONS(2064),
    [anon_sym_EQ_EQ] = ACTIONS(2066),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2064),
    [anon_sym_BANG_EQ] = ACTIONS(2066),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2064),
    [anon_sym_GT_EQ] = ACTIONS(2064),
    [anon_sym_instanceof] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2066),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [1594] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(4054),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1595] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [ts_builtin_sym_end] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_STAR] = ACTIONS(3103),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_COMMA] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_else] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_in] = ACTIONS(3105),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3105),
    [anon_sym_GT] = ACTIONS(3105),
    [anon_sym_SLASH] = ACTIONS(3105),
    [anon_sym_DOT] = ACTIONS(3105),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3103),
    [anon_sym_AMP_AMP] = ACTIONS(3103),
    [anon_sym_PIPE_PIPE] = ACTIONS(3103),
    [anon_sym_GT_GT] = ACTIONS(3105),
    [anon_sym_GT_GT_GT] = ACTIONS(3103),
    [anon_sym_LT_LT] = ACTIONS(3103),
    [anon_sym_AMP] = ACTIONS(3105),
    [anon_sym_CARET] = ACTIONS(3103),
    [anon_sym_PIPE] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_PERCENT] = ACTIONS(3103),
    [anon_sym_LT_EQ] = ACTIONS(3103),
    [anon_sym_EQ_EQ] = ACTIONS(3105),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3103),
    [anon_sym_BANG_EQ] = ACTIONS(3105),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3103),
    [anon_sym_GT_EQ] = ACTIONS(3103),
    [anon_sym_instanceof] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3105),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [1596] = {
    [sym_statement_block] = STATE(1862),
    [anon_sym_LBRACE] = ACTIONS(2684),
    [sym_comment] = ACTIONS(71),
  },
  [1597] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [ts_builtin_sym_end] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_STAR] = ACTIONS(3107),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_COMMA] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_else] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_in] = ACTIONS(3109),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3109),
    [anon_sym_GT] = ACTIONS(3109),
    [anon_sym_SLASH] = ACTIONS(3109),
    [anon_sym_DOT] = ACTIONS(3109),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_AMP_AMP] = ACTIONS(3107),
    [anon_sym_PIPE_PIPE] = ACTIONS(3107),
    [anon_sym_GT_GT] = ACTIONS(3109),
    [anon_sym_GT_GT_GT] = ACTIONS(3107),
    [anon_sym_LT_LT] = ACTIONS(3107),
    [anon_sym_AMP] = ACTIONS(3109),
    [anon_sym_CARET] = ACTIONS(3107),
    [anon_sym_PIPE] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_PERCENT] = ACTIONS(3107),
    [anon_sym_LT_EQ] = ACTIONS(3107),
    [anon_sym_EQ_EQ] = ACTIONS(3109),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3107),
    [anon_sym_BANG_EQ] = ACTIONS(3109),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3107),
    [anon_sym_GT_EQ] = ACTIONS(3107),
    [anon_sym_instanceof] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3109),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [1598] = {
    [sym_statement_block] = STATE(1863),
    [anon_sym_LBRACE] = ACTIONS(2684),
    [sym_comment] = ACTIONS(71),
  },
  [1599] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [ts_builtin_sym_end] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_STAR] = ACTIONS(3115),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_COMMA] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_else] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_in] = ACTIONS(3117),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3117),
    [anon_sym_GT] = ACTIONS(3117),
    [anon_sym_SLASH] = ACTIONS(3117),
    [anon_sym_DOT] = ACTIONS(3117),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_QMARK] = ACTIONS(3115),
    [anon_sym_AMP_AMP] = ACTIONS(3115),
    [anon_sym_PIPE_PIPE] = ACTIONS(3115),
    [anon_sym_GT_GT] = ACTIONS(3117),
    [anon_sym_GT_GT_GT] = ACTIONS(3115),
    [anon_sym_LT_LT] = ACTIONS(3115),
    [anon_sym_AMP] = ACTIONS(3117),
    [anon_sym_CARET] = ACTIONS(3115),
    [anon_sym_PIPE] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_PERCENT] = ACTIONS(3115),
    [anon_sym_LT_EQ] = ACTIONS(3115),
    [anon_sym_EQ_EQ] = ACTIONS(3117),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3115),
    [anon_sym_BANG_EQ] = ACTIONS(3117),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3115),
    [anon_sym_GT_EQ] = ACTIONS(3115),
    [anon_sym_instanceof] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3117),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [1600] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_in] = ACTIONS(484),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(484),
    [anon_sym_GT] = ACTIONS(484),
    [anon_sym_SLASH] = ACTIONS(484),
    [anon_sym_DOT] = ACTIONS(484),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(482),
    [anon_sym_AMP_AMP] = ACTIONS(482),
    [anon_sym_PIPE_PIPE] = ACTIONS(482),
    [anon_sym_GT_GT] = ACTIONS(484),
    [anon_sym_GT_GT_GT] = ACTIONS(482),
    [anon_sym_LT_LT] = ACTIONS(482),
    [anon_sym_AMP] = ACTIONS(484),
    [anon_sym_CARET] = ACTIONS(482),
    [anon_sym_PIPE] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_PERCENT] = ACTIONS(482),
    [anon_sym_LT_EQ] = ACTIONS(482),
    [anon_sym_EQ_EQ] = ACTIONS(484),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(482),
    [anon_sym_BANG_EQ] = ACTIONS(484),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(482),
    [anon_sym_GT_EQ] = ACTIONS(482),
    [anon_sym_instanceof] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1601] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4056),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1602] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(4058),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(4058),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1603] = {
    [sym_class_body] = STATE(1866),
    [anon_sym_LBRACE] = ACTIONS(2678),
    [sym_comment] = ACTIONS(71),
  },
  [1604] = {
    [sym_export_clause] = STATE(1872),
    [sym__declaration] = STATE(1873),
    [sym_variable_declaration] = STATE(1873),
    [sym_lexical_declaration] = STATE(1873),
    [sym_class] = STATE(1874),
    [sym_function] = STATE(1874),
    [sym_generator_function] = STATE(1874),
    [sym_decorator] = STATE(1875),
    [aux_sym_export_statement_repeat1] = STATE(1875),
    [anon_sym_STAR] = ACTIONS(4060),
    [anon_sym_default] = ACTIONS(4062),
    [anon_sym_LBRACE] = ACTIONS(93),
    [anon_sym_var] = ACTIONS(4064),
    [anon_sym_let] = ACTIONS(4066),
    [anon_sym_const] = ACTIONS(4066),
    [anon_sym_class] = ACTIONS(4068),
    [anon_sym_async] = ACTIONS(4070),
    [anon_sym_function] = ACTIONS(4072),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [1605] = {
    [sym_export_statement] = STATE(1877),
    [sym__declaration] = STATE(1877),
    [sym_import_statement] = STATE(1877),
    [sym_expression_statement] = STATE(1877),
    [sym_variable_declaration] = STATE(1877),
    [sym_lexical_declaration] = STATE(1877),
    [sym_statement_block] = STATE(1877),
    [sym_if_statement] = STATE(1877),
    [sym_switch_statement] = STATE(1877),
    [sym_for_statement] = STATE(1877),
    [sym_for_in_statement] = STATE(1877),
    [sym_for_of_statement] = STATE(1877),
    [sym_while_statement] = STATE(1877),
    [sym_do_statement] = STATE(1877),
    [sym_try_statement] = STATE(1877),
    [sym_with_statement] = STATE(1877),
    [sym_break_statement] = STATE(1877),
    [sym_continue_statement] = STATE(1877),
    [sym_debugger_statement] = STATE(1877),
    [sym_return_statement] = STATE(1877),
    [sym_throw_statement] = STATE(1877),
    [sym_empty_statement] = STATE(1877),
    [sym_labeled_statement] = STATE(1877),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(1877),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(4074),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [1606] = {
    [sym_import_clause] = STATE(1879),
    [sym_namespace_import] = STATE(87),
    [sym_named_imports] = STATE(87),
    [anon_sym_STAR] = ACTIONS(133),
    [anon_sym_LBRACE] = ACTIONS(135),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4076),
    [sym_identifier] = ACTIONS(139),
  },
  [1607] = {
    [sym_variable_declarator] = STATE(1880),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [1608] = {
    [sym_variable_declarator] = STATE(1881),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [1609] = {
    [sym_parenthesized_expression] = STATE(1882),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [1610] = {
    [sym_parenthesized_expression] = STATE(1883),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [1611] = {
    [anon_sym_LPAREN] = ACTIONS(4078),
    [sym_comment] = ACTIONS(71),
  },
  [1612] = {
    [sym_parenthesized_expression] = STATE(1885),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [1613] = {
    [sym_export_statement] = STATE(1886),
    [sym__declaration] = STATE(1886),
    [sym_import_statement] = STATE(1886),
    [sym_expression_statement] = STATE(1886),
    [sym_variable_declaration] = STATE(1886),
    [sym_lexical_declaration] = STATE(1886),
    [sym_statement_block] = STATE(1886),
    [sym_if_statement] = STATE(1886),
    [sym_switch_statement] = STATE(1886),
    [sym_for_statement] = STATE(1886),
    [sym_for_in_statement] = STATE(1886),
    [sym_for_of_statement] = STATE(1886),
    [sym_while_statement] = STATE(1886),
    [sym_do_statement] = STATE(1886),
    [sym_try_statement] = STATE(1886),
    [sym_with_statement] = STATE(1886),
    [sym_break_statement] = STATE(1886),
    [sym_continue_statement] = STATE(1886),
    [sym_debugger_statement] = STATE(1886),
    [sym_return_statement] = STATE(1886),
    [sym_throw_statement] = STATE(1886),
    [sym_empty_statement] = STATE(1886),
    [sym_labeled_statement] = STATE(1886),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1614] = {
    [sym_statement_block] = STATE(1888),
    [anon_sym_LBRACE] = ACTIONS(4080),
    [sym_comment] = ACTIONS(71),
  },
  [1615] = {
    [sym_parenthesized_expression] = STATE(1889),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [1616] = {
    [sym__automatic_semicolon] = ACTIONS(4082),
    [anon_sym_SEMI] = ACTIONS(4082),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4084),
  },
  [1617] = {
    [sym__automatic_semicolon] = ACTIONS(4086),
    [anon_sym_SEMI] = ACTIONS(4086),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4088),
  },
  [1618] = {
    [sym__automatic_semicolon] = ACTIONS(4090),
    [anon_sym_SEMI] = ACTIONS(4090),
    [sym_comment] = ACTIONS(71),
  },
  [1619] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1896),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1896),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(4092),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(4092),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1620] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1897),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1897),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1621] = {
    [anon_sym_export] = ACTIONS(237),
    [anon_sym_default] = ACTIONS(237),
    [anon_sym_LBRACE] = ACTIONS(235),
    [anon_sym_RBRACE] = ACTIONS(235),
    [anon_sym_import] = ACTIONS(237),
    [anon_sym_var] = ACTIONS(237),
    [anon_sym_let] = ACTIONS(237),
    [anon_sym_const] = ACTIONS(237),
    [anon_sym_if] = ACTIONS(237),
    [anon_sym_switch] = ACTIONS(237),
    [anon_sym_for] = ACTIONS(237),
    [anon_sym_LPAREN] = ACTIONS(235),
    [anon_sym_while] = ACTIONS(237),
    [anon_sym_do] = ACTIONS(237),
    [anon_sym_try] = ACTIONS(237),
    [anon_sym_with] = ACTIONS(237),
    [anon_sym_break] = ACTIONS(237),
    [anon_sym_continue] = ACTIONS(237),
    [anon_sym_debugger] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_throw] = ACTIONS(237),
    [anon_sym_SEMI] = ACTIONS(235),
    [anon_sym_case] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(237),
    [anon_sym_LBRACK] = ACTIONS(235),
    [anon_sym_LT] = ACTIONS(235),
    [anon_sym_class] = ACTIONS(237),
    [anon_sym_async] = ACTIONS(237),
    [anon_sym_function] = ACTIONS(237),
    [anon_sym_EQ_GT] = ACTIONS(235),
    [anon_sym_new] = ACTIONS(237),
    [anon_sym_await] = ACTIONS(237),
    [anon_sym_PLUS] = ACTIONS(237),
    [anon_sym_DASH] = ACTIONS(237),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_TILDE] = ACTIONS(235),
    [anon_sym_typeof] = ACTIONS(237),
    [anon_sym_void] = ACTIONS(237),
    [anon_sym_delete] = ACTIONS(237),
    [anon_sym_PLUS_PLUS] = ACTIONS(235),
    [anon_sym_DASH_DASH] = ACTIONS(235),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(235),
    [anon_sym_BQUOTE] = ACTIONS(235),
    [sym_regex] = ACTIONS(237),
    [sym_number] = ACTIONS(235),
    [sym_identifier] = ACTIONS(237),
    [sym_this] = ACTIONS(237),
    [sym_super] = ACTIONS(237),
    [sym_true] = ACTIONS(237),
    [sym_false] = ACTIONS(237),
    [sym_null] = ACTIONS(237),
    [sym_undefined] = ACTIONS(237),
    [anon_sym_AT] = ACTIONS(235),
    [anon_sym_get] = ACTIONS(237),
    [anon_sym_set] = ACTIONS(237),
  },
  [1622] = {
    [sym_class_heritage] = STATE(187),
    [sym_class_body] = STATE(188),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4094),
  },
  [1623] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_function] = ACTIONS(4096),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(308),
  },
  [1624] = {
    [sym_formal_parameters] = STATE(1902),
    [anon_sym_STAR] = ACTIONS(4098),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4100),
  },
  [1625] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(4102),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1626] = {
    [sym__automatic_semicolon] = ACTIONS(4104),
    [anon_sym_SEMI] = ACTIONS(4104),
    [sym_comment] = ACTIONS(71),
  },
  [1627] = {
    [sym__automatic_semicolon] = ACTIONS(4106),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_default] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_case] = ACTIONS(402),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [1628] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(4106),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_default] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_case] = ACTIONS(402),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [1629] = {
    [sym_export_statement] = STATE(1906),
    [sym__declaration] = STATE(1906),
    [sym_import_statement] = STATE(1906),
    [sym_expression_statement] = STATE(1906),
    [sym_variable_declaration] = STATE(1906),
    [sym_lexical_declaration] = STATE(1906),
    [sym_statement_block] = STATE(1906),
    [sym_if_statement] = STATE(1906),
    [sym_switch_statement] = STATE(1906),
    [sym_for_statement] = STATE(1906),
    [sym_for_in_statement] = STATE(1906),
    [sym_for_of_statement] = STATE(1906),
    [sym_while_statement] = STATE(1906),
    [sym_do_statement] = STATE(1906),
    [sym_try_statement] = STATE(1906),
    [sym_with_statement] = STATE(1906),
    [sym_break_statement] = STATE(1906),
    [sym_continue_statement] = STATE(1906),
    [sym_debugger_statement] = STATE(1906),
    [sym_return_statement] = STATE(1906),
    [sym_throw_statement] = STATE(1906),
    [sym_empty_statement] = STATE(1906),
    [sym_labeled_statement] = STATE(1906),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1906),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_default] = ACTIONS(4108),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_RBRACE] = ACTIONS(4110),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_case] = ACTIONS(4108),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1630] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(4112),
    [anon_sym_class] = ACTIONS(4114),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [1631] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(882),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1632] = {
    [sym_parenthesized_expression] = STATE(1909),
    [sym__expression] = STATE(1909),
    [sym_yield_expression] = STATE(1909),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1909),
    [sym_jsx_fragment] = STATE(1909),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1909),
    [sym_anonymous_class] = STATE(1909),
    [sym_class] = STATE(1909),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1909),
    [sym_generator_function] = STATE(1909),
    [sym_call_expression] = STATE(1909),
    [sym_new_expression] = STATE(1909),
    [sym_await_expression] = STATE(1909),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1909),
    [sym_augmented_assignment_expression] = STATE(1909),
    [sym_ternary_expression] = STATE(1909),
    [sym_binary_expression] = STATE(1909),
    [sym_unary_expression] = STATE(1909),
    [sym_update_expression] = STATE(1909),
    [sym_template_string] = STATE(1909),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4116),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4118),
    [sym_number] = ACTIONS(4116),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4118),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4118),
    [sym_false] = ACTIONS(4118),
    [sym_null] = ACTIONS(4118),
    [sym_undefined] = ACTIONS(4118),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1633] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1910),
    [sym__expression] = STATE(1910),
    [sym_yield_expression] = STATE(1910),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1910),
    [sym_jsx_fragment] = STATE(1910),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1910),
    [sym_anonymous_class] = STATE(1910),
    [sym_class] = STATE(1910),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1910),
    [sym_generator_function] = STATE(1910),
    [sym_call_expression] = STATE(1910),
    [sym_new_expression] = STATE(1910),
    [sym_await_expression] = STATE(1910),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1910),
    [sym_augmented_assignment_expression] = STATE(1910),
    [sym_ternary_expression] = STATE(1910),
    [sym_binary_expression] = STATE(1910),
    [sym_unary_expression] = STATE(1910),
    [sym_update_expression] = STATE(1910),
    [sym_template_string] = STATE(1910),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4120),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4122),
    [sym_number] = ACTIONS(4120),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4122),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4122),
    [sym_false] = ACTIONS(4122),
    [sym_null] = ACTIONS(4122),
    [sym_undefined] = ACTIONS(4122),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1634] = {
    [sym_parenthesized_expression] = STATE(1911),
    [sym__expression] = STATE(1911),
    [sym_yield_expression] = STATE(1911),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1911),
    [sym_jsx_fragment] = STATE(1911),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1911),
    [sym_anonymous_class] = STATE(1911),
    [sym_class] = STATE(1911),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1911),
    [sym_generator_function] = STATE(1911),
    [sym_call_expression] = STATE(1911),
    [sym_new_expression] = STATE(1911),
    [sym_await_expression] = STATE(1911),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1911),
    [sym_augmented_assignment_expression] = STATE(1911),
    [sym_ternary_expression] = STATE(1911),
    [sym_binary_expression] = STATE(1911),
    [sym_unary_expression] = STATE(1911),
    [sym_update_expression] = STATE(1911),
    [sym_template_string] = STATE(1911),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4124),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4126),
    [sym_number] = ACTIONS(4124),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4126),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4126),
    [sym_false] = ACTIONS(4126),
    [sym_null] = ACTIONS(4126),
    [sym_undefined] = ACTIONS(4126),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1635] = {
    [anon_sym_EQ_GT] = ACTIONS(4128),
    [sym_comment] = ACTIONS(71),
  },
  [1636] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1637] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(1056),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1638] = {
    [sym_parenthesized_expression] = STATE(1913),
    [sym__expression] = STATE(1913),
    [sym_yield_expression] = STATE(1913),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1913),
    [sym_jsx_fragment] = STATE(1913),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1913),
    [sym_anonymous_class] = STATE(1913),
    [sym_class] = STATE(1913),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1913),
    [sym_generator_function] = STATE(1913),
    [sym_call_expression] = STATE(1913),
    [sym_new_expression] = STATE(1913),
    [sym_await_expression] = STATE(1913),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1913),
    [sym_augmented_assignment_expression] = STATE(1913),
    [sym_ternary_expression] = STATE(1913),
    [sym_binary_expression] = STATE(1913),
    [sym_unary_expression] = STATE(1913),
    [sym_update_expression] = STATE(1913),
    [sym_template_string] = STATE(1913),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4130),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4132),
    [sym_number] = ACTIONS(4130),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4132),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4132),
    [sym_false] = ACTIONS(4132),
    [sym_null] = ACTIONS(4132),
    [sym_undefined] = ACTIONS(4132),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1639] = {
    [sym_parenthesized_expression] = STATE(1914),
    [sym__expression] = STATE(1914),
    [sym_yield_expression] = STATE(1914),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1914),
    [sym_jsx_fragment] = STATE(1914),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1914),
    [sym_anonymous_class] = STATE(1914),
    [sym_class] = STATE(1914),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1914),
    [sym_generator_function] = STATE(1914),
    [sym_call_expression] = STATE(1914),
    [sym_new_expression] = STATE(1914),
    [sym_await_expression] = STATE(1914),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1914),
    [sym_augmented_assignment_expression] = STATE(1914),
    [sym_ternary_expression] = STATE(1914),
    [sym_binary_expression] = STATE(1914),
    [sym_unary_expression] = STATE(1914),
    [sym_update_expression] = STATE(1914),
    [sym_template_string] = STATE(1914),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4134),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4136),
    [sym_number] = ACTIONS(4134),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4136),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4136),
    [sym_false] = ACTIONS(4136),
    [sym_null] = ACTIONS(4136),
    [sym_undefined] = ACTIONS(4136),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1640] = {
    [sym_export_statement] = STATE(1915),
    [sym__declaration] = STATE(1915),
    [sym_import_statement] = STATE(1915),
    [sym_expression_statement] = STATE(1915),
    [sym_variable_declaration] = STATE(1915),
    [sym_lexical_declaration] = STATE(1915),
    [sym_statement_block] = STATE(1915),
    [sym_if_statement] = STATE(1915),
    [sym_switch_statement] = STATE(1915),
    [sym_for_statement] = STATE(1915),
    [sym_for_in_statement] = STATE(1915),
    [sym_for_of_statement] = STATE(1915),
    [sym_while_statement] = STATE(1915),
    [sym_do_statement] = STATE(1915),
    [sym_try_statement] = STATE(1915),
    [sym_with_statement] = STATE(1915),
    [sym_break_statement] = STATE(1915),
    [sym_continue_statement] = STATE(1915),
    [sym_debugger_statement] = STATE(1915),
    [sym_return_statement] = STATE(1915),
    [sym_throw_statement] = STATE(1915),
    [sym_empty_statement] = STATE(1915),
    [sym_labeled_statement] = STATE(1915),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1915),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_default] = ACTIONS(4138),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_RBRACE] = ACTIONS(4140),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_case] = ACTIONS(4138),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1641] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1642] = {
    [sym_parenthesized_expression] = STATE(1916),
    [sym__expression] = STATE(1916),
    [sym_yield_expression] = STATE(1916),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1916),
    [sym_jsx_fragment] = STATE(1916),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1916),
    [sym_anonymous_class] = STATE(1916),
    [sym_class] = STATE(1916),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1916),
    [sym_generator_function] = STATE(1916),
    [sym_call_expression] = STATE(1916),
    [sym_new_expression] = STATE(1916),
    [sym_await_expression] = STATE(1916),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1916),
    [sym_augmented_assignment_expression] = STATE(1916),
    [sym_ternary_expression] = STATE(1916),
    [sym_binary_expression] = STATE(1916),
    [sym_unary_expression] = STATE(1916),
    [sym_update_expression] = STATE(1916),
    [sym_sequence_expression] = STATE(902),
    [sym_template_string] = STATE(1916),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4142),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4144),
    [sym_number] = ACTIONS(4142),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4144),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4144),
    [sym_false] = ACTIONS(4144),
    [sym_null] = ACTIONS(4144),
    [sym_undefined] = ACTIONS(4144),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1643] = {
    [sym_parenthesized_expression] = STATE(1917),
    [sym__expression] = STATE(1917),
    [sym_yield_expression] = STATE(1917),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1917),
    [sym_jsx_fragment] = STATE(1917),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1917),
    [sym_anonymous_class] = STATE(1917),
    [sym_class] = STATE(1917),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1917),
    [sym_generator_function] = STATE(1917),
    [sym_call_expression] = STATE(1917),
    [sym_new_expression] = STATE(1917),
    [sym_await_expression] = STATE(1917),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1917),
    [sym_augmented_assignment_expression] = STATE(1917),
    [sym_ternary_expression] = STATE(1917),
    [sym_binary_expression] = STATE(1917),
    [sym_unary_expression] = STATE(1917),
    [sym_update_expression] = STATE(1917),
    [sym_template_string] = STATE(1917),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4146),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4148),
    [sym_number] = ACTIONS(4146),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(4148),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4148),
    [sym_false] = ACTIONS(4148),
    [sym_null] = ACTIONS(4148),
    [sym_undefined] = ACTIONS(4148),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1644] = {
    [sym_parenthesized_expression] = STATE(1918),
    [sym__expression] = STATE(1918),
    [sym_yield_expression] = STATE(1918),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1918),
    [sym_jsx_fragment] = STATE(1918),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1918),
    [sym_anonymous_class] = STATE(1918),
    [sym_class] = STATE(1918),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1918),
    [sym_generator_function] = STATE(1918),
    [sym_call_expression] = STATE(1918),
    [sym_new_expression] = STATE(1918),
    [sym_await_expression] = STATE(1918),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1918),
    [sym_augmented_assignment_expression] = STATE(1918),
    [sym_ternary_expression] = STATE(1918),
    [sym_binary_expression] = STATE(1918),
    [sym_unary_expression] = STATE(1918),
    [sym_update_expression] = STATE(1918),
    [sym_template_string] = STATE(1918),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4150),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4152),
    [sym_number] = ACTIONS(4150),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4152),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4152),
    [sym_false] = ACTIONS(4152),
    [sym_null] = ACTIONS(4152),
    [sym_undefined] = ACTIONS(4152),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1645] = {
    [sym_parenthesized_expression] = STATE(1919),
    [sym__expression] = STATE(1919),
    [sym_yield_expression] = STATE(1919),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1919),
    [sym_jsx_fragment] = STATE(1919),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1919),
    [sym_anonymous_class] = STATE(1919),
    [sym_class] = STATE(1919),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1919),
    [sym_generator_function] = STATE(1919),
    [sym_call_expression] = STATE(1919),
    [sym_new_expression] = STATE(1919),
    [sym_await_expression] = STATE(1919),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1919),
    [sym_augmented_assignment_expression] = STATE(1919),
    [sym_ternary_expression] = STATE(1919),
    [sym_binary_expression] = STATE(1919),
    [sym_unary_expression] = STATE(1919),
    [sym_update_expression] = STATE(1919),
    [sym_template_string] = STATE(1919),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4154),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4156),
    [sym_number] = ACTIONS(4154),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4156),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4156),
    [sym_false] = ACTIONS(4156),
    [sym_null] = ACTIONS(4156),
    [sym_undefined] = ACTIONS(4156),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1646] = {
    [sym_parenthesized_expression] = STATE(1920),
    [sym__expression] = STATE(1920),
    [sym_yield_expression] = STATE(1920),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(1920),
    [sym_jsx_fragment] = STATE(1920),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1920),
    [sym_anonymous_class] = STATE(1920),
    [sym_class] = STATE(1920),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1920),
    [sym_generator_function] = STATE(1920),
    [sym_call_expression] = STATE(1920),
    [sym_new_expression] = STATE(1920),
    [sym_await_expression] = STATE(1920),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(1920),
    [sym_augmented_assignment_expression] = STATE(1920),
    [sym_ternary_expression] = STATE(1920),
    [sym_binary_expression] = STATE(1920),
    [sym_unary_expression] = STATE(1920),
    [sym_update_expression] = STATE(1920),
    [sym_template_string] = STATE(1920),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4158),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4160),
    [sym_number] = ACTIONS(4158),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4160),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4160),
    [sym_false] = ACTIONS(4160),
    [sym_null] = ACTIONS(4160),
    [sym_undefined] = ACTIONS(4160),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1647] = {
    [anon_sym_RPAREN] = ACTIONS(4162),
    [sym_comment] = ACTIONS(71),
  },
  [1648] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4164),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1649] = {
    [anon_sym_LBRACE] = ACTIONS(2619),
    [anon_sym_LPAREN] = ACTIONS(2619),
    [anon_sym_SEMI] = ACTIONS(2619),
    [anon_sym_yield] = ACTIONS(2621),
    [anon_sym_LBRACK] = ACTIONS(2619),
    [anon_sym_LT] = ACTIONS(2619),
    [anon_sym_class] = ACTIONS(2621),
    [anon_sym_async] = ACTIONS(2621),
    [anon_sym_function] = ACTIONS(2621),
    [anon_sym_EQ_GT] = ACTIONS(2619),
    [anon_sym_new] = ACTIONS(2621),
    [anon_sym_await] = ACTIONS(2621),
    [anon_sym_PLUS] = ACTIONS(2621),
    [anon_sym_DASH] = ACTIONS(2621),
    [anon_sym_BANG] = ACTIONS(2619),
    [anon_sym_TILDE] = ACTIONS(2619),
    [anon_sym_typeof] = ACTIONS(2621),
    [anon_sym_void] = ACTIONS(2621),
    [anon_sym_delete] = ACTIONS(2621),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2619),
    [anon_sym_BQUOTE] = ACTIONS(2619),
    [sym_regex] = ACTIONS(2621),
    [sym_number] = ACTIONS(2619),
    [sym_identifier] = ACTIONS(2621),
    [sym_this] = ACTIONS(2621),
    [sym_super] = ACTIONS(2621),
    [sym_true] = ACTIONS(2621),
    [sym_false] = ACTIONS(2621),
    [sym_null] = ACTIONS(2621),
    [sym_undefined] = ACTIONS(2621),
    [anon_sym_AT] = ACTIONS(2619),
    [anon_sym_get] = ACTIONS(2621),
    [anon_sym_set] = ACTIONS(2621),
  },
  [1650] = {
    [anon_sym_RPAREN] = ACTIONS(4166),
    [sym_comment] = ACTIONS(71),
  },
  [1651] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4168),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1652] = {
    [anon_sym_RPAREN] = ACTIONS(4170),
    [sym_comment] = ACTIONS(71),
  },
  [1653] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4172),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1654] = {
    [anon_sym_LBRACE] = ACTIONS(2626),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_SEMI] = ACTIONS(2626),
    [anon_sym_yield] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2628),
    [anon_sym_async] = ACTIONS(2628),
    [anon_sym_function] = ACTIONS(2628),
    [anon_sym_EQ_GT] = ACTIONS(2626),
    [anon_sym_new] = ACTIONS(2628),
    [anon_sym_await] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_DASH] = ACTIONS(2628),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2626),
    [anon_sym_typeof] = ACTIONS(2628),
    [anon_sym_void] = ACTIONS(2628),
    [anon_sym_delete] = ACTIONS(2628),
    [anon_sym_PLUS_PLUS] = ACTIONS(2626),
    [anon_sym_DASH_DASH] = ACTIONS(2626),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2626),
    [anon_sym_BQUOTE] = ACTIONS(2626),
    [sym_regex] = ACTIONS(2628),
    [sym_number] = ACTIONS(2626),
    [sym_identifier] = ACTIONS(2628),
    [sym_this] = ACTIONS(2628),
    [sym_super] = ACTIONS(2628),
    [sym_true] = ACTIONS(2628),
    [sym_false] = ACTIONS(2628),
    [sym_null] = ACTIONS(2628),
    [sym_undefined] = ACTIONS(2628),
    [anon_sym_AT] = ACTIONS(2626),
    [anon_sym_get] = ACTIONS(2628),
    [anon_sym_set] = ACTIONS(2628),
  },
  [1655] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(882),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(882),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(882),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(882),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1656] = {
    [sym_parenthesized_expression] = STATE(1927),
    [sym__expression] = STATE(1927),
    [sym_yield_expression] = STATE(1927),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1927),
    [sym_jsx_fragment] = STATE(1927),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1927),
    [sym_anonymous_class] = STATE(1927),
    [sym_class] = STATE(1927),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1927),
    [sym_generator_function] = STATE(1927),
    [sym_call_expression] = STATE(1927),
    [sym_new_expression] = STATE(1927),
    [sym_await_expression] = STATE(1927),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1927),
    [sym_augmented_assignment_expression] = STATE(1927),
    [sym_ternary_expression] = STATE(1927),
    [sym_binary_expression] = STATE(1927),
    [sym_unary_expression] = STATE(1927),
    [sym_update_expression] = STATE(1927),
    [sym_template_string] = STATE(1927),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4188),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4190),
    [sym_number] = ACTIONS(4188),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4190),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4190),
    [sym_false] = ACTIONS(4190),
    [sym_null] = ACTIONS(4190),
    [sym_undefined] = ACTIONS(4190),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1657] = {
    [sym_statement_block] = STATE(584),
    [sym_parenthesized_expression] = STATE(1928),
    [sym__expression] = STATE(1928),
    [sym_yield_expression] = STATE(1928),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1928),
    [sym_jsx_fragment] = STATE(1928),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1928),
    [sym_anonymous_class] = STATE(1928),
    [sym_class] = STATE(1928),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1928),
    [sym_generator_function] = STATE(1928),
    [sym_call_expression] = STATE(1928),
    [sym_new_expression] = STATE(1928),
    [sym_await_expression] = STATE(1928),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1928),
    [sym_augmented_assignment_expression] = STATE(1928),
    [sym_ternary_expression] = STATE(1928),
    [sym_binary_expression] = STATE(1928),
    [sym_unary_expression] = STATE(1928),
    [sym_update_expression] = STATE(1928),
    [sym_template_string] = STATE(1928),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4192),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4194),
    [sym_number] = ACTIONS(4192),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4194),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4194),
    [sym_false] = ACTIONS(4194),
    [sym_null] = ACTIONS(4194),
    [sym_undefined] = ACTIONS(4194),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1658] = {
    [sym_parenthesized_expression] = STATE(1929),
    [sym__expression] = STATE(1929),
    [sym_yield_expression] = STATE(1929),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1929),
    [sym_jsx_fragment] = STATE(1929),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1929),
    [sym_anonymous_class] = STATE(1929),
    [sym_class] = STATE(1929),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1929),
    [sym_generator_function] = STATE(1929),
    [sym_call_expression] = STATE(1929),
    [sym_new_expression] = STATE(1929),
    [sym_await_expression] = STATE(1929),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1929),
    [sym_augmented_assignment_expression] = STATE(1929),
    [sym_ternary_expression] = STATE(1929),
    [sym_binary_expression] = STATE(1929),
    [sym_unary_expression] = STATE(1929),
    [sym_update_expression] = STATE(1929),
    [sym_template_string] = STATE(1929),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4196),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4198),
    [sym_number] = ACTIONS(4196),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4198),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4198),
    [sym_false] = ACTIONS(4198),
    [sym_null] = ACTIONS(4198),
    [sym_undefined] = ACTIONS(4198),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1659] = {
    [anon_sym_EQ_GT] = ACTIONS(4200),
    [sym_comment] = ACTIONS(71),
  },
  [1660] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1048),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(1048),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(1048),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(1048),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1661] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(1056),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(1056),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(1056),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(1056),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1662] = {
    [sym_parenthesized_expression] = STATE(1931),
    [sym__expression] = STATE(1931),
    [sym_yield_expression] = STATE(1931),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1931),
    [sym_jsx_fragment] = STATE(1931),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1931),
    [sym_anonymous_class] = STATE(1931),
    [sym_class] = STATE(1931),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1931),
    [sym_generator_function] = STATE(1931),
    [sym_call_expression] = STATE(1931),
    [sym_new_expression] = STATE(1931),
    [sym_await_expression] = STATE(1931),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1931),
    [sym_augmented_assignment_expression] = STATE(1931),
    [sym_ternary_expression] = STATE(1931),
    [sym_binary_expression] = STATE(1931),
    [sym_unary_expression] = STATE(1931),
    [sym_update_expression] = STATE(1931),
    [sym_template_string] = STATE(1931),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4202),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4204),
    [sym_number] = ACTIONS(4202),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4204),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4204),
    [sym_false] = ACTIONS(4204),
    [sym_null] = ACTIONS(4204),
    [sym_undefined] = ACTIONS(4204),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1663] = {
    [sym_parenthesized_expression] = STATE(1932),
    [sym__expression] = STATE(1932),
    [sym_yield_expression] = STATE(1932),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1932),
    [sym_jsx_fragment] = STATE(1932),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1932),
    [sym_anonymous_class] = STATE(1932),
    [sym_class] = STATE(1932),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1932),
    [sym_generator_function] = STATE(1932),
    [sym_call_expression] = STATE(1932),
    [sym_new_expression] = STATE(1932),
    [sym_await_expression] = STATE(1932),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1932),
    [sym_augmented_assignment_expression] = STATE(1932),
    [sym_ternary_expression] = STATE(1932),
    [sym_binary_expression] = STATE(1932),
    [sym_unary_expression] = STATE(1932),
    [sym_update_expression] = STATE(1932),
    [sym_template_string] = STATE(1932),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4206),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4208),
    [sym_number] = ACTIONS(4206),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4208),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4208),
    [sym_false] = ACTIONS(4208),
    [sym_null] = ACTIONS(4208),
    [sym_undefined] = ACTIONS(4208),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1664] = {
    [sym_export_statement] = STATE(1933),
    [sym__declaration] = STATE(1933),
    [sym_import_statement] = STATE(1933),
    [sym_expression_statement] = STATE(1933),
    [sym_variable_declaration] = STATE(1933),
    [sym_lexical_declaration] = STATE(1933),
    [sym_statement_block] = STATE(1933),
    [sym_if_statement] = STATE(1933),
    [sym_switch_statement] = STATE(1933),
    [sym_for_statement] = STATE(1933),
    [sym_for_in_statement] = STATE(1933),
    [sym_for_of_statement] = STATE(1933),
    [sym_while_statement] = STATE(1933),
    [sym_do_statement] = STATE(1933),
    [sym_try_statement] = STATE(1933),
    [sym_with_statement] = STATE(1933),
    [sym_break_statement] = STATE(1933),
    [sym_continue_statement] = STATE(1933),
    [sym_debugger_statement] = STATE(1933),
    [sym_return_statement] = STATE(1933),
    [sym_throw_statement] = STATE(1933),
    [sym_empty_statement] = STATE(1933),
    [sym_labeled_statement] = STATE(1933),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1665] = {
    [sym_parenthesized_expression] = STATE(613),
    [sym__expression] = STATE(613),
    [sym_yield_expression] = STATE(613),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(613),
    [sym_jsx_fragment] = STATE(613),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(613),
    [sym_anonymous_class] = STATE(613),
    [sym_class] = STATE(613),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(613),
    [sym_generator_function] = STATE(613),
    [sym_call_expression] = STATE(613),
    [sym_new_expression] = STATE(613),
    [sym_await_expression] = STATE(613),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(613),
    [sym_augmented_assignment_expression] = STATE(613),
    [sym_ternary_expression] = STATE(613),
    [sym_binary_expression] = STATE(613),
    [sym_unary_expression] = STATE(613),
    [sym_update_expression] = STATE(613),
    [sym_template_string] = STATE(613),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1126),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(1128),
    [sym_number] = ACTIONS(1126),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(1128),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [sym_null] = ACTIONS(1128),
    [sym_undefined] = ACTIONS(1128),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1666] = {
    [sym_parenthesized_expression] = STATE(1934),
    [sym__expression] = STATE(1934),
    [sym_yield_expression] = STATE(1934),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1934),
    [sym_jsx_fragment] = STATE(1934),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1934),
    [sym_anonymous_class] = STATE(1934),
    [sym_class] = STATE(1934),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1934),
    [sym_generator_function] = STATE(1934),
    [sym_call_expression] = STATE(1934),
    [sym_new_expression] = STATE(1934),
    [sym_await_expression] = STATE(1934),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1934),
    [sym_augmented_assignment_expression] = STATE(1934),
    [sym_ternary_expression] = STATE(1934),
    [sym_binary_expression] = STATE(1934),
    [sym_unary_expression] = STATE(1934),
    [sym_update_expression] = STATE(1934),
    [sym_template_string] = STATE(1934),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4210),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4212),
    [sym_number] = ACTIONS(4210),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(4212),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4212),
    [sym_false] = ACTIONS(4212),
    [sym_null] = ACTIONS(4212),
    [sym_undefined] = ACTIONS(4212),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1667] = {
    [sym_parenthesized_expression] = STATE(1935),
    [sym__expression] = STATE(1935),
    [sym_yield_expression] = STATE(1935),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1935),
    [sym_jsx_fragment] = STATE(1935),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1935),
    [sym_anonymous_class] = STATE(1935),
    [sym_class] = STATE(1935),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1935),
    [sym_generator_function] = STATE(1935),
    [sym_call_expression] = STATE(1935),
    [sym_new_expression] = STATE(1935),
    [sym_await_expression] = STATE(1935),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1935),
    [sym_augmented_assignment_expression] = STATE(1935),
    [sym_ternary_expression] = STATE(1935),
    [sym_binary_expression] = STATE(1935),
    [sym_unary_expression] = STATE(1935),
    [sym_update_expression] = STATE(1935),
    [sym_template_string] = STATE(1935),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4214),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4216),
    [sym_number] = ACTIONS(4214),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4216),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4216),
    [sym_false] = ACTIONS(4216),
    [sym_null] = ACTIONS(4216),
    [sym_undefined] = ACTIONS(4216),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1668] = {
    [sym_parenthesized_expression] = STATE(1936),
    [sym__expression] = STATE(1936),
    [sym_yield_expression] = STATE(1936),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1936),
    [sym_jsx_fragment] = STATE(1936),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1936),
    [sym_anonymous_class] = STATE(1936),
    [sym_class] = STATE(1936),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1936),
    [sym_generator_function] = STATE(1936),
    [sym_call_expression] = STATE(1936),
    [sym_new_expression] = STATE(1936),
    [sym_await_expression] = STATE(1936),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1936),
    [sym_augmented_assignment_expression] = STATE(1936),
    [sym_ternary_expression] = STATE(1936),
    [sym_binary_expression] = STATE(1936),
    [sym_unary_expression] = STATE(1936),
    [sym_update_expression] = STATE(1936),
    [sym_template_string] = STATE(1936),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4218),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4220),
    [sym_number] = ACTIONS(4218),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4220),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4220),
    [sym_false] = ACTIONS(4220),
    [sym_null] = ACTIONS(4220),
    [sym_undefined] = ACTIONS(4220),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1669] = {
    [sym_parenthesized_expression] = STATE(1937),
    [sym__expression] = STATE(1937),
    [sym_yield_expression] = STATE(1937),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1937),
    [sym_jsx_fragment] = STATE(1937),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1937),
    [sym_anonymous_class] = STATE(1937),
    [sym_class] = STATE(1937),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1937),
    [sym_generator_function] = STATE(1937),
    [sym_call_expression] = STATE(1937),
    [sym_new_expression] = STATE(1937),
    [sym_await_expression] = STATE(1937),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1937),
    [sym_augmented_assignment_expression] = STATE(1937),
    [sym_ternary_expression] = STATE(1937),
    [sym_binary_expression] = STATE(1937),
    [sym_unary_expression] = STATE(1937),
    [sym_update_expression] = STATE(1937),
    [sym_template_string] = STATE(1937),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4222),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4224),
    [sym_number] = ACTIONS(4222),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4224),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4224),
    [sym_false] = ACTIONS(4224),
    [sym_null] = ACTIONS(4224),
    [sym_undefined] = ACTIONS(4224),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1670] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(882),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1671] = {
    [sym_parenthesized_expression] = STATE(1938),
    [sym__expression] = STATE(1938),
    [sym_yield_expression] = STATE(1938),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1938),
    [sym_jsx_fragment] = STATE(1938),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1938),
    [sym_anonymous_class] = STATE(1938),
    [sym_class] = STATE(1938),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1938),
    [sym_generator_function] = STATE(1938),
    [sym_call_expression] = STATE(1938),
    [sym_new_expression] = STATE(1938),
    [sym_await_expression] = STATE(1938),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1938),
    [sym_augmented_assignment_expression] = STATE(1938),
    [sym_ternary_expression] = STATE(1938),
    [sym_binary_expression] = STATE(1938),
    [sym_unary_expression] = STATE(1938),
    [sym_update_expression] = STATE(1938),
    [sym_template_string] = STATE(1938),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4226),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4228),
    [sym_number] = ACTIONS(4226),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4228),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4228),
    [sym_false] = ACTIONS(4228),
    [sym_null] = ACTIONS(4228),
    [sym_undefined] = ACTIONS(4228),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1672] = {
    [sym_statement_block] = STATE(875),
    [sym_parenthesized_expression] = STATE(1939),
    [sym__expression] = STATE(1939),
    [sym_yield_expression] = STATE(1939),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1939),
    [sym_jsx_fragment] = STATE(1939),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1939),
    [sym_anonymous_class] = STATE(1939),
    [sym_class] = STATE(1939),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1939),
    [sym_generator_function] = STATE(1939),
    [sym_call_expression] = STATE(1939),
    [sym_new_expression] = STATE(1939),
    [sym_await_expression] = STATE(1939),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1939),
    [sym_augmented_assignment_expression] = STATE(1939),
    [sym_ternary_expression] = STATE(1939),
    [sym_binary_expression] = STATE(1939),
    [sym_unary_expression] = STATE(1939),
    [sym_update_expression] = STATE(1939),
    [sym_template_string] = STATE(1939),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4230),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4232),
    [sym_number] = ACTIONS(4230),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4232),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4232),
    [sym_false] = ACTIONS(4232),
    [sym_null] = ACTIONS(4232),
    [sym_undefined] = ACTIONS(4232),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1673] = {
    [sym_parenthesized_expression] = STATE(1940),
    [sym__expression] = STATE(1940),
    [sym_yield_expression] = STATE(1940),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1940),
    [sym_jsx_fragment] = STATE(1940),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1940),
    [sym_anonymous_class] = STATE(1940),
    [sym_class] = STATE(1940),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1940),
    [sym_generator_function] = STATE(1940),
    [sym_call_expression] = STATE(1940),
    [sym_new_expression] = STATE(1940),
    [sym_await_expression] = STATE(1940),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1940),
    [sym_augmented_assignment_expression] = STATE(1940),
    [sym_ternary_expression] = STATE(1940),
    [sym_binary_expression] = STATE(1940),
    [sym_unary_expression] = STATE(1940),
    [sym_update_expression] = STATE(1940),
    [sym_template_string] = STATE(1940),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4234),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4236),
    [sym_number] = ACTIONS(4234),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4236),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4236),
    [sym_false] = ACTIONS(4236),
    [sym_null] = ACTIONS(4236),
    [sym_undefined] = ACTIONS(4236),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1674] = {
    [anon_sym_EQ_GT] = ACTIONS(4238),
    [sym_comment] = ACTIONS(71),
  },
  [1675] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1048),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1676] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(1056),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1677] = {
    [sym_parenthesized_expression] = STATE(1942),
    [sym__expression] = STATE(1942),
    [sym_yield_expression] = STATE(1942),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1942),
    [sym_jsx_fragment] = STATE(1942),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1942),
    [sym_anonymous_class] = STATE(1942),
    [sym_class] = STATE(1942),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1942),
    [sym_generator_function] = STATE(1942),
    [sym_call_expression] = STATE(1942),
    [sym_new_expression] = STATE(1942),
    [sym_await_expression] = STATE(1942),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1942),
    [sym_augmented_assignment_expression] = STATE(1942),
    [sym_ternary_expression] = STATE(1942),
    [sym_binary_expression] = STATE(1942),
    [sym_unary_expression] = STATE(1942),
    [sym_update_expression] = STATE(1942),
    [sym_template_string] = STATE(1942),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4240),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4242),
    [sym_number] = ACTIONS(4240),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4242),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4242),
    [sym_false] = ACTIONS(4242),
    [sym_null] = ACTIONS(4242),
    [sym_undefined] = ACTIONS(4242),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1678] = {
    [sym_parenthesized_expression] = STATE(1943),
    [sym__expression] = STATE(1943),
    [sym_yield_expression] = STATE(1943),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1943),
    [sym_jsx_fragment] = STATE(1943),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1943),
    [sym_anonymous_class] = STATE(1943),
    [sym_class] = STATE(1943),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1943),
    [sym_generator_function] = STATE(1943),
    [sym_call_expression] = STATE(1943),
    [sym_new_expression] = STATE(1943),
    [sym_await_expression] = STATE(1943),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1943),
    [sym_augmented_assignment_expression] = STATE(1943),
    [sym_ternary_expression] = STATE(1943),
    [sym_binary_expression] = STATE(1943),
    [sym_unary_expression] = STATE(1943),
    [sym_update_expression] = STATE(1943),
    [sym_template_string] = STATE(1943),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4244),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4246),
    [sym_number] = ACTIONS(4244),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4246),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4246),
    [sym_false] = ACTIONS(4246),
    [sym_null] = ACTIONS(4246),
    [sym_undefined] = ACTIONS(4246),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1679] = {
    [sym_parenthesized_expression] = STATE(900),
    [sym__expression] = STATE(900),
    [sym_yield_expression] = STATE(900),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(900),
    [sym_jsx_fragment] = STATE(900),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(900),
    [sym_anonymous_class] = STATE(900),
    [sym_class] = STATE(900),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(900),
    [sym_generator_function] = STATE(900),
    [sym_call_expression] = STATE(900),
    [sym_new_expression] = STATE(900),
    [sym_await_expression] = STATE(900),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(900),
    [sym_augmented_assignment_expression] = STATE(900),
    [sym_ternary_expression] = STATE(900),
    [sym_binary_expression] = STATE(900),
    [sym_unary_expression] = STATE(900),
    [sym_update_expression] = STATE(900),
    [sym_template_string] = STATE(900),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1825),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(1827),
    [sym_number] = ACTIONS(1825),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(1827),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(1827),
    [sym_false] = ACTIONS(1827),
    [sym_null] = ACTIONS(1827),
    [sym_undefined] = ACTIONS(1827),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1680] = {
    [sym_export_statement] = STATE(1944),
    [sym__declaration] = STATE(1944),
    [sym_import_statement] = STATE(1944),
    [sym_expression_statement] = STATE(1944),
    [sym_variable_declaration] = STATE(1944),
    [sym_lexical_declaration] = STATE(1944),
    [sym_statement_block] = STATE(1944),
    [sym_if_statement] = STATE(1944),
    [sym_switch_statement] = STATE(1944),
    [sym_for_statement] = STATE(1944),
    [sym_for_in_statement] = STATE(1944),
    [sym_for_of_statement] = STATE(1944),
    [sym_while_statement] = STATE(1944),
    [sym_do_statement] = STATE(1944),
    [sym_try_statement] = STATE(1944),
    [sym_with_statement] = STATE(1944),
    [sym_break_statement] = STATE(1944),
    [sym_continue_statement] = STATE(1944),
    [sym_debugger_statement] = STATE(1944),
    [sym_return_statement] = STATE(1944),
    [sym_throw_statement] = STATE(1944),
    [sym_empty_statement] = STATE(1944),
    [sym_labeled_statement] = STATE(1944),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1681] = {
    [sym_parenthesized_expression] = STATE(1945),
    [sym__expression] = STATE(1945),
    [sym_yield_expression] = STATE(1945),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(1945),
    [sym_jsx_fragment] = STATE(1945),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1945),
    [sym_anonymous_class] = STATE(1945),
    [sym_class] = STATE(1945),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1945),
    [sym_generator_function] = STATE(1945),
    [sym_call_expression] = STATE(1945),
    [sym_new_expression] = STATE(1945),
    [sym_await_expression] = STATE(1945),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(1945),
    [sym_augmented_assignment_expression] = STATE(1945),
    [sym_ternary_expression] = STATE(1945),
    [sym_binary_expression] = STATE(1945),
    [sym_unary_expression] = STATE(1945),
    [sym_update_expression] = STATE(1945),
    [sym_template_string] = STATE(1945),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4248),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4250),
    [sym_number] = ACTIONS(4248),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(4250),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4250),
    [sym_false] = ACTIONS(4250),
    [sym_null] = ACTIONS(4250),
    [sym_undefined] = ACTIONS(4250),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1682] = {
    [sym_parenthesized_expression] = STATE(1946),
    [sym__expression] = STATE(1946),
    [sym_yield_expression] = STATE(1946),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1946),
    [sym_jsx_fragment] = STATE(1946),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1946),
    [sym_anonymous_class] = STATE(1946),
    [sym_class] = STATE(1946),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1946),
    [sym_generator_function] = STATE(1946),
    [sym_call_expression] = STATE(1946),
    [sym_new_expression] = STATE(1946),
    [sym_await_expression] = STATE(1946),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1946),
    [sym_augmented_assignment_expression] = STATE(1946),
    [sym_ternary_expression] = STATE(1946),
    [sym_binary_expression] = STATE(1946),
    [sym_unary_expression] = STATE(1946),
    [sym_update_expression] = STATE(1946),
    [sym_template_string] = STATE(1946),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4252),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4254),
    [sym_number] = ACTIONS(4252),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4254),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4254),
    [sym_false] = ACTIONS(4254),
    [sym_null] = ACTIONS(4254),
    [sym_undefined] = ACTIONS(4254),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1683] = {
    [sym_parenthesized_expression] = STATE(1947),
    [sym__expression] = STATE(1947),
    [sym_yield_expression] = STATE(1947),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1947),
    [sym_jsx_fragment] = STATE(1947),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1947),
    [sym_anonymous_class] = STATE(1947),
    [sym_class] = STATE(1947),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1947),
    [sym_generator_function] = STATE(1947),
    [sym_call_expression] = STATE(1947),
    [sym_new_expression] = STATE(1947),
    [sym_await_expression] = STATE(1947),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1947),
    [sym_augmented_assignment_expression] = STATE(1947),
    [sym_ternary_expression] = STATE(1947),
    [sym_binary_expression] = STATE(1947),
    [sym_unary_expression] = STATE(1947),
    [sym_update_expression] = STATE(1947),
    [sym_template_string] = STATE(1947),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4256),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4258),
    [sym_number] = ACTIONS(4256),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4258),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4258),
    [sym_false] = ACTIONS(4258),
    [sym_null] = ACTIONS(4258),
    [sym_undefined] = ACTIONS(4258),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1684] = {
    [sym_parenthesized_expression] = STATE(1948),
    [sym__expression] = STATE(1948),
    [sym_yield_expression] = STATE(1948),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1948),
    [sym_jsx_fragment] = STATE(1948),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1948),
    [sym_anonymous_class] = STATE(1948),
    [sym_class] = STATE(1948),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1948),
    [sym_generator_function] = STATE(1948),
    [sym_call_expression] = STATE(1948),
    [sym_new_expression] = STATE(1948),
    [sym_await_expression] = STATE(1948),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1948),
    [sym_augmented_assignment_expression] = STATE(1948),
    [sym_ternary_expression] = STATE(1948),
    [sym_binary_expression] = STATE(1948),
    [sym_unary_expression] = STATE(1948),
    [sym_update_expression] = STATE(1948),
    [sym_template_string] = STATE(1948),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4260),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4262),
    [sym_number] = ACTIONS(4260),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4262),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4262),
    [sym_false] = ACTIONS(4262),
    [sym_null] = ACTIONS(4262),
    [sym_undefined] = ACTIONS(4262),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1685] = {
    [ts_builtin_sym_end] = ACTIONS(4264),
    [anon_sym_export] = ACTIONS(4266),
    [anon_sym_LBRACE] = ACTIONS(4264),
    [anon_sym_RBRACE] = ACTIONS(4264),
    [anon_sym_import] = ACTIONS(4266),
    [anon_sym_var] = ACTIONS(4266),
    [anon_sym_let] = ACTIONS(4266),
    [anon_sym_const] = ACTIONS(4266),
    [anon_sym_if] = ACTIONS(4266),
    [anon_sym_switch] = ACTIONS(4266),
    [anon_sym_for] = ACTIONS(4266),
    [anon_sym_LPAREN] = ACTIONS(4264),
    [anon_sym_while] = ACTIONS(4266),
    [anon_sym_do] = ACTIONS(4266),
    [anon_sym_try] = ACTIONS(4266),
    [anon_sym_with] = ACTIONS(4266),
    [anon_sym_break] = ACTIONS(4266),
    [anon_sym_continue] = ACTIONS(4266),
    [anon_sym_debugger] = ACTIONS(4266),
    [anon_sym_return] = ACTIONS(4266),
    [anon_sym_throw] = ACTIONS(4266),
    [anon_sym_SEMI] = ACTIONS(4264),
    [anon_sym_yield] = ACTIONS(4266),
    [anon_sym_LBRACK] = ACTIONS(4264),
    [anon_sym_LT] = ACTIONS(4264),
    [anon_sym_class] = ACTIONS(4266),
    [anon_sym_async] = ACTIONS(4266),
    [anon_sym_function] = ACTIONS(4266),
    [anon_sym_EQ_GT] = ACTIONS(4264),
    [anon_sym_new] = ACTIONS(4266),
    [anon_sym_await] = ACTIONS(4266),
    [anon_sym_PLUS] = ACTIONS(4266),
    [anon_sym_DASH] = ACTIONS(4266),
    [anon_sym_BANG] = ACTIONS(4264),
    [anon_sym_TILDE] = ACTIONS(4264),
    [anon_sym_typeof] = ACTIONS(4266),
    [anon_sym_void] = ACTIONS(4266),
    [anon_sym_delete] = ACTIONS(4266),
    [anon_sym_PLUS_PLUS] = ACTIONS(4264),
    [anon_sym_DASH_DASH] = ACTIONS(4264),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4264),
    [anon_sym_BQUOTE] = ACTIONS(4264),
    [sym_regex] = ACTIONS(4266),
    [sym_number] = ACTIONS(4264),
    [sym_identifier] = ACTIONS(4266),
    [sym_this] = ACTIONS(4266),
    [sym_super] = ACTIONS(4266),
    [sym_true] = ACTIONS(4266),
    [sym_false] = ACTIONS(4266),
    [sym_null] = ACTIONS(4266),
    [sym_undefined] = ACTIONS(4266),
    [anon_sym_AT] = ACTIONS(4264),
    [anon_sym_get] = ACTIONS(4266),
    [anon_sym_set] = ACTIONS(4266),
  },
  [1686] = {
    [sym_export_statement] = STATE(1949),
    [sym__declaration] = STATE(1949),
    [sym_import_statement] = STATE(1949),
    [sym_expression_statement] = STATE(1949),
    [sym_variable_declaration] = STATE(1949),
    [sym_lexical_declaration] = STATE(1949),
    [sym_statement_block] = STATE(1949),
    [sym_if_statement] = STATE(1949),
    [sym_switch_statement] = STATE(1949),
    [sym_for_statement] = STATE(1949),
    [sym_for_in_statement] = STATE(1949),
    [sym_for_of_statement] = STATE(1949),
    [sym_while_statement] = STATE(1949),
    [sym_do_statement] = STATE(1949),
    [sym_try_statement] = STATE(1949),
    [sym_with_statement] = STATE(1949),
    [sym_break_statement] = STATE(1949),
    [sym_continue_statement] = STATE(1949),
    [sym_debugger_statement] = STATE(1949),
    [sym_return_statement] = STATE(1949),
    [sym_throw_statement] = STATE(1949),
    [sym_empty_statement] = STATE(1949),
    [sym_labeled_statement] = STATE(1949),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1687] = {
    [sym_export_statement] = STATE(1950),
    [sym__declaration] = STATE(1950),
    [sym_import_statement] = STATE(1950),
    [sym_expression_statement] = STATE(1950),
    [sym_variable_declaration] = STATE(1950),
    [sym_lexical_declaration] = STATE(1950),
    [sym_statement_block] = STATE(1950),
    [sym_if_statement] = STATE(1950),
    [sym_switch_statement] = STATE(1950),
    [sym_for_statement] = STATE(1950),
    [sym_for_in_statement] = STATE(1950),
    [sym_for_of_statement] = STATE(1950),
    [sym_while_statement] = STATE(1950),
    [sym_do_statement] = STATE(1950),
    [sym_try_statement] = STATE(1950),
    [sym_with_statement] = STATE(1950),
    [sym_break_statement] = STATE(1950),
    [sym_continue_statement] = STATE(1950),
    [sym_debugger_statement] = STATE(1950),
    [sym_return_statement] = STATE(1950),
    [sym_throw_statement] = STATE(1950),
    [sym_empty_statement] = STATE(1950),
    [sym_labeled_statement] = STATE(1950),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1688] = {
    [sym_export_statement] = STATE(1951),
    [sym__declaration] = STATE(1951),
    [sym_import_statement] = STATE(1951),
    [sym_expression_statement] = STATE(1951),
    [sym_variable_declaration] = STATE(1951),
    [sym_lexical_declaration] = STATE(1951),
    [sym_statement_block] = STATE(1951),
    [sym_if_statement] = STATE(1951),
    [sym_switch_statement] = STATE(1951),
    [sym_for_statement] = STATE(1951),
    [sym_for_in_statement] = STATE(1951),
    [sym_for_of_statement] = STATE(1951),
    [sym_while_statement] = STATE(1951),
    [sym_do_statement] = STATE(1951),
    [sym_try_statement] = STATE(1951),
    [sym_with_statement] = STATE(1951),
    [sym_break_statement] = STATE(1951),
    [sym_continue_statement] = STATE(1951),
    [sym_debugger_statement] = STATE(1951),
    [sym_return_statement] = STATE(1951),
    [sym_throw_statement] = STATE(1951),
    [sym_empty_statement] = STATE(1951),
    [sym_labeled_statement] = STATE(1951),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1689] = {
    [sym_export_statement] = STATE(1952),
    [sym__declaration] = STATE(1952),
    [sym_import_statement] = STATE(1952),
    [sym_expression_statement] = STATE(1952),
    [sym_variable_declaration] = STATE(1952),
    [sym_lexical_declaration] = STATE(1952),
    [sym_statement_block] = STATE(1952),
    [sym_if_statement] = STATE(1952),
    [sym_switch_statement] = STATE(1952),
    [sym_for_statement] = STATE(1952),
    [sym_for_in_statement] = STATE(1952),
    [sym_for_of_statement] = STATE(1952),
    [sym_while_statement] = STATE(1952),
    [sym_do_statement] = STATE(1952),
    [sym_try_statement] = STATE(1952),
    [sym_with_statement] = STATE(1952),
    [sym_break_statement] = STATE(1952),
    [sym_continue_statement] = STATE(1952),
    [sym_debugger_statement] = STATE(1952),
    [sym_return_statement] = STATE(1952),
    [sym_throw_statement] = STATE(1952),
    [sym_empty_statement] = STATE(1952),
    [sym_labeled_statement] = STATE(1952),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1690] = {
    [sym_export_statement] = STATE(1953),
    [sym__declaration] = STATE(1953),
    [sym_import_statement] = STATE(1953),
    [sym_expression_statement] = STATE(1953),
    [sym_variable_declaration] = STATE(1953),
    [sym_lexical_declaration] = STATE(1953),
    [sym_statement_block] = STATE(1953),
    [sym_if_statement] = STATE(1953),
    [sym_switch_statement] = STATE(1953),
    [sym_for_statement] = STATE(1953),
    [sym_for_in_statement] = STATE(1953),
    [sym_for_of_statement] = STATE(1953),
    [sym_while_statement] = STATE(1953),
    [sym_do_statement] = STATE(1953),
    [sym_try_statement] = STATE(1953),
    [sym_with_statement] = STATE(1953),
    [sym_break_statement] = STATE(1953),
    [sym_continue_statement] = STATE(1953),
    [sym_debugger_statement] = STATE(1953),
    [sym_return_statement] = STATE(1953),
    [sym_throw_statement] = STATE(1953),
    [sym_empty_statement] = STATE(1953),
    [sym_labeled_statement] = STATE(1953),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1691] = {
    [anon_sym_STAR] = ACTIONS(3417),
    [anon_sym_LBRACE] = ACTIONS(3417),
    [anon_sym_COMMA] = ACTIONS(3417),
    [anon_sym_RBRACE] = ACTIONS(3417),
    [anon_sym_LPAREN] = ACTIONS(3417),
    [anon_sym_RPAREN] = ACTIONS(3417),
    [anon_sym_in] = ACTIONS(3419),
    [anon_sym_COLON] = ACTIONS(3417),
    [anon_sym_LBRACK] = ACTIONS(3417),
    [anon_sym_RBRACK] = ACTIONS(3417),
    [anon_sym_LT] = ACTIONS(3419),
    [anon_sym_GT] = ACTIONS(3419),
    [anon_sym_SLASH] = ACTIONS(3419),
    [anon_sym_DOT] = ACTIONS(3417),
    [anon_sym_EQ] = ACTIONS(3419),
    [anon_sym_QMARK] = ACTIONS(3417),
    [anon_sym_AMP_AMP] = ACTIONS(3417),
    [anon_sym_PIPE_PIPE] = ACTIONS(3417),
    [anon_sym_GT_GT] = ACTIONS(3419),
    [anon_sym_GT_GT_GT] = ACTIONS(3417),
    [anon_sym_LT_LT] = ACTIONS(3417),
    [anon_sym_AMP] = ACTIONS(3419),
    [anon_sym_CARET] = ACTIONS(3417),
    [anon_sym_PIPE] = ACTIONS(3419),
    [anon_sym_PLUS] = ACTIONS(3419),
    [anon_sym_DASH] = ACTIONS(3419),
    [anon_sym_PERCENT] = ACTIONS(3417),
    [anon_sym_LT_EQ] = ACTIONS(3417),
    [anon_sym_EQ_EQ] = ACTIONS(3419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3417),
    [anon_sym_BANG_EQ] = ACTIONS(3419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3417),
    [anon_sym_GT_EQ] = ACTIONS(3417),
    [anon_sym_instanceof] = ACTIONS(3417),
    [anon_sym_PLUS_PLUS] = ACTIONS(3417),
    [anon_sym_DASH_DASH] = ACTIONS(3417),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3417),
  },
  [1692] = {
    [anon_sym_STAR] = ACTIONS(3798),
    [anon_sym_LBRACE] = ACTIONS(3798),
    [anon_sym_COMMA] = ACTIONS(3798),
    [anon_sym_RBRACE] = ACTIONS(3798),
    [anon_sym_LPAREN] = ACTIONS(3798),
    [anon_sym_RPAREN] = ACTIONS(3798),
    [anon_sym_in] = ACTIONS(3800),
    [anon_sym_COLON] = ACTIONS(3798),
    [anon_sym_LBRACK] = ACTIONS(3798),
    [anon_sym_RBRACK] = ACTIONS(3798),
    [anon_sym_LT] = ACTIONS(3800),
    [anon_sym_GT] = ACTIONS(3800),
    [anon_sym_SLASH] = ACTIONS(3800),
    [anon_sym_DOT] = ACTIONS(3798),
    [anon_sym_QMARK] = ACTIONS(3798),
    [anon_sym_AMP_AMP] = ACTIONS(3798),
    [anon_sym_PIPE_PIPE] = ACTIONS(3798),
    [anon_sym_GT_GT] = ACTIONS(3800),
    [anon_sym_GT_GT_GT] = ACTIONS(3798),
    [anon_sym_LT_LT] = ACTIONS(3798),
    [anon_sym_AMP] = ACTIONS(3800),
    [anon_sym_CARET] = ACTIONS(3798),
    [anon_sym_PIPE] = ACTIONS(3800),
    [anon_sym_PLUS] = ACTIONS(3800),
    [anon_sym_DASH] = ACTIONS(3800),
    [anon_sym_PERCENT] = ACTIONS(3798),
    [anon_sym_LT_EQ] = ACTIONS(3798),
    [anon_sym_EQ_EQ] = ACTIONS(3800),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3798),
    [anon_sym_BANG_EQ] = ACTIONS(3800),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3798),
    [anon_sym_GT_EQ] = ACTIONS(3798),
    [anon_sym_instanceof] = ACTIONS(3798),
    [anon_sym_PLUS_PLUS] = ACTIONS(3798),
    [anon_sym_DASH_DASH] = ACTIONS(3798),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3798),
  },
  [1693] = {
    [anon_sym_GT] = ACTIONS(4268),
    [sym_comment] = ACTIONS(71),
  },
  [1694] = {
    [anon_sym_STAR] = ACTIONS(3824),
    [anon_sym_LBRACE] = ACTIONS(3824),
    [anon_sym_COMMA] = ACTIONS(3824),
    [anon_sym_RBRACE] = ACTIONS(3824),
    [anon_sym_LPAREN] = ACTIONS(3824),
    [anon_sym_RPAREN] = ACTIONS(3824),
    [anon_sym_in] = ACTIONS(3826),
    [anon_sym_COLON] = ACTIONS(3824),
    [anon_sym_LBRACK] = ACTIONS(3824),
    [anon_sym_RBRACK] = ACTIONS(3824),
    [anon_sym_LT] = ACTIONS(3826),
    [anon_sym_GT] = ACTIONS(3826),
    [anon_sym_SLASH] = ACTIONS(3826),
    [anon_sym_DOT] = ACTIONS(3824),
    [anon_sym_QMARK] = ACTIONS(3824),
    [anon_sym_AMP_AMP] = ACTIONS(3824),
    [anon_sym_PIPE_PIPE] = ACTIONS(3824),
    [anon_sym_GT_GT] = ACTIONS(3826),
    [anon_sym_GT_GT_GT] = ACTIONS(3824),
    [anon_sym_LT_LT] = ACTIONS(3824),
    [anon_sym_AMP] = ACTIONS(3826),
    [anon_sym_CARET] = ACTIONS(3824),
    [anon_sym_PIPE] = ACTIONS(3826),
    [anon_sym_PLUS] = ACTIONS(3826),
    [anon_sym_DASH] = ACTIONS(3826),
    [anon_sym_PERCENT] = ACTIONS(3824),
    [anon_sym_LT_EQ] = ACTIONS(3824),
    [anon_sym_EQ_EQ] = ACTIONS(3826),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3824),
    [anon_sym_BANG_EQ] = ACTIONS(3826),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3824),
    [anon_sym_GT_EQ] = ACTIONS(3824),
    [anon_sym_instanceof] = ACTIONS(3824),
    [anon_sym_PLUS_PLUS] = ACTIONS(3824),
    [anon_sym_DASH_DASH] = ACTIONS(3824),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3824),
  },
  [1695] = {
    [anon_sym_STAR] = ACTIONS(3828),
    [anon_sym_LBRACE] = ACTIONS(3828),
    [anon_sym_COMMA] = ACTIONS(3828),
    [anon_sym_RBRACE] = ACTIONS(3828),
    [anon_sym_LPAREN] = ACTIONS(3828),
    [anon_sym_RPAREN] = ACTIONS(3828),
    [anon_sym_in] = ACTIONS(3830),
    [anon_sym_COLON] = ACTIONS(3828),
    [anon_sym_LBRACK] = ACTIONS(3828),
    [anon_sym_RBRACK] = ACTIONS(3828),
    [anon_sym_LT] = ACTIONS(3830),
    [anon_sym_GT] = ACTIONS(3830),
    [anon_sym_SLASH] = ACTIONS(3830),
    [anon_sym_DOT] = ACTIONS(3828),
    [anon_sym_QMARK] = ACTIONS(3828),
    [anon_sym_AMP_AMP] = ACTIONS(3828),
    [anon_sym_PIPE_PIPE] = ACTIONS(3828),
    [anon_sym_GT_GT] = ACTIONS(3830),
    [anon_sym_GT_GT_GT] = ACTIONS(3828),
    [anon_sym_LT_LT] = ACTIONS(3828),
    [anon_sym_AMP] = ACTIONS(3830),
    [anon_sym_CARET] = ACTIONS(3828),
    [anon_sym_PIPE] = ACTIONS(3830),
    [anon_sym_PLUS] = ACTIONS(3830),
    [anon_sym_DASH] = ACTIONS(3830),
    [anon_sym_PERCENT] = ACTIONS(3828),
    [anon_sym_LT_EQ] = ACTIONS(3828),
    [anon_sym_EQ_EQ] = ACTIONS(3830),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3828),
    [anon_sym_BANG_EQ] = ACTIONS(3830),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3828),
    [anon_sym_GT_EQ] = ACTIONS(3828),
    [anon_sym_instanceof] = ACTIONS(3828),
    [anon_sym_PLUS_PLUS] = ACTIONS(3828),
    [anon_sym_DASH_DASH] = ACTIONS(3828),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3828),
  },
  [1696] = {
    [anon_sym_STAR] = ACTIONS(3856),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_COMMA] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_RPAREN] = ACTIONS(3856),
    [anon_sym_in] = ACTIONS(3858),
    [anon_sym_COLON] = ACTIONS(3856),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_RBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_GT] = ACTIONS(3858),
    [anon_sym_SLASH] = ACTIONS(3858),
    [anon_sym_DOT] = ACTIONS(3856),
    [anon_sym_QMARK] = ACTIONS(3856),
    [anon_sym_AMP_AMP] = ACTIONS(3856),
    [anon_sym_PIPE_PIPE] = ACTIONS(3856),
    [anon_sym_GT_GT] = ACTIONS(3858),
    [anon_sym_GT_GT_GT] = ACTIONS(3856),
    [anon_sym_LT_LT] = ACTIONS(3856),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_CARET] = ACTIONS(3856),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_PERCENT] = ACTIONS(3856),
    [anon_sym_LT_EQ] = ACTIONS(3856),
    [anon_sym_EQ_EQ] = ACTIONS(3858),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3856),
    [anon_sym_BANG_EQ] = ACTIONS(3858),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3856),
    [anon_sym_GT_EQ] = ACTIONS(3856),
    [anon_sym_instanceof] = ACTIONS(3856),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3856),
  },
  [1697] = {
    [anon_sym_STAR] = ACTIONS(3860),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_COMMA] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_RPAREN] = ACTIONS(3860),
    [anon_sym_in] = ACTIONS(3862),
    [anon_sym_COLON] = ACTIONS(3860),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_RBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_GT] = ACTIONS(3862),
    [anon_sym_SLASH] = ACTIONS(3862),
    [anon_sym_DOT] = ACTIONS(3860),
    [anon_sym_QMARK] = ACTIONS(3860),
    [anon_sym_AMP_AMP] = ACTIONS(3860),
    [anon_sym_PIPE_PIPE] = ACTIONS(3860),
    [anon_sym_GT_GT] = ACTIONS(3862),
    [anon_sym_GT_GT_GT] = ACTIONS(3860),
    [anon_sym_LT_LT] = ACTIONS(3860),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_CARET] = ACTIONS(3860),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_PERCENT] = ACTIONS(3860),
    [anon_sym_LT_EQ] = ACTIONS(3860),
    [anon_sym_EQ_EQ] = ACTIONS(3862),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3860),
    [anon_sym_BANG_EQ] = ACTIONS(3862),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3860),
    [anon_sym_GT_EQ] = ACTIONS(3860),
    [anon_sym_instanceof] = ACTIONS(3860),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3860),
  },
  [1698] = {
    [anon_sym_export] = ACTIONS(3864),
    [anon_sym_STAR] = ACTIONS(3864),
    [anon_sym_LBRACE] = ACTIONS(3864),
    [anon_sym_COMMA] = ACTIONS(3864),
    [anon_sym_RBRACE] = ACTIONS(3864),
    [anon_sym_LPAREN] = ACTIONS(3864),
    [anon_sym_RPAREN] = ACTIONS(3864),
    [anon_sym_in] = ACTIONS(3866),
    [anon_sym_COLON] = ACTIONS(3864),
    [anon_sym_LBRACK] = ACTIONS(3864),
    [anon_sym_RBRACK] = ACTIONS(3864),
    [anon_sym_LT] = ACTIONS(3866),
    [anon_sym_GT] = ACTIONS(3866),
    [anon_sym_SLASH] = ACTIONS(3866),
    [anon_sym_DOT] = ACTIONS(3864),
    [anon_sym_class] = ACTIONS(3864),
    [anon_sym_QMARK] = ACTIONS(3864),
    [anon_sym_AMP_AMP] = ACTIONS(3864),
    [anon_sym_PIPE_PIPE] = ACTIONS(3864),
    [anon_sym_GT_GT] = ACTIONS(3866),
    [anon_sym_GT_GT_GT] = ACTIONS(3864),
    [anon_sym_LT_LT] = ACTIONS(3864),
    [anon_sym_AMP] = ACTIONS(3866),
    [anon_sym_CARET] = ACTIONS(3864),
    [anon_sym_PIPE] = ACTIONS(3866),
    [anon_sym_PLUS] = ACTIONS(3866),
    [anon_sym_DASH] = ACTIONS(3866),
    [anon_sym_PERCENT] = ACTIONS(3864),
    [anon_sym_LT_EQ] = ACTIONS(3864),
    [anon_sym_EQ_EQ] = ACTIONS(3866),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3864),
    [anon_sym_BANG_EQ] = ACTIONS(3866),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3864),
    [anon_sym_GT_EQ] = ACTIONS(3864),
    [anon_sym_instanceof] = ACTIONS(3864),
    [anon_sym_PLUS_PLUS] = ACTIONS(3864),
    [anon_sym_DASH_DASH] = ACTIONS(3864),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3864),
    [anon_sym_AT] = ACTIONS(3864),
  },
  [1699] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3872),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1700] = {
    [anon_sym_STAR] = ACTIONS(3914),
    [anon_sym_LBRACE] = ACTIONS(3914),
    [anon_sym_COMMA] = ACTIONS(3914),
    [anon_sym_RBRACE] = ACTIONS(3914),
    [anon_sym_LPAREN] = ACTIONS(3914),
    [anon_sym_RPAREN] = ACTIONS(3914),
    [anon_sym_in] = ACTIONS(3916),
    [anon_sym_COLON] = ACTIONS(3914),
    [anon_sym_LBRACK] = ACTIONS(3914),
    [anon_sym_RBRACK] = ACTIONS(3914),
    [anon_sym_LT] = ACTIONS(3916),
    [anon_sym_GT] = ACTIONS(3916),
    [anon_sym_SLASH] = ACTIONS(3916),
    [anon_sym_DOT] = ACTIONS(3914),
    [anon_sym_QMARK] = ACTIONS(3914),
    [anon_sym_AMP_AMP] = ACTIONS(3914),
    [anon_sym_PIPE_PIPE] = ACTIONS(3914),
    [anon_sym_GT_GT] = ACTIONS(3916),
    [anon_sym_GT_GT_GT] = ACTIONS(3914),
    [anon_sym_LT_LT] = ACTIONS(3914),
    [anon_sym_AMP] = ACTIONS(3916),
    [anon_sym_CARET] = ACTIONS(3914),
    [anon_sym_PIPE] = ACTIONS(3916),
    [anon_sym_PLUS] = ACTIONS(3916),
    [anon_sym_DASH] = ACTIONS(3916),
    [anon_sym_PERCENT] = ACTIONS(3914),
    [anon_sym_LT_EQ] = ACTIONS(3914),
    [anon_sym_EQ_EQ] = ACTIONS(3916),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3914),
    [anon_sym_BANG_EQ] = ACTIONS(3916),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3914),
    [anon_sym_GT_EQ] = ACTIONS(3914),
    [anon_sym_instanceof] = ACTIONS(3914),
    [anon_sym_PLUS_PLUS] = ACTIONS(3914),
    [anon_sym_DASH_DASH] = ACTIONS(3914),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3914),
  },
  [1701] = {
    [anon_sym_STAR] = ACTIONS(3918),
    [anon_sym_LBRACE] = ACTIONS(3918),
    [anon_sym_COMMA] = ACTIONS(3918),
    [anon_sym_RBRACE] = ACTIONS(3918),
    [anon_sym_LPAREN] = ACTIONS(3918),
    [anon_sym_RPAREN] = ACTIONS(3918),
    [anon_sym_in] = ACTIONS(3920),
    [anon_sym_COLON] = ACTIONS(3918),
    [anon_sym_LBRACK] = ACTIONS(3918),
    [anon_sym_RBRACK] = ACTIONS(3918),
    [anon_sym_LT] = ACTIONS(3920),
    [anon_sym_GT] = ACTIONS(3920),
    [anon_sym_SLASH] = ACTIONS(3920),
    [anon_sym_DOT] = ACTIONS(3918),
    [anon_sym_QMARK] = ACTIONS(3918),
    [anon_sym_AMP_AMP] = ACTIONS(3918),
    [anon_sym_PIPE_PIPE] = ACTIONS(3918),
    [anon_sym_GT_GT] = ACTIONS(3920),
    [anon_sym_GT_GT_GT] = ACTIONS(3918),
    [anon_sym_LT_LT] = ACTIONS(3918),
    [anon_sym_AMP] = ACTIONS(3920),
    [anon_sym_CARET] = ACTIONS(3918),
    [anon_sym_PIPE] = ACTIONS(3920),
    [anon_sym_PLUS] = ACTIONS(3920),
    [anon_sym_DASH] = ACTIONS(3920),
    [anon_sym_PERCENT] = ACTIONS(3918),
    [anon_sym_LT_EQ] = ACTIONS(3918),
    [anon_sym_EQ_EQ] = ACTIONS(3920),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3918),
    [anon_sym_BANG_EQ] = ACTIONS(3920),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3918),
    [anon_sym_GT_EQ] = ACTIONS(3918),
    [anon_sym_instanceof] = ACTIONS(3918),
    [anon_sym_PLUS_PLUS] = ACTIONS(3918),
    [anon_sym_DASH_DASH] = ACTIONS(3918),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3918),
  },
  [1702] = {
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_COMMA] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_RPAREN] = ACTIONS(3934),
    [anon_sym_in] = ACTIONS(3936),
    [anon_sym_COLON] = ACTIONS(3934),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_RBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3936),
    [anon_sym_GT] = ACTIONS(3936),
    [anon_sym_SLASH] = ACTIONS(3936),
    [anon_sym_DOT] = ACTIONS(3934),
    [anon_sym_QMARK] = ACTIONS(3934),
    [anon_sym_AMP_AMP] = ACTIONS(3934),
    [anon_sym_PIPE_PIPE] = ACTIONS(3934),
    [anon_sym_GT_GT] = ACTIONS(3936),
    [anon_sym_GT_GT_GT] = ACTIONS(3934),
    [anon_sym_LT_LT] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3936),
    [anon_sym_CARET] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_PERCENT] = ACTIONS(3934),
    [anon_sym_LT_EQ] = ACTIONS(3934),
    [anon_sym_EQ_EQ] = ACTIONS(3936),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3934),
    [anon_sym_BANG_EQ] = ACTIONS(3936),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3934),
    [anon_sym_GT_EQ] = ACTIONS(3934),
    [anon_sym_instanceof] = ACTIONS(3934),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3934),
  },
  [1703] = {
    [sym_statement_block] = STATE(1448),
    [anon_sym_LBRACE] = ACTIONS(2857),
    [sym_comment] = ACTIONS(71),
  },
  [1704] = {
    [sym_statement_block] = STATE(1449),
    [anon_sym_LBRACE] = ACTIONS(2857),
    [sym_comment] = ACTIONS(71),
  },
  [1705] = {
    [sym_class_body] = STATE(1490),
    [anon_sym_LBRACE] = ACTIONS(285),
    [sym_comment] = ACTIONS(71),
  },
  [1706] = {
    [sym__automatic_semicolon] = ACTIONS(4270),
    [anon_sym_SEMI] = ACTIONS(4270),
    [sym_comment] = ACTIONS(71),
  },
  [1707] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(4270),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(4270),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1708] = {
    [anon_sym_else] = ACTIONS(1397),
    [anon_sym_while] = ACTIONS(1397),
    [sym_comment] = ACTIONS(71),
  },
  [1709] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4272),
  },
  [1710] = {
    [anon_sym_else] = ACTIONS(1627),
    [anon_sym_while] = ACTIONS(1627),
    [sym_comment] = ACTIONS(71),
  },
  [1711] = {
    [sym__automatic_semicolon] = ACTIONS(4274),
    [anon_sym_SEMI] = ACTIONS(4274),
    [sym_comment] = ACTIONS(71),
  },
  [1712] = {
    [anon_sym_else] = ACTIONS(1641),
    [anon_sym_while] = ACTIONS(1641),
    [sym_comment] = ACTIONS(71),
  },
  [1713] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(4276),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4276),
    [sym_comment] = ACTIONS(71),
  },
  [1714] = {
    [anon_sym_else] = ACTIONS(1651),
    [anon_sym_while] = ACTIONS(1651),
    [sym_comment] = ACTIONS(71),
  },
  [1715] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(4278),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4278),
    [sym_comment] = ACTIONS(71),
  },
  [1716] = {
    [anon_sym_else] = ACTIONS(4280),
    [anon_sym_while] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
  },
  [1717] = {
    [sym_switch_case] = STATE(1961),
    [sym_switch_default] = STATE(1961),
    [aux_sym_switch_body_repeat1] = STATE(1961),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(4282),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [1718] = {
    [anon_sym_else] = ACTIONS(1723),
    [anon_sym_while] = ACTIONS(1723),
    [sym_comment] = ACTIONS(71),
  },
  [1719] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(1963),
    [sym_array] = STATE(1964),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4284),
  },
  [1720] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(1963),
    [sym_array] = STATE(1964),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4284),
  },
  [1721] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(4286),
    [anon_sym_of] = ACTIONS(4288),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1722] = {
    [sym_expression_statement] = STATE(1967),
    [sym_empty_statement] = STATE(1967),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1723] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(4290),
    [anon_sym_of] = ACTIONS(4292),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1724] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(4294),
    [anon_sym_of] = ACTIONS(4296),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [1725] = {
    [anon_sym_else] = ACTIONS(1861),
    [anon_sym_while] = ACTIONS(1861),
    [sym_comment] = ACTIONS(71),
  },
  [1726] = {
    [sym_parenthesized_expression] = STATE(1972),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [sym_comment] = ACTIONS(71),
  },
  [1727] = {
    [sym_statement_block] = STATE(1974),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [anon_sym_LPAREN] = ACTIONS(4298),
    [sym_comment] = ACTIONS(71),
  },
  [1728] = {
    [sym_statement_block] = STATE(1975),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [1729] = {
    [sym_finally_clause] = STATE(1976),
    [anon_sym_else] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1941),
    [anon_sym_finally] = ACTIONS(3746),
    [sym_comment] = ACTIONS(71),
  },
  [1730] = {
    [anon_sym_else] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
  },
  [1731] = {
    [anon_sym_else] = ACTIONS(1945),
    [anon_sym_while] = ACTIONS(1945),
    [sym_comment] = ACTIONS(71),
  },
  [1732] = {
    [anon_sym_else] = ACTIONS(1949),
    [anon_sym_while] = ACTIONS(1949),
    [sym_comment] = ACTIONS(71),
  },
  [1733] = {
    [anon_sym_else] = ACTIONS(1953),
    [anon_sym_while] = ACTIONS(1953),
    [sym_comment] = ACTIONS(71),
  },
  [1734] = {
    [anon_sym_else] = ACTIONS(1957),
    [anon_sym_while] = ACTIONS(1957),
    [sym_comment] = ACTIONS(71),
  },
  [1735] = {
    [anon_sym_else] = ACTIONS(1961),
    [anon_sym_while] = ACTIONS(1961),
    [sym_comment] = ACTIONS(71),
  },
  [1736] = {
    [anon_sym_else] = ACTIONS(2204),
    [anon_sym_while] = ACTIONS(2204),
    [sym_comment] = ACTIONS(71),
  },
  [1737] = {
    [sym_parenthesized_expression] = STATE(1977),
    [sym__expression] = STATE(1977),
    [sym_yield_expression] = STATE(1977),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(1977),
    [sym_jsx_fragment] = STATE(1977),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1977),
    [sym_anonymous_class] = STATE(1977),
    [sym_class] = STATE(1977),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1977),
    [sym_generator_function] = STATE(1977),
    [sym_call_expression] = STATE(1977),
    [sym_new_expression] = STATE(1977),
    [sym_await_expression] = STATE(1977),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(1977),
    [sym_augmented_assignment_expression] = STATE(1977),
    [sym_ternary_expression] = STATE(1977),
    [sym_binary_expression] = STATE(1977),
    [sym_unary_expression] = STATE(1977),
    [sym_update_expression] = STATE(1977),
    [sym_template_string] = STATE(1977),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4300),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4302),
    [sym_number] = ACTIONS(4300),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(4302),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4302),
    [sym_false] = ACTIONS(4302),
    [sym_null] = ACTIONS(4302),
    [sym_undefined] = ACTIONS(4302),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [1738] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1978),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1978),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1739] = {
    [sym_parenthesized_expression] = STATE(1979),
    [sym__expression] = STATE(1979),
    [sym_yield_expression] = STATE(1979),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1979),
    [sym_jsx_fragment] = STATE(1979),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1979),
    [sym_anonymous_class] = STATE(1979),
    [sym_class] = STATE(1979),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1979),
    [sym_generator_function] = STATE(1979),
    [sym_call_expression] = STATE(1979),
    [sym_new_expression] = STATE(1979),
    [sym_await_expression] = STATE(1979),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1979),
    [sym_augmented_assignment_expression] = STATE(1979),
    [sym_ternary_expression] = STATE(1979),
    [sym_binary_expression] = STATE(1979),
    [sym_unary_expression] = STATE(1979),
    [sym_update_expression] = STATE(1979),
    [sym_template_string] = STATE(1979),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4304),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4306),
    [sym_number] = ACTIONS(4304),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4306),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4306),
    [sym_false] = ACTIONS(4306),
    [sym_null] = ACTIONS(4306),
    [sym_undefined] = ACTIONS(4306),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1740] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1980),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1980),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1741] = {
    [sym_parenthesized_expression] = STATE(1981),
    [sym__expression] = STATE(1981),
    [sym_yield_expression] = STATE(1981),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1981),
    [sym_jsx_fragment] = STATE(1981),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1981),
    [sym_anonymous_class] = STATE(1981),
    [sym_class] = STATE(1981),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1981),
    [sym_generator_function] = STATE(1981),
    [sym_call_expression] = STATE(1981),
    [sym_new_expression] = STATE(1981),
    [sym_await_expression] = STATE(1981),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1981),
    [sym_augmented_assignment_expression] = STATE(1981),
    [sym_ternary_expression] = STATE(1981),
    [sym_binary_expression] = STATE(1981),
    [sym_unary_expression] = STATE(1981),
    [sym_update_expression] = STATE(1981),
    [sym_template_string] = STATE(1981),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4308),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4310),
    [sym_number] = ACTIONS(4308),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4310),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4310),
    [sym_false] = ACTIONS(4310),
    [sym_null] = ACTIONS(4310),
    [sym_undefined] = ACTIONS(4310),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1742] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(1982),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(1982),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1743] = {
    [sym_parenthesized_expression] = STATE(1983),
    [sym__expression] = STATE(1983),
    [sym_yield_expression] = STATE(1983),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(1983),
    [sym_jsx_fragment] = STATE(1983),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(1983),
    [sym_anonymous_class] = STATE(1983),
    [sym_class] = STATE(1983),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(1983),
    [sym_generator_function] = STATE(1983),
    [sym_call_expression] = STATE(1983),
    [sym_new_expression] = STATE(1983),
    [sym_await_expression] = STATE(1983),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(1983),
    [sym_augmented_assignment_expression] = STATE(1983),
    [sym_ternary_expression] = STATE(1983),
    [sym_binary_expression] = STATE(1983),
    [sym_unary_expression] = STATE(1983),
    [sym_update_expression] = STATE(1983),
    [sym_template_string] = STATE(1983),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4312),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4314),
    [sym_number] = ACTIONS(4312),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4314),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4314),
    [sym_false] = ACTIONS(4314),
    [sym_null] = ACTIONS(4314),
    [sym_undefined] = ACTIONS(4314),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1744] = {
    [anon_sym_RPAREN] = ACTIONS(4316),
    [sym_comment] = ACTIONS(71),
  },
  [1745] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4318),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1746] = {
    [sym_export_statement] = STATE(1685),
    [sym__declaration] = STATE(1685),
    [sym_import_statement] = STATE(1685),
    [sym_expression_statement] = STATE(1685),
    [sym_variable_declaration] = STATE(1685),
    [sym_lexical_declaration] = STATE(1685),
    [sym_statement_block] = STATE(1685),
    [sym_if_statement] = STATE(1685),
    [sym_switch_statement] = STATE(1685),
    [sym_for_statement] = STATE(1685),
    [sym_for_in_statement] = STATE(1685),
    [sym_for_of_statement] = STATE(1685),
    [sym_while_statement] = STATE(1685),
    [sym_do_statement] = STATE(1685),
    [sym_try_statement] = STATE(1685),
    [sym_with_statement] = STATE(1685),
    [sym_break_statement] = STATE(1685),
    [sym_continue_statement] = STATE(1685),
    [sym_debugger_statement] = STATE(1685),
    [sym_return_statement] = STATE(1685),
    [sym_throw_statement] = STATE(1685),
    [sym_empty_statement] = STATE(1685),
    [sym_labeled_statement] = STATE(1685),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1747] = {
    [anon_sym_RPAREN] = ACTIONS(4320),
    [sym_comment] = ACTIONS(71),
  },
  [1748] = {
    [anon_sym_RPAREN] = ACTIONS(4322),
    [sym_comment] = ACTIONS(71),
  },
  [1749] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4324),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1750] = {
    [anon_sym_RPAREN] = ACTIONS(4326),
    [sym_comment] = ACTIONS(71),
  },
  [1751] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4328),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1752] = {
    [anon_sym_RPAREN] = ACTIONS(4330),
    [sym_comment] = ACTIONS(71),
  },
  [1753] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [anon_sym_STAR] = ACTIONS(3856),
    [anon_sym_COMMA] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_else] = ACTIONS(3856),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_RPAREN] = ACTIONS(3856),
    [anon_sym_in] = ACTIONS(3858),
    [anon_sym_while] = ACTIONS(3856),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_GT] = ACTIONS(3858),
    [anon_sym_SLASH] = ACTIONS(3858),
    [anon_sym_DOT] = ACTIONS(3856),
    [anon_sym_QMARK] = ACTIONS(3856),
    [anon_sym_AMP_AMP] = ACTIONS(3856),
    [anon_sym_PIPE_PIPE] = ACTIONS(3856),
    [anon_sym_GT_GT] = ACTIONS(3858),
    [anon_sym_GT_GT_GT] = ACTIONS(3856),
    [anon_sym_LT_LT] = ACTIONS(3856),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_CARET] = ACTIONS(3856),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_PERCENT] = ACTIONS(3856),
    [anon_sym_LT_EQ] = ACTIONS(3856),
    [anon_sym_EQ_EQ] = ACTIONS(3858),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3856),
    [anon_sym_BANG_EQ] = ACTIONS(3858),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3856),
    [anon_sym_GT_EQ] = ACTIONS(3856),
    [anon_sym_instanceof] = ACTIONS(3856),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3856),
  },
  [1754] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [anon_sym_STAR] = ACTIONS(3860),
    [anon_sym_COMMA] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_else] = ACTIONS(3860),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_RPAREN] = ACTIONS(3860),
    [anon_sym_in] = ACTIONS(3862),
    [anon_sym_while] = ACTIONS(3860),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_GT] = ACTIONS(3862),
    [anon_sym_SLASH] = ACTIONS(3862),
    [anon_sym_DOT] = ACTIONS(3860),
    [anon_sym_QMARK] = ACTIONS(3860),
    [anon_sym_AMP_AMP] = ACTIONS(3860),
    [anon_sym_PIPE_PIPE] = ACTIONS(3860),
    [anon_sym_GT_GT] = ACTIONS(3862),
    [anon_sym_GT_GT_GT] = ACTIONS(3860),
    [anon_sym_LT_LT] = ACTIONS(3860),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_CARET] = ACTIONS(3860),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_PERCENT] = ACTIONS(3860),
    [anon_sym_LT_EQ] = ACTIONS(3860),
    [anon_sym_EQ_EQ] = ACTIONS(3862),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3860),
    [anon_sym_BANG_EQ] = ACTIONS(3862),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3860),
    [anon_sym_GT_EQ] = ACTIONS(3860),
    [anon_sym_instanceof] = ACTIONS(3860),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3860),
  },
  [1755] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_COMMA] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_else] = ACTIONS(3934),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_RPAREN] = ACTIONS(3934),
    [anon_sym_in] = ACTIONS(3936),
    [anon_sym_while] = ACTIONS(3934),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3936),
    [anon_sym_GT] = ACTIONS(3936),
    [anon_sym_SLASH] = ACTIONS(3936),
    [anon_sym_DOT] = ACTIONS(3934),
    [anon_sym_QMARK] = ACTIONS(3934),
    [anon_sym_AMP_AMP] = ACTIONS(3934),
    [anon_sym_PIPE_PIPE] = ACTIONS(3934),
    [anon_sym_GT_GT] = ACTIONS(3936),
    [anon_sym_GT_GT_GT] = ACTIONS(3934),
    [anon_sym_LT_LT] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3936),
    [anon_sym_CARET] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_PERCENT] = ACTIONS(3934),
    [anon_sym_LT_EQ] = ACTIONS(3934),
    [anon_sym_EQ_EQ] = ACTIONS(3936),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3934),
    [anon_sym_BANG_EQ] = ACTIONS(3936),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3934),
    [anon_sym_GT_EQ] = ACTIONS(3934),
    [anon_sym_instanceof] = ACTIONS(3934),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(3934),
  },
  [1756] = {
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [1757] = {
    [sym_statement_block] = STATE(1992),
    [anon_sym_LBRACE] = ACTIONS(1935),
    [sym_comment] = ACTIONS(71),
  },
  [1758] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(922),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(918),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(3872),
    [anon_sym_LT] = ACTIONS(918),
    [anon_sym_GT] = ACTIONS(918),
    [anon_sym_SLASH] = ACTIONS(926),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(928),
    [anon_sym_AMP_AMP] = ACTIONS(930),
    [anon_sym_PIPE_PIPE] = ACTIONS(932),
    [anon_sym_GT_GT] = ACTIONS(926),
    [anon_sym_GT_GT_GT] = ACTIONS(922),
    [anon_sym_LT_LT] = ACTIONS(922),
    [anon_sym_AMP] = ACTIONS(934),
    [anon_sym_CARET] = ACTIONS(932),
    [anon_sym_PIPE] = ACTIONS(936),
    [anon_sym_PLUS] = ACTIONS(938),
    [anon_sym_DASH] = ACTIONS(938),
    [anon_sym_PERCENT] = ACTIONS(922),
    [anon_sym_LT_EQ] = ACTIONS(940),
    [anon_sym_EQ_EQ] = ACTIONS(918),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(940),
    [anon_sym_BANG_EQ] = ACTIONS(918),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(940),
    [anon_sym_GT_EQ] = ACTIONS(940),
    [anon_sym_instanceof] = ACTIONS(940),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1759] = {
    [sym__automatic_semicolon] = ACTIONS(4332),
    [anon_sym_STAR] = ACTIONS(4332),
    [anon_sym_COMMA] = ACTIONS(4332),
    [anon_sym_RBRACE] = ACTIONS(4332),
    [anon_sym_LPAREN] = ACTIONS(4332),
    [anon_sym_RPAREN] = ACTIONS(4332),
    [anon_sym_in] = ACTIONS(4334),
    [anon_sym_SEMI] = ACTIONS(4332),
    [anon_sym_LBRACK] = ACTIONS(4332),
    [anon_sym_LT] = ACTIONS(4334),
    [anon_sym_GT] = ACTIONS(4334),
    [anon_sym_SLASH] = ACTIONS(4334),
    [anon_sym_DOT] = ACTIONS(4332),
    [anon_sym_QMARK] = ACTIONS(4332),
    [anon_sym_AMP_AMP] = ACTIONS(4332),
    [anon_sym_PIPE_PIPE] = ACTIONS(4332),
    [anon_sym_GT_GT] = ACTIONS(4334),
    [anon_sym_GT_GT_GT] = ACTIONS(4332),
    [anon_sym_LT_LT] = ACTIONS(4332),
    [anon_sym_AMP] = ACTIONS(4334),
    [anon_sym_CARET] = ACTIONS(4332),
    [anon_sym_PIPE] = ACTIONS(4334),
    [anon_sym_PLUS] = ACTIONS(4334),
    [anon_sym_DASH] = ACTIONS(4334),
    [anon_sym_PERCENT] = ACTIONS(4332),
    [anon_sym_LT_EQ] = ACTIONS(4332),
    [anon_sym_EQ_EQ] = ACTIONS(4334),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4332),
    [anon_sym_BANG_EQ] = ACTIONS(4334),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4332),
    [anon_sym_GT_EQ] = ACTIONS(4332),
    [anon_sym_instanceof] = ACTIONS(4332),
    [anon_sym_PLUS_PLUS] = ACTIONS(4332),
    [anon_sym_DASH_DASH] = ACTIONS(4332),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(4332),
  },
  [1760] = {
    [sym_jsx_element] = STATE(1994),
    [sym_jsx_expression] = STATE(1994),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(1994),
    [aux_sym_jsx_element_repeat1] = STATE(1994),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(4336),
    [sym_jsx_text] = ACTIONS(4338),
    [sym_comment] = ACTIONS(71),
  },
  [1761] = {
    [sym_jsx_expression] = STATE(1996),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1996),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1996),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(950),
    [anon_sym_SLASH] = ACTIONS(4340),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1762] = {
    [sym_jsx_expression] = STATE(1998),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1998),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1998),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(4342),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1763] = {
    [sym_jsx_expression] = STATE(1998),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1998),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1998),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(4342),
    [sym_jsx_identifier] = ACTIONS(954),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1764] = {
    [sym_jsx_expression] = STATE(1998),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(1998),
    [aux_sym_jsx_opening_element_repeat1] = STATE(1998),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(960),
    [anon_sym_SLASH] = ACTIONS(4342),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1765] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(4344),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [1766] = {
    [anon_sym_LBRACE] = ACTIONS(1196),
    [anon_sym_GT] = ACTIONS(1196),
    [anon_sym_SLASH] = ACTIONS(1198),
    [sym_jsx_identifier] = ACTIONS(1196),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1198),
  },
  [1767] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_closing_element] = STATE(2000),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(3816),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [1768] = {
    [sym_statement_block] = STATE(1541),
    [anon_sym_LBRACE] = ACTIONS(3030),
    [sym_comment] = ACTIONS(71),
  },
  [1769] = {
    [anon_sym_STAR] = ACTIONS(2218),
    [anon_sym_LBRACK] = ACTIONS(2218),
    [anon_sym_async] = ACTIONS(2220),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2218),
    [sym_number] = ACTIONS(2218),
    [sym_identifier] = ACTIONS(2220),
    [anon_sym_AT] = ACTIONS(2218),
    [anon_sym_static] = ACTIONS(2220),
    [anon_sym_get] = ACTIONS(2220),
    [anon_sym_set] = ACTIONS(2220),
  },
  [1770] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [aux_sym_array_repeat1] = STATE(2002),
    [anon_sym_STAR] = ACTIONS(776),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4346),
    [anon_sym_in] = ACTIONS(764),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(764),
    [anon_sym_GT] = ACTIONS(764),
    [anon_sym_SLASH] = ACTIONS(780),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(782),
    [anon_sym_AMP_AMP] = ACTIONS(784),
    [anon_sym_PIPE_PIPE] = ACTIONS(786),
    [anon_sym_GT_GT] = ACTIONS(780),
    [anon_sym_GT_GT_GT] = ACTIONS(776),
    [anon_sym_LT_LT] = ACTIONS(776),
    [anon_sym_AMP] = ACTIONS(788),
    [anon_sym_CARET] = ACTIONS(786),
    [anon_sym_PIPE] = ACTIONS(790),
    [anon_sym_PLUS] = ACTIONS(792),
    [anon_sym_DASH] = ACTIONS(792),
    [anon_sym_PERCENT] = ACTIONS(776),
    [anon_sym_LT_EQ] = ACTIONS(794),
    [anon_sym_EQ_EQ] = ACTIONS(764),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(794),
    [anon_sym_BANG_EQ] = ACTIONS(764),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(794),
    [anon_sym_GT_EQ] = ACTIONS(794),
    [anon_sym_instanceof] = ACTIONS(794),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1771] = {
    [aux_sym_array_repeat1] = STATE(2002),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(4346),
    [sym_comment] = ACTIONS(71),
  },
  [1772] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(4346),
    [sym_comment] = ACTIONS(71),
  },
  [1773] = {
    [anon_sym_STAR] = ACTIONS(3146),
    [anon_sym_LPAREN] = ACTIONS(3146),
    [anon_sym_LBRACK] = ACTIONS(3146),
    [anon_sym_DOT] = ACTIONS(3413),
    [anon_sym_async] = ACTIONS(3413),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3146),
    [sym_number] = ACTIONS(3146),
    [sym_identifier] = ACTIONS(3413),
    [anon_sym_AT] = ACTIONS(3146),
    [anon_sym_static] = ACTIONS(3413),
    [anon_sym_get] = ACTIONS(3413),
    [anon_sym_set] = ACTIONS(3413),
  },
  [1774] = {
    [anon_sym_STAR] = ACTIONS(3148),
    [anon_sym_LPAREN] = ACTIONS(3148),
    [anon_sym_LBRACK] = ACTIONS(3148),
    [anon_sym_DOT] = ACTIONS(3415),
    [anon_sym_async] = ACTIONS(3415),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3148),
    [sym_number] = ACTIONS(3148),
    [sym_identifier] = ACTIONS(3415),
    [anon_sym_AT] = ACTIONS(3148),
    [anon_sym_static] = ACTIONS(3415),
    [anon_sym_get] = ACTIONS(3415),
    [anon_sym_set] = ACTIONS(3415),
  },
  [1775] = {
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_async] = ACTIONS(484),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_static] = ACTIONS(484),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1776] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4348),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1777] = {
    [anon_sym_STAR] = ACTIONS(4350),
    [anon_sym_RBRACE] = ACTIONS(4350),
    [anon_sym_LBRACK] = ACTIONS(4350),
    [anon_sym_async] = ACTIONS(4352),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4350),
    [sym_number] = ACTIONS(4350),
    [sym_identifier] = ACTIONS(4352),
    [anon_sym_AT] = ACTIONS(4350),
    [anon_sym_static] = ACTIONS(4352),
    [anon_sym_get] = ACTIONS(4352),
    [anon_sym_set] = ACTIONS(4352),
  },
  [1778] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1779] = {
    [sym_parenthesized_expression] = STATE(2004),
    [sym__expression] = STATE(2004),
    [sym_yield_expression] = STATE(2004),
    [sym_object] = STATE(567),
    [sym_array] = STATE(568),
    [sym_jsx_element] = STATE(2004),
    [sym_jsx_fragment] = STATE(2004),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2004),
    [sym_anonymous_class] = STATE(2004),
    [sym_class] = STATE(2004),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2004),
    [sym_generator_function] = STATE(2004),
    [sym_call_expression] = STATE(2004),
    [sym_new_expression] = STATE(2004),
    [sym_await_expression] = STATE(2004),
    [sym_member_expression] = STATE(569),
    [sym_subscript_expression] = STATE(569),
    [sym_assignment_expression] = STATE(2004),
    [sym_augmented_assignment_expression] = STATE(2004),
    [sym_ternary_expression] = STATE(2004),
    [sym_binary_expression] = STATE(2004),
    [sym_unary_expression] = STATE(2004),
    [sym_update_expression] = STATE(2004),
    [sym_template_string] = STATE(2004),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(570),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(984),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(986),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(988),
    [anon_sym_new] = ACTIONS(990),
    [anon_sym_await] = ACTIONS(992),
    [anon_sym_PLUS] = ACTIONS(994),
    [anon_sym_DASH] = ACTIONS(994),
    [anon_sym_BANG] = ACTIONS(996),
    [anon_sym_TILDE] = ACTIONS(996),
    [anon_sym_typeof] = ACTIONS(994),
    [anon_sym_void] = ACTIONS(994),
    [anon_sym_delete] = ACTIONS(994),
    [anon_sym_PLUS_PLUS] = ACTIONS(998),
    [anon_sym_DASH_DASH] = ACTIONS(998),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4354),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4356),
    [sym_number] = ACTIONS(4354),
    [sym_identifier] = ACTIONS(1004),
    [sym_this] = ACTIONS(4356),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4356),
    [sym_false] = ACTIONS(4356),
    [sym_null] = ACTIONS(4356),
    [sym_undefined] = ACTIONS(4356),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1006),
    [anon_sym_set] = ACTIONS(1006),
  },
  [1780] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1781] = {
    [sym_parenthesized_expression] = STATE(2005),
    [sym__expression] = STATE(2005),
    [sym_yield_expression] = STATE(2005),
    [sym_object] = STATE(626),
    [sym_array] = STATE(627),
    [sym_jsx_element] = STATE(2005),
    [sym_jsx_fragment] = STATE(2005),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2005),
    [sym_anonymous_class] = STATE(2005),
    [sym_class] = STATE(2005),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2005),
    [sym_generator_function] = STATE(2005),
    [sym_call_expression] = STATE(2005),
    [sym_new_expression] = STATE(2005),
    [sym_await_expression] = STATE(2005),
    [sym_member_expression] = STATE(628),
    [sym_subscript_expression] = STATE(628),
    [sym_assignment_expression] = STATE(2005),
    [sym_augmented_assignment_expression] = STATE(2005),
    [sym_ternary_expression] = STATE(2005),
    [sym_binary_expression] = STATE(2005),
    [sym_unary_expression] = STATE(2005),
    [sym_update_expression] = STATE(2005),
    [sym_template_string] = STATE(2005),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(629),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1134),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1136),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1138),
    [anon_sym_new] = ACTIONS(1140),
    [anon_sym_await] = ACTIONS(1142),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_typeof] = ACTIONS(1144),
    [anon_sym_void] = ACTIONS(1144),
    [anon_sym_delete] = ACTIONS(1144),
    [anon_sym_PLUS_PLUS] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1148),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4358),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4360),
    [sym_number] = ACTIONS(4358),
    [sym_identifier] = ACTIONS(1154),
    [sym_this] = ACTIONS(4360),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4360),
    [sym_false] = ACTIONS(4360),
    [sym_null] = ACTIONS(4360),
    [sym_undefined] = ACTIONS(4360),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1156),
    [anon_sym_set] = ACTIONS(1156),
  },
  [1782] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1783] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1784] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1785] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(2006),
    [sym__expression] = STATE(2006),
    [sym_yield_expression] = STATE(2006),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(2006),
    [sym_jsx_fragment] = STATE(2006),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2006),
    [sym_anonymous_class] = STATE(2006),
    [sym_class] = STATE(2006),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2006),
    [sym_generator_function] = STATE(2006),
    [sym_call_expression] = STATE(2006),
    [sym_new_expression] = STATE(2006),
    [sym_await_expression] = STATE(2006),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(2006),
    [sym_augmented_assignment_expression] = STATE(2006),
    [sym_ternary_expression] = STATE(2006),
    [sym_binary_expression] = STATE(2006),
    [sym_unary_expression] = STATE(2006),
    [sym_update_expression] = STATE(2006),
    [sym_template_string] = STATE(2006),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4362),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4364),
    [sym_number] = ACTIONS(4362),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(4364),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4364),
    [sym_false] = ACTIONS(4364),
    [sym_null] = ACTIONS(4364),
    [sym_undefined] = ACTIONS(4364),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [1786] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1787] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1788] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(4366),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1789] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1790] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1791] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1792] = {
    [anon_sym_LBRACE] = ACTIONS(3826),
    [anon_sym_LT] = ACTIONS(3826),
    [sym_jsx_text] = ACTIONS(3826),
    [sym_comment] = ACTIONS(71),
  },
  [1793] = {
    [anon_sym_LBRACE] = ACTIONS(3830),
    [anon_sym_LT] = ACTIONS(3830),
    [sym_jsx_text] = ACTIONS(3830),
    [sym_comment] = ACTIONS(71),
  },
  [1794] = {
    [anon_sym_LBRACE] = ACTIONS(3916),
    [anon_sym_LT] = ACTIONS(3916),
    [sym_jsx_text] = ACTIONS(3916),
    [sym_comment] = ACTIONS(71),
  },
  [1795] = {
    [anon_sym_LBRACE] = ACTIONS(3920),
    [anon_sym_LT] = ACTIONS(3920),
    [sym_jsx_text] = ACTIONS(3920),
    [sym_comment] = ACTIONS(71),
  },
  [1796] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3872),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(3872),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1797] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(3111),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1798] = {
    [sym_parenthesized_expression] = STATE(2008),
    [sym__expression] = STATE(2008),
    [sym_yield_expression] = STATE(2008),
    [sym_object] = STATE(691),
    [sym_array] = STATE(692),
    [sym_jsx_element] = STATE(2008),
    [sym_jsx_fragment] = STATE(2008),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2008),
    [sym_anonymous_class] = STATE(2008),
    [sym_class] = STATE(2008),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2008),
    [sym_generator_function] = STATE(2008),
    [sym_call_expression] = STATE(2008),
    [sym_new_expression] = STATE(2008),
    [sym_await_expression] = STATE(2008),
    [sym_member_expression] = STATE(693),
    [sym_subscript_expression] = STATE(693),
    [sym_assignment_expression] = STATE(2008),
    [sym_augmented_assignment_expression] = STATE(2008),
    [sym_ternary_expression] = STATE(2008),
    [sym_binary_expression] = STATE(2008),
    [sym_unary_expression] = STATE(2008),
    [sym_update_expression] = STATE(2008),
    [sym_template_string] = STATE(2008),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(694),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(1403),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(1405),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(1407),
    [anon_sym_new] = ACTIONS(1409),
    [anon_sym_await] = ACTIONS(1411),
    [anon_sym_PLUS] = ACTIONS(1413),
    [anon_sym_DASH] = ACTIONS(1413),
    [anon_sym_BANG] = ACTIONS(1415),
    [anon_sym_TILDE] = ACTIONS(1415),
    [anon_sym_typeof] = ACTIONS(1413),
    [anon_sym_void] = ACTIONS(1413),
    [anon_sym_delete] = ACTIONS(1413),
    [anon_sym_PLUS_PLUS] = ACTIONS(1417),
    [anon_sym_DASH_DASH] = ACTIONS(1417),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4368),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4370),
    [sym_number] = ACTIONS(4368),
    [sym_identifier] = ACTIONS(1423),
    [sym_this] = ACTIONS(4370),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4370),
    [sym_false] = ACTIONS(4370),
    [sym_null] = ACTIONS(4370),
    [sym_undefined] = ACTIONS(4370),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1425),
    [anon_sym_set] = ACTIONS(1425),
  },
  [1799] = {
    [anon_sym_COMMA] = ACTIONS(3417),
    [anon_sym_RPAREN] = ACTIONS(3417),
    [anon_sym_EQ] = ACTIONS(3417),
    [sym_comment] = ACTIONS(71),
  },
  [1800] = {
    [anon_sym_STAR] = ACTIONS(3129),
    [anon_sym_LBRACK] = ACTIONS(3129),
    [anon_sym_async] = ACTIONS(3131),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3129),
    [sym_number] = ACTIONS(3129),
    [sym_identifier] = ACTIONS(3131),
    [anon_sym_AT] = ACTIONS(3129),
    [anon_sym_get] = ACTIONS(3131),
    [anon_sym_set] = ACTIONS(3131),
  },
  [1801] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(4372),
    [sym_comment] = ACTIONS(71),
  },
  [1802] = {
    [sym_export_statement] = STATE(1840),
    [sym__declaration] = STATE(1840),
    [sym_import_statement] = STATE(1840),
    [sym_expression_statement] = STATE(1840),
    [sym_variable_declaration] = STATE(1840),
    [sym_lexical_declaration] = STATE(1840),
    [sym_statement_block] = STATE(1840),
    [sym_if_statement] = STATE(1840),
    [sym_switch_statement] = STATE(1840),
    [sym_for_statement] = STATE(1840),
    [sym_for_in_statement] = STATE(1840),
    [sym_for_of_statement] = STATE(1840),
    [sym_while_statement] = STATE(1840),
    [sym_do_statement] = STATE(1840),
    [sym_try_statement] = STATE(1840),
    [sym_with_statement] = STATE(1840),
    [sym_break_statement] = STATE(1840),
    [sym_continue_statement] = STATE(1840),
    [sym_debugger_statement] = STATE(1840),
    [sym_return_statement] = STATE(1840),
    [sym_throw_statement] = STATE(1840),
    [sym_empty_statement] = STATE(1840),
    [sym_labeled_statement] = STATE(1840),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1803] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(4374),
    [anon_sym_of] = ACTIONS(4376),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1804] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(4378),
    [anon_sym_of] = ACTIONS(4380),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1805] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(4382),
    [anon_sym_of] = ACTIONS(4384),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1806] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2016),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2016),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1807] = {
    [sym_parenthesized_expression] = STATE(2017),
    [sym__expression] = STATE(2017),
    [sym_yield_expression] = STATE(2017),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2017),
    [sym_jsx_fragment] = STATE(2017),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2017),
    [sym_anonymous_class] = STATE(2017),
    [sym_class] = STATE(2017),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2017),
    [sym_generator_function] = STATE(2017),
    [sym_call_expression] = STATE(2017),
    [sym_new_expression] = STATE(2017),
    [sym_await_expression] = STATE(2017),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2017),
    [sym_augmented_assignment_expression] = STATE(2017),
    [sym_ternary_expression] = STATE(2017),
    [sym_binary_expression] = STATE(2017),
    [sym_unary_expression] = STATE(2017),
    [sym_update_expression] = STATE(2017),
    [sym_template_string] = STATE(2017),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4386),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4388),
    [sym_number] = ACTIONS(4386),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4388),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4388),
    [sym_false] = ACTIONS(4388),
    [sym_null] = ACTIONS(4388),
    [sym_undefined] = ACTIONS(4388),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1808] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2019),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2019),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(4390),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1809] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2020),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2020),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1810] = {
    [sym_parenthesized_expression] = STATE(2021),
    [sym__expression] = STATE(2021),
    [sym_yield_expression] = STATE(2021),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2021),
    [sym_jsx_fragment] = STATE(2021),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2021),
    [sym_anonymous_class] = STATE(2021),
    [sym_class] = STATE(2021),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2021),
    [sym_generator_function] = STATE(2021),
    [sym_call_expression] = STATE(2021),
    [sym_new_expression] = STATE(2021),
    [sym_await_expression] = STATE(2021),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2021),
    [sym_augmented_assignment_expression] = STATE(2021),
    [sym_ternary_expression] = STATE(2021),
    [sym_binary_expression] = STATE(2021),
    [sym_unary_expression] = STATE(2021),
    [sym_update_expression] = STATE(2021),
    [sym_template_string] = STATE(2021),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4392),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4394),
    [sym_number] = ACTIONS(4392),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4394),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4394),
    [sym_false] = ACTIONS(4394),
    [sym_null] = ACTIONS(4394),
    [sym_undefined] = ACTIONS(4394),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1811] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2022),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2022),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1812] = {
    [sym_parenthesized_expression] = STATE(2023),
    [sym__expression] = STATE(2023),
    [sym_yield_expression] = STATE(2023),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2023),
    [sym_jsx_fragment] = STATE(2023),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2023),
    [sym_anonymous_class] = STATE(2023),
    [sym_class] = STATE(2023),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2023),
    [sym_generator_function] = STATE(2023),
    [sym_call_expression] = STATE(2023),
    [sym_new_expression] = STATE(2023),
    [sym_await_expression] = STATE(2023),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2023),
    [sym_augmented_assignment_expression] = STATE(2023),
    [sym_ternary_expression] = STATE(2023),
    [sym_binary_expression] = STATE(2023),
    [sym_unary_expression] = STATE(2023),
    [sym_update_expression] = STATE(2023),
    [sym_template_string] = STATE(2023),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4396),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4398),
    [sym_number] = ACTIONS(4396),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4398),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4398),
    [sym_false] = ACTIONS(4398),
    [sym_null] = ACTIONS(4398),
    [sym_undefined] = ACTIONS(4398),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1813] = {
    [anon_sym_RPAREN] = ACTIONS(4400),
    [sym_comment] = ACTIONS(71),
  },
  [1814] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4402),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1815] = {
    [anon_sym_RPAREN] = ACTIONS(4404),
    [sym_comment] = ACTIONS(71),
  },
  [1816] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4406),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1817] = {
    [anon_sym_RPAREN] = ACTIONS(4408),
    [sym_comment] = ACTIONS(71),
  },
  [1818] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4410),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1819] = {
    [sym_export_statement] = STATE(1933),
    [sym__declaration] = STATE(1933),
    [sym_import_statement] = STATE(1933),
    [sym_expression_statement] = STATE(1933),
    [sym_variable_declaration] = STATE(1933),
    [sym_lexical_declaration] = STATE(1933),
    [sym_statement_block] = STATE(1933),
    [sym_if_statement] = STATE(1933),
    [sym_switch_statement] = STATE(1933),
    [sym_for_statement] = STATE(1933),
    [sym_for_in_statement] = STATE(1933),
    [sym_for_of_statement] = STATE(1933),
    [sym_while_statement] = STATE(1933),
    [sym_do_statement] = STATE(1933),
    [sym_try_statement] = STATE(1933),
    [sym_with_statement] = STATE(1933),
    [sym_break_statement] = STATE(1933),
    [sym_continue_statement] = STATE(1933),
    [sym_debugger_statement] = STATE(1933),
    [sym_return_statement] = STATE(1933),
    [sym_throw_statement] = STATE(1933),
    [sym_empty_statement] = STATE(1933),
    [sym_labeled_statement] = STATE(1933),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1820] = {
    [sym_export_statement] = STATE(1944),
    [sym__declaration] = STATE(1944),
    [sym_import_statement] = STATE(1944),
    [sym_expression_statement] = STATE(1944),
    [sym_variable_declaration] = STATE(1944),
    [sym_lexical_declaration] = STATE(1944),
    [sym_statement_block] = STATE(1944),
    [sym_if_statement] = STATE(1944),
    [sym_switch_statement] = STATE(1944),
    [sym_for_statement] = STATE(1944),
    [sym_for_in_statement] = STATE(1944),
    [sym_for_of_statement] = STATE(1944),
    [sym_while_statement] = STATE(1944),
    [sym_do_statement] = STATE(1944),
    [sym_try_statement] = STATE(1944),
    [sym_with_statement] = STATE(1944),
    [sym_break_statement] = STATE(1944),
    [sym_continue_statement] = STATE(1944),
    [sym_debugger_statement] = STATE(1944),
    [sym_return_statement] = STATE(1944),
    [sym_throw_statement] = STATE(1944),
    [sym_empty_statement] = STATE(1944),
    [sym_labeled_statement] = STATE(1944),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1821] = {
    [sym_export_statement] = STATE(1949),
    [sym__declaration] = STATE(1949),
    [sym_import_statement] = STATE(1949),
    [sym_expression_statement] = STATE(1949),
    [sym_variable_declaration] = STATE(1949),
    [sym_lexical_declaration] = STATE(1949),
    [sym_statement_block] = STATE(1949),
    [sym_if_statement] = STATE(1949),
    [sym_switch_statement] = STATE(1949),
    [sym_for_statement] = STATE(1949),
    [sym_for_in_statement] = STATE(1949),
    [sym_for_of_statement] = STATE(1949),
    [sym_while_statement] = STATE(1949),
    [sym_do_statement] = STATE(1949),
    [sym_try_statement] = STATE(1949),
    [sym_with_statement] = STATE(1949),
    [sym_break_statement] = STATE(1949),
    [sym_continue_statement] = STATE(1949),
    [sym_debugger_statement] = STATE(1949),
    [sym_return_statement] = STATE(1949),
    [sym_throw_statement] = STATE(1949),
    [sym_empty_statement] = STATE(1949),
    [sym_labeled_statement] = STATE(1949),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1822] = {
    [sym_export_statement] = STATE(1950),
    [sym__declaration] = STATE(1950),
    [sym_import_statement] = STATE(1950),
    [sym_expression_statement] = STATE(1950),
    [sym_variable_declaration] = STATE(1950),
    [sym_lexical_declaration] = STATE(1950),
    [sym_statement_block] = STATE(1950),
    [sym_if_statement] = STATE(1950),
    [sym_switch_statement] = STATE(1950),
    [sym_for_statement] = STATE(1950),
    [sym_for_in_statement] = STATE(1950),
    [sym_for_of_statement] = STATE(1950),
    [sym_while_statement] = STATE(1950),
    [sym_do_statement] = STATE(1950),
    [sym_try_statement] = STATE(1950),
    [sym_with_statement] = STATE(1950),
    [sym_break_statement] = STATE(1950),
    [sym_continue_statement] = STATE(1950),
    [sym_debugger_statement] = STATE(1950),
    [sym_return_statement] = STATE(1950),
    [sym_throw_statement] = STATE(1950),
    [sym_empty_statement] = STATE(1950),
    [sym_labeled_statement] = STATE(1950),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1823] = {
    [sym_export_statement] = STATE(1951),
    [sym__declaration] = STATE(1951),
    [sym_import_statement] = STATE(1951),
    [sym_expression_statement] = STATE(1951),
    [sym_variable_declaration] = STATE(1951),
    [sym_lexical_declaration] = STATE(1951),
    [sym_statement_block] = STATE(1951),
    [sym_if_statement] = STATE(1951),
    [sym_switch_statement] = STATE(1951),
    [sym_for_statement] = STATE(1951),
    [sym_for_in_statement] = STATE(1951),
    [sym_for_of_statement] = STATE(1951),
    [sym_while_statement] = STATE(1951),
    [sym_do_statement] = STATE(1951),
    [sym_try_statement] = STATE(1951),
    [sym_with_statement] = STATE(1951),
    [sym_break_statement] = STATE(1951),
    [sym_continue_statement] = STATE(1951),
    [sym_debugger_statement] = STATE(1951),
    [sym_return_statement] = STATE(1951),
    [sym_throw_statement] = STATE(1951),
    [sym_empty_statement] = STATE(1951),
    [sym_labeled_statement] = STATE(1951),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1824] = {
    [sym_export_statement] = STATE(1952),
    [sym__declaration] = STATE(1952),
    [sym_import_statement] = STATE(1952),
    [sym_expression_statement] = STATE(1952),
    [sym_variable_declaration] = STATE(1952),
    [sym_lexical_declaration] = STATE(1952),
    [sym_statement_block] = STATE(1952),
    [sym_if_statement] = STATE(1952),
    [sym_switch_statement] = STATE(1952),
    [sym_for_statement] = STATE(1952),
    [sym_for_in_statement] = STATE(1952),
    [sym_for_of_statement] = STATE(1952),
    [sym_while_statement] = STATE(1952),
    [sym_do_statement] = STATE(1952),
    [sym_try_statement] = STATE(1952),
    [sym_with_statement] = STATE(1952),
    [sym_break_statement] = STATE(1952),
    [sym_continue_statement] = STATE(1952),
    [sym_debugger_statement] = STATE(1952),
    [sym_return_statement] = STATE(1952),
    [sym_throw_statement] = STATE(1952),
    [sym_empty_statement] = STATE(1952),
    [sym_labeled_statement] = STATE(1952),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1825] = {
    [sym_export_statement] = STATE(1953),
    [sym__declaration] = STATE(1953),
    [sym_import_statement] = STATE(1953),
    [sym_expression_statement] = STATE(1953),
    [sym_variable_declaration] = STATE(1953),
    [sym_lexical_declaration] = STATE(1953),
    [sym_statement_block] = STATE(1953),
    [sym_if_statement] = STATE(1953),
    [sym_switch_statement] = STATE(1953),
    [sym_for_statement] = STATE(1953),
    [sym_for_in_statement] = STATE(1953),
    [sym_for_of_statement] = STATE(1953),
    [sym_while_statement] = STATE(1953),
    [sym_do_statement] = STATE(1953),
    [sym_try_statement] = STATE(1953),
    [sym_with_statement] = STATE(1953),
    [sym_break_statement] = STATE(1953),
    [sym_continue_statement] = STATE(1953),
    [sym_debugger_statement] = STATE(1953),
    [sym_return_statement] = STATE(1953),
    [sym_throw_statement] = STATE(1953),
    [sym_empty_statement] = STATE(1953),
    [sym_labeled_statement] = STATE(1953),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1826] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(1503),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_RBRACE] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(1499),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(1499),
    [anon_sym_GT] = ACTIONS(1499),
    [anon_sym_SLASH] = ACTIONS(1507),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(1509),
    [anon_sym_AMP_AMP] = ACTIONS(1511),
    [anon_sym_PIPE_PIPE] = ACTIONS(1513),
    [anon_sym_GT_GT] = ACTIONS(1507),
    [anon_sym_GT_GT_GT] = ACTIONS(1503),
    [anon_sym_LT_LT] = ACTIONS(1503),
    [anon_sym_AMP] = ACTIONS(1515),
    [anon_sym_CARET] = ACTIONS(1513),
    [anon_sym_PIPE] = ACTIONS(1517),
    [anon_sym_PLUS] = ACTIONS(1519),
    [anon_sym_DASH] = ACTIONS(1519),
    [anon_sym_PERCENT] = ACTIONS(1503),
    [anon_sym_LT_EQ] = ACTIONS(1521),
    [anon_sym_EQ_EQ] = ACTIONS(1499),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1521),
    [anon_sym_BANG_EQ] = ACTIONS(1499),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1521),
    [anon_sym_GT_EQ] = ACTIONS(1521),
    [anon_sym_instanceof] = ACTIONS(1521),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1827] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3111),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_RBRACE] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1828] = {
    [sym_parenthesized_expression] = STATE(2030),
    [sym__expression] = STATE(2030),
    [sym_yield_expression] = STATE(2030),
    [sym_object] = STATE(755),
    [sym_array] = STATE(756),
    [sym_jsx_element] = STATE(2030),
    [sym_jsx_fragment] = STATE(2030),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2030),
    [sym_anonymous_class] = STATE(2030),
    [sym_class] = STATE(2030),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2030),
    [sym_generator_function] = STATE(2030),
    [sym_call_expression] = STATE(2030),
    [sym_new_expression] = STATE(2030),
    [sym_await_expression] = STATE(2030),
    [sym_member_expression] = STATE(757),
    [sym_subscript_expression] = STATE(757),
    [sym_assignment_expression] = STATE(2030),
    [sym_augmented_assignment_expression] = STATE(2030),
    [sym_ternary_expression] = STATE(2030),
    [sym_binary_expression] = STATE(2030),
    [sym_unary_expression] = STATE(2030),
    [sym_update_expression] = STATE(2030),
    [sym_template_string] = STATE(2030),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(758),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(1523),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(1525),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(1527),
    [anon_sym_new] = ACTIONS(1529),
    [anon_sym_await] = ACTIONS(1531),
    [anon_sym_PLUS] = ACTIONS(1533),
    [anon_sym_DASH] = ACTIONS(1533),
    [anon_sym_BANG] = ACTIONS(1535),
    [anon_sym_TILDE] = ACTIONS(1535),
    [anon_sym_typeof] = ACTIONS(1533),
    [anon_sym_void] = ACTIONS(1533),
    [anon_sym_delete] = ACTIONS(1533),
    [anon_sym_PLUS_PLUS] = ACTIONS(1537),
    [anon_sym_DASH_DASH] = ACTIONS(1537),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4412),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4414),
    [sym_number] = ACTIONS(4412),
    [sym_identifier] = ACTIONS(1543),
    [sym_this] = ACTIONS(4414),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4414),
    [sym_false] = ACTIONS(4414),
    [sym_null] = ACTIONS(4414),
    [sym_undefined] = ACTIONS(4414),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(1545),
    [anon_sym_set] = ACTIONS(1545),
  },
  [1829] = {
    [anon_sym_export] = ACTIONS(3866),
    [anon_sym_STAR] = ACTIONS(3864),
    [anon_sym_LBRACK] = ACTIONS(3864),
    [anon_sym_class] = ACTIONS(3866),
    [anon_sym_async] = ACTIONS(3866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3864),
    [sym_number] = ACTIONS(3864),
    [sym_identifier] = ACTIONS(3866),
    [anon_sym_AT] = ACTIONS(3864),
    [anon_sym_get] = ACTIONS(3866),
    [anon_sym_set] = ACTIONS(3866),
  },
  [1830] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [ts_builtin_sym_end] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_else] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2064),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2064),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [1831] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(4416),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [1832] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [ts_builtin_sym_end] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_else] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3103),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3103),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [1833] = {
    [sym_statement_block] = STATE(2032),
    [anon_sym_LBRACE] = ACTIONS(3442),
    [sym_comment] = ACTIONS(71),
  },
  [1834] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [ts_builtin_sym_end] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_else] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3107),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3107),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [1835] = {
    [sym_statement_block] = STATE(2033),
    [anon_sym_LBRACE] = ACTIONS(3442),
    [sym_comment] = ACTIONS(71),
  },
  [1836] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [ts_builtin_sym_end] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_else] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3115),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3115),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [1837] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1838] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4418),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1839] = {
    [sym_class_body] = STATE(2035),
    [anon_sym_LBRACE] = ACTIONS(3436),
    [sym_comment] = ACTIONS(71),
  },
  [1840] = {
    [ts_builtin_sym_end] = ACTIONS(3482),
    [anon_sym_export] = ACTIONS(3484),
    [anon_sym_LBRACE] = ACTIONS(3482),
    [anon_sym_RBRACE] = ACTIONS(3482),
    [anon_sym_import] = ACTIONS(3484),
    [anon_sym_var] = ACTIONS(3484),
    [anon_sym_let] = ACTIONS(3484),
    [anon_sym_const] = ACTIONS(3484),
    [anon_sym_if] = ACTIONS(3484),
    [anon_sym_else] = ACTIONS(3484),
    [anon_sym_switch] = ACTIONS(3484),
    [anon_sym_for] = ACTIONS(3484),
    [anon_sym_LPAREN] = ACTIONS(3482),
    [anon_sym_while] = ACTIONS(3484),
    [anon_sym_do] = ACTIONS(3484),
    [anon_sym_try] = ACTIONS(3484),
    [anon_sym_with] = ACTIONS(3484),
    [anon_sym_break] = ACTIONS(3484),
    [anon_sym_continue] = ACTIONS(3484),
    [anon_sym_debugger] = ACTIONS(3484),
    [anon_sym_return] = ACTIONS(3484),
    [anon_sym_throw] = ACTIONS(3484),
    [anon_sym_SEMI] = ACTIONS(3482),
    [anon_sym_yield] = ACTIONS(3484),
    [anon_sym_LBRACK] = ACTIONS(3482),
    [anon_sym_LT] = ACTIONS(3482),
    [anon_sym_class] = ACTIONS(3484),
    [anon_sym_async] = ACTIONS(3484),
    [anon_sym_function] = ACTIONS(3484),
    [anon_sym_EQ_GT] = ACTIONS(3482),
    [anon_sym_new] = ACTIONS(3484),
    [anon_sym_await] = ACTIONS(3484),
    [anon_sym_PLUS] = ACTIONS(3484),
    [anon_sym_DASH] = ACTIONS(3484),
    [anon_sym_BANG] = ACTIONS(3482),
    [anon_sym_TILDE] = ACTIONS(3482),
    [anon_sym_typeof] = ACTIONS(3484),
    [anon_sym_void] = ACTIONS(3484),
    [anon_sym_delete] = ACTIONS(3484),
    [anon_sym_PLUS_PLUS] = ACTIONS(3482),
    [anon_sym_DASH_DASH] = ACTIONS(3482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3482),
    [anon_sym_BQUOTE] = ACTIONS(3482),
    [sym_regex] = ACTIONS(3484),
    [sym_number] = ACTIONS(3482),
    [sym_identifier] = ACTIONS(3484),
    [sym_this] = ACTIONS(3484),
    [sym_super] = ACTIONS(3484),
    [sym_true] = ACTIONS(3484),
    [sym_false] = ACTIONS(3484),
    [sym_null] = ACTIONS(3484),
    [sym_undefined] = ACTIONS(3484),
    [anon_sym_AT] = ACTIONS(3482),
    [anon_sym_get] = ACTIONS(3484),
    [anon_sym_set] = ACTIONS(3484),
  },
  [1841] = {
    [ts_builtin_sym_end] = ACTIONS(3584),
    [anon_sym_export] = ACTIONS(3586),
    [anon_sym_LBRACE] = ACTIONS(3584),
    [anon_sym_RBRACE] = ACTIONS(3584),
    [anon_sym_import] = ACTIONS(3586),
    [anon_sym_var] = ACTIONS(3586),
    [anon_sym_let] = ACTIONS(3586),
    [anon_sym_const] = ACTIONS(3586),
    [anon_sym_if] = ACTIONS(3586),
    [anon_sym_else] = ACTIONS(3586),
    [anon_sym_switch] = ACTIONS(3586),
    [anon_sym_for] = ACTIONS(3586),
    [anon_sym_LPAREN] = ACTIONS(3584),
    [anon_sym_while] = ACTIONS(3586),
    [anon_sym_do] = ACTIONS(3586),
    [anon_sym_try] = ACTIONS(3586),
    [anon_sym_with] = ACTIONS(3586),
    [anon_sym_break] = ACTIONS(3586),
    [anon_sym_continue] = ACTIONS(3586),
    [anon_sym_debugger] = ACTIONS(3586),
    [anon_sym_return] = ACTIONS(3586),
    [anon_sym_throw] = ACTIONS(3586),
    [anon_sym_SEMI] = ACTIONS(3584),
    [anon_sym_yield] = ACTIONS(3586),
    [anon_sym_LBRACK] = ACTIONS(3584),
    [anon_sym_LT] = ACTIONS(3584),
    [anon_sym_class] = ACTIONS(3586),
    [anon_sym_async] = ACTIONS(3586),
    [anon_sym_function] = ACTIONS(3586),
    [anon_sym_EQ_GT] = ACTIONS(3584),
    [anon_sym_new] = ACTIONS(3586),
    [anon_sym_await] = ACTIONS(3586),
    [anon_sym_PLUS] = ACTIONS(3586),
    [anon_sym_DASH] = ACTIONS(3586),
    [anon_sym_BANG] = ACTIONS(3584),
    [anon_sym_TILDE] = ACTIONS(3584),
    [anon_sym_typeof] = ACTIONS(3586),
    [anon_sym_void] = ACTIONS(3586),
    [anon_sym_delete] = ACTIONS(3586),
    [anon_sym_PLUS_PLUS] = ACTIONS(3584),
    [anon_sym_DASH_DASH] = ACTIONS(3584),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3584),
    [anon_sym_BQUOTE] = ACTIONS(3584),
    [sym_regex] = ACTIONS(3586),
    [sym_number] = ACTIONS(3584),
    [sym_identifier] = ACTIONS(3586),
    [sym_this] = ACTIONS(3586),
    [sym_super] = ACTIONS(3586),
    [sym_true] = ACTIONS(3586),
    [sym_false] = ACTIONS(3586),
    [sym_null] = ACTIONS(3586),
    [sym_undefined] = ACTIONS(3586),
    [anon_sym_AT] = ACTIONS(3584),
    [anon_sym_get] = ACTIONS(3586),
    [anon_sym_set] = ACTIONS(3586),
  },
  [1842] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2036),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2036),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1843] = {
    [sym_parenthesized_expression] = STATE(2037),
    [sym__expression] = STATE(2037),
    [sym_yield_expression] = STATE(2037),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2037),
    [sym_jsx_fragment] = STATE(2037),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2037),
    [sym_anonymous_class] = STATE(2037),
    [sym_class] = STATE(2037),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2037),
    [sym_generator_function] = STATE(2037),
    [sym_call_expression] = STATE(2037),
    [sym_new_expression] = STATE(2037),
    [sym_await_expression] = STATE(2037),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2037),
    [sym_augmented_assignment_expression] = STATE(2037),
    [sym_ternary_expression] = STATE(2037),
    [sym_binary_expression] = STATE(2037),
    [sym_unary_expression] = STATE(2037),
    [sym_update_expression] = STATE(2037),
    [sym_template_string] = STATE(2037),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4420),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4422),
    [sym_number] = ACTIONS(4420),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4422),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4422),
    [sym_false] = ACTIONS(4422),
    [sym_null] = ACTIONS(4422),
    [sym_undefined] = ACTIONS(4422),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1844] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2038),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2038),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1845] = {
    [sym_parenthesized_expression] = STATE(2039),
    [sym__expression] = STATE(2039),
    [sym_yield_expression] = STATE(2039),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2039),
    [sym_jsx_fragment] = STATE(2039),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2039),
    [sym_anonymous_class] = STATE(2039),
    [sym_class] = STATE(2039),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2039),
    [sym_generator_function] = STATE(2039),
    [sym_call_expression] = STATE(2039),
    [sym_new_expression] = STATE(2039),
    [sym_await_expression] = STATE(2039),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2039),
    [sym_augmented_assignment_expression] = STATE(2039),
    [sym_ternary_expression] = STATE(2039),
    [sym_binary_expression] = STATE(2039),
    [sym_unary_expression] = STATE(2039),
    [sym_update_expression] = STATE(2039),
    [sym_template_string] = STATE(2039),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4424),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4426),
    [sym_number] = ACTIONS(4424),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4426),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4426),
    [sym_false] = ACTIONS(4426),
    [sym_null] = ACTIONS(4426),
    [sym_undefined] = ACTIONS(4426),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1846] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2040),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2040),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1847] = {
    [sym_parenthesized_expression] = STATE(2041),
    [sym__expression] = STATE(2041),
    [sym_yield_expression] = STATE(2041),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2041),
    [sym_jsx_fragment] = STATE(2041),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2041),
    [sym_anonymous_class] = STATE(2041),
    [sym_class] = STATE(2041),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2041),
    [sym_generator_function] = STATE(2041),
    [sym_call_expression] = STATE(2041),
    [sym_new_expression] = STATE(2041),
    [sym_await_expression] = STATE(2041),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2041),
    [sym_augmented_assignment_expression] = STATE(2041),
    [sym_ternary_expression] = STATE(2041),
    [sym_binary_expression] = STATE(2041),
    [sym_unary_expression] = STATE(2041),
    [sym_update_expression] = STATE(2041),
    [sym_template_string] = STATE(2041),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4428),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4430),
    [sym_number] = ACTIONS(4428),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4430),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4430),
    [sym_false] = ACTIONS(4430),
    [sym_null] = ACTIONS(4430),
    [sym_undefined] = ACTIONS(4430),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1848] = {
    [anon_sym_RPAREN] = ACTIONS(4432),
    [sym_comment] = ACTIONS(71),
  },
  [1849] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4434),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1850] = {
    [sym_export_statement] = STATE(2044),
    [sym__declaration] = STATE(2044),
    [sym_import_statement] = STATE(2044),
    [sym_expression_statement] = STATE(2044),
    [sym_variable_declaration] = STATE(2044),
    [sym_lexical_declaration] = STATE(2044),
    [sym_statement_block] = STATE(2044),
    [sym_if_statement] = STATE(2044),
    [sym_switch_statement] = STATE(2044),
    [sym_for_statement] = STATE(2044),
    [sym_for_in_statement] = STATE(2044),
    [sym_for_of_statement] = STATE(2044),
    [sym_while_statement] = STATE(2044),
    [sym_do_statement] = STATE(2044),
    [sym_try_statement] = STATE(2044),
    [sym_with_statement] = STATE(2044),
    [sym_break_statement] = STATE(2044),
    [sym_continue_statement] = STATE(2044),
    [sym_debugger_statement] = STATE(2044),
    [sym_return_statement] = STATE(2044),
    [sym_throw_statement] = STATE(2044),
    [sym_empty_statement] = STATE(2044),
    [sym_labeled_statement] = STATE(2044),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1851] = {
    [anon_sym_RPAREN] = ACTIONS(4436),
    [sym_comment] = ACTIONS(71),
  },
  [1852] = {
    [anon_sym_RPAREN] = ACTIONS(4438),
    [sym_comment] = ACTIONS(71),
  },
  [1853] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4440),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1854] = {
    [anon_sym_RPAREN] = ACTIONS(4442),
    [sym_comment] = ACTIONS(71),
  },
  [1855] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4444),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1856] = {
    [ts_builtin_sym_end] = ACTIONS(3786),
    [anon_sym_export] = ACTIONS(3788),
    [anon_sym_LBRACE] = ACTIONS(3786),
    [anon_sym_RBRACE] = ACTIONS(3786),
    [anon_sym_import] = ACTIONS(3788),
    [anon_sym_var] = ACTIONS(3788),
    [anon_sym_let] = ACTIONS(3788),
    [anon_sym_const] = ACTIONS(3788),
    [anon_sym_if] = ACTIONS(3788),
    [anon_sym_else] = ACTIONS(3788),
    [anon_sym_switch] = ACTIONS(3788),
    [anon_sym_for] = ACTIONS(3788),
    [anon_sym_LPAREN] = ACTIONS(3786),
    [anon_sym_while] = ACTIONS(3788),
    [anon_sym_do] = ACTIONS(3788),
    [anon_sym_try] = ACTIONS(3788),
    [anon_sym_with] = ACTIONS(3788),
    [anon_sym_break] = ACTIONS(3788),
    [anon_sym_continue] = ACTIONS(3788),
    [anon_sym_debugger] = ACTIONS(3788),
    [anon_sym_return] = ACTIONS(3788),
    [anon_sym_throw] = ACTIONS(3788),
    [anon_sym_SEMI] = ACTIONS(3786),
    [anon_sym_yield] = ACTIONS(3788),
    [anon_sym_LBRACK] = ACTIONS(3786),
    [anon_sym_LT] = ACTIONS(3786),
    [anon_sym_class] = ACTIONS(3788),
    [anon_sym_async] = ACTIONS(3788),
    [anon_sym_function] = ACTIONS(3788),
    [anon_sym_EQ_GT] = ACTIONS(3786),
    [anon_sym_new] = ACTIONS(3788),
    [anon_sym_await] = ACTIONS(3788),
    [anon_sym_PLUS] = ACTIONS(3788),
    [anon_sym_DASH] = ACTIONS(3788),
    [anon_sym_BANG] = ACTIONS(3786),
    [anon_sym_TILDE] = ACTIONS(3786),
    [anon_sym_typeof] = ACTIONS(3788),
    [anon_sym_void] = ACTIONS(3788),
    [anon_sym_delete] = ACTIONS(3788),
    [anon_sym_PLUS_PLUS] = ACTIONS(3786),
    [anon_sym_DASH_DASH] = ACTIONS(3786),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3786),
    [anon_sym_BQUOTE] = ACTIONS(3786),
    [sym_regex] = ACTIONS(3788),
    [sym_number] = ACTIONS(3786),
    [sym_identifier] = ACTIONS(3788),
    [sym_this] = ACTIONS(3788),
    [sym_super] = ACTIONS(3788),
    [sym_true] = ACTIONS(3788),
    [sym_false] = ACTIONS(3788),
    [sym_null] = ACTIONS(3788),
    [sym_undefined] = ACTIONS(3788),
    [anon_sym_AT] = ACTIONS(3786),
    [anon_sym_get] = ACTIONS(3788),
    [anon_sym_set] = ACTIONS(3788),
  },
  [1857] = {
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1858] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4446),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1859] = {
    [anon_sym_RPAREN] = ACTIONS(4448),
    [sym_comment] = ACTIONS(71),
  },
  [1860] = {
    [ts_builtin_sym_end] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1861] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [ts_builtin_sym_end] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_STAR] = ACTIONS(3035),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_else] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_in] = ACTIONS(3037),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3037),
    [anon_sym_GT] = ACTIONS(3037),
    [anon_sym_SLASH] = ACTIONS(3037),
    [anon_sym_DOT] = ACTIONS(3037),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_QMARK] = ACTIONS(3035),
    [anon_sym_AMP_AMP] = ACTIONS(3035),
    [anon_sym_PIPE_PIPE] = ACTIONS(3035),
    [anon_sym_GT_GT] = ACTIONS(3037),
    [anon_sym_GT_GT_GT] = ACTIONS(3035),
    [anon_sym_LT_LT] = ACTIONS(3035),
    [anon_sym_AMP] = ACTIONS(3037),
    [anon_sym_CARET] = ACTIONS(3035),
    [anon_sym_PIPE] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_PERCENT] = ACTIONS(3035),
    [anon_sym_LT_EQ] = ACTIONS(3035),
    [anon_sym_EQ_EQ] = ACTIONS(3037),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3035),
    [anon_sym_BANG_EQ] = ACTIONS(3037),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3035),
    [anon_sym_GT_EQ] = ACTIONS(3035),
    [anon_sym_instanceof] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3037),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [1862] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [ts_builtin_sym_end] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_STAR] = ACTIONS(3856),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_COMMA] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_else] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_in] = ACTIONS(3858),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_GT] = ACTIONS(3858),
    [anon_sym_SLASH] = ACTIONS(3858),
    [anon_sym_DOT] = ACTIONS(3858),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_QMARK] = ACTIONS(3856),
    [anon_sym_AMP_AMP] = ACTIONS(3856),
    [anon_sym_PIPE_PIPE] = ACTIONS(3856),
    [anon_sym_GT_GT] = ACTIONS(3858),
    [anon_sym_GT_GT_GT] = ACTIONS(3856),
    [anon_sym_LT_LT] = ACTIONS(3856),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_CARET] = ACTIONS(3856),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_PERCENT] = ACTIONS(3856),
    [anon_sym_LT_EQ] = ACTIONS(3856),
    [anon_sym_EQ_EQ] = ACTIONS(3858),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3856),
    [anon_sym_BANG_EQ] = ACTIONS(3858),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3856),
    [anon_sym_GT_EQ] = ACTIONS(3856),
    [anon_sym_instanceof] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3858),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [1863] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [ts_builtin_sym_end] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_STAR] = ACTIONS(3860),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_COMMA] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_else] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_in] = ACTIONS(3862),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_GT] = ACTIONS(3862),
    [anon_sym_SLASH] = ACTIONS(3862),
    [anon_sym_DOT] = ACTIONS(3862),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_QMARK] = ACTIONS(3860),
    [anon_sym_AMP_AMP] = ACTIONS(3860),
    [anon_sym_PIPE_PIPE] = ACTIONS(3860),
    [anon_sym_GT_GT] = ACTIONS(3862),
    [anon_sym_GT_GT_GT] = ACTIONS(3860),
    [anon_sym_LT_LT] = ACTIONS(3860),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_CARET] = ACTIONS(3860),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_PERCENT] = ACTIONS(3860),
    [anon_sym_LT_EQ] = ACTIONS(3860),
    [anon_sym_EQ_EQ] = ACTIONS(3862),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3860),
    [anon_sym_BANG_EQ] = ACTIONS(3862),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3860),
    [anon_sym_GT_EQ] = ACTIONS(3860),
    [anon_sym_instanceof] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3862),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [1864] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_in] = ACTIONS(1590),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1590),
    [anon_sym_SLASH] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1590),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_QMARK] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1588),
    [anon_sym_PIPE_PIPE] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1590),
    [anon_sym_GT_GT_GT] = ACTIONS(1588),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1590),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_LT_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1588),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1588),
    [anon_sym_instanceof] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1590),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [1865] = {
    [ts_builtin_sym_end] = ACTIONS(3930),
    [anon_sym_export] = ACTIONS(3932),
    [anon_sym_LBRACE] = ACTIONS(3930),
    [anon_sym_RBRACE] = ACTIONS(3930),
    [anon_sym_import] = ACTIONS(3932),
    [anon_sym_var] = ACTIONS(3932),
    [anon_sym_let] = ACTIONS(3932),
    [anon_sym_const] = ACTIONS(3932),
    [anon_sym_if] = ACTIONS(3932),
    [anon_sym_else] = ACTIONS(3932),
    [anon_sym_switch] = ACTIONS(3932),
    [anon_sym_for] = ACTIONS(3932),
    [anon_sym_LPAREN] = ACTIONS(3930),
    [anon_sym_while] = ACTIONS(3932),
    [anon_sym_do] = ACTIONS(3932),
    [anon_sym_try] = ACTIONS(3932),
    [anon_sym_with] = ACTIONS(3932),
    [anon_sym_break] = ACTIONS(3932),
    [anon_sym_continue] = ACTIONS(3932),
    [anon_sym_debugger] = ACTIONS(3932),
    [anon_sym_return] = ACTIONS(3932),
    [anon_sym_throw] = ACTIONS(3932),
    [anon_sym_SEMI] = ACTIONS(3930),
    [anon_sym_yield] = ACTIONS(3932),
    [anon_sym_LBRACK] = ACTIONS(3930),
    [anon_sym_LT] = ACTIONS(3930),
    [anon_sym_class] = ACTIONS(3932),
    [anon_sym_async] = ACTIONS(3932),
    [anon_sym_function] = ACTIONS(3932),
    [anon_sym_EQ_GT] = ACTIONS(3930),
    [anon_sym_new] = ACTIONS(3932),
    [anon_sym_await] = ACTIONS(3932),
    [anon_sym_PLUS] = ACTIONS(3932),
    [anon_sym_DASH] = ACTIONS(3932),
    [anon_sym_BANG] = ACTIONS(3930),
    [anon_sym_TILDE] = ACTIONS(3930),
    [anon_sym_typeof] = ACTIONS(3932),
    [anon_sym_void] = ACTIONS(3932),
    [anon_sym_delete] = ACTIONS(3932),
    [anon_sym_PLUS_PLUS] = ACTIONS(3930),
    [anon_sym_DASH_DASH] = ACTIONS(3930),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3930),
    [anon_sym_BQUOTE] = ACTIONS(3930),
    [sym_regex] = ACTIONS(3932),
    [sym_number] = ACTIONS(3930),
    [sym_identifier] = ACTIONS(3932),
    [sym_this] = ACTIONS(3932),
    [sym_super] = ACTIONS(3932),
    [sym_true] = ACTIONS(3932),
    [sym_false] = ACTIONS(3932),
    [sym_null] = ACTIONS(3932),
    [sym_undefined] = ACTIONS(3932),
    [anon_sym_AT] = ACTIONS(3930),
    [anon_sym_get] = ACTIONS(3932),
    [anon_sym_set] = ACTIONS(3932),
  },
  [1866] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [ts_builtin_sym_end] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_COMMA] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_else] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_in] = ACTIONS(3936),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3936),
    [anon_sym_GT] = ACTIONS(3936),
    [anon_sym_SLASH] = ACTIONS(3936),
    [anon_sym_DOT] = ACTIONS(3936),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_QMARK] = ACTIONS(3934),
    [anon_sym_AMP_AMP] = ACTIONS(3934),
    [anon_sym_PIPE_PIPE] = ACTIONS(3934),
    [anon_sym_GT_GT] = ACTIONS(3936),
    [anon_sym_GT_GT_GT] = ACTIONS(3934),
    [anon_sym_LT_LT] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3936),
    [anon_sym_CARET] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_PERCENT] = ACTIONS(3934),
    [anon_sym_LT_EQ] = ACTIONS(3934),
    [anon_sym_EQ_EQ] = ACTIONS(3936),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3934),
    [anon_sym_BANG_EQ] = ACTIONS(3936),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3934),
    [anon_sym_GT_EQ] = ACTIONS(3934),
    [anon_sym_instanceof] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3936),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [1867] = {
    [sym__from_clause] = STATE(2052),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [1868] = {
    [sym_parenthesized_expression] = STATE(2053),
    [sym__expression] = STATE(2053),
    [sym_yield_expression] = STATE(2053),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(2053),
    [sym_jsx_fragment] = STATE(2053),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2053),
    [sym_anonymous_class] = STATE(2053),
    [sym_class] = STATE(2053),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2053),
    [sym_generator_function] = STATE(2053),
    [sym_call_expression] = STATE(2053),
    [sym_new_expression] = STATE(2053),
    [sym_await_expression] = STATE(2053),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(2053),
    [sym_augmented_assignment_expression] = STATE(2053),
    [sym_ternary_expression] = STATE(2053),
    [sym_binary_expression] = STATE(2053),
    [sym_unary_expression] = STATE(2053),
    [sym_update_expression] = STATE(2053),
    [sym_template_string] = STATE(2053),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4450),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4452),
    [sym_number] = ACTIONS(4450),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(4452),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4452),
    [sym_false] = ACTIONS(4452),
    [sym_null] = ACTIONS(4452),
    [sym_undefined] = ACTIONS(4452),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [1869] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4454),
  },
  [1870] = {
    [anon_sym_function] = ACTIONS(4456),
    [sym_comment] = ACTIONS(71),
  },
  [1871] = {
    [sym_formal_parameters] = STATE(2058),
    [anon_sym_STAR] = ACTIONS(4458),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4460),
  },
  [1872] = {
    [sym__from_clause] = STATE(2052),
    [sym__automatic_semicolon] = ACTIONS(4462),
    [anon_sym_from] = ACTIONS(410),
    [anon_sym_SEMI] = ACTIONS(4462),
    [sym_comment] = ACTIONS(71),
  },
  [1873] = {
    [anon_sym_export] = ACTIONS(454),
    [anon_sym_default] = ACTIONS(454),
    [anon_sym_LBRACE] = ACTIONS(452),
    [anon_sym_RBRACE] = ACTIONS(452),
    [anon_sym_import] = ACTIONS(454),
    [anon_sym_var] = ACTIONS(454),
    [anon_sym_let] = ACTIONS(454),
    [anon_sym_const] = ACTIONS(454),
    [anon_sym_if] = ACTIONS(454),
    [anon_sym_switch] = ACTIONS(454),
    [anon_sym_for] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(452),
    [anon_sym_while] = ACTIONS(454),
    [anon_sym_do] = ACTIONS(454),
    [anon_sym_try] = ACTIONS(454),
    [anon_sym_with] = ACTIONS(454),
    [anon_sym_break] = ACTIONS(454),
    [anon_sym_continue] = ACTIONS(454),
    [anon_sym_debugger] = ACTIONS(454),
    [anon_sym_return] = ACTIONS(454),
    [anon_sym_throw] = ACTIONS(454),
    [anon_sym_SEMI] = ACTIONS(452),
    [anon_sym_case] = ACTIONS(454),
    [anon_sym_yield] = ACTIONS(454),
    [anon_sym_LBRACK] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(452),
    [anon_sym_class] = ACTIONS(454),
    [anon_sym_async] = ACTIONS(454),
    [anon_sym_function] = ACTIONS(454),
    [anon_sym_EQ_GT] = ACTIONS(452),
    [anon_sym_new] = ACTIONS(454),
    [anon_sym_await] = ACTIONS(454),
    [anon_sym_PLUS] = ACTIONS(454),
    [anon_sym_DASH] = ACTIONS(454),
    [anon_sym_BANG] = ACTIONS(452),
    [anon_sym_TILDE] = ACTIONS(452),
    [anon_sym_typeof] = ACTIONS(454),
    [anon_sym_void] = ACTIONS(454),
    [anon_sym_delete] = ACTIONS(454),
    [anon_sym_PLUS_PLUS] = ACTIONS(452),
    [anon_sym_DASH_DASH] = ACTIONS(452),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(452),
    [anon_sym_BQUOTE] = ACTIONS(452),
    [sym_regex] = ACTIONS(454),
    [sym_number] = ACTIONS(452),
    [sym_identifier] = ACTIONS(454),
    [sym_this] = ACTIONS(454),
    [sym_super] = ACTIONS(454),
    [sym_true] = ACTIONS(454),
    [sym_false] = ACTIONS(454),
    [sym_null] = ACTIONS(454),
    [sym_undefined] = ACTIONS(454),
    [anon_sym_AT] = ACTIONS(452),
    [anon_sym_get] = ACTIONS(454),
    [anon_sym_set] = ACTIONS(454),
  },
  [1874] = {
    [sym__automatic_semicolon] = ACTIONS(4106),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_default] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_case] = ACTIONS(402),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(400),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_BANG] = ACTIONS(400),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [1875] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(4464),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [1876] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(486),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(486),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(480),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [1877] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4466),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1878] = {
    [sym__automatic_semicolon] = ACTIONS(4468),
    [anon_sym_SEMI] = ACTIONS(4468),
    [sym_comment] = ACTIONS(71),
  },
  [1879] = {
    [sym__from_clause] = STATE(2063),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [1880] = {
    [aux_sym_variable_declaration_repeat1] = STATE(2065),
    [sym__automatic_semicolon] = ACTIONS(4470),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4470),
    [sym_comment] = ACTIONS(71),
  },
  [1881] = {
    [aux_sym_variable_declaration_repeat1] = STATE(2067),
    [sym__automatic_semicolon] = ACTIONS(4472),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4472),
    [sym_comment] = ACTIONS(71),
  },
  [1882] = {
    [sym_export_statement] = STATE(2090),
    [sym__declaration] = STATE(2090),
    [sym_import_statement] = STATE(2090),
    [sym_expression_statement] = STATE(2090),
    [sym_variable_declaration] = STATE(2090),
    [sym_lexical_declaration] = STATE(2090),
    [sym_statement_block] = STATE(2090),
    [sym_if_statement] = STATE(2090),
    [sym_switch_statement] = STATE(2090),
    [sym_for_statement] = STATE(2090),
    [sym_for_in_statement] = STATE(2090),
    [sym_for_of_statement] = STATE(2090),
    [sym_while_statement] = STATE(2090),
    [sym_do_statement] = STATE(2090),
    [sym_try_statement] = STATE(2090),
    [sym_with_statement] = STATE(2090),
    [sym_break_statement] = STATE(2090),
    [sym_continue_statement] = STATE(2090),
    [sym_debugger_statement] = STATE(2090),
    [sym_return_statement] = STATE(2090),
    [sym_throw_statement] = STATE(2090),
    [sym_empty_statement] = STATE(2090),
    [sym_labeled_statement] = STATE(2090),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1883] = {
    [sym_switch_body] = STATE(2096),
    [anon_sym_LBRACE] = ACTIONS(4518),
    [sym_comment] = ACTIONS(71),
  },
  [1884] = {
    [sym_expression_statement] = STATE(2100),
    [sym_variable_declaration] = STATE(2100),
    [sym_lexical_declaration] = STATE(2100),
    [sym_empty_statement] = STATE(2100),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(2101),
    [sym_array] = STATE(2102),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(4520),
    [anon_sym_let] = ACTIONS(4522),
    [anon_sym_const] = ACTIONS(4522),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4524),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1885] = {
    [sym_export_statement] = STATE(2103),
    [sym__declaration] = STATE(2103),
    [sym_import_statement] = STATE(2103),
    [sym_expression_statement] = STATE(2103),
    [sym_variable_declaration] = STATE(2103),
    [sym_lexical_declaration] = STATE(2103),
    [sym_statement_block] = STATE(2103),
    [sym_if_statement] = STATE(2103),
    [sym_switch_statement] = STATE(2103),
    [sym_for_statement] = STATE(2103),
    [sym_for_in_statement] = STATE(2103),
    [sym_for_of_statement] = STATE(2103),
    [sym_while_statement] = STATE(2103),
    [sym_do_statement] = STATE(2103),
    [sym_try_statement] = STATE(2103),
    [sym_with_statement] = STATE(2103),
    [sym_break_statement] = STATE(2103),
    [sym_continue_statement] = STATE(2103),
    [sym_debugger_statement] = STATE(2103),
    [sym_return_statement] = STATE(2103),
    [sym_throw_statement] = STATE(2103),
    [sym_empty_statement] = STATE(2103),
    [sym_labeled_statement] = STATE(2103),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1886] = {
    [anon_sym_while] = ACTIONS(4526),
    [sym_comment] = ACTIONS(71),
  },
  [1887] = {
    [sym_export_statement] = STATE(2106),
    [sym__declaration] = STATE(2106),
    [sym_import_statement] = STATE(2106),
    [sym_expression_statement] = STATE(2106),
    [sym_variable_declaration] = STATE(2106),
    [sym_lexical_declaration] = STATE(2106),
    [sym_statement_block] = STATE(2106),
    [sym_if_statement] = STATE(2106),
    [sym_switch_statement] = STATE(2106),
    [sym_for_statement] = STATE(2106),
    [sym_for_in_statement] = STATE(2106),
    [sym_for_of_statement] = STATE(2106),
    [sym_while_statement] = STATE(2106),
    [sym_do_statement] = STATE(2106),
    [sym_try_statement] = STATE(2106),
    [sym_with_statement] = STATE(2106),
    [sym_break_statement] = STATE(2106),
    [sym_continue_statement] = STATE(2106),
    [sym_debugger_statement] = STATE(2106),
    [sym_return_statement] = STATE(2106),
    [sym_throw_statement] = STATE(2106),
    [sym_empty_statement] = STATE(2106),
    [sym_labeled_statement] = STATE(2106),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2106),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4528),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1888] = {
    [sym_catch_clause] = STATE(2109),
    [sym_finally_clause] = STATE(2110),
    [anon_sym_export] = ACTIONS(844),
    [anon_sym_default] = ACTIONS(844),
    [anon_sym_LBRACE] = ACTIONS(842),
    [anon_sym_RBRACE] = ACTIONS(842),
    [anon_sym_import] = ACTIONS(844),
    [anon_sym_var] = ACTIONS(844),
    [anon_sym_let] = ACTIONS(844),
    [anon_sym_const] = ACTIONS(844),
    [anon_sym_if] = ACTIONS(844),
    [anon_sym_switch] = ACTIONS(844),
    [anon_sym_for] = ACTIONS(844),
    [anon_sym_LPAREN] = ACTIONS(842),
    [anon_sym_while] = ACTIONS(844),
    [anon_sym_do] = ACTIONS(844),
    [anon_sym_try] = ACTIONS(844),
    [anon_sym_with] = ACTIONS(844),
    [anon_sym_break] = ACTIONS(844),
    [anon_sym_continue] = ACTIONS(844),
    [anon_sym_debugger] = ACTIONS(844),
    [anon_sym_return] = ACTIONS(844),
    [anon_sym_throw] = ACTIONS(844),
    [anon_sym_SEMI] = ACTIONS(842),
    [anon_sym_case] = ACTIONS(844),
    [anon_sym_catch] = ACTIONS(4530),
    [anon_sym_finally] = ACTIONS(4532),
    [anon_sym_yield] = ACTIONS(844),
    [anon_sym_LBRACK] = ACTIONS(842),
    [anon_sym_LT] = ACTIONS(842),
    [anon_sym_class] = ACTIONS(844),
    [anon_sym_async] = ACTIONS(844),
    [anon_sym_function] = ACTIONS(844),
    [anon_sym_EQ_GT] = ACTIONS(842),
    [anon_sym_new] = ACTIONS(844),
    [anon_sym_await] = ACTIONS(844),
    [anon_sym_PLUS] = ACTIONS(844),
    [anon_sym_DASH] = ACTIONS(844),
    [anon_sym_BANG] = ACTIONS(842),
    [anon_sym_TILDE] = ACTIONS(842),
    [anon_sym_typeof] = ACTIONS(844),
    [anon_sym_void] = ACTIONS(844),
    [anon_sym_delete] = ACTIONS(844),
    [anon_sym_PLUS_PLUS] = ACTIONS(842),
    [anon_sym_DASH_DASH] = ACTIONS(842),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(842),
    [anon_sym_BQUOTE] = ACTIONS(842),
    [sym_regex] = ACTIONS(844),
    [sym_number] = ACTIONS(842),
    [sym_identifier] = ACTIONS(844),
    [sym_this] = ACTIONS(844),
    [sym_super] = ACTIONS(844),
    [sym_true] = ACTIONS(844),
    [sym_false] = ACTIONS(844),
    [sym_null] = ACTIONS(844),
    [sym_undefined] = ACTIONS(844),
    [anon_sym_AT] = ACTIONS(842),
    [anon_sym_get] = ACTIONS(844),
    [anon_sym_set] = ACTIONS(844),
  },
  [1889] = {
    [sym_export_statement] = STATE(2111),
    [sym__declaration] = STATE(2111),
    [sym_import_statement] = STATE(2111),
    [sym_expression_statement] = STATE(2111),
    [sym_variable_declaration] = STATE(2111),
    [sym_lexical_declaration] = STATE(2111),
    [sym_statement_block] = STATE(2111),
    [sym_if_statement] = STATE(2111),
    [sym_switch_statement] = STATE(2111),
    [sym_for_statement] = STATE(2111),
    [sym_for_in_statement] = STATE(2111),
    [sym_for_of_statement] = STATE(2111),
    [sym_while_statement] = STATE(2111),
    [sym_do_statement] = STATE(2111),
    [sym_try_statement] = STATE(2111),
    [sym_with_statement] = STATE(2111),
    [sym_break_statement] = STATE(2111),
    [sym_continue_statement] = STATE(2111),
    [sym_debugger_statement] = STATE(2111),
    [sym_return_statement] = STATE(2111),
    [sym_throw_statement] = STATE(2111),
    [sym_empty_statement] = STATE(2111),
    [sym_labeled_statement] = STATE(2111),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1890] = {
    [anon_sym_export] = ACTIONS(852),
    [anon_sym_default] = ACTIONS(852),
    [anon_sym_LBRACE] = ACTIONS(850),
    [anon_sym_RBRACE] = ACTIONS(850),
    [anon_sym_import] = ACTIONS(852),
    [anon_sym_var] = ACTIONS(852),
    [anon_sym_let] = ACTIONS(852),
    [anon_sym_const] = ACTIONS(852),
    [anon_sym_if] = ACTIONS(852),
    [anon_sym_switch] = ACTIONS(852),
    [anon_sym_for] = ACTIONS(852),
    [anon_sym_LPAREN] = ACTIONS(850),
    [anon_sym_while] = ACTIONS(852),
    [anon_sym_do] = ACTIONS(852),
    [anon_sym_try] = ACTIONS(852),
    [anon_sym_with] = ACTIONS(852),
    [anon_sym_break] = ACTIONS(852),
    [anon_sym_continue] = ACTIONS(852),
    [anon_sym_debugger] = ACTIONS(852),
    [anon_sym_return] = ACTIONS(852),
    [anon_sym_throw] = ACTIONS(852),
    [anon_sym_SEMI] = ACTIONS(850),
    [anon_sym_case] = ACTIONS(852),
    [anon_sym_yield] = ACTIONS(852),
    [anon_sym_LBRACK] = ACTIONS(850),
    [anon_sym_LT] = ACTIONS(850),
    [anon_sym_class] = ACTIONS(852),
    [anon_sym_async] = ACTIONS(852),
    [anon_sym_function] = ACTIONS(852),
    [anon_sym_EQ_GT] = ACTIONS(850),
    [anon_sym_new] = ACTIONS(852),
    [anon_sym_await] = ACTIONS(852),
    [anon_sym_PLUS] = ACTIONS(852),
    [anon_sym_DASH] = ACTIONS(852),
    [anon_sym_BANG] = ACTIONS(850),
    [anon_sym_TILDE] = ACTIONS(850),
    [anon_sym_typeof] = ACTIONS(852),
    [anon_sym_void] = ACTIONS(852),
    [anon_sym_delete] = ACTIONS(852),
    [anon_sym_PLUS_PLUS] = ACTIONS(850),
    [anon_sym_DASH_DASH] = ACTIONS(850),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(850),
    [anon_sym_BQUOTE] = ACTIONS(850),
    [sym_regex] = ACTIONS(852),
    [sym_number] = ACTIONS(850),
    [sym_identifier] = ACTIONS(852),
    [sym_this] = ACTIONS(852),
    [sym_super] = ACTIONS(852),
    [sym_true] = ACTIONS(852),
    [sym_false] = ACTIONS(852),
    [sym_null] = ACTIONS(852),
    [sym_undefined] = ACTIONS(852),
    [anon_sym_AT] = ACTIONS(850),
    [anon_sym_get] = ACTIONS(852),
    [anon_sym_set] = ACTIONS(852),
  },
  [1891] = {
    [sym__automatic_semicolon] = ACTIONS(4534),
    [anon_sym_SEMI] = ACTIONS(4534),
    [sym_comment] = ACTIONS(71),
  },
  [1892] = {
    [anon_sym_export] = ACTIONS(858),
    [anon_sym_default] = ACTIONS(858),
    [anon_sym_LBRACE] = ACTIONS(856),
    [anon_sym_RBRACE] = ACTIONS(856),
    [anon_sym_import] = ACTIONS(858),
    [anon_sym_var] = ACTIONS(858),
    [anon_sym_let] = ACTIONS(858),
    [anon_sym_const] = ACTIONS(858),
    [anon_sym_if] = ACTIONS(858),
    [anon_sym_switch] = ACTIONS(858),
    [anon_sym_for] = ACTIONS(858),
    [anon_sym_LPAREN] = ACTIONS(856),
    [anon_sym_while] = ACTIONS(858),
    [anon_sym_do] = ACTIONS(858),
    [anon_sym_try] = ACTIONS(858),
    [anon_sym_with] = ACTIONS(858),
    [anon_sym_break] = ACTIONS(858),
    [anon_sym_continue] = ACTIONS(858),
    [anon_sym_debugger] = ACTIONS(858),
    [anon_sym_return] = ACTIONS(858),
    [anon_sym_throw] = ACTIONS(858),
    [anon_sym_SEMI] = ACTIONS(856),
    [anon_sym_case] = ACTIONS(858),
    [anon_sym_yield] = ACTIONS(858),
    [anon_sym_LBRACK] = ACTIONS(856),
    [anon_sym_LT] = ACTIONS(856),
    [anon_sym_class] = ACTIONS(858),
    [anon_sym_async] = ACTIONS(858),
    [anon_sym_function] = ACTIONS(858),
    [anon_sym_EQ_GT] = ACTIONS(856),
    [anon_sym_new] = ACTIONS(858),
    [anon_sym_await] = ACTIONS(858),
    [anon_sym_PLUS] = ACTIONS(858),
    [anon_sym_DASH] = ACTIONS(858),
    [anon_sym_BANG] = ACTIONS(856),
    [anon_sym_TILDE] = ACTIONS(856),
    [anon_sym_typeof] = ACTIONS(858),
    [anon_sym_void] = ACTIONS(858),
    [anon_sym_delete] = ACTIONS(858),
    [anon_sym_PLUS_PLUS] = ACTIONS(856),
    [anon_sym_DASH_DASH] = ACTIONS(856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(856),
    [anon_sym_BQUOTE] = ACTIONS(856),
    [sym_regex] = ACTIONS(858),
    [sym_number] = ACTIONS(856),
    [sym_identifier] = ACTIONS(858),
    [sym_this] = ACTIONS(858),
    [sym_super] = ACTIONS(858),
    [sym_true] = ACTIONS(858),
    [sym_false] = ACTIONS(858),
    [sym_null] = ACTIONS(858),
    [sym_undefined] = ACTIONS(858),
    [anon_sym_AT] = ACTIONS(856),
    [anon_sym_get] = ACTIONS(858),
    [anon_sym_set] = ACTIONS(858),
  },
  [1893] = {
    [sym__automatic_semicolon] = ACTIONS(4536),
    [anon_sym_SEMI] = ACTIONS(4536),
    [sym_comment] = ACTIONS(71),
  },
  [1894] = {
    [anon_sym_export] = ACTIONS(864),
    [anon_sym_default] = ACTIONS(864),
    [anon_sym_LBRACE] = ACTIONS(862),
    [anon_sym_RBRACE] = ACTIONS(862),
    [anon_sym_import] = ACTIONS(864),
    [anon_sym_var] = ACTIONS(864),
    [anon_sym_let] = ACTIONS(864),
    [anon_sym_const] = ACTIONS(864),
    [anon_sym_if] = ACTIONS(864),
    [anon_sym_switch] = ACTIONS(864),
    [anon_sym_for] = ACTIONS(864),
    [anon_sym_LPAREN] = ACTIONS(862),
    [anon_sym_while] = ACTIONS(864),
    [anon_sym_do] = ACTIONS(864),
    [anon_sym_try] = ACTIONS(864),
    [anon_sym_with] = ACTIONS(864),
    [anon_sym_break] = ACTIONS(864),
    [anon_sym_continue] = ACTIONS(864),
    [anon_sym_debugger] = ACTIONS(864),
    [anon_sym_return] = ACTIONS(864),
    [anon_sym_throw] = ACTIONS(864),
    [anon_sym_SEMI] = ACTIONS(862),
    [anon_sym_case] = ACTIONS(864),
    [anon_sym_yield] = ACTIONS(864),
    [anon_sym_LBRACK] = ACTIONS(862),
    [anon_sym_LT] = ACTIONS(862),
    [anon_sym_class] = ACTIONS(864),
    [anon_sym_async] = ACTIONS(864),
    [anon_sym_function] = ACTIONS(864),
    [anon_sym_EQ_GT] = ACTIONS(862),
    [anon_sym_new] = ACTIONS(864),
    [anon_sym_await] = ACTIONS(864),
    [anon_sym_PLUS] = ACTIONS(864),
    [anon_sym_DASH] = ACTIONS(864),
    [anon_sym_BANG] = ACTIONS(862),
    [anon_sym_TILDE] = ACTIONS(862),
    [anon_sym_typeof] = ACTIONS(864),
    [anon_sym_void] = ACTIONS(864),
    [anon_sym_delete] = ACTIONS(864),
    [anon_sym_PLUS_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(862),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(862),
    [anon_sym_BQUOTE] = ACTIONS(862),
    [sym_regex] = ACTIONS(864),
    [sym_number] = ACTIONS(862),
    [sym_identifier] = ACTIONS(864),
    [sym_this] = ACTIONS(864),
    [sym_super] = ACTIONS(864),
    [sym_true] = ACTIONS(864),
    [sym_false] = ACTIONS(864),
    [sym_null] = ACTIONS(864),
    [sym_undefined] = ACTIONS(864),
    [anon_sym_AT] = ACTIONS(862),
    [anon_sym_get] = ACTIONS(864),
    [anon_sym_set] = ACTIONS(864),
  },
  [1895] = {
    [anon_sym_export] = ACTIONS(868),
    [anon_sym_default] = ACTIONS(868),
    [anon_sym_LBRACE] = ACTIONS(866),
    [anon_sym_RBRACE] = ACTIONS(866),
    [anon_sym_import] = ACTIONS(868),
    [anon_sym_var] = ACTIONS(868),
    [anon_sym_let] = ACTIONS(868),
    [anon_sym_const] = ACTIONS(868),
    [anon_sym_if] = ACTIONS(868),
    [anon_sym_switch] = ACTIONS(868),
    [anon_sym_for] = ACTIONS(868),
    [anon_sym_LPAREN] = ACTIONS(866),
    [anon_sym_while] = ACTIONS(868),
    [anon_sym_do] = ACTIONS(868),
    [anon_sym_try] = ACTIONS(868),
    [anon_sym_with] = ACTIONS(868),
    [anon_sym_break] = ACTIONS(868),
    [anon_sym_continue] = ACTIONS(868),
    [anon_sym_debugger] = ACTIONS(868),
    [anon_sym_return] = ACTIONS(868),
    [anon_sym_throw] = ACTIONS(868),
    [anon_sym_SEMI] = ACTIONS(866),
    [anon_sym_case] = ACTIONS(868),
    [anon_sym_yield] = ACTIONS(868),
    [anon_sym_LBRACK] = ACTIONS(866),
    [anon_sym_LT] = ACTIONS(866),
    [anon_sym_class] = ACTIONS(868),
    [anon_sym_async] = ACTIONS(868),
    [anon_sym_function] = ACTIONS(868),
    [anon_sym_EQ_GT] = ACTIONS(866),
    [anon_sym_new] = ACTIONS(868),
    [anon_sym_await] = ACTIONS(868),
    [anon_sym_PLUS] = ACTIONS(868),
    [anon_sym_DASH] = ACTIONS(868),
    [anon_sym_BANG] = ACTIONS(866),
    [anon_sym_TILDE] = ACTIONS(866),
    [anon_sym_typeof] = ACTIONS(868),
    [anon_sym_void] = ACTIONS(868),
    [anon_sym_delete] = ACTIONS(868),
    [anon_sym_PLUS_PLUS] = ACTIONS(866),
    [anon_sym_DASH_DASH] = ACTIONS(866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(866),
    [anon_sym_BQUOTE] = ACTIONS(866),
    [sym_regex] = ACTIONS(868),
    [sym_number] = ACTIONS(866),
    [sym_identifier] = ACTIONS(868),
    [sym_this] = ACTIONS(868),
    [sym_super] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_null] = ACTIONS(868),
    [sym_undefined] = ACTIONS(868),
    [anon_sym_AT] = ACTIONS(866),
    [anon_sym_get] = ACTIONS(868),
    [anon_sym_set] = ACTIONS(868),
  },
  [1896] = {
    [sym__automatic_semicolon] = ACTIONS(4538),
    [anon_sym_SEMI] = ACTIONS(4538),
    [sym_comment] = ACTIONS(71),
  },
  [1897] = {
    [sym__automatic_semicolon] = ACTIONS(4540),
    [anon_sym_SEMI] = ACTIONS(4540),
    [sym_comment] = ACTIONS(71),
  },
  [1898] = {
    [sym_class_heritage] = STATE(2117),
    [sym_class_body] = STATE(2118),
    [anon_sym_LBRACE] = ACTIONS(4542),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [1899] = {
    [sym_formal_parameters] = STATE(2120),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4544),
  },
  [1900] = {
    [sym_formal_parameters] = STATE(2122),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4546),
  },
  [1901] = {
    [sym_formal_parameters] = STATE(2120),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [1902] = {
    [sym_statement_block] = STATE(2124),
    [anon_sym_LBRACE] = ACTIONS(4548),
    [sym_comment] = ACTIONS(71),
  },
  [1903] = {
    [sym_export_statement] = STATE(2125),
    [sym__declaration] = STATE(2125),
    [sym_import_statement] = STATE(2125),
    [sym_expression_statement] = STATE(2125),
    [sym_variable_declaration] = STATE(2125),
    [sym_lexical_declaration] = STATE(2125),
    [sym_statement_block] = STATE(2125),
    [sym_if_statement] = STATE(2125),
    [sym_switch_statement] = STATE(2125),
    [sym_for_statement] = STATE(2125),
    [sym_for_in_statement] = STATE(2125),
    [sym_for_of_statement] = STATE(2125),
    [sym_while_statement] = STATE(2125),
    [sym_do_statement] = STATE(2125),
    [sym_try_statement] = STATE(2125),
    [sym_with_statement] = STATE(2125),
    [sym_break_statement] = STATE(2125),
    [sym_continue_statement] = STATE(2125),
    [sym_debugger_statement] = STATE(2125),
    [sym_return_statement] = STATE(2125),
    [sym_throw_statement] = STATE(2125),
    [sym_empty_statement] = STATE(2125),
    [sym_labeled_statement] = STATE(2125),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1904] = {
    [anon_sym_export] = ACTIONS(1124),
    [anon_sym_default] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_RBRACE] = ACTIONS(1122),
    [anon_sym_import] = ACTIONS(1124),
    [anon_sym_var] = ACTIONS(1124),
    [anon_sym_let] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_LPAREN] = ACTIONS(1122),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_try] = ACTIONS(1124),
    [anon_sym_with] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_debugger] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_throw] = ACTIONS(1124),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1122),
    [anon_sym_LT] = ACTIONS(1122),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_async] = ACTIONS(1124),
    [anon_sym_function] = ACTIONS(1124),
    [anon_sym_EQ_GT] = ACTIONS(1122),
    [anon_sym_new] = ACTIONS(1124),
    [anon_sym_await] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_typeof] = ACTIONS(1124),
    [anon_sym_void] = ACTIONS(1124),
    [anon_sym_delete] = ACTIONS(1124),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1122),
    [anon_sym_BQUOTE] = ACTIONS(1122),
    [sym_regex] = ACTIONS(1124),
    [sym_number] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1124),
    [sym_this] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_null] = ACTIONS(1124),
    [sym_undefined] = ACTIONS(1124),
    [anon_sym_AT] = ACTIONS(1122),
    [anon_sym_get] = ACTIONS(1124),
    [anon_sym_set] = ACTIONS(1124),
  },
  [1905] = {
    [anon_sym_export] = ACTIONS(1204),
    [anon_sym_default] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_RBRACE] = ACTIONS(1202),
    [anon_sym_import] = ACTIONS(1204),
    [anon_sym_var] = ACTIONS(1204),
    [anon_sym_let] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_LPAREN] = ACTIONS(1202),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_try] = ACTIONS(1204),
    [anon_sym_with] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_debugger] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_throw] = ACTIONS(1204),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym_case] = ACTIONS(1204),
    [anon_sym_yield] = ACTIONS(1204),
    [anon_sym_LBRACK] = ACTIONS(1202),
    [anon_sym_LT] = ACTIONS(1202),
    [anon_sym_class] = ACTIONS(1204),
    [anon_sym_async] = ACTIONS(1204),
    [anon_sym_function] = ACTIONS(1204),
    [anon_sym_EQ_GT] = ACTIONS(1202),
    [anon_sym_new] = ACTIONS(1204),
    [anon_sym_await] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_typeof] = ACTIONS(1204),
    [anon_sym_void] = ACTIONS(1204),
    [anon_sym_delete] = ACTIONS(1204),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1202),
    [anon_sym_BQUOTE] = ACTIONS(1202),
    [sym_regex] = ACTIONS(1204),
    [sym_number] = ACTIONS(1202),
    [sym_identifier] = ACTIONS(1204),
    [sym_this] = ACTIONS(1204),
    [sym_super] = ACTIONS(1204),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [sym_null] = ACTIONS(1204),
    [sym_undefined] = ACTIONS(1204),
    [anon_sym_AT] = ACTIONS(1202),
    [anon_sym_get] = ACTIONS(1204),
    [anon_sym_set] = ACTIONS(1204),
  },
  [1906] = {
    [sym_export_statement] = STATE(1906),
    [sym__declaration] = STATE(1906),
    [sym_import_statement] = STATE(1906),
    [sym_expression_statement] = STATE(1906),
    [sym_variable_declaration] = STATE(1906),
    [sym_lexical_declaration] = STATE(1906),
    [sym_statement_block] = STATE(1906),
    [sym_if_statement] = STATE(1906),
    [sym_switch_statement] = STATE(1906),
    [sym_for_statement] = STATE(1906),
    [sym_for_in_statement] = STATE(1906),
    [sym_for_of_statement] = STATE(1906),
    [sym_while_statement] = STATE(1906),
    [sym_do_statement] = STATE(1906),
    [sym_try_statement] = STATE(1906),
    [sym_with_statement] = STATE(1906),
    [sym_break_statement] = STATE(1906),
    [sym_continue_statement] = STATE(1906),
    [sym_debugger_statement] = STATE(1906),
    [sym_return_statement] = STATE(1906),
    [sym_throw_statement] = STATE(1906),
    [sym_empty_statement] = STATE(1906),
    [sym_labeled_statement] = STATE(1906),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1906),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(4550),
    [anon_sym_default] = ACTIONS(4553),
    [anon_sym_LBRACE] = ACTIONS(4555),
    [anon_sym_RBRACE] = ACTIONS(1206),
    [anon_sym_import] = ACTIONS(4558),
    [anon_sym_var] = ACTIONS(4561),
    [anon_sym_let] = ACTIONS(4564),
    [anon_sym_const] = ACTIONS(4564),
    [anon_sym_if] = ACTIONS(4567),
    [anon_sym_switch] = ACTIONS(4570),
    [anon_sym_for] = ACTIONS(4573),
    [anon_sym_LPAREN] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(4576),
    [anon_sym_do] = ACTIONS(4579),
    [anon_sym_try] = ACTIONS(4582),
    [anon_sym_with] = ACTIONS(4585),
    [anon_sym_break] = ACTIONS(4588),
    [anon_sym_continue] = ACTIONS(4591),
    [anon_sym_debugger] = ACTIONS(4594),
    [anon_sym_return] = ACTIONS(4597),
    [anon_sym_throw] = ACTIONS(4600),
    [anon_sym_SEMI] = ACTIONS(4603),
    [anon_sym_case] = ACTIONS(4553),
    [anon_sym_yield] = ACTIONS(1265),
    [anon_sym_LBRACK] = ACTIONS(1268),
    [anon_sym_LT] = ACTIONS(1271),
    [anon_sym_class] = ACTIONS(4606),
    [anon_sym_async] = ACTIONS(4609),
    [anon_sym_function] = ACTIONS(4612),
    [anon_sym_EQ_GT] = ACTIONS(1283),
    [anon_sym_new] = ACTIONS(1286),
    [anon_sym_await] = ACTIONS(1289),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_BANG] = ACTIONS(1295),
    [anon_sym_TILDE] = ACTIONS(1295),
    [anon_sym_typeof] = ACTIONS(1292),
    [anon_sym_void] = ACTIONS(1292),
    [anon_sym_delete] = ACTIONS(1292),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1301),
    [anon_sym_BQUOTE] = ACTIONS(1304),
    [sym_regex] = ACTIONS(1307),
    [sym_number] = ACTIONS(1301),
    [sym_identifier] = ACTIONS(4615),
    [sym_this] = ACTIONS(1307),
    [sym_super] = ACTIONS(1313),
    [sym_true] = ACTIONS(1307),
    [sym_false] = ACTIONS(1307),
    [sym_null] = ACTIONS(1307),
    [sym_undefined] = ACTIONS(1307),
    [anon_sym_AT] = ACTIONS(1316),
    [anon_sym_get] = ACTIONS(1319),
    [anon_sym_set] = ACTIONS(1319),
  },
  [1907] = {
    [sym__declaration] = STATE(2059),
    [sym_variable_declaration] = STATE(2059),
    [sym_lexical_declaration] = STATE(2059),
    [sym_class] = STATE(1874),
    [sym_function] = STATE(1874),
    [sym_generator_function] = STATE(1874),
    [sym_decorator] = STATE(1875),
    [aux_sym_export_statement_repeat1] = STATE(1875),
    [anon_sym_default] = ACTIONS(4618),
    [anon_sym_var] = ACTIONS(4064),
    [anon_sym_let] = ACTIONS(4066),
    [anon_sym_const] = ACTIONS(4066),
    [anon_sym_class] = ACTIONS(4068),
    [anon_sym_async] = ACTIONS(4070),
    [anon_sym_function] = ACTIONS(4072),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [1908] = {
    [sym_class_heritage] = STATE(649),
    [sym_class_body] = STATE(574),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4620),
  },
  [1909] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2164),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1910] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1911] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1912] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(2128),
    [sym__expression] = STATE(2128),
    [sym_yield_expression] = STATE(2128),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(2128),
    [sym_jsx_fragment] = STATE(2128),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2128),
    [sym_anonymous_class] = STATE(2128),
    [sym_class] = STATE(2128),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2128),
    [sym_generator_function] = STATE(2128),
    [sym_call_expression] = STATE(2128),
    [sym_new_expression] = STATE(2128),
    [sym_await_expression] = STATE(2128),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(2128),
    [sym_augmented_assignment_expression] = STATE(2128),
    [sym_ternary_expression] = STATE(2128),
    [sym_binary_expression] = STATE(2128),
    [sym_unary_expression] = STATE(2128),
    [sym_update_expression] = STATE(2128),
    [sym_template_string] = STATE(2128),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4622),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4624),
    [sym_number] = ACTIONS(4622),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4624),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4624),
    [sym_false] = ACTIONS(4624),
    [sym_null] = ACTIONS(4624),
    [sym_undefined] = ACTIONS(4624),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [1913] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1914] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1915] = {
    [sym_export_statement] = STATE(1906),
    [sym__declaration] = STATE(1906),
    [sym_import_statement] = STATE(1906),
    [sym_expression_statement] = STATE(1906),
    [sym_variable_declaration] = STATE(1906),
    [sym_lexical_declaration] = STATE(1906),
    [sym_statement_block] = STATE(1906),
    [sym_if_statement] = STATE(1906),
    [sym_switch_statement] = STATE(1906),
    [sym_for_statement] = STATE(1906),
    [sym_for_in_statement] = STATE(1906),
    [sym_for_of_statement] = STATE(1906),
    [sym_while_statement] = STATE(1906),
    [sym_do_statement] = STATE(1906),
    [sym_try_statement] = STATE(1906),
    [sym_with_statement] = STATE(1906),
    [sym_break_statement] = STATE(1906),
    [sym_continue_statement] = STATE(1906),
    [sym_debugger_statement] = STATE(1906),
    [sym_return_statement] = STATE(1906),
    [sym_throw_statement] = STATE(1906),
    [sym_empty_statement] = STATE(1906),
    [sym_labeled_statement] = STATE(1906),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1906),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_default] = ACTIONS(4626),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_RBRACE] = ACTIONS(4628),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_case] = ACTIONS(4626),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1916] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(3566),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2240),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1917] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(4630),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1918] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1919] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1920] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1921] = {
    [sym_export_statement] = STATE(2130),
    [sym__declaration] = STATE(2130),
    [sym_import_statement] = STATE(2130),
    [sym_expression_statement] = STATE(2130),
    [sym_variable_declaration] = STATE(2130),
    [sym_lexical_declaration] = STATE(2130),
    [sym_statement_block] = STATE(2130),
    [sym_if_statement] = STATE(2130),
    [sym_switch_statement] = STATE(2130),
    [sym_for_statement] = STATE(2130),
    [sym_for_in_statement] = STATE(2130),
    [sym_for_of_statement] = STATE(2130),
    [sym_while_statement] = STATE(2130),
    [sym_do_statement] = STATE(2130),
    [sym_try_statement] = STATE(2130),
    [sym_with_statement] = STATE(2130),
    [sym_break_statement] = STATE(2130),
    [sym_continue_statement] = STATE(2130),
    [sym_debugger_statement] = STATE(2130),
    [sym_return_statement] = STATE(2130),
    [sym_throw_statement] = STATE(2130),
    [sym_empty_statement] = STATE(2130),
    [sym_labeled_statement] = STATE(2130),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1922] = {
    [sym_export_statement] = STATE(2131),
    [sym__declaration] = STATE(2131),
    [sym_import_statement] = STATE(2131),
    [sym_expression_statement] = STATE(2131),
    [sym_variable_declaration] = STATE(2131),
    [sym_lexical_declaration] = STATE(2131),
    [sym_statement_block] = STATE(2131),
    [sym_if_statement] = STATE(2131),
    [sym_switch_statement] = STATE(2131),
    [sym_for_statement] = STATE(2131),
    [sym_for_in_statement] = STATE(2131),
    [sym_for_of_statement] = STATE(2131),
    [sym_while_statement] = STATE(2131),
    [sym_do_statement] = STATE(2131),
    [sym_try_statement] = STATE(2131),
    [sym_with_statement] = STATE(2131),
    [sym_break_statement] = STATE(2131),
    [sym_continue_statement] = STATE(2131),
    [sym_debugger_statement] = STATE(2131),
    [sym_return_statement] = STATE(2131),
    [sym_throw_statement] = STATE(2131),
    [sym_empty_statement] = STATE(2131),
    [sym_labeled_statement] = STATE(2131),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1923] = {
    [sym_export_statement] = STATE(2132),
    [sym__declaration] = STATE(2132),
    [sym_import_statement] = STATE(2132),
    [sym_expression_statement] = STATE(2132),
    [sym_variable_declaration] = STATE(2132),
    [sym_lexical_declaration] = STATE(2132),
    [sym_statement_block] = STATE(2132),
    [sym_if_statement] = STATE(2132),
    [sym_switch_statement] = STATE(2132),
    [sym_for_statement] = STATE(2132),
    [sym_for_in_statement] = STATE(2132),
    [sym_for_of_statement] = STATE(2132),
    [sym_while_statement] = STATE(2132),
    [sym_do_statement] = STATE(2132),
    [sym_try_statement] = STATE(2132),
    [sym_with_statement] = STATE(2132),
    [sym_break_statement] = STATE(2132),
    [sym_continue_statement] = STATE(2132),
    [sym_debugger_statement] = STATE(2132),
    [sym_return_statement] = STATE(2132),
    [sym_throw_statement] = STATE(2132),
    [sym_empty_statement] = STATE(2132),
    [sym_labeled_statement] = STATE(2132),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1924] = {
    [sym_export_statement] = STATE(2133),
    [sym__declaration] = STATE(2133),
    [sym_import_statement] = STATE(2133),
    [sym_expression_statement] = STATE(2133),
    [sym_variable_declaration] = STATE(2133),
    [sym_lexical_declaration] = STATE(2133),
    [sym_statement_block] = STATE(2133),
    [sym_if_statement] = STATE(2133),
    [sym_switch_statement] = STATE(2133),
    [sym_for_statement] = STATE(2133),
    [sym_for_in_statement] = STATE(2133),
    [sym_for_of_statement] = STATE(2133),
    [sym_while_statement] = STATE(2133),
    [sym_do_statement] = STATE(2133),
    [sym_try_statement] = STATE(2133),
    [sym_with_statement] = STATE(2133),
    [sym_break_statement] = STATE(2133),
    [sym_continue_statement] = STATE(2133),
    [sym_debugger_statement] = STATE(2133),
    [sym_return_statement] = STATE(2133),
    [sym_throw_statement] = STATE(2133),
    [sym_empty_statement] = STATE(2133),
    [sym_labeled_statement] = STATE(2133),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1925] = {
    [sym_export_statement] = STATE(2134),
    [sym__declaration] = STATE(2134),
    [sym_import_statement] = STATE(2134),
    [sym_expression_statement] = STATE(2134),
    [sym_variable_declaration] = STATE(2134),
    [sym_lexical_declaration] = STATE(2134),
    [sym_statement_block] = STATE(2134),
    [sym_if_statement] = STATE(2134),
    [sym_switch_statement] = STATE(2134),
    [sym_for_statement] = STATE(2134),
    [sym_for_in_statement] = STATE(2134),
    [sym_for_of_statement] = STATE(2134),
    [sym_while_statement] = STATE(2134),
    [sym_do_statement] = STATE(2134),
    [sym_try_statement] = STATE(2134),
    [sym_with_statement] = STATE(2134),
    [sym_break_statement] = STATE(2134),
    [sym_continue_statement] = STATE(2134),
    [sym_debugger_statement] = STATE(2134),
    [sym_return_statement] = STATE(2134),
    [sym_throw_statement] = STATE(2134),
    [sym_empty_statement] = STATE(2134),
    [sym_labeled_statement] = STATE(2134),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1926] = {
    [sym_export_statement] = STATE(2135),
    [sym__declaration] = STATE(2135),
    [sym_import_statement] = STATE(2135),
    [sym_expression_statement] = STATE(2135),
    [sym_variable_declaration] = STATE(2135),
    [sym_lexical_declaration] = STATE(2135),
    [sym_statement_block] = STATE(2135),
    [sym_if_statement] = STATE(2135),
    [sym_switch_statement] = STATE(2135),
    [sym_for_statement] = STATE(2135),
    [sym_for_in_statement] = STATE(2135),
    [sym_for_of_statement] = STATE(2135),
    [sym_while_statement] = STATE(2135),
    [sym_do_statement] = STATE(2135),
    [sym_try_statement] = STATE(2135),
    [sym_with_statement] = STATE(2135),
    [sym_break_statement] = STATE(2135),
    [sym_continue_statement] = STATE(2135),
    [sym_debugger_statement] = STATE(2135),
    [sym_return_statement] = STATE(2135),
    [sym_throw_statement] = STATE(2135),
    [sym_empty_statement] = STATE(2135),
    [sym_labeled_statement] = STATE(2135),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(19),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(23),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(27),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(33),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(79),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1927] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2164),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2164),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2164),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2164),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1928] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2166),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2166),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2166),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2166),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1929] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2170),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2170),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2170),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2170),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1930] = {
    [sym_statement_block] = STATE(1037),
    [sym_parenthesized_expression] = STATE(2136),
    [sym__expression] = STATE(2136),
    [sym_yield_expression] = STATE(2136),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(2136),
    [sym_jsx_fragment] = STATE(2136),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2136),
    [sym_anonymous_class] = STATE(2136),
    [sym_class] = STATE(2136),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2136),
    [sym_generator_function] = STATE(2136),
    [sym_call_expression] = STATE(2136),
    [sym_new_expression] = STATE(2136),
    [sym_await_expression] = STATE(2136),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(2136),
    [sym_augmented_assignment_expression] = STATE(2136),
    [sym_ternary_expression] = STATE(2136),
    [sym_binary_expression] = STATE(2136),
    [sym_unary_expression] = STATE(2136),
    [sym_update_expression] = STATE(2136),
    [sym_template_string] = STATE(2136),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(316),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4632),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4634),
    [sym_number] = ACTIONS(4632),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4634),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4634),
    [sym_false] = ACTIONS(4634),
    [sym_null] = ACTIONS(4634),
    [sym_undefined] = ACTIONS(4634),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1931] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1932] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2212),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2212),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2212),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2212),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1933] = {
    [ts_builtin_sym_end] = ACTIONS(4636),
    [anon_sym_export] = ACTIONS(4638),
    [anon_sym_LBRACE] = ACTIONS(4636),
    [anon_sym_RBRACE] = ACTIONS(4636),
    [anon_sym_import] = ACTIONS(4638),
    [anon_sym_var] = ACTIONS(4638),
    [anon_sym_let] = ACTIONS(4638),
    [anon_sym_const] = ACTIONS(4638),
    [anon_sym_if] = ACTIONS(4638),
    [anon_sym_switch] = ACTIONS(4638),
    [anon_sym_for] = ACTIONS(4638),
    [anon_sym_LPAREN] = ACTIONS(4636),
    [anon_sym_while] = ACTIONS(4638),
    [anon_sym_do] = ACTIONS(4638),
    [anon_sym_try] = ACTIONS(4638),
    [anon_sym_with] = ACTIONS(4638),
    [anon_sym_break] = ACTIONS(4638),
    [anon_sym_continue] = ACTIONS(4638),
    [anon_sym_debugger] = ACTIONS(4638),
    [anon_sym_return] = ACTIONS(4638),
    [anon_sym_throw] = ACTIONS(4638),
    [anon_sym_SEMI] = ACTIONS(4636),
    [anon_sym_yield] = ACTIONS(4638),
    [anon_sym_LBRACK] = ACTIONS(4636),
    [anon_sym_LT] = ACTIONS(4636),
    [anon_sym_class] = ACTIONS(4638),
    [anon_sym_async] = ACTIONS(4638),
    [anon_sym_function] = ACTIONS(4638),
    [anon_sym_EQ_GT] = ACTIONS(4636),
    [anon_sym_new] = ACTIONS(4638),
    [anon_sym_await] = ACTIONS(4638),
    [anon_sym_PLUS] = ACTIONS(4638),
    [anon_sym_DASH] = ACTIONS(4638),
    [anon_sym_BANG] = ACTIONS(4636),
    [anon_sym_TILDE] = ACTIONS(4636),
    [anon_sym_typeof] = ACTIONS(4638),
    [anon_sym_void] = ACTIONS(4638),
    [anon_sym_delete] = ACTIONS(4638),
    [anon_sym_PLUS_PLUS] = ACTIONS(4636),
    [anon_sym_DASH_DASH] = ACTIONS(4636),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4636),
    [anon_sym_BQUOTE] = ACTIONS(4636),
    [sym_regex] = ACTIONS(4638),
    [sym_number] = ACTIONS(4636),
    [sym_identifier] = ACTIONS(4638),
    [sym_this] = ACTIONS(4638),
    [sym_super] = ACTIONS(4638),
    [sym_true] = ACTIONS(4638),
    [sym_false] = ACTIONS(4638),
    [sym_null] = ACTIONS(4638),
    [sym_undefined] = ACTIONS(4638),
    [anon_sym_AT] = ACTIONS(4636),
    [anon_sym_get] = ACTIONS(4638),
    [anon_sym_set] = ACTIONS(4638),
  },
  [1934] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(4640),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1935] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1936] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1937] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(2200),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(2200),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(2200),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1938] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2164),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1939] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2166),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1940] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2170),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1941] = {
    [sym_statement_block] = STATE(1336),
    [sym_parenthesized_expression] = STATE(2138),
    [sym__expression] = STATE(2138),
    [sym_yield_expression] = STATE(2138),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2138),
    [sym_jsx_fragment] = STATE(2138),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2138),
    [sym_anonymous_class] = STATE(2138),
    [sym_class] = STATE(2138),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2138),
    [sym_generator_function] = STATE(2138),
    [sym_call_expression] = STATE(2138),
    [sym_new_expression] = STATE(2138),
    [sym_await_expression] = STATE(2138),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2138),
    [sym_augmented_assignment_expression] = STATE(2138),
    [sym_ternary_expression] = STATE(2138),
    [sym_binary_expression] = STATE(2138),
    [sym_unary_expression] = STATE(2138),
    [sym_update_expression] = STATE(2138),
    [sym_template_string] = STATE(2138),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(730),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4642),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4644),
    [sym_number] = ACTIONS(4642),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4644),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4644),
    [sym_false] = ACTIONS(4644),
    [sym_null] = ACTIONS(4644),
    [sym_undefined] = ACTIONS(4644),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1942] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(2202),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2202),
    [anon_sym_GT] = ACTIONS(2202),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(2200),
    [anon_sym_EQ_EQ] = ACTIONS(2202),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2200),
    [anon_sym_BANG_EQ] = ACTIONS(2202),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2200),
    [anon_sym_GT_EQ] = ACTIONS(2200),
    [anon_sym_instanceof] = ACTIONS(2200),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1943] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2212),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1944] = {
    [ts_builtin_sym_end] = ACTIONS(4646),
    [anon_sym_export] = ACTIONS(4648),
    [anon_sym_LBRACE] = ACTIONS(4646),
    [anon_sym_RBRACE] = ACTIONS(4646),
    [anon_sym_import] = ACTIONS(4648),
    [anon_sym_var] = ACTIONS(4648),
    [anon_sym_let] = ACTIONS(4648),
    [anon_sym_const] = ACTIONS(4648),
    [anon_sym_if] = ACTIONS(4648),
    [anon_sym_switch] = ACTIONS(4648),
    [anon_sym_for] = ACTIONS(4648),
    [anon_sym_LPAREN] = ACTIONS(4646),
    [anon_sym_while] = ACTIONS(4648),
    [anon_sym_do] = ACTIONS(4648),
    [anon_sym_try] = ACTIONS(4648),
    [anon_sym_with] = ACTIONS(4648),
    [anon_sym_break] = ACTIONS(4648),
    [anon_sym_continue] = ACTIONS(4648),
    [anon_sym_debugger] = ACTIONS(4648),
    [anon_sym_return] = ACTIONS(4648),
    [anon_sym_throw] = ACTIONS(4648),
    [anon_sym_SEMI] = ACTIONS(4646),
    [anon_sym_yield] = ACTIONS(4648),
    [anon_sym_LBRACK] = ACTIONS(4646),
    [anon_sym_LT] = ACTIONS(4646),
    [anon_sym_class] = ACTIONS(4648),
    [anon_sym_async] = ACTIONS(4648),
    [anon_sym_function] = ACTIONS(4648),
    [anon_sym_EQ_GT] = ACTIONS(4646),
    [anon_sym_new] = ACTIONS(4648),
    [anon_sym_await] = ACTIONS(4648),
    [anon_sym_PLUS] = ACTIONS(4648),
    [anon_sym_DASH] = ACTIONS(4648),
    [anon_sym_BANG] = ACTIONS(4646),
    [anon_sym_TILDE] = ACTIONS(4646),
    [anon_sym_typeof] = ACTIONS(4648),
    [anon_sym_void] = ACTIONS(4648),
    [anon_sym_delete] = ACTIONS(4648),
    [anon_sym_PLUS_PLUS] = ACTIONS(4646),
    [anon_sym_DASH_DASH] = ACTIONS(4646),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4646),
    [anon_sym_BQUOTE] = ACTIONS(4646),
    [sym_regex] = ACTIONS(4648),
    [sym_number] = ACTIONS(4646),
    [sym_identifier] = ACTIONS(4648),
    [sym_this] = ACTIONS(4648),
    [sym_super] = ACTIONS(4648),
    [sym_true] = ACTIONS(4648),
    [sym_false] = ACTIONS(4648),
    [sym_null] = ACTIONS(4648),
    [sym_undefined] = ACTIONS(4648),
    [anon_sym_AT] = ACTIONS(4646),
    [anon_sym_get] = ACTIONS(4648),
    [anon_sym_set] = ACTIONS(4648),
  },
  [1945] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(4650),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1946] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1947] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1948] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(2200),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2200),
    [anon_sym_AMP_AMP] = ACTIONS(2200),
    [anon_sym_PIPE_PIPE] = ACTIONS(2200),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(2202),
    [anon_sym_CARET] = ACTIONS(2200),
    [anon_sym_PIPE] = ACTIONS(2202),
    [anon_sym_PLUS] = ACTIONS(2202),
    [anon_sym_DASH] = ACTIONS(2202),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1949] = {
    [ts_builtin_sym_end] = ACTIONS(4652),
    [anon_sym_export] = ACTIONS(4654),
    [anon_sym_LBRACE] = ACTIONS(4652),
    [anon_sym_RBRACE] = ACTIONS(4652),
    [anon_sym_import] = ACTIONS(4654),
    [anon_sym_var] = ACTIONS(4654),
    [anon_sym_let] = ACTIONS(4654),
    [anon_sym_const] = ACTIONS(4654),
    [anon_sym_if] = ACTIONS(4654),
    [anon_sym_switch] = ACTIONS(4654),
    [anon_sym_for] = ACTIONS(4654),
    [anon_sym_LPAREN] = ACTIONS(4652),
    [anon_sym_while] = ACTIONS(4654),
    [anon_sym_do] = ACTIONS(4654),
    [anon_sym_try] = ACTIONS(4654),
    [anon_sym_with] = ACTIONS(4654),
    [anon_sym_break] = ACTIONS(4654),
    [anon_sym_continue] = ACTIONS(4654),
    [anon_sym_debugger] = ACTIONS(4654),
    [anon_sym_return] = ACTIONS(4654),
    [anon_sym_throw] = ACTIONS(4654),
    [anon_sym_SEMI] = ACTIONS(4652),
    [anon_sym_yield] = ACTIONS(4654),
    [anon_sym_LBRACK] = ACTIONS(4652),
    [anon_sym_LT] = ACTIONS(4652),
    [anon_sym_class] = ACTIONS(4654),
    [anon_sym_async] = ACTIONS(4654),
    [anon_sym_function] = ACTIONS(4654),
    [anon_sym_EQ_GT] = ACTIONS(4652),
    [anon_sym_new] = ACTIONS(4654),
    [anon_sym_await] = ACTIONS(4654),
    [anon_sym_PLUS] = ACTIONS(4654),
    [anon_sym_DASH] = ACTIONS(4654),
    [anon_sym_BANG] = ACTIONS(4652),
    [anon_sym_TILDE] = ACTIONS(4652),
    [anon_sym_typeof] = ACTIONS(4654),
    [anon_sym_void] = ACTIONS(4654),
    [anon_sym_delete] = ACTIONS(4654),
    [anon_sym_PLUS_PLUS] = ACTIONS(4652),
    [anon_sym_DASH_DASH] = ACTIONS(4652),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4652),
    [anon_sym_BQUOTE] = ACTIONS(4652),
    [sym_regex] = ACTIONS(4654),
    [sym_number] = ACTIONS(4652),
    [sym_identifier] = ACTIONS(4654),
    [sym_this] = ACTIONS(4654),
    [sym_super] = ACTIONS(4654),
    [sym_true] = ACTIONS(4654),
    [sym_false] = ACTIONS(4654),
    [sym_null] = ACTIONS(4654),
    [sym_undefined] = ACTIONS(4654),
    [anon_sym_AT] = ACTIONS(4652),
    [anon_sym_get] = ACTIONS(4654),
    [anon_sym_set] = ACTIONS(4654),
  },
  [1950] = {
    [ts_builtin_sym_end] = ACTIONS(4656),
    [anon_sym_export] = ACTIONS(4658),
    [anon_sym_LBRACE] = ACTIONS(4656),
    [anon_sym_RBRACE] = ACTIONS(4656),
    [anon_sym_import] = ACTIONS(4658),
    [anon_sym_var] = ACTIONS(4658),
    [anon_sym_let] = ACTIONS(4658),
    [anon_sym_const] = ACTIONS(4658),
    [anon_sym_if] = ACTIONS(4658),
    [anon_sym_switch] = ACTIONS(4658),
    [anon_sym_for] = ACTIONS(4658),
    [anon_sym_LPAREN] = ACTIONS(4656),
    [anon_sym_while] = ACTIONS(4658),
    [anon_sym_do] = ACTIONS(4658),
    [anon_sym_try] = ACTIONS(4658),
    [anon_sym_with] = ACTIONS(4658),
    [anon_sym_break] = ACTIONS(4658),
    [anon_sym_continue] = ACTIONS(4658),
    [anon_sym_debugger] = ACTIONS(4658),
    [anon_sym_return] = ACTIONS(4658),
    [anon_sym_throw] = ACTIONS(4658),
    [anon_sym_SEMI] = ACTIONS(4656),
    [anon_sym_yield] = ACTIONS(4658),
    [anon_sym_LBRACK] = ACTIONS(4656),
    [anon_sym_LT] = ACTIONS(4656),
    [anon_sym_class] = ACTIONS(4658),
    [anon_sym_async] = ACTIONS(4658),
    [anon_sym_function] = ACTIONS(4658),
    [anon_sym_EQ_GT] = ACTIONS(4656),
    [anon_sym_new] = ACTIONS(4658),
    [anon_sym_await] = ACTIONS(4658),
    [anon_sym_PLUS] = ACTIONS(4658),
    [anon_sym_DASH] = ACTIONS(4658),
    [anon_sym_BANG] = ACTIONS(4656),
    [anon_sym_TILDE] = ACTIONS(4656),
    [anon_sym_typeof] = ACTIONS(4658),
    [anon_sym_void] = ACTIONS(4658),
    [anon_sym_delete] = ACTIONS(4658),
    [anon_sym_PLUS_PLUS] = ACTIONS(4656),
    [anon_sym_DASH_DASH] = ACTIONS(4656),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4656),
    [anon_sym_BQUOTE] = ACTIONS(4656),
    [sym_regex] = ACTIONS(4658),
    [sym_number] = ACTIONS(4656),
    [sym_identifier] = ACTIONS(4658),
    [sym_this] = ACTIONS(4658),
    [sym_super] = ACTIONS(4658),
    [sym_true] = ACTIONS(4658),
    [sym_false] = ACTIONS(4658),
    [sym_null] = ACTIONS(4658),
    [sym_undefined] = ACTIONS(4658),
    [anon_sym_AT] = ACTIONS(4656),
    [anon_sym_get] = ACTIONS(4658),
    [anon_sym_set] = ACTIONS(4658),
  },
  [1951] = {
    [ts_builtin_sym_end] = ACTIONS(4660),
    [anon_sym_export] = ACTIONS(4662),
    [anon_sym_LBRACE] = ACTIONS(4660),
    [anon_sym_RBRACE] = ACTIONS(4660),
    [anon_sym_import] = ACTIONS(4662),
    [anon_sym_var] = ACTIONS(4662),
    [anon_sym_let] = ACTIONS(4662),
    [anon_sym_const] = ACTIONS(4662),
    [anon_sym_if] = ACTIONS(4662),
    [anon_sym_switch] = ACTIONS(4662),
    [anon_sym_for] = ACTIONS(4662),
    [anon_sym_LPAREN] = ACTIONS(4660),
    [anon_sym_while] = ACTIONS(4662),
    [anon_sym_do] = ACTIONS(4662),
    [anon_sym_try] = ACTIONS(4662),
    [anon_sym_with] = ACTIONS(4662),
    [anon_sym_break] = ACTIONS(4662),
    [anon_sym_continue] = ACTIONS(4662),
    [anon_sym_debugger] = ACTIONS(4662),
    [anon_sym_return] = ACTIONS(4662),
    [anon_sym_throw] = ACTIONS(4662),
    [anon_sym_SEMI] = ACTIONS(4660),
    [anon_sym_yield] = ACTIONS(4662),
    [anon_sym_LBRACK] = ACTIONS(4660),
    [anon_sym_LT] = ACTIONS(4660),
    [anon_sym_class] = ACTIONS(4662),
    [anon_sym_async] = ACTIONS(4662),
    [anon_sym_function] = ACTIONS(4662),
    [anon_sym_EQ_GT] = ACTIONS(4660),
    [anon_sym_new] = ACTIONS(4662),
    [anon_sym_await] = ACTIONS(4662),
    [anon_sym_PLUS] = ACTIONS(4662),
    [anon_sym_DASH] = ACTIONS(4662),
    [anon_sym_BANG] = ACTIONS(4660),
    [anon_sym_TILDE] = ACTIONS(4660),
    [anon_sym_typeof] = ACTIONS(4662),
    [anon_sym_void] = ACTIONS(4662),
    [anon_sym_delete] = ACTIONS(4662),
    [anon_sym_PLUS_PLUS] = ACTIONS(4660),
    [anon_sym_DASH_DASH] = ACTIONS(4660),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4660),
    [anon_sym_BQUOTE] = ACTIONS(4660),
    [sym_regex] = ACTIONS(4662),
    [sym_number] = ACTIONS(4660),
    [sym_identifier] = ACTIONS(4662),
    [sym_this] = ACTIONS(4662),
    [sym_super] = ACTIONS(4662),
    [sym_true] = ACTIONS(4662),
    [sym_false] = ACTIONS(4662),
    [sym_null] = ACTIONS(4662),
    [sym_undefined] = ACTIONS(4662),
    [anon_sym_AT] = ACTIONS(4660),
    [anon_sym_get] = ACTIONS(4662),
    [anon_sym_set] = ACTIONS(4662),
  },
  [1952] = {
    [ts_builtin_sym_end] = ACTIONS(4664),
    [anon_sym_export] = ACTIONS(4666),
    [anon_sym_LBRACE] = ACTIONS(4664),
    [anon_sym_RBRACE] = ACTIONS(4664),
    [anon_sym_import] = ACTIONS(4666),
    [anon_sym_var] = ACTIONS(4666),
    [anon_sym_let] = ACTIONS(4666),
    [anon_sym_const] = ACTIONS(4666),
    [anon_sym_if] = ACTIONS(4666),
    [anon_sym_switch] = ACTIONS(4666),
    [anon_sym_for] = ACTIONS(4666),
    [anon_sym_LPAREN] = ACTIONS(4664),
    [anon_sym_while] = ACTIONS(4666),
    [anon_sym_do] = ACTIONS(4666),
    [anon_sym_try] = ACTIONS(4666),
    [anon_sym_with] = ACTIONS(4666),
    [anon_sym_break] = ACTIONS(4666),
    [anon_sym_continue] = ACTIONS(4666),
    [anon_sym_debugger] = ACTIONS(4666),
    [anon_sym_return] = ACTIONS(4666),
    [anon_sym_throw] = ACTIONS(4666),
    [anon_sym_SEMI] = ACTIONS(4664),
    [anon_sym_yield] = ACTIONS(4666),
    [anon_sym_LBRACK] = ACTIONS(4664),
    [anon_sym_LT] = ACTIONS(4664),
    [anon_sym_class] = ACTIONS(4666),
    [anon_sym_async] = ACTIONS(4666),
    [anon_sym_function] = ACTIONS(4666),
    [anon_sym_EQ_GT] = ACTIONS(4664),
    [anon_sym_new] = ACTIONS(4666),
    [anon_sym_await] = ACTIONS(4666),
    [anon_sym_PLUS] = ACTIONS(4666),
    [anon_sym_DASH] = ACTIONS(4666),
    [anon_sym_BANG] = ACTIONS(4664),
    [anon_sym_TILDE] = ACTIONS(4664),
    [anon_sym_typeof] = ACTIONS(4666),
    [anon_sym_void] = ACTIONS(4666),
    [anon_sym_delete] = ACTIONS(4666),
    [anon_sym_PLUS_PLUS] = ACTIONS(4664),
    [anon_sym_DASH_DASH] = ACTIONS(4664),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4664),
    [anon_sym_BQUOTE] = ACTIONS(4664),
    [sym_regex] = ACTIONS(4666),
    [sym_number] = ACTIONS(4664),
    [sym_identifier] = ACTIONS(4666),
    [sym_this] = ACTIONS(4666),
    [sym_super] = ACTIONS(4666),
    [sym_true] = ACTIONS(4666),
    [sym_false] = ACTIONS(4666),
    [sym_null] = ACTIONS(4666),
    [sym_undefined] = ACTIONS(4666),
    [anon_sym_AT] = ACTIONS(4664),
    [anon_sym_get] = ACTIONS(4666),
    [anon_sym_set] = ACTIONS(4666),
  },
  [1953] = {
    [ts_builtin_sym_end] = ACTIONS(4668),
    [anon_sym_export] = ACTIONS(4670),
    [anon_sym_LBRACE] = ACTIONS(4668),
    [anon_sym_RBRACE] = ACTIONS(4668),
    [anon_sym_import] = ACTIONS(4670),
    [anon_sym_var] = ACTIONS(4670),
    [anon_sym_let] = ACTIONS(4670),
    [anon_sym_const] = ACTIONS(4670),
    [anon_sym_if] = ACTIONS(4670),
    [anon_sym_switch] = ACTIONS(4670),
    [anon_sym_for] = ACTIONS(4670),
    [anon_sym_LPAREN] = ACTIONS(4668),
    [anon_sym_while] = ACTIONS(4670),
    [anon_sym_do] = ACTIONS(4670),
    [anon_sym_try] = ACTIONS(4670),
    [anon_sym_with] = ACTIONS(4670),
    [anon_sym_break] = ACTIONS(4670),
    [anon_sym_continue] = ACTIONS(4670),
    [anon_sym_debugger] = ACTIONS(4670),
    [anon_sym_return] = ACTIONS(4670),
    [anon_sym_throw] = ACTIONS(4670),
    [anon_sym_SEMI] = ACTIONS(4668),
    [anon_sym_yield] = ACTIONS(4670),
    [anon_sym_LBRACK] = ACTIONS(4668),
    [anon_sym_LT] = ACTIONS(4668),
    [anon_sym_class] = ACTIONS(4670),
    [anon_sym_async] = ACTIONS(4670),
    [anon_sym_function] = ACTIONS(4670),
    [anon_sym_EQ_GT] = ACTIONS(4668),
    [anon_sym_new] = ACTIONS(4670),
    [anon_sym_await] = ACTIONS(4670),
    [anon_sym_PLUS] = ACTIONS(4670),
    [anon_sym_DASH] = ACTIONS(4670),
    [anon_sym_BANG] = ACTIONS(4668),
    [anon_sym_TILDE] = ACTIONS(4668),
    [anon_sym_typeof] = ACTIONS(4670),
    [anon_sym_void] = ACTIONS(4670),
    [anon_sym_delete] = ACTIONS(4670),
    [anon_sym_PLUS_PLUS] = ACTIONS(4668),
    [anon_sym_DASH_DASH] = ACTIONS(4668),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4668),
    [anon_sym_BQUOTE] = ACTIONS(4668),
    [sym_regex] = ACTIONS(4670),
    [sym_number] = ACTIONS(4668),
    [sym_identifier] = ACTIONS(4670),
    [sym_this] = ACTIONS(4670),
    [sym_super] = ACTIONS(4670),
    [sym_true] = ACTIONS(4670),
    [sym_false] = ACTIONS(4670),
    [sym_null] = ACTIONS(4670),
    [sym_undefined] = ACTIONS(4670),
    [anon_sym_AT] = ACTIONS(4668),
    [anon_sym_get] = ACTIONS(4670),
    [anon_sym_set] = ACTIONS(4670),
  },
  [1954] = {
    [anon_sym_STAR] = ACTIONS(4332),
    [anon_sym_LBRACE] = ACTIONS(4332),
    [anon_sym_COMMA] = ACTIONS(4332),
    [anon_sym_RBRACE] = ACTIONS(4332),
    [anon_sym_LPAREN] = ACTIONS(4332),
    [anon_sym_RPAREN] = ACTIONS(4332),
    [anon_sym_in] = ACTIONS(4334),
    [anon_sym_COLON] = ACTIONS(4332),
    [anon_sym_LBRACK] = ACTIONS(4332),
    [anon_sym_RBRACK] = ACTIONS(4332),
    [anon_sym_LT] = ACTIONS(4334),
    [anon_sym_GT] = ACTIONS(4334),
    [anon_sym_SLASH] = ACTIONS(4334),
    [anon_sym_DOT] = ACTIONS(4332),
    [anon_sym_QMARK] = ACTIONS(4332),
    [anon_sym_AMP_AMP] = ACTIONS(4332),
    [anon_sym_PIPE_PIPE] = ACTIONS(4332),
    [anon_sym_GT_GT] = ACTIONS(4334),
    [anon_sym_GT_GT_GT] = ACTIONS(4332),
    [anon_sym_LT_LT] = ACTIONS(4332),
    [anon_sym_AMP] = ACTIONS(4334),
    [anon_sym_CARET] = ACTIONS(4332),
    [anon_sym_PIPE] = ACTIONS(4334),
    [anon_sym_PLUS] = ACTIONS(4334),
    [anon_sym_DASH] = ACTIONS(4334),
    [anon_sym_PERCENT] = ACTIONS(4332),
    [anon_sym_LT_EQ] = ACTIONS(4332),
    [anon_sym_EQ_EQ] = ACTIONS(4334),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4332),
    [anon_sym_BANG_EQ] = ACTIONS(4334),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4332),
    [anon_sym_GT_EQ] = ACTIONS(4332),
    [anon_sym_instanceof] = ACTIONS(4332),
    [anon_sym_PLUS_PLUS] = ACTIONS(4332),
    [anon_sym_DASH_DASH] = ACTIONS(4332),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(4332),
  },
  [1955] = {
    [anon_sym_else] = ACTIONS(2343),
    [anon_sym_while] = ACTIONS(2343),
    [sym_comment] = ACTIONS(71),
  },
  [1956] = {
    [anon_sym_else] = ACTIONS(2607),
    [anon_sym_while] = ACTIONS(2607),
    [sym_comment] = ACTIONS(71),
  },
  [1957] = {
    [anon_sym_else] = ACTIONS(2619),
    [anon_sym_while] = ACTIONS(2619),
    [sym_comment] = ACTIONS(71),
  },
  [1958] = {
    [anon_sym_else] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2626),
    [sym_comment] = ACTIONS(71),
  },
  [1959] = {
    [sym_export_statement] = STATE(2140),
    [sym__declaration] = STATE(2140),
    [sym_import_statement] = STATE(2140),
    [sym_expression_statement] = STATE(2140),
    [sym_variable_declaration] = STATE(2140),
    [sym_lexical_declaration] = STATE(2140),
    [sym_statement_block] = STATE(2140),
    [sym_if_statement] = STATE(2140),
    [sym_switch_statement] = STATE(2140),
    [sym_for_statement] = STATE(2140),
    [sym_for_in_statement] = STATE(2140),
    [sym_for_of_statement] = STATE(2140),
    [sym_while_statement] = STATE(2140),
    [sym_do_statement] = STATE(2140),
    [sym_try_statement] = STATE(2140),
    [sym_with_statement] = STATE(2140),
    [sym_break_statement] = STATE(2140),
    [sym_continue_statement] = STATE(2140),
    [sym_debugger_statement] = STATE(2140),
    [sym_return_statement] = STATE(2140),
    [sym_throw_statement] = STATE(2140),
    [sym_empty_statement] = STATE(2140),
    [sym_labeled_statement] = STATE(2140),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1960] = {
    [anon_sym_else] = ACTIONS(2692),
    [anon_sym_while] = ACTIONS(2692),
    [sym_comment] = ACTIONS(71),
  },
  [1961] = {
    [sym_switch_case] = STATE(1276),
    [sym_switch_default] = STATE(1276),
    [aux_sym_switch_body_repeat1] = STATE(1276),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(4672),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [1962] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(4674),
    [anon_sym_of] = ACTIONS(4676),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1963] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(4678),
    [anon_sym_of] = ACTIONS(4680),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1964] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(4682),
    [anon_sym_of] = ACTIONS(4684),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [1965] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2148),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2148),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1966] = {
    [sym_parenthesized_expression] = STATE(2149),
    [sym__expression] = STATE(2149),
    [sym_yield_expression] = STATE(2149),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2149),
    [sym_jsx_fragment] = STATE(2149),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2149),
    [sym_anonymous_class] = STATE(2149),
    [sym_class] = STATE(2149),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2149),
    [sym_generator_function] = STATE(2149),
    [sym_call_expression] = STATE(2149),
    [sym_new_expression] = STATE(2149),
    [sym_await_expression] = STATE(2149),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2149),
    [sym_augmented_assignment_expression] = STATE(2149),
    [sym_ternary_expression] = STATE(2149),
    [sym_binary_expression] = STATE(2149),
    [sym_unary_expression] = STATE(2149),
    [sym_update_expression] = STATE(2149),
    [sym_template_string] = STATE(2149),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4686),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4688),
    [sym_number] = ACTIONS(4686),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4688),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4688),
    [sym_false] = ACTIONS(4688),
    [sym_null] = ACTIONS(4688),
    [sym_undefined] = ACTIONS(4688),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1967] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2151),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2151),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(4690),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [1968] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2152),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2152),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1969] = {
    [sym_parenthesized_expression] = STATE(2153),
    [sym__expression] = STATE(2153),
    [sym_yield_expression] = STATE(2153),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2153),
    [sym_jsx_fragment] = STATE(2153),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2153),
    [sym_anonymous_class] = STATE(2153),
    [sym_class] = STATE(2153),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2153),
    [sym_generator_function] = STATE(2153),
    [sym_call_expression] = STATE(2153),
    [sym_new_expression] = STATE(2153),
    [sym_await_expression] = STATE(2153),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2153),
    [sym_augmented_assignment_expression] = STATE(2153),
    [sym_ternary_expression] = STATE(2153),
    [sym_binary_expression] = STATE(2153),
    [sym_unary_expression] = STATE(2153),
    [sym_update_expression] = STATE(2153),
    [sym_template_string] = STATE(2153),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4692),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4694),
    [sym_number] = ACTIONS(4692),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4694),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4694),
    [sym_false] = ACTIONS(4694),
    [sym_null] = ACTIONS(4694),
    [sym_undefined] = ACTIONS(4694),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1970] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2154),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2154),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [1971] = {
    [sym_parenthesized_expression] = STATE(2155),
    [sym__expression] = STATE(2155),
    [sym_yield_expression] = STATE(2155),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2155),
    [sym_jsx_fragment] = STATE(2155),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2155),
    [sym_anonymous_class] = STATE(2155),
    [sym_class] = STATE(2155),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2155),
    [sym_generator_function] = STATE(2155),
    [sym_call_expression] = STATE(2155),
    [sym_new_expression] = STATE(2155),
    [sym_await_expression] = STATE(2155),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2155),
    [sym_augmented_assignment_expression] = STATE(2155),
    [sym_ternary_expression] = STATE(2155),
    [sym_binary_expression] = STATE(2155),
    [sym_unary_expression] = STATE(2155),
    [sym_update_expression] = STATE(2155),
    [sym_template_string] = STATE(2155),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4696),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4698),
    [sym_number] = ACTIONS(4696),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4698),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4698),
    [sym_false] = ACTIONS(4698),
    [sym_null] = ACTIONS(4698),
    [sym_undefined] = ACTIONS(4698),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [1972] = {
    [sym__automatic_semicolon] = ACTIONS(4700),
    [anon_sym_SEMI] = ACTIONS(4700),
    [sym_comment] = ACTIONS(71),
  },
  [1973] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4702),
  },
  [1974] = {
    [anon_sym_else] = ACTIONS(2925),
    [anon_sym_while] = ACTIONS(2925),
    [anon_sym_finally] = ACTIONS(2925),
    [sym_comment] = ACTIONS(71),
  },
  [1975] = {
    [anon_sym_else] = ACTIONS(2931),
    [anon_sym_while] = ACTIONS(2931),
    [sym_comment] = ACTIONS(71),
  },
  [1976] = {
    [anon_sym_else] = ACTIONS(2935),
    [anon_sym_while] = ACTIONS(2935),
    [sym_comment] = ACTIONS(71),
  },
  [1977] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(4704),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(4704),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [1978] = {
    [anon_sym_RPAREN] = ACTIONS(4706),
    [sym_comment] = ACTIONS(71),
  },
  [1979] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4708),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1980] = {
    [anon_sym_RPAREN] = ACTIONS(4710),
    [sym_comment] = ACTIONS(71),
  },
  [1981] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4712),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1982] = {
    [anon_sym_RPAREN] = ACTIONS(4714),
    [sym_comment] = ACTIONS(71),
  },
  [1983] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4716),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [1984] = {
    [sym_export_statement] = STATE(1933),
    [sym__declaration] = STATE(1933),
    [sym_import_statement] = STATE(1933),
    [sym_expression_statement] = STATE(1933),
    [sym_variable_declaration] = STATE(1933),
    [sym_lexical_declaration] = STATE(1933),
    [sym_statement_block] = STATE(1933),
    [sym_if_statement] = STATE(1933),
    [sym_switch_statement] = STATE(1933),
    [sym_for_statement] = STATE(1933),
    [sym_for_in_statement] = STATE(1933),
    [sym_for_of_statement] = STATE(1933),
    [sym_while_statement] = STATE(1933),
    [sym_do_statement] = STATE(1933),
    [sym_try_statement] = STATE(1933),
    [sym_with_statement] = STATE(1933),
    [sym_break_statement] = STATE(1933),
    [sym_continue_statement] = STATE(1933),
    [sym_debugger_statement] = STATE(1933),
    [sym_return_statement] = STATE(1933),
    [sym_throw_statement] = STATE(1933),
    [sym_empty_statement] = STATE(1933),
    [sym_labeled_statement] = STATE(1933),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1985] = {
    [sym_export_statement] = STATE(1944),
    [sym__declaration] = STATE(1944),
    [sym_import_statement] = STATE(1944),
    [sym_expression_statement] = STATE(1944),
    [sym_variable_declaration] = STATE(1944),
    [sym_lexical_declaration] = STATE(1944),
    [sym_statement_block] = STATE(1944),
    [sym_if_statement] = STATE(1944),
    [sym_switch_statement] = STATE(1944),
    [sym_for_statement] = STATE(1944),
    [sym_for_in_statement] = STATE(1944),
    [sym_for_of_statement] = STATE(1944),
    [sym_while_statement] = STATE(1944),
    [sym_do_statement] = STATE(1944),
    [sym_try_statement] = STATE(1944),
    [sym_with_statement] = STATE(1944),
    [sym_break_statement] = STATE(1944),
    [sym_continue_statement] = STATE(1944),
    [sym_debugger_statement] = STATE(1944),
    [sym_return_statement] = STATE(1944),
    [sym_throw_statement] = STATE(1944),
    [sym_empty_statement] = STATE(1944),
    [sym_labeled_statement] = STATE(1944),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1986] = {
    [sym_export_statement] = STATE(1949),
    [sym__declaration] = STATE(1949),
    [sym_import_statement] = STATE(1949),
    [sym_expression_statement] = STATE(1949),
    [sym_variable_declaration] = STATE(1949),
    [sym_lexical_declaration] = STATE(1949),
    [sym_statement_block] = STATE(1949),
    [sym_if_statement] = STATE(1949),
    [sym_switch_statement] = STATE(1949),
    [sym_for_statement] = STATE(1949),
    [sym_for_in_statement] = STATE(1949),
    [sym_for_of_statement] = STATE(1949),
    [sym_while_statement] = STATE(1949),
    [sym_do_statement] = STATE(1949),
    [sym_try_statement] = STATE(1949),
    [sym_with_statement] = STATE(1949),
    [sym_break_statement] = STATE(1949),
    [sym_continue_statement] = STATE(1949),
    [sym_debugger_statement] = STATE(1949),
    [sym_return_statement] = STATE(1949),
    [sym_throw_statement] = STATE(1949),
    [sym_empty_statement] = STATE(1949),
    [sym_labeled_statement] = STATE(1949),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1987] = {
    [sym_export_statement] = STATE(1950),
    [sym__declaration] = STATE(1950),
    [sym_import_statement] = STATE(1950),
    [sym_expression_statement] = STATE(1950),
    [sym_variable_declaration] = STATE(1950),
    [sym_lexical_declaration] = STATE(1950),
    [sym_statement_block] = STATE(1950),
    [sym_if_statement] = STATE(1950),
    [sym_switch_statement] = STATE(1950),
    [sym_for_statement] = STATE(1950),
    [sym_for_in_statement] = STATE(1950),
    [sym_for_of_statement] = STATE(1950),
    [sym_while_statement] = STATE(1950),
    [sym_do_statement] = STATE(1950),
    [sym_try_statement] = STATE(1950),
    [sym_with_statement] = STATE(1950),
    [sym_break_statement] = STATE(1950),
    [sym_continue_statement] = STATE(1950),
    [sym_debugger_statement] = STATE(1950),
    [sym_return_statement] = STATE(1950),
    [sym_throw_statement] = STATE(1950),
    [sym_empty_statement] = STATE(1950),
    [sym_labeled_statement] = STATE(1950),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1988] = {
    [sym_export_statement] = STATE(1951),
    [sym__declaration] = STATE(1951),
    [sym_import_statement] = STATE(1951),
    [sym_expression_statement] = STATE(1951),
    [sym_variable_declaration] = STATE(1951),
    [sym_lexical_declaration] = STATE(1951),
    [sym_statement_block] = STATE(1951),
    [sym_if_statement] = STATE(1951),
    [sym_switch_statement] = STATE(1951),
    [sym_for_statement] = STATE(1951),
    [sym_for_in_statement] = STATE(1951),
    [sym_for_of_statement] = STATE(1951),
    [sym_while_statement] = STATE(1951),
    [sym_do_statement] = STATE(1951),
    [sym_try_statement] = STATE(1951),
    [sym_with_statement] = STATE(1951),
    [sym_break_statement] = STATE(1951),
    [sym_continue_statement] = STATE(1951),
    [sym_debugger_statement] = STATE(1951),
    [sym_return_statement] = STATE(1951),
    [sym_throw_statement] = STATE(1951),
    [sym_empty_statement] = STATE(1951),
    [sym_labeled_statement] = STATE(1951),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1989] = {
    [sym_export_statement] = STATE(1952),
    [sym__declaration] = STATE(1952),
    [sym_import_statement] = STATE(1952),
    [sym_expression_statement] = STATE(1952),
    [sym_variable_declaration] = STATE(1952),
    [sym_lexical_declaration] = STATE(1952),
    [sym_statement_block] = STATE(1952),
    [sym_if_statement] = STATE(1952),
    [sym_switch_statement] = STATE(1952),
    [sym_for_statement] = STATE(1952),
    [sym_for_in_statement] = STATE(1952),
    [sym_for_of_statement] = STATE(1952),
    [sym_while_statement] = STATE(1952),
    [sym_do_statement] = STATE(1952),
    [sym_try_statement] = STATE(1952),
    [sym_with_statement] = STATE(1952),
    [sym_break_statement] = STATE(1952),
    [sym_continue_statement] = STATE(1952),
    [sym_debugger_statement] = STATE(1952),
    [sym_return_statement] = STATE(1952),
    [sym_throw_statement] = STATE(1952),
    [sym_empty_statement] = STATE(1952),
    [sym_labeled_statement] = STATE(1952),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1990] = {
    [sym_export_statement] = STATE(1953),
    [sym__declaration] = STATE(1953),
    [sym_import_statement] = STATE(1953),
    [sym_expression_statement] = STATE(1953),
    [sym_variable_declaration] = STATE(1953),
    [sym_lexical_declaration] = STATE(1953),
    [sym_statement_block] = STATE(1953),
    [sym_if_statement] = STATE(1953),
    [sym_switch_statement] = STATE(1953),
    [sym_for_statement] = STATE(1953),
    [sym_for_in_statement] = STATE(1953),
    [sym_for_of_statement] = STATE(1953),
    [sym_while_statement] = STATE(1953),
    [sym_do_statement] = STATE(1953),
    [sym_try_statement] = STATE(1953),
    [sym_with_statement] = STATE(1953),
    [sym_break_statement] = STATE(1953),
    [sym_continue_statement] = STATE(1953),
    [sym_debugger_statement] = STATE(1953),
    [sym_return_statement] = STATE(1953),
    [sym_throw_statement] = STATE(1953),
    [sym_empty_statement] = STATE(1953),
    [sym_labeled_statement] = STATE(1953),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [1991] = {
    [sym_statement_block] = STATE(1992),
    [anon_sym_LBRACE] = ACTIONS(217),
    [sym_comment] = ACTIONS(71),
  },
  [1992] = {
    [ts_builtin_sym_end] = ACTIONS(4718),
    [anon_sym_export] = ACTIONS(4720),
    [anon_sym_LBRACE] = ACTIONS(4718),
    [anon_sym_RBRACE] = ACTIONS(4718),
    [anon_sym_import] = ACTIONS(4720),
    [anon_sym_var] = ACTIONS(4720),
    [anon_sym_let] = ACTIONS(4720),
    [anon_sym_const] = ACTIONS(4720),
    [anon_sym_if] = ACTIONS(4720),
    [anon_sym_switch] = ACTIONS(4720),
    [anon_sym_for] = ACTIONS(4720),
    [anon_sym_LPAREN] = ACTIONS(4718),
    [anon_sym_while] = ACTIONS(4720),
    [anon_sym_do] = ACTIONS(4720),
    [anon_sym_try] = ACTIONS(4720),
    [anon_sym_with] = ACTIONS(4720),
    [anon_sym_break] = ACTIONS(4720),
    [anon_sym_continue] = ACTIONS(4720),
    [anon_sym_debugger] = ACTIONS(4720),
    [anon_sym_return] = ACTIONS(4720),
    [anon_sym_throw] = ACTIONS(4720),
    [anon_sym_SEMI] = ACTIONS(4718),
    [anon_sym_finally] = ACTIONS(4720),
    [anon_sym_yield] = ACTIONS(4720),
    [anon_sym_LBRACK] = ACTIONS(4718),
    [anon_sym_LT] = ACTIONS(4718),
    [anon_sym_class] = ACTIONS(4720),
    [anon_sym_async] = ACTIONS(4720),
    [anon_sym_function] = ACTIONS(4720),
    [anon_sym_EQ_GT] = ACTIONS(4718),
    [anon_sym_new] = ACTIONS(4720),
    [anon_sym_await] = ACTIONS(4720),
    [anon_sym_PLUS] = ACTIONS(4720),
    [anon_sym_DASH] = ACTIONS(4720),
    [anon_sym_BANG] = ACTIONS(4718),
    [anon_sym_TILDE] = ACTIONS(4718),
    [anon_sym_typeof] = ACTIONS(4720),
    [anon_sym_void] = ACTIONS(4720),
    [anon_sym_delete] = ACTIONS(4720),
    [anon_sym_PLUS_PLUS] = ACTIONS(4718),
    [anon_sym_DASH_DASH] = ACTIONS(4718),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4718),
    [anon_sym_BQUOTE] = ACTIONS(4718),
    [sym_regex] = ACTIONS(4720),
    [sym_number] = ACTIONS(4718),
    [sym_identifier] = ACTIONS(4720),
    [sym_this] = ACTIONS(4720),
    [sym_super] = ACTIONS(4720),
    [sym_true] = ACTIONS(4720),
    [sym_false] = ACTIONS(4720),
    [sym_null] = ACTIONS(4720),
    [sym_undefined] = ACTIONS(4720),
    [anon_sym_AT] = ACTIONS(4718),
    [anon_sym_get] = ACTIONS(4720),
    [anon_sym_set] = ACTIONS(4720),
  },
  [1993] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(4722),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [1994] = {
    [sym_jsx_element] = STATE(646),
    [sym_jsx_expression] = STATE(646),
    [sym_jsx_opening_element] = STATE(234),
    [sym_jsx_self_closing_element] = STATE(646),
    [aux_sym_jsx_element_repeat1] = STATE(646),
    [anon_sym_LBRACE] = ACTIONS(392),
    [anon_sym_LT] = ACTIONS(4724),
    [sym_jsx_text] = ACTIONS(1200),
    [sym_comment] = ACTIONS(71),
  },
  [1995] = {
    [anon_sym_GT] = ACTIONS(4726),
    [sym_comment] = ACTIONS(71),
  },
  [1996] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2044),
    [anon_sym_SLASH] = ACTIONS(4728),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1997] = {
    [anon_sym_GT] = ACTIONS(4730),
    [sym_comment] = ACTIONS(71),
  },
  [1998] = {
    [sym_jsx_expression] = STATE(993),
    [sym_jsx_namespace_name] = STATE(537),
    [sym_jsx_attribute] = STATE(993),
    [aux_sym_jsx_opening_element_repeat1] = STATE(993),
    [anon_sym_LBRACE] = ACTIONS(946),
    [anon_sym_GT] = ACTIONS(2058),
    [anon_sym_SLASH] = ACTIONS(4732),
    [sym_jsx_identifier] = ACTIONS(954),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(956),
  },
  [1999] = {
    [sym_nested_identifier] = STATE(2173),
    [sym_jsx_namespace_name] = STATE(2174),
    [sym_jsx_identifier] = ACTIONS(4734),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4736),
  },
  [2000] = {
    [anon_sym_LBRACE] = ACTIONS(2324),
    [anon_sym_GT] = ACTIONS(2324),
    [anon_sym_SLASH] = ACTIONS(2326),
    [sym_jsx_identifier] = ACTIONS(2324),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2326),
  },
  [2001] = {
    [anon_sym_STAR] = ACTIONS(3129),
    [anon_sym_LBRACK] = ACTIONS(3129),
    [anon_sym_async] = ACTIONS(3131),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3129),
    [sym_number] = ACTIONS(3129),
    [sym_identifier] = ACTIONS(3131),
    [anon_sym_AT] = ACTIONS(3129),
    [anon_sym_static] = ACTIONS(3131),
    [anon_sym_get] = ACTIONS(3131),
    [anon_sym_set] = ACTIONS(3131),
  },
  [2002] = {
    [aux_sym_array_repeat1] = STATE(1050),
    [anon_sym_COMMA] = ACTIONS(1092),
    [anon_sym_RPAREN] = ACTIONS(4738),
    [sym_comment] = ACTIONS(71),
  },
  [2003] = {
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_async] = ACTIONS(1590),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_static] = ACTIONS(1590),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2004] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2118),
    [anon_sym_LBRACE] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2114),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2114),
    [anon_sym_GT] = ACTIONS(2114),
    [anon_sym_SLASH] = ACTIONS(2122),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2124),
    [anon_sym_AMP_AMP] = ACTIONS(2126),
    [anon_sym_PIPE_PIPE] = ACTIONS(2128),
    [anon_sym_GT_GT] = ACTIONS(2122),
    [anon_sym_GT_GT_GT] = ACTIONS(2118),
    [anon_sym_LT_LT] = ACTIONS(2118),
    [anon_sym_AMP] = ACTIONS(2130),
    [anon_sym_CARET] = ACTIONS(2128),
    [anon_sym_PIPE] = ACTIONS(2132),
    [anon_sym_PLUS] = ACTIONS(2134),
    [anon_sym_DASH] = ACTIONS(2134),
    [anon_sym_PERCENT] = ACTIONS(2118),
    [anon_sym_LT_EQ] = ACTIONS(2136),
    [anon_sym_EQ_EQ] = ACTIONS(2114),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2136),
    [anon_sym_BANG_EQ] = ACTIONS(2114),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2136),
    [anon_sym_GT_EQ] = ACTIONS(2136),
    [anon_sym_instanceof] = ACTIONS(2136),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2005] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2266),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2262),
    [anon_sym_COLON] = ACTIONS(3872),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(2262),
    [anon_sym_GT] = ACTIONS(2262),
    [anon_sym_SLASH] = ACTIONS(2270),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2272),
    [anon_sym_AMP_AMP] = ACTIONS(2274),
    [anon_sym_PIPE_PIPE] = ACTIONS(2276),
    [anon_sym_GT_GT] = ACTIONS(2270),
    [anon_sym_GT_GT_GT] = ACTIONS(2266),
    [anon_sym_LT_LT] = ACTIONS(2266),
    [anon_sym_AMP] = ACTIONS(2278),
    [anon_sym_CARET] = ACTIONS(2276),
    [anon_sym_PIPE] = ACTIONS(2280),
    [anon_sym_PLUS] = ACTIONS(2282),
    [anon_sym_DASH] = ACTIONS(2282),
    [anon_sym_PERCENT] = ACTIONS(2266),
    [anon_sym_LT_EQ] = ACTIONS(2284),
    [anon_sym_EQ_EQ] = ACTIONS(2262),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2284),
    [anon_sym_BANG_EQ] = ACTIONS(2262),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2284),
    [anon_sym_GT_EQ] = ACTIONS(2284),
    [anon_sym_instanceof] = ACTIONS(2284),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2006] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2007] = {
    [sym_parenthesized_expression] = STATE(2176),
    [sym__expression] = STATE(2176),
    [sym_yield_expression] = STATE(2176),
    [sym_object] = STATE(1080),
    [sym_array] = STATE(1081),
    [sym_jsx_element] = STATE(2176),
    [sym_jsx_fragment] = STATE(2176),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2176),
    [sym_anonymous_class] = STATE(2176),
    [sym_class] = STATE(2176),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2176),
    [sym_generator_function] = STATE(2176),
    [sym_call_expression] = STATE(2176),
    [sym_new_expression] = STATE(2176),
    [sym_await_expression] = STATE(2176),
    [sym_member_expression] = STATE(1082),
    [sym_subscript_expression] = STATE(1082),
    [sym_assignment_expression] = STATE(2176),
    [sym_augmented_assignment_expression] = STATE(2176),
    [sym_ternary_expression] = STATE(2176),
    [sym_binary_expression] = STATE(2176),
    [sym_unary_expression] = STATE(2176),
    [sym_update_expression] = STATE(2176),
    [sym_template_string] = STATE(2176),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1083),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2288),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2290),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2292),
    [anon_sym_new] = ACTIONS(2294),
    [anon_sym_await] = ACTIONS(2296),
    [anon_sym_PLUS] = ACTIONS(2298),
    [anon_sym_DASH] = ACTIONS(2298),
    [anon_sym_BANG] = ACTIONS(2300),
    [anon_sym_TILDE] = ACTIONS(2300),
    [anon_sym_typeof] = ACTIONS(2298),
    [anon_sym_void] = ACTIONS(2298),
    [anon_sym_delete] = ACTIONS(2298),
    [anon_sym_PLUS_PLUS] = ACTIONS(2302),
    [anon_sym_DASH_DASH] = ACTIONS(2302),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4740),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4742),
    [sym_number] = ACTIONS(4740),
    [sym_identifier] = ACTIONS(2308),
    [sym_this] = ACTIONS(4742),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4742),
    [sym_false] = ACTIONS(4742),
    [sym_null] = ACTIONS(4742),
    [sym_undefined] = ACTIONS(4742),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2310),
    [anon_sym_set] = ACTIONS(2310),
  },
  [2008] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(2423),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(2419),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_RBRACK] = ACTIONS(3872),
    [anon_sym_LT] = ACTIONS(2419),
    [anon_sym_GT] = ACTIONS(2419),
    [anon_sym_SLASH] = ACTIONS(2427),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(2429),
    [anon_sym_AMP_AMP] = ACTIONS(2431),
    [anon_sym_PIPE_PIPE] = ACTIONS(2433),
    [anon_sym_GT_GT] = ACTIONS(2427),
    [anon_sym_GT_GT_GT] = ACTIONS(2423),
    [anon_sym_LT_LT] = ACTIONS(2423),
    [anon_sym_AMP] = ACTIONS(2435),
    [anon_sym_CARET] = ACTIONS(2433),
    [anon_sym_PIPE] = ACTIONS(2437),
    [anon_sym_PLUS] = ACTIONS(2439),
    [anon_sym_DASH] = ACTIONS(2439),
    [anon_sym_PERCENT] = ACTIONS(2423),
    [anon_sym_LT_EQ] = ACTIONS(2441),
    [anon_sym_EQ_EQ] = ACTIONS(2419),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2441),
    [anon_sym_BANG_EQ] = ACTIONS(2419),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2441),
    [anon_sym_GT_EQ] = ACTIONS(2441),
    [anon_sym_instanceof] = ACTIONS(2441),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2009] = {
    [anon_sym_STAR] = ACTIONS(3864),
    [anon_sym_LBRACK] = ACTIONS(3864),
    [anon_sym_async] = ACTIONS(3866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3864),
    [sym_number] = ACTIONS(3864),
    [sym_identifier] = ACTIONS(3866),
    [anon_sym_AT] = ACTIONS(3864),
    [anon_sym_get] = ACTIONS(3866),
    [anon_sym_set] = ACTIONS(3866),
  },
  [2010] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2177),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2177),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2011] = {
    [sym_parenthesized_expression] = STATE(2178),
    [sym__expression] = STATE(2178),
    [sym_yield_expression] = STATE(2178),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2178),
    [sym_jsx_fragment] = STATE(2178),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2178),
    [sym_anonymous_class] = STATE(2178),
    [sym_class] = STATE(2178),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2178),
    [sym_generator_function] = STATE(2178),
    [sym_call_expression] = STATE(2178),
    [sym_new_expression] = STATE(2178),
    [sym_await_expression] = STATE(2178),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2178),
    [sym_augmented_assignment_expression] = STATE(2178),
    [sym_ternary_expression] = STATE(2178),
    [sym_binary_expression] = STATE(2178),
    [sym_unary_expression] = STATE(2178),
    [sym_update_expression] = STATE(2178),
    [sym_template_string] = STATE(2178),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4744),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4746),
    [sym_number] = ACTIONS(4744),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4746),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4746),
    [sym_false] = ACTIONS(4746),
    [sym_null] = ACTIONS(4746),
    [sym_undefined] = ACTIONS(4746),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2012] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2179),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2179),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2013] = {
    [sym_parenthesized_expression] = STATE(2180),
    [sym__expression] = STATE(2180),
    [sym_yield_expression] = STATE(2180),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2180),
    [sym_jsx_fragment] = STATE(2180),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2180),
    [sym_anonymous_class] = STATE(2180),
    [sym_class] = STATE(2180),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2180),
    [sym_generator_function] = STATE(2180),
    [sym_call_expression] = STATE(2180),
    [sym_new_expression] = STATE(2180),
    [sym_await_expression] = STATE(2180),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2180),
    [sym_augmented_assignment_expression] = STATE(2180),
    [sym_ternary_expression] = STATE(2180),
    [sym_binary_expression] = STATE(2180),
    [sym_unary_expression] = STATE(2180),
    [sym_update_expression] = STATE(2180),
    [sym_template_string] = STATE(2180),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4748),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4750),
    [sym_number] = ACTIONS(4748),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4750),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4750),
    [sym_false] = ACTIONS(4750),
    [sym_null] = ACTIONS(4750),
    [sym_undefined] = ACTIONS(4750),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2014] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2181),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2181),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2015] = {
    [sym_parenthesized_expression] = STATE(2182),
    [sym__expression] = STATE(2182),
    [sym_yield_expression] = STATE(2182),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2182),
    [sym_jsx_fragment] = STATE(2182),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2182),
    [sym_anonymous_class] = STATE(2182),
    [sym_class] = STATE(2182),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2182),
    [sym_generator_function] = STATE(2182),
    [sym_call_expression] = STATE(2182),
    [sym_new_expression] = STATE(2182),
    [sym_await_expression] = STATE(2182),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2182),
    [sym_augmented_assignment_expression] = STATE(2182),
    [sym_ternary_expression] = STATE(2182),
    [sym_binary_expression] = STATE(2182),
    [sym_unary_expression] = STATE(2182),
    [sym_update_expression] = STATE(2182),
    [sym_template_string] = STATE(2182),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4752),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4754),
    [sym_number] = ACTIONS(4752),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4754),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4754),
    [sym_false] = ACTIONS(4754),
    [sym_null] = ACTIONS(4754),
    [sym_undefined] = ACTIONS(4754),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2016] = {
    [anon_sym_RPAREN] = ACTIONS(4756),
    [sym_comment] = ACTIONS(71),
  },
  [2017] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4758),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2018] = {
    [sym_export_statement] = STATE(2044),
    [sym__declaration] = STATE(2044),
    [sym_import_statement] = STATE(2044),
    [sym_expression_statement] = STATE(2044),
    [sym_variable_declaration] = STATE(2044),
    [sym_lexical_declaration] = STATE(2044),
    [sym_statement_block] = STATE(2044),
    [sym_if_statement] = STATE(2044),
    [sym_switch_statement] = STATE(2044),
    [sym_for_statement] = STATE(2044),
    [sym_for_in_statement] = STATE(2044),
    [sym_for_of_statement] = STATE(2044),
    [sym_while_statement] = STATE(2044),
    [sym_do_statement] = STATE(2044),
    [sym_try_statement] = STATE(2044),
    [sym_with_statement] = STATE(2044),
    [sym_break_statement] = STATE(2044),
    [sym_continue_statement] = STATE(2044),
    [sym_debugger_statement] = STATE(2044),
    [sym_return_statement] = STATE(2044),
    [sym_throw_statement] = STATE(2044),
    [sym_empty_statement] = STATE(2044),
    [sym_labeled_statement] = STATE(2044),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2019] = {
    [anon_sym_RPAREN] = ACTIONS(4760),
    [sym_comment] = ACTIONS(71),
  },
  [2020] = {
    [anon_sym_RPAREN] = ACTIONS(4762),
    [sym_comment] = ACTIONS(71),
  },
  [2021] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4764),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2022] = {
    [anon_sym_RPAREN] = ACTIONS(4766),
    [sym_comment] = ACTIONS(71),
  },
  [2023] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4768),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2024] = {
    [sym_export_statement] = STATE(2130),
    [sym__declaration] = STATE(2130),
    [sym_import_statement] = STATE(2130),
    [sym_expression_statement] = STATE(2130),
    [sym_variable_declaration] = STATE(2130),
    [sym_lexical_declaration] = STATE(2130),
    [sym_statement_block] = STATE(2130),
    [sym_if_statement] = STATE(2130),
    [sym_switch_statement] = STATE(2130),
    [sym_for_statement] = STATE(2130),
    [sym_for_in_statement] = STATE(2130),
    [sym_for_of_statement] = STATE(2130),
    [sym_while_statement] = STATE(2130),
    [sym_do_statement] = STATE(2130),
    [sym_try_statement] = STATE(2130),
    [sym_with_statement] = STATE(2130),
    [sym_break_statement] = STATE(2130),
    [sym_continue_statement] = STATE(2130),
    [sym_debugger_statement] = STATE(2130),
    [sym_return_statement] = STATE(2130),
    [sym_throw_statement] = STATE(2130),
    [sym_empty_statement] = STATE(2130),
    [sym_labeled_statement] = STATE(2130),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2025] = {
    [sym_export_statement] = STATE(2131),
    [sym__declaration] = STATE(2131),
    [sym_import_statement] = STATE(2131),
    [sym_expression_statement] = STATE(2131),
    [sym_variable_declaration] = STATE(2131),
    [sym_lexical_declaration] = STATE(2131),
    [sym_statement_block] = STATE(2131),
    [sym_if_statement] = STATE(2131),
    [sym_switch_statement] = STATE(2131),
    [sym_for_statement] = STATE(2131),
    [sym_for_in_statement] = STATE(2131),
    [sym_for_of_statement] = STATE(2131),
    [sym_while_statement] = STATE(2131),
    [sym_do_statement] = STATE(2131),
    [sym_try_statement] = STATE(2131),
    [sym_with_statement] = STATE(2131),
    [sym_break_statement] = STATE(2131),
    [sym_continue_statement] = STATE(2131),
    [sym_debugger_statement] = STATE(2131),
    [sym_return_statement] = STATE(2131),
    [sym_throw_statement] = STATE(2131),
    [sym_empty_statement] = STATE(2131),
    [sym_labeled_statement] = STATE(2131),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2026] = {
    [sym_export_statement] = STATE(2132),
    [sym__declaration] = STATE(2132),
    [sym_import_statement] = STATE(2132),
    [sym_expression_statement] = STATE(2132),
    [sym_variable_declaration] = STATE(2132),
    [sym_lexical_declaration] = STATE(2132),
    [sym_statement_block] = STATE(2132),
    [sym_if_statement] = STATE(2132),
    [sym_switch_statement] = STATE(2132),
    [sym_for_statement] = STATE(2132),
    [sym_for_in_statement] = STATE(2132),
    [sym_for_of_statement] = STATE(2132),
    [sym_while_statement] = STATE(2132),
    [sym_do_statement] = STATE(2132),
    [sym_try_statement] = STATE(2132),
    [sym_with_statement] = STATE(2132),
    [sym_break_statement] = STATE(2132),
    [sym_continue_statement] = STATE(2132),
    [sym_debugger_statement] = STATE(2132),
    [sym_return_statement] = STATE(2132),
    [sym_throw_statement] = STATE(2132),
    [sym_empty_statement] = STATE(2132),
    [sym_labeled_statement] = STATE(2132),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2027] = {
    [sym_export_statement] = STATE(2133),
    [sym__declaration] = STATE(2133),
    [sym_import_statement] = STATE(2133),
    [sym_expression_statement] = STATE(2133),
    [sym_variable_declaration] = STATE(2133),
    [sym_lexical_declaration] = STATE(2133),
    [sym_statement_block] = STATE(2133),
    [sym_if_statement] = STATE(2133),
    [sym_switch_statement] = STATE(2133),
    [sym_for_statement] = STATE(2133),
    [sym_for_in_statement] = STATE(2133),
    [sym_for_of_statement] = STATE(2133),
    [sym_while_statement] = STATE(2133),
    [sym_do_statement] = STATE(2133),
    [sym_try_statement] = STATE(2133),
    [sym_with_statement] = STATE(2133),
    [sym_break_statement] = STATE(2133),
    [sym_continue_statement] = STATE(2133),
    [sym_debugger_statement] = STATE(2133),
    [sym_return_statement] = STATE(2133),
    [sym_throw_statement] = STATE(2133),
    [sym_empty_statement] = STATE(2133),
    [sym_labeled_statement] = STATE(2133),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2028] = {
    [sym_export_statement] = STATE(2134),
    [sym__declaration] = STATE(2134),
    [sym_import_statement] = STATE(2134),
    [sym_expression_statement] = STATE(2134),
    [sym_variable_declaration] = STATE(2134),
    [sym_lexical_declaration] = STATE(2134),
    [sym_statement_block] = STATE(2134),
    [sym_if_statement] = STATE(2134),
    [sym_switch_statement] = STATE(2134),
    [sym_for_statement] = STATE(2134),
    [sym_for_in_statement] = STATE(2134),
    [sym_for_of_statement] = STATE(2134),
    [sym_while_statement] = STATE(2134),
    [sym_do_statement] = STATE(2134),
    [sym_try_statement] = STATE(2134),
    [sym_with_statement] = STATE(2134),
    [sym_break_statement] = STATE(2134),
    [sym_continue_statement] = STATE(2134),
    [sym_debugger_statement] = STATE(2134),
    [sym_return_statement] = STATE(2134),
    [sym_throw_statement] = STATE(2134),
    [sym_empty_statement] = STATE(2134),
    [sym_labeled_statement] = STATE(2134),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2029] = {
    [sym_export_statement] = STATE(2135),
    [sym__declaration] = STATE(2135),
    [sym_import_statement] = STATE(2135),
    [sym_expression_statement] = STATE(2135),
    [sym_variable_declaration] = STATE(2135),
    [sym_lexical_declaration] = STATE(2135),
    [sym_statement_block] = STATE(2135),
    [sym_if_statement] = STATE(2135),
    [sym_switch_statement] = STATE(2135),
    [sym_for_statement] = STATE(2135),
    [sym_for_in_statement] = STATE(2135),
    [sym_for_of_statement] = STATE(2135),
    [sym_while_statement] = STATE(2135),
    [sym_do_statement] = STATE(2135),
    [sym_try_statement] = STATE(2135),
    [sym_with_statement] = STATE(2135),
    [sym_break_statement] = STATE(2135),
    [sym_continue_statement] = STATE(2135),
    [sym_debugger_statement] = STATE(2135),
    [sym_return_statement] = STATE(2135),
    [sym_throw_statement] = STATE(2135),
    [sym_empty_statement] = STATE(2135),
    [sym_labeled_statement] = STATE(2135),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2030] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3872),
    [anon_sym_STAR] = ACTIONS(2551),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_RBRACE] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(2547),
    [anon_sym_SEMI] = ACTIONS(3872),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(2547),
    [anon_sym_GT] = ACTIONS(2547),
    [anon_sym_SLASH] = ACTIONS(2553),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(2555),
    [anon_sym_AMP_AMP] = ACTIONS(2557),
    [anon_sym_PIPE_PIPE] = ACTIONS(2559),
    [anon_sym_GT_GT] = ACTIONS(2553),
    [anon_sym_GT_GT_GT] = ACTIONS(2551),
    [anon_sym_LT_LT] = ACTIONS(2551),
    [anon_sym_AMP] = ACTIONS(2561),
    [anon_sym_CARET] = ACTIONS(2559),
    [anon_sym_PIPE] = ACTIONS(2563),
    [anon_sym_PLUS] = ACTIONS(2565),
    [anon_sym_DASH] = ACTIONS(2565),
    [anon_sym_PERCENT] = ACTIONS(2551),
    [anon_sym_LT_EQ] = ACTIONS(2567),
    [anon_sym_EQ_EQ] = ACTIONS(2547),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2567),
    [anon_sym_BANG_EQ] = ACTIONS(2547),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2567),
    [anon_sym_GT_EQ] = ACTIONS(2567),
    [anon_sym_instanceof] = ACTIONS(2567),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2031] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [ts_builtin_sym_end] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_else] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3035),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3035),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [2032] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [ts_builtin_sym_end] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_else] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3856),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3856),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [2033] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [ts_builtin_sym_end] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_else] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3860),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3860),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [2034] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2035] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [ts_builtin_sym_end] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_else] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3934),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3934),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [2036] = {
    [anon_sym_RPAREN] = ACTIONS(4770),
    [sym_comment] = ACTIONS(71),
  },
  [2037] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4772),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2038] = {
    [anon_sym_RPAREN] = ACTIONS(4774),
    [sym_comment] = ACTIONS(71),
  },
  [2039] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4776),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2040] = {
    [anon_sym_RPAREN] = ACTIONS(4778),
    [sym_comment] = ACTIONS(71),
  },
  [2041] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4780),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2042] = {
    [sym_export_statement] = STATE(2196),
    [sym__declaration] = STATE(2196),
    [sym_import_statement] = STATE(2196),
    [sym_expression_statement] = STATE(2196),
    [sym_variable_declaration] = STATE(2196),
    [sym_lexical_declaration] = STATE(2196),
    [sym_statement_block] = STATE(2196),
    [sym_if_statement] = STATE(2196),
    [sym_switch_statement] = STATE(2196),
    [sym_for_statement] = STATE(2196),
    [sym_for_in_statement] = STATE(2196),
    [sym_for_of_statement] = STATE(2196),
    [sym_while_statement] = STATE(2196),
    [sym_do_statement] = STATE(2196),
    [sym_try_statement] = STATE(2196),
    [sym_with_statement] = STATE(2196),
    [sym_break_statement] = STATE(2196),
    [sym_continue_statement] = STATE(2196),
    [sym_debugger_statement] = STATE(2196),
    [sym_return_statement] = STATE(2196),
    [sym_throw_statement] = STATE(2196),
    [sym_empty_statement] = STATE(2196),
    [sym_labeled_statement] = STATE(2196),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2043] = {
    [sym_export_statement] = STATE(2197),
    [sym__declaration] = STATE(2197),
    [sym_import_statement] = STATE(2197),
    [sym_expression_statement] = STATE(2197),
    [sym_variable_declaration] = STATE(2197),
    [sym_lexical_declaration] = STATE(2197),
    [sym_statement_block] = STATE(2197),
    [sym_if_statement] = STATE(2197),
    [sym_switch_statement] = STATE(2197),
    [sym_for_statement] = STATE(2197),
    [sym_for_in_statement] = STATE(2197),
    [sym_for_of_statement] = STATE(2197),
    [sym_while_statement] = STATE(2197),
    [sym_do_statement] = STATE(2197),
    [sym_try_statement] = STATE(2197),
    [sym_with_statement] = STATE(2197),
    [sym_break_statement] = STATE(2197),
    [sym_continue_statement] = STATE(2197),
    [sym_debugger_statement] = STATE(2197),
    [sym_return_statement] = STATE(2197),
    [sym_throw_statement] = STATE(2197),
    [sym_empty_statement] = STATE(2197),
    [sym_labeled_statement] = STATE(2197),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2044] = {
    [ts_builtin_sym_end] = ACTIONS(4264),
    [anon_sym_export] = ACTIONS(4266),
    [anon_sym_LBRACE] = ACTIONS(4264),
    [anon_sym_RBRACE] = ACTIONS(4264),
    [anon_sym_import] = ACTIONS(4266),
    [anon_sym_var] = ACTIONS(4266),
    [anon_sym_let] = ACTIONS(4266),
    [anon_sym_const] = ACTIONS(4266),
    [anon_sym_if] = ACTIONS(4266),
    [anon_sym_else] = ACTIONS(4266),
    [anon_sym_switch] = ACTIONS(4266),
    [anon_sym_for] = ACTIONS(4266),
    [anon_sym_LPAREN] = ACTIONS(4264),
    [anon_sym_while] = ACTIONS(4266),
    [anon_sym_do] = ACTIONS(4266),
    [anon_sym_try] = ACTIONS(4266),
    [anon_sym_with] = ACTIONS(4266),
    [anon_sym_break] = ACTIONS(4266),
    [anon_sym_continue] = ACTIONS(4266),
    [anon_sym_debugger] = ACTIONS(4266),
    [anon_sym_return] = ACTIONS(4266),
    [anon_sym_throw] = ACTIONS(4266),
    [anon_sym_SEMI] = ACTIONS(4264),
    [anon_sym_yield] = ACTIONS(4266),
    [anon_sym_LBRACK] = ACTIONS(4264),
    [anon_sym_LT] = ACTIONS(4264),
    [anon_sym_class] = ACTIONS(4266),
    [anon_sym_async] = ACTIONS(4266),
    [anon_sym_function] = ACTIONS(4266),
    [anon_sym_EQ_GT] = ACTIONS(4264),
    [anon_sym_new] = ACTIONS(4266),
    [anon_sym_await] = ACTIONS(4266),
    [anon_sym_PLUS] = ACTIONS(4266),
    [anon_sym_DASH] = ACTIONS(4266),
    [anon_sym_BANG] = ACTIONS(4264),
    [anon_sym_TILDE] = ACTIONS(4264),
    [anon_sym_typeof] = ACTIONS(4266),
    [anon_sym_void] = ACTIONS(4266),
    [anon_sym_delete] = ACTIONS(4266),
    [anon_sym_PLUS_PLUS] = ACTIONS(4264),
    [anon_sym_DASH_DASH] = ACTIONS(4264),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4264),
    [anon_sym_BQUOTE] = ACTIONS(4264),
    [sym_regex] = ACTIONS(4266),
    [sym_number] = ACTIONS(4264),
    [sym_identifier] = ACTIONS(4266),
    [sym_this] = ACTIONS(4266),
    [sym_super] = ACTIONS(4266),
    [sym_true] = ACTIONS(4266),
    [sym_false] = ACTIONS(4266),
    [sym_null] = ACTIONS(4266),
    [sym_undefined] = ACTIONS(4266),
    [anon_sym_AT] = ACTIONS(4264),
    [anon_sym_get] = ACTIONS(4266),
    [anon_sym_set] = ACTIONS(4266),
  },
  [2045] = {
    [sym_export_statement] = STATE(2198),
    [sym__declaration] = STATE(2198),
    [sym_import_statement] = STATE(2198),
    [sym_expression_statement] = STATE(2198),
    [sym_variable_declaration] = STATE(2198),
    [sym_lexical_declaration] = STATE(2198),
    [sym_statement_block] = STATE(2198),
    [sym_if_statement] = STATE(2198),
    [sym_switch_statement] = STATE(2198),
    [sym_for_statement] = STATE(2198),
    [sym_for_in_statement] = STATE(2198),
    [sym_for_of_statement] = STATE(2198),
    [sym_while_statement] = STATE(2198),
    [sym_do_statement] = STATE(2198),
    [sym_try_statement] = STATE(2198),
    [sym_with_statement] = STATE(2198),
    [sym_break_statement] = STATE(2198),
    [sym_continue_statement] = STATE(2198),
    [sym_debugger_statement] = STATE(2198),
    [sym_return_statement] = STATE(2198),
    [sym_throw_statement] = STATE(2198),
    [sym_empty_statement] = STATE(2198),
    [sym_labeled_statement] = STATE(2198),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2046] = {
    [sym_export_statement] = STATE(2199),
    [sym__declaration] = STATE(2199),
    [sym_import_statement] = STATE(2199),
    [sym_expression_statement] = STATE(2199),
    [sym_variable_declaration] = STATE(2199),
    [sym_lexical_declaration] = STATE(2199),
    [sym_statement_block] = STATE(2199),
    [sym_if_statement] = STATE(2199),
    [sym_switch_statement] = STATE(2199),
    [sym_for_statement] = STATE(2199),
    [sym_for_in_statement] = STATE(2199),
    [sym_for_of_statement] = STATE(2199),
    [sym_while_statement] = STATE(2199),
    [sym_do_statement] = STATE(2199),
    [sym_try_statement] = STATE(2199),
    [sym_with_statement] = STATE(2199),
    [sym_break_statement] = STATE(2199),
    [sym_continue_statement] = STATE(2199),
    [sym_debugger_statement] = STATE(2199),
    [sym_return_statement] = STATE(2199),
    [sym_throw_statement] = STATE(2199),
    [sym_empty_statement] = STATE(2199),
    [sym_labeled_statement] = STATE(2199),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2047] = {
    [sym_export_statement] = STATE(2200),
    [sym__declaration] = STATE(2200),
    [sym_import_statement] = STATE(2200),
    [sym_expression_statement] = STATE(2200),
    [sym_variable_declaration] = STATE(2200),
    [sym_lexical_declaration] = STATE(2200),
    [sym_statement_block] = STATE(2200),
    [sym_if_statement] = STATE(2200),
    [sym_switch_statement] = STATE(2200),
    [sym_for_statement] = STATE(2200),
    [sym_for_in_statement] = STATE(2200),
    [sym_for_of_statement] = STATE(2200),
    [sym_while_statement] = STATE(2200),
    [sym_do_statement] = STATE(2200),
    [sym_try_statement] = STATE(2200),
    [sym_with_statement] = STATE(2200),
    [sym_break_statement] = STATE(2200),
    [sym_continue_statement] = STATE(2200),
    [sym_debugger_statement] = STATE(2200),
    [sym_return_statement] = STATE(2200),
    [sym_throw_statement] = STATE(2200),
    [sym_empty_statement] = STATE(2200),
    [sym_labeled_statement] = STATE(2200),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2048] = {
    [sym_export_statement] = STATE(2201),
    [sym__declaration] = STATE(2201),
    [sym_import_statement] = STATE(2201),
    [sym_expression_statement] = STATE(2201),
    [sym_variable_declaration] = STATE(2201),
    [sym_lexical_declaration] = STATE(2201),
    [sym_statement_block] = STATE(2201),
    [sym_if_statement] = STATE(2201),
    [sym_switch_statement] = STATE(2201),
    [sym_for_statement] = STATE(2201),
    [sym_for_in_statement] = STATE(2201),
    [sym_for_of_statement] = STATE(2201),
    [sym_while_statement] = STATE(2201),
    [sym_do_statement] = STATE(2201),
    [sym_try_statement] = STATE(2201),
    [sym_with_statement] = STATE(2201),
    [sym_break_statement] = STATE(2201),
    [sym_continue_statement] = STATE(2201),
    [sym_debugger_statement] = STATE(2201),
    [sym_return_statement] = STATE(2201),
    [sym_throw_statement] = STATE(2201),
    [sym_empty_statement] = STATE(2201),
    [sym_labeled_statement] = STATE(2201),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2049] = {
    [sym_export_statement] = STATE(2202),
    [sym__declaration] = STATE(2202),
    [sym_import_statement] = STATE(2202),
    [sym_expression_statement] = STATE(2202),
    [sym_variable_declaration] = STATE(2202),
    [sym_lexical_declaration] = STATE(2202),
    [sym_statement_block] = STATE(2202),
    [sym_if_statement] = STATE(2202),
    [sym_switch_statement] = STATE(2202),
    [sym_for_statement] = STATE(2202),
    [sym_for_in_statement] = STATE(2202),
    [sym_for_of_statement] = STATE(2202),
    [sym_while_statement] = STATE(2202),
    [sym_do_statement] = STATE(2202),
    [sym_try_statement] = STATE(2202),
    [sym_with_statement] = STATE(2202),
    [sym_break_statement] = STATE(2202),
    [sym_continue_statement] = STATE(2202),
    [sym_debugger_statement] = STATE(2202),
    [sym_return_statement] = STATE(2202),
    [sym_throw_statement] = STATE(2202),
    [sym_empty_statement] = STATE(2202),
    [sym_labeled_statement] = STATE(2202),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2050] = {
    [ts_builtin_sym_end] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2051] = {
    [sym_statement_block] = STATE(2203),
    [anon_sym_LBRACE] = ACTIONS(3472),
    [sym_comment] = ACTIONS(71),
  },
  [2052] = {
    [sym__automatic_semicolon] = ACTIONS(4782),
    [anon_sym_SEMI] = ACTIONS(4782),
    [sym_comment] = ACTIONS(71),
  },
  [2053] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(4782),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(4782),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2054] = {
    [sym_class_heritage] = STATE(2206),
    [sym_class_body] = STATE(2207),
    [anon_sym_LBRACE] = ACTIONS(4784),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2055] = {
    [sym_formal_parameters] = STATE(2209),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4786),
  },
  [2056] = {
    [sym_formal_parameters] = STATE(2211),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4788),
  },
  [2057] = {
    [sym_formal_parameters] = STATE(2209),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2058] = {
    [sym_statement_block] = STATE(2213),
    [anon_sym_LBRACE] = ACTIONS(4790),
    [sym_comment] = ACTIONS(71),
  },
  [2059] = {
    [anon_sym_export] = ACTIONS(1399),
    [anon_sym_default] = ACTIONS(1399),
    [anon_sym_LBRACE] = ACTIONS(1397),
    [anon_sym_RBRACE] = ACTIONS(1397),
    [anon_sym_import] = ACTIONS(1399),
    [anon_sym_var] = ACTIONS(1399),
    [anon_sym_let] = ACTIONS(1399),
    [anon_sym_const] = ACTIONS(1399),
    [anon_sym_if] = ACTIONS(1399),
    [anon_sym_switch] = ACTIONS(1399),
    [anon_sym_for] = ACTIONS(1399),
    [anon_sym_LPAREN] = ACTIONS(1397),
    [anon_sym_while] = ACTIONS(1399),
    [anon_sym_do] = ACTIONS(1399),
    [anon_sym_try] = ACTIONS(1399),
    [anon_sym_with] = ACTIONS(1399),
    [anon_sym_break] = ACTIONS(1399),
    [anon_sym_continue] = ACTIONS(1399),
    [anon_sym_debugger] = ACTIONS(1399),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_throw] = ACTIONS(1399),
    [anon_sym_SEMI] = ACTIONS(1397),
    [anon_sym_case] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1399),
    [anon_sym_LBRACK] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1397),
    [anon_sym_class] = ACTIONS(1399),
    [anon_sym_async] = ACTIONS(1399),
    [anon_sym_function] = ACTIONS(1399),
    [anon_sym_EQ_GT] = ACTIONS(1397),
    [anon_sym_new] = ACTIONS(1399),
    [anon_sym_await] = ACTIONS(1399),
    [anon_sym_PLUS] = ACTIONS(1399),
    [anon_sym_DASH] = ACTIONS(1399),
    [anon_sym_BANG] = ACTIONS(1397),
    [anon_sym_TILDE] = ACTIONS(1397),
    [anon_sym_typeof] = ACTIONS(1399),
    [anon_sym_void] = ACTIONS(1399),
    [anon_sym_delete] = ACTIONS(1399),
    [anon_sym_PLUS_PLUS] = ACTIONS(1397),
    [anon_sym_DASH_DASH] = ACTIONS(1397),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1397),
    [anon_sym_BQUOTE] = ACTIONS(1397),
    [sym_regex] = ACTIONS(1399),
    [sym_number] = ACTIONS(1397),
    [sym_identifier] = ACTIONS(1399),
    [sym_this] = ACTIONS(1399),
    [sym_super] = ACTIONS(1399),
    [sym_true] = ACTIONS(1399),
    [sym_false] = ACTIONS(1399),
    [sym_null] = ACTIONS(1399),
    [sym_undefined] = ACTIONS(1399),
    [anon_sym_AT] = ACTIONS(1397),
    [anon_sym_get] = ACTIONS(1399),
    [anon_sym_set] = ACTIONS(1399),
  },
  [2060] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4792),
  },
  [2061] = {
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2062] = {
    [anon_sym_export] = ACTIONS(1629),
    [anon_sym_default] = ACTIONS(1629),
    [anon_sym_LBRACE] = ACTIONS(1627),
    [anon_sym_RBRACE] = ACTIONS(1627),
    [anon_sym_import] = ACTIONS(1629),
    [anon_sym_var] = ACTIONS(1629),
    [anon_sym_let] = ACTIONS(1629),
    [anon_sym_const] = ACTIONS(1629),
    [anon_sym_if] = ACTIONS(1629),
    [anon_sym_switch] = ACTIONS(1629),
    [anon_sym_for] = ACTIONS(1629),
    [anon_sym_LPAREN] = ACTIONS(1627),
    [anon_sym_while] = ACTIONS(1629),
    [anon_sym_do] = ACTIONS(1629),
    [anon_sym_try] = ACTIONS(1629),
    [anon_sym_with] = ACTIONS(1629),
    [anon_sym_break] = ACTIONS(1629),
    [anon_sym_continue] = ACTIONS(1629),
    [anon_sym_debugger] = ACTIONS(1629),
    [anon_sym_return] = ACTIONS(1629),
    [anon_sym_throw] = ACTIONS(1629),
    [anon_sym_SEMI] = ACTIONS(1627),
    [anon_sym_case] = ACTIONS(1629),
    [anon_sym_yield] = ACTIONS(1629),
    [anon_sym_LBRACK] = ACTIONS(1627),
    [anon_sym_LT] = ACTIONS(1627),
    [anon_sym_class] = ACTIONS(1629),
    [anon_sym_async] = ACTIONS(1629),
    [anon_sym_function] = ACTIONS(1629),
    [anon_sym_EQ_GT] = ACTIONS(1627),
    [anon_sym_new] = ACTIONS(1629),
    [anon_sym_await] = ACTIONS(1629),
    [anon_sym_PLUS] = ACTIONS(1629),
    [anon_sym_DASH] = ACTIONS(1629),
    [anon_sym_BANG] = ACTIONS(1627),
    [anon_sym_TILDE] = ACTIONS(1627),
    [anon_sym_typeof] = ACTIONS(1629),
    [anon_sym_void] = ACTIONS(1629),
    [anon_sym_delete] = ACTIONS(1629),
    [anon_sym_PLUS_PLUS] = ACTIONS(1627),
    [anon_sym_DASH_DASH] = ACTIONS(1627),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1627),
    [anon_sym_BQUOTE] = ACTIONS(1627),
    [sym_regex] = ACTIONS(1629),
    [sym_number] = ACTIONS(1627),
    [sym_identifier] = ACTIONS(1629),
    [sym_this] = ACTIONS(1629),
    [sym_super] = ACTIONS(1629),
    [sym_true] = ACTIONS(1629),
    [sym_false] = ACTIONS(1629),
    [sym_null] = ACTIONS(1629),
    [sym_undefined] = ACTIONS(1629),
    [anon_sym_AT] = ACTIONS(1627),
    [anon_sym_get] = ACTIONS(1629),
    [anon_sym_set] = ACTIONS(1629),
  },
  [2063] = {
    [sym__automatic_semicolon] = ACTIONS(4794),
    [anon_sym_SEMI] = ACTIONS(4794),
    [sym_comment] = ACTIONS(71),
  },
  [2064] = {
    [anon_sym_export] = ACTIONS(1643),
    [anon_sym_default] = ACTIONS(1643),
    [anon_sym_LBRACE] = ACTIONS(1641),
    [anon_sym_RBRACE] = ACTIONS(1641),
    [anon_sym_import] = ACTIONS(1643),
    [anon_sym_var] = ACTIONS(1643),
    [anon_sym_let] = ACTIONS(1643),
    [anon_sym_const] = ACTIONS(1643),
    [anon_sym_if] = ACTIONS(1643),
    [anon_sym_switch] = ACTIONS(1643),
    [anon_sym_for] = ACTIONS(1643),
    [anon_sym_LPAREN] = ACTIONS(1641),
    [anon_sym_while] = ACTIONS(1643),
    [anon_sym_do] = ACTIONS(1643),
    [anon_sym_try] = ACTIONS(1643),
    [anon_sym_with] = ACTIONS(1643),
    [anon_sym_break] = ACTIONS(1643),
    [anon_sym_continue] = ACTIONS(1643),
    [anon_sym_debugger] = ACTIONS(1643),
    [anon_sym_return] = ACTIONS(1643),
    [anon_sym_throw] = ACTIONS(1643),
    [anon_sym_SEMI] = ACTIONS(1641),
    [anon_sym_case] = ACTIONS(1643),
    [anon_sym_yield] = ACTIONS(1643),
    [anon_sym_LBRACK] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1641),
    [anon_sym_class] = ACTIONS(1643),
    [anon_sym_async] = ACTIONS(1643),
    [anon_sym_function] = ACTIONS(1643),
    [anon_sym_EQ_GT] = ACTIONS(1641),
    [anon_sym_new] = ACTIONS(1643),
    [anon_sym_await] = ACTIONS(1643),
    [anon_sym_PLUS] = ACTIONS(1643),
    [anon_sym_DASH] = ACTIONS(1643),
    [anon_sym_BANG] = ACTIONS(1641),
    [anon_sym_TILDE] = ACTIONS(1641),
    [anon_sym_typeof] = ACTIONS(1643),
    [anon_sym_void] = ACTIONS(1643),
    [anon_sym_delete] = ACTIONS(1643),
    [anon_sym_PLUS_PLUS] = ACTIONS(1641),
    [anon_sym_DASH_DASH] = ACTIONS(1641),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1641),
    [anon_sym_BQUOTE] = ACTIONS(1641),
    [sym_regex] = ACTIONS(1643),
    [sym_number] = ACTIONS(1641),
    [sym_identifier] = ACTIONS(1643),
    [sym_this] = ACTIONS(1643),
    [sym_super] = ACTIONS(1643),
    [sym_true] = ACTIONS(1643),
    [sym_false] = ACTIONS(1643),
    [sym_null] = ACTIONS(1643),
    [sym_undefined] = ACTIONS(1643),
    [anon_sym_AT] = ACTIONS(1641),
    [anon_sym_get] = ACTIONS(1643),
    [anon_sym_set] = ACTIONS(1643),
  },
  [2065] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(4796),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4796),
    [sym_comment] = ACTIONS(71),
  },
  [2066] = {
    [anon_sym_export] = ACTIONS(1653),
    [anon_sym_default] = ACTIONS(1653),
    [anon_sym_LBRACE] = ACTIONS(1651),
    [anon_sym_RBRACE] = ACTIONS(1651),
    [anon_sym_import] = ACTIONS(1653),
    [anon_sym_var] = ACTIONS(1653),
    [anon_sym_let] = ACTIONS(1653),
    [anon_sym_const] = ACTIONS(1653),
    [anon_sym_if] = ACTIONS(1653),
    [anon_sym_switch] = ACTIONS(1653),
    [anon_sym_for] = ACTIONS(1653),
    [anon_sym_LPAREN] = ACTIONS(1651),
    [anon_sym_while] = ACTIONS(1653),
    [anon_sym_do] = ACTIONS(1653),
    [anon_sym_try] = ACTIONS(1653),
    [anon_sym_with] = ACTIONS(1653),
    [anon_sym_break] = ACTIONS(1653),
    [anon_sym_continue] = ACTIONS(1653),
    [anon_sym_debugger] = ACTIONS(1653),
    [anon_sym_return] = ACTIONS(1653),
    [anon_sym_throw] = ACTIONS(1653),
    [anon_sym_SEMI] = ACTIONS(1651),
    [anon_sym_case] = ACTIONS(1653),
    [anon_sym_yield] = ACTIONS(1653),
    [anon_sym_LBRACK] = ACTIONS(1651),
    [anon_sym_LT] = ACTIONS(1651),
    [anon_sym_class] = ACTIONS(1653),
    [anon_sym_async] = ACTIONS(1653),
    [anon_sym_function] = ACTIONS(1653),
    [anon_sym_EQ_GT] = ACTIONS(1651),
    [anon_sym_new] = ACTIONS(1653),
    [anon_sym_await] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1653),
    [anon_sym_BANG] = ACTIONS(1651),
    [anon_sym_TILDE] = ACTIONS(1651),
    [anon_sym_typeof] = ACTIONS(1653),
    [anon_sym_void] = ACTIONS(1653),
    [anon_sym_delete] = ACTIONS(1653),
    [anon_sym_PLUS_PLUS] = ACTIONS(1651),
    [anon_sym_DASH_DASH] = ACTIONS(1651),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1651),
    [anon_sym_BQUOTE] = ACTIONS(1651),
    [sym_regex] = ACTIONS(1653),
    [sym_number] = ACTIONS(1651),
    [sym_identifier] = ACTIONS(1653),
    [sym_this] = ACTIONS(1653),
    [sym_super] = ACTIONS(1653),
    [sym_true] = ACTIONS(1653),
    [sym_false] = ACTIONS(1653),
    [sym_null] = ACTIONS(1653),
    [sym_undefined] = ACTIONS(1653),
    [anon_sym_AT] = ACTIONS(1651),
    [anon_sym_get] = ACTIONS(1653),
    [anon_sym_set] = ACTIONS(1653),
  },
  [2067] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(4798),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4798),
    [sym_comment] = ACTIONS(71),
  },
  [2068] = {
    [sym_export_clause] = STATE(2223),
    [sym__declaration] = STATE(2224),
    [sym_variable_declaration] = STATE(2224),
    [sym_lexical_declaration] = STATE(2224),
    [sym_class] = STATE(2225),
    [sym_function] = STATE(2225),
    [sym_generator_function] = STATE(2225),
    [sym_decorator] = STATE(2226),
    [aux_sym_export_statement_repeat1] = STATE(2226),
    [anon_sym_STAR] = ACTIONS(4800),
    [anon_sym_default] = ACTIONS(4802),
    [anon_sym_LBRACE] = ACTIONS(93),
    [anon_sym_var] = ACTIONS(4804),
    [anon_sym_let] = ACTIONS(4806),
    [anon_sym_const] = ACTIONS(4806),
    [anon_sym_class] = ACTIONS(4808),
    [anon_sym_async] = ACTIONS(4810),
    [anon_sym_function] = ACTIONS(4812),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [2069] = {
    [sym_export_statement] = STATE(2228),
    [sym__declaration] = STATE(2228),
    [sym_import_statement] = STATE(2228),
    [sym_expression_statement] = STATE(2228),
    [sym_variable_declaration] = STATE(2228),
    [sym_lexical_declaration] = STATE(2228),
    [sym_statement_block] = STATE(2228),
    [sym_if_statement] = STATE(2228),
    [sym_switch_statement] = STATE(2228),
    [sym_for_statement] = STATE(2228),
    [sym_for_in_statement] = STATE(2228),
    [sym_for_of_statement] = STATE(2228),
    [sym_while_statement] = STATE(2228),
    [sym_do_statement] = STATE(2228),
    [sym_try_statement] = STATE(2228),
    [sym_with_statement] = STATE(2228),
    [sym_break_statement] = STATE(2228),
    [sym_continue_statement] = STATE(2228),
    [sym_debugger_statement] = STATE(2228),
    [sym_return_statement] = STATE(2228),
    [sym_throw_statement] = STATE(2228),
    [sym_empty_statement] = STATE(2228),
    [sym_labeled_statement] = STATE(2228),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(75),
    [sym_assignment_pattern] = STATE(76),
    [sym_array] = STATE(77),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_spread_element] = STATE(76),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(80),
    [sym_formal_parameters] = STATE(47),
    [sym_method_definition] = STATE(76),
    [sym_pair] = STATE(76),
    [sym__property_name] = STATE(78),
    [sym_computed_property_name] = STATE(78),
    [aux_sym_program_repeat1] = STATE(2228),
    [aux_sym_export_statement_repeat1] = STATE(80),
    [aux_sym_object_repeat1] = STATE(81),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_STAR] = ACTIONS(105),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_COMMA] = ACTIONS(107),
    [anon_sym_RBRACE] = ACTIONS(4814),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(119),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(121),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(123),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(125),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(125),
    [sym_identifier] = ACTIONS(127),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(129),
    [anon_sym_get] = ACTIONS(131),
    [anon_sym_set] = ACTIONS(131),
  },
  [2070] = {
    [sym_import_clause] = STATE(2230),
    [sym_namespace_import] = STATE(87),
    [sym_named_imports] = STATE(87),
    [anon_sym_STAR] = ACTIONS(133),
    [anon_sym_LBRACE] = ACTIONS(135),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4816),
    [sym_identifier] = ACTIONS(139),
  },
  [2071] = {
    [sym_variable_declarator] = STATE(2231),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [2072] = {
    [sym_variable_declarator] = STATE(2232),
    [sym_object] = STATE(92),
    [sym_array] = STATE(93),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(145),
  },
  [2073] = {
    [sym_parenthesized_expression] = STATE(2233),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [2074] = {
    [sym_parenthesized_expression] = STATE(2234),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [2075] = {
    [anon_sym_LPAREN] = ACTIONS(4818),
    [sym_comment] = ACTIONS(71),
  },
  [2076] = {
    [sym_parenthesized_expression] = STATE(2236),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [2077] = {
    [sym_export_statement] = STATE(2237),
    [sym__declaration] = STATE(2237),
    [sym_import_statement] = STATE(2237),
    [sym_expression_statement] = STATE(2237),
    [sym_variable_declaration] = STATE(2237),
    [sym_lexical_declaration] = STATE(2237),
    [sym_statement_block] = STATE(2237),
    [sym_if_statement] = STATE(2237),
    [sym_switch_statement] = STATE(2237),
    [sym_for_statement] = STATE(2237),
    [sym_for_in_statement] = STATE(2237),
    [sym_for_of_statement] = STATE(2237),
    [sym_while_statement] = STATE(2237),
    [sym_do_statement] = STATE(2237),
    [sym_try_statement] = STATE(2237),
    [sym_with_statement] = STATE(2237),
    [sym_break_statement] = STATE(2237),
    [sym_continue_statement] = STATE(2237),
    [sym_debugger_statement] = STATE(2237),
    [sym_return_statement] = STATE(2237),
    [sym_throw_statement] = STATE(2237),
    [sym_empty_statement] = STATE(2237),
    [sym_labeled_statement] = STATE(2237),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2078] = {
    [sym_statement_block] = STATE(2239),
    [anon_sym_LBRACE] = ACTIONS(4820),
    [sym_comment] = ACTIONS(71),
  },
  [2079] = {
    [sym_parenthesized_expression] = STATE(2240),
    [anon_sym_LPAREN] = ACTIONS(147),
    [sym_comment] = ACTIONS(71),
  },
  [2080] = {
    [sym__automatic_semicolon] = ACTIONS(4822),
    [anon_sym_SEMI] = ACTIONS(4822),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4824),
  },
  [2081] = {
    [sym__automatic_semicolon] = ACTIONS(4826),
    [anon_sym_SEMI] = ACTIONS(4826),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4828),
  },
  [2082] = {
    [sym__automatic_semicolon] = ACTIONS(4830),
    [anon_sym_SEMI] = ACTIONS(4830),
    [sym_comment] = ACTIONS(71),
  },
  [2083] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2247),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2247),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [sym__automatic_semicolon] = ACTIONS(4832),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(4832),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2084] = {
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2248),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2248),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2085] = {
    [anon_sym_export] = ACTIONS(237),
    [anon_sym_default] = ACTIONS(237),
    [anon_sym_LBRACE] = ACTIONS(235),
    [anon_sym_RBRACE] = ACTIONS(235),
    [anon_sym_import] = ACTIONS(237),
    [anon_sym_var] = ACTIONS(237),
    [anon_sym_let] = ACTIONS(237),
    [anon_sym_const] = ACTIONS(237),
    [anon_sym_if] = ACTIONS(237),
    [anon_sym_else] = ACTIONS(237),
    [anon_sym_switch] = ACTIONS(237),
    [anon_sym_for] = ACTIONS(237),
    [anon_sym_LPAREN] = ACTIONS(235),
    [anon_sym_while] = ACTIONS(237),
    [anon_sym_do] = ACTIONS(237),
    [anon_sym_try] = ACTIONS(237),
    [anon_sym_with] = ACTIONS(237),
    [anon_sym_break] = ACTIONS(237),
    [anon_sym_continue] = ACTIONS(237),
    [anon_sym_debugger] = ACTIONS(237),
    [anon_sym_return] = ACTIONS(237),
    [anon_sym_throw] = ACTIONS(237),
    [anon_sym_SEMI] = ACTIONS(235),
    [anon_sym_case] = ACTIONS(237),
    [anon_sym_yield] = ACTIONS(237),
    [anon_sym_LBRACK] = ACTIONS(235),
    [anon_sym_LT] = ACTIONS(235),
    [anon_sym_class] = ACTIONS(237),
    [anon_sym_async] = ACTIONS(237),
    [anon_sym_function] = ACTIONS(237),
    [anon_sym_EQ_GT] = ACTIONS(235),
    [anon_sym_new] = ACTIONS(237),
    [anon_sym_await] = ACTIONS(237),
    [anon_sym_PLUS] = ACTIONS(237),
    [anon_sym_DASH] = ACTIONS(237),
    [anon_sym_BANG] = ACTIONS(235),
    [anon_sym_TILDE] = ACTIONS(235),
    [anon_sym_typeof] = ACTIONS(237),
    [anon_sym_void] = ACTIONS(237),
    [anon_sym_delete] = ACTIONS(237),
    [anon_sym_PLUS_PLUS] = ACTIONS(235),
    [anon_sym_DASH_DASH] = ACTIONS(235),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(235),
    [anon_sym_BQUOTE] = ACTIONS(235),
    [sym_regex] = ACTIONS(237),
    [sym_number] = ACTIONS(235),
    [sym_identifier] = ACTIONS(237),
    [sym_this] = ACTIONS(237),
    [sym_super] = ACTIONS(237),
    [sym_true] = ACTIONS(237),
    [sym_false] = ACTIONS(237),
    [sym_null] = ACTIONS(237),
    [sym_undefined] = ACTIONS(237),
    [anon_sym_AT] = ACTIONS(235),
    [anon_sym_get] = ACTIONS(237),
    [anon_sym_set] = ACTIONS(237),
  },
  [2086] = {
    [sym_class_heritage] = STATE(187),
    [sym_class_body] = STATE(188),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4834),
  },
  [2087] = {
    [sym__initializer] = STATE(196),
    [sym_formal_parameters] = STATE(195),
    [sym__automatic_semicolon] = ACTIONS(291),
    [anon_sym_STAR] = ACTIONS(293),
    [anon_sym_COMMA] = ACTIONS(291),
    [anon_sym_LPAREN] = ACTIONS(295),
    [anon_sym_in] = ACTIONS(293),
    [anon_sym_SEMI] = ACTIONS(291),
    [anon_sym_LBRACK] = ACTIONS(291),
    [anon_sym_LT] = ACTIONS(293),
    [anon_sym_GT] = ACTIONS(293),
    [anon_sym_SLASH] = ACTIONS(293),
    [anon_sym_DOT] = ACTIONS(298),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_function] = ACTIONS(4836),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(306),
    [anon_sym_DASH_EQ] = ACTIONS(306),
    [anon_sym_STAR_EQ] = ACTIONS(306),
    [anon_sym_SLASH_EQ] = ACTIONS(306),
    [anon_sym_PERCENT_EQ] = ACTIONS(306),
    [anon_sym_CARET_EQ] = ACTIONS(306),
    [anon_sym_AMP_EQ] = ACTIONS(306),
    [anon_sym_PIPE_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(306),
    [anon_sym_LT_LT_EQ] = ACTIONS(306),
    [anon_sym_QMARK] = ACTIONS(291),
    [anon_sym_AMP_AMP] = ACTIONS(291),
    [anon_sym_PIPE_PIPE] = ACTIONS(291),
    [anon_sym_GT_GT] = ACTIONS(293),
    [anon_sym_GT_GT_GT] = ACTIONS(293),
    [anon_sym_LT_LT] = ACTIONS(293),
    [anon_sym_AMP] = ACTIONS(293),
    [anon_sym_CARET] = ACTIONS(293),
    [anon_sym_PIPE] = ACTIONS(293),
    [anon_sym_PLUS] = ACTIONS(293),
    [anon_sym_DASH] = ACTIONS(293),
    [anon_sym_PERCENT] = ACTIONS(293),
    [anon_sym_LT_EQ] = ACTIONS(291),
    [anon_sym_EQ_EQ] = ACTIONS(293),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(291),
    [anon_sym_BANG_EQ] = ACTIONS(293),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(291),
    [anon_sym_GT_EQ] = ACTIONS(291),
    [anon_sym_instanceof] = ACTIONS(293),
    [anon_sym_PLUS_PLUS] = ACTIONS(291),
    [anon_sym_DASH_DASH] = ACTIONS(291),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(291),
    [sym_identifier] = ACTIONS(308),
  },
  [2088] = {
    [sym_formal_parameters] = STATE(2253),
    [anon_sym_STAR] = ACTIONS(4838),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4840),
  },
  [2089] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(348),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_COLON] = ACTIONS(4842),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2090] = {
    [anon_sym_export] = ACTIONS(1705),
    [anon_sym_default] = ACTIONS(1705),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_import] = ACTIONS(1705),
    [anon_sym_var] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_else] = ACTIONS(4844),
    [anon_sym_switch] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_do] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [anon_sym_with] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_debugger] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_throw] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_case] = ACTIONS(1705),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_class] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_function] = ACTIONS(1705),
    [anon_sym_EQ_GT] = ACTIONS(1703),
    [anon_sym_new] = ACTIONS(1705),
    [anon_sym_await] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_typeof] = ACTIONS(1705),
    [anon_sym_void] = ACTIONS(1705),
    [anon_sym_delete] = ACTIONS(1705),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1703),
    [anon_sym_BQUOTE] = ACTIONS(1703),
    [sym_regex] = ACTIONS(1705),
    [sym_number] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_this] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_null] = ACTIONS(1705),
    [sym_undefined] = ACTIONS(1705),
    [anon_sym_AT] = ACTIONS(1703),
    [anon_sym_get] = ACTIONS(1705),
    [anon_sym_set] = ACTIONS(1705),
  },
  [2091] = {
    [sym__automatic_semicolon] = ACTIONS(4846),
    [anon_sym_SEMI] = ACTIONS(4846),
    [sym_comment] = ACTIONS(71),
  },
  [2092] = {
    [sym__automatic_semicolon] = ACTIONS(4848),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_default] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_else] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_case] = ACTIONS(402),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [2093] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(4848),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_default] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_else] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(346),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_case] = ACTIONS(402),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(402),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(346),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(346),
    [anon_sym_BANG] = ACTIONS(402),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [2094] = {
    [sym_decorator] = STATE(241),
    [aux_sym_export_statement_repeat1] = STATE(241),
    [anon_sym_export] = ACTIONS(4850),
    [anon_sym_class] = ACTIONS(4852),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [2095] = {
    [sym_switch_case] = STATE(2261),
    [sym_switch_default] = STATE(2261),
    [aux_sym_switch_body_repeat1] = STATE(2261),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(4854),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [2096] = {
    [anon_sym_export] = ACTIONS(1725),
    [anon_sym_default] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1723),
    [anon_sym_RBRACE] = ACTIONS(1723),
    [anon_sym_import] = ACTIONS(1725),
    [anon_sym_var] = ACTIONS(1725),
    [anon_sym_let] = ACTIONS(1725),
    [anon_sym_const] = ACTIONS(1725),
    [anon_sym_if] = ACTIONS(1725),
    [anon_sym_switch] = ACTIONS(1725),
    [anon_sym_for] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_while] = ACTIONS(1725),
    [anon_sym_do] = ACTIONS(1725),
    [anon_sym_try] = ACTIONS(1725),
    [anon_sym_with] = ACTIONS(1725),
    [anon_sym_break] = ACTIONS(1725),
    [anon_sym_continue] = ACTIONS(1725),
    [anon_sym_debugger] = ACTIONS(1725),
    [anon_sym_return] = ACTIONS(1725),
    [anon_sym_throw] = ACTIONS(1725),
    [anon_sym_SEMI] = ACTIONS(1723),
    [anon_sym_case] = ACTIONS(1725),
    [anon_sym_yield] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1723),
    [anon_sym_LT] = ACTIONS(1723),
    [anon_sym_class] = ACTIONS(1725),
    [anon_sym_async] = ACTIONS(1725),
    [anon_sym_function] = ACTIONS(1725),
    [anon_sym_EQ_GT] = ACTIONS(1723),
    [anon_sym_new] = ACTIONS(1725),
    [anon_sym_await] = ACTIONS(1725),
    [anon_sym_PLUS] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1725),
    [anon_sym_BANG] = ACTIONS(1723),
    [anon_sym_TILDE] = ACTIONS(1723),
    [anon_sym_typeof] = ACTIONS(1725),
    [anon_sym_void] = ACTIONS(1725),
    [anon_sym_delete] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1723),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1723),
    [anon_sym_BQUOTE] = ACTIONS(1723),
    [sym_regex] = ACTIONS(1725),
    [sym_number] = ACTIONS(1723),
    [sym_identifier] = ACTIONS(1725),
    [sym_this] = ACTIONS(1725),
    [sym_super] = ACTIONS(1725),
    [sym_true] = ACTIONS(1725),
    [sym_false] = ACTIONS(1725),
    [sym_null] = ACTIONS(1725),
    [sym_undefined] = ACTIONS(1725),
    [anon_sym_AT] = ACTIONS(1723),
    [anon_sym_get] = ACTIONS(1725),
    [anon_sym_set] = ACTIONS(1725),
  },
  [2097] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(2263),
    [sym_array] = STATE(2264),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4856),
  },
  [2098] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(2263),
    [sym_array] = STATE(2264),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4856),
  },
  [2099] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(4858),
    [anon_sym_of] = ACTIONS(4860),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2100] = {
    [sym_expression_statement] = STATE(2267),
    [sym_empty_statement] = STATE(2267),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2101] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(4862),
    [anon_sym_of] = ACTIONS(4864),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2102] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(4866),
    [anon_sym_of] = ACTIONS(4868),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2103] = {
    [anon_sym_export] = ACTIONS(1863),
    [anon_sym_default] = ACTIONS(1863),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_RBRACE] = ACTIONS(1861),
    [anon_sym_import] = ACTIONS(1863),
    [anon_sym_var] = ACTIONS(1863),
    [anon_sym_let] = ACTIONS(1863),
    [anon_sym_const] = ACTIONS(1863),
    [anon_sym_if] = ACTIONS(1863),
    [anon_sym_switch] = ACTIONS(1863),
    [anon_sym_for] = ACTIONS(1863),
    [anon_sym_LPAREN] = ACTIONS(1861),
    [anon_sym_while] = ACTIONS(1863),
    [anon_sym_do] = ACTIONS(1863),
    [anon_sym_try] = ACTIONS(1863),
    [anon_sym_with] = ACTIONS(1863),
    [anon_sym_break] = ACTIONS(1863),
    [anon_sym_continue] = ACTIONS(1863),
    [anon_sym_debugger] = ACTIONS(1863),
    [anon_sym_return] = ACTIONS(1863),
    [anon_sym_throw] = ACTIONS(1863),
    [anon_sym_SEMI] = ACTIONS(1861),
    [anon_sym_case] = ACTIONS(1863),
    [anon_sym_yield] = ACTIONS(1863),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LT] = ACTIONS(1861),
    [anon_sym_class] = ACTIONS(1863),
    [anon_sym_async] = ACTIONS(1863),
    [anon_sym_function] = ACTIONS(1863),
    [anon_sym_EQ_GT] = ACTIONS(1861),
    [anon_sym_new] = ACTIONS(1863),
    [anon_sym_await] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_typeof] = ACTIONS(1863),
    [anon_sym_void] = ACTIONS(1863),
    [anon_sym_delete] = ACTIONS(1863),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1861),
    [anon_sym_BQUOTE] = ACTIONS(1861),
    [sym_regex] = ACTIONS(1863),
    [sym_number] = ACTIONS(1861),
    [sym_identifier] = ACTIONS(1863),
    [sym_this] = ACTIONS(1863),
    [sym_super] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_null] = ACTIONS(1863),
    [sym_undefined] = ACTIONS(1863),
    [anon_sym_AT] = ACTIONS(1861),
    [anon_sym_get] = ACTIONS(1863),
    [anon_sym_set] = ACTIONS(1863),
  },
  [2104] = {
    [sym_parenthesized_expression] = STATE(2272),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [sym_comment] = ACTIONS(71),
  },
  [2105] = {
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_catch] = ACTIONS(484),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2106] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4870),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2107] = {
    [sym_statement_block] = STATE(2276),
    [anon_sym_LBRACE] = ACTIONS(4872),
    [anon_sym_LPAREN] = ACTIONS(4874),
    [sym_comment] = ACTIONS(71),
  },
  [2108] = {
    [sym_statement_block] = STATE(2278),
    [anon_sym_LBRACE] = ACTIONS(4876),
    [sym_comment] = ACTIONS(71),
  },
  [2109] = {
    [sym_finally_clause] = STATE(2279),
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_default] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_case] = ACTIONS(1943),
    [anon_sym_finally] = ACTIONS(4532),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [2110] = {
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_default] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_case] = ACTIONS(1943),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [2111] = {
    [anon_sym_export] = ACTIONS(1947),
    [anon_sym_default] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1945),
    [anon_sym_RBRACE] = ACTIONS(1945),
    [anon_sym_import] = ACTIONS(1947),
    [anon_sym_var] = ACTIONS(1947),
    [anon_sym_let] = ACTIONS(1947),
    [anon_sym_const] = ACTIONS(1947),
    [anon_sym_if] = ACTIONS(1947),
    [anon_sym_switch] = ACTIONS(1947),
    [anon_sym_for] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1945),
    [anon_sym_while] = ACTIONS(1947),
    [anon_sym_do] = ACTIONS(1947),
    [anon_sym_try] = ACTIONS(1947),
    [anon_sym_with] = ACTIONS(1947),
    [anon_sym_break] = ACTIONS(1947),
    [anon_sym_continue] = ACTIONS(1947),
    [anon_sym_debugger] = ACTIONS(1947),
    [anon_sym_return] = ACTIONS(1947),
    [anon_sym_throw] = ACTIONS(1947),
    [anon_sym_SEMI] = ACTIONS(1945),
    [anon_sym_case] = ACTIONS(1947),
    [anon_sym_yield] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1945),
    [anon_sym_LT] = ACTIONS(1945),
    [anon_sym_class] = ACTIONS(1947),
    [anon_sym_async] = ACTIONS(1947),
    [anon_sym_function] = ACTIONS(1947),
    [anon_sym_EQ_GT] = ACTIONS(1945),
    [anon_sym_new] = ACTIONS(1947),
    [anon_sym_await] = ACTIONS(1947),
    [anon_sym_PLUS] = ACTIONS(1947),
    [anon_sym_DASH] = ACTIONS(1947),
    [anon_sym_BANG] = ACTIONS(1945),
    [anon_sym_TILDE] = ACTIONS(1945),
    [anon_sym_typeof] = ACTIONS(1947),
    [anon_sym_void] = ACTIONS(1947),
    [anon_sym_delete] = ACTIONS(1947),
    [anon_sym_PLUS_PLUS] = ACTIONS(1945),
    [anon_sym_DASH_DASH] = ACTIONS(1945),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1945),
    [anon_sym_BQUOTE] = ACTIONS(1945),
    [sym_regex] = ACTIONS(1947),
    [sym_number] = ACTIONS(1945),
    [sym_identifier] = ACTIONS(1947),
    [sym_this] = ACTIONS(1947),
    [sym_super] = ACTIONS(1947),
    [sym_true] = ACTIONS(1947),
    [sym_false] = ACTIONS(1947),
    [sym_null] = ACTIONS(1947),
    [sym_undefined] = ACTIONS(1947),
    [anon_sym_AT] = ACTIONS(1945),
    [anon_sym_get] = ACTIONS(1947),
    [anon_sym_set] = ACTIONS(1947),
  },
  [2112] = {
    [anon_sym_export] = ACTIONS(1951),
    [anon_sym_default] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1949),
    [anon_sym_RBRACE] = ACTIONS(1949),
    [anon_sym_import] = ACTIONS(1951),
    [anon_sym_var] = ACTIONS(1951),
    [anon_sym_let] = ACTIONS(1951),
    [anon_sym_const] = ACTIONS(1951),
    [anon_sym_if] = ACTIONS(1951),
    [anon_sym_switch] = ACTIONS(1951),
    [anon_sym_for] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1949),
    [anon_sym_while] = ACTIONS(1951),
    [anon_sym_do] = ACTIONS(1951),
    [anon_sym_try] = ACTIONS(1951),
    [anon_sym_with] = ACTIONS(1951),
    [anon_sym_break] = ACTIONS(1951),
    [anon_sym_continue] = ACTIONS(1951),
    [anon_sym_debugger] = ACTIONS(1951),
    [anon_sym_return] = ACTIONS(1951),
    [anon_sym_throw] = ACTIONS(1951),
    [anon_sym_SEMI] = ACTIONS(1949),
    [anon_sym_case] = ACTIONS(1951),
    [anon_sym_yield] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1949),
    [anon_sym_LT] = ACTIONS(1949),
    [anon_sym_class] = ACTIONS(1951),
    [anon_sym_async] = ACTIONS(1951),
    [anon_sym_function] = ACTIONS(1951),
    [anon_sym_EQ_GT] = ACTIONS(1949),
    [anon_sym_new] = ACTIONS(1951),
    [anon_sym_await] = ACTIONS(1951),
    [anon_sym_PLUS] = ACTIONS(1951),
    [anon_sym_DASH] = ACTIONS(1951),
    [anon_sym_BANG] = ACTIONS(1949),
    [anon_sym_TILDE] = ACTIONS(1949),
    [anon_sym_typeof] = ACTIONS(1951),
    [anon_sym_void] = ACTIONS(1951),
    [anon_sym_delete] = ACTIONS(1951),
    [anon_sym_PLUS_PLUS] = ACTIONS(1949),
    [anon_sym_DASH_DASH] = ACTIONS(1949),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1949),
    [anon_sym_BQUOTE] = ACTIONS(1949),
    [sym_regex] = ACTIONS(1951),
    [sym_number] = ACTIONS(1949),
    [sym_identifier] = ACTIONS(1951),
    [sym_this] = ACTIONS(1951),
    [sym_super] = ACTIONS(1951),
    [sym_true] = ACTIONS(1951),
    [sym_false] = ACTIONS(1951),
    [sym_null] = ACTIONS(1951),
    [sym_undefined] = ACTIONS(1951),
    [anon_sym_AT] = ACTIONS(1949),
    [anon_sym_get] = ACTIONS(1951),
    [anon_sym_set] = ACTIONS(1951),
  },
  [2113] = {
    [anon_sym_export] = ACTIONS(1955),
    [anon_sym_default] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1953),
    [anon_sym_RBRACE] = ACTIONS(1953),
    [anon_sym_import] = ACTIONS(1955),
    [anon_sym_var] = ACTIONS(1955),
    [anon_sym_let] = ACTIONS(1955),
    [anon_sym_const] = ACTIONS(1955),
    [anon_sym_if] = ACTIONS(1955),
    [anon_sym_switch] = ACTIONS(1955),
    [anon_sym_for] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1953),
    [anon_sym_while] = ACTIONS(1955),
    [anon_sym_do] = ACTIONS(1955),
    [anon_sym_try] = ACTIONS(1955),
    [anon_sym_with] = ACTIONS(1955),
    [anon_sym_break] = ACTIONS(1955),
    [anon_sym_continue] = ACTIONS(1955),
    [anon_sym_debugger] = ACTIONS(1955),
    [anon_sym_return] = ACTIONS(1955),
    [anon_sym_throw] = ACTIONS(1955),
    [anon_sym_SEMI] = ACTIONS(1953),
    [anon_sym_case] = ACTIONS(1955),
    [anon_sym_yield] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1953),
    [anon_sym_LT] = ACTIONS(1953),
    [anon_sym_class] = ACTIONS(1955),
    [anon_sym_async] = ACTIONS(1955),
    [anon_sym_function] = ACTIONS(1955),
    [anon_sym_EQ_GT] = ACTIONS(1953),
    [anon_sym_new] = ACTIONS(1955),
    [anon_sym_await] = ACTIONS(1955),
    [anon_sym_PLUS] = ACTIONS(1955),
    [anon_sym_DASH] = ACTIONS(1955),
    [anon_sym_BANG] = ACTIONS(1953),
    [anon_sym_TILDE] = ACTIONS(1953),
    [anon_sym_typeof] = ACTIONS(1955),
    [anon_sym_void] = ACTIONS(1955),
    [anon_sym_delete] = ACTIONS(1955),
    [anon_sym_PLUS_PLUS] = ACTIONS(1953),
    [anon_sym_DASH_DASH] = ACTIONS(1953),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1953),
    [anon_sym_BQUOTE] = ACTIONS(1953),
    [sym_regex] = ACTIONS(1955),
    [sym_number] = ACTIONS(1953),
    [sym_identifier] = ACTIONS(1955),
    [sym_this] = ACTIONS(1955),
    [sym_super] = ACTIONS(1955),
    [sym_true] = ACTIONS(1955),
    [sym_false] = ACTIONS(1955),
    [sym_null] = ACTIONS(1955),
    [sym_undefined] = ACTIONS(1955),
    [anon_sym_AT] = ACTIONS(1953),
    [anon_sym_get] = ACTIONS(1955),
    [anon_sym_set] = ACTIONS(1955),
  },
  [2114] = {
    [anon_sym_export] = ACTIONS(1959),
    [anon_sym_default] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1957),
    [anon_sym_RBRACE] = ACTIONS(1957),
    [anon_sym_import] = ACTIONS(1959),
    [anon_sym_var] = ACTIONS(1959),
    [anon_sym_let] = ACTIONS(1959),
    [anon_sym_const] = ACTIONS(1959),
    [anon_sym_if] = ACTIONS(1959),
    [anon_sym_switch] = ACTIONS(1959),
    [anon_sym_for] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1957),
    [anon_sym_while] = ACTIONS(1959),
    [anon_sym_do] = ACTIONS(1959),
    [anon_sym_try] = ACTIONS(1959),
    [anon_sym_with] = ACTIONS(1959),
    [anon_sym_break] = ACTIONS(1959),
    [anon_sym_continue] = ACTIONS(1959),
    [anon_sym_debugger] = ACTIONS(1959),
    [anon_sym_return] = ACTIONS(1959),
    [anon_sym_throw] = ACTIONS(1959),
    [anon_sym_SEMI] = ACTIONS(1957),
    [anon_sym_case] = ACTIONS(1959),
    [anon_sym_yield] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1957),
    [anon_sym_LT] = ACTIONS(1957),
    [anon_sym_class] = ACTIONS(1959),
    [anon_sym_async] = ACTIONS(1959),
    [anon_sym_function] = ACTIONS(1959),
    [anon_sym_EQ_GT] = ACTIONS(1957),
    [anon_sym_new] = ACTIONS(1959),
    [anon_sym_await] = ACTIONS(1959),
    [anon_sym_PLUS] = ACTIONS(1959),
    [anon_sym_DASH] = ACTIONS(1959),
    [anon_sym_BANG] = ACTIONS(1957),
    [anon_sym_TILDE] = ACTIONS(1957),
    [anon_sym_typeof] = ACTIONS(1959),
    [anon_sym_void] = ACTIONS(1959),
    [anon_sym_delete] = ACTIONS(1959),
    [anon_sym_PLUS_PLUS] = ACTIONS(1957),
    [anon_sym_DASH_DASH] = ACTIONS(1957),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1957),
    [anon_sym_BQUOTE] = ACTIONS(1957),
    [sym_regex] = ACTIONS(1959),
    [sym_number] = ACTIONS(1957),
    [sym_identifier] = ACTIONS(1959),
    [sym_this] = ACTIONS(1959),
    [sym_super] = ACTIONS(1959),
    [sym_true] = ACTIONS(1959),
    [sym_false] = ACTIONS(1959),
    [sym_null] = ACTIONS(1959),
    [sym_undefined] = ACTIONS(1959),
    [anon_sym_AT] = ACTIONS(1957),
    [anon_sym_get] = ACTIONS(1959),
    [anon_sym_set] = ACTIONS(1959),
  },
  [2115] = {
    [anon_sym_export] = ACTIONS(1963),
    [anon_sym_default] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1961),
    [anon_sym_RBRACE] = ACTIONS(1961),
    [anon_sym_import] = ACTIONS(1963),
    [anon_sym_var] = ACTIONS(1963),
    [anon_sym_let] = ACTIONS(1963),
    [anon_sym_const] = ACTIONS(1963),
    [anon_sym_if] = ACTIONS(1963),
    [anon_sym_switch] = ACTIONS(1963),
    [anon_sym_for] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1961),
    [anon_sym_while] = ACTIONS(1963),
    [anon_sym_do] = ACTIONS(1963),
    [anon_sym_try] = ACTIONS(1963),
    [anon_sym_with] = ACTIONS(1963),
    [anon_sym_break] = ACTIONS(1963),
    [anon_sym_continue] = ACTIONS(1963),
    [anon_sym_debugger] = ACTIONS(1963),
    [anon_sym_return] = ACTIONS(1963),
    [anon_sym_throw] = ACTIONS(1963),
    [anon_sym_SEMI] = ACTIONS(1961),
    [anon_sym_case] = ACTIONS(1963),
    [anon_sym_yield] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1961),
    [anon_sym_LT] = ACTIONS(1961),
    [anon_sym_class] = ACTIONS(1963),
    [anon_sym_async] = ACTIONS(1963),
    [anon_sym_function] = ACTIONS(1963),
    [anon_sym_EQ_GT] = ACTIONS(1961),
    [anon_sym_new] = ACTIONS(1963),
    [anon_sym_await] = ACTIONS(1963),
    [anon_sym_PLUS] = ACTIONS(1963),
    [anon_sym_DASH] = ACTIONS(1963),
    [anon_sym_BANG] = ACTIONS(1961),
    [anon_sym_TILDE] = ACTIONS(1961),
    [anon_sym_typeof] = ACTIONS(1963),
    [anon_sym_void] = ACTIONS(1963),
    [anon_sym_delete] = ACTIONS(1963),
    [anon_sym_PLUS_PLUS] = ACTIONS(1961),
    [anon_sym_DASH_DASH] = ACTIONS(1961),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1961),
    [anon_sym_BQUOTE] = ACTIONS(1961),
    [sym_regex] = ACTIONS(1963),
    [sym_number] = ACTIONS(1961),
    [sym_identifier] = ACTIONS(1963),
    [sym_this] = ACTIONS(1963),
    [sym_super] = ACTIONS(1963),
    [sym_true] = ACTIONS(1963),
    [sym_false] = ACTIONS(1963),
    [sym_null] = ACTIONS(1963),
    [sym_undefined] = ACTIONS(1963),
    [anon_sym_AT] = ACTIONS(1961),
    [anon_sym_get] = ACTIONS(1963),
    [anon_sym_set] = ACTIONS(1963),
  },
  [2116] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(2281),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(4878),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2117] = {
    [sym_class_body] = STATE(2282),
    [anon_sym_LBRACE] = ACTIONS(4542),
    [sym_comment] = ACTIONS(71),
  },
  [2118] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_STAR] = ACTIONS(2140),
    [anon_sym_default] = ACTIONS(2142),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_COMMA] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_in] = ACTIONS(2142),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_case] = ACTIONS(2142),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2142),
    [anon_sym_GT] = ACTIONS(2142),
    [anon_sym_SLASH] = ACTIONS(2142),
    [anon_sym_DOT] = ACTIONS(2142),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_QMARK] = ACTIONS(2140),
    [anon_sym_AMP_AMP] = ACTIONS(2140),
    [anon_sym_PIPE_PIPE] = ACTIONS(2140),
    [anon_sym_GT_GT] = ACTIONS(2142),
    [anon_sym_GT_GT_GT] = ACTIONS(2140),
    [anon_sym_LT_LT] = ACTIONS(2140),
    [anon_sym_AMP] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(2140),
    [anon_sym_PIPE] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_PERCENT] = ACTIONS(2140),
    [anon_sym_LT_EQ] = ACTIONS(2140),
    [anon_sym_EQ_EQ] = ACTIONS(2142),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2140),
    [anon_sym_BANG_EQ] = ACTIONS(2142),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2140),
    [anon_sym_GT_EQ] = ACTIONS(2140),
    [anon_sym_instanceof] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2142),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [2119] = {
    [sym_formal_parameters] = STATE(2283),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2120] = {
    [sym_statement_block] = STATE(2284),
    [anon_sym_LBRACE] = ACTIONS(4548),
    [sym_comment] = ACTIONS(71),
  },
  [2121] = {
    [sym_formal_parameters] = STATE(2285),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2122] = {
    [sym_statement_block] = STATE(2286),
    [anon_sym_LBRACE] = ACTIONS(4548),
    [sym_comment] = ACTIONS(71),
  },
  [2123] = {
    [sym_export_statement] = STATE(2288),
    [sym__declaration] = STATE(2288),
    [sym_import_statement] = STATE(2288),
    [sym_expression_statement] = STATE(2288),
    [sym_variable_declaration] = STATE(2288),
    [sym_lexical_declaration] = STATE(2288),
    [sym_statement_block] = STATE(2288),
    [sym_if_statement] = STATE(2288),
    [sym_switch_statement] = STATE(2288),
    [sym_for_statement] = STATE(2288),
    [sym_for_in_statement] = STATE(2288),
    [sym_for_of_statement] = STATE(2288),
    [sym_while_statement] = STATE(2288),
    [sym_do_statement] = STATE(2288),
    [sym_try_statement] = STATE(2288),
    [sym_with_statement] = STATE(2288),
    [sym_break_statement] = STATE(2288),
    [sym_continue_statement] = STATE(2288),
    [sym_debugger_statement] = STATE(2288),
    [sym_return_statement] = STATE(2288),
    [sym_throw_statement] = STATE(2288),
    [sym_empty_statement] = STATE(2288),
    [sym_labeled_statement] = STATE(2288),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2288),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4880),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2124] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_default] = ACTIONS(2180),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_COMMA] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_in] = ACTIONS(2180),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_case] = ACTIONS(2180),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2180),
    [anon_sym_GT] = ACTIONS(2180),
    [anon_sym_SLASH] = ACTIONS(2180),
    [anon_sym_DOT] = ACTIONS(2180),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_QMARK] = ACTIONS(2178),
    [anon_sym_AMP_AMP] = ACTIONS(2178),
    [anon_sym_PIPE_PIPE] = ACTIONS(2178),
    [anon_sym_GT_GT] = ACTIONS(2180),
    [anon_sym_GT_GT_GT] = ACTIONS(2178),
    [anon_sym_LT_LT] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2180),
    [anon_sym_CARET] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_PERCENT] = ACTIONS(2178),
    [anon_sym_LT_EQ] = ACTIONS(2178),
    [anon_sym_EQ_EQ] = ACTIONS(2180),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2178),
    [anon_sym_BANG_EQ] = ACTIONS(2180),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2178),
    [anon_sym_GT_EQ] = ACTIONS(2178),
    [anon_sym_instanceof] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2180),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [2125] = {
    [anon_sym_export] = ACTIONS(2206),
    [anon_sym_default] = ACTIONS(2206),
    [anon_sym_LBRACE] = ACTIONS(2204),
    [anon_sym_RBRACE] = ACTIONS(2204),
    [anon_sym_import] = ACTIONS(2206),
    [anon_sym_var] = ACTIONS(2206),
    [anon_sym_let] = ACTIONS(2206),
    [anon_sym_const] = ACTIONS(2206),
    [anon_sym_if] = ACTIONS(2206),
    [anon_sym_switch] = ACTIONS(2206),
    [anon_sym_for] = ACTIONS(2206),
    [anon_sym_LPAREN] = ACTIONS(2204),
    [anon_sym_while] = ACTIONS(2206),
    [anon_sym_do] = ACTIONS(2206),
    [anon_sym_try] = ACTIONS(2206),
    [anon_sym_with] = ACTIONS(2206),
    [anon_sym_break] = ACTIONS(2206),
    [anon_sym_continue] = ACTIONS(2206),
    [anon_sym_debugger] = ACTIONS(2206),
    [anon_sym_return] = ACTIONS(2206),
    [anon_sym_throw] = ACTIONS(2206),
    [anon_sym_SEMI] = ACTIONS(2204),
    [anon_sym_case] = ACTIONS(2206),
    [anon_sym_yield] = ACTIONS(2206),
    [anon_sym_LBRACK] = ACTIONS(2204),
    [anon_sym_LT] = ACTIONS(2204),
    [anon_sym_class] = ACTIONS(2206),
    [anon_sym_async] = ACTIONS(2206),
    [anon_sym_function] = ACTIONS(2206),
    [anon_sym_EQ_GT] = ACTIONS(2204),
    [anon_sym_new] = ACTIONS(2206),
    [anon_sym_await] = ACTIONS(2206),
    [anon_sym_PLUS] = ACTIONS(2206),
    [anon_sym_DASH] = ACTIONS(2206),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_typeof] = ACTIONS(2206),
    [anon_sym_void] = ACTIONS(2206),
    [anon_sym_delete] = ACTIONS(2206),
    [anon_sym_PLUS_PLUS] = ACTIONS(2204),
    [anon_sym_DASH_DASH] = ACTIONS(2204),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2204),
    [anon_sym_BQUOTE] = ACTIONS(2204),
    [sym_regex] = ACTIONS(2206),
    [sym_number] = ACTIONS(2204),
    [sym_identifier] = ACTIONS(2206),
    [sym_this] = ACTIONS(2206),
    [sym_super] = ACTIONS(2206),
    [sym_true] = ACTIONS(2206),
    [sym_false] = ACTIONS(2206),
    [sym_null] = ACTIONS(2206),
    [sym_undefined] = ACTIONS(2206),
    [anon_sym_AT] = ACTIONS(2204),
    [anon_sym_get] = ACTIONS(2206),
    [anon_sym_set] = ACTIONS(2206),
  },
  [2126] = {
    [sym_parenthesized_expression] = STATE(2289),
    [sym__expression] = STATE(2289),
    [sym_yield_expression] = STATE(2289),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(2289),
    [sym_jsx_fragment] = STATE(2289),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2289),
    [sym_anonymous_class] = STATE(2289),
    [sym_class] = STATE(2289),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2289),
    [sym_generator_function] = STATE(2289),
    [sym_call_expression] = STATE(2289),
    [sym_new_expression] = STATE(2289),
    [sym_await_expression] = STATE(2289),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(2289),
    [sym_augmented_assignment_expression] = STATE(2289),
    [sym_ternary_expression] = STATE(2289),
    [sym_binary_expression] = STATE(2289),
    [sym_unary_expression] = STATE(2289),
    [sym_update_expression] = STATE(2289),
    [sym_template_string] = STATE(2289),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4882),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4884),
    [sym_number] = ACTIONS(4882),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(4884),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4884),
    [sym_false] = ACTIONS(4884),
    [sym_null] = ACTIONS(4884),
    [sym_undefined] = ACTIONS(4884),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [2127] = {
    [sym_class_heritage] = STATE(2290),
    [sym_class_body] = STATE(2282),
    [anon_sym_LBRACE] = ACTIONS(4542),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2128] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2129] = {
    [sym_parenthesized_expression] = STATE(2291),
    [sym__expression] = STATE(2291),
    [sym_yield_expression] = STATE(2291),
    [sym_object] = STATE(1271),
    [sym_array] = STATE(1272),
    [sym_jsx_element] = STATE(2291),
    [sym_jsx_fragment] = STATE(2291),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2291),
    [sym_anonymous_class] = STATE(2291),
    [sym_class] = STATE(2291),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2291),
    [sym_generator_function] = STATE(2291),
    [sym_call_expression] = STATE(2291),
    [sym_new_expression] = STATE(2291),
    [sym_await_expression] = STATE(2291),
    [sym_member_expression] = STATE(1273),
    [sym_subscript_expression] = STATE(1273),
    [sym_assignment_expression] = STATE(2291),
    [sym_augmented_assignment_expression] = STATE(2291),
    [sym_ternary_expression] = STATE(2291),
    [sym_binary_expression] = STATE(2291),
    [sym_unary_expression] = STATE(2291),
    [sym_update_expression] = STATE(2291),
    [sym_template_string] = STATE(2291),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1274),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2696),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2698),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2700),
    [anon_sym_new] = ACTIONS(2702),
    [anon_sym_await] = ACTIONS(2704),
    [anon_sym_PLUS] = ACTIONS(2706),
    [anon_sym_DASH] = ACTIONS(2706),
    [anon_sym_BANG] = ACTIONS(2708),
    [anon_sym_TILDE] = ACTIONS(2708),
    [anon_sym_typeof] = ACTIONS(2706),
    [anon_sym_void] = ACTIONS(2706),
    [anon_sym_delete] = ACTIONS(2706),
    [anon_sym_PLUS_PLUS] = ACTIONS(2710),
    [anon_sym_DASH_DASH] = ACTIONS(2710),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4886),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4888),
    [sym_number] = ACTIONS(4886),
    [sym_identifier] = ACTIONS(2716),
    [sym_this] = ACTIONS(4888),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4888),
    [sym_false] = ACTIONS(4888),
    [sym_null] = ACTIONS(4888),
    [sym_undefined] = ACTIONS(4888),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2718),
    [anon_sym_set] = ACTIONS(2718),
  },
  [2130] = {
    [ts_builtin_sym_end] = ACTIONS(4890),
    [anon_sym_export] = ACTIONS(4892),
    [anon_sym_LBRACE] = ACTIONS(4890),
    [anon_sym_RBRACE] = ACTIONS(4890),
    [anon_sym_import] = ACTIONS(4892),
    [anon_sym_var] = ACTIONS(4892),
    [anon_sym_let] = ACTIONS(4892),
    [anon_sym_const] = ACTIONS(4892),
    [anon_sym_if] = ACTIONS(4892),
    [anon_sym_switch] = ACTIONS(4892),
    [anon_sym_for] = ACTIONS(4892),
    [anon_sym_LPAREN] = ACTIONS(4890),
    [anon_sym_while] = ACTIONS(4892),
    [anon_sym_do] = ACTIONS(4892),
    [anon_sym_try] = ACTIONS(4892),
    [anon_sym_with] = ACTIONS(4892),
    [anon_sym_break] = ACTIONS(4892),
    [anon_sym_continue] = ACTIONS(4892),
    [anon_sym_debugger] = ACTIONS(4892),
    [anon_sym_return] = ACTIONS(4892),
    [anon_sym_throw] = ACTIONS(4892),
    [anon_sym_SEMI] = ACTIONS(4890),
    [anon_sym_yield] = ACTIONS(4892),
    [anon_sym_LBRACK] = ACTIONS(4890),
    [anon_sym_LT] = ACTIONS(4890),
    [anon_sym_class] = ACTIONS(4892),
    [anon_sym_async] = ACTIONS(4892),
    [anon_sym_function] = ACTIONS(4892),
    [anon_sym_EQ_GT] = ACTIONS(4890),
    [anon_sym_new] = ACTIONS(4892),
    [anon_sym_await] = ACTIONS(4892),
    [anon_sym_PLUS] = ACTIONS(4892),
    [anon_sym_DASH] = ACTIONS(4892),
    [anon_sym_BANG] = ACTIONS(4890),
    [anon_sym_TILDE] = ACTIONS(4890),
    [anon_sym_typeof] = ACTIONS(4892),
    [anon_sym_void] = ACTIONS(4892),
    [anon_sym_delete] = ACTIONS(4892),
    [anon_sym_PLUS_PLUS] = ACTIONS(4890),
    [anon_sym_DASH_DASH] = ACTIONS(4890),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4890),
    [anon_sym_BQUOTE] = ACTIONS(4890),
    [sym_regex] = ACTIONS(4892),
    [sym_number] = ACTIONS(4890),
    [sym_identifier] = ACTIONS(4892),
    [sym_this] = ACTIONS(4892),
    [sym_super] = ACTIONS(4892),
    [sym_true] = ACTIONS(4892),
    [sym_false] = ACTIONS(4892),
    [sym_null] = ACTIONS(4892),
    [sym_undefined] = ACTIONS(4892),
    [anon_sym_AT] = ACTIONS(4890),
    [anon_sym_get] = ACTIONS(4892),
    [anon_sym_set] = ACTIONS(4892),
  },
  [2131] = {
    [ts_builtin_sym_end] = ACTIONS(4894),
    [anon_sym_export] = ACTIONS(4896),
    [anon_sym_LBRACE] = ACTIONS(4894),
    [anon_sym_RBRACE] = ACTIONS(4894),
    [anon_sym_import] = ACTIONS(4896),
    [anon_sym_var] = ACTIONS(4896),
    [anon_sym_let] = ACTIONS(4896),
    [anon_sym_const] = ACTIONS(4896),
    [anon_sym_if] = ACTIONS(4896),
    [anon_sym_switch] = ACTIONS(4896),
    [anon_sym_for] = ACTIONS(4896),
    [anon_sym_LPAREN] = ACTIONS(4894),
    [anon_sym_while] = ACTIONS(4896),
    [anon_sym_do] = ACTIONS(4896),
    [anon_sym_try] = ACTIONS(4896),
    [anon_sym_with] = ACTIONS(4896),
    [anon_sym_break] = ACTIONS(4896),
    [anon_sym_continue] = ACTIONS(4896),
    [anon_sym_debugger] = ACTIONS(4896),
    [anon_sym_return] = ACTIONS(4896),
    [anon_sym_throw] = ACTIONS(4896),
    [anon_sym_SEMI] = ACTIONS(4894),
    [anon_sym_yield] = ACTIONS(4896),
    [anon_sym_LBRACK] = ACTIONS(4894),
    [anon_sym_LT] = ACTIONS(4894),
    [anon_sym_class] = ACTIONS(4896),
    [anon_sym_async] = ACTIONS(4896),
    [anon_sym_function] = ACTIONS(4896),
    [anon_sym_EQ_GT] = ACTIONS(4894),
    [anon_sym_new] = ACTIONS(4896),
    [anon_sym_await] = ACTIONS(4896),
    [anon_sym_PLUS] = ACTIONS(4896),
    [anon_sym_DASH] = ACTIONS(4896),
    [anon_sym_BANG] = ACTIONS(4894),
    [anon_sym_TILDE] = ACTIONS(4894),
    [anon_sym_typeof] = ACTIONS(4896),
    [anon_sym_void] = ACTIONS(4896),
    [anon_sym_delete] = ACTIONS(4896),
    [anon_sym_PLUS_PLUS] = ACTIONS(4894),
    [anon_sym_DASH_DASH] = ACTIONS(4894),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4894),
    [anon_sym_BQUOTE] = ACTIONS(4894),
    [sym_regex] = ACTIONS(4896),
    [sym_number] = ACTIONS(4894),
    [sym_identifier] = ACTIONS(4896),
    [sym_this] = ACTIONS(4896),
    [sym_super] = ACTIONS(4896),
    [sym_true] = ACTIONS(4896),
    [sym_false] = ACTIONS(4896),
    [sym_null] = ACTIONS(4896),
    [sym_undefined] = ACTIONS(4896),
    [anon_sym_AT] = ACTIONS(4894),
    [anon_sym_get] = ACTIONS(4896),
    [anon_sym_set] = ACTIONS(4896),
  },
  [2132] = {
    [ts_builtin_sym_end] = ACTIONS(4898),
    [anon_sym_export] = ACTIONS(4900),
    [anon_sym_LBRACE] = ACTIONS(4898),
    [anon_sym_RBRACE] = ACTIONS(4898),
    [anon_sym_import] = ACTIONS(4900),
    [anon_sym_var] = ACTIONS(4900),
    [anon_sym_let] = ACTIONS(4900),
    [anon_sym_const] = ACTIONS(4900),
    [anon_sym_if] = ACTIONS(4900),
    [anon_sym_switch] = ACTIONS(4900),
    [anon_sym_for] = ACTIONS(4900),
    [anon_sym_LPAREN] = ACTIONS(4898),
    [anon_sym_while] = ACTIONS(4900),
    [anon_sym_do] = ACTIONS(4900),
    [anon_sym_try] = ACTIONS(4900),
    [anon_sym_with] = ACTIONS(4900),
    [anon_sym_break] = ACTIONS(4900),
    [anon_sym_continue] = ACTIONS(4900),
    [anon_sym_debugger] = ACTIONS(4900),
    [anon_sym_return] = ACTIONS(4900),
    [anon_sym_throw] = ACTIONS(4900),
    [anon_sym_SEMI] = ACTIONS(4898),
    [anon_sym_yield] = ACTIONS(4900),
    [anon_sym_LBRACK] = ACTIONS(4898),
    [anon_sym_LT] = ACTIONS(4898),
    [anon_sym_class] = ACTIONS(4900),
    [anon_sym_async] = ACTIONS(4900),
    [anon_sym_function] = ACTIONS(4900),
    [anon_sym_EQ_GT] = ACTIONS(4898),
    [anon_sym_new] = ACTIONS(4900),
    [anon_sym_await] = ACTIONS(4900),
    [anon_sym_PLUS] = ACTIONS(4900),
    [anon_sym_DASH] = ACTIONS(4900),
    [anon_sym_BANG] = ACTIONS(4898),
    [anon_sym_TILDE] = ACTIONS(4898),
    [anon_sym_typeof] = ACTIONS(4900),
    [anon_sym_void] = ACTIONS(4900),
    [anon_sym_delete] = ACTIONS(4900),
    [anon_sym_PLUS_PLUS] = ACTIONS(4898),
    [anon_sym_DASH_DASH] = ACTIONS(4898),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4898),
    [anon_sym_BQUOTE] = ACTIONS(4898),
    [sym_regex] = ACTIONS(4900),
    [sym_number] = ACTIONS(4898),
    [sym_identifier] = ACTIONS(4900),
    [sym_this] = ACTIONS(4900),
    [sym_super] = ACTIONS(4900),
    [sym_true] = ACTIONS(4900),
    [sym_false] = ACTIONS(4900),
    [sym_null] = ACTIONS(4900),
    [sym_undefined] = ACTIONS(4900),
    [anon_sym_AT] = ACTIONS(4898),
    [anon_sym_get] = ACTIONS(4900),
    [anon_sym_set] = ACTIONS(4900),
  },
  [2133] = {
    [ts_builtin_sym_end] = ACTIONS(4902),
    [anon_sym_export] = ACTIONS(4904),
    [anon_sym_LBRACE] = ACTIONS(4902),
    [anon_sym_RBRACE] = ACTIONS(4902),
    [anon_sym_import] = ACTIONS(4904),
    [anon_sym_var] = ACTIONS(4904),
    [anon_sym_let] = ACTIONS(4904),
    [anon_sym_const] = ACTIONS(4904),
    [anon_sym_if] = ACTIONS(4904),
    [anon_sym_switch] = ACTIONS(4904),
    [anon_sym_for] = ACTIONS(4904),
    [anon_sym_LPAREN] = ACTIONS(4902),
    [anon_sym_while] = ACTIONS(4904),
    [anon_sym_do] = ACTIONS(4904),
    [anon_sym_try] = ACTIONS(4904),
    [anon_sym_with] = ACTIONS(4904),
    [anon_sym_break] = ACTIONS(4904),
    [anon_sym_continue] = ACTIONS(4904),
    [anon_sym_debugger] = ACTIONS(4904),
    [anon_sym_return] = ACTIONS(4904),
    [anon_sym_throw] = ACTIONS(4904),
    [anon_sym_SEMI] = ACTIONS(4902),
    [anon_sym_yield] = ACTIONS(4904),
    [anon_sym_LBRACK] = ACTIONS(4902),
    [anon_sym_LT] = ACTIONS(4902),
    [anon_sym_class] = ACTIONS(4904),
    [anon_sym_async] = ACTIONS(4904),
    [anon_sym_function] = ACTIONS(4904),
    [anon_sym_EQ_GT] = ACTIONS(4902),
    [anon_sym_new] = ACTIONS(4904),
    [anon_sym_await] = ACTIONS(4904),
    [anon_sym_PLUS] = ACTIONS(4904),
    [anon_sym_DASH] = ACTIONS(4904),
    [anon_sym_BANG] = ACTIONS(4902),
    [anon_sym_TILDE] = ACTIONS(4902),
    [anon_sym_typeof] = ACTIONS(4904),
    [anon_sym_void] = ACTIONS(4904),
    [anon_sym_delete] = ACTIONS(4904),
    [anon_sym_PLUS_PLUS] = ACTIONS(4902),
    [anon_sym_DASH_DASH] = ACTIONS(4902),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4902),
    [anon_sym_BQUOTE] = ACTIONS(4902),
    [sym_regex] = ACTIONS(4904),
    [sym_number] = ACTIONS(4902),
    [sym_identifier] = ACTIONS(4904),
    [sym_this] = ACTIONS(4904),
    [sym_super] = ACTIONS(4904),
    [sym_true] = ACTIONS(4904),
    [sym_false] = ACTIONS(4904),
    [sym_null] = ACTIONS(4904),
    [sym_undefined] = ACTIONS(4904),
    [anon_sym_AT] = ACTIONS(4902),
    [anon_sym_get] = ACTIONS(4904),
    [anon_sym_set] = ACTIONS(4904),
  },
  [2134] = {
    [ts_builtin_sym_end] = ACTIONS(4906),
    [anon_sym_export] = ACTIONS(4908),
    [anon_sym_LBRACE] = ACTIONS(4906),
    [anon_sym_RBRACE] = ACTIONS(4906),
    [anon_sym_import] = ACTIONS(4908),
    [anon_sym_var] = ACTIONS(4908),
    [anon_sym_let] = ACTIONS(4908),
    [anon_sym_const] = ACTIONS(4908),
    [anon_sym_if] = ACTIONS(4908),
    [anon_sym_switch] = ACTIONS(4908),
    [anon_sym_for] = ACTIONS(4908),
    [anon_sym_LPAREN] = ACTIONS(4906),
    [anon_sym_while] = ACTIONS(4908),
    [anon_sym_do] = ACTIONS(4908),
    [anon_sym_try] = ACTIONS(4908),
    [anon_sym_with] = ACTIONS(4908),
    [anon_sym_break] = ACTIONS(4908),
    [anon_sym_continue] = ACTIONS(4908),
    [anon_sym_debugger] = ACTIONS(4908),
    [anon_sym_return] = ACTIONS(4908),
    [anon_sym_throw] = ACTIONS(4908),
    [anon_sym_SEMI] = ACTIONS(4906),
    [anon_sym_yield] = ACTIONS(4908),
    [anon_sym_LBRACK] = ACTIONS(4906),
    [anon_sym_LT] = ACTIONS(4906),
    [anon_sym_class] = ACTIONS(4908),
    [anon_sym_async] = ACTIONS(4908),
    [anon_sym_function] = ACTIONS(4908),
    [anon_sym_EQ_GT] = ACTIONS(4906),
    [anon_sym_new] = ACTIONS(4908),
    [anon_sym_await] = ACTIONS(4908),
    [anon_sym_PLUS] = ACTIONS(4908),
    [anon_sym_DASH] = ACTIONS(4908),
    [anon_sym_BANG] = ACTIONS(4906),
    [anon_sym_TILDE] = ACTIONS(4906),
    [anon_sym_typeof] = ACTIONS(4908),
    [anon_sym_void] = ACTIONS(4908),
    [anon_sym_delete] = ACTIONS(4908),
    [anon_sym_PLUS_PLUS] = ACTIONS(4906),
    [anon_sym_DASH_DASH] = ACTIONS(4906),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4906),
    [anon_sym_BQUOTE] = ACTIONS(4906),
    [sym_regex] = ACTIONS(4908),
    [sym_number] = ACTIONS(4906),
    [sym_identifier] = ACTIONS(4908),
    [sym_this] = ACTIONS(4908),
    [sym_super] = ACTIONS(4908),
    [sym_true] = ACTIONS(4908),
    [sym_false] = ACTIONS(4908),
    [sym_null] = ACTIONS(4908),
    [sym_undefined] = ACTIONS(4908),
    [anon_sym_AT] = ACTIONS(4906),
    [anon_sym_get] = ACTIONS(4908),
    [anon_sym_set] = ACTIONS(4908),
  },
  [2135] = {
    [ts_builtin_sym_end] = ACTIONS(4910),
    [anon_sym_export] = ACTIONS(4912),
    [anon_sym_LBRACE] = ACTIONS(4910),
    [anon_sym_RBRACE] = ACTIONS(4910),
    [anon_sym_import] = ACTIONS(4912),
    [anon_sym_var] = ACTIONS(4912),
    [anon_sym_let] = ACTIONS(4912),
    [anon_sym_const] = ACTIONS(4912),
    [anon_sym_if] = ACTIONS(4912),
    [anon_sym_switch] = ACTIONS(4912),
    [anon_sym_for] = ACTIONS(4912),
    [anon_sym_LPAREN] = ACTIONS(4910),
    [anon_sym_while] = ACTIONS(4912),
    [anon_sym_do] = ACTIONS(4912),
    [anon_sym_try] = ACTIONS(4912),
    [anon_sym_with] = ACTIONS(4912),
    [anon_sym_break] = ACTIONS(4912),
    [anon_sym_continue] = ACTIONS(4912),
    [anon_sym_debugger] = ACTIONS(4912),
    [anon_sym_return] = ACTIONS(4912),
    [anon_sym_throw] = ACTIONS(4912),
    [anon_sym_SEMI] = ACTIONS(4910),
    [anon_sym_yield] = ACTIONS(4912),
    [anon_sym_LBRACK] = ACTIONS(4910),
    [anon_sym_LT] = ACTIONS(4910),
    [anon_sym_class] = ACTIONS(4912),
    [anon_sym_async] = ACTIONS(4912),
    [anon_sym_function] = ACTIONS(4912),
    [anon_sym_EQ_GT] = ACTIONS(4910),
    [anon_sym_new] = ACTIONS(4912),
    [anon_sym_await] = ACTIONS(4912),
    [anon_sym_PLUS] = ACTIONS(4912),
    [anon_sym_DASH] = ACTIONS(4912),
    [anon_sym_BANG] = ACTIONS(4910),
    [anon_sym_TILDE] = ACTIONS(4910),
    [anon_sym_typeof] = ACTIONS(4912),
    [anon_sym_void] = ACTIONS(4912),
    [anon_sym_delete] = ACTIONS(4912),
    [anon_sym_PLUS_PLUS] = ACTIONS(4910),
    [anon_sym_DASH_DASH] = ACTIONS(4910),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4910),
    [anon_sym_BQUOTE] = ACTIONS(4910),
    [sym_regex] = ACTIONS(4912),
    [sym_number] = ACTIONS(4910),
    [sym_identifier] = ACTIONS(4912),
    [sym_this] = ACTIONS(4912),
    [sym_super] = ACTIONS(4912),
    [sym_true] = ACTIONS(4912),
    [sym_false] = ACTIONS(4912),
    [sym_null] = ACTIONS(4912),
    [sym_undefined] = ACTIONS(4912),
    [anon_sym_AT] = ACTIONS(4910),
    [anon_sym_get] = ACTIONS(4912),
    [anon_sym_set] = ACTIONS(4912),
  },
  [2136] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3111),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(3111),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(3111),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(3111),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2137] = {
    [sym_parenthesized_expression] = STATE(2292),
    [sym__expression] = STATE(2292),
    [sym_yield_expression] = STATE(2292),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(2292),
    [sym_jsx_fragment] = STATE(2292),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2292),
    [sym_anonymous_class] = STATE(2292),
    [sym_class] = STATE(2292),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2292),
    [sym_generator_function] = STATE(2292),
    [sym_call_expression] = STATE(2292),
    [sym_new_expression] = STATE(2292),
    [sym_await_expression] = STATE(2292),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(2292),
    [sym_augmented_assignment_expression] = STATE(2292),
    [sym_ternary_expression] = STATE(2292),
    [sym_binary_expression] = STATE(2292),
    [sym_unary_expression] = STATE(2292),
    [sym_update_expression] = STATE(2292),
    [sym_template_string] = STATE(2292),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4914),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4916),
    [sym_number] = ACTIONS(4914),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(4916),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4916),
    [sym_false] = ACTIONS(4916),
    [sym_null] = ACTIONS(4916),
    [sym_undefined] = ACTIONS(4916),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2138] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3111),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2139] = {
    [sym_parenthesized_expression] = STATE(2293),
    [sym__expression] = STATE(2293),
    [sym_yield_expression] = STATE(2293),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2293),
    [sym_jsx_fragment] = STATE(2293),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2293),
    [sym_anonymous_class] = STATE(2293),
    [sym_class] = STATE(2293),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2293),
    [sym_generator_function] = STATE(2293),
    [sym_call_expression] = STATE(2293),
    [sym_new_expression] = STATE(2293),
    [sym_await_expression] = STATE(2293),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2293),
    [sym_augmented_assignment_expression] = STATE(2293),
    [sym_ternary_expression] = STATE(2293),
    [sym_binary_expression] = STATE(2293),
    [sym_unary_expression] = STATE(2293),
    [sym_update_expression] = STATE(2293),
    [sym_template_string] = STATE(2293),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4918),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4920),
    [sym_number] = ACTIONS(4918),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4920),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4920),
    [sym_false] = ACTIONS(4920),
    [sym_null] = ACTIONS(4920),
    [sym_undefined] = ACTIONS(4920),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2140] = {
    [anon_sym_else] = ACTIONS(3482),
    [anon_sym_while] = ACTIONS(3482),
    [sym_comment] = ACTIONS(71),
  },
  [2141] = {
    [anon_sym_else] = ACTIONS(3584),
    [anon_sym_while] = ACTIONS(3584),
    [sym_comment] = ACTIONS(71),
  },
  [2142] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2294),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2294),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2143] = {
    [sym_parenthesized_expression] = STATE(2295),
    [sym__expression] = STATE(2295),
    [sym_yield_expression] = STATE(2295),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2295),
    [sym_jsx_fragment] = STATE(2295),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2295),
    [sym_anonymous_class] = STATE(2295),
    [sym_class] = STATE(2295),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2295),
    [sym_generator_function] = STATE(2295),
    [sym_call_expression] = STATE(2295),
    [sym_new_expression] = STATE(2295),
    [sym_await_expression] = STATE(2295),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2295),
    [sym_augmented_assignment_expression] = STATE(2295),
    [sym_ternary_expression] = STATE(2295),
    [sym_binary_expression] = STATE(2295),
    [sym_unary_expression] = STATE(2295),
    [sym_update_expression] = STATE(2295),
    [sym_template_string] = STATE(2295),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4922),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4924),
    [sym_number] = ACTIONS(4922),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4924),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4924),
    [sym_false] = ACTIONS(4924),
    [sym_null] = ACTIONS(4924),
    [sym_undefined] = ACTIONS(4924),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2144] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2296),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2296),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2145] = {
    [sym_parenthesized_expression] = STATE(2297),
    [sym__expression] = STATE(2297),
    [sym_yield_expression] = STATE(2297),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2297),
    [sym_jsx_fragment] = STATE(2297),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2297),
    [sym_anonymous_class] = STATE(2297),
    [sym_class] = STATE(2297),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2297),
    [sym_generator_function] = STATE(2297),
    [sym_call_expression] = STATE(2297),
    [sym_new_expression] = STATE(2297),
    [sym_await_expression] = STATE(2297),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2297),
    [sym_augmented_assignment_expression] = STATE(2297),
    [sym_ternary_expression] = STATE(2297),
    [sym_binary_expression] = STATE(2297),
    [sym_unary_expression] = STATE(2297),
    [sym_update_expression] = STATE(2297),
    [sym_template_string] = STATE(2297),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4926),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4928),
    [sym_number] = ACTIONS(4926),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4928),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4928),
    [sym_false] = ACTIONS(4928),
    [sym_null] = ACTIONS(4928),
    [sym_undefined] = ACTIONS(4928),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2146] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2298),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2298),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2147] = {
    [sym_parenthesized_expression] = STATE(2299),
    [sym__expression] = STATE(2299),
    [sym_yield_expression] = STATE(2299),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2299),
    [sym_jsx_fragment] = STATE(2299),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2299),
    [sym_anonymous_class] = STATE(2299),
    [sym_class] = STATE(2299),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2299),
    [sym_generator_function] = STATE(2299),
    [sym_call_expression] = STATE(2299),
    [sym_new_expression] = STATE(2299),
    [sym_await_expression] = STATE(2299),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2299),
    [sym_augmented_assignment_expression] = STATE(2299),
    [sym_ternary_expression] = STATE(2299),
    [sym_binary_expression] = STATE(2299),
    [sym_unary_expression] = STATE(2299),
    [sym_update_expression] = STATE(2299),
    [sym_template_string] = STATE(2299),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4930),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(4932),
    [sym_number] = ACTIONS(4930),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(4932),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(4932),
    [sym_false] = ACTIONS(4932),
    [sym_null] = ACTIONS(4932),
    [sym_undefined] = ACTIONS(4932),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2148] = {
    [anon_sym_RPAREN] = ACTIONS(4934),
    [sym_comment] = ACTIONS(71),
  },
  [2149] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4936),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2150] = {
    [sym_export_statement] = STATE(2302),
    [sym__declaration] = STATE(2302),
    [sym_import_statement] = STATE(2302),
    [sym_expression_statement] = STATE(2302),
    [sym_variable_declaration] = STATE(2302),
    [sym_lexical_declaration] = STATE(2302),
    [sym_statement_block] = STATE(2302),
    [sym_if_statement] = STATE(2302),
    [sym_switch_statement] = STATE(2302),
    [sym_for_statement] = STATE(2302),
    [sym_for_in_statement] = STATE(2302),
    [sym_for_of_statement] = STATE(2302),
    [sym_while_statement] = STATE(2302),
    [sym_do_statement] = STATE(2302),
    [sym_try_statement] = STATE(2302),
    [sym_with_statement] = STATE(2302),
    [sym_break_statement] = STATE(2302),
    [sym_continue_statement] = STATE(2302),
    [sym_debugger_statement] = STATE(2302),
    [sym_return_statement] = STATE(2302),
    [sym_throw_statement] = STATE(2302),
    [sym_empty_statement] = STATE(2302),
    [sym_labeled_statement] = STATE(2302),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2151] = {
    [anon_sym_RPAREN] = ACTIONS(4938),
    [sym_comment] = ACTIONS(71),
  },
  [2152] = {
    [anon_sym_RPAREN] = ACTIONS(4940),
    [sym_comment] = ACTIONS(71),
  },
  [2153] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4942),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2154] = {
    [anon_sym_RPAREN] = ACTIONS(4944),
    [sym_comment] = ACTIONS(71),
  },
  [2155] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4946),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2156] = {
    [anon_sym_else] = ACTIONS(3786),
    [anon_sym_while] = ACTIONS(3786),
    [sym_comment] = ACTIONS(71),
  },
  [2157] = {
    [anon_sym_RPAREN] = ACTIONS(4948),
    [sym_comment] = ACTIONS(71),
  },
  [2158] = {
    [anon_sym_else] = ACTIONS(3930),
    [anon_sym_while] = ACTIONS(3930),
    [sym_comment] = ACTIONS(71),
  },
  [2159] = {
    [sym_export_statement] = STATE(2130),
    [sym__declaration] = STATE(2130),
    [sym_import_statement] = STATE(2130),
    [sym_expression_statement] = STATE(2130),
    [sym_variable_declaration] = STATE(2130),
    [sym_lexical_declaration] = STATE(2130),
    [sym_statement_block] = STATE(2130),
    [sym_if_statement] = STATE(2130),
    [sym_switch_statement] = STATE(2130),
    [sym_for_statement] = STATE(2130),
    [sym_for_in_statement] = STATE(2130),
    [sym_for_of_statement] = STATE(2130),
    [sym_while_statement] = STATE(2130),
    [sym_do_statement] = STATE(2130),
    [sym_try_statement] = STATE(2130),
    [sym_with_statement] = STATE(2130),
    [sym_break_statement] = STATE(2130),
    [sym_continue_statement] = STATE(2130),
    [sym_debugger_statement] = STATE(2130),
    [sym_return_statement] = STATE(2130),
    [sym_throw_statement] = STATE(2130),
    [sym_empty_statement] = STATE(2130),
    [sym_labeled_statement] = STATE(2130),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2160] = {
    [sym_export_statement] = STATE(2131),
    [sym__declaration] = STATE(2131),
    [sym_import_statement] = STATE(2131),
    [sym_expression_statement] = STATE(2131),
    [sym_variable_declaration] = STATE(2131),
    [sym_lexical_declaration] = STATE(2131),
    [sym_statement_block] = STATE(2131),
    [sym_if_statement] = STATE(2131),
    [sym_switch_statement] = STATE(2131),
    [sym_for_statement] = STATE(2131),
    [sym_for_in_statement] = STATE(2131),
    [sym_for_of_statement] = STATE(2131),
    [sym_while_statement] = STATE(2131),
    [sym_do_statement] = STATE(2131),
    [sym_try_statement] = STATE(2131),
    [sym_with_statement] = STATE(2131),
    [sym_break_statement] = STATE(2131),
    [sym_continue_statement] = STATE(2131),
    [sym_debugger_statement] = STATE(2131),
    [sym_return_statement] = STATE(2131),
    [sym_throw_statement] = STATE(2131),
    [sym_empty_statement] = STATE(2131),
    [sym_labeled_statement] = STATE(2131),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2161] = {
    [sym_export_statement] = STATE(2132),
    [sym__declaration] = STATE(2132),
    [sym_import_statement] = STATE(2132),
    [sym_expression_statement] = STATE(2132),
    [sym_variable_declaration] = STATE(2132),
    [sym_lexical_declaration] = STATE(2132),
    [sym_statement_block] = STATE(2132),
    [sym_if_statement] = STATE(2132),
    [sym_switch_statement] = STATE(2132),
    [sym_for_statement] = STATE(2132),
    [sym_for_in_statement] = STATE(2132),
    [sym_for_of_statement] = STATE(2132),
    [sym_while_statement] = STATE(2132),
    [sym_do_statement] = STATE(2132),
    [sym_try_statement] = STATE(2132),
    [sym_with_statement] = STATE(2132),
    [sym_break_statement] = STATE(2132),
    [sym_continue_statement] = STATE(2132),
    [sym_debugger_statement] = STATE(2132),
    [sym_return_statement] = STATE(2132),
    [sym_throw_statement] = STATE(2132),
    [sym_empty_statement] = STATE(2132),
    [sym_labeled_statement] = STATE(2132),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2162] = {
    [sym_export_statement] = STATE(2133),
    [sym__declaration] = STATE(2133),
    [sym_import_statement] = STATE(2133),
    [sym_expression_statement] = STATE(2133),
    [sym_variable_declaration] = STATE(2133),
    [sym_lexical_declaration] = STATE(2133),
    [sym_statement_block] = STATE(2133),
    [sym_if_statement] = STATE(2133),
    [sym_switch_statement] = STATE(2133),
    [sym_for_statement] = STATE(2133),
    [sym_for_in_statement] = STATE(2133),
    [sym_for_of_statement] = STATE(2133),
    [sym_while_statement] = STATE(2133),
    [sym_do_statement] = STATE(2133),
    [sym_try_statement] = STATE(2133),
    [sym_with_statement] = STATE(2133),
    [sym_break_statement] = STATE(2133),
    [sym_continue_statement] = STATE(2133),
    [sym_debugger_statement] = STATE(2133),
    [sym_return_statement] = STATE(2133),
    [sym_throw_statement] = STATE(2133),
    [sym_empty_statement] = STATE(2133),
    [sym_labeled_statement] = STATE(2133),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2163] = {
    [sym_export_statement] = STATE(2134),
    [sym__declaration] = STATE(2134),
    [sym_import_statement] = STATE(2134),
    [sym_expression_statement] = STATE(2134),
    [sym_variable_declaration] = STATE(2134),
    [sym_lexical_declaration] = STATE(2134),
    [sym_statement_block] = STATE(2134),
    [sym_if_statement] = STATE(2134),
    [sym_switch_statement] = STATE(2134),
    [sym_for_statement] = STATE(2134),
    [sym_for_in_statement] = STATE(2134),
    [sym_for_of_statement] = STATE(2134),
    [sym_while_statement] = STATE(2134),
    [sym_do_statement] = STATE(2134),
    [sym_try_statement] = STATE(2134),
    [sym_with_statement] = STATE(2134),
    [sym_break_statement] = STATE(2134),
    [sym_continue_statement] = STATE(2134),
    [sym_debugger_statement] = STATE(2134),
    [sym_return_statement] = STATE(2134),
    [sym_throw_statement] = STATE(2134),
    [sym_empty_statement] = STATE(2134),
    [sym_labeled_statement] = STATE(2134),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2164] = {
    [sym_export_statement] = STATE(2135),
    [sym__declaration] = STATE(2135),
    [sym_import_statement] = STATE(2135),
    [sym_expression_statement] = STATE(2135),
    [sym_variable_declaration] = STATE(2135),
    [sym_lexical_declaration] = STATE(2135),
    [sym_statement_block] = STATE(2135),
    [sym_if_statement] = STATE(2135),
    [sym_switch_statement] = STATE(2135),
    [sym_for_statement] = STATE(2135),
    [sym_for_in_statement] = STATE(2135),
    [sym_for_of_statement] = STATE(2135),
    [sym_while_statement] = STATE(2135),
    [sym_do_statement] = STATE(2135),
    [sym_try_statement] = STATE(2135),
    [sym_with_statement] = STATE(2135),
    [sym_break_statement] = STATE(2135),
    [sym_continue_statement] = STATE(2135),
    [sym_debugger_statement] = STATE(2135),
    [sym_return_statement] = STATE(2135),
    [sym_throw_statement] = STATE(2135),
    [sym_empty_statement] = STATE(2135),
    [sym_labeled_statement] = STATE(2135),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(141),
    [sym_function] = STATE(142),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(141),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(143),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(143),
    [anon_sym_export] = ACTIONS(195),
    [anon_sym_LBRACE] = ACTIONS(197),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(199),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(201),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(203),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(205),
    [anon_sym_with] = ACTIONS(207),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(215),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2165] = {
    [anon_sym_GT] = ACTIONS(4950),
    [sym_comment] = ACTIONS(71),
  },
  [2166] = {
    [sym_nested_identifier] = STATE(640),
    [sym_jsx_namespace_name] = STATE(641),
    [anon_sym_SLASH] = ACTIONS(4952),
    [sym_jsx_identifier] = ACTIONS(1188),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(1190),
  },
  [2167] = {
    [anon_sym_LBRACE] = ACTIONS(2965),
    [anon_sym_GT] = ACTIONS(2965),
    [anon_sym_SLASH] = ACTIONS(2967),
    [sym_jsx_identifier] = ACTIONS(2965),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(2967),
  },
  [2168] = {
    [anon_sym_GT] = ACTIONS(4954),
    [sym_comment] = ACTIONS(71),
  },
  [2169] = {
    [anon_sym_LBRACE] = ACTIONS(3000),
    [anon_sym_GT] = ACTIONS(3000),
    [anon_sym_SLASH] = ACTIONS(3002),
    [sym_jsx_identifier] = ACTIONS(3000),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3002),
  },
  [2170] = {
    [anon_sym_GT] = ACTIONS(4956),
    [sym_comment] = ACTIONS(71),
  },
  [2171] = {
    [anon_sym_COLON] = ACTIONS(948),
    [anon_sym_GT] = ACTIONS(4958),
    [sym_comment] = ACTIONS(71),
  },
  [2172] = {
    [anon_sym_COLON] = ACTIONS(958),
    [anon_sym_GT] = ACTIONS(4960),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [2173] = {
    [anon_sym_GT] = ACTIONS(4960),
    [anon_sym_DOT] = ACTIONS(964),
    [sym_comment] = ACTIONS(71),
  },
  [2174] = {
    [anon_sym_GT] = ACTIONS(4960),
    [sym_comment] = ACTIONS(71),
  },
  [2175] = {
    [anon_sym_STAR] = ACTIONS(3864),
    [anon_sym_LBRACK] = ACTIONS(3864),
    [anon_sym_async] = ACTIONS(3866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3864),
    [sym_number] = ACTIONS(3864),
    [sym_identifier] = ACTIONS(3866),
    [anon_sym_AT] = ACTIONS(3864),
    [anon_sym_static] = ACTIONS(3866),
    [anon_sym_get] = ACTIONS(3866),
    [anon_sym_set] = ACTIONS(3866),
  },
  [2176] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3216),
    [anon_sym_RBRACE] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3212),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3212),
    [anon_sym_GT] = ACTIONS(3212),
    [anon_sym_SLASH] = ACTIONS(3218),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3220),
    [anon_sym_AMP_AMP] = ACTIONS(3222),
    [anon_sym_PIPE_PIPE] = ACTIONS(3224),
    [anon_sym_GT_GT] = ACTIONS(3218),
    [anon_sym_GT_GT_GT] = ACTIONS(3216),
    [anon_sym_LT_LT] = ACTIONS(3216),
    [anon_sym_AMP] = ACTIONS(3226),
    [anon_sym_CARET] = ACTIONS(3224),
    [anon_sym_PIPE] = ACTIONS(3228),
    [anon_sym_PLUS] = ACTIONS(3230),
    [anon_sym_DASH] = ACTIONS(3230),
    [anon_sym_PERCENT] = ACTIONS(3216),
    [anon_sym_LT_EQ] = ACTIONS(3232),
    [anon_sym_EQ_EQ] = ACTIONS(3212),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3232),
    [anon_sym_BANG_EQ] = ACTIONS(3212),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3232),
    [anon_sym_GT_EQ] = ACTIONS(3232),
    [anon_sym_instanceof] = ACTIONS(3232),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2177] = {
    [anon_sym_RPAREN] = ACTIONS(4962),
    [sym_comment] = ACTIONS(71),
  },
  [2178] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4964),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2179] = {
    [anon_sym_RPAREN] = ACTIONS(4966),
    [sym_comment] = ACTIONS(71),
  },
  [2180] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4968),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2181] = {
    [anon_sym_RPAREN] = ACTIONS(4970),
    [sym_comment] = ACTIONS(71),
  },
  [2182] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(4972),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2183] = {
    [sym_export_statement] = STATE(2196),
    [sym__declaration] = STATE(2196),
    [sym_import_statement] = STATE(2196),
    [sym_expression_statement] = STATE(2196),
    [sym_variable_declaration] = STATE(2196),
    [sym_lexical_declaration] = STATE(2196),
    [sym_statement_block] = STATE(2196),
    [sym_if_statement] = STATE(2196),
    [sym_switch_statement] = STATE(2196),
    [sym_for_statement] = STATE(2196),
    [sym_for_in_statement] = STATE(2196),
    [sym_for_of_statement] = STATE(2196),
    [sym_while_statement] = STATE(2196),
    [sym_do_statement] = STATE(2196),
    [sym_try_statement] = STATE(2196),
    [sym_with_statement] = STATE(2196),
    [sym_break_statement] = STATE(2196),
    [sym_continue_statement] = STATE(2196),
    [sym_debugger_statement] = STATE(2196),
    [sym_return_statement] = STATE(2196),
    [sym_throw_statement] = STATE(2196),
    [sym_empty_statement] = STATE(2196),
    [sym_labeled_statement] = STATE(2196),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2184] = {
    [sym_export_statement] = STATE(2197),
    [sym__declaration] = STATE(2197),
    [sym_import_statement] = STATE(2197),
    [sym_expression_statement] = STATE(2197),
    [sym_variable_declaration] = STATE(2197),
    [sym_lexical_declaration] = STATE(2197),
    [sym_statement_block] = STATE(2197),
    [sym_if_statement] = STATE(2197),
    [sym_switch_statement] = STATE(2197),
    [sym_for_statement] = STATE(2197),
    [sym_for_in_statement] = STATE(2197),
    [sym_for_of_statement] = STATE(2197),
    [sym_while_statement] = STATE(2197),
    [sym_do_statement] = STATE(2197),
    [sym_try_statement] = STATE(2197),
    [sym_with_statement] = STATE(2197),
    [sym_break_statement] = STATE(2197),
    [sym_continue_statement] = STATE(2197),
    [sym_debugger_statement] = STATE(2197),
    [sym_return_statement] = STATE(2197),
    [sym_throw_statement] = STATE(2197),
    [sym_empty_statement] = STATE(2197),
    [sym_labeled_statement] = STATE(2197),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2185] = {
    [sym_export_statement] = STATE(2198),
    [sym__declaration] = STATE(2198),
    [sym_import_statement] = STATE(2198),
    [sym_expression_statement] = STATE(2198),
    [sym_variable_declaration] = STATE(2198),
    [sym_lexical_declaration] = STATE(2198),
    [sym_statement_block] = STATE(2198),
    [sym_if_statement] = STATE(2198),
    [sym_switch_statement] = STATE(2198),
    [sym_for_statement] = STATE(2198),
    [sym_for_in_statement] = STATE(2198),
    [sym_for_of_statement] = STATE(2198),
    [sym_while_statement] = STATE(2198),
    [sym_do_statement] = STATE(2198),
    [sym_try_statement] = STATE(2198),
    [sym_with_statement] = STATE(2198),
    [sym_break_statement] = STATE(2198),
    [sym_continue_statement] = STATE(2198),
    [sym_debugger_statement] = STATE(2198),
    [sym_return_statement] = STATE(2198),
    [sym_throw_statement] = STATE(2198),
    [sym_empty_statement] = STATE(2198),
    [sym_labeled_statement] = STATE(2198),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2186] = {
    [sym_export_statement] = STATE(2199),
    [sym__declaration] = STATE(2199),
    [sym_import_statement] = STATE(2199),
    [sym_expression_statement] = STATE(2199),
    [sym_variable_declaration] = STATE(2199),
    [sym_lexical_declaration] = STATE(2199),
    [sym_statement_block] = STATE(2199),
    [sym_if_statement] = STATE(2199),
    [sym_switch_statement] = STATE(2199),
    [sym_for_statement] = STATE(2199),
    [sym_for_in_statement] = STATE(2199),
    [sym_for_of_statement] = STATE(2199),
    [sym_while_statement] = STATE(2199),
    [sym_do_statement] = STATE(2199),
    [sym_try_statement] = STATE(2199),
    [sym_with_statement] = STATE(2199),
    [sym_break_statement] = STATE(2199),
    [sym_continue_statement] = STATE(2199),
    [sym_debugger_statement] = STATE(2199),
    [sym_return_statement] = STATE(2199),
    [sym_throw_statement] = STATE(2199),
    [sym_empty_statement] = STATE(2199),
    [sym_labeled_statement] = STATE(2199),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2187] = {
    [sym_export_statement] = STATE(2200),
    [sym__declaration] = STATE(2200),
    [sym_import_statement] = STATE(2200),
    [sym_expression_statement] = STATE(2200),
    [sym_variable_declaration] = STATE(2200),
    [sym_lexical_declaration] = STATE(2200),
    [sym_statement_block] = STATE(2200),
    [sym_if_statement] = STATE(2200),
    [sym_switch_statement] = STATE(2200),
    [sym_for_statement] = STATE(2200),
    [sym_for_in_statement] = STATE(2200),
    [sym_for_of_statement] = STATE(2200),
    [sym_while_statement] = STATE(2200),
    [sym_do_statement] = STATE(2200),
    [sym_try_statement] = STATE(2200),
    [sym_with_statement] = STATE(2200),
    [sym_break_statement] = STATE(2200),
    [sym_continue_statement] = STATE(2200),
    [sym_debugger_statement] = STATE(2200),
    [sym_return_statement] = STATE(2200),
    [sym_throw_statement] = STATE(2200),
    [sym_empty_statement] = STATE(2200),
    [sym_labeled_statement] = STATE(2200),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2188] = {
    [sym_export_statement] = STATE(2201),
    [sym__declaration] = STATE(2201),
    [sym_import_statement] = STATE(2201),
    [sym_expression_statement] = STATE(2201),
    [sym_variable_declaration] = STATE(2201),
    [sym_lexical_declaration] = STATE(2201),
    [sym_statement_block] = STATE(2201),
    [sym_if_statement] = STATE(2201),
    [sym_switch_statement] = STATE(2201),
    [sym_for_statement] = STATE(2201),
    [sym_for_in_statement] = STATE(2201),
    [sym_for_of_statement] = STATE(2201),
    [sym_while_statement] = STATE(2201),
    [sym_do_statement] = STATE(2201),
    [sym_try_statement] = STATE(2201),
    [sym_with_statement] = STATE(2201),
    [sym_break_statement] = STATE(2201),
    [sym_continue_statement] = STATE(2201),
    [sym_debugger_statement] = STATE(2201),
    [sym_return_statement] = STATE(2201),
    [sym_throw_statement] = STATE(2201),
    [sym_empty_statement] = STATE(2201),
    [sym_labeled_statement] = STATE(2201),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2189] = {
    [sym_export_statement] = STATE(2202),
    [sym__declaration] = STATE(2202),
    [sym_import_statement] = STATE(2202),
    [sym_expression_statement] = STATE(2202),
    [sym_variable_declaration] = STATE(2202),
    [sym_lexical_declaration] = STATE(2202),
    [sym_statement_block] = STATE(2202),
    [sym_if_statement] = STATE(2202),
    [sym_switch_statement] = STATE(2202),
    [sym_for_statement] = STATE(2202),
    [sym_for_in_statement] = STATE(2202),
    [sym_for_of_statement] = STATE(2202),
    [sym_while_statement] = STATE(2202),
    [sym_do_statement] = STATE(2202),
    [sym_try_statement] = STATE(2202),
    [sym_with_statement] = STATE(2202),
    [sym_break_statement] = STATE(2202),
    [sym_continue_statement] = STATE(2202),
    [sym_debugger_statement] = STATE(2202),
    [sym_return_statement] = STATE(2202),
    [sym_throw_statement] = STATE(2202),
    [sym_empty_statement] = STATE(2202),
    [sym_labeled_statement] = STATE(2202),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2190] = {
    [sym_export_statement] = STATE(2321),
    [sym__declaration] = STATE(2321),
    [sym_import_statement] = STATE(2321),
    [sym_expression_statement] = STATE(2321),
    [sym_variable_declaration] = STATE(2321),
    [sym_lexical_declaration] = STATE(2321),
    [sym_statement_block] = STATE(2321),
    [sym_if_statement] = STATE(2321),
    [sym_switch_statement] = STATE(2321),
    [sym_for_statement] = STATE(2321),
    [sym_for_in_statement] = STATE(2321),
    [sym_for_of_statement] = STATE(2321),
    [sym_while_statement] = STATE(2321),
    [sym_do_statement] = STATE(2321),
    [sym_try_statement] = STATE(2321),
    [sym_with_statement] = STATE(2321),
    [sym_break_statement] = STATE(2321),
    [sym_continue_statement] = STATE(2321),
    [sym_debugger_statement] = STATE(2321),
    [sym_return_statement] = STATE(2321),
    [sym_throw_statement] = STATE(2321),
    [sym_empty_statement] = STATE(2321),
    [sym_labeled_statement] = STATE(2321),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2191] = {
    [sym_export_statement] = STATE(2322),
    [sym__declaration] = STATE(2322),
    [sym_import_statement] = STATE(2322),
    [sym_expression_statement] = STATE(2322),
    [sym_variable_declaration] = STATE(2322),
    [sym_lexical_declaration] = STATE(2322),
    [sym_statement_block] = STATE(2322),
    [sym_if_statement] = STATE(2322),
    [sym_switch_statement] = STATE(2322),
    [sym_for_statement] = STATE(2322),
    [sym_for_in_statement] = STATE(2322),
    [sym_for_of_statement] = STATE(2322),
    [sym_while_statement] = STATE(2322),
    [sym_do_statement] = STATE(2322),
    [sym_try_statement] = STATE(2322),
    [sym_with_statement] = STATE(2322),
    [sym_break_statement] = STATE(2322),
    [sym_continue_statement] = STATE(2322),
    [sym_debugger_statement] = STATE(2322),
    [sym_return_statement] = STATE(2322),
    [sym_throw_statement] = STATE(2322),
    [sym_empty_statement] = STATE(2322),
    [sym_labeled_statement] = STATE(2322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2192] = {
    [sym_export_statement] = STATE(2323),
    [sym__declaration] = STATE(2323),
    [sym_import_statement] = STATE(2323),
    [sym_expression_statement] = STATE(2323),
    [sym_variable_declaration] = STATE(2323),
    [sym_lexical_declaration] = STATE(2323),
    [sym_statement_block] = STATE(2323),
    [sym_if_statement] = STATE(2323),
    [sym_switch_statement] = STATE(2323),
    [sym_for_statement] = STATE(2323),
    [sym_for_in_statement] = STATE(2323),
    [sym_for_of_statement] = STATE(2323),
    [sym_while_statement] = STATE(2323),
    [sym_do_statement] = STATE(2323),
    [sym_try_statement] = STATE(2323),
    [sym_with_statement] = STATE(2323),
    [sym_break_statement] = STATE(2323),
    [sym_continue_statement] = STATE(2323),
    [sym_debugger_statement] = STATE(2323),
    [sym_return_statement] = STATE(2323),
    [sym_throw_statement] = STATE(2323),
    [sym_empty_statement] = STATE(2323),
    [sym_labeled_statement] = STATE(2323),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2193] = {
    [sym_export_statement] = STATE(2324),
    [sym__declaration] = STATE(2324),
    [sym_import_statement] = STATE(2324),
    [sym_expression_statement] = STATE(2324),
    [sym_variable_declaration] = STATE(2324),
    [sym_lexical_declaration] = STATE(2324),
    [sym_statement_block] = STATE(2324),
    [sym_if_statement] = STATE(2324),
    [sym_switch_statement] = STATE(2324),
    [sym_for_statement] = STATE(2324),
    [sym_for_in_statement] = STATE(2324),
    [sym_for_of_statement] = STATE(2324),
    [sym_while_statement] = STATE(2324),
    [sym_do_statement] = STATE(2324),
    [sym_try_statement] = STATE(2324),
    [sym_with_statement] = STATE(2324),
    [sym_break_statement] = STATE(2324),
    [sym_continue_statement] = STATE(2324),
    [sym_debugger_statement] = STATE(2324),
    [sym_return_statement] = STATE(2324),
    [sym_throw_statement] = STATE(2324),
    [sym_empty_statement] = STATE(2324),
    [sym_labeled_statement] = STATE(2324),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2194] = {
    [sym_export_statement] = STATE(2325),
    [sym__declaration] = STATE(2325),
    [sym_import_statement] = STATE(2325),
    [sym_expression_statement] = STATE(2325),
    [sym_variable_declaration] = STATE(2325),
    [sym_lexical_declaration] = STATE(2325),
    [sym_statement_block] = STATE(2325),
    [sym_if_statement] = STATE(2325),
    [sym_switch_statement] = STATE(2325),
    [sym_for_statement] = STATE(2325),
    [sym_for_in_statement] = STATE(2325),
    [sym_for_of_statement] = STATE(2325),
    [sym_while_statement] = STATE(2325),
    [sym_do_statement] = STATE(2325),
    [sym_try_statement] = STATE(2325),
    [sym_with_statement] = STATE(2325),
    [sym_break_statement] = STATE(2325),
    [sym_continue_statement] = STATE(2325),
    [sym_debugger_statement] = STATE(2325),
    [sym_return_statement] = STATE(2325),
    [sym_throw_statement] = STATE(2325),
    [sym_empty_statement] = STATE(2325),
    [sym_labeled_statement] = STATE(2325),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2195] = {
    [sym_export_statement] = STATE(2326),
    [sym__declaration] = STATE(2326),
    [sym_import_statement] = STATE(2326),
    [sym_expression_statement] = STATE(2326),
    [sym_variable_declaration] = STATE(2326),
    [sym_lexical_declaration] = STATE(2326),
    [sym_statement_block] = STATE(2326),
    [sym_if_statement] = STATE(2326),
    [sym_switch_statement] = STATE(2326),
    [sym_for_statement] = STATE(2326),
    [sym_for_in_statement] = STATE(2326),
    [sym_for_of_statement] = STATE(2326),
    [sym_while_statement] = STATE(2326),
    [sym_do_statement] = STATE(2326),
    [sym_try_statement] = STATE(2326),
    [sym_with_statement] = STATE(2326),
    [sym_break_statement] = STATE(2326),
    [sym_continue_statement] = STATE(2326),
    [sym_debugger_statement] = STATE(2326),
    [sym_return_statement] = STATE(2326),
    [sym_throw_statement] = STATE(2326),
    [sym_empty_statement] = STATE(2326),
    [sym_labeled_statement] = STATE(2326),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(632),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(636),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(638),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(644),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(664),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2196] = {
    [ts_builtin_sym_end] = ACTIONS(4636),
    [anon_sym_export] = ACTIONS(4638),
    [anon_sym_LBRACE] = ACTIONS(4636),
    [anon_sym_RBRACE] = ACTIONS(4636),
    [anon_sym_import] = ACTIONS(4638),
    [anon_sym_var] = ACTIONS(4638),
    [anon_sym_let] = ACTIONS(4638),
    [anon_sym_const] = ACTIONS(4638),
    [anon_sym_if] = ACTIONS(4638),
    [anon_sym_else] = ACTIONS(4638),
    [anon_sym_switch] = ACTIONS(4638),
    [anon_sym_for] = ACTIONS(4638),
    [anon_sym_LPAREN] = ACTIONS(4636),
    [anon_sym_while] = ACTIONS(4638),
    [anon_sym_do] = ACTIONS(4638),
    [anon_sym_try] = ACTIONS(4638),
    [anon_sym_with] = ACTIONS(4638),
    [anon_sym_break] = ACTIONS(4638),
    [anon_sym_continue] = ACTIONS(4638),
    [anon_sym_debugger] = ACTIONS(4638),
    [anon_sym_return] = ACTIONS(4638),
    [anon_sym_throw] = ACTIONS(4638),
    [anon_sym_SEMI] = ACTIONS(4636),
    [anon_sym_yield] = ACTIONS(4638),
    [anon_sym_LBRACK] = ACTIONS(4636),
    [anon_sym_LT] = ACTIONS(4636),
    [anon_sym_class] = ACTIONS(4638),
    [anon_sym_async] = ACTIONS(4638),
    [anon_sym_function] = ACTIONS(4638),
    [anon_sym_EQ_GT] = ACTIONS(4636),
    [anon_sym_new] = ACTIONS(4638),
    [anon_sym_await] = ACTIONS(4638),
    [anon_sym_PLUS] = ACTIONS(4638),
    [anon_sym_DASH] = ACTIONS(4638),
    [anon_sym_BANG] = ACTIONS(4636),
    [anon_sym_TILDE] = ACTIONS(4636),
    [anon_sym_typeof] = ACTIONS(4638),
    [anon_sym_void] = ACTIONS(4638),
    [anon_sym_delete] = ACTIONS(4638),
    [anon_sym_PLUS_PLUS] = ACTIONS(4636),
    [anon_sym_DASH_DASH] = ACTIONS(4636),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4636),
    [anon_sym_BQUOTE] = ACTIONS(4636),
    [sym_regex] = ACTIONS(4638),
    [sym_number] = ACTIONS(4636),
    [sym_identifier] = ACTIONS(4638),
    [sym_this] = ACTIONS(4638),
    [sym_super] = ACTIONS(4638),
    [sym_true] = ACTIONS(4638),
    [sym_false] = ACTIONS(4638),
    [sym_null] = ACTIONS(4638),
    [sym_undefined] = ACTIONS(4638),
    [anon_sym_AT] = ACTIONS(4636),
    [anon_sym_get] = ACTIONS(4638),
    [anon_sym_set] = ACTIONS(4638),
  },
  [2197] = {
    [ts_builtin_sym_end] = ACTIONS(4646),
    [anon_sym_export] = ACTIONS(4648),
    [anon_sym_LBRACE] = ACTIONS(4646),
    [anon_sym_RBRACE] = ACTIONS(4646),
    [anon_sym_import] = ACTIONS(4648),
    [anon_sym_var] = ACTIONS(4648),
    [anon_sym_let] = ACTIONS(4648),
    [anon_sym_const] = ACTIONS(4648),
    [anon_sym_if] = ACTIONS(4648),
    [anon_sym_else] = ACTIONS(4648),
    [anon_sym_switch] = ACTIONS(4648),
    [anon_sym_for] = ACTIONS(4648),
    [anon_sym_LPAREN] = ACTIONS(4646),
    [anon_sym_while] = ACTIONS(4648),
    [anon_sym_do] = ACTIONS(4648),
    [anon_sym_try] = ACTIONS(4648),
    [anon_sym_with] = ACTIONS(4648),
    [anon_sym_break] = ACTIONS(4648),
    [anon_sym_continue] = ACTIONS(4648),
    [anon_sym_debugger] = ACTIONS(4648),
    [anon_sym_return] = ACTIONS(4648),
    [anon_sym_throw] = ACTIONS(4648),
    [anon_sym_SEMI] = ACTIONS(4646),
    [anon_sym_yield] = ACTIONS(4648),
    [anon_sym_LBRACK] = ACTIONS(4646),
    [anon_sym_LT] = ACTIONS(4646),
    [anon_sym_class] = ACTIONS(4648),
    [anon_sym_async] = ACTIONS(4648),
    [anon_sym_function] = ACTIONS(4648),
    [anon_sym_EQ_GT] = ACTIONS(4646),
    [anon_sym_new] = ACTIONS(4648),
    [anon_sym_await] = ACTIONS(4648),
    [anon_sym_PLUS] = ACTIONS(4648),
    [anon_sym_DASH] = ACTIONS(4648),
    [anon_sym_BANG] = ACTIONS(4646),
    [anon_sym_TILDE] = ACTIONS(4646),
    [anon_sym_typeof] = ACTIONS(4648),
    [anon_sym_void] = ACTIONS(4648),
    [anon_sym_delete] = ACTIONS(4648),
    [anon_sym_PLUS_PLUS] = ACTIONS(4646),
    [anon_sym_DASH_DASH] = ACTIONS(4646),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4646),
    [anon_sym_BQUOTE] = ACTIONS(4646),
    [sym_regex] = ACTIONS(4648),
    [sym_number] = ACTIONS(4646),
    [sym_identifier] = ACTIONS(4648),
    [sym_this] = ACTIONS(4648),
    [sym_super] = ACTIONS(4648),
    [sym_true] = ACTIONS(4648),
    [sym_false] = ACTIONS(4648),
    [sym_null] = ACTIONS(4648),
    [sym_undefined] = ACTIONS(4648),
    [anon_sym_AT] = ACTIONS(4646),
    [anon_sym_get] = ACTIONS(4648),
    [anon_sym_set] = ACTIONS(4648),
  },
  [2198] = {
    [ts_builtin_sym_end] = ACTIONS(4652),
    [anon_sym_export] = ACTIONS(4654),
    [anon_sym_LBRACE] = ACTIONS(4652),
    [anon_sym_RBRACE] = ACTIONS(4652),
    [anon_sym_import] = ACTIONS(4654),
    [anon_sym_var] = ACTIONS(4654),
    [anon_sym_let] = ACTIONS(4654),
    [anon_sym_const] = ACTIONS(4654),
    [anon_sym_if] = ACTIONS(4654),
    [anon_sym_else] = ACTIONS(4654),
    [anon_sym_switch] = ACTIONS(4654),
    [anon_sym_for] = ACTIONS(4654),
    [anon_sym_LPAREN] = ACTIONS(4652),
    [anon_sym_while] = ACTIONS(4654),
    [anon_sym_do] = ACTIONS(4654),
    [anon_sym_try] = ACTIONS(4654),
    [anon_sym_with] = ACTIONS(4654),
    [anon_sym_break] = ACTIONS(4654),
    [anon_sym_continue] = ACTIONS(4654),
    [anon_sym_debugger] = ACTIONS(4654),
    [anon_sym_return] = ACTIONS(4654),
    [anon_sym_throw] = ACTIONS(4654),
    [anon_sym_SEMI] = ACTIONS(4652),
    [anon_sym_yield] = ACTIONS(4654),
    [anon_sym_LBRACK] = ACTIONS(4652),
    [anon_sym_LT] = ACTIONS(4652),
    [anon_sym_class] = ACTIONS(4654),
    [anon_sym_async] = ACTIONS(4654),
    [anon_sym_function] = ACTIONS(4654),
    [anon_sym_EQ_GT] = ACTIONS(4652),
    [anon_sym_new] = ACTIONS(4654),
    [anon_sym_await] = ACTIONS(4654),
    [anon_sym_PLUS] = ACTIONS(4654),
    [anon_sym_DASH] = ACTIONS(4654),
    [anon_sym_BANG] = ACTIONS(4652),
    [anon_sym_TILDE] = ACTIONS(4652),
    [anon_sym_typeof] = ACTIONS(4654),
    [anon_sym_void] = ACTIONS(4654),
    [anon_sym_delete] = ACTIONS(4654),
    [anon_sym_PLUS_PLUS] = ACTIONS(4652),
    [anon_sym_DASH_DASH] = ACTIONS(4652),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4652),
    [anon_sym_BQUOTE] = ACTIONS(4652),
    [sym_regex] = ACTIONS(4654),
    [sym_number] = ACTIONS(4652),
    [sym_identifier] = ACTIONS(4654),
    [sym_this] = ACTIONS(4654),
    [sym_super] = ACTIONS(4654),
    [sym_true] = ACTIONS(4654),
    [sym_false] = ACTIONS(4654),
    [sym_null] = ACTIONS(4654),
    [sym_undefined] = ACTIONS(4654),
    [anon_sym_AT] = ACTIONS(4652),
    [anon_sym_get] = ACTIONS(4654),
    [anon_sym_set] = ACTIONS(4654),
  },
  [2199] = {
    [ts_builtin_sym_end] = ACTIONS(4656),
    [anon_sym_export] = ACTIONS(4658),
    [anon_sym_LBRACE] = ACTIONS(4656),
    [anon_sym_RBRACE] = ACTIONS(4656),
    [anon_sym_import] = ACTIONS(4658),
    [anon_sym_var] = ACTIONS(4658),
    [anon_sym_let] = ACTIONS(4658),
    [anon_sym_const] = ACTIONS(4658),
    [anon_sym_if] = ACTIONS(4658),
    [anon_sym_else] = ACTIONS(4658),
    [anon_sym_switch] = ACTIONS(4658),
    [anon_sym_for] = ACTIONS(4658),
    [anon_sym_LPAREN] = ACTIONS(4656),
    [anon_sym_while] = ACTIONS(4658),
    [anon_sym_do] = ACTIONS(4658),
    [anon_sym_try] = ACTIONS(4658),
    [anon_sym_with] = ACTIONS(4658),
    [anon_sym_break] = ACTIONS(4658),
    [anon_sym_continue] = ACTIONS(4658),
    [anon_sym_debugger] = ACTIONS(4658),
    [anon_sym_return] = ACTIONS(4658),
    [anon_sym_throw] = ACTIONS(4658),
    [anon_sym_SEMI] = ACTIONS(4656),
    [anon_sym_yield] = ACTIONS(4658),
    [anon_sym_LBRACK] = ACTIONS(4656),
    [anon_sym_LT] = ACTIONS(4656),
    [anon_sym_class] = ACTIONS(4658),
    [anon_sym_async] = ACTIONS(4658),
    [anon_sym_function] = ACTIONS(4658),
    [anon_sym_EQ_GT] = ACTIONS(4656),
    [anon_sym_new] = ACTIONS(4658),
    [anon_sym_await] = ACTIONS(4658),
    [anon_sym_PLUS] = ACTIONS(4658),
    [anon_sym_DASH] = ACTIONS(4658),
    [anon_sym_BANG] = ACTIONS(4656),
    [anon_sym_TILDE] = ACTIONS(4656),
    [anon_sym_typeof] = ACTIONS(4658),
    [anon_sym_void] = ACTIONS(4658),
    [anon_sym_delete] = ACTIONS(4658),
    [anon_sym_PLUS_PLUS] = ACTIONS(4656),
    [anon_sym_DASH_DASH] = ACTIONS(4656),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4656),
    [anon_sym_BQUOTE] = ACTIONS(4656),
    [sym_regex] = ACTIONS(4658),
    [sym_number] = ACTIONS(4656),
    [sym_identifier] = ACTIONS(4658),
    [sym_this] = ACTIONS(4658),
    [sym_super] = ACTIONS(4658),
    [sym_true] = ACTIONS(4658),
    [sym_false] = ACTIONS(4658),
    [sym_null] = ACTIONS(4658),
    [sym_undefined] = ACTIONS(4658),
    [anon_sym_AT] = ACTIONS(4656),
    [anon_sym_get] = ACTIONS(4658),
    [anon_sym_set] = ACTIONS(4658),
  },
  [2200] = {
    [ts_builtin_sym_end] = ACTIONS(4660),
    [anon_sym_export] = ACTIONS(4662),
    [anon_sym_LBRACE] = ACTIONS(4660),
    [anon_sym_RBRACE] = ACTIONS(4660),
    [anon_sym_import] = ACTIONS(4662),
    [anon_sym_var] = ACTIONS(4662),
    [anon_sym_let] = ACTIONS(4662),
    [anon_sym_const] = ACTIONS(4662),
    [anon_sym_if] = ACTIONS(4662),
    [anon_sym_else] = ACTIONS(4662),
    [anon_sym_switch] = ACTIONS(4662),
    [anon_sym_for] = ACTIONS(4662),
    [anon_sym_LPAREN] = ACTIONS(4660),
    [anon_sym_while] = ACTIONS(4662),
    [anon_sym_do] = ACTIONS(4662),
    [anon_sym_try] = ACTIONS(4662),
    [anon_sym_with] = ACTIONS(4662),
    [anon_sym_break] = ACTIONS(4662),
    [anon_sym_continue] = ACTIONS(4662),
    [anon_sym_debugger] = ACTIONS(4662),
    [anon_sym_return] = ACTIONS(4662),
    [anon_sym_throw] = ACTIONS(4662),
    [anon_sym_SEMI] = ACTIONS(4660),
    [anon_sym_yield] = ACTIONS(4662),
    [anon_sym_LBRACK] = ACTIONS(4660),
    [anon_sym_LT] = ACTIONS(4660),
    [anon_sym_class] = ACTIONS(4662),
    [anon_sym_async] = ACTIONS(4662),
    [anon_sym_function] = ACTIONS(4662),
    [anon_sym_EQ_GT] = ACTIONS(4660),
    [anon_sym_new] = ACTIONS(4662),
    [anon_sym_await] = ACTIONS(4662),
    [anon_sym_PLUS] = ACTIONS(4662),
    [anon_sym_DASH] = ACTIONS(4662),
    [anon_sym_BANG] = ACTIONS(4660),
    [anon_sym_TILDE] = ACTIONS(4660),
    [anon_sym_typeof] = ACTIONS(4662),
    [anon_sym_void] = ACTIONS(4662),
    [anon_sym_delete] = ACTIONS(4662),
    [anon_sym_PLUS_PLUS] = ACTIONS(4660),
    [anon_sym_DASH_DASH] = ACTIONS(4660),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4660),
    [anon_sym_BQUOTE] = ACTIONS(4660),
    [sym_regex] = ACTIONS(4662),
    [sym_number] = ACTIONS(4660),
    [sym_identifier] = ACTIONS(4662),
    [sym_this] = ACTIONS(4662),
    [sym_super] = ACTIONS(4662),
    [sym_true] = ACTIONS(4662),
    [sym_false] = ACTIONS(4662),
    [sym_null] = ACTIONS(4662),
    [sym_undefined] = ACTIONS(4662),
    [anon_sym_AT] = ACTIONS(4660),
    [anon_sym_get] = ACTIONS(4662),
    [anon_sym_set] = ACTIONS(4662),
  },
  [2201] = {
    [ts_builtin_sym_end] = ACTIONS(4664),
    [anon_sym_export] = ACTIONS(4666),
    [anon_sym_LBRACE] = ACTIONS(4664),
    [anon_sym_RBRACE] = ACTIONS(4664),
    [anon_sym_import] = ACTIONS(4666),
    [anon_sym_var] = ACTIONS(4666),
    [anon_sym_let] = ACTIONS(4666),
    [anon_sym_const] = ACTIONS(4666),
    [anon_sym_if] = ACTIONS(4666),
    [anon_sym_else] = ACTIONS(4666),
    [anon_sym_switch] = ACTIONS(4666),
    [anon_sym_for] = ACTIONS(4666),
    [anon_sym_LPAREN] = ACTIONS(4664),
    [anon_sym_while] = ACTIONS(4666),
    [anon_sym_do] = ACTIONS(4666),
    [anon_sym_try] = ACTIONS(4666),
    [anon_sym_with] = ACTIONS(4666),
    [anon_sym_break] = ACTIONS(4666),
    [anon_sym_continue] = ACTIONS(4666),
    [anon_sym_debugger] = ACTIONS(4666),
    [anon_sym_return] = ACTIONS(4666),
    [anon_sym_throw] = ACTIONS(4666),
    [anon_sym_SEMI] = ACTIONS(4664),
    [anon_sym_yield] = ACTIONS(4666),
    [anon_sym_LBRACK] = ACTIONS(4664),
    [anon_sym_LT] = ACTIONS(4664),
    [anon_sym_class] = ACTIONS(4666),
    [anon_sym_async] = ACTIONS(4666),
    [anon_sym_function] = ACTIONS(4666),
    [anon_sym_EQ_GT] = ACTIONS(4664),
    [anon_sym_new] = ACTIONS(4666),
    [anon_sym_await] = ACTIONS(4666),
    [anon_sym_PLUS] = ACTIONS(4666),
    [anon_sym_DASH] = ACTIONS(4666),
    [anon_sym_BANG] = ACTIONS(4664),
    [anon_sym_TILDE] = ACTIONS(4664),
    [anon_sym_typeof] = ACTIONS(4666),
    [anon_sym_void] = ACTIONS(4666),
    [anon_sym_delete] = ACTIONS(4666),
    [anon_sym_PLUS_PLUS] = ACTIONS(4664),
    [anon_sym_DASH_DASH] = ACTIONS(4664),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4664),
    [anon_sym_BQUOTE] = ACTIONS(4664),
    [sym_regex] = ACTIONS(4666),
    [sym_number] = ACTIONS(4664),
    [sym_identifier] = ACTIONS(4666),
    [sym_this] = ACTIONS(4666),
    [sym_super] = ACTIONS(4666),
    [sym_true] = ACTIONS(4666),
    [sym_false] = ACTIONS(4666),
    [sym_null] = ACTIONS(4666),
    [sym_undefined] = ACTIONS(4666),
    [anon_sym_AT] = ACTIONS(4664),
    [anon_sym_get] = ACTIONS(4666),
    [anon_sym_set] = ACTIONS(4666),
  },
  [2202] = {
    [ts_builtin_sym_end] = ACTIONS(4668),
    [anon_sym_export] = ACTIONS(4670),
    [anon_sym_LBRACE] = ACTIONS(4668),
    [anon_sym_RBRACE] = ACTIONS(4668),
    [anon_sym_import] = ACTIONS(4670),
    [anon_sym_var] = ACTIONS(4670),
    [anon_sym_let] = ACTIONS(4670),
    [anon_sym_const] = ACTIONS(4670),
    [anon_sym_if] = ACTIONS(4670),
    [anon_sym_else] = ACTIONS(4670),
    [anon_sym_switch] = ACTIONS(4670),
    [anon_sym_for] = ACTIONS(4670),
    [anon_sym_LPAREN] = ACTIONS(4668),
    [anon_sym_while] = ACTIONS(4670),
    [anon_sym_do] = ACTIONS(4670),
    [anon_sym_try] = ACTIONS(4670),
    [anon_sym_with] = ACTIONS(4670),
    [anon_sym_break] = ACTIONS(4670),
    [anon_sym_continue] = ACTIONS(4670),
    [anon_sym_debugger] = ACTIONS(4670),
    [anon_sym_return] = ACTIONS(4670),
    [anon_sym_throw] = ACTIONS(4670),
    [anon_sym_SEMI] = ACTIONS(4668),
    [anon_sym_yield] = ACTIONS(4670),
    [anon_sym_LBRACK] = ACTIONS(4668),
    [anon_sym_LT] = ACTIONS(4668),
    [anon_sym_class] = ACTIONS(4670),
    [anon_sym_async] = ACTIONS(4670),
    [anon_sym_function] = ACTIONS(4670),
    [anon_sym_EQ_GT] = ACTIONS(4668),
    [anon_sym_new] = ACTIONS(4670),
    [anon_sym_await] = ACTIONS(4670),
    [anon_sym_PLUS] = ACTIONS(4670),
    [anon_sym_DASH] = ACTIONS(4670),
    [anon_sym_BANG] = ACTIONS(4668),
    [anon_sym_TILDE] = ACTIONS(4668),
    [anon_sym_typeof] = ACTIONS(4670),
    [anon_sym_void] = ACTIONS(4670),
    [anon_sym_delete] = ACTIONS(4670),
    [anon_sym_PLUS_PLUS] = ACTIONS(4668),
    [anon_sym_DASH_DASH] = ACTIONS(4668),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4668),
    [anon_sym_BQUOTE] = ACTIONS(4668),
    [sym_regex] = ACTIONS(4670),
    [sym_number] = ACTIONS(4668),
    [sym_identifier] = ACTIONS(4670),
    [sym_this] = ACTIONS(4670),
    [sym_super] = ACTIONS(4670),
    [sym_true] = ACTIONS(4670),
    [sym_false] = ACTIONS(4670),
    [sym_null] = ACTIONS(4670),
    [sym_undefined] = ACTIONS(4670),
    [anon_sym_AT] = ACTIONS(4668),
    [anon_sym_get] = ACTIONS(4670),
    [anon_sym_set] = ACTIONS(4670),
  },
  [2203] = {
    [ts_builtin_sym_end] = ACTIONS(4718),
    [anon_sym_export] = ACTIONS(4720),
    [anon_sym_LBRACE] = ACTIONS(4718),
    [anon_sym_RBRACE] = ACTIONS(4718),
    [anon_sym_import] = ACTIONS(4720),
    [anon_sym_var] = ACTIONS(4720),
    [anon_sym_let] = ACTIONS(4720),
    [anon_sym_const] = ACTIONS(4720),
    [anon_sym_if] = ACTIONS(4720),
    [anon_sym_else] = ACTIONS(4720),
    [anon_sym_switch] = ACTIONS(4720),
    [anon_sym_for] = ACTIONS(4720),
    [anon_sym_LPAREN] = ACTIONS(4718),
    [anon_sym_while] = ACTIONS(4720),
    [anon_sym_do] = ACTIONS(4720),
    [anon_sym_try] = ACTIONS(4720),
    [anon_sym_with] = ACTIONS(4720),
    [anon_sym_break] = ACTIONS(4720),
    [anon_sym_continue] = ACTIONS(4720),
    [anon_sym_debugger] = ACTIONS(4720),
    [anon_sym_return] = ACTIONS(4720),
    [anon_sym_throw] = ACTIONS(4720),
    [anon_sym_SEMI] = ACTIONS(4718),
    [anon_sym_finally] = ACTIONS(4720),
    [anon_sym_yield] = ACTIONS(4720),
    [anon_sym_LBRACK] = ACTIONS(4718),
    [anon_sym_LT] = ACTIONS(4718),
    [anon_sym_class] = ACTIONS(4720),
    [anon_sym_async] = ACTIONS(4720),
    [anon_sym_function] = ACTIONS(4720),
    [anon_sym_EQ_GT] = ACTIONS(4718),
    [anon_sym_new] = ACTIONS(4720),
    [anon_sym_await] = ACTIONS(4720),
    [anon_sym_PLUS] = ACTIONS(4720),
    [anon_sym_DASH] = ACTIONS(4720),
    [anon_sym_BANG] = ACTIONS(4718),
    [anon_sym_TILDE] = ACTIONS(4718),
    [anon_sym_typeof] = ACTIONS(4720),
    [anon_sym_void] = ACTIONS(4720),
    [anon_sym_delete] = ACTIONS(4720),
    [anon_sym_PLUS_PLUS] = ACTIONS(4718),
    [anon_sym_DASH_DASH] = ACTIONS(4718),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4718),
    [anon_sym_BQUOTE] = ACTIONS(4718),
    [sym_regex] = ACTIONS(4720),
    [sym_number] = ACTIONS(4718),
    [sym_identifier] = ACTIONS(4720),
    [sym_this] = ACTIONS(4720),
    [sym_super] = ACTIONS(4720),
    [sym_true] = ACTIONS(4720),
    [sym_false] = ACTIONS(4720),
    [sym_null] = ACTIONS(4720),
    [sym_undefined] = ACTIONS(4720),
    [anon_sym_AT] = ACTIONS(4718),
    [anon_sym_get] = ACTIONS(4720),
    [anon_sym_set] = ACTIONS(4720),
  },
  [2204] = {
    [anon_sym_export] = ACTIONS(2345),
    [anon_sym_default] = ACTIONS(2345),
    [anon_sym_LBRACE] = ACTIONS(2343),
    [anon_sym_RBRACE] = ACTIONS(2343),
    [anon_sym_import] = ACTIONS(2345),
    [anon_sym_var] = ACTIONS(2345),
    [anon_sym_let] = ACTIONS(2345),
    [anon_sym_const] = ACTIONS(2345),
    [anon_sym_if] = ACTIONS(2345),
    [anon_sym_switch] = ACTIONS(2345),
    [anon_sym_for] = ACTIONS(2345),
    [anon_sym_LPAREN] = ACTIONS(2343),
    [anon_sym_while] = ACTIONS(2345),
    [anon_sym_do] = ACTIONS(2345),
    [anon_sym_try] = ACTIONS(2345),
    [anon_sym_with] = ACTIONS(2345),
    [anon_sym_break] = ACTIONS(2345),
    [anon_sym_continue] = ACTIONS(2345),
    [anon_sym_debugger] = ACTIONS(2345),
    [anon_sym_return] = ACTIONS(2345),
    [anon_sym_throw] = ACTIONS(2345),
    [anon_sym_SEMI] = ACTIONS(2343),
    [anon_sym_case] = ACTIONS(2345),
    [anon_sym_yield] = ACTIONS(2345),
    [anon_sym_LBRACK] = ACTIONS(2343),
    [anon_sym_LT] = ACTIONS(2343),
    [anon_sym_class] = ACTIONS(2345),
    [anon_sym_async] = ACTIONS(2345),
    [anon_sym_function] = ACTIONS(2345),
    [anon_sym_EQ_GT] = ACTIONS(2343),
    [anon_sym_new] = ACTIONS(2345),
    [anon_sym_await] = ACTIONS(2345),
    [anon_sym_PLUS] = ACTIONS(2345),
    [anon_sym_DASH] = ACTIONS(2345),
    [anon_sym_BANG] = ACTIONS(2343),
    [anon_sym_TILDE] = ACTIONS(2343),
    [anon_sym_typeof] = ACTIONS(2345),
    [anon_sym_void] = ACTIONS(2345),
    [anon_sym_delete] = ACTIONS(2345),
    [anon_sym_PLUS_PLUS] = ACTIONS(2343),
    [anon_sym_DASH_DASH] = ACTIONS(2343),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2343),
    [anon_sym_BQUOTE] = ACTIONS(2343),
    [sym_regex] = ACTIONS(2345),
    [sym_number] = ACTIONS(2343),
    [sym_identifier] = ACTIONS(2345),
    [sym_this] = ACTIONS(2345),
    [sym_super] = ACTIONS(2345),
    [sym_true] = ACTIONS(2345),
    [sym_false] = ACTIONS(2345),
    [sym_null] = ACTIONS(2345),
    [sym_undefined] = ACTIONS(2345),
    [anon_sym_AT] = ACTIONS(2343),
    [anon_sym_get] = ACTIONS(2345),
    [anon_sym_set] = ACTIONS(2345),
  },
  [2205] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(2328),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(4974),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2206] = {
    [sym_class_body] = STATE(2329),
    [anon_sym_LBRACE] = ACTIONS(4784),
    [sym_comment] = ACTIONS(71),
  },
  [2207] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_default] = ACTIONS(2142),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_case] = ACTIONS(2142),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2140),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2140),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [2208] = {
    [sym_formal_parameters] = STATE(2330),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2209] = {
    [sym_statement_block] = STATE(2331),
    [anon_sym_LBRACE] = ACTIONS(4790),
    [sym_comment] = ACTIONS(71),
  },
  [2210] = {
    [sym_formal_parameters] = STATE(2332),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2211] = {
    [sym_statement_block] = STATE(2333),
    [anon_sym_LBRACE] = ACTIONS(4790),
    [sym_comment] = ACTIONS(71),
  },
  [2212] = {
    [sym_export_statement] = STATE(2335),
    [sym__declaration] = STATE(2335),
    [sym_import_statement] = STATE(2335),
    [sym_expression_statement] = STATE(2335),
    [sym_variable_declaration] = STATE(2335),
    [sym_lexical_declaration] = STATE(2335),
    [sym_statement_block] = STATE(2335),
    [sym_if_statement] = STATE(2335),
    [sym_switch_statement] = STATE(2335),
    [sym_for_statement] = STATE(2335),
    [sym_for_in_statement] = STATE(2335),
    [sym_for_of_statement] = STATE(2335),
    [sym_while_statement] = STATE(2335),
    [sym_do_statement] = STATE(2335),
    [sym_try_statement] = STATE(2335),
    [sym_with_statement] = STATE(2335),
    [sym_break_statement] = STATE(2335),
    [sym_continue_statement] = STATE(2335),
    [sym_debugger_statement] = STATE(2335),
    [sym_return_statement] = STATE(2335),
    [sym_throw_statement] = STATE(2335),
    [sym_empty_statement] = STATE(2335),
    [sym_labeled_statement] = STATE(2335),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2335),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4976),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2213] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_default] = ACTIONS(2180),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_case] = ACTIONS(2180),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2178),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2178),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [2214] = {
    [sym_class_heritage] = STATE(2336),
    [sym_class_body] = STATE(2329),
    [anon_sym_LBRACE] = ACTIONS(4784),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2215] = {
    [anon_sym_export] = ACTIONS(2609),
    [anon_sym_default] = ACTIONS(2609),
    [anon_sym_LBRACE] = ACTIONS(2607),
    [anon_sym_RBRACE] = ACTIONS(2607),
    [anon_sym_import] = ACTIONS(2609),
    [anon_sym_var] = ACTIONS(2609),
    [anon_sym_let] = ACTIONS(2609),
    [anon_sym_const] = ACTIONS(2609),
    [anon_sym_if] = ACTIONS(2609),
    [anon_sym_switch] = ACTIONS(2609),
    [anon_sym_for] = ACTIONS(2609),
    [anon_sym_LPAREN] = ACTIONS(2607),
    [anon_sym_while] = ACTIONS(2609),
    [anon_sym_do] = ACTIONS(2609),
    [anon_sym_try] = ACTIONS(2609),
    [anon_sym_with] = ACTIONS(2609),
    [anon_sym_break] = ACTIONS(2609),
    [anon_sym_continue] = ACTIONS(2609),
    [anon_sym_debugger] = ACTIONS(2609),
    [anon_sym_return] = ACTIONS(2609),
    [anon_sym_throw] = ACTIONS(2609),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_case] = ACTIONS(2609),
    [anon_sym_yield] = ACTIONS(2609),
    [anon_sym_LBRACK] = ACTIONS(2607),
    [anon_sym_LT] = ACTIONS(2607),
    [anon_sym_class] = ACTIONS(2609),
    [anon_sym_async] = ACTIONS(2609),
    [anon_sym_function] = ACTIONS(2609),
    [anon_sym_EQ_GT] = ACTIONS(2607),
    [anon_sym_new] = ACTIONS(2609),
    [anon_sym_await] = ACTIONS(2609),
    [anon_sym_PLUS] = ACTIONS(2609),
    [anon_sym_DASH] = ACTIONS(2609),
    [anon_sym_BANG] = ACTIONS(2607),
    [anon_sym_TILDE] = ACTIONS(2607),
    [anon_sym_typeof] = ACTIONS(2609),
    [anon_sym_void] = ACTIONS(2609),
    [anon_sym_delete] = ACTIONS(2609),
    [anon_sym_PLUS_PLUS] = ACTIONS(2607),
    [anon_sym_DASH_DASH] = ACTIONS(2607),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2607),
    [anon_sym_BQUOTE] = ACTIONS(2607),
    [sym_regex] = ACTIONS(2609),
    [sym_number] = ACTIONS(2607),
    [sym_identifier] = ACTIONS(2609),
    [sym_this] = ACTIONS(2609),
    [sym_super] = ACTIONS(2609),
    [sym_true] = ACTIONS(2609),
    [sym_false] = ACTIONS(2609),
    [sym_null] = ACTIONS(2609),
    [sym_undefined] = ACTIONS(2609),
    [anon_sym_AT] = ACTIONS(2607),
    [anon_sym_get] = ACTIONS(2609),
    [anon_sym_set] = ACTIONS(2609),
  },
  [2216] = {
    [anon_sym_export] = ACTIONS(2621),
    [anon_sym_default] = ACTIONS(2621),
    [anon_sym_LBRACE] = ACTIONS(2619),
    [anon_sym_RBRACE] = ACTIONS(2619),
    [anon_sym_import] = ACTIONS(2621),
    [anon_sym_var] = ACTIONS(2621),
    [anon_sym_let] = ACTIONS(2621),
    [anon_sym_const] = ACTIONS(2621),
    [anon_sym_if] = ACTIONS(2621),
    [anon_sym_switch] = ACTIONS(2621),
    [anon_sym_for] = ACTIONS(2621),
    [anon_sym_LPAREN] = ACTIONS(2619),
    [anon_sym_while] = ACTIONS(2621),
    [anon_sym_do] = ACTIONS(2621),
    [anon_sym_try] = ACTIONS(2621),
    [anon_sym_with] = ACTIONS(2621),
    [anon_sym_break] = ACTIONS(2621),
    [anon_sym_continue] = ACTIONS(2621),
    [anon_sym_debugger] = ACTIONS(2621),
    [anon_sym_return] = ACTIONS(2621),
    [anon_sym_throw] = ACTIONS(2621),
    [anon_sym_SEMI] = ACTIONS(2619),
    [anon_sym_case] = ACTIONS(2621),
    [anon_sym_yield] = ACTIONS(2621),
    [anon_sym_LBRACK] = ACTIONS(2619),
    [anon_sym_LT] = ACTIONS(2619),
    [anon_sym_class] = ACTIONS(2621),
    [anon_sym_async] = ACTIONS(2621),
    [anon_sym_function] = ACTIONS(2621),
    [anon_sym_EQ_GT] = ACTIONS(2619),
    [anon_sym_new] = ACTIONS(2621),
    [anon_sym_await] = ACTIONS(2621),
    [anon_sym_PLUS] = ACTIONS(2621),
    [anon_sym_DASH] = ACTIONS(2621),
    [anon_sym_BANG] = ACTIONS(2619),
    [anon_sym_TILDE] = ACTIONS(2619),
    [anon_sym_typeof] = ACTIONS(2621),
    [anon_sym_void] = ACTIONS(2621),
    [anon_sym_delete] = ACTIONS(2621),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2619),
    [anon_sym_BQUOTE] = ACTIONS(2619),
    [sym_regex] = ACTIONS(2621),
    [sym_number] = ACTIONS(2619),
    [sym_identifier] = ACTIONS(2621),
    [sym_this] = ACTIONS(2621),
    [sym_super] = ACTIONS(2621),
    [sym_true] = ACTIONS(2621),
    [sym_false] = ACTIONS(2621),
    [sym_null] = ACTIONS(2621),
    [sym_undefined] = ACTIONS(2621),
    [anon_sym_AT] = ACTIONS(2619),
    [anon_sym_get] = ACTIONS(2621),
    [anon_sym_set] = ACTIONS(2621),
  },
  [2217] = {
    [anon_sym_export] = ACTIONS(2628),
    [anon_sym_default] = ACTIONS(2628),
    [anon_sym_LBRACE] = ACTIONS(2626),
    [anon_sym_RBRACE] = ACTIONS(2626),
    [anon_sym_import] = ACTIONS(2628),
    [anon_sym_var] = ACTIONS(2628),
    [anon_sym_let] = ACTIONS(2628),
    [anon_sym_const] = ACTIONS(2628),
    [anon_sym_if] = ACTIONS(2628),
    [anon_sym_switch] = ACTIONS(2628),
    [anon_sym_for] = ACTIONS(2628),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2628),
    [anon_sym_do] = ACTIONS(2628),
    [anon_sym_try] = ACTIONS(2628),
    [anon_sym_with] = ACTIONS(2628),
    [anon_sym_break] = ACTIONS(2628),
    [anon_sym_continue] = ACTIONS(2628),
    [anon_sym_debugger] = ACTIONS(2628),
    [anon_sym_return] = ACTIONS(2628),
    [anon_sym_throw] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2628),
    [anon_sym_yield] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2628),
    [anon_sym_async] = ACTIONS(2628),
    [anon_sym_function] = ACTIONS(2628),
    [anon_sym_EQ_GT] = ACTIONS(2626),
    [anon_sym_new] = ACTIONS(2628),
    [anon_sym_await] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_DASH] = ACTIONS(2628),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2626),
    [anon_sym_typeof] = ACTIONS(2628),
    [anon_sym_void] = ACTIONS(2628),
    [anon_sym_delete] = ACTIONS(2628),
    [anon_sym_PLUS_PLUS] = ACTIONS(2626),
    [anon_sym_DASH_DASH] = ACTIONS(2626),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2626),
    [anon_sym_BQUOTE] = ACTIONS(2626),
    [sym_regex] = ACTIONS(2628),
    [sym_number] = ACTIONS(2626),
    [sym_identifier] = ACTIONS(2628),
    [sym_this] = ACTIONS(2628),
    [sym_super] = ACTIONS(2628),
    [sym_true] = ACTIONS(2628),
    [sym_false] = ACTIONS(2628),
    [sym_null] = ACTIONS(2628),
    [sym_undefined] = ACTIONS(2628),
    [anon_sym_AT] = ACTIONS(2626),
    [anon_sym_get] = ACTIONS(2628),
    [anon_sym_set] = ACTIONS(2628),
  },
  [2218] = {
    [sym__from_clause] = STATE(2337),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [2219] = {
    [sym_parenthesized_expression] = STATE(2338),
    [sym__expression] = STATE(2338),
    [sym_yield_expression] = STATE(2338),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(2338),
    [sym_jsx_fragment] = STATE(2338),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2338),
    [sym_anonymous_class] = STATE(2338),
    [sym_class] = STATE(2338),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2338),
    [sym_generator_function] = STATE(2338),
    [sym_call_expression] = STATE(2338),
    [sym_new_expression] = STATE(2338),
    [sym_await_expression] = STATE(2338),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(2338),
    [sym_augmented_assignment_expression] = STATE(2338),
    [sym_ternary_expression] = STATE(2338),
    [sym_binary_expression] = STATE(2338),
    [sym_unary_expression] = STATE(2338),
    [sym_update_expression] = STATE(2338),
    [sym_template_string] = STATE(2338),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4978),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(4980),
    [sym_number] = ACTIONS(4978),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(4980),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(4980),
    [sym_false] = ACTIONS(4980),
    [sym_null] = ACTIONS(4980),
    [sym_undefined] = ACTIONS(4980),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [2220] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4982),
  },
  [2221] = {
    [anon_sym_function] = ACTIONS(4984),
    [sym_comment] = ACTIONS(71),
  },
  [2222] = {
    [sym_formal_parameters] = STATE(2343),
    [anon_sym_STAR] = ACTIONS(4986),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4988),
  },
  [2223] = {
    [sym__from_clause] = STATE(2337),
    [sym__automatic_semicolon] = ACTIONS(4990),
    [anon_sym_from] = ACTIONS(410),
    [anon_sym_SEMI] = ACTIONS(4990),
    [sym_comment] = ACTIONS(71),
  },
  [2224] = {
    [anon_sym_export] = ACTIONS(454),
    [anon_sym_default] = ACTIONS(454),
    [anon_sym_LBRACE] = ACTIONS(452),
    [anon_sym_RBRACE] = ACTIONS(452),
    [anon_sym_import] = ACTIONS(454),
    [anon_sym_var] = ACTIONS(454),
    [anon_sym_let] = ACTIONS(454),
    [anon_sym_const] = ACTIONS(454),
    [anon_sym_if] = ACTIONS(454),
    [anon_sym_else] = ACTIONS(454),
    [anon_sym_switch] = ACTIONS(454),
    [anon_sym_for] = ACTIONS(454),
    [anon_sym_LPAREN] = ACTIONS(452),
    [anon_sym_while] = ACTIONS(454),
    [anon_sym_do] = ACTIONS(454),
    [anon_sym_try] = ACTIONS(454),
    [anon_sym_with] = ACTIONS(454),
    [anon_sym_break] = ACTIONS(454),
    [anon_sym_continue] = ACTIONS(454),
    [anon_sym_debugger] = ACTIONS(454),
    [anon_sym_return] = ACTIONS(454),
    [anon_sym_throw] = ACTIONS(454),
    [anon_sym_SEMI] = ACTIONS(452),
    [anon_sym_case] = ACTIONS(454),
    [anon_sym_yield] = ACTIONS(454),
    [anon_sym_LBRACK] = ACTIONS(452),
    [anon_sym_LT] = ACTIONS(452),
    [anon_sym_class] = ACTIONS(454),
    [anon_sym_async] = ACTIONS(454),
    [anon_sym_function] = ACTIONS(454),
    [anon_sym_EQ_GT] = ACTIONS(452),
    [anon_sym_new] = ACTIONS(454),
    [anon_sym_await] = ACTIONS(454),
    [anon_sym_PLUS] = ACTIONS(454),
    [anon_sym_DASH] = ACTIONS(454),
    [anon_sym_BANG] = ACTIONS(452),
    [anon_sym_TILDE] = ACTIONS(452),
    [anon_sym_typeof] = ACTIONS(454),
    [anon_sym_void] = ACTIONS(454),
    [anon_sym_delete] = ACTIONS(454),
    [anon_sym_PLUS_PLUS] = ACTIONS(452),
    [anon_sym_DASH_DASH] = ACTIONS(452),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(452),
    [anon_sym_BQUOTE] = ACTIONS(452),
    [sym_regex] = ACTIONS(454),
    [sym_number] = ACTIONS(452),
    [sym_identifier] = ACTIONS(454),
    [sym_this] = ACTIONS(454),
    [sym_super] = ACTIONS(454),
    [sym_true] = ACTIONS(454),
    [sym_false] = ACTIONS(454),
    [sym_null] = ACTIONS(454),
    [sym_undefined] = ACTIONS(454),
    [anon_sym_AT] = ACTIONS(452),
    [anon_sym_get] = ACTIONS(454),
    [anon_sym_set] = ACTIONS(454),
  },
  [2225] = {
    [sym__automatic_semicolon] = ACTIONS(4848),
    [anon_sym_export] = ACTIONS(402),
    [anon_sym_default] = ACTIONS(402),
    [anon_sym_LBRACE] = ACTIONS(400),
    [anon_sym_RBRACE] = ACTIONS(400),
    [anon_sym_import] = ACTIONS(402),
    [anon_sym_var] = ACTIONS(402),
    [anon_sym_let] = ACTIONS(402),
    [anon_sym_const] = ACTIONS(402),
    [anon_sym_if] = ACTIONS(402),
    [anon_sym_else] = ACTIONS(402),
    [anon_sym_switch] = ACTIONS(402),
    [anon_sym_for] = ACTIONS(402),
    [anon_sym_LPAREN] = ACTIONS(400),
    [anon_sym_while] = ACTIONS(402),
    [anon_sym_do] = ACTIONS(402),
    [anon_sym_try] = ACTIONS(402),
    [anon_sym_with] = ACTIONS(402),
    [anon_sym_break] = ACTIONS(402),
    [anon_sym_continue] = ACTIONS(402),
    [anon_sym_debugger] = ACTIONS(402),
    [anon_sym_return] = ACTIONS(402),
    [anon_sym_throw] = ACTIONS(402),
    [anon_sym_SEMI] = ACTIONS(400),
    [anon_sym_case] = ACTIONS(402),
    [anon_sym_yield] = ACTIONS(402),
    [anon_sym_LBRACK] = ACTIONS(400),
    [anon_sym_LT] = ACTIONS(400),
    [anon_sym_class] = ACTIONS(402),
    [anon_sym_async] = ACTIONS(402),
    [anon_sym_function] = ACTIONS(402),
    [anon_sym_EQ_GT] = ACTIONS(400),
    [anon_sym_new] = ACTIONS(402),
    [anon_sym_await] = ACTIONS(402),
    [anon_sym_PLUS] = ACTIONS(402),
    [anon_sym_DASH] = ACTIONS(402),
    [anon_sym_BANG] = ACTIONS(400),
    [anon_sym_TILDE] = ACTIONS(400),
    [anon_sym_typeof] = ACTIONS(402),
    [anon_sym_void] = ACTIONS(402),
    [anon_sym_delete] = ACTIONS(402),
    [anon_sym_PLUS_PLUS] = ACTIONS(400),
    [anon_sym_DASH_DASH] = ACTIONS(400),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(400),
    [anon_sym_BQUOTE] = ACTIONS(400),
    [sym_regex] = ACTIONS(402),
    [sym_number] = ACTIONS(400),
    [sym_identifier] = ACTIONS(402),
    [sym_this] = ACTIONS(402),
    [sym_super] = ACTIONS(402),
    [sym_true] = ACTIONS(402),
    [sym_false] = ACTIONS(402),
    [sym_null] = ACTIONS(402),
    [sym_undefined] = ACTIONS(402),
    [anon_sym_AT] = ACTIONS(400),
    [anon_sym_get] = ACTIONS(402),
    [anon_sym_set] = ACTIONS(402),
  },
  [2226] = {
    [sym_decorator] = STATE(269),
    [aux_sym_export_statement_repeat1] = STATE(269),
    [anon_sym_class] = ACTIONS(4992),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [2227] = {
    [sym__automatic_semicolon] = ACTIONS(480),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(480),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(480),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(480),
    [anon_sym_in] = ACTIONS(486),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(480),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(480),
    [anon_sym_LT] = ACTIONS(486),
    [anon_sym_GT] = ACTIONS(486),
    [anon_sym_SLASH] = ACTIONS(486),
    [anon_sym_DOT] = ACTIONS(486),
    [anon_sym_EQ] = ACTIONS(486),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(480),
    [anon_sym_AMP_AMP] = ACTIONS(480),
    [anon_sym_PIPE_PIPE] = ACTIONS(480),
    [anon_sym_GT_GT] = ACTIONS(486),
    [anon_sym_GT_GT_GT] = ACTIONS(480),
    [anon_sym_LT_LT] = ACTIONS(480),
    [anon_sym_AMP] = ACTIONS(486),
    [anon_sym_CARET] = ACTIONS(480),
    [anon_sym_PIPE] = ACTIONS(486),
    [anon_sym_PLUS] = ACTIONS(486),
    [anon_sym_DASH] = ACTIONS(486),
    [anon_sym_PERCENT] = ACTIONS(480),
    [anon_sym_LT_EQ] = ACTIONS(480),
    [anon_sym_EQ_EQ] = ACTIONS(486),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(480),
    [anon_sym_BANG_EQ] = ACTIONS(486),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(480),
    [anon_sym_GT_EQ] = ACTIONS(480),
    [anon_sym_instanceof] = ACTIONS(486),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(480),
    [anon_sym_DASH_DASH] = ACTIONS(480),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(480),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2228] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(4994),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2229] = {
    [sym__automatic_semicolon] = ACTIONS(4996),
    [anon_sym_SEMI] = ACTIONS(4996),
    [sym_comment] = ACTIONS(71),
  },
  [2230] = {
    [sym__from_clause] = STATE(2348),
    [anon_sym_from] = ACTIONS(410),
    [sym_comment] = ACTIONS(71),
  },
  [2231] = {
    [aux_sym_variable_declaration_repeat1] = STATE(2350),
    [sym__automatic_semicolon] = ACTIONS(4998),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(4998),
    [sym_comment] = ACTIONS(71),
  },
  [2232] = {
    [aux_sym_variable_declaration_repeat1] = STATE(2352),
    [sym__automatic_semicolon] = ACTIONS(5000),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(5000),
    [sym_comment] = ACTIONS(71),
  },
  [2233] = {
    [sym_export_statement] = STATE(2353),
    [sym__declaration] = STATE(2353),
    [sym_import_statement] = STATE(2353),
    [sym_expression_statement] = STATE(2353),
    [sym_variable_declaration] = STATE(2353),
    [sym_lexical_declaration] = STATE(2353),
    [sym_statement_block] = STATE(2353),
    [sym_if_statement] = STATE(2353),
    [sym_switch_statement] = STATE(2353),
    [sym_for_statement] = STATE(2353),
    [sym_for_in_statement] = STATE(2353),
    [sym_for_of_statement] = STATE(2353),
    [sym_while_statement] = STATE(2353),
    [sym_do_statement] = STATE(2353),
    [sym_try_statement] = STATE(2353),
    [sym_with_statement] = STATE(2353),
    [sym_break_statement] = STATE(2353),
    [sym_continue_statement] = STATE(2353),
    [sym_debugger_statement] = STATE(2353),
    [sym_return_statement] = STATE(2353),
    [sym_throw_statement] = STATE(2353),
    [sym_empty_statement] = STATE(2353),
    [sym_labeled_statement] = STATE(2353),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2234] = {
    [sym_switch_body] = STATE(2355),
    [anon_sym_LBRACE] = ACTIONS(5002),
    [sym_comment] = ACTIONS(71),
  },
  [2235] = {
    [sym_expression_statement] = STATE(2359),
    [sym_variable_declaration] = STATE(2359),
    [sym_lexical_declaration] = STATE(2359),
    [sym_empty_statement] = STATE(2359),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(2360),
    [sym_array] = STATE(2361),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_var] = ACTIONS(5004),
    [anon_sym_let] = ACTIONS(5006),
    [anon_sym_const] = ACTIONS(5006),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(5008),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2236] = {
    [sym_export_statement] = STATE(2362),
    [sym__declaration] = STATE(2362),
    [sym_import_statement] = STATE(2362),
    [sym_expression_statement] = STATE(2362),
    [sym_variable_declaration] = STATE(2362),
    [sym_lexical_declaration] = STATE(2362),
    [sym_statement_block] = STATE(2362),
    [sym_if_statement] = STATE(2362),
    [sym_switch_statement] = STATE(2362),
    [sym_for_statement] = STATE(2362),
    [sym_for_in_statement] = STATE(2362),
    [sym_for_of_statement] = STATE(2362),
    [sym_while_statement] = STATE(2362),
    [sym_do_statement] = STATE(2362),
    [sym_try_statement] = STATE(2362),
    [sym_with_statement] = STATE(2362),
    [sym_break_statement] = STATE(2362),
    [sym_continue_statement] = STATE(2362),
    [sym_debugger_statement] = STATE(2362),
    [sym_return_statement] = STATE(2362),
    [sym_throw_statement] = STATE(2362),
    [sym_empty_statement] = STATE(2362),
    [sym_labeled_statement] = STATE(2362),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2237] = {
    [anon_sym_while] = ACTIONS(5010),
    [sym_comment] = ACTIONS(71),
  },
  [2238] = {
    [sym_export_statement] = STATE(2365),
    [sym__declaration] = STATE(2365),
    [sym_import_statement] = STATE(2365),
    [sym_expression_statement] = STATE(2365),
    [sym_variable_declaration] = STATE(2365),
    [sym_lexical_declaration] = STATE(2365),
    [sym_statement_block] = STATE(2365),
    [sym_if_statement] = STATE(2365),
    [sym_switch_statement] = STATE(2365),
    [sym_for_statement] = STATE(2365),
    [sym_for_in_statement] = STATE(2365),
    [sym_for_of_statement] = STATE(2365),
    [sym_while_statement] = STATE(2365),
    [sym_do_statement] = STATE(2365),
    [sym_try_statement] = STATE(2365),
    [sym_with_statement] = STATE(2365),
    [sym_break_statement] = STATE(2365),
    [sym_continue_statement] = STATE(2365),
    [sym_debugger_statement] = STATE(2365),
    [sym_return_statement] = STATE(2365),
    [sym_throw_statement] = STATE(2365),
    [sym_empty_statement] = STATE(2365),
    [sym_labeled_statement] = STATE(2365),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2365),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5012),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2239] = {
    [sym_catch_clause] = STATE(2368),
    [sym_finally_clause] = STATE(2369),
    [anon_sym_export] = ACTIONS(844),
    [anon_sym_default] = ACTIONS(844),
    [anon_sym_LBRACE] = ACTIONS(842),
    [anon_sym_RBRACE] = ACTIONS(842),
    [anon_sym_import] = ACTIONS(844),
    [anon_sym_var] = ACTIONS(844),
    [anon_sym_let] = ACTIONS(844),
    [anon_sym_const] = ACTIONS(844),
    [anon_sym_if] = ACTIONS(844),
    [anon_sym_else] = ACTIONS(844),
    [anon_sym_switch] = ACTIONS(844),
    [anon_sym_for] = ACTIONS(844),
    [anon_sym_LPAREN] = ACTIONS(842),
    [anon_sym_while] = ACTIONS(844),
    [anon_sym_do] = ACTIONS(844),
    [anon_sym_try] = ACTIONS(844),
    [anon_sym_with] = ACTIONS(844),
    [anon_sym_break] = ACTIONS(844),
    [anon_sym_continue] = ACTIONS(844),
    [anon_sym_debugger] = ACTIONS(844),
    [anon_sym_return] = ACTIONS(844),
    [anon_sym_throw] = ACTIONS(844),
    [anon_sym_SEMI] = ACTIONS(842),
    [anon_sym_case] = ACTIONS(844),
    [anon_sym_catch] = ACTIONS(5014),
    [anon_sym_finally] = ACTIONS(5016),
    [anon_sym_yield] = ACTIONS(844),
    [anon_sym_LBRACK] = ACTIONS(842),
    [anon_sym_LT] = ACTIONS(842),
    [anon_sym_class] = ACTIONS(844),
    [anon_sym_async] = ACTIONS(844),
    [anon_sym_function] = ACTIONS(844),
    [anon_sym_EQ_GT] = ACTIONS(842),
    [anon_sym_new] = ACTIONS(844),
    [anon_sym_await] = ACTIONS(844),
    [anon_sym_PLUS] = ACTIONS(844),
    [anon_sym_DASH] = ACTIONS(844),
    [anon_sym_BANG] = ACTIONS(842),
    [anon_sym_TILDE] = ACTIONS(842),
    [anon_sym_typeof] = ACTIONS(844),
    [anon_sym_void] = ACTIONS(844),
    [anon_sym_delete] = ACTIONS(844),
    [anon_sym_PLUS_PLUS] = ACTIONS(842),
    [anon_sym_DASH_DASH] = ACTIONS(842),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(842),
    [anon_sym_BQUOTE] = ACTIONS(842),
    [sym_regex] = ACTIONS(844),
    [sym_number] = ACTIONS(842),
    [sym_identifier] = ACTIONS(844),
    [sym_this] = ACTIONS(844),
    [sym_super] = ACTIONS(844),
    [sym_true] = ACTIONS(844),
    [sym_false] = ACTIONS(844),
    [sym_null] = ACTIONS(844),
    [sym_undefined] = ACTIONS(844),
    [anon_sym_AT] = ACTIONS(842),
    [anon_sym_get] = ACTIONS(844),
    [anon_sym_set] = ACTIONS(844),
  },
  [2240] = {
    [sym_export_statement] = STATE(2370),
    [sym__declaration] = STATE(2370),
    [sym_import_statement] = STATE(2370),
    [sym_expression_statement] = STATE(2370),
    [sym_variable_declaration] = STATE(2370),
    [sym_lexical_declaration] = STATE(2370),
    [sym_statement_block] = STATE(2370),
    [sym_if_statement] = STATE(2370),
    [sym_switch_statement] = STATE(2370),
    [sym_for_statement] = STATE(2370),
    [sym_for_in_statement] = STATE(2370),
    [sym_for_of_statement] = STATE(2370),
    [sym_while_statement] = STATE(2370),
    [sym_do_statement] = STATE(2370),
    [sym_try_statement] = STATE(2370),
    [sym_with_statement] = STATE(2370),
    [sym_break_statement] = STATE(2370),
    [sym_continue_statement] = STATE(2370),
    [sym_debugger_statement] = STATE(2370),
    [sym_return_statement] = STATE(2370),
    [sym_throw_statement] = STATE(2370),
    [sym_empty_statement] = STATE(2370),
    [sym_labeled_statement] = STATE(2370),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2241] = {
    [anon_sym_export] = ACTIONS(852),
    [anon_sym_default] = ACTIONS(852),
    [anon_sym_LBRACE] = ACTIONS(850),
    [anon_sym_RBRACE] = ACTIONS(850),
    [anon_sym_import] = ACTIONS(852),
    [anon_sym_var] = ACTIONS(852),
    [anon_sym_let] = ACTIONS(852),
    [anon_sym_const] = ACTIONS(852),
    [anon_sym_if] = ACTIONS(852),
    [anon_sym_else] = ACTIONS(852),
    [anon_sym_switch] = ACTIONS(852),
    [anon_sym_for] = ACTIONS(852),
    [anon_sym_LPAREN] = ACTIONS(850),
    [anon_sym_while] = ACTIONS(852),
    [anon_sym_do] = ACTIONS(852),
    [anon_sym_try] = ACTIONS(852),
    [anon_sym_with] = ACTIONS(852),
    [anon_sym_break] = ACTIONS(852),
    [anon_sym_continue] = ACTIONS(852),
    [anon_sym_debugger] = ACTIONS(852),
    [anon_sym_return] = ACTIONS(852),
    [anon_sym_throw] = ACTIONS(852),
    [anon_sym_SEMI] = ACTIONS(850),
    [anon_sym_case] = ACTIONS(852),
    [anon_sym_yield] = ACTIONS(852),
    [anon_sym_LBRACK] = ACTIONS(850),
    [anon_sym_LT] = ACTIONS(850),
    [anon_sym_class] = ACTIONS(852),
    [anon_sym_async] = ACTIONS(852),
    [anon_sym_function] = ACTIONS(852),
    [anon_sym_EQ_GT] = ACTIONS(850),
    [anon_sym_new] = ACTIONS(852),
    [anon_sym_await] = ACTIONS(852),
    [anon_sym_PLUS] = ACTIONS(852),
    [anon_sym_DASH] = ACTIONS(852),
    [anon_sym_BANG] = ACTIONS(850),
    [anon_sym_TILDE] = ACTIONS(850),
    [anon_sym_typeof] = ACTIONS(852),
    [anon_sym_void] = ACTIONS(852),
    [anon_sym_delete] = ACTIONS(852),
    [anon_sym_PLUS_PLUS] = ACTIONS(850),
    [anon_sym_DASH_DASH] = ACTIONS(850),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(850),
    [anon_sym_BQUOTE] = ACTIONS(850),
    [sym_regex] = ACTIONS(852),
    [sym_number] = ACTIONS(850),
    [sym_identifier] = ACTIONS(852),
    [sym_this] = ACTIONS(852),
    [sym_super] = ACTIONS(852),
    [sym_true] = ACTIONS(852),
    [sym_false] = ACTIONS(852),
    [sym_null] = ACTIONS(852),
    [sym_undefined] = ACTIONS(852),
    [anon_sym_AT] = ACTIONS(850),
    [anon_sym_get] = ACTIONS(852),
    [anon_sym_set] = ACTIONS(852),
  },
  [2242] = {
    [sym__automatic_semicolon] = ACTIONS(5018),
    [anon_sym_SEMI] = ACTIONS(5018),
    [sym_comment] = ACTIONS(71),
  },
  [2243] = {
    [anon_sym_export] = ACTIONS(858),
    [anon_sym_default] = ACTIONS(858),
    [anon_sym_LBRACE] = ACTIONS(856),
    [anon_sym_RBRACE] = ACTIONS(856),
    [anon_sym_import] = ACTIONS(858),
    [anon_sym_var] = ACTIONS(858),
    [anon_sym_let] = ACTIONS(858),
    [anon_sym_const] = ACTIONS(858),
    [anon_sym_if] = ACTIONS(858),
    [anon_sym_else] = ACTIONS(858),
    [anon_sym_switch] = ACTIONS(858),
    [anon_sym_for] = ACTIONS(858),
    [anon_sym_LPAREN] = ACTIONS(856),
    [anon_sym_while] = ACTIONS(858),
    [anon_sym_do] = ACTIONS(858),
    [anon_sym_try] = ACTIONS(858),
    [anon_sym_with] = ACTIONS(858),
    [anon_sym_break] = ACTIONS(858),
    [anon_sym_continue] = ACTIONS(858),
    [anon_sym_debugger] = ACTIONS(858),
    [anon_sym_return] = ACTIONS(858),
    [anon_sym_throw] = ACTIONS(858),
    [anon_sym_SEMI] = ACTIONS(856),
    [anon_sym_case] = ACTIONS(858),
    [anon_sym_yield] = ACTIONS(858),
    [anon_sym_LBRACK] = ACTIONS(856),
    [anon_sym_LT] = ACTIONS(856),
    [anon_sym_class] = ACTIONS(858),
    [anon_sym_async] = ACTIONS(858),
    [anon_sym_function] = ACTIONS(858),
    [anon_sym_EQ_GT] = ACTIONS(856),
    [anon_sym_new] = ACTIONS(858),
    [anon_sym_await] = ACTIONS(858),
    [anon_sym_PLUS] = ACTIONS(858),
    [anon_sym_DASH] = ACTIONS(858),
    [anon_sym_BANG] = ACTIONS(856),
    [anon_sym_TILDE] = ACTIONS(856),
    [anon_sym_typeof] = ACTIONS(858),
    [anon_sym_void] = ACTIONS(858),
    [anon_sym_delete] = ACTIONS(858),
    [anon_sym_PLUS_PLUS] = ACTIONS(856),
    [anon_sym_DASH_DASH] = ACTIONS(856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(856),
    [anon_sym_BQUOTE] = ACTIONS(856),
    [sym_regex] = ACTIONS(858),
    [sym_number] = ACTIONS(856),
    [sym_identifier] = ACTIONS(858),
    [sym_this] = ACTIONS(858),
    [sym_super] = ACTIONS(858),
    [sym_true] = ACTIONS(858),
    [sym_false] = ACTIONS(858),
    [sym_null] = ACTIONS(858),
    [sym_undefined] = ACTIONS(858),
    [anon_sym_AT] = ACTIONS(856),
    [anon_sym_get] = ACTIONS(858),
    [anon_sym_set] = ACTIONS(858),
  },
  [2244] = {
    [sym__automatic_semicolon] = ACTIONS(5020),
    [anon_sym_SEMI] = ACTIONS(5020),
    [sym_comment] = ACTIONS(71),
  },
  [2245] = {
    [anon_sym_export] = ACTIONS(864),
    [anon_sym_default] = ACTIONS(864),
    [anon_sym_LBRACE] = ACTIONS(862),
    [anon_sym_RBRACE] = ACTIONS(862),
    [anon_sym_import] = ACTIONS(864),
    [anon_sym_var] = ACTIONS(864),
    [anon_sym_let] = ACTIONS(864),
    [anon_sym_const] = ACTIONS(864),
    [anon_sym_if] = ACTIONS(864),
    [anon_sym_else] = ACTIONS(864),
    [anon_sym_switch] = ACTIONS(864),
    [anon_sym_for] = ACTIONS(864),
    [anon_sym_LPAREN] = ACTIONS(862),
    [anon_sym_while] = ACTIONS(864),
    [anon_sym_do] = ACTIONS(864),
    [anon_sym_try] = ACTIONS(864),
    [anon_sym_with] = ACTIONS(864),
    [anon_sym_break] = ACTIONS(864),
    [anon_sym_continue] = ACTIONS(864),
    [anon_sym_debugger] = ACTIONS(864),
    [anon_sym_return] = ACTIONS(864),
    [anon_sym_throw] = ACTIONS(864),
    [anon_sym_SEMI] = ACTIONS(862),
    [anon_sym_case] = ACTIONS(864),
    [anon_sym_yield] = ACTIONS(864),
    [anon_sym_LBRACK] = ACTIONS(862),
    [anon_sym_LT] = ACTIONS(862),
    [anon_sym_class] = ACTIONS(864),
    [anon_sym_async] = ACTIONS(864),
    [anon_sym_function] = ACTIONS(864),
    [anon_sym_EQ_GT] = ACTIONS(862),
    [anon_sym_new] = ACTIONS(864),
    [anon_sym_await] = ACTIONS(864),
    [anon_sym_PLUS] = ACTIONS(864),
    [anon_sym_DASH] = ACTIONS(864),
    [anon_sym_BANG] = ACTIONS(862),
    [anon_sym_TILDE] = ACTIONS(862),
    [anon_sym_typeof] = ACTIONS(864),
    [anon_sym_void] = ACTIONS(864),
    [anon_sym_delete] = ACTIONS(864),
    [anon_sym_PLUS_PLUS] = ACTIONS(862),
    [anon_sym_DASH_DASH] = ACTIONS(862),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(862),
    [anon_sym_BQUOTE] = ACTIONS(862),
    [sym_regex] = ACTIONS(864),
    [sym_number] = ACTIONS(862),
    [sym_identifier] = ACTIONS(864),
    [sym_this] = ACTIONS(864),
    [sym_super] = ACTIONS(864),
    [sym_true] = ACTIONS(864),
    [sym_false] = ACTIONS(864),
    [sym_null] = ACTIONS(864),
    [sym_undefined] = ACTIONS(864),
    [anon_sym_AT] = ACTIONS(862),
    [anon_sym_get] = ACTIONS(864),
    [anon_sym_set] = ACTIONS(864),
  },
  [2246] = {
    [anon_sym_export] = ACTIONS(868),
    [anon_sym_default] = ACTIONS(868),
    [anon_sym_LBRACE] = ACTIONS(866),
    [anon_sym_RBRACE] = ACTIONS(866),
    [anon_sym_import] = ACTIONS(868),
    [anon_sym_var] = ACTIONS(868),
    [anon_sym_let] = ACTIONS(868),
    [anon_sym_const] = ACTIONS(868),
    [anon_sym_if] = ACTIONS(868),
    [anon_sym_else] = ACTIONS(868),
    [anon_sym_switch] = ACTIONS(868),
    [anon_sym_for] = ACTIONS(868),
    [anon_sym_LPAREN] = ACTIONS(866),
    [anon_sym_while] = ACTIONS(868),
    [anon_sym_do] = ACTIONS(868),
    [anon_sym_try] = ACTIONS(868),
    [anon_sym_with] = ACTIONS(868),
    [anon_sym_break] = ACTIONS(868),
    [anon_sym_continue] = ACTIONS(868),
    [anon_sym_debugger] = ACTIONS(868),
    [anon_sym_return] = ACTIONS(868),
    [anon_sym_throw] = ACTIONS(868),
    [anon_sym_SEMI] = ACTIONS(866),
    [anon_sym_case] = ACTIONS(868),
    [anon_sym_yield] = ACTIONS(868),
    [anon_sym_LBRACK] = ACTIONS(866),
    [anon_sym_LT] = ACTIONS(866),
    [anon_sym_class] = ACTIONS(868),
    [anon_sym_async] = ACTIONS(868),
    [anon_sym_function] = ACTIONS(868),
    [anon_sym_EQ_GT] = ACTIONS(866),
    [anon_sym_new] = ACTIONS(868),
    [anon_sym_await] = ACTIONS(868),
    [anon_sym_PLUS] = ACTIONS(868),
    [anon_sym_DASH] = ACTIONS(868),
    [anon_sym_BANG] = ACTIONS(866),
    [anon_sym_TILDE] = ACTIONS(866),
    [anon_sym_typeof] = ACTIONS(868),
    [anon_sym_void] = ACTIONS(868),
    [anon_sym_delete] = ACTIONS(868),
    [anon_sym_PLUS_PLUS] = ACTIONS(866),
    [anon_sym_DASH_DASH] = ACTIONS(866),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(866),
    [anon_sym_BQUOTE] = ACTIONS(866),
    [sym_regex] = ACTIONS(868),
    [sym_number] = ACTIONS(866),
    [sym_identifier] = ACTIONS(868),
    [sym_this] = ACTIONS(868),
    [sym_super] = ACTIONS(868),
    [sym_true] = ACTIONS(868),
    [sym_false] = ACTIONS(868),
    [sym_null] = ACTIONS(868),
    [sym_undefined] = ACTIONS(868),
    [anon_sym_AT] = ACTIONS(866),
    [anon_sym_get] = ACTIONS(868),
    [anon_sym_set] = ACTIONS(868),
  },
  [2247] = {
    [sym__automatic_semicolon] = ACTIONS(5022),
    [anon_sym_SEMI] = ACTIONS(5022),
    [sym_comment] = ACTIONS(71),
  },
  [2248] = {
    [sym__automatic_semicolon] = ACTIONS(5024),
    [anon_sym_SEMI] = ACTIONS(5024),
    [sym_comment] = ACTIONS(71),
  },
  [2249] = {
    [sym_class_heritage] = STATE(2376),
    [sym_class_body] = STATE(2377),
    [anon_sym_LBRACE] = ACTIONS(5026),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2250] = {
    [sym_formal_parameters] = STATE(2379),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5028),
  },
  [2251] = {
    [sym_formal_parameters] = STATE(2381),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5030),
  },
  [2252] = {
    [sym_formal_parameters] = STATE(2379),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2253] = {
    [sym_statement_block] = STATE(2383),
    [anon_sym_LBRACE] = ACTIONS(5032),
    [sym_comment] = ACTIONS(71),
  },
  [2254] = {
    [sym_export_statement] = STATE(2384),
    [sym__declaration] = STATE(2384),
    [sym_import_statement] = STATE(2384),
    [sym_expression_statement] = STATE(2384),
    [sym_variable_declaration] = STATE(2384),
    [sym_lexical_declaration] = STATE(2384),
    [sym_statement_block] = STATE(2384),
    [sym_if_statement] = STATE(2384),
    [sym_switch_statement] = STATE(2384),
    [sym_for_statement] = STATE(2384),
    [sym_for_in_statement] = STATE(2384),
    [sym_for_of_statement] = STATE(2384),
    [sym_while_statement] = STATE(2384),
    [sym_do_statement] = STATE(2384),
    [sym_try_statement] = STATE(2384),
    [sym_with_statement] = STATE(2384),
    [sym_break_statement] = STATE(2384),
    [sym_continue_statement] = STATE(2384),
    [sym_debugger_statement] = STATE(2384),
    [sym_return_statement] = STATE(2384),
    [sym_throw_statement] = STATE(2384),
    [sym_empty_statement] = STATE(2384),
    [sym_labeled_statement] = STATE(2384),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2255] = {
    [sym_export_statement] = STATE(2385),
    [sym__declaration] = STATE(2385),
    [sym_import_statement] = STATE(2385),
    [sym_expression_statement] = STATE(2385),
    [sym_variable_declaration] = STATE(2385),
    [sym_lexical_declaration] = STATE(2385),
    [sym_statement_block] = STATE(2385),
    [sym_if_statement] = STATE(2385),
    [sym_switch_statement] = STATE(2385),
    [sym_for_statement] = STATE(2385),
    [sym_for_in_statement] = STATE(2385),
    [sym_for_of_statement] = STATE(2385),
    [sym_while_statement] = STATE(2385),
    [sym_do_statement] = STATE(2385),
    [sym_try_statement] = STATE(2385),
    [sym_with_statement] = STATE(2385),
    [sym_break_statement] = STATE(2385),
    [sym_continue_statement] = STATE(2385),
    [sym_debugger_statement] = STATE(2385),
    [sym_return_statement] = STATE(2385),
    [sym_throw_statement] = STATE(2385),
    [sym_empty_statement] = STATE(2385),
    [sym_labeled_statement] = STATE(2385),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2256] = {
    [anon_sym_export] = ACTIONS(1124),
    [anon_sym_default] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_RBRACE] = ACTIONS(1122),
    [anon_sym_import] = ACTIONS(1124),
    [anon_sym_var] = ACTIONS(1124),
    [anon_sym_let] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_LPAREN] = ACTIONS(1122),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_try] = ACTIONS(1124),
    [anon_sym_with] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_debugger] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_throw] = ACTIONS(1124),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_yield] = ACTIONS(1124),
    [anon_sym_LBRACK] = ACTIONS(1122),
    [anon_sym_LT] = ACTIONS(1122),
    [anon_sym_class] = ACTIONS(1124),
    [anon_sym_async] = ACTIONS(1124),
    [anon_sym_function] = ACTIONS(1124),
    [anon_sym_EQ_GT] = ACTIONS(1122),
    [anon_sym_new] = ACTIONS(1124),
    [anon_sym_await] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_typeof] = ACTIONS(1124),
    [anon_sym_void] = ACTIONS(1124),
    [anon_sym_delete] = ACTIONS(1124),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1122),
    [anon_sym_BQUOTE] = ACTIONS(1122),
    [sym_regex] = ACTIONS(1124),
    [sym_number] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1124),
    [sym_this] = ACTIONS(1124),
    [sym_super] = ACTIONS(1124),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [sym_null] = ACTIONS(1124),
    [sym_undefined] = ACTIONS(1124),
    [anon_sym_AT] = ACTIONS(1122),
    [anon_sym_get] = ACTIONS(1124),
    [anon_sym_set] = ACTIONS(1124),
  },
  [2257] = {
    [anon_sym_export] = ACTIONS(1204),
    [anon_sym_default] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_RBRACE] = ACTIONS(1202),
    [anon_sym_import] = ACTIONS(1204),
    [anon_sym_var] = ACTIONS(1204),
    [anon_sym_let] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_else] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_LPAREN] = ACTIONS(1202),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_try] = ACTIONS(1204),
    [anon_sym_with] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_debugger] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_throw] = ACTIONS(1204),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym_case] = ACTIONS(1204),
    [anon_sym_yield] = ACTIONS(1204),
    [anon_sym_LBRACK] = ACTIONS(1202),
    [anon_sym_LT] = ACTIONS(1202),
    [anon_sym_class] = ACTIONS(1204),
    [anon_sym_async] = ACTIONS(1204),
    [anon_sym_function] = ACTIONS(1204),
    [anon_sym_EQ_GT] = ACTIONS(1202),
    [anon_sym_new] = ACTIONS(1204),
    [anon_sym_await] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_typeof] = ACTIONS(1204),
    [anon_sym_void] = ACTIONS(1204),
    [anon_sym_delete] = ACTIONS(1204),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1202),
    [anon_sym_BQUOTE] = ACTIONS(1202),
    [sym_regex] = ACTIONS(1204),
    [sym_number] = ACTIONS(1202),
    [sym_identifier] = ACTIONS(1204),
    [sym_this] = ACTIONS(1204),
    [sym_super] = ACTIONS(1204),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [sym_null] = ACTIONS(1204),
    [sym_undefined] = ACTIONS(1204),
    [anon_sym_AT] = ACTIONS(1202),
    [anon_sym_get] = ACTIONS(1204),
    [anon_sym_set] = ACTIONS(1204),
  },
  [2258] = {
    [sym__declaration] = STATE(2344),
    [sym_variable_declaration] = STATE(2344),
    [sym_lexical_declaration] = STATE(2344),
    [sym_class] = STATE(2225),
    [sym_function] = STATE(2225),
    [sym_generator_function] = STATE(2225),
    [sym_decorator] = STATE(2226),
    [aux_sym_export_statement_repeat1] = STATE(2226),
    [anon_sym_default] = ACTIONS(5034),
    [anon_sym_var] = ACTIONS(4804),
    [anon_sym_let] = ACTIONS(4806),
    [anon_sym_const] = ACTIONS(4806),
    [anon_sym_class] = ACTIONS(4808),
    [anon_sym_async] = ACTIONS(4810),
    [anon_sym_function] = ACTIONS(4812),
    [sym_comment] = ACTIONS(71),
    [anon_sym_AT] = ACTIONS(83),
  },
  [2259] = {
    [sym_class_heritage] = STATE(649),
    [sym_class_body] = STATE(574),
    [anon_sym_LBRACE] = ACTIONS(285),
    [anon_sym_extends] = ACTIONS(287),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5036),
  },
  [2260] = {
    [anon_sym_export] = ACTIONS(2694),
    [anon_sym_default] = ACTIONS(2694),
    [anon_sym_LBRACE] = ACTIONS(2692),
    [anon_sym_RBRACE] = ACTIONS(2692),
    [anon_sym_import] = ACTIONS(2694),
    [anon_sym_var] = ACTIONS(2694),
    [anon_sym_let] = ACTIONS(2694),
    [anon_sym_const] = ACTIONS(2694),
    [anon_sym_if] = ACTIONS(2694),
    [anon_sym_switch] = ACTIONS(2694),
    [anon_sym_for] = ACTIONS(2694),
    [anon_sym_LPAREN] = ACTIONS(2692),
    [anon_sym_while] = ACTIONS(2694),
    [anon_sym_do] = ACTIONS(2694),
    [anon_sym_try] = ACTIONS(2694),
    [anon_sym_with] = ACTIONS(2694),
    [anon_sym_break] = ACTIONS(2694),
    [anon_sym_continue] = ACTIONS(2694),
    [anon_sym_debugger] = ACTIONS(2694),
    [anon_sym_return] = ACTIONS(2694),
    [anon_sym_throw] = ACTIONS(2694),
    [anon_sym_SEMI] = ACTIONS(2692),
    [anon_sym_case] = ACTIONS(2694),
    [anon_sym_yield] = ACTIONS(2694),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_LT] = ACTIONS(2692),
    [anon_sym_class] = ACTIONS(2694),
    [anon_sym_async] = ACTIONS(2694),
    [anon_sym_function] = ACTIONS(2694),
    [anon_sym_EQ_GT] = ACTIONS(2692),
    [anon_sym_new] = ACTIONS(2694),
    [anon_sym_await] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2694),
    [anon_sym_DASH] = ACTIONS(2694),
    [anon_sym_BANG] = ACTIONS(2692),
    [anon_sym_TILDE] = ACTIONS(2692),
    [anon_sym_typeof] = ACTIONS(2694),
    [anon_sym_void] = ACTIONS(2694),
    [anon_sym_delete] = ACTIONS(2694),
    [anon_sym_PLUS_PLUS] = ACTIONS(2692),
    [anon_sym_DASH_DASH] = ACTIONS(2692),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2692),
    [anon_sym_BQUOTE] = ACTIONS(2692),
    [sym_regex] = ACTIONS(2694),
    [sym_number] = ACTIONS(2692),
    [sym_identifier] = ACTIONS(2694),
    [sym_this] = ACTIONS(2694),
    [sym_super] = ACTIONS(2694),
    [sym_true] = ACTIONS(2694),
    [sym_false] = ACTIONS(2694),
    [sym_null] = ACTIONS(2694),
    [sym_undefined] = ACTIONS(2694),
    [anon_sym_AT] = ACTIONS(2692),
    [anon_sym_get] = ACTIONS(2694),
    [anon_sym_set] = ACTIONS(2694),
  },
  [2261] = {
    [sym_switch_case] = STATE(1276),
    [sym_switch_default] = STATE(1276),
    [aux_sym_switch_body_repeat1] = STATE(1276),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(5038),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [2262] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(5040),
    [anon_sym_of] = ACTIONS(5042),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [2263] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(5044),
    [anon_sym_of] = ACTIONS(5046),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [2264] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(5048),
    [anon_sym_of] = ACTIONS(5050),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [2265] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2395),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2395),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2266] = {
    [sym_parenthesized_expression] = STATE(2396),
    [sym__expression] = STATE(2396),
    [sym_yield_expression] = STATE(2396),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2396),
    [sym_jsx_fragment] = STATE(2396),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2396),
    [sym_anonymous_class] = STATE(2396),
    [sym_class] = STATE(2396),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2396),
    [sym_generator_function] = STATE(2396),
    [sym_call_expression] = STATE(2396),
    [sym_new_expression] = STATE(2396),
    [sym_await_expression] = STATE(2396),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2396),
    [sym_augmented_assignment_expression] = STATE(2396),
    [sym_ternary_expression] = STATE(2396),
    [sym_binary_expression] = STATE(2396),
    [sym_unary_expression] = STATE(2396),
    [sym_update_expression] = STATE(2396),
    [sym_template_string] = STATE(2396),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5052),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5054),
    [sym_number] = ACTIONS(5052),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5054),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5054),
    [sym_false] = ACTIONS(5054),
    [sym_null] = ACTIONS(5054),
    [sym_undefined] = ACTIONS(5054),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2267] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2398),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2398),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(5056),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2268] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2399),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2399),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2269] = {
    [sym_parenthesized_expression] = STATE(2400),
    [sym__expression] = STATE(2400),
    [sym_yield_expression] = STATE(2400),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2400),
    [sym_jsx_fragment] = STATE(2400),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2400),
    [sym_anonymous_class] = STATE(2400),
    [sym_class] = STATE(2400),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2400),
    [sym_generator_function] = STATE(2400),
    [sym_call_expression] = STATE(2400),
    [sym_new_expression] = STATE(2400),
    [sym_await_expression] = STATE(2400),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2400),
    [sym_augmented_assignment_expression] = STATE(2400),
    [sym_ternary_expression] = STATE(2400),
    [sym_binary_expression] = STATE(2400),
    [sym_unary_expression] = STATE(2400),
    [sym_update_expression] = STATE(2400),
    [sym_template_string] = STATE(2400),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5058),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5060),
    [sym_number] = ACTIONS(5058),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5060),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5060),
    [sym_false] = ACTIONS(5060),
    [sym_null] = ACTIONS(5060),
    [sym_undefined] = ACTIONS(5060),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2270] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2401),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2401),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2271] = {
    [sym_parenthesized_expression] = STATE(2402),
    [sym__expression] = STATE(2402),
    [sym_yield_expression] = STATE(2402),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2402),
    [sym_jsx_fragment] = STATE(2402),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2402),
    [sym_anonymous_class] = STATE(2402),
    [sym_class] = STATE(2402),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2402),
    [sym_generator_function] = STATE(2402),
    [sym_call_expression] = STATE(2402),
    [sym_new_expression] = STATE(2402),
    [sym_await_expression] = STATE(2402),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2402),
    [sym_augmented_assignment_expression] = STATE(2402),
    [sym_ternary_expression] = STATE(2402),
    [sym_binary_expression] = STATE(2402),
    [sym_unary_expression] = STATE(2402),
    [sym_update_expression] = STATE(2402),
    [sym_template_string] = STATE(2402),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5062),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5064),
    [sym_number] = ACTIONS(5062),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5064),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5064),
    [sym_false] = ACTIONS(5064),
    [sym_null] = ACTIONS(5064),
    [sym_undefined] = ACTIONS(5064),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2272] = {
    [sym__automatic_semicolon] = ACTIONS(5066),
    [anon_sym_SEMI] = ACTIONS(5066),
    [sym_comment] = ACTIONS(71),
  },
  [2273] = {
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_catch] = ACTIONS(1590),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2274] = {
    [sym_export_statement] = STATE(2405),
    [sym__declaration] = STATE(2405),
    [sym_import_statement] = STATE(2405),
    [sym_expression_statement] = STATE(2405),
    [sym_variable_declaration] = STATE(2405),
    [sym_lexical_declaration] = STATE(2405),
    [sym_statement_block] = STATE(2405),
    [sym_if_statement] = STATE(2405),
    [sym_switch_statement] = STATE(2405),
    [sym_for_statement] = STATE(2405),
    [sym_for_in_statement] = STATE(2405),
    [sym_for_of_statement] = STATE(2405),
    [sym_while_statement] = STATE(2405),
    [sym_do_statement] = STATE(2405),
    [sym_try_statement] = STATE(2405),
    [sym_with_statement] = STATE(2405),
    [sym_break_statement] = STATE(2405),
    [sym_continue_statement] = STATE(2405),
    [sym_debugger_statement] = STATE(2405),
    [sym_return_statement] = STATE(2405),
    [sym_throw_statement] = STATE(2405),
    [sym_empty_statement] = STATE(2405),
    [sym_labeled_statement] = STATE(2405),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2405),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5068),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2275] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5070),
  },
  [2276] = {
    [anon_sym_export] = ACTIONS(2927),
    [anon_sym_default] = ACTIONS(2927),
    [anon_sym_LBRACE] = ACTIONS(2925),
    [anon_sym_RBRACE] = ACTIONS(2925),
    [anon_sym_import] = ACTIONS(2927),
    [anon_sym_var] = ACTIONS(2927),
    [anon_sym_let] = ACTIONS(2927),
    [anon_sym_const] = ACTIONS(2927),
    [anon_sym_if] = ACTIONS(2927),
    [anon_sym_switch] = ACTIONS(2927),
    [anon_sym_for] = ACTIONS(2927),
    [anon_sym_LPAREN] = ACTIONS(2925),
    [anon_sym_while] = ACTIONS(2927),
    [anon_sym_do] = ACTIONS(2927),
    [anon_sym_try] = ACTIONS(2927),
    [anon_sym_with] = ACTIONS(2927),
    [anon_sym_break] = ACTIONS(2927),
    [anon_sym_continue] = ACTIONS(2927),
    [anon_sym_debugger] = ACTIONS(2927),
    [anon_sym_return] = ACTIONS(2927),
    [anon_sym_throw] = ACTIONS(2927),
    [anon_sym_SEMI] = ACTIONS(2925),
    [anon_sym_case] = ACTIONS(2927),
    [anon_sym_finally] = ACTIONS(2927),
    [anon_sym_yield] = ACTIONS(2927),
    [anon_sym_LBRACK] = ACTIONS(2925),
    [anon_sym_LT] = ACTIONS(2925),
    [anon_sym_class] = ACTIONS(2927),
    [anon_sym_async] = ACTIONS(2927),
    [anon_sym_function] = ACTIONS(2927),
    [anon_sym_EQ_GT] = ACTIONS(2925),
    [anon_sym_new] = ACTIONS(2927),
    [anon_sym_await] = ACTIONS(2927),
    [anon_sym_PLUS] = ACTIONS(2927),
    [anon_sym_DASH] = ACTIONS(2927),
    [anon_sym_BANG] = ACTIONS(2925),
    [anon_sym_TILDE] = ACTIONS(2925),
    [anon_sym_typeof] = ACTIONS(2927),
    [anon_sym_void] = ACTIONS(2927),
    [anon_sym_delete] = ACTIONS(2927),
    [anon_sym_PLUS_PLUS] = ACTIONS(2925),
    [anon_sym_DASH_DASH] = ACTIONS(2925),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2925),
    [anon_sym_BQUOTE] = ACTIONS(2925),
    [sym_regex] = ACTIONS(2927),
    [sym_number] = ACTIONS(2925),
    [sym_identifier] = ACTIONS(2927),
    [sym_this] = ACTIONS(2927),
    [sym_super] = ACTIONS(2927),
    [sym_true] = ACTIONS(2927),
    [sym_false] = ACTIONS(2927),
    [sym_null] = ACTIONS(2927),
    [sym_undefined] = ACTIONS(2927),
    [anon_sym_AT] = ACTIONS(2925),
    [anon_sym_get] = ACTIONS(2927),
    [anon_sym_set] = ACTIONS(2927),
  },
  [2277] = {
    [sym_export_statement] = STATE(1877),
    [sym__declaration] = STATE(1877),
    [sym_import_statement] = STATE(1877),
    [sym_expression_statement] = STATE(1877),
    [sym_variable_declaration] = STATE(1877),
    [sym_lexical_declaration] = STATE(1877),
    [sym_statement_block] = STATE(1877),
    [sym_if_statement] = STATE(1877),
    [sym_switch_statement] = STATE(1877),
    [sym_for_statement] = STATE(1877),
    [sym_for_in_statement] = STATE(1877),
    [sym_for_of_statement] = STATE(1877),
    [sym_while_statement] = STATE(1877),
    [sym_do_statement] = STATE(1877),
    [sym_try_statement] = STATE(1877),
    [sym_with_statement] = STATE(1877),
    [sym_break_statement] = STATE(1877),
    [sym_continue_statement] = STATE(1877),
    [sym_debugger_statement] = STATE(1877),
    [sym_return_statement] = STATE(1877),
    [sym_throw_statement] = STATE(1877),
    [sym_empty_statement] = STATE(1877),
    [sym_labeled_statement] = STATE(1877),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(1877),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5072),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2278] = {
    [anon_sym_export] = ACTIONS(2933),
    [anon_sym_default] = ACTIONS(2933),
    [anon_sym_LBRACE] = ACTIONS(2931),
    [anon_sym_RBRACE] = ACTIONS(2931),
    [anon_sym_import] = ACTIONS(2933),
    [anon_sym_var] = ACTIONS(2933),
    [anon_sym_let] = ACTIONS(2933),
    [anon_sym_const] = ACTIONS(2933),
    [anon_sym_if] = ACTIONS(2933),
    [anon_sym_switch] = ACTIONS(2933),
    [anon_sym_for] = ACTIONS(2933),
    [anon_sym_LPAREN] = ACTIONS(2931),
    [anon_sym_while] = ACTIONS(2933),
    [anon_sym_do] = ACTIONS(2933),
    [anon_sym_try] = ACTIONS(2933),
    [anon_sym_with] = ACTIONS(2933),
    [anon_sym_break] = ACTIONS(2933),
    [anon_sym_continue] = ACTIONS(2933),
    [anon_sym_debugger] = ACTIONS(2933),
    [anon_sym_return] = ACTIONS(2933),
    [anon_sym_throw] = ACTIONS(2933),
    [anon_sym_SEMI] = ACTIONS(2931),
    [anon_sym_case] = ACTIONS(2933),
    [anon_sym_yield] = ACTIONS(2933),
    [anon_sym_LBRACK] = ACTIONS(2931),
    [anon_sym_LT] = ACTIONS(2931),
    [anon_sym_class] = ACTIONS(2933),
    [anon_sym_async] = ACTIONS(2933),
    [anon_sym_function] = ACTIONS(2933),
    [anon_sym_EQ_GT] = ACTIONS(2931),
    [anon_sym_new] = ACTIONS(2933),
    [anon_sym_await] = ACTIONS(2933),
    [anon_sym_PLUS] = ACTIONS(2933),
    [anon_sym_DASH] = ACTIONS(2933),
    [anon_sym_BANG] = ACTIONS(2931),
    [anon_sym_TILDE] = ACTIONS(2931),
    [anon_sym_typeof] = ACTIONS(2933),
    [anon_sym_void] = ACTIONS(2933),
    [anon_sym_delete] = ACTIONS(2933),
    [anon_sym_PLUS_PLUS] = ACTIONS(2931),
    [anon_sym_DASH_DASH] = ACTIONS(2931),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2931),
    [anon_sym_BQUOTE] = ACTIONS(2931),
    [sym_regex] = ACTIONS(2933),
    [sym_number] = ACTIONS(2931),
    [sym_identifier] = ACTIONS(2933),
    [sym_this] = ACTIONS(2933),
    [sym_super] = ACTIONS(2933),
    [sym_true] = ACTIONS(2933),
    [sym_false] = ACTIONS(2933),
    [sym_null] = ACTIONS(2933),
    [sym_undefined] = ACTIONS(2933),
    [anon_sym_AT] = ACTIONS(2931),
    [anon_sym_get] = ACTIONS(2933),
    [anon_sym_set] = ACTIONS(2933),
  },
  [2279] = {
    [anon_sym_export] = ACTIONS(2937),
    [anon_sym_default] = ACTIONS(2937),
    [anon_sym_LBRACE] = ACTIONS(2935),
    [anon_sym_RBRACE] = ACTIONS(2935),
    [anon_sym_import] = ACTIONS(2937),
    [anon_sym_var] = ACTIONS(2937),
    [anon_sym_let] = ACTIONS(2937),
    [anon_sym_const] = ACTIONS(2937),
    [anon_sym_if] = ACTIONS(2937),
    [anon_sym_switch] = ACTIONS(2937),
    [anon_sym_for] = ACTIONS(2937),
    [anon_sym_LPAREN] = ACTIONS(2935),
    [anon_sym_while] = ACTIONS(2937),
    [anon_sym_do] = ACTIONS(2937),
    [anon_sym_try] = ACTIONS(2937),
    [anon_sym_with] = ACTIONS(2937),
    [anon_sym_break] = ACTIONS(2937),
    [anon_sym_continue] = ACTIONS(2937),
    [anon_sym_debugger] = ACTIONS(2937),
    [anon_sym_return] = ACTIONS(2937),
    [anon_sym_throw] = ACTIONS(2937),
    [anon_sym_SEMI] = ACTIONS(2935),
    [anon_sym_case] = ACTIONS(2937),
    [anon_sym_yield] = ACTIONS(2937),
    [anon_sym_LBRACK] = ACTIONS(2935),
    [anon_sym_LT] = ACTIONS(2935),
    [anon_sym_class] = ACTIONS(2937),
    [anon_sym_async] = ACTIONS(2937),
    [anon_sym_function] = ACTIONS(2937),
    [anon_sym_EQ_GT] = ACTIONS(2935),
    [anon_sym_new] = ACTIONS(2937),
    [anon_sym_await] = ACTIONS(2937),
    [anon_sym_PLUS] = ACTIONS(2937),
    [anon_sym_DASH] = ACTIONS(2937),
    [anon_sym_BANG] = ACTIONS(2935),
    [anon_sym_TILDE] = ACTIONS(2935),
    [anon_sym_typeof] = ACTIONS(2937),
    [anon_sym_void] = ACTIONS(2937),
    [anon_sym_delete] = ACTIONS(2937),
    [anon_sym_PLUS_PLUS] = ACTIONS(2935),
    [anon_sym_DASH_DASH] = ACTIONS(2935),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2935),
    [anon_sym_BQUOTE] = ACTIONS(2935),
    [sym_regex] = ACTIONS(2937),
    [sym_number] = ACTIONS(2935),
    [sym_identifier] = ACTIONS(2937),
    [sym_this] = ACTIONS(2937),
    [sym_super] = ACTIONS(2937),
    [sym_true] = ACTIONS(2937),
    [sym_false] = ACTIONS(2937),
    [sym_null] = ACTIONS(2937),
    [sym_undefined] = ACTIONS(2937),
    [anon_sym_AT] = ACTIONS(2935),
    [anon_sym_get] = ACTIONS(2937),
    [anon_sym_set] = ACTIONS(2937),
  },
  [2280] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_STAR] = ACTIONS(2064),
    [anon_sym_default] = ACTIONS(2066),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_COMMA] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_in] = ACTIONS(2066),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_case] = ACTIONS(2066),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2066),
    [anon_sym_GT] = ACTIONS(2066),
    [anon_sym_SLASH] = ACTIONS(2066),
    [anon_sym_DOT] = ACTIONS(2066),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_QMARK] = ACTIONS(2064),
    [anon_sym_AMP_AMP] = ACTIONS(2064),
    [anon_sym_PIPE_PIPE] = ACTIONS(2064),
    [anon_sym_GT_GT] = ACTIONS(2066),
    [anon_sym_GT_GT_GT] = ACTIONS(2064),
    [anon_sym_LT_LT] = ACTIONS(2064),
    [anon_sym_AMP] = ACTIONS(2066),
    [anon_sym_CARET] = ACTIONS(2064),
    [anon_sym_PIPE] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_PERCENT] = ACTIONS(2064),
    [anon_sym_LT_EQ] = ACTIONS(2064),
    [anon_sym_EQ_EQ] = ACTIONS(2066),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2064),
    [anon_sym_BANG_EQ] = ACTIONS(2066),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2064),
    [anon_sym_GT_EQ] = ACTIONS(2064),
    [anon_sym_instanceof] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2066),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [2281] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(5074),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2282] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_STAR] = ACTIONS(3103),
    [anon_sym_default] = ACTIONS(3105),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_COMMA] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_in] = ACTIONS(3105),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_case] = ACTIONS(3105),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3105),
    [anon_sym_GT] = ACTIONS(3105),
    [anon_sym_SLASH] = ACTIONS(3105),
    [anon_sym_DOT] = ACTIONS(3105),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3103),
    [anon_sym_AMP_AMP] = ACTIONS(3103),
    [anon_sym_PIPE_PIPE] = ACTIONS(3103),
    [anon_sym_GT_GT] = ACTIONS(3105),
    [anon_sym_GT_GT_GT] = ACTIONS(3103),
    [anon_sym_LT_LT] = ACTIONS(3103),
    [anon_sym_AMP] = ACTIONS(3105),
    [anon_sym_CARET] = ACTIONS(3103),
    [anon_sym_PIPE] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_PERCENT] = ACTIONS(3103),
    [anon_sym_LT_EQ] = ACTIONS(3103),
    [anon_sym_EQ_EQ] = ACTIONS(3105),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3103),
    [anon_sym_BANG_EQ] = ACTIONS(3105),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3103),
    [anon_sym_GT_EQ] = ACTIONS(3103),
    [anon_sym_instanceof] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3105),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [2283] = {
    [sym_statement_block] = STATE(2409),
    [anon_sym_LBRACE] = ACTIONS(4548),
    [sym_comment] = ACTIONS(71),
  },
  [2284] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_STAR] = ACTIONS(3107),
    [anon_sym_default] = ACTIONS(3109),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_COMMA] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_in] = ACTIONS(3109),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_case] = ACTIONS(3109),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3109),
    [anon_sym_GT] = ACTIONS(3109),
    [anon_sym_SLASH] = ACTIONS(3109),
    [anon_sym_DOT] = ACTIONS(3109),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_AMP_AMP] = ACTIONS(3107),
    [anon_sym_PIPE_PIPE] = ACTIONS(3107),
    [anon_sym_GT_GT] = ACTIONS(3109),
    [anon_sym_GT_GT_GT] = ACTIONS(3107),
    [anon_sym_LT_LT] = ACTIONS(3107),
    [anon_sym_AMP] = ACTIONS(3109),
    [anon_sym_CARET] = ACTIONS(3107),
    [anon_sym_PIPE] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_PERCENT] = ACTIONS(3107),
    [anon_sym_LT_EQ] = ACTIONS(3107),
    [anon_sym_EQ_EQ] = ACTIONS(3109),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3107),
    [anon_sym_BANG_EQ] = ACTIONS(3109),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3107),
    [anon_sym_GT_EQ] = ACTIONS(3107),
    [anon_sym_instanceof] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3109),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [2285] = {
    [sym_statement_block] = STATE(2410),
    [anon_sym_LBRACE] = ACTIONS(4548),
    [sym_comment] = ACTIONS(71),
  },
  [2286] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_STAR] = ACTIONS(3115),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_COMMA] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_in] = ACTIONS(3117),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_case] = ACTIONS(3117),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3117),
    [anon_sym_GT] = ACTIONS(3117),
    [anon_sym_SLASH] = ACTIONS(3117),
    [anon_sym_DOT] = ACTIONS(3117),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_QMARK] = ACTIONS(3115),
    [anon_sym_AMP_AMP] = ACTIONS(3115),
    [anon_sym_PIPE_PIPE] = ACTIONS(3115),
    [anon_sym_GT_GT] = ACTIONS(3117),
    [anon_sym_GT_GT_GT] = ACTIONS(3115),
    [anon_sym_LT_LT] = ACTIONS(3115),
    [anon_sym_AMP] = ACTIONS(3117),
    [anon_sym_CARET] = ACTIONS(3115),
    [anon_sym_PIPE] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_PERCENT] = ACTIONS(3115),
    [anon_sym_LT_EQ] = ACTIONS(3115),
    [anon_sym_EQ_EQ] = ACTIONS(3117),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3115),
    [anon_sym_BANG_EQ] = ACTIONS(3117),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3115),
    [anon_sym_GT_EQ] = ACTIONS(3115),
    [anon_sym_instanceof] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3117),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [2287] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_in] = ACTIONS(484),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(484),
    [anon_sym_GT] = ACTIONS(484),
    [anon_sym_SLASH] = ACTIONS(484),
    [anon_sym_DOT] = ACTIONS(484),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(482),
    [anon_sym_AMP_AMP] = ACTIONS(482),
    [anon_sym_PIPE_PIPE] = ACTIONS(482),
    [anon_sym_GT_GT] = ACTIONS(484),
    [anon_sym_GT_GT_GT] = ACTIONS(482),
    [anon_sym_LT_LT] = ACTIONS(482),
    [anon_sym_AMP] = ACTIONS(484),
    [anon_sym_CARET] = ACTIONS(482),
    [anon_sym_PIPE] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_PERCENT] = ACTIONS(482),
    [anon_sym_LT_EQ] = ACTIONS(482),
    [anon_sym_EQ_EQ] = ACTIONS(484),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(482),
    [anon_sym_BANG_EQ] = ACTIONS(484),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(482),
    [anon_sym_GT_EQ] = ACTIONS(482),
    [anon_sym_instanceof] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2288] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5076),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2289] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(5078),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(5078),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2290] = {
    [sym_class_body] = STATE(2413),
    [anon_sym_LBRACE] = ACTIONS(4542),
    [sym_comment] = ACTIONS(71),
  },
  [2291] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3564),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_in] = ACTIONS(3558),
    [anon_sym_COLON] = ACTIONS(3872),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3558),
    [anon_sym_GT] = ACTIONS(3558),
    [anon_sym_SLASH] = ACTIONS(3568),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3570),
    [anon_sym_AMP_AMP] = ACTIONS(3572),
    [anon_sym_PIPE_PIPE] = ACTIONS(3574),
    [anon_sym_GT_GT] = ACTIONS(3568),
    [anon_sym_GT_GT_GT] = ACTIONS(3564),
    [anon_sym_LT_LT] = ACTIONS(3564),
    [anon_sym_AMP] = ACTIONS(3576),
    [anon_sym_CARET] = ACTIONS(3574),
    [anon_sym_PIPE] = ACTIONS(3578),
    [anon_sym_PLUS] = ACTIONS(3580),
    [anon_sym_DASH] = ACTIONS(3580),
    [anon_sym_PERCENT] = ACTIONS(3564),
    [anon_sym_LT_EQ] = ACTIONS(3582),
    [anon_sym_EQ_EQ] = ACTIONS(3558),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3582),
    [anon_sym_BANG_EQ] = ACTIONS(3558),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3582),
    [anon_sym_GT_EQ] = ACTIONS(3582),
    [anon_sym_instanceof] = ACTIONS(3582),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2292] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(3872),
    [anon_sym_STAR] = ACTIONS(3638),
    [anon_sym_COMMA] = ACTIONS(3872),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_RPAREN] = ACTIONS(3872),
    [anon_sym_in] = ACTIONS(3632),
    [anon_sym_SEMI] = ACTIONS(3872),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(3632),
    [anon_sym_GT] = ACTIONS(3632),
    [anon_sym_SLASH] = ACTIONS(3640),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(4174),
    [anon_sym_AMP_AMP] = ACTIONS(4176),
    [anon_sym_PIPE_PIPE] = ACTIONS(4178),
    [anon_sym_GT_GT] = ACTIONS(3640),
    [anon_sym_GT_GT_GT] = ACTIONS(3638),
    [anon_sym_LT_LT] = ACTIONS(3638),
    [anon_sym_AMP] = ACTIONS(4180),
    [anon_sym_CARET] = ACTIONS(4178),
    [anon_sym_PIPE] = ACTIONS(4182),
    [anon_sym_PLUS] = ACTIONS(4184),
    [anon_sym_DASH] = ACTIONS(4184),
    [anon_sym_PERCENT] = ACTIONS(3638),
    [anon_sym_LT_EQ] = ACTIONS(4186),
    [anon_sym_EQ_EQ] = ACTIONS(3632),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(4186),
    [anon_sym_BANG_EQ] = ACTIONS(3632),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(4186),
    [anon_sym_GT_EQ] = ACTIONS(4186),
    [anon_sym_instanceof] = ACTIONS(4186),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2293] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(3872),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2294] = {
    [anon_sym_RPAREN] = ACTIONS(5080),
    [sym_comment] = ACTIONS(71),
  },
  [2295] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5082),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2296] = {
    [anon_sym_RPAREN] = ACTIONS(5084),
    [sym_comment] = ACTIONS(71),
  },
  [2297] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5086),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2298] = {
    [anon_sym_RPAREN] = ACTIONS(5088),
    [sym_comment] = ACTIONS(71),
  },
  [2299] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5090),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2300] = {
    [sym_export_statement] = STATE(2420),
    [sym__declaration] = STATE(2420),
    [sym_import_statement] = STATE(2420),
    [sym_expression_statement] = STATE(2420),
    [sym_variable_declaration] = STATE(2420),
    [sym_lexical_declaration] = STATE(2420),
    [sym_statement_block] = STATE(2420),
    [sym_if_statement] = STATE(2420),
    [sym_switch_statement] = STATE(2420),
    [sym_for_statement] = STATE(2420),
    [sym_for_in_statement] = STATE(2420),
    [sym_for_of_statement] = STATE(2420),
    [sym_while_statement] = STATE(2420),
    [sym_do_statement] = STATE(2420),
    [sym_try_statement] = STATE(2420),
    [sym_with_statement] = STATE(2420),
    [sym_break_statement] = STATE(2420),
    [sym_continue_statement] = STATE(2420),
    [sym_debugger_statement] = STATE(2420),
    [sym_return_statement] = STATE(2420),
    [sym_throw_statement] = STATE(2420),
    [sym_empty_statement] = STATE(2420),
    [sym_labeled_statement] = STATE(2420),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2301] = {
    [sym_export_statement] = STATE(2421),
    [sym__declaration] = STATE(2421),
    [sym_import_statement] = STATE(2421),
    [sym_expression_statement] = STATE(2421),
    [sym_variable_declaration] = STATE(2421),
    [sym_lexical_declaration] = STATE(2421),
    [sym_statement_block] = STATE(2421),
    [sym_if_statement] = STATE(2421),
    [sym_switch_statement] = STATE(2421),
    [sym_for_statement] = STATE(2421),
    [sym_for_in_statement] = STATE(2421),
    [sym_for_of_statement] = STATE(2421),
    [sym_while_statement] = STATE(2421),
    [sym_do_statement] = STATE(2421),
    [sym_try_statement] = STATE(2421),
    [sym_with_statement] = STATE(2421),
    [sym_break_statement] = STATE(2421),
    [sym_continue_statement] = STATE(2421),
    [sym_debugger_statement] = STATE(2421),
    [sym_return_statement] = STATE(2421),
    [sym_throw_statement] = STATE(2421),
    [sym_empty_statement] = STATE(2421),
    [sym_labeled_statement] = STATE(2421),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2302] = {
    [anon_sym_else] = ACTIONS(4264),
    [anon_sym_while] = ACTIONS(4264),
    [sym_comment] = ACTIONS(71),
  },
  [2303] = {
    [sym_export_statement] = STATE(2422),
    [sym__declaration] = STATE(2422),
    [sym_import_statement] = STATE(2422),
    [sym_expression_statement] = STATE(2422),
    [sym_variable_declaration] = STATE(2422),
    [sym_lexical_declaration] = STATE(2422),
    [sym_statement_block] = STATE(2422),
    [sym_if_statement] = STATE(2422),
    [sym_switch_statement] = STATE(2422),
    [sym_for_statement] = STATE(2422),
    [sym_for_in_statement] = STATE(2422),
    [sym_for_of_statement] = STATE(2422),
    [sym_while_statement] = STATE(2422),
    [sym_do_statement] = STATE(2422),
    [sym_try_statement] = STATE(2422),
    [sym_with_statement] = STATE(2422),
    [sym_break_statement] = STATE(2422),
    [sym_continue_statement] = STATE(2422),
    [sym_debugger_statement] = STATE(2422),
    [sym_return_statement] = STATE(2422),
    [sym_throw_statement] = STATE(2422),
    [sym_empty_statement] = STATE(2422),
    [sym_labeled_statement] = STATE(2422),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2304] = {
    [sym_export_statement] = STATE(2423),
    [sym__declaration] = STATE(2423),
    [sym_import_statement] = STATE(2423),
    [sym_expression_statement] = STATE(2423),
    [sym_variable_declaration] = STATE(2423),
    [sym_lexical_declaration] = STATE(2423),
    [sym_statement_block] = STATE(2423),
    [sym_if_statement] = STATE(2423),
    [sym_switch_statement] = STATE(2423),
    [sym_for_statement] = STATE(2423),
    [sym_for_in_statement] = STATE(2423),
    [sym_for_of_statement] = STATE(2423),
    [sym_while_statement] = STATE(2423),
    [sym_do_statement] = STATE(2423),
    [sym_try_statement] = STATE(2423),
    [sym_with_statement] = STATE(2423),
    [sym_break_statement] = STATE(2423),
    [sym_continue_statement] = STATE(2423),
    [sym_debugger_statement] = STATE(2423),
    [sym_return_statement] = STATE(2423),
    [sym_throw_statement] = STATE(2423),
    [sym_empty_statement] = STATE(2423),
    [sym_labeled_statement] = STATE(2423),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2305] = {
    [sym_export_statement] = STATE(2424),
    [sym__declaration] = STATE(2424),
    [sym_import_statement] = STATE(2424),
    [sym_expression_statement] = STATE(2424),
    [sym_variable_declaration] = STATE(2424),
    [sym_lexical_declaration] = STATE(2424),
    [sym_statement_block] = STATE(2424),
    [sym_if_statement] = STATE(2424),
    [sym_switch_statement] = STATE(2424),
    [sym_for_statement] = STATE(2424),
    [sym_for_in_statement] = STATE(2424),
    [sym_for_of_statement] = STATE(2424),
    [sym_while_statement] = STATE(2424),
    [sym_do_statement] = STATE(2424),
    [sym_try_statement] = STATE(2424),
    [sym_with_statement] = STATE(2424),
    [sym_break_statement] = STATE(2424),
    [sym_continue_statement] = STATE(2424),
    [sym_debugger_statement] = STATE(2424),
    [sym_return_statement] = STATE(2424),
    [sym_throw_statement] = STATE(2424),
    [sym_empty_statement] = STATE(2424),
    [sym_labeled_statement] = STATE(2424),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2306] = {
    [sym_export_statement] = STATE(2425),
    [sym__declaration] = STATE(2425),
    [sym_import_statement] = STATE(2425),
    [sym_expression_statement] = STATE(2425),
    [sym_variable_declaration] = STATE(2425),
    [sym_lexical_declaration] = STATE(2425),
    [sym_statement_block] = STATE(2425),
    [sym_if_statement] = STATE(2425),
    [sym_switch_statement] = STATE(2425),
    [sym_for_statement] = STATE(2425),
    [sym_for_in_statement] = STATE(2425),
    [sym_for_of_statement] = STATE(2425),
    [sym_while_statement] = STATE(2425),
    [sym_do_statement] = STATE(2425),
    [sym_try_statement] = STATE(2425),
    [sym_with_statement] = STATE(2425),
    [sym_break_statement] = STATE(2425),
    [sym_continue_statement] = STATE(2425),
    [sym_debugger_statement] = STATE(2425),
    [sym_return_statement] = STATE(2425),
    [sym_throw_statement] = STATE(2425),
    [sym_empty_statement] = STATE(2425),
    [sym_labeled_statement] = STATE(2425),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2307] = {
    [sym_export_statement] = STATE(2426),
    [sym__declaration] = STATE(2426),
    [sym_import_statement] = STATE(2426),
    [sym_expression_statement] = STATE(2426),
    [sym_variable_declaration] = STATE(2426),
    [sym_lexical_declaration] = STATE(2426),
    [sym_statement_block] = STATE(2426),
    [sym_if_statement] = STATE(2426),
    [sym_switch_statement] = STATE(2426),
    [sym_for_statement] = STATE(2426),
    [sym_for_in_statement] = STATE(2426),
    [sym_for_of_statement] = STATE(2426),
    [sym_while_statement] = STATE(2426),
    [sym_do_statement] = STATE(2426),
    [sym_try_statement] = STATE(2426),
    [sym_with_statement] = STATE(2426),
    [sym_break_statement] = STATE(2426),
    [sym_continue_statement] = STATE(2426),
    [sym_debugger_statement] = STATE(2426),
    [sym_return_statement] = STATE(2426),
    [sym_throw_statement] = STATE(2426),
    [sym_empty_statement] = STATE(2426),
    [sym_labeled_statement] = STATE(2426),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2308] = {
    [sym_statement_block] = STATE(2427),
    [anon_sym_LBRACE] = ACTIONS(1789),
    [sym_comment] = ACTIONS(71),
  },
  [2309] = {
    [anon_sym_LBRACE] = ACTIONS(3798),
    [anon_sym_GT] = ACTIONS(3798),
    [anon_sym_SLASH] = ACTIONS(3800),
    [sym_jsx_identifier] = ACTIONS(3798),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3800),
  },
  [2310] = {
    [anon_sym_GT] = ACTIONS(5092),
    [sym_comment] = ACTIONS(71),
  },
  [2311] = {
    [anon_sym_LBRACE] = ACTIONS(3824),
    [anon_sym_GT] = ACTIONS(3824),
    [anon_sym_SLASH] = ACTIONS(3826),
    [sym_jsx_identifier] = ACTIONS(3824),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3826),
  },
  [2312] = {
    [anon_sym_LBRACE] = ACTIONS(3828),
    [anon_sym_GT] = ACTIONS(3828),
    [anon_sym_SLASH] = ACTIONS(3830),
    [sym_jsx_identifier] = ACTIONS(3828),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3830),
  },
  [2313] = {
    [anon_sym_LBRACE] = ACTIONS(3914),
    [anon_sym_GT] = ACTIONS(3914),
    [anon_sym_SLASH] = ACTIONS(3916),
    [sym_jsx_identifier] = ACTIONS(3914),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3916),
  },
  [2314] = {
    [anon_sym_LBRACE] = ACTIONS(3918),
    [anon_sym_GT] = ACTIONS(3918),
    [anon_sym_SLASH] = ACTIONS(3920),
    [sym_jsx_identifier] = ACTIONS(3918),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(3920),
  },
  [2315] = {
    [sym_export_statement] = STATE(2321),
    [sym__declaration] = STATE(2321),
    [sym_import_statement] = STATE(2321),
    [sym_expression_statement] = STATE(2321),
    [sym_variable_declaration] = STATE(2321),
    [sym_lexical_declaration] = STATE(2321),
    [sym_statement_block] = STATE(2321),
    [sym_if_statement] = STATE(2321),
    [sym_switch_statement] = STATE(2321),
    [sym_for_statement] = STATE(2321),
    [sym_for_in_statement] = STATE(2321),
    [sym_for_of_statement] = STATE(2321),
    [sym_while_statement] = STATE(2321),
    [sym_do_statement] = STATE(2321),
    [sym_try_statement] = STATE(2321),
    [sym_with_statement] = STATE(2321),
    [sym_break_statement] = STATE(2321),
    [sym_continue_statement] = STATE(2321),
    [sym_debugger_statement] = STATE(2321),
    [sym_return_statement] = STATE(2321),
    [sym_throw_statement] = STATE(2321),
    [sym_empty_statement] = STATE(2321),
    [sym_labeled_statement] = STATE(2321),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2316] = {
    [sym_export_statement] = STATE(2322),
    [sym__declaration] = STATE(2322),
    [sym_import_statement] = STATE(2322),
    [sym_expression_statement] = STATE(2322),
    [sym_variable_declaration] = STATE(2322),
    [sym_lexical_declaration] = STATE(2322),
    [sym_statement_block] = STATE(2322),
    [sym_if_statement] = STATE(2322),
    [sym_switch_statement] = STATE(2322),
    [sym_for_statement] = STATE(2322),
    [sym_for_in_statement] = STATE(2322),
    [sym_for_of_statement] = STATE(2322),
    [sym_while_statement] = STATE(2322),
    [sym_do_statement] = STATE(2322),
    [sym_try_statement] = STATE(2322),
    [sym_with_statement] = STATE(2322),
    [sym_break_statement] = STATE(2322),
    [sym_continue_statement] = STATE(2322),
    [sym_debugger_statement] = STATE(2322),
    [sym_return_statement] = STATE(2322),
    [sym_throw_statement] = STATE(2322),
    [sym_empty_statement] = STATE(2322),
    [sym_labeled_statement] = STATE(2322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2317] = {
    [sym_export_statement] = STATE(2323),
    [sym__declaration] = STATE(2323),
    [sym_import_statement] = STATE(2323),
    [sym_expression_statement] = STATE(2323),
    [sym_variable_declaration] = STATE(2323),
    [sym_lexical_declaration] = STATE(2323),
    [sym_statement_block] = STATE(2323),
    [sym_if_statement] = STATE(2323),
    [sym_switch_statement] = STATE(2323),
    [sym_for_statement] = STATE(2323),
    [sym_for_in_statement] = STATE(2323),
    [sym_for_of_statement] = STATE(2323),
    [sym_while_statement] = STATE(2323),
    [sym_do_statement] = STATE(2323),
    [sym_try_statement] = STATE(2323),
    [sym_with_statement] = STATE(2323),
    [sym_break_statement] = STATE(2323),
    [sym_continue_statement] = STATE(2323),
    [sym_debugger_statement] = STATE(2323),
    [sym_return_statement] = STATE(2323),
    [sym_throw_statement] = STATE(2323),
    [sym_empty_statement] = STATE(2323),
    [sym_labeled_statement] = STATE(2323),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2318] = {
    [sym_export_statement] = STATE(2324),
    [sym__declaration] = STATE(2324),
    [sym_import_statement] = STATE(2324),
    [sym_expression_statement] = STATE(2324),
    [sym_variable_declaration] = STATE(2324),
    [sym_lexical_declaration] = STATE(2324),
    [sym_statement_block] = STATE(2324),
    [sym_if_statement] = STATE(2324),
    [sym_switch_statement] = STATE(2324),
    [sym_for_statement] = STATE(2324),
    [sym_for_in_statement] = STATE(2324),
    [sym_for_of_statement] = STATE(2324),
    [sym_while_statement] = STATE(2324),
    [sym_do_statement] = STATE(2324),
    [sym_try_statement] = STATE(2324),
    [sym_with_statement] = STATE(2324),
    [sym_break_statement] = STATE(2324),
    [sym_continue_statement] = STATE(2324),
    [sym_debugger_statement] = STATE(2324),
    [sym_return_statement] = STATE(2324),
    [sym_throw_statement] = STATE(2324),
    [sym_empty_statement] = STATE(2324),
    [sym_labeled_statement] = STATE(2324),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2319] = {
    [sym_export_statement] = STATE(2325),
    [sym__declaration] = STATE(2325),
    [sym_import_statement] = STATE(2325),
    [sym_expression_statement] = STATE(2325),
    [sym_variable_declaration] = STATE(2325),
    [sym_lexical_declaration] = STATE(2325),
    [sym_statement_block] = STATE(2325),
    [sym_if_statement] = STATE(2325),
    [sym_switch_statement] = STATE(2325),
    [sym_for_statement] = STATE(2325),
    [sym_for_in_statement] = STATE(2325),
    [sym_for_of_statement] = STATE(2325),
    [sym_while_statement] = STATE(2325),
    [sym_do_statement] = STATE(2325),
    [sym_try_statement] = STATE(2325),
    [sym_with_statement] = STATE(2325),
    [sym_break_statement] = STATE(2325),
    [sym_continue_statement] = STATE(2325),
    [sym_debugger_statement] = STATE(2325),
    [sym_return_statement] = STATE(2325),
    [sym_throw_statement] = STATE(2325),
    [sym_empty_statement] = STATE(2325),
    [sym_labeled_statement] = STATE(2325),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2320] = {
    [sym_export_statement] = STATE(2326),
    [sym__declaration] = STATE(2326),
    [sym_import_statement] = STATE(2326),
    [sym_expression_statement] = STATE(2326),
    [sym_variable_declaration] = STATE(2326),
    [sym_lexical_declaration] = STATE(2326),
    [sym_statement_block] = STATE(2326),
    [sym_if_statement] = STATE(2326),
    [sym_switch_statement] = STATE(2326),
    [sym_for_statement] = STATE(2326),
    [sym_for_in_statement] = STATE(2326),
    [sym_for_of_statement] = STATE(2326),
    [sym_while_statement] = STATE(2326),
    [sym_do_statement] = STATE(2326),
    [sym_try_statement] = STATE(2326),
    [sym_with_statement] = STATE(2326),
    [sym_break_statement] = STATE(2326),
    [sym_continue_statement] = STATE(2326),
    [sym_debugger_statement] = STATE(2326),
    [sym_return_statement] = STATE(2326),
    [sym_throw_statement] = STATE(2326),
    [sym_empty_statement] = STATE(2326),
    [sym_labeled_statement] = STATE(2326),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(384),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(385),
    [sym_function] = STATE(386),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(385),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(384),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(387),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(387),
    [anon_sym_export] = ACTIONS(622),
    [anon_sym_LBRACE] = ACTIONS(624),
    [anon_sym_import] = ACTIONS(626),
    [anon_sym_var] = ACTIONS(628),
    [anon_sym_let] = ACTIONS(630),
    [anon_sym_const] = ACTIONS(630),
    [anon_sym_if] = ACTIONS(1453),
    [anon_sym_switch] = ACTIONS(634),
    [anon_sym_for] = ACTIONS(1455),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1457),
    [anon_sym_do] = ACTIONS(640),
    [anon_sym_try] = ACTIONS(642),
    [anon_sym_with] = ACTIONS(1459),
    [anon_sym_break] = ACTIONS(646),
    [anon_sym_continue] = ACTIONS(648),
    [anon_sym_debugger] = ACTIONS(650),
    [anon_sym_return] = ACTIONS(652),
    [anon_sym_throw] = ACTIONS(654),
    [anon_sym_SEMI] = ACTIONS(656),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(658),
    [anon_sym_async] = ACTIONS(660),
    [anon_sym_function] = ACTIONS(662),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1461),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2321] = {
    [ts_builtin_sym_end] = ACTIONS(4890),
    [anon_sym_export] = ACTIONS(4892),
    [anon_sym_LBRACE] = ACTIONS(4890),
    [anon_sym_RBRACE] = ACTIONS(4890),
    [anon_sym_import] = ACTIONS(4892),
    [anon_sym_var] = ACTIONS(4892),
    [anon_sym_let] = ACTIONS(4892),
    [anon_sym_const] = ACTIONS(4892),
    [anon_sym_if] = ACTIONS(4892),
    [anon_sym_else] = ACTIONS(4892),
    [anon_sym_switch] = ACTIONS(4892),
    [anon_sym_for] = ACTIONS(4892),
    [anon_sym_LPAREN] = ACTIONS(4890),
    [anon_sym_while] = ACTIONS(4892),
    [anon_sym_do] = ACTIONS(4892),
    [anon_sym_try] = ACTIONS(4892),
    [anon_sym_with] = ACTIONS(4892),
    [anon_sym_break] = ACTIONS(4892),
    [anon_sym_continue] = ACTIONS(4892),
    [anon_sym_debugger] = ACTIONS(4892),
    [anon_sym_return] = ACTIONS(4892),
    [anon_sym_throw] = ACTIONS(4892),
    [anon_sym_SEMI] = ACTIONS(4890),
    [anon_sym_yield] = ACTIONS(4892),
    [anon_sym_LBRACK] = ACTIONS(4890),
    [anon_sym_LT] = ACTIONS(4890),
    [anon_sym_class] = ACTIONS(4892),
    [anon_sym_async] = ACTIONS(4892),
    [anon_sym_function] = ACTIONS(4892),
    [anon_sym_EQ_GT] = ACTIONS(4890),
    [anon_sym_new] = ACTIONS(4892),
    [anon_sym_await] = ACTIONS(4892),
    [anon_sym_PLUS] = ACTIONS(4892),
    [anon_sym_DASH] = ACTIONS(4892),
    [anon_sym_BANG] = ACTIONS(4890),
    [anon_sym_TILDE] = ACTIONS(4890),
    [anon_sym_typeof] = ACTIONS(4892),
    [anon_sym_void] = ACTIONS(4892),
    [anon_sym_delete] = ACTIONS(4892),
    [anon_sym_PLUS_PLUS] = ACTIONS(4890),
    [anon_sym_DASH_DASH] = ACTIONS(4890),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4890),
    [anon_sym_BQUOTE] = ACTIONS(4890),
    [sym_regex] = ACTIONS(4892),
    [sym_number] = ACTIONS(4890),
    [sym_identifier] = ACTIONS(4892),
    [sym_this] = ACTIONS(4892),
    [sym_super] = ACTIONS(4892),
    [sym_true] = ACTIONS(4892),
    [sym_false] = ACTIONS(4892),
    [sym_null] = ACTIONS(4892),
    [sym_undefined] = ACTIONS(4892),
    [anon_sym_AT] = ACTIONS(4890),
    [anon_sym_get] = ACTIONS(4892),
    [anon_sym_set] = ACTIONS(4892),
  },
  [2322] = {
    [ts_builtin_sym_end] = ACTIONS(4894),
    [anon_sym_export] = ACTIONS(4896),
    [anon_sym_LBRACE] = ACTIONS(4894),
    [anon_sym_RBRACE] = ACTIONS(4894),
    [anon_sym_import] = ACTIONS(4896),
    [anon_sym_var] = ACTIONS(4896),
    [anon_sym_let] = ACTIONS(4896),
    [anon_sym_const] = ACTIONS(4896),
    [anon_sym_if] = ACTIONS(4896),
    [anon_sym_else] = ACTIONS(4896),
    [anon_sym_switch] = ACTIONS(4896),
    [anon_sym_for] = ACTIONS(4896),
    [anon_sym_LPAREN] = ACTIONS(4894),
    [anon_sym_while] = ACTIONS(4896),
    [anon_sym_do] = ACTIONS(4896),
    [anon_sym_try] = ACTIONS(4896),
    [anon_sym_with] = ACTIONS(4896),
    [anon_sym_break] = ACTIONS(4896),
    [anon_sym_continue] = ACTIONS(4896),
    [anon_sym_debugger] = ACTIONS(4896),
    [anon_sym_return] = ACTIONS(4896),
    [anon_sym_throw] = ACTIONS(4896),
    [anon_sym_SEMI] = ACTIONS(4894),
    [anon_sym_yield] = ACTIONS(4896),
    [anon_sym_LBRACK] = ACTIONS(4894),
    [anon_sym_LT] = ACTIONS(4894),
    [anon_sym_class] = ACTIONS(4896),
    [anon_sym_async] = ACTIONS(4896),
    [anon_sym_function] = ACTIONS(4896),
    [anon_sym_EQ_GT] = ACTIONS(4894),
    [anon_sym_new] = ACTIONS(4896),
    [anon_sym_await] = ACTIONS(4896),
    [anon_sym_PLUS] = ACTIONS(4896),
    [anon_sym_DASH] = ACTIONS(4896),
    [anon_sym_BANG] = ACTIONS(4894),
    [anon_sym_TILDE] = ACTIONS(4894),
    [anon_sym_typeof] = ACTIONS(4896),
    [anon_sym_void] = ACTIONS(4896),
    [anon_sym_delete] = ACTIONS(4896),
    [anon_sym_PLUS_PLUS] = ACTIONS(4894),
    [anon_sym_DASH_DASH] = ACTIONS(4894),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4894),
    [anon_sym_BQUOTE] = ACTIONS(4894),
    [sym_regex] = ACTIONS(4896),
    [sym_number] = ACTIONS(4894),
    [sym_identifier] = ACTIONS(4896),
    [sym_this] = ACTIONS(4896),
    [sym_super] = ACTIONS(4896),
    [sym_true] = ACTIONS(4896),
    [sym_false] = ACTIONS(4896),
    [sym_null] = ACTIONS(4896),
    [sym_undefined] = ACTIONS(4896),
    [anon_sym_AT] = ACTIONS(4894),
    [anon_sym_get] = ACTIONS(4896),
    [anon_sym_set] = ACTIONS(4896),
  },
  [2323] = {
    [ts_builtin_sym_end] = ACTIONS(4898),
    [anon_sym_export] = ACTIONS(4900),
    [anon_sym_LBRACE] = ACTIONS(4898),
    [anon_sym_RBRACE] = ACTIONS(4898),
    [anon_sym_import] = ACTIONS(4900),
    [anon_sym_var] = ACTIONS(4900),
    [anon_sym_let] = ACTIONS(4900),
    [anon_sym_const] = ACTIONS(4900),
    [anon_sym_if] = ACTIONS(4900),
    [anon_sym_else] = ACTIONS(4900),
    [anon_sym_switch] = ACTIONS(4900),
    [anon_sym_for] = ACTIONS(4900),
    [anon_sym_LPAREN] = ACTIONS(4898),
    [anon_sym_while] = ACTIONS(4900),
    [anon_sym_do] = ACTIONS(4900),
    [anon_sym_try] = ACTIONS(4900),
    [anon_sym_with] = ACTIONS(4900),
    [anon_sym_break] = ACTIONS(4900),
    [anon_sym_continue] = ACTIONS(4900),
    [anon_sym_debugger] = ACTIONS(4900),
    [anon_sym_return] = ACTIONS(4900),
    [anon_sym_throw] = ACTIONS(4900),
    [anon_sym_SEMI] = ACTIONS(4898),
    [anon_sym_yield] = ACTIONS(4900),
    [anon_sym_LBRACK] = ACTIONS(4898),
    [anon_sym_LT] = ACTIONS(4898),
    [anon_sym_class] = ACTIONS(4900),
    [anon_sym_async] = ACTIONS(4900),
    [anon_sym_function] = ACTIONS(4900),
    [anon_sym_EQ_GT] = ACTIONS(4898),
    [anon_sym_new] = ACTIONS(4900),
    [anon_sym_await] = ACTIONS(4900),
    [anon_sym_PLUS] = ACTIONS(4900),
    [anon_sym_DASH] = ACTIONS(4900),
    [anon_sym_BANG] = ACTIONS(4898),
    [anon_sym_TILDE] = ACTIONS(4898),
    [anon_sym_typeof] = ACTIONS(4900),
    [anon_sym_void] = ACTIONS(4900),
    [anon_sym_delete] = ACTIONS(4900),
    [anon_sym_PLUS_PLUS] = ACTIONS(4898),
    [anon_sym_DASH_DASH] = ACTIONS(4898),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4898),
    [anon_sym_BQUOTE] = ACTIONS(4898),
    [sym_regex] = ACTIONS(4900),
    [sym_number] = ACTIONS(4898),
    [sym_identifier] = ACTIONS(4900),
    [sym_this] = ACTIONS(4900),
    [sym_super] = ACTIONS(4900),
    [sym_true] = ACTIONS(4900),
    [sym_false] = ACTIONS(4900),
    [sym_null] = ACTIONS(4900),
    [sym_undefined] = ACTIONS(4900),
    [anon_sym_AT] = ACTIONS(4898),
    [anon_sym_get] = ACTIONS(4900),
    [anon_sym_set] = ACTIONS(4900),
  },
  [2324] = {
    [ts_builtin_sym_end] = ACTIONS(4902),
    [anon_sym_export] = ACTIONS(4904),
    [anon_sym_LBRACE] = ACTIONS(4902),
    [anon_sym_RBRACE] = ACTIONS(4902),
    [anon_sym_import] = ACTIONS(4904),
    [anon_sym_var] = ACTIONS(4904),
    [anon_sym_let] = ACTIONS(4904),
    [anon_sym_const] = ACTIONS(4904),
    [anon_sym_if] = ACTIONS(4904),
    [anon_sym_else] = ACTIONS(4904),
    [anon_sym_switch] = ACTIONS(4904),
    [anon_sym_for] = ACTIONS(4904),
    [anon_sym_LPAREN] = ACTIONS(4902),
    [anon_sym_while] = ACTIONS(4904),
    [anon_sym_do] = ACTIONS(4904),
    [anon_sym_try] = ACTIONS(4904),
    [anon_sym_with] = ACTIONS(4904),
    [anon_sym_break] = ACTIONS(4904),
    [anon_sym_continue] = ACTIONS(4904),
    [anon_sym_debugger] = ACTIONS(4904),
    [anon_sym_return] = ACTIONS(4904),
    [anon_sym_throw] = ACTIONS(4904),
    [anon_sym_SEMI] = ACTIONS(4902),
    [anon_sym_yield] = ACTIONS(4904),
    [anon_sym_LBRACK] = ACTIONS(4902),
    [anon_sym_LT] = ACTIONS(4902),
    [anon_sym_class] = ACTIONS(4904),
    [anon_sym_async] = ACTIONS(4904),
    [anon_sym_function] = ACTIONS(4904),
    [anon_sym_EQ_GT] = ACTIONS(4902),
    [anon_sym_new] = ACTIONS(4904),
    [anon_sym_await] = ACTIONS(4904),
    [anon_sym_PLUS] = ACTIONS(4904),
    [anon_sym_DASH] = ACTIONS(4904),
    [anon_sym_BANG] = ACTIONS(4902),
    [anon_sym_TILDE] = ACTIONS(4902),
    [anon_sym_typeof] = ACTIONS(4904),
    [anon_sym_void] = ACTIONS(4904),
    [anon_sym_delete] = ACTIONS(4904),
    [anon_sym_PLUS_PLUS] = ACTIONS(4902),
    [anon_sym_DASH_DASH] = ACTIONS(4902),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4902),
    [anon_sym_BQUOTE] = ACTIONS(4902),
    [sym_regex] = ACTIONS(4904),
    [sym_number] = ACTIONS(4902),
    [sym_identifier] = ACTIONS(4904),
    [sym_this] = ACTIONS(4904),
    [sym_super] = ACTIONS(4904),
    [sym_true] = ACTIONS(4904),
    [sym_false] = ACTIONS(4904),
    [sym_null] = ACTIONS(4904),
    [sym_undefined] = ACTIONS(4904),
    [anon_sym_AT] = ACTIONS(4902),
    [anon_sym_get] = ACTIONS(4904),
    [anon_sym_set] = ACTIONS(4904),
  },
  [2325] = {
    [ts_builtin_sym_end] = ACTIONS(4906),
    [anon_sym_export] = ACTIONS(4908),
    [anon_sym_LBRACE] = ACTIONS(4906),
    [anon_sym_RBRACE] = ACTIONS(4906),
    [anon_sym_import] = ACTIONS(4908),
    [anon_sym_var] = ACTIONS(4908),
    [anon_sym_let] = ACTIONS(4908),
    [anon_sym_const] = ACTIONS(4908),
    [anon_sym_if] = ACTIONS(4908),
    [anon_sym_else] = ACTIONS(4908),
    [anon_sym_switch] = ACTIONS(4908),
    [anon_sym_for] = ACTIONS(4908),
    [anon_sym_LPAREN] = ACTIONS(4906),
    [anon_sym_while] = ACTIONS(4908),
    [anon_sym_do] = ACTIONS(4908),
    [anon_sym_try] = ACTIONS(4908),
    [anon_sym_with] = ACTIONS(4908),
    [anon_sym_break] = ACTIONS(4908),
    [anon_sym_continue] = ACTIONS(4908),
    [anon_sym_debugger] = ACTIONS(4908),
    [anon_sym_return] = ACTIONS(4908),
    [anon_sym_throw] = ACTIONS(4908),
    [anon_sym_SEMI] = ACTIONS(4906),
    [anon_sym_yield] = ACTIONS(4908),
    [anon_sym_LBRACK] = ACTIONS(4906),
    [anon_sym_LT] = ACTIONS(4906),
    [anon_sym_class] = ACTIONS(4908),
    [anon_sym_async] = ACTIONS(4908),
    [anon_sym_function] = ACTIONS(4908),
    [anon_sym_EQ_GT] = ACTIONS(4906),
    [anon_sym_new] = ACTIONS(4908),
    [anon_sym_await] = ACTIONS(4908),
    [anon_sym_PLUS] = ACTIONS(4908),
    [anon_sym_DASH] = ACTIONS(4908),
    [anon_sym_BANG] = ACTIONS(4906),
    [anon_sym_TILDE] = ACTIONS(4906),
    [anon_sym_typeof] = ACTIONS(4908),
    [anon_sym_void] = ACTIONS(4908),
    [anon_sym_delete] = ACTIONS(4908),
    [anon_sym_PLUS_PLUS] = ACTIONS(4906),
    [anon_sym_DASH_DASH] = ACTIONS(4906),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4906),
    [anon_sym_BQUOTE] = ACTIONS(4906),
    [sym_regex] = ACTIONS(4908),
    [sym_number] = ACTIONS(4906),
    [sym_identifier] = ACTIONS(4908),
    [sym_this] = ACTIONS(4908),
    [sym_super] = ACTIONS(4908),
    [sym_true] = ACTIONS(4908),
    [sym_false] = ACTIONS(4908),
    [sym_null] = ACTIONS(4908),
    [sym_undefined] = ACTIONS(4908),
    [anon_sym_AT] = ACTIONS(4906),
    [anon_sym_get] = ACTIONS(4908),
    [anon_sym_set] = ACTIONS(4908),
  },
  [2326] = {
    [ts_builtin_sym_end] = ACTIONS(4910),
    [anon_sym_export] = ACTIONS(4912),
    [anon_sym_LBRACE] = ACTIONS(4910),
    [anon_sym_RBRACE] = ACTIONS(4910),
    [anon_sym_import] = ACTIONS(4912),
    [anon_sym_var] = ACTIONS(4912),
    [anon_sym_let] = ACTIONS(4912),
    [anon_sym_const] = ACTIONS(4912),
    [anon_sym_if] = ACTIONS(4912),
    [anon_sym_else] = ACTIONS(4912),
    [anon_sym_switch] = ACTIONS(4912),
    [anon_sym_for] = ACTIONS(4912),
    [anon_sym_LPAREN] = ACTIONS(4910),
    [anon_sym_while] = ACTIONS(4912),
    [anon_sym_do] = ACTIONS(4912),
    [anon_sym_try] = ACTIONS(4912),
    [anon_sym_with] = ACTIONS(4912),
    [anon_sym_break] = ACTIONS(4912),
    [anon_sym_continue] = ACTIONS(4912),
    [anon_sym_debugger] = ACTIONS(4912),
    [anon_sym_return] = ACTIONS(4912),
    [anon_sym_throw] = ACTIONS(4912),
    [anon_sym_SEMI] = ACTIONS(4910),
    [anon_sym_yield] = ACTIONS(4912),
    [anon_sym_LBRACK] = ACTIONS(4910),
    [anon_sym_LT] = ACTIONS(4910),
    [anon_sym_class] = ACTIONS(4912),
    [anon_sym_async] = ACTIONS(4912),
    [anon_sym_function] = ACTIONS(4912),
    [anon_sym_EQ_GT] = ACTIONS(4910),
    [anon_sym_new] = ACTIONS(4912),
    [anon_sym_await] = ACTIONS(4912),
    [anon_sym_PLUS] = ACTIONS(4912),
    [anon_sym_DASH] = ACTIONS(4912),
    [anon_sym_BANG] = ACTIONS(4910),
    [anon_sym_TILDE] = ACTIONS(4910),
    [anon_sym_typeof] = ACTIONS(4912),
    [anon_sym_void] = ACTIONS(4912),
    [anon_sym_delete] = ACTIONS(4912),
    [anon_sym_PLUS_PLUS] = ACTIONS(4910),
    [anon_sym_DASH_DASH] = ACTIONS(4910),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4910),
    [anon_sym_BQUOTE] = ACTIONS(4910),
    [sym_regex] = ACTIONS(4912),
    [sym_number] = ACTIONS(4910),
    [sym_identifier] = ACTIONS(4912),
    [sym_this] = ACTIONS(4912),
    [sym_super] = ACTIONS(4912),
    [sym_true] = ACTIONS(4912),
    [sym_false] = ACTIONS(4912),
    [sym_null] = ACTIONS(4912),
    [sym_undefined] = ACTIONS(4912),
    [anon_sym_AT] = ACTIONS(4910),
    [anon_sym_get] = ACTIONS(4912),
    [anon_sym_set] = ACTIONS(4912),
  },
  [2327] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_default] = ACTIONS(2066),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_case] = ACTIONS(2066),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2064),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2064),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [2328] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(5094),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2329] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_default] = ACTIONS(3105),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_case] = ACTIONS(3105),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3103),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3103),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [2330] = {
    [sym_statement_block] = STATE(2430),
    [anon_sym_LBRACE] = ACTIONS(4790),
    [sym_comment] = ACTIONS(71),
  },
  [2331] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_default] = ACTIONS(3109),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_case] = ACTIONS(3109),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3107),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3107),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [2332] = {
    [sym_statement_block] = STATE(2431),
    [anon_sym_LBRACE] = ACTIONS(4790),
    [sym_comment] = ACTIONS(71),
  },
  [2333] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_case] = ACTIONS(3117),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3115),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3115),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [2334] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2335] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5096),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2336] = {
    [sym_class_body] = STATE(2433),
    [anon_sym_LBRACE] = ACTIONS(4784),
    [sym_comment] = ACTIONS(71),
  },
  [2337] = {
    [sym__automatic_semicolon] = ACTIONS(5098),
    [anon_sym_SEMI] = ACTIONS(5098),
    [sym_comment] = ACTIONS(71),
  },
  [2338] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(5098),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(5098),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2339] = {
    [sym_class_heritage] = STATE(2436),
    [sym_class_body] = STATE(2437),
    [anon_sym_LBRACE] = ACTIONS(5100),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2340] = {
    [sym_formal_parameters] = STATE(2439),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5102),
  },
  [2341] = {
    [sym_formal_parameters] = STATE(2441),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5104),
  },
  [2342] = {
    [sym_formal_parameters] = STATE(2439),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2343] = {
    [sym_statement_block] = STATE(2443),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [sym_comment] = ACTIONS(71),
  },
  [2344] = {
    [anon_sym_export] = ACTIONS(1399),
    [anon_sym_default] = ACTIONS(1399),
    [anon_sym_LBRACE] = ACTIONS(1397),
    [anon_sym_RBRACE] = ACTIONS(1397),
    [anon_sym_import] = ACTIONS(1399),
    [anon_sym_var] = ACTIONS(1399),
    [anon_sym_let] = ACTIONS(1399),
    [anon_sym_const] = ACTIONS(1399),
    [anon_sym_if] = ACTIONS(1399),
    [anon_sym_else] = ACTIONS(1399),
    [anon_sym_switch] = ACTIONS(1399),
    [anon_sym_for] = ACTIONS(1399),
    [anon_sym_LPAREN] = ACTIONS(1397),
    [anon_sym_while] = ACTIONS(1399),
    [anon_sym_do] = ACTIONS(1399),
    [anon_sym_try] = ACTIONS(1399),
    [anon_sym_with] = ACTIONS(1399),
    [anon_sym_break] = ACTIONS(1399),
    [anon_sym_continue] = ACTIONS(1399),
    [anon_sym_debugger] = ACTIONS(1399),
    [anon_sym_return] = ACTIONS(1399),
    [anon_sym_throw] = ACTIONS(1399),
    [anon_sym_SEMI] = ACTIONS(1397),
    [anon_sym_case] = ACTIONS(1399),
    [anon_sym_yield] = ACTIONS(1399),
    [anon_sym_LBRACK] = ACTIONS(1397),
    [anon_sym_LT] = ACTIONS(1397),
    [anon_sym_class] = ACTIONS(1399),
    [anon_sym_async] = ACTIONS(1399),
    [anon_sym_function] = ACTIONS(1399),
    [anon_sym_EQ_GT] = ACTIONS(1397),
    [anon_sym_new] = ACTIONS(1399),
    [anon_sym_await] = ACTIONS(1399),
    [anon_sym_PLUS] = ACTIONS(1399),
    [anon_sym_DASH] = ACTIONS(1399),
    [anon_sym_BANG] = ACTIONS(1397),
    [anon_sym_TILDE] = ACTIONS(1397),
    [anon_sym_typeof] = ACTIONS(1399),
    [anon_sym_void] = ACTIONS(1399),
    [anon_sym_delete] = ACTIONS(1399),
    [anon_sym_PLUS_PLUS] = ACTIONS(1397),
    [anon_sym_DASH_DASH] = ACTIONS(1397),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1397),
    [anon_sym_BQUOTE] = ACTIONS(1397),
    [sym_regex] = ACTIONS(1399),
    [sym_number] = ACTIONS(1397),
    [sym_identifier] = ACTIONS(1399),
    [sym_this] = ACTIONS(1399),
    [sym_super] = ACTIONS(1399),
    [sym_true] = ACTIONS(1399),
    [sym_false] = ACTIONS(1399),
    [sym_null] = ACTIONS(1399),
    [sym_undefined] = ACTIONS(1399),
    [anon_sym_AT] = ACTIONS(1397),
    [anon_sym_get] = ACTIONS(1399),
    [anon_sym_set] = ACTIONS(1399),
  },
  [2345] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5108),
  },
  [2346] = {
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2347] = {
    [anon_sym_export] = ACTIONS(1629),
    [anon_sym_default] = ACTIONS(1629),
    [anon_sym_LBRACE] = ACTIONS(1627),
    [anon_sym_RBRACE] = ACTIONS(1627),
    [anon_sym_import] = ACTIONS(1629),
    [anon_sym_var] = ACTIONS(1629),
    [anon_sym_let] = ACTIONS(1629),
    [anon_sym_const] = ACTIONS(1629),
    [anon_sym_if] = ACTIONS(1629),
    [anon_sym_else] = ACTIONS(1629),
    [anon_sym_switch] = ACTIONS(1629),
    [anon_sym_for] = ACTIONS(1629),
    [anon_sym_LPAREN] = ACTIONS(1627),
    [anon_sym_while] = ACTIONS(1629),
    [anon_sym_do] = ACTIONS(1629),
    [anon_sym_try] = ACTIONS(1629),
    [anon_sym_with] = ACTIONS(1629),
    [anon_sym_break] = ACTIONS(1629),
    [anon_sym_continue] = ACTIONS(1629),
    [anon_sym_debugger] = ACTIONS(1629),
    [anon_sym_return] = ACTIONS(1629),
    [anon_sym_throw] = ACTIONS(1629),
    [anon_sym_SEMI] = ACTIONS(1627),
    [anon_sym_case] = ACTIONS(1629),
    [anon_sym_yield] = ACTIONS(1629),
    [anon_sym_LBRACK] = ACTIONS(1627),
    [anon_sym_LT] = ACTIONS(1627),
    [anon_sym_class] = ACTIONS(1629),
    [anon_sym_async] = ACTIONS(1629),
    [anon_sym_function] = ACTIONS(1629),
    [anon_sym_EQ_GT] = ACTIONS(1627),
    [anon_sym_new] = ACTIONS(1629),
    [anon_sym_await] = ACTIONS(1629),
    [anon_sym_PLUS] = ACTIONS(1629),
    [anon_sym_DASH] = ACTIONS(1629),
    [anon_sym_BANG] = ACTIONS(1627),
    [anon_sym_TILDE] = ACTIONS(1627),
    [anon_sym_typeof] = ACTIONS(1629),
    [anon_sym_void] = ACTIONS(1629),
    [anon_sym_delete] = ACTIONS(1629),
    [anon_sym_PLUS_PLUS] = ACTIONS(1627),
    [anon_sym_DASH_DASH] = ACTIONS(1627),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1627),
    [anon_sym_BQUOTE] = ACTIONS(1627),
    [sym_regex] = ACTIONS(1629),
    [sym_number] = ACTIONS(1627),
    [sym_identifier] = ACTIONS(1629),
    [sym_this] = ACTIONS(1629),
    [sym_super] = ACTIONS(1629),
    [sym_true] = ACTIONS(1629),
    [sym_false] = ACTIONS(1629),
    [sym_null] = ACTIONS(1629),
    [sym_undefined] = ACTIONS(1629),
    [anon_sym_AT] = ACTIONS(1627),
    [anon_sym_get] = ACTIONS(1629),
    [anon_sym_set] = ACTIONS(1629),
  },
  [2348] = {
    [sym__automatic_semicolon] = ACTIONS(5110),
    [anon_sym_SEMI] = ACTIONS(5110),
    [sym_comment] = ACTIONS(71),
  },
  [2349] = {
    [anon_sym_export] = ACTIONS(1643),
    [anon_sym_default] = ACTIONS(1643),
    [anon_sym_LBRACE] = ACTIONS(1641),
    [anon_sym_RBRACE] = ACTIONS(1641),
    [anon_sym_import] = ACTIONS(1643),
    [anon_sym_var] = ACTIONS(1643),
    [anon_sym_let] = ACTIONS(1643),
    [anon_sym_const] = ACTIONS(1643),
    [anon_sym_if] = ACTIONS(1643),
    [anon_sym_else] = ACTIONS(1643),
    [anon_sym_switch] = ACTIONS(1643),
    [anon_sym_for] = ACTIONS(1643),
    [anon_sym_LPAREN] = ACTIONS(1641),
    [anon_sym_while] = ACTIONS(1643),
    [anon_sym_do] = ACTIONS(1643),
    [anon_sym_try] = ACTIONS(1643),
    [anon_sym_with] = ACTIONS(1643),
    [anon_sym_break] = ACTIONS(1643),
    [anon_sym_continue] = ACTIONS(1643),
    [anon_sym_debugger] = ACTIONS(1643),
    [anon_sym_return] = ACTIONS(1643),
    [anon_sym_throw] = ACTIONS(1643),
    [anon_sym_SEMI] = ACTIONS(1641),
    [anon_sym_case] = ACTIONS(1643),
    [anon_sym_yield] = ACTIONS(1643),
    [anon_sym_LBRACK] = ACTIONS(1641),
    [anon_sym_LT] = ACTIONS(1641),
    [anon_sym_class] = ACTIONS(1643),
    [anon_sym_async] = ACTIONS(1643),
    [anon_sym_function] = ACTIONS(1643),
    [anon_sym_EQ_GT] = ACTIONS(1641),
    [anon_sym_new] = ACTIONS(1643),
    [anon_sym_await] = ACTIONS(1643),
    [anon_sym_PLUS] = ACTIONS(1643),
    [anon_sym_DASH] = ACTIONS(1643),
    [anon_sym_BANG] = ACTIONS(1641),
    [anon_sym_TILDE] = ACTIONS(1641),
    [anon_sym_typeof] = ACTIONS(1643),
    [anon_sym_void] = ACTIONS(1643),
    [anon_sym_delete] = ACTIONS(1643),
    [anon_sym_PLUS_PLUS] = ACTIONS(1641),
    [anon_sym_DASH_DASH] = ACTIONS(1641),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1641),
    [anon_sym_BQUOTE] = ACTIONS(1641),
    [sym_regex] = ACTIONS(1643),
    [sym_number] = ACTIONS(1641),
    [sym_identifier] = ACTIONS(1643),
    [sym_this] = ACTIONS(1643),
    [sym_super] = ACTIONS(1643),
    [sym_true] = ACTIONS(1643),
    [sym_false] = ACTIONS(1643),
    [sym_null] = ACTIONS(1643),
    [sym_undefined] = ACTIONS(1643),
    [anon_sym_AT] = ACTIONS(1641),
    [anon_sym_get] = ACTIONS(1643),
    [anon_sym_set] = ACTIONS(1643),
  },
  [2350] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(5112),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(5112),
    [sym_comment] = ACTIONS(71),
  },
  [2351] = {
    [anon_sym_export] = ACTIONS(1653),
    [anon_sym_default] = ACTIONS(1653),
    [anon_sym_LBRACE] = ACTIONS(1651),
    [anon_sym_RBRACE] = ACTIONS(1651),
    [anon_sym_import] = ACTIONS(1653),
    [anon_sym_var] = ACTIONS(1653),
    [anon_sym_let] = ACTIONS(1653),
    [anon_sym_const] = ACTIONS(1653),
    [anon_sym_if] = ACTIONS(1653),
    [anon_sym_else] = ACTIONS(1653),
    [anon_sym_switch] = ACTIONS(1653),
    [anon_sym_for] = ACTIONS(1653),
    [anon_sym_LPAREN] = ACTIONS(1651),
    [anon_sym_while] = ACTIONS(1653),
    [anon_sym_do] = ACTIONS(1653),
    [anon_sym_try] = ACTIONS(1653),
    [anon_sym_with] = ACTIONS(1653),
    [anon_sym_break] = ACTIONS(1653),
    [anon_sym_continue] = ACTIONS(1653),
    [anon_sym_debugger] = ACTIONS(1653),
    [anon_sym_return] = ACTIONS(1653),
    [anon_sym_throw] = ACTIONS(1653),
    [anon_sym_SEMI] = ACTIONS(1651),
    [anon_sym_case] = ACTIONS(1653),
    [anon_sym_yield] = ACTIONS(1653),
    [anon_sym_LBRACK] = ACTIONS(1651),
    [anon_sym_LT] = ACTIONS(1651),
    [anon_sym_class] = ACTIONS(1653),
    [anon_sym_async] = ACTIONS(1653),
    [anon_sym_function] = ACTIONS(1653),
    [anon_sym_EQ_GT] = ACTIONS(1651),
    [anon_sym_new] = ACTIONS(1653),
    [anon_sym_await] = ACTIONS(1653),
    [anon_sym_PLUS] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1653),
    [anon_sym_BANG] = ACTIONS(1651),
    [anon_sym_TILDE] = ACTIONS(1651),
    [anon_sym_typeof] = ACTIONS(1653),
    [anon_sym_void] = ACTIONS(1653),
    [anon_sym_delete] = ACTIONS(1653),
    [anon_sym_PLUS_PLUS] = ACTIONS(1651),
    [anon_sym_DASH_DASH] = ACTIONS(1651),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1651),
    [anon_sym_BQUOTE] = ACTIONS(1651),
    [sym_regex] = ACTIONS(1653),
    [sym_number] = ACTIONS(1651),
    [sym_identifier] = ACTIONS(1653),
    [sym_this] = ACTIONS(1653),
    [sym_super] = ACTIONS(1653),
    [sym_true] = ACTIONS(1653),
    [sym_false] = ACTIONS(1653),
    [sym_null] = ACTIONS(1653),
    [sym_undefined] = ACTIONS(1653),
    [anon_sym_AT] = ACTIONS(1651),
    [anon_sym_get] = ACTIONS(1653),
    [anon_sym_set] = ACTIONS(1653),
  },
  [2352] = {
    [aux_sym_variable_declaration_repeat1] = STATE(785),
    [sym__automatic_semicolon] = ACTIONS(5114),
    [anon_sym_COMMA] = ACTIONS(608),
    [anon_sym_SEMI] = ACTIONS(5114),
    [sym_comment] = ACTIONS(71),
  },
  [2353] = {
    [anon_sym_export] = ACTIONS(1705),
    [anon_sym_default] = ACTIONS(1705),
    [anon_sym_LBRACE] = ACTIONS(1703),
    [anon_sym_RBRACE] = ACTIONS(1703),
    [anon_sym_import] = ACTIONS(1705),
    [anon_sym_var] = ACTIONS(1705),
    [anon_sym_let] = ACTIONS(1705),
    [anon_sym_const] = ACTIONS(1705),
    [anon_sym_if] = ACTIONS(1705),
    [anon_sym_else] = ACTIONS(5116),
    [anon_sym_switch] = ACTIONS(1705),
    [anon_sym_for] = ACTIONS(1705),
    [anon_sym_LPAREN] = ACTIONS(1703),
    [anon_sym_while] = ACTIONS(1705),
    [anon_sym_do] = ACTIONS(1705),
    [anon_sym_try] = ACTIONS(1705),
    [anon_sym_with] = ACTIONS(1705),
    [anon_sym_break] = ACTIONS(1705),
    [anon_sym_continue] = ACTIONS(1705),
    [anon_sym_debugger] = ACTIONS(1705),
    [anon_sym_return] = ACTIONS(1705),
    [anon_sym_throw] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1703),
    [anon_sym_case] = ACTIONS(1705),
    [anon_sym_yield] = ACTIONS(1705),
    [anon_sym_LBRACK] = ACTIONS(1703),
    [anon_sym_LT] = ACTIONS(1703),
    [anon_sym_class] = ACTIONS(1705),
    [anon_sym_async] = ACTIONS(1705),
    [anon_sym_function] = ACTIONS(1705),
    [anon_sym_EQ_GT] = ACTIONS(1703),
    [anon_sym_new] = ACTIONS(1705),
    [anon_sym_await] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_BANG] = ACTIONS(1703),
    [anon_sym_TILDE] = ACTIONS(1703),
    [anon_sym_typeof] = ACTIONS(1705),
    [anon_sym_void] = ACTIONS(1705),
    [anon_sym_delete] = ACTIONS(1705),
    [anon_sym_PLUS_PLUS] = ACTIONS(1703),
    [anon_sym_DASH_DASH] = ACTIONS(1703),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1703),
    [anon_sym_BQUOTE] = ACTIONS(1703),
    [sym_regex] = ACTIONS(1705),
    [sym_number] = ACTIONS(1703),
    [sym_identifier] = ACTIONS(1705),
    [sym_this] = ACTIONS(1705),
    [sym_super] = ACTIONS(1705),
    [sym_true] = ACTIONS(1705),
    [sym_false] = ACTIONS(1705),
    [sym_null] = ACTIONS(1705),
    [sym_undefined] = ACTIONS(1705),
    [anon_sym_AT] = ACTIONS(1703),
    [anon_sym_get] = ACTIONS(1705),
    [anon_sym_set] = ACTIONS(1705),
  },
  [2354] = {
    [sym_switch_case] = STATE(2450),
    [sym_switch_default] = STATE(2450),
    [aux_sym_switch_body_repeat1] = STATE(2450),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(5118),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [2355] = {
    [anon_sym_export] = ACTIONS(1725),
    [anon_sym_default] = ACTIONS(1725),
    [anon_sym_LBRACE] = ACTIONS(1723),
    [anon_sym_RBRACE] = ACTIONS(1723),
    [anon_sym_import] = ACTIONS(1725),
    [anon_sym_var] = ACTIONS(1725),
    [anon_sym_let] = ACTIONS(1725),
    [anon_sym_const] = ACTIONS(1725),
    [anon_sym_if] = ACTIONS(1725),
    [anon_sym_else] = ACTIONS(1725),
    [anon_sym_switch] = ACTIONS(1725),
    [anon_sym_for] = ACTIONS(1725),
    [anon_sym_LPAREN] = ACTIONS(1723),
    [anon_sym_while] = ACTIONS(1725),
    [anon_sym_do] = ACTIONS(1725),
    [anon_sym_try] = ACTIONS(1725),
    [anon_sym_with] = ACTIONS(1725),
    [anon_sym_break] = ACTIONS(1725),
    [anon_sym_continue] = ACTIONS(1725),
    [anon_sym_debugger] = ACTIONS(1725),
    [anon_sym_return] = ACTIONS(1725),
    [anon_sym_throw] = ACTIONS(1725),
    [anon_sym_SEMI] = ACTIONS(1723),
    [anon_sym_case] = ACTIONS(1725),
    [anon_sym_yield] = ACTIONS(1725),
    [anon_sym_LBRACK] = ACTIONS(1723),
    [anon_sym_LT] = ACTIONS(1723),
    [anon_sym_class] = ACTIONS(1725),
    [anon_sym_async] = ACTIONS(1725),
    [anon_sym_function] = ACTIONS(1725),
    [anon_sym_EQ_GT] = ACTIONS(1723),
    [anon_sym_new] = ACTIONS(1725),
    [anon_sym_await] = ACTIONS(1725),
    [anon_sym_PLUS] = ACTIONS(1725),
    [anon_sym_DASH] = ACTIONS(1725),
    [anon_sym_BANG] = ACTIONS(1723),
    [anon_sym_TILDE] = ACTIONS(1723),
    [anon_sym_typeof] = ACTIONS(1725),
    [anon_sym_void] = ACTIONS(1725),
    [anon_sym_delete] = ACTIONS(1725),
    [anon_sym_PLUS_PLUS] = ACTIONS(1723),
    [anon_sym_DASH_DASH] = ACTIONS(1723),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1723),
    [anon_sym_BQUOTE] = ACTIONS(1723),
    [sym_regex] = ACTIONS(1725),
    [sym_number] = ACTIONS(1723),
    [sym_identifier] = ACTIONS(1725),
    [sym_this] = ACTIONS(1725),
    [sym_super] = ACTIONS(1725),
    [sym_true] = ACTIONS(1725),
    [sym_false] = ACTIONS(1725),
    [sym_null] = ACTIONS(1725),
    [sym_undefined] = ACTIONS(1725),
    [anon_sym_AT] = ACTIONS(1723),
    [anon_sym_get] = ACTIONS(1725),
    [anon_sym_set] = ACTIONS(1725),
  },
  [2356] = {
    [sym_variable_declarator] = STATE(835),
    [sym_object] = STATE(2452),
    [sym_array] = STATE(2453),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5120),
  },
  [2357] = {
    [sym_variable_declarator] = STATE(838),
    [sym_object] = STATE(2452),
    [sym_array] = STATE(2453),
    [anon_sym_LBRACE] = ACTIONS(141),
    [anon_sym_LBRACK] = ACTIONS(143),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5120),
  },
  [2358] = {
    [sym__initializer] = STATE(214),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(346),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(5122),
    [anon_sym_of] = ACTIONS(5124),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_EQ_GT] = ACTIONS(304),
    [anon_sym_PLUS_EQ] = ACTIONS(354),
    [anon_sym_DASH_EQ] = ACTIONS(354),
    [anon_sym_STAR_EQ] = ACTIONS(354),
    [anon_sym_SLASH_EQ] = ACTIONS(354),
    [anon_sym_PERCENT_EQ] = ACTIONS(354),
    [anon_sym_CARET_EQ] = ACTIONS(354),
    [anon_sym_AMP_EQ] = ACTIONS(354),
    [anon_sym_PIPE_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_GT_GT_GT_EQ] = ACTIONS(354),
    [anon_sym_LT_LT_EQ] = ACTIONS(354),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(346),
    [anon_sym_LT_LT] = ACTIONS(346),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(346),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(346),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2359] = {
    [sym_expression_statement] = STATE(2456),
    [sym_empty_statement] = STATE(2456),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(395),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(40),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(40),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(395),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(672),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(233),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2360] = {
    [sym__initializer] = STATE(230),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(5126),
    [anon_sym_of] = ACTIONS(5128),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2361] = {
    [sym__initializer] = STATE(231),
    [sym__automatic_semicolon] = ACTIONS(344),
    [anon_sym_STAR] = ACTIONS(344),
    [anon_sym_COMMA] = ACTIONS(344),
    [anon_sym_LPAREN] = ACTIONS(344),
    [anon_sym_in] = ACTIONS(5130),
    [anon_sym_of] = ACTIONS(5132),
    [anon_sym_SEMI] = ACTIONS(344),
    [anon_sym_LBRACK] = ACTIONS(344),
    [anon_sym_LT] = ACTIONS(346),
    [anon_sym_GT] = ACTIONS(346),
    [anon_sym_SLASH] = ACTIONS(346),
    [anon_sym_DOT] = ACTIONS(344),
    [anon_sym_EQ] = ACTIONS(300),
    [anon_sym_QMARK] = ACTIONS(344),
    [anon_sym_AMP_AMP] = ACTIONS(344),
    [anon_sym_PIPE_PIPE] = ACTIONS(344),
    [anon_sym_GT_GT] = ACTIONS(346),
    [anon_sym_GT_GT_GT] = ACTIONS(344),
    [anon_sym_LT_LT] = ACTIONS(344),
    [anon_sym_AMP] = ACTIONS(346),
    [anon_sym_CARET] = ACTIONS(344),
    [anon_sym_PIPE] = ACTIONS(346),
    [anon_sym_PLUS] = ACTIONS(346),
    [anon_sym_DASH] = ACTIONS(346),
    [anon_sym_PERCENT] = ACTIONS(344),
    [anon_sym_LT_EQ] = ACTIONS(344),
    [anon_sym_EQ_EQ] = ACTIONS(346),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(344),
    [anon_sym_BANG_EQ] = ACTIONS(346),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(344),
    [anon_sym_GT_EQ] = ACTIONS(344),
    [anon_sym_instanceof] = ACTIONS(344),
    [anon_sym_PLUS_PLUS] = ACTIONS(344),
    [anon_sym_DASH_DASH] = ACTIONS(344),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(344),
  },
  [2362] = {
    [anon_sym_export] = ACTIONS(1863),
    [anon_sym_default] = ACTIONS(1863),
    [anon_sym_LBRACE] = ACTIONS(1861),
    [anon_sym_RBRACE] = ACTIONS(1861),
    [anon_sym_import] = ACTIONS(1863),
    [anon_sym_var] = ACTIONS(1863),
    [anon_sym_let] = ACTIONS(1863),
    [anon_sym_const] = ACTIONS(1863),
    [anon_sym_if] = ACTIONS(1863),
    [anon_sym_else] = ACTIONS(1863),
    [anon_sym_switch] = ACTIONS(1863),
    [anon_sym_for] = ACTIONS(1863),
    [anon_sym_LPAREN] = ACTIONS(1861),
    [anon_sym_while] = ACTIONS(1863),
    [anon_sym_do] = ACTIONS(1863),
    [anon_sym_try] = ACTIONS(1863),
    [anon_sym_with] = ACTIONS(1863),
    [anon_sym_break] = ACTIONS(1863),
    [anon_sym_continue] = ACTIONS(1863),
    [anon_sym_debugger] = ACTIONS(1863),
    [anon_sym_return] = ACTIONS(1863),
    [anon_sym_throw] = ACTIONS(1863),
    [anon_sym_SEMI] = ACTIONS(1861),
    [anon_sym_case] = ACTIONS(1863),
    [anon_sym_yield] = ACTIONS(1863),
    [anon_sym_LBRACK] = ACTIONS(1861),
    [anon_sym_LT] = ACTIONS(1861),
    [anon_sym_class] = ACTIONS(1863),
    [anon_sym_async] = ACTIONS(1863),
    [anon_sym_function] = ACTIONS(1863),
    [anon_sym_EQ_GT] = ACTIONS(1861),
    [anon_sym_new] = ACTIONS(1863),
    [anon_sym_await] = ACTIONS(1863),
    [anon_sym_PLUS] = ACTIONS(1863),
    [anon_sym_DASH] = ACTIONS(1863),
    [anon_sym_BANG] = ACTIONS(1861),
    [anon_sym_TILDE] = ACTIONS(1861),
    [anon_sym_typeof] = ACTIONS(1863),
    [anon_sym_void] = ACTIONS(1863),
    [anon_sym_delete] = ACTIONS(1863),
    [anon_sym_PLUS_PLUS] = ACTIONS(1861),
    [anon_sym_DASH_DASH] = ACTIONS(1861),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1861),
    [anon_sym_BQUOTE] = ACTIONS(1861),
    [sym_regex] = ACTIONS(1863),
    [sym_number] = ACTIONS(1861),
    [sym_identifier] = ACTIONS(1863),
    [sym_this] = ACTIONS(1863),
    [sym_super] = ACTIONS(1863),
    [sym_true] = ACTIONS(1863),
    [sym_false] = ACTIONS(1863),
    [sym_null] = ACTIONS(1863),
    [sym_undefined] = ACTIONS(1863),
    [anon_sym_AT] = ACTIONS(1861),
    [anon_sym_get] = ACTIONS(1863),
    [anon_sym_set] = ACTIONS(1863),
  },
  [2363] = {
    [sym_parenthesized_expression] = STATE(2461),
    [anon_sym_LPAREN] = ACTIONS(1929),
    [sym_comment] = ACTIONS(71),
  },
  [2364] = {
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_catch] = ACTIONS(484),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2365] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5134),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2366] = {
    [sym_statement_block] = STATE(2465),
    [anon_sym_LBRACE] = ACTIONS(5136),
    [anon_sym_LPAREN] = ACTIONS(5138),
    [sym_comment] = ACTIONS(71),
  },
  [2367] = {
    [sym_statement_block] = STATE(2467),
    [anon_sym_LBRACE] = ACTIONS(5140),
    [sym_comment] = ACTIONS(71),
  },
  [2368] = {
    [sym_finally_clause] = STATE(2468),
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_default] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_else] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_case] = ACTIONS(1943),
    [anon_sym_finally] = ACTIONS(5016),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [2369] = {
    [anon_sym_export] = ACTIONS(1943),
    [anon_sym_default] = ACTIONS(1943),
    [anon_sym_LBRACE] = ACTIONS(1941),
    [anon_sym_RBRACE] = ACTIONS(1941),
    [anon_sym_import] = ACTIONS(1943),
    [anon_sym_var] = ACTIONS(1943),
    [anon_sym_let] = ACTIONS(1943),
    [anon_sym_const] = ACTIONS(1943),
    [anon_sym_if] = ACTIONS(1943),
    [anon_sym_else] = ACTIONS(1943),
    [anon_sym_switch] = ACTIONS(1943),
    [anon_sym_for] = ACTIONS(1943),
    [anon_sym_LPAREN] = ACTIONS(1941),
    [anon_sym_while] = ACTIONS(1943),
    [anon_sym_do] = ACTIONS(1943),
    [anon_sym_try] = ACTIONS(1943),
    [anon_sym_with] = ACTIONS(1943),
    [anon_sym_break] = ACTIONS(1943),
    [anon_sym_continue] = ACTIONS(1943),
    [anon_sym_debugger] = ACTIONS(1943),
    [anon_sym_return] = ACTIONS(1943),
    [anon_sym_throw] = ACTIONS(1943),
    [anon_sym_SEMI] = ACTIONS(1941),
    [anon_sym_case] = ACTIONS(1943),
    [anon_sym_yield] = ACTIONS(1943),
    [anon_sym_LBRACK] = ACTIONS(1941),
    [anon_sym_LT] = ACTIONS(1941),
    [anon_sym_class] = ACTIONS(1943),
    [anon_sym_async] = ACTIONS(1943),
    [anon_sym_function] = ACTIONS(1943),
    [anon_sym_EQ_GT] = ACTIONS(1941),
    [anon_sym_new] = ACTIONS(1943),
    [anon_sym_await] = ACTIONS(1943),
    [anon_sym_PLUS] = ACTIONS(1943),
    [anon_sym_DASH] = ACTIONS(1943),
    [anon_sym_BANG] = ACTIONS(1941),
    [anon_sym_TILDE] = ACTIONS(1941),
    [anon_sym_typeof] = ACTIONS(1943),
    [anon_sym_void] = ACTIONS(1943),
    [anon_sym_delete] = ACTIONS(1943),
    [anon_sym_PLUS_PLUS] = ACTIONS(1941),
    [anon_sym_DASH_DASH] = ACTIONS(1941),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1941),
    [anon_sym_BQUOTE] = ACTIONS(1941),
    [sym_regex] = ACTIONS(1943),
    [sym_number] = ACTIONS(1941),
    [sym_identifier] = ACTIONS(1943),
    [sym_this] = ACTIONS(1943),
    [sym_super] = ACTIONS(1943),
    [sym_true] = ACTIONS(1943),
    [sym_false] = ACTIONS(1943),
    [sym_null] = ACTIONS(1943),
    [sym_undefined] = ACTIONS(1943),
    [anon_sym_AT] = ACTIONS(1941),
    [anon_sym_get] = ACTIONS(1943),
    [anon_sym_set] = ACTIONS(1943),
  },
  [2370] = {
    [anon_sym_export] = ACTIONS(1947),
    [anon_sym_default] = ACTIONS(1947),
    [anon_sym_LBRACE] = ACTIONS(1945),
    [anon_sym_RBRACE] = ACTIONS(1945),
    [anon_sym_import] = ACTIONS(1947),
    [anon_sym_var] = ACTIONS(1947),
    [anon_sym_let] = ACTIONS(1947),
    [anon_sym_const] = ACTIONS(1947),
    [anon_sym_if] = ACTIONS(1947),
    [anon_sym_else] = ACTIONS(1947),
    [anon_sym_switch] = ACTIONS(1947),
    [anon_sym_for] = ACTIONS(1947),
    [anon_sym_LPAREN] = ACTIONS(1945),
    [anon_sym_while] = ACTIONS(1947),
    [anon_sym_do] = ACTIONS(1947),
    [anon_sym_try] = ACTIONS(1947),
    [anon_sym_with] = ACTIONS(1947),
    [anon_sym_break] = ACTIONS(1947),
    [anon_sym_continue] = ACTIONS(1947),
    [anon_sym_debugger] = ACTIONS(1947),
    [anon_sym_return] = ACTIONS(1947),
    [anon_sym_throw] = ACTIONS(1947),
    [anon_sym_SEMI] = ACTIONS(1945),
    [anon_sym_case] = ACTIONS(1947),
    [anon_sym_yield] = ACTIONS(1947),
    [anon_sym_LBRACK] = ACTIONS(1945),
    [anon_sym_LT] = ACTIONS(1945),
    [anon_sym_class] = ACTIONS(1947),
    [anon_sym_async] = ACTIONS(1947),
    [anon_sym_function] = ACTIONS(1947),
    [anon_sym_EQ_GT] = ACTIONS(1945),
    [anon_sym_new] = ACTIONS(1947),
    [anon_sym_await] = ACTIONS(1947),
    [anon_sym_PLUS] = ACTIONS(1947),
    [anon_sym_DASH] = ACTIONS(1947),
    [anon_sym_BANG] = ACTIONS(1945),
    [anon_sym_TILDE] = ACTIONS(1945),
    [anon_sym_typeof] = ACTIONS(1947),
    [anon_sym_void] = ACTIONS(1947),
    [anon_sym_delete] = ACTIONS(1947),
    [anon_sym_PLUS_PLUS] = ACTIONS(1945),
    [anon_sym_DASH_DASH] = ACTIONS(1945),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1945),
    [anon_sym_BQUOTE] = ACTIONS(1945),
    [sym_regex] = ACTIONS(1947),
    [sym_number] = ACTIONS(1945),
    [sym_identifier] = ACTIONS(1947),
    [sym_this] = ACTIONS(1947),
    [sym_super] = ACTIONS(1947),
    [sym_true] = ACTIONS(1947),
    [sym_false] = ACTIONS(1947),
    [sym_null] = ACTIONS(1947),
    [sym_undefined] = ACTIONS(1947),
    [anon_sym_AT] = ACTIONS(1945),
    [anon_sym_get] = ACTIONS(1947),
    [anon_sym_set] = ACTIONS(1947),
  },
  [2371] = {
    [anon_sym_export] = ACTIONS(1951),
    [anon_sym_default] = ACTIONS(1951),
    [anon_sym_LBRACE] = ACTIONS(1949),
    [anon_sym_RBRACE] = ACTIONS(1949),
    [anon_sym_import] = ACTIONS(1951),
    [anon_sym_var] = ACTIONS(1951),
    [anon_sym_let] = ACTIONS(1951),
    [anon_sym_const] = ACTIONS(1951),
    [anon_sym_if] = ACTIONS(1951),
    [anon_sym_else] = ACTIONS(1951),
    [anon_sym_switch] = ACTIONS(1951),
    [anon_sym_for] = ACTIONS(1951),
    [anon_sym_LPAREN] = ACTIONS(1949),
    [anon_sym_while] = ACTIONS(1951),
    [anon_sym_do] = ACTIONS(1951),
    [anon_sym_try] = ACTIONS(1951),
    [anon_sym_with] = ACTIONS(1951),
    [anon_sym_break] = ACTIONS(1951),
    [anon_sym_continue] = ACTIONS(1951),
    [anon_sym_debugger] = ACTIONS(1951),
    [anon_sym_return] = ACTIONS(1951),
    [anon_sym_throw] = ACTIONS(1951),
    [anon_sym_SEMI] = ACTIONS(1949),
    [anon_sym_case] = ACTIONS(1951),
    [anon_sym_yield] = ACTIONS(1951),
    [anon_sym_LBRACK] = ACTIONS(1949),
    [anon_sym_LT] = ACTIONS(1949),
    [anon_sym_class] = ACTIONS(1951),
    [anon_sym_async] = ACTIONS(1951),
    [anon_sym_function] = ACTIONS(1951),
    [anon_sym_EQ_GT] = ACTIONS(1949),
    [anon_sym_new] = ACTIONS(1951),
    [anon_sym_await] = ACTIONS(1951),
    [anon_sym_PLUS] = ACTIONS(1951),
    [anon_sym_DASH] = ACTIONS(1951),
    [anon_sym_BANG] = ACTIONS(1949),
    [anon_sym_TILDE] = ACTIONS(1949),
    [anon_sym_typeof] = ACTIONS(1951),
    [anon_sym_void] = ACTIONS(1951),
    [anon_sym_delete] = ACTIONS(1951),
    [anon_sym_PLUS_PLUS] = ACTIONS(1949),
    [anon_sym_DASH_DASH] = ACTIONS(1949),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1949),
    [anon_sym_BQUOTE] = ACTIONS(1949),
    [sym_regex] = ACTIONS(1951),
    [sym_number] = ACTIONS(1949),
    [sym_identifier] = ACTIONS(1951),
    [sym_this] = ACTIONS(1951),
    [sym_super] = ACTIONS(1951),
    [sym_true] = ACTIONS(1951),
    [sym_false] = ACTIONS(1951),
    [sym_null] = ACTIONS(1951),
    [sym_undefined] = ACTIONS(1951),
    [anon_sym_AT] = ACTIONS(1949),
    [anon_sym_get] = ACTIONS(1951),
    [anon_sym_set] = ACTIONS(1951),
  },
  [2372] = {
    [anon_sym_export] = ACTIONS(1955),
    [anon_sym_default] = ACTIONS(1955),
    [anon_sym_LBRACE] = ACTIONS(1953),
    [anon_sym_RBRACE] = ACTIONS(1953),
    [anon_sym_import] = ACTIONS(1955),
    [anon_sym_var] = ACTIONS(1955),
    [anon_sym_let] = ACTIONS(1955),
    [anon_sym_const] = ACTIONS(1955),
    [anon_sym_if] = ACTIONS(1955),
    [anon_sym_else] = ACTIONS(1955),
    [anon_sym_switch] = ACTIONS(1955),
    [anon_sym_for] = ACTIONS(1955),
    [anon_sym_LPAREN] = ACTIONS(1953),
    [anon_sym_while] = ACTIONS(1955),
    [anon_sym_do] = ACTIONS(1955),
    [anon_sym_try] = ACTIONS(1955),
    [anon_sym_with] = ACTIONS(1955),
    [anon_sym_break] = ACTIONS(1955),
    [anon_sym_continue] = ACTIONS(1955),
    [anon_sym_debugger] = ACTIONS(1955),
    [anon_sym_return] = ACTIONS(1955),
    [anon_sym_throw] = ACTIONS(1955),
    [anon_sym_SEMI] = ACTIONS(1953),
    [anon_sym_case] = ACTIONS(1955),
    [anon_sym_yield] = ACTIONS(1955),
    [anon_sym_LBRACK] = ACTIONS(1953),
    [anon_sym_LT] = ACTIONS(1953),
    [anon_sym_class] = ACTIONS(1955),
    [anon_sym_async] = ACTIONS(1955),
    [anon_sym_function] = ACTIONS(1955),
    [anon_sym_EQ_GT] = ACTIONS(1953),
    [anon_sym_new] = ACTIONS(1955),
    [anon_sym_await] = ACTIONS(1955),
    [anon_sym_PLUS] = ACTIONS(1955),
    [anon_sym_DASH] = ACTIONS(1955),
    [anon_sym_BANG] = ACTIONS(1953),
    [anon_sym_TILDE] = ACTIONS(1953),
    [anon_sym_typeof] = ACTIONS(1955),
    [anon_sym_void] = ACTIONS(1955),
    [anon_sym_delete] = ACTIONS(1955),
    [anon_sym_PLUS_PLUS] = ACTIONS(1953),
    [anon_sym_DASH_DASH] = ACTIONS(1953),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1953),
    [anon_sym_BQUOTE] = ACTIONS(1953),
    [sym_regex] = ACTIONS(1955),
    [sym_number] = ACTIONS(1953),
    [sym_identifier] = ACTIONS(1955),
    [sym_this] = ACTIONS(1955),
    [sym_super] = ACTIONS(1955),
    [sym_true] = ACTIONS(1955),
    [sym_false] = ACTIONS(1955),
    [sym_null] = ACTIONS(1955),
    [sym_undefined] = ACTIONS(1955),
    [anon_sym_AT] = ACTIONS(1953),
    [anon_sym_get] = ACTIONS(1955),
    [anon_sym_set] = ACTIONS(1955),
  },
  [2373] = {
    [anon_sym_export] = ACTIONS(1959),
    [anon_sym_default] = ACTIONS(1959),
    [anon_sym_LBRACE] = ACTIONS(1957),
    [anon_sym_RBRACE] = ACTIONS(1957),
    [anon_sym_import] = ACTIONS(1959),
    [anon_sym_var] = ACTIONS(1959),
    [anon_sym_let] = ACTIONS(1959),
    [anon_sym_const] = ACTIONS(1959),
    [anon_sym_if] = ACTIONS(1959),
    [anon_sym_else] = ACTIONS(1959),
    [anon_sym_switch] = ACTIONS(1959),
    [anon_sym_for] = ACTIONS(1959),
    [anon_sym_LPAREN] = ACTIONS(1957),
    [anon_sym_while] = ACTIONS(1959),
    [anon_sym_do] = ACTIONS(1959),
    [anon_sym_try] = ACTIONS(1959),
    [anon_sym_with] = ACTIONS(1959),
    [anon_sym_break] = ACTIONS(1959),
    [anon_sym_continue] = ACTIONS(1959),
    [anon_sym_debugger] = ACTIONS(1959),
    [anon_sym_return] = ACTIONS(1959),
    [anon_sym_throw] = ACTIONS(1959),
    [anon_sym_SEMI] = ACTIONS(1957),
    [anon_sym_case] = ACTIONS(1959),
    [anon_sym_yield] = ACTIONS(1959),
    [anon_sym_LBRACK] = ACTIONS(1957),
    [anon_sym_LT] = ACTIONS(1957),
    [anon_sym_class] = ACTIONS(1959),
    [anon_sym_async] = ACTIONS(1959),
    [anon_sym_function] = ACTIONS(1959),
    [anon_sym_EQ_GT] = ACTIONS(1957),
    [anon_sym_new] = ACTIONS(1959),
    [anon_sym_await] = ACTIONS(1959),
    [anon_sym_PLUS] = ACTIONS(1959),
    [anon_sym_DASH] = ACTIONS(1959),
    [anon_sym_BANG] = ACTIONS(1957),
    [anon_sym_TILDE] = ACTIONS(1957),
    [anon_sym_typeof] = ACTIONS(1959),
    [anon_sym_void] = ACTIONS(1959),
    [anon_sym_delete] = ACTIONS(1959),
    [anon_sym_PLUS_PLUS] = ACTIONS(1957),
    [anon_sym_DASH_DASH] = ACTIONS(1957),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1957),
    [anon_sym_BQUOTE] = ACTIONS(1957),
    [sym_regex] = ACTIONS(1959),
    [sym_number] = ACTIONS(1957),
    [sym_identifier] = ACTIONS(1959),
    [sym_this] = ACTIONS(1959),
    [sym_super] = ACTIONS(1959),
    [sym_true] = ACTIONS(1959),
    [sym_false] = ACTIONS(1959),
    [sym_null] = ACTIONS(1959),
    [sym_undefined] = ACTIONS(1959),
    [anon_sym_AT] = ACTIONS(1957),
    [anon_sym_get] = ACTIONS(1959),
    [anon_sym_set] = ACTIONS(1959),
  },
  [2374] = {
    [anon_sym_export] = ACTIONS(1963),
    [anon_sym_default] = ACTIONS(1963),
    [anon_sym_LBRACE] = ACTIONS(1961),
    [anon_sym_RBRACE] = ACTIONS(1961),
    [anon_sym_import] = ACTIONS(1963),
    [anon_sym_var] = ACTIONS(1963),
    [anon_sym_let] = ACTIONS(1963),
    [anon_sym_const] = ACTIONS(1963),
    [anon_sym_if] = ACTIONS(1963),
    [anon_sym_else] = ACTIONS(1963),
    [anon_sym_switch] = ACTIONS(1963),
    [anon_sym_for] = ACTIONS(1963),
    [anon_sym_LPAREN] = ACTIONS(1961),
    [anon_sym_while] = ACTIONS(1963),
    [anon_sym_do] = ACTIONS(1963),
    [anon_sym_try] = ACTIONS(1963),
    [anon_sym_with] = ACTIONS(1963),
    [anon_sym_break] = ACTIONS(1963),
    [anon_sym_continue] = ACTIONS(1963),
    [anon_sym_debugger] = ACTIONS(1963),
    [anon_sym_return] = ACTIONS(1963),
    [anon_sym_throw] = ACTIONS(1963),
    [anon_sym_SEMI] = ACTIONS(1961),
    [anon_sym_case] = ACTIONS(1963),
    [anon_sym_yield] = ACTIONS(1963),
    [anon_sym_LBRACK] = ACTIONS(1961),
    [anon_sym_LT] = ACTIONS(1961),
    [anon_sym_class] = ACTIONS(1963),
    [anon_sym_async] = ACTIONS(1963),
    [anon_sym_function] = ACTIONS(1963),
    [anon_sym_EQ_GT] = ACTIONS(1961),
    [anon_sym_new] = ACTIONS(1963),
    [anon_sym_await] = ACTIONS(1963),
    [anon_sym_PLUS] = ACTIONS(1963),
    [anon_sym_DASH] = ACTIONS(1963),
    [anon_sym_BANG] = ACTIONS(1961),
    [anon_sym_TILDE] = ACTIONS(1961),
    [anon_sym_typeof] = ACTIONS(1963),
    [anon_sym_void] = ACTIONS(1963),
    [anon_sym_delete] = ACTIONS(1963),
    [anon_sym_PLUS_PLUS] = ACTIONS(1961),
    [anon_sym_DASH_DASH] = ACTIONS(1961),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1961),
    [anon_sym_BQUOTE] = ACTIONS(1961),
    [sym_regex] = ACTIONS(1963),
    [sym_number] = ACTIONS(1961),
    [sym_identifier] = ACTIONS(1963),
    [sym_this] = ACTIONS(1963),
    [sym_super] = ACTIONS(1963),
    [sym_true] = ACTIONS(1963),
    [sym_false] = ACTIONS(1963),
    [sym_null] = ACTIONS(1963),
    [sym_undefined] = ACTIONS(1963),
    [anon_sym_AT] = ACTIONS(1961),
    [anon_sym_get] = ACTIONS(1963),
    [anon_sym_set] = ACTIONS(1963),
  },
  [2375] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(2470),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(5142),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2376] = {
    [sym_class_body] = STATE(2471),
    [anon_sym_LBRACE] = ACTIONS(5026),
    [sym_comment] = ACTIONS(71),
  },
  [2377] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_STAR] = ACTIONS(2140),
    [anon_sym_default] = ACTIONS(2142),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_COMMA] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_else] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_in] = ACTIONS(2142),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_case] = ACTIONS(2142),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2142),
    [anon_sym_GT] = ACTIONS(2142),
    [anon_sym_SLASH] = ACTIONS(2142),
    [anon_sym_DOT] = ACTIONS(2142),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_QMARK] = ACTIONS(2140),
    [anon_sym_AMP_AMP] = ACTIONS(2140),
    [anon_sym_PIPE_PIPE] = ACTIONS(2140),
    [anon_sym_GT_GT] = ACTIONS(2142),
    [anon_sym_GT_GT_GT] = ACTIONS(2140),
    [anon_sym_LT_LT] = ACTIONS(2140),
    [anon_sym_AMP] = ACTIONS(2142),
    [anon_sym_CARET] = ACTIONS(2140),
    [anon_sym_PIPE] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_PERCENT] = ACTIONS(2140),
    [anon_sym_LT_EQ] = ACTIONS(2140),
    [anon_sym_EQ_EQ] = ACTIONS(2142),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2140),
    [anon_sym_BANG_EQ] = ACTIONS(2142),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2140),
    [anon_sym_GT_EQ] = ACTIONS(2140),
    [anon_sym_instanceof] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2142),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [2378] = {
    [sym_formal_parameters] = STATE(2472),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2379] = {
    [sym_statement_block] = STATE(2473),
    [anon_sym_LBRACE] = ACTIONS(5032),
    [sym_comment] = ACTIONS(71),
  },
  [2380] = {
    [sym_formal_parameters] = STATE(2474),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2381] = {
    [sym_statement_block] = STATE(2475),
    [anon_sym_LBRACE] = ACTIONS(5032),
    [sym_comment] = ACTIONS(71),
  },
  [2382] = {
    [sym_export_statement] = STATE(2477),
    [sym__declaration] = STATE(2477),
    [sym_import_statement] = STATE(2477),
    [sym_expression_statement] = STATE(2477),
    [sym_variable_declaration] = STATE(2477),
    [sym_lexical_declaration] = STATE(2477),
    [sym_statement_block] = STATE(2477),
    [sym_if_statement] = STATE(2477),
    [sym_switch_statement] = STATE(2477),
    [sym_for_statement] = STATE(2477),
    [sym_for_in_statement] = STATE(2477),
    [sym_for_of_statement] = STATE(2477),
    [sym_while_statement] = STATE(2477),
    [sym_do_statement] = STATE(2477),
    [sym_try_statement] = STATE(2477),
    [sym_with_statement] = STATE(2477),
    [sym_break_statement] = STATE(2477),
    [sym_continue_statement] = STATE(2477),
    [sym_debugger_statement] = STATE(2477),
    [sym_return_statement] = STATE(2477),
    [sym_throw_statement] = STATE(2477),
    [sym_empty_statement] = STATE(2477),
    [sym_labeled_statement] = STATE(2477),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2477),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5144),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2383] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_STAR] = ACTIONS(2178),
    [anon_sym_default] = ACTIONS(2180),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_COMMA] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_else] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_in] = ACTIONS(2180),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_case] = ACTIONS(2180),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2180),
    [anon_sym_GT] = ACTIONS(2180),
    [anon_sym_SLASH] = ACTIONS(2180),
    [anon_sym_DOT] = ACTIONS(2180),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_QMARK] = ACTIONS(2178),
    [anon_sym_AMP_AMP] = ACTIONS(2178),
    [anon_sym_PIPE_PIPE] = ACTIONS(2178),
    [anon_sym_GT_GT] = ACTIONS(2180),
    [anon_sym_GT_GT_GT] = ACTIONS(2178),
    [anon_sym_LT_LT] = ACTIONS(2178),
    [anon_sym_AMP] = ACTIONS(2180),
    [anon_sym_CARET] = ACTIONS(2178),
    [anon_sym_PIPE] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_PERCENT] = ACTIONS(2178),
    [anon_sym_LT_EQ] = ACTIONS(2178),
    [anon_sym_EQ_EQ] = ACTIONS(2180),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2178),
    [anon_sym_BANG_EQ] = ACTIONS(2180),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2178),
    [anon_sym_GT_EQ] = ACTIONS(2178),
    [anon_sym_instanceof] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2180),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [2384] = {
    [anon_sym_export] = ACTIONS(2206),
    [anon_sym_default] = ACTIONS(2206),
    [anon_sym_LBRACE] = ACTIONS(2204),
    [anon_sym_RBRACE] = ACTIONS(2204),
    [anon_sym_import] = ACTIONS(2206),
    [anon_sym_var] = ACTIONS(2206),
    [anon_sym_let] = ACTIONS(2206),
    [anon_sym_const] = ACTIONS(2206),
    [anon_sym_if] = ACTIONS(2206),
    [anon_sym_else] = ACTIONS(2206),
    [anon_sym_switch] = ACTIONS(2206),
    [anon_sym_for] = ACTIONS(2206),
    [anon_sym_LPAREN] = ACTIONS(2204),
    [anon_sym_while] = ACTIONS(2206),
    [anon_sym_do] = ACTIONS(2206),
    [anon_sym_try] = ACTIONS(2206),
    [anon_sym_with] = ACTIONS(2206),
    [anon_sym_break] = ACTIONS(2206),
    [anon_sym_continue] = ACTIONS(2206),
    [anon_sym_debugger] = ACTIONS(2206),
    [anon_sym_return] = ACTIONS(2206),
    [anon_sym_throw] = ACTIONS(2206),
    [anon_sym_SEMI] = ACTIONS(2204),
    [anon_sym_case] = ACTIONS(2206),
    [anon_sym_yield] = ACTIONS(2206),
    [anon_sym_LBRACK] = ACTIONS(2204),
    [anon_sym_LT] = ACTIONS(2204),
    [anon_sym_class] = ACTIONS(2206),
    [anon_sym_async] = ACTIONS(2206),
    [anon_sym_function] = ACTIONS(2206),
    [anon_sym_EQ_GT] = ACTIONS(2204),
    [anon_sym_new] = ACTIONS(2206),
    [anon_sym_await] = ACTIONS(2206),
    [anon_sym_PLUS] = ACTIONS(2206),
    [anon_sym_DASH] = ACTIONS(2206),
    [anon_sym_BANG] = ACTIONS(2204),
    [anon_sym_TILDE] = ACTIONS(2204),
    [anon_sym_typeof] = ACTIONS(2206),
    [anon_sym_void] = ACTIONS(2206),
    [anon_sym_delete] = ACTIONS(2206),
    [anon_sym_PLUS_PLUS] = ACTIONS(2204),
    [anon_sym_DASH_DASH] = ACTIONS(2204),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2204),
    [anon_sym_BQUOTE] = ACTIONS(2204),
    [sym_regex] = ACTIONS(2206),
    [sym_number] = ACTIONS(2204),
    [sym_identifier] = ACTIONS(2206),
    [sym_this] = ACTIONS(2206),
    [sym_super] = ACTIONS(2206),
    [sym_true] = ACTIONS(2206),
    [sym_false] = ACTIONS(2206),
    [sym_null] = ACTIONS(2206),
    [sym_undefined] = ACTIONS(2206),
    [anon_sym_AT] = ACTIONS(2204),
    [anon_sym_get] = ACTIONS(2206),
    [anon_sym_set] = ACTIONS(2206),
  },
  [2385] = {
    [anon_sym_export] = ACTIONS(3484),
    [anon_sym_default] = ACTIONS(3484),
    [anon_sym_LBRACE] = ACTIONS(3482),
    [anon_sym_RBRACE] = ACTIONS(3482),
    [anon_sym_import] = ACTIONS(3484),
    [anon_sym_var] = ACTIONS(3484),
    [anon_sym_let] = ACTIONS(3484),
    [anon_sym_const] = ACTIONS(3484),
    [anon_sym_if] = ACTIONS(3484),
    [anon_sym_switch] = ACTIONS(3484),
    [anon_sym_for] = ACTIONS(3484),
    [anon_sym_LPAREN] = ACTIONS(3482),
    [anon_sym_while] = ACTIONS(3484),
    [anon_sym_do] = ACTIONS(3484),
    [anon_sym_try] = ACTIONS(3484),
    [anon_sym_with] = ACTIONS(3484),
    [anon_sym_break] = ACTIONS(3484),
    [anon_sym_continue] = ACTIONS(3484),
    [anon_sym_debugger] = ACTIONS(3484),
    [anon_sym_return] = ACTIONS(3484),
    [anon_sym_throw] = ACTIONS(3484),
    [anon_sym_SEMI] = ACTIONS(3482),
    [anon_sym_case] = ACTIONS(3484),
    [anon_sym_yield] = ACTIONS(3484),
    [anon_sym_LBRACK] = ACTIONS(3482),
    [anon_sym_LT] = ACTIONS(3482),
    [anon_sym_class] = ACTIONS(3484),
    [anon_sym_async] = ACTIONS(3484),
    [anon_sym_function] = ACTIONS(3484),
    [anon_sym_EQ_GT] = ACTIONS(3482),
    [anon_sym_new] = ACTIONS(3484),
    [anon_sym_await] = ACTIONS(3484),
    [anon_sym_PLUS] = ACTIONS(3484),
    [anon_sym_DASH] = ACTIONS(3484),
    [anon_sym_BANG] = ACTIONS(3482),
    [anon_sym_TILDE] = ACTIONS(3482),
    [anon_sym_typeof] = ACTIONS(3484),
    [anon_sym_void] = ACTIONS(3484),
    [anon_sym_delete] = ACTIONS(3484),
    [anon_sym_PLUS_PLUS] = ACTIONS(3482),
    [anon_sym_DASH_DASH] = ACTIONS(3482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3482),
    [anon_sym_BQUOTE] = ACTIONS(3482),
    [sym_regex] = ACTIONS(3484),
    [sym_number] = ACTIONS(3482),
    [sym_identifier] = ACTIONS(3484),
    [sym_this] = ACTIONS(3484),
    [sym_super] = ACTIONS(3484),
    [sym_true] = ACTIONS(3484),
    [sym_false] = ACTIONS(3484),
    [sym_null] = ACTIONS(3484),
    [sym_undefined] = ACTIONS(3484),
    [anon_sym_AT] = ACTIONS(3482),
    [anon_sym_get] = ACTIONS(3484),
    [anon_sym_set] = ACTIONS(3484),
  },
  [2386] = {
    [sym_parenthesized_expression] = STATE(2478),
    [sym__expression] = STATE(2478),
    [sym_yield_expression] = STATE(2478),
    [sym_object] = STATE(254),
    [sym_array] = STATE(255),
    [sym_jsx_element] = STATE(2478),
    [sym_jsx_fragment] = STATE(2478),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(2478),
    [sym_anonymous_class] = STATE(2478),
    [sym_class] = STATE(2478),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(2478),
    [sym_generator_function] = STATE(2478),
    [sym_call_expression] = STATE(2478),
    [sym_new_expression] = STATE(2478),
    [sym_await_expression] = STATE(2478),
    [sym_member_expression] = STATE(256),
    [sym_subscript_expression] = STATE(256),
    [sym_assignment_expression] = STATE(2478),
    [sym_augmented_assignment_expression] = STATE(2478),
    [sym_ternary_expression] = STATE(2478),
    [sym_binary_expression] = STATE(2478),
    [sym_unary_expression] = STATE(2478),
    [sym_update_expression] = STATE(2478),
    [sym_template_string] = STATE(2478),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(257),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(412),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(414),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(416),
    [anon_sym_new] = ACTIONS(418),
    [anon_sym_await] = ACTIONS(420),
    [anon_sym_PLUS] = ACTIONS(422),
    [anon_sym_DASH] = ACTIONS(422),
    [anon_sym_BANG] = ACTIONS(424),
    [anon_sym_TILDE] = ACTIONS(424),
    [anon_sym_typeof] = ACTIONS(422),
    [anon_sym_void] = ACTIONS(422),
    [anon_sym_delete] = ACTIONS(422),
    [anon_sym_PLUS_PLUS] = ACTIONS(426),
    [anon_sym_DASH_DASH] = ACTIONS(426),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5146),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(5148),
    [sym_number] = ACTIONS(5146),
    [sym_identifier] = ACTIONS(432),
    [sym_this] = ACTIONS(5148),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(5148),
    [sym_false] = ACTIONS(5148),
    [sym_null] = ACTIONS(5148),
    [sym_undefined] = ACTIONS(5148),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(434),
    [anon_sym_set] = ACTIONS(434),
  },
  [2387] = {
    [sym_class_heritage] = STATE(2479),
    [sym_class_body] = STATE(2471),
    [anon_sym_LBRACE] = ACTIONS(5026),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2388] = {
    [anon_sym_export] = ACTIONS(3586),
    [anon_sym_default] = ACTIONS(3586),
    [anon_sym_LBRACE] = ACTIONS(3584),
    [anon_sym_RBRACE] = ACTIONS(3584),
    [anon_sym_import] = ACTIONS(3586),
    [anon_sym_var] = ACTIONS(3586),
    [anon_sym_let] = ACTIONS(3586),
    [anon_sym_const] = ACTIONS(3586),
    [anon_sym_if] = ACTIONS(3586),
    [anon_sym_switch] = ACTIONS(3586),
    [anon_sym_for] = ACTIONS(3586),
    [anon_sym_LPAREN] = ACTIONS(3584),
    [anon_sym_while] = ACTIONS(3586),
    [anon_sym_do] = ACTIONS(3586),
    [anon_sym_try] = ACTIONS(3586),
    [anon_sym_with] = ACTIONS(3586),
    [anon_sym_break] = ACTIONS(3586),
    [anon_sym_continue] = ACTIONS(3586),
    [anon_sym_debugger] = ACTIONS(3586),
    [anon_sym_return] = ACTIONS(3586),
    [anon_sym_throw] = ACTIONS(3586),
    [anon_sym_SEMI] = ACTIONS(3584),
    [anon_sym_case] = ACTIONS(3586),
    [anon_sym_yield] = ACTIONS(3586),
    [anon_sym_LBRACK] = ACTIONS(3584),
    [anon_sym_LT] = ACTIONS(3584),
    [anon_sym_class] = ACTIONS(3586),
    [anon_sym_async] = ACTIONS(3586),
    [anon_sym_function] = ACTIONS(3586),
    [anon_sym_EQ_GT] = ACTIONS(3584),
    [anon_sym_new] = ACTIONS(3586),
    [anon_sym_await] = ACTIONS(3586),
    [anon_sym_PLUS] = ACTIONS(3586),
    [anon_sym_DASH] = ACTIONS(3586),
    [anon_sym_BANG] = ACTIONS(3584),
    [anon_sym_TILDE] = ACTIONS(3584),
    [anon_sym_typeof] = ACTIONS(3586),
    [anon_sym_void] = ACTIONS(3586),
    [anon_sym_delete] = ACTIONS(3586),
    [anon_sym_PLUS_PLUS] = ACTIONS(3584),
    [anon_sym_DASH_DASH] = ACTIONS(3584),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3584),
    [anon_sym_BQUOTE] = ACTIONS(3584),
    [sym_regex] = ACTIONS(3586),
    [sym_number] = ACTIONS(3584),
    [sym_identifier] = ACTIONS(3586),
    [sym_this] = ACTIONS(3586),
    [sym_super] = ACTIONS(3586),
    [sym_true] = ACTIONS(3586),
    [sym_false] = ACTIONS(3586),
    [sym_null] = ACTIONS(3586),
    [sym_undefined] = ACTIONS(3586),
    [anon_sym_AT] = ACTIONS(3584),
    [anon_sym_get] = ACTIONS(3586),
    [anon_sym_set] = ACTIONS(3586),
  },
  [2389] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2480),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2480),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2390] = {
    [sym_parenthesized_expression] = STATE(2481),
    [sym__expression] = STATE(2481),
    [sym_yield_expression] = STATE(2481),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2481),
    [sym_jsx_fragment] = STATE(2481),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2481),
    [sym_anonymous_class] = STATE(2481),
    [sym_class] = STATE(2481),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2481),
    [sym_generator_function] = STATE(2481),
    [sym_call_expression] = STATE(2481),
    [sym_new_expression] = STATE(2481),
    [sym_await_expression] = STATE(2481),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2481),
    [sym_augmented_assignment_expression] = STATE(2481),
    [sym_ternary_expression] = STATE(2481),
    [sym_binary_expression] = STATE(2481),
    [sym_unary_expression] = STATE(2481),
    [sym_update_expression] = STATE(2481),
    [sym_template_string] = STATE(2481),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5150),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5152),
    [sym_number] = ACTIONS(5150),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5152),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5152),
    [sym_false] = ACTIONS(5152),
    [sym_null] = ACTIONS(5152),
    [sym_undefined] = ACTIONS(5152),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2391] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2482),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2482),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2392] = {
    [sym_parenthesized_expression] = STATE(2483),
    [sym__expression] = STATE(2483),
    [sym_yield_expression] = STATE(2483),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2483),
    [sym_jsx_fragment] = STATE(2483),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2483),
    [sym_anonymous_class] = STATE(2483),
    [sym_class] = STATE(2483),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2483),
    [sym_generator_function] = STATE(2483),
    [sym_call_expression] = STATE(2483),
    [sym_new_expression] = STATE(2483),
    [sym_await_expression] = STATE(2483),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2483),
    [sym_augmented_assignment_expression] = STATE(2483),
    [sym_ternary_expression] = STATE(2483),
    [sym_binary_expression] = STATE(2483),
    [sym_unary_expression] = STATE(2483),
    [sym_update_expression] = STATE(2483),
    [sym_template_string] = STATE(2483),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5154),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5156),
    [sym_number] = ACTIONS(5154),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5156),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5156),
    [sym_false] = ACTIONS(5156),
    [sym_null] = ACTIONS(5156),
    [sym_undefined] = ACTIONS(5156),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2393] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2484),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2484),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2394] = {
    [sym_parenthesized_expression] = STATE(2485),
    [sym__expression] = STATE(2485),
    [sym_yield_expression] = STATE(2485),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2485),
    [sym_jsx_fragment] = STATE(2485),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2485),
    [sym_anonymous_class] = STATE(2485),
    [sym_class] = STATE(2485),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2485),
    [sym_generator_function] = STATE(2485),
    [sym_call_expression] = STATE(2485),
    [sym_new_expression] = STATE(2485),
    [sym_await_expression] = STATE(2485),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2485),
    [sym_augmented_assignment_expression] = STATE(2485),
    [sym_ternary_expression] = STATE(2485),
    [sym_binary_expression] = STATE(2485),
    [sym_unary_expression] = STATE(2485),
    [sym_update_expression] = STATE(2485),
    [sym_template_string] = STATE(2485),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5158),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5160),
    [sym_number] = ACTIONS(5158),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5160),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5160),
    [sym_false] = ACTIONS(5160),
    [sym_null] = ACTIONS(5160),
    [sym_undefined] = ACTIONS(5160),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2395] = {
    [anon_sym_RPAREN] = ACTIONS(5162),
    [sym_comment] = ACTIONS(71),
  },
  [2396] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5164),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2397] = {
    [sym_export_statement] = STATE(2488),
    [sym__declaration] = STATE(2488),
    [sym_import_statement] = STATE(2488),
    [sym_expression_statement] = STATE(2488),
    [sym_variable_declaration] = STATE(2488),
    [sym_lexical_declaration] = STATE(2488),
    [sym_statement_block] = STATE(2488),
    [sym_if_statement] = STATE(2488),
    [sym_switch_statement] = STATE(2488),
    [sym_for_statement] = STATE(2488),
    [sym_for_in_statement] = STATE(2488),
    [sym_for_of_statement] = STATE(2488),
    [sym_while_statement] = STATE(2488),
    [sym_do_statement] = STATE(2488),
    [sym_try_statement] = STATE(2488),
    [sym_with_statement] = STATE(2488),
    [sym_break_statement] = STATE(2488),
    [sym_continue_statement] = STATE(2488),
    [sym_debugger_statement] = STATE(2488),
    [sym_return_statement] = STATE(2488),
    [sym_throw_statement] = STATE(2488),
    [sym_empty_statement] = STATE(2488),
    [sym_labeled_statement] = STATE(2488),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2398] = {
    [anon_sym_RPAREN] = ACTIONS(5166),
    [sym_comment] = ACTIONS(71),
  },
  [2399] = {
    [anon_sym_RPAREN] = ACTIONS(5168),
    [sym_comment] = ACTIONS(71),
  },
  [2400] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5170),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2401] = {
    [anon_sym_RPAREN] = ACTIONS(5172),
    [sym_comment] = ACTIONS(71),
  },
  [2402] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5174),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2403] = {
    [anon_sym_export] = ACTIONS(3788),
    [anon_sym_default] = ACTIONS(3788),
    [anon_sym_LBRACE] = ACTIONS(3786),
    [anon_sym_RBRACE] = ACTIONS(3786),
    [anon_sym_import] = ACTIONS(3788),
    [anon_sym_var] = ACTIONS(3788),
    [anon_sym_let] = ACTIONS(3788),
    [anon_sym_const] = ACTIONS(3788),
    [anon_sym_if] = ACTIONS(3788),
    [anon_sym_switch] = ACTIONS(3788),
    [anon_sym_for] = ACTIONS(3788),
    [anon_sym_LPAREN] = ACTIONS(3786),
    [anon_sym_while] = ACTIONS(3788),
    [anon_sym_do] = ACTIONS(3788),
    [anon_sym_try] = ACTIONS(3788),
    [anon_sym_with] = ACTIONS(3788),
    [anon_sym_break] = ACTIONS(3788),
    [anon_sym_continue] = ACTIONS(3788),
    [anon_sym_debugger] = ACTIONS(3788),
    [anon_sym_return] = ACTIONS(3788),
    [anon_sym_throw] = ACTIONS(3788),
    [anon_sym_SEMI] = ACTIONS(3786),
    [anon_sym_case] = ACTIONS(3788),
    [anon_sym_yield] = ACTIONS(3788),
    [anon_sym_LBRACK] = ACTIONS(3786),
    [anon_sym_LT] = ACTIONS(3786),
    [anon_sym_class] = ACTIONS(3788),
    [anon_sym_async] = ACTIONS(3788),
    [anon_sym_function] = ACTIONS(3788),
    [anon_sym_EQ_GT] = ACTIONS(3786),
    [anon_sym_new] = ACTIONS(3788),
    [anon_sym_await] = ACTIONS(3788),
    [anon_sym_PLUS] = ACTIONS(3788),
    [anon_sym_DASH] = ACTIONS(3788),
    [anon_sym_BANG] = ACTIONS(3786),
    [anon_sym_TILDE] = ACTIONS(3786),
    [anon_sym_typeof] = ACTIONS(3788),
    [anon_sym_void] = ACTIONS(3788),
    [anon_sym_delete] = ACTIONS(3788),
    [anon_sym_PLUS_PLUS] = ACTIONS(3786),
    [anon_sym_DASH_DASH] = ACTIONS(3786),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3786),
    [anon_sym_BQUOTE] = ACTIONS(3786),
    [sym_regex] = ACTIONS(3788),
    [sym_number] = ACTIONS(3786),
    [sym_identifier] = ACTIONS(3788),
    [sym_this] = ACTIONS(3788),
    [sym_super] = ACTIONS(3788),
    [sym_true] = ACTIONS(3788),
    [sym_false] = ACTIONS(3788),
    [sym_null] = ACTIONS(3788),
    [sym_undefined] = ACTIONS(3788),
    [anon_sym_AT] = ACTIONS(3786),
    [anon_sym_get] = ACTIONS(3788),
    [anon_sym_set] = ACTIONS(3788),
  },
  [2404] = {
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2405] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5176),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2406] = {
    [anon_sym_RPAREN] = ACTIONS(5178),
    [sym_comment] = ACTIONS(71),
  },
  [2407] = {
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2408] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_STAR] = ACTIONS(3035),
    [anon_sym_default] = ACTIONS(3037),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_in] = ACTIONS(3037),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_case] = ACTIONS(3037),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3037),
    [anon_sym_GT] = ACTIONS(3037),
    [anon_sym_SLASH] = ACTIONS(3037),
    [anon_sym_DOT] = ACTIONS(3037),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_QMARK] = ACTIONS(3035),
    [anon_sym_AMP_AMP] = ACTIONS(3035),
    [anon_sym_PIPE_PIPE] = ACTIONS(3035),
    [anon_sym_GT_GT] = ACTIONS(3037),
    [anon_sym_GT_GT_GT] = ACTIONS(3035),
    [anon_sym_LT_LT] = ACTIONS(3035),
    [anon_sym_AMP] = ACTIONS(3037),
    [anon_sym_CARET] = ACTIONS(3035),
    [anon_sym_PIPE] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_PERCENT] = ACTIONS(3035),
    [anon_sym_LT_EQ] = ACTIONS(3035),
    [anon_sym_EQ_EQ] = ACTIONS(3037),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3035),
    [anon_sym_BANG_EQ] = ACTIONS(3037),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3035),
    [anon_sym_GT_EQ] = ACTIONS(3035),
    [anon_sym_instanceof] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3037),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [2409] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_STAR] = ACTIONS(3856),
    [anon_sym_default] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_COMMA] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_in] = ACTIONS(3858),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_case] = ACTIONS(3858),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_GT] = ACTIONS(3858),
    [anon_sym_SLASH] = ACTIONS(3858),
    [anon_sym_DOT] = ACTIONS(3858),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_QMARK] = ACTIONS(3856),
    [anon_sym_AMP_AMP] = ACTIONS(3856),
    [anon_sym_PIPE_PIPE] = ACTIONS(3856),
    [anon_sym_GT_GT] = ACTIONS(3858),
    [anon_sym_GT_GT_GT] = ACTIONS(3856),
    [anon_sym_LT_LT] = ACTIONS(3856),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_CARET] = ACTIONS(3856),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_PERCENT] = ACTIONS(3856),
    [anon_sym_LT_EQ] = ACTIONS(3856),
    [anon_sym_EQ_EQ] = ACTIONS(3858),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3856),
    [anon_sym_BANG_EQ] = ACTIONS(3858),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3856),
    [anon_sym_GT_EQ] = ACTIONS(3856),
    [anon_sym_instanceof] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3858),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [2410] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_STAR] = ACTIONS(3860),
    [anon_sym_default] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_COMMA] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_in] = ACTIONS(3862),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_case] = ACTIONS(3862),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_GT] = ACTIONS(3862),
    [anon_sym_SLASH] = ACTIONS(3862),
    [anon_sym_DOT] = ACTIONS(3862),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_QMARK] = ACTIONS(3860),
    [anon_sym_AMP_AMP] = ACTIONS(3860),
    [anon_sym_PIPE_PIPE] = ACTIONS(3860),
    [anon_sym_GT_GT] = ACTIONS(3862),
    [anon_sym_GT_GT_GT] = ACTIONS(3860),
    [anon_sym_LT_LT] = ACTIONS(3860),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_CARET] = ACTIONS(3860),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_PERCENT] = ACTIONS(3860),
    [anon_sym_LT_EQ] = ACTIONS(3860),
    [anon_sym_EQ_EQ] = ACTIONS(3862),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3860),
    [anon_sym_BANG_EQ] = ACTIONS(3862),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3860),
    [anon_sym_GT_EQ] = ACTIONS(3860),
    [anon_sym_instanceof] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3862),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [2411] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_in] = ACTIONS(1590),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1590),
    [anon_sym_SLASH] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1590),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_QMARK] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1588),
    [anon_sym_PIPE_PIPE] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1590),
    [anon_sym_GT_GT_GT] = ACTIONS(1588),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1590),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_LT_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1588),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1588),
    [anon_sym_instanceof] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1590),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2412] = {
    [anon_sym_export] = ACTIONS(3932),
    [anon_sym_default] = ACTIONS(3932),
    [anon_sym_LBRACE] = ACTIONS(3930),
    [anon_sym_RBRACE] = ACTIONS(3930),
    [anon_sym_import] = ACTIONS(3932),
    [anon_sym_var] = ACTIONS(3932),
    [anon_sym_let] = ACTIONS(3932),
    [anon_sym_const] = ACTIONS(3932),
    [anon_sym_if] = ACTIONS(3932),
    [anon_sym_switch] = ACTIONS(3932),
    [anon_sym_for] = ACTIONS(3932),
    [anon_sym_LPAREN] = ACTIONS(3930),
    [anon_sym_while] = ACTIONS(3932),
    [anon_sym_do] = ACTIONS(3932),
    [anon_sym_try] = ACTIONS(3932),
    [anon_sym_with] = ACTIONS(3932),
    [anon_sym_break] = ACTIONS(3932),
    [anon_sym_continue] = ACTIONS(3932),
    [anon_sym_debugger] = ACTIONS(3932),
    [anon_sym_return] = ACTIONS(3932),
    [anon_sym_throw] = ACTIONS(3932),
    [anon_sym_SEMI] = ACTIONS(3930),
    [anon_sym_case] = ACTIONS(3932),
    [anon_sym_yield] = ACTIONS(3932),
    [anon_sym_LBRACK] = ACTIONS(3930),
    [anon_sym_LT] = ACTIONS(3930),
    [anon_sym_class] = ACTIONS(3932),
    [anon_sym_async] = ACTIONS(3932),
    [anon_sym_function] = ACTIONS(3932),
    [anon_sym_EQ_GT] = ACTIONS(3930),
    [anon_sym_new] = ACTIONS(3932),
    [anon_sym_await] = ACTIONS(3932),
    [anon_sym_PLUS] = ACTIONS(3932),
    [anon_sym_DASH] = ACTIONS(3932),
    [anon_sym_BANG] = ACTIONS(3930),
    [anon_sym_TILDE] = ACTIONS(3930),
    [anon_sym_typeof] = ACTIONS(3932),
    [anon_sym_void] = ACTIONS(3932),
    [anon_sym_delete] = ACTIONS(3932),
    [anon_sym_PLUS_PLUS] = ACTIONS(3930),
    [anon_sym_DASH_DASH] = ACTIONS(3930),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3930),
    [anon_sym_BQUOTE] = ACTIONS(3930),
    [sym_regex] = ACTIONS(3932),
    [sym_number] = ACTIONS(3930),
    [sym_identifier] = ACTIONS(3932),
    [sym_this] = ACTIONS(3932),
    [sym_super] = ACTIONS(3932),
    [sym_true] = ACTIONS(3932),
    [sym_false] = ACTIONS(3932),
    [sym_null] = ACTIONS(3932),
    [sym_undefined] = ACTIONS(3932),
    [anon_sym_AT] = ACTIONS(3930),
    [anon_sym_get] = ACTIONS(3932),
    [anon_sym_set] = ACTIONS(3932),
  },
  [2413] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_default] = ACTIONS(3936),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_COMMA] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_in] = ACTIONS(3936),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_case] = ACTIONS(3936),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3936),
    [anon_sym_GT] = ACTIONS(3936),
    [anon_sym_SLASH] = ACTIONS(3936),
    [anon_sym_DOT] = ACTIONS(3936),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_QMARK] = ACTIONS(3934),
    [anon_sym_AMP_AMP] = ACTIONS(3934),
    [anon_sym_PIPE_PIPE] = ACTIONS(3934),
    [anon_sym_GT_GT] = ACTIONS(3936),
    [anon_sym_GT_GT_GT] = ACTIONS(3934),
    [anon_sym_LT_LT] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3936),
    [anon_sym_CARET] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_PERCENT] = ACTIONS(3934),
    [anon_sym_LT_EQ] = ACTIONS(3934),
    [anon_sym_EQ_EQ] = ACTIONS(3936),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3934),
    [anon_sym_BANG_EQ] = ACTIONS(3936),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3934),
    [anon_sym_GT_EQ] = ACTIONS(3934),
    [anon_sym_instanceof] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3936),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [2414] = {
    [sym_export_statement] = STATE(2496),
    [sym__declaration] = STATE(2496),
    [sym_import_statement] = STATE(2496),
    [sym_expression_statement] = STATE(2496),
    [sym_variable_declaration] = STATE(2496),
    [sym_lexical_declaration] = STATE(2496),
    [sym_statement_block] = STATE(2496),
    [sym_if_statement] = STATE(2496),
    [sym_switch_statement] = STATE(2496),
    [sym_for_statement] = STATE(2496),
    [sym_for_in_statement] = STATE(2496),
    [sym_for_of_statement] = STATE(2496),
    [sym_while_statement] = STATE(2496),
    [sym_do_statement] = STATE(2496),
    [sym_try_statement] = STATE(2496),
    [sym_with_statement] = STATE(2496),
    [sym_break_statement] = STATE(2496),
    [sym_continue_statement] = STATE(2496),
    [sym_debugger_statement] = STATE(2496),
    [sym_return_statement] = STATE(2496),
    [sym_throw_statement] = STATE(2496),
    [sym_empty_statement] = STATE(2496),
    [sym_labeled_statement] = STATE(2496),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2415] = {
    [sym_export_statement] = STATE(2497),
    [sym__declaration] = STATE(2497),
    [sym_import_statement] = STATE(2497),
    [sym_expression_statement] = STATE(2497),
    [sym_variable_declaration] = STATE(2497),
    [sym_lexical_declaration] = STATE(2497),
    [sym_statement_block] = STATE(2497),
    [sym_if_statement] = STATE(2497),
    [sym_switch_statement] = STATE(2497),
    [sym_for_statement] = STATE(2497),
    [sym_for_in_statement] = STATE(2497),
    [sym_for_of_statement] = STATE(2497),
    [sym_while_statement] = STATE(2497),
    [sym_do_statement] = STATE(2497),
    [sym_try_statement] = STATE(2497),
    [sym_with_statement] = STATE(2497),
    [sym_break_statement] = STATE(2497),
    [sym_continue_statement] = STATE(2497),
    [sym_debugger_statement] = STATE(2497),
    [sym_return_statement] = STATE(2497),
    [sym_throw_statement] = STATE(2497),
    [sym_empty_statement] = STATE(2497),
    [sym_labeled_statement] = STATE(2497),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2416] = {
    [sym_export_statement] = STATE(2498),
    [sym__declaration] = STATE(2498),
    [sym_import_statement] = STATE(2498),
    [sym_expression_statement] = STATE(2498),
    [sym_variable_declaration] = STATE(2498),
    [sym_lexical_declaration] = STATE(2498),
    [sym_statement_block] = STATE(2498),
    [sym_if_statement] = STATE(2498),
    [sym_switch_statement] = STATE(2498),
    [sym_for_statement] = STATE(2498),
    [sym_for_in_statement] = STATE(2498),
    [sym_for_of_statement] = STATE(2498),
    [sym_while_statement] = STATE(2498),
    [sym_do_statement] = STATE(2498),
    [sym_try_statement] = STATE(2498),
    [sym_with_statement] = STATE(2498),
    [sym_break_statement] = STATE(2498),
    [sym_continue_statement] = STATE(2498),
    [sym_debugger_statement] = STATE(2498),
    [sym_return_statement] = STATE(2498),
    [sym_throw_statement] = STATE(2498),
    [sym_empty_statement] = STATE(2498),
    [sym_labeled_statement] = STATE(2498),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2417] = {
    [sym_export_statement] = STATE(2499),
    [sym__declaration] = STATE(2499),
    [sym_import_statement] = STATE(2499),
    [sym_expression_statement] = STATE(2499),
    [sym_variable_declaration] = STATE(2499),
    [sym_lexical_declaration] = STATE(2499),
    [sym_statement_block] = STATE(2499),
    [sym_if_statement] = STATE(2499),
    [sym_switch_statement] = STATE(2499),
    [sym_for_statement] = STATE(2499),
    [sym_for_in_statement] = STATE(2499),
    [sym_for_of_statement] = STATE(2499),
    [sym_while_statement] = STATE(2499),
    [sym_do_statement] = STATE(2499),
    [sym_try_statement] = STATE(2499),
    [sym_with_statement] = STATE(2499),
    [sym_break_statement] = STATE(2499),
    [sym_continue_statement] = STATE(2499),
    [sym_debugger_statement] = STATE(2499),
    [sym_return_statement] = STATE(2499),
    [sym_throw_statement] = STATE(2499),
    [sym_empty_statement] = STATE(2499),
    [sym_labeled_statement] = STATE(2499),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2418] = {
    [sym_export_statement] = STATE(2500),
    [sym__declaration] = STATE(2500),
    [sym_import_statement] = STATE(2500),
    [sym_expression_statement] = STATE(2500),
    [sym_variable_declaration] = STATE(2500),
    [sym_lexical_declaration] = STATE(2500),
    [sym_statement_block] = STATE(2500),
    [sym_if_statement] = STATE(2500),
    [sym_switch_statement] = STATE(2500),
    [sym_for_statement] = STATE(2500),
    [sym_for_in_statement] = STATE(2500),
    [sym_for_of_statement] = STATE(2500),
    [sym_while_statement] = STATE(2500),
    [sym_do_statement] = STATE(2500),
    [sym_try_statement] = STATE(2500),
    [sym_with_statement] = STATE(2500),
    [sym_break_statement] = STATE(2500),
    [sym_continue_statement] = STATE(2500),
    [sym_debugger_statement] = STATE(2500),
    [sym_return_statement] = STATE(2500),
    [sym_throw_statement] = STATE(2500),
    [sym_empty_statement] = STATE(2500),
    [sym_labeled_statement] = STATE(2500),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2419] = {
    [sym_export_statement] = STATE(2501),
    [sym__declaration] = STATE(2501),
    [sym_import_statement] = STATE(2501),
    [sym_expression_statement] = STATE(2501),
    [sym_variable_declaration] = STATE(2501),
    [sym_lexical_declaration] = STATE(2501),
    [sym_statement_block] = STATE(2501),
    [sym_if_statement] = STATE(2501),
    [sym_switch_statement] = STATE(2501),
    [sym_for_statement] = STATE(2501),
    [sym_for_in_statement] = STATE(2501),
    [sym_for_of_statement] = STATE(2501),
    [sym_while_statement] = STATE(2501),
    [sym_do_statement] = STATE(2501),
    [sym_try_statement] = STATE(2501),
    [sym_with_statement] = STATE(2501),
    [sym_break_statement] = STATE(2501),
    [sym_continue_statement] = STATE(2501),
    [sym_debugger_statement] = STATE(2501),
    [sym_return_statement] = STATE(2501),
    [sym_throw_statement] = STATE(2501),
    [sym_empty_statement] = STATE(2501),
    [sym_labeled_statement] = STATE(2501),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(939),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(940),
    [sym_function] = STATE(941),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(940),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(939),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(942),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(942),
    [anon_sym_export] = ACTIONS(1875),
    [anon_sym_LBRACE] = ACTIONS(1877),
    [anon_sym_import] = ACTIONS(1879),
    [anon_sym_var] = ACTIONS(1881),
    [anon_sym_let] = ACTIONS(1883),
    [anon_sym_const] = ACTIONS(1883),
    [anon_sym_if] = ACTIONS(1885),
    [anon_sym_switch] = ACTIONS(1887),
    [anon_sym_for] = ACTIONS(1889),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(1891),
    [anon_sym_do] = ACTIONS(1893),
    [anon_sym_try] = ACTIONS(1895),
    [anon_sym_with] = ACTIONS(1897),
    [anon_sym_break] = ACTIONS(1899),
    [anon_sym_continue] = ACTIONS(1901),
    [anon_sym_debugger] = ACTIONS(1903),
    [anon_sym_return] = ACTIONS(1905),
    [anon_sym_throw] = ACTIONS(1907),
    [anon_sym_SEMI] = ACTIONS(1909),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(211),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(1911),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2420] = {
    [anon_sym_else] = ACTIONS(4636),
    [anon_sym_while] = ACTIONS(4636),
    [sym_comment] = ACTIONS(71),
  },
  [2421] = {
    [anon_sym_else] = ACTIONS(4646),
    [anon_sym_while] = ACTIONS(4646),
    [sym_comment] = ACTIONS(71),
  },
  [2422] = {
    [anon_sym_else] = ACTIONS(4652),
    [anon_sym_while] = ACTIONS(4652),
    [sym_comment] = ACTIONS(71),
  },
  [2423] = {
    [anon_sym_else] = ACTIONS(4656),
    [anon_sym_while] = ACTIONS(4656),
    [sym_comment] = ACTIONS(71),
  },
  [2424] = {
    [anon_sym_else] = ACTIONS(4660),
    [anon_sym_while] = ACTIONS(4660),
    [sym_comment] = ACTIONS(71),
  },
  [2425] = {
    [anon_sym_else] = ACTIONS(4664),
    [anon_sym_while] = ACTIONS(4664),
    [sym_comment] = ACTIONS(71),
  },
  [2426] = {
    [anon_sym_else] = ACTIONS(4668),
    [anon_sym_while] = ACTIONS(4668),
    [sym_comment] = ACTIONS(71),
  },
  [2427] = {
    [anon_sym_else] = ACTIONS(4718),
    [anon_sym_while] = ACTIONS(4718),
    [anon_sym_finally] = ACTIONS(4718),
    [sym_comment] = ACTIONS(71),
  },
  [2428] = {
    [anon_sym_LBRACE] = ACTIONS(4332),
    [anon_sym_GT] = ACTIONS(4332),
    [anon_sym_SLASH] = ACTIONS(4334),
    [sym_jsx_identifier] = ACTIONS(4332),
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(4334),
  },
  [2429] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_default] = ACTIONS(3037),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_case] = ACTIONS(3037),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3035),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3035),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [2430] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_default] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_case] = ACTIONS(3858),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3856),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3856),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [2431] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_default] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_case] = ACTIONS(3862),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3860),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3860),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [2432] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2433] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_default] = ACTIONS(3936),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_case] = ACTIONS(3936),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3934),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3934),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [2434] = {
    [anon_sym_export] = ACTIONS(2345),
    [anon_sym_default] = ACTIONS(2345),
    [anon_sym_LBRACE] = ACTIONS(2343),
    [anon_sym_RBRACE] = ACTIONS(2343),
    [anon_sym_import] = ACTIONS(2345),
    [anon_sym_var] = ACTIONS(2345),
    [anon_sym_let] = ACTIONS(2345),
    [anon_sym_const] = ACTIONS(2345),
    [anon_sym_if] = ACTIONS(2345),
    [anon_sym_else] = ACTIONS(2345),
    [anon_sym_switch] = ACTIONS(2345),
    [anon_sym_for] = ACTIONS(2345),
    [anon_sym_LPAREN] = ACTIONS(2343),
    [anon_sym_while] = ACTIONS(2345),
    [anon_sym_do] = ACTIONS(2345),
    [anon_sym_try] = ACTIONS(2345),
    [anon_sym_with] = ACTIONS(2345),
    [anon_sym_break] = ACTIONS(2345),
    [anon_sym_continue] = ACTIONS(2345),
    [anon_sym_debugger] = ACTIONS(2345),
    [anon_sym_return] = ACTIONS(2345),
    [anon_sym_throw] = ACTIONS(2345),
    [anon_sym_SEMI] = ACTIONS(2343),
    [anon_sym_case] = ACTIONS(2345),
    [anon_sym_yield] = ACTIONS(2345),
    [anon_sym_LBRACK] = ACTIONS(2343),
    [anon_sym_LT] = ACTIONS(2343),
    [anon_sym_class] = ACTIONS(2345),
    [anon_sym_async] = ACTIONS(2345),
    [anon_sym_function] = ACTIONS(2345),
    [anon_sym_EQ_GT] = ACTIONS(2343),
    [anon_sym_new] = ACTIONS(2345),
    [anon_sym_await] = ACTIONS(2345),
    [anon_sym_PLUS] = ACTIONS(2345),
    [anon_sym_DASH] = ACTIONS(2345),
    [anon_sym_BANG] = ACTIONS(2343),
    [anon_sym_TILDE] = ACTIONS(2343),
    [anon_sym_typeof] = ACTIONS(2345),
    [anon_sym_void] = ACTIONS(2345),
    [anon_sym_delete] = ACTIONS(2345),
    [anon_sym_PLUS_PLUS] = ACTIONS(2343),
    [anon_sym_DASH_DASH] = ACTIONS(2343),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2343),
    [anon_sym_BQUOTE] = ACTIONS(2343),
    [sym_regex] = ACTIONS(2345),
    [sym_number] = ACTIONS(2343),
    [sym_identifier] = ACTIONS(2345),
    [sym_this] = ACTIONS(2345),
    [sym_super] = ACTIONS(2345),
    [sym_true] = ACTIONS(2345),
    [sym_false] = ACTIONS(2345),
    [sym_null] = ACTIONS(2345),
    [sym_undefined] = ACTIONS(2345),
    [anon_sym_AT] = ACTIONS(2343),
    [anon_sym_get] = ACTIONS(2345),
    [anon_sym_set] = ACTIONS(2345),
  },
  [2435] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(2503),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(5180),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2436] = {
    [sym_class_body] = STATE(2504),
    [anon_sym_LBRACE] = ACTIONS(5100),
    [sym_comment] = ACTIONS(71),
  },
  [2437] = {
    [sym__automatic_semicolon] = ACTIONS(2140),
    [anon_sym_export] = ACTIONS(2142),
    [anon_sym_default] = ACTIONS(2142),
    [anon_sym_LBRACE] = ACTIONS(2140),
    [anon_sym_RBRACE] = ACTIONS(2140),
    [anon_sym_import] = ACTIONS(2142),
    [anon_sym_var] = ACTIONS(2142),
    [anon_sym_let] = ACTIONS(2142),
    [anon_sym_const] = ACTIONS(2142),
    [anon_sym_if] = ACTIONS(2142),
    [anon_sym_else] = ACTIONS(2142),
    [anon_sym_switch] = ACTIONS(2142),
    [anon_sym_for] = ACTIONS(2142),
    [anon_sym_LPAREN] = ACTIONS(2140),
    [anon_sym_while] = ACTIONS(2142),
    [anon_sym_do] = ACTIONS(2142),
    [anon_sym_try] = ACTIONS(2142),
    [anon_sym_with] = ACTIONS(2142),
    [anon_sym_break] = ACTIONS(2142),
    [anon_sym_continue] = ACTIONS(2142),
    [anon_sym_debugger] = ACTIONS(2142),
    [anon_sym_return] = ACTIONS(2142),
    [anon_sym_throw] = ACTIONS(2142),
    [anon_sym_SEMI] = ACTIONS(2140),
    [anon_sym_case] = ACTIONS(2142),
    [anon_sym_yield] = ACTIONS(2142),
    [anon_sym_LBRACK] = ACTIONS(2140),
    [anon_sym_LT] = ACTIONS(2140),
    [anon_sym_class] = ACTIONS(2142),
    [anon_sym_async] = ACTIONS(2142),
    [anon_sym_function] = ACTIONS(2142),
    [anon_sym_EQ_GT] = ACTIONS(2140),
    [anon_sym_new] = ACTIONS(2142),
    [anon_sym_await] = ACTIONS(2142),
    [anon_sym_PLUS] = ACTIONS(2142),
    [anon_sym_DASH] = ACTIONS(2142),
    [anon_sym_BANG] = ACTIONS(2140),
    [anon_sym_TILDE] = ACTIONS(2140),
    [anon_sym_typeof] = ACTIONS(2142),
    [anon_sym_void] = ACTIONS(2142),
    [anon_sym_delete] = ACTIONS(2142),
    [anon_sym_PLUS_PLUS] = ACTIONS(2140),
    [anon_sym_DASH_DASH] = ACTIONS(2140),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2140),
    [anon_sym_BQUOTE] = ACTIONS(2140),
    [sym_regex] = ACTIONS(2142),
    [sym_number] = ACTIONS(2140),
    [sym_identifier] = ACTIONS(2142),
    [sym_this] = ACTIONS(2142),
    [sym_super] = ACTIONS(2142),
    [sym_true] = ACTIONS(2142),
    [sym_false] = ACTIONS(2142),
    [sym_null] = ACTIONS(2142),
    [sym_undefined] = ACTIONS(2142),
    [anon_sym_AT] = ACTIONS(2140),
    [anon_sym_get] = ACTIONS(2142),
    [anon_sym_set] = ACTIONS(2142),
  },
  [2438] = {
    [sym_formal_parameters] = STATE(2505),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2439] = {
    [sym_statement_block] = STATE(2506),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [sym_comment] = ACTIONS(71),
  },
  [2440] = {
    [sym_formal_parameters] = STATE(2507),
    [anon_sym_LPAREN] = ACTIONS(312),
    [sym_comment] = ACTIONS(71),
  },
  [2441] = {
    [sym_statement_block] = STATE(2508),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [sym_comment] = ACTIONS(71),
  },
  [2442] = {
    [sym_export_statement] = STATE(2510),
    [sym__declaration] = STATE(2510),
    [sym_import_statement] = STATE(2510),
    [sym_expression_statement] = STATE(2510),
    [sym_variable_declaration] = STATE(2510),
    [sym_lexical_declaration] = STATE(2510),
    [sym_statement_block] = STATE(2510),
    [sym_if_statement] = STATE(2510),
    [sym_switch_statement] = STATE(2510),
    [sym_for_statement] = STATE(2510),
    [sym_for_in_statement] = STATE(2510),
    [sym_for_of_statement] = STATE(2510),
    [sym_while_statement] = STATE(2510),
    [sym_do_statement] = STATE(2510),
    [sym_try_statement] = STATE(2510),
    [sym_with_statement] = STATE(2510),
    [sym_break_statement] = STATE(2510),
    [sym_continue_statement] = STATE(2510),
    [sym_debugger_statement] = STATE(2510),
    [sym_return_statement] = STATE(2510),
    [sym_throw_statement] = STATE(2510),
    [sym_empty_statement] = STATE(2510),
    [sym_labeled_statement] = STATE(2510),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2510),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5182),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2443] = {
    [sym__automatic_semicolon] = ACTIONS(2178),
    [anon_sym_export] = ACTIONS(2180),
    [anon_sym_default] = ACTIONS(2180),
    [anon_sym_LBRACE] = ACTIONS(2178),
    [anon_sym_RBRACE] = ACTIONS(2178),
    [anon_sym_import] = ACTIONS(2180),
    [anon_sym_var] = ACTIONS(2180),
    [anon_sym_let] = ACTIONS(2180),
    [anon_sym_const] = ACTIONS(2180),
    [anon_sym_if] = ACTIONS(2180),
    [anon_sym_else] = ACTIONS(2180),
    [anon_sym_switch] = ACTIONS(2180),
    [anon_sym_for] = ACTIONS(2180),
    [anon_sym_LPAREN] = ACTIONS(2178),
    [anon_sym_while] = ACTIONS(2180),
    [anon_sym_do] = ACTIONS(2180),
    [anon_sym_try] = ACTIONS(2180),
    [anon_sym_with] = ACTIONS(2180),
    [anon_sym_break] = ACTIONS(2180),
    [anon_sym_continue] = ACTIONS(2180),
    [anon_sym_debugger] = ACTIONS(2180),
    [anon_sym_return] = ACTIONS(2180),
    [anon_sym_throw] = ACTIONS(2180),
    [anon_sym_SEMI] = ACTIONS(2178),
    [anon_sym_case] = ACTIONS(2180),
    [anon_sym_yield] = ACTIONS(2180),
    [anon_sym_LBRACK] = ACTIONS(2178),
    [anon_sym_LT] = ACTIONS(2178),
    [anon_sym_class] = ACTIONS(2180),
    [anon_sym_async] = ACTIONS(2180),
    [anon_sym_function] = ACTIONS(2180),
    [anon_sym_EQ_GT] = ACTIONS(2178),
    [anon_sym_new] = ACTIONS(2180),
    [anon_sym_await] = ACTIONS(2180),
    [anon_sym_PLUS] = ACTIONS(2180),
    [anon_sym_DASH] = ACTIONS(2180),
    [anon_sym_BANG] = ACTIONS(2178),
    [anon_sym_TILDE] = ACTIONS(2178),
    [anon_sym_typeof] = ACTIONS(2180),
    [anon_sym_void] = ACTIONS(2180),
    [anon_sym_delete] = ACTIONS(2180),
    [anon_sym_PLUS_PLUS] = ACTIONS(2178),
    [anon_sym_DASH_DASH] = ACTIONS(2178),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2178),
    [anon_sym_BQUOTE] = ACTIONS(2178),
    [sym_regex] = ACTIONS(2180),
    [sym_number] = ACTIONS(2178),
    [sym_identifier] = ACTIONS(2180),
    [sym_this] = ACTIONS(2180),
    [sym_super] = ACTIONS(2180),
    [sym_true] = ACTIONS(2180),
    [sym_false] = ACTIONS(2180),
    [sym_null] = ACTIONS(2180),
    [sym_undefined] = ACTIONS(2180),
    [anon_sym_AT] = ACTIONS(2178),
    [anon_sym_get] = ACTIONS(2180),
    [anon_sym_set] = ACTIONS(2180),
  },
  [2444] = {
    [sym_class_heritage] = STATE(2511),
    [sym_class_body] = STATE(2504),
    [anon_sym_LBRACE] = ACTIONS(5100),
    [anon_sym_extends] = ACTIONS(1010),
    [sym_comment] = ACTIONS(71),
  },
  [2445] = {
    [anon_sym_export] = ACTIONS(2609),
    [anon_sym_default] = ACTIONS(2609),
    [anon_sym_LBRACE] = ACTIONS(2607),
    [anon_sym_RBRACE] = ACTIONS(2607),
    [anon_sym_import] = ACTIONS(2609),
    [anon_sym_var] = ACTIONS(2609),
    [anon_sym_let] = ACTIONS(2609),
    [anon_sym_const] = ACTIONS(2609),
    [anon_sym_if] = ACTIONS(2609),
    [anon_sym_else] = ACTIONS(2609),
    [anon_sym_switch] = ACTIONS(2609),
    [anon_sym_for] = ACTIONS(2609),
    [anon_sym_LPAREN] = ACTIONS(2607),
    [anon_sym_while] = ACTIONS(2609),
    [anon_sym_do] = ACTIONS(2609),
    [anon_sym_try] = ACTIONS(2609),
    [anon_sym_with] = ACTIONS(2609),
    [anon_sym_break] = ACTIONS(2609),
    [anon_sym_continue] = ACTIONS(2609),
    [anon_sym_debugger] = ACTIONS(2609),
    [anon_sym_return] = ACTIONS(2609),
    [anon_sym_throw] = ACTIONS(2609),
    [anon_sym_SEMI] = ACTIONS(2607),
    [anon_sym_case] = ACTIONS(2609),
    [anon_sym_yield] = ACTIONS(2609),
    [anon_sym_LBRACK] = ACTIONS(2607),
    [anon_sym_LT] = ACTIONS(2607),
    [anon_sym_class] = ACTIONS(2609),
    [anon_sym_async] = ACTIONS(2609),
    [anon_sym_function] = ACTIONS(2609),
    [anon_sym_EQ_GT] = ACTIONS(2607),
    [anon_sym_new] = ACTIONS(2609),
    [anon_sym_await] = ACTIONS(2609),
    [anon_sym_PLUS] = ACTIONS(2609),
    [anon_sym_DASH] = ACTIONS(2609),
    [anon_sym_BANG] = ACTIONS(2607),
    [anon_sym_TILDE] = ACTIONS(2607),
    [anon_sym_typeof] = ACTIONS(2609),
    [anon_sym_void] = ACTIONS(2609),
    [anon_sym_delete] = ACTIONS(2609),
    [anon_sym_PLUS_PLUS] = ACTIONS(2607),
    [anon_sym_DASH_DASH] = ACTIONS(2607),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2607),
    [anon_sym_BQUOTE] = ACTIONS(2607),
    [sym_regex] = ACTIONS(2609),
    [sym_number] = ACTIONS(2607),
    [sym_identifier] = ACTIONS(2609),
    [sym_this] = ACTIONS(2609),
    [sym_super] = ACTIONS(2609),
    [sym_true] = ACTIONS(2609),
    [sym_false] = ACTIONS(2609),
    [sym_null] = ACTIONS(2609),
    [sym_undefined] = ACTIONS(2609),
    [anon_sym_AT] = ACTIONS(2607),
    [anon_sym_get] = ACTIONS(2609),
    [anon_sym_set] = ACTIONS(2609),
  },
  [2446] = {
    [anon_sym_export] = ACTIONS(2621),
    [anon_sym_default] = ACTIONS(2621),
    [anon_sym_LBRACE] = ACTIONS(2619),
    [anon_sym_RBRACE] = ACTIONS(2619),
    [anon_sym_import] = ACTIONS(2621),
    [anon_sym_var] = ACTIONS(2621),
    [anon_sym_let] = ACTIONS(2621),
    [anon_sym_const] = ACTIONS(2621),
    [anon_sym_if] = ACTIONS(2621),
    [anon_sym_else] = ACTIONS(2621),
    [anon_sym_switch] = ACTIONS(2621),
    [anon_sym_for] = ACTIONS(2621),
    [anon_sym_LPAREN] = ACTIONS(2619),
    [anon_sym_while] = ACTIONS(2621),
    [anon_sym_do] = ACTIONS(2621),
    [anon_sym_try] = ACTIONS(2621),
    [anon_sym_with] = ACTIONS(2621),
    [anon_sym_break] = ACTIONS(2621),
    [anon_sym_continue] = ACTIONS(2621),
    [anon_sym_debugger] = ACTIONS(2621),
    [anon_sym_return] = ACTIONS(2621),
    [anon_sym_throw] = ACTIONS(2621),
    [anon_sym_SEMI] = ACTIONS(2619),
    [anon_sym_case] = ACTIONS(2621),
    [anon_sym_yield] = ACTIONS(2621),
    [anon_sym_LBRACK] = ACTIONS(2619),
    [anon_sym_LT] = ACTIONS(2619),
    [anon_sym_class] = ACTIONS(2621),
    [anon_sym_async] = ACTIONS(2621),
    [anon_sym_function] = ACTIONS(2621),
    [anon_sym_EQ_GT] = ACTIONS(2619),
    [anon_sym_new] = ACTIONS(2621),
    [anon_sym_await] = ACTIONS(2621),
    [anon_sym_PLUS] = ACTIONS(2621),
    [anon_sym_DASH] = ACTIONS(2621),
    [anon_sym_BANG] = ACTIONS(2619),
    [anon_sym_TILDE] = ACTIONS(2619),
    [anon_sym_typeof] = ACTIONS(2621),
    [anon_sym_void] = ACTIONS(2621),
    [anon_sym_delete] = ACTIONS(2621),
    [anon_sym_PLUS_PLUS] = ACTIONS(2619),
    [anon_sym_DASH_DASH] = ACTIONS(2619),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2619),
    [anon_sym_BQUOTE] = ACTIONS(2619),
    [sym_regex] = ACTIONS(2621),
    [sym_number] = ACTIONS(2619),
    [sym_identifier] = ACTIONS(2621),
    [sym_this] = ACTIONS(2621),
    [sym_super] = ACTIONS(2621),
    [sym_true] = ACTIONS(2621),
    [sym_false] = ACTIONS(2621),
    [sym_null] = ACTIONS(2621),
    [sym_undefined] = ACTIONS(2621),
    [anon_sym_AT] = ACTIONS(2619),
    [anon_sym_get] = ACTIONS(2621),
    [anon_sym_set] = ACTIONS(2621),
  },
  [2447] = {
    [anon_sym_export] = ACTIONS(2628),
    [anon_sym_default] = ACTIONS(2628),
    [anon_sym_LBRACE] = ACTIONS(2626),
    [anon_sym_RBRACE] = ACTIONS(2626),
    [anon_sym_import] = ACTIONS(2628),
    [anon_sym_var] = ACTIONS(2628),
    [anon_sym_let] = ACTIONS(2628),
    [anon_sym_const] = ACTIONS(2628),
    [anon_sym_if] = ACTIONS(2628),
    [anon_sym_else] = ACTIONS(2628),
    [anon_sym_switch] = ACTIONS(2628),
    [anon_sym_for] = ACTIONS(2628),
    [anon_sym_LPAREN] = ACTIONS(2626),
    [anon_sym_while] = ACTIONS(2628),
    [anon_sym_do] = ACTIONS(2628),
    [anon_sym_try] = ACTIONS(2628),
    [anon_sym_with] = ACTIONS(2628),
    [anon_sym_break] = ACTIONS(2628),
    [anon_sym_continue] = ACTIONS(2628),
    [anon_sym_debugger] = ACTIONS(2628),
    [anon_sym_return] = ACTIONS(2628),
    [anon_sym_throw] = ACTIONS(2628),
    [anon_sym_SEMI] = ACTIONS(2626),
    [anon_sym_case] = ACTIONS(2628),
    [anon_sym_yield] = ACTIONS(2628),
    [anon_sym_LBRACK] = ACTIONS(2626),
    [anon_sym_LT] = ACTIONS(2626),
    [anon_sym_class] = ACTIONS(2628),
    [anon_sym_async] = ACTIONS(2628),
    [anon_sym_function] = ACTIONS(2628),
    [anon_sym_EQ_GT] = ACTIONS(2626),
    [anon_sym_new] = ACTIONS(2628),
    [anon_sym_await] = ACTIONS(2628),
    [anon_sym_PLUS] = ACTIONS(2628),
    [anon_sym_DASH] = ACTIONS(2628),
    [anon_sym_BANG] = ACTIONS(2626),
    [anon_sym_TILDE] = ACTIONS(2626),
    [anon_sym_typeof] = ACTIONS(2628),
    [anon_sym_void] = ACTIONS(2628),
    [anon_sym_delete] = ACTIONS(2628),
    [anon_sym_PLUS_PLUS] = ACTIONS(2626),
    [anon_sym_DASH_DASH] = ACTIONS(2626),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2626),
    [anon_sym_BQUOTE] = ACTIONS(2626),
    [sym_regex] = ACTIONS(2628),
    [sym_number] = ACTIONS(2626),
    [sym_identifier] = ACTIONS(2628),
    [sym_this] = ACTIONS(2628),
    [sym_super] = ACTIONS(2628),
    [sym_true] = ACTIONS(2628),
    [sym_false] = ACTIONS(2628),
    [sym_null] = ACTIONS(2628),
    [sym_undefined] = ACTIONS(2628),
    [anon_sym_AT] = ACTIONS(2626),
    [anon_sym_get] = ACTIONS(2628),
    [anon_sym_set] = ACTIONS(2628),
  },
  [2448] = {
    [sym_export_statement] = STATE(2512),
    [sym__declaration] = STATE(2512),
    [sym_import_statement] = STATE(2512),
    [sym_expression_statement] = STATE(2512),
    [sym_variable_declaration] = STATE(2512),
    [sym_lexical_declaration] = STATE(2512),
    [sym_statement_block] = STATE(2512),
    [sym_if_statement] = STATE(2512),
    [sym_switch_statement] = STATE(2512),
    [sym_for_statement] = STATE(2512),
    [sym_for_in_statement] = STATE(2512),
    [sym_for_of_statement] = STATE(2512),
    [sym_while_statement] = STATE(2512),
    [sym_do_statement] = STATE(2512),
    [sym_try_statement] = STATE(2512),
    [sym_with_statement] = STATE(2512),
    [sym_break_statement] = STATE(2512),
    [sym_continue_statement] = STATE(2512),
    [sym_debugger_statement] = STATE(2512),
    [sym_return_statement] = STATE(2512),
    [sym_throw_statement] = STATE(2512),
    [sym_empty_statement] = STATE(2512),
    [sym_labeled_statement] = STATE(2512),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2449] = {
    [anon_sym_export] = ACTIONS(2694),
    [anon_sym_default] = ACTIONS(2694),
    [anon_sym_LBRACE] = ACTIONS(2692),
    [anon_sym_RBRACE] = ACTIONS(2692),
    [anon_sym_import] = ACTIONS(2694),
    [anon_sym_var] = ACTIONS(2694),
    [anon_sym_let] = ACTIONS(2694),
    [anon_sym_const] = ACTIONS(2694),
    [anon_sym_if] = ACTIONS(2694),
    [anon_sym_else] = ACTIONS(2694),
    [anon_sym_switch] = ACTIONS(2694),
    [anon_sym_for] = ACTIONS(2694),
    [anon_sym_LPAREN] = ACTIONS(2692),
    [anon_sym_while] = ACTIONS(2694),
    [anon_sym_do] = ACTIONS(2694),
    [anon_sym_try] = ACTIONS(2694),
    [anon_sym_with] = ACTIONS(2694),
    [anon_sym_break] = ACTIONS(2694),
    [anon_sym_continue] = ACTIONS(2694),
    [anon_sym_debugger] = ACTIONS(2694),
    [anon_sym_return] = ACTIONS(2694),
    [anon_sym_throw] = ACTIONS(2694),
    [anon_sym_SEMI] = ACTIONS(2692),
    [anon_sym_case] = ACTIONS(2694),
    [anon_sym_yield] = ACTIONS(2694),
    [anon_sym_LBRACK] = ACTIONS(2692),
    [anon_sym_LT] = ACTIONS(2692),
    [anon_sym_class] = ACTIONS(2694),
    [anon_sym_async] = ACTIONS(2694),
    [anon_sym_function] = ACTIONS(2694),
    [anon_sym_EQ_GT] = ACTIONS(2692),
    [anon_sym_new] = ACTIONS(2694),
    [anon_sym_await] = ACTIONS(2694),
    [anon_sym_PLUS] = ACTIONS(2694),
    [anon_sym_DASH] = ACTIONS(2694),
    [anon_sym_BANG] = ACTIONS(2692),
    [anon_sym_TILDE] = ACTIONS(2692),
    [anon_sym_typeof] = ACTIONS(2694),
    [anon_sym_void] = ACTIONS(2694),
    [anon_sym_delete] = ACTIONS(2694),
    [anon_sym_PLUS_PLUS] = ACTIONS(2692),
    [anon_sym_DASH_DASH] = ACTIONS(2692),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2692),
    [anon_sym_BQUOTE] = ACTIONS(2692),
    [sym_regex] = ACTIONS(2694),
    [sym_number] = ACTIONS(2692),
    [sym_identifier] = ACTIONS(2694),
    [sym_this] = ACTIONS(2694),
    [sym_super] = ACTIONS(2694),
    [sym_true] = ACTIONS(2694),
    [sym_false] = ACTIONS(2694),
    [sym_null] = ACTIONS(2694),
    [sym_undefined] = ACTIONS(2694),
    [anon_sym_AT] = ACTIONS(2692),
    [anon_sym_get] = ACTIONS(2694),
    [anon_sym_set] = ACTIONS(2694),
  },
  [2450] = {
    [sym_switch_case] = STATE(1276),
    [sym_switch_default] = STATE(1276),
    [aux_sym_switch_body_repeat1] = STATE(1276),
    [anon_sym_default] = ACTIONS(1717),
    [anon_sym_RBRACE] = ACTIONS(5184),
    [anon_sym_case] = ACTIONS(1721),
    [sym_comment] = ACTIONS(71),
  },
  [2451] = {
    [sym__initializer] = STATE(347),
    [sym__automatic_semicolon] = ACTIONS(602),
    [anon_sym_COMMA] = ACTIONS(602),
    [anon_sym_in] = ACTIONS(5186),
    [anon_sym_of] = ACTIONS(5188),
    [anon_sym_SEMI] = ACTIONS(602),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [2452] = {
    [sym__initializer] = STATE(351),
    [sym__automatic_semicolon] = ACTIONS(610),
    [anon_sym_COMMA] = ACTIONS(610),
    [anon_sym_in] = ACTIONS(5190),
    [anon_sym_of] = ACTIONS(5192),
    [anon_sym_SEMI] = ACTIONS(610),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [2453] = {
    [sym__initializer] = STATE(352),
    [sym__automatic_semicolon] = ACTIONS(612),
    [anon_sym_COMMA] = ACTIONS(612),
    [anon_sym_in] = ACTIONS(5194),
    [anon_sym_of] = ACTIONS(5196),
    [anon_sym_SEMI] = ACTIONS(612),
    [anon_sym_EQ] = ACTIONS(604),
    [sym_comment] = ACTIONS(71),
  },
  [2454] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2520),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2520),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2455] = {
    [sym_parenthesized_expression] = STATE(2521),
    [sym__expression] = STATE(2521),
    [sym_yield_expression] = STATE(2521),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2521),
    [sym_jsx_fragment] = STATE(2521),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2521),
    [sym_anonymous_class] = STATE(2521),
    [sym_class] = STATE(2521),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2521),
    [sym_generator_function] = STATE(2521),
    [sym_call_expression] = STATE(2521),
    [sym_new_expression] = STATE(2521),
    [sym_await_expression] = STATE(2521),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2521),
    [sym_augmented_assignment_expression] = STATE(2521),
    [sym_ternary_expression] = STATE(2521),
    [sym_binary_expression] = STATE(2521),
    [sym_unary_expression] = STATE(2521),
    [sym_update_expression] = STATE(2521),
    [sym_template_string] = STATE(2521),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5198),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5200),
    [sym_number] = ACTIONS(5198),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5200),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5200),
    [sym_false] = ACTIONS(5200),
    [sym_null] = ACTIONS(5200),
    [sym_undefined] = ACTIONS(5200),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2456] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2523),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2523),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_RPAREN] = ACTIONS(5202),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2457] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2524),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2524),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2458] = {
    [sym_parenthesized_expression] = STATE(2525),
    [sym__expression] = STATE(2525),
    [sym_yield_expression] = STATE(2525),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2525),
    [sym_jsx_fragment] = STATE(2525),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2525),
    [sym_anonymous_class] = STATE(2525),
    [sym_class] = STATE(2525),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2525),
    [sym_generator_function] = STATE(2525),
    [sym_call_expression] = STATE(2525),
    [sym_new_expression] = STATE(2525),
    [sym_await_expression] = STATE(2525),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2525),
    [sym_augmented_assignment_expression] = STATE(2525),
    [sym_ternary_expression] = STATE(2525),
    [sym_binary_expression] = STATE(2525),
    [sym_unary_expression] = STATE(2525),
    [sym_update_expression] = STATE(2525),
    [sym_template_string] = STATE(2525),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5204),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5206),
    [sym_number] = ACTIONS(5204),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5206),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5206),
    [sym_false] = ACTIONS(5206),
    [sym_null] = ACTIONS(5206),
    [sym_undefined] = ACTIONS(5206),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2459] = {
    [sym_parenthesized_expression] = STATE(1297),
    [sym__expressions] = STATE(2526),
    [sym__expression] = STATE(1297),
    [sym_yield_expression] = STATE(1297),
    [sym_object] = STATE(1298),
    [sym_array] = STATE(1299),
    [sym_jsx_element] = STATE(1297),
    [sym_jsx_fragment] = STATE(1297),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(1297),
    [sym_anonymous_class] = STATE(1297),
    [sym_class] = STATE(1297),
    [sym_function] = STATE(156),
    [sym_arrow_function] = STATE(1297),
    [sym_generator_function] = STATE(1297),
    [sym_call_expression] = STATE(1297),
    [sym_new_expression] = STATE(1297),
    [sym_await_expression] = STATE(1297),
    [sym_member_expression] = STATE(1300),
    [sym_subscript_expression] = STATE(1300),
    [sym_assignment_expression] = STATE(1297),
    [sym_augmented_assignment_expression] = STATE(1297),
    [sym_ternary_expression] = STATE(1297),
    [sym_binary_expression] = STATE(1297),
    [sym_unary_expression] = STATE(1297),
    [sym_update_expression] = STATE(1297),
    [sym_sequence_expression] = STATE(2526),
    [sym_template_string] = STATE(1297),
    [sym_decorator] = STATE(157),
    [sym_formal_parameters] = STATE(1301),
    [aux_sym_export_statement_repeat1] = STATE(157),
    [anon_sym_LBRACE] = ACTIONS(231),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_yield] = ACTIONS(2738),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(209),
    [anon_sym_async] = ACTIONS(2740),
    [anon_sym_function] = ACTIONS(213),
    [anon_sym_EQ_GT] = ACTIONS(2742),
    [anon_sym_new] = ACTIONS(2744),
    [anon_sym_await] = ACTIONS(2746),
    [anon_sym_PLUS] = ACTIONS(2748),
    [anon_sym_DASH] = ACTIONS(2748),
    [anon_sym_BANG] = ACTIONS(2750),
    [anon_sym_TILDE] = ACTIONS(2750),
    [anon_sym_typeof] = ACTIONS(2748),
    [anon_sym_void] = ACTIONS(2748),
    [anon_sym_delete] = ACTIONS(2748),
    [anon_sym_PLUS_PLUS] = ACTIONS(2752),
    [anon_sym_DASH_DASH] = ACTIONS(2752),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2754),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(2756),
    [sym_number] = ACTIONS(2754),
    [sym_identifier] = ACTIONS(2758),
    [sym_this] = ACTIONS(2756),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(2756),
    [sym_false] = ACTIONS(2756),
    [sym_null] = ACTIONS(2756),
    [sym_undefined] = ACTIONS(2756),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2760),
    [anon_sym_set] = ACTIONS(2760),
  },
  [2460] = {
    [sym_parenthesized_expression] = STATE(2527),
    [sym__expression] = STATE(2527),
    [sym_yield_expression] = STATE(2527),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2527),
    [sym_jsx_fragment] = STATE(2527),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2527),
    [sym_anonymous_class] = STATE(2527),
    [sym_class] = STATE(2527),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2527),
    [sym_generator_function] = STATE(2527),
    [sym_call_expression] = STATE(2527),
    [sym_new_expression] = STATE(2527),
    [sym_await_expression] = STATE(2527),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2527),
    [sym_augmented_assignment_expression] = STATE(2527),
    [sym_ternary_expression] = STATE(2527),
    [sym_binary_expression] = STATE(2527),
    [sym_unary_expression] = STATE(2527),
    [sym_update_expression] = STATE(2527),
    [sym_template_string] = STATE(2527),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5208),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5210),
    [sym_number] = ACTIONS(5208),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5210),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5210),
    [sym_false] = ACTIONS(5210),
    [sym_null] = ACTIONS(5210),
    [sym_undefined] = ACTIONS(5210),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2461] = {
    [sym__automatic_semicolon] = ACTIONS(5212),
    [anon_sym_SEMI] = ACTIONS(5212),
    [sym_comment] = ACTIONS(71),
  },
  [2462] = {
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_catch] = ACTIONS(1590),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2463] = {
    [sym_export_statement] = STATE(2530),
    [sym__declaration] = STATE(2530),
    [sym_import_statement] = STATE(2530),
    [sym_expression_statement] = STATE(2530),
    [sym_variable_declaration] = STATE(2530),
    [sym_lexical_declaration] = STATE(2530),
    [sym_statement_block] = STATE(2530),
    [sym_if_statement] = STATE(2530),
    [sym_switch_statement] = STATE(2530),
    [sym_for_statement] = STATE(2530),
    [sym_for_in_statement] = STATE(2530),
    [sym_for_of_statement] = STATE(2530),
    [sym_while_statement] = STATE(2530),
    [sym_do_statement] = STATE(2530),
    [sym_try_statement] = STATE(2530),
    [sym_with_statement] = STATE(2530),
    [sym_break_statement] = STATE(2530),
    [sym_continue_statement] = STATE(2530),
    [sym_debugger_statement] = STATE(2530),
    [sym_return_statement] = STATE(2530),
    [sym_throw_statement] = STATE(2530),
    [sym_empty_statement] = STATE(2530),
    [sym_labeled_statement] = STATE(2530),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2530),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5214),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2464] = {
    [sym_comment] = ACTIONS(71),
    [sym_identifier] = ACTIONS(5216),
  },
  [2465] = {
    [anon_sym_export] = ACTIONS(2927),
    [anon_sym_default] = ACTIONS(2927),
    [anon_sym_LBRACE] = ACTIONS(2925),
    [anon_sym_RBRACE] = ACTIONS(2925),
    [anon_sym_import] = ACTIONS(2927),
    [anon_sym_var] = ACTIONS(2927),
    [anon_sym_let] = ACTIONS(2927),
    [anon_sym_const] = ACTIONS(2927),
    [anon_sym_if] = ACTIONS(2927),
    [anon_sym_else] = ACTIONS(2927),
    [anon_sym_switch] = ACTIONS(2927),
    [anon_sym_for] = ACTIONS(2927),
    [anon_sym_LPAREN] = ACTIONS(2925),
    [anon_sym_while] = ACTIONS(2927),
    [anon_sym_do] = ACTIONS(2927),
    [anon_sym_try] = ACTIONS(2927),
    [anon_sym_with] = ACTIONS(2927),
    [anon_sym_break] = ACTIONS(2927),
    [anon_sym_continue] = ACTIONS(2927),
    [anon_sym_debugger] = ACTIONS(2927),
    [anon_sym_return] = ACTIONS(2927),
    [anon_sym_throw] = ACTIONS(2927),
    [anon_sym_SEMI] = ACTIONS(2925),
    [anon_sym_case] = ACTIONS(2927),
    [anon_sym_finally] = ACTIONS(2927),
    [anon_sym_yield] = ACTIONS(2927),
    [anon_sym_LBRACK] = ACTIONS(2925),
    [anon_sym_LT] = ACTIONS(2925),
    [anon_sym_class] = ACTIONS(2927),
    [anon_sym_async] = ACTIONS(2927),
    [anon_sym_function] = ACTIONS(2927),
    [anon_sym_EQ_GT] = ACTIONS(2925),
    [anon_sym_new] = ACTIONS(2927),
    [anon_sym_await] = ACTIONS(2927),
    [anon_sym_PLUS] = ACTIONS(2927),
    [anon_sym_DASH] = ACTIONS(2927),
    [anon_sym_BANG] = ACTIONS(2925),
    [anon_sym_TILDE] = ACTIONS(2925),
    [anon_sym_typeof] = ACTIONS(2927),
    [anon_sym_void] = ACTIONS(2927),
    [anon_sym_delete] = ACTIONS(2927),
    [anon_sym_PLUS_PLUS] = ACTIONS(2925),
    [anon_sym_DASH_DASH] = ACTIONS(2925),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2925),
    [anon_sym_BQUOTE] = ACTIONS(2925),
    [sym_regex] = ACTIONS(2927),
    [sym_number] = ACTIONS(2925),
    [sym_identifier] = ACTIONS(2927),
    [sym_this] = ACTIONS(2927),
    [sym_super] = ACTIONS(2927),
    [sym_true] = ACTIONS(2927),
    [sym_false] = ACTIONS(2927),
    [sym_null] = ACTIONS(2927),
    [sym_undefined] = ACTIONS(2927),
    [anon_sym_AT] = ACTIONS(2925),
    [anon_sym_get] = ACTIONS(2927),
    [anon_sym_set] = ACTIONS(2927),
  },
  [2466] = {
    [sym_export_statement] = STATE(2228),
    [sym__declaration] = STATE(2228),
    [sym_import_statement] = STATE(2228),
    [sym_expression_statement] = STATE(2228),
    [sym_variable_declaration] = STATE(2228),
    [sym_lexical_declaration] = STATE(2228),
    [sym_statement_block] = STATE(2228),
    [sym_if_statement] = STATE(2228),
    [sym_switch_statement] = STATE(2228),
    [sym_for_statement] = STATE(2228),
    [sym_for_in_statement] = STATE(2228),
    [sym_for_of_statement] = STATE(2228),
    [sym_while_statement] = STATE(2228),
    [sym_do_statement] = STATE(2228),
    [sym_try_statement] = STATE(2228),
    [sym_with_statement] = STATE(2228),
    [sym_break_statement] = STATE(2228),
    [sym_continue_statement] = STATE(2228),
    [sym_debugger_statement] = STATE(2228),
    [sym_return_statement] = STATE(2228),
    [sym_throw_statement] = STATE(2228),
    [sym_empty_statement] = STATE(2228),
    [sym_labeled_statement] = STATE(2228),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(2228),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5218),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2467] = {
    [anon_sym_export] = ACTIONS(2933),
    [anon_sym_default] = ACTIONS(2933),
    [anon_sym_LBRACE] = ACTIONS(2931),
    [anon_sym_RBRACE] = ACTIONS(2931),
    [anon_sym_import] = ACTIONS(2933),
    [anon_sym_var] = ACTIONS(2933),
    [anon_sym_let] = ACTIONS(2933),
    [anon_sym_const] = ACTIONS(2933),
    [anon_sym_if] = ACTIONS(2933),
    [anon_sym_else] = ACTIONS(2933),
    [anon_sym_switch] = ACTIONS(2933),
    [anon_sym_for] = ACTIONS(2933),
    [anon_sym_LPAREN] = ACTIONS(2931),
    [anon_sym_while] = ACTIONS(2933),
    [anon_sym_do] = ACTIONS(2933),
    [anon_sym_try] = ACTIONS(2933),
    [anon_sym_with] = ACTIONS(2933),
    [anon_sym_break] = ACTIONS(2933),
    [anon_sym_continue] = ACTIONS(2933),
    [anon_sym_debugger] = ACTIONS(2933),
    [anon_sym_return] = ACTIONS(2933),
    [anon_sym_throw] = ACTIONS(2933),
    [anon_sym_SEMI] = ACTIONS(2931),
    [anon_sym_case] = ACTIONS(2933),
    [anon_sym_yield] = ACTIONS(2933),
    [anon_sym_LBRACK] = ACTIONS(2931),
    [anon_sym_LT] = ACTIONS(2931),
    [anon_sym_class] = ACTIONS(2933),
    [anon_sym_async] = ACTIONS(2933),
    [anon_sym_function] = ACTIONS(2933),
    [anon_sym_EQ_GT] = ACTIONS(2931),
    [anon_sym_new] = ACTIONS(2933),
    [anon_sym_await] = ACTIONS(2933),
    [anon_sym_PLUS] = ACTIONS(2933),
    [anon_sym_DASH] = ACTIONS(2933),
    [anon_sym_BANG] = ACTIONS(2931),
    [anon_sym_TILDE] = ACTIONS(2931),
    [anon_sym_typeof] = ACTIONS(2933),
    [anon_sym_void] = ACTIONS(2933),
    [anon_sym_delete] = ACTIONS(2933),
    [anon_sym_PLUS_PLUS] = ACTIONS(2931),
    [anon_sym_DASH_DASH] = ACTIONS(2931),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2931),
    [anon_sym_BQUOTE] = ACTIONS(2931),
    [sym_regex] = ACTIONS(2933),
    [sym_number] = ACTIONS(2931),
    [sym_identifier] = ACTIONS(2933),
    [sym_this] = ACTIONS(2933),
    [sym_super] = ACTIONS(2933),
    [sym_true] = ACTIONS(2933),
    [sym_false] = ACTIONS(2933),
    [sym_null] = ACTIONS(2933),
    [sym_undefined] = ACTIONS(2933),
    [anon_sym_AT] = ACTIONS(2931),
    [anon_sym_get] = ACTIONS(2933),
    [anon_sym_set] = ACTIONS(2933),
  },
  [2468] = {
    [anon_sym_export] = ACTIONS(2937),
    [anon_sym_default] = ACTIONS(2937),
    [anon_sym_LBRACE] = ACTIONS(2935),
    [anon_sym_RBRACE] = ACTIONS(2935),
    [anon_sym_import] = ACTIONS(2937),
    [anon_sym_var] = ACTIONS(2937),
    [anon_sym_let] = ACTIONS(2937),
    [anon_sym_const] = ACTIONS(2937),
    [anon_sym_if] = ACTIONS(2937),
    [anon_sym_else] = ACTIONS(2937),
    [anon_sym_switch] = ACTIONS(2937),
    [anon_sym_for] = ACTIONS(2937),
    [anon_sym_LPAREN] = ACTIONS(2935),
    [anon_sym_while] = ACTIONS(2937),
    [anon_sym_do] = ACTIONS(2937),
    [anon_sym_try] = ACTIONS(2937),
    [anon_sym_with] = ACTIONS(2937),
    [anon_sym_break] = ACTIONS(2937),
    [anon_sym_continue] = ACTIONS(2937),
    [anon_sym_debugger] = ACTIONS(2937),
    [anon_sym_return] = ACTIONS(2937),
    [anon_sym_throw] = ACTIONS(2937),
    [anon_sym_SEMI] = ACTIONS(2935),
    [anon_sym_case] = ACTIONS(2937),
    [anon_sym_yield] = ACTIONS(2937),
    [anon_sym_LBRACK] = ACTIONS(2935),
    [anon_sym_LT] = ACTIONS(2935),
    [anon_sym_class] = ACTIONS(2937),
    [anon_sym_async] = ACTIONS(2937),
    [anon_sym_function] = ACTIONS(2937),
    [anon_sym_EQ_GT] = ACTIONS(2935),
    [anon_sym_new] = ACTIONS(2937),
    [anon_sym_await] = ACTIONS(2937),
    [anon_sym_PLUS] = ACTIONS(2937),
    [anon_sym_DASH] = ACTIONS(2937),
    [anon_sym_BANG] = ACTIONS(2935),
    [anon_sym_TILDE] = ACTIONS(2935),
    [anon_sym_typeof] = ACTIONS(2937),
    [anon_sym_void] = ACTIONS(2937),
    [anon_sym_delete] = ACTIONS(2937),
    [anon_sym_PLUS_PLUS] = ACTIONS(2935),
    [anon_sym_DASH_DASH] = ACTIONS(2935),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2935),
    [anon_sym_BQUOTE] = ACTIONS(2935),
    [sym_regex] = ACTIONS(2937),
    [sym_number] = ACTIONS(2935),
    [sym_identifier] = ACTIONS(2937),
    [sym_this] = ACTIONS(2937),
    [sym_super] = ACTIONS(2937),
    [sym_true] = ACTIONS(2937),
    [sym_false] = ACTIONS(2937),
    [sym_null] = ACTIONS(2937),
    [sym_undefined] = ACTIONS(2937),
    [anon_sym_AT] = ACTIONS(2935),
    [anon_sym_get] = ACTIONS(2937),
    [anon_sym_set] = ACTIONS(2937),
  },
  [2469] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_STAR] = ACTIONS(2064),
    [anon_sym_default] = ACTIONS(2066),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_COMMA] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_else] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_in] = ACTIONS(2066),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_case] = ACTIONS(2066),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2066),
    [anon_sym_GT] = ACTIONS(2066),
    [anon_sym_SLASH] = ACTIONS(2066),
    [anon_sym_DOT] = ACTIONS(2066),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_QMARK] = ACTIONS(2064),
    [anon_sym_AMP_AMP] = ACTIONS(2064),
    [anon_sym_PIPE_PIPE] = ACTIONS(2064),
    [anon_sym_GT_GT] = ACTIONS(2066),
    [anon_sym_GT_GT_GT] = ACTIONS(2064),
    [anon_sym_LT_LT] = ACTIONS(2064),
    [anon_sym_AMP] = ACTIONS(2066),
    [anon_sym_CARET] = ACTIONS(2064),
    [anon_sym_PIPE] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_PERCENT] = ACTIONS(2064),
    [anon_sym_LT_EQ] = ACTIONS(2064),
    [anon_sym_EQ_EQ] = ACTIONS(2066),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(2064),
    [anon_sym_BANG_EQ] = ACTIONS(2066),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(2064),
    [anon_sym_GT_EQ] = ACTIONS(2064),
    [anon_sym_instanceof] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2066),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [2470] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(5220),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2471] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_STAR] = ACTIONS(3103),
    [anon_sym_default] = ACTIONS(3105),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_COMMA] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_else] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_in] = ACTIONS(3105),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_case] = ACTIONS(3105),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3105),
    [anon_sym_GT] = ACTIONS(3105),
    [anon_sym_SLASH] = ACTIONS(3105),
    [anon_sym_DOT] = ACTIONS(3105),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_QMARK] = ACTIONS(3103),
    [anon_sym_AMP_AMP] = ACTIONS(3103),
    [anon_sym_PIPE_PIPE] = ACTIONS(3103),
    [anon_sym_GT_GT] = ACTIONS(3105),
    [anon_sym_GT_GT_GT] = ACTIONS(3103),
    [anon_sym_LT_LT] = ACTIONS(3103),
    [anon_sym_AMP] = ACTIONS(3105),
    [anon_sym_CARET] = ACTIONS(3103),
    [anon_sym_PIPE] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_PERCENT] = ACTIONS(3103),
    [anon_sym_LT_EQ] = ACTIONS(3103),
    [anon_sym_EQ_EQ] = ACTIONS(3105),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3103),
    [anon_sym_BANG_EQ] = ACTIONS(3105),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3103),
    [anon_sym_GT_EQ] = ACTIONS(3103),
    [anon_sym_instanceof] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3105),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [2472] = {
    [sym_statement_block] = STATE(2534),
    [anon_sym_LBRACE] = ACTIONS(5032),
    [sym_comment] = ACTIONS(71),
  },
  [2473] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_STAR] = ACTIONS(3107),
    [anon_sym_default] = ACTIONS(3109),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_COMMA] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_else] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_in] = ACTIONS(3109),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_case] = ACTIONS(3109),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3109),
    [anon_sym_GT] = ACTIONS(3109),
    [anon_sym_SLASH] = ACTIONS(3109),
    [anon_sym_DOT] = ACTIONS(3109),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_QMARK] = ACTIONS(3107),
    [anon_sym_AMP_AMP] = ACTIONS(3107),
    [anon_sym_PIPE_PIPE] = ACTIONS(3107),
    [anon_sym_GT_GT] = ACTIONS(3109),
    [anon_sym_GT_GT_GT] = ACTIONS(3107),
    [anon_sym_LT_LT] = ACTIONS(3107),
    [anon_sym_AMP] = ACTIONS(3109),
    [anon_sym_CARET] = ACTIONS(3107),
    [anon_sym_PIPE] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_PERCENT] = ACTIONS(3107),
    [anon_sym_LT_EQ] = ACTIONS(3107),
    [anon_sym_EQ_EQ] = ACTIONS(3109),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3107),
    [anon_sym_BANG_EQ] = ACTIONS(3109),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3107),
    [anon_sym_GT_EQ] = ACTIONS(3107),
    [anon_sym_instanceof] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3109),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [2474] = {
    [sym_statement_block] = STATE(2535),
    [anon_sym_LBRACE] = ACTIONS(5032),
    [sym_comment] = ACTIONS(71),
  },
  [2475] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_STAR] = ACTIONS(3115),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_COMMA] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_else] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_in] = ACTIONS(3117),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_case] = ACTIONS(3117),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3117),
    [anon_sym_GT] = ACTIONS(3117),
    [anon_sym_SLASH] = ACTIONS(3117),
    [anon_sym_DOT] = ACTIONS(3117),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_QMARK] = ACTIONS(3115),
    [anon_sym_AMP_AMP] = ACTIONS(3115),
    [anon_sym_PIPE_PIPE] = ACTIONS(3115),
    [anon_sym_GT_GT] = ACTIONS(3117),
    [anon_sym_GT_GT_GT] = ACTIONS(3115),
    [anon_sym_LT_LT] = ACTIONS(3115),
    [anon_sym_AMP] = ACTIONS(3117),
    [anon_sym_CARET] = ACTIONS(3115),
    [anon_sym_PIPE] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_PERCENT] = ACTIONS(3115),
    [anon_sym_LT_EQ] = ACTIONS(3115),
    [anon_sym_EQ_EQ] = ACTIONS(3117),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3115),
    [anon_sym_BANG_EQ] = ACTIONS(3117),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3115),
    [anon_sym_GT_EQ] = ACTIONS(3115),
    [anon_sym_instanceof] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3117),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [2476] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_STAR] = ACTIONS(482),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_COMMA] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_in] = ACTIONS(484),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(484),
    [anon_sym_GT] = ACTIONS(484),
    [anon_sym_SLASH] = ACTIONS(484),
    [anon_sym_DOT] = ACTIONS(484),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_QMARK] = ACTIONS(482),
    [anon_sym_AMP_AMP] = ACTIONS(482),
    [anon_sym_PIPE_PIPE] = ACTIONS(482),
    [anon_sym_GT_GT] = ACTIONS(484),
    [anon_sym_GT_GT_GT] = ACTIONS(482),
    [anon_sym_LT_LT] = ACTIONS(482),
    [anon_sym_AMP] = ACTIONS(484),
    [anon_sym_CARET] = ACTIONS(482),
    [anon_sym_PIPE] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_PERCENT] = ACTIONS(482),
    [anon_sym_LT_EQ] = ACTIONS(482),
    [anon_sym_EQ_EQ] = ACTIONS(484),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(482),
    [anon_sym_BANG_EQ] = ACTIONS(484),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(482),
    [anon_sym_GT_EQ] = ACTIONS(482),
    [anon_sym_instanceof] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(484),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2477] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5222),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2478] = {
    [sym_template_string] = STATE(217),
    [sym_arguments] = STATE(217),
    [sym__automatic_semicolon] = ACTIONS(5224),
    [anon_sym_STAR] = ACTIONS(1359),
    [anon_sym_LPAREN] = ACTIONS(356),
    [anon_sym_in] = ACTIONS(1355),
    [anon_sym_SEMI] = ACTIONS(5224),
    [anon_sym_LBRACK] = ACTIONS(358),
    [anon_sym_LT] = ACTIONS(1355),
    [anon_sym_GT] = ACTIONS(1355),
    [anon_sym_SLASH] = ACTIONS(1361),
    [anon_sym_DOT] = ACTIONS(352),
    [anon_sym_QMARK] = ACTIONS(1363),
    [anon_sym_AMP_AMP] = ACTIONS(1365),
    [anon_sym_PIPE_PIPE] = ACTIONS(1367),
    [anon_sym_GT_GT] = ACTIONS(1361),
    [anon_sym_GT_GT_GT] = ACTIONS(1359),
    [anon_sym_LT_LT] = ACTIONS(1359),
    [anon_sym_AMP] = ACTIONS(1369),
    [anon_sym_CARET] = ACTIONS(1367),
    [anon_sym_PIPE] = ACTIONS(1371),
    [anon_sym_PLUS] = ACTIONS(1373),
    [anon_sym_DASH] = ACTIONS(1373),
    [anon_sym_PERCENT] = ACTIONS(1359),
    [anon_sym_LT_EQ] = ACTIONS(1375),
    [anon_sym_EQ_EQ] = ACTIONS(1355),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1375),
    [anon_sym_BANG_EQ] = ACTIONS(1355),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1375),
    [anon_sym_GT_EQ] = ACTIONS(1375),
    [anon_sym_instanceof] = ACTIONS(1375),
    [anon_sym_PLUS_PLUS] = ACTIONS(390),
    [anon_sym_DASH_DASH] = ACTIONS(390),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(75),
  },
  [2479] = {
    [sym_class_body] = STATE(2538),
    [anon_sym_LBRACE] = ACTIONS(5026),
    [sym_comment] = ACTIONS(71),
  },
  [2480] = {
    [anon_sym_RPAREN] = ACTIONS(5226),
    [sym_comment] = ACTIONS(71),
  },
  [2481] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5228),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2482] = {
    [anon_sym_RPAREN] = ACTIONS(5230),
    [sym_comment] = ACTIONS(71),
  },
  [2483] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5232),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2484] = {
    [anon_sym_RPAREN] = ACTIONS(5234),
    [sym_comment] = ACTIONS(71),
  },
  [2485] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5236),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2486] = {
    [sym_export_statement] = STATE(2545),
    [sym__declaration] = STATE(2545),
    [sym_import_statement] = STATE(2545),
    [sym_expression_statement] = STATE(2545),
    [sym_variable_declaration] = STATE(2545),
    [sym_lexical_declaration] = STATE(2545),
    [sym_statement_block] = STATE(2545),
    [sym_if_statement] = STATE(2545),
    [sym_switch_statement] = STATE(2545),
    [sym_for_statement] = STATE(2545),
    [sym_for_in_statement] = STATE(2545),
    [sym_for_of_statement] = STATE(2545),
    [sym_while_statement] = STATE(2545),
    [sym_do_statement] = STATE(2545),
    [sym_try_statement] = STATE(2545),
    [sym_with_statement] = STATE(2545),
    [sym_break_statement] = STATE(2545),
    [sym_continue_statement] = STATE(2545),
    [sym_debugger_statement] = STATE(2545),
    [sym_return_statement] = STATE(2545),
    [sym_throw_statement] = STATE(2545),
    [sym_empty_statement] = STATE(2545),
    [sym_labeled_statement] = STATE(2545),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2487] = {
    [sym_export_statement] = STATE(2546),
    [sym__declaration] = STATE(2546),
    [sym_import_statement] = STATE(2546),
    [sym_expression_statement] = STATE(2546),
    [sym_variable_declaration] = STATE(2546),
    [sym_lexical_declaration] = STATE(2546),
    [sym_statement_block] = STATE(2546),
    [sym_if_statement] = STATE(2546),
    [sym_switch_statement] = STATE(2546),
    [sym_for_statement] = STATE(2546),
    [sym_for_in_statement] = STATE(2546),
    [sym_for_of_statement] = STATE(2546),
    [sym_while_statement] = STATE(2546),
    [sym_do_statement] = STATE(2546),
    [sym_try_statement] = STATE(2546),
    [sym_with_statement] = STATE(2546),
    [sym_break_statement] = STATE(2546),
    [sym_continue_statement] = STATE(2546),
    [sym_debugger_statement] = STATE(2546),
    [sym_return_statement] = STATE(2546),
    [sym_throw_statement] = STATE(2546),
    [sym_empty_statement] = STATE(2546),
    [sym_labeled_statement] = STATE(2546),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2488] = {
    [anon_sym_export] = ACTIONS(4266),
    [anon_sym_default] = ACTIONS(4266),
    [anon_sym_LBRACE] = ACTIONS(4264),
    [anon_sym_RBRACE] = ACTIONS(4264),
    [anon_sym_import] = ACTIONS(4266),
    [anon_sym_var] = ACTIONS(4266),
    [anon_sym_let] = ACTIONS(4266),
    [anon_sym_const] = ACTIONS(4266),
    [anon_sym_if] = ACTIONS(4266),
    [anon_sym_switch] = ACTIONS(4266),
    [anon_sym_for] = ACTIONS(4266),
    [anon_sym_LPAREN] = ACTIONS(4264),
    [anon_sym_while] = ACTIONS(4266),
    [anon_sym_do] = ACTIONS(4266),
    [anon_sym_try] = ACTIONS(4266),
    [anon_sym_with] = ACTIONS(4266),
    [anon_sym_break] = ACTIONS(4266),
    [anon_sym_continue] = ACTIONS(4266),
    [anon_sym_debugger] = ACTIONS(4266),
    [anon_sym_return] = ACTIONS(4266),
    [anon_sym_throw] = ACTIONS(4266),
    [anon_sym_SEMI] = ACTIONS(4264),
    [anon_sym_case] = ACTIONS(4266),
    [anon_sym_yield] = ACTIONS(4266),
    [anon_sym_LBRACK] = ACTIONS(4264),
    [anon_sym_LT] = ACTIONS(4264),
    [anon_sym_class] = ACTIONS(4266),
    [anon_sym_async] = ACTIONS(4266),
    [anon_sym_function] = ACTIONS(4266),
    [anon_sym_EQ_GT] = ACTIONS(4264),
    [anon_sym_new] = ACTIONS(4266),
    [anon_sym_await] = ACTIONS(4266),
    [anon_sym_PLUS] = ACTIONS(4266),
    [anon_sym_DASH] = ACTIONS(4266),
    [anon_sym_BANG] = ACTIONS(4264),
    [anon_sym_TILDE] = ACTIONS(4264),
    [anon_sym_typeof] = ACTIONS(4266),
    [anon_sym_void] = ACTIONS(4266),
    [anon_sym_delete] = ACTIONS(4266),
    [anon_sym_PLUS_PLUS] = ACTIONS(4264),
    [anon_sym_DASH_DASH] = ACTIONS(4264),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4264),
    [anon_sym_BQUOTE] = ACTIONS(4264),
    [sym_regex] = ACTIONS(4266),
    [sym_number] = ACTIONS(4264),
    [sym_identifier] = ACTIONS(4266),
    [sym_this] = ACTIONS(4266),
    [sym_super] = ACTIONS(4266),
    [sym_true] = ACTIONS(4266),
    [sym_false] = ACTIONS(4266),
    [sym_null] = ACTIONS(4266),
    [sym_undefined] = ACTIONS(4266),
    [anon_sym_AT] = ACTIONS(4264),
    [anon_sym_get] = ACTIONS(4266),
    [anon_sym_set] = ACTIONS(4266),
  },
  [2489] = {
    [sym_export_statement] = STATE(2547),
    [sym__declaration] = STATE(2547),
    [sym_import_statement] = STATE(2547),
    [sym_expression_statement] = STATE(2547),
    [sym_variable_declaration] = STATE(2547),
    [sym_lexical_declaration] = STATE(2547),
    [sym_statement_block] = STATE(2547),
    [sym_if_statement] = STATE(2547),
    [sym_switch_statement] = STATE(2547),
    [sym_for_statement] = STATE(2547),
    [sym_for_in_statement] = STATE(2547),
    [sym_for_of_statement] = STATE(2547),
    [sym_while_statement] = STATE(2547),
    [sym_do_statement] = STATE(2547),
    [sym_try_statement] = STATE(2547),
    [sym_with_statement] = STATE(2547),
    [sym_break_statement] = STATE(2547),
    [sym_continue_statement] = STATE(2547),
    [sym_debugger_statement] = STATE(2547),
    [sym_return_statement] = STATE(2547),
    [sym_throw_statement] = STATE(2547),
    [sym_empty_statement] = STATE(2547),
    [sym_labeled_statement] = STATE(2547),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2490] = {
    [sym_export_statement] = STATE(2548),
    [sym__declaration] = STATE(2548),
    [sym_import_statement] = STATE(2548),
    [sym_expression_statement] = STATE(2548),
    [sym_variable_declaration] = STATE(2548),
    [sym_lexical_declaration] = STATE(2548),
    [sym_statement_block] = STATE(2548),
    [sym_if_statement] = STATE(2548),
    [sym_switch_statement] = STATE(2548),
    [sym_for_statement] = STATE(2548),
    [sym_for_in_statement] = STATE(2548),
    [sym_for_of_statement] = STATE(2548),
    [sym_while_statement] = STATE(2548),
    [sym_do_statement] = STATE(2548),
    [sym_try_statement] = STATE(2548),
    [sym_with_statement] = STATE(2548),
    [sym_break_statement] = STATE(2548),
    [sym_continue_statement] = STATE(2548),
    [sym_debugger_statement] = STATE(2548),
    [sym_return_statement] = STATE(2548),
    [sym_throw_statement] = STATE(2548),
    [sym_empty_statement] = STATE(2548),
    [sym_labeled_statement] = STATE(2548),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2491] = {
    [sym_export_statement] = STATE(2549),
    [sym__declaration] = STATE(2549),
    [sym_import_statement] = STATE(2549),
    [sym_expression_statement] = STATE(2549),
    [sym_variable_declaration] = STATE(2549),
    [sym_lexical_declaration] = STATE(2549),
    [sym_statement_block] = STATE(2549),
    [sym_if_statement] = STATE(2549),
    [sym_switch_statement] = STATE(2549),
    [sym_for_statement] = STATE(2549),
    [sym_for_in_statement] = STATE(2549),
    [sym_for_of_statement] = STATE(2549),
    [sym_while_statement] = STATE(2549),
    [sym_do_statement] = STATE(2549),
    [sym_try_statement] = STATE(2549),
    [sym_with_statement] = STATE(2549),
    [sym_break_statement] = STATE(2549),
    [sym_continue_statement] = STATE(2549),
    [sym_debugger_statement] = STATE(2549),
    [sym_return_statement] = STATE(2549),
    [sym_throw_statement] = STATE(2549),
    [sym_empty_statement] = STATE(2549),
    [sym_labeled_statement] = STATE(2549),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2492] = {
    [sym_export_statement] = STATE(2550),
    [sym__declaration] = STATE(2550),
    [sym_import_statement] = STATE(2550),
    [sym_expression_statement] = STATE(2550),
    [sym_variable_declaration] = STATE(2550),
    [sym_lexical_declaration] = STATE(2550),
    [sym_statement_block] = STATE(2550),
    [sym_if_statement] = STATE(2550),
    [sym_switch_statement] = STATE(2550),
    [sym_for_statement] = STATE(2550),
    [sym_for_in_statement] = STATE(2550),
    [sym_for_of_statement] = STATE(2550),
    [sym_while_statement] = STATE(2550),
    [sym_do_statement] = STATE(2550),
    [sym_try_statement] = STATE(2550),
    [sym_with_statement] = STATE(2550),
    [sym_break_statement] = STATE(2550),
    [sym_continue_statement] = STATE(2550),
    [sym_debugger_statement] = STATE(2550),
    [sym_return_statement] = STATE(2550),
    [sym_throw_statement] = STATE(2550),
    [sym_empty_statement] = STATE(2550),
    [sym_labeled_statement] = STATE(2550),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2493] = {
    [sym_export_statement] = STATE(2551),
    [sym__declaration] = STATE(2551),
    [sym_import_statement] = STATE(2551),
    [sym_expression_statement] = STATE(2551),
    [sym_variable_declaration] = STATE(2551),
    [sym_lexical_declaration] = STATE(2551),
    [sym_statement_block] = STATE(2551),
    [sym_if_statement] = STATE(2551),
    [sym_switch_statement] = STATE(2551),
    [sym_for_statement] = STATE(2551),
    [sym_for_in_statement] = STATE(2551),
    [sym_for_of_statement] = STATE(2551),
    [sym_while_statement] = STATE(2551),
    [sym_do_statement] = STATE(2551),
    [sym_try_statement] = STATE(2551),
    [sym_with_statement] = STATE(2551),
    [sym_break_statement] = STATE(2551),
    [sym_continue_statement] = STATE(2551),
    [sym_debugger_statement] = STATE(2551),
    [sym_return_statement] = STATE(2551),
    [sym_throw_statement] = STATE(2551),
    [sym_empty_statement] = STATE(2551),
    [sym_labeled_statement] = STATE(2551),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2494] = {
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2495] = {
    [sym_statement_block] = STATE(2552),
    [anon_sym_LBRACE] = ACTIONS(4872),
    [sym_comment] = ACTIONS(71),
  },
  [2496] = {
    [anon_sym_else] = ACTIONS(4890),
    [anon_sym_while] = ACTIONS(4890),
    [sym_comment] = ACTIONS(71),
  },
  [2497] = {
    [anon_sym_else] = ACTIONS(4894),
    [anon_sym_while] = ACTIONS(4894),
    [sym_comment] = ACTIONS(71),
  },
  [2498] = {
    [anon_sym_else] = ACTIONS(4898),
    [anon_sym_while] = ACTIONS(4898),
    [sym_comment] = ACTIONS(71),
  },
  [2499] = {
    [anon_sym_else] = ACTIONS(4902),
    [anon_sym_while] = ACTIONS(4902),
    [sym_comment] = ACTIONS(71),
  },
  [2500] = {
    [anon_sym_else] = ACTIONS(4906),
    [anon_sym_while] = ACTIONS(4906),
    [sym_comment] = ACTIONS(71),
  },
  [2501] = {
    [anon_sym_else] = ACTIONS(4910),
    [anon_sym_while] = ACTIONS(4910),
    [sym_comment] = ACTIONS(71),
  },
  [2502] = {
    [sym__automatic_semicolon] = ACTIONS(2064),
    [anon_sym_export] = ACTIONS(2066),
    [anon_sym_default] = ACTIONS(2066),
    [anon_sym_LBRACE] = ACTIONS(2064),
    [anon_sym_RBRACE] = ACTIONS(2064),
    [anon_sym_import] = ACTIONS(2066),
    [anon_sym_var] = ACTIONS(2066),
    [anon_sym_let] = ACTIONS(2066),
    [anon_sym_const] = ACTIONS(2066),
    [anon_sym_if] = ACTIONS(2066),
    [anon_sym_else] = ACTIONS(2066),
    [anon_sym_switch] = ACTIONS(2066),
    [anon_sym_for] = ACTIONS(2066),
    [anon_sym_LPAREN] = ACTIONS(2064),
    [anon_sym_while] = ACTIONS(2066),
    [anon_sym_do] = ACTIONS(2066),
    [anon_sym_try] = ACTIONS(2066),
    [anon_sym_with] = ACTIONS(2066),
    [anon_sym_break] = ACTIONS(2066),
    [anon_sym_continue] = ACTIONS(2066),
    [anon_sym_debugger] = ACTIONS(2066),
    [anon_sym_return] = ACTIONS(2066),
    [anon_sym_throw] = ACTIONS(2066),
    [anon_sym_SEMI] = ACTIONS(2064),
    [anon_sym_case] = ACTIONS(2066),
    [anon_sym_yield] = ACTIONS(2066),
    [anon_sym_LBRACK] = ACTIONS(2064),
    [anon_sym_LT] = ACTIONS(2064),
    [anon_sym_class] = ACTIONS(2066),
    [anon_sym_async] = ACTIONS(2066),
    [anon_sym_function] = ACTIONS(2066),
    [anon_sym_EQ_GT] = ACTIONS(2064),
    [anon_sym_new] = ACTIONS(2066),
    [anon_sym_await] = ACTIONS(2066),
    [anon_sym_PLUS] = ACTIONS(2066),
    [anon_sym_DASH] = ACTIONS(2066),
    [anon_sym_BANG] = ACTIONS(2064),
    [anon_sym_TILDE] = ACTIONS(2064),
    [anon_sym_typeof] = ACTIONS(2066),
    [anon_sym_void] = ACTIONS(2066),
    [anon_sym_delete] = ACTIONS(2066),
    [anon_sym_PLUS_PLUS] = ACTIONS(2064),
    [anon_sym_DASH_DASH] = ACTIONS(2064),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(2064),
    [anon_sym_BQUOTE] = ACTIONS(2064),
    [sym_regex] = ACTIONS(2066),
    [sym_number] = ACTIONS(2064),
    [sym_identifier] = ACTIONS(2066),
    [sym_this] = ACTIONS(2066),
    [sym_super] = ACTIONS(2066),
    [sym_true] = ACTIONS(2066),
    [sym_false] = ACTIONS(2066),
    [sym_null] = ACTIONS(2066),
    [sym_undefined] = ACTIONS(2066),
    [anon_sym_AT] = ACTIONS(2064),
    [anon_sym_get] = ACTIONS(2066),
    [anon_sym_set] = ACTIONS(2066),
  },
  [2503] = {
    [sym_decorator] = STATE(555),
    [sym_public_field_definition] = STATE(552),
    [sym_method_definition] = STATE(553),
    [sym__property_name] = STATE(554),
    [sym_computed_property_name] = STATE(554),
    [aux_sym_export_statement_repeat1] = STATE(555),
    [aux_sym_class_body_repeat1] = STATE(1016),
    [anon_sym_STAR] = ACTIONS(966),
    [anon_sym_RBRACE] = ACTIONS(5238),
    [anon_sym_LBRACK] = ACTIONS(970),
    [anon_sym_async] = ACTIONS(972),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(974),
    [sym_number] = ACTIONS(974),
    [sym_identifier] = ACTIONS(976),
    [anon_sym_AT] = ACTIONS(978),
    [anon_sym_static] = ACTIONS(980),
    [anon_sym_get] = ACTIONS(982),
    [anon_sym_set] = ACTIONS(982),
  },
  [2504] = {
    [sym__automatic_semicolon] = ACTIONS(3103),
    [anon_sym_export] = ACTIONS(3105),
    [anon_sym_default] = ACTIONS(3105),
    [anon_sym_LBRACE] = ACTIONS(3103),
    [anon_sym_RBRACE] = ACTIONS(3103),
    [anon_sym_import] = ACTIONS(3105),
    [anon_sym_var] = ACTIONS(3105),
    [anon_sym_let] = ACTIONS(3105),
    [anon_sym_const] = ACTIONS(3105),
    [anon_sym_if] = ACTIONS(3105),
    [anon_sym_else] = ACTIONS(3105),
    [anon_sym_switch] = ACTIONS(3105),
    [anon_sym_for] = ACTIONS(3105),
    [anon_sym_LPAREN] = ACTIONS(3103),
    [anon_sym_while] = ACTIONS(3105),
    [anon_sym_do] = ACTIONS(3105),
    [anon_sym_try] = ACTIONS(3105),
    [anon_sym_with] = ACTIONS(3105),
    [anon_sym_break] = ACTIONS(3105),
    [anon_sym_continue] = ACTIONS(3105),
    [anon_sym_debugger] = ACTIONS(3105),
    [anon_sym_return] = ACTIONS(3105),
    [anon_sym_throw] = ACTIONS(3105),
    [anon_sym_SEMI] = ACTIONS(3103),
    [anon_sym_case] = ACTIONS(3105),
    [anon_sym_yield] = ACTIONS(3105),
    [anon_sym_LBRACK] = ACTIONS(3103),
    [anon_sym_LT] = ACTIONS(3103),
    [anon_sym_class] = ACTIONS(3105),
    [anon_sym_async] = ACTIONS(3105),
    [anon_sym_function] = ACTIONS(3105),
    [anon_sym_EQ_GT] = ACTIONS(3103),
    [anon_sym_new] = ACTIONS(3105),
    [anon_sym_await] = ACTIONS(3105),
    [anon_sym_PLUS] = ACTIONS(3105),
    [anon_sym_DASH] = ACTIONS(3105),
    [anon_sym_BANG] = ACTIONS(3103),
    [anon_sym_TILDE] = ACTIONS(3103),
    [anon_sym_typeof] = ACTIONS(3105),
    [anon_sym_void] = ACTIONS(3105),
    [anon_sym_delete] = ACTIONS(3105),
    [anon_sym_PLUS_PLUS] = ACTIONS(3103),
    [anon_sym_DASH_DASH] = ACTIONS(3103),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3103),
    [anon_sym_BQUOTE] = ACTIONS(3103),
    [sym_regex] = ACTIONS(3105),
    [sym_number] = ACTIONS(3103),
    [sym_identifier] = ACTIONS(3105),
    [sym_this] = ACTIONS(3105),
    [sym_super] = ACTIONS(3105),
    [sym_true] = ACTIONS(3105),
    [sym_false] = ACTIONS(3105),
    [sym_null] = ACTIONS(3105),
    [sym_undefined] = ACTIONS(3105),
    [anon_sym_AT] = ACTIONS(3103),
    [anon_sym_get] = ACTIONS(3105),
    [anon_sym_set] = ACTIONS(3105),
  },
  [2505] = {
    [sym_statement_block] = STATE(2554),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [sym_comment] = ACTIONS(71),
  },
  [2506] = {
    [sym__automatic_semicolon] = ACTIONS(3107),
    [anon_sym_export] = ACTIONS(3109),
    [anon_sym_default] = ACTIONS(3109),
    [anon_sym_LBRACE] = ACTIONS(3107),
    [anon_sym_RBRACE] = ACTIONS(3107),
    [anon_sym_import] = ACTIONS(3109),
    [anon_sym_var] = ACTIONS(3109),
    [anon_sym_let] = ACTIONS(3109),
    [anon_sym_const] = ACTIONS(3109),
    [anon_sym_if] = ACTIONS(3109),
    [anon_sym_else] = ACTIONS(3109),
    [anon_sym_switch] = ACTIONS(3109),
    [anon_sym_for] = ACTIONS(3109),
    [anon_sym_LPAREN] = ACTIONS(3107),
    [anon_sym_while] = ACTIONS(3109),
    [anon_sym_do] = ACTIONS(3109),
    [anon_sym_try] = ACTIONS(3109),
    [anon_sym_with] = ACTIONS(3109),
    [anon_sym_break] = ACTIONS(3109),
    [anon_sym_continue] = ACTIONS(3109),
    [anon_sym_debugger] = ACTIONS(3109),
    [anon_sym_return] = ACTIONS(3109),
    [anon_sym_throw] = ACTIONS(3109),
    [anon_sym_SEMI] = ACTIONS(3107),
    [anon_sym_case] = ACTIONS(3109),
    [anon_sym_yield] = ACTIONS(3109),
    [anon_sym_LBRACK] = ACTIONS(3107),
    [anon_sym_LT] = ACTIONS(3107),
    [anon_sym_class] = ACTIONS(3109),
    [anon_sym_async] = ACTIONS(3109),
    [anon_sym_function] = ACTIONS(3109),
    [anon_sym_EQ_GT] = ACTIONS(3107),
    [anon_sym_new] = ACTIONS(3109),
    [anon_sym_await] = ACTIONS(3109),
    [anon_sym_PLUS] = ACTIONS(3109),
    [anon_sym_DASH] = ACTIONS(3109),
    [anon_sym_BANG] = ACTIONS(3107),
    [anon_sym_TILDE] = ACTIONS(3107),
    [anon_sym_typeof] = ACTIONS(3109),
    [anon_sym_void] = ACTIONS(3109),
    [anon_sym_delete] = ACTIONS(3109),
    [anon_sym_PLUS_PLUS] = ACTIONS(3107),
    [anon_sym_DASH_DASH] = ACTIONS(3107),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3107),
    [anon_sym_BQUOTE] = ACTIONS(3107),
    [sym_regex] = ACTIONS(3109),
    [sym_number] = ACTIONS(3107),
    [sym_identifier] = ACTIONS(3109),
    [sym_this] = ACTIONS(3109),
    [sym_super] = ACTIONS(3109),
    [sym_true] = ACTIONS(3109),
    [sym_false] = ACTIONS(3109),
    [sym_null] = ACTIONS(3109),
    [sym_undefined] = ACTIONS(3109),
    [anon_sym_AT] = ACTIONS(3107),
    [anon_sym_get] = ACTIONS(3109),
    [anon_sym_set] = ACTIONS(3109),
  },
  [2507] = {
    [sym_statement_block] = STATE(2555),
    [anon_sym_LBRACE] = ACTIONS(5106),
    [sym_comment] = ACTIONS(71),
  },
  [2508] = {
    [sym__automatic_semicolon] = ACTIONS(3115),
    [anon_sym_export] = ACTIONS(3117),
    [anon_sym_default] = ACTIONS(3117),
    [anon_sym_LBRACE] = ACTIONS(3115),
    [anon_sym_RBRACE] = ACTIONS(3115),
    [anon_sym_import] = ACTIONS(3117),
    [anon_sym_var] = ACTIONS(3117),
    [anon_sym_let] = ACTIONS(3117),
    [anon_sym_const] = ACTIONS(3117),
    [anon_sym_if] = ACTIONS(3117),
    [anon_sym_else] = ACTIONS(3117),
    [anon_sym_switch] = ACTIONS(3117),
    [anon_sym_for] = ACTIONS(3117),
    [anon_sym_LPAREN] = ACTIONS(3115),
    [anon_sym_while] = ACTIONS(3117),
    [anon_sym_do] = ACTIONS(3117),
    [anon_sym_try] = ACTIONS(3117),
    [anon_sym_with] = ACTIONS(3117),
    [anon_sym_break] = ACTIONS(3117),
    [anon_sym_continue] = ACTIONS(3117),
    [anon_sym_debugger] = ACTIONS(3117),
    [anon_sym_return] = ACTIONS(3117),
    [anon_sym_throw] = ACTIONS(3117),
    [anon_sym_SEMI] = ACTIONS(3115),
    [anon_sym_case] = ACTIONS(3117),
    [anon_sym_yield] = ACTIONS(3117),
    [anon_sym_LBRACK] = ACTIONS(3115),
    [anon_sym_LT] = ACTIONS(3115),
    [anon_sym_class] = ACTIONS(3117),
    [anon_sym_async] = ACTIONS(3117),
    [anon_sym_function] = ACTIONS(3117),
    [anon_sym_EQ_GT] = ACTIONS(3115),
    [anon_sym_new] = ACTIONS(3117),
    [anon_sym_await] = ACTIONS(3117),
    [anon_sym_PLUS] = ACTIONS(3117),
    [anon_sym_DASH] = ACTIONS(3117),
    [anon_sym_BANG] = ACTIONS(3115),
    [anon_sym_TILDE] = ACTIONS(3115),
    [anon_sym_typeof] = ACTIONS(3117),
    [anon_sym_void] = ACTIONS(3117),
    [anon_sym_delete] = ACTIONS(3117),
    [anon_sym_PLUS_PLUS] = ACTIONS(3115),
    [anon_sym_DASH_DASH] = ACTIONS(3115),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3115),
    [anon_sym_BQUOTE] = ACTIONS(3115),
    [sym_regex] = ACTIONS(3117),
    [sym_number] = ACTIONS(3115),
    [sym_identifier] = ACTIONS(3117),
    [sym_this] = ACTIONS(3117),
    [sym_super] = ACTIONS(3117),
    [sym_true] = ACTIONS(3117),
    [sym_false] = ACTIONS(3117),
    [sym_null] = ACTIONS(3117),
    [sym_undefined] = ACTIONS(3117),
    [anon_sym_AT] = ACTIONS(3115),
    [anon_sym_get] = ACTIONS(3117),
    [anon_sym_set] = ACTIONS(3117),
  },
  [2509] = {
    [sym__automatic_semicolon] = ACTIONS(482),
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2510] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5240),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2511] = {
    [sym_class_body] = STATE(2557),
    [anon_sym_LBRACE] = ACTIONS(5100),
    [sym_comment] = ACTIONS(71),
  },
  [2512] = {
    [anon_sym_export] = ACTIONS(3484),
    [anon_sym_default] = ACTIONS(3484),
    [anon_sym_LBRACE] = ACTIONS(3482),
    [anon_sym_RBRACE] = ACTIONS(3482),
    [anon_sym_import] = ACTIONS(3484),
    [anon_sym_var] = ACTIONS(3484),
    [anon_sym_let] = ACTIONS(3484),
    [anon_sym_const] = ACTIONS(3484),
    [anon_sym_if] = ACTIONS(3484),
    [anon_sym_else] = ACTIONS(3484),
    [anon_sym_switch] = ACTIONS(3484),
    [anon_sym_for] = ACTIONS(3484),
    [anon_sym_LPAREN] = ACTIONS(3482),
    [anon_sym_while] = ACTIONS(3484),
    [anon_sym_do] = ACTIONS(3484),
    [anon_sym_try] = ACTIONS(3484),
    [anon_sym_with] = ACTIONS(3484),
    [anon_sym_break] = ACTIONS(3484),
    [anon_sym_continue] = ACTIONS(3484),
    [anon_sym_debugger] = ACTIONS(3484),
    [anon_sym_return] = ACTIONS(3484),
    [anon_sym_throw] = ACTIONS(3484),
    [anon_sym_SEMI] = ACTIONS(3482),
    [anon_sym_case] = ACTIONS(3484),
    [anon_sym_yield] = ACTIONS(3484),
    [anon_sym_LBRACK] = ACTIONS(3482),
    [anon_sym_LT] = ACTIONS(3482),
    [anon_sym_class] = ACTIONS(3484),
    [anon_sym_async] = ACTIONS(3484),
    [anon_sym_function] = ACTIONS(3484),
    [anon_sym_EQ_GT] = ACTIONS(3482),
    [anon_sym_new] = ACTIONS(3484),
    [anon_sym_await] = ACTIONS(3484),
    [anon_sym_PLUS] = ACTIONS(3484),
    [anon_sym_DASH] = ACTIONS(3484),
    [anon_sym_BANG] = ACTIONS(3482),
    [anon_sym_TILDE] = ACTIONS(3482),
    [anon_sym_typeof] = ACTIONS(3484),
    [anon_sym_void] = ACTIONS(3484),
    [anon_sym_delete] = ACTIONS(3484),
    [anon_sym_PLUS_PLUS] = ACTIONS(3482),
    [anon_sym_DASH_DASH] = ACTIONS(3482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3482),
    [anon_sym_BQUOTE] = ACTIONS(3482),
    [sym_regex] = ACTIONS(3484),
    [sym_number] = ACTIONS(3482),
    [sym_identifier] = ACTIONS(3484),
    [sym_this] = ACTIONS(3484),
    [sym_super] = ACTIONS(3484),
    [sym_true] = ACTIONS(3484),
    [sym_false] = ACTIONS(3484),
    [sym_null] = ACTIONS(3484),
    [sym_undefined] = ACTIONS(3484),
    [anon_sym_AT] = ACTIONS(3482),
    [anon_sym_get] = ACTIONS(3484),
    [anon_sym_set] = ACTIONS(3484),
  },
  [2513] = {
    [anon_sym_export] = ACTIONS(3586),
    [anon_sym_default] = ACTIONS(3586),
    [anon_sym_LBRACE] = ACTIONS(3584),
    [anon_sym_RBRACE] = ACTIONS(3584),
    [anon_sym_import] = ACTIONS(3586),
    [anon_sym_var] = ACTIONS(3586),
    [anon_sym_let] = ACTIONS(3586),
    [anon_sym_const] = ACTIONS(3586),
    [anon_sym_if] = ACTIONS(3586),
    [anon_sym_else] = ACTIONS(3586),
    [anon_sym_switch] = ACTIONS(3586),
    [anon_sym_for] = ACTIONS(3586),
    [anon_sym_LPAREN] = ACTIONS(3584),
    [anon_sym_while] = ACTIONS(3586),
    [anon_sym_do] = ACTIONS(3586),
    [anon_sym_try] = ACTIONS(3586),
    [anon_sym_with] = ACTIONS(3586),
    [anon_sym_break] = ACTIONS(3586),
    [anon_sym_continue] = ACTIONS(3586),
    [anon_sym_debugger] = ACTIONS(3586),
    [anon_sym_return] = ACTIONS(3586),
    [anon_sym_throw] = ACTIONS(3586),
    [anon_sym_SEMI] = ACTIONS(3584),
    [anon_sym_case] = ACTIONS(3586),
    [anon_sym_yield] = ACTIONS(3586),
    [anon_sym_LBRACK] = ACTIONS(3584),
    [anon_sym_LT] = ACTIONS(3584),
    [anon_sym_class] = ACTIONS(3586),
    [anon_sym_async] = ACTIONS(3586),
    [anon_sym_function] = ACTIONS(3586),
    [anon_sym_EQ_GT] = ACTIONS(3584),
    [anon_sym_new] = ACTIONS(3586),
    [anon_sym_await] = ACTIONS(3586),
    [anon_sym_PLUS] = ACTIONS(3586),
    [anon_sym_DASH] = ACTIONS(3586),
    [anon_sym_BANG] = ACTIONS(3584),
    [anon_sym_TILDE] = ACTIONS(3584),
    [anon_sym_typeof] = ACTIONS(3586),
    [anon_sym_void] = ACTIONS(3586),
    [anon_sym_delete] = ACTIONS(3586),
    [anon_sym_PLUS_PLUS] = ACTIONS(3584),
    [anon_sym_DASH_DASH] = ACTIONS(3584),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3584),
    [anon_sym_BQUOTE] = ACTIONS(3584),
    [sym_regex] = ACTIONS(3586),
    [sym_number] = ACTIONS(3584),
    [sym_identifier] = ACTIONS(3586),
    [sym_this] = ACTIONS(3586),
    [sym_super] = ACTIONS(3586),
    [sym_true] = ACTIONS(3586),
    [sym_false] = ACTIONS(3586),
    [sym_null] = ACTIONS(3586),
    [sym_undefined] = ACTIONS(3586),
    [anon_sym_AT] = ACTIONS(3584),
    [anon_sym_get] = ACTIONS(3586),
    [anon_sym_set] = ACTIONS(3586),
  },
  [2514] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2558),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2558),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2515] = {
    [sym_parenthesized_expression] = STATE(2559),
    [sym__expression] = STATE(2559),
    [sym_yield_expression] = STATE(2559),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2559),
    [sym_jsx_fragment] = STATE(2559),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2559),
    [sym_anonymous_class] = STATE(2559),
    [sym_class] = STATE(2559),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2559),
    [sym_generator_function] = STATE(2559),
    [sym_call_expression] = STATE(2559),
    [sym_new_expression] = STATE(2559),
    [sym_await_expression] = STATE(2559),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2559),
    [sym_augmented_assignment_expression] = STATE(2559),
    [sym_ternary_expression] = STATE(2559),
    [sym_binary_expression] = STATE(2559),
    [sym_unary_expression] = STATE(2559),
    [sym_update_expression] = STATE(2559),
    [sym_template_string] = STATE(2559),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5242),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5244),
    [sym_number] = ACTIONS(5242),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5244),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5244),
    [sym_false] = ACTIONS(5244),
    [sym_null] = ACTIONS(5244),
    [sym_undefined] = ACTIONS(5244),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2516] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2560),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2560),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2517] = {
    [sym_parenthesized_expression] = STATE(2561),
    [sym__expression] = STATE(2561),
    [sym_yield_expression] = STATE(2561),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2561),
    [sym_jsx_fragment] = STATE(2561),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2561),
    [sym_anonymous_class] = STATE(2561),
    [sym_class] = STATE(2561),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2561),
    [sym_generator_function] = STATE(2561),
    [sym_call_expression] = STATE(2561),
    [sym_new_expression] = STATE(2561),
    [sym_await_expression] = STATE(2561),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2561),
    [sym_augmented_assignment_expression] = STATE(2561),
    [sym_ternary_expression] = STATE(2561),
    [sym_binary_expression] = STATE(2561),
    [sym_unary_expression] = STATE(2561),
    [sym_update_expression] = STATE(2561),
    [sym_template_string] = STATE(2561),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5246),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5248),
    [sym_number] = ACTIONS(5246),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5248),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5248),
    [sym_false] = ACTIONS(5248),
    [sym_null] = ACTIONS(5248),
    [sym_undefined] = ACTIONS(5248),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2518] = {
    [sym_parenthesized_expression] = STATE(119),
    [sym__expressions] = STATE(2562),
    [sym__expression] = STATE(119),
    [sym_yield_expression] = STATE(119),
    [sym_object] = STATE(359),
    [sym_array] = STATE(360),
    [sym_jsx_element] = STATE(119),
    [sym_jsx_fragment] = STATE(119),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(119),
    [sym_anonymous_class] = STATE(119),
    [sym_class] = STATE(119),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(119),
    [sym_generator_function] = STATE(119),
    [sym_call_expression] = STATE(119),
    [sym_new_expression] = STATE(119),
    [sym_await_expression] = STATE(119),
    [sym_member_expression] = STATE(125),
    [sym_subscript_expression] = STATE(125),
    [sym_assignment_expression] = STATE(119),
    [sym_augmented_assignment_expression] = STATE(119),
    [sym_ternary_expression] = STATE(119),
    [sym_binary_expression] = STATE(119),
    [sym_unary_expression] = STATE(119),
    [sym_update_expression] = STATE(119),
    [sym_sequence_expression] = STATE(2562),
    [sym_template_string] = STATE(119),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(126),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(157),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(616),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(169),
    [anon_sym_new] = ACTIONS(171),
    [anon_sym_await] = ACTIONS(173),
    [anon_sym_PLUS] = ACTIONS(177),
    [anon_sym_DASH] = ACTIONS(177),
    [anon_sym_BANG] = ACTIONS(179),
    [anon_sym_TILDE] = ACTIONS(179),
    [anon_sym_typeof] = ACTIONS(177),
    [anon_sym_void] = ACTIONS(177),
    [anon_sym_delete] = ACTIONS(177),
    [anon_sym_PLUS_PLUS] = ACTIONS(181),
    [anon_sym_DASH_DASH] = ACTIONS(181),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(183),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(187),
    [sym_number] = ACTIONS(183),
    [sym_identifier] = ACTIONS(618),
    [sym_this] = ACTIONS(187),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(187),
    [sym_false] = ACTIONS(187),
    [sym_null] = ACTIONS(187),
    [sym_undefined] = ACTIONS(187),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(620),
    [anon_sym_set] = ACTIONS(620),
  },
  [2519] = {
    [sym_parenthesized_expression] = STATE(2563),
    [sym__expression] = STATE(2563),
    [sym_yield_expression] = STATE(2563),
    [sym_object] = STATE(1312),
    [sym_array] = STATE(1313),
    [sym_jsx_element] = STATE(2563),
    [sym_jsx_fragment] = STATE(2563),
    [sym_jsx_opening_element] = STATE(123),
    [sym_jsx_self_closing_element] = STATE(2563),
    [sym_anonymous_class] = STATE(2563),
    [sym_class] = STATE(2563),
    [sym_function] = STATE(124),
    [sym_arrow_function] = STATE(2563),
    [sym_generator_function] = STATE(2563),
    [sym_call_expression] = STATE(2563),
    [sym_new_expression] = STATE(2563),
    [sym_await_expression] = STATE(2563),
    [sym_member_expression] = STATE(1314),
    [sym_subscript_expression] = STATE(1314),
    [sym_assignment_expression] = STATE(2563),
    [sym_augmented_assignment_expression] = STATE(2563),
    [sym_ternary_expression] = STATE(2563),
    [sym_binary_expression] = STATE(2563),
    [sym_unary_expression] = STATE(2563),
    [sym_update_expression] = STATE(2563),
    [sym_template_string] = STATE(2563),
    [sym_decorator] = STATE(127),
    [sym_formal_parameters] = STATE(1315),
    [aux_sym_export_statement_repeat1] = STATE(127),
    [anon_sym_LBRACE] = ACTIONS(151),
    [anon_sym_LPAREN] = ACTIONS(153),
    [anon_sym_yield] = ACTIONS(2762),
    [anon_sym_LBRACK] = ACTIONS(159),
    [anon_sym_LT] = ACTIONS(161),
    [anon_sym_class] = ACTIONS(163),
    [anon_sym_async] = ACTIONS(2764),
    [anon_sym_function] = ACTIONS(167),
    [anon_sym_EQ_GT] = ACTIONS(2766),
    [anon_sym_new] = ACTIONS(2768),
    [anon_sym_await] = ACTIONS(2770),
    [anon_sym_PLUS] = ACTIONS(2772),
    [anon_sym_DASH] = ACTIONS(2772),
    [anon_sym_BANG] = ACTIONS(2774),
    [anon_sym_TILDE] = ACTIONS(2774),
    [anon_sym_typeof] = ACTIONS(2772),
    [anon_sym_void] = ACTIONS(2772),
    [anon_sym_delete] = ACTIONS(2772),
    [anon_sym_PLUS_PLUS] = ACTIONS(2776),
    [anon_sym_DASH_DASH] = ACTIONS(2776),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(5250),
    [anon_sym_BQUOTE] = ACTIONS(185),
    [sym_regex] = ACTIONS(5252),
    [sym_number] = ACTIONS(5250),
    [sym_identifier] = ACTIONS(2782),
    [sym_this] = ACTIONS(5252),
    [sym_super] = ACTIONS(191),
    [sym_true] = ACTIONS(5252),
    [sym_false] = ACTIONS(5252),
    [sym_null] = ACTIONS(5252),
    [sym_undefined] = ACTIONS(5252),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(2784),
    [anon_sym_set] = ACTIONS(2784),
  },
  [2520] = {
    [anon_sym_RPAREN] = ACTIONS(5254),
    [sym_comment] = ACTIONS(71),
  },
  [2521] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5256),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2522] = {
    [sym_export_statement] = STATE(2566),
    [sym__declaration] = STATE(2566),
    [sym_import_statement] = STATE(2566),
    [sym_expression_statement] = STATE(2566),
    [sym_variable_declaration] = STATE(2566),
    [sym_lexical_declaration] = STATE(2566),
    [sym_statement_block] = STATE(2566),
    [sym_if_statement] = STATE(2566),
    [sym_switch_statement] = STATE(2566),
    [sym_for_statement] = STATE(2566),
    [sym_for_in_statement] = STATE(2566),
    [sym_for_of_statement] = STATE(2566),
    [sym_while_statement] = STATE(2566),
    [sym_do_statement] = STATE(2566),
    [sym_try_statement] = STATE(2566),
    [sym_with_statement] = STATE(2566),
    [sym_break_statement] = STATE(2566),
    [sym_continue_statement] = STATE(2566),
    [sym_debugger_statement] = STATE(2566),
    [sym_return_statement] = STATE(2566),
    [sym_throw_statement] = STATE(2566),
    [sym_empty_statement] = STATE(2566),
    [sym_labeled_statement] = STATE(2566),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2523] = {
    [anon_sym_RPAREN] = ACTIONS(5258),
    [sym_comment] = ACTIONS(71),
  },
  [2524] = {
    [anon_sym_RPAREN] = ACTIONS(5260),
    [sym_comment] = ACTIONS(71),
  },
  [2525] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5262),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2526] = {
    [anon_sym_RPAREN] = ACTIONS(5264),
    [sym_comment] = ACTIONS(71),
  },
  [2527] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5266),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2528] = {
    [anon_sym_export] = ACTIONS(3788),
    [anon_sym_default] = ACTIONS(3788),
    [anon_sym_LBRACE] = ACTIONS(3786),
    [anon_sym_RBRACE] = ACTIONS(3786),
    [anon_sym_import] = ACTIONS(3788),
    [anon_sym_var] = ACTIONS(3788),
    [anon_sym_let] = ACTIONS(3788),
    [anon_sym_const] = ACTIONS(3788),
    [anon_sym_if] = ACTIONS(3788),
    [anon_sym_else] = ACTIONS(3788),
    [anon_sym_switch] = ACTIONS(3788),
    [anon_sym_for] = ACTIONS(3788),
    [anon_sym_LPAREN] = ACTIONS(3786),
    [anon_sym_while] = ACTIONS(3788),
    [anon_sym_do] = ACTIONS(3788),
    [anon_sym_try] = ACTIONS(3788),
    [anon_sym_with] = ACTIONS(3788),
    [anon_sym_break] = ACTIONS(3788),
    [anon_sym_continue] = ACTIONS(3788),
    [anon_sym_debugger] = ACTIONS(3788),
    [anon_sym_return] = ACTIONS(3788),
    [anon_sym_throw] = ACTIONS(3788),
    [anon_sym_SEMI] = ACTIONS(3786),
    [anon_sym_case] = ACTIONS(3788),
    [anon_sym_yield] = ACTIONS(3788),
    [anon_sym_LBRACK] = ACTIONS(3786),
    [anon_sym_LT] = ACTIONS(3786),
    [anon_sym_class] = ACTIONS(3788),
    [anon_sym_async] = ACTIONS(3788),
    [anon_sym_function] = ACTIONS(3788),
    [anon_sym_EQ_GT] = ACTIONS(3786),
    [anon_sym_new] = ACTIONS(3788),
    [anon_sym_await] = ACTIONS(3788),
    [anon_sym_PLUS] = ACTIONS(3788),
    [anon_sym_DASH] = ACTIONS(3788),
    [anon_sym_BANG] = ACTIONS(3786),
    [anon_sym_TILDE] = ACTIONS(3786),
    [anon_sym_typeof] = ACTIONS(3788),
    [anon_sym_void] = ACTIONS(3788),
    [anon_sym_delete] = ACTIONS(3788),
    [anon_sym_PLUS_PLUS] = ACTIONS(3786),
    [anon_sym_DASH_DASH] = ACTIONS(3786),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3786),
    [anon_sym_BQUOTE] = ACTIONS(3786),
    [sym_regex] = ACTIONS(3788),
    [sym_number] = ACTIONS(3786),
    [sym_identifier] = ACTIONS(3788),
    [sym_this] = ACTIONS(3788),
    [sym_super] = ACTIONS(3788),
    [sym_true] = ACTIONS(3788),
    [sym_false] = ACTIONS(3788),
    [sym_null] = ACTIONS(3788),
    [sym_undefined] = ACTIONS(3788),
    [anon_sym_AT] = ACTIONS(3786),
    [anon_sym_get] = ACTIONS(3788),
    [anon_sym_set] = ACTIONS(3788),
  },
  [2529] = {
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_finally] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2530] = {
    [sym_export_statement] = STATE(322),
    [sym__declaration] = STATE(322),
    [sym_import_statement] = STATE(322),
    [sym_expression_statement] = STATE(322),
    [sym_variable_declaration] = STATE(322),
    [sym_lexical_declaration] = STATE(322),
    [sym_statement_block] = STATE(322),
    [sym_if_statement] = STATE(322),
    [sym_switch_statement] = STATE(322),
    [sym_for_statement] = STATE(322),
    [sym_for_in_statement] = STATE(322),
    [sym_for_of_statement] = STATE(322),
    [sym_while_statement] = STATE(322),
    [sym_do_statement] = STATE(322),
    [sym_try_statement] = STATE(322),
    [sym_with_statement] = STATE(322),
    [sym_break_statement] = STATE(322),
    [sym_continue_statement] = STATE(322),
    [sym_debugger_statement] = STATE(322),
    [sym_return_statement] = STATE(322),
    [sym_throw_statement] = STATE(322),
    [sym_empty_statement] = STATE(322),
    [sym_labeled_statement] = STATE(322),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(39),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(44),
    [sym_function] = STATE(45),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(44),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(39),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(49),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_program_repeat1] = STATE(322),
    [aux_sym_export_statement_repeat1] = STATE(49),
    [anon_sym_export] = ACTIONS(9),
    [anon_sym_LBRACE] = ACTIONS(11),
    [anon_sym_RBRACE] = ACTIONS(5268),
    [anon_sym_import] = ACTIONS(13),
    [anon_sym_var] = ACTIONS(15),
    [anon_sym_let] = ACTIONS(17),
    [anon_sym_const] = ACTIONS(17),
    [anon_sym_if] = ACTIONS(111),
    [anon_sym_switch] = ACTIONS(21),
    [anon_sym_for] = ACTIONS(113),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(115),
    [anon_sym_do] = ACTIONS(29),
    [anon_sym_try] = ACTIONS(31),
    [anon_sym_with] = ACTIONS(117),
    [anon_sym_break] = ACTIONS(35),
    [anon_sym_continue] = ACTIONS(37),
    [anon_sym_debugger] = ACTIONS(39),
    [anon_sym_return] = ACTIONS(41),
    [anon_sym_throw] = ACTIONS(43),
    [anon_sym_SEMI] = ACTIONS(45),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(53),
    [anon_sym_async] = ACTIONS(55),
    [anon_sym_function] = ACTIONS(57),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(564),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2531] = {
    [anon_sym_RPAREN] = ACTIONS(5270),
    [sym_comment] = ACTIONS(71),
  },
  [2532] = {
    [anon_sym_export] = ACTIONS(484),
    [anon_sym_default] = ACTIONS(484),
    [anon_sym_LBRACE] = ACTIONS(482),
    [anon_sym_RBRACE] = ACTIONS(482),
    [anon_sym_import] = ACTIONS(484),
    [anon_sym_var] = ACTIONS(484),
    [anon_sym_let] = ACTIONS(484),
    [anon_sym_const] = ACTIONS(484),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_else] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(484),
    [anon_sym_for] = ACTIONS(484),
    [anon_sym_LPAREN] = ACTIONS(482),
    [anon_sym_while] = ACTIONS(484),
    [anon_sym_do] = ACTIONS(484),
    [anon_sym_try] = ACTIONS(484),
    [anon_sym_with] = ACTIONS(484),
    [anon_sym_break] = ACTIONS(484),
    [anon_sym_continue] = ACTIONS(484),
    [anon_sym_debugger] = ACTIONS(484),
    [anon_sym_return] = ACTIONS(484),
    [anon_sym_throw] = ACTIONS(484),
    [anon_sym_SEMI] = ACTIONS(482),
    [anon_sym_case] = ACTIONS(484),
    [anon_sym_yield] = ACTIONS(484),
    [anon_sym_LBRACK] = ACTIONS(482),
    [anon_sym_LT] = ACTIONS(482),
    [anon_sym_class] = ACTIONS(484),
    [anon_sym_async] = ACTIONS(484),
    [anon_sym_function] = ACTIONS(484),
    [anon_sym_EQ_GT] = ACTIONS(482),
    [anon_sym_new] = ACTIONS(484),
    [anon_sym_await] = ACTIONS(484),
    [anon_sym_PLUS] = ACTIONS(484),
    [anon_sym_DASH] = ACTIONS(484),
    [anon_sym_BANG] = ACTIONS(482),
    [anon_sym_TILDE] = ACTIONS(482),
    [anon_sym_typeof] = ACTIONS(484),
    [anon_sym_void] = ACTIONS(484),
    [anon_sym_delete] = ACTIONS(484),
    [anon_sym_PLUS_PLUS] = ACTIONS(482),
    [anon_sym_DASH_DASH] = ACTIONS(482),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(482),
    [anon_sym_BQUOTE] = ACTIONS(482),
    [sym_regex] = ACTIONS(484),
    [sym_number] = ACTIONS(482),
    [sym_identifier] = ACTIONS(484),
    [sym_this] = ACTIONS(484),
    [sym_super] = ACTIONS(484),
    [sym_true] = ACTIONS(484),
    [sym_false] = ACTIONS(484),
    [sym_null] = ACTIONS(484),
    [sym_undefined] = ACTIONS(484),
    [anon_sym_AT] = ACTIONS(482),
    [anon_sym_get] = ACTIONS(484),
    [anon_sym_set] = ACTIONS(484),
  },
  [2533] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_STAR] = ACTIONS(3035),
    [anon_sym_default] = ACTIONS(3037),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_COMMA] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_else] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_in] = ACTIONS(3037),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_case] = ACTIONS(3037),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3037),
    [anon_sym_GT] = ACTIONS(3037),
    [anon_sym_SLASH] = ACTIONS(3037),
    [anon_sym_DOT] = ACTIONS(3037),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_QMARK] = ACTIONS(3035),
    [anon_sym_AMP_AMP] = ACTIONS(3035),
    [anon_sym_PIPE_PIPE] = ACTIONS(3035),
    [anon_sym_GT_GT] = ACTIONS(3037),
    [anon_sym_GT_GT_GT] = ACTIONS(3035),
    [anon_sym_LT_LT] = ACTIONS(3035),
    [anon_sym_AMP] = ACTIONS(3037),
    [anon_sym_CARET] = ACTIONS(3035),
    [anon_sym_PIPE] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_PERCENT] = ACTIONS(3035),
    [anon_sym_LT_EQ] = ACTIONS(3035),
    [anon_sym_EQ_EQ] = ACTIONS(3037),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3035),
    [anon_sym_BANG_EQ] = ACTIONS(3037),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3035),
    [anon_sym_GT_EQ] = ACTIONS(3035),
    [anon_sym_instanceof] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3037),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [2534] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_STAR] = ACTIONS(3856),
    [anon_sym_default] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_COMMA] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_else] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_in] = ACTIONS(3858),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_case] = ACTIONS(3858),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3858),
    [anon_sym_GT] = ACTIONS(3858),
    [anon_sym_SLASH] = ACTIONS(3858),
    [anon_sym_DOT] = ACTIONS(3858),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_QMARK] = ACTIONS(3856),
    [anon_sym_AMP_AMP] = ACTIONS(3856),
    [anon_sym_PIPE_PIPE] = ACTIONS(3856),
    [anon_sym_GT_GT] = ACTIONS(3858),
    [anon_sym_GT_GT_GT] = ACTIONS(3856),
    [anon_sym_LT_LT] = ACTIONS(3856),
    [anon_sym_AMP] = ACTIONS(3858),
    [anon_sym_CARET] = ACTIONS(3856),
    [anon_sym_PIPE] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_PERCENT] = ACTIONS(3856),
    [anon_sym_LT_EQ] = ACTIONS(3856),
    [anon_sym_EQ_EQ] = ACTIONS(3858),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3856),
    [anon_sym_BANG_EQ] = ACTIONS(3858),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3856),
    [anon_sym_GT_EQ] = ACTIONS(3856),
    [anon_sym_instanceof] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3858),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [2535] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_STAR] = ACTIONS(3860),
    [anon_sym_default] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_COMMA] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_else] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_in] = ACTIONS(3862),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_case] = ACTIONS(3862),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3862),
    [anon_sym_GT] = ACTIONS(3862),
    [anon_sym_SLASH] = ACTIONS(3862),
    [anon_sym_DOT] = ACTIONS(3862),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_QMARK] = ACTIONS(3860),
    [anon_sym_AMP_AMP] = ACTIONS(3860),
    [anon_sym_PIPE_PIPE] = ACTIONS(3860),
    [anon_sym_GT_GT] = ACTIONS(3862),
    [anon_sym_GT_GT_GT] = ACTIONS(3860),
    [anon_sym_LT_LT] = ACTIONS(3860),
    [anon_sym_AMP] = ACTIONS(3862),
    [anon_sym_CARET] = ACTIONS(3860),
    [anon_sym_PIPE] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_PERCENT] = ACTIONS(3860),
    [anon_sym_LT_EQ] = ACTIONS(3860),
    [anon_sym_EQ_EQ] = ACTIONS(3862),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3860),
    [anon_sym_BANG_EQ] = ACTIONS(3862),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3860),
    [anon_sym_GT_EQ] = ACTIONS(3860),
    [anon_sym_instanceof] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3862),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [2536] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_STAR] = ACTIONS(1588),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_COMMA] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_in] = ACTIONS(1590),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1590),
    [anon_sym_GT] = ACTIONS(1590),
    [anon_sym_SLASH] = ACTIONS(1590),
    [anon_sym_DOT] = ACTIONS(1590),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_QMARK] = ACTIONS(1588),
    [anon_sym_AMP_AMP] = ACTIONS(1588),
    [anon_sym_PIPE_PIPE] = ACTIONS(1588),
    [anon_sym_GT_GT] = ACTIONS(1590),
    [anon_sym_GT_GT_GT] = ACTIONS(1588),
    [anon_sym_LT_LT] = ACTIONS(1588),
    [anon_sym_AMP] = ACTIONS(1590),
    [anon_sym_CARET] = ACTIONS(1588),
    [anon_sym_PIPE] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_PERCENT] = ACTIONS(1588),
    [anon_sym_LT_EQ] = ACTIONS(1588),
    [anon_sym_EQ_EQ] = ACTIONS(1590),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(1588),
    [anon_sym_BANG_EQ] = ACTIONS(1590),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(1588),
    [anon_sym_GT_EQ] = ACTIONS(1588),
    [anon_sym_instanceof] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1590),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2537] = {
    [anon_sym_export] = ACTIONS(3932),
    [anon_sym_default] = ACTIONS(3932),
    [anon_sym_LBRACE] = ACTIONS(3930),
    [anon_sym_RBRACE] = ACTIONS(3930),
    [anon_sym_import] = ACTIONS(3932),
    [anon_sym_var] = ACTIONS(3932),
    [anon_sym_let] = ACTIONS(3932),
    [anon_sym_const] = ACTIONS(3932),
    [anon_sym_if] = ACTIONS(3932),
    [anon_sym_else] = ACTIONS(3932),
    [anon_sym_switch] = ACTIONS(3932),
    [anon_sym_for] = ACTIONS(3932),
    [anon_sym_LPAREN] = ACTIONS(3930),
    [anon_sym_while] = ACTIONS(3932),
    [anon_sym_do] = ACTIONS(3932),
    [anon_sym_try] = ACTIONS(3932),
    [anon_sym_with] = ACTIONS(3932),
    [anon_sym_break] = ACTIONS(3932),
    [anon_sym_continue] = ACTIONS(3932),
    [anon_sym_debugger] = ACTIONS(3932),
    [anon_sym_return] = ACTIONS(3932),
    [anon_sym_throw] = ACTIONS(3932),
    [anon_sym_SEMI] = ACTIONS(3930),
    [anon_sym_case] = ACTIONS(3932),
    [anon_sym_yield] = ACTIONS(3932),
    [anon_sym_LBRACK] = ACTIONS(3930),
    [anon_sym_LT] = ACTIONS(3930),
    [anon_sym_class] = ACTIONS(3932),
    [anon_sym_async] = ACTIONS(3932),
    [anon_sym_function] = ACTIONS(3932),
    [anon_sym_EQ_GT] = ACTIONS(3930),
    [anon_sym_new] = ACTIONS(3932),
    [anon_sym_await] = ACTIONS(3932),
    [anon_sym_PLUS] = ACTIONS(3932),
    [anon_sym_DASH] = ACTIONS(3932),
    [anon_sym_BANG] = ACTIONS(3930),
    [anon_sym_TILDE] = ACTIONS(3930),
    [anon_sym_typeof] = ACTIONS(3932),
    [anon_sym_void] = ACTIONS(3932),
    [anon_sym_delete] = ACTIONS(3932),
    [anon_sym_PLUS_PLUS] = ACTIONS(3930),
    [anon_sym_DASH_DASH] = ACTIONS(3930),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3930),
    [anon_sym_BQUOTE] = ACTIONS(3930),
    [sym_regex] = ACTIONS(3932),
    [sym_number] = ACTIONS(3930),
    [sym_identifier] = ACTIONS(3932),
    [sym_this] = ACTIONS(3932),
    [sym_super] = ACTIONS(3932),
    [sym_true] = ACTIONS(3932),
    [sym_false] = ACTIONS(3932),
    [sym_null] = ACTIONS(3932),
    [sym_undefined] = ACTIONS(3932),
    [anon_sym_AT] = ACTIONS(3930),
    [anon_sym_get] = ACTIONS(3932),
    [anon_sym_set] = ACTIONS(3932),
  },
  [2538] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_STAR] = ACTIONS(3934),
    [anon_sym_default] = ACTIONS(3936),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_COMMA] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_else] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_in] = ACTIONS(3936),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_case] = ACTIONS(3936),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3936),
    [anon_sym_GT] = ACTIONS(3936),
    [anon_sym_SLASH] = ACTIONS(3936),
    [anon_sym_DOT] = ACTIONS(3936),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_QMARK] = ACTIONS(3934),
    [anon_sym_AMP_AMP] = ACTIONS(3934),
    [anon_sym_PIPE_PIPE] = ACTIONS(3934),
    [anon_sym_GT_GT] = ACTIONS(3936),
    [anon_sym_GT_GT_GT] = ACTIONS(3934),
    [anon_sym_LT_LT] = ACTIONS(3934),
    [anon_sym_AMP] = ACTIONS(3936),
    [anon_sym_CARET] = ACTIONS(3934),
    [anon_sym_PIPE] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_PERCENT] = ACTIONS(3934),
    [anon_sym_LT_EQ] = ACTIONS(3934),
    [anon_sym_EQ_EQ] = ACTIONS(3936),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3934),
    [anon_sym_BANG_EQ] = ACTIONS(3936),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3934),
    [anon_sym_GT_EQ] = ACTIONS(3934),
    [anon_sym_instanceof] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3936),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [2539] = {
    [sym_export_statement] = STATE(2574),
    [sym__declaration] = STATE(2574),
    [sym_import_statement] = STATE(2574),
    [sym_expression_statement] = STATE(2574),
    [sym_variable_declaration] = STATE(2574),
    [sym_lexical_declaration] = STATE(2574),
    [sym_statement_block] = STATE(2574),
    [sym_if_statement] = STATE(2574),
    [sym_switch_statement] = STATE(2574),
    [sym_for_statement] = STATE(2574),
    [sym_for_in_statement] = STATE(2574),
    [sym_for_of_statement] = STATE(2574),
    [sym_while_statement] = STATE(2574),
    [sym_do_statement] = STATE(2574),
    [sym_try_statement] = STATE(2574),
    [sym_with_statement] = STATE(2574),
    [sym_break_statement] = STATE(2574),
    [sym_continue_statement] = STATE(2574),
    [sym_debugger_statement] = STATE(2574),
    [sym_return_statement] = STATE(2574),
    [sym_throw_statement] = STATE(2574),
    [sym_empty_statement] = STATE(2574),
    [sym_labeled_statement] = STATE(2574),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2540] = {
    [sym_export_statement] = STATE(2575),
    [sym__declaration] = STATE(2575),
    [sym_import_statement] = STATE(2575),
    [sym_expression_statement] = STATE(2575),
    [sym_variable_declaration] = STATE(2575),
    [sym_lexical_declaration] = STATE(2575),
    [sym_statement_block] = STATE(2575),
    [sym_if_statement] = STATE(2575),
    [sym_switch_statement] = STATE(2575),
    [sym_for_statement] = STATE(2575),
    [sym_for_in_statement] = STATE(2575),
    [sym_for_of_statement] = STATE(2575),
    [sym_while_statement] = STATE(2575),
    [sym_do_statement] = STATE(2575),
    [sym_try_statement] = STATE(2575),
    [sym_with_statement] = STATE(2575),
    [sym_break_statement] = STATE(2575),
    [sym_continue_statement] = STATE(2575),
    [sym_debugger_statement] = STATE(2575),
    [sym_return_statement] = STATE(2575),
    [sym_throw_statement] = STATE(2575),
    [sym_empty_statement] = STATE(2575),
    [sym_labeled_statement] = STATE(2575),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2541] = {
    [sym_export_statement] = STATE(2576),
    [sym__declaration] = STATE(2576),
    [sym_import_statement] = STATE(2576),
    [sym_expression_statement] = STATE(2576),
    [sym_variable_declaration] = STATE(2576),
    [sym_lexical_declaration] = STATE(2576),
    [sym_statement_block] = STATE(2576),
    [sym_if_statement] = STATE(2576),
    [sym_switch_statement] = STATE(2576),
    [sym_for_statement] = STATE(2576),
    [sym_for_in_statement] = STATE(2576),
    [sym_for_of_statement] = STATE(2576),
    [sym_while_statement] = STATE(2576),
    [sym_do_statement] = STATE(2576),
    [sym_try_statement] = STATE(2576),
    [sym_with_statement] = STATE(2576),
    [sym_break_statement] = STATE(2576),
    [sym_continue_statement] = STATE(2576),
    [sym_debugger_statement] = STATE(2576),
    [sym_return_statement] = STATE(2576),
    [sym_throw_statement] = STATE(2576),
    [sym_empty_statement] = STATE(2576),
    [sym_labeled_statement] = STATE(2576),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2542] = {
    [sym_export_statement] = STATE(2577),
    [sym__declaration] = STATE(2577),
    [sym_import_statement] = STATE(2577),
    [sym_expression_statement] = STATE(2577),
    [sym_variable_declaration] = STATE(2577),
    [sym_lexical_declaration] = STATE(2577),
    [sym_statement_block] = STATE(2577),
    [sym_if_statement] = STATE(2577),
    [sym_switch_statement] = STATE(2577),
    [sym_for_statement] = STATE(2577),
    [sym_for_in_statement] = STATE(2577),
    [sym_for_of_statement] = STATE(2577),
    [sym_while_statement] = STATE(2577),
    [sym_do_statement] = STATE(2577),
    [sym_try_statement] = STATE(2577),
    [sym_with_statement] = STATE(2577),
    [sym_break_statement] = STATE(2577),
    [sym_continue_statement] = STATE(2577),
    [sym_debugger_statement] = STATE(2577),
    [sym_return_statement] = STATE(2577),
    [sym_throw_statement] = STATE(2577),
    [sym_empty_statement] = STATE(2577),
    [sym_labeled_statement] = STATE(2577),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2543] = {
    [sym_export_statement] = STATE(2578),
    [sym__declaration] = STATE(2578),
    [sym_import_statement] = STATE(2578),
    [sym_expression_statement] = STATE(2578),
    [sym_variable_declaration] = STATE(2578),
    [sym_lexical_declaration] = STATE(2578),
    [sym_statement_block] = STATE(2578),
    [sym_if_statement] = STATE(2578),
    [sym_switch_statement] = STATE(2578),
    [sym_for_statement] = STATE(2578),
    [sym_for_in_statement] = STATE(2578),
    [sym_for_of_statement] = STATE(2578),
    [sym_while_statement] = STATE(2578),
    [sym_do_statement] = STATE(2578),
    [sym_try_statement] = STATE(2578),
    [sym_with_statement] = STATE(2578),
    [sym_break_statement] = STATE(2578),
    [sym_continue_statement] = STATE(2578),
    [sym_debugger_statement] = STATE(2578),
    [sym_return_statement] = STATE(2578),
    [sym_throw_statement] = STATE(2578),
    [sym_empty_statement] = STATE(2578),
    [sym_labeled_statement] = STATE(2578),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2544] = {
    [sym_export_statement] = STATE(2579),
    [sym__declaration] = STATE(2579),
    [sym_import_statement] = STATE(2579),
    [sym_expression_statement] = STATE(2579),
    [sym_variable_declaration] = STATE(2579),
    [sym_lexical_declaration] = STATE(2579),
    [sym_statement_block] = STATE(2579),
    [sym_if_statement] = STATE(2579),
    [sym_switch_statement] = STATE(2579),
    [sym_for_statement] = STATE(2579),
    [sym_for_in_statement] = STATE(2579),
    [sym_for_of_statement] = STATE(2579),
    [sym_while_statement] = STATE(2579),
    [sym_do_statement] = STATE(2579),
    [sym_try_statement] = STATE(2579),
    [sym_with_statement] = STATE(2579),
    [sym_break_statement] = STATE(2579),
    [sym_continue_statement] = STATE(2579),
    [sym_debugger_statement] = STATE(2579),
    [sym_return_statement] = STATE(2579),
    [sym_throw_statement] = STATE(2579),
    [sym_empty_statement] = STATE(2579),
    [sym_labeled_statement] = STATE(2579),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(1626),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(1627),
    [sym_function] = STATE(1628),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(1627),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(1626),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(1630),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(1630),
    [anon_sym_export] = ACTIONS(3490),
    [anon_sym_LBRACE] = ACTIONS(3494),
    [anon_sym_import] = ACTIONS(3498),
    [anon_sym_var] = ACTIONS(3500),
    [anon_sym_let] = ACTIONS(3502),
    [anon_sym_const] = ACTIONS(3502),
    [anon_sym_if] = ACTIONS(3504),
    [anon_sym_switch] = ACTIONS(3506),
    [anon_sym_for] = ACTIONS(3508),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(3510),
    [anon_sym_do] = ACTIONS(3512),
    [anon_sym_try] = ACTIONS(3514),
    [anon_sym_with] = ACTIONS(3516),
    [anon_sym_break] = ACTIONS(3518),
    [anon_sym_continue] = ACTIONS(3520),
    [anon_sym_debugger] = ACTIONS(3522),
    [anon_sym_return] = ACTIONS(3524),
    [anon_sym_throw] = ACTIONS(3526),
    [anon_sym_SEMI] = ACTIONS(3528),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(3530),
    [anon_sym_async] = ACTIONS(3532),
    [anon_sym_function] = ACTIONS(3534),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(3536),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2545] = {
    [anon_sym_export] = ACTIONS(4638),
    [anon_sym_default] = ACTIONS(4638),
    [anon_sym_LBRACE] = ACTIONS(4636),
    [anon_sym_RBRACE] = ACTIONS(4636),
    [anon_sym_import] = ACTIONS(4638),
    [anon_sym_var] = ACTIONS(4638),
    [anon_sym_let] = ACTIONS(4638),
    [anon_sym_const] = ACTIONS(4638),
    [anon_sym_if] = ACTIONS(4638),
    [anon_sym_switch] = ACTIONS(4638),
    [anon_sym_for] = ACTIONS(4638),
    [anon_sym_LPAREN] = ACTIONS(4636),
    [anon_sym_while] = ACTIONS(4638),
    [anon_sym_do] = ACTIONS(4638),
    [anon_sym_try] = ACTIONS(4638),
    [anon_sym_with] = ACTIONS(4638),
    [anon_sym_break] = ACTIONS(4638),
    [anon_sym_continue] = ACTIONS(4638),
    [anon_sym_debugger] = ACTIONS(4638),
    [anon_sym_return] = ACTIONS(4638),
    [anon_sym_throw] = ACTIONS(4638),
    [anon_sym_SEMI] = ACTIONS(4636),
    [anon_sym_case] = ACTIONS(4638),
    [anon_sym_yield] = ACTIONS(4638),
    [anon_sym_LBRACK] = ACTIONS(4636),
    [anon_sym_LT] = ACTIONS(4636),
    [anon_sym_class] = ACTIONS(4638),
    [anon_sym_async] = ACTIONS(4638),
    [anon_sym_function] = ACTIONS(4638),
    [anon_sym_EQ_GT] = ACTIONS(4636),
    [anon_sym_new] = ACTIONS(4638),
    [anon_sym_await] = ACTIONS(4638),
    [anon_sym_PLUS] = ACTIONS(4638),
    [anon_sym_DASH] = ACTIONS(4638),
    [anon_sym_BANG] = ACTIONS(4636),
    [anon_sym_TILDE] = ACTIONS(4636),
    [anon_sym_typeof] = ACTIONS(4638),
    [anon_sym_void] = ACTIONS(4638),
    [anon_sym_delete] = ACTIONS(4638),
    [anon_sym_PLUS_PLUS] = ACTIONS(4636),
    [anon_sym_DASH_DASH] = ACTIONS(4636),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4636),
    [anon_sym_BQUOTE] = ACTIONS(4636),
    [sym_regex] = ACTIONS(4638),
    [sym_number] = ACTIONS(4636),
    [sym_identifier] = ACTIONS(4638),
    [sym_this] = ACTIONS(4638),
    [sym_super] = ACTIONS(4638),
    [sym_true] = ACTIONS(4638),
    [sym_false] = ACTIONS(4638),
    [sym_null] = ACTIONS(4638),
    [sym_undefined] = ACTIONS(4638),
    [anon_sym_AT] = ACTIONS(4636),
    [anon_sym_get] = ACTIONS(4638),
    [anon_sym_set] = ACTIONS(4638),
  },
  [2546] = {
    [anon_sym_export] = ACTIONS(4648),
    [anon_sym_default] = ACTIONS(4648),
    [anon_sym_LBRACE] = ACTIONS(4646),
    [anon_sym_RBRACE] = ACTIONS(4646),
    [anon_sym_import] = ACTIONS(4648),
    [anon_sym_var] = ACTIONS(4648),
    [anon_sym_let] = ACTIONS(4648),
    [anon_sym_const] = ACTIONS(4648),
    [anon_sym_if] = ACTIONS(4648),
    [anon_sym_switch] = ACTIONS(4648),
    [anon_sym_for] = ACTIONS(4648),
    [anon_sym_LPAREN] = ACTIONS(4646),
    [anon_sym_while] = ACTIONS(4648),
    [anon_sym_do] = ACTIONS(4648),
    [anon_sym_try] = ACTIONS(4648),
    [anon_sym_with] = ACTIONS(4648),
    [anon_sym_break] = ACTIONS(4648),
    [anon_sym_continue] = ACTIONS(4648),
    [anon_sym_debugger] = ACTIONS(4648),
    [anon_sym_return] = ACTIONS(4648),
    [anon_sym_throw] = ACTIONS(4648),
    [anon_sym_SEMI] = ACTIONS(4646),
    [anon_sym_case] = ACTIONS(4648),
    [anon_sym_yield] = ACTIONS(4648),
    [anon_sym_LBRACK] = ACTIONS(4646),
    [anon_sym_LT] = ACTIONS(4646),
    [anon_sym_class] = ACTIONS(4648),
    [anon_sym_async] = ACTIONS(4648),
    [anon_sym_function] = ACTIONS(4648),
    [anon_sym_EQ_GT] = ACTIONS(4646),
    [anon_sym_new] = ACTIONS(4648),
    [anon_sym_await] = ACTIONS(4648),
    [anon_sym_PLUS] = ACTIONS(4648),
    [anon_sym_DASH] = ACTIONS(4648),
    [anon_sym_BANG] = ACTIONS(4646),
    [anon_sym_TILDE] = ACTIONS(4646),
    [anon_sym_typeof] = ACTIONS(4648),
    [anon_sym_void] = ACTIONS(4648),
    [anon_sym_delete] = ACTIONS(4648),
    [anon_sym_PLUS_PLUS] = ACTIONS(4646),
    [anon_sym_DASH_DASH] = ACTIONS(4646),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4646),
    [anon_sym_BQUOTE] = ACTIONS(4646),
    [sym_regex] = ACTIONS(4648),
    [sym_number] = ACTIONS(4646),
    [sym_identifier] = ACTIONS(4648),
    [sym_this] = ACTIONS(4648),
    [sym_super] = ACTIONS(4648),
    [sym_true] = ACTIONS(4648),
    [sym_false] = ACTIONS(4648),
    [sym_null] = ACTIONS(4648),
    [sym_undefined] = ACTIONS(4648),
    [anon_sym_AT] = ACTIONS(4646),
    [anon_sym_get] = ACTIONS(4648),
    [anon_sym_set] = ACTIONS(4648),
  },
  [2547] = {
    [anon_sym_export] = ACTIONS(4654),
    [anon_sym_default] = ACTIONS(4654),
    [anon_sym_LBRACE] = ACTIONS(4652),
    [anon_sym_RBRACE] = ACTIONS(4652),
    [anon_sym_import] = ACTIONS(4654),
    [anon_sym_var] = ACTIONS(4654),
    [anon_sym_let] = ACTIONS(4654),
    [anon_sym_const] = ACTIONS(4654),
    [anon_sym_if] = ACTIONS(4654),
    [anon_sym_switch] = ACTIONS(4654),
    [anon_sym_for] = ACTIONS(4654),
    [anon_sym_LPAREN] = ACTIONS(4652),
    [anon_sym_while] = ACTIONS(4654),
    [anon_sym_do] = ACTIONS(4654),
    [anon_sym_try] = ACTIONS(4654),
    [anon_sym_with] = ACTIONS(4654),
    [anon_sym_break] = ACTIONS(4654),
    [anon_sym_continue] = ACTIONS(4654),
    [anon_sym_debugger] = ACTIONS(4654),
    [anon_sym_return] = ACTIONS(4654),
    [anon_sym_throw] = ACTIONS(4654),
    [anon_sym_SEMI] = ACTIONS(4652),
    [anon_sym_case] = ACTIONS(4654),
    [anon_sym_yield] = ACTIONS(4654),
    [anon_sym_LBRACK] = ACTIONS(4652),
    [anon_sym_LT] = ACTIONS(4652),
    [anon_sym_class] = ACTIONS(4654),
    [anon_sym_async] = ACTIONS(4654),
    [anon_sym_function] = ACTIONS(4654),
    [anon_sym_EQ_GT] = ACTIONS(4652),
    [anon_sym_new] = ACTIONS(4654),
    [anon_sym_await] = ACTIONS(4654),
    [anon_sym_PLUS] = ACTIONS(4654),
    [anon_sym_DASH] = ACTIONS(4654),
    [anon_sym_BANG] = ACTIONS(4652),
    [anon_sym_TILDE] = ACTIONS(4652),
    [anon_sym_typeof] = ACTIONS(4654),
    [anon_sym_void] = ACTIONS(4654),
    [anon_sym_delete] = ACTIONS(4654),
    [anon_sym_PLUS_PLUS] = ACTIONS(4652),
    [anon_sym_DASH_DASH] = ACTIONS(4652),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4652),
    [anon_sym_BQUOTE] = ACTIONS(4652),
    [sym_regex] = ACTIONS(4654),
    [sym_number] = ACTIONS(4652),
    [sym_identifier] = ACTIONS(4654),
    [sym_this] = ACTIONS(4654),
    [sym_super] = ACTIONS(4654),
    [sym_true] = ACTIONS(4654),
    [sym_false] = ACTIONS(4654),
    [sym_null] = ACTIONS(4654),
    [sym_undefined] = ACTIONS(4654),
    [anon_sym_AT] = ACTIONS(4652),
    [anon_sym_get] = ACTIONS(4654),
    [anon_sym_set] = ACTIONS(4654),
  },
  [2548] = {
    [anon_sym_export] = ACTIONS(4658),
    [anon_sym_default] = ACTIONS(4658),
    [anon_sym_LBRACE] = ACTIONS(4656),
    [anon_sym_RBRACE] = ACTIONS(4656),
    [anon_sym_import] = ACTIONS(4658),
    [anon_sym_var] = ACTIONS(4658),
    [anon_sym_let] = ACTIONS(4658),
    [anon_sym_const] = ACTIONS(4658),
    [anon_sym_if] = ACTIONS(4658),
    [anon_sym_switch] = ACTIONS(4658),
    [anon_sym_for] = ACTIONS(4658),
    [anon_sym_LPAREN] = ACTIONS(4656),
    [anon_sym_while] = ACTIONS(4658),
    [anon_sym_do] = ACTIONS(4658),
    [anon_sym_try] = ACTIONS(4658),
    [anon_sym_with] = ACTIONS(4658),
    [anon_sym_break] = ACTIONS(4658),
    [anon_sym_continue] = ACTIONS(4658),
    [anon_sym_debugger] = ACTIONS(4658),
    [anon_sym_return] = ACTIONS(4658),
    [anon_sym_throw] = ACTIONS(4658),
    [anon_sym_SEMI] = ACTIONS(4656),
    [anon_sym_case] = ACTIONS(4658),
    [anon_sym_yield] = ACTIONS(4658),
    [anon_sym_LBRACK] = ACTIONS(4656),
    [anon_sym_LT] = ACTIONS(4656),
    [anon_sym_class] = ACTIONS(4658),
    [anon_sym_async] = ACTIONS(4658),
    [anon_sym_function] = ACTIONS(4658),
    [anon_sym_EQ_GT] = ACTIONS(4656),
    [anon_sym_new] = ACTIONS(4658),
    [anon_sym_await] = ACTIONS(4658),
    [anon_sym_PLUS] = ACTIONS(4658),
    [anon_sym_DASH] = ACTIONS(4658),
    [anon_sym_BANG] = ACTIONS(4656),
    [anon_sym_TILDE] = ACTIONS(4656),
    [anon_sym_typeof] = ACTIONS(4658),
    [anon_sym_void] = ACTIONS(4658),
    [anon_sym_delete] = ACTIONS(4658),
    [anon_sym_PLUS_PLUS] = ACTIONS(4656),
    [anon_sym_DASH_DASH] = ACTIONS(4656),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4656),
    [anon_sym_BQUOTE] = ACTIONS(4656),
    [sym_regex] = ACTIONS(4658),
    [sym_number] = ACTIONS(4656),
    [sym_identifier] = ACTIONS(4658),
    [sym_this] = ACTIONS(4658),
    [sym_super] = ACTIONS(4658),
    [sym_true] = ACTIONS(4658),
    [sym_false] = ACTIONS(4658),
    [sym_null] = ACTIONS(4658),
    [sym_undefined] = ACTIONS(4658),
    [anon_sym_AT] = ACTIONS(4656),
    [anon_sym_get] = ACTIONS(4658),
    [anon_sym_set] = ACTIONS(4658),
  },
  [2549] = {
    [anon_sym_export] = ACTIONS(4662),
    [anon_sym_default] = ACTIONS(4662),
    [anon_sym_LBRACE] = ACTIONS(4660),
    [anon_sym_RBRACE] = ACTIONS(4660),
    [anon_sym_import] = ACTIONS(4662),
    [anon_sym_var] = ACTIONS(4662),
    [anon_sym_let] = ACTIONS(4662),
    [anon_sym_const] = ACTIONS(4662),
    [anon_sym_if] = ACTIONS(4662),
    [anon_sym_switch] = ACTIONS(4662),
    [anon_sym_for] = ACTIONS(4662),
    [anon_sym_LPAREN] = ACTIONS(4660),
    [anon_sym_while] = ACTIONS(4662),
    [anon_sym_do] = ACTIONS(4662),
    [anon_sym_try] = ACTIONS(4662),
    [anon_sym_with] = ACTIONS(4662),
    [anon_sym_break] = ACTIONS(4662),
    [anon_sym_continue] = ACTIONS(4662),
    [anon_sym_debugger] = ACTIONS(4662),
    [anon_sym_return] = ACTIONS(4662),
    [anon_sym_throw] = ACTIONS(4662),
    [anon_sym_SEMI] = ACTIONS(4660),
    [anon_sym_case] = ACTIONS(4662),
    [anon_sym_yield] = ACTIONS(4662),
    [anon_sym_LBRACK] = ACTIONS(4660),
    [anon_sym_LT] = ACTIONS(4660),
    [anon_sym_class] = ACTIONS(4662),
    [anon_sym_async] = ACTIONS(4662),
    [anon_sym_function] = ACTIONS(4662),
    [anon_sym_EQ_GT] = ACTIONS(4660),
    [anon_sym_new] = ACTIONS(4662),
    [anon_sym_await] = ACTIONS(4662),
    [anon_sym_PLUS] = ACTIONS(4662),
    [anon_sym_DASH] = ACTIONS(4662),
    [anon_sym_BANG] = ACTIONS(4660),
    [anon_sym_TILDE] = ACTIONS(4660),
    [anon_sym_typeof] = ACTIONS(4662),
    [anon_sym_void] = ACTIONS(4662),
    [anon_sym_delete] = ACTIONS(4662),
    [anon_sym_PLUS_PLUS] = ACTIONS(4660),
    [anon_sym_DASH_DASH] = ACTIONS(4660),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4660),
    [anon_sym_BQUOTE] = ACTIONS(4660),
    [sym_regex] = ACTIONS(4662),
    [sym_number] = ACTIONS(4660),
    [sym_identifier] = ACTIONS(4662),
    [sym_this] = ACTIONS(4662),
    [sym_super] = ACTIONS(4662),
    [sym_true] = ACTIONS(4662),
    [sym_false] = ACTIONS(4662),
    [sym_null] = ACTIONS(4662),
    [sym_undefined] = ACTIONS(4662),
    [anon_sym_AT] = ACTIONS(4660),
    [anon_sym_get] = ACTIONS(4662),
    [anon_sym_set] = ACTIONS(4662),
  },
  [2550] = {
    [anon_sym_export] = ACTIONS(4666),
    [anon_sym_default] = ACTIONS(4666),
    [anon_sym_LBRACE] = ACTIONS(4664),
    [anon_sym_RBRACE] = ACTIONS(4664),
    [anon_sym_import] = ACTIONS(4666),
    [anon_sym_var] = ACTIONS(4666),
    [anon_sym_let] = ACTIONS(4666),
    [anon_sym_const] = ACTIONS(4666),
    [anon_sym_if] = ACTIONS(4666),
    [anon_sym_switch] = ACTIONS(4666),
    [anon_sym_for] = ACTIONS(4666),
    [anon_sym_LPAREN] = ACTIONS(4664),
    [anon_sym_while] = ACTIONS(4666),
    [anon_sym_do] = ACTIONS(4666),
    [anon_sym_try] = ACTIONS(4666),
    [anon_sym_with] = ACTIONS(4666),
    [anon_sym_break] = ACTIONS(4666),
    [anon_sym_continue] = ACTIONS(4666),
    [anon_sym_debugger] = ACTIONS(4666),
    [anon_sym_return] = ACTIONS(4666),
    [anon_sym_throw] = ACTIONS(4666),
    [anon_sym_SEMI] = ACTIONS(4664),
    [anon_sym_case] = ACTIONS(4666),
    [anon_sym_yield] = ACTIONS(4666),
    [anon_sym_LBRACK] = ACTIONS(4664),
    [anon_sym_LT] = ACTIONS(4664),
    [anon_sym_class] = ACTIONS(4666),
    [anon_sym_async] = ACTIONS(4666),
    [anon_sym_function] = ACTIONS(4666),
    [anon_sym_EQ_GT] = ACTIONS(4664),
    [anon_sym_new] = ACTIONS(4666),
    [anon_sym_await] = ACTIONS(4666),
    [anon_sym_PLUS] = ACTIONS(4666),
    [anon_sym_DASH] = ACTIONS(4666),
    [anon_sym_BANG] = ACTIONS(4664),
    [anon_sym_TILDE] = ACTIONS(4664),
    [anon_sym_typeof] = ACTIONS(4666),
    [anon_sym_void] = ACTIONS(4666),
    [anon_sym_delete] = ACTIONS(4666),
    [anon_sym_PLUS_PLUS] = ACTIONS(4664),
    [anon_sym_DASH_DASH] = ACTIONS(4664),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4664),
    [anon_sym_BQUOTE] = ACTIONS(4664),
    [sym_regex] = ACTIONS(4666),
    [sym_number] = ACTIONS(4664),
    [sym_identifier] = ACTIONS(4666),
    [sym_this] = ACTIONS(4666),
    [sym_super] = ACTIONS(4666),
    [sym_true] = ACTIONS(4666),
    [sym_false] = ACTIONS(4666),
    [sym_null] = ACTIONS(4666),
    [sym_undefined] = ACTIONS(4666),
    [anon_sym_AT] = ACTIONS(4664),
    [anon_sym_get] = ACTIONS(4666),
    [anon_sym_set] = ACTIONS(4666),
  },
  [2551] = {
    [anon_sym_export] = ACTIONS(4670),
    [anon_sym_default] = ACTIONS(4670),
    [anon_sym_LBRACE] = ACTIONS(4668),
    [anon_sym_RBRACE] = ACTIONS(4668),
    [anon_sym_import] = ACTIONS(4670),
    [anon_sym_var] = ACTIONS(4670),
    [anon_sym_let] = ACTIONS(4670),
    [anon_sym_const] = ACTIONS(4670),
    [anon_sym_if] = ACTIONS(4670),
    [anon_sym_switch] = ACTIONS(4670),
    [anon_sym_for] = ACTIONS(4670),
    [anon_sym_LPAREN] = ACTIONS(4668),
    [anon_sym_while] = ACTIONS(4670),
    [anon_sym_do] = ACTIONS(4670),
    [anon_sym_try] = ACTIONS(4670),
    [anon_sym_with] = ACTIONS(4670),
    [anon_sym_break] = ACTIONS(4670),
    [anon_sym_continue] = ACTIONS(4670),
    [anon_sym_debugger] = ACTIONS(4670),
    [anon_sym_return] = ACTIONS(4670),
    [anon_sym_throw] = ACTIONS(4670),
    [anon_sym_SEMI] = ACTIONS(4668),
    [anon_sym_case] = ACTIONS(4670),
    [anon_sym_yield] = ACTIONS(4670),
    [anon_sym_LBRACK] = ACTIONS(4668),
    [anon_sym_LT] = ACTIONS(4668),
    [anon_sym_class] = ACTIONS(4670),
    [anon_sym_async] = ACTIONS(4670),
    [anon_sym_function] = ACTIONS(4670),
    [anon_sym_EQ_GT] = ACTIONS(4668),
    [anon_sym_new] = ACTIONS(4670),
    [anon_sym_await] = ACTIONS(4670),
    [anon_sym_PLUS] = ACTIONS(4670),
    [anon_sym_DASH] = ACTIONS(4670),
    [anon_sym_BANG] = ACTIONS(4668),
    [anon_sym_TILDE] = ACTIONS(4668),
    [anon_sym_typeof] = ACTIONS(4670),
    [anon_sym_void] = ACTIONS(4670),
    [anon_sym_delete] = ACTIONS(4670),
    [anon_sym_PLUS_PLUS] = ACTIONS(4668),
    [anon_sym_DASH_DASH] = ACTIONS(4668),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4668),
    [anon_sym_BQUOTE] = ACTIONS(4668),
    [sym_regex] = ACTIONS(4670),
    [sym_number] = ACTIONS(4668),
    [sym_identifier] = ACTIONS(4670),
    [sym_this] = ACTIONS(4670),
    [sym_super] = ACTIONS(4670),
    [sym_true] = ACTIONS(4670),
    [sym_false] = ACTIONS(4670),
    [sym_null] = ACTIONS(4670),
    [sym_undefined] = ACTIONS(4670),
    [anon_sym_AT] = ACTIONS(4668),
    [anon_sym_get] = ACTIONS(4670),
    [anon_sym_set] = ACTIONS(4670),
  },
  [2552] = {
    [anon_sym_export] = ACTIONS(4720),
    [anon_sym_default] = ACTIONS(4720),
    [anon_sym_LBRACE] = ACTIONS(4718),
    [anon_sym_RBRACE] = ACTIONS(4718),
    [anon_sym_import] = ACTIONS(4720),
    [anon_sym_var] = ACTIONS(4720),
    [anon_sym_let] = ACTIONS(4720),
    [anon_sym_const] = ACTIONS(4720),
    [anon_sym_if] = ACTIONS(4720),
    [anon_sym_switch] = ACTIONS(4720),
    [anon_sym_for] = ACTIONS(4720),
    [anon_sym_LPAREN] = ACTIONS(4718),
    [anon_sym_while] = ACTIONS(4720),
    [anon_sym_do] = ACTIONS(4720),
    [anon_sym_try] = ACTIONS(4720),
    [anon_sym_with] = ACTIONS(4720),
    [anon_sym_break] = ACTIONS(4720),
    [anon_sym_continue] = ACTIONS(4720),
    [anon_sym_debugger] = ACTIONS(4720),
    [anon_sym_return] = ACTIONS(4720),
    [anon_sym_throw] = ACTIONS(4720),
    [anon_sym_SEMI] = ACTIONS(4718),
    [anon_sym_case] = ACTIONS(4720),
    [anon_sym_finally] = ACTIONS(4720),
    [anon_sym_yield] = ACTIONS(4720),
    [anon_sym_LBRACK] = ACTIONS(4718),
    [anon_sym_LT] = ACTIONS(4718),
    [anon_sym_class] = ACTIONS(4720),
    [anon_sym_async] = ACTIONS(4720),
    [anon_sym_function] = ACTIONS(4720),
    [anon_sym_EQ_GT] = ACTIONS(4718),
    [anon_sym_new] = ACTIONS(4720),
    [anon_sym_await] = ACTIONS(4720),
    [anon_sym_PLUS] = ACTIONS(4720),
    [anon_sym_DASH] = ACTIONS(4720),
    [anon_sym_BANG] = ACTIONS(4718),
    [anon_sym_TILDE] = ACTIONS(4718),
    [anon_sym_typeof] = ACTIONS(4720),
    [anon_sym_void] = ACTIONS(4720),
    [anon_sym_delete] = ACTIONS(4720),
    [anon_sym_PLUS_PLUS] = ACTIONS(4718),
    [anon_sym_DASH_DASH] = ACTIONS(4718),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4718),
    [anon_sym_BQUOTE] = ACTIONS(4718),
    [sym_regex] = ACTIONS(4720),
    [sym_number] = ACTIONS(4718),
    [sym_identifier] = ACTIONS(4720),
    [sym_this] = ACTIONS(4720),
    [sym_super] = ACTIONS(4720),
    [sym_true] = ACTIONS(4720),
    [sym_false] = ACTIONS(4720),
    [sym_null] = ACTIONS(4720),
    [sym_undefined] = ACTIONS(4720),
    [anon_sym_AT] = ACTIONS(4718),
    [anon_sym_get] = ACTIONS(4720),
    [anon_sym_set] = ACTIONS(4720),
  },
  [2553] = {
    [sym__automatic_semicolon] = ACTIONS(3035),
    [anon_sym_export] = ACTIONS(3037),
    [anon_sym_default] = ACTIONS(3037),
    [anon_sym_LBRACE] = ACTIONS(3035),
    [anon_sym_RBRACE] = ACTIONS(3035),
    [anon_sym_import] = ACTIONS(3037),
    [anon_sym_var] = ACTIONS(3037),
    [anon_sym_let] = ACTIONS(3037),
    [anon_sym_const] = ACTIONS(3037),
    [anon_sym_if] = ACTIONS(3037),
    [anon_sym_else] = ACTIONS(3037),
    [anon_sym_switch] = ACTIONS(3037),
    [anon_sym_for] = ACTIONS(3037),
    [anon_sym_LPAREN] = ACTIONS(3035),
    [anon_sym_while] = ACTIONS(3037),
    [anon_sym_do] = ACTIONS(3037),
    [anon_sym_try] = ACTIONS(3037),
    [anon_sym_with] = ACTIONS(3037),
    [anon_sym_break] = ACTIONS(3037),
    [anon_sym_continue] = ACTIONS(3037),
    [anon_sym_debugger] = ACTIONS(3037),
    [anon_sym_return] = ACTIONS(3037),
    [anon_sym_throw] = ACTIONS(3037),
    [anon_sym_SEMI] = ACTIONS(3035),
    [anon_sym_case] = ACTIONS(3037),
    [anon_sym_yield] = ACTIONS(3037),
    [anon_sym_LBRACK] = ACTIONS(3035),
    [anon_sym_LT] = ACTIONS(3035),
    [anon_sym_class] = ACTIONS(3037),
    [anon_sym_async] = ACTIONS(3037),
    [anon_sym_function] = ACTIONS(3037),
    [anon_sym_EQ_GT] = ACTIONS(3035),
    [anon_sym_new] = ACTIONS(3037),
    [anon_sym_await] = ACTIONS(3037),
    [anon_sym_PLUS] = ACTIONS(3037),
    [anon_sym_DASH] = ACTIONS(3037),
    [anon_sym_BANG] = ACTIONS(3035),
    [anon_sym_TILDE] = ACTIONS(3035),
    [anon_sym_typeof] = ACTIONS(3037),
    [anon_sym_void] = ACTIONS(3037),
    [anon_sym_delete] = ACTIONS(3037),
    [anon_sym_PLUS_PLUS] = ACTIONS(3035),
    [anon_sym_DASH_DASH] = ACTIONS(3035),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3035),
    [anon_sym_BQUOTE] = ACTIONS(3035),
    [sym_regex] = ACTIONS(3037),
    [sym_number] = ACTIONS(3035),
    [sym_identifier] = ACTIONS(3037),
    [sym_this] = ACTIONS(3037),
    [sym_super] = ACTIONS(3037),
    [sym_true] = ACTIONS(3037),
    [sym_false] = ACTIONS(3037),
    [sym_null] = ACTIONS(3037),
    [sym_undefined] = ACTIONS(3037),
    [anon_sym_AT] = ACTIONS(3035),
    [anon_sym_get] = ACTIONS(3037),
    [anon_sym_set] = ACTIONS(3037),
  },
  [2554] = {
    [sym__automatic_semicolon] = ACTIONS(3856),
    [anon_sym_export] = ACTIONS(3858),
    [anon_sym_default] = ACTIONS(3858),
    [anon_sym_LBRACE] = ACTIONS(3856),
    [anon_sym_RBRACE] = ACTIONS(3856),
    [anon_sym_import] = ACTIONS(3858),
    [anon_sym_var] = ACTIONS(3858),
    [anon_sym_let] = ACTIONS(3858),
    [anon_sym_const] = ACTIONS(3858),
    [anon_sym_if] = ACTIONS(3858),
    [anon_sym_else] = ACTIONS(3858),
    [anon_sym_switch] = ACTIONS(3858),
    [anon_sym_for] = ACTIONS(3858),
    [anon_sym_LPAREN] = ACTIONS(3856),
    [anon_sym_while] = ACTIONS(3858),
    [anon_sym_do] = ACTIONS(3858),
    [anon_sym_try] = ACTIONS(3858),
    [anon_sym_with] = ACTIONS(3858),
    [anon_sym_break] = ACTIONS(3858),
    [anon_sym_continue] = ACTIONS(3858),
    [anon_sym_debugger] = ACTIONS(3858),
    [anon_sym_return] = ACTIONS(3858),
    [anon_sym_throw] = ACTIONS(3858),
    [anon_sym_SEMI] = ACTIONS(3856),
    [anon_sym_case] = ACTIONS(3858),
    [anon_sym_yield] = ACTIONS(3858),
    [anon_sym_LBRACK] = ACTIONS(3856),
    [anon_sym_LT] = ACTIONS(3856),
    [anon_sym_class] = ACTIONS(3858),
    [anon_sym_async] = ACTIONS(3858),
    [anon_sym_function] = ACTIONS(3858),
    [anon_sym_EQ_GT] = ACTIONS(3856),
    [anon_sym_new] = ACTIONS(3858),
    [anon_sym_await] = ACTIONS(3858),
    [anon_sym_PLUS] = ACTIONS(3858),
    [anon_sym_DASH] = ACTIONS(3858),
    [anon_sym_BANG] = ACTIONS(3856),
    [anon_sym_TILDE] = ACTIONS(3856),
    [anon_sym_typeof] = ACTIONS(3858),
    [anon_sym_void] = ACTIONS(3858),
    [anon_sym_delete] = ACTIONS(3858),
    [anon_sym_PLUS_PLUS] = ACTIONS(3856),
    [anon_sym_DASH_DASH] = ACTIONS(3856),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3856),
    [anon_sym_BQUOTE] = ACTIONS(3856),
    [sym_regex] = ACTIONS(3858),
    [sym_number] = ACTIONS(3856),
    [sym_identifier] = ACTIONS(3858),
    [sym_this] = ACTIONS(3858),
    [sym_super] = ACTIONS(3858),
    [sym_true] = ACTIONS(3858),
    [sym_false] = ACTIONS(3858),
    [sym_null] = ACTIONS(3858),
    [sym_undefined] = ACTIONS(3858),
    [anon_sym_AT] = ACTIONS(3856),
    [anon_sym_get] = ACTIONS(3858),
    [anon_sym_set] = ACTIONS(3858),
  },
  [2555] = {
    [sym__automatic_semicolon] = ACTIONS(3860),
    [anon_sym_export] = ACTIONS(3862),
    [anon_sym_default] = ACTIONS(3862),
    [anon_sym_LBRACE] = ACTIONS(3860),
    [anon_sym_RBRACE] = ACTIONS(3860),
    [anon_sym_import] = ACTIONS(3862),
    [anon_sym_var] = ACTIONS(3862),
    [anon_sym_let] = ACTIONS(3862),
    [anon_sym_const] = ACTIONS(3862),
    [anon_sym_if] = ACTIONS(3862),
    [anon_sym_else] = ACTIONS(3862),
    [anon_sym_switch] = ACTIONS(3862),
    [anon_sym_for] = ACTIONS(3862),
    [anon_sym_LPAREN] = ACTIONS(3860),
    [anon_sym_while] = ACTIONS(3862),
    [anon_sym_do] = ACTIONS(3862),
    [anon_sym_try] = ACTIONS(3862),
    [anon_sym_with] = ACTIONS(3862),
    [anon_sym_break] = ACTIONS(3862),
    [anon_sym_continue] = ACTIONS(3862),
    [anon_sym_debugger] = ACTIONS(3862),
    [anon_sym_return] = ACTIONS(3862),
    [anon_sym_throw] = ACTIONS(3862),
    [anon_sym_SEMI] = ACTIONS(3860),
    [anon_sym_case] = ACTIONS(3862),
    [anon_sym_yield] = ACTIONS(3862),
    [anon_sym_LBRACK] = ACTIONS(3860),
    [anon_sym_LT] = ACTIONS(3860),
    [anon_sym_class] = ACTIONS(3862),
    [anon_sym_async] = ACTIONS(3862),
    [anon_sym_function] = ACTIONS(3862),
    [anon_sym_EQ_GT] = ACTIONS(3860),
    [anon_sym_new] = ACTIONS(3862),
    [anon_sym_await] = ACTIONS(3862),
    [anon_sym_PLUS] = ACTIONS(3862),
    [anon_sym_DASH] = ACTIONS(3862),
    [anon_sym_BANG] = ACTIONS(3860),
    [anon_sym_TILDE] = ACTIONS(3860),
    [anon_sym_typeof] = ACTIONS(3862),
    [anon_sym_void] = ACTIONS(3862),
    [anon_sym_delete] = ACTIONS(3862),
    [anon_sym_PLUS_PLUS] = ACTIONS(3860),
    [anon_sym_DASH_DASH] = ACTIONS(3860),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3860),
    [anon_sym_BQUOTE] = ACTIONS(3860),
    [sym_regex] = ACTIONS(3862),
    [sym_number] = ACTIONS(3860),
    [sym_identifier] = ACTIONS(3862),
    [sym_this] = ACTIONS(3862),
    [sym_super] = ACTIONS(3862),
    [sym_true] = ACTIONS(3862),
    [sym_false] = ACTIONS(3862),
    [sym_null] = ACTIONS(3862),
    [sym_undefined] = ACTIONS(3862),
    [anon_sym_AT] = ACTIONS(3860),
    [anon_sym_get] = ACTIONS(3862),
    [anon_sym_set] = ACTIONS(3862),
  },
  [2556] = {
    [sym__automatic_semicolon] = ACTIONS(1588),
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2557] = {
    [sym__automatic_semicolon] = ACTIONS(3934),
    [anon_sym_export] = ACTIONS(3936),
    [anon_sym_default] = ACTIONS(3936),
    [anon_sym_LBRACE] = ACTIONS(3934),
    [anon_sym_RBRACE] = ACTIONS(3934),
    [anon_sym_import] = ACTIONS(3936),
    [anon_sym_var] = ACTIONS(3936),
    [anon_sym_let] = ACTIONS(3936),
    [anon_sym_const] = ACTIONS(3936),
    [anon_sym_if] = ACTIONS(3936),
    [anon_sym_else] = ACTIONS(3936),
    [anon_sym_switch] = ACTIONS(3936),
    [anon_sym_for] = ACTIONS(3936),
    [anon_sym_LPAREN] = ACTIONS(3934),
    [anon_sym_while] = ACTIONS(3936),
    [anon_sym_do] = ACTIONS(3936),
    [anon_sym_try] = ACTIONS(3936),
    [anon_sym_with] = ACTIONS(3936),
    [anon_sym_break] = ACTIONS(3936),
    [anon_sym_continue] = ACTIONS(3936),
    [anon_sym_debugger] = ACTIONS(3936),
    [anon_sym_return] = ACTIONS(3936),
    [anon_sym_throw] = ACTIONS(3936),
    [anon_sym_SEMI] = ACTIONS(3934),
    [anon_sym_case] = ACTIONS(3936),
    [anon_sym_yield] = ACTIONS(3936),
    [anon_sym_LBRACK] = ACTIONS(3934),
    [anon_sym_LT] = ACTIONS(3934),
    [anon_sym_class] = ACTIONS(3936),
    [anon_sym_async] = ACTIONS(3936),
    [anon_sym_function] = ACTIONS(3936),
    [anon_sym_EQ_GT] = ACTIONS(3934),
    [anon_sym_new] = ACTIONS(3936),
    [anon_sym_await] = ACTIONS(3936),
    [anon_sym_PLUS] = ACTIONS(3936),
    [anon_sym_DASH] = ACTIONS(3936),
    [anon_sym_BANG] = ACTIONS(3934),
    [anon_sym_TILDE] = ACTIONS(3934),
    [anon_sym_typeof] = ACTIONS(3936),
    [anon_sym_void] = ACTIONS(3936),
    [anon_sym_delete] = ACTIONS(3936),
    [anon_sym_PLUS_PLUS] = ACTIONS(3934),
    [anon_sym_DASH_DASH] = ACTIONS(3934),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(3934),
    [anon_sym_BQUOTE] = ACTIONS(3934),
    [sym_regex] = ACTIONS(3936),
    [sym_number] = ACTIONS(3934),
    [sym_identifier] = ACTIONS(3936),
    [sym_this] = ACTIONS(3936),
    [sym_super] = ACTIONS(3936),
    [sym_true] = ACTIONS(3936),
    [sym_false] = ACTIONS(3936),
    [sym_null] = ACTIONS(3936),
    [sym_undefined] = ACTIONS(3936),
    [anon_sym_AT] = ACTIONS(3934),
    [anon_sym_get] = ACTIONS(3936),
    [anon_sym_set] = ACTIONS(3936),
  },
  [2558] = {
    [anon_sym_RPAREN] = ACTIONS(5272),
    [sym_comment] = ACTIONS(71),
  },
  [2559] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5274),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2560] = {
    [anon_sym_RPAREN] = ACTIONS(5276),
    [sym_comment] = ACTIONS(71),
  },
  [2561] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5278),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2562] = {
    [anon_sym_RPAREN] = ACTIONS(5280),
    [sym_comment] = ACTIONS(71),
  },
  [2563] = {
    [sym_template_string] = STATE(451),
    [sym_arguments] = STATE(451),
    [anon_sym_STAR] = ACTIONS(3666),
    [anon_sym_LPAREN] = ACTIONS(770),
    [anon_sym_RPAREN] = ACTIONS(5282),
    [anon_sym_in] = ACTIONS(3662),
    [anon_sym_LBRACK] = ACTIONS(772),
    [anon_sym_LT] = ACTIONS(3662),
    [anon_sym_GT] = ACTIONS(3662),
    [anon_sym_SLASH] = ACTIONS(3670),
    [anon_sym_DOT] = ACTIONS(766),
    [anon_sym_QMARK] = ACTIONS(3672),
    [anon_sym_AMP_AMP] = ACTIONS(3674),
    [anon_sym_PIPE_PIPE] = ACTIONS(3676),
    [anon_sym_GT_GT] = ACTIONS(3670),
    [anon_sym_GT_GT_GT] = ACTIONS(3666),
    [anon_sym_LT_LT] = ACTIONS(3666),
    [anon_sym_AMP] = ACTIONS(3678),
    [anon_sym_CARET] = ACTIONS(3676),
    [anon_sym_PIPE] = ACTIONS(3680),
    [anon_sym_PLUS] = ACTIONS(3682),
    [anon_sym_DASH] = ACTIONS(3682),
    [anon_sym_PERCENT] = ACTIONS(3666),
    [anon_sym_LT_EQ] = ACTIONS(3684),
    [anon_sym_EQ_EQ] = ACTIONS(3662),
    [anon_sym_EQ_EQ_EQ] = ACTIONS(3684),
    [anon_sym_BANG_EQ] = ACTIONS(3662),
    [anon_sym_BANG_EQ_EQ] = ACTIONS(3684),
    [anon_sym_GT_EQ] = ACTIONS(3684),
    [anon_sym_instanceof] = ACTIONS(3684),
    [anon_sym_PLUS_PLUS] = ACTIONS(796),
    [anon_sym_DASH_DASH] = ACTIONS(796),
    [sym_comment] = ACTIONS(71),
    [anon_sym_BQUOTE] = ACTIONS(185),
  },
  [2564] = {
    [sym_export_statement] = STATE(2586),
    [sym__declaration] = STATE(2586),
    [sym_import_statement] = STATE(2586),
    [sym_expression_statement] = STATE(2586),
    [sym_variable_declaration] = STATE(2586),
    [sym_lexical_declaration] = STATE(2586),
    [sym_statement_block] = STATE(2586),
    [sym_if_statement] = STATE(2586),
    [sym_switch_statement] = STATE(2586),
    [sym_for_statement] = STATE(2586),
    [sym_for_in_statement] = STATE(2586),
    [sym_for_of_statement] = STATE(2586),
    [sym_while_statement] = STATE(2586),
    [sym_do_statement] = STATE(2586),
    [sym_try_statement] = STATE(2586),
    [sym_with_statement] = STATE(2586),
    [sym_break_statement] = STATE(2586),
    [sym_continue_statement] = STATE(2586),
    [sym_debugger_statement] = STATE(2586),
    [sym_return_statement] = STATE(2586),
    [sym_throw_statement] = STATE(2586),
    [sym_empty_statement] = STATE(2586),
    [sym_labeled_statement] = STATE(2586),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2565] = {
    [sym_export_statement] = STATE(2587),
    [sym__declaration] = STATE(2587),
    [sym_import_statement] = STATE(2587),
    [sym_expression_statement] = STATE(2587),
    [sym_variable_declaration] = STATE(2587),
    [sym_lexical_declaration] = STATE(2587),
    [sym_statement_block] = STATE(2587),
    [sym_if_statement] = STATE(2587),
    [sym_switch_statement] = STATE(2587),
    [sym_for_statement] = STATE(2587),
    [sym_for_in_statement] = STATE(2587),
    [sym_for_of_statement] = STATE(2587),
    [sym_while_statement] = STATE(2587),
    [sym_do_statement] = STATE(2587),
    [sym_try_statement] = STATE(2587),
    [sym_with_statement] = STATE(2587),
    [sym_break_statement] = STATE(2587),
    [sym_continue_statement] = STATE(2587),
    [sym_debugger_statement] = STATE(2587),
    [sym_return_statement] = STATE(2587),
    [sym_throw_statement] = STATE(2587),
    [sym_empty_statement] = STATE(2587),
    [sym_labeled_statement] = STATE(2587),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2566] = {
    [anon_sym_export] = ACTIONS(4266),
    [anon_sym_default] = ACTIONS(4266),
    [anon_sym_LBRACE] = ACTIONS(4264),
    [anon_sym_RBRACE] = ACTIONS(4264),
    [anon_sym_import] = ACTIONS(4266),
    [anon_sym_var] = ACTIONS(4266),
    [anon_sym_let] = ACTIONS(4266),
    [anon_sym_const] = ACTIONS(4266),
    [anon_sym_if] = ACTIONS(4266),
    [anon_sym_else] = ACTIONS(4266),
    [anon_sym_switch] = ACTIONS(4266),
    [anon_sym_for] = ACTIONS(4266),
    [anon_sym_LPAREN] = ACTIONS(4264),
    [anon_sym_while] = ACTIONS(4266),
    [anon_sym_do] = ACTIONS(4266),
    [anon_sym_try] = ACTIONS(4266),
    [anon_sym_with] = ACTIONS(4266),
    [anon_sym_break] = ACTIONS(4266),
    [anon_sym_continue] = ACTIONS(4266),
    [anon_sym_debugger] = ACTIONS(4266),
    [anon_sym_return] = ACTIONS(4266),
    [anon_sym_throw] = ACTIONS(4266),
    [anon_sym_SEMI] = ACTIONS(4264),
    [anon_sym_case] = ACTIONS(4266),
    [anon_sym_yield] = ACTIONS(4266),
    [anon_sym_LBRACK] = ACTIONS(4264),
    [anon_sym_LT] = ACTIONS(4264),
    [anon_sym_class] = ACTIONS(4266),
    [anon_sym_async] = ACTIONS(4266),
    [anon_sym_function] = ACTIONS(4266),
    [anon_sym_EQ_GT] = ACTIONS(4264),
    [anon_sym_new] = ACTIONS(4266),
    [anon_sym_await] = ACTIONS(4266),
    [anon_sym_PLUS] = ACTIONS(4266),
    [anon_sym_DASH] = ACTIONS(4266),
    [anon_sym_BANG] = ACTIONS(4264),
    [anon_sym_TILDE] = ACTIONS(4264),
    [anon_sym_typeof] = ACTIONS(4266),
    [anon_sym_void] = ACTIONS(4266),
    [anon_sym_delete] = ACTIONS(4266),
    [anon_sym_PLUS_PLUS] = ACTIONS(4264),
    [anon_sym_DASH_DASH] = ACTIONS(4264),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4264),
    [anon_sym_BQUOTE] = ACTIONS(4264),
    [sym_regex] = ACTIONS(4266),
    [sym_number] = ACTIONS(4264),
    [sym_identifier] = ACTIONS(4266),
    [sym_this] = ACTIONS(4266),
    [sym_super] = ACTIONS(4266),
    [sym_true] = ACTIONS(4266),
    [sym_false] = ACTIONS(4266),
    [sym_null] = ACTIONS(4266),
    [sym_undefined] = ACTIONS(4266),
    [anon_sym_AT] = ACTIONS(4264),
    [anon_sym_get] = ACTIONS(4266),
    [anon_sym_set] = ACTIONS(4266),
  },
  [2567] = {
    [sym_export_statement] = STATE(2588),
    [sym__declaration] = STATE(2588),
    [sym_import_statement] = STATE(2588),
    [sym_expression_statement] = STATE(2588),
    [sym_variable_declaration] = STATE(2588),
    [sym_lexical_declaration] = STATE(2588),
    [sym_statement_block] = STATE(2588),
    [sym_if_statement] = STATE(2588),
    [sym_switch_statement] = STATE(2588),
    [sym_for_statement] = STATE(2588),
    [sym_for_in_statement] = STATE(2588),
    [sym_for_of_statement] = STATE(2588),
    [sym_while_statement] = STATE(2588),
    [sym_do_statement] = STATE(2588),
    [sym_try_statement] = STATE(2588),
    [sym_with_statement] = STATE(2588),
    [sym_break_statement] = STATE(2588),
    [sym_continue_statement] = STATE(2588),
    [sym_debugger_statement] = STATE(2588),
    [sym_return_statement] = STATE(2588),
    [sym_throw_statement] = STATE(2588),
    [sym_empty_statement] = STATE(2588),
    [sym_labeled_statement] = STATE(2588),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2568] = {
    [sym_export_statement] = STATE(2589),
    [sym__declaration] = STATE(2589),
    [sym_import_statement] = STATE(2589),
    [sym_expression_statement] = STATE(2589),
    [sym_variable_declaration] = STATE(2589),
    [sym_lexical_declaration] = STATE(2589),
    [sym_statement_block] = STATE(2589),
    [sym_if_statement] = STATE(2589),
    [sym_switch_statement] = STATE(2589),
    [sym_for_statement] = STATE(2589),
    [sym_for_in_statement] = STATE(2589),
    [sym_for_of_statement] = STATE(2589),
    [sym_while_statement] = STATE(2589),
    [sym_do_statement] = STATE(2589),
    [sym_try_statement] = STATE(2589),
    [sym_with_statement] = STATE(2589),
    [sym_break_statement] = STATE(2589),
    [sym_continue_statement] = STATE(2589),
    [sym_debugger_statement] = STATE(2589),
    [sym_return_statement] = STATE(2589),
    [sym_throw_statement] = STATE(2589),
    [sym_empty_statement] = STATE(2589),
    [sym_labeled_statement] = STATE(2589),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2569] = {
    [sym_export_statement] = STATE(2590),
    [sym__declaration] = STATE(2590),
    [sym_import_statement] = STATE(2590),
    [sym_expression_statement] = STATE(2590),
    [sym_variable_declaration] = STATE(2590),
    [sym_lexical_declaration] = STATE(2590),
    [sym_statement_block] = STATE(2590),
    [sym_if_statement] = STATE(2590),
    [sym_switch_statement] = STATE(2590),
    [sym_for_statement] = STATE(2590),
    [sym_for_in_statement] = STATE(2590),
    [sym_for_of_statement] = STATE(2590),
    [sym_while_statement] = STATE(2590),
    [sym_do_statement] = STATE(2590),
    [sym_try_statement] = STATE(2590),
    [sym_with_statement] = STATE(2590),
    [sym_break_statement] = STATE(2590),
    [sym_continue_statement] = STATE(2590),
    [sym_debugger_statement] = STATE(2590),
    [sym_return_statement] = STATE(2590),
    [sym_throw_statement] = STATE(2590),
    [sym_empty_statement] = STATE(2590),
    [sym_labeled_statement] = STATE(2590),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2570] = {
    [sym_export_statement] = STATE(2591),
    [sym__declaration] = STATE(2591),
    [sym_import_statement] = STATE(2591),
    [sym_expression_statement] = STATE(2591),
    [sym_variable_declaration] = STATE(2591),
    [sym_lexical_declaration] = STATE(2591),
    [sym_statement_block] = STATE(2591),
    [sym_if_statement] = STATE(2591),
    [sym_switch_statement] = STATE(2591),
    [sym_for_statement] = STATE(2591),
    [sym_for_in_statement] = STATE(2591),
    [sym_for_of_statement] = STATE(2591),
    [sym_while_statement] = STATE(2591),
    [sym_do_statement] = STATE(2591),
    [sym_try_statement] = STATE(2591),
    [sym_with_statement] = STATE(2591),
    [sym_break_statement] = STATE(2591),
    [sym_continue_statement] = STATE(2591),
    [sym_debugger_statement] = STATE(2591),
    [sym_return_statement] = STATE(2591),
    [sym_throw_statement] = STATE(2591),
    [sym_empty_statement] = STATE(2591),
    [sym_labeled_statement] = STATE(2591),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2571] = {
    [sym_export_statement] = STATE(2592),
    [sym__declaration] = STATE(2592),
    [sym_import_statement] = STATE(2592),
    [sym_expression_statement] = STATE(2592),
    [sym_variable_declaration] = STATE(2592),
    [sym_lexical_declaration] = STATE(2592),
    [sym_statement_block] = STATE(2592),
    [sym_if_statement] = STATE(2592),
    [sym_switch_statement] = STATE(2592),
    [sym_for_statement] = STATE(2592),
    [sym_for_in_statement] = STATE(2592),
    [sym_for_of_statement] = STATE(2592),
    [sym_while_statement] = STATE(2592),
    [sym_do_statement] = STATE(2592),
    [sym_try_statement] = STATE(2592),
    [sym_with_statement] = STATE(2592),
    [sym_break_statement] = STATE(2592),
    [sym_continue_statement] = STATE(2592),
    [sym_debugger_statement] = STATE(2592),
    [sym_return_statement] = STATE(2592),
    [sym_throw_statement] = STATE(2592),
    [sym_empty_statement] = STATE(2592),
    [sym_labeled_statement] = STATE(2592),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2572] = {
    [anon_sym_export] = ACTIONS(1590),
    [anon_sym_default] = ACTIONS(1590),
    [anon_sym_LBRACE] = ACTIONS(1588),
    [anon_sym_RBRACE] = ACTIONS(1588),
    [anon_sym_import] = ACTIONS(1590),
    [anon_sym_var] = ACTIONS(1590),
    [anon_sym_let] = ACTIONS(1590),
    [anon_sym_const] = ACTIONS(1590),
    [anon_sym_if] = ACTIONS(1590),
    [anon_sym_else] = ACTIONS(1590),
    [anon_sym_switch] = ACTIONS(1590),
    [anon_sym_for] = ACTIONS(1590),
    [anon_sym_LPAREN] = ACTIONS(1588),
    [anon_sym_while] = ACTIONS(1590),
    [anon_sym_do] = ACTIONS(1590),
    [anon_sym_try] = ACTIONS(1590),
    [anon_sym_with] = ACTIONS(1590),
    [anon_sym_break] = ACTIONS(1590),
    [anon_sym_continue] = ACTIONS(1590),
    [anon_sym_debugger] = ACTIONS(1590),
    [anon_sym_return] = ACTIONS(1590),
    [anon_sym_throw] = ACTIONS(1590),
    [anon_sym_SEMI] = ACTIONS(1588),
    [anon_sym_case] = ACTIONS(1590),
    [anon_sym_finally] = ACTIONS(1590),
    [anon_sym_yield] = ACTIONS(1590),
    [anon_sym_LBRACK] = ACTIONS(1588),
    [anon_sym_LT] = ACTIONS(1588),
    [anon_sym_class] = ACTIONS(1590),
    [anon_sym_async] = ACTIONS(1590),
    [anon_sym_function] = ACTIONS(1590),
    [anon_sym_EQ_GT] = ACTIONS(1588),
    [anon_sym_new] = ACTIONS(1590),
    [anon_sym_await] = ACTIONS(1590),
    [anon_sym_PLUS] = ACTIONS(1590),
    [anon_sym_DASH] = ACTIONS(1590),
    [anon_sym_BANG] = ACTIONS(1588),
    [anon_sym_TILDE] = ACTIONS(1588),
    [anon_sym_typeof] = ACTIONS(1590),
    [anon_sym_void] = ACTIONS(1590),
    [anon_sym_delete] = ACTIONS(1590),
    [anon_sym_PLUS_PLUS] = ACTIONS(1588),
    [anon_sym_DASH_DASH] = ACTIONS(1588),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(1588),
    [anon_sym_BQUOTE] = ACTIONS(1588),
    [sym_regex] = ACTIONS(1590),
    [sym_number] = ACTIONS(1588),
    [sym_identifier] = ACTIONS(1590),
    [sym_this] = ACTIONS(1590),
    [sym_super] = ACTIONS(1590),
    [sym_true] = ACTIONS(1590),
    [sym_false] = ACTIONS(1590),
    [sym_null] = ACTIONS(1590),
    [sym_undefined] = ACTIONS(1590),
    [anon_sym_AT] = ACTIONS(1588),
    [anon_sym_get] = ACTIONS(1590),
    [anon_sym_set] = ACTIONS(1590),
  },
  [2573] = {
    [sym_statement_block] = STATE(2593),
    [anon_sym_LBRACE] = ACTIONS(5136),
    [sym_comment] = ACTIONS(71),
  },
  [2574] = {
    [anon_sym_export] = ACTIONS(4892),
    [anon_sym_default] = ACTIONS(4892),
    [anon_sym_LBRACE] = ACTIONS(4890),
    [anon_sym_RBRACE] = ACTIONS(4890),
    [anon_sym_import] = ACTIONS(4892),
    [anon_sym_var] = ACTIONS(4892),
    [anon_sym_let] = ACTIONS(4892),
    [anon_sym_const] = ACTIONS(4892),
    [anon_sym_if] = ACTIONS(4892),
    [anon_sym_switch] = ACTIONS(4892),
    [anon_sym_for] = ACTIONS(4892),
    [anon_sym_LPAREN] = ACTIONS(4890),
    [anon_sym_while] = ACTIONS(4892),
    [anon_sym_do] = ACTIONS(4892),
    [anon_sym_try] = ACTIONS(4892),
    [anon_sym_with] = ACTIONS(4892),
    [anon_sym_break] = ACTIONS(4892),
    [anon_sym_continue] = ACTIONS(4892),
    [anon_sym_debugger] = ACTIONS(4892),
    [anon_sym_return] = ACTIONS(4892),
    [anon_sym_throw] = ACTIONS(4892),
    [anon_sym_SEMI] = ACTIONS(4890),
    [anon_sym_case] = ACTIONS(4892),
    [anon_sym_yield] = ACTIONS(4892),
    [anon_sym_LBRACK] = ACTIONS(4890),
    [anon_sym_LT] = ACTIONS(4890),
    [anon_sym_class] = ACTIONS(4892),
    [anon_sym_async] = ACTIONS(4892),
    [anon_sym_function] = ACTIONS(4892),
    [anon_sym_EQ_GT] = ACTIONS(4890),
    [anon_sym_new] = ACTIONS(4892),
    [anon_sym_await] = ACTIONS(4892),
    [anon_sym_PLUS] = ACTIONS(4892),
    [anon_sym_DASH] = ACTIONS(4892),
    [anon_sym_BANG] = ACTIONS(4890),
    [anon_sym_TILDE] = ACTIONS(4890),
    [anon_sym_typeof] = ACTIONS(4892),
    [anon_sym_void] = ACTIONS(4892),
    [anon_sym_delete] = ACTIONS(4892),
    [anon_sym_PLUS_PLUS] = ACTIONS(4890),
    [anon_sym_DASH_DASH] = ACTIONS(4890),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4890),
    [anon_sym_BQUOTE] = ACTIONS(4890),
    [sym_regex] = ACTIONS(4892),
    [sym_number] = ACTIONS(4890),
    [sym_identifier] = ACTIONS(4892),
    [sym_this] = ACTIONS(4892),
    [sym_super] = ACTIONS(4892),
    [sym_true] = ACTIONS(4892),
    [sym_false] = ACTIONS(4892),
    [sym_null] = ACTIONS(4892),
    [sym_undefined] = ACTIONS(4892),
    [anon_sym_AT] = ACTIONS(4890),
    [anon_sym_get] = ACTIONS(4892),
    [anon_sym_set] = ACTIONS(4892),
  },
  [2575] = {
    [anon_sym_export] = ACTIONS(4896),
    [anon_sym_default] = ACTIONS(4896),
    [anon_sym_LBRACE] = ACTIONS(4894),
    [anon_sym_RBRACE] = ACTIONS(4894),
    [anon_sym_import] = ACTIONS(4896),
    [anon_sym_var] = ACTIONS(4896),
    [anon_sym_let] = ACTIONS(4896),
    [anon_sym_const] = ACTIONS(4896),
    [anon_sym_if] = ACTIONS(4896),
    [anon_sym_switch] = ACTIONS(4896),
    [anon_sym_for] = ACTIONS(4896),
    [anon_sym_LPAREN] = ACTIONS(4894),
    [anon_sym_while] = ACTIONS(4896),
    [anon_sym_do] = ACTIONS(4896),
    [anon_sym_try] = ACTIONS(4896),
    [anon_sym_with] = ACTIONS(4896),
    [anon_sym_break] = ACTIONS(4896),
    [anon_sym_continue] = ACTIONS(4896),
    [anon_sym_debugger] = ACTIONS(4896),
    [anon_sym_return] = ACTIONS(4896),
    [anon_sym_throw] = ACTIONS(4896),
    [anon_sym_SEMI] = ACTIONS(4894),
    [anon_sym_case] = ACTIONS(4896),
    [anon_sym_yield] = ACTIONS(4896),
    [anon_sym_LBRACK] = ACTIONS(4894),
    [anon_sym_LT] = ACTIONS(4894),
    [anon_sym_class] = ACTIONS(4896),
    [anon_sym_async] = ACTIONS(4896),
    [anon_sym_function] = ACTIONS(4896),
    [anon_sym_EQ_GT] = ACTIONS(4894),
    [anon_sym_new] = ACTIONS(4896),
    [anon_sym_await] = ACTIONS(4896),
    [anon_sym_PLUS] = ACTIONS(4896),
    [anon_sym_DASH] = ACTIONS(4896),
    [anon_sym_BANG] = ACTIONS(4894),
    [anon_sym_TILDE] = ACTIONS(4894),
    [anon_sym_typeof] = ACTIONS(4896),
    [anon_sym_void] = ACTIONS(4896),
    [anon_sym_delete] = ACTIONS(4896),
    [anon_sym_PLUS_PLUS] = ACTIONS(4894),
    [anon_sym_DASH_DASH] = ACTIONS(4894),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4894),
    [anon_sym_BQUOTE] = ACTIONS(4894),
    [sym_regex] = ACTIONS(4896),
    [sym_number] = ACTIONS(4894),
    [sym_identifier] = ACTIONS(4896),
    [sym_this] = ACTIONS(4896),
    [sym_super] = ACTIONS(4896),
    [sym_true] = ACTIONS(4896),
    [sym_false] = ACTIONS(4896),
    [sym_null] = ACTIONS(4896),
    [sym_undefined] = ACTIONS(4896),
    [anon_sym_AT] = ACTIONS(4894),
    [anon_sym_get] = ACTIONS(4896),
    [anon_sym_set] = ACTIONS(4896),
  },
  [2576] = {
    [anon_sym_export] = ACTIONS(4900),
    [anon_sym_default] = ACTIONS(4900),
    [anon_sym_LBRACE] = ACTIONS(4898),
    [anon_sym_RBRACE] = ACTIONS(4898),
    [anon_sym_import] = ACTIONS(4900),
    [anon_sym_var] = ACTIONS(4900),
    [anon_sym_let] = ACTIONS(4900),
    [anon_sym_const] = ACTIONS(4900),
    [anon_sym_if] = ACTIONS(4900),
    [anon_sym_switch] = ACTIONS(4900),
    [anon_sym_for] = ACTIONS(4900),
    [anon_sym_LPAREN] = ACTIONS(4898),
    [anon_sym_while] = ACTIONS(4900),
    [anon_sym_do] = ACTIONS(4900),
    [anon_sym_try] = ACTIONS(4900),
    [anon_sym_with] = ACTIONS(4900),
    [anon_sym_break] = ACTIONS(4900),
    [anon_sym_continue] = ACTIONS(4900),
    [anon_sym_debugger] = ACTIONS(4900),
    [anon_sym_return] = ACTIONS(4900),
    [anon_sym_throw] = ACTIONS(4900),
    [anon_sym_SEMI] = ACTIONS(4898),
    [anon_sym_case] = ACTIONS(4900),
    [anon_sym_yield] = ACTIONS(4900),
    [anon_sym_LBRACK] = ACTIONS(4898),
    [anon_sym_LT] = ACTIONS(4898),
    [anon_sym_class] = ACTIONS(4900),
    [anon_sym_async] = ACTIONS(4900),
    [anon_sym_function] = ACTIONS(4900),
    [anon_sym_EQ_GT] = ACTIONS(4898),
    [anon_sym_new] = ACTIONS(4900),
    [anon_sym_await] = ACTIONS(4900),
    [anon_sym_PLUS] = ACTIONS(4900),
    [anon_sym_DASH] = ACTIONS(4900),
    [anon_sym_BANG] = ACTIONS(4898),
    [anon_sym_TILDE] = ACTIONS(4898),
    [anon_sym_typeof] = ACTIONS(4900),
    [anon_sym_void] = ACTIONS(4900),
    [anon_sym_delete] = ACTIONS(4900),
    [anon_sym_PLUS_PLUS] = ACTIONS(4898),
    [anon_sym_DASH_DASH] = ACTIONS(4898),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4898),
    [anon_sym_BQUOTE] = ACTIONS(4898),
    [sym_regex] = ACTIONS(4900),
    [sym_number] = ACTIONS(4898),
    [sym_identifier] = ACTIONS(4900),
    [sym_this] = ACTIONS(4900),
    [sym_super] = ACTIONS(4900),
    [sym_true] = ACTIONS(4900),
    [sym_false] = ACTIONS(4900),
    [sym_null] = ACTIONS(4900),
    [sym_undefined] = ACTIONS(4900),
    [anon_sym_AT] = ACTIONS(4898),
    [anon_sym_get] = ACTIONS(4900),
    [anon_sym_set] = ACTIONS(4900),
  },
  [2577] = {
    [anon_sym_export] = ACTIONS(4904),
    [anon_sym_default] = ACTIONS(4904),
    [anon_sym_LBRACE] = ACTIONS(4902),
    [anon_sym_RBRACE] = ACTIONS(4902),
    [anon_sym_import] = ACTIONS(4904),
    [anon_sym_var] = ACTIONS(4904),
    [anon_sym_let] = ACTIONS(4904),
    [anon_sym_const] = ACTIONS(4904),
    [anon_sym_if] = ACTIONS(4904),
    [anon_sym_switch] = ACTIONS(4904),
    [anon_sym_for] = ACTIONS(4904),
    [anon_sym_LPAREN] = ACTIONS(4902),
    [anon_sym_while] = ACTIONS(4904),
    [anon_sym_do] = ACTIONS(4904),
    [anon_sym_try] = ACTIONS(4904),
    [anon_sym_with] = ACTIONS(4904),
    [anon_sym_break] = ACTIONS(4904),
    [anon_sym_continue] = ACTIONS(4904),
    [anon_sym_debugger] = ACTIONS(4904),
    [anon_sym_return] = ACTIONS(4904),
    [anon_sym_throw] = ACTIONS(4904),
    [anon_sym_SEMI] = ACTIONS(4902),
    [anon_sym_case] = ACTIONS(4904),
    [anon_sym_yield] = ACTIONS(4904),
    [anon_sym_LBRACK] = ACTIONS(4902),
    [anon_sym_LT] = ACTIONS(4902),
    [anon_sym_class] = ACTIONS(4904),
    [anon_sym_async] = ACTIONS(4904),
    [anon_sym_function] = ACTIONS(4904),
    [anon_sym_EQ_GT] = ACTIONS(4902),
    [anon_sym_new] = ACTIONS(4904),
    [anon_sym_await] = ACTIONS(4904),
    [anon_sym_PLUS] = ACTIONS(4904),
    [anon_sym_DASH] = ACTIONS(4904),
    [anon_sym_BANG] = ACTIONS(4902),
    [anon_sym_TILDE] = ACTIONS(4902),
    [anon_sym_typeof] = ACTIONS(4904),
    [anon_sym_void] = ACTIONS(4904),
    [anon_sym_delete] = ACTIONS(4904),
    [anon_sym_PLUS_PLUS] = ACTIONS(4902),
    [anon_sym_DASH_DASH] = ACTIONS(4902),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4902),
    [anon_sym_BQUOTE] = ACTIONS(4902),
    [sym_regex] = ACTIONS(4904),
    [sym_number] = ACTIONS(4902),
    [sym_identifier] = ACTIONS(4904),
    [sym_this] = ACTIONS(4904),
    [sym_super] = ACTIONS(4904),
    [sym_true] = ACTIONS(4904),
    [sym_false] = ACTIONS(4904),
    [sym_null] = ACTIONS(4904),
    [sym_undefined] = ACTIONS(4904),
    [anon_sym_AT] = ACTIONS(4902),
    [anon_sym_get] = ACTIONS(4904),
    [anon_sym_set] = ACTIONS(4904),
  },
  [2578] = {
    [anon_sym_export] = ACTIONS(4908),
    [anon_sym_default] = ACTIONS(4908),
    [anon_sym_LBRACE] = ACTIONS(4906),
    [anon_sym_RBRACE] = ACTIONS(4906),
    [anon_sym_import] = ACTIONS(4908),
    [anon_sym_var] = ACTIONS(4908),
    [anon_sym_let] = ACTIONS(4908),
    [anon_sym_const] = ACTIONS(4908),
    [anon_sym_if] = ACTIONS(4908),
    [anon_sym_switch] = ACTIONS(4908),
    [anon_sym_for] = ACTIONS(4908),
    [anon_sym_LPAREN] = ACTIONS(4906),
    [anon_sym_while] = ACTIONS(4908),
    [anon_sym_do] = ACTIONS(4908),
    [anon_sym_try] = ACTIONS(4908),
    [anon_sym_with] = ACTIONS(4908),
    [anon_sym_break] = ACTIONS(4908),
    [anon_sym_continue] = ACTIONS(4908),
    [anon_sym_debugger] = ACTIONS(4908),
    [anon_sym_return] = ACTIONS(4908),
    [anon_sym_throw] = ACTIONS(4908),
    [anon_sym_SEMI] = ACTIONS(4906),
    [anon_sym_case] = ACTIONS(4908),
    [anon_sym_yield] = ACTIONS(4908),
    [anon_sym_LBRACK] = ACTIONS(4906),
    [anon_sym_LT] = ACTIONS(4906),
    [anon_sym_class] = ACTIONS(4908),
    [anon_sym_async] = ACTIONS(4908),
    [anon_sym_function] = ACTIONS(4908),
    [anon_sym_EQ_GT] = ACTIONS(4906),
    [anon_sym_new] = ACTIONS(4908),
    [anon_sym_await] = ACTIONS(4908),
    [anon_sym_PLUS] = ACTIONS(4908),
    [anon_sym_DASH] = ACTIONS(4908),
    [anon_sym_BANG] = ACTIONS(4906),
    [anon_sym_TILDE] = ACTIONS(4906),
    [anon_sym_typeof] = ACTIONS(4908),
    [anon_sym_void] = ACTIONS(4908),
    [anon_sym_delete] = ACTIONS(4908),
    [anon_sym_PLUS_PLUS] = ACTIONS(4906),
    [anon_sym_DASH_DASH] = ACTIONS(4906),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4906),
    [anon_sym_BQUOTE] = ACTIONS(4906),
    [sym_regex] = ACTIONS(4908),
    [sym_number] = ACTIONS(4906),
    [sym_identifier] = ACTIONS(4908),
    [sym_this] = ACTIONS(4908),
    [sym_super] = ACTIONS(4908),
    [sym_true] = ACTIONS(4908),
    [sym_false] = ACTIONS(4908),
    [sym_null] = ACTIONS(4908),
    [sym_undefined] = ACTIONS(4908),
    [anon_sym_AT] = ACTIONS(4906),
    [anon_sym_get] = ACTIONS(4908),
    [anon_sym_set] = ACTIONS(4908),
  },
  [2579] = {
    [anon_sym_export] = ACTIONS(4912),
    [anon_sym_default] = ACTIONS(4912),
    [anon_sym_LBRACE] = ACTIONS(4910),
    [anon_sym_RBRACE] = ACTIONS(4910),
    [anon_sym_import] = ACTIONS(4912),
    [anon_sym_var] = ACTIONS(4912),
    [anon_sym_let] = ACTIONS(4912),
    [anon_sym_const] = ACTIONS(4912),
    [anon_sym_if] = ACTIONS(4912),
    [anon_sym_switch] = ACTIONS(4912),
    [anon_sym_for] = ACTIONS(4912),
    [anon_sym_LPAREN] = ACTIONS(4910),
    [anon_sym_while] = ACTIONS(4912),
    [anon_sym_do] = ACTIONS(4912),
    [anon_sym_try] = ACTIONS(4912),
    [anon_sym_with] = ACTIONS(4912),
    [anon_sym_break] = ACTIONS(4912),
    [anon_sym_continue] = ACTIONS(4912),
    [anon_sym_debugger] = ACTIONS(4912),
    [anon_sym_return] = ACTIONS(4912),
    [anon_sym_throw] = ACTIONS(4912),
    [anon_sym_SEMI] = ACTIONS(4910),
    [anon_sym_case] = ACTIONS(4912),
    [anon_sym_yield] = ACTIONS(4912),
    [anon_sym_LBRACK] = ACTIONS(4910),
    [anon_sym_LT] = ACTIONS(4910),
    [anon_sym_class] = ACTIONS(4912),
    [anon_sym_async] = ACTIONS(4912),
    [anon_sym_function] = ACTIONS(4912),
    [anon_sym_EQ_GT] = ACTIONS(4910),
    [anon_sym_new] = ACTIONS(4912),
    [anon_sym_await] = ACTIONS(4912),
    [anon_sym_PLUS] = ACTIONS(4912),
    [anon_sym_DASH] = ACTIONS(4912),
    [anon_sym_BANG] = ACTIONS(4910),
    [anon_sym_TILDE] = ACTIONS(4910),
    [anon_sym_typeof] = ACTIONS(4912),
    [anon_sym_void] = ACTIONS(4912),
    [anon_sym_delete] = ACTIONS(4912),
    [anon_sym_PLUS_PLUS] = ACTIONS(4910),
    [anon_sym_DASH_DASH] = ACTIONS(4910),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4910),
    [anon_sym_BQUOTE] = ACTIONS(4910),
    [sym_regex] = ACTIONS(4912),
    [sym_number] = ACTIONS(4910),
    [sym_identifier] = ACTIONS(4912),
    [sym_this] = ACTIONS(4912),
    [sym_super] = ACTIONS(4912),
    [sym_true] = ACTIONS(4912),
    [sym_false] = ACTIONS(4912),
    [sym_null] = ACTIONS(4912),
    [sym_undefined] = ACTIONS(4912),
    [anon_sym_AT] = ACTIONS(4910),
    [anon_sym_get] = ACTIONS(4912),
    [anon_sym_set] = ACTIONS(4912),
  },
  [2580] = {
    [sym_export_statement] = STATE(2594),
    [sym__declaration] = STATE(2594),
    [sym_import_statement] = STATE(2594),
    [sym_expression_statement] = STATE(2594),
    [sym_variable_declaration] = STATE(2594),
    [sym_lexical_declaration] = STATE(2594),
    [sym_statement_block] = STATE(2594),
    [sym_if_statement] = STATE(2594),
    [sym_switch_statement] = STATE(2594),
    [sym_for_statement] = STATE(2594),
    [sym_for_in_statement] = STATE(2594),
    [sym_for_of_statement] = STATE(2594),
    [sym_while_statement] = STATE(2594),
    [sym_do_statement] = STATE(2594),
    [sym_try_statement] = STATE(2594),
    [sym_with_statement] = STATE(2594),
    [sym_break_statement] = STATE(2594),
    [sym_continue_statement] = STATE(2594),
    [sym_debugger_statement] = STATE(2594),
    [sym_return_statement] = STATE(2594),
    [sym_throw_statement] = STATE(2594),
    [sym_empty_statement] = STATE(2594),
    [sym_labeled_statement] = STATE(2594),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2581] = {
    [sym_export_statement] = STATE(2595),
    [sym__declaration] = STATE(2595),
    [sym_import_statement] = STATE(2595),
    [sym_expression_statement] = STATE(2595),
    [sym_variable_declaration] = STATE(2595),
    [sym_lexical_declaration] = STATE(2595),
    [sym_statement_block] = STATE(2595),
    [sym_if_statement] = STATE(2595),
    [sym_switch_statement] = STATE(2595),
    [sym_for_statement] = STATE(2595),
    [sym_for_in_statement] = STATE(2595),
    [sym_for_of_statement] = STATE(2595),
    [sym_while_statement] = STATE(2595),
    [sym_do_statement] = STATE(2595),
    [sym_try_statement] = STATE(2595),
    [sym_with_statement] = STATE(2595),
    [sym_break_statement] = STATE(2595),
    [sym_continue_statement] = STATE(2595),
    [sym_debugger_statement] = STATE(2595),
    [sym_return_statement] = STATE(2595),
    [sym_throw_statement] = STATE(2595),
    [sym_empty_statement] = STATE(2595),
    [sym_labeled_statement] = STATE(2595),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2582] = {
    [sym_export_statement] = STATE(2596),
    [sym__declaration] = STATE(2596),
    [sym_import_statement] = STATE(2596),
    [sym_expression_statement] = STATE(2596),
    [sym_variable_declaration] = STATE(2596),
    [sym_lexical_declaration] = STATE(2596),
    [sym_statement_block] = STATE(2596),
    [sym_if_statement] = STATE(2596),
    [sym_switch_statement] = STATE(2596),
    [sym_for_statement] = STATE(2596),
    [sym_for_in_statement] = STATE(2596),
    [sym_for_of_statement] = STATE(2596),
    [sym_while_statement] = STATE(2596),
    [sym_do_statement] = STATE(2596),
    [sym_try_statement] = STATE(2596),
    [sym_with_statement] = STATE(2596),
    [sym_break_statement] = STATE(2596),
    [sym_continue_statement] = STATE(2596),
    [sym_debugger_statement] = STATE(2596),
    [sym_return_statement] = STATE(2596),
    [sym_throw_statement] = STATE(2596),
    [sym_empty_statement] = STATE(2596),
    [sym_labeled_statement] = STATE(2596),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2583] = {
    [sym_export_statement] = STATE(2597),
    [sym__declaration] = STATE(2597),
    [sym_import_statement] = STATE(2597),
    [sym_expression_statement] = STATE(2597),
    [sym_variable_declaration] = STATE(2597),
    [sym_lexical_declaration] = STATE(2597),
    [sym_statement_block] = STATE(2597),
    [sym_if_statement] = STATE(2597),
    [sym_switch_statement] = STATE(2597),
    [sym_for_statement] = STATE(2597),
    [sym_for_in_statement] = STATE(2597),
    [sym_for_of_statement] = STATE(2597),
    [sym_while_statement] = STATE(2597),
    [sym_do_statement] = STATE(2597),
    [sym_try_statement] = STATE(2597),
    [sym_with_statement] = STATE(2597),
    [sym_break_statement] = STATE(2597),
    [sym_continue_statement] = STATE(2597),
    [sym_debugger_statement] = STATE(2597),
    [sym_return_statement] = STATE(2597),
    [sym_throw_statement] = STATE(2597),
    [sym_empty_statement] = STATE(2597),
    [sym_labeled_statement] = STATE(2597),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2584] = {
    [sym_export_statement] = STATE(2598),
    [sym__declaration] = STATE(2598),
    [sym_import_statement] = STATE(2598),
    [sym_expression_statement] = STATE(2598),
    [sym_variable_declaration] = STATE(2598),
    [sym_lexical_declaration] = STATE(2598),
    [sym_statement_block] = STATE(2598),
    [sym_if_statement] = STATE(2598),
    [sym_switch_statement] = STATE(2598),
    [sym_for_statement] = STATE(2598),
    [sym_for_in_statement] = STATE(2598),
    [sym_for_of_statement] = STATE(2598),
    [sym_while_statement] = STATE(2598),
    [sym_do_statement] = STATE(2598),
    [sym_try_statement] = STATE(2598),
    [sym_with_statement] = STATE(2598),
    [sym_break_statement] = STATE(2598),
    [sym_continue_statement] = STATE(2598),
    [sym_debugger_statement] = STATE(2598),
    [sym_return_statement] = STATE(2598),
    [sym_throw_statement] = STATE(2598),
    [sym_empty_statement] = STATE(2598),
    [sym_labeled_statement] = STATE(2598),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2585] = {
    [sym_export_statement] = STATE(2599),
    [sym__declaration] = STATE(2599),
    [sym_import_statement] = STATE(2599),
    [sym_expression_statement] = STATE(2599),
    [sym_variable_declaration] = STATE(2599),
    [sym_lexical_declaration] = STATE(2599),
    [sym_statement_block] = STATE(2599),
    [sym_if_statement] = STATE(2599),
    [sym_switch_statement] = STATE(2599),
    [sym_for_statement] = STATE(2599),
    [sym_for_in_statement] = STATE(2599),
    [sym_for_of_statement] = STATE(2599),
    [sym_while_statement] = STATE(2599),
    [sym_do_statement] = STATE(2599),
    [sym_try_statement] = STATE(2599),
    [sym_with_statement] = STATE(2599),
    [sym_break_statement] = STATE(2599),
    [sym_continue_statement] = STATE(2599),
    [sym_debugger_statement] = STATE(2599),
    [sym_return_statement] = STATE(2599),
    [sym_throw_statement] = STATE(2599),
    [sym_empty_statement] = STATE(2599),
    [sym_labeled_statement] = STATE(2599),
    [sym_parenthesized_expression] = STATE(40),
    [sym__expressions] = STATE(2091),
    [sym__expression] = STATE(40),
    [sym_yield_expression] = STATE(40),
    [sym_object] = STATE(41),
    [sym_array] = STATE(42),
    [sym_jsx_element] = STATE(40),
    [sym_jsx_fragment] = STATE(40),
    [sym_jsx_opening_element] = STATE(43),
    [sym_jsx_self_closing_element] = STATE(40),
    [sym_anonymous_class] = STATE(40),
    [sym_class] = STATE(2092),
    [sym_function] = STATE(2093),
    [sym_arrow_function] = STATE(40),
    [sym_generator_function] = STATE(2092),
    [sym_call_expression] = STATE(40),
    [sym_new_expression] = STATE(40),
    [sym_await_expression] = STATE(40),
    [sym_member_expression] = STATE(46),
    [sym_subscript_expression] = STATE(46),
    [sym_assignment_expression] = STATE(40),
    [sym_augmented_assignment_expression] = STATE(40),
    [sym_ternary_expression] = STATE(40),
    [sym_binary_expression] = STATE(40),
    [sym_unary_expression] = STATE(40),
    [sym_update_expression] = STATE(40),
    [sym_sequence_expression] = STATE(2091),
    [sym_template_string] = STATE(40),
    [sym_decorator] = STATE(2094),
    [sym_formal_parameters] = STATE(47),
    [aux_sym_export_statement_repeat1] = STATE(2094),
    [anon_sym_export] = ACTIONS(4474),
    [anon_sym_LBRACE] = ACTIONS(4476),
    [anon_sym_import] = ACTIONS(4478),
    [anon_sym_var] = ACTIONS(4480),
    [anon_sym_let] = ACTIONS(4482),
    [anon_sym_const] = ACTIONS(4482),
    [anon_sym_if] = ACTIONS(4484),
    [anon_sym_switch] = ACTIONS(4486),
    [anon_sym_for] = ACTIONS(4488),
    [anon_sym_LPAREN] = ACTIONS(25),
    [anon_sym_while] = ACTIONS(4490),
    [anon_sym_do] = ACTIONS(4492),
    [anon_sym_try] = ACTIONS(4494),
    [anon_sym_with] = ACTIONS(4496),
    [anon_sym_break] = ACTIONS(4498),
    [anon_sym_continue] = ACTIONS(4500),
    [anon_sym_debugger] = ACTIONS(4502),
    [anon_sym_return] = ACTIONS(4504),
    [anon_sym_throw] = ACTIONS(4506),
    [anon_sym_SEMI] = ACTIONS(4508),
    [anon_sym_yield] = ACTIONS(47),
    [anon_sym_LBRACK] = ACTIONS(49),
    [anon_sym_LT] = ACTIONS(51),
    [anon_sym_class] = ACTIONS(4510),
    [anon_sym_async] = ACTIONS(4512),
    [anon_sym_function] = ACTIONS(4514),
    [anon_sym_EQ_GT] = ACTIONS(59),
    [anon_sym_new] = ACTIONS(61),
    [anon_sym_await] = ACTIONS(63),
    [anon_sym_PLUS] = ACTIONS(65),
    [anon_sym_DASH] = ACTIONS(65),
    [anon_sym_BANG] = ACTIONS(67),
    [anon_sym_TILDE] = ACTIONS(67),
    [anon_sym_typeof] = ACTIONS(65),
    [anon_sym_void] = ACTIONS(65),
    [anon_sym_delete] = ACTIONS(65),
    [anon_sym_PLUS_PLUS] = ACTIONS(69),
    [anon_sym_DASH_DASH] = ACTIONS(69),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(73),
    [anon_sym_BQUOTE] = ACTIONS(75),
    [sym_regex] = ACTIONS(77),
    [sym_number] = ACTIONS(73),
    [sym_identifier] = ACTIONS(4516),
    [sym_this] = ACTIONS(77),
    [sym_super] = ACTIONS(81),
    [sym_true] = ACTIONS(77),
    [sym_false] = ACTIONS(77),
    [sym_null] = ACTIONS(77),
    [sym_undefined] = ACTIONS(77),
    [anon_sym_AT] = ACTIONS(83),
    [anon_sym_get] = ACTIONS(85),
    [anon_sym_set] = ACTIONS(85),
  },
  [2586] = {
    [anon_sym_export] = ACTIONS(4638),
    [anon_sym_default] = ACTIONS(4638),
    [anon_sym_LBRACE] = ACTIONS(4636),
    [anon_sym_RBRACE] = ACTIONS(4636),
    [anon_sym_import] = ACTIONS(4638),
    [anon_sym_var] = ACTIONS(4638),
    [anon_sym_let] = ACTIONS(4638),
    [anon_sym_const] = ACTIONS(4638),
    [anon_sym_if] = ACTIONS(4638),
    [anon_sym_else] = ACTIONS(4638),
    [anon_sym_switch] = ACTIONS(4638),
    [anon_sym_for] = ACTIONS(4638),
    [anon_sym_LPAREN] = ACTIONS(4636),
    [anon_sym_while] = ACTIONS(4638),
    [anon_sym_do] = ACTIONS(4638),
    [anon_sym_try] = ACTIONS(4638),
    [anon_sym_with] = ACTIONS(4638),
    [anon_sym_break] = ACTIONS(4638),
    [anon_sym_continue] = ACTIONS(4638),
    [anon_sym_debugger] = ACTIONS(4638),
    [anon_sym_return] = ACTIONS(4638),
    [anon_sym_throw] = ACTIONS(4638),
    [anon_sym_SEMI] = ACTIONS(4636),
    [anon_sym_case] = ACTIONS(4638),
    [anon_sym_yield] = ACTIONS(4638),
    [anon_sym_LBRACK] = ACTIONS(4636),
    [anon_sym_LT] = ACTIONS(4636),
    [anon_sym_class] = ACTIONS(4638),
    [anon_sym_async] = ACTIONS(4638),
    [anon_sym_function] = ACTIONS(4638),
    [anon_sym_EQ_GT] = ACTIONS(4636),
    [anon_sym_new] = ACTIONS(4638),
    [anon_sym_await] = ACTIONS(4638),
    [anon_sym_PLUS] = ACTIONS(4638),
    [anon_sym_DASH] = ACTIONS(4638),
    [anon_sym_BANG] = ACTIONS(4636),
    [anon_sym_TILDE] = ACTIONS(4636),
    [anon_sym_typeof] = ACTIONS(4638),
    [anon_sym_void] = ACTIONS(4638),
    [anon_sym_delete] = ACTIONS(4638),
    [anon_sym_PLUS_PLUS] = ACTIONS(4636),
    [anon_sym_DASH_DASH] = ACTIONS(4636),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4636),
    [anon_sym_BQUOTE] = ACTIONS(4636),
    [sym_regex] = ACTIONS(4638),
    [sym_number] = ACTIONS(4636),
    [sym_identifier] = ACTIONS(4638),
    [sym_this] = ACTIONS(4638),
    [sym_super] = ACTIONS(4638),
    [sym_true] = ACTIONS(4638),
    [sym_false] = ACTIONS(4638),
    [sym_null] = ACTIONS(4638),
    [sym_undefined] = ACTIONS(4638),
    [anon_sym_AT] = ACTIONS(4636),
    [anon_sym_get] = ACTIONS(4638),
    [anon_sym_set] = ACTIONS(4638),
  },
  [2587] = {
    [anon_sym_export] = ACTIONS(4648),
    [anon_sym_default] = ACTIONS(4648),
    [anon_sym_LBRACE] = ACTIONS(4646),
    [anon_sym_RBRACE] = ACTIONS(4646),
    [anon_sym_import] = ACTIONS(4648),
    [anon_sym_var] = ACTIONS(4648),
    [anon_sym_let] = ACTIONS(4648),
    [anon_sym_const] = ACTIONS(4648),
    [anon_sym_if] = ACTIONS(4648),
    [anon_sym_else] = ACTIONS(4648),
    [anon_sym_switch] = ACTIONS(4648),
    [anon_sym_for] = ACTIONS(4648),
    [anon_sym_LPAREN] = ACTIONS(4646),
    [anon_sym_while] = ACTIONS(4648),
    [anon_sym_do] = ACTIONS(4648),
    [anon_sym_try] = ACTIONS(4648),
    [anon_sym_with] = ACTIONS(4648),
    [anon_sym_break] = ACTIONS(4648),
    [anon_sym_continue] = ACTIONS(4648),
    [anon_sym_debugger] = ACTIONS(4648),
    [anon_sym_return] = ACTIONS(4648),
    [anon_sym_throw] = ACTIONS(4648),
    [anon_sym_SEMI] = ACTIONS(4646),
    [anon_sym_case] = ACTIONS(4648),
    [anon_sym_yield] = ACTIONS(4648),
    [anon_sym_LBRACK] = ACTIONS(4646),
    [anon_sym_LT] = ACTIONS(4646),
    [anon_sym_class] = ACTIONS(4648),
    [anon_sym_async] = ACTIONS(4648),
    [anon_sym_function] = ACTIONS(4648),
    [anon_sym_EQ_GT] = ACTIONS(4646),
    [anon_sym_new] = ACTIONS(4648),
    [anon_sym_await] = ACTIONS(4648),
    [anon_sym_PLUS] = ACTIONS(4648),
    [anon_sym_DASH] = ACTIONS(4648),
    [anon_sym_BANG] = ACTIONS(4646),
    [anon_sym_TILDE] = ACTIONS(4646),
    [anon_sym_typeof] = ACTIONS(4648),
    [anon_sym_void] = ACTIONS(4648),
    [anon_sym_delete] = ACTIONS(4648),
    [anon_sym_PLUS_PLUS] = ACTIONS(4646),
    [anon_sym_DASH_DASH] = ACTIONS(4646),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4646),
    [anon_sym_BQUOTE] = ACTIONS(4646),
    [sym_regex] = ACTIONS(4648),
    [sym_number] = ACTIONS(4646),
    [sym_identifier] = ACTIONS(4648),
    [sym_this] = ACTIONS(4648),
    [sym_super] = ACTIONS(4648),
    [sym_true] = ACTIONS(4648),
    [sym_false] = ACTIONS(4648),
    [sym_null] = ACTIONS(4648),
    [sym_undefined] = ACTIONS(4648),
    [anon_sym_AT] = ACTIONS(4646),
    [anon_sym_get] = ACTIONS(4648),
    [anon_sym_set] = ACTIONS(4648),
  },
  [2588] = {
    [anon_sym_export] = ACTIONS(4654),
    [anon_sym_default] = ACTIONS(4654),
    [anon_sym_LBRACE] = ACTIONS(4652),
    [anon_sym_RBRACE] = ACTIONS(4652),
    [anon_sym_import] = ACTIONS(4654),
    [anon_sym_var] = ACTIONS(4654),
    [anon_sym_let] = ACTIONS(4654),
    [anon_sym_const] = ACTIONS(4654),
    [anon_sym_if] = ACTIONS(4654),
    [anon_sym_else] = ACTIONS(4654),
    [anon_sym_switch] = ACTIONS(4654),
    [anon_sym_for] = ACTIONS(4654),
    [anon_sym_LPAREN] = ACTIONS(4652),
    [anon_sym_while] = ACTIONS(4654),
    [anon_sym_do] = ACTIONS(4654),
    [anon_sym_try] = ACTIONS(4654),
    [anon_sym_with] = ACTIONS(4654),
    [anon_sym_break] = ACTIONS(4654),
    [anon_sym_continue] = ACTIONS(4654),
    [anon_sym_debugger] = ACTIONS(4654),
    [anon_sym_return] = ACTIONS(4654),
    [anon_sym_throw] = ACTIONS(4654),
    [anon_sym_SEMI] = ACTIONS(4652),
    [anon_sym_case] = ACTIONS(4654),
    [anon_sym_yield] = ACTIONS(4654),
    [anon_sym_LBRACK] = ACTIONS(4652),
    [anon_sym_LT] = ACTIONS(4652),
    [anon_sym_class] = ACTIONS(4654),
    [anon_sym_async] = ACTIONS(4654),
    [anon_sym_function] = ACTIONS(4654),
    [anon_sym_EQ_GT] = ACTIONS(4652),
    [anon_sym_new] = ACTIONS(4654),
    [anon_sym_await] = ACTIONS(4654),
    [anon_sym_PLUS] = ACTIONS(4654),
    [anon_sym_DASH] = ACTIONS(4654),
    [anon_sym_BANG] = ACTIONS(4652),
    [anon_sym_TILDE] = ACTIONS(4652),
    [anon_sym_typeof] = ACTIONS(4654),
    [anon_sym_void] = ACTIONS(4654),
    [anon_sym_delete] = ACTIONS(4654),
    [anon_sym_PLUS_PLUS] = ACTIONS(4652),
    [anon_sym_DASH_DASH] = ACTIONS(4652),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4652),
    [anon_sym_BQUOTE] = ACTIONS(4652),
    [sym_regex] = ACTIONS(4654),
    [sym_number] = ACTIONS(4652),
    [sym_identifier] = ACTIONS(4654),
    [sym_this] = ACTIONS(4654),
    [sym_super] = ACTIONS(4654),
    [sym_true] = ACTIONS(4654),
    [sym_false] = ACTIONS(4654),
    [sym_null] = ACTIONS(4654),
    [sym_undefined] = ACTIONS(4654),
    [anon_sym_AT] = ACTIONS(4652),
    [anon_sym_get] = ACTIONS(4654),
    [anon_sym_set] = ACTIONS(4654),
  },
  [2589] = {
    [anon_sym_export] = ACTIONS(4658),
    [anon_sym_default] = ACTIONS(4658),
    [anon_sym_LBRACE] = ACTIONS(4656),
    [anon_sym_RBRACE] = ACTIONS(4656),
    [anon_sym_import] = ACTIONS(4658),
    [anon_sym_var] = ACTIONS(4658),
    [anon_sym_let] = ACTIONS(4658),
    [anon_sym_const] = ACTIONS(4658),
    [anon_sym_if] = ACTIONS(4658),
    [anon_sym_else] = ACTIONS(4658),
    [anon_sym_switch] = ACTIONS(4658),
    [anon_sym_for] = ACTIONS(4658),
    [anon_sym_LPAREN] = ACTIONS(4656),
    [anon_sym_while] = ACTIONS(4658),
    [anon_sym_do] = ACTIONS(4658),
    [anon_sym_try] = ACTIONS(4658),
    [anon_sym_with] = ACTIONS(4658),
    [anon_sym_break] = ACTIONS(4658),
    [anon_sym_continue] = ACTIONS(4658),
    [anon_sym_debugger] = ACTIONS(4658),
    [anon_sym_return] = ACTIONS(4658),
    [anon_sym_throw] = ACTIONS(4658),
    [anon_sym_SEMI] = ACTIONS(4656),
    [anon_sym_case] = ACTIONS(4658),
    [anon_sym_yield] = ACTIONS(4658),
    [anon_sym_LBRACK] = ACTIONS(4656),
    [anon_sym_LT] = ACTIONS(4656),
    [anon_sym_class] = ACTIONS(4658),
    [anon_sym_async] = ACTIONS(4658),
    [anon_sym_function] = ACTIONS(4658),
    [anon_sym_EQ_GT] = ACTIONS(4656),
    [anon_sym_new] = ACTIONS(4658),
    [anon_sym_await] = ACTIONS(4658),
    [anon_sym_PLUS] = ACTIONS(4658),
    [anon_sym_DASH] = ACTIONS(4658),
    [anon_sym_BANG] = ACTIONS(4656),
    [anon_sym_TILDE] = ACTIONS(4656),
    [anon_sym_typeof] = ACTIONS(4658),
    [anon_sym_void] = ACTIONS(4658),
    [anon_sym_delete] = ACTIONS(4658),
    [anon_sym_PLUS_PLUS] = ACTIONS(4656),
    [anon_sym_DASH_DASH] = ACTIONS(4656),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4656),
    [anon_sym_BQUOTE] = ACTIONS(4656),
    [sym_regex] = ACTIONS(4658),
    [sym_number] = ACTIONS(4656),
    [sym_identifier] = ACTIONS(4658),
    [sym_this] = ACTIONS(4658),
    [sym_super] = ACTIONS(4658),
    [sym_true] = ACTIONS(4658),
    [sym_false] = ACTIONS(4658),
    [sym_null] = ACTIONS(4658),
    [sym_undefined] = ACTIONS(4658),
    [anon_sym_AT] = ACTIONS(4656),
    [anon_sym_get] = ACTIONS(4658),
    [anon_sym_set] = ACTIONS(4658),
  },
  [2590] = {
    [anon_sym_export] = ACTIONS(4662),
    [anon_sym_default] = ACTIONS(4662),
    [anon_sym_LBRACE] = ACTIONS(4660),
    [anon_sym_RBRACE] = ACTIONS(4660),
    [anon_sym_import] = ACTIONS(4662),
    [anon_sym_var] = ACTIONS(4662),
    [anon_sym_let] = ACTIONS(4662),
    [anon_sym_const] = ACTIONS(4662),
    [anon_sym_if] = ACTIONS(4662),
    [anon_sym_else] = ACTIONS(4662),
    [anon_sym_switch] = ACTIONS(4662),
    [anon_sym_for] = ACTIONS(4662),
    [anon_sym_LPAREN] = ACTIONS(4660),
    [anon_sym_while] = ACTIONS(4662),
    [anon_sym_do] = ACTIONS(4662),
    [anon_sym_try] = ACTIONS(4662),
    [anon_sym_with] = ACTIONS(4662),
    [anon_sym_break] = ACTIONS(4662),
    [anon_sym_continue] = ACTIONS(4662),
    [anon_sym_debugger] = ACTIONS(4662),
    [anon_sym_return] = ACTIONS(4662),
    [anon_sym_throw] = ACTIONS(4662),
    [anon_sym_SEMI] = ACTIONS(4660),
    [anon_sym_case] = ACTIONS(4662),
    [anon_sym_yield] = ACTIONS(4662),
    [anon_sym_LBRACK] = ACTIONS(4660),
    [anon_sym_LT] = ACTIONS(4660),
    [anon_sym_class] = ACTIONS(4662),
    [anon_sym_async] = ACTIONS(4662),
    [anon_sym_function] = ACTIONS(4662),
    [anon_sym_EQ_GT] = ACTIONS(4660),
    [anon_sym_new] = ACTIONS(4662),
    [anon_sym_await] = ACTIONS(4662),
    [anon_sym_PLUS] = ACTIONS(4662),
    [anon_sym_DASH] = ACTIONS(4662),
    [anon_sym_BANG] = ACTIONS(4660),
    [anon_sym_TILDE] = ACTIONS(4660),
    [anon_sym_typeof] = ACTIONS(4662),
    [anon_sym_void] = ACTIONS(4662),
    [anon_sym_delete] = ACTIONS(4662),
    [anon_sym_PLUS_PLUS] = ACTIONS(4660),
    [anon_sym_DASH_DASH] = ACTIONS(4660),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4660),
    [anon_sym_BQUOTE] = ACTIONS(4660),
    [sym_regex] = ACTIONS(4662),
    [sym_number] = ACTIONS(4660),
    [sym_identifier] = ACTIONS(4662),
    [sym_this] = ACTIONS(4662),
    [sym_super] = ACTIONS(4662),
    [sym_true] = ACTIONS(4662),
    [sym_false] = ACTIONS(4662),
    [sym_null] = ACTIONS(4662),
    [sym_undefined] = ACTIONS(4662),
    [anon_sym_AT] = ACTIONS(4660),
    [anon_sym_get] = ACTIONS(4662),
    [anon_sym_set] = ACTIONS(4662),
  },
  [2591] = {
    [anon_sym_export] = ACTIONS(4666),
    [anon_sym_default] = ACTIONS(4666),
    [anon_sym_LBRACE] = ACTIONS(4664),
    [anon_sym_RBRACE] = ACTIONS(4664),
    [anon_sym_import] = ACTIONS(4666),
    [anon_sym_var] = ACTIONS(4666),
    [anon_sym_let] = ACTIONS(4666),
    [anon_sym_const] = ACTIONS(4666),
    [anon_sym_if] = ACTIONS(4666),
    [anon_sym_else] = ACTIONS(4666),
    [anon_sym_switch] = ACTIONS(4666),
    [anon_sym_for] = ACTIONS(4666),
    [anon_sym_LPAREN] = ACTIONS(4664),
    [anon_sym_while] = ACTIONS(4666),
    [anon_sym_do] = ACTIONS(4666),
    [anon_sym_try] = ACTIONS(4666),
    [anon_sym_with] = ACTIONS(4666),
    [anon_sym_break] = ACTIONS(4666),
    [anon_sym_continue] = ACTIONS(4666),
    [anon_sym_debugger] = ACTIONS(4666),
    [anon_sym_return] = ACTIONS(4666),
    [anon_sym_throw] = ACTIONS(4666),
    [anon_sym_SEMI] = ACTIONS(4664),
    [anon_sym_case] = ACTIONS(4666),
    [anon_sym_yield] = ACTIONS(4666),
    [anon_sym_LBRACK] = ACTIONS(4664),
    [anon_sym_LT] = ACTIONS(4664),
    [anon_sym_class] = ACTIONS(4666),
    [anon_sym_async] = ACTIONS(4666),
    [anon_sym_function] = ACTIONS(4666),
    [anon_sym_EQ_GT] = ACTIONS(4664),
    [anon_sym_new] = ACTIONS(4666),
    [anon_sym_await] = ACTIONS(4666),
    [anon_sym_PLUS] = ACTIONS(4666),
    [anon_sym_DASH] = ACTIONS(4666),
    [anon_sym_BANG] = ACTIONS(4664),
    [anon_sym_TILDE] = ACTIONS(4664),
    [anon_sym_typeof] = ACTIONS(4666),
    [anon_sym_void] = ACTIONS(4666),
    [anon_sym_delete] = ACTIONS(4666),
    [anon_sym_PLUS_PLUS] = ACTIONS(4664),
    [anon_sym_DASH_DASH] = ACTIONS(4664),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4664),
    [anon_sym_BQUOTE] = ACTIONS(4664),
    [sym_regex] = ACTIONS(4666),
    [sym_number] = ACTIONS(4664),
    [sym_identifier] = ACTIONS(4666),
    [sym_this] = ACTIONS(4666),
    [sym_super] = ACTIONS(4666),
    [sym_true] = ACTIONS(4666),
    [sym_false] = ACTIONS(4666),
    [sym_null] = ACTIONS(4666),
    [sym_undefined] = ACTIONS(4666),
    [anon_sym_AT] = ACTIONS(4664),
    [anon_sym_get] = ACTIONS(4666),
    [anon_sym_set] = ACTIONS(4666),
  },
  [2592] = {
    [anon_sym_export] = ACTIONS(4670),
    [anon_sym_default] = ACTIONS(4670),
    [anon_sym_LBRACE] = ACTIONS(4668),
    [anon_sym_RBRACE] = ACTIONS(4668),
    [anon_sym_import] = ACTIONS(4670),
    [anon_sym_var] = ACTIONS(4670),
    [anon_sym_let] = ACTIONS(4670),
    [anon_sym_const] = ACTIONS(4670),
    [anon_sym_if] = ACTIONS(4670),
    [anon_sym_else] = ACTIONS(4670),
    [anon_sym_switch] = ACTIONS(4670),
    [anon_sym_for] = ACTIONS(4670),
    [anon_sym_LPAREN] = ACTIONS(4668),
    [anon_sym_while] = ACTIONS(4670),
    [anon_sym_do] = ACTIONS(4670),
    [anon_sym_try] = ACTIONS(4670),
    [anon_sym_with] = ACTIONS(4670),
    [anon_sym_break] = ACTIONS(4670),
    [anon_sym_continue] = ACTIONS(4670),
    [anon_sym_debugger] = ACTIONS(4670),
    [anon_sym_return] = ACTIONS(4670),
    [anon_sym_throw] = ACTIONS(4670),
    [anon_sym_SEMI] = ACTIONS(4668),
    [anon_sym_case] = ACTIONS(4670),
    [anon_sym_yield] = ACTIONS(4670),
    [anon_sym_LBRACK] = ACTIONS(4668),
    [anon_sym_LT] = ACTIONS(4668),
    [anon_sym_class] = ACTIONS(4670),
    [anon_sym_async] = ACTIONS(4670),
    [anon_sym_function] = ACTIONS(4670),
    [anon_sym_EQ_GT] = ACTIONS(4668),
    [anon_sym_new] = ACTIONS(4670),
    [anon_sym_await] = ACTIONS(4670),
    [anon_sym_PLUS] = ACTIONS(4670),
    [anon_sym_DASH] = ACTIONS(4670),
    [anon_sym_BANG] = ACTIONS(4668),
    [anon_sym_TILDE] = ACTIONS(4668),
    [anon_sym_typeof] = ACTIONS(4670),
    [anon_sym_void] = ACTIONS(4670),
    [anon_sym_delete] = ACTIONS(4670),
    [anon_sym_PLUS_PLUS] = ACTIONS(4668),
    [anon_sym_DASH_DASH] = ACTIONS(4668),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4668),
    [anon_sym_BQUOTE] = ACTIONS(4668),
    [sym_regex] = ACTIONS(4670),
    [sym_number] = ACTIONS(4668),
    [sym_identifier] = ACTIONS(4670),
    [sym_this] = ACTIONS(4670),
    [sym_super] = ACTIONS(4670),
    [sym_true] = ACTIONS(4670),
    [sym_false] = ACTIONS(4670),
    [sym_null] = ACTIONS(4670),
    [sym_undefined] = ACTIONS(4670),
    [anon_sym_AT] = ACTIONS(4668),
    [anon_sym_get] = ACTIONS(4670),
    [anon_sym_set] = ACTIONS(4670),
  },
  [2593] = {
    [anon_sym_export] = ACTIONS(4720),
    [anon_sym_default] = ACTIONS(4720),
    [anon_sym_LBRACE] = ACTIONS(4718),
    [anon_sym_RBRACE] = ACTIONS(4718),
    [anon_sym_import] = ACTIONS(4720),
    [anon_sym_var] = ACTIONS(4720),
    [anon_sym_let] = ACTIONS(4720),
    [anon_sym_const] = ACTIONS(4720),
    [anon_sym_if] = ACTIONS(4720),
    [anon_sym_else] = ACTIONS(4720),
    [anon_sym_switch] = ACTIONS(4720),
    [anon_sym_for] = ACTIONS(4720),
    [anon_sym_LPAREN] = ACTIONS(4718),
    [anon_sym_while] = ACTIONS(4720),
    [anon_sym_do] = ACTIONS(4720),
    [anon_sym_try] = ACTIONS(4720),
    [anon_sym_with] = ACTIONS(4720),
    [anon_sym_break] = ACTIONS(4720),
    [anon_sym_continue] = ACTIONS(4720),
    [anon_sym_debugger] = ACTIONS(4720),
    [anon_sym_return] = ACTIONS(4720),
    [anon_sym_throw] = ACTIONS(4720),
    [anon_sym_SEMI] = ACTIONS(4718),
    [anon_sym_case] = ACTIONS(4720),
    [anon_sym_finally] = ACTIONS(4720),
    [anon_sym_yield] = ACTIONS(4720),
    [anon_sym_LBRACK] = ACTIONS(4718),
    [anon_sym_LT] = ACTIONS(4718),
    [anon_sym_class] = ACTIONS(4720),
    [anon_sym_async] = ACTIONS(4720),
    [anon_sym_function] = ACTIONS(4720),
    [anon_sym_EQ_GT] = ACTIONS(4718),
    [anon_sym_new] = ACTIONS(4720),
    [anon_sym_await] = ACTIONS(4720),
    [anon_sym_PLUS] = ACTIONS(4720),
    [anon_sym_DASH] = ACTIONS(4720),
    [anon_sym_BANG] = ACTIONS(4718),
    [anon_sym_TILDE] = ACTIONS(4718),
    [anon_sym_typeof] = ACTIONS(4720),
    [anon_sym_void] = ACTIONS(4720),
    [anon_sym_delete] = ACTIONS(4720),
    [anon_sym_PLUS_PLUS] = ACTIONS(4718),
    [anon_sym_DASH_DASH] = ACTIONS(4718),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4718),
    [anon_sym_BQUOTE] = ACTIONS(4718),
    [sym_regex] = ACTIONS(4720),
    [sym_number] = ACTIONS(4718),
    [sym_identifier] = ACTIONS(4720),
    [sym_this] = ACTIONS(4720),
    [sym_super] = ACTIONS(4720),
    [sym_true] = ACTIONS(4720),
    [sym_false] = ACTIONS(4720),
    [sym_null] = ACTIONS(4720),
    [sym_undefined] = ACTIONS(4720),
    [anon_sym_AT] = ACTIONS(4718),
    [anon_sym_get] = ACTIONS(4720),
    [anon_sym_set] = ACTIONS(4720),
  },
  [2594] = {
    [anon_sym_export] = ACTIONS(4892),
    [anon_sym_default] = ACTIONS(4892),
    [anon_sym_LBRACE] = ACTIONS(4890),
    [anon_sym_RBRACE] = ACTIONS(4890),
    [anon_sym_import] = ACTIONS(4892),
    [anon_sym_var] = ACTIONS(4892),
    [anon_sym_let] = ACTIONS(4892),
    [anon_sym_const] = ACTIONS(4892),
    [anon_sym_if] = ACTIONS(4892),
    [anon_sym_else] = ACTIONS(4892),
    [anon_sym_switch] = ACTIONS(4892),
    [anon_sym_for] = ACTIONS(4892),
    [anon_sym_LPAREN] = ACTIONS(4890),
    [anon_sym_while] = ACTIONS(4892),
    [anon_sym_do] = ACTIONS(4892),
    [anon_sym_try] = ACTIONS(4892),
    [anon_sym_with] = ACTIONS(4892),
    [anon_sym_break] = ACTIONS(4892),
    [anon_sym_continue] = ACTIONS(4892),
    [anon_sym_debugger] = ACTIONS(4892),
    [anon_sym_return] = ACTIONS(4892),
    [anon_sym_throw] = ACTIONS(4892),
    [anon_sym_SEMI] = ACTIONS(4890),
    [anon_sym_case] = ACTIONS(4892),
    [anon_sym_yield] = ACTIONS(4892),
    [anon_sym_LBRACK] = ACTIONS(4890),
    [anon_sym_LT] = ACTIONS(4890),
    [anon_sym_class] = ACTIONS(4892),
    [anon_sym_async] = ACTIONS(4892),
    [anon_sym_function] = ACTIONS(4892),
    [anon_sym_EQ_GT] = ACTIONS(4890),
    [anon_sym_new] = ACTIONS(4892),
    [anon_sym_await] = ACTIONS(4892),
    [anon_sym_PLUS] = ACTIONS(4892),
    [anon_sym_DASH] = ACTIONS(4892),
    [anon_sym_BANG] = ACTIONS(4890),
    [anon_sym_TILDE] = ACTIONS(4890),
    [anon_sym_typeof] = ACTIONS(4892),
    [anon_sym_void] = ACTIONS(4892),
    [anon_sym_delete] = ACTIONS(4892),
    [anon_sym_PLUS_PLUS] = ACTIONS(4890),
    [anon_sym_DASH_DASH] = ACTIONS(4890),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4890),
    [anon_sym_BQUOTE] = ACTIONS(4890),
    [sym_regex] = ACTIONS(4892),
    [sym_number] = ACTIONS(4890),
    [sym_identifier] = ACTIONS(4892),
    [sym_this] = ACTIONS(4892),
    [sym_super] = ACTIONS(4892),
    [sym_true] = ACTIONS(4892),
    [sym_false] = ACTIONS(4892),
    [sym_null] = ACTIONS(4892),
    [sym_undefined] = ACTIONS(4892),
    [anon_sym_AT] = ACTIONS(4890),
    [anon_sym_get] = ACTIONS(4892),
    [anon_sym_set] = ACTIONS(4892),
  },
  [2595] = {
    [anon_sym_export] = ACTIONS(4896),
    [anon_sym_default] = ACTIONS(4896),
    [anon_sym_LBRACE] = ACTIONS(4894),
    [anon_sym_RBRACE] = ACTIONS(4894),
    [anon_sym_import] = ACTIONS(4896),
    [anon_sym_var] = ACTIONS(4896),
    [anon_sym_let] = ACTIONS(4896),
    [anon_sym_const] = ACTIONS(4896),
    [anon_sym_if] = ACTIONS(4896),
    [anon_sym_else] = ACTIONS(4896),
    [anon_sym_switch] = ACTIONS(4896),
    [anon_sym_for] = ACTIONS(4896),
    [anon_sym_LPAREN] = ACTIONS(4894),
    [anon_sym_while] = ACTIONS(4896),
    [anon_sym_do] = ACTIONS(4896),
    [anon_sym_try] = ACTIONS(4896),
    [anon_sym_with] = ACTIONS(4896),
    [anon_sym_break] = ACTIONS(4896),
    [anon_sym_continue] = ACTIONS(4896),
    [anon_sym_debugger] = ACTIONS(4896),
    [anon_sym_return] = ACTIONS(4896),
    [anon_sym_throw] = ACTIONS(4896),
    [anon_sym_SEMI] = ACTIONS(4894),
    [anon_sym_case] = ACTIONS(4896),
    [anon_sym_yield] = ACTIONS(4896),
    [anon_sym_LBRACK] = ACTIONS(4894),
    [anon_sym_LT] = ACTIONS(4894),
    [anon_sym_class] = ACTIONS(4896),
    [anon_sym_async] = ACTIONS(4896),
    [anon_sym_function] = ACTIONS(4896),
    [anon_sym_EQ_GT] = ACTIONS(4894),
    [anon_sym_new] = ACTIONS(4896),
    [anon_sym_await] = ACTIONS(4896),
    [anon_sym_PLUS] = ACTIONS(4896),
    [anon_sym_DASH] = ACTIONS(4896),
    [anon_sym_BANG] = ACTIONS(4894),
    [anon_sym_TILDE] = ACTIONS(4894),
    [anon_sym_typeof] = ACTIONS(4896),
    [anon_sym_void] = ACTIONS(4896),
    [anon_sym_delete] = ACTIONS(4896),
    [anon_sym_PLUS_PLUS] = ACTIONS(4894),
    [anon_sym_DASH_DASH] = ACTIONS(4894),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4894),
    [anon_sym_BQUOTE] = ACTIONS(4894),
    [sym_regex] = ACTIONS(4896),
    [sym_number] = ACTIONS(4894),
    [sym_identifier] = ACTIONS(4896),
    [sym_this] = ACTIONS(4896),
    [sym_super] = ACTIONS(4896),
    [sym_true] = ACTIONS(4896),
    [sym_false] = ACTIONS(4896),
    [sym_null] = ACTIONS(4896),
    [sym_undefined] = ACTIONS(4896),
    [anon_sym_AT] = ACTIONS(4894),
    [anon_sym_get] = ACTIONS(4896),
    [anon_sym_set] = ACTIONS(4896),
  },
  [2596] = {
    [anon_sym_export] = ACTIONS(4900),
    [anon_sym_default] = ACTIONS(4900),
    [anon_sym_LBRACE] = ACTIONS(4898),
    [anon_sym_RBRACE] = ACTIONS(4898),
    [anon_sym_import] = ACTIONS(4900),
    [anon_sym_var] = ACTIONS(4900),
    [anon_sym_let] = ACTIONS(4900),
    [anon_sym_const] = ACTIONS(4900),
    [anon_sym_if] = ACTIONS(4900),
    [anon_sym_else] = ACTIONS(4900),
    [anon_sym_switch] = ACTIONS(4900),
    [anon_sym_for] = ACTIONS(4900),
    [anon_sym_LPAREN] = ACTIONS(4898),
    [anon_sym_while] = ACTIONS(4900),
    [anon_sym_do] = ACTIONS(4900),
    [anon_sym_try] = ACTIONS(4900),
    [anon_sym_with] = ACTIONS(4900),
    [anon_sym_break] = ACTIONS(4900),
    [anon_sym_continue] = ACTIONS(4900),
    [anon_sym_debugger] = ACTIONS(4900),
    [anon_sym_return] = ACTIONS(4900),
    [anon_sym_throw] = ACTIONS(4900),
    [anon_sym_SEMI] = ACTIONS(4898),
    [anon_sym_case] = ACTIONS(4900),
    [anon_sym_yield] = ACTIONS(4900),
    [anon_sym_LBRACK] = ACTIONS(4898),
    [anon_sym_LT] = ACTIONS(4898),
    [anon_sym_class] = ACTIONS(4900),
    [anon_sym_async] = ACTIONS(4900),
    [anon_sym_function] = ACTIONS(4900),
    [anon_sym_EQ_GT] = ACTIONS(4898),
    [anon_sym_new] = ACTIONS(4900),
    [anon_sym_await] = ACTIONS(4900),
    [anon_sym_PLUS] = ACTIONS(4900),
    [anon_sym_DASH] = ACTIONS(4900),
    [anon_sym_BANG] = ACTIONS(4898),
    [anon_sym_TILDE] = ACTIONS(4898),
    [anon_sym_typeof] = ACTIONS(4900),
    [anon_sym_void] = ACTIONS(4900),
    [anon_sym_delete] = ACTIONS(4900),
    [anon_sym_PLUS_PLUS] = ACTIONS(4898),
    [anon_sym_DASH_DASH] = ACTIONS(4898),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4898),
    [anon_sym_BQUOTE] = ACTIONS(4898),
    [sym_regex] = ACTIONS(4900),
    [sym_number] = ACTIONS(4898),
    [sym_identifier] = ACTIONS(4900),
    [sym_this] = ACTIONS(4900),
    [sym_super] = ACTIONS(4900),
    [sym_true] = ACTIONS(4900),
    [sym_false] = ACTIONS(4900),
    [sym_null] = ACTIONS(4900),
    [sym_undefined] = ACTIONS(4900),
    [anon_sym_AT] = ACTIONS(4898),
    [anon_sym_get] = ACTIONS(4900),
    [anon_sym_set] = ACTIONS(4900),
  },
  [2597] = {
    [anon_sym_export] = ACTIONS(4904),
    [anon_sym_default] = ACTIONS(4904),
    [anon_sym_LBRACE] = ACTIONS(4902),
    [anon_sym_RBRACE] = ACTIONS(4902),
    [anon_sym_import] = ACTIONS(4904),
    [anon_sym_var] = ACTIONS(4904),
    [anon_sym_let] = ACTIONS(4904),
    [anon_sym_const] = ACTIONS(4904),
    [anon_sym_if] = ACTIONS(4904),
    [anon_sym_else] = ACTIONS(4904),
    [anon_sym_switch] = ACTIONS(4904),
    [anon_sym_for] = ACTIONS(4904),
    [anon_sym_LPAREN] = ACTIONS(4902),
    [anon_sym_while] = ACTIONS(4904),
    [anon_sym_do] = ACTIONS(4904),
    [anon_sym_try] = ACTIONS(4904),
    [anon_sym_with] = ACTIONS(4904),
    [anon_sym_break] = ACTIONS(4904),
    [anon_sym_continue] = ACTIONS(4904),
    [anon_sym_debugger] = ACTIONS(4904),
    [anon_sym_return] = ACTIONS(4904),
    [anon_sym_throw] = ACTIONS(4904),
    [anon_sym_SEMI] = ACTIONS(4902),
    [anon_sym_case] = ACTIONS(4904),
    [anon_sym_yield] = ACTIONS(4904),
    [anon_sym_LBRACK] = ACTIONS(4902),
    [anon_sym_LT] = ACTIONS(4902),
    [anon_sym_class] = ACTIONS(4904),
    [anon_sym_async] = ACTIONS(4904),
    [anon_sym_function] = ACTIONS(4904),
    [anon_sym_EQ_GT] = ACTIONS(4902),
    [anon_sym_new] = ACTIONS(4904),
    [anon_sym_await] = ACTIONS(4904),
    [anon_sym_PLUS] = ACTIONS(4904),
    [anon_sym_DASH] = ACTIONS(4904),
    [anon_sym_BANG] = ACTIONS(4902),
    [anon_sym_TILDE] = ACTIONS(4902),
    [anon_sym_typeof] = ACTIONS(4904),
    [anon_sym_void] = ACTIONS(4904),
    [anon_sym_delete] = ACTIONS(4904),
    [anon_sym_PLUS_PLUS] = ACTIONS(4902),
    [anon_sym_DASH_DASH] = ACTIONS(4902),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4902),
    [anon_sym_BQUOTE] = ACTIONS(4902),
    [sym_regex] = ACTIONS(4904),
    [sym_number] = ACTIONS(4902),
    [sym_identifier] = ACTIONS(4904),
    [sym_this] = ACTIONS(4904),
    [sym_super] = ACTIONS(4904),
    [sym_true] = ACTIONS(4904),
    [sym_false] = ACTIONS(4904),
    [sym_null] = ACTIONS(4904),
    [sym_undefined] = ACTIONS(4904),
    [anon_sym_AT] = ACTIONS(4902),
    [anon_sym_get] = ACTIONS(4904),
    [anon_sym_set] = ACTIONS(4904),
  },
  [2598] = {
    [anon_sym_export] = ACTIONS(4908),
    [anon_sym_default] = ACTIONS(4908),
    [anon_sym_LBRACE] = ACTIONS(4906),
    [anon_sym_RBRACE] = ACTIONS(4906),
    [anon_sym_import] = ACTIONS(4908),
    [anon_sym_var] = ACTIONS(4908),
    [anon_sym_let] = ACTIONS(4908),
    [anon_sym_const] = ACTIONS(4908),
    [anon_sym_if] = ACTIONS(4908),
    [anon_sym_else] = ACTIONS(4908),
    [anon_sym_switch] = ACTIONS(4908),
    [anon_sym_for] = ACTIONS(4908),
    [anon_sym_LPAREN] = ACTIONS(4906),
    [anon_sym_while] = ACTIONS(4908),
    [anon_sym_do] = ACTIONS(4908),
    [anon_sym_try] = ACTIONS(4908),
    [anon_sym_with] = ACTIONS(4908),
    [anon_sym_break] = ACTIONS(4908),
    [anon_sym_continue] = ACTIONS(4908),
    [anon_sym_debugger] = ACTIONS(4908),
    [anon_sym_return] = ACTIONS(4908),
    [anon_sym_throw] = ACTIONS(4908),
    [anon_sym_SEMI] = ACTIONS(4906),
    [anon_sym_case] = ACTIONS(4908),
    [anon_sym_yield] = ACTIONS(4908),
    [anon_sym_LBRACK] = ACTIONS(4906),
    [anon_sym_LT] = ACTIONS(4906),
    [anon_sym_class] = ACTIONS(4908),
    [anon_sym_async] = ACTIONS(4908),
    [anon_sym_function] = ACTIONS(4908),
    [anon_sym_EQ_GT] = ACTIONS(4906),
    [anon_sym_new] = ACTIONS(4908),
    [anon_sym_await] = ACTIONS(4908),
    [anon_sym_PLUS] = ACTIONS(4908),
    [anon_sym_DASH] = ACTIONS(4908),
    [anon_sym_BANG] = ACTIONS(4906),
    [anon_sym_TILDE] = ACTIONS(4906),
    [anon_sym_typeof] = ACTIONS(4908),
    [anon_sym_void] = ACTIONS(4908),
    [anon_sym_delete] = ACTIONS(4908),
    [anon_sym_PLUS_PLUS] = ACTIONS(4906),
    [anon_sym_DASH_DASH] = ACTIONS(4906),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4906),
    [anon_sym_BQUOTE] = ACTIONS(4906),
    [sym_regex] = ACTIONS(4908),
    [sym_number] = ACTIONS(4906),
    [sym_identifier] = ACTIONS(4908),
    [sym_this] = ACTIONS(4908),
    [sym_super] = ACTIONS(4908),
    [sym_true] = ACTIONS(4908),
    [sym_false] = ACTIONS(4908),
    [sym_null] = ACTIONS(4908),
    [sym_undefined] = ACTIONS(4908),
    [anon_sym_AT] = ACTIONS(4906),
    [anon_sym_get] = ACTIONS(4908),
    [anon_sym_set] = ACTIONS(4908),
  },
  [2599] = {
    [anon_sym_export] = ACTIONS(4912),
    [anon_sym_default] = ACTIONS(4912),
    [anon_sym_LBRACE] = ACTIONS(4910),
    [anon_sym_RBRACE] = ACTIONS(4910),
    [anon_sym_import] = ACTIONS(4912),
    [anon_sym_var] = ACTIONS(4912),
    [anon_sym_let] = ACTIONS(4912),
    [anon_sym_const] = ACTIONS(4912),
    [anon_sym_if] = ACTIONS(4912),
    [anon_sym_else] = ACTIONS(4912),
    [anon_sym_switch] = ACTIONS(4912),
    [anon_sym_for] = ACTIONS(4912),
    [anon_sym_LPAREN] = ACTIONS(4910),
    [anon_sym_while] = ACTIONS(4912),
    [anon_sym_do] = ACTIONS(4912),
    [anon_sym_try] = ACTIONS(4912),
    [anon_sym_with] = ACTIONS(4912),
    [anon_sym_break] = ACTIONS(4912),
    [anon_sym_continue] = ACTIONS(4912),
    [anon_sym_debugger] = ACTIONS(4912),
    [anon_sym_return] = ACTIONS(4912),
    [anon_sym_throw] = ACTIONS(4912),
    [anon_sym_SEMI] = ACTIONS(4910),
    [anon_sym_case] = ACTIONS(4912),
    [anon_sym_yield] = ACTIONS(4912),
    [anon_sym_LBRACK] = ACTIONS(4910),
    [anon_sym_LT] = ACTIONS(4910),
    [anon_sym_class] = ACTIONS(4912),
    [anon_sym_async] = ACTIONS(4912),
    [anon_sym_function] = ACTIONS(4912),
    [anon_sym_EQ_GT] = ACTIONS(4910),
    [anon_sym_new] = ACTIONS(4912),
    [anon_sym_await] = ACTIONS(4912),
    [anon_sym_PLUS] = ACTIONS(4912),
    [anon_sym_DASH] = ACTIONS(4912),
    [anon_sym_BANG] = ACTIONS(4910),
    [anon_sym_TILDE] = ACTIONS(4910),
    [anon_sym_typeof] = ACTIONS(4912),
    [anon_sym_void] = ACTIONS(4912),
    [anon_sym_delete] = ACTIONS(4912),
    [anon_sym_PLUS_PLUS] = ACTIONS(4910),
    [anon_sym_DASH_DASH] = ACTIONS(4910),
    [sym_comment] = ACTIONS(71),
    [sym_string] = ACTIONS(4910),
    [anon_sym_BQUOTE] = ACTIONS(4910),
    [sym_regex] = ACTIONS(4912),
    [sym_number] = ACTIONS(4910),
    [sym_identifier] = ACTIONS(4912),
    [sym_this] = ACTIONS(4912),
    [sym_super] = ACTIONS(4912),
    [sym_true] = ACTIONS(4912),
    [sym_false] = ACTIONS(4912),
    [sym_null] = ACTIONS(4912),
    [sym_undefined] = ACTIONS(4912),
    [anon_sym_AT] = ACTIONS(4910),
    [anon_sym_get] = ACTIONS(4912),
    [anon_sym_set] = ACTIONS(4912),
  },
};

static TSParseActionEntry ts_parse_actions[] = {
  [0] = {.count = 0, .reusable = false},
  [1] = {.count = 1, .reusable = true}, RECOVER(),
  [3] = {.count = 1, .reusable = false}, RECOVER(),
  [5] = {.count = 1, .reusable = true}, REDUCE(sym_program, 0),
  [7] = {.count = 1, .reusable = true}, SHIFT(2),
  [9] = {.count = 1, .reusable = false}, SHIFT(3),
  [11] = {.count = 1, .reusable = true}, SHIFT(4),
  [13] = {.count = 1, .reusable = false}, SHIFT(5),
  [15] = {.count = 1, .reusable = false}, SHIFT(6),
  [17] = {.count = 1, .reusable = false}, SHIFT(7),
  [19] = {.count = 1, .reusable = false}, SHIFT(8),
  [21] = {.count = 1, .reusable = false}, SHIFT(9),
  [23] = {.count = 1, .reusable = false}, SHIFT(10),
  [25] = {.count = 1, .reusable = true}, SHIFT(11),
  [27] = {.count = 1, .reusable = false}, SHIFT(12),
  [29] = {.count = 1, .reusable = false}, SHIFT(13),
  [31] = {.count = 1, .reusable = false}, SHIFT(14),
  [33] = {.count = 1, .reusable = false}, SHIFT(15),
  [35] = {.count = 1, .reusable = false}, SHIFT(16),
  [37] = {.count = 1, .reusable = false}, SHIFT(17),
  [39] = {.count = 1, .reusable = false}, SHIFT(18),
  [41] = {.count = 1, .reusable = false}, SHIFT(19),
  [43] = {.count = 1, .reusable = false}, SHIFT(20),
  [45] = {.count = 1, .reusable = true}, SHIFT(21),
  [47] = {.count = 1, .reusable = false}, SHIFT(22),
  [49] = {.count = 1, .reusable = true}, SHIFT(23),
  [51] = {.count = 1, .reusable = true}, SHIFT(24),
  [53] = {.count = 1, .reusable = false}, SHIFT(25),
  [55] = {.count = 1, .reusable = false}, SHIFT(26),
  [57] = {.count = 1, .reusable = false}, SHIFT(27),
  [59] = {.count = 1, .reusable = true}, SHIFT(28),
  [61] = {.count = 1, .reusable = false}, SHIFT(29),
  [63] = {.count = 1, .reusable = false}, SHIFT(30),
  [65] = {.count = 1, .reusable = false}, SHIFT(31),
  [67] = {.count = 1, .reusable = true}, SHIFT(31),
  [69] = {.count = 1, .reusable = true}, SHIFT(32),
  [71] = {.count = 1, .reusable = true}, SHIFT_EXTRA(),
  [73] = {.count = 1, .reusable = true}, SHIFT(40),
  [75] = {.count = 1, .reusable = true}, SHIFT(33),
  [77] = {.count = 1, .reusable = false}, SHIFT(40),
  [79] = {.count = 1, .reusable = false}, SHIFT(34),
  [81] = {.count = 1, .reusable = false}, SHIFT(35),
  [83] = {.count = 1, .reusable = true}, SHIFT(36),
  [85] = {.count = 1, .reusable = false}, SHIFT(37),
  [87] = {.count = 1, .reusable = true}, REDUCE(sym_program, 1),
  [89] = {.count = 1, .reusable = true}, SHIFT(51),
  [91] = {.count = 1, .reusable = true}, SHIFT(52),
  [93] = {.count = 1, .reusable = true}, SHIFT(53),
  [95] = {.count = 1, .reusable = true}, SHIFT(6),
  [97] = {.count = 1, .reusable = true}, SHIFT(7),
  [99] = {.count = 1, .reusable = true}, SHIFT(54),
  [101] = {.count = 1, .reusable = true}, SHIFT(55),
  [103] = {.count = 1, .reusable = true}, SHIFT(56),
  [105] = {.count = 1, .reusable = true}, SHIFT(61),
  [107] = {.count = 1, .reusable = true}, SHIFT(62),
  [109] = {.count = 1, .reusable = true}, SHIFT(63),
  [111] = {.count = 1, .reusable = false}, SHIFT(64),
  [113] = {.count = 1, .reusable = false}, SHIFT(65),
  [115] = {.count = 1, .reusable = false}, SHIFT(66),
  [117] = {.count = 1, .reusable = false}, SHIFT(67),
  [119] = {.count = 1, .reusable = true}, SHIFT(68),
  [121] = {.count = 1, .reusable = false}, SHIFT(69),
  [123] = {.count = 1, .reusable = true}, SHIFT(70),
  [125] = {.count = 1, .reusable = true}, SHIFT(71),
  [127] = {.count = 1, .reusable = false}, SHIFT(72),
  [129] = {.count = 1, .reusable = true}, SHIFT(73),
  [131] = {.count = 1, .reusable = false}, SHIFT(74),
  [133] = {.count = 1, .reusable = true}, SHIFT(82),
  [135] = {.count = 1, .reusable = true}, SHIFT(83),
  [137] = {.count = 1, .reusable = true}, SHIFT(84),
  [139] = {.count = 1, .reusable = true}, SHIFT(85),
  [141] = {.count = 1, .reusable = true}, SHIFT(88),
  [143] = {.count = 1, .reusable = true}, SHIFT(89),
  [145] = {.count = 1, .reusable = true}, SHIFT(90),
  [147] = {.count = 1, .reusable = true}, SHIFT(95),
  [149] = {.count = 1, .reusable = true}, SHIFT(98),
  [151] = {.count = 1, .reusable = true}, SHIFT(99),
  [153] = {.count = 1, .reusable = true}, SHIFT(100),
  [155] = {.count = 1, .reusable = true}, SHIFT(101),
  [157] = {.count = 1, .reusable = false}, SHIFT(102),
  [159] = {.count = 1, .reusable = true}, SHIFT(103),
  [161] = {.count = 1, .reusable = true}, SHIFT(104),
  [163] = {.count = 1, .reusable = false}, SHIFT(105),
  [165] = {.count = 1, .reusable = false}, SHIFT(106),
  [167] = {.count = 1, .reusable = false}, SHIFT(107),
  [169] = {.count = 1, .reusable = true}, SHIFT(108),
  [171] = {.count = 1, .reusable = false}, SHIFT(109),
  [173] = {.count = 1, .reusable = false}, SHIFT(110),
  [175] = {.count = 1, .reusable = true}, SHIFT(111),
  [177] = {.count = 1, .reusable = false}, SHIFT(112),
  [179] = {.count = 1, .reusable = true}, SHIFT(112),
  [181] = {.count = 1, .reusable = true}, SHIFT(113),
  [183] = {.count = 1, .reusable = true}, SHIFT(119),
  [185] = {.count = 1, .reusable = true}, SHIFT(114),
  [187] = {.count = 1, .reusable = false}, SHIFT(119),
  [189] = {.count = 1, .reusable = false}, SHIFT(115),
  [191] = {.count = 1, .reusable = false}, SHIFT(116),
  [193] = {.count = 1, .reusable = false}, SHIFT(117),
  [195] = {.count = 1, .reusable = false}, SHIFT(129),
  [197] = {.count = 1, .reusable = true}, SHIFT(130),
  [199] = {.count = 1, .reusable = false}, SHIFT(131),
  [201] = {.count = 1, .reusable = false}, SHIFT(132),
  [203] = {.count = 1, .reusable = false}, SHIFT(133),
  [205] = {.count = 1, .reusable = false}, SHIFT(134),
  [207] = {.count = 1, .reusable = false}, SHIFT(135),
  [209] = {.count = 1, .reusable = false}, SHIFT(136),
  [211] = {.count = 1, .reusable = false}, SHIFT(137),
  [213] = {.count = 1, .reusable = false}, SHIFT(138),
  [215] = {.count = 1, .reusable = false}, SHIFT(139),
  [217] = {.count = 1, .reusable = true}, SHIFT(144),
  [219] = {.count = 1, .reusable = true}, SHIFT(147),
  [221] = {.count = 1, .reusable = true}, SHIFT(148),
  [223] = {.count = 1, .reusable = true}, SHIFT(149),
  [225] = {.count = 1, .reusable = true}, SHIFT(150),
  [227] = {.count = 1, .reusable = true}, SHIFT(151),
  [229] = {.count = 1, .reusable = true}, SHIFT(152),
  [231] = {.count = 1, .reusable = true}, SHIFT(153),
  [233] = {.count = 1, .reusable = false}, SHIFT(154),
  [235] = {.count = 1, .reusable = true}, REDUCE(sym_empty_statement, 1),
  [237] = {.count = 1, .reusable = false}, REDUCE(sym_empty_statement, 1),
  [239] = {.count = 1, .reusable = true}, REDUCE(sym_yield_expression, 1),
  [241] = {.count = 1, .reusable = false}, REDUCE(sym_yield_expression, 1),
  [243] = {.count = 1, .reusable = false}, SHIFT(24),
  [245] = {.count = 1, .reusable = true}, SHIFT(159),
  [247] = {.count = 1, .reusable = false}, SHIFT(159),
  [249] = {.count = 1, .reusable = true}, SHIFT(160),
  [251] = {.count = 1, .reusable = false}, SHIFT(161),
  [253] = {.count = 1, .reusable = true}, SHIFT(162),
  [255] = {.count = 1, .reusable = false}, SHIFT(163),
  [257] = {.count = 1, .reusable = true}, SHIFT(164),
  [259] = {.count = 1, .reusable = false}, SHIFT(165),
  [261] = {.count = 1, .reusable = false}, SHIFT(166),
  [263] = {.count = 1, .reusable = true}, SHIFT(167),
  [265] = {.count = 1, .reusable = false}, SHIFT(168),
  [267] = {.count = 1, .reusable = true}, SHIFT(168),
  [269] = {.count = 1, .reusable = true}, SHIFT(169),
  [271] = {.count = 1, .reusable = true}, SHIFT(172),
  [273] = {.count = 1, .reusable = false}, SHIFT(172),
  [275] = {.count = 1, .reusable = false}, SHIFT(170),
  [277] = {.count = 1, .reusable = false}, SHIFT(171),
  [279] = {.count = 1, .reusable = true}, SHIFT(179),
  [281] = {.count = 1, .reusable = true}, SHIFT(180),
  [283] = {.count = 1, .reusable = false}, SHIFT(181),
  [285] = {.count = 1, .reusable = true}, SHIFT(184),
  [287] = {.count = 1, .reusable = false}, SHIFT(185),
  [289] = {.count = 1, .reusable = false}, SHIFT(186),
  [291] = {.count = 1, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1),
  [293] = {.count = 1, .reusable = false}, REDUCE(sym__expression, 1, .alias_sequence_id = 1),
  [295] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), SHIFT(189),
  [298] = {.count = 1, .reusable = true}, SHIFT(190),
  [300] = {.count = 1, .reusable = false}, SHIFT(191),
  [302] = {.count = 1, .reusable = false}, SHIFT(192),
  [304] = {.count = 1, .reusable = true}, SHIFT(193),
  [306] = {.count = 1, .reusable = true}, SHIFT(194),
  [308] = {.count = 1, .reusable = false}, SHIFT(195),
  [310] = {.count = 1, .reusable = true}, SHIFT(197),
  [312] = {.count = 1, .reusable = true}, SHIFT(189),
  [314] = {.count = 1, .reusable = true}, SHIFT(198),
  [316] = {.count = 1, .reusable = true}, SHIFT(200),
  [318] = {.count = 1, .reusable = true}, SHIFT(202),
  [320] = {.count = 1, .reusable = false}, SHIFT(202),
  [322] = {.count = 1, .reusable = true}, SHIFT(203),
  [324] = {.count = 1, .reusable = false}, SHIFT(203),
  [326] = {.count = 1, .reusable = true}, SHIFT(204),
  [328] = {.count = 1, .reusable = false}, SHIFT(204),
  [330] = {.count = 1, .reusable = true}, SHIFT(205),
  [332] = {.count = 1, .reusable = false}, SHIFT(205),
  [334] = {.count = 1, .reusable = true}, SHIFT(206),
  [336] = {.count = 1, .reusable = false}, SHIFT(206),
  [338] = {.count = 1, .reusable = false}, SHIFT(207),
  [340] = {.count = 1, .reusable = false}, SHIFT(209),
  [342] = {.count = 1, .reusable = true}, SHIFT(208),
  [344] = {.count = 1, .reusable = true}, REDUCE(sym__expression, 1),
  [346] = {.count = 1, .reusable = false}, REDUCE(sym__expression, 1),
  [348] = {.count = 1, .reusable = false}, SHIFT(210),
  [350] = {.count = 1, .reusable = true}, SHIFT(211),
  [352] = {.count = 1, .reusable = true}, SHIFT(212),
  [354] = {.count = 1, .reusable = true}, SHIFT(213),
  [356] = {.count = 1, .reusable = true}, SHIFT(215),
  [358] = {.count = 1, .reusable = true}, SHIFT(216),
  [360] = {.count = 1, .reusable = false}, SHIFT(218),
  [362] = {.count = 1, .reusable = false}, SHIFT(219),
  [364] = {.count = 1, .reusable = true}, ACCEPT_INPUT(),
  [366] = {.count = 1, .reusable = true}, SHIFT(222),
  [368] = {.count = 1, .reusable = true}, REDUCE(sym__expressions, 1),
  [370] = {.count = 1, .reusable = true}, SHIFT(223),
  [372] = {.count = 1, .reusable = true}, SHIFT(224),
  [374] = {.count = 1, .reusable = false}, SHIFT(223),
  [376] = {.count = 1, .reusable = true}, SHIFT(225),
  [378] = {.count = 1, .reusable = true}, SHIFT(226),
  [380] = {.count = 1, .reusable = true}, SHIFT(227),
  [382] = {.count = 1, .reusable = false}, SHIFT(226),
  [384] = {.count = 1, .reusable = false}, SHIFT(227),
  [386] = {.count = 1, .reusable = false}, SHIFT(228),
  [388] = {.count = 1, .reusable = true}, SHIFT(210),
  [390] = {.count = 1, .reusable = true}, SHIFT(229),
  [392] = {.count = 1, .reusable = false}, SHIFT(232),
  [394] = {.count = 1, .reusable = false}, SHIFT(233),
  [396] = {.count = 1, .reusable = false}, SHIFT(236),
  [398] = {.count = 1, .reusable = true}, SHIFT(237),
  [400] = {.count = 1, .reusable = true}, REDUCE(sym__declaration, 1),
  [402] = {.count = 1, .reusable = false}, REDUCE(sym__declaration, 1),
  [404] = {.count = 1, .reusable = true}, SHIFT(239),
  [406] = {.count = 1, .reusable = true}, SHIFT(240),
  [408] = {.count = 1, .reusable = true}, REDUCE(sym_program, 2),
  [410] = {.count = 1, .reusable = true}, SHIFT(242),
  [412] = {.count = 1, .reusable = false}, SHIFT(244),
  [414] = {.count = 1, .reusable = false}, SHIFT(245),
  [416] = {.count = 1, .reusable = true}, SHIFT(246),
  [418] = {.count = 1, .reusable = false}, SHIFT(247),
  [420] = {.count = 1, .reusable = false}, SHIFT(248),
  [422] = {.count = 1, .reusable = false}, SHIFT(249),
  [424] = {.count = 1, .reusable = true}, SHIFT(249),
  [426] = {.count = 1, .reusable = true}, SHIFT(250),
  [428] = {.count = 1, .reusable = true}, SHIFT(253),
  [430] = {.count = 1, .reusable = false}, SHIFT(253),
  [432] = {.count = 1, .reusable = false}, SHIFT(251),
  [434] = {.count = 1, .reusable = false}, SHIFT(252),
  [436] = {.count = 1, .reusable = true}, SHIFT(258),
  [438] = {.count = 1, .reusable = true}, SHIFT(259),
  [440] = {.count = 1, .reusable = true}, SHIFT(260),
  [442] = {.count = 1, .reusable = true}, SHIFT(262),
  [444] = {.count = 1, .reusable = true}, SHIFT(263),
  [446] = {.count = 1, .reusable = true}, SHIFT(264),
  [448] = {.count = 1, .reusable = true}, SHIFT(265),
  [450] = {.count = 1, .reusable = true}, SHIFT(267),
  [452] = {.count = 1, .reusable = true}, REDUCE(sym_export_statement, 2),
  [454] = {.count = 1, .reusable = false}, REDUCE(sym_export_statement, 2),
  [456] = {.count = 1, .reusable = true}, SHIFT(268),
  [458] = {.count = 1, .reusable = true}, SHIFT(270),
  [460] = {.count = 1, .reusable = false}, SHIFT(271),
  [462] = {.count = 1, .reusable = true}, SHIFT(272),
  [464] = {.count = 1, .reusable = true}, SHIFT(273),
  [466] = {.count = 1, .reusable = true}, REDUCE(aux_sym_object_repeat1, 1),
  [468] = {.count = 1, .reusable = true}, SHIFT(274),
  [470] = {.count = 1, .reusable = false}, SHIFT(275),
  [472] = {.count = 1, .reusable = true}, SHIFT(78),
  [474] = {.count = 1, .reusable = false}, SHIFT(276),
  [476] = {.count = 1, .reusable = true}, SHIFT(277),
  [478] = {.count = 1, .reusable = false}, SHIFT(278),
  [480] = {.count = 1, .reusable = true}, REDUCE(sym_object, 2),
  [482] = {.count = 1, .reusable = true}, REDUCE(sym_statement_block, 2),
  [484] = {.count = 1, .reusable = false}, REDUCE(sym_statement_block, 2),
  [486] = {.count = 1, .reusable = false}, REDUCE(sym_object, 2),
  [488] = {.count = 1, .reusable = true}, SHIFT(284),
  [490] = {.count = 1, .reusable = true}, SHIFT(287),
  [492] = {.count = 1, .reusable = false}, SHIFT(287),
  [494] = {.count = 2, .reusable = false}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), SHIFT(288),
  [497] = {.count = 1, .reusable = true}, SHIFT(289),
  [499] = {.count = 3, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), REDUCE(sym__property_name, 1, .alias_sequence_id = 2), SHIFT(189),
  [503] = {.count = 1, .reusable = true}, REDUCE(sym__property_name, 1, .alias_sequence_id = 2),
  [505] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), SHIFT(270),
  [508] = {.count = 1, .reusable = false}, SHIFT(190),
  [510] = {.count = 1, .reusable = false}, SHIFT(290),
  [512] = {.count = 1, .reusable = false}, SHIFT(291),
  [514] = {.count = 1, .reusable = false}, SHIFT(293),
  [516] = {.count = 1, .reusable = false}, SHIFT(294),
  [518] = {.count = 1, .reusable = true}, SHIFT(295),
  [520] = {.count = 1, .reusable = false}, SHIFT(296),
  [522] = {.count = 1, .reusable = false}, SHIFT(297),
  [524] = {.count = 1, .reusable = false}, SHIFT(298),
  [526] = {.count = 1, .reusable = true}, SHIFT(298),
  [528] = {.count = 1, .reusable = true}, SHIFT(299),
  [530] = {.count = 1, .reusable = true}, SHIFT(302),
  [532] = {.count = 1, .reusable = false}, SHIFT(302),
  [534] = {.count = 1, .reusable = false}, SHIFT(300),
  [536] = {.count = 1, .reusable = false}, SHIFT(301),
  [538] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1), REDUCE(sym__property_name, 1),
  [541] = {.count = 1, .reusable = true}, REDUCE(sym__property_name, 1),
  [543] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1), REDUCE(sym__property_name, 1, .alias_sequence_id = 2),
  [546] = {.count = 2, .reusable = true}, REDUCE(sym__property_name, 1, .alias_sequence_id = 2), SHIFT(307),
  [549] = {.count = 1, .reusable = false}, SHIFT(308),
  [551] = {.count = 1, .reusable = false}, SHIFT(310),
  [553] = {.count = 1, .reusable = false}, SHIFT(311),
  [555] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), REDUCE(sym__property_name, 1, .alias_sequence_id = 2),
  [558] = {.count = 1, .reusable = true}, SHIFT(315),
  [560] = {.count = 1, .reusable = true}, SHIFT(318),
  [562] = {.count = 1, .reusable = true}, SHIFT(320),
  [564] = {.count = 1, .reusable = false}, SHIFT(321),
  [566] = {.count = 1, .reusable = false}, SHIFT(239),
  [568] = {.count = 1, .reusable = false}, SHIFT(240),
  [570] = {.count = 1, .reusable = false}, SHIFT(323),
  [572] = {.count = 1, .reusable = true}, SHIFT(326),
  [574] = {.count = 1, .reusable = false}, SHIFT(324),
  [576] = {.count = 1, .reusable = true}, SHIFT(329),
  [578] = {.count = 1, .reusable = true}, SHIFT(330),
  [580] = {.count = 1, .reusable = true}, SHIFT(331),
  [582] = {.count = 1, .reusable = true}, SHIFT(333),
  [584] = {.count = 1, .reusable = true}, SHIFT(334),
  [586] = {.count = 1, .reusable = true}, REDUCE(sym_import_clause, 1),
  [588] = {.count = 1, .reusable = true}, SHIFT(336),
  [590] = {.count = 1, .reusable = false}, SHIFT(337),
  [592] = {.count = 1, .reusable = false}, SHIFT(338),
  [594] = {.count = 1, .reusable = false}, SHIFT(339),
  [596] = {.count = 1, .reusable = true}, SHIFT(343),
  [598] = {.count = 1, .reusable = true}, SHIFT(344),
  [600] = {.count = 1, .reusable = false}, SHIFT(344),
  [602] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declarator, 1),
  [604] = {.count = 1, .reusable = true}, SHIFT(191),
  [606] = {.count = 1, .reusable = true}, SHIFT(348),
  [608] = {.count = 1, .reusable = true}, SHIFT(349),
  [610] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declarator, 1, .alias_sequence_id = 3),
  [612] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declarator, 1, .alias_sequence_id = 4),
  [614] = {.count = 1, .reusable = true}, SHIFT(353),
  [616] = {.count = 1, .reusable = false}, SHIFT(355),
  [618] = {.count = 1, .reusable = false}, SHIFT(356),
  [620] = {.count = 1, .reusable = false}, SHIFT(357),
  [622] = {.count = 1, .reusable = false}, SHIFT(361),
  [624] = {.count = 1, .reusable = true}, SHIFT(362),
  [626] = {.count = 1, .reusable = false}, SHIFT(363),
  [628] = {.count = 1, .reusable = false}, SHIFT(364),
  [630] = {.count = 1, .reusable = false}, SHIFT(365),
  [632] = {.count = 1, .reusable = false}, SHIFT(366),
  [634] = {.count = 1, .reusable = false}, SHIFT(367),
  [636] = {.count = 1, .reusable = false}, SHIFT(368),
  [638] = {.count = 1, .reusable = false}, SHIFT(369),
  [640] = {.count = 1, .reusable = false}, SHIFT(370),
  [642] = {.count = 1, .reusable = false}, SHIFT(371),
  [644] = {.count = 1, .reusable = false}, SHIFT(372),
  [646] = {.count = 1, .reusable = false}, SHIFT(373),
  [648] = {.count = 1, .reusable = false}, SHIFT(374),
  [650] = {.count = 1, .reusable = false}, SHIFT(375),
  [652] = {.count = 1, .reusable = false}, SHIFT(376),
  [654] = {.count = 1, .reusable = false}, SHIFT(377),
  [656] = {.count = 1, .reusable = true}, SHIFT(378),
  [658] = {.count = 1, .reusable = false}, SHIFT(379),
  [660] = {.count = 1, .reusable = false}, SHIFT(380),
  [662] = {.count = 1, .reusable = false}, SHIFT(381),
  [664] = {.count = 1, .reusable = false}, SHIFT(382),
  [666] = {.count = 1, .reusable = true}, SHIFT(388),
  [668] = {.count = 1, .reusable = false}, SHIFT(390),
  [670] = {.count = 1, .reusable = false}, SHIFT(391),
  [672] = {.count = 1, .reusable = true}, SHIFT(392),
  [674] = {.count = 1, .reusable = false}, SHIFT(393),
  [676] = {.count = 1, .reusable = true}, SHIFT(398),
  [678] = {.count = 1, .reusable = false}, SHIFT(399),
  [680] = {.count = 1, .reusable = false}, SHIFT(400),
  [682] = {.count = 1, .reusable = false}, SHIFT(401),
  [684] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 2),
  [686] = {.count = 1, .reusable = false}, SHIFT(104),
  [688] = {.count = 1, .reusable = true}, SHIFT(406),
  [690] = {.count = 1, .reusable = false}, SHIFT(406),
  [692] = {.count = 1, .reusable = true}, SHIFT(407),
  [694] = {.count = 1, .reusable = true}, SHIFT(408),
  [696] = {.count = 1, .reusable = false}, SHIFT(408),
  [698] = {.count = 1, .reusable = true}, SHIFT(411),
  [700] = {.count = 1, .reusable = true}, SHIFT(412),
  [702] = {.count = 1, .reusable = false}, SHIFT(413),
  [704] = {.count = 1, .reusable = true}, SHIFT(416),
  [706] = {.count = 1, .reusable = false}, SHIFT(417),
  [708] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), SHIFT(420),
  [711] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1, .alias_sequence_id = 1), SHIFT(421),
  [714] = {.count = 1, .reusable = true}, SHIFT(422),
  [716] = {.count = 1, .reusable = false}, SHIFT(423),
  [718] = {.count = 1, .reusable = false}, SHIFT(424),
  [720] = {.count = 1, .reusable = true}, SHIFT(425),
  [722] = {.count = 1, .reusable = true}, SHIFT(426),
  [724] = {.count = 1, .reusable = false}, SHIFT(427),
  [726] = {.count = 1, .reusable = true}, SHIFT(430),
  [728] = {.count = 1, .reusable = true}, SHIFT(431),
  [730] = {.count = 1, .reusable = true}, SHIFT(433),
  [732] = {.count = 1, .reusable = true}, SHIFT(435),
  [734] = {.count = 1, .reusable = false}, SHIFT(435),
  [736] = {.count = 1, .reusable = true}, SHIFT(436),
  [738] = {.count = 1, .reusable = false}, SHIFT(436),
  [740] = {.count = 1, .reusable = true}, SHIFT(437),
  [742] = {.count = 1, .reusable = false}, SHIFT(437),
  [744] = {.count = 1, .reusable = true}, SHIFT(438),
  [746] = {.count = 1, .reusable = true}, SHIFT(439),
  [748] = {.count = 1, .reusable = false}, SHIFT(439),
  [750] = {.count = 1, .reusable = true}, SHIFT(440),
  [752] = {.count = 1, .reusable = false}, SHIFT(440),
  [754] = {.count = 1, .reusable = false}, SHIFT(441),
  [756] = {.count = 1, .reusable = false}, SHIFT(442),
  [758] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1), SHIFT(420),
  [761] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1), SHIFT(443),
  [764] = {.count = 1, .reusable = false}, SHIFT(444),
  [766] = {.count = 1, .reusable = true}, SHIFT(445),
  [768] = {.count = 1, .reusable = true}, SHIFT(446),
  [770] = {.count = 1, .reusable = true}, SHIFT(449),
  [772] = {.count = 1, .reusable = true}, SHIFT(450),
  [774] = {.count = 1, .reusable = true}, SHIFT(452),
  [776] = {.count = 1, .reusable = true}, SHIFT(453),
  [778] = {.count = 1, .reusable = true}, SHIFT(454),
  [780] = {.count = 1, .reusable = false}, SHIFT(453),
  [782] = {.count = 1, .reusable = true}, SHIFT(455),
  [784] = {.count = 1, .reusable = true}, SHIFT(456),
  [786] = {.count = 1, .reusable = true}, SHIFT(457),
  [788] = {.count = 1, .reusable = false}, SHIFT(456),
  [790] = {.count = 1, .reusable = false}, SHIFT(457),
  [792] = {.count = 1, .reusable = false}, SHIFT(458),
  [794] = {.count = 1, .reusable = true}, SHIFT(444),
  [796] = {.count = 1, .reusable = true}, SHIFT(459),
  [798] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1), SHIFT(460),
  [801] = {.count = 1, .reusable = true}, SHIFT(420),
  [803] = {.count = 1, .reusable = true}, SHIFT(443),
  [805] = {.count = 2, .reusable = true}, REDUCE(sym__expression, 1), SHIFT(463),
  [808] = {.count = 1, .reusable = false}, SHIFT(466),
  [810] = {.count = 1, .reusable = false}, SHIFT(468),
  [812] = {.count = 1, .reusable = true}, SHIFT(470),
  [814] = {.count = 1, .reusable = true}, SHIFT(472),
  [816] = {.count = 1, .reusable = true}, SHIFT(473),
  [818] = {.count = 1, .reusable = true}, SHIFT(474),
  [820] = {.count = 1, .reusable = true}, SHIFT(476),
  [822] = {.count = 1, .reusable = true}, SHIFT(478),
  [824] = {.count = 1, .reusable = false}, SHIFT(482),
  [826] = {.count = 1, .reusable = false}, SHIFT(483),
  [828] = {.count = 1, .reusable = true}, SHIFT(484),
  [830] = {.count = 1, .reusable = true}, SHIFT(485),
  [832] = {.count = 1, .reusable = true}, SHIFT(487),
  [834] = {.count = 1, .reusable = true}, SHIFT(488),
  [836] = {.count = 1, .reusable = true}, SHIFT(489),
  [838] = {.count = 1, .reusable = true}, SHIFT(490),
  [840] = {.count = 1, .reusable = true}, SHIFT(491),
  [842] = {.count = 1, .reusable = true}, REDUCE(sym_try_statement, 2),
  [844] = {.count = 1, .reusable = false}, REDUCE(sym_try_statement, 2),
  [846] = {.count = 1, .reusable = false}, SHIFT(493),
  [848] = {.count = 1, .reusable = false}, SHIFT(494),
  [850] = {.count = 1, .reusable = true}, REDUCE(sym_break_statement, 2),
  [852] = {.count = 1, .reusable = false}, REDUCE(sym_break_statement, 2),
  [854] = {.count = 1, .reusable = true}, SHIFT(498),
  [856] = {.count = 1, .reusable = true}, REDUCE(sym_continue_statement, 2),
  [858] = {.count = 1, .reusable = false}, REDUCE(sym_continue_statement, 2),
  [860] = {.count = 1, .reusable = true}, SHIFT(499),
  [862] = {.count = 1, .reusable = true}, REDUCE(sym_debugger_statement, 2),
  [864] = {.count = 1, .reusable = false}, REDUCE(sym_debugger_statement, 2),
  [866] = {.count = 1, .reusable = true}, REDUCE(sym_return_statement, 2),
  [868] = {.count = 1, .reusable = false}, REDUCE(sym_return_statement, 2),
  [870] = {.count = 1, .reusable = true}, SHIFT(500),
  [872] = {.count = 1, .reusable = false}, SHIFT(501),
  [874] = {.count = 1, .reusable = false}, SHIFT(502),
  [876] = {.count = 1, .reusable = false}, SHIFT(503),
  [878] = {.count = 1, .reusable = true}, SHIFT(505),
  [880] = {.count = 1, .reusable = true}, SHIFT(506),
  [882] = {.count = 1, .reusable = true}, REDUCE(sym_yield_expression, 2),
  [884] = {.count = 1, .reusable = true}, REDUCE(aux_sym_array_repeat1, 1),
  [886] = {.count = 1, .reusable = true}, SHIFT(507),
  [888] = {.count = 1, .reusable = false}, SHIFT(507),
  [890] = {.count = 1, .reusable = true}, SHIFT(509),
  [892] = {.count = 1, .reusable = false}, SHIFT(509),
  [894] = {.count = 1, .reusable = true}, REDUCE(sym_array, 2),
  [896] = {.count = 1, .reusable = false}, REDUCE(sym_array, 2),
  [898] = {.count = 1, .reusable = false}, SHIFT(510),
  [900] = {.count = 1, .reusable = true}, SHIFT(511),
  [902] = {.count = 1, .reusable = true}, SHIFT(512),
  [904] = {.count = 1, .reusable = false}, SHIFT(513),
  [906] = {.count = 1, .reusable = true}, SHIFT(514),
  [908] = {.count = 1, .reusable = false}, SHIFT(514),
  [910] = {.count = 1, .reusable = true}, SHIFT(515),
  [912] = {.count = 1, .reusable = false}, SHIFT(515),
  [914] = {.count = 1, .reusable = true}, SHIFT(516),
  [916] = {.count = 1, .reusable = false}, SHIFT(516),
  [918] = {.count = 1, .reusable = false}, SHIFT(517),
  [920] = {.count = 1, .reusable = true}, SHIFT(518),
  [922] = {.count = 1, .reusable = true}, SHIFT(519),
  [924] = {.count = 1, .reusable = true}, SHIFT(520),
  [926] = {.count = 1, .reusable = false}, SHIFT(519),
  [928] = {.count = 1, .reusable = true}, SHIFT(521),
  [930] = {.count = 1, .reusable = true}, SHIFT(522),
  [932] = {.count = 1, .reusable = true}, SHIFT(523),
  [934] = {.count = 1, .reusable = false}, SHIFT(522),
  [936] = {.count = 1, .reusable = false}, SHIFT(523),
  [938] = {.count = 1, .reusable = false}, SHIFT(524),
  [940] = {.count = 1, .reusable = true}, SHIFT(517),
  [942] = {.count = 1, .reusable = false}, SHIFT(529),
  [944] = {.count = 1, .reusable = false}, SHIFT(530),
  [946] = {.count = 1, .reusable = true}, SHIFT(531),
  [948] = {.count = 1, .reusable = true}, SHIFT(532),
  [950] = {.count = 1, .reusable = true}, SHIFT(533),
  [952] = {.count = 1, .reusable = false}, SHIFT(534),
  [954] = {.count = 1, .reusable = true}, SHIFT(535),
  [956] = {.count = 1, .reusable = false}, SHIFT(536),
  [958] = {.count = 1, .reusable = true}, SHIFT(539),
  [960] = {.count = 1, .reusable = true}, SHIFT(540),
  [962] = {.count = 1, .reusable = false}, SHIFT(541),
  [964] = {.count = 1, .reusable = true}, SHIFT(542),
  [966] = {.count = 1, .reusable = true}, SHIFT(544),
  [968] = {.count = 1, .reusable = true}, SHIFT(545),
  [970] = {.count = 1, .reusable = true}, SHIFT(546),
  [972] = {.count = 1, .reusable = false}, SHIFT(547),
  [974] = {.count = 1, .reusable = true}, SHIFT(554),
  [976] = {.count = 1, .reusable = false}, SHIFT(548),
  [978] = {.count = 1, .reusable = true}, SHIFT(549),
  [980] = {.count = 1, .reusable = false}, SHIFT(550),
  [982] = {.count = 1, .reusable = false}, SHIFT(551),
  [984] = {.count = 1, .reusable = false}, SHIFT(557),
  [986] = {.count = 1, .reusable = false}, SHIFT(558),
  [988] = {.count = 1, .reusable = true}, SHIFT(559),
  [990] = {.count = 1, .reusable = false}, SHIFT(560),
  [992] = {.count = 1, .reusable = false}, SHIFT(561),
  [994] = {.count = 1, .reusable = false}, SHIFT(562),
  [996] = {.count = 1, .reusable = true}, SHIFT(562),
  [998] = {.count = 1, .reusable = true}, SHIFT(563),
  [1000] = {.count = 1, .reusable = true}, SHIFT(566),
  [1002] = {.count = 1, .reusable = false}, SHIFT(566),
  [1004] = {.count = 1, .reusable = false}, SHIFT(564),
  [1006] = {.count = 1, .reusable = false}, SHIFT(565),
  [1008] = {.count = 1, .reusable = true}, SHIFT(571),
  [1010] = {.count = 1, .reusable = true}, SHIFT(185),
  [1012] = {.count = 1, .reusable = true}, REDUCE(sym_anonymous_class, 2),
  [1014] = {.count = 1, .reusable = false}, REDUCE(sym_anonymous_class, 2),
  [1016] = {.count = 1, .reusable = true}, SHIFT(575),
  [1018] = {.count = 1, .reusable = false}, SHIFT(576),
  [1020] = {.count = 1, .reusable = false}, SHIFT(577),
  [1022] = {.count = 1, .reusable = true}, SHIFT(580),
  [1024] = {.count = 1, .reusable = true}, SHIFT(581),
  [1026] = {.count = 1, .reusable = false}, SHIFT(581),
  [1028] = {.count = 1, .reusable = true}, SHIFT(582),
  [1030] = {.count = 1, .reusable = true}, SHIFT(585),
  [1032] = {.count = 1, .reusable = false}, SHIFT(585),
  [1034] = {.count = 1, .reusable = true}, SHIFT(586),
  [1036] = {.count = 1, .reusable = false}, SHIFT(586),
  [1038] = {.count = 1, .reusable = true}, SHIFT(587),
  [1040] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 1),
  [1042] = {.count = 1, .reusable = false}, REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 1),
  [1044] = {.count = 1, .reusable = true}, SHIFT(588),
  [1046] = {.count = 1, .reusable = true}, SHIFT(590),
  [1048] = {.count = 1, .reusable = true}, REDUCE(sym_arrow_function, 2),
  [1050] = {.count = 1, .reusable = false}, REDUCE(sym_arrow_function, 2),
  [1052] = {.count = 1, .reusable = true}, REDUCE(sym_new_expression, 2),
  [1054] = {.count = 1, .reusable = false}, REDUCE(sym_new_expression, 2),
  [1056] = {.count = 1, .reusable = true}, REDUCE(sym_await_expression, 2),
  [1058] = {.count = 1, .reusable = true}, REDUCE(sym_unary_expression, 2),
  [1060] = {.count = 1, .reusable = false}, REDUCE(sym_unary_expression, 2),
  [1062] = {.count = 1, .reusable = true}, REDUCE(sym_update_expression, 2),
  [1064] = {.count = 1, .reusable = false}, REDUCE(sym_update_expression, 2),
  [1066] = {.count = 1, .reusable = true}, REDUCE(sym_template_string, 2),
  [1068] = {.count = 1, .reusable = false}, REDUCE(sym_template_string, 2),
  [1070] = {.count = 1, .reusable = true}, SHIFT(594),
  [1072] = {.count = 1, .reusable = false}, SHIFT(594),
  [1074] = {.count = 1, .reusable = false}, SHIFT(595),
  [1076] = {.count = 1, .reusable = false}, SHIFT(596),
  [1078] = {.count = 1, .reusable = true}, SHIFT(597),
  [1080] = {.count = 1, .reusable = false}, SHIFT(597),
  [1082] = {.count = 1, .reusable = true}, SHIFT(599),
  [1084] = {.count = 1, .reusable = true}, SHIFT(600),
  [1086] = {.count = 1, .reusable = false}, SHIFT(600),
  [1088] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_expression, 2),
  [1090] = {.count = 1, .reusable = false}, REDUCE(sym_assignment_expression, 2),
  [1092] = {.count = 1, .reusable = true}, SHIFT(601),
  [1094] = {.count = 1, .reusable = true}, SHIFT(602),
  [1096] = {.count = 1, .reusable = true}, SHIFT(603),
  [1098] = {.count = 1, .reusable = true}, SHIFT(604),
  [1100] = {.count = 1, .reusable = false}, SHIFT(604),
  [1102] = {.count = 1, .reusable = true}, SHIFT(608),
  [1104] = {.count = 1, .reusable = false}, SHIFT(608),
  [1106] = {.count = 1, .reusable = true}, REDUCE(sym_call_expression, 2),
  [1108] = {.count = 1, .reusable = false}, REDUCE(sym_call_expression, 2),
  [1110] = {.count = 1, .reusable = true}, REDUCE(sym__identifier_reference, 1, .alias_sequence_id = 1),
  [1112] = {.count = 1, .reusable = true}, REDUCE(sym_decorator, 2),
  [1114] = {.count = 1, .reusable = true}, SHIFT(609),
  [1116] = {.count = 1, .reusable = true}, REDUCE(sym_decorator, 2, .alias_sequence_id = 5),
  [1118] = {.count = 1, .reusable = true}, SHIFT(611),
  [1120] = {.count = 1, .reusable = true}, REDUCE(sym_decorator, 2, .alias_sequence_id = 6),
  [1122] = {.count = 1, .reusable = true}, REDUCE(sym_expression_statement, 2),
  [1124] = {.count = 1, .reusable = false}, REDUCE(sym_expression_statement, 2),
  [1126] = {.count = 1, .reusable = true}, SHIFT(613),
  [1128] = {.count = 1, .reusable = false}, SHIFT(613),
  [1130] = {.count = 1, .reusable = true}, SHIFT(614),
  [1132] = {.count = 1, .reusable = false}, SHIFT(614),
  [1134] = {.count = 1, .reusable = false}, SHIFT(616),
  [1136] = {.count = 1, .reusable = false}, SHIFT(617),
  [1138] = {.count = 1, .reusable = true}, SHIFT(618),
  [1140] = {.count = 1, .reusable = false}, SHIFT(619),
  [1142] = {.count = 1, .reusable = false}, SHIFT(620),
  [1144] = {.count = 1, .reusable = false}, SHIFT(621),
  [1146] = {.count = 1, .reusable = true}, SHIFT(621),
  [1148] = {.count = 1, .reusable = true}, SHIFT(622),
  [1150] = {.count = 1, .reusable = true}, SHIFT(625),
  [1152] = {.count = 1, .reusable = false}, SHIFT(625),
  [1154] = {.count = 1, .reusable = false}, SHIFT(623),
  [1156] = {.count = 1, .reusable = false}, SHIFT(624),
  [1158] = {.count = 1, .reusable = true}, SHIFT(630),
  [1160] = {.count = 1, .reusable = false}, SHIFT(630),
  [1162] = {.count = 1, .reusable = true}, SHIFT(631),
  [1164] = {.count = 1, .reusable = false}, SHIFT(631),
  [1166] = {.count = 1, .reusable = true}, SHIFT(632),
  [1168] = {.count = 1, .reusable = false}, SHIFT(632),
  [1170] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 3),
  [1172] = {.count = 1, .reusable = false}, REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 3),
  [1174] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 4),
  [1176] = {.count = 1, .reusable = false}, REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 4),
  [1178] = {.count = 1, .reusable = true}, SHIFT(633),
  [1180] = {.count = 1, .reusable = true}, SHIFT(634),
  [1182] = {.count = 1, .reusable = true}, SHIFT(635),
  [1184] = {.count = 1, .reusable = false}, SHIFT(635),
  [1186] = {.count = 1, .reusable = false}, SHIFT(637),
  [1188] = {.count = 1, .reusable = true}, SHIFT(638),
  [1190] = {.count = 1, .reusable = false}, SHIFT(639),
  [1192] = {.count = 1, .reusable = false}, SHIFT(642),
  [1194] = {.count = 1, .reusable = false}, SHIFT(644),
  [1196] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_element, 2),
  [1198] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_element, 2),
  [1200] = {.count = 1, .reusable = false}, SHIFT(646),
  [1202] = {.count = 1, .reusable = true}, REDUCE(sym__declaration, 2),
  [1204] = {.count = 1, .reusable = false}, REDUCE(sym__declaration, 2),
  [1206] = {.count = 1, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2),
  [1208] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(3),
  [1211] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(4),
  [1214] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(5),
  [1217] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(6),
  [1220] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(7),
  [1223] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(8),
  [1226] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(9),
  [1229] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(10),
  [1232] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(11),
  [1235] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(12),
  [1238] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(13),
  [1241] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(14),
  [1244] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(15),
  [1247] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(16),
  [1250] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(17),
  [1253] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(18),
  [1256] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(19),
  [1259] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(20),
  [1262] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(21),
  [1265] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(22),
  [1268] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(23),
  [1271] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(24),
  [1274] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(25),
  [1277] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(26),
  [1280] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(27),
  [1283] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(28),
  [1286] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(29),
  [1289] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(30),
  [1292] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(31),
  [1295] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(31),
  [1298] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(32),
  [1301] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(40),
  [1304] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(33),
  [1307] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(40),
  [1310] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(34),
  [1313] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(35),
  [1316] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(36),
  [1319] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(37),
  [1322] = {.count = 1, .reusable = true}, SHIFT(647),
  [1324] = {.count = 1, .reusable = false}, SHIFT(648),
  [1326] = {.count = 1, .reusable = true}, REDUCE(aux_sym_export_statement_repeat1, 2),
  [1328] = {.count = 2, .reusable = true}, REDUCE(aux_sym_export_statement_repeat1, 2), SHIFT_REPEAT(36),
  [1331] = {.count = 1, .reusable = true}, SHIFT(650),
  [1333] = {.count = 1, .reusable = true}, SHIFT(651),
  [1335] = {.count = 1, .reusable = true}, SHIFT(652),
  [1337] = {.count = 1, .reusable = false}, SHIFT(652),
  [1339] = {.count = 1, .reusable = false}, SHIFT(653),
  [1341] = {.count = 1, .reusable = true}, SHIFT(654),
  [1343] = {.count = 1, .reusable = true}, SHIFT(655),
  [1345] = {.count = 1, .reusable = false}, SHIFT(656),
  [1347] = {.count = 1, .reusable = true}, SHIFT(657),
  [1349] = {.count = 1, .reusable = false}, SHIFT(657),
  [1351] = {.count = 1, .reusable = true}, SHIFT(658),
  [1353] = {.count = 1, .reusable = false}, SHIFT(658),
  [1355] = {.count = 1, .reusable = false}, SHIFT(659),
  [1357] = {.count = 1, .reusable = true}, SHIFT(660),
  [1359] = {.count = 1, .reusable = true}, SHIFT(661),
  [1361] = {.count = 1, .reusable = false}, SHIFT(661),
  [1363] = {.count = 1, .reusable = true}, SHIFT(662),
  [1365] = {.count = 1, .reusable = true}, SHIFT(663),
  [1367] = {.count = 1, .reusable = true}, SHIFT(664),
  [1369] = {.count = 1, .reusable = false}, SHIFT(663),
  [1371] = {.count = 1, .reusable = false}, SHIFT(664),
  [1373] = {.count = 1, .reusable = false}, SHIFT(665),
  [1375] = {.count = 1, .reusable = true}, SHIFT(659),
  [1377] = {.count = 1, .reusable = true}, SHIFT(666),
  [1379] = {.count = 1, .reusable = true}, REDUCE(sym_export_clause, 2),
  [1381] = {.count = 1, .reusable = true}, REDUCE(sym__import_export_specifier, 1),
  [1383] = {.count = 1, .reusable = true}, SHIFT(667),
  [1385] = {.count = 1, .reusable = true}, SHIFT(668),
  [1387] = {.count = 1, .reusable = true}, SHIFT(669),
  [1389] = {.count = 1, .reusable = true}, SHIFT(671),
  [1391] = {.count = 1, .reusable = true}, SHIFT(674),
  [1393] = {.count = 1, .reusable = true}, SHIFT(676),
  [1395] = {.count = 1, .reusable = true}, SHIFT(678),
  [1397] = {.count = 1, .reusable = true}, REDUCE(sym_export_statement, 3),
  [1399] = {.count = 1, .reusable = false}, REDUCE(sym_export_statement, 3),
  [1401] = {.count = 1, .reusable = true}, SHIFT(680),
  [1403] = {.count = 1, .reusable = false}, SHIFT(681),
  [1405] = {.count = 1, .reusable = false}, SHIFT(682),
  [1407] = {.count = 1, .reusable = true}, SHIFT(683),
  [1409] = {.count = 1, .reusable = false}, SHIFT(684),
  [1411] = {.count = 1, .reusable = false}, SHIFT(685),
  [1413] = {.count = 1, .reusable = false}, SHIFT(686),
  [1415] = {.count = 1, .reusable = true}, SHIFT(686),
  [1417] = {.count = 1, .reusable = true}, SHIFT(687),
  [1419] = {.count = 1, .reusable = true}, SHIFT(690),
  [1421] = {.count = 1, .reusable = false}, SHIFT(690),
  [1423] = {.count = 1, .reusable = false}, SHIFT(688),
  [1425] = {.count = 1, .reusable = false}, SHIFT(689),
  [1427] = {.count = 1, .reusable = true}, SHIFT(696),
  [1429] = {.count = 1, .reusable = false}, SHIFT(697),
  [1431] = {.count = 1, .reusable = false}, SHIFT(698),
  [1433] = {.count = 1, .reusable = false}, SHIFT(699),
  [1435] = {.count = 1, .reusable = true}, SHIFT(703),
  [1437] = {.count = 1, .reusable = true}, SHIFT(704),
  [1439] = {.count = 1, .reusable = false}, SHIFT(704),
  [1441] = {.count = 1, .reusable = true}, SHIFT(288),
  [1443] = {.count = 1, .reusable = true}, REDUCE(aux_sym_object_repeat1, 2, .alias_sequence_id = 7),
  [1445] = {.count = 1, .reusable = true}, SHIFT(707),
  [1447] = {.count = 1, .reusable = false}, SHIFT(709),
  [1449] = {.count = 1, .reusable = false}, SHIFT(710),
  [1451] = {.count = 1, .reusable = true}, REDUCE(aux_sym_object_repeat1, 2),
  [1453] = {.count = 1, .reusable = false}, SHIFT(717),
  [1455] = {.count = 1, .reusable = false}, SHIFT(718),
  [1457] = {.count = 1, .reusable = false}, SHIFT(719),
  [1459] = {.count = 1, .reusable = false}, SHIFT(720),
  [1461] = {.count = 1, .reusable = false}, SHIFT(721),
  [1463] = {.count = 1, .reusable = false}, SHIFT(723),
  [1465] = {.count = 1, .reusable = false}, SHIFT(724),
  [1467] = {.count = 1, .reusable = false}, SHIFT(725),
  [1469] = {.count = 1, .reusable = true}, SHIFT(729),
  [1471] = {.count = 1, .reusable = true}, SHIFT(730),
  [1473] = {.count = 1, .reusable = true}, REDUCE(sym_object, 3, .alias_sequence_id = 7),
  [1475] = {.count = 1, .reusable = false}, REDUCE(sym_object, 3, .alias_sequence_id = 7),
  [1477] = {.count = 1, .reusable = true}, SHIFT(731),
  [1479] = {.count = 1, .reusable = true}, SHIFT(732),
  [1481] = {.count = 1, .reusable = false}, SHIFT(732),
  [1483] = {.count = 1, .reusable = false}, SHIFT(707),
  [1485] = {.count = 1, .reusable = true}, SHIFT(733),
  [1487] = {.count = 1, .reusable = true}, SHIFT(734),
  [1489] = {.count = 1, .reusable = false}, SHIFT(735),
  [1491] = {.count = 1, .reusable = true}, SHIFT(736),
  [1493] = {.count = 1, .reusable = false}, SHIFT(736),
  [1495] = {.count = 1, .reusable = true}, SHIFT(737),
  [1497] = {.count = 1, .reusable = false}, SHIFT(737),
  [1499] = {.count = 1, .reusable = false}, SHIFT(738),
  [1501] = {.count = 1, .reusable = true}, SHIFT(739),
  [1503] = {.count = 1, .reusable = true}, SHIFT(740),
  [1505] = {.count = 1, .reusable = true}, REDUCE(sym_spread_element, 2),
  [1507] = {.count = 1, .reusable = false}, SHIFT(740),
  [1509] = {.count = 1, .reusable = true}, SHIFT(741),
  [1511] = {.count = 1, .reusable = true}, SHIFT(742),
  [1513] = {.count = 1, .reusable = true}, SHIFT(743),
  [1515] = {.count = 1, .reusable = false}, SHIFT(742),
  [1517] = {.count = 1, .reusable = false}, SHIFT(743),
  [1519] = {.count = 1, .reusable = false}, SHIFT(744),
  [1521] = {.count = 1, .reusable = true}, SHIFT(738),
  [1523] = {.count = 1, .reusable = false}, SHIFT(745),
  [1525] = {.count = 1, .reusable = false}, SHIFT(746),
  [1527] = {.count = 1, .reusable = true}, SHIFT(747),
  [1529] = {.count = 1, .reusable = false}, SHIFT(748),
  [1531] = {.count = 1, .reusable = false}, SHIFT(749),
  [1533] = {.count = 1, .reusable = false}, SHIFT(750),
  [1535] = {.count = 1, .reusable = true}, SHIFT(750),
  [1537] = {.count = 1, .reusable = true}, SHIFT(751),
  [1539] = {.count = 1, .reusable = true}, SHIFT(754),
  [1541] = {.count = 1, .reusable = false}, SHIFT(754),
  [1543] = {.count = 1, .reusable = false}, SHIFT(752),
  [1545] = {.count = 1, .reusable = false}, SHIFT(753),
  [1547] = {.count = 2, .reusable = true}, REDUCE(sym_assignment_pattern, 2, .alias_sequence_id = 8), REDUCE(sym_assignment_expression, 2),
  [1550] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_pattern, 2, .alias_sequence_id = 8),
  [1552] = {.count = 1, .reusable = false}, REDUCE(sym__identifier_reference, 1, .alias_sequence_id = 1),
  [1554] = {.count = 1, .reusable = false}, REDUCE(sym_decorator, 2),
  [1556] = {.count = 1, .reusable = true}, SHIFT(759),
  [1558] = {.count = 1, .reusable = false}, SHIFT(760),
  [1560] = {.count = 1, .reusable = false}, REDUCE(sym_decorator, 2, .alias_sequence_id = 5),
  [1562] = {.count = 1, .reusable = false}, SHIFT(762),
  [1564] = {.count = 1, .reusable = false}, REDUCE(sym_decorator, 2, .alias_sequence_id = 6),
  [1566] = {.count = 2, .reusable = true}, REDUCE(sym_assignment_pattern, 2, .alias_sequence_id = 3), REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 3),
  [1569] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_pattern, 2, .alias_sequence_id = 3),
  [1571] = {.count = 1, .reusable = true}, REDUCE(sym_object, 3),
  [1573] = {.count = 1, .reusable = false}, REDUCE(sym_object, 3),
  [1575] = {.count = 1, .reusable = true}, SHIFT(764),
  [1577] = {.count = 2, .reusable = true}, REDUCE(sym_assignment_pattern, 2, .alias_sequence_id = 4), REDUCE(sym_assignment_expression, 2, .alias_sequence_id = 4),
  [1580] = {.count = 1, .reusable = true}, REDUCE(sym_assignment_pattern, 2, .alias_sequence_id = 4),
  [1582] = {.count = 1, .reusable = true}, SHIFT(765),
  [1584] = {.count = 1, .reusable = false}, SHIFT(765),
  [1586] = {.count = 1, .reusable = true}, SHIFT(766),
  [1588] = {.count = 1, .reusable = true}, REDUCE(sym_statement_block, 3),
  [1590] = {.count = 1, .reusable = false}, REDUCE(sym_statement_block, 3),
  [1592] = {.count = 1, .reusable = true}, SHIFT(307),
  [1594] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(64),
  [1597] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(65),
  [1600] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(66),
  [1603] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(67),
  [1606] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(321),
  [1609] = {.count = 1, .reusable = false}, REDUCE(aux_sym_export_statement_repeat1, 2),
  [1611] = {.count = 2, .reusable = true}, REDUCE(aux_sym_export_statement_repeat1, 2), SHIFT_REPEAT(73),
  [1614] = {.count = 2, .reusable = true}, REDUCE(aux_sym_object_repeat1, 2), SHIFT_REPEAT(62),
  [1617] = {.count = 1, .reusable = true}, SHIFT(769),
  [1619] = {.count = 1, .reusable = true}, SHIFT(770),
  [1621] = {.count = 1, .reusable = true}, REDUCE(sym_named_imports, 2),
  [1623] = {.count = 1, .reusable = true}, SHIFT(771),
  [1625] = {.count = 1, .reusable = true}, SHIFT(772),
  [1627] = {.count = 1, .reusable = true}, REDUCE(sym_import_statement, 3),
  [1629] = {.count = 1, .reusable = false}, REDUCE(sym_import_statement, 3),
  [1631] = {.count = 1, .reusable = true}, SHIFT(775),
  [1633] = {.count = 1, .reusable = true}, SHIFT(776),
  [1635] = {.count = 1, .reusable = true}, SHIFT(778),
  [1637] = {.count = 1, .reusable = true}, SHIFT(781),
  [1639] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declarator, 2),
  [1641] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declaration, 3),
  [1643] = {.count = 1, .reusable = false}, REDUCE(sym_variable_declaration, 3),
  [1645] = {.count = 1, .reusable = true}, SHIFT(784),
  [1647] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declarator, 2, .alias_sequence_id = 3),
  [1649] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declarator, 2, .alias_sequence_id = 4),
  [1651] = {.count = 1, .reusable = true}, REDUCE(sym_lexical_declaration, 3),
  [1653] = {.count = 1, .reusable = false}, REDUCE(sym_lexical_declaration, 3),
  [1655] = {.count = 1, .reusable = true}, SHIFT(786),
  [1657] = {.count = 1, .reusable = true}, SHIFT(787),
  [1659] = {.count = 1, .reusable = true}, SHIFT(788),
  [1661] = {.count = 1, .reusable = true}, SHIFT(789),
  [1663] = {.count = 1, .reusable = true}, SHIFT(364),
  [1665] = {.count = 1, .reusable = true}, SHIFT(365),
  [1667] = {.count = 1, .reusable = true}, SHIFT(790),
  [1669] = {.count = 1, .reusable = true}, SHIFT(791),
  [1671] = {.count = 1, .reusable = true}, SHIFT(792),
  [1673] = {.count = 1, .reusable = true}, SHIFT(797),
  [1675] = {.count = 1, .reusable = true}, SHIFT(799),
  [1677] = {.count = 1, .reusable = true}, SHIFT(805),
  [1679] = {.count = 1, .reusable = true}, SHIFT(808),
  [1681] = {.count = 1, .reusable = true}, SHIFT(811),
  [1683] = {.count = 1, .reusable = true}, SHIFT(812),
  [1685] = {.count = 1, .reusable = true}, SHIFT(813),
  [1687] = {.count = 1, .reusable = true}, SHIFT(814),
  [1689] = {.count = 1, .reusable = true}, SHIFT(815),
  [1691] = {.count = 1, .reusable = true}, SHIFT(816),
  [1693] = {.count = 1, .reusable = false}, SHIFT(819),
  [1695] = {.count = 1, .reusable = false}, SHIFT(820),
  [1697] = {.count = 1, .reusable = true}, SHIFT(821),
  [1699] = {.count = 1, .reusable = true}, SHIFT(822),
  [1701] = {.count = 1, .reusable = true}, SHIFT(824),
  [1703] = {.count = 1, .reusable = true}, REDUCE(sym_if_statement, 3),
  [1705] = {.count = 1, .reusable = false}, REDUCE(sym_if_statement, 3),
  [1707] = {.count = 1, .reusable = false}, SHIFT(825),
  [1709] = {.count = 1, .reusable = true}, SHIFT(826),
  [1711] = {.count = 1, .reusable = true}, SHIFT(827),
  [1713] = {.count = 1, .reusable = true}, SHIFT(828),
  [1715] = {.count = 1, .reusable = true}, SHIFT(829),
  [1717] = {.count = 1, .reusable = true}, SHIFT(830),
  [1719] = {.count = 1, .reusable = true}, SHIFT(831),
  [1721] = {.count = 1, .reusable = true}, SHIFT(832),
  [1723] = {.count = 1, .reusable = true}, REDUCE(sym_switch_statement, 3),
  [1725] = {.count = 1, .reusable = false}, REDUCE(sym_switch_statement, 3),
  [1727] = {.count = 1, .reusable = true}, SHIFT(834),
  [1729] = {.count = 1, .reusable = false}, SHIFT(839),
  [1731] = {.count = 1, .reusable = true}, SHIFT(840),
  [1733] = {.count = 1, .reusable = true}, SHIFT(842),
  [1735] = {.count = 1, .reusable = false}, SHIFT(843),
  [1737] = {.count = 1, .reusable = true}, SHIFT(844),
  [1739] = {.count = 1, .reusable = false}, SHIFT(845),
  [1741] = {.count = 1, .reusable = true}, SHIFT(846),
  [1743] = {.count = 1, .reusable = true}, SHIFT(847),
  [1745] = {.count = 1, .reusable = true}, SHIFT(849),
  [1747] = {.count = 1, .reusable = true}, SHIFT(852),
  [1749] = {.count = 1, .reusable = true}, SHIFT(853),
  [1751] = {.count = 1, .reusable = false}, SHIFT(855),
  [1753] = {.count = 1, .reusable = false}, SHIFT(856),
  [1755] = {.count = 1, .reusable = false}, SHIFT(857),
  [1757] = {.count = 1, .reusable = false}, SHIFT(859),
  [1759] = {.count = 1, .reusable = true}, SHIFT(861),
  [1761] = {.count = 1, .reusable = false}, SHIFT(866),
  [1763] = {.count = 1, .reusable = false}, SHIFT(867),
  [1765] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 3, .alias_sequence_id = 9),
  [1767] = {.count = 1, .reusable = true}, SHIFT(871),
  [1769] = {.count = 1, .reusable = true}, SHIFT(872),
  [1771] = {.count = 1, .reusable = false}, SHIFT(872),
  [1773] = {.count = 1, .reusable = true}, SHIFT(873),
  [1775] = {.count = 1, .reusable = true}, SHIFT(876),
  [1777] = {.count = 1, .reusable = false}, SHIFT(876),
  [1779] = {.count = 1, .reusable = true}, SHIFT(877),
  [1781] = {.count = 1, .reusable = false}, SHIFT(877),
  [1783] = {.count = 1, .reusable = true}, SHIFT(878),
  [1785] = {.count = 1, .reusable = true}, SHIFT(879),
  [1787] = {.count = 1, .reusable = true}, SHIFT(881),
  [1789] = {.count = 1, .reusable = true}, SHIFT(883),
  [1791] = {.count = 1, .reusable = false}, SHIFT(885),
  [1793] = {.count = 1, .reusable = false}, SHIFT(886),
  [1795] = {.count = 1, .reusable = false}, SHIFT(887),
  [1797] = {.count = 1, .reusable = true}, REDUCE(sym_rest_parameter, 2),
  [1799] = {.count = 1, .reusable = false}, SHIFT(890),
  [1801] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 3),
  [1803] = {.count = 1, .reusable = true}, SHIFT(891),
  [1805] = {.count = 1, .reusable = false}, SHIFT(891),
  [1807] = {.count = 1, .reusable = true}, SHIFT(892),
  [1809] = {.count = 1, .reusable = true}, SHIFT(893),
  [1811] = {.count = 1, .reusable = false}, SHIFT(893),
  [1813] = {.count = 1, .reusable = true}, SHIFT(894),
  [1815] = {.count = 1, .reusable = true}, SHIFT(895),
  [1817] = {.count = 1, .reusable = true}, SHIFT(896),
  [1819] = {.count = 1, .reusable = false}, SHIFT(896),
  [1821] = {.count = 1, .reusable = true}, REDUCE(sym_parenthesized_expression, 3),
  [1823] = {.count = 1, .reusable = false}, REDUCE(sym_parenthesized_expression, 3),
  [1825] = {.count = 1, .reusable = true}, SHIFT(900),
  [1827] = {.count = 1, .reusable = false}, SHIFT(900),
  [1829] = {.count = 1, .reusable = true}, SHIFT(901),
  [1831] = {.count = 1, .reusable = false}, SHIFT(901),
  [1833] = {.count = 1, .reusable = true}, SHIFT(903),
  [1835] = {.count = 1, .reusable = false}, SHIFT(903),
  [1837] = {.count = 1, .reusable = true}, SHIFT(904),
  [1839] = {.count = 1, .reusable = false}, SHIFT(904),
  [1841] = {.count = 1, .reusable = true}, SHIFT(905),
  [1843] = {.count = 1, .reusable = false}, SHIFT(905),
  [1845] = {.count = 1, .reusable = true}, SHIFT(906),
  [1847] = {.count = 1, .reusable = false}, SHIFT(906),
  [1849] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 3, .alias_sequence_id = 10),
  [1851] = {.count = 1, .reusable = true}, SHIFT(907),
  [1853] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 3, .alias_sequence_id = 11),
  [1855] = {.count = 1, .reusable = true}, SHIFT(908),
  [1857] = {.count = 1, .reusable = false}, SHIFT(909),
  [1859] = {.count = 1, .reusable = false}, SHIFT(911),
  [1861] = {.count = 1, .reusable = true}, REDUCE(sym_while_statement, 3),
  [1863] = {.count = 1, .reusable = false}, REDUCE(sym_while_statement, 3),
  [1865] = {.count = 1, .reusable = true}, SHIFT(913),
  [1867] = {.count = 1, .reusable = true}, SHIFT(914),
  [1869] = {.count = 1, .reusable = true}, SHIFT(915),
  [1871] = {.count = 1, .reusable = true}, SHIFT(916),
  [1873] = {.count = 1, .reusable = true}, SHIFT(918),
  [1875] = {.count = 1, .reusable = false}, SHIFT(919),
  [1877] = {.count = 1, .reusable = true}, SHIFT(920),
  [1879] = {.count = 1, .reusable = false}, SHIFT(921),
  [1881] = {.count = 1, .reusable = false}, SHIFT(922),
  [1883] = {.count = 1, .reusable = false}, SHIFT(923),
  [1885] = {.count = 1, .reusable = false}, SHIFT(924),
  [1887] = {.count = 1, .reusable = false}, SHIFT(925),
  [1889] = {.count = 1, .reusable = false}, SHIFT(926),
  [1891] = {.count = 1, .reusable = false}, SHIFT(927),
  [1893] = {.count = 1, .reusable = false}, SHIFT(928),
  [1895] = {.count = 1, .reusable = false}, SHIFT(929),
  [1897] = {.count = 1, .reusable = false}, SHIFT(930),
  [1899] = {.count = 1, .reusable = false}, SHIFT(931),
  [1901] = {.count = 1, .reusable = false}, SHIFT(932),
  [1903] = {.count = 1, .reusable = false}, SHIFT(933),
  [1905] = {.count = 1, .reusable = false}, SHIFT(934),
  [1907] = {.count = 1, .reusable = false}, SHIFT(935),
  [1909] = {.count = 1, .reusable = true}, SHIFT(936),
  [1911] = {.count = 1, .reusable = false}, SHIFT(937),
  [1913] = {.count = 1, .reusable = false}, SHIFT(943),
  [1915] = {.count = 1, .reusable = false}, SHIFT(944),
  [1917] = {.count = 1, .reusable = false}, SHIFT(945),
  [1919] = {.count = 1, .reusable = true}, SHIFT(949),
  [1921] = {.count = 1, .reusable = true}, SHIFT(950),
  [1923] = {.count = 1, .reusable = true}, SHIFT(954),
  [1925] = {.count = 1, .reusable = true}, SHIFT(956),
  [1927] = {.count = 1, .reusable = true}, SHIFT(958),
  [1929] = {.count = 1, .reusable = true}, SHIFT(960),
  [1931] = {.count = 1, .reusable = false}, SHIFT(962),
  [1933] = {.count = 1, .reusable = true}, SHIFT(963),
  [1935] = {.count = 1, .reusable = true}, SHIFT(964),
  [1937] = {.count = 1, .reusable = true}, SHIFT(965),
  [1939] = {.count = 1, .reusable = true}, SHIFT(967),
  [1941] = {.count = 1, .reusable = true}, REDUCE(sym_try_statement, 3),
  [1943] = {.count = 1, .reusable = false}, REDUCE(sym_try_statement, 3),
  [1945] = {.count = 1, .reusable = true}, REDUCE(sym_with_statement, 3),
  [1947] = {.count = 1, .reusable = false}, REDUCE(sym_with_statement, 3),
  [1949] = {.count = 1, .reusable = true}, REDUCE(sym_break_statement, 3, .alias_sequence_id = 12),
  [1951] = {.count = 1, .reusable = false}, REDUCE(sym_break_statement, 3, .alias_sequence_id = 12),
  [1953] = {.count = 1, .reusable = true}, REDUCE(sym_continue_statement, 3, .alias_sequence_id = 12),
  [1955] = {.count = 1, .reusable = false}, REDUCE(sym_continue_statement, 3, .alias_sequence_id = 12),
  [1957] = {.count = 1, .reusable = true}, REDUCE(sym_return_statement, 3),
  [1959] = {.count = 1, .reusable = false}, REDUCE(sym_return_statement, 3),
  [1961] = {.count = 1, .reusable = true}, REDUCE(sym_throw_statement, 3),
  [1963] = {.count = 1, .reusable = false}, REDUCE(sym_throw_statement, 3),
  [1965] = {.count = 1, .reusable = true}, REDUCE(aux_sym_array_repeat1, 2),
  [1967] = {.count = 1, .reusable = true}, SHIFT(970),
  [1969] = {.count = 1, .reusable = false}, SHIFT(970),
  [1971] = {.count = 1, .reusable = true}, SHIFT(971),
  [1973] = {.count = 1, .reusable = false}, SHIFT(971),
  [1975] = {.count = 1, .reusable = true}, SHIFT(972),
  [1977] = {.count = 1, .reusable = false}, SHIFT(972),
  [1979] = {.count = 1, .reusable = true}, SHIFT(973),
  [1981] = {.count = 1, .reusable = true}, SHIFT(974),
  [1983] = {.count = 1, .reusable = false}, SHIFT(974),
  [1985] = {.count = 1, .reusable = true}, SHIFT(975),
  [1987] = {.count = 1, .reusable = false}, SHIFT(975),
  [1989] = {.count = 1, .reusable = true}, REDUCE(sym_array, 3),
  [1991] = {.count = 1, .reusable = false}, REDUCE(sym_array, 3),
  [1993] = {.count = 1, .reusable = true}, SHIFT(976),
  [1995] = {.count = 1, .reusable = false}, SHIFT(976),
  [1997] = {.count = 1, .reusable = true}, SHIFT(977),
  [1999] = {.count = 1, .reusable = false}, SHIFT(977),
  [2001] = {.count = 1, .reusable = true}, SHIFT(978),
  [2003] = {.count = 1, .reusable = false}, SHIFT(978),
  [2005] = {.count = 1, .reusable = true}, SHIFT(979),
  [2007] = {.count = 1, .reusable = false}, SHIFT(979),
  [2009] = {.count = 1, .reusable = true}, SHIFT(980),
  [2011] = {.count = 2, .reusable = true}, REDUCE(aux_sym_array_repeat1, 2), SHIFT_REPEAT(160),
  [2014] = {.count = 1, .reusable = false}, SHIFT(981),
  [2016] = {.count = 1, .reusable = false}, SHIFT(982),
  [2018] = {.count = 1, .reusable = true}, SHIFT(983),
  [2020] = {.count = 1, .reusable = true}, SHIFT(984),
  [2022] = {.count = 1, .reusable = false}, SHIFT(984),
  [2024] = {.count = 1, .reusable = true}, SHIFT(986),
  [2026] = {.count = 1, .reusable = false}, SHIFT(987),
  [2028] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_opening_element, 3, .dynamic_precedence = -1, .alias_sequence_id = 9),
  [2030] = {.count = 1, .reusable = true}, SHIFT(988),
  [2032] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_attribute, 1, .alias_sequence_id = 2),
  [2034] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_attribute, 1, .alias_sequence_id = 2),
  [2036] = {.count = 1, .reusable = true}, SHIFT(989),
  [2038] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_attribute, 1),
  [2040] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_attribute, 1),
  [2042] = {.count = 1, .reusable = true}, SHIFT(990),
  [2044] = {.count = 1, .reusable = true}, SHIFT(991),
  [2046] = {.count = 1, .reusable = false}, SHIFT(992),
  [2048] = {.count = 1, .reusable = true}, SHIFT(994),
  [2050] = {.count = 1, .reusable = false}, SHIFT(995),
  [2052] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_opening_element, 3, .dynamic_precedence = -1),
  [2054] = {.count = 1, .reusable = true}, SHIFT(996),
  [2056] = {.count = 1, .reusable = true}, SHIFT(997),
  [2058] = {.count = 1, .reusable = true}, SHIFT(998),
  [2060] = {.count = 1, .reusable = false}, SHIFT(999),
  [2062] = {.count = 1, .reusable = true}, SHIFT(1000),
  [2064] = {.count = 1, .reusable = true}, REDUCE(sym_class_body, 2),
  [2066] = {.count = 1, .reusable = false}, REDUCE(sym_class_body, 2),
  [2068] = {.count = 1, .reusable = true}, SHIFT(1001),
  [2070] = {.count = 1, .reusable = false}, SHIFT(1001),
  [2072] = {.count = 1, .reusable = true}, SHIFT(1002),
  [2074] = {.count = 1, .reusable = false}, SHIFT(1003),
  [2076] = {.count = 1, .reusable = false}, SHIFT(1004),
  [2078] = {.count = 1, .reusable = false}, SHIFT(1005),
  [2080] = {.count = 1, .reusable = true}, SHIFT(1010),
  [2082] = {.count = 1, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 1),
  [2084] = {.count = 1, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 1),
  [2086] = {.count = 1, .reusable = true}, REDUCE(sym_public_field_definition, 1),
  [2088] = {.count = 1, .reusable = true}, SHIFT(653),
  [2090] = {.count = 1, .reusable = false}, SHIFT(1013),
  [2092] = {.count = 1, .reusable = true}, SHIFT(1015),
  [2094] = {.count = 1, .reusable = true}, SHIFT(1017),
  [2096] = {.count = 1, .reusable = false}, SHIFT(1017),
  [2098] = {.count = 1, .reusable = false}, SHIFT(1018),
  [2100] = {.count = 1, .reusable = true}, SHIFT(1019),
  [2102] = {.count = 1, .reusable = true}, SHIFT(1020),
  [2104] = {.count = 1, .reusable = false}, SHIFT(1021),
  [2106] = {.count = 1, .reusable = true}, SHIFT(1022),
  [2108] = {.count = 1, .reusable = false}, SHIFT(1022),
  [2110] = {.count = 1, .reusable = true}, SHIFT(1023),
  [2112] = {.count = 1, .reusable = false}, SHIFT(1023),
  [2114] = {.count = 1, .reusable = false}, SHIFT(1024),
  [2116] = {.count = 1, .reusable = true}, SHIFT(1025),
  [2118] = {.count = 1, .reusable = true}, SHIFT(1026),
  [2120] = {.count = 1, .reusable = true}, REDUCE(sym_class_heritage, 2),
  [2122] = {.count = 1, .reusable = false}, SHIFT(1026),
  [2124] = {.count = 1, .reusable = true}, SHIFT(1027),
  [2126] = {.count = 1, .reusable = true}, SHIFT(1028),
  [2128] = {.count = 1, .reusable = true}, SHIFT(1029),
  [2130] = {.count = 1, .reusable = false}, SHIFT(1028),
  [2132] = {.count = 1, .reusable = false}, SHIFT(1029),
  [2134] = {.count = 1, .reusable = false}, SHIFT(1030),
  [2136] = {.count = 1, .reusable = true}, SHIFT(1024),
  [2138] = {.count = 1, .reusable = true}, SHIFT(1031),
  [2140] = {.count = 1, .reusable = true}, REDUCE(sym_class, 3),
  [2142] = {.count = 1, .reusable = false}, REDUCE(sym_class, 3),
  [2144] = {.count = 1, .reusable = true}, REDUCE(sym_anonymous_class, 3),
  [2146] = {.count = 1, .reusable = false}, REDUCE(sym_anonymous_class, 3),
  [2148] = {.count = 1, .reusable = true}, SHIFT(1034),
  [2150] = {.count = 1, .reusable = false}, SHIFT(1034),
  [2152] = {.count = 1, .reusable = true}, SHIFT(421),
  [2154] = {.count = 1, .reusable = true}, SHIFT(423),
  [2156] = {.count = 1, .reusable = true}, SHIFT(460),
  [2158] = {.count = 1, .reusable = true}, SHIFT(463),
  [2160] = {.count = 1, .reusable = true}, REDUCE(sym_member_expression, 3, .alias_sequence_id = 13),
  [2162] = {.count = 1, .reusable = false}, REDUCE(sym_member_expression, 3, .alias_sequence_id = 13),
  [2164] = {.count = 1, .reusable = true}, REDUCE(sym__initializer, 2),
  [2166] = {.count = 1, .reusable = true}, REDUCE(sym_arrow_function, 3),
  [2168] = {.count = 1, .reusable = false}, REDUCE(sym_arrow_function, 3),
  [2170] = {.count = 1, .reusable = true}, REDUCE(sym_augmented_assignment_expression, 3, .alias_sequence_id = 1),
  [2172] = {.count = 1, .reusable = true}, SHIFT(1038),
  [2174] = {.count = 1, .reusable = false}, SHIFT(1038),
  [2176] = {.count = 1, .reusable = true}, SHIFT(1041),
  [2178] = {.count = 1, .reusable = true}, REDUCE(sym_function, 3),
  [2180] = {.count = 1, .reusable = false}, REDUCE(sym_function, 3),
  [2182] = {.count = 1, .reusable = true}, SHIFT(1043),
  [2184] = {.count = 1, .reusable = true}, SHIFT(1044),
  [2186] = {.count = 1, .reusable = true}, SHIFT(1045),
  [2188] = {.count = 1, .reusable = true}, REDUCE(sym_template_string, 3),
  [2190] = {.count = 1, .reusable = false}, REDUCE(sym_template_string, 3),
  [2192] = {.count = 1, .reusable = false}, REDUCE(aux_sym_template_string_repeat1, 2),
  [2194] = {.count = 2, .reusable = false}, REDUCE(aux_sym_template_string_repeat1, 2), SHIFT_REPEAT(596),
  [2197] = {.count = 2, .reusable = true}, REDUCE(aux_sym_template_string_repeat1, 2), SHIFT_REPEAT(208),
  [2200] = {.count = 1, .reusable = true}, REDUCE(sym_binary_expression, 3),
  [2202] = {.count = 1, .reusable = false}, REDUCE(sym_binary_expression, 3),
  [2204] = {.count = 1, .reusable = true}, REDUCE(sym_labeled_statement, 3, .alias_sequence_id = 14),
  [2206] = {.count = 1, .reusable = false}, REDUCE(sym_labeled_statement, 3, .alias_sequence_id = 14),
  [2208] = {.count = 1, .reusable = true}, REDUCE(sym_member_expression, 3, .alias_sequence_id = 15),
  [2210] = {.count = 1, .reusable = false}, REDUCE(sym_member_expression, 3, .alias_sequence_id = 15),
  [2212] = {.count = 1, .reusable = true}, REDUCE(sym_augmented_assignment_expression, 3),
  [2214] = {.count = 1, .reusable = true}, SHIFT(1046),
  [2216] = {.count = 1, .reusable = false}, SHIFT(1046),
  [2218] = {.count = 1, .reusable = true}, REDUCE(sym_arguments, 2),
  [2220] = {.count = 1, .reusable = false}, REDUCE(sym_arguments, 2),
  [2222] = {.count = 1, .reusable = true}, SHIFT(1047),
  [2224] = {.count = 1, .reusable = false}, SHIFT(1047),
  [2226] = {.count = 1, .reusable = true}, SHIFT(1048),
  [2228] = {.count = 1, .reusable = true}, SHIFT(1051),
  [2230] = {.count = 1, .reusable = true}, SHIFT(1052),
  [2232] = {.count = 1, .reusable = true}, SHIFT(1053),
  [2234] = {.count = 1, .reusable = true}, REDUCE(sym_decorator_call_expression, 2),
  [2236] = {.count = 1, .reusable = true}, SHIFT(1054),
  [2238] = {.count = 1, .reusable = true}, REDUCE(sym_decorator_call_expression, 2, .alias_sequence_id = 16),
  [2240] = {.count = 1, .reusable = true}, REDUCE(sym_sequence_expression, 3),
  [2242] = {.count = 1, .reusable = true}, SHIFT(1055),
  [2244] = {.count = 1, .reusable = false}, SHIFT(1055),
  [2246] = {.count = 1, .reusable = false}, SHIFT(1056),
  [2248] = {.count = 1, .reusable = true}, SHIFT(1057),
  [2250] = {.count = 1, .reusable = true}, SHIFT(1058),
  [2252] = {.count = 1, .reusable = false}, SHIFT(1059),
  [2254] = {.count = 1, .reusable = true}, SHIFT(1060),
  [2256] = {.count = 1, .reusable = false}, SHIFT(1060),
  [2258] = {.count = 1, .reusable = true}, SHIFT(1061),
  [2260] = {.count = 1, .reusable = false}, SHIFT(1061),
  [2262] = {.count = 1, .reusable = false}, SHIFT(1062),
  [2264] = {.count = 1, .reusable = true}, SHIFT(1063),
  [2266] = {.count = 1, .reusable = true}, SHIFT(1064),
  [2268] = {.count = 1, .reusable = true}, SHIFT(1065),
  [2270] = {.count = 1, .reusable = false}, SHIFT(1064),
  [2272] = {.count = 1, .reusable = true}, SHIFT(1066),
  [2274] = {.count = 1, .reusable = true}, SHIFT(1067),
  [2276] = {.count = 1, .reusable = true}, SHIFT(1068),
  [2278] = {.count = 1, .reusable = false}, SHIFT(1067),
  [2280] = {.count = 1, .reusable = false}, SHIFT(1068),
  [2282] = {.count = 1, .reusable = false}, SHIFT(1069),
  [2284] = {.count = 1, .reusable = true}, SHIFT(1062),
  [2286] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_expression, 2),
  [2288] = {.count = 1, .reusable = false}, SHIFT(1070),
  [2290] = {.count = 1, .reusable = false}, SHIFT(1071),
  [2292] = {.count = 1, .reusable = true}, SHIFT(1072),
  [2294] = {.count = 1, .reusable = false}, SHIFT(1073),
  [2296] = {.count = 1, .reusable = false}, SHIFT(1074),
  [2298] = {.count = 1, .reusable = false}, SHIFT(1075),
  [2300] = {.count = 1, .reusable = true}, SHIFT(1075),
  [2302] = {.count = 1, .reusable = true}, SHIFT(1076),
  [2304] = {.count = 1, .reusable = true}, SHIFT(1079),
  [2306] = {.count = 1, .reusable = false}, SHIFT(1079),
  [2308] = {.count = 1, .reusable = false}, SHIFT(1077),
  [2310] = {.count = 1, .reusable = false}, SHIFT(1078),
  [2312] = {.count = 1, .reusable = true}, SHIFT(1084),
  [2314] = {.count = 1, .reusable = true}, SHIFT(1085),
  [2316] = {.count = 1, .reusable = false}, SHIFT(1086),
  [2318] = {.count = 1, .reusable = false}, SHIFT(1089),
  [2320] = {.count = 1, .reusable = false}, SHIFT(1091),
  [2322] = {.count = 1, .reusable = false}, SHIFT(1093),
  [2324] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_element, 3),
  [2326] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_element, 3),
  [2328] = {.count = 2, .reusable = false}, REDUCE(aux_sym_jsx_element_repeat1, 2), SHIFT_REPEAT(232),
  [2331] = {.count = 2, .reusable = false}, REDUCE(aux_sym_jsx_element_repeat1, 2), SHIFT_REPEAT(1095),
  [2334] = {.count = 2, .reusable = false}, REDUCE(aux_sym_jsx_element_repeat1, 2), SHIFT_REPEAT(646),
  [2337] = {.count = 1, .reusable = true}, SHIFT(1096),
  [2339] = {.count = 1, .reusable = false}, SHIFT(1096),
  [2341] = {.count = 1, .reusable = true}, REDUCE(sym__from_clause, 2),
  [2343] = {.count = 1, .reusable = true}, REDUCE(sym_export_statement, 4),
  [2345] = {.count = 1, .reusable = false}, REDUCE(sym_export_statement, 4),
  [2347] = {.count = 1, .reusable = true}, SHIFT(1099),
  [2349] = {.count = 1, .reusable = false}, SHIFT(1099),
  [2351] = {.count = 1, .reusable = true}, SHIFT(1100),
  [2353] = {.count = 1, .reusable = false}, SHIFT(1100),
  [2355] = {.count = 1, .reusable = true}, SHIFT(1101),
  [2357] = {.count = 1, .reusable = false}, SHIFT(1101),
  [2359] = {.count = 1, .reusable = true}, SHIFT(1102),
  [2361] = {.count = 1, .reusable = true}, SHIFT(1103),
  [2363] = {.count = 1, .reusable = false}, SHIFT(1103),
  [2365] = {.count = 1, .reusable = true}, SHIFT(1104),
  [2367] = {.count = 1, .reusable = false}, SHIFT(1104),
  [2369] = {.count = 1, .reusable = true}, SHIFT(1105),
  [2371] = {.count = 1, .reusable = false}, SHIFT(1105),
  [2373] = {.count = 1, .reusable = true}, SHIFT(1106),
  [2375] = {.count = 1, .reusable = false}, SHIFT(1106),
  [2377] = {.count = 1, .reusable = true}, SHIFT(1107),
  [2379] = {.count = 1, .reusable = false}, SHIFT(1107),
  [2381] = {.count = 1, .reusable = true}, SHIFT(1108),
  [2383] = {.count = 1, .reusable = false}, SHIFT(1108),
  [2385] = {.count = 1, .reusable = true}, REDUCE(sym_export_clause, 3),
  [2387] = {.count = 1, .reusable = true}, SHIFT(1109),
  [2389] = {.count = 1, .reusable = true}, SHIFT(1110),
  [2391] = {.count = 1, .reusable = true}, REDUCE(sym_export_clause, 3, .alias_sequence_id = 17),
  [2393] = {.count = 1, .reusable = true}, SHIFT(1112),
  [2395] = {.count = 1, .reusable = true}, SHIFT(1114),
  [2397] = {.count = 1, .reusable = true}, SHIFT(1121),
  [2399] = {.count = 1, .reusable = true}, SHIFT(1124),
  [2401] = {.count = 1, .reusable = false}, SHIFT(1124),
  [2403] = {.count = 1, .reusable = false}, SHIFT(1125),
  [2405] = {.count = 1, .reusable = true}, SHIFT(1126),
  [2407] = {.count = 1, .reusable = true}, SHIFT(1127),
  [2409] = {.count = 1, .reusable = false}, SHIFT(1128),
  [2411] = {.count = 1, .reusable = true}, SHIFT(1129),
  [2413] = {.count = 1, .reusable = false}, SHIFT(1129),
  [2415] = {.count = 1, .reusable = true}, SHIFT(1130),
  [2417] = {.count = 1, .reusable = false}, SHIFT(1130),
  [2419] = {.count = 1, .reusable = false}, SHIFT(1131),
  [2421] = {.count = 1, .reusable = true}, SHIFT(1132),
  [2423] = {.count = 1, .reusable = true}, SHIFT(1133),
  [2425] = {.count = 1, .reusable = true}, SHIFT(1134),
  [2427] = {.count = 1, .reusable = false}, SHIFT(1133),
  [2429] = {.count = 1, .reusable = true}, SHIFT(1135),
  [2431] = {.count = 1, .reusable = true}, SHIFT(1136),
  [2433] = {.count = 1, .reusable = true}, SHIFT(1137),
  [2435] = {.count = 1, .reusable = false}, SHIFT(1136),
  [2437] = {.count = 1, .reusable = false}, SHIFT(1137),
  [2439] = {.count = 1, .reusable = false}, SHIFT(1138),
  [2441] = {.count = 1, .reusable = true}, SHIFT(1131),
  [2443] = {.count = 1, .reusable = true}, SHIFT(1140),
  [2445] = {.count = 1, .reusable = true}, SHIFT(1142),
  [2447] = {.count = 1, .reusable = true}, SHIFT(1145),
  [2449] = {.count = 1, .reusable = true}, SHIFT(1147),
  [2451] = {.count = 1, .reusable = true}, SHIFT(1148),
  [2453] = {.count = 1, .reusable = false}, SHIFT(1148),
  [2455] = {.count = 1, .reusable = true}, SHIFT(1149),
  [2457] = {.count = 1, .reusable = false}, SHIFT(1150),
  [2459] = {.count = 1, .reusable = false}, SHIFT(1152),
  [2461] = {.count = 1, .reusable = true}, REDUCE(aux_sym_object_repeat1, 3),
  [2463] = {.count = 2, .reusable = true}, REDUCE(aux_sym_export_statement_repeat1, 2), SHIFT_REPEAT(277),
  [2466] = {.count = 1, .reusable = true}, SHIFT(1155),
  [2468] = {.count = 1, .reusable = true}, SHIFT(1158),
  [2470] = {.count = 1, .reusable = false}, SHIFT(1159),
  [2472] = {.count = 1, .reusable = true}, SHIFT(1160),
  [2474] = {.count = 1, .reusable = false}, SHIFT(1163),
  [2476] = {.count = 1, .reusable = true}, SHIFT(1164),
  [2478] = {.count = 1, .reusable = false}, SHIFT(1166),
  [2480] = {.count = 1, .reusable = true}, SHIFT(1167),
  [2482] = {.count = 1, .reusable = false}, SHIFT(1168),
  [2484] = {.count = 1, .reusable = true}, SHIFT(1169),
  [2486] = {.count = 2, .reusable = true}, REDUCE(sym_array, 3), REDUCE(sym_computed_property_name, 3),
  [2489] = {.count = 1, .reusable = true}, REDUCE(sym_computed_property_name, 3),
  [2491] = {.count = 1, .reusable = true}, REDUCE(sym_object, 4, .alias_sequence_id = 7),
  [2493] = {.count = 1, .reusable = false}, REDUCE(sym_object, 4, .alias_sequence_id = 7),
  [2495] = {.count = 1, .reusable = true}, SHIFT(1171),
  [2497] = {.count = 1, .reusable = false}, SHIFT(1171),
  [2499] = {.count = 1, .reusable = true}, SHIFT(1172),
  [2501] = {.count = 1, .reusable = false}, SHIFT(1172),
  [2503] = {.count = 1, .reusable = true}, SHIFT(1173),
  [2505] = {.count = 1, .reusable = true}, SHIFT(1174),
  [2507] = {.count = 1, .reusable = false}, SHIFT(1174),
  [2509] = {.count = 1, .reusable = true}, SHIFT(1175),
  [2511] = {.count = 1, .reusable = false}, SHIFT(1175),
  [2513] = {.count = 1, .reusable = true}, SHIFT(1176),
  [2515] = {.count = 1, .reusable = false}, SHIFT(1176),
  [2517] = {.count = 1, .reusable = true}, SHIFT(1177),
  [2519] = {.count = 1, .reusable = false}, SHIFT(1177),
  [2521] = {.count = 1, .reusable = true}, SHIFT(1178),
  [2523] = {.count = 1, .reusable = false}, SHIFT(1178),
  [2525] = {.count = 1, .reusable = true}, SHIFT(1179),
  [2527] = {.count = 1, .reusable = false}, SHIFT(1179),
  [2529] = {.count = 1, .reusable = true}, SHIFT(1180),
  [2531] = {.count = 1, .reusable = false}, SHIFT(1180),
  [2533] = {.count = 1, .reusable = true}, SHIFT(1181),
  [2535] = {.count = 1, .reusable = true}, SHIFT(1182),
  [2537] = {.count = 1, .reusable = false}, SHIFT(1183),
  [2539] = {.count = 1, .reusable = true}, SHIFT(1184),
  [2541] = {.count = 1, .reusable = false}, SHIFT(1184),
  [2543] = {.count = 1, .reusable = true}, SHIFT(1185),
  [2545] = {.count = 1, .reusable = false}, SHIFT(1185),
  [2547] = {.count = 1, .reusable = false}, SHIFT(1186),
  [2549] = {.count = 1, .reusable = true}, SHIFT(1187),
  [2551] = {.count = 1, .reusable = true}, SHIFT(1188),
  [2553] = {.count = 1, .reusable = false}, SHIFT(1188),
  [2555] = {.count = 1, .reusable = true}, SHIFT(1189),
  [2557] = {.count = 1, .reusable = true}, SHIFT(1190),
  [2559] = {.count = 1, .reusable = true}, SHIFT(1191),
  [2561] = {.count = 1, .reusable = false}, SHIFT(1190),
  [2563] = {.count = 1, .reusable = false}, SHIFT(1191),
  [2565] = {.count = 1, .reusable = false}, SHIFT(1192),
  [2567] = {.count = 1, .reusable = true}, SHIFT(1186),
  [2569] = {.count = 1, .reusable = true}, SHIFT(1193),
  [2571] = {.count = 1, .reusable = true}, SHIFT(1194),
  [2573] = {.count = 1, .reusable = false}, SHIFT(1194),
  [2575] = {.count = 1, .reusable = true}, SHIFT(1197),
  [2577] = {.count = 1, .reusable = false}, REDUCE(sym_decorator_call_expression, 2),
  [2579] = {.count = 1, .reusable = true}, SHIFT(1198),
  [2581] = {.count = 1, .reusable = false}, REDUCE(sym_decorator_call_expression, 2, .alias_sequence_id = 16),
  [2583] = {.count = 1, .reusable = true}, REDUCE(sym_object, 4),
  [2585] = {.count = 1, .reusable = false}, REDUCE(sym_object, 4),
  [2587] = {.count = 1, .reusable = true}, REDUCE(sym_pair, 3),
  [2589] = {.count = 1, .reusable = true}, REDUCE(sym_method_definition, 3),
  [2591] = {.count = 1, .reusable = false}, REDUCE(sym_method_definition, 3),
  [2593] = {.count = 1, .reusable = true}, SHIFT(1199),
  [2595] = {.count = 1, .reusable = true}, REDUCE(sym_namespace_import, 3),
  [2597] = {.count = 1, .reusable = true}, REDUCE(sym_named_imports, 3),
  [2599] = {.count = 1, .reusable = true}, SHIFT(1200),
  [2601] = {.count = 1, .reusable = true}, REDUCE(sym_named_imports, 3, .alias_sequence_id = 18),
  [2603] = {.count = 1, .reusable = true}, SHIFT(1202),
  [2605] = {.count = 1, .reusable = true}, REDUCE(sym_import_clause, 3),
  [2607] = {.count = 1, .reusable = true}, REDUCE(sym_import_statement, 4),
  [2609] = {.count = 1, .reusable = false}, REDUCE(sym_import_statement, 4),
  [2611] = {.count = 1, .reusable = true}, SHIFT(1204),
  [2613] = {.count = 1, .reusable = true}, SHIFT(1205),
  [2615] = {.count = 1, .reusable = true}, SHIFT(1207),
  [2617] = {.count = 1, .reusable = true}, REDUCE(aux_sym_variable_declaration_repeat1, 2),
  [2619] = {.count = 1, .reusable = true}, REDUCE(sym_variable_declaration, 4),
  [2621] = {.count = 1, .reusable = false}, REDUCE(sym_variable_declaration, 4),
  [2623] = {.count = 2, .reusable = true}, REDUCE(aux_sym_variable_declaration_repeat1, 2), SHIFT_REPEAT(349),
  [2626] = {.count = 1, .reusable = true}, REDUCE(sym_lexical_declaration, 4),
  [2628] = {.count = 1, .reusable = false}, REDUCE(sym_lexical_declaration, 4),
  [2630] = {.count = 1, .reusable = true}, SHIFT(1209),
  [2632] = {.count = 1, .reusable = false}, SHIFT(1209),
  [2634] = {.count = 1, .reusable = true}, SHIFT(1210),
  [2636] = {.count = 1, .reusable = true}, SHIFT(1211),
  [2638] = {.count = 1, .reusable = true}, SHIFT(1212),
  [2640] = {.count = 1, .reusable = true}, SHIFT(1213),
  [2642] = {.count = 1, .reusable = true}, SHIFT(1215),
  [2644] = {.count = 1, .reusable = true}, SHIFT(1216),
  [2646] = {.count = 1, .reusable = true}, SHIFT(1217),
  [2648] = {.count = 1, .reusable = true}, SHIFT(1218),
  [2650] = {.count = 1, .reusable = true}, SHIFT(1220),
  [2652] = {.count = 1, .reusable = true}, SHIFT(1222),
  [2654] = {.count = 1, .reusable = true}, SHIFT(1225),
  [2656] = {.count = 1, .reusable = false}, SHIFT(1227),
  [2658] = {.count = 1, .reusable = false}, SHIFT(1228),
  [2660] = {.count = 1, .reusable = false}, SHIFT(1229),
  [2662] = {.count = 1, .reusable = true}, SHIFT(1234),
  [2664] = {.count = 1, .reusable = true}, SHIFT(1235),
  [2666] = {.count = 1, .reusable = false}, SHIFT(1237),
  [2668] = {.count = 1, .reusable = false}, SHIFT(1238),
  [2670] = {.count = 1, .reusable = true}, SHIFT(1242),
  [2672] = {.count = 1, .reusable = true}, SHIFT(1243),
  [2674] = {.count = 1, .reusable = true}, SHIFT(1244),
  [2676] = {.count = 1, .reusable = true}, SHIFT(1245),
  [2678] = {.count = 1, .reusable = true}, SHIFT(1246),
  [2680] = {.count = 1, .reusable = true}, SHIFT(1249),
  [2682] = {.count = 1, .reusable = true}, SHIFT(1251),
  [2684] = {.count = 1, .reusable = true}, SHIFT(1253),
  [2686] = {.count = 1, .reusable = true}, SHIFT(1257),
  [2688] = {.count = 1, .reusable = false}, SHIFT(1258),
  [2690] = {.count = 1, .reusable = true}, SHIFT(1259),
  [2692] = {.count = 1, .reusable = true}, REDUCE(sym_switch_body, 2),
  [2694] = {.count = 1, .reusable = false}, REDUCE(sym_switch_body, 2),
  [2696] = {.count = 1, .reusable = false}, SHIFT(1260),
  [2698] = {.count = 1, .reusable = false}, SHIFT(1261),
  [2700] = {.count = 1, .reusable = true}, SHIFT(1262),
  [2702] = {.count = 1, .reusable = false}, SHIFT(1263),
  [2704] = {.count = 1, .reusable = false}, SHIFT(1264),
  [2706] = {.count = 1, .reusable = false}, SHIFT(1265),
  [2708] = {.count = 1, .reusable = true}, SHIFT(1265),
  [2710] = {.count = 1, .reusable = true}, SHIFT(1266),
  [2712] = {.count = 1, .reusable = true}, SHIFT(1270),
  [2714] = {.count = 1, .reusable = false}, SHIFT(1270),
  [2716] = {.count = 1, .reusable = false}, SHIFT(1267),
  [2718] = {.count = 1, .reusable = false}, SHIFT(1268),
  [2720] = {.count = 1, .reusable = true}, SHIFT(1275),
  [2722] = {.count = 1, .reusable = true}, SHIFT(1277),
  [2724] = {.count = 1, .reusable = true}, SHIFT(1278),
  [2726] = {.count = 1, .reusable = true}, SHIFT(1279),
  [2728] = {.count = 1, .reusable = true}, SHIFT(1281),
  [2730] = {.count = 1, .reusable = true}, SHIFT(1282),
  [2732] = {.count = 1, .reusable = true}, SHIFT(1283),
  [2734] = {.count = 1, .reusable = true}, SHIFT(1284),
  [2736] = {.count = 1, .reusable = true}, SHIFT(1285),
  [2738] = {.count = 1, .reusable = false}, SHIFT(1287),
  [2740] = {.count = 1, .reusable = false}, SHIFT(1288),
  [2742] = {.count = 1, .reusable = true}, SHIFT(1289),
  [2744] = {.count = 1, .reusable = false}, SHIFT(1290),
  [2746] = {.count = 1, .reusable = false}, SHIFT(1291),
  [2748] = {.count = 1, .reusable = false}, SHIFT(1292),
  [2750] = {.count = 1, .reusable = true}, SHIFT(1292),
  [2752] = {.count = 1, .reusable = true}, SHIFT(1293),
  [2754] = {.count = 1, .reusable = true}, SHIFT(1297),
  [2756] = {.count = 1, .reusable = false}, SHIFT(1297),
  [2758] = {.count = 1, .reusable = false}, SHIFT(1294),
  [2760] = {.count = 1, .reusable = false}, SHIFT(1295),
  [2762] = {.count = 1, .reusable = false}, SHIFT(1302),
  [2764] = {.count = 1, .reusable = false}, SHIFT(1303),
  [2766] = {.count = 1, .reusable = true}, SHIFT(1304),
  [2768] = {.count = 1, .reusable = false}, SHIFT(1305),
  [2770] = {.count = 1, .reusable = false}, SHIFT(1306),
  [2772] = {.count = 1, .reusable = false}, SHIFT(1307),
  [2774] = {.count = 1, .reusable = true}, SHIFT(1307),
  [2776] = {.count = 1, .reusable = true}, SHIFT(1308),
  [2778] = {.count = 1, .reusable = true}, SHIFT(1311),
  [2780] = {.count = 1, .reusable = false}, SHIFT(1311),
  [2782] = {.count = 1, .reusable = false}, SHIFT(1309),
  [2784] = {.count = 1, .reusable = false}, SHIFT(1310),
  [2786] = {.count = 1, .reusable = true}, SHIFT(1316),
  [2788] = {.count = 1, .reusable = true}, SHIFT(1319),
  [2790] = {.count = 1, .reusable = false}, SHIFT(1319),
  [2792] = {.count = 1, .reusable = true}, SHIFT(1321),
  [2794] = {.count = 1, .reusable = false}, SHIFT(1321),
  [2796] = {.count = 1, .reusable = true}, SHIFT(1322),
  [2798] = {.count = 1, .reusable = true}, SHIFT(1323),
  [2800] = {.count = 1, .reusable = true}, SHIFT(1325),
  [2802] = {.count = 1, .reusable = false}, SHIFT(1326),
  [2804] = {.count = 1, .reusable = false}, SHIFT(1327),
  [2806] = {.count = 1, .reusable = true}, SHIFT(1328),
  [2808] = {.count = 1, .reusable = false}, SHIFT(1329),
  [2810] = {.count = 1, .reusable = true}, SHIFT(1330),
  [2812] = {.count = 1, .reusable = false}, SHIFT(1331),
  [2814] = {.count = 1, .reusable = true}, SHIFT(1332),
  [2816] = {.count = 1, .reusable = true}, REDUCE(aux_sym_formal_parameters_repeat1, 2, .alias_sequence_id = 9),
  [2818] = {.count = 1, .reusable = true}, REDUCE(aux_sym_formal_parameters_repeat1, 2),
  [2820] = {.count = 1, .reusable = true}, REDUCE(aux_sym_formal_parameters_repeat1, 2, .alias_sequence_id = 10),
  [2822] = {.count = 1, .reusable = true}, REDUCE(aux_sym_formal_parameters_repeat1, 2, .alias_sequence_id = 11),
  [2824] = {.count = 1, .reusable = true}, SHIFT(1337),
  [2826] = {.count = 1, .reusable = false}, SHIFT(1337),
  [2828] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 4, .alias_sequence_id = 9),
  [2830] = {.count = 2, .reusable = true}, REDUCE(aux_sym_formal_parameters_repeat1, 2), SHIFT_REPEAT(420),
  [2833] = {.count = 1, .reusable = true}, SHIFT(1340),
  [2835] = {.count = 1, .reusable = true}, SHIFT(1341),
  [2837] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 4),
  [2839] = {.count = 1, .reusable = true}, SHIFT(1342),
  [2841] = {.count = 1, .reusable = true}, SHIFT(1344),
  [2843] = {.count = 1, .reusable = true}, SHIFT(1345),
  [2845] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 4, .alias_sequence_id = 10),
  [2847] = {.count = 1, .reusable = true}, REDUCE(sym_formal_parameters, 4, .alias_sequence_id = 11),
  [2849] = {.count = 1, .reusable = true}, SHIFT(1346),
  [2851] = {.count = 1, .reusable = false}, SHIFT(1347),
  [2853] = {.count = 1, .reusable = true}, SHIFT(1353),
  [2855] = {.count = 1, .reusable = true}, SHIFT(1355),
  [2857] = {.count = 1, .reusable = true}, SHIFT(1357),
  [2859] = {.count = 1, .reusable = true}, SHIFT(1358),
  [2861] = {.count = 1, .reusable = true}, SHIFT(1359),
  [2863] = {.count = 1, .reusable = true}, SHIFT(1360),
  [2865] = {.count = 1, .reusable = true}, SHIFT(922),
  [2867] = {.count = 1, .reusable = true}, SHIFT(923),
  [2869] = {.count = 1, .reusable = true}, SHIFT(1361),
  [2871] = {.count = 1, .reusable = true}, SHIFT(1362),
  [2873] = {.count = 1, .reusable = true}, SHIFT(138),
  [2875] = {.count = 1, .reusable = true}, SHIFT(1367),
  [2877] = {.count = 1, .reusable = true}, SHIFT(1373),
  [2879] = {.count = 1, .reusable = true}, SHIFT(1378),
  [2881] = {.count = 1, .reusable = true}, SHIFT(1379),
  [2883] = {.count = 1, .reusable = true}, SHIFT(1380),
  [2885] = {.count = 1, .reusable = true}, SHIFT(1381),
  [2887] = {.count = 1, .reusable = true}, SHIFT(1382),
  [2889] = {.count = 1, .reusable = true}, SHIFT(1383),
  [2891] = {.count = 1, .reusable = true}, SHIFT(1386),
  [2893] = {.count = 1, .reusable = true}, SHIFT(1387),
  [2895] = {.count = 1, .reusable = true}, SHIFT(1388),
  [2897] = {.count = 1, .reusable = true}, SHIFT(1389),
  [2899] = {.count = 1, .reusable = true}, SHIFT(1390),
  [2901] = {.count = 1, .reusable = true}, SHIFT(1391),
  [2903] = {.count = 1, .reusable = false}, SHIFT(1394),
  [2905] = {.count = 1, .reusable = true}, SHIFT(1395),
  [2907] = {.count = 1, .reusable = false}, SHIFT(1397),
  [2909] = {.count = 1, .reusable = true}, SHIFT(1398),
  [2911] = {.count = 1, .reusable = false}, SHIFT(1399),
  [2913] = {.count = 1, .reusable = true}, SHIFT(1400),
  [2915] = {.count = 1, .reusable = true}, SHIFT(1401),
  [2917] = {.count = 1, .reusable = true}, SHIFT(1407),
  [2919] = {.count = 1, .reusable = true}, SHIFT(1408),
  [2921] = {.count = 1, .reusable = true}, SHIFT(1410),
  [2923] = {.count = 1, .reusable = true}, SHIFT(1412),
  [2925] = {.count = 1, .reusable = true}, REDUCE(sym_catch_clause, 2),
  [2927] = {.count = 1, .reusable = false}, REDUCE(sym_catch_clause, 2),
  [2929] = {.count = 1, .reusable = true}, SHIFT(1413),
  [2931] = {.count = 1, .reusable = true}, REDUCE(sym_finally_clause, 2),
  [2933] = {.count = 1, .reusable = false}, REDUCE(sym_finally_clause, 2),
  [2935] = {.count = 1, .reusable = true}, REDUCE(sym_try_statement, 4),
  [2937] = {.count = 1, .reusable = false}, REDUCE(sym_try_statement, 4),
  [2939] = {.count = 1, .reusable = true}, SHIFT(1414),
  [2941] = {.count = 1, .reusable = false}, SHIFT(1414),
  [2943] = {.count = 1, .reusable = true}, SHIFT(1415),
  [2945] = {.count = 1, .reusable = true}, REDUCE(sym_array, 4),
  [2947] = {.count = 1, .reusable = false}, REDUCE(sym_array, 4),
  [2949] = {.count = 1, .reusable = true}, SHIFT(1416),
  [2951] = {.count = 1, .reusable = false}, SHIFT(1417),
  [2953] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_expression, 2),
  [2955] = {.count = 1, .reusable = true}, SHIFT(1418),
  [2957] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_namespace_name, 3, .alias_sequence_id = 19),
  [2959] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_namespace_name, 3, .alias_sequence_id = 19),
  [2961] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_namespace_name, 3, .alias_sequence_id = 1),
  [2963] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_namespace_name, 3, .alias_sequence_id = 1),
  [2965] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_self_closing_element, 4, .alias_sequence_id = 9),
  [2967] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_self_closing_element, 4, .alias_sequence_id = 9),
  [2969] = {.count = 1, .reusable = true}, SHIFT(1419),
  [2971] = {.count = 1, .reusable = true}, SHIFT(1420),
  [2973] = {.count = 1, .reusable = true}, SHIFT(1422),
  [2975] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_opening_element, 4, .dynamic_precedence = -1, .alias_sequence_id = 9),
  [2977] = {.count = 1, .reusable = true}, SHIFT(1423),
  [2979] = {.count = 2, .reusable = true}, REDUCE(aux_sym_jsx_opening_element_repeat1, 2), SHIFT_REPEAT(531),
  [2982] = {.count = 1, .reusable = true}, REDUCE(aux_sym_jsx_opening_element_repeat1, 2),
  [2984] = {.count = 1, .reusable = false}, REDUCE(aux_sym_jsx_opening_element_repeat1, 2),
  [2986] = {.count = 2, .reusable = true}, REDUCE(aux_sym_jsx_opening_element_repeat1, 2), SHIFT_REPEAT(535),
  [2989] = {.count = 2, .reusable = false}, REDUCE(aux_sym_jsx_opening_element_repeat1, 2), SHIFT_REPEAT(536),
  [2992] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_namespace_name, 3, .alias_sequence_id = 20),
  [2994] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_namespace_name, 3, .alias_sequence_id = 20),
  [2996] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_namespace_name, 3),
  [2998] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_namespace_name, 3),
  [3000] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_self_closing_element, 4),
  [3002] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_self_closing_element, 4),
  [3004] = {.count = 1, .reusable = true}, REDUCE(sym_nested_identifier, 3),
  [3006] = {.count = 1, .reusable = false}, REDUCE(sym_nested_identifier, 3),
  [3008] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_opening_element, 4, .dynamic_precedence = -1),
  [3010] = {.count = 1, .reusable = true}, SHIFT(1424),
  [3012] = {.count = 1, .reusable = true}, SHIFT(1426),
  [3014] = {.count = 1, .reusable = true}, SHIFT(1427),
  [3016] = {.count = 1, .reusable = true}, SHIFT(1428),
  [3018] = {.count = 1, .reusable = false}, SHIFT(1429),
  [3020] = {.count = 1, .reusable = false}, SHIFT(1431),
  [3022] = {.count = 1, .reusable = true}, SHIFT(1433),
  [3024] = {.count = 1, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 2),
  [3026] = {.count = 1, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 2),
  [3028] = {.count = 1, .reusable = true}, REDUCE(sym_public_field_definition, 2),
  [3030] = {.count = 1, .reusable = true}, SHIFT(1434),
  [3032] = {.count = 2, .reusable = true}, REDUCE(aux_sym_export_statement_repeat1, 2), SHIFT_REPEAT(549),
  [3035] = {.count = 1, .reusable = true}, REDUCE(sym_class_body, 3),
  [3037] = {.count = 1, .reusable = false}, REDUCE(sym_class_body, 3),
  [3039] = {.count = 2, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(544),
  [3042] = {.count = 2, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(546),
  [3045] = {.count = 2, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(547),
  [3048] = {.count = 2, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(554),
  [3051] = {.count = 2, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(548),
  [3054] = {.count = 2, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(549),
  [3057] = {.count = 2, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(550),
  [3060] = {.count = 2, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 2), SHIFT_REPEAT(551),
  [3063] = {.count = 1, .reusable = true}, SHIFT(1437),
  [3065] = {.count = 1, .reusable = false}, SHIFT(1437),
  [3067] = {.count = 1, .reusable = true}, SHIFT(1438),
  [3069] = {.count = 1, .reusable = false}, SHIFT(1438),
  [3071] = {.count = 1, .reusable = true}, SHIFT(1439),
  [3073] = {.count = 1, .reusable = false}, SHIFT(1439),
  [3075] = {.count = 1, .reusable = true}, SHIFT(1440),
  [3077] = {.count = 1, .reusable = true}, SHIFT(1441),
  [3079] = {.count = 1, .reusable = false}, SHIFT(1441),
  [3081] = {.count = 1, .reusable = true}, SHIFT(1442),
  [3083] = {.count = 1, .reusable = false}, SHIFT(1442),
  [3085] = {.count = 1, .reusable = true}, SHIFT(1443),
  [3087] = {.count = 1, .reusable = false}, SHIFT(1443),
  [3089] = {.count = 1, .reusable = true}, SHIFT(1444),
  [3091] = {.count = 1, .reusable = false}, SHIFT(1444),
  [3093] = {.count = 1, .reusable = true}, SHIFT(1445),
  [3095] = {.count = 1, .reusable = false}, SHIFT(1445),
  [3097] = {.count = 1, .reusable = true}, SHIFT(1446),
  [3099] = {.count = 1, .reusable = false}, SHIFT(1446),
  [3101] = {.count = 1, .reusable = true}, SHIFT(1447),
  [3103] = {.count = 1, .reusable = true}, REDUCE(sym_class, 4),
  [3105] = {.count = 1, .reusable = false}, REDUCE(sym_class, 4),
  [3107] = {.count = 1, .reusable = true}, REDUCE(sym_function, 4),
  [3109] = {.count = 1, .reusable = false}, REDUCE(sym_function, 4),
  [3111] = {.count = 1, .reusable = true}, REDUCE(sym_arrow_function, 4),
  [3113] = {.count = 1, .reusable = false}, REDUCE(sym_arrow_function, 4),
  [3115] = {.count = 1, .reusable = true}, REDUCE(sym_generator_function, 4),
  [3117] = {.count = 1, .reusable = false}, REDUCE(sym_generator_function, 4),
  [3119] = {.count = 1, .reusable = true}, SHIFT(1450),
  [3121] = {.count = 1, .reusable = false}, REDUCE(sym_template_substitution, 3),
  [3123] = {.count = 1, .reusable = true}, REDUCE(sym_template_substitution, 3),
  [3125] = {.count = 1, .reusable = true}, SHIFT(1451),
  [3127] = {.count = 1, .reusable = false}, SHIFT(1451),
  [3129] = {.count = 1, .reusable = true}, REDUCE(sym_arguments, 3),
  [3131] = {.count = 1, .reusable = false}, REDUCE(sym_arguments, 3),
  [3133] = {.count = 1, .reusable = true}, SHIFT(1452),
  [3135] = {.count = 2, .reusable = true}, REDUCE(aux_sym_array_repeat1, 2), SHIFT_REPEAT(601),
  [3138] = {.count = 1, .reusable = true}, REDUCE(sym_subscript_expression, 4),
  [3140] = {.count = 1, .reusable = false}, REDUCE(sym_subscript_expression, 4),
  [3142] = {.count = 1, .reusable = true}, SHIFT(1453),
  [3144] = {.count = 1, .reusable = false}, SHIFT(1453),
  [3146] = {.count = 1, .reusable = true}, REDUCE(sym_decorator_member_expression, 3, .alias_sequence_id = 15),
  [3148] = {.count = 1, .reusable = true}, REDUCE(sym_decorator_member_expression, 3, .alias_sequence_id = 21),
  [3150] = {.count = 1, .reusable = true}, SHIFT(1454),
  [3152] = {.count = 1, .reusable = false}, SHIFT(1454),
  [3154] = {.count = 1, .reusable = true}, SHIFT(1455),
  [3156] = {.count = 1, .reusable = false}, SHIFT(1455),
  [3158] = {.count = 1, .reusable = true}, SHIFT(1456),
  [3160] = {.count = 1, .reusable = false}, SHIFT(1456),
  [3162] = {.count = 1, .reusable = true}, SHIFT(1457),
  [3164] = {.count = 1, .reusable = true}, SHIFT(1458),
  [3166] = {.count = 1, .reusable = false}, SHIFT(1458),
  [3168] = {.count = 1, .reusable = true}, SHIFT(1459),
  [3170] = {.count = 1, .reusable = false}, SHIFT(1459),
  [3172] = {.count = 1, .reusable = true}, SHIFT(1460),
  [3174] = {.count = 1, .reusable = false}, SHIFT(1460),
  [3176] = {.count = 1, .reusable = true}, SHIFT(1461),
  [3178] = {.count = 1, .reusable = false}, SHIFT(1461),
  [3180] = {.count = 1, .reusable = true}, SHIFT(1462),
  [3182] = {.count = 1, .reusable = false}, SHIFT(1462),
  [3184] = {.count = 1, .reusable = true}, SHIFT(1463),
  [3186] = {.count = 1, .reusable = false}, SHIFT(1463),
  [3188] = {.count = 1, .reusable = true}, SHIFT(1464),
  [3190] = {.count = 1, .reusable = false}, SHIFT(1464),
  [3192] = {.count = 1, .reusable = true}, SHIFT(1465),
  [3194] = {.count = 1, .reusable = false}, SHIFT(1465),
  [3196] = {.count = 1, .reusable = false}, SHIFT(1466),
  [3198] = {.count = 1, .reusable = true}, SHIFT(1467),
  [3200] = {.count = 1, .reusable = true}, SHIFT(1468),
  [3202] = {.count = 1, .reusable = false}, SHIFT(1469),
  [3204] = {.count = 1, .reusable = true}, SHIFT(1470),
  [3206] = {.count = 1, .reusable = false}, SHIFT(1470),
  [3208] = {.count = 1, .reusable = true}, SHIFT(1471),
  [3210] = {.count = 1, .reusable = false}, SHIFT(1471),
  [3212] = {.count = 1, .reusable = false}, SHIFT(1472),
  [3214] = {.count = 1, .reusable = true}, SHIFT(1473),
  [3216] = {.count = 1, .reusable = true}, SHIFT(1474),
  [3218] = {.count = 1, .reusable = false}, SHIFT(1474),
  [3220] = {.count = 1, .reusable = true}, SHIFT(1475),
  [3222] = {.count = 1, .reusable = true}, SHIFT(1476),
  [3224] = {.count = 1, .reusable = true}, SHIFT(1477),
  [3226] = {.count = 1, .reusable = false}, SHIFT(1476),
  [3228] = {.count = 1, .reusable = false}, SHIFT(1477),
  [3230] = {.count = 1, .reusable = false}, SHIFT(1478),
  [3232] = {.count = 1, .reusable = true}, SHIFT(1472),
  [3234] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_expression, 3),
  [3236] = {.count = 1, .reusable = true}, SHIFT(1479),
  [3238] = {.count = 1, .reusable = true}, SHIFT(1480),
  [3240] = {.count = 1, .reusable = true}, SHIFT(1481),
  [3242] = {.count = 1, .reusable = false}, SHIFT(1482),
  [3244] = {.count = 1, .reusable = true}, SHIFT(1483),
  [3246] = {.count = 1, .reusable = false}, SHIFT(1484),
  [3248] = {.count = 1, .reusable = true}, SHIFT(1485),
  [3250] = {.count = 1, .reusable = false}, SHIFT(1486),
  [3252] = {.count = 1, .reusable = true}, SHIFT(1489),
  [3254] = {.count = 1, .reusable = true}, REDUCE(sym_anonymous_class, 4),
  [3256] = {.count = 1, .reusable = false}, REDUCE(sym_anonymous_class, 4),
  [3258] = {.count = 1, .reusable = true}, SHIFT(1491),
  [3260] = {.count = 1, .reusable = false}, SHIFT(1491),
  [3262] = {.count = 1, .reusable = true}, SHIFT(1492),
  [3264] = {.count = 1, .reusable = true}, REDUCE(sym__import_export_specifier, 3),
  [3266] = {.count = 1, .reusable = true}, REDUCE(sym_export_clause, 4, .alias_sequence_id = 17),
  [3268] = {.count = 1, .reusable = true}, REDUCE(aux_sym_export_clause_repeat1, 2, .alias_sequence_id = 17),
  [3270] = {.count = 1, .reusable = true}, SHIFT(1493),
  [3272] = {.count = 2, .reusable = true}, REDUCE(aux_sym_export_clause_repeat1, 2), SHIFT_REPEAT(1494),
  [3275] = {.count = 1, .reusable = true}, REDUCE(aux_sym_export_clause_repeat1, 2),
  [3277] = {.count = 1, .reusable = true}, SHIFT(1495),
  [3279] = {.count = 1, .reusable = true}, SHIFT(1498),
  [3281] = {.count = 1, .reusable = true}, SHIFT(1500),
  [3283] = {.count = 1, .reusable = false}, SHIFT(1500),
  [3285] = {.count = 1, .reusable = true}, SHIFT(1501),
  [3287] = {.count = 1, .reusable = false}, SHIFT(1501),
  [3289] = {.count = 1, .reusable = true}, SHIFT(1502),
  [3291] = {.count = 1, .reusable = false}, SHIFT(1502),
  [3293] = {.count = 1, .reusable = true}, SHIFT(1503),
  [3295] = {.count = 1, .reusable = true}, SHIFT(1504),
  [3297] = {.count = 1, .reusable = false}, SHIFT(1504),
  [3299] = {.count = 1, .reusable = true}, SHIFT(1505),
  [3301] = {.count = 1, .reusable = false}, SHIFT(1505),
  [3303] = {.count = 1, .reusable = true}, SHIFT(1506),
  [3305] = {.count = 1, .reusable = false}, SHIFT(1506),
  [3307] = {.count = 1, .reusable = true}, SHIFT(1507),
  [3309] = {.count = 1, .reusable = false}, SHIFT(1507),
  [3311] = {.count = 1, .reusable = true}, SHIFT(1508),
  [3313] = {.count = 1, .reusable = false}, SHIFT(1508),
  [3315] = {.count = 1, .reusable = true}, SHIFT(1509),
  [3317] = {.count = 1, .reusable = false}, SHIFT(1509),
  [3319] = {.count = 1, .reusable = true}, REDUCE(sym_method_definition, 4),
  [3321] = {.count = 1, .reusable = false}, REDUCE(sym_method_definition, 4),
  [3323] = {.count = 1, .reusable = true}, SHIFT(1510),
  [3325] = {.count = 1, .reusable = true}, SHIFT(1511),
  [3327] = {.count = 1, .reusable = true}, SHIFT(1513),
  [3329] = {.count = 1, .reusable = true}, SHIFT(1514),
  [3331] = {.count = 1, .reusable = true}, SHIFT(1515),
  [3333] = {.count = 1, .reusable = false}, SHIFT(1515),
  [3335] = {.count = 1, .reusable = true}, SHIFT(1518),
  [3337] = {.count = 1, .reusable = true}, SHIFT(1519),
  [3339] = {.count = 1, .reusable = false}, SHIFT(1521),
  [3341] = {.count = 1, .reusable = false}, SHIFT(1522),
  [3343] = {.count = 1, .reusable = false}, SHIFT(1523),
  [3345] = {.count = 1, .reusable = true}, SHIFT(1527),
  [3347] = {.count = 1, .reusable = true}, SHIFT(1528),
  [3349] = {.count = 1, .reusable = true}, SHIFT(1529),
  [3351] = {.count = 1, .reusable = true}, SHIFT(1530),
  [3353] = {.count = 1, .reusable = true}, SHIFT(1531),
  [3355] = {.count = 1, .reusable = true}, SHIFT(1532),
  [3357] = {.count = 1, .reusable = true}, SHIFT(1534),
  [3359] = {.count = 1, .reusable = false}, SHIFT(1534),
  [3361] = {.count = 1, .reusable = true}, SHIFT(1535),
  [3363] = {.count = 1, .reusable = true}, SHIFT(1538),
  [3365] = {.count = 1, .reusable = false}, SHIFT(1538),
  [3367] = {.count = 1, .reusable = true}, SHIFT(1540),
  [3369] = {.count = 1, .reusable = false}, SHIFT(1540),
  [3371] = {.count = 1, .reusable = true}, SHIFT(1542),
  [3373] = {.count = 1, .reusable = false}, SHIFT(1542),
  [3375] = {.count = 1, .reusable = true}, SHIFT(1543),
  [3377] = {.count = 1, .reusable = true}, SHIFT(1544),
  [3379] = {.count = 1, .reusable = false}, SHIFT(1544),
  [3381] = {.count = 1, .reusable = true}, SHIFT(1545),
  [3383] = {.count = 1, .reusable = false}, SHIFT(1545),
  [3385] = {.count = 1, .reusable = true}, SHIFT(1546),
  [3387] = {.count = 1, .reusable = true}, SHIFT(1547),
  [3389] = {.count = 1, .reusable = false}, SHIFT(1547),
  [3391] = {.count = 1, .reusable = true}, SHIFT(1548),
  [3393] = {.count = 1, .reusable = false}, SHIFT(1548),
  [3395] = {.count = 1, .reusable = true}, SHIFT(1549),
  [3397] = {.count = 1, .reusable = false}, SHIFT(1549),
  [3399] = {.count = 1, .reusable = true}, SHIFT(1550),
  [3401] = {.count = 1, .reusable = false}, SHIFT(1550),
  [3403] = {.count = 1, .reusable = true}, SHIFT(1551),
  [3405] = {.count = 1, .reusable = false}, SHIFT(1551),
  [3407] = {.count = 1, .reusable = true}, SHIFT(1552),
  [3409] = {.count = 1, .reusable = false}, SHIFT(1552),
  [3411] = {.count = 1, .reusable = true}, SHIFT(1553),
  [3413] = {.count = 1, .reusable = false}, REDUCE(sym_decorator_member_expression, 3, .alias_sequence_id = 15),
  [3415] = {.count = 1, .reusable = false}, REDUCE(sym_decorator_member_expression, 3, .alias_sequence_id = 21),
  [3417] = {.count = 1, .reusable = true}, REDUCE(sym_object, 5),
  [3419] = {.count = 1, .reusable = false}, REDUCE(sym_object, 5),
  [3421] = {.count = 1, .reusable = true}, REDUCE(sym_named_imports, 4, .alias_sequence_id = 18),
  [3423] = {.count = 1, .reusable = true}, REDUCE(aux_sym_named_imports_repeat1, 2, .alias_sequence_id = 18),
  [3425] = {.count = 1, .reusable = true}, SHIFT(1555),
  [3427] = {.count = 2, .reusable = true}, REDUCE(aux_sym_named_imports_repeat1, 2), SHIFT_REPEAT(1556),
  [3430] = {.count = 1, .reusable = true}, REDUCE(aux_sym_named_imports_repeat1, 2),
  [3432] = {.count = 1, .reusable = true}, SHIFT(1557),
  [3434] = {.count = 1, .reusable = true}, SHIFT(1558),
  [3436] = {.count = 1, .reusable = true}, SHIFT(1559),
  [3438] = {.count = 1, .reusable = true}, SHIFT(1562),
  [3440] = {.count = 1, .reusable = true}, SHIFT(1564),
  [3442] = {.count = 1, .reusable = true}, SHIFT(1566),
  [3444] = {.count = 1, .reusable = true}, SHIFT(1568),
  [3446] = {.count = 1, .reusable = true}, SHIFT(1569),
  [3448] = {.count = 1, .reusable = true}, SHIFT(1570),
  [3450] = {.count = 1, .reusable = true}, SHIFT(1571),
  [3452] = {.count = 1, .reusable = false}, SHIFT(1572),
  [3454] = {.count = 1, .reusable = true}, SHIFT(1573),
  [3456] = {.count = 1, .reusable = true}, SHIFT(1575),
  [3458] = {.count = 1, .reusable = false}, SHIFT(1578),
  [3460] = {.count = 1, .reusable = true}, SHIFT(1579),
  [3462] = {.count = 1, .reusable = false}, SHIFT(1581),
  [3464] = {.count = 1, .reusable = true}, SHIFT(1582),
  [3466] = {.count = 1, .reusable = false}, SHIFT(1583),
  [3468] = {.count = 1, .reusable = true}, SHIFT(1584),
  [3470] = {.count = 1, .reusable = true}, SHIFT(1586),
  [3472] = {.count = 1, .reusable = true}, SHIFT(1587),
  [3474] = {.count = 1, .reusable = true}, SHIFT(1588),
  [3476] = {.count = 1, .reusable = true}, SHIFT(1590),
  [3478] = {.count = 1, .reusable = true}, SHIFT(1593),
  [3480] = {.count = 1, .reusable = true}, SHIFT(1600),
  [3482] = {.count = 1, .reusable = true}, REDUCE(sym_if_statement, 5),
  [3484] = {.count = 1, .reusable = false}, REDUCE(sym_if_statement, 5),
  [3486] = {.count = 1, .reusable = true}, SHIFT(1602),
  [3488] = {.count = 1, .reusable = false}, SHIFT(1602),
  [3490] = {.count = 1, .reusable = false}, SHIFT(1604),
  [3492] = {.count = 1, .reusable = false}, REDUCE(sym_switch_default, 2),
  [3494] = {.count = 1, .reusable = true}, SHIFT(1605),
  [3496] = {.count = 1, .reusable = true}, REDUCE(sym_switch_default, 2),
  [3498] = {.count = 1, .reusable = false}, SHIFT(1606),
  [3500] = {.count = 1, .reusable = false}, SHIFT(1607),
  [3502] = {.count = 1, .reusable = false}, SHIFT(1608),
  [3504] = {.count = 1, .reusable = false}, SHIFT(1609),
  [3506] = {.count = 1, .reusable = false}, SHIFT(1610),
  [3508] = {.count = 1, .reusable = false}, SHIFT(1611),
  [3510] = {.count = 1, .reusable = false}, SHIFT(1612),
  [3512] = {.count = 1, .reusable = false}, SHIFT(1613),
  [3514] = {.count = 1, .reusable = false}, SHIFT(1614),
  [3516] = {.count = 1, .reusable = false}, SHIFT(1615),
  [3518] = {.count = 1, .reusable = false}, SHIFT(1616),
  [3520] = {.count = 1, .reusable = false}, SHIFT(1617),
  [3522] = {.count = 1, .reusable = false}, SHIFT(1618),
  [3524] = {.count = 1, .reusable = false}, SHIFT(1619),
  [3526] = {.count = 1, .reusable = false}, SHIFT(1620),
  [3528] = {.count = 1, .reusable = true}, SHIFT(1621),
  [3530] = {.count = 1, .reusable = false}, SHIFT(1622),
  [3532] = {.count = 1, .reusable = false}, SHIFT(1623),
  [3534] = {.count = 1, .reusable = false}, SHIFT(1624),
  [3536] = {.count = 1, .reusable = false}, SHIFT(1625),
  [3538] = {.count = 1, .reusable = true}, SHIFT(1631),
  [3540] = {.count = 1, .reusable = false}, SHIFT(1631),
  [3542] = {.count = 1, .reusable = false}, SHIFT(1632),
  [3544] = {.count = 1, .reusable = true}, SHIFT(1633),
  [3546] = {.count = 1, .reusable = true}, SHIFT(1634),
  [3548] = {.count = 1, .reusable = false}, SHIFT(1635),
  [3550] = {.count = 1, .reusable = true}, SHIFT(1636),
  [3552] = {.count = 1, .reusable = false}, SHIFT(1636),
  [3554] = {.count = 1, .reusable = true}, SHIFT(1637),
  [3556] = {.count = 1, .reusable = false}, SHIFT(1637),
  [3558] = {.count = 1, .reusable = false}, SHIFT(1638),
  [3560] = {.count = 1, .reusable = true}, SHIFT(1639),
  [3562] = {.count = 1, .reusable = true}, SHIFT(1640),
  [3564] = {.count = 1, .reusable = true}, SHIFT(1641),
  [3566] = {.count = 1, .reusable = true}, SHIFT(1642),
  [3568] = {.count = 1, .reusable = false}, SHIFT(1641),
  [3570] = {.count = 1, .reusable = true}, SHIFT(1643),
  [3572] = {.count = 1, .reusable = true}, SHIFT(1644),
  [3574] = {.count = 1, .reusable = true}, SHIFT(1645),
  [3576] = {.count = 1, .reusable = false}, SHIFT(1644),
  [3578] = {.count = 1, .reusable = false}, SHIFT(1645),
  [3580] = {.count = 1, .reusable = false}, SHIFT(1646),
  [3582] = {.count = 1, .reusable = true}, SHIFT(1638),
  [3584] = {.count = 1, .reusable = true}, REDUCE(sym_switch_body, 3),
  [3586] = {.count = 1, .reusable = false}, REDUCE(sym_switch_body, 3),
  [3588] = {.count = 2, .reusable = true}, REDUCE(aux_sym_switch_body_repeat1, 2), SHIFT_REPEAT(830),
  [3591] = {.count = 1, .reusable = true}, REDUCE(aux_sym_switch_body_repeat1, 2),
  [3593] = {.count = 2, .reusable = true}, REDUCE(aux_sym_switch_body_repeat1, 2), SHIFT_REPEAT(832),
  [3596] = {.count = 1, .reusable = true}, SHIFT(1648),
  [3598] = {.count = 1, .reusable = false}, SHIFT(1648),
  [3600] = {.count = 1, .reusable = true}, SHIFT(1649),
  [3602] = {.count = 1, .reusable = true}, SHIFT(1651),
  [3604] = {.count = 1, .reusable = false}, SHIFT(1651),
  [3606] = {.count = 1, .reusable = true}, SHIFT(1653),
  [3608] = {.count = 1, .reusable = false}, SHIFT(1653),
  [3610] = {.count = 1, .reusable = true}, SHIFT(1654),
  [3612] = {.count = 1, .reusable = true}, SHIFT(1655),
  [3614] = {.count = 1, .reusable = false}, SHIFT(1655),
  [3616] = {.count = 1, .reusable = false}, SHIFT(1656),
  [3618] = {.count = 1, .reusable = true}, SHIFT(1657),
  [3620] = {.count = 1, .reusable = true}, SHIFT(1658),
  [3622] = {.count = 1, .reusable = false}, SHIFT(1659),
  [3624] = {.count = 1, .reusable = true}, SHIFT(1660),
  [3626] = {.count = 1, .reusable = false}, SHIFT(1660),
  [3628] = {.count = 1, .reusable = true}, SHIFT(1661),
  [3630] = {.count = 1, .reusable = false}, SHIFT(1661),
  [3632] = {.count = 1, .reusable = false}, SHIFT(1662),
  [3634] = {.count = 1, .reusable = true}, SHIFT(1663),
  [3636] = {.count = 1, .reusable = true}, SHIFT(1664),
  [3638] = {.count = 1, .reusable = true}, SHIFT(1665),
  [3640] = {.count = 1, .reusable = false}, SHIFT(1665),
  [3642] = {.count = 1, .reusable = true}, SHIFT(1670),
  [3644] = {.count = 1, .reusable = false}, SHIFT(1670),
  [3646] = {.count = 1, .reusable = false}, SHIFT(1671),
  [3648] = {.count = 1, .reusable = true}, SHIFT(1672),
  [3650] = {.count = 1, .reusable = true}, SHIFT(1673),
  [3652] = {.count = 1, .reusable = false}, SHIFT(1674),
  [3654] = {.count = 1, .reusable = true}, SHIFT(1675),
  [3656] = {.count = 1, .reusable = false}, SHIFT(1675),
  [3658] = {.count = 1, .reusable = true}, SHIFT(1676),
  [3660] = {.count = 1, .reusable = false}, SHIFT(1676),
  [3662] = {.count = 1, .reusable = false}, SHIFT(1677),
  [3664] = {.count = 1, .reusable = true}, SHIFT(1678),
  [3666] = {.count = 1, .reusable = true}, SHIFT(1679),
  [3668] = {.count = 1, .reusable = true}, SHIFT(1680),
  [3670] = {.count = 1, .reusable = false}, SHIFT(1679),
  [3672] = {.count = 1, .reusable = true}, SHIFT(1681),
  [3674] = {.count = 1, .reusable = true}, SHIFT(1682),
  [3676] = {.count = 1, .reusable = true}, SHIFT(1683),
  [3678] = {.count = 1, .reusable = false}, SHIFT(1682),
  [3680] = {.count = 1, .reusable = false}, SHIFT(1683),
  [3682] = {.count = 1, .reusable = false}, SHIFT(1684),
  [3684] = {.count = 1, .reusable = true}, SHIFT(1677),
  [3686] = {.count = 1, .reusable = true}, SHIFT(1686),
  [3688] = {.count = 1, .reusable = true}, SHIFT(1687),
  [3690] = {.count = 1, .reusable = true}, SHIFT(1688),
  [3692] = {.count = 1, .reusable = true}, SHIFT(1689),
  [3694] = {.count = 1, .reusable = true}, SHIFT(1690),
  [3696] = {.count = 1, .reusable = true}, SHIFT(1691),
  [3698] = {.count = 1, .reusable = true}, SHIFT(1692),
  [3700] = {.count = 1, .reusable = false}, SHIFT(1693),
  [3702] = {.count = 1, .reusable = true}, SHIFT(1694),
  [3704] = {.count = 1, .reusable = true}, SHIFT(1695),
  [3706] = {.count = 1, .reusable = true}, SHIFT(1698),
  [3708] = {.count = 1, .reusable = true}, SHIFT(1699),
  [3710] = {.count = 1, .reusable = false}, SHIFT(1699),
  [3712] = {.count = 1, .reusable = true}, SHIFT(1700),
  [3714] = {.count = 1, .reusable = true}, SHIFT(1701),
  [3716] = {.count = 1, .reusable = true}, SHIFT(1707),
  [3718] = {.count = 1, .reusable = false}, SHIFT(1707),
  [3720] = {.count = 1, .reusable = true}, SHIFT(482),
  [3722] = {.count = 1, .reusable = true}, SHIFT(483),
  [3724] = {.count = 1, .reusable = true}, SHIFT(1708),
  [3726] = {.count = 1, .reusable = true}, SHIFT(1709),
  [3728] = {.count = 1, .reusable = true}, SHIFT(1710),
  [3730] = {.count = 1, .reusable = true}, SHIFT(1712),
  [3732] = {.count = 1, .reusable = true}, SHIFT(1714),
  [3734] = {.count = 1, .reusable = true}, SHIFT(1717),
  [3736] = {.count = 1, .reusable = false}, SHIFT(1719),
  [3738] = {.count = 1, .reusable = false}, SHIFT(1720),
  [3740] = {.count = 1, .reusable = false}, SHIFT(1721),
  [3742] = {.count = 1, .reusable = true}, SHIFT(1726),
  [3744] = {.count = 1, .reusable = true}, SHIFT(1727),
  [3746] = {.count = 1, .reusable = true}, SHIFT(1728),
  [3748] = {.count = 1, .reusable = true}, SHIFT(1732),
  [3750] = {.count = 1, .reusable = true}, SHIFT(1733),
  [3752] = {.count = 1, .reusable = true}, SHIFT(1734),
  [3754] = {.count = 1, .reusable = true}, SHIFT(1735),
  [3756] = {.count = 1, .reusable = true}, SHIFT(1737),
  [3758] = {.count = 1, .reusable = true}, SHIFT(1738),
  [3760] = {.count = 1, .reusable = true}, SHIFT(1739),
  [3762] = {.count = 1, .reusable = true}, SHIFT(1740),
  [3764] = {.count = 1, .reusable = true}, SHIFT(1741),
  [3766] = {.count = 1, .reusable = true}, SHIFT(1742),
  [3768] = {.count = 1, .reusable = true}, SHIFT(1743),
  [3770] = {.count = 1, .reusable = true}, SHIFT(1745),
  [3772] = {.count = 1, .reusable = false}, SHIFT(1745),
  [3774] = {.count = 1, .reusable = true}, SHIFT(1746),
  [3776] = {.count = 1, .reusable = true}, SHIFT(1749),
  [3778] = {.count = 1, .reusable = false}, SHIFT(1749),
  [3780] = {.count = 1, .reusable = true}, SHIFT(1751),
  [3782] = {.count = 1, .reusable = false}, SHIFT(1751),
  [3784] = {.count = 1, .reusable = true}, SHIFT(1752),
  [3786] = {.count = 1, .reusable = true}, REDUCE(sym_do_statement, 5),
  [3788] = {.count = 1, .reusable = false}, REDUCE(sym_do_statement, 5),
  [3790] = {.count = 1, .reusable = true}, SHIFT(1756),
  [3792] = {.count = 1, .reusable = true}, SHIFT(1757),
  [3794] = {.count = 1, .reusable = true}, SHIFT(1758),
  [3796] = {.count = 1, .reusable = false}, SHIFT(1758),
  [3798] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_fragment, 5),
  [3800] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_fragment, 5),
  [3802] = {.count = 1, .reusable = true}, SHIFT(1759),
  [3804] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_expression, 3),
  [3806] = {.count = 1, .reusable = true}, SHIFT(1760),
  [3808] = {.count = 1, .reusable = true}, SHIFT(1761),
  [3810] = {.count = 1, .reusable = false}, SHIFT(1762),
  [3812] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_attribute, 3, .alias_sequence_id = 2),
  [3814] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_attribute, 3, .alias_sequence_id = 2),
  [3816] = {.count = 1, .reusable = false}, SHIFT(1765),
  [3818] = {.count = 1, .reusable = false}, SHIFT(1767),
  [3820] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_attribute, 3),
  [3822] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_attribute, 3),
  [3824] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_self_closing_element, 5, .alias_sequence_id = 9),
  [3826] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_self_closing_element, 5, .alias_sequence_id = 9),
  [3828] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_self_closing_element, 5),
  [3830] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_self_closing_element, 5),
  [3832] = {.count = 1, .reusable = true}, SHIFT(1769),
  [3834] = {.count = 1, .reusable = true}, SHIFT(1770),
  [3836] = {.count = 1, .reusable = false}, SHIFT(1770),
  [3838] = {.count = 1, .reusable = true}, SHIFT(1773),
  [3840] = {.count = 1, .reusable = true}, SHIFT(1774),
  [3842] = {.count = 1, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 3),
  [3844] = {.count = 1, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 3),
  [3846] = {.count = 1, .reusable = true}, SHIFT(1775),
  [3848] = {.count = 1, .reusable = true}, SHIFT(1777),
  [3850] = {.count = 1, .reusable = true}, SHIFT(1778),
  [3852] = {.count = 1, .reusable = false}, SHIFT(1778),
  [3854] = {.count = 1, .reusable = true}, SHIFT(1779),
  [3856] = {.count = 1, .reusable = true}, REDUCE(sym_function, 5),
  [3858] = {.count = 1, .reusable = false}, REDUCE(sym_function, 5),
  [3860] = {.count = 1, .reusable = true}, REDUCE(sym_generator_function, 5),
  [3862] = {.count = 1, .reusable = false}, REDUCE(sym_generator_function, 5),
  [3864] = {.count = 1, .reusable = true}, REDUCE(sym_arguments, 4),
  [3866] = {.count = 1, .reusable = false}, REDUCE(sym_arguments, 4),
  [3868] = {.count = 1, .reusable = true}, SHIFT(1780),
  [3870] = {.count = 1, .reusable = false}, SHIFT(1780),
  [3872] = {.count = 1, .reusable = true}, REDUCE(sym_ternary_expression, 5),
  [3874] = {.count = 1, .reusable = true}, SHIFT(1781),
  [3876] = {.count = 1, .reusable = true}, SHIFT(1782),
  [3878] = {.count = 1, .reusable = false}, SHIFT(1782),
  [3880] = {.count = 1, .reusable = true}, SHIFT(1783),
  [3882] = {.count = 1, .reusable = false}, SHIFT(1783),
  [3884] = {.count = 1, .reusable = true}, SHIFT(1784),
  [3886] = {.count = 1, .reusable = false}, SHIFT(1784),
  [3888] = {.count = 1, .reusable = true}, SHIFT(1785),
  [3890] = {.count = 1, .reusable = true}, SHIFT(1786),
  [3892] = {.count = 1, .reusable = false}, SHIFT(1786),
  [3894] = {.count = 1, .reusable = true}, SHIFT(1787),
  [3896] = {.count = 1, .reusable = false}, SHIFT(1787),
  [3898] = {.count = 1, .reusable = true}, SHIFT(1788),
  [3900] = {.count = 1, .reusable = false}, SHIFT(1788),
  [3902] = {.count = 1, .reusable = true}, SHIFT(1789),
  [3904] = {.count = 1, .reusable = false}, SHIFT(1789),
  [3906] = {.count = 1, .reusable = true}, SHIFT(1790),
  [3908] = {.count = 1, .reusable = false}, SHIFT(1790),
  [3910] = {.count = 1, .reusable = true}, SHIFT(1791),
  [3912] = {.count = 1, .reusable = false}, SHIFT(1791),
  [3914] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_closing_element, 4, .alias_sequence_id = 20),
  [3916] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_closing_element, 4, .alias_sequence_id = 20),
  [3918] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_closing_element, 4),
  [3920] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_closing_element, 4),
  [3922] = {.count = 1, .reusable = true}, SHIFT(1792),
  [3924] = {.count = 1, .reusable = true}, SHIFT(1793),
  [3926] = {.count = 1, .reusable = true}, SHIFT(1794),
  [3928] = {.count = 1, .reusable = true}, SHIFT(1795),
  [3930] = {.count = 1, .reusable = true}, REDUCE(sym_export_statement, 5),
  [3932] = {.count = 1, .reusable = false}, REDUCE(sym_export_statement, 5),
  [3934] = {.count = 1, .reusable = true}, REDUCE(sym_class, 5),
  [3936] = {.count = 1, .reusable = false}, REDUCE(sym_class, 5),
  [3938] = {.count = 1, .reusable = true}, SHIFT(1796),
  [3940] = {.count = 1, .reusable = false}, SHIFT(1796),
  [3942] = {.count = 1, .reusable = true}, REDUCE(sym_export_clause, 5, .alias_sequence_id = 17),
  [3944] = {.count = 1, .reusable = true}, SHIFT(1797),
  [3946] = {.count = 1, .reusable = false}, SHIFT(1797),
  [3948] = {.count = 1, .reusable = true}, SHIFT(1798),
  [3950] = {.count = 1, .reusable = true}, SHIFT(1799),
  [3952] = {.count = 1, .reusable = true}, SHIFT(1800),
  [3954] = {.count = 1, .reusable = false}, SHIFT(1802),
  [3956] = {.count = 1, .reusable = true}, SHIFT(1803),
  [3958] = {.count = 1, .reusable = false}, SHIFT(1806),
  [3960] = {.count = 1, .reusable = true}, SHIFT(1807),
  [3962] = {.count = 1, .reusable = false}, SHIFT(1809),
  [3964] = {.count = 1, .reusable = true}, SHIFT(1810),
  [3966] = {.count = 1, .reusable = false}, SHIFT(1811),
  [3968] = {.count = 1, .reusable = true}, SHIFT(1812),
  [3970] = {.count = 1, .reusable = true}, SHIFT(1814),
  [3972] = {.count = 1, .reusable = false}, SHIFT(1814),
  [3974] = {.count = 1, .reusable = true}, SHIFT(1816),
  [3976] = {.count = 1, .reusable = false}, SHIFT(1816),
  [3978] = {.count = 1, .reusable = true}, SHIFT(1818),
  [3980] = {.count = 1, .reusable = false}, SHIFT(1818),
  [3982] = {.count = 1, .reusable = true}, SHIFT(1819),
  [3984] = {.count = 1, .reusable = true}, SHIFT(1820),
  [3986] = {.count = 1, .reusable = true}, SHIFT(1821),
  [3988] = {.count = 1, .reusable = true}, SHIFT(1822),
  [3990] = {.count = 1, .reusable = true}, SHIFT(1823),
  [3992] = {.count = 1, .reusable = true}, SHIFT(1824),
  [3994] = {.count = 1, .reusable = true}, SHIFT(1825),
  [3996] = {.count = 1, .reusable = true}, REDUCE(sym_method_definition, 5),
  [3998] = {.count = 1, .reusable = false}, REDUCE(sym_method_definition, 5),
  [4000] = {.count = 1, .reusable = true}, SHIFT(1826),
  [4002] = {.count = 1, .reusable = false}, SHIFT(1826),
  [4004] = {.count = 1, .reusable = true}, SHIFT(1827),
  [4006] = {.count = 1, .reusable = false}, SHIFT(1827),
  [4008] = {.count = 1, .reusable = true}, SHIFT(1828),
  [4010] = {.count = 1, .reusable = true}, SHIFT(1829),
  [4012] = {.count = 1, .reusable = true}, REDUCE(sym_named_imports, 5, .alias_sequence_id = 18),
  [4014] = {.count = 1, .reusable = true}, SHIFT(1830),
  [4016] = {.count = 1, .reusable = true}, SHIFT(1837),
  [4018] = {.count = 1, .reusable = true}, SHIFT(1841),
  [4020] = {.count = 1, .reusable = true}, SHIFT(1842),
  [4022] = {.count = 1, .reusable = true}, SHIFT(1843),
  [4024] = {.count = 1, .reusable = true}, SHIFT(1844),
  [4026] = {.count = 1, .reusable = true}, SHIFT(1845),
  [4028] = {.count = 1, .reusable = true}, SHIFT(1846),
  [4030] = {.count = 1, .reusable = true}, SHIFT(1847),
  [4032] = {.count = 1, .reusable = true}, SHIFT(1849),
  [4034] = {.count = 1, .reusable = false}, SHIFT(1849),
  [4036] = {.count = 1, .reusable = true}, SHIFT(1850),
  [4038] = {.count = 1, .reusable = true}, SHIFT(1853),
  [4040] = {.count = 1, .reusable = false}, SHIFT(1853),
  [4042] = {.count = 1, .reusable = true}, SHIFT(1855),
  [4044] = {.count = 1, .reusable = false}, SHIFT(1855),
  [4046] = {.count = 1, .reusable = true}, SHIFT(1856),
  [4048] = {.count = 1, .reusable = true}, SHIFT(1857),
  [4050] = {.count = 1, .reusable = true}, SHIFT(1859),
  [4052] = {.count = 1, .reusable = true}, SHIFT(1860),
  [4054] = {.count = 1, .reusable = true}, SHIFT(1861),
  [4056] = {.count = 1, .reusable = true}, SHIFT(1864),
  [4058] = {.count = 1, .reusable = true}, SHIFT(1865),
  [4060] = {.count = 1, .reusable = true}, SHIFT(1867),
  [4062] = {.count = 1, .reusable = true}, SHIFT(1868),
  [4064] = {.count = 1, .reusable = true}, SHIFT(1607),
  [4066] = {.count = 1, .reusable = true}, SHIFT(1608),
  [4068] = {.count = 1, .reusable = true}, SHIFT(1869),
  [4070] = {.count = 1, .reusable = true}, SHIFT(1870),
  [4072] = {.count = 1, .reusable = true}, SHIFT(1871),
  [4074] = {.count = 1, .reusable = true}, SHIFT(1876),
  [4076] = {.count = 1, .reusable = true}, SHIFT(1878),
  [4078] = {.count = 1, .reusable = true}, SHIFT(1884),
  [4080] = {.count = 1, .reusable = true}, SHIFT(1887),
  [4082] = {.count = 1, .reusable = true}, SHIFT(1890),
  [4084] = {.count = 1, .reusable = true}, SHIFT(1891),
  [4086] = {.count = 1, .reusable = true}, SHIFT(1892),
  [4088] = {.count = 1, .reusable = true}, SHIFT(1893),
  [4090] = {.count = 1, .reusable = true}, SHIFT(1894),
  [4092] = {.count = 1, .reusable = true}, SHIFT(1895),
  [4094] = {.count = 1, .reusable = false}, SHIFT(1898),
  [4096] = {.count = 1, .reusable = false}, SHIFT(1899),
  [4098] = {.count = 1, .reusable = true}, SHIFT(1900),
  [4100] = {.count = 1, .reusable = true}, SHIFT(1901),
  [4102] = {.count = 1, .reusable = true}, SHIFT(1903),
  [4104] = {.count = 1, .reusable = true}, SHIFT(1904),
  [4106] = {.count = 1, .reusable = true}, SHIFT(1905),
  [4108] = {.count = 1, .reusable = false}, REDUCE(sym_switch_default, 3),
  [4110] = {.count = 1, .reusable = true}, REDUCE(sym_switch_default, 3),
  [4112] = {.count = 1, .reusable = true}, SHIFT(1907),
  [4114] = {.count = 1, .reusable = true}, SHIFT(1908),
  [4116] = {.count = 1, .reusable = true}, SHIFT(1909),
  [4118] = {.count = 1, .reusable = false}, SHIFT(1909),
  [4120] = {.count = 1, .reusable = true}, SHIFT(1910),
  [4122] = {.count = 1, .reusable = false}, SHIFT(1910),
  [4124] = {.count = 1, .reusable = true}, SHIFT(1911),
  [4126] = {.count = 1, .reusable = false}, SHIFT(1911),
  [4128] = {.count = 1, .reusable = true}, SHIFT(1912),
  [4130] = {.count = 1, .reusable = true}, SHIFT(1913),
  [4132] = {.count = 1, .reusable = false}, SHIFT(1913),
  [4134] = {.count = 1, .reusable = true}, SHIFT(1914),
  [4136] = {.count = 1, .reusable = false}, SHIFT(1914),
  [4138] = {.count = 1, .reusable = false}, REDUCE(sym_switch_case, 3),
  [4140] = {.count = 1, .reusable = true}, REDUCE(sym_switch_case, 3),
  [4142] = {.count = 1, .reusable = true}, SHIFT(1916),
  [4144] = {.count = 1, .reusable = false}, SHIFT(1916),
  [4146] = {.count = 1, .reusable = true}, SHIFT(1917),
  [4148] = {.count = 1, .reusable = false}, SHIFT(1917),
  [4150] = {.count = 1, .reusable = true}, SHIFT(1918),
  [4152] = {.count = 1, .reusable = false}, SHIFT(1918),
  [4154] = {.count = 1, .reusable = true}, SHIFT(1919),
  [4156] = {.count = 1, .reusable = false}, SHIFT(1919),
  [4158] = {.count = 1, .reusable = true}, SHIFT(1920),
  [4160] = {.count = 1, .reusable = false}, SHIFT(1920),
  [4162] = {.count = 1, .reusable = true}, SHIFT(1921),
  [4164] = {.count = 1, .reusable = true}, SHIFT(1922),
  [4166] = {.count = 1, .reusable = true}, SHIFT(1923),
  [4168] = {.count = 1, .reusable = true}, SHIFT(1924),
  [4170] = {.count = 1, .reusable = true}, SHIFT(1925),
  [4172] = {.count = 1, .reusable = true}, SHIFT(1926),
  [4174] = {.count = 1, .reusable = true}, SHIFT(1666),
  [4176] = {.count = 1, .reusable = true}, SHIFT(1667),
  [4178] = {.count = 1, .reusable = true}, SHIFT(1668),
  [4180] = {.count = 1, .reusable = false}, SHIFT(1667),
  [4182] = {.count = 1, .reusable = false}, SHIFT(1668),
  [4184] = {.count = 1, .reusable = false}, SHIFT(1669),
  [4186] = {.count = 1, .reusable = true}, SHIFT(1662),
  [4188] = {.count = 1, .reusable = true}, SHIFT(1927),
  [4190] = {.count = 1, .reusable = false}, SHIFT(1927),
  [4192] = {.count = 1, .reusable = true}, SHIFT(1928),
  [4194] = {.count = 1, .reusable = false}, SHIFT(1928),
  [4196] = {.count = 1, .reusable = true}, SHIFT(1929),
  [4198] = {.count = 1, .reusable = false}, SHIFT(1929),
  [4200] = {.count = 1, .reusable = true}, SHIFT(1930),
  [4202] = {.count = 1, .reusable = true}, SHIFT(1931),
  [4204] = {.count = 1, .reusable = false}, SHIFT(1931),
  [4206] = {.count = 1, .reusable = true}, SHIFT(1932),
  [4208] = {.count = 1, .reusable = false}, SHIFT(1932),
  [4210] = {.count = 1, .reusable = true}, SHIFT(1934),
  [4212] = {.count = 1, .reusable = false}, SHIFT(1934),
  [4214] = {.count = 1, .reusable = true}, SHIFT(1935),
  [4216] = {.count = 1, .reusable = false}, SHIFT(1935),
  [4218] = {.count = 1, .reusable = true}, SHIFT(1936),
  [4220] = {.count = 1, .reusable = false}, SHIFT(1936),
  [4222] = {.count = 1, .reusable = true}, SHIFT(1937),
  [4224] = {.count = 1, .reusable = false}, SHIFT(1937),
  [4226] = {.count = 1, .reusable = true}, SHIFT(1938),
  [4228] = {.count = 1, .reusable = false}, SHIFT(1938),
  [4230] = {.count = 1, .reusable = true}, SHIFT(1939),
  [4232] = {.count = 1, .reusable = false}, SHIFT(1939),
  [4234] = {.count = 1, .reusable = true}, SHIFT(1940),
  [4236] = {.count = 1, .reusable = false}, SHIFT(1940),
  [4238] = {.count = 1, .reusable = true}, SHIFT(1941),
  [4240] = {.count = 1, .reusable = true}, SHIFT(1942),
  [4242] = {.count = 1, .reusable = false}, SHIFT(1942),
  [4244] = {.count = 1, .reusable = true}, SHIFT(1943),
  [4246] = {.count = 1, .reusable = false}, SHIFT(1943),
  [4248] = {.count = 1, .reusable = true}, SHIFT(1945),
  [4250] = {.count = 1, .reusable = false}, SHIFT(1945),
  [4252] = {.count = 1, .reusable = true}, SHIFT(1946),
  [4254] = {.count = 1, .reusable = false}, SHIFT(1946),
  [4256] = {.count = 1, .reusable = true}, SHIFT(1947),
  [4258] = {.count = 1, .reusable = false}, SHIFT(1947),
  [4260] = {.count = 1, .reusable = true}, SHIFT(1948),
  [4262] = {.count = 1, .reusable = false}, SHIFT(1948),
  [4264] = {.count = 1, .reusable = true}, REDUCE(sym_for_statement, 6),
  [4266] = {.count = 1, .reusable = false}, REDUCE(sym_for_statement, 6),
  [4268] = {.count = 1, .reusable = true}, SHIFT(1954),
  [4270] = {.count = 1, .reusable = true}, SHIFT(1955),
  [4272] = {.count = 1, .reusable = true}, SHIFT(962),
  [4274] = {.count = 1, .reusable = true}, SHIFT(1956),
  [4276] = {.count = 1, .reusable = true}, SHIFT(1957),
  [4278] = {.count = 1, .reusable = true}, SHIFT(1958),
  [4280] = {.count = 1, .reusable = true}, SHIFT(1959),
  [4282] = {.count = 1, .reusable = true}, SHIFT(1960),
  [4284] = {.count = 1, .reusable = true}, SHIFT(1962),
  [4286] = {.count = 1, .reusable = false}, SHIFT(1965),
  [4288] = {.count = 1, .reusable = true}, SHIFT(1966),
  [4290] = {.count = 1, .reusable = false}, SHIFT(1968),
  [4292] = {.count = 1, .reusable = true}, SHIFT(1969),
  [4294] = {.count = 1, .reusable = false}, SHIFT(1970),
  [4296] = {.count = 1, .reusable = true}, SHIFT(1971),
  [4298] = {.count = 1, .reusable = true}, SHIFT(1973),
  [4300] = {.count = 1, .reusable = true}, SHIFT(1977),
  [4302] = {.count = 1, .reusable = false}, SHIFT(1977),
  [4304] = {.count = 1, .reusable = true}, SHIFT(1979),
  [4306] = {.count = 1, .reusable = false}, SHIFT(1979),
  [4308] = {.count = 1, .reusable = true}, SHIFT(1981),
  [4310] = {.count = 1, .reusable = false}, SHIFT(1981),
  [4312] = {.count = 1, .reusable = true}, SHIFT(1983),
  [4314] = {.count = 1, .reusable = false}, SHIFT(1983),
  [4316] = {.count = 1, .reusable = true}, SHIFT(1984),
  [4318] = {.count = 1, .reusable = true}, SHIFT(1985),
  [4320] = {.count = 1, .reusable = true}, SHIFT(1986),
  [4322] = {.count = 1, .reusable = true}, SHIFT(1987),
  [4324] = {.count = 1, .reusable = true}, SHIFT(1988),
  [4326] = {.count = 1, .reusable = true}, SHIFT(1989),
  [4328] = {.count = 1, .reusable = true}, SHIFT(1990),
  [4330] = {.count = 1, .reusable = true}, SHIFT(1991),
  [4332] = {.count = 1, .reusable = true}, REDUCE(sym_jsx_fragment, 6),
  [4334] = {.count = 1, .reusable = false}, REDUCE(sym_jsx_fragment, 6),
  [4336] = {.count = 1, .reusable = false}, SHIFT(1993),
  [4338] = {.count = 1, .reusable = false}, SHIFT(1994),
  [4340] = {.count = 1, .reusable = false}, SHIFT(1995),
  [4342] = {.count = 1, .reusable = false}, SHIFT(1997),
  [4344] = {.count = 1, .reusable = false}, SHIFT(1999),
  [4346] = {.count = 1, .reusable = true}, SHIFT(2001),
  [4348] = {.count = 1, .reusable = true}, SHIFT(2003),
  [4350] = {.count = 1, .reusable = true}, REDUCE(aux_sym_class_body_repeat1, 4),
  [4352] = {.count = 1, .reusable = false}, REDUCE(aux_sym_class_body_repeat1, 4),
  [4354] = {.count = 1, .reusable = true}, SHIFT(2004),
  [4356] = {.count = 1, .reusable = false}, SHIFT(2004),
  [4358] = {.count = 1, .reusable = true}, SHIFT(2005),
  [4360] = {.count = 1, .reusable = false}, SHIFT(2005),
  [4362] = {.count = 1, .reusable = true}, SHIFT(2006),
  [4364] = {.count = 1, .reusable = false}, SHIFT(2006),
  [4366] = {.count = 1, .reusable = true}, SHIFT(2007),
  [4368] = {.count = 1, .reusable = true}, SHIFT(2008),
  [4370] = {.count = 1, .reusable = false}, SHIFT(2008),
  [4372] = {.count = 1, .reusable = true}, SHIFT(2009),
  [4374] = {.count = 1, .reusable = true}, SHIFT(2010),
  [4376] = {.count = 1, .reusable = true}, SHIFT(2011),
  [4378] = {.count = 1, .reusable = true}, SHIFT(2012),
  [4380] = {.count = 1, .reusable = true}, SHIFT(2013),
  [4382] = {.count = 1, .reusable = true}, SHIFT(2014),
  [4384] = {.count = 1, .reusable = true}, SHIFT(2015),
  [4386] = {.count = 1, .reusable = true}, SHIFT(2017),
  [4388] = {.count = 1, .reusable = false}, SHIFT(2017),
  [4390] = {.count = 1, .reusable = true}, SHIFT(2018),
  [4392] = {.count = 1, .reusable = true}, SHIFT(2021),
  [4394] = {.count = 1, .reusable = false}, SHIFT(2021),
  [4396] = {.count = 1, .reusable = true}, SHIFT(2023),
  [4398] = {.count = 1, .reusable = false}, SHIFT(2023),
  [4400] = {.count = 1, .reusable = true}, SHIFT(2024),
  [4402] = {.count = 1, .reusable = true}, SHIFT(2025),
  [4404] = {.count = 1, .reusable = true}, SHIFT(2026),
  [4406] = {.count = 1, .reusable = true}, SHIFT(2027),
  [4408] = {.count = 1, .reusable = true}, SHIFT(2028),
  [4410] = {.count = 1, .reusable = true}, SHIFT(2029),
  [4412] = {.count = 1, .reusable = true}, SHIFT(2030),
  [4414] = {.count = 1, .reusable = false}, SHIFT(2030),
  [4416] = {.count = 1, .reusable = true}, SHIFT(2031),
  [4418] = {.count = 1, .reusable = true}, SHIFT(2034),
  [4420] = {.count = 1, .reusable = true}, SHIFT(2037),
  [4422] = {.count = 1, .reusable = false}, SHIFT(2037),
  [4424] = {.count = 1, .reusable = true}, SHIFT(2039),
  [4426] = {.count = 1, .reusable = false}, SHIFT(2039),
  [4428] = {.count = 1, .reusable = true}, SHIFT(2041),
  [4430] = {.count = 1, .reusable = false}, SHIFT(2041),
  [4432] = {.count = 1, .reusable = true}, SHIFT(2042),
  [4434] = {.count = 1, .reusable = true}, SHIFT(2043),
  [4436] = {.count = 1, .reusable = true}, SHIFT(2045),
  [4438] = {.count = 1, .reusable = true}, SHIFT(2046),
  [4440] = {.count = 1, .reusable = true}, SHIFT(2047),
  [4442] = {.count = 1, .reusable = true}, SHIFT(2048),
  [4444] = {.count = 1, .reusable = true}, SHIFT(2049),
  [4446] = {.count = 1, .reusable = true}, SHIFT(2050),
  [4448] = {.count = 1, .reusable = true}, SHIFT(2051),
  [4450] = {.count = 1, .reusable = true}, SHIFT(2053),
  [4452] = {.count = 1, .reusable = false}, SHIFT(2053),
  [4454] = {.count = 1, .reusable = true}, SHIFT(2054),
  [4456] = {.count = 1, .reusable = true}, SHIFT(2055),
  [4458] = {.count = 1, .reusable = true}, SHIFT(2056),
  [4460] = {.count = 1, .reusable = true}, SHIFT(2057),
  [4462] = {.count = 1, .reusable = true}, SHIFT(2059),
  [4464] = {.count = 1, .reusable = true}, SHIFT(2060),
  [4466] = {.count = 1, .reusable = true}, SHIFT(2061),
  [4468] = {.count = 1, .reusable = true}, SHIFT(2062),
  [4470] = {.count = 1, .reusable = true}, SHIFT(2064),
  [4472] = {.count = 1, .reusable = true}, SHIFT(2066),
  [4474] = {.count = 1, .reusable = false}, SHIFT(2068),
  [4476] = {.count = 1, .reusable = true}, SHIFT(2069),
  [4478] = {.count = 1, .reusable = false}, SHIFT(2070),
  [4480] = {.count = 1, .reusable = false}, SHIFT(2071),
  [4482] = {.count = 1, .reusable = false}, SHIFT(2072),
  [4484] = {.count = 1, .reusable = false}, SHIFT(2073),
  [4486] = {.count = 1, .reusable = false}, SHIFT(2074),
  [4488] = {.count = 1, .reusable = false}, SHIFT(2075),
  [4490] = {.count = 1, .reusable = false}, SHIFT(2076),
  [4492] = {.count = 1, .reusable = false}, SHIFT(2077),
  [4494] = {.count = 1, .reusable = false}, SHIFT(2078),
  [4496] = {.count = 1, .reusable = false}, SHIFT(2079),
  [4498] = {.count = 1, .reusable = false}, SHIFT(2080),
  [4500] = {.count = 1, .reusable = false}, SHIFT(2081),
  [4502] = {.count = 1, .reusable = false}, SHIFT(2082),
  [4504] = {.count = 1, .reusable = false}, SHIFT(2083),
  [4506] = {.count = 1, .reusable = false}, SHIFT(2084),
  [4508] = {.count = 1, .reusable = true}, SHIFT(2085),
  [4510] = {.count = 1, .reusable = false}, SHIFT(2086),
  [4512] = {.count = 1, .reusable = false}, SHIFT(2087),
  [4514] = {.count = 1, .reusable = false}, SHIFT(2088),
  [4516] = {.count = 1, .reusable = false}, SHIFT(2089),
  [4518] = {.count = 1, .reusable = true}, SHIFT(2095),
  [4520] = {.count = 1, .reusable = false}, SHIFT(2097),
  [4522] = {.count = 1, .reusable = false}, SHIFT(2098),
  [4524] = {.count = 1, .reusable = false}, SHIFT(2099),
  [4526] = {.count = 1, .reusable = true}, SHIFT(2104),
  [4528] = {.count = 1, .reusable = true}, SHIFT(2105),
  [4530] = {.count = 1, .reusable = false}, SHIFT(2107),
  [4532] = {.count = 1, .reusable = false}, SHIFT(2108),
  [4534] = {.count = 1, .reusable = true}, SHIFT(2112),
  [4536] = {.count = 1, .reusable = true}, SHIFT(2113),
  [4538] = {.count = 1, .reusable = true}, SHIFT(2114),
  [4540] = {.count = 1, .reusable = true}, SHIFT(2115),
  [4542] = {.count = 1, .reusable = true}, SHIFT(2116),
  [4544] = {.count = 1, .reusable = true}, SHIFT(2119),
  [4546] = {.count = 1, .reusable = true}, SHIFT(2121),
  [4548] = {.count = 1, .reusable = true}, SHIFT(2123),
  [4550] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1604),
  [4553] = {.count = 1, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2),
  [4555] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1605),
  [4558] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1606),
  [4561] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1607),
  [4564] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1608),
  [4567] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1609),
  [4570] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1610),
  [4573] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1611),
  [4576] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1612),
  [4579] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1613),
  [4582] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1614),
  [4585] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1615),
  [4588] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1616),
  [4591] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1617),
  [4594] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1618),
  [4597] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1619),
  [4600] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1620),
  [4603] = {.count = 2, .reusable = true}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1621),
  [4606] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1622),
  [4609] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1623),
  [4612] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1624),
  [4615] = {.count = 2, .reusable = false}, REDUCE(aux_sym_program_repeat1, 2), SHIFT_REPEAT(1625),
  [4618] = {.count = 1, .reusable = true}, SHIFT(2126),
  [4620] = {.count = 1, .reusable = false}, SHIFT(2127),
  [4622] = {.count = 1, .reusable = true}, SHIFT(2128),
  [4624] = {.count = 1, .reusable = false}, SHIFT(2128),
  [4626] = {.count = 1, .reusable = false}, REDUCE(sym_switch_case, 4),
  [4628] = {.count = 1, .reusable = true}, REDUCE(sym_switch_case, 4),
  [4630] = {.count = 1, .reusable = true}, SHIFT(2129),
  [4632] = {.count = 1, .reusable = true}, SHIFT(2136),
  [4634] = {.count = 1, .reusable = false}, SHIFT(2136),
  [4636] = {.count = 1, .reusable = true}, REDUCE(sym_for_in_statement, 7),
  [4638] = {.count = 1, .reusable = false}, REDUCE(sym_for_in_statement, 7),
  [4640] = {.count = 1, .reusable = true}, SHIFT(2137),
  [4642] = {.count = 1, .reusable = true}, SHIFT(2138),
  [4644] = {.count = 1, .reusable = false}, SHIFT(2138),
  [4646] = {.count = 1, .reusable = true}, REDUCE(sym_for_of_statement, 7),
  [4648] = {.count = 1, .reusable = false}, REDUCE(sym_for_of_statement, 7),
  [4650] = {.count = 1, .reusable = true}, SHIFT(2139),
  [4652] = {.count = 1, .reusable = true}, REDUCE(sym_for_statement, 7),
  [4654] = {.count = 1, .reusable = false}, REDUCE(sym_for_statement, 7),
  [4656] = {.count = 1, .reusable = true}, REDUCE(sym_for_in_statement, 7, .alias_sequence_id = 22),
  [4658] = {.count = 1, .reusable = false}, REDUCE(sym_for_in_statement, 7, .alias_sequence_id = 22),
  [4660] = {.count = 1, .reusable = true}, REDUCE(sym_for_of_statement, 7, .alias_sequence_id = 22),
  [4662] = {.count = 1, .reusable = false}, REDUCE(sym_for_of_statement, 7, .alias_sequence_id = 22),
  [4664] = {.count = 1, .reusable = true}, REDUCE(sym_for_in_statement, 7, .alias_sequence_id = 23),
  [4666] = {.count = 1, .reusable = false}, REDUCE(sym_for_in_statement, 7, .alias_sequence_id = 23),
  [4668] = {.count = 1, .reusable = true}, REDUCE(sym_for_of_statement, 7, .alias_sequence_id = 23),
  [4670] = {.count = 1, .reusable = false}, REDUCE(sym_for_of_statement, 7, .alias_sequence_id = 23),
  [4672] = {.count = 1, .reusable = true}, SHIFT(2141),
  [4674] = {.count = 1, .reusable = true}, SHIFT(2142),
  [4676] = {.count = 1, .reusable = true}, SHIFT(2143),
  [4678] = {.count = 1, .reusable = true}, SHIFT(2144),
  [4680] = {.count = 1, .reusable = true}, SHIFT(2145),
  [4682] = {.count = 1, .reusable = true}, SHIFT(2146),
  [4684] = {.count = 1, .reusable = true}, SHIFT(2147),
  [4686] = {.count = 1, .reusable = true}, SHIFT(2149),
  [4688] = {.count = 1, .reusable = false}, SHIFT(2149),
  [4690] = {.count = 1, .reusable = true}, SHIFT(2150),
  [4692] = {.count = 1, .reusable = true}, SHIFT(2153),
  [4694] = {.count = 1, .reusable = false}, SHIFT(2153),
  [4696] = {.count = 1, .reusable = true}, SHIFT(2155),
  [4698] = {.count = 1, .reusable = false}, SHIFT(2155),
  [4700] = {.count = 1, .reusable = true}, SHIFT(2156),
  [4702] = {.count = 1, .reusable = true}, SHIFT(2157),
  [4704] = {.count = 1, .reusable = true}, SHIFT(2158),
  [4706] = {.count = 1, .reusable = true}, SHIFT(2159),
  [4708] = {.count = 1, .reusable = true}, SHIFT(2160),
  [4710] = {.count = 1, .reusable = true}, SHIFT(2161),
  [4712] = {.count = 1, .reusable = true}, SHIFT(2162),
  [4714] = {.count = 1, .reusable = true}, SHIFT(2163),
  [4716] = {.count = 1, .reusable = true}, SHIFT(2164),
  [4718] = {.count = 1, .reusable = true}, REDUCE(sym_catch_clause, 5),
  [4720] = {.count = 1, .reusable = false}, REDUCE(sym_catch_clause, 5),
  [4722] = {.count = 1, .reusable = false}, SHIFT(2165),
  [4724] = {.count = 1, .reusable = false}, SHIFT(2166),
  [4726] = {.count = 1, .reusable = true}, SHIFT(2167),
  [4728] = {.count = 1, .reusable = false}, SHIFT(2168),
  [4730] = {.count = 1, .reusable = true}, SHIFT(2169),
  [4732] = {.count = 1, .reusable = false}, SHIFT(2170),
  [4734] = {.count = 1, .reusable = true}, SHIFT(2171),
  [4736] = {.count = 1, .reusable = false}, SHIFT(2172),
  [4738] = {.count = 1, .reusable = true}, SHIFT(2175),
  [4740] = {.count = 1, .reusable = true}, SHIFT(2176),
  [4742] = {.count = 1, .reusable = false}, SHIFT(2176),
  [4744] = {.count = 1, .reusable = true}, SHIFT(2178),
  [4746] = {.count = 1, .reusable = false}, SHIFT(2178),
  [4748] = {.count = 1, .reusable = true}, SHIFT(2180),
  [4750] = {.count = 1, .reusable = false}, SHIFT(2180),
  [4752] = {.count = 1, .reusable = true}, SHIFT(2182),
  [4754] = {.count = 1, .reusable = false}, SHIFT(2182),
  [4756] = {.count = 1, .reusable = true}, SHIFT(2183),
  [4758] = {.count = 1, .reusable = true}, SHIFT(2184),
  [4760] = {.count = 1, .reusable = true}, SHIFT(2185),
  [4762] = {.count = 1, .reusable = true}, SHIFT(2186),
  [4764] = {.count = 1, .reusable = true}, SHIFT(2187),
  [4766] = {.count = 1, .reusable = true}, SHIFT(2188),
  [4768] = {.count = 1, .reusable = true}, SHIFT(2189),
  [4770] = {.count = 1, .reusable = true}, SHIFT(2190),
  [4772] = {.count = 1, .reusable = true}, SHIFT(2191),
  [4774] = {.count = 1, .reusable = true}, SHIFT(2192),
  [4776] = {.count = 1, .reusable = true}, SHIFT(2193),
  [4778] = {.count = 1, .reusable = true}, SHIFT(2194),
  [4780] = {.count = 1, .reusable = true}, SHIFT(2195),
  [4782] = {.count = 1, .reusable = true}, SHIFT(2204),
  [4784] = {.count = 1, .reusable = true}, SHIFT(2205),
  [4786] = {.count = 1, .reusable = true}, SHIFT(2208),
  [4788] = {.count = 1, .reusable = true}, SHIFT(2210),
  [4790] = {.count = 1, .reusable = true}, SHIFT(2212),
  [4792] = {.count = 1, .reusable = true}, SHIFT(2214),
  [4794] = {.count = 1, .reusable = true}, SHIFT(2215),
  [4796] = {.count = 1, .reusable = true}, SHIFT(2216),
  [4798] = {.count = 1, .reusable = true}, SHIFT(2217),
  [4800] = {.count = 1, .reusable = true}, SHIFT(2218),
  [4802] = {.count = 1, .reusable = true}, SHIFT(2219),
  [4804] = {.count = 1, .reusable = true}, SHIFT(2071),
  [4806] = {.count = 1, .reusable = true}, SHIFT(2072),
  [4808] = {.count = 1, .reusable = true}, SHIFT(2220),
  [4810] = {.count = 1, .reusable = true}, SHIFT(2221),
  [4812] = {.count = 1, .reusable = true}, SHIFT(2222),
  [4814] = {.count = 1, .reusable = true}, SHIFT(2227),
  [4816] = {.count = 1, .reusable = true}, SHIFT(2229),
  [4818] = {.count = 1, .reusable = true}, SHIFT(2235),
  [4820] = {.count = 1, .reusable = true}, SHIFT(2238),
  [4822] = {.count = 1, .reusable = true}, SHIFT(2241),
  [4824] = {.count = 1, .reusable = true}, SHIFT(2242),
  [4826] = {.count = 1, .reusable = true}, SHIFT(2243),
  [4828] = {.count = 1, .reusable = true}, SHIFT(2244),
  [4830] = {.count = 1, .reusable = true}, SHIFT(2245),
  [4832] = {.count = 1, .reusable = true}, SHIFT(2246),
  [4834] = {.count = 1, .reusable = false}, SHIFT(2249),
  [4836] = {.count = 1, .reusable = false}, SHIFT(2250),
  [4838] = {.count = 1, .reusable = true}, SHIFT(2251),
  [4840] = {.count = 1, .reusable = true}, SHIFT(2252),
  [4842] = {.count = 1, .reusable = true}, SHIFT(2254),
  [4844] = {.count = 1, .reusable = false}, SHIFT(2255),
  [4846] = {.count = 1, .reusable = true}, SHIFT(2256),
  [4848] = {.count = 1, .reusable = true}, SHIFT(2257),
  [4850] = {.count = 1, .reusable = true}, SHIFT(2258),
  [4852] = {.count = 1, .reusable = true}, SHIFT(2259),
  [4854] = {.count = 1, .reusable = true}, SHIFT(2260),
  [4856] = {.count = 1, .reusable = true}, SHIFT(2262),
  [4858] = {.count = 1, .reusable = false}, SHIFT(2265),
  [4860] = {.count = 1, .reusable = true}, SHIFT(2266),
  [4862] = {.count = 1, .reusable = false}, SHIFT(2268),
  [4864] = {.count = 1, .reusable = true}, SHIFT(2269),
  [4866] = {.count = 1, .reusable = false}, SHIFT(2270),
  [4868] = {.count = 1, .reusable = true}, SHIFT(2271),
  [4870] = {.count = 1, .reusable = true}, SHIFT(2273),
  [4872] = {.count = 1, .reusable = true}, SHIFT(2274),
  [4874] = {.count = 1, .reusable = true}, SHIFT(2275),
  [4876] = {.count = 1, .reusable = true}, SHIFT(2277),
  [4878] = {.count = 1, .reusable = true}, SHIFT(2280),
  [4880] = {.count = 1, .reusable = true}, SHIFT(2287),
  [4882] = {.count = 1, .reusable = true}, SHIFT(2289),
  [4884] = {.count = 1, .reusable = false}, SHIFT(2289),
  [4886] = {.count = 1, .reusable = true}, SHIFT(2291),
  [4888] = {.count = 1, .reusable = false}, SHIFT(2291),
  [4890] = {.count = 1, .reusable = true}, REDUCE(sym_for_in_statement, 8),
  [4892] = {.count = 1, .reusable = false}, REDUCE(sym_for_in_statement, 8),
  [4894] = {.count = 1, .reusable = true}, REDUCE(sym_for_of_statement, 8),
  [4896] = {.count = 1, .reusable = false}, REDUCE(sym_for_of_statement, 8),
  [4898] = {.count = 1, .reusable = true}, REDUCE(sym_for_in_statement, 8, .alias_sequence_id = 24),
  [4900] = {.count = 1, .reusable = false}, REDUCE(sym_for_in_statement, 8, .alias_sequence_id = 24),
  [4902] = {.count = 1, .reusable = true}, REDUCE(sym_for_of_statement, 8, .alias_sequence_id = 24),
  [4904] = {.count = 1, .reusable = false}, REDUCE(sym_for_of_statement, 8, .alias_sequence_id = 24),
  [4906] = {.count = 1, .reusable = true}, REDUCE(sym_for_in_statement, 8, .alias_sequence_id = 25),
  [4908] = {.count = 1, .reusable = false}, REDUCE(sym_for_in_statement, 8, .alias_sequence_id = 25),
  [4910] = {.count = 1, .reusable = true}, REDUCE(sym_for_of_statement, 8, .alias_sequence_id = 25),
  [4912] = {.count = 1, .reusable = false}, REDUCE(sym_for_of_statement, 8, .alias_sequence_id = 25),
  [4914] = {.count = 1, .reusable = true}, SHIFT(2292),
  [4916] = {.count = 1, .reusable = false}, SHIFT(2292),
  [4918] = {.count = 1, .reusable = true}, SHIFT(2293),
  [4920] = {.count = 1, .reusable = false}, SHIFT(2293),
  [4922] = {.count = 1, .reusable = true}, SHIFT(2295),
  [4924] = {.count = 1, .reusable = false}, SHIFT(2295),
  [4926] = {.count = 1, .reusable = true}, SHIFT(2297),
  [4928] = {.count = 1, .reusable = false}, SHIFT(2297),
  [4930] = {.count = 1, .reusable = true}, SHIFT(2299),
  [4932] = {.count = 1, .reusable = false}, SHIFT(2299),
  [4934] = {.count = 1, .reusable = true}, SHIFT(2300),
  [4936] = {.count = 1, .reusable = true}, SHIFT(2301),
  [4938] = {.count = 1, .reusable = true}, SHIFT(2303),
  [4940] = {.count = 1, .reusable = true}, SHIFT(2304),
  [4942] = {.count = 1, .reusable = true}, SHIFT(2305),
  [4944] = {.count = 1, .reusable = true}, SHIFT(2306),
  [4946] = {.count = 1, .reusable = true}, SHIFT(2307),
  [4948] = {.count = 1, .reusable = true}, SHIFT(2308),
  [4950] = {.count = 1, .reusable = true}, SHIFT(2309),
  [4952] = {.count = 1, .reusable = false}, SHIFT(2310),
  [4954] = {.count = 1, .reusable = true}, SHIFT(2311),
  [4956] = {.count = 1, .reusable = true}, SHIFT(2312),
  [4958] = {.count = 1, .reusable = true}, SHIFT(2313),
  [4960] = {.count = 1, .reusable = true}, SHIFT(2314),
  [4962] = {.count = 1, .reusable = true}, SHIFT(2315),
  [4964] = {.count = 1, .reusable = true}, SHIFT(2316),
  [4966] = {.count = 1, .reusable = true}, SHIFT(2317),
  [4968] = {.count = 1, .reusable = true}, SHIFT(2318),
  [4970] = {.count = 1, .reusable = true}, SHIFT(2319),
  [4972] = {.count = 1, .reusable = true}, SHIFT(2320),
  [4974] = {.count = 1, .reusable = true}, SHIFT(2327),
  [4976] = {.count = 1, .reusable = true}, SHIFT(2334),
  [4978] = {.count = 1, .reusable = true}, SHIFT(2338),
  [4980] = {.count = 1, .reusable = false}, SHIFT(2338),
  [4982] = {.count = 1, .reusable = true}, SHIFT(2339),
  [4984] = {.count = 1, .reusable = true}, SHIFT(2340),
  [4986] = {.count = 1, .reusable = true}, SHIFT(2341),
  [4988] = {.count = 1, .reusable = true}, SHIFT(2342),
  [4990] = {.count = 1, .reusable = true}, SHIFT(2344),
  [4992] = {.count = 1, .reusable = true}, SHIFT(2345),
  [4994] = {.count = 1, .reusable = true}, SHIFT(2346),
  [4996] = {.count = 1, .reusable = true}, SHIFT(2347),
  [4998] = {.count = 1, .reusable = true}, SHIFT(2349),
  [5000] = {.count = 1, .reusable = true}, SHIFT(2351),
  [5002] = {.count = 1, .reusable = true}, SHIFT(2354),
  [5004] = {.count = 1, .reusable = false}, SHIFT(2356),
  [5006] = {.count = 1, .reusable = false}, SHIFT(2357),
  [5008] = {.count = 1, .reusable = false}, SHIFT(2358),
  [5010] = {.count = 1, .reusable = true}, SHIFT(2363),
  [5012] = {.count = 1, .reusable = true}, SHIFT(2364),
  [5014] = {.count = 1, .reusable = false}, SHIFT(2366),
  [5016] = {.count = 1, .reusable = false}, SHIFT(2367),
  [5018] = {.count = 1, .reusable = true}, SHIFT(2371),
  [5020] = {.count = 1, .reusable = true}, SHIFT(2372),
  [5022] = {.count = 1, .reusable = true}, SHIFT(2373),
  [5024] = {.count = 1, .reusable = true}, SHIFT(2374),
  [5026] = {.count = 1, .reusable = true}, SHIFT(2375),
  [5028] = {.count = 1, .reusable = true}, SHIFT(2378),
  [5030] = {.count = 1, .reusable = true}, SHIFT(2380),
  [5032] = {.count = 1, .reusable = true}, SHIFT(2382),
  [5034] = {.count = 1, .reusable = true}, SHIFT(2386),
  [5036] = {.count = 1, .reusable = false}, SHIFT(2387),
  [5038] = {.count = 1, .reusable = true}, SHIFT(2388),
  [5040] = {.count = 1, .reusable = true}, SHIFT(2389),
  [5042] = {.count = 1, .reusable = true}, SHIFT(2390),
  [5044] = {.count = 1, .reusable = true}, SHIFT(2391),
  [5046] = {.count = 1, .reusable = true}, SHIFT(2392),
  [5048] = {.count = 1, .reusable = true}, SHIFT(2393),
  [5050] = {.count = 1, .reusable = true}, SHIFT(2394),
  [5052] = {.count = 1, .reusable = true}, SHIFT(2396),
  [5054] = {.count = 1, .reusable = false}, SHIFT(2396),
  [5056] = {.count = 1, .reusable = true}, SHIFT(2397),
  [5058] = {.count = 1, .reusable = true}, SHIFT(2400),
  [5060] = {.count = 1, .reusable = false}, SHIFT(2400),
  [5062] = {.count = 1, .reusable = true}, SHIFT(2402),
  [5064] = {.count = 1, .reusable = false}, SHIFT(2402),
  [5066] = {.count = 1, .reusable = true}, SHIFT(2403),
  [5068] = {.count = 1, .reusable = true}, SHIFT(2404),
  [5070] = {.count = 1, .reusable = true}, SHIFT(2406),
  [5072] = {.count = 1, .reusable = true}, SHIFT(2407),
  [5074] = {.count = 1, .reusable = true}, SHIFT(2408),
  [5076] = {.count = 1, .reusable = true}, SHIFT(2411),
  [5078] = {.count = 1, .reusable = true}, SHIFT(2412),
  [5080] = {.count = 1, .reusable = true}, SHIFT(2414),
  [5082] = {.count = 1, .reusable = true}, SHIFT(2415),
  [5084] = {.count = 1, .reusable = true}, SHIFT(2416),
  [5086] = {.count = 1, .reusable = true}, SHIFT(2417),
  [5088] = {.count = 1, .reusable = true}, SHIFT(2418),
  [5090] = {.count = 1, .reusable = true}, SHIFT(2419),
  [5092] = {.count = 1, .reusable = true}, SHIFT(2428),
  [5094] = {.count = 1, .reusable = true}, SHIFT(2429),
  [5096] = {.count = 1, .reusable = true}, SHIFT(2432),
  [5098] = {.count = 1, .reusable = true}, SHIFT(2434),
  [5100] = {.count = 1, .reusable = true}, SHIFT(2435),
  [5102] = {.count = 1, .reusable = true}, SHIFT(2438),
  [5104] = {.count = 1, .reusable = true}, SHIFT(2440),
  [5106] = {.count = 1, .reusable = true}, SHIFT(2442),
  [5108] = {.count = 1, .reusable = true}, SHIFT(2444),
  [5110] = {.count = 1, .reusable = true}, SHIFT(2445),
  [5112] = {.count = 1, .reusable = true}, SHIFT(2446),
  [5114] = {.count = 1, .reusable = true}, SHIFT(2447),
  [5116] = {.count = 1, .reusable = false}, SHIFT(2448),
  [5118] = {.count = 1, .reusable = true}, SHIFT(2449),
  [5120] = {.count = 1, .reusable = true}, SHIFT(2451),
  [5122] = {.count = 1, .reusable = false}, SHIFT(2454),
  [5124] = {.count = 1, .reusable = true}, SHIFT(2455),
  [5126] = {.count = 1, .reusable = false}, SHIFT(2457),
  [5128] = {.count = 1, .reusable = true}, SHIFT(2458),
  [5130] = {.count = 1, .reusable = false}, SHIFT(2459),
  [5132] = {.count = 1, .reusable = true}, SHIFT(2460),
  [5134] = {.count = 1, .reusable = true}, SHIFT(2462),
  [5136] = {.count = 1, .reusable = true}, SHIFT(2463),
  [5138] = {.count = 1, .reusable = true}, SHIFT(2464),
  [5140] = {.count = 1, .reusable = true}, SHIFT(2466),
  [5142] = {.count = 1, .reusable = true}, SHIFT(2469),
  [5144] = {.count = 1, .reusable = true}, SHIFT(2476),
  [5146] = {.count = 1, .reusable = true}, SHIFT(2478),
  [5148] = {.count = 1, .reusable = false}, SHIFT(2478),
  [5150] = {.count = 1, .reusable = true}, SHIFT(2481),
  [5152] = {.count = 1, .reusable = false}, SHIFT(2481),
  [5154] = {.count = 1, .reusable = true}, SHIFT(2483),
  [5156] = {.count = 1, .reusable = false}, SHIFT(2483),
  [5158] = {.count = 1, .reusable = true}, SHIFT(2485),
  [5160] = {.count = 1, .reusable = false}, SHIFT(2485),
  [5162] = {.count = 1, .reusable = true}, SHIFT(2486),
  [5164] = {.count = 1, .reusable = true}, SHIFT(2487),
  [5166] = {.count = 1, .reusable = true}, SHIFT(2489),
  [5168] = {.count = 1, .reusable = true}, SHIFT(2490),
  [5170] = {.count = 1, .reusable = true}, SHIFT(2491),
  [5172] = {.count = 1, .reusable = true}, SHIFT(2492),
  [5174] = {.count = 1, .reusable = true}, SHIFT(2493),
  [5176] = {.count = 1, .reusable = true}, SHIFT(2494),
  [5178] = {.count = 1, .reusable = true}, SHIFT(2495),
  [5180] = {.count = 1, .reusable = true}, SHIFT(2502),
  [5182] = {.count = 1, .reusable = true}, SHIFT(2509),
  [5184] = {.count = 1, .reusable = true}, SHIFT(2513),
  [5186] = {.count = 1, .reusable = true}, SHIFT(2514),
  [5188] = {.count = 1, .reusable = true}, SHIFT(2515),
  [5190] = {.count = 1, .reusable = true}, SHIFT(2516),
  [5192] = {.count = 1, .reusable = true}, SHIFT(2517),
  [5194] = {.count = 1, .reusable = true}, SHIFT(2518),
  [5196] = {.count = 1, .reusable = true}, SHIFT(2519),
  [5198] = {.count = 1, .reusable = true}, SHIFT(2521),
  [5200] = {.count = 1, .reusable = false}, SHIFT(2521),
  [5202] = {.count = 1, .reusable = true}, SHIFT(2522),
  [5204] = {.count = 1, .reusable = true}, SHIFT(2525),
  [5206] = {.count = 1, .reusable = false}, SHIFT(2525),
  [5208] = {.count = 1, .reusable = true}, SHIFT(2527),
  [5210] = {.count = 1, .reusable = false}, SHIFT(2527),
  [5212] = {.count = 1, .reusable = true}, SHIFT(2528),
  [5214] = {.count = 1, .reusable = true}, SHIFT(2529),
  [5216] = {.count = 1, .reusable = true}, SHIFT(2531),
  [5218] = {.count = 1, .reusable = true}, SHIFT(2532),
  [5220] = {.count = 1, .reusable = true}, SHIFT(2533),
  [5222] = {.count = 1, .reusable = true}, SHIFT(2536),
  [5224] = {.count = 1, .reusable = true}, SHIFT(2537),
  [5226] = {.count = 1, .reusable = true}, SHIFT(2539),
  [5228] = {.count = 1, .reusable = true}, SHIFT(2540),
  [5230] = {.count = 1, .reusable = true}, SHIFT(2541),
  [5232] = {.count = 1, .reusable = true}, SHIFT(2542),
  [5234] = {.count = 1, .reusable = true}, SHIFT(2543),
  [5236] = {.count = 1, .reusable = true}, SHIFT(2544),
  [5238] = {.count = 1, .reusable = true}, SHIFT(2553),
  [5240] = {.count = 1, .reusable = true}, SHIFT(2556),
  [5242] = {.count = 1, .reusable = true}, SHIFT(2559),
  [5244] = {.count = 1, .reusable = false}, SHIFT(2559),
  [5246] = {.count = 1, .reusable = true}, SHIFT(2561),
  [5248] = {.count = 1, .reusable = false}, SHIFT(2561),
  [5250] = {.count = 1, .reusable = true}, SHIFT(2563),
  [5252] = {.count = 1, .reusable = false}, SHIFT(2563),
  [5254] = {.count = 1, .reusable = true}, SHIFT(2564),
  [5256] = {.count = 1, .reusable = true}, SHIFT(2565),
  [5258] = {.count = 1, .reusable = true}, SHIFT(2567),
  [5260] = {.count = 1, .reusable = true}, SHIFT(2568),
  [5262] = {.count = 1, .reusable = true}, SHIFT(2569),
  [5264] = {.count = 1, .reusable = true}, SHIFT(2570),
  [5266] = {.count = 1, .reusable = true}, SHIFT(2571),
  [5268] = {.count = 1, .reusable = true}, SHIFT(2572),
  [5270] = {.count = 1, .reusable = true}, SHIFT(2573),
  [5272] = {.count = 1, .reusable = true}, SHIFT(2580),
  [5274] = {.count = 1, .reusable = true}, SHIFT(2581),
  [5276] = {.count = 1, .reusable = true}, SHIFT(2582),
  [5278] = {.count = 1, .reusable = true}, SHIFT(2583),
  [5280] = {.count = 1, .reusable = true}, SHIFT(2584),
  [5282] = {.count = 1, .reusable = true}, SHIFT(2585),
};

void *tree_sitter_javascript_external_scanner_create();
void tree_sitter_javascript_external_scanner_destroy(void *);
bool tree_sitter_javascript_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_javascript_external_scanner_serialize(void *, char *);
void tree_sitter_javascript_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef _WIN32
#define extern __declspec(dllexport)
#endif

extern const TSLanguage *tree_sitter_javascript() {
  static TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .symbol_metadata = ts_symbol_metadata,
    .parse_table = (const unsigned short *)ts_parse_table,
    .parse_actions = ts_parse_actions,
    .lex_modes = ts_lex_modes,
    .symbol_names = ts_symbol_names,
    .alias_sequences = (const TSSymbol *)ts_alias_sequences,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .external_scanner = {
      (const bool *)ts_external_scanner_states,
      ts_external_scanner_symbol_map,
      tree_sitter_javascript_external_scanner_create,
      tree_sitter_javascript_external_scanner_destroy,
      tree_sitter_javascript_external_scanner_scan,
      tree_sitter_javascript_external_scanner_serialize,
      tree_sitter_javascript_external_scanner_deserialize,
    },
  };
  return &language;
}
